=== modified file '.bzrignore'
--- .bzrignore	2005-08-04 19:31:20 +0000
+++ .bzrignore	2006-02-13 01:28:03 +0000
@@ -16,3 +16,7 @@
 test.log
 ./tmp
 build
+./ID
+./tags
+.bzr-shelf
+BRANCH-INFO

=== modified file '.rsyncexclude'
--- .rsyncexclude	2005-05-05 06:38:18 +0000
+++ .rsyncexclude	2005-11-22 04:30:14 +0000
@@ -15,3 +15,9 @@
 /tmp
 # do want this after all
 + CHANGELOG
+/build
+test*.tmp
+.*.swp
+*.orig
+.*.orig
+.bzr-shelf*

=== added file 'BRANCH.TODO'
--- BRANCH.TODO	1970-01-01 00:00:00 +0000
+++ BRANCH.TODO	2006-04-18 04:38:44 +0000
@@ -0,0 +1,3 @@
+# This file is for listing TODOs for branches that are being worked on.
+# It should ALWAYS be empty in the mainline or in integration branches.
+# 

=== modified file 'HACKING'
--- HACKING	2005-08-17 02:11:25 +0000
+++ HACKING	2006-04-06 10:13:27 +0000
@@ -1,16 +1,26 @@
 ============================
-guidelines for modifying bzr
+Guidelines for modifying bzr
 ============================
 
+.. contents::
+
+(The current version of this document is available in the file ``HACKING``
+in the source tree, or at http://bazaar-ng.org/hacking.html)
+
+Overall
+=======
+
 * New functionality should have test cases.  Preferably write the
   test before writing the code.
 
   In general, you can test at either the command-line level or the
-  internal API level.  Choose whichever is appropriate: if adding a
-  new command, or a new command option, then call through run_bzr().
-  It is not necessary to do both.
+  internal API level.  See Writing Tests below for more detail.
 
-* Before fixing a bug, write a test case so that it does not regress.
+* Try to practice Test-Driven Development.  before fixing a bug, write a
+  test case so that it does not regress.  Similarly for adding a new
+  feature: write a test case for a small version of the new feature before
+  starting on the code itself.  Check the test fails on the old code, then
+  add the feature or fix and check it passes.
 
 * Exceptions should be defined inside bzrlib.errors, so that we can
   see the whole tree at a glance.
@@ -20,14 +30,58 @@
   function runs.  Import statements have a cost, so try to make sure
   they don't run inside hot functions.
 
-* Please write PEP-8__ compliant code.
-
-__ http://www.python.org/peps/pep-0008.html
-
 * Module names should always be given fully-qualified,
   i.e. ``bzrlib.hashcache`` not just ``hashcache``.
 
-
+* Commands should return non-zero when they encounter circumstances that
+  the user should really pay attention to - which includes trivial shell
+  pipelines.
+
+  Recommended values are 
+    0- OK, 
+    1- Conflicts in merge-like operations, or changes are present in
+       diff-like operations. 
+    2- Unrepresentable diff changes (i.e. binary files that we cannot show 
+       a diff of).
+    3- An error or exception has occurred.
+
+Evolving interfaces
+-------------------
+
+We have a commitment to 6 months API stability - any supported symbol in a
+release of bzr MUST NOT be altered in any way that would result in
+breaking existing code that uses it. That means that method names,
+parameter ordering, parameter names, variable and attribute names etc must
+not be changed without leaving a 'deprecated forwarder' behind. This even
+applies to modules and classes.
+
+If you wish to change the behaviour of a supported API in an incompatible
+way, you need to change its name as well. For instance, if I add a optional keyword
+parameter to branch.commit - that's fine. On the other hand, if I add a
+keyword parameter to branch.commit which is a *required* transaction
+object, I should rename the API - i.e. to 'branch.commit_transaction'. 
+
+When renaming such supported API's, be sure to leave a deprecated_method (or
+_function or ...) behind which forwards to the new API. See the
+bzrlib.symbol_versioning module for decorators that take care of the
+details for you - such as updating the docstring, and issuing a warning
+when the old api is used.
+
+For unsupported API's, it does not hurt to follow this discipline, but its
+not required. Minimally though, please try to rename things so that
+callers will at least get an AttributeError rather than weird results.
+
+
+Standard parameter types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be check via 'bzrlib.osutils.safe_unicode'. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
 
 Documentation
 =============
@@ -35,9 +89,252 @@
 If you change the behaviour of a command, please update its docstring
 in bzrlib/commands.py.  This is displayed by the 'bzr help' command.
 
+NEWS file
+---------
+
 If you make a user-visible change, please add a note to the NEWS file.
 The description should be written to make sense to someone who's just
 a user of bzr, not a developer: new functions or classes shouldn't be
 mentioned, but new commands, changes in behaviour or fixed nontrivial
 bugs should be listed.  See the existing entries for an idea of what
 should be done.
+
+Within each release, entries in the news file should have the most
+user-visible changes first.  So the order should be approximately:
+
+ * changes to existing behaviour - the highest priority because the 
+   user's existing knowledge is incorrect
+ * new features - should be brought to their attention
+ * bug fixes - may be of interest if the bug was affecting them, and
+   should include the bug number if any
+ * major documentation changes
+ * changes to internal interfaces
+
+People who made significant contributions to each change are listed in
+parenthesis.  This can include reporting bugs (particularly with good
+details or reproduction recipes), submitting patches, etc.
+
+API documentation
+-----------------
+
+Functions, methods, classes and modules should have docstrings
+describing how they are used. 
+
+The first line of the docstring should be a self-contained sentence.
+
+For the special case of Command classes, this acts as the user-visible
+documentation shown by the help command.
+
+The docstrings should be formatted as reStructuredText_ (like this
+document), suitable for processing using the epydoc_ tool into HTML
+documentation.
+
+.. _reStructuredText: http://docutils.sourceforge.net/rst.html
+.. _epydoc: http://epydoc.sourceforge.net/
+
+
+
+Coding style
+============
+
+Please write PEP-8__ compliant code.  
+
+One often-missed requirement is that the first line of docstrings
+should be a self-contained one-sentence summary.
+
+__ http://www.python.org/peps/pep-0008.html
+
+
+
+Naming
+------
+
+Functions, methods or members that are in some sense "private" are given
+a leading underscore prefix.  This is just a hint that code outside the
+implementation should probably not use that interface.
+
+We prefer class names to be concatenated capital words (``TestCase``)
+and variables, methods and functions to be lowercase words joined by
+underscores (``revision_id``, ``get_revision``).
+
+For the purposes of naming some names are treated as single compound
+words: "filename", "revno".
+
+Consider naming classes as nouns and functions/methods as verbs.
+
+
+Standard names
+--------------
+
+``revision_id`` not ``rev_id`` or ``revid``
+
+Functions that transform one thing to another should be named ``x_to_y``
+(not ``x2y`` as occurs in some old code.)
+
+
+Destructors
+-----------
+
+Python destructors (``__del__``) work differently to those of other
+languages.  In particular, bear in mind that destructors may be called
+immediately when the object apparently becomes unreferenced, or at some
+later time, or possibly never at all.  Therefore we have restrictions on
+what can be done inside them.
+
+ 0. Never use a __del__ method without asking Martin/Robert first.
+
+ 1. Never rely on a ``__del__`` method running.  If there is code that
+    must run, do it from a ``finally`` block instead.
+
+ 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
+    interpreter!!
+
+ 3. In some places we raise a warning from the destructor if the object
+    has not been cleaned up or closed.  This is considered OK: the warning
+    may not catch every case but it's still useful sometimes.
+
+
+Writing output
+==============
+
+(The strategy described here is what we want to get to, but it's not
+consistently followed in the code at the moment.)
+
+bzrlib is intended to be a generically reusable library.  It shouldn't
+write messages to stdout or stderr, because some programs that use it
+might want to display that information through a GUI or some other
+mechanism.
+
+We can distinguish two types of output from the library:
+
+ 1. Structured data representing the progress or result of an
+    operation.  For example, for a commit command this will be a list
+    of the modified files and the finally committed revision number
+    and id.
+
+    These should be exposed either through the return code or by calls
+    to a callback parameter.
+
+    A special case of this is progress indicators for long-lived
+    operations, where the caller should pass a ProgressBar object.
+
+ 2. Unstructured log/debug messages, mostly for the benefit of the
+    developers or users trying to debug problems.  This should always
+    be sent through ``bzrlib.trace`` and Python ``logging``, so that
+    it can be redirected by the client.
+
+The distinction between the two is a bit subjective, but in general if
+there is any chance that a library would want to see something as
+structured data, we should make it so.
+
+The policy about how output is presented in the text-mode client
+should be only in the command-line tool.
+
+
+Writing tests
+=============
+In general tests should be placed in a file named test_FOO.py where 
+FOO is the logical thing under test. That file should be placed in the
+tests subdirectory under the package being tested.
+
+For example, tests for merge3 in bzrlib belong in bzrlib/tests/test_merge3.py.
+See bzrlib/selftest/test_sampler.py for a template test script.
+
+Tests can be written for the UI or for individual areas of the library.
+Choose whichever is appropriate: if adding a new command, or a new command 
+option, then you should be writing a UI test.  If you are both adding UI
+functionality and library functionality, you will want to write tests for 
+both the UI and the core behaviours.  We call UI tests 'blackbox' tests
+and they are found in bzrlib/tests/blackbox/*.py. 
+
+When writing blackbox tests please honour the following conventions:
+
+ 1. Place the tests for the command 'name' in
+    bzrlib/tests/blackbox/test_name.py. This makes it easy for developers
+    to locate the test script for a faulty command.
+
+ 2. Use the 'self.run_bzr("name")' utility function to invoke the command
+    rather than running bzr in a subprocess or invoking the
+    cmd_object.run() method directly. This is a lot faster than
+    subprocesses and generates the same logging output as running it in a
+    subprocess (which invoking the method directly does not).
+ 
+ 3. Only test the one command in a single test script. Use the bzrlib 
+    library when setting up tests and when evaluating the side-effects of
+    the command. We do this so that the library api has continual pressure
+    on it to be as functional as the command line in a simple manner, and
+    to isolate knock-on effects throughout the blackbox test suite when a
+    command changes it name or signature. Ideally only the tests for a
+    given command are affected when a given command is changed.
+
+Running tests
+=============
+Currently, bzr selftest is used to invoke tests.
+You can provide a pattern argument to run a subset. For example, 
+to run just the blackbox tests, run::
+
+  ./bzr selftest -v blackbox
+
+
+Errors and exceptions
+=====================
+
+Errors are handled through Python exceptions.  They can represent user
+errors, environmental errors or program bugs.  Sometimes we can't be sure
+at the time it's raised which case applies.  See bzrlib/errors.py for 
+details on the error-handling practices.
+
+
+Jargon
+======
+
+revno
+    Integer identifier for a revision on the main line of a branch.
+    Revision 0 is always the null revision; others are 1-based
+    indexes into the branch's revision history.
+
+
+Merge/review process
+====================
+
+If you'd like to propose a change, please post to the
+bazaar-ng@lists.canonical.com list with a patch, bzr changeset, or link to a
+branch.  Please put '[patch]' in the subject so we can pick them out, and
+include some text explaining the change.  Remember to put an update to the NEWS
+file in your diff, if it makes any changes visible to users or plugin
+developers.  Please include a diff against mainline if you're giving a link to
+a branch.
+
+Please indicate if you think the code is ready to merge, or if it's just a
+draft or for discussion.  If you want comments from many developers rather than
+to be merged, you can put '[rfc]' in the subject lines.
+
+Anyone is welcome to review code.  There are broadly three gates for
+code to get in:
+
+ * Doesn't reduce test coverage: if it adds new methods or commands,
+   there should be tests for them.  There is a good test framework
+   and plenty of examples to crib from, but if you are having trouble
+   working out how to test something feel free to post a draft patch
+   and ask for help.
+
+ * Doesn't reduce design clarity, such as by entangling objects
+   we're trying to separate.  This is mostly something the more
+   experienced reviewers need to help check.
+
+ * Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should pass all three.
+
+If you read a patch please reply and say so.  We can use a numeric scale
+of -1, -0, +0, +1, meaning respectively "really don't want it in current
+form", "somewhat uncomfortable", "ok with me", and "please put it in".
+Anyone can "vote".   (It's not really voting, just a terse expression.)
+
+If something gets say two +1 votes from core reviewers, and no
+vetos, then it's OK to come in.  Any of the core developers can bring it
+into their integration branch, which I'll merge regularly.  (If you do
+so, please reply and say so.)
+
+
+:: vim:tw=74:ai

=== added file 'INSTALL'
--- INSTALL	1970-01-01 00:00:00 +0000
+++ INSTALL	2005-10-19 07:04:01 +0000
@@ -0,0 +1,26 @@
+bzr install instructions
+************************
+
+Dependencies
+------------
+
+bzr requires Python2.4 and cElementTree:
+
+  http://effbot.org/zone/element-index.htm
+
+If you wish to access branches over sftp, you will need paramiko and
+pycrypto:
+
+  http://www.lag.net/paramiko/
+
+Installation
+------------
+
+To install bzr as a user, run
+
+  python setup.py install --home ~ 
+
+To install system-wide, run (as root)
+
+  python setup.py install
+

=== modified file 'Makefile'
--- Makefile	2005-08-23 04:40:35 +0000
+++ Makefile	2006-01-23 22:33:23 +0000
@@ -1,7 +1,25 @@
-all:
-	chmod u+x ./bzr
+all: 
 
 check:
-	./bzr selftest
+	./bzr selftest $(tests)
+	@echo "Running all tests with no locale."
+	LC_CTYPE= LANG=C LC_ALL= ./bzr selftest $(tests)
+
+check-msgeditor:
+	./bzr --no-plugins selftest -v msgeditor
+
+clean: 
+	./setup.py clean
+	-find . -name "*.pyc" -o -name "*.pyo" | xargs rm -f
+	rm -rf test????.tmp
 
 .PHONY: all
+
+
+# build emacs cross-reference
+tag_files=./bzr ./bzrlib/*py ./bzrlib/selftest/*.py
+TAGS: $(tag_files)
+	ctags-exuberant -e $(tag_files)
+
+tutorial.html: tutorial.txt
+	rest2html tutorial.txt > tutorial.html

=== modified file 'NEWS'
--- NEWS	2005-08-22 17:52:11 +0000
+++ NEWS	2006-04-18 22:41:16 +0000
@@ -1,16 +1,919 @@
-bzr development head
+bzr 0.8rc1  2006-04-14
+
+  IMPROVEMENTS:
+
+    * Sftp paths can now be relative, or local, according to the lftp
+      convention. Paths now take the form:
+      sftp://user:pass@host:port/~/relative/path
+      or
+      sftp://user:pass@host:port/absolute/path
+
+    * The FTP transport now tries to reconnect after a temporary
+      failure. ftp put is made atomic. (Matthieu Moy)
+
+    * The FTP transport now maintains a pool of connections, and
+      reuses them to avoid multiple connections to the same host (like
+      sftp did). (Daniel Silverstone)
+
+    * The bzr_man.py file has been removed. To create the man page now,
+      use ./generate_docs.py man. The new program can also create other files.
+      Run "python generate_docs.py --help" for usage information. (Hans
+      Ulrich Niedermann & James Blackwell).
+
+    * Man Page now gives full help (James Blackwell). Help also updated to 
+      reflect user config now being stored in .bazaar (Hans Ulrich
+      Niedermann)
+
+    * It's now possible to set aliases in bazaar.conf (Erik Bågfors)
+
+    * Pull now accepts a --revision argument (Erik Bågfors)
+
+    * 'bzr re-sign' now allows multiple revisions to be supplied on the command
+      line. You can now use the following command to sign all of your old commits.
+        find .bzr/revision-store// -name my@email-* \
+          | sed 's/.*\/\/..\///' \
+          | xargs bzr re-sign
+
+    * Upgrade can now upgrade over the network. (Robert Collins)
+
+    * Two new commands 'bzr checkout' and 'bzr update' allow for CVS/SVN-alike
+      behaviour.  By default they will cache history in the checkout, but
+      with --lightweight almost all data is kept in the master branch.
+      (Robert Collins)
+
+    * 'revert' unversions newly-versioned files, instead of deleting them.
+
+    * 'merge' is more robust.  Conflict messages have changed.
+
+    * 'merge' and 'revert' no longer clobber existing files that end in '~' or
+      '.moved'.
+
+    * Default log format can be set in configuration and plugins can register
+      their own formatters. (Erik Bågfors)
+
+    * New 'reconcile' command will check branch consistency and repair indexes
+      that can become out of sync in pre 0.8 formats. (Robert Collins,
+      Daniel Silverstone)
+
+    * New 'bzr init --format' and 'bzr upgrade --format' option to control 
+      what storage format is created or produced.  (Robert Collins, 
+      Martin Pool)
+
+    * Add parent location to 'bzr info', if there is one.  (Olaf Conradi)
+
+    * New developer commands 'weave-list' and 'weave-join'.  (Martin Pool)
+
+    * New 'init-repository' command, plus support for repositories in 'init'
+      and 'branch' (Aaron Bentley, Erik Bågfors, Robert Collins)
+
+    * Improve output of 'info' command. Show all relevant locations related to
+      working tree, branch and repository. Use kibibytes for binary quantities.
+      Fix off-by-one error in missing revisions of working tree.  Make 'info'
+      work on branches, repositories and remote locations.  (Olaf Conradi)
+
+    * 'pull' and 'push' now normalise the revision history, so that any two
+      branches with the same tip revision will have the same output from 'log'.
+      (Robert Collins)
+
+    * 'merge' accepts --remember option to store parent location, like 'push'
+      and 'pull'. (Olaf Conradi)
+
+    * 'Metadir' is now the default disk format. This improves behaviour in
+      SFTP using circumstances and allows binding and rebinding and easier
+      use of repositories. (Robert Collins, Aaron Bentley).
+
+  CHANGES:
+
+    * Commit is now verbose by default, and shows changed filenames and the 
+      new revision number.  (Robert Collins, Martin Pool)
+
+    * Unify 'mv', 'move', 'rename'.  (#5379, Matthew Fuller)
+
+    * 'bzr -h' shows help.  (#35940, Martin Pool, Ian Bicking)
+
+    * Make 'pull' and 'push' remember location on failure using --remember.
+      (Olaf Conradi)
+
+  INTERNALS:
+  
+    * The internal storage of history, and logical branch identity have now
+      been split into Branch, and Repository. The common locking and file 
+      management routines are now in bzrlib.lockablefiles. 
+      (Aaron Bentley, Robert Collins, Martin Pool)
+
+    * Transports can now raise DependencyNotPresent if they need a library
+      which is not installed, and then another implementation will be 
+      tried.  (Martin Pool)
+
+    * Remove obsolete (and no-op) `decode` parameter to `Transport.get`.  
+      (Martin Pool)
+
+    * Using Tree Transform for merge, revert, tree-building
+
+    * WorkingTree.create, Branch.create, WorkingTree.create_standalone,
+      Branch.initialize are now deprecated. Please see BzrDir.create_* for
+      replacement API's. (Robert Collins)
+
+    * New BzrDir class represents the .bzr control directory and manages
+      formatting issues. (Robert Collins)
+
+    * New repository.InterRepository class encapsulates Repository to 
+      Repository actions and allows for clean selection of optimised code
+      paths. (Robert Collins)
+
+    * bzrlib.fetch.fetch and bzrlib.fetch.greedy_fetch are now deprecated,
+      please use 'branch.fetch' or 'repository.fetch' depending on your
+      needs. (Robert Collins)
+
+    * deprecated methods now have a 'is_deprecated' flag on them that can
+      be checked, if you need to determine whether a given callable is 
+      deprecated at runtime. (Robert Collins)
+
+    * Progress bars are now nested - see
+      bzrlib.ui.ui_factory.nested_progress_bar. (Robert Collins, Robey Pointer)
+
+    * New API call get_format_description() for each type of format.
+      (Olaf Conradi)
+
+    * Changed branch.set_parent() to accept None to remove parent.
+      (Olaf Conradi)
+
+    * Deprecated BzrError AmbiguousBase.  (Olaf Conradi)
+
+  TESTING:
+
+    * SFTP tests now shortcut the SSH negotiation, reducing test overhead
+      for testing SFTP protocol support. (Robey Pointer)
+
+    * Branch formats are now tested once per implementation (see bzrlib.
+      tests.branch_implementations. This is analagous to the transport
+      interface tests, and has been followed up with working tree,
+      repository and BzrDir tests. (Robert Collins)
+
+    * New test base class TestCaseWithTransport provides a transport aware
+      test environment, useful for testing any transport-interface using
+      code. The test suite option --transport controls the transport used
+      by this class (when its not being used as part of implementation
+      contract testing). (Robert Collins)
+
+    * Close logging handler on disabling the test log. This will remove the
+      handler from the internal list inside python's logging module,
+      preventing shutdown from closing it twice.  (Olaf Conradi)
+
+    * Move test case for uncommit to blackbox tests.  (Olaf Conradi)
+
+bzr 0.7rc1 2006-01-09
+
+  CHANGES:
+
+    * .bzrignore is excluded from exports, on the grounds that it's a bzr 
+      internal-use file and may not be wanted.  (Jamie Wilkinson)
+
+    * The "bzr directories" command were removed in favor of the new
+      --kind option to the "bzr inventory" command.  To list all 
+      versioned directories, now use "bzr inventory --kind directory".  
+      (Johan Rydberg)
+
+    * Under Windows configuration directory is now %APPDATA%\bazaar\2.0
+      by default. (John Arbash Meinel)
+
+    * The parent of Bzr configuration directory can be set by BZR_HOME
+      environment variable. Now the path for it is searched in BZR_HOME, then
+      in HOME. Under Windows the order is: BZR_HOME, APPDATA (usually
+      points to C:\Documents and Settings\User Name\Application Data), HOME.
+      (John Arbash Meinel)
+
+    * Plugins with the same name in different directories in the bzr plugin
+      path are no longer loaded: only the first successfully loaded one is
+      used. (Robert Collins)
+
+    * Use systems' external ssh command to open connections if possible.  
+      This gives better integration with user settings such as ProxyCommand.
+      (James Henstridge)
+
+    * Permissions on files underneath .bzr/ are inherited from the .bzr 
+      directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
+      will mean that future file will be created with group write permissions.
+
+    * configure.in and config.guess are no longer in the builtin default 
+      ignore list.
+
+    * '.sw[nop]' pattern ignored, to ignore vim swap files for nameless
+      files.  (John Arbash Meinel, Martin Pool)
+
+  IMPROVEMENTS:
+
+    * "bzr INIT dir" now initializes the specified directory, and creates 
+      it if it does not exist.  (John Arbash Meinel)
+
+    * New remerge command (Aaron Bentley)
+
+    * Better zsh completion script.  (Steve Borho)
+
+    * 'bzr diff' now returns 1 when there are changes in the working 
+      tree. (Robert Collins)
+
+    * 'bzr push' now exists and can push changes to a remote location. 
+      This uses the transport infrastructure, and can store the remote
+      location in the ~/.bazaar/branches.conf configuration file.
+      (Robert Collins)
+
+    * Test directories are only kept if the test fails and the user requests
+      that they be kept.
+
+    * Tweaks to short log printing
+
+    * Added branch nicks, new nick command, printing them in log output. 
+      (Aaron Bentley)
+
+    * If $BZR_PDB is set, pop into the debugger when an uncaught exception 
+      occurs.  (Martin Pool)
+
+    * Accept 'bzr resolved' (an alias for 'bzr resolve'), as this is
+      the same as Subversion.  (Martin Pool)
+
+    * New ftp transport support (on ftplib), for ftp:// and aftp:// 
+      URLs.  (Daniel Silverstone)
+
+    * Commit editor temporary files now start with 'bzr_log.', to allow 
+      text editors to match the file name and set up appropriate modes or 
+      settings.  (Magnus Therning)
+
+    * Improved performance when integrating changes from a remote weave.  
+      (Goffredo Baroncelli)
+
+    * Sftp will attempt to cache the connection, so it is more likely that
+      a connection will be reused, rather than requiring multiple password
+      requests.
+
+    * bzr revno now takes an optional argument indicating the branch whose
+      revno should be printed.  (Michael Ellerman)
+
+    * bzr cat defaults to printing the last version of the file.  
+      (#3632, Matthieu Moy)
+
+    * New global option 'bzr --lsprof COMMAND' runs bzr under the lsprof 
+      profiler.  (Denys Duchier)
+
+    * Faster commits by reading only the headers of affected weave files. 
+      (Denys Duchier)
+
+    * 'bzr add' now takes a --dry-run parameter which shows you what would be
+      added, but doesn't actually add anything. (Michael Ellerman)
+
+    * 'bzr add' now lists how many files were ignored per glob.  add --verbose
+      lists the specific files.  (Aaron Bentley)
+
+    * 'bzr missing' now supports displaying changes in diverged trees and can
+      be limited to show what either end of the comparison is missing.
+      (Aaron Bently, with a little prompting from Daniel Silverstone)
+
+  BUG FIXES:
+
+    * SFTP can walk up to the root path without index errors. (Robert Collins)
+
+    * Fix bugs in running bzr with 'python -O'.  (Martin Pool)
+
+    * Error when run with -OO
+
+    * Fix bug in reporting http errors that don't have an http error code.
+      (Martin Pool)
+
+    * Handle more cases of pipe errors in display commands
+
+    * Change status to 3 for all errors
+
+    * Files that are added and unlinked before committing are completely
+      ignored by diff and status
+
+    * Stores with some compressed texts and some uncompressed texts are now
+      able to be used. (John A Meinel)
+
+    * Fix for bzr pull failing sometimes under windows
+
+    * Fix for sftp transport under windows when using interactive auth
+
+    * Show files which are both renamed and modified as such in 'bzr 
+      status' output.  (#4503, Daniel Silverstone)
+
+    * Make annotate cope better with revisions committed without a valid 
+      email address.  (Marien Zwart)
+
+    * Fix representation of tab characters in commit messages.  (Harald 
+      Meland)
+
+    * List of plugin directories in BZR_PLUGIN_PATH environment variable is
+      now parsed properly under Windows. (Alexander Belchenko)
+
+    * Show number of revisions pushed/pulled/merged. (Robey Pointer)
+
+    * Keep a cached copy of the basis inventory to speed up operations 
+      that need to refer to it.  (Johan Rydberg, Martin Pool)
+
+    * Fix bugs in bzr status display of non-ascii characters.  (Martin 
+      Pool)
+
+    * Remove Makefile.in from default ignore list.  (#6413, Tollef Fog 
+      Heen, Martin Pool)
+
+    * Fix failure in 'bzr added'.  (Nathan McCallum, Martin Pool)
+
+  TESTING:
+
+    * Fix selftest asking for passwords when there are no SFTP keys.  
+      (Robey Pointer, Jelmer Vernooij) 
+
+    * Fix selftest run with 'python -O'.  (Martin Pool)
+
+    * Fix HTTP tests under Windows. (John Arbash Meinel)
+
+    * Make tests work even if HOME is not set (Aaron Bentley)
+
+    * Updated build_tree to use fixed line-endings for tests which read 
+      the file cotents and compare. Make some tests use this to pass under
+      Windows. (John Arbash Meinel)
+
+    * Skip stat and symlink tests under Windows. (Alexander Belchenko)
+
+    * Delay in selftest/testhashcash is now issued under win32 and Cygwin.
+      (John Arbash Meinel)
+
+    * Use terminal width to align verbose test output.  (Martin Pool)
+
+    * Blackbox tests are maintained within the bzrlib.tests.blackbox directory.
+      If adding a new test script please add that to
+      bzrlib.tests.blackbox.__init__. (Robert Collins)
+
+    * Much better error message if one of the test suites can't be 
+      imported.  (Martin Pool)
+
+    * Make check now runs the test suite twice - once with the default locale,
+      and once with all locales forced to C, to expose bugs. This is not 
+      trivially done within python, so for now its only triggered by running
+      Make check. Integrators and packagers who wish to check for full 
+      platform support should run 'make check' to test the source.
+      (Robert Collins)
+
+    * Tests can now run TestSkipped if they can't execute for any reason.
+      (Martin Pool) (NB: TestSkipped should only be raised for correctable
+      reasons - see the wiki spec ImprovingBzrTestSuite).
+
+    * Test sftp with relative, absolute-in-homedir and absolute-not-in-homedir
+      paths for the transport tests. Introduce blackbox remote sftp tests that
+      test the same permutations. (Robert Collins, Robey Pointer)
+
+    * Transport implementation tests are now independent of the local file
+      system, which allows tests for esoteric transports, and for features
+      not available in the local file system. They also repeat for variations
+      on the URL scheme that can introduce issues in the transport code,
+      see bzrlib.transport.TransportTestProviderAdapter() for this.
+      (Robert Collins).
+
+    * TestCase.build_tree uses the transport interface to build trees, pass
+      in a transport parameter to give it an existing connection.
+      (Robert Collins).
+
+  INTERNALS:
+
+    * WorkingTree.pull has been split across Branch and WorkingTree,
+      to allow Branch only pulls. (Robert Collins)
+
+    * commands.display_command now returns the result of the decorated 
+      function. (Robert Collins)
+
+    * LocationConfig now has a set_user_option(key, value) call to save
+      a setting in its matching location section (a new one is created
+      if needed). (Robert Collins)
+
+    * Branch has two new methods, get_push_location and set_push_location
+      to respectively, get and set the push location. (Robert Collins)
+
+    * commands.register_command now takes an optional flag to signal that
+      the registrant is planning to decorate an existing command. When 
+      given multiple plugins registering a command is not an error, and
+      the original command class (whether built in or a plugin based one) is
+      returned to the caller. There is a new error 'MustUseDecorated' for
+      signalling when a wrapping command should switch to the original
+      version. (Robert Collins)
+
+    * Some option parsing errors will raise 'BzrOptionError', allowing 
+      granular detection for decorating commands. (Robert Collins).
+
+    * Branch.read_working_inventory has moved to
+      WorkingTree.read_working_inventory. This necessitated changes to
+      Branch.get_root_id, and a move of Branch.set_inventory to WorkingTree
+      as well. To make it clear that a WorkingTree cannot always be obtained
+      Branch.working_tree() will raise 'errors.NoWorkingTree' if one cannot
+      be obtained. (Robert Collins)
+
+    * All pending merges operations from Branch are now on WorkingTree.
+      (Robert Collins)
+
+    * The follow operations from Branch have moved to WorkingTree:
+      add()
+      commit()
+      move()
+      rename_one()
+      unknowns()
+      (Robert Collins)
+
+    * bzrlib.add.smart_add_branch is now smart_add_tree. (Robert Collins)
+
+    * New "rio" serialization format, similar to rfc-822. (Martin Pool)
+
+    * Rename selftests to `bzrlib.tests.test_foo`.  (John A Meinel, Martin 
+      Pool)
+
+    * bzrlib.plugin.all_plugins has been changed from an attribute to a 
+      query method. (Robert Collins)
+ 
+    * New options to read only the table-of-contents of a weave.  
+      (Denys Duchier)
+
+    * Raise NoSuchFile when someone tries to add a non-existant file.
+      (Michael Ellerman)
+
+    * Simplify handling of DivergedBranches in cmd_pull().
+      (Michael Ellerman)
+		   
+   
+    * Branch.controlfile* logic has moved to lockablefiles.LockableFiles, which
+      is exposed as Branch().control_files. Also this has been altered with the
+      controlfile pre/suffix replaced by simple method names like 'get' and
+      'put'. (Aaron Bentley, Robert Collins).
+
+    * Deprecated functions and methods can now be marked as such using the 
+      bzrlib.symbol_versioning module. Marked method have their docstring
+      updated and will issue a DeprecationWarning using the warnings module
+      when they are used. (Robert Collins)
+
+    * bzrlib.osutils.safe_unicode now exists to provide parameter coercion
+      for functions that need unicode strings. (Robert Collins)
+
+bzr 0.6 2005-10-28
+
+  IMPROVEMENTS:
+  
+    * pull now takes --verbose to show you what revisions are added or removed
+      (John A Meinel)
+
+    * merge now takes a --show-base option to include the base text in
+      conflicts.
+      (Aaron Bentley)
+
+    * The config files are now read using ConfigObj, so '=' should be used as
+      a separator, not ':'.
+      (Aaron Bentley)
+
+    * New 'bzr commit --strict' option refuses to commit if there are 
+      any unknown files in the tree.  To commit, make sure all files are 
+      either ignored, added, or deleted.  (Michael Ellerman)
+
+    * The config directory is now ~/.bazaar, and there is a single file 
+      ~/.bazaar/bazaar.conf storing email, editor and other preferences.
+      (Robert Collins)
+
+    * 'bzr add' no longer takes a --verbose option, and a --quiet option
+      has been added that suppresses all output.
+
+    * Improved zsh completion support in contrib/zsh, from Clint
+      Adams.
+
+    * Builtin 'bzr annotate' command, by Martin Pool with improvements from 
+      Goffredo Baroncelli.
+    
+    * 'bzr check' now accepts -v for verbose reporting, and checks for
+      ghosts in the branch. (Robert Collins)
+
+    * New command 're-sign' which will regenerate the gpg signature for 
+      a revision. (Robert Collins)
+
+    * If you set check_signatures=require for a path in 
+      ~/.bazaar/branches.conf then bzr will invoke your
+      gpg_signing_command (defaults to gpg) and record a digital signature
+      of your commit. (Robert Collins)
+
+    * New sftp transport, based on Paramiko.  (Robey Pointer)
+
+    * 'bzr pull' now accepts '--clobber' which will discard local changes
+      and make this branch identical to the source branch. (Robert Collins)
+
+    * Just give a quieter warning if a plugin can't be loaded, and 
+      put the details in .bzr.log.  (Martin Pool)
+
+    * 'bzr branch' will now set the branch-name to the last component of the
+      output directory, if one was supplied.
+
+    * If the option 'post_commit' is set to one (or more) python function
+      names (must be in the bzrlib namespace), then they will be invoked
+      after the commit has completed, with the branch and revision_id as
+      parameters. (Robert Collins)
+
+    * Merge now has a retcode of 1 when conflicts occur. (Robert Collins)
+
+    * --merge-type weave is now supported for file contents.  Tree-shape
+      changes are still three-way based.  (Martin Pool, Aaron Bentley)
+
+    * 'bzr check' allows the first revision on revision-history to have
+      parents - something that is expected for cheap checkouts, and occurs
+      when conversions from baz do not have all history.  (Robert Collins).
+
+   * 'bzr merge' can now graft unrelated trees together, if your specify
+     0 as a base. (Aaron Bentley)
+
+   * 'bzr commit branch' and 'bzr commit branch/file1 branch/file2' now work
+     (Aaron Bentley)
+
+    * Add '.sconsign*' to default ignore list.  (Alexander Belchenko)
+
+   * 'bzr merge --reprocess' minimizes conflicts
+
+  TESTING:
+
+    * The 'bzr selftest --pattern' option for has been removed, now 
+      test specifiers on the command line can be simple strings, or 
+      regexps, or both. (Robert Collins)
+
+    * Passing -v to selftest will now show the time each test took to 
+      complete, which will aid in analysing performance regressions and
+      related questions. (Robert Collins)
+
+    * 'bzr selftest' runs all tests, even if one fails, unless '--one'
+      is given. (Martin Pool)
+
+    * There is a new method for TestCaseInTempDir, assertFileEqual, which
+      will check that a given content is equal to the content of the named
+      file. (Robert Collins)
+
+    * Fix test suite's habit of leaving many temporary log files in $TMPDIR.
+      (Martin Pool)
+
+  INTERNALS:
+
+    * New 'testament' command and concept for making gpg-signatures 
+      of revisions that are not tied to a particular internal
+      representation.  (Martin Pool).
+
+    * Per-revision properties ('revprops') as key-value associated 
+      strings on each revision created when the revision is committed.
+      Intended mainly for the use of external tools.  (Martin Pool).
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      (Robert Collins)
+
+    * Improved command line option definitions allowing explanations
+      for individual options, among other things.  Contributed by 
+      Magnus Therning.
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      Configuration is now done via the config.Config interface:
+      Depending on whether you have a Branch, a Location or no information
+      available, construct a *Config, and use its signature_checking,
+      username and user_email methods. (Robert Collins)
+
+    * Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
+      they are made available for other plugins to use. You should not 
+      import other plugins during the __init__ of your plugin though, as 
+      no ordering is guaranteed, and the plugins directory is not on the
+      python path. (Robert Collins)
+
+    * Branch.relpath has been moved to WorkingTree.relpath. WorkingTree no
+      no longer takes an inventory, rather it takes an option branch
+      parameter, and if None is given will open the branch at basedir 
+      implicitly. (Robert Collins)
+
+    * Cleaner exception structure and error reporting.  Suggested by 
+      Scott James Remnant.  (Martin Pool)
+
+    * Branch.remove has been moved to WorkingTree, which has also gained
+      lock_read, lock_write and unlock methods for convenience. (Robert
+      Collins)
+
+    * Two decorators, needs_read_lock and needs_write_lock have been added
+      to the branch module. Use these to cause a function to run in a
+      read or write lock respectively. (Robert Collins)
+
+    * Branch.open_containing now returns a tuple (Branch, relative-path),
+      which allows direct access to the common case of 'get me this file
+      from its branch'. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'pull' has been factored out of the command as WorkingTree.pull().
+      A new option to WorkingTree.pull has been added, clobber, which will
+      ignore diverged history and pull anyway.
+      (Robert Collins)
+
+    * config.Config has a 'get_user_option' call that accepts an option name.
+      This will be looked up in branches.conf and bazaar.conf as normal.
+      It is intended that this be used by plugins to support options - 
+      options of built in programs should have specific methods on the config.
+      (Robert Collins)
+
+    * merge.merge_inner now has tempdir as an optional parameter. (Robert
+      Collins)
+
+    * Tree.kind is not recorded at the top level of the hierarchy, as it was
+      missing on EmptyTree, leading to a bug with merge on EmptyTrees.
+      (Robert Collins)
+
+    * WorkingTree.__del__ has been removed, it was non deterministic and not 
+      doing what it was intended to. See WorkingTree.__init__ for a comment
+      about future directions. (Robert Collins/Martin Pool)
+
+    * bzrlib.transport.http has been modified so that only 404 urllib errors
+      are returned as NoSuchFile. Other exceptions will propogate as normal.
+      This allows debuging of actual errors. (Robert Collins)
+
+    * bzrlib.transport.Transport now accepts *ONLY* url escaped relative paths
+      to apis like 'put', 'get' and 'has'. This is to provide consistent
+      behaviour - it operates on url's only. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'merge_flex' no longer calls conflict_handler.finalize(), instead that
+      is called by merge_inner. This is so that the conflict count can be 
+      retrieved (and potentially manipulated) before returning to the caller
+      of merge_inner. Likewise 'merge' now returns the conflict count to the
+      caller. (Robert Collins)
+
+    * 'revision.revision_graph can handle having only partial history for
+      a revision - that is no revisions in the graph with no parents.
+      (Robert Collins).
+
+    * New builtins.branch_files uses the standard file_list rules to produce
+      a branch and a list of paths, relative to that branch (Aaron Bentley)
+
+    * New TestCase.addCleanup facility.
+
+    * New bzrlib.version_info tuple (similar to sys.version_info), which can
+      be used by programs importing bzrlib.
+
+  BUG FIXES:
+
+    * Better handling of branches in directories with non-ascii names. 
+      (Joel Rosdahl, Panagiotis Papadakos)
+
+    * Upgrades of trees with no commits will not fail due to accessing
+      [-1] in the revision-history. (Andres Salomon)
+
+
+bzr 0.1.1 2005-10-12
+
+  BUG FIXES:
+
+    * Fix problem in pulling over http from machines that do not 
+      allow directories to be listed.
+
+    * Avoid harmless warning about invalid hash cache after 
+      upgrading branch format.
+
+  PERFORMANCE: 
+  
+    * Avoid some unnecessary http operations in branch and pull.
+
+
+bzr 0.1 2005-10-11
+
+  NOTES:
+
+    * 'bzr branch' over http initially gives a very high estimate
+      of completion time but it should fall as the first few 
+      revisions are pulled in.  branch is still slow on 
+      high-latency connections.
+
+  BUG FIXES:
+  
+    * bzr-man.py has been updated to work again. Contributed by
+      Rob Weir.
+
+    * Locking is now done with fcntl.lockf which works with NFS
+      file systems. Contributed by Harald Meland.
+
+    * When a merge encounters a file that has been deleted on
+      one side and modified on the other, the old contents are
+      written out to foo.BASE and foo.SIDE, where SIDE is this
+      or OTHER. Contributed by Aaron Bentley.
+
+    * Export was choosing incorrect file paths for the content of
+      the tarball, this has been fixed by Aaron Bentley.
+
+    * Commit will no longer commit without a log message, an 
+      error is returned instead. Contributed by Jelmer Vernooij.
+
+    * If you commit a specific file in a sub directory, any of its
+      parent directories that are added but not listed will be 
+      automatically included. Suggested by Michael Ellerman.
+
+    * bzr commit and upgrade did not correctly record new revisions
+      for files with only a change to their executable status.
+      bzr will correct this when it encounters it. Fixed by
+      Robert Collins
+
+    * HTTP tests now force off the use of http_proxy for the duration.
+      Contributed by Gustavo Niemeyer.
+
+    * Fix problems in merging weave-based branches that have 
+      different partial views of history.
+
+    * Symlink support: working with symlinks when not in the root of a 
+      bzr tree was broken, patch from Scott James Remnant.
+
+
+  IMPROVEMENTS:
+
+    * 'branch' now accepts a --basis parameter which will take advantage
+      of local history when making a new branch. This allows faster 
+      branching of remote branches. Contributed by Aaron Bentley.
+
+    * New tree format based on weave files, called version 5.
+      Existing branches can be upgraded to this format using 
+      'bzr upgrade'.
+
+    * Symlinks are now versionable. Initial patch by 
+      Erik Toubro Nielsen, updated to head by Robert Collins.
+
+    * Executable bits are tracked on files. Patch from Gustavo
+      Niemeyer.
+
+    * 'bzr status' now shows unknown files inside a selected directory.
+      Patch from Heikki Paajanen.
+
+    * Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
+      and 'resolve' have needed added, which list and remove those 
+      merge conflicts respectively. A conflicted tree cannot be committed
+      in. Contributed by Aaron Bentley.
+
+    * 'rm' is now an alias for 'remove'.
+
+    * Stores now split out their content in a single byte prefixed hash,
+      dropping the density of files per directory by 256. Contributed by
+      Gustavo Niemeyer.
+
+    * 'bzr diff -r branch:URL' will now perform a diff between two branches.
+      Contributed by Robert Collins.
+
+    * 'bzr log' with the default formatter will show merged revisions,
+      indented to the right. Initial implementation contributed by Gustavo
+      Niemeyer, made incremental by Robert Collins.
+
+
+  INTERNALS:
+
+    * Test case failures have the exception printed after the log 
+      for your viewing pleasure.
+
+    * InventoryEntry is now an abstract base class, use one of the
+      concrete InventoryDirectory etc classes instead.
+
+    * Branch raises an UnsupportedFormatError when it detects a 
+      bzr branch it cannot understand. This allows for precise
+      handling of such circumstances.
+
+
+  TESTING:
+
+    * Removed testsweet module so that tests can be run after 
+      bzr installed by 'bzr selftest'.
+
+    * 'bzr selftest' command-line arguments can now be partial ids
+      of tests to run, e.g. 'bzr selftest test_weave'
+
+      
+bzr 0.0.9 2005-09-23
+
+  BUG FIXES:
+
+    * Fixed "branch -r" option.
+
+    * Fix remote access to branches containing non-compressed history.
+      (Robert Collins).
+
+    * Better reliability of http server tests.  (John Arbash-Meinel)
+
+    * Merge graph maximum distance calculation fix.  (Aaron Bentley)
+   
+    * Various minor bug in windows support have been fixed, largely in the
+      test suite. Contributed by Alexander Belchenko.
+
+  IMPROVEMENTS:
+
+    * Status now accepts a -r argument to give status between chosen
+      revisions. Contributed by Heikki Paajanen.
+
+    * Revision arguments no longer use +/-/= to control ranges, instead
+      there is a 'before' namespace, which limits the successive namespace.
+      For example '$ bzr log -r date:yesterday..before:date:today' will
+      select everything from yesterday and before today. Contributed by
+      Robey Pointer
+
+    * There is now a bzr.bat file created by distutils when building on 
+      Windows. Contributed by Alexander Belchenko.
+
+  INTERNALS:
+
+    * Removed uuid() as it was unused.
+
+    * Improved 'fetch' code for pulling revisions from one branch into
+      another (used by pull, merged, etc.)
+
+
+bzr 0.0.8 2005-09-20
+
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+    * Ignore .DS_Store (contains Mac metadata) by default.  Patch from
+      Nir Soffer.
+
+    * If you set BZR_EDITOR in the environment, it is checked in
+      preference to EDITOR and the config file for the interactive commit
+      editing program. Related to this is a bugfix where a missing program
+      set in EDITOR would cause editing to fail, now the fallback program
+      for the operating system is still tried.
+
+    * Files that are not directories/symlinks/regular files will no longer
+      cause bzr to fail, it will just ignore them by default. You cannot add
+      them to the tree though - they are not versionable.
+
+
+  INTERNALS:
+
+    * Refactor xml packing/unpacking.
+
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+    * Make sure that the hashcache gets written out if it can't be
+      read.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+    * Quieten warnings about locking; patch from Matt Lavin.
+
+
+bzr-0.0.7 2005-09-02
 
   NEW FEATURES:
 
     * ``bzr shell-complete`` command contributed by Clint Adams to
       help with intelligent shell completion.
 
+    * New expert command ``bzr find-merge-base`` for debugging merges.
+
+
+  ENHANCEMENTS:
+
+    * Much better merge support.
+
+    * merge3 conflicts are now reported with markers like '<<<<<<<'
+      (seven characters) which is the same as CVS and pleases things
+      like emacs smerge.
+
 
   BUG FIXES:
 
     * ``bzr upgrade`` no longer fails when trying to fix trees that
       mention revisions that are not present.
 
+    * Fixed bugs in listing plugins from ``bzr plugins``.
+
+    * Fix case of $EDITOR containing options for the editor.
+
+    * Fix log -r refusing to show the last revision.
+      (Patch from Goffredo Baroncelli.)
+
+
+  CHANGES:
+
+    * ``bzr log --show-ids`` shows the revision ids of all parents.
+
+    * Externally provided commands on your $BZRPATH no longer need
+      to recognize --bzr-usage to work properly, and can just handle
+      --help themselves.
+
+
+  LIBRARY:
+
+    * Changed trace messages to go through the standard logging
+      framework, so that they can more easily be redirected by
+      libraries.
+
+
 
 bzr-0.0.6 2005-08-18
 
@@ -39,7 +942,7 @@
       specified by putting it into .bzr/email within a branch.  (Based
       on a patch from Heikki Paajanen.)
 
-    
+
   ENHANCEMENTS:
 
     * Faster working tree operations.

=== added file 'NEWS.developers'
--- NEWS.developers	1970-01-01 00:00:00 +0000
+++ NEWS.developers	2005-10-13 07:30:57 +0000
@@ -0,0 +1,25 @@
+DEVELOPMENT HEAD
+
+ * Properties can be set on Revisions by passing 'revprops=' to
+   Branch.commit(), and retrieved by looking at Revision.properties.
+
+bzr 0.1
+
+ * Remove RevisionReference class; Revision.parent_ids is now simply a
+   list of their ids and parent_sha1s is a list of their corresponding
+   sha1s (for old branches only at the moment.)
+
+ * New method-object style interface for Commit() and Fetch().
+
+ * Renamed Branch.last_patch() to Branch.last_revision(), since
+   we call them revisions not patches.
+
+ * Move copy_branch to bzrlib.clone.copy_branch.  The destination
+   directory is created if it doesn't exist.
+
+ * Inventories now identify the files which were present by 
+   giving the revision *of that file*.
+
+ * Inventory and Revision XML contains a version identifier.  
+   This must be consistent with the overall branch version
+   but allows for more flexibility in future upgrades.

=== modified file 'README'
--- README	2005-07-22 23:32:00 +0000
+++ README	2006-03-16 20:20:32 +0000
@@ -1,57 +1,32 @@
-*********************************
-Release notes for Bazaar-NG 0.0.5
-*********************************
-
-mbp@sourcefrog.net, May 2005, Canberra
-
-
-
-Caveats
--------
-
-
-
-* Don't use this for critical data; at the very least keep separate
-  regular snapshots of your tree.
-
-
-Dependencies
-------------
-
-This is mostly developed on Linux (Ubuntu); it should work on Unix,
-Windows, or OS X with relatively little trouble.
-
-The only dependency is Python 2.4.
-
-You may optionally install cElementTree to speed up some operations.
-This is strongly recommended if you are dealing with large trees.
-
-This package includes copies of two Python libraries, elementtree and
-urlgrabber.
-
-If you are building .deb or .rpm packages for bzr, please don't
-include these libraries but instead package them separately and depend
-on that.
-
-
-Installation
-------------
-
-The best way to install bzr is to symlink the ``bzr`` command onto a
-directory on your path.  For example::
-
-  ln -s ~/work/bzr/bzr ~/bin/bzr
-
-If you use a symlink for this, Python will be able to automatically
-find the bzr libraries.  Otherwise you must ensure they are listed on
-your $PYTHONPATH.
-
-After installing, please run the test suite to identify any problems
-on your platform::
-
-  ./bzr selftest
-
-If you use the setup.py script then bzr will be installed into the
-specified path.  In this case you must install ElementTree and
-urlgrabber separately.
-
+*******************************
+Release notes for Bazaar-NG 0.8
+*******************************
+
+bazaar-ng is a distributed/decentralized version control system.
+
+Features include:
+
+ * simple to use
+
+ * runs on Unix and Windows
+
+ * branches can be hosted on an http server with no special software
+
+ * full directory versioning including tracking subdirectories, 
+   file renames, symlinks and execute-bits
+   
+ * friendly interface that should be familiar to users of cvs, 
+   svn or other systems
+
+ * Python interface for extending and embedding, including a 
+   plugin interface
+
+ * strong integrity protection, including gpg-signing
+
+ * vibrant and active community
+
+For usage instructions see 
+http://bazaar.canonical.com/IntroductionToBzr
+or run "bzr help"
+
+For installation instructions see ./INSTALL

=== modified file 'TODO'
--- TODO	2005-08-17 03:31:19 +0000
+++ TODO	2006-01-08 23:34:37 +0000
@@ -17,14 +17,12 @@
 Small things
 ------------
 
-* ``bzr status`` should show some description of patches that are merged but 
-  not yet committed.  Preferably in a very compact format so that they
-  just fit on a single line; this could also be used for say ``log
-  --tiny``::
-
-  2005-11-05      mbp  Flowers for stephane
-
-* print a message at the end of running the tests telling them tht the
+* Fix tests so that import errors caused by modules don't produce false reports
+  that the tests themselves don't exist.
+
+* Fix tests so that one test failure doesn't prevent other tests from running
+
+* print a message at the end of running the tests telling them that the
   test log and output exists but can be removed
 
 * tests for running the commit editor, and fix problem of not passing in 
@@ -54,8 +52,6 @@
 
 * -r option should take a revision-id as well as a revno.
 
-* allow ``bzr st -r 300`` to show a summary of changes since then.
-
 * ``bzr info`` should count only people with distinct email addresses as
   different committers.  (Or perhaps only distinct userids?)
 
@@ -86,15 +82,6 @@
 * ``bzr inventory -r REV`` and perhaps unify this with ``bzr ls``,
   giving options to display ids, types, etc.
 
-* Split BzrError into various more specific subclasses for different
-  errors people might want to catch.
-
-* If the export destination ends in '.tar', '.tar.gz', etc then create
-  a tarball instead of a directory.  (Need to actually make a
-  temporary directory and then tar that up.)
-
-  http://www.gelato.unsw.edu.au/archives/git/0504/2194.html
-
 * RemoteBranch could maintain a cache either in memory or on disk.  We
   know more than an external cache might about which files are
   immutable and which can vary.  On the other hand, it's much simpler
@@ -140,10 +127,6 @@
   methods return object, but what we really want is the raw XML, which
   can be popped into our own store.  That needs to be refactored.
 
-* ``bzr status FOO`` where foo is ignored should say so.
-
-* ``bzr mkdir A...`` should just create and add A.
-
 * Guard against repeatedly merging any particular patch.
 
 * More options for diff:
@@ -164,8 +147,6 @@
 
   - and, of course, tests for all this
 
-* ``bzr add`` with no arguments should probably be the same as ``bzr add .``
-
 * Reproducible performance benchmark to measure whether performance is 
   getting better or worse.
 
@@ -201,12 +182,11 @@
 * sometimes gives "conflicting add" even when the contents are in fact
   the same???
 
+* BZRDIR should be in branch.py not __init__.py.
 
 Medium things
 -------------
 
-* ~/.bzr.log is not written anymore for some reason.
-
 * merge should add all revision and inventory XML to the local store.
 
 * check should give a warning for revisions that are named in the
@@ -219,9 +199,6 @@
   just get the id for the selected files, look up their location and
   diff just those files.  No need to traverse the entire inventories.
 
-* ``bzr status DIR`` or ``bzr diff DIR`` should report on all changes
-  under that directory.
-
 * Fix up Inventory objects to represent root object as an entry.
 
 * Don't convert entire entry from ElementTree to an object when it is
@@ -304,13 +281,6 @@
   - Class that describes the state of a working tree so we can just
     assert it's equal.
 
-* There are too many methods on Branch() that really manipulate the
-  WorkingTree.  They should be moved across.
-
-  Also there are some methods which are duplicated on Tree and
-  Inventory objects, and it should be made more clear which ones are
-  proxies and which ones behave differently, and how.
-
 * Try using XSLT to add some formatting to REST-generated HTML.  Or
   maybe write a small Python program that specifies a header and foot
   for the pages and calls into the docutils libraries.
@@ -332,7 +302,7 @@
 
   - Hold the ElementTree in memory in the Inventory object and work
     directly on that, rather than converting into Python objects every
-    time it is read in.  Probably still exposoe it through some kind of
+    time it is read in.  Probably still expose it through some kind of
     object interface though, but perhaps that should just be a proxy
     for the elements.
 
@@ -344,7 +314,7 @@
 
 * stat cache should perhaps only stat files as necessary, rather than
   doing them all up-front.  On the other hand, that disallows the
-  opimization of stating them in inode order.
+  optimization of stating them in inode order.
 
 * It'd be nice to pipeline multiple HTTP requests.  Often we can
   predict what will be wanted in future: all revisions, or all texts
@@ -355,8 +325,6 @@
 
 * Paranoid mode where we never trust SHA-1 matches.
 
-* Don't commit if there are no changes unless forced.
-
 * --dry-run mode for commit?  (Or maybe just run with
   check-command=false?)
 
@@ -368,14 +336,12 @@
 * XML attributes might have trouble with filenames containing \n and
   \r.  Do we really want to support this?  I think perhaps not.
 
-* Remember execute bits, so that exports will work OK.
-
 * Unify smart_add and plain Branch.add(); perhaps smart_add should
   just build a list of files to add and pass that to the regular add
   function.
 
 * Function to list a directory, saying in which revision each file was
-  last modified.  Useful for web and gui interfaces, and slow to
+  last modified.  Useful for web and GUI interfaces, and slow to
   compute one file at a time.
   
   This will be done when we track file texts by referring to the
@@ -386,40 +352,20 @@
 * Perhaps attempts to get locks should timeout after some period of
   time, or at least display a progress message.
 
-* Split out upgrade functionality from check command into a separate
-  ``bzr upgrade``.
-
 * Don't pass around command classes but rather pass objects.  This'd
   make it cleaner to construct objects wrapping external commands.
 
 * Track all merged-in revisions in a versioned add-only metafile.
 
-* ``pull --clobber`` should discard any local changes not present
-  remotely.  Not generally what you want, but possibly useful when
-  you're just mirroring another branch and want to keep tracking it
-  even when they e.g. uncommit or make similar non-forward movements.
-  Also for push I suppose.  Clobber may not be the best name, maybe
-  ``--destroy``?
-
 * ``uncommit`` command that removes a revision from the end of the
   revision-history; just doing this is enough to remove the commit,
   and a new commit will automatically be made against the
   predecessor.  This can be repeated.
 
-  It only makes sense to delete from the tail of history, not from the
-  end.
-
-  The revision, its inventory and texts remain floating in the store.
-  We should perhaps add the revision to a list of removed-commits, so
-  that it can be restored or at least accounted for when checking
-  consistency.  This file would not be versioned, and probably should
-  not propagate when branched.
-
-  If we track merged revisions then we need to update this list too.
-  If the list is stored in a weave it's easy (implicit): the version
-  of the list can remain but it won't be referenced anymore.  It's
-  probably best to just store this list in a weave in the first place
-  and be done.
+  It only makes sense to delete from the tail of history.
+
+  This has been implemented, but it does not remove the texts from
+  the store.
 
 
 Large things

=== modified file 'build-api'
--- build-api	2005-05-16 02:19:13 +0000
+++ build-api	2005-11-18 08:10:07 +0000
@@ -1,2 +1,3 @@
+#!/bin/bash
 mkdir -p api
 PYTHONPATH=$PWD epydoc -o api/html --docformat restructuredtext bzr bzrlib

=== modified file 'bzr' (properties changed: -x to +x)
--- bzr	2005-07-26 14:06:11 +0000
+++ bzr	2006-01-13 06:31:42 +0000
@@ -15,16 +15,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
+"""Bazaar-NG -- a free distributed version-control tool"""
 import os, sys
-
+if __doc__ is None:
+    print "bzr does not support python -OO."
+    sys.exit(2)
 try:
     version_info = sys.version_info
 except AttributeError:
     version_info = 1, 5 # 1.5 or older
 
-
-REINVOKE = "__BZR_REINVOKE"    
+REINVOKE = "__BZR_REINVOKE"
 NEED_VERS = (2, 4)
 KNOWN_PYTHONS = ('python2.4',)
 
@@ -40,9 +41,28 @@
     print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-os.unsetenv(REINVOKE)
-
-import bzrlib, bzrlib.commands
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
+
+try:
+    import bzrlib
+    import bzrlib.commands
+    import bzrlib.trace
+except ImportError, e:
+    sys.stderr.write("bzr: ERROR: "
+                     "Couldn't import bzrlib and dependencies.\n"
+                     "Please check bzrlib is on your PYTHONPATH.\n"
+                     "\n")
+    raise
+
+if bzrlib.version_info[:3] != (0, 8, 0):
+    sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
+            "This may indicate an installation problem.\n"
+            "bzrlib from %s is version %r\n"
+            % (bzrlib.__path__, bzrlib.version_info))
 
 if __name__ == '__main__':
+    bzrlib.trace.enable_default_logging()
     sys.exit(bzrlib.commands.main(sys.argv))
+else:
+    pass    # should this give an error? - it can't be used as a lib

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-08-22 14:15:16 +0000
+++ bzrlib/__init__.py	2006-04-14 01:26:52 +0000
@@ -16,44 +16,101 @@
 
 """bzr library"""
 
-# TODO: Do less imports here
-from branch import Branch, ScratchBranch, find_branch
-from errors import BzrError
-
-BZRDIR = ".bzr"
-
-DEFAULT_IGNORE = ['.bzr.log',
-                  '*~', '#*#', '*$', '.#*',
-                  '.*.sw[nop]', '.*.tmp',
-                  '*.tmp', '*.bak', '*.BAK', '*.orig',
-                  '*.o', '*.obj', '*.a', '*.py[oc]', '*.so', '*.exe', '*.elc', 
-                  '{arch}', 'CVS', 'CVS.adm', '.svn', '_darcs', 'SCCS', 'RCS',
+
+# please keep these sorted (in C locale order) to aid merging
+DEFAULT_IGNORE = [
+                  '#*#',
+                  '*$',
                   '*,v',
+                  '*.BAK',
+                  '*.a',
+                  '*.bak',
+                  '*.elc',
+                  '*.exe',
+                  '*.la',
+                  '*.lo',
+                  '*.o',
+                  '*.o',
+                  '*.obj',
+                  '*.orig',
+                  '*.py[oc]',
+                  '*.so',
+                  '*.tmp',
+                  '.*.tmp',
+                  '*~',
+                  '.#*',
+                  '.*.sw[nop]',
+                  '.sw[nop]',    # vim editing nameless file
+                  '.DS_Store',
+                  '.arch-ids',
+                  '.arch-inventory',
+                  '.bzr.log',
+                  '.del-*',
+                  '.git',
+                  '.jamdeps'
+                  '.libs',
+                  '.make.state',
+                  '.sconsign*',
+                  '.svn',
+                  '.tmp*',
                   'BitKeeper',
-                  '.git',
-                  'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*']
+                  'CVS',
+                  'CVS.adm',
+                  'RCS',
+                  'SCCS',
+                  'TAGS',
+                  '_darcs',
+                  'aclocal.m4',
+                  'autom4te*',
+                  'config.h',
+                  'config.h.in',
+                  'config.log',
+                  'config.status',
+                  'config.sub',
+                  'stamp-h',
+                  'stamp-h.in',
+                  'stamp-h1',
+                  '{arch}',
+                  ]
 
 IGNORE_FILENAME = ".bzrignore"
 
-import locale
+import os
+import sys
+if sys.platform == 'darwin':
+    # work around egregious python 2.4 bug
+    sys.platform = 'posix'
+    import locale
+    sys.platform = 'darwin'
+else:
+    import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 
-__copyright__ = "Copyright 2005 Canonical Development Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.0.7pre'
-
-
+__copyright__ = "Copyright 2005,06 Canonical Development Ltd."
+__version__ = version_string = '0.8rc1'
+# same format as sys.version_info
+version_info = (0, 8, 0, 'rc', 1)
+
+
+from bzrlib.symbol_versioning import deprecated_function, zero_seven
+
+@deprecated_function(zero_seven)
 def get_bzr_revision():
-    """If bzr is run from a branch, return (revno,revid) or None"""
+    """If bzr is run from a branch, return (revno,revid) or None."""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    
     try:
-        branch = Branch(__path__[0])
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]
         else:
             return None
-    except BzrError:
+    except bzrlib.errors.BzrError:
         return None
     
+def test_suite():
+    import tests
+    return tests.test_suite()

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-07-23 14:06:37 +0000
+++ bzrlib/add.py	2006-02-16 02:53:00 +0000
@@ -14,11 +14,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter, note
+from os.path import dirname
+
+import bzrlib.errors as errors
+from bzrlib.inventory import InventoryEntry
+from bzrlib.trace import mutter, note, warning
+from bzrlib.errors import NotBranchError
+from bzrlib.workingtree import is_control_file
+import bzrlib.osutils
+from bzrlib.workingtree import WorkingTree
 
 def glob_expand_for_win32(file_list):
+    if not file_list:
+        return
     import glob
-    
     expanded_file_list = []
     for possible_glob in file_list:
         glob_files = glob.glob(possible_glob)
@@ -31,79 +40,150 @@
             expanded_file_list += glob_files
     return expanded_file_list
 
-def smart_add(file_list, verbose=True, recurse=True):
-    """Add files to version, optionally recursing into directories.
 
-    This is designed more towards DWIM for humans than API simplicity.
-    For the specific behaviour see the help for cmd_add().
-    """
-    import os
+def _prepare_file_list(file_list):
+    """Prepare a file list for use by smart_add_*."""
     import sys
-    from bzrlib.osutils import quotefn, kind_marker
-    from bzrlib.errors import BadFileKindError, ForbiddenFileError
-    import bzrlib.branch
-    import bzrlib.osutils
-
     if sys.platform == 'win32':
         file_list = glob_expand_for_win32(file_list)
-        
     if not file_list:
-        file_list = ['.']
-    
+        file_list = [u'.']
+    file_list = list(file_list)
+    return file_list
+
+
+def add_action_null(inv, path, kind):
+    """Absorb add actions and do nothing."""
+    pass
+
+def add_action_print(inv, path, kind):
+    """Print a line to stdout for each file that would be added."""
+    print "added", bzrlib.osutils.quotefn(path)
+
+def add_action_add(inv, path, kind):
+    """Add each file to the given inventory. Produce no output."""
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+
+def add_action_add_and_print(inv, path, kind):
+    """Add each file to the given inventory, and print a line to stdout."""
+    add_action_add(inv, path, kind)
+    add_action_print(inv, path, kind)
+
+
+def smart_add(file_list, recurse=True, action=add_action_add):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    Returns the number of files added.
+    """
+    file_list = _prepare_file_list(file_list)
+    tree = WorkingTree.open_containing(file_list[0])[0]
+    return smart_add_tree(tree, file_list, recurse, action)
+
+def smart_add_tree(tree, file_list, recurse=True, action=add_action_add):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    This calls reporter with each (path, kind, file_id) of added files.
+
+    Returns the number of files added.
+    """
+    import os, errno
+    from bzrlib.errors import BadFileKindError, ForbiddenFileError
+    assert isinstance(recurse, bool)
+
+    file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
-    assert not isinstance(file_list, basestring)
-    b = bzrlib.branch.Branch(file_list[0], find_root=True)
-    inv = b.read_working_inventory()
-    tree = b.working_tree()
-    count = 0
+    inv = tree.read_working_inventory()
+    added = []
+    ignored = {}
 
     for f in file_list:
-        rf = b.relpath(f)
-        af = b.abspath(rf)
-
-        kind = bzrlib.osutils.file_kind(af)
-
-        if kind != 'file' and kind != 'directory':
+        rf = tree.relpath(f)
+        af = tree.abspath(rf)
+
+        try:
+            kind = bzrlib.osutils.file_kind(af)
+        except OSError, e:
+            if hasattr(e, 'errno') and e.errno == errno.ENOENT:
+                raise errors.NoSuchFile(rf)
+            raise
+
+        if not InventoryEntry.versionable_kind(kind):
             if f in user_list:
                 raise BadFileKindError("cannot add %s of type %s" % (f, kind))
             else:
-                print "skipping %s (can't add file of kind '%s')" % (f, kind)
+                warning("skipping %s (can't add file of kind '%s')", f, kind)
                 continue
 
-        mutter("smart add of %r, abs=%r" % (f, af))
+        mutter("smart add of %r, abs=%r", f, af)
         
-        if bzrlib.branch.is_control_file(af):
+        if tree.is_control_filename(af):
             raise ForbiddenFileError('cannot add control file %s' % f)
             
         versioned = (inv.path2id(rf) != None)
 
+        if kind == 'directory':
+            try:
+                sub_branch = WorkingTree.open(af)
+                sub_tree = True
+            except NotBranchError:
+                sub_tree = False
+            except errors.UnsupportedFormatError:
+                sub_tree = True
+        else:
+            sub_tree = False
+
         if rf == '':
-            mutter("branch root doesn't need to be added")
+            mutter("tree root doesn't need to be added")
+            sub_tree = False
         elif versioned:
-            mutter("%r is already versioned" % f)
+            mutter("%r is already versioned", f)
+        elif sub_tree:
+            mutter("%r is a bzr tree", f)
         else:
-            file_id = bzrlib.branch.gen_file_id(rf)
-            inv.add_path(rf, kind=kind, file_id=file_id)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, file_id))
-            count += 1 
-
-            print 'added', quotefn(f)
-
-        if kind == 'directory' and recurse:
+            added.extend(__add_one(tree, inv, rf, kind, action))
+
+        if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
-                subp = os.path.join(rf, subf)
-                if subf == bzrlib.BZRDIR:
-                    mutter("skip control directory %r" % subp)
-                elif tree.is_ignored(subp):
-                    mutter("skip ignored sub-file %r" % subp)
+                subp = bzrlib.osutils.pathjoin(rf, subf)
+                if tree.is_control_filename(subp):
+                    mutter("skip control directory %r", subp)
                 else:
-                    mutter("queue to add sub-file %r" % subp)
-                    file_list.append(b.abspath(subp))
-
-    if count > 0:
-        if verbose:
-            note('added %d' % count)
-        b._write_inventory(inv)
-    else:
-        print "nothing new to add"
-        # should this return 1 to the shell?
+                    ignore_glob = tree.is_ignored(subp)
+                    if ignore_glob is not None:
+                        mutter("skip ignored sub-file %r", subp)
+                        if ignore_glob not in ignored:
+                            ignored[ignore_glob] = []
+                        ignored[ignore_glob].append(subp)
+                    else:
+                        mutter("queue to add sub-file %r", subp)
+                        file_list.append(tree.abspath(subp))
+
+
+    mutter('added %d entries', len(added))
+    
+    if len(added) > 0:
+        tree._write_inventory(inv)
+
+    return added, ignored
+
+def __add_one(tree, inv, path, kind, action):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the tree root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return []
+
+    # add parent
+    added = __add_one(tree, inv, dirname(path), 'directory', action)
+    action(inv, path, kind)
+
+    return added + [path]

=== added file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/annotate.py	2006-02-22 10:35:05 +0000
@@ -0,0 +1,85 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""File annotate based on weave storage"""
+
+# TODO: Choice of more or less verbose formats:
+# 
+# interposed: show more details between blocks of modified lines
+
+# TODO: Show which revision caused a line to merge into the parent
+
+# TODO: perhaps abbreviate timescales depending on how recent they are
+# e.g. "3:12 Tue", "13 Oct", "Oct 2005", etc.  
+
+import sys
+import os
+import time
+
+import bzrlib.weave
+from bzrlib.config import extract_email_address
+from bzrlib.errors import BzrError
+
+
+def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
+        to_file=None):
+    if to_file is None:
+        to_file = sys.stdout
+
+    prevanno=''
+    annotation = list(_annotate_file(branch, rev_id, file_id))
+    max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
+    for (revno_str, author, date_str, line_rev_id, text ) in annotation:
+        if verbose:
+            anno = '%5s %-*s %8s ' % (revno_str, max_origin_len, author, date_str)
+        else:
+            anno = "%5s %-7s " % ( revno_str, author[:7] )
+
+        if anno.lstrip() == "" and full: anno = prevanno
+        print >>to_file, '%s| %s' % (anno, text)
+        prevanno=anno
+
+def _annotate_file(branch, rev_id, file_id ):
+
+    rh = branch.revision_history()
+    w = branch.repository.weave_store.get_weave(file_id, 
+        branch.repository.get_transaction())
+    last_origin = None
+    for origin, text in w.annotate_iter(rev_id):
+        text = text.rstrip('\r\n')
+        if origin == last_origin:
+            (revno_str, author, date_str) = ('','','')
+        else:
+            last_origin = origin
+            if not branch.repository.has_revision(origin):
+                (revno_str, author, date_str) = ('?','?','?')
+            else:
+                if origin in rh:
+                    revno_str = str(rh.index(origin) + 1)
+                else:
+                    revno_str = 'merge'
+            rev = branch.repository.get_revision(origin)
+            tz = rev.timezone or 0
+            date_str = time.strftime('%Y%m%d', 
+                                     time.gmtime(rev.timestamp + tz))
+            # a lazy way to get something like the email address
+            # TODO: Get real email address
+            author = rev.committer
+            try:
+                author = extract_email_address(author)
+            except BzrError:
+                pass        # use the whole name
+        yield (revno_str, author, date_str, origin, text)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-05-26 16:09:45 +0000
+++ bzrlib/atomicfile.py	2005-12-18 04:08:46 +0000
@@ -17,7 +17,8 @@
 
 
 from warnings import warn
-
+from bzrlib.osutils import rename
+import errno
 
 class AtomicFile(object):
     """A file that does an atomic-rename to move into place.
@@ -30,7 +31,7 @@
     An encoding can be specified; otherwise the default is ascii.
     """
 
-    def __init__(self, filename, mode='wb', encoding=None):
+    def __init__(self, filename, mode='wb', encoding=None, new_mode=None):
         if mode != 'wb' and mode != 'wt':
             raise ValueError("invalid AtomicFile mode %r" % mode)
 
@@ -47,6 +48,7 @@
 
         self.write = self.f.write
         self.closed = False
+        self._new_mode = new_mode
 
 
     def __repr__(self):
@@ -65,16 +67,16 @@
         self.f.close()
         self.f = None
         
-        if sys.platform == 'win32':
-            # windows cannot rename over an existing file
-            try:
-                os.remove(self.realfilename)
-            except OSError, e:
-                import errno
-                if e.errno != errno.ENOENT:
-                    raise
-                
-        os.rename(self.tmpfilename, self.realfilename)
+        try:
+            if self._new_mode is None:
+                self._new_mode = os.lstat(self.realfilename).st_mode
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        else:
+            os.chmod(self.tmpfilename, self._new_mode)
+
+        rename(self.tmpfilename, self.realfilename)
 
 
     def abort(self):
@@ -97,6 +99,6 @@
 
 
     def __del__(self):
-        if not self.closed:
+        if hasattr(self, 'closed') and not self.closed:
             warn("%r leaked" % self)
         

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-08-18 04:25:42 +0000
+++ bzrlib/branch.py	2006-04-13 22:54:24 +0000
@@ -1,140 +1,82 @@
-# Copyright (C) 2005 Canonical Ltd
-
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+from copy import deepcopy
+from cStringIO import StringIO
+import errno
+import os
+import shutil
 import sys
-import os
+from unittest import TestSuite
+from warnings import warn
 
 import bzrlib
+import bzrlib.bzrdir as bzrdir
+from bzrlib.config import TreeConfig
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.delta import compare_trees
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
+                           NoSuchRevision, HistoryMissing, NotBranchError,
+                           DivergedBranches, LockError,
+                           UninitializableFormat,
+                           UnlistableStore,
+                           UnlistableBranch, NoSuchFile, NotVersionedError,
+                           NoWorkingTree)
+import bzrlib.inventory as inventory
+from bzrlib.inventory import Inventory
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
+from bzrlib.osutils import (isdir, quotefn,
+                            rename, splitpath, sha_file,
+                            file_kind, abspath, normpath, pathjoin,
+                            safe_unicode,
+                            )
+from bzrlib.textui import show_status
 from bzrlib.trace import mutter, note
-from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, \
-     splitpath, \
-     sha_file, appendpath, file_kind
-from bzrlib.errors import BzrError, InvalidRevisionNumber, InvalidRevisionId
-import bzrlib.errors
-from bzrlib.textui import show_status
-from bzrlib.revision import Revision
-from bzrlib.xml import unpack_xml
-from bzrlib.delta import compare_trees
-from bzrlib.tree import EmptyTree, RevisionTree
-        
-BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
-## TODO: Maybe include checks for common corruption of newlines, etc?
-
+from bzrlib.tree import EmptyTree, RevisionTree
+from bzrlib.repository import Repository
+from bzrlib.revision import (
+                             get_intervening_revisions,
+                             is_ancestor,
+                             NULL_REVISION,
+                             Revision,
+                             )
+from bzrlib.store import copy_all
+from bzrlib.symbol_versioning import *
+import bzrlib.transactions as transactions
+from bzrlib.transport import Transport, get_transport
+from bzrlib.tree import EmptyTree, RevisionTree
+import bzrlib.ui
+import bzrlib.xml5
+
+
+BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
+BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
+BZR_BRANCH_FORMAT_6 = "Bazaar-NG branch, format 6\n"
+
+
+# TODO: Maybe include checks for common corruption of newlines, etc?
 
 # TODO: Some operations like log might retrieve the same revisions
 # repeatedly to calculate deltas.  We could perhaps have a weakref
-# cache in memory to make this faster.
-
-# TODO: please move the revision-string syntax stuff out of the branch
-# object; it's clutter
-
-
-def find_branch(f, **args):
-    if f and (f.startswith('http://') or f.startswith('https://')):
-        import remotebranch 
-        return remotebranch.RemoteBranch(f, **args)
-    else:
-        return Branch(f, **args)
-
-
-def find_cached_branch(f, cache_root, **args):
-    from remotebranch import RemoteBranch
-    br = find_branch(f, **args)
-    def cacheify(br, store_name):
-        from meta_store import CachedStore
-        cache_path = os.path.join(cache_root, store_name)
-        os.mkdir(cache_path)
-        new_store = CachedStore(getattr(br, store_name), cache_path)
-        setattr(br, store_name, new_store)
-
-    if isinstance(br, RemoteBranch):
-        cacheify(br, 'inventory_store')
-        cacheify(br, 'text_store')
-        cacheify(br, 'revision_store')
-    return br
-
-
-def _relpath(base, path):
-    """Return path relative to base, or raise exception.
-
-    The path may be either an absolute path or a path relative to the
-    current working directory.
-
-    Lifted out of Branch.relpath for ease of testing.
-
-    os.path.commonprefix (python2.4) has a bad bug that it works just
-    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
-    avoids that problem."""
-    rp = os.path.abspath(path)
-
-    s = []
-    head = rp
-    while len(head) >= len(base):
-        if head == base:
-            break
-        head, tail = os.path.split(head)
-        if tail:
-            s.insert(0, tail)
-    else:
-        from errors import NotBranchError
-        raise NotBranchError("path %r is not within branch %r" % (rp, base))
-
-    return os.sep.join(s)
-        
-
-def find_branch_root(f=None):
-    """Find the branch root enclosing f, or pwd.
-
-    f may be a filename or a URL.
-
-    It is not necessary that f exists.
-
-    Basically we keep looking up until we find the control directory or
-    run into the root.  If there isn't one, raises NotBranchError.
-    """
-    if f == None:
-        f = os.getcwd()
-    elif hasattr(os.path, 'realpath'):
-        f = os.path.realpath(f)
-    else:
-        f = os.path.abspath(f)
-    if not os.path.exists(f):
-        raise BzrError('%r does not exist' % f)
-        
-
-    orig_f = f
-
-    while True:
-        if os.path.exists(os.path.join(f, bzrlib.BZRDIR)):
-            return f
-        head, tail = os.path.split(f)
-        if head == f:
-            # reached the root, whatever that may be
-            raise bzrlib.errors.NotBranchError('%r is not in a branch' % orig_f)
-        f = head
-
-
-
-# XXX: move into bzrlib.errors; subclass BzrError    
-class DivergedBranches(Exception):
-    def __init__(self, branch1, branch2):
-        self.branch1 = branch1
-        self.branch2 = branch2
-        Exception.__init__(self, "These branches have diverged.")
+# cache in memory to make this faster.  In general anything can be
+# cached in memory between lock and unlock operations. .. nb thats
+# what the transaction identity map provides
 
 
 ######################################################################
@@ -144,612 +86,174 @@
     """Branch holding a history of revisions.
 
     base
-        Base directory of the branch.
-
-    _lock_mode
-        None, or 'r' or 'w'
-
-    _lock_count
-        If _lock_mode is true, a positive count of the number of times the
-        lock has been taken.
-
-    _lock
-        Lock object from bzrlib.lock.
+        Base directory/url of the branch.
     """
+    # this is really an instance variable - FIXME move it there
+    # - RBC 20060112
     base = None
-    _lock_mode = None
-    _lock_count = None
-    _lock = None
-    
-    # Map some sort of prefix into a namespace
-    # stuff like "revno:10", "revid:", etc.
-    # This should match a prefix with a function which accepts
-    REVISION_NAMESPACES = {}
-
-    def __init__(self, base, init=False, find_root=True):
-        """Create new branch object at a particular location.
-
-        base -- Base directory for the branch.
-        
-        init -- If True, create new control files in a previously
-             unversioned directory.  If False, the branch must already
-             be versioned.
-
-        find_root -- If true and init is false, find the root of the
-             existing branch containing base.
-
-        In the test suite, creation of new trees is tested using the
-        `ScratchBranch` class.
-        """
-        from bzrlib.store import ImmutableStore
-        if init:
-            self.base = os.path.realpath(base)
-            self._make_control()
-        elif find_root:
-            self.base = find_branch_root(base)
-        else:
-            self.base = os.path.realpath(base)
-            if not isdir(self.controlfilename('.')):
-                from errors import NotBranchError
-                raise NotBranchError("not a bzr branch: %s" % quotefn(base),
-                                     ['use "bzr init" to initialize a new working tree',
-                                      'current bzr can only operate from top-of-tree'])
-        self._check_format()
-
-        self.text_store = ImmutableStore(self.controlfilename('text-store'))
-        self.revision_store = ImmutableStore(self.controlfilename('revision-store'))
-        self.inventory_store = ImmutableStore(self.controlfilename('inventory-store'))
-
-
-    def __str__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.base)
-
-
-    __repr__ = __str__
-
-
-    def __del__(self):
-        if self._lock_mode or self._lock:
-            from warnings import warn
-            warn("branch %r was not explicitly unlocked" % self)
-            self._lock.unlock()
-
-
-
+
+    def __init__(self, *ignored, **ignored_too):
+        raise NotImplementedError('The Branch class is abstract')
+
+    @staticmethod
+    @deprecated_method(zero_eight)
+    def open_downlevel(base):
+        """Open a branch which may be of an old format."""
+        return Branch.open(base, _unsupported=True)
+        
+    @staticmethod
+    def open(base, _unsupported=False):
+        """Open the repository rooted at base.
+
+        For instance, if the repository is at URL/.bzr/repository,
+        Repository.open(URL) -> a Repository instance.
+        """
+        control = bzrdir.BzrDir.open(base, _unsupported)
+        return control.open_branch(_unsupported)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        This probes for a branch at url, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one and it is either an unrecognised format or an unsupported 
+        format, UnknownFormatError or UnsupportedFormatError are raised.
+        If there is one, it is returned, along with the unused portion of url.
+        """
+        control, relpath = bzrdir.BzrDir.open_containing(url)
+        return control.open_branch(), relpath
+
+    @staticmethod
+    @deprecated_function(zero_eight)
+    def initialize(base):
+        """Create a new working tree and branch, rooted at 'base' (url)
+
+        NOTE: This will soon be deprecated in favour of creation
+        through a BzrDir.
+        """
+        return bzrdir.BzrDir.create_standalone_workingtree(base).branch
+
+    def setup_caching(self, cache_root):
+        """Subclasses that care about caching should override this, and set
+        up cached stores located under cache_root.
+        """
+        # seems to be unused, 2006-01-13 mbp
+        warn('%s is deprecated' % self.setup_caching)
+        self.cache_root = cache_root
+
+    def _get_nick(self):
+        cfg = self.tree_config()
+        return cfg.get_option(u"nickname", default=self.base.split('/')[-2])
+
+    def _set_nick(self, nick):
+        cfg = self.tree_config()
+        cfg.set_option(nick, "nickname")
+        assert cfg.get_option("nickname") == nick
+
+    nick = property(_get_nick, _set_nick)
+        
     def lock_write(self):
-        if self._lock_mode:
-            if self._lock_mode != 'w':
-                from errors import LockError
-                raise LockError("can't upgrade to a write lock from %r" %
-                                self._lock_mode)
-            self._lock_count += 1
-        else:
-            from bzrlib.lock import WriteLock
-
-            self._lock = WriteLock(self.controlfilename('branch-lock'))
-            self._lock_mode = 'w'
-            self._lock_count = 1
-
-
-
+        raise NotImplementedError('lock_write is abstract')
+        
     def lock_read(self):
-        if self._lock_mode:
-            assert self._lock_mode in ('r', 'w'), \
-                   "invalid lock mode %r" % self._lock_mode
-            self._lock_count += 1
-        else:
-            from bzrlib.lock import ReadLock
-
-            self._lock = ReadLock(self.controlfilename('branch-lock'))
-            self._lock_mode = 'r'
-            self._lock_count = 1
-                        
-
-            
+        raise NotImplementedError('lock_read is abstract')
+
     def unlock(self):
-        if not self._lock_mode:
-            from errors import LockError
-            raise LockError('branch %r is not locked' % (self))
-
-        if self._lock_count > 1:
-            self._lock_count -= 1
-        else:
-            self._lock.unlock()
-            self._lock = None
-            self._lock_mode = self._lock_count = None
-
+        raise NotImplementedError('unlock is abstract')
+
+    def peek_lock_mode(self):
+        """Return lock mode for the Branch: 'r', 'w' or None"""
+        raise NotImplementedError(self.peek_lock_mode)
 
     def abspath(self, name):
-        """Return absolute filename for something in the branch"""
-        return os.path.join(self.base, name)
-
-
-    def relpath(self, path):
-        """Return path relative to this branch of something inside it.
-
-        Raises an error if path is not in this branch."""
-        return _relpath(self.base, path)
-
-
-    def controlfilename(self, file_or_path):
-        """Return location relative to branch."""
-        if isinstance(file_or_path, basestring):
-            file_or_path = [file_or_path]
-        return os.path.join(self.base, bzrlib.BZRDIR, *file_or_path)
-
-
-    def controlfile(self, file_or_path, mode='r'):
-        """Open a control file for this branch.
-
-        There are two classes of file in the control directory: text
-        and binary.  binary files are untranslated byte streams.  Text
-        control files are stored with Unix newlines and in UTF-8, even
-        if the platform or locale defaults are different.
-
-        Controlfiles should almost never be opened in write mode but
-        rather should be atomically copied and replaced using atomicfile.
-        """
-
-        fn = self.controlfilename(file_or_path)
-
-        if mode == 'rb' or mode == 'wb':
-            return file(fn, mode)
-        elif mode == 'r' or mode == 'w':
-            # open in binary mode anyhow so there's no newline translation;
-            # codecs uses line buffering by default; don't want that.
-            import codecs
-            return codecs.open(fn, mode + 'b', 'utf-8',
-                               buffering=60000)
+        """Return absolute filename for something in the branch
+        
+        XXX: Robert Collins 20051017 what is this used for? why is it a branch
+        method and not a tree method.
+        """
+        raise NotImplementedError('abspath is abstract')
+
+    def bind(self, other):
+        """Bind the local branch the other branch.
+
+        :param other: The branch to bind to
+        :type other: Branch
+        """
+        raise errors.UpgradeRequired(self.base)
+
+    @needs_write_lock
+    def fetch(self, from_branch, last_revision=None, pb=None):
+        """Copy revisions from from_branch into this branch.
+
+        :param from_branch: Where to copy from.
+        :param last_revision: What revision to stop at (None for at the end
+                              of the branch.
+        :param pb: An optional progress bar to use.
+
+        Returns the copied revision count and the failed revisions in a tuple:
+        (copied, failures).
+        """
+        if self.base == from_branch.base:
+            return (0, [])
+        if pb is None:
+            nested_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            pb = nested_pb
         else:
-            raise BzrError("invalid controlfile mode %r" % mode)
-
-
-
-    def _make_control(self):
-        from bzrlib.inventory import Inventory
-        from bzrlib.xml import pack_xml
+            nested_pb = None
+
+        from_branch.lock_read()
+        try:
+            if last_revision is None:
+                pb.update('get source history')
+                from_history = from_branch.revision_history()
+                if from_history:
+                    last_revision = from_history[-1]
+                else:
+                    # no history in the source branch
+                    last_revision = NULL_REVISION
+            return self.repository.fetch(from_branch.repository,
+                                         revision_id=last_revision,
+                                         pb=nested_pb)
+        finally:
+            if nested_pb is not None:
+                nested_pb.finished()
+            from_branch.unlock()
+
+    def get_bound_location(self):
+        """Return the URL of the branch we are bound to.
+
+        Older format branches cannot bind, please be sure to use a metadir
+        branch.
+        """
+        return None
+
+    def get_master_branch(self):
+        """Return the branch we are bound to.
         
-        os.mkdir(self.controlfilename([]))
-        self.controlfile('README', 'w').write(
-            "This is a Bazaar-NG control directory.\n"
-            "Do not change any files in this directory.\n")
-        self.controlfile('branch-format', 'w').write(BZR_BRANCH_FORMAT)
-        for d in ('text-store', 'inventory-store', 'revision-store'):
-            os.mkdir(self.controlfilename(d))
-        for f in ('revision-history', 'merged-patches',
-                  'pending-merged-patches', 'branch-name',
-                  'branch-lock',
-                  'pending-merges'):
-            self.controlfile(f, 'w').write('')
-        mutter('created control directory in ' + self.base)
-
-        # if we want per-tree root ids then this is the place to set
-        # them; they're not needed for now and so ommitted for
-        # simplicity.
-        pack_xml(Inventory(), self.controlfile('inventory','w'))
-
-
-    def _check_format(self):
-        """Check this branch format is supported.
-
-        The current tool only supports the current unstable format.
-
-        In the future, we might need different in-memory Branch
-        classes to support downlevel branches.  But not yet.
+        :return: Either a Branch, or None
         """
-        # This ignores newlines so that we can open branches created
-        # on Windows from Linux and so on.  I think it might be better
-        # to always make all internal files in unix format.
-        fmt = self.controlfile('branch-format', 'r').read()
-        fmt.replace('\r\n', '')
-        if fmt != BZR_BRANCH_FORMAT:
-            raise BzrError('sorry, branch format %r not supported' % fmt,
-                           ['use a different bzr version',
-                            'or remove the .bzr directory and "bzr init" again'])
+        return None
 
     def get_root_id(self):
         """Return the id of this branches root"""
-        inv = self.read_working_inventory()
-        return inv.root.file_id
-
-    def set_root_id(self, file_id):
-        inv = self.read_working_inventory()
-        orig_root_id = inv.root.file_id
-        del inv._byid[inv.root.file_id]
-        inv.root.file_id = file_id
-        inv._byid[inv.root.file_id] = inv.root
-        for fid in inv:
-            entry = inv[fid]
-            if entry.parent_id in (None, orig_root_id):
-                entry.parent_id = inv.root.file_id
-        self._write_inventory(inv)
-
-    def read_working_inventory(self):
-        """Read the working inventory."""
-        from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-        from time import time
-        before = time()
-        self.lock_read()
-        try:
-            # ElementTree does its own conversion from UTF-8, so open in
-            # binary.
-            inv = unpack_xml(Inventory,
-                             self.controlfile('inventory', 'rb'))
-            mutter("loaded inventory of %d items in %f"
-                   % (len(inv), time() - before))
-            return inv
-        finally:
-            self.unlock()
-            
-
-    def _write_inventory(self, inv):
-        """Update the working inventory.
-
-        That is to say, the inventory describing changes underway, that
-        will be committed to the next revision.
-        """
-        from bzrlib.atomicfile import AtomicFile
-        from bzrlib.xml import pack_xml
-        
-        self.lock_write()
-        try:
-            f = AtomicFile(self.controlfilename('inventory'), 'wb')
-            try:
-                pack_xml(inv, f)
-                f.commit()
-            finally:
-                f.close()
-        finally:
-            self.unlock()
-        
-        mutter('wrote working inventory')
-            
-
-    inventory = property(read_working_inventory, _write_inventory, None,
-                         """Inventory for the working copy.""")
-
-
-    def add(self, files, verbose=False, ids=None):
-        """Make files versioned.
-
-        Note that the command line normally calls smart_add instead.
-
-        This puts the files in the Added state, so that they will be
-        recorded by the next commit.
-
-        files
-            List of paths to add, relative to the base of the tree.
-
-        ids
-            If set, use these instead of automatically generated ids.
-            Must be the same length as the list of files, but may
-            contain None for ids that are to be autogenerated.
-
-        TODO: Perhaps have an option to add the ids even if the files do
-              not (yet) exist.
-
-        TODO: Perhaps return the ids of the files?  But then again it
-              is easy to retrieve them if they're needed.
-
-        TODO: Adding a directory should optionally recurse down and
-              add all non-ignored children.  Perhaps do that in a
-              higher-level method.
-        """
-        # TODO: Re-adding a file that is removed in the working copy
-        # should probably put it back with the previous ID.
-        if isinstance(files, basestring):
-            assert(ids is None or isinstance(ids, basestring))
-            files = [files]
-            if ids is not None:
-                ids = [ids]
-
-        if ids is None:
-            ids = [None] * len(files)
-        else:
-            assert(len(ids) == len(files))
-
-        self.lock_write()
-        try:
-            inv = self.read_working_inventory()
-            for f,file_id in zip(files, ids):
-                if is_control_file(f):
-                    raise BzrError("cannot add control file %s" % quotefn(f))
-
-                fp = splitpath(f)
-
-                if len(fp) == 0:
-                    raise BzrError("cannot add top-level %r" % f)
-
-                fullpath = os.path.normpath(self.abspath(f))
-
-                try:
-                    kind = file_kind(fullpath)
-                except OSError:
-                    # maybe something better?
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
-
-                if kind != 'file' and kind != 'directory':
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
-
-                if file_id is None:
-                    file_id = gen_file_id(f)
-                inv.add_path(f, kind=kind, file_id=file_id)
-
-                if verbose:
-                    print 'added', quotefn(f)
-
-                mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-            
-
-    def print_file(self, file, revno):
+        raise NotImplementedError('get_root_id is abstract')
+
+    def print_file(self, file, revision_id):
         """Print `file` to stdout."""
-        self.lock_read()
-        try:
-            tree = self.revision_tree(self.lookup_revision(revno))
-            # use inventory as it was in that revision
-            file_id = tree.inventory.path2id(file)
-            if not file_id:
-                raise BzrError("%r is not present in revision %s" % (file, revno))
-            tree.print_file(file_id)
-        finally:
-            self.unlock()
-
-
-    def remove(self, files, verbose=False):
-        """Mark nominated files for removal from the inventory.
-
-        This does not remove their text.  This does not run on 
-
-        TODO: Refuse to remove modified files unless --force is given?
-
-        TODO: Do something useful with directories.
-
-        TODO: Should this remove the text or not?  Tough call; not
-        removing may be useful and the user can just use use rm, and
-        is the opposite of add.  Removing it is consistent with most
-        other tools.  Maybe an option.
-        """
-        ## TODO: Normalize names
-        ## TODO: Remove nested loops; better scalability
-        if isinstance(files, basestring):
-            files = [files]
-
-        self.lock_write()
-
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-
-            # do this before any modifications
-            for f in files:
-                fid = inv.path2id(f)
-                if not fid:
-                    raise BzrError("cannot remove unversioned file %s" % quotefn(f))
-                mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
-                if verbose:
-                    # having remove it, it must be either ignored or unknown
-                    if tree.is_ignored(f):
-                        new_status = 'I'
-                    else:
-                        new_status = '?'
-                    show_status(new_status, inv[fid].kind, quotefn(f))
-                del inv[fid]
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
-    # FIXME: this doesn't need to be a branch method
-    def set_inventory(self, new_inventory_list):
-        from bzrlib.inventory import Inventory, InventoryEntry
-        inv = Inventory(self.get_root_id())
-        for path, file_id, parent, kind in new_inventory_list:
-            name = os.path.basename(path)
-            if name == "":
-                continue
-            inv.add(InventoryEntry(file_id, name, kind, parent))
-        self._write_inventory(inv)
-
-
-    def unknowns(self):
-        """Return all unknown files.
-
-        These are files in the working directory that are not versioned or
-        control files or ignored.
-        
-        >>> b = ScratchBranch(files=['foo', 'foo~'])
-        >>> list(b.unknowns())
-        ['foo']
-        >>> b.add('foo')
-        >>> list(b.unknowns())
-        []
-        >>> b.remove('foo')
-        >>> list(b.unknowns())
-        ['foo']
-        """
-        return self.working_tree().unknowns()
-
+        raise NotImplementedError('print_file is abstract')
 
     def append_revision(self, *revision_ids):
-        from bzrlib.atomicfile import AtomicFile
-
-        for revision_id in revision_ids:
-            mutter("add {%s} to revision-history" % revision_id)
-
-        rev_history = self.revision_history()
-        rev_history.extend(revision_ids)
-
-        f = AtomicFile(self.controlfilename('revision-history'))
-        try:
-            for rev_id in rev_history:
-                print >>f, rev_id
-            f.commit()
-        finally:
-            f.close()
-
-
-    def get_revision_xml(self, revision_id):
-        """Return XML file object for revision object."""
-        if not revision_id or not isinstance(revision_id, basestring):
-            raise InvalidRevisionId(revision_id)
-
-        self.lock_read()
-        try:
-            try:
-                return self.revision_store[revision_id]
-            except IndexError:
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        finally:
-            self.unlock()
-
-
-    def get_revision(self, revision_id):
-        """Return the Revision object for a named revision"""
-        xml_file = self.get_revision_xml(revision_id)
-
-        try:
-            r = unpack_xml(Revision, xml_file)
-        except SyntaxError, e:
-            raise bzrlib.errors.BzrError('failed to unpack revision_xml',
-                                         [revision_id,
-                                          str(e)])
-            
-        assert r.revision_id == revision_id
-        return r
-
-
-    def get_revision_delta(self, revno):
-        """Return the delta for one revision.
-
-        The delta is relative to its mainline predecessor, or the
-        empty tree for revision 1.
-        """
-        assert isinstance(revno, int)
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
-            raise InvalidRevisionNumber(revno)
-
-        # revno is 1-based; list is 0-based
-
-        new_tree = self.revision_tree(rh[revno-1])
-        if revno == 1:
-            old_tree = EmptyTree()
-        else:
-            old_tree = self.revision_tree(rh[revno-2])
-
-        return compare_trees(old_tree, new_tree)
-
-        
-
-    def get_revision_sha1(self, revision_id):
-        """Hash the stored value of a revision, and return it."""
-        # In the future, revision entries will be signed. At that
-        # point, it is probably best *not* to include the signature
-        # in the revision hash. Because that lets you re-sign
-        # the revision, (add signatures/remove signatures) and still
-        # have all hash pointers stay consistent.
-        # But for now, just hash the contents.
-        return bzrlib.osutils.sha_file(self.get_revision_xml(revision_id))
-
-
-    def get_inventory(self, inventory_id):
-        """Get Inventory object by hash.
-
-        TODO: Perhaps for this and similar methods, take a revision
-               parameter which can be either an integer revno or a
-               string hash."""
-        from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-
-        return unpack_xml(Inventory, self.get_inventory_xml(inventory_id))
-
-
-    def get_inventory_xml(self, inventory_id):
-        """Get inventory XML as a file object."""
-        return self.inventory_store[inventory_id]
-            
-
-    def get_inventory_sha1(self, inventory_id):
-        """Return the sha1 hash of the inventory entry
-        """
-        return sha_file(self.get_inventory_xml(inventory_id))
-
-
-    def get_revision_inventory(self, revision_id):
-        """Return inventory of a past revision."""
-        # bzr 0.0.6 imposes the constraint that the inventory_id
-        # must be the same as its revision, so this is trivial.
-        if revision_id == None:
-            from bzrlib.inventory import Inventory
-            return Inventory(self.get_root_id())
-        else:
-            return self.get_inventory(revision_id)
-
+        raise NotImplementedError('append_revision is abstract')
+
+    def set_revision_history(self, rev_history):
+        raise NotImplementedError('set_revision_history is abstract')
 
     def revision_history(self):
-        """Return sequence of revision hashes on to this branch.
-
-        >>> ScratchBranch().revision_history()
-        []
-        """
-        self.lock_read()
-        try:
-            return [l.rstrip('\r\n') for l in
-                    self.controlfile('revision-history', 'r').readlines()]
-        finally:
-            self.unlock()
-
-
-    def common_ancestor(self, other, self_revno=None, other_revno=None):
-        """
-        >>> import commit
-        >>> sb = ScratchBranch(files=['foo', 'foo~'])
-        >>> sb.common_ancestor(sb) == (None, None)
-        True
-        >>> commit.commit(sb, "Committing first revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        1
-        >>> clone = sb.clone()
-        >>> commit.commit(sb, "Committing second revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        2
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> commit.commit(clone, "Committing divergent second revision", 
-        ...               verbose=False)
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> sb.common_ancestor(clone) == clone.common_ancestor(sb)
-        True
-        >>> sb.common_ancestor(sb) != clone.common_ancestor(clone)
-        True
-        >>> clone2 = sb.clone()
-        >>> sb.common_ancestor(clone2)[0]
-        2
-        >>> sb.common_ancestor(clone2, self_revno=1)[0]
-        1
-        >>> sb.common_ancestor(clone2, other_revno=1)[0]
-        1
-        """
-        my_history = self.revision_history()
-        other_history = other.revision_history()
-        if self_revno is None:
-            self_revno = len(my_history)
-        if other_revno is None:
-            other_revno = len(other_history)
-        indices = range(min((self_revno, other_revno)))
-        indices.reverse()
-        for r in indices:
-            if my_history[r] == other_history[r]:
-                return r+1, my_history[r]
-        return None, None
-
+        """Return sequence of revision hashes on to this branch."""
+        raise NotImplementedError('revision_history is abstract')
 
     def revno(self):
         """Return current revision number for this branch.
@@ -759,43 +263,49 @@
         """
         return len(self.revision_history())
 
+    def unbind(self):
+        """Older format branches cannot bind or unbind."""
+        raise errors.UpgradeRequired(self.base)
 
-    def last_patch(self):
-        """Return last patch hash, or None if no history.
-        """
+    def last_revision(self):
+        """Return last patch hash, or None if no history."""
         ph = self.revision_history()
         if ph:
             return ph[-1]
         else:
             return None
 
-
     def missing_revisions(self, other, stop_revision=None):
-        """
+        """Return a list of new revisions that would perfectly fit.
+        
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
 
-        >>> from bzrlib.commit import commit
+        >>> from bzrlib.workingtree import WorkingTree
         >>> bzrlib.trace.silent = True
-        >>> br1 = ScratchBranch()
-        >>> br2 = ScratchBranch()
+        >>> d1 = bzrdir.ScratchDir()
+        >>> br1 = d1.open_branch()
+        >>> wt1 = d1.open_workingtree()
+        >>> d2 = bzrdir.ScratchDir()
+        >>> br2 = d2.open_branch()
+        >>> wt2 = d2.open_workingtree()
         >>> br1.missing_revisions(br2)
         []
-        >>> commit(br2, "lala!", rev_id="REVISION-ID-1")
+        >>> wt2.commit("lala!", rev_id="REVISION-ID-1")
         >>> br1.missing_revisions(br2)
         [u'REVISION-ID-1']
         >>> br2.missing_revisions(br1)
         []
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-1")
+        >>> wt1.commit("lala!", rev_id="REVISION-ID-1")
         >>> br1.missing_revisions(br2)
         []
-        >>> commit(br2, "lala!", rev_id="REVISION-ID-2A")
+        >>> wt2.commit("lala!", rev_id="REVISION-ID-2A")
         >>> br1.missing_revisions(br2)
         [u'REVISION-ID-2A']
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-2B")
+        >>> wt1.commit("lala!", rev_id="REVISION-ID-2B")
         >>> br1.missing_revisions(br2)
         Traceback (most recent call last):
-        DivergedBranches: These branches have diverged.
+        DivergedBranches: These branches have diverged.  Try merge.
         """
         self_history = self.revision_history()
         self_len = len(self_history)
@@ -808,326 +318,57 @@
 
         if stop_revision is None:
             stop_revision = other_len
-        elif stop_revision > other_len:
-            raise NoSuchRevision(self, stop_revision)
-        
+        else:
+            assert isinstance(stop_revision, int)
+            if stop_revision > other_len:
+                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
-
     def update_revisions(self, other, stop_revision=None):
-        """Pull in all new revisions from other branch.
-        
-        >>> from bzrlib.commit import commit
-        >>> bzrlib.trace.silent = True
-        >>> br1 = ScratchBranch(files=['foo', 'bar'])
-        >>> br1.add('foo')
-        >>> br1.add('bar')
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
-        >>> br2 = ScratchBranch()
-        >>> br2.update_revisions(br1)
-        Added 2 texts.
-        Added 1 inventories.
-        Added 1 revisions.
-        >>> br2.revision_history()
-        [u'REVISION-ID-1']
-        >>> br2.update_revisions(br1)
-        Added 0 texts.
-        Added 0 inventories.
-        Added 0 revisions.
-        >>> br1.text_store.total_size() == br2.text_store.total_size()
-        True
-        """
-        from bzrlib.progress import ProgressBar
-
-        pb = ProgressBar()
-
-        pb.update('comparing histories')
-        revision_ids = self.missing_revisions(other, stop_revision)
-
-        if hasattr(other.revision_store, "prefetch"):
-            other.revision_store.prefetch(revision_ids)
-        if hasattr(other.inventory_store, "prefetch"):
-            inventory_ids = [other.get_revision(r).inventory_id
-                             for r in revision_ids]
-            other.inventory_store.prefetch(inventory_ids)
-                
-        revisions = []
-        needed_texts = set()
-        i = 0
-        for rev_id in revision_ids:
-            i += 1
-            pb.update('fetching revision', i, len(revision_ids))
-            rev = other.get_revision(rev_id)
-            revisions.append(rev)
-            inv = other.get_inventory(str(rev.inventory_id))
-            for key, entry in inv.iter_entries():
-                if entry.text_id is None:
-                    continue
-                if entry.text_id not in self.text_store:
-                    needed_texts.add(entry.text_id)
-
-        pb.clear()
-                    
-        count = self.text_store.copy_multi(other.text_store, needed_texts)
-        print "Added %d texts." % count 
-        inventory_ids = [ f.inventory_id for f in revisions ]
-        count = self.inventory_store.copy_multi(other.inventory_store, 
-                                                inventory_ids)
-        print "Added %d inventories." % count 
-        revision_ids = [ f.revision_id for f in revisions]
-        count = self.revision_store.copy_multi(other.revision_store, 
-                                               revision_ids)
-        for revision_id in revision_ids:
-            self.append_revision(revision_id)
-        print "Added %d revisions." % count
-                    
-        
-    def commit(self, *args, **kw):
-        from bzrlib.commit import commit
-        commit(self, *args, **kw)
-        
-
-    def lookup_revision(self, revision):
-        """Return the revision identifier for a given revision information."""
-        revno, info = self.get_revision_info(revision)
-        return info
-
-    def get_revision_info(self, revision):
-        """Return (revno, revision id) for revision identifier.
-
-        revision can be an integer, in which case it is assumed to be revno (though
-            this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something like
-            'date:' or 'revid:' etc.
-        """
-        if revision is None:
-            return 0, None
-        revno = None
-        try:# Convert to int if possible
-            revision = int(revision)
-        except ValueError:
-            pass
-        revs = self.revision_history()
-        if isinstance(revision, int):
-            if revision == 0:
-                return 0, None
-            # Mabye we should do this first, but we don't need it if revision == 0
-            if revision < 0:
-                revno = len(revs) + revision + 1
-            else:
-                revno = revision
-        elif isinstance(revision, basestring):
-            for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
-                if revision.startswith(prefix):
-                    revno = func(self, revs, revision)
-                    break
-            else:
-                raise BzrError('No namespace registered for string: %r' % revision)
-
-        if revno is None or revno <= 0 or revno > len(revs):
-            raise BzrError("no such revision %s" % revision)
-        return revno, revs[revno-1]
-
-    def _namespace_revno(self, revs, revision):
-        """Lookup a revision by revision number"""
-        assert revision.startswith('revno:')
-        try:
-            return int(revision[6:])
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revno:'] = _namespace_revno
-
-    def _namespace_revid(self, revs, revision):
-        assert revision.startswith('revid:')
-        try:
-            return revs.index(revision[6:]) + 1
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revid:'] = _namespace_revid
-
-    def _namespace_last(self, revs, revision):
-        assert revision.startswith('last:')
-        try:
-            offset = int(revision[5:])
-        except ValueError:
-            return None
-        else:
-            if offset <= 0:
-                raise BzrError('You must supply a positive value for --revision last:XXX')
-            return len(revs) - offset + 1
-    REVISION_NAMESPACES['last:'] = _namespace_last
-
-    def _namespace_tag(self, revs, revision):
-        assert revision.startswith('tag:')
-        raise BzrError('tag: namespace registered, but not implemented.')
-    REVISION_NAMESPACES['tag:'] = _namespace_tag
-
-    def _namespace_date(self, revs, revision):
-        assert revision.startswith('date:')
-        import datetime
-        # Spec for date revisions:
-        #   date:value
-        #   value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
-        #   it can also start with a '+/-/='. '+' says match the first
-        #   entry after the given date. '-' is match the first entry before the date
-        #   '=' is match the first entry after, but still on the given date.
-        #
-        #   +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
-        #   -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
-        #   =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
-        #       May 13th, 2005 at 0:00
-        #
-        #   So the proper way of saying 'give me all entries for today' is:
-        #       -r {date:+today}:{date:-tomorrow}
-        #   The default is '=' when not supplied
-        val = revision[5:]
-        match_style = '='
-        if val[:1] in ('+', '-', '='):
-            match_style = val[:1]
-            val = val[1:]
-
-        today = datetime.datetime.today().replace(hour=0,minute=0,second=0,microsecond=0)
-        if val.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif val.lower() == 'today':
-            dt = today
-        elif val.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
-        else:
-            import re
-            # This should be done outside the function to avoid recompiling it.
-            _date_re = re.compile(
-                    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-                    r'(,|T)?\s*'
-                    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-                )
-            m = _date_re.match(val)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise BzrError('Invalid revision date %r' % revision)
-
-            if m.group('date'):
-                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
-            else:
-                year, month, day = today.year, today.month, today.day
-            if m.group('time'):
-                hour = int(m.group('hour'))
-                minute = int(m.group('minute'))
-                if m.group('second'):
-                    second = int(m.group('second'))
-                else:
-                    second = 0
-            else:
-                hour, minute, second = 0,0,0
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                    hour=hour, minute=minute, second=second)
-        first = dt
-        last = None
-        reversed = False
-        if match_style == '-':
-            reversed = True
-        elif match_style == '=':
-            last = dt + datetime.timedelta(days=1)
-
-        if reversed:
-            for i in range(len(revs)-1, -1, -1):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first >= dt and (last is None or dt >= last):
-                    return i+1
-        else:
-            for i in range(len(revs)):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first <= dt and (last is None or dt <= last):
-                    return i+1
-    REVISION_NAMESPACES['date:'] = _namespace_date
-
-    def revision_tree(self, revision_id):
-        """Return Tree for a revision on this branch.
-
-        `revision_id` may be None for the null revision, in which case
-        an `EmptyTree` is returned."""
-        # TODO: refactor this to use an existing revision object
-        # so we don't need to read it in twice.
-        if revision_id == None:
-            return EmptyTree()
-        else:
-            inv = self.get_revision_inventory(revision_id)
-            return RevisionTree(self.text_store, inv)
-
-
-    def working_tree(self):
-        """Return a `Tree` for the working copy."""
-        from workingtree import WorkingTree
-        return WorkingTree(self.base, self.read_working_inventory())
-
+        """Pull in new perfect-fit revisions.
+
+        :param other: Another Branch to pull from
+        :param stop_revision: Updated until the given revision
+        :return: None
+        """
+        raise NotImplementedError('update_revisions is abstract')
+
+    def revision_id_to_revno(self, revision_id):
+        """Given a revision id, return its revno"""
+        if revision_id is None:
+            return 0
+        history = self.revision_history()
+        try:
+            return history.index(revision_id) + 1
+        except ValueError:
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
+    def get_rev_id(self, revno, history=None):
+        """Find the revision id of the specified revno."""
+        if revno == 0:
+            return None
+        if history is None:
+            history = self.revision_history()
+        elif revno <= 0 or revno > len(history):
+            raise bzrlib.errors.NoSuchRevision(self, revno)
+        return history[revno - 1]
+
+    def pull(self, source, overwrite=False, stop_revision=None):
+        raise NotImplementedError('pull is abstract')
 
     def basis_tree(self):
         """Return `Tree` object for last revision.
 
         If there are no revisions yet, return an `EmptyTree`.
         """
-        r = self.last_patch()
-        if r == None:
-            return EmptyTree()
-        else:
-            return RevisionTree(self.text_store, self.get_revision_inventory(r))
-
-
+        return self.repository.revision_tree(self.last_revision())
 
     def rename_one(self, from_rel, to_rel):
         """Rename one file.
 
         This can change the directory or the filename or both.
         """
-        self.lock_write()
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-            if not tree.has_filename(from_rel):
-                raise BzrError("can't rename: old working file %r does not exist" % from_rel)
-            if tree.has_filename(to_rel):
-                raise BzrError("can't rename: new working file %r already exists" % to_rel)
-
-            file_id = inv.path2id(from_rel)
-            if file_id == None:
-                raise BzrError("can't rename: old name %r is not versioned" % from_rel)
-
-            if inv.path2id(to_rel):
-                raise BzrError("can't rename: new name %r is already versioned" % to_rel)
-
-            to_dir, to_tail = os.path.split(to_rel)
-            to_dir_id = inv.path2id(to_dir)
-            if to_dir_id == None and to_dir != '':
-                raise BzrError("can't determine destination directory id for %r" % to_dir)
-
-            mutter("rename_one:")
-            mutter("  file_id    {%s}" % file_id)
-            mutter("  from_rel   %r" % from_rel)
-            mutter("  to_rel     %r" % to_rel)
-            mutter("  to_dir     %r" % to_dir)
-            mutter("  to_dir_id  {%s}" % to_dir_id)
-
-            inv.rename(file_id, to_dir_id, to_tail)
-
-            print "%s => %s" % (from_rel, to_rel)
-
-            from_abs = self.abspath(from_rel)
-            to_abs = self.abspath(to_rel)
-            try:
-                os.rename(from_abs, to_abs)
-            except OSError, e:
-                raise BzrError("failed to rename %r to %r: %s"
-                        % (from_abs, to_abs, e[1]),
-                        ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
+        raise NotImplementedError('rename_one is abstract')
 
     def move(self, from_paths, to_name):
         """Rename files.
@@ -1139,265 +380,963 @@
 
         Note that to_name is only the last component of the new name;
         this doesn't change the directory.
-        """
-        self.lock_write()
-        try:
-            ## TODO: Option to move IDs only
-            assert not isinstance(from_paths, basestring)
-            tree = self.working_tree()
-            inv = tree.inventory
-            to_abs = self.abspath(to_name)
-            if not isdir(to_abs):
-                raise BzrError("destination %r is not a directory" % to_abs)
-            if not tree.has_filename(to_name):
-                raise BzrError("destination %r not in working directory" % to_abs)
-            to_dir_id = inv.path2id(to_name)
-            if to_dir_id == None and to_name != '':
-                raise BzrError("destination %r is not a versioned directory" % to_name)
-            to_dir_ie = inv[to_dir_id]
-            if to_dir_ie.kind not in ('directory', 'root_directory'):
-                raise BzrError("destination %r is not a directory" % to_abs)
-
-            to_idpath = inv.get_idpath(to_dir_id)
-
-            for f in from_paths:
-                if not tree.has_filename(f):
-                    raise BzrError("%r does not exist in working tree" % f)
-                f_id = inv.path2id(f)
-                if f_id == None:
-                    raise BzrError("%r is not versioned" % f)
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                if tree.has_filename(dest_path):
-                    raise BzrError("destination %r already exists" % dest_path)
-                if f_id in to_idpath:
-                    raise BzrError("can't move %r to a subdirectory of itself" % f)
-
-            # OK, so there's a race here, it's possible that someone will
-            # create a file in this interval and then the rename might be
-            # left half-done.  But we should have caught most problems.
-
-            for f in from_paths:
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                print "%s => %s" % (f, dest_path)
-                inv.rename(inv.path2id(f), to_dir_id, name_tail)
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
+        """
+        raise NotImplementedError('move is abstract')
+
+    def get_parent(self):
+        """Return the parent location of the branch.
+
+        This is the default location for push/pull/missing.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        raise NotImplementedError('get_parent is abstract')
+
+    def get_push_location(self):
+        """Return the None or the location to push this branch to."""
+        raise NotImplementedError('get_push_location is abstract')
+
+    def set_push_location(self, location):
+        """Set a new push location for this branch."""
+        raise NotImplementedError('set_push_location is abstract')
+
+    def set_parent(self, url):
+        raise NotImplementedError('set_parent is abstract')
+
+    @needs_write_lock
+    def update(self):
+        """Synchronise this branch with the master branch if any. 
+
+        :return: None or the last_revision pivoted out during the update.
+        """
+        return None
+
+    def check_revno(self, revno):
+        """\
+        Check whether a revno corresponds to any revision.
+        Zero (the NULL revision) is considered valid.
+        """
+        if revno != 0:
+            self.check_real_revno(revno)
+            
+    def check_real_revno(self, revno):
+        """\
+        Check whether a revno corresponds to a real revision.
+        Zero (the NULL revision) is considered invalid
+        """
+        if revno < 1 or revno > self.revno():
+            raise InvalidRevisionNumber(revno)
+
+    @needs_read_lock
+    def clone(self, *args, **kwargs):
+        """Clone this branch into to_bzrdir preserving all semantic values.
+        
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        # for API compatability, until 0.8 releases we provide the old api:
+        # def clone(self, to_location, revision=None, basis_branch=None, to_branch_format=None):
+        # after 0.8 releases, the *args and **kwargs should be changed:
+        # def clone(self, to_bzrdir, revision_id=None):
+        if (kwargs.get('to_location', None) or
+            kwargs.get('revision', None) or
+            kwargs.get('basis_branch', None) or
+            (len(args) and isinstance(args[0], basestring))):
+            # backwards compatability api:
+            warn("Branch.clone() has been deprecated for BzrDir.clone() from"
+                 " bzrlib 0.8.", DeprecationWarning, stacklevel=3)
+            # get basis_branch
+            if len(args) > 2:
+                basis_branch = args[2]
+            else:
+                basis_branch = kwargs.get('basis_branch', None)
+            if basis_branch:
+                basis = basis_branch.bzrdir
+            else:
+                basis = None
+            # get revision
+            if len(args) > 1:
+                revision_id = args[1]
+            else:
+                revision_id = kwargs.get('revision', None)
+            # get location
+            if len(args):
+                url = args[0]
+            else:
+                # no default to raise if not provided.
+                url = kwargs.get('to_location')
+            return self.bzrdir.clone(url,
+                                     revision_id=revision_id,
+                                     basis=basis).open_branch()
+        # new cleaner api.
+        # generate args by hand 
+        if len(args) > 1:
+            revision_id = args[1]
+        else:
+            revision_id = kwargs.get('revision_id', None)
+        if len(args):
+            to_bzrdir = args[0]
+        else:
+            # no default to raise if not provided.
+            to_bzrdir = kwargs.get('to_bzrdir')
+        result = self._format.initialize(to_bzrdir)
+        self.copy_content_into(result, revision_id=revision_id)
+        return  result
+
+    @needs_read_lock
+    def sprout(self, to_bzrdir, revision_id=None):
+        """Create a new line of development from the branch, into to_bzrdir.
+        
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        result = self._format.initialize(to_bzrdir)
+        self.copy_content_into(result, revision_id=revision_id)
+        result.set_parent(self.bzrdir.root_transport.base)
+        return result
+
+    @needs_read_lock
+    def copy_content_into(self, destination, revision_id=None):
+        """Copy the content of self into destination.
+
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        new_history = self.revision_history()
+        if revision_id is not None:
+            try:
+                new_history = new_history[:new_history.index(revision_id) + 1]
+            except ValueError:
+                rev = self.repository.get_revision(revision_id)
+                new_history = rev.get_history(self.repository)[1:]
+        destination.set_revision_history(new_history)
+        parent = self.get_parent()
+        if parent:
+            destination.set_parent(parent)
+
+
+class BranchFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in an dict by their format string for reference 
+    during branch opening. Its not required that these be instances, they
+    can be classes themselves with class methods - it simply depends on 
+    whether state is needed for a given format or not.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every time regardless.
+    """
+
+    _default_format = None
+    """The default format used for new branches."""
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def find_format(klass, a_bzrdir):
+        """Return the format for the branch object in a_bzrdir."""
+        try:
+            transport = a_bzrdir.get_branch_transport(None)
+            format_string = transport.get("format").read()
+            return klass._formats[format_string]
+        except NoSuchFile:
+            raise NotBranchError(path=transport.base)
+        except KeyError:
+            raise errors.UnknownFormatError(format_string)
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def get_format_description(self):
+        """Return the short format description for this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def initialize(self, a_bzrdir):
+        """Create a branch of this format in a_bzrdir."""
+        raise NotImplementedError(self.initialized)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats can be initialized and opened.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the branch object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        raise NotImplementedError(self.open)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+    def __str__(self):
+        return self.get_format_string().rstrip()
+
+
+class BzrBranchFormat4(BranchFormat):
+    """Bzr branch format 4.
+
+    This format has:
+     - a revision-history file.
+     - a branch-lock lock file [ to be shared with the bzrdir ]
+    """
+
+    def get_format_description(self):
+        """See BranchFormat.get_format_description()."""
+        return "Branch format 4"
+
+    def initialize(self, a_bzrdir):
+        """Create a branch of this format in a_bzrdir."""
+        mutter('creating branch in %s', a_bzrdir.transport.base)
+        branch_transport = a_bzrdir.get_branch_transport(self)
+        utf8_files = [('revision-history', ''),
+                      ('branch-name', ''),
+                      ]
+        control_files = LockableFiles(branch_transport, 'branch-lock',
+                                      TransportLock)
+        control_files.create_lock()
+        control_files.lock_write()
+        try:
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(a_bzrdir, _found=True)
+
+    def __init__(self):
+        super(BzrBranchFormat4, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirFormat6()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the branch object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        return BzrBranch(_format=self,
+                         _control_files=a_bzrdir._control_files,
+                         a_bzrdir=a_bzrdir,
+                         _repository=a_bzrdir.open_repository())
+
+    def __str__(self):
+        return "Bazaar-NG branch format 4"
+
+
+class BzrBranchFormat5(BranchFormat):
+    """Bzr branch format 5.
+
+    This format has:
+     - a revision-history file.
+     - a format string
+     - a lock dir guarding the branch itself
+     - all of this stored in a branch/ subdirectory
+     - works with shared repositories.
+
+    This format is new in bzr 0.8.
+    """
+
+    def get_format_string(self):
+        """See BranchFormat.get_format_string()."""
+        return "Bazaar-NG branch format 5\n"
+
+    def get_format_description(self):
+        """See BranchFormat.get_format_description()."""
+        return "Branch format 5"
+        
+    def initialize(self, a_bzrdir):
+        """Create a branch of this format in a_bzrdir."""
+        mutter('creating branch %r in %s', self, a_bzrdir.transport.base)
+        branch_transport = a_bzrdir.get_branch_transport(self)
+        utf8_files = [('revision-history', ''),
+                      ('branch-name', ''),
+                      ]
+        control_files = LockableFiles(branch_transport, 'lock', LockDir)
+        control_files.create_lock()
+        control_files.lock_write()
+        control_files.put_utf8('format', self.get_format_string())
+        try:
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(a_bzrdir, _found=True, )
+
+    def __init__(self):
+        super(BzrBranchFormat5, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the branch object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        if not _found:
+            format = BranchFormat.find_format(a_bzrdir)
+            assert format.__class__ == self.__class__
+        transport = a_bzrdir.get_branch_transport(None)
+        control_files = LockableFiles(transport, 'lock', LockDir)
+        return BzrBranch5(_format=self,
+                          _control_files=control_files,
+                          a_bzrdir=a_bzrdir,
+                          _repository=a_bzrdir.find_repository())
+
+    def __str__(self):
+        return "Bazaar-NG Metadir branch format 5"
+
+
+class BranchReferenceFormat(BranchFormat):
+    """Bzr branch reference format.
+
+    Branch references are used in implementing checkouts, they
+    act as an alias to the real branch which is at some other url.
+
+    This format has:
+     - A location file
+     - a format string
+    """
+
+    def get_format_string(self):
+        """See BranchFormat.get_format_string()."""
+        return "Bazaar-NG Branch Reference Format 1\n"
+
+    def get_format_description(self):
+        """See BranchFormat.get_format_description()."""
+        return "Checkout reference format 1"
+        
+    def initialize(self, a_bzrdir, target_branch=None):
+        """Create a branch of this format in a_bzrdir."""
+        if target_branch is None:
+            # this format does not implement branch itself, thus the implicit
+            # creation contract must see it as uninitializable
+            raise errors.UninitializableFormat(self)
+        mutter('creating branch reference in %s', a_bzrdir.transport.base)
+        branch_transport = a_bzrdir.get_branch_transport(self)
+        # FIXME rbc 20060209 one j-a-ms encoding branch lands this str() cast is not needed.
+        branch_transport.put('location', StringIO(str(target_branch.bzrdir.root_transport.base)))
+        branch_transport.put('format', StringIO(self.get_format_string()))
+        return self.open(a_bzrdir, _found=True)
+
+    def __init__(self):
+        super(BranchReferenceFormat, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
+
+    def _make_reference_clone_function(format, a_branch):
+        """Create a clone() routine for a branch dynamically."""
+        def clone(to_bzrdir, revision_id=None):
+            """See Branch.clone()."""
+            return format.initialize(to_bzrdir, a_branch)
+            # cannot obey revision_id limits when cloning a reference ...
+            # FIXME RBC 20060210 either nuke revision_id for clone, or
+            # emit some sort of warning/error to the caller ?!
+        return clone
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the branch that the branch reference in a_bzrdir points at.
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        if not _found:
+            format = BranchFormat.find_format(a_bzrdir)
+            assert format.__class__ == self.__class__
+        transport = a_bzrdir.get_branch_transport(None)
+        real_bzrdir = bzrdir.BzrDir.open(transport.get('location').read())
+        result = real_bzrdir.open_branch()
+        # this changes the behaviour of result.clone to create a new reference
+        # rather than a copy of the content of the branch.
+        # I did not use a proxy object because that needs much more extensive
+        # testing, and we are only changing one behaviour at the moment.
+        # If we decide to alter more behaviours - i.e. the implicit nickname
+        # then this should be refactored to introduce a tested proxy branch
+        # and a subclass of that for use in overriding clone() and ....
+        # - RBC 20060210
+        result.clone = self._make_reference_clone_function(result)
+        return result
+
+
+# formats which have no format string are not discoverable
+# and not independently creatable, so are not registered.
+__default_format = BzrBranchFormat5()
+BranchFormat.register_format(__default_format)
+BranchFormat.register_format(BranchReferenceFormat())
+BranchFormat.set_default_format(__default_format)
+_legacy_formats = [BzrBranchFormat4(),
+                   ]
+
+class BzrBranch(Branch):
+    """A branch stored in the actual filesystem.
+
+    Note that it's "local" in the context of the filesystem; it doesn't
+    really matter if it's on an nfs/smb/afs/coda/... share, as long as
+    it's writable, and can be accessed via the normal filesystem API.
+    """
+    
+    def __init__(self, transport=DEPRECATED_PARAMETER, init=DEPRECATED_PARAMETER,
+                 relax_version_check=DEPRECATED_PARAMETER, _format=None,
+                 _control_files=None, a_bzrdir=None, _repository=None):
+        """Create new branch object at a particular location.
+
+        transport -- A Transport object, defining how to access files.
+        
+        init -- If True, create new control files in a previously
+             unversioned directory.  If False, the branch must already
+             be versioned.
+
+        relax_version_check -- If true, the usual check for the branch
+            version is not applied.  This is intended only for
+            upgrade/recovery type use; it's not guaranteed that
+            all operations will work on old format branches.
+        """
+        if a_bzrdir is None:
+            self.bzrdir = bzrdir.BzrDir.open(transport.base)
+        else:
+            self.bzrdir = a_bzrdir
+        self._transport = self.bzrdir.transport.clone('..')
+        self._base = self._transport.base
+        self._format = _format
+        if _control_files is None:
+            raise BzrBadParameterMissing('_control_files')
+        self.control_files = _control_files
+        if deprecated_passed(init):
+            warn("BzrBranch.__init__(..., init=XXX): The init parameter is "
+                 "deprecated as of bzr 0.8. Please use Branch.create().",
+                 DeprecationWarning,
+                 stacklevel=2)
+            if init:
+                # this is slower than before deprecation, oh well never mind.
+                # -> its deprecated.
+                self._initialize(transport.base)
+        self._check_format(_format)
+        if deprecated_passed(relax_version_check):
+            warn("BzrBranch.__init__(..., relax_version_check=XXX_: The "
+                 "relax_version_check parameter is deprecated as of bzr 0.8. "
+                 "Please use BzrDir.open_downlevel, or a BzrBranchFormat's "
+                 "open() method.",
+                 DeprecationWarning,
+                 stacklevel=2)
+            if (not relax_version_check
+                and not self._format.is_supported()):
+                raise errors.UnsupportedFormatError(
+                        'sorry, branch format %r not supported' % fmt,
+                        ['use a different bzr version',
+                         'or remove the .bzr directory'
+                         ' and "bzr init" again'])
+        if deprecated_passed(transport):
+            warn("BzrBranch.__init__(transport=XXX...): The transport "
+                 "parameter is deprecated as of bzr 0.8. "
+                 "Please use Branch.open, or bzrdir.open_branch().",
+                 DeprecationWarning,
+                 stacklevel=2)
+        self.repository = _repository
+
+    def __str__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.base)
+
+    __repr__ = __str__
+
+    def __del__(self):
+        # TODO: It might be best to do this somewhere else,
+        # but it is nice for a Branch object to automatically
+        # cache it's information.
+        # Alternatively, we could have the Transport objects cache requests
+        # See the earlier discussion about how major objects (like Branch)
+        # should never expect their __del__ function to run.
+        # XXX: cache_root seems to be unused, 2006-01-13 mbp
+        if hasattr(self, 'cache_root') and self.cache_root is not None:
+            try:
+                shutil.rmtree(self.cache_root)
+            except:
+                pass
+            self.cache_root = None
+
+    def _get_base(self):
+        return self._base
+
+    base = property(_get_base, doc="The URL for the root of this branch.")
+
+    def _finish_transaction(self):
+        """Exit the current transaction."""
+        return self.control_files._finish_transaction()
+
+    def get_transaction(self):
+        """Return the current active transaction.
+
+        If no transaction is active, this returns a passthrough object
+        for which all data is immediately flushed and no caching happens.
+        """
+        # this is an explicit function so that we can do tricky stuff
+        # when the storage in rev_storage is elsewhere.
+        # we probably need to hook the two 'lock a location' and 
+        # 'have a transaction' together more delicately, so that
+        # we can have two locks (branch and storage) and one transaction
+        # ... and finishing the transaction unlocks both, but unlocking
+        # does not. - RBC 20051121
+        return self.control_files.get_transaction()
+
+    def _set_transaction(self, transaction):
+        """Set a new active transaction."""
+        return self.control_files._set_transaction(transaction)
+
+    def abspath(self, name):
+        """See Branch.abspath."""
+        return self.control_files._transport.abspath(name)
+
+    def _check_format(self, format):
+        """Identify the branch format if needed.
+
+        The format is stored as a reference to the format object in
+        self._format for code that needs to check it later.
+
+        The format parameter is either None or the branch format class
+        used to open this branch.
+
+        FIXME: DELETE THIS METHOD when pre 0.8 support is removed.
+        """
+        if format is None:
+            format = BzrBranchFormat.find_format(self.bzrdir)
+        self._format = format
+        mutter("got branch format %s", self._format)
+
+    @needs_read_lock
+    def get_root_id(self):
+        """See Branch.get_root_id."""
+        tree = self.repository.revision_tree(self.last_revision())
+        return tree.inventory.root.file_id
+
+    def lock_write(self):
+        # TODO: test for failed two phase locks. This is known broken.
+        self.control_files.lock_write()
+        self.repository.lock_write()
+
+    def lock_read(self):
+        # TODO: test for failed two phase locks. This is known broken.
+        self.control_files.lock_read()
+        self.repository.lock_read()
+
+    def unlock(self):
+        # TODO: test for failed two phase locks. This is known broken.
+        self.repository.unlock()
+        self.control_files.unlock()
+        
+    def peek_lock_mode(self):
+        if self.control_files._lock_count == 0:
+            return None
+        else:
+            return self.control_files._lock_mode
+
+    @needs_read_lock
+    def print_file(self, file, revision_id):
+        """See Branch.print_file."""
+        return self.repository.print_file(file, revision_id)
+
+    @needs_write_lock
+    def append_revision(self, *revision_ids):
+        """See Branch.append_revision."""
+        for revision_id in revision_ids:
+            mutter("add {%s} to revision-history" % revision_id)
+        rev_history = self.revision_history()
+        rev_history.extend(revision_ids)
+        self.set_revision_history(rev_history)
+
+    @needs_write_lock
+    def set_revision_history(self, rev_history):
+        """See Branch.set_revision_history."""
+        self.control_files.put_utf8(
+            'revision-history', '\n'.join(rev_history))
+        transaction = self.get_transaction()
+        history = transaction.map.find_revision_history()
+        if history is not None:
+            # update the revision history in the identity map.
+            history[:] = list(rev_history)
+            # this call is disabled because revision_history is 
+            # not really an object yet, and the transaction is for objects.
+            # transaction.register_dirty(history)
+        else:
+            transaction.map.add_revision_history(rev_history)
+            # this call is disabled because revision_history is 
+            # not really an object yet, and the transaction is for objects.
+            # transaction.register_clean(history)
+
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+
+        # revno is 1-based; list is 0-based
+
+        new_tree = self.repository.revision_tree(rh[revno-1])
+        if revno == 1:
+            old_tree = EmptyTree()
+        else:
+            old_tree = self.repository.revision_tree(rh[revno-2])
+        return compare_trees(old_tree, new_tree)
+
+    @needs_read_lock
+    def revision_history(self):
+        """See Branch.revision_history."""
+        transaction = self.get_transaction()
+        history = transaction.map.find_revision_history()
+        if history is not None:
+            mutter("cache hit for revision-history in %s", self)
+            return list(history)
+        history = [l.rstrip('\r\n') for l in
+                self.control_files.get_utf8('revision-history').readlines()]
+        transaction.map.add_revision_history(history)
+        # this call is disabled because revision_history is 
+        # not really an object yet, and the transaction is for objects.
+        # transaction.register_clean(history, precious=True)
+        return list(history)
+
+    @needs_write_lock
+    def update_revisions(self, other, stop_revision=None):
+        """See Branch.update_revisions."""
+        other.lock_read()
+        try:
+            if stop_revision is None:
+                stop_revision = other.last_revision()
+                if stop_revision is None:
+                    # if there are no commits, we're done.
+                    return
+            # whats the current last revision, before we fetch [and change it
+            # possibly]
+            last_rev = self.last_revision()
+            # we fetch here regardless of whether we need to so that we pickup
+            # filled in ghosts.
+            self.fetch(other, stop_revision)
+            my_ancestry = self.repository.get_ancestry(last_rev)
+            if stop_revision in my_ancestry:
+                # last_revision is a descendant of stop_revision
+                return
+            # stop_revision must be a descendant of last_revision
+            stop_graph = self.repository.get_revision_graph(stop_revision)
+            if last_rev is not None and last_rev not in stop_graph:
+                # our previous tip is not merged into stop_revision
+                raise errors.DivergedBranches(self, other)
+            # make a new revision history from the graph
+            current_rev_id = stop_revision
+            new_history = []
+            while current_rev_id not in (None, NULL_REVISION):
+                new_history.append(current_rev_id)
+                current_rev_id_parents = stop_graph[current_rev_id]
                 try:
-                    os.rename(self.abspath(f), self.abspath(dest_path))
-                except OSError, e:
-                    raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
-                            ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
-    def revert(self, filenames, old_tree=None, backups=True):
-        """Restore selected files to the versions from a previous tree.
-
-        backups
-            If true (default) backups are made of files before
-            they're renamed.
-        """
-        from bzrlib.errors import NotVersionedError, BzrError
-        from bzrlib.atomicfile import AtomicFile
-        from bzrlib.osutils import backup_file
-        
-        inv = self.read_working_inventory()
-        if old_tree is None:
-            old_tree = self.basis_tree()
-        old_inv = old_tree.inventory
-
-        nids = []
-        for fn in filenames:
-            file_id = inv.path2id(fn)
-            if not file_id:
-                raise NotVersionedError("not a versioned file", fn)
-            if not old_inv.has_id(file_id):
-                raise BzrError("file not present in old tree", fn, file_id)
-            nids.append((fn, file_id))
-            
-        # TODO: Rename back if it was previously at a different location
-
-        # TODO: If given a directory, restore the entire contents from
-        # the previous version.
-
-        # TODO: Make a backup to a temporary file.
-
-        # TODO: If the file previously didn't exist, delete it?
-        for fn, file_id in nids:
-            backup_file(fn)
-            
-            f = AtomicFile(fn, 'wb')
-            try:
-                f.write(old_tree.get_file(file_id).read())
-                f.commit()
-            finally:
-                f.close()
-
-
-    def pending_merges(self):
-        """Return a list of pending merges.
-
-        These are revisions that have been merged into the working
-        directory but not yet committed.
-        """
-        cfn = self.controlfilename('pending-merges')
-        if not os.path.exists(cfn):
-            return []
-        p = []
-        for l in self.controlfile('pending-merges', 'r').readlines():
-            p.append(l.rstrip('\n'))
-        return p
-
-
-    def add_pending_merge(self, revision_id):
-        from bzrlib.revision import validate_revision_id
-
-        validate_revision_id(revision_id)
-
-        p = self.pending_merges()
-        if revision_id in p:
-            return
-        p.append(revision_id)
-        self.set_pending_merges(p)
-
-
-    def set_pending_merges(self, rev_list):
-        from bzrlib.atomicfile import AtomicFile
-        self.lock_write()
-        try:
-            f = AtomicFile(self.controlfilename('pending-merges'))
-            try:
-                for l in rev_list:
-                    print >>f, l
-                f.commit()
-            finally:
-                f.close()
-        finally:
-            self.unlock()
-
-
-
-class ScratchBranch(Branch):
-    """Special test class: a branch that cleans up after itself.
-
-    >>> b = ScratchBranch()
-    >>> isdir(b.base)
-    True
-    >>> bd = b.base
-    >>> b.destroy()
-    >>> isdir(bd)
-    False
-    """
-    def __init__(self, files=[], dirs=[], base=None):
-        """Make a test branch.
-
-        This creates a temporary directory and runs init-tree in it.
-
-        If any files are listed, they are created in the working copy.
-        """
-        from tempfile import mkdtemp
-        init = False
-        if base is None:
-            base = mkdtemp()
-            init = True
-        Branch.__init__(self, base, init=init)
-        for d in dirs:
-            os.mkdir(self.abspath(d))
-            
-        for f in files:
-            file(os.path.join(self.base, f), 'w').write('content of %s' % f)
-
-
-    def clone(self):
-        """
-        >>> orig = ScratchBranch(files=["file1", "file2"])
-        >>> clone = orig.clone()
-        >>> os.path.samefile(orig.base, clone.base)
-        False
-        >>> os.path.isfile(os.path.join(clone.base, "file1"))
-        True
-        """
-        from shutil import copytree
-        from tempfile import mkdtemp
-        base = mkdtemp()
-        os.rmdir(base)
-        copytree(self.base, base, symlinks=True)
-        return ScratchBranch(base=base)
-        
-    def __del__(self):
-        self.destroy()
-
-    def destroy(self):
-        """Destroy the test branch, removing the scratch directory."""
-        from shutil import rmtree
-        try:
-            if self.base:
-                mutter("delete ScratchBranch %s" % self.base)
-                rmtree(self.base)
-        except OSError, e:
-            # Work around for shutil.rmtree failing on Windows when
-            # readonly files are encountered
-            mutter("hit exception in destroying ScratchBranch: %s" % e)
-            for root, dirs, files in os.walk(self.base, topdown=False):
-                for name in files:
-                    os.chmod(os.path.join(root, name), 0700)
-            rmtree(self.base)
-        self.base = None
-
+                    current_rev_id = current_rev_id_parents[0]
+                except IndexError:
+                    current_rev_id = None
+            new_history.reverse()
+            self.set_revision_history(new_history)
+        finally:
+            other.unlock()
+
+    @deprecated_method(zero_eight)
+    def pullable_revisions(self, other, stop_revision):
+        """Please use bzrlib.missing instead."""
+        other_revno = other.revision_id_to_revno(stop_revision)
+        try:
+            return self.missing_revisions(other, other_revno)
+        except DivergedBranches, e:
+            try:
+                pullable_revs = get_intervening_revisions(self.last_revision(),
+                                                          stop_revision, 
+                                                          self.repository)
+                assert self.last_revision() not in pullable_revs
+                return pullable_revs
+            except bzrlib.errors.NotAncestor:
+                if is_ancestor(self.last_revision(), stop_revision, self):
+                    return []
+                else:
+                    raise e
+        
+    def basis_tree(self):
+        """See Branch.basis_tree."""
+        return self.repository.revision_tree(self.last_revision())
+
+    @deprecated_method(zero_eight)
+    def working_tree(self):
+        """Create a Working tree object for this branch."""
+        from bzrlib.workingtree import WorkingTree
+        from bzrlib.transport.local import LocalTransport
+        if (self.base.find('://') != -1 or 
+            not isinstance(self._transport, LocalTransport)):
+            raise NoWorkingTree(self.base)
+        return self.bzrdir.open_workingtree()
+
+    @needs_write_lock
+    def pull(self, source, overwrite=False, stop_revision=None):
+        """See Branch.pull."""
+        source.lock_read()
+        try:
+            old_count = len(self.revision_history())
+            try:
+                self.update_revisions(source,stop_revision)
+            except DivergedBranches:
+                if not overwrite:
+                    raise
+            if overwrite:
+                self.set_revision_history(source.revision_history())
+            new_count = len(self.revision_history())
+            return new_count - old_count
+        finally:
+            source.unlock()
+
+    def get_parent(self):
+        """See Branch.get_parent."""
+        import errno
+        _locs = ['parent', 'pull', 'x-pull']
+        for l in _locs:
+            try:
+                return self.control_files.get_utf8(l).read().strip('\n')
+            except NoSuchFile:
+                pass
+        return None
+
+    def get_push_location(self):
+        """See Branch.get_push_location."""
+        config = bzrlib.config.BranchConfig(self)
+        push_loc = config.get_user_option('push_location')
+        return push_loc
+
+    def set_push_location(self, location):
+        """See Branch.set_push_location."""
+        config = bzrlib.config.LocationConfig(self.base)
+        config.set_user_option('push_location', location)
+
+    @needs_write_lock
+    def set_parent(self, url):
+        """See Branch.set_parent."""
+        # TODO: Maybe delete old location files?
+        # URLs should never be unicode, even on the local fs,
+        # FIXUP this and get_parent in a future branch format bump:
+        # read and rewrite the file, and have the new format code read
+        # using .get not .get_utf8. RBC 20060125
+        if url is None:
+            self.control_files._transport.delete('parent')
+        else:
+            self.control_files.put_utf8('parent', url + '\n')
+
+    def tree_config(self):
+        return TreeConfig(self)
+
+
+class BzrBranch5(BzrBranch):
+    """A format 5 branch. This supports new features over plan branches.
+
+    It has support for a master_branch which is the data for bound branches.
+    """
+
+    def __init__(self,
+                 _format,
+                 _control_files,
+                 a_bzrdir,
+                 _repository):
+        super(BzrBranch5, self).__init__(_format=_format,
+                                         _control_files=_control_files,
+                                         a_bzrdir=a_bzrdir,
+                                         _repository=_repository)
+        
+    @needs_write_lock
+    def pull(self, source, overwrite=False, stop_revision=None):
+        """Updates branch.pull to be bound branch aware."""
+        bound_location = self.get_bound_location()
+        if source.base != bound_location:
+            # not pulling from master, so we need to update master.
+            master_branch = self.get_master_branch()
+            if master_branch:
+                master_branch.pull(source)
+                source = master_branch
+        return super(BzrBranch5, self).pull(source, overwrite, stop_revision)
+
+    def get_bound_location(self):
+        try:
+            return self.control_files.get_utf8('bound').read()[:-1]
+        except errors.NoSuchFile:
+            return None
+
+    @needs_read_lock
+    def get_master_branch(self):
+        """Return the branch we are bound to.
+        
+        :return: Either a Branch, or None
+
+        This could memoise the branch, but if thats done
+        it must be revalidated on each new lock.
+        So for now we just dont memoise it.
+        # RBC 20060304 review this decision.
+        """
+        bound_loc = self.get_bound_location()
+        if not bound_loc:
+            return None
+        try:
+            return Branch.open(bound_loc)
+        except (errors.NotBranchError, errors.ConnectionError), e:
+            raise errors.BoundBranchConnectionFailure(
+                    self, bound_loc, e)
+
+    @needs_write_lock
+    def set_bound_location(self, location):
+        """Set the target where this branch is bound to.
+
+        :param location: URL to the target branch
+        """
+        if location:
+            self.control_files.put_utf8('bound', location+'\n')
+        else:
+            try:
+                self.control_files._transport.delete('bound')
+            except NoSuchFile:
+                return False
+            return True
+
+    @needs_write_lock
+    def bind(self, other):
+        """Bind the local branch the other branch.
+
+        :param other: The branch to bind to
+        :type other: Branch
+        """
+        # TODO: jam 20051230 Consider checking if the target is bound
+        #       It is debatable whether you should be able to bind to
+        #       a branch which is itself bound.
+        #       Committing is obviously forbidden,
+        #       but binding itself may not be.
+        #       Since we *have* to check at commit time, we don't
+        #       *need* to check here
+        self.pull(other)
+
+        # we are now equal to or a suffix of other.
+
+        # Since we have 'pulled' from the remote location,
+        # now we should try to pull in the opposite direction
+        # in case the local tree has more revisions than the
+        # remote one.
+        # There may be a different check you could do here
+        # rather than actually trying to install revisions remotely.
+        # TODO: capture an exception which indicates the remote branch
+        #       is not writeable. 
+        #       If it is up-to-date, this probably should not be a failure
+        
+        # lock other for write so the revision-history syncing cannot race
+        other.lock_write()
+        try:
+            other.pull(self)
+            # if this does not error, other now has the same last rev we do
+            # it can only error if the pull from other was concurrent with
+            # a commit to other from someone else.
+
+            # until we ditch revision-history, we need to sync them up:
+            self.set_revision_history(other.revision_history())
+            # now other and self are up to date with each other and have the
+            # same revision-history.
+        finally:
+            other.unlock()
+
+        self.set_bound_location(other.base)
+
+    @needs_write_lock
+    def unbind(self):
+        """If bound, unbind"""
+        return self.set_bound_location(None)
+
+    @needs_write_lock
+    def update(self):
+        """Synchronise this branch with the master branch if any. 
+
+        :return: None or the last_revision that was pivoted out during the
+                 update.
+        """
+        master = self.get_master_branch()
+        if master is not None:
+            old_tip = self.last_revision()
+            self.pull(master, overwrite=True)
+            if old_tip in self.repository.get_ancestry(self.last_revision()):
+                return None
+            return old_tip
+        return None
+
+
+class BranchTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple branch formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and branch_format
+    classes into each copy. Each copy is also given a new id() to make it
+    easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
     
+    def adapt(self, test):
+        result = TestSuite()
+        for branch_format, bzrdir_format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = bzrdir_format
+            new_test.branch_format = branch_format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), branch_format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
 
 ######################################################################
 # predicates
 
 
-def is_control_file(filename):
-    ## FIXME: better check
-    filename = os.path.normpath(filename)
-    while filename != '':
-        head, tail = os.path.split(filename)
-        ## mutter('check %r for control file' % ((head, tail), ))
-        if tail == bzrlib.BZRDIR:
-            return True
-        if filename == head:
-            break
-        filename = head
-    return False
-
-
-
-def gen_file_id(name):
-    """Return new file id.
-
-    This should probably generate proper UUIDs, but for the moment we
-    cope with just randomness because running uuidgen every time is
-    slow."""
-    import re
-    from binascii import hexlify
-    from time import time
-
-    # get last component
-    idx = name.rfind('/')
-    if idx != -1:
-        name = name[idx+1 : ]
-    idx = name.rfind('\\')
-    if idx != -1:
-        name = name[idx+1 : ]
-
-    # make it not a hidden file
-    name = name.lstrip('.')
-
-    # remove any wierd characters; we don't escape them but rather
-    # just pull them out
-    name = re.sub(r'[^\w.]', '', name)
-
-    s = hexlify(rand_bytes(8))
-    return '-'.join((name, compact_date(time()), s))
-
-
-def gen_root_id():
-    """Return a new tree-root file id."""
-    return gen_file_id('TREE_ROOT')
-
+@deprecated_function(zero_eight)
+def ScratchBranch(*args, **kwargs):
+    """See bzrlib.bzrdir.ScratchDir."""
+    d = ScratchDir(*args, **kwargs)
+    return d.open_branch()
+
+
+@deprecated_function(zero_eight)
+def is_control_file(*args, **kwargs):
+    """See bzrlib.workingtree.is_control_file."""
+    return bzrlib.workingtree.is_control_file(*args, **kwargs)

=== added file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	1970-01-01 00:00:00 +0000
+++ bzrlib/builtins.py	2006-04-19 08:31:08 +0000
@@ -0,0 +1,2511 @@
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""builtin bzr commands"""
+
+
+import errno
+import os
+from shutil import rmtree
+import sys
+
+import bzrlib
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.commands import Command, display_command
+from bzrlib.revision import common_ancestor
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, BzrCheckError, BzrCommandError, 
+                           NotBranchError, DivergedBranches, NotConflicted,
+                           NoSuchFile, NoWorkingTree, FileInWrongBranch,
+                           NotVersionedError)
+from bzrlib.log import show_one_log
+from bzrlib.merge import Merge3Merger
+from bzrlib.option import Option
+from bzrlib.progress import DummyProgress, ProgressPhase
+from bzrlib.revisionspec import RevisionSpec
+import bzrlib.trace
+from bzrlib.trace import mutter, note, log_error, warning, is_quiet
+from bzrlib.transport.local import LocalTransport
+import bzrlib.ui
+from bzrlib.workingtree import WorkingTree
+
+
+def tree_files(file_list, default_branch=u'.'):
+    try:
+        return internal_tree_files(file_list, default_branch)
+    except FileInWrongBranch, e:
+        raise BzrCommandError("%s is not in the same branch as %s" %
+                             (e.path, file_list[0]))
+
+
+# XXX: Bad function name; should possibly also be a class method of
+# WorkingTree rather than a function.
+def internal_tree_files(file_list, default_branch=u'.'):
+    """Convert command-line paths to a WorkingTree and relative paths.
+
+    This is typically used for command-line processors that take one or
+    more filenames, and infer the workingtree that contains them.
+
+    The filenames given are not required to exist.
+
+    :param file_list: Filenames to convert.  
+
+    :param default_branch: Fallback tree path to use if file_list is empty or None.
+
+    :return: workingtree, [relative_paths]
+    """
+    if file_list is None or len(file_list) == 0:
+        return WorkingTree.open_containing(default_branch)[0], file_list
+    tree = WorkingTree.open_containing(file_list[0])[0]
+    new_list = []
+    for filename in file_list:
+        try:
+            new_list.append(tree.relpath(filename))
+        except errors.PathNotChild:
+            raise FileInWrongBranch(tree.branch, filename)
+    return tree, new_list
+
+
+def get_format_type(typestring):
+    """Parse and return a format specifier."""
+    if typestring == "weave":
+        return bzrdir.BzrDirFormat6()
+    if typestring == "metadir":
+        return bzrdir.BzrDirMetaFormat1()
+    if typestring == "knit":
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = bzrlib.repository.RepositoryFormatKnit1()
+        return format
+    msg = "No known bzr-dir format %s. Supported types are: weave, metadir\n" %\
+        (typestring)
+    raise BzrCommandError(msg)
+
+
+# TODO: Make sure no commands unconditionally use the working directory as a
+# branch.  If a filename argument is used, the first of them should be used to
+# specify the branch.  (Perhaps this can be factored out into some kind of
+# Argument class, representing a file in a branch, where the first occurrence
+# opens the branch?)
+
+class cmd_status(Command):
+    """Display status summary.
+
+    This reports on versioned and unknown files, reporting them
+    grouped by state.  Possible states are:
+
+    added
+        Versioned in the working copy but not in the previous revision.
+
+    removed
+        Versioned in the previous revision but removed or deleted
+        in the working copy.
+
+    renamed
+        Path of this file changed from the previous revision;
+        the text may also have changed.  This includes files whose
+        parent directory was renamed.
+
+    modified
+        Text has changed since the previous revision.
+
+    unchanged
+        Nothing about this file has changed since the previous revision.
+        Only shown with --all.
+
+    unknown
+        Not versioned and not matching an ignore pattern.
+
+    To see ignored files use 'bzr ignored'.  For details in the
+    changes to file texts, use 'bzr diff'.
+
+    If no arguments are specified, the status of the entire working
+    directory is shown.  Otherwise, only the status of the specified
+    files or directories is reported.  If a directory is given, status
+    is reported for everything inside that directory.
+
+    If a revision argument is given, the status is calculated against
+    that revision, or between two revisions if two are provided.
+    """
+    
+    # TODO: --no-recurse, --recurse options
+    
+    takes_args = ['file*']
+    takes_options = ['all', 'show-ids', 'revision']
+    aliases = ['st', 'stat']
+    
+    @display_command
+    def run(self, all=False, show_ids=False, file_list=None, revision=None):
+        tree, file_list = tree_files(file_list)
+            
+        from bzrlib.status import show_tree_status
+        show_tree_status(tree, show_unchanged=all, show_ids=show_ids,
+                         specific_files=file_list, revision=revision)
+
+
+class cmd_cat_revision(Command):
+    """Write out metadata for a revision.
+    
+    The revision to print can either be specified by a specific
+    revision identifier, or you can use --revision.
+    """
+
+    hidden = True
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
+    
+    @display_command
+    def run(self, revision_id=None, revision=None):
+
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = WorkingTree.open_containing(u'.')[0].branch
+        if revision_id is not None:
+            sys.stdout.write(b.repository.get_revision_xml(revision_id))
+        elif revision is not None:
+            for rev in revision:
+                if rev is None:
+                    raise BzrCommandError('You cannot specify a NULL revision.')
+                revno, rev_id = rev.in_history(b)
+                sys.stdout.write(b.repository.get_revision_xml(rev_id))
+    
+
+class cmd_revno(Command):
+    """Show current revision number.
+
+    This is equal to the number of revisions on this branch."""
+    takes_args = ['location?']
+    @display_command
+    def run(self, location=u'.'):
+        print Branch.open_containing(location)[0].revno()
+
+
+class cmd_revision_info(Command):
+    """Show revision number and revision id for a given revision identifier.
+    """
+    hidden = True
+    takes_args = ['revision_info*']
+    takes_options = ['revision']
+    @display_command
+    def run(self, revision=None, revision_info_list=[]):
+
+        revs = []
+        if revision is not None:
+            revs.extend(revision)
+        if revision_info_list is not None:
+            for rev in revision_info_list:
+                revs.append(RevisionSpec(rev))
+        if len(revs) == 0:
+            raise BzrCommandError('You must supply a revision identifier')
+
+        b = WorkingTree.open_containing(u'.')[0].branch
+
+        for rev in revs:
+            revinfo = rev.in_history(b)
+            if revinfo.revno is None:
+                print '     %s' % revinfo.rev_id
+            else:
+                print '%4d %s' % (revinfo.revno, revinfo.rev_id)
+
+    
+class cmd_add(Command):
+    """Add specified files or directories.
+
+    In non-recursive mode, all the named items are added, regardless
+    of whether they were previously ignored.  A warning is given if
+    any of the named files are already versioned.
+
+    In recursive mode (the default), files are treated the same way
+    but the behaviour for directories is different.  Directories that
+    are already versioned do not give a warning.  All directories,
+    whether already versioned or not, are searched for files or
+    subdirectories that are neither versioned or ignored, and these
+    are added.  This search proceeds recursively into versioned
+    directories.  If no names are given '.' is assumed.
+
+    Therefore simply saying 'bzr add' will version all files that
+    are currently unknown.
+
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
+
+    --dry-run will show which files would be added, but not actually 
+    add them.
+    """
+    takes_args = ['file*']
+    takes_options = ['no-recurse', 'dry-run', 'verbose']
+
+    def run(self, file_list, no_recurse=False, dry_run=False, verbose=False):
+        import bzrlib.add
+
+        if dry_run:
+            if is_quiet():
+                # This is pointless, but I'd rather not raise an error
+                action = bzrlib.add.add_action_null
+            else:
+                action = bzrlib.add.add_action_print
+        elif is_quiet():
+            action = bzrlib.add.add_action_add
+        else:
+            action = bzrlib.add.add_action_add_and_print
+
+        added, ignored = bzrlib.add.smart_add(file_list, not no_recurse, 
+                                              action)
+        if len(ignored) > 0:
+            for glob in sorted(ignored.keys()):
+                match_len = len(ignored[glob])
+                if verbose:
+                    for path in ignored[glob]:
+                        print "ignored %s matching \"%s\"" % (path, glob)
+                else:
+                    print "ignored %d file(s) matching \"%s\"" % (match_len,
+                                                              glob)
+            print "If you wish to add some of these files, please add them"\
+                " by name."
+
+
+class cmd_mkdir(Command):
+    """Create a new versioned directory.
+
+    This is equivalent to creating the directory and then adding it.
+    """
+    takes_args = ['dir+']
+
+    def run(self, dir_list):
+        for d in dir_list:
+            os.mkdir(d)
+            wt, dd = WorkingTree.open_containing(d)
+            wt.add([dd])
+            print 'added', d
+
+
+class cmd_relpath(Command):
+    """Show path of a file relative to root"""
+    takes_args = ['filename']
+    hidden = True
+    
+    @display_command
+    def run(self, filename):
+        tree, relpath = WorkingTree.open_containing(filename)
+        print relpath
+
+
+class cmd_inventory(Command):
+    """Show inventory of the current working copy or a revision.
+
+    It is possible to limit the output to a particular entry
+    type using the --kind option.  For example; --kind file.
+    """
+    takes_options = ['revision', 'show-ids', 'kind']
+    
+    @display_command
+    def run(self, revision=None, show_ids=False, kind=None):
+        if kind and kind not in ['file', 'directory', 'symlink']:
+            raise BzrCommandError('invalid kind specified')
+        tree = WorkingTree.open_containing(u'.')[0]
+        if revision is None:
+            inv = tree.read_working_inventory()
+        else:
+            if len(revision) > 1:
+                raise BzrCommandError('bzr inventory --revision takes'
+                    ' exactly one revision identifier')
+            inv = tree.branch.repository.get_revision_inventory(
+                revision[0].in_history(tree.branch).rev_id)
+
+        for path, entry in inv.entries():
+            if kind and kind != entry.kind:
+                continue
+            if show_ids:
+                print '%-50s %s' % (path, entry.file_id)
+            else:
+                print path
+
+
+class cmd_mv(Command):
+    """Move or rename a file.
+
+    usage:
+        bzr mv OLDNAME NEWNAME
+        bzr mv SOURCE... DESTINATION
+
+    If the last argument is a versioned directory, all the other names
+    are moved into it.  Otherwise, there must be exactly two arguments
+    and the file is changed to a new name, which must not already exist.
+
+    Files cannot be moved between branches.
+    """
+    takes_args = ['names*']
+    aliases = ['move', 'rename']
+
+    def run(self, names_list):
+        if len(names_list) < 2:
+            raise BzrCommandError("missing file argument")
+        tree, rel_names = tree_files(names_list)
+        
+        if os.path.isdir(names_list[-1]):
+            # move into existing directory
+            for pair in tree.move(rel_names[:-1], rel_names[-1]):
+                print "%s => %s" % pair
+        else:
+            if len(names_list) != 2:
+                raise BzrCommandError('to mv multiple files the destination '
+                                      'must be a versioned directory')
+            tree.rename_one(rel_names[0], rel_names[1])
+            print "%s => %s" % (rel_names[0], rel_names[1])
+            
+    
+class cmd_pull(Command):
+    """Turn this branch into a mirror of another branch.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if the destination branch's most recent commit is one
+    that has not been merged (directly or indirectly) into the parent.
+
+    If branches have diverged, you can use 'bzr merge' to integrate the changes
+    from one into the other.  Once one branch has merged, the other should
+    be able to pull it again.
+
+    If branches have diverged, you can use 'bzr merge' to pull the text changes
+    from one into the other.  Once one branch has merged, the other should
+    be able to pull it again.
+
+    If you want to forget your local changes and just update your branch to
+    match the remote one, use pull --overwrite.
+
+    If there is no default location set, the first pull will set it.  After
+    that, you can omit the location to use the default.  To change the
+    default, use --remember.
+    """
+    takes_options = ['remember', 'overwrite', 'revision', 'verbose']
+    takes_args = ['location?']
+
+    def run(self, location=None, remember=False, overwrite=False, revision=None, verbose=False):
+        # FIXME: too much stuff is in the command class
+        try:
+            tree_to = WorkingTree.open_containing(u'.')[0]
+            branch_to = tree_to.branch
+        except NoWorkingTree:
+            tree_to = None
+            branch_to = Branch.open_containing(u'.')[0] 
+        stored_loc = branch_to.get_parent()
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No pull location known or specified.")
+            else:
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+
+        if branch_to.get_parent() is None or remember:
+            branch_to.set_parent(location)
+
+        branch_from = Branch.open(location)
+
+        if revision is None:
+            rev_id = None
+        elif len(revision) == 1:
+            rev_id = revision[0].in_history(branch_from).rev_id
+        else:
+            raise BzrCommandError('bzr pull --revision takes one value.')
+
+        old_rh = branch_to.revision_history()
+        if tree_to is not None:
+            count = tree_to.pull(branch_from, overwrite, rev_id)
+        else:
+            count = branch_to.pull(branch_from, overwrite, rev_id)
+        note('%d revision(s) pulled.' % (count,))
+
+        if verbose:
+            new_rh = branch_to.revision_history()
+            if old_rh != new_rh:
+                # Something changed
+                from bzrlib.log import show_changed_revisions
+                show_changed_revisions(branch_to, old_rh, new_rh)
+
+
+class cmd_push(Command):
+    """Update a mirror of this branch.
+    
+    The target branch will not have its working tree populated because this
+    is both expensive, and is not supported on remote file systems.
+    
+    Some smart servers or protocols *may* put the working tree in place in
+    the future.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if the destination branch's most recent commit is one
+    that has not been merged (directly or indirectly) by the source branch.
+
+    If branches have diverged, you can use 'bzr push --overwrite' to replace
+    the other branch completely, discarding its unmerged changes.
+    
+    If you want to ensure you have the different changes in the other branch,
+    do a merge (see bzr help merge) from the other branch, and commit that.
+    After that you will be able to do a push without '--overwrite'.
+
+    If there is no default push location set, the first push will set it.
+    After that, you can omit the location to use the default.  To change the
+    default, use --remember.
+    """
+    takes_options = ['remember', 'overwrite', 
+                     Option('create-prefix', 
+                            help='Create the path leading up to the branch '
+                                 'if it does not already exist')]
+    takes_args = ['location?']
+
+    def run(self, location=None, remember=False, overwrite=False,
+            create_prefix=False, verbose=False):
+        # FIXME: Way too big!  Put this into a function called from the
+        # command.
+        from bzrlib.transport import get_transport
+        
+        tree_from = WorkingTree.open_containing(u'.')[0]
+        br_from = tree_from.branch
+        stored_loc = tree_from.branch.get_push_location()
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No push location known or specified.")
+            else:
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+        if br_from.get_push_location() is None or remember:
+            br_from.set_push_location(location)
+        try:
+            dir_to = bzrlib.bzrdir.BzrDir.open(location)
+            br_to = dir_to.open_branch()
+        except NotBranchError:
+            # create a branch.
+            transport = get_transport(location).clone('..')
+            if not create_prefix:
+                try:
+                    transport.mkdir(transport.relpath(location))
+                except NoSuchFile:
+                    raise BzrCommandError("Parent directory of %s "
+                                          "does not exist." % location)
+            else:
+                current = transport.base
+                needed = [(transport, transport.relpath(location))]
+                while needed:
+                    try:
+                        transport, relpath = needed[-1]
+                        transport.mkdir(relpath)
+                        needed.pop()
+                    except NoSuchFile:
+                        new_transport = transport.clone('..')
+                        needed.append((new_transport,
+                                       new_transport.relpath(transport.base)))
+                        if new_transport.base == transport.base:
+                            raise BzrCommandError("Could not create "
+                                                  "path prefix.")
+            dir_to = br_from.bzrdir.clone(location)
+            br_to = dir_to.open_branch()
+        old_rh = br_to.revision_history()
+        try:
+            try:
+                tree_to = dir_to.open_workingtree()
+            except errors.NotLocalUrl:
+                # TODO: This should be updated for branches which don't have a
+                # working tree, as opposed to ones where we just couldn't 
+                # update the tree.
+                warning('This transport does not update the working '
+                        'tree of: %s' % (br_to.base,))
+                count = br_to.pull(br_from, overwrite)
+            except NoWorkingTree:
+                count = br_to.pull(br_from, overwrite)
+            else:
+                count = tree_to.pull(br_from, overwrite)
+        except DivergedBranches:
+            raise BzrCommandError("These branches have diverged."
+                                  "  Try a merge then push with overwrite.")
+        note('%d revision(s) pushed.' % (count,))
+
+        if verbose:
+            new_rh = br_to.revision_history()
+            if old_rh != new_rh:
+                # Something changed
+                from bzrlib.log import show_changed_revisions
+                show_changed_revisions(br_to, old_rh, new_rh)
+
+
+class cmd_branch(Command):
+    """Create a new copy of a branch.
+
+    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
+    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
+
+    To retrieve the branch as of a particular revision, supply the --revision
+    parameter, as in "branch foo/bar -r 5".
+
+    --basis is to speed up branching from remote branches.  When specified, it
+    copies all the file-contents, inventory and revision data from the basis
+    branch before copying anything from the remote branch.
+    """
+    takes_args = ['from_location', 'to_location?']
+    takes_options = ['revision', 'basis']
+    aliases = ['get', 'clone']
+
+    def run(self, from_location, to_location=None, revision=None, basis=None):
+        if revision is None:
+            revision = [None]
+        elif len(revision) > 1:
+            raise BzrCommandError(
+                'bzr branch --revision takes exactly 1 revision value')
+        try:
+            br_from = Branch.open(from_location)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise BzrCommandError('Source location "%s" does not'
+                                      ' exist.' % to_location)
+            else:
+                raise
+        br_from.lock_read()
+        try:
+            if basis is not None:
+                basis_dir = bzrdir.BzrDir.open_containing(basis)[0]
+            else:
+                basis_dir = None
+            if len(revision) == 1 and revision[0] is not None:
+                revision_id = revision[0].in_history(br_from)[1]
+            else:
+                # FIXME - wt.last_revision, fallback to branch, fall back to
+                # None or perhaps NULL_REVISION to mean copy nothing
+                # RBC 20060209
+                revision_id = br_from.last_revision()
+            if to_location is None:
+                to_location = os.path.basename(from_location.rstrip("/\\"))
+                name = None
+            else:
+                name = os.path.basename(to_location) + '\n'
+            try:
+                os.mkdir(to_location)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    raise BzrCommandError('Target directory "%s" already'
+                                          ' exists.' % to_location)
+                if e.errno == errno.ENOENT:
+                    raise BzrCommandError('Parent of "%s" does not exist.' %
+                                          to_location)
+                else:
+                    raise
+            try:
+                # preserve whatever source format we have.
+                dir = br_from.bzrdir.sprout(to_location, revision_id, basis_dir)
+                branch = dir.open_branch()
+            except bzrlib.errors.NoSuchRevision:
+                rmtree(to_location)
+                msg = "The branch %s has no revision %s." % (from_location, revision[0])
+                raise BzrCommandError(msg)
+            except bzrlib.errors.UnlistableBranch:
+                rmtree(to_location)
+                msg = "The branch %s cannot be used as a --basis" % (basis,)
+                raise BzrCommandError(msg)
+            if name:
+                branch.control_files.put_utf8('branch-name', name)
+
+            note('Branched %d revision(s).' % branch.revno())
+        finally:
+            br_from.unlock()
+
+
+class cmd_checkout(Command):
+    """Create a new checkout of an existing branch.
+
+    If BRANCH_LOCATION is omitted, checkout will reconstitute a working tree for
+    the branch found in '.'. This is useful if you have removed the working tree
+    or if it was never created - i.e. if you pushed the branch to its current
+    location using SFTP.
+    
+    If the TO_LOCATION is omitted, the last component of the BRANCH_LOCATION will
+    be used.  In other words, "checkout ../foo/bar" will attempt to create ./bar.
+
+    To retrieve the branch as of a particular revision, supply the --revision
+    parameter, as in "checkout foo/bar -r 5". Note that this will be immediately
+    out of date [so you cannot commit] but it may be useful (i.e. to examine old
+    code.)
+
+    --basis is to speed up checking out from remote branches.  When specified, it
+    uses the inventory and file contents from the basis branch in preference to the
+    branch being checked out. [Not implemented yet.]
+    """
+    takes_args = ['branch_location?', 'to_location?']
+    takes_options = ['revision', # , 'basis']
+                     Option('lightweight',
+                            help="perform a lightweight checkout. Lightweight "
+                                 "checkouts depend on access to the branch for "
+                                 "every operation. Normal checkouts can perform "
+                                 "common operations like diff and status without "
+                                 "such access, and also support local commits."
+                            ),
+                     ]
+
+    def run(self, branch_location=None, to_location=None, revision=None, basis=None,
+            lightweight=False):
+        if revision is None:
+            revision = [None]
+        elif len(revision) > 1:
+            raise BzrCommandError(
+                'bzr checkout --revision takes exactly 1 revision value')
+        if branch_location is None:
+            branch_location = bzrlib.osutils.getcwd()
+            to_location = branch_location
+        source = Branch.open(branch_location)
+        if len(revision) == 1 and revision[0] is not None:
+            revision_id = revision[0].in_history(source)[1]
+        else:
+            revision_id = None
+        if to_location is None:
+            to_location = os.path.basename(branch_location.rstrip("/\\"))
+        # if the source and to_location are the same, 
+        # and there is no working tree,
+        # then reconstitute a branch
+        if (bzrlib.osutils.abspath(to_location) == 
+            bzrlib.osutils.abspath(branch_location)):
+            try:
+                source.bzrdir.open_workingtree()
+            except errors.NoWorkingTree:
+                source.bzrdir.create_workingtree()
+                return
+        try:
+            os.mkdir(to_location)
+        except OSError, e:
+            if e.errno == errno.EEXIST:
+                raise BzrCommandError('Target directory "%s" already'
+                                      ' exists.' % to_location)
+            if e.errno == errno.ENOENT:
+                raise BzrCommandError('Parent of "%s" does not exist.' %
+                                      to_location)
+            else:
+                raise
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            if lightweight:
+                checkout = bzrdir.BzrDirMetaFormat1().initialize(to_location)
+                bzrlib.branch.BranchReferenceFormat().initialize(checkout, source)
+            else:
+                checkout_branch =  bzrlib.bzrdir.BzrDir.create_branch_convenience(
+                    to_location, force_new_tree=False)
+                checkout = checkout_branch.bzrdir
+                checkout_branch.bind(source)
+                if revision_id is not None:
+                    rh = checkout_branch.revision_history()
+                    checkout_branch.set_revision_history(rh[:rh.index(revision_id) + 1])
+            checkout.create_workingtree(revision_id)
+        finally:
+            bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+
+
+class cmd_renames(Command):
+    """Show list of renamed files.
+    """
+    # TODO: Option to show renames between two historical versions.
+
+    # TODO: Only show renames under dir, rather than in the whole branch.
+    takes_args = ['dir?']
+
+    @display_command
+    def run(self, dir=u'.'):
+        tree = WorkingTree.open_containing(dir)[0]
+        old_inv = tree.basis_tree().inventory
+        new_inv = tree.read_working_inventory()
+
+        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
+        renames.sort()
+        for old_name, new_name in renames:
+            print "%s => %s" % (old_name, new_name)        
+
+
+class cmd_update(Command):
+    """Update a tree to have the latest code committed to its branch.
+    
+    This will perform a merge into the working tree, and may generate
+    conflicts. If you have any local changes, you will still 
+    need to commit them after the update for the update to be complete.
+    
+    If you want to discard your local changes, you can just do a 
+    'bzr revert' instead of 'bzr commit' after the update.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        tree = WorkingTree.open_containing(dir)[0]
+        tree.lock_write()
+        try:
+            if tree.last_revision() == tree.branch.last_revision():
+                # may be up to date, check master too.
+                master = tree.branch.get_master_branch()
+                if master is None or master.last_revision == tree.last_revision():
+                    note("Tree is up to date.")
+                    return
+            conflicts = tree.update()
+            note('Updated to revision %d.' %
+                 (tree.branch.revision_id_to_revno(tree.last_revision()),))
+            if conflicts != 0:
+                return 1
+            else:
+                return 0
+        finally:
+            tree.unlock()
+
+
+class cmd_info(Command):
+    """Show statistical information about a branch."""
+    takes_args = ['branch?']
+    takes_options = ['verbose']
+    
+    @display_command
+    def run(self, branch=None, verbose=False):
+        import bzrlib.info
+        bzrlib.info.show_bzrdir_info(bzrdir.BzrDir.open_containing(branch)[0],
+                                     verbose=verbose)
+
+
+class cmd_remove(Command):
+    """Make a file unversioned.
+
+    This makes bzr stop tracking changes to a versioned file.  It does
+    not delete the working copy.
+    """
+    takes_args = ['file+']
+    takes_options = ['verbose']
+    aliases = ['rm']
+    
+    def run(self, file_list, verbose=False):
+        tree, file_list = tree_files(file_list)
+        tree.remove(file_list, verbose=verbose)
+
+
+class cmd_file_id(Command):
+    """Print file_id of a particular file or directory.
+
+    The file_id is assigned when the file is first added and remains the
+    same through all revisions where the file exists, even when it is
+    moved or renamed.
+    """
+    hidden = True
+    takes_args = ['filename']
+    @display_command
+    def run(self, filename):
+        tree, relpath = WorkingTree.open_containing(filename)
+        i = tree.inventory.path2id(relpath)
+        if i == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        else:
+            print i
+
+
+class cmd_file_path(Command):
+    """Print path of file_ids to a file or directory.
+
+    This prints one line for each directory down to the target,
+    starting at the branch root."""
+    hidden = True
+    takes_args = ['filename']
+    @display_command
+    def run(self, filename):
+        tree, relpath = WorkingTree.open_containing(filename)
+        inv = tree.inventory
+        fid = inv.path2id(relpath)
+        if fid == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        for fip in inv.get_idpath(fid):
+            print fip
+
+
+class cmd_reconcile(Command):
+    """Reconcile bzr metadata in a branch.
+
+    This can correct data mismatches that may have been caused by
+    previous ghost operations or bzr upgrades. You should only
+    need to run this command if 'bzr check' or a bzr developer 
+    advises you to run it.
+
+    If a second branch is provided, cross-branch reconciliation is
+    also attempted, which will check that data like the tree root
+    id which was not present in very early bzr versions is represented
+    correctly in both branches.
+
+    At the same time it is run it may recompress data resulting in 
+    a potential saving in disk space or performance gain.
+
+    The branch *MUST* be on a listable system such as local disk or sftp.
+    """
+    takes_args = ['branch?']
+
+    def run(self, branch="."):
+        from bzrlib.reconcile import reconcile
+        dir = bzrlib.bzrdir.BzrDir.open(branch)
+        reconcile(dir)
+
+
+class cmd_revision_history(Command):
+    """Display list of revision ids on this branch."""
+    hidden = True
+    @display_command
+    def run(self):
+        branch = WorkingTree.open_containing(u'.')[0].branch
+        for patchid in branch.revision_history():
+            print patchid
+
+
+class cmd_ancestry(Command):
+    """List all revisions merged into this branch."""
+    hidden = True
+    @display_command
+    def run(self):
+        tree = WorkingTree.open_containing(u'.')[0]
+        b = tree.branch
+        # FIXME. should be tree.last_revision
+        for revision_id in b.repository.get_ancestry(b.last_revision()):
+            print revision_id
+
+
+class cmd_init(Command):
+    """Make a directory into a versioned branch.
+
+    Use this to create an empty branch, or before importing an
+    existing project.
+
+    Recipe for importing a tree of files:
+        cd ~/project
+        bzr init
+        bzr add .
+        bzr status
+        bzr commit -m 'imported project'
+    """
+    takes_args = ['location?']
+    takes_options = [
+                     Option('format', 
+                            help='Create a specific format rather than the'
+                                 ' current default format. Currently this '
+                                 ' option only accepts "metadir"',
+                            type=get_format_type),
+                     ]
+    def run(self, location=None, format=None):
+        from bzrlib.branch import Branch
+        if location is None:
+            location = u'.'
+        else:
+            # The path has to exist to initialize a
+            # branch inside of it.
+            # Just using os.mkdir, since I don't
+            # believe that we want to create a bunch of
+            # locations if the user supplies an extended path
+            if not os.path.exists(location):
+                os.mkdir(location)
+        try:
+            existing = bzrdir.BzrDir.open(location)
+        except NotBranchError:
+            bzrdir.BzrDir.create_branch_convenience(location, format=format)
+        else:
+            try:
+                existing.open_branch()
+            except NotBranchError:
+                existing.create_branch()
+                existing.create_workingtree()
+            else:
+                raise errors.AlreadyBranchError(location)
+
+
+class cmd_init_repository(Command):
+    """Create a shared repository to hold branches.
+
+    New branches created under the repository directory will store their revisions
+    in the repository, not in the branch directory, if the branch format supports
+    shared storage.
+
+    example:    
+        bzr init-repo repo
+        bzr init --format=metadir repo/trunk
+        cd repo/trunk
+        (add files here)
+    """
+    takes_args = ["location"] 
+    takes_options = [Option('format', 
+                            help='Use a specific format rather than the'
+                            ' current default format. Currently this'
+                            ' option only accepts "metadir" and "knit"'
+                            ' WARNING: the knit format is currently unstable'
+                            ' and only for experimental use.',
+                            type=get_format_type),
+                     Option('trees',
+                             help='Allows branches in repository to have'
+                             ' a working tree')]
+    aliases = ["init-repo"]
+    def run(self, location, format=None, trees=False):
+        from bzrlib.bzrdir import BzrDirMetaFormat1
+        from bzrlib.transport import get_transport
+        if format is None:
+            format = BzrDirMetaFormat1()
+        transport = get_transport(location)
+        if not transport.has('.'):
+            transport.mkdir('')
+        newdir = format.initialize_on_transport(transport)
+        repo = newdir.create_repository(shared=True)
+        repo.set_make_working_trees(trees)
+
+
+class cmd_diff(Command):
+    """Show differences in working tree.
+    
+    If files are listed, only the changes in those files are listed.
+    Otherwise, all changes for the tree are listed.
+
+    examples:
+        bzr diff
+        bzr diff -r1
+        bzr diff -r1..2
+    """
+    # TODO: Allow diff across branches.
+    # TODO: Option to use external diff command; could be GNU diff, wdiff,
+    #       or a graphical diff.
+
+    # TODO: Python difflib is not exactly the same as unidiff; should
+    #       either fix it up or prefer to use an external diff.
+
+    # TODO: If a directory is given, diff everything under that.
+
+    # TODO: Selected-file diff is inefficient and doesn't show you
+    #       deleted files.
+
+    # TODO: This probably handles non-Unix newlines poorly.
+    
+    takes_args = ['file*']
+    takes_options = ['revision', 'diff-options']
+    aliases = ['di', 'dif']
+
+    @display_command
+    def run(self, revision=None, file_list=None, diff_options=None):
+        from bzrlib.diff import diff_cmd_helper, show_diff_trees
+        try:
+            tree1, file_list = internal_tree_files(file_list)
+            tree2 = None
+            b = None
+            b2 = None
+        except FileInWrongBranch:
+            if len(file_list) != 2:
+                raise BzrCommandError("Files are in different branches")
+
+            tree1, file1 = WorkingTree.open_containing(file_list[0])
+            tree2, file2 = WorkingTree.open_containing(file_list[1])
+            if file1 != "" or file2 != "":
+                # FIXME diff those two files. rbc 20051123
+                raise BzrCommandError("Files are in different branches")
+            file_list = None
+        if revision is not None:
+            if tree2 is not None:
+                raise BzrCommandError("Can't specify -r with two branches")
+            if (len(revision) == 1) or (revision[1].spec is None):
+                return diff_cmd_helper(tree1, file_list, diff_options,
+                                       revision[0])
+            elif len(revision) == 2:
+                return diff_cmd_helper(tree1, file_list, diff_options,
+                                       revision[0], revision[1])
+            else:
+                raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
+        else:
+            if tree2 is not None:
+                return show_diff_trees(tree1, tree2, sys.stdout, 
+                                       specific_files=file_list,
+                                       external_diff_options=diff_options)
+            else:
+                return diff_cmd_helper(tree1, file_list, diff_options)
+
+
+class cmd_deleted(Command):
+    """List files deleted in the working tree.
+    """
+    # TODO: Show files deleted since a previous revision, or
+    # between two revisions.
+    # TODO: Much more efficient way to do this: read in new
+    # directories with readdir, rather than stating each one.  Same
+    # level of effort but possibly much less IO.  (Or possibly not,
+    # if the directories are very large...)
+    @display_command
+    def run(self, show_ids=False):
+        tree = WorkingTree.open_containing(u'.')[0]
+        old = tree.basis_tree()
+        for path, ie in old.inventory.iter_entries():
+            if not tree.has_id(ie.file_id):
+                if show_ids:
+                    print '%-50s %s' % (path, ie.file_id)
+                else:
+                    print path
+
+
+class cmd_modified(Command):
+    """List files modified in working tree."""
+    hidden = True
+    @display_command
+    def run(self):
+        from bzrlib.delta import compare_trees
+
+        tree = WorkingTree.open_containing(u'.')[0]
+        td = compare_trees(tree.basis_tree(), tree)
+
+        for path, id, kind, text_modified, meta_modified in td.modified:
+            print path
+
+
+
+class cmd_added(Command):
+    """List files added in working tree."""
+    hidden = True
+    @display_command
+    def run(self):
+        wt = WorkingTree.open_containing(u'.')[0]
+        basis_inv = wt.basis_tree().inventory
+        inv = wt.inventory
+        for file_id in inv:
+            if file_id in basis_inv:
+                continue
+            path = inv.id2path(file_id)
+            if not os.access(bzrlib.osutils.abspath(path), os.F_OK):
+                continue
+            print path
+                
+        
+
+class cmd_root(Command):
+    """Show the tree root directory.
+
+    The root is the nearest enclosing directory with a .bzr control
+    directory."""
+    takes_args = ['filename?']
+    @display_command
+    def run(self, filename=None):
+        """Print the branch root."""
+        tree = WorkingTree.open_containing(filename)[0]
+        print tree.basedir
+
+
+class cmd_log(Command):
+    """Show log of a branch, file, or directory.
+
+    By default show the log of the branch containing the working directory.
+
+    To request a range of logs, you can use the command -r begin..end
+    -r revision requests a specific revision, -r ..end or -r begin.. are
+    also valid.
+
+    examples:
+        bzr log
+        bzr log foo.c
+        bzr log -r -10.. http://server/branch
+    """
+
+    # TODO: Make --revision support uuid: and hash: [future tag:] notation.
+
+    takes_args = ['location?']
+    takes_options = [Option('forward', 
+                            help='show from oldest to newest'),
+                     'timezone', 
+                     Option('verbose', 
+                             help='show files changed in each revision'),
+                     'show-ids', 'revision',
+                     'log-format',
+                     'line', 'long', 
+                     Option('message',
+                            help='show revisions whose message matches this regexp',
+                            type=str),
+                     'short',
+                     ]
+    @display_command
+    def run(self, location=None, timezone='original',
+            verbose=False,
+            show_ids=False,
+            forward=False,
+            revision=None,
+            log_format=None,
+            message=None,
+            long=False,
+            short=False,
+            line=False):
+        from bzrlib.log import log_formatter, show_log
+        import codecs
+        assert message is None or isinstance(message, basestring), \
+            "invalid message argument %r" % message
+        direction = (forward and 'forward') or 'reverse'
+        
+        # log everything
+        file_id = None
+        if location:
+            # find the file id to log:
+
+            dir, fp = bzrdir.BzrDir.open_containing(location)
+            b = dir.open_branch()
+            if fp != '':
+                try:
+                    # might be a tree:
+                    inv = dir.open_workingtree().inventory
+                except (errors.NotBranchError, errors.NotLocalUrl):
+                    # either no tree, or is remote.
+                    inv = b.basis_tree().inventory
+                file_id = inv.path2id(fp)
+        else:
+            # local dir only
+            # FIXME ? log the current subdir only RBC 20060203 
+            dir, relpath = bzrdir.BzrDir.open_containing('.')
+            b = dir.open_branch()
+
+        if revision is None:
+            rev1 = None
+            rev2 = None
+        elif len(revision) == 1:
+            rev1 = rev2 = revision[0].in_history(b).revno
+        elif len(revision) == 2:
+            if revision[0].spec is None:
+                # missing begin-range means first revision
+                rev1 = 1
+            else:
+                rev1 = revision[0].in_history(b).revno
+
+            if revision[1].spec is None:
+                # missing end-range means last known revision
+                rev2 = b.revno()
+            else:
+                rev2 = revision[1].in_history(b).revno
+        else:
+            raise BzrCommandError('bzr log --revision takes one or two values.')
+
+        # By this point, the revision numbers are converted to the +ve
+        # form if they were supplied in the -ve form, so we can do
+        # this comparison in relative safety
+        if rev1 > rev2:
+            (rev2, rev1) = (rev1, rev2)
+
+        mutter('encoding log as %r', bzrlib.user_encoding)
+
+        # use 'replace' so that we don't abort if trying to write out
+        # in e.g. the default C locale.
+        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+
+        if (log_format == None):
+            default = bzrlib.config.BranchConfig(b).log_format()
+            log_format = get_log_format(long=long, short=short, line=line, default=default)
+
+        lf = log_formatter(log_format,
+                           show_ids=show_ids,
+                           to_file=outf,
+                           show_timezone=timezone)
+
+        show_log(b,
+                 lf,
+                 file_id,
+                 verbose=verbose,
+                 direction=direction,
+                 start_revision=rev1,
+                 end_revision=rev2,
+                 search=message)
+
+
+def get_log_format(long=False, short=False, line=False, default='long'):
+    log_format = default
+    if long:
+        log_format = 'long'
+    if short:
+        log_format = 'short'
+    if line:
+        log_format = 'line'
+    return log_format
+
+
+class cmd_touching_revisions(Command):
+    """Return revision-ids which affected a particular file.
+
+    A more user-friendly interface is "bzr log FILE"."""
+    hidden = True
+    takes_args = ["filename"]
+    @display_command
+    def run(self, filename):
+        tree, relpath = WorkingTree.open_containing(filename)
+        b = tree.branch
+        inv = tree.read_working_inventory()
+        file_id = inv.path2id(relpath)
+        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
+            print "%6d %s" % (revno, what)
+
+
+class cmd_ls(Command):
+    """List files in a tree.
+    """
+    # TODO: Take a revision or remote path and list that tree instead.
+    hidden = True
+    takes_options = ['verbose', 'revision',
+                     Option('non-recursive',
+                            help='don\'t recurse into sub-directories'),
+                     Option('from-root',
+                            help='Print all paths from the root of the branch.'),
+                     Option('unknown', help='Print unknown files'),
+                     Option('versioned', help='Print versioned files'),
+                     Option('ignored', help='Print ignored files'),
+
+                     Option('null', help='Null separate the files'),
+                    ]
+    @display_command
+    def run(self, revision=None, verbose=False, 
+            non_recursive=False, from_root=False,
+            unknown=False, versioned=False, ignored=False,
+            null=False):
+
+        if verbose and null:
+            raise BzrCommandError('Cannot set both --verbose and --null')
+        all = not (unknown or versioned or ignored)
+
+        selection = {'I':ignored, '?':unknown, 'V':versioned}
+
+        tree, relpath = WorkingTree.open_containing(u'.')
+        if from_root:
+            relpath = u''
+        elif relpath:
+            relpath += '/'
+        if revision is not None:
+            tree = tree.branch.repository.revision_tree(
+                revision[0].in_history(tree.branch).rev_id)
+        for fp, fc, kind, fid, entry in tree.list_files():
+            if fp.startswith(relpath):
+                fp = fp[len(relpath):]
+                if non_recursive and '/' in fp:
+                    continue
+                if not all and not selection[fc]:
+                    continue
+                if verbose:
+                    kindch = entry.kind_character()
+                    print '%-8s %s%s' % (fc, fp, kindch)
+                elif null:
+                    sys.stdout.write(fp)
+                    sys.stdout.write('\0')
+                    sys.stdout.flush()
+                else:
+                    print fp
+
+
+class cmd_unknowns(Command):
+    """List unknown files."""
+    @display_command
+    def run(self):
+        from bzrlib.osutils import quotefn
+        for f in WorkingTree.open_containing(u'.')[0].unknowns():
+            print quotefn(f)
+
+
+class cmd_ignore(Command):
+    """Ignore a command or pattern.
+
+    To remove patterns from the ignore list, edit the .bzrignore file.
+
+    If the pattern contains a slash, it is compared to the whole path
+    from the branch root.  Otherwise, it is compared to only the last
+    component of the path.  To match a file only in the root directory,
+    prepend './'.
+
+    Ignore patterns are case-insensitive on case-insensitive systems.
+
+    Note: wildcards must be quoted from the shell on Unix.
+
+    examples:
+        bzr ignore ./Makefile
+        bzr ignore '*.class'
+    """
+    # TODO: Complain if the filename is absolute
+    takes_args = ['name_pattern']
+    
+    def run(self, name_pattern):
+        from bzrlib.atomicfile import AtomicFile
+        import os.path
+
+        tree, relpath = WorkingTree.open_containing(u'.')
+        ifn = tree.abspath('.bzrignore')
+
+        if os.path.exists(ifn):
+            f = open(ifn, 'rt')
+            try:
+                igns = f.read().decode('utf-8')
+            finally:
+                f.close()
+        else:
+            igns = ''
+
+        # TODO: If the file already uses crlf-style termination, maybe
+        # we should use that for the newly added lines?
+
+        if igns and igns[-1] != '\n':
+            igns += '\n'
+        igns += name_pattern + '\n'
+
+        try:
+            f = AtomicFile(ifn, 'wt')
+            f.write(igns.encode('utf-8'))
+            f.commit()
+        finally:
+            f.close()
+
+        inv = tree.inventory
+        if inv.path2id('.bzrignore'):
+            mutter('.bzrignore is already versioned')
+        else:
+            mutter('need to make new .bzrignore file versioned')
+            tree.add(['.bzrignore'])
+
+
+class cmd_ignored(Command):
+    """List ignored files and the patterns that matched them.
+
+    See also: bzr ignore"""
+    @display_command
+    def run(self):
+        tree = WorkingTree.open_containing(u'.')[0]
+        for path, file_class, kind, file_id, entry in tree.list_files():
+            if file_class != 'I':
+                continue
+            ## XXX: Slightly inefficient since this was already calculated
+            pat = tree.is_ignored(path)
+            print '%-50s %s' % (path, pat)
+
+
+class cmd_lookup_revision(Command):
+    """Lookup the revision-id from a revision-number
+
+    example:
+        bzr lookup-revision 33
+    """
+    hidden = True
+    takes_args = ['revno']
+    
+    @display_command
+    def run(self, revno):
+        try:
+            revno = int(revno)
+        except ValueError:
+            raise BzrCommandError("not a valid revision-number: %r" % revno)
+
+        print WorkingTree.open_containing(u'.')[0].branch.get_rev_id(revno)
+
+
+class cmd_export(Command):
+    """Export past revision to destination directory.
+
+    If no revision is specified this exports the last committed revision.
+
+    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
+    given, try to find the format with the extension. If no extension
+    is found exports to a directory (equivalent to --format=dir).
+
+    Root may be the top directory for tar, tgz and tbz2 formats. If none
+    is given, the top directory will be the root name of the file.
+
+    Note: export of tree with non-ascii filenames to zip is not supported.
+
+     Supported formats       Autodetected by extension
+     -----------------       -------------------------
+         dir                            -
+         tar                          .tar
+         tbz2                    .tar.bz2, .tbz2
+         tgz                      .tar.gz, .tgz
+         zip                          .zip
+    """
+    takes_args = ['dest']
+    takes_options = ['revision', 'format', 'root']
+    def run(self, dest, revision=None, format=None, root=None):
+        import os.path
+        from bzrlib.export import export
+        tree = WorkingTree.open_containing(u'.')[0]
+        b = tree.branch
+        if revision is None:
+            # should be tree.last_revision  FIXME
+            rev_id = b.last_revision()
+        else:
+            if len(revision) != 1:
+                raise BzrError('bzr export --revision takes exactly 1 argument')
+            rev_id = revision[0].in_history(b).rev_id
+        t = b.repository.revision_tree(rev_id)
+        try:
+            export(t, dest, format, root)
+        except errors.NoSuchExportFormat, e:
+            raise BzrCommandError('Unsupported export format: %s' % e.format)
+
+
+class cmd_cat(Command):
+    """Write a file's text from a previous revision."""
+
+    takes_options = ['revision']
+    takes_args = ['filename']
+
+    @display_command
+    def run(self, filename, revision=None):
+        if revision is not None and len(revision) != 1:
+            raise BzrCommandError("bzr cat --revision takes exactly one number")
+        tree = None
+        try:
+            tree, relpath = WorkingTree.open_containing(filename)
+            b = tree.branch
+        except NotBranchError:
+            pass
+
+        if tree is None:
+            b, relpath = Branch.open_containing(filename)
+        if revision is None:
+            revision_id = b.last_revision()
+        else:
+            revision_id = revision[0].in_history(b).rev_id
+        b.print_file(relpath, revision_id)
+
+
+class cmd_local_time_offset(Command):
+    """Show the offset in seconds from GMT to local time."""
+    hidden = True    
+    @display_command
+    def run(self):
+        print bzrlib.osutils.local_time_offset()
+
+
+
+class cmd_commit(Command):
+    """Commit changes into a new revision.
+    
+    If no arguments are given, the entire tree is committed.
+
+    If selected files are specified, only changes to those files are
+    committed.  If a directory is specified then the directory and everything 
+    within it is committed.
+
+    A selected-file commit may fail in some cases where the committed
+    tree would be invalid, such as trying to commit a file in a
+    newly-added directory that is not itself committed.
+    """
+    # TODO: Run hooks on tree to-be-committed, and after commit.
+
+    # TODO: Strict commit that fails if there are deleted files.
+    #       (what does "deleted files" mean ??)
+
+    # TODO: Give better message for -s, --summary, used by tla people
+
+    # XXX: verbose currently does nothing
+
+    takes_args = ['selected*']
+    takes_options = ['message', 'verbose', 
+                     Option('unchanged',
+                            help='commit even if nothing has changed'),
+                     Option('file', type=str, 
+                            argname='msgfile',
+                            help='file containing commit message'),
+                     Option('strict',
+                            help="refuse to commit if there are unknown "
+                            "files in the working tree."),
+                     Option('local',
+                            help="perform a local only commit in a bound "
+                                 "branch. Such commits are not pushed to "
+                                 "the master branch until a normal commit "
+                                 "is performed."
+                            ),
+                     ]
+    aliases = ['ci', 'checkin']
+
+    def run(self, message=None, file=None, verbose=True, selected_list=None,
+            unchanged=False, strict=False, local=False):
+        from bzrlib.commit import (NullCommitReporter, ReportCommitToLog)
+        from bzrlib.errors import (PointlessCommit, ConflictsInTree,
+                StrictCommitFailed)
+        from bzrlib.msgeditor import edit_commit_message, \
+                make_commit_message_template
+        from tempfile import TemporaryFile
+        import codecs
+
+        # TODO: Need a blackbox test for invoking the external editor; may be
+        # slightly problematic to run this cross-platform.
+
+        # TODO: do more checks that the commit will succeed before 
+        # spending the user's valuable time typing a commit message.
+        #
+        # TODO: if the commit *does* happen to fail, then save the commit 
+        # message to a temporary file where it can be recovered
+        tree, selected_list = tree_files(selected_list)
+        if local and not tree.branch.get_bound_location():
+            raise errors.LocalRequiresBoundBranch()
+        if message is None and not file:
+            template = make_commit_message_template(tree, selected_list)
+            message = edit_commit_message(template)
+            if message is None:
+                raise BzrCommandError("please specify a commit message"
+                                      " with either --message or --file")
+        elif message and file:
+            raise BzrCommandError("please specify either --message or --file")
+        
+        if file:
+            import codecs
+            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
+
+        if message == "":
+                raise BzrCommandError("empty commit message specified")
+        
+        if verbose:
+            reporter = ReportCommitToLog()
+        else:
+            reporter = NullCommitReporter()
+        
+        try:
+            tree.commit(message, specific_files=selected_list,
+                        allow_pointless=unchanged, strict=strict, local=local,
+                        reporter=reporter)
+        except PointlessCommit:
+            # FIXME: This should really happen before the file is read in;
+            # perhaps prepare the commit; get the message; then actually commit
+            raise BzrCommandError("no changes to commit",
+                                  ["use --unchanged to commit anyhow"])
+        except ConflictsInTree:
+            raise BzrCommandError("Conflicts detected in working tree.  "
+                'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+        except StrictCommitFailed:
+            raise BzrCommandError("Commit refused because there are unknown "
+                                  "files in the working tree.")
+        except errors.BoundBranchOutOfDate, e:
+            raise BzrCommandError(str(e)
+                                  + ' Either unbind, update, or'
+                                    ' pass --local to commit.')
+
+
+class cmd_check(Command):
+    """Validate consistency of branch history.
+
+    This command checks various invariants about the branch storage to
+    detect data corruption or bzr bugs.
+    """
+    takes_args = ['branch?']
+    takes_options = ['verbose']
+
+    def run(self, branch=None, verbose=False):
+        from bzrlib.check import check
+        if branch is None:
+            tree = WorkingTree.open_containing()[0]
+            branch = tree.branch
+        else:
+            branch = Branch.open(branch)
+        check(branch, verbose)
+
+
+class cmd_scan_cache(Command):
+    hidden = True
+    def run(self):
+        from bzrlib.hashcache import HashCache
+
+        c = HashCache(u'.')
+        c.read()
+        c.scan()
+            
+        print '%6d stats' % c.stat_count
+        print '%6d in hashcache' % len(c._cache)
+        print '%6d files removed from cache' % c.removed_count
+        print '%6d hashes updated' % c.update_count
+        print '%6d files changed too recently to cache' % c.danger_count
+
+        if c.needs_write:
+            c.write()
+
+
+class cmd_upgrade(Command):
+    """Upgrade branch storage to current format.
+
+    The check command or bzr developers may sometimes advise you to run
+    this command. When the default format has changed you may also be warned
+    during other operations to upgrade.
+    """
+    takes_args = ['url?']
+    takes_options = [
+                     Option('format', 
+                            help='Upgrade to a specific format rather than the'
+                                 ' current default format. Currently this'
+                                 ' option only accepts "metadir" and "knit".'
+                                 ' WARNING: the knit format is currently'
+                                 ' unstable and only for experimental use.',
+                            type=get_format_type),
+                    ]
+
+
+    def run(self, url='.', format=None):
+        from bzrlib.upgrade import upgrade
+        upgrade(url, format)
+
+
+class cmd_whoami(Command):
+    """Show bzr user id."""
+    takes_options = ['email']
+    
+    @display_command
+    def run(self, email=False):
+        try:
+            b = WorkingTree.open_containing(u'.')[0].branch
+            config = bzrlib.config.BranchConfig(b)
+        except NotBranchError:
+            config = bzrlib.config.GlobalConfig()
+        
+        if email:
+            print config.user_email()
+        else:
+            print config.username()
+
+
+class cmd_nick(Command):
+    """Print or set the branch nickname.  
+
+    If unset, the tree root directory name is used as the nickname
+    To print the current nickname, execute with no argument.  
+    """
+    takes_args = ['nickname?']
+    def run(self, nickname=None):
+        branch = Branch.open_containing(u'.')[0]
+        if nickname is None:
+            self.printme(branch)
+        else:
+            branch.nick = nickname
+
+    @display_command
+    def printme(self, branch):
+        print branch.nick 
+
+
+class cmd_selftest(Command):
+    """Run internal test suite.
+    
+    This creates temporary test directories in the working directory,
+    but not existing data is affected.  These directories are deleted
+    if the tests pass, or left behind to help in debugging if they
+    fail and --keep-output is specified.
+    
+    If arguments are given, they are regular expressions that say
+    which tests should run.
+
+    If the global option '--no-plugins' is given, plugins are not loaded
+    before running the selftests.  This has two effects: features provided or
+    modified by plugins will not be tested, and tests provided by plugins will
+    not be run.
+
+    examples:
+        bzr selftest ignore
+        bzr --no-plugins selftest -v
+    """
+    # TODO: --list should give a list of all available tests
+
+    # NB: this is used from the class without creating an instance, which is
+    # why it does not have a self parameter.
+    def get_transport_type(typestring):
+        """Parse and return a transport specifier."""
+        if typestring == "sftp":
+            from bzrlib.transport.sftp import SFTPAbsoluteServer
+            return SFTPAbsoluteServer
+        if typestring == "memory":
+            from bzrlib.transport.memory import MemoryServer
+            return MemoryServer
+        if typestring == "fakenfs":
+            from bzrlib.transport.fakenfs import FakeNFSServer
+            return FakeNFSServer
+        msg = "No known transport type %s. Supported types are: sftp\n" %\
+            (typestring)
+        raise BzrCommandError(msg)
+
+    hidden = True
+    takes_args = ['testspecs*']
+    takes_options = ['verbose',
+                     Option('one', help='stop when one test fails'),
+                     Option('keep-output', 
+                            help='keep output directories when tests fail'),
+                     Option('transport', 
+                            help='Use a different transport by default '
+                                 'throughout the test suite.',
+                            type=get_transport_type),
+                    ]
+
+    def run(self, testspecs_list=None, verbose=False, one=False,
+            keep_output=False, transport=None):
+        import bzrlib.ui
+        from bzrlib.tests import selftest
+        # we don't want progress meters from the tests to go to the
+        # real output; and we don't want log messages cluttering up
+        # the real logs.
+        save_ui = bzrlib.ui.ui_factory
+        bzrlib.trace.info('running tests...')
+        try:
+            bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+            if testspecs_list is not None:
+                pattern = '|'.join(testspecs_list)
+            else:
+                pattern = ".*"
+            result = selftest(verbose=verbose, 
+                              pattern=pattern,
+                              stop_on_failure=one, 
+                              keep_output=keep_output,
+                              transport=transport)
+            if result:
+                bzrlib.trace.info('tests passed')
+            else:
+                bzrlib.trace.info('tests failed')
+            return int(not result)
+        finally:
+            bzrlib.ui.ui_factory = save_ui
+
+
+def _get_bzr_branch():
+    """If bzr is run from a branch, return Branch or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    from bzrlib.osutils import abspath
+    from os.path import dirname
+    
+    try:
+        branch = Branch.open(dirname(abspath(dirname(__file__))))
+        return branch
+    except bzrlib.errors.BzrError:
+        return None
+    
+
+def show_version():
+    print "bzr (bazaar-ng) %s" % bzrlib.__version__
+    # is bzrlib itself in a branch?
+    branch = _get_bzr_branch()
+    if branch:
+        rh = branch.revision_history()
+        revno = len(rh)
+        print "  bzr checkout, revision %d" % (revno,)
+        print "  nick: %s" % (branch.nick,)
+        if rh:
+            print "  revid: %s" % (rh[-1],)
+    print bzrlib.__copyright__
+    print "http://bazaar-ng.org/"
+    print
+    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
+    print "you may use, modify and redistribute it under the terms of the GNU"
+    print "General Public License version 2 or later."
+
+
+class cmd_version(Command):
+    """Show version of bzr."""
+    @display_command
+    def run(self):
+        show_version()
+
+class cmd_rocks(Command):
+    """Statement of optimism."""
+    hidden = True
+    @display_command
+    def run(self):
+        print "it sure does!"
+
+
+class cmd_find_merge_base(Command):
+    """Find and print a base revision for merging two branches.
+    """
+    # TODO: Options to specify revisions on either side, as if
+    #       merging only part of the history.
+    takes_args = ['branch', 'other']
+    hidden = True
+    
+    @display_command
+    def run(self, branch, other):
+        from bzrlib.revision import common_ancestor, MultipleRevisionSources
+        
+        branch1 = Branch.open_containing(branch)[0]
+        branch2 = Branch.open_containing(other)[0]
+
+        history_1 = branch1.revision_history()
+        history_2 = branch2.revision_history()
+
+        last1 = branch1.last_revision()
+        last2 = branch2.last_revision()
+
+        source = MultipleRevisionSources(branch1.repository, 
+                                         branch2.repository)
+        
+        base_rev_id = common_ancestor(last1, last2, source)
+
+        print 'merge base is revision %s' % base_rev_id
+        
+        return
+
+        if base_revno is None:
+            raise bzrlib.errors.UnrelatedBranches()
+
+        print ' r%-6d in %s' % (base_revno, branch)
+
+        other_revno = branch2.revision_id_to_revno(base_revid)
+        
+        print ' r%-6d in %s' % (other_revno, other)
+
+
+
+class cmd_merge(Command):
+    """Perform a three-way merge.
+    
+    The branch is the branch you will merge from.  By default, it will
+    merge the latest revision.  If you specify a revision, that
+    revision will be merged.  If you specify two revisions, the first
+    will be used as a BASE, and the second one as OTHER.  Revision
+    numbers are always relative to the specified branch.
+
+    By default, bzr will try to merge in all new work from the other
+    branch, automatically determining an appropriate base.  If this
+    fails, you may need to give an explicit base.
+    
+    Merge will do its best to combine the changes in two branches, but there
+    are some kinds of problems only a human can fix.  When it encounters those,
+    it will mark a conflict.  A conflict means that you need to fix something,
+    before you should commit.
+
+    Use bzr resolve when you have fixed a problem.  See also bzr conflicts.
+
+    If there is no default branch set, the first merge will set it. After
+    that, you can omit the branch to use the default.  To change the
+    default, use --remember.
+
+    Examples:
+
+    To merge the latest revision from bzr.dev
+    bzr merge ../bzr.dev
+
+    To merge changes up to and including revision 82 from bzr.dev
+    bzr merge -r 82 ../bzr.dev
+
+    To merge the changes introduced by 82, without previous changes:
+    bzr merge -r 81..82 ../bzr.dev
+    
+    merge refuses to run if there are any uncommitted changes, unless
+    --force is given.
+    """
+    takes_args = ['branch?']
+    takes_options = ['revision', 'force', 'merge-type', 'reprocess', 'remember',
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
+
+    def run(self, branch=None, revision=None, force=False, merge_type=None,
+            show_base=False, reprocess=False, remember=False):
+        if merge_type is None:
+            merge_type = Merge3Merger
+
+        tree = WorkingTree.open_containing(u'.')[0]
+        stored_loc = tree.branch.get_parent()
+        if branch is None:
+            if stored_loc is None:
+                raise BzrCommandError("No merge branch known or specified.")
+            else:
+                print "Using saved branch: %s" % stored_loc
+                branch = stored_loc
+
+        if tree.branch.get_parent() is None or remember:
+            tree.branch.set_parent(branch)
+
+        if revision is None or len(revision) < 1:
+            base = [None, None]
+            other = [branch, -1]
+            other_branch, path = Branch.open_containing(branch)
+        else:
+            if len(revision) == 1:
+                base = [None, None]
+                other_branch, path = Branch.open_containing(branch)
+                revno = revision[0].in_history(other_branch).revno
+                other = [branch, revno]
+            else:
+                assert len(revision) == 2
+                if None in revision:
+                    raise BzrCommandError(
+                        "Merge doesn't permit that revision specifier.")
+                b, path = Branch.open_containing(branch)
+
+                base = [branch, revision[0].in_history(b).revno]
+                other = [branch, revision[1].in_history(b).revno]
+        if path != "":
+            interesting_files = [path]
+        else:
+            interesting_files = None
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            try:
+                conflict_count = merge(other, base, check_clean=(not force),
+                                       merge_type=merge_type, 
+                                       reprocess=reprocess,
+                                       show_base=show_base, 
+                                       pb=pb, file_list=interesting_files)
+            finally:
+                pb.finished()
+            if conflict_count != 0:
+                return 1
+            else:
+                return 0
+        except bzrlib.errors.AmbiguousBase, e:
+            m = ("sorry, bzr can't determine the right merge base yet\n"
+                 "candidates are:\n  "
+                 + "\n  ".join(e.bases)
+                 + "\n"
+                 "please specify an explicit base with -r,\n"
+                 "and (if you want) report this to the bzr developers\n")
+            log_error(m)
+
+
+class cmd_remerge(Command):
+    """Redo a merge.
+    """
+    takes_args = ['file*']
+    takes_options = ['merge-type', 'reprocess',
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
+
+    def run(self, file_list=None, merge_type=None, show_base=False,
+            reprocess=False):
+        from bzrlib.merge import merge_inner, transform_tree
+        if merge_type is None:
+            merge_type = Merge3Merger
+        tree, file_list = tree_files(file_list)
+        tree.lock_write()
+        try:
+            pending_merges = tree.pending_merges() 
+            if len(pending_merges) != 1:
+                raise BzrCommandError("Sorry, remerge only works after normal"
+                                      + " merges.  Not cherrypicking or"
+                                      + "multi-merges.")
+            repository = tree.branch.repository
+            base_revision = common_ancestor(tree.branch.last_revision(), 
+                                            pending_merges[0], repository)
+            base_tree = repository.revision_tree(base_revision)
+            other_tree = repository.revision_tree(pending_merges[0])
+            interesting_ids = None
+            if file_list is not None:
+                interesting_ids = set()
+                for filename in file_list:
+                    file_id = tree.path2id(filename)
+                    if file_id is None:
+                        raise NotVersionedError(filename)
+                    interesting_ids.add(file_id)
+                    if tree.kind(file_id) != "directory":
+                        continue
+                    
+                    for name, ie in tree.inventory.iter_entries(file_id):
+                        interesting_ids.add(ie.file_id)
+            transform_tree(tree, tree.basis_tree(), interesting_ids)
+            if file_list is None:
+                restore_files = list(tree.iter_conflicts())
+            else:
+                restore_files = file_list
+            for filename in restore_files:
+                try:
+                    restore(tree.abspath(filename))
+                except NotConflicted:
+                    pass
+            conflicts =  merge_inner(tree.branch, other_tree, base_tree,
+                                     this_tree=tree,
+                                     interesting_ids = interesting_ids, 
+                                     other_rev_id=pending_merges[0], 
+                                     merge_type=merge_type, 
+                                     show_base=show_base,
+                                     reprocess=reprocess)
+        finally:
+            tree.unlock()
+        if conflicts > 0:
+            return 1
+        else:
+            return 0
+
+class cmd_revert(Command):
+    """Reverse all changes since the last commit.
+
+    Only versioned files are affected.  Specify filenames to revert only 
+    those files.  By default, any files that are changed will be backed up
+    first.  Backup files have a '~' appended to their name.
+    """
+    takes_options = ['revision', 'no-backup']
+    takes_args = ['file*']
+    aliases = ['merge-revert']
+
+    def run(self, revision=None, no_backup=False, file_list=None):
+        from bzrlib.commands import parse_spec
+        if file_list is not None:
+            if len(file_list) == 0:
+                raise BzrCommandError("No files specified")
+        else:
+            file_list = []
+        
+        tree, file_list = tree_files(file_list)
+        if revision is None:
+            # FIXME should be tree.last_revision
+            rev_id = tree.last_revision()
+        elif len(revision) != 1:
+            raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
+        else:
+            rev_id = revision[0].in_history(tree.branch).rev_id
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            tree.revert(file_list, 
+                        tree.branch.repository.revision_tree(rev_id),
+                        not no_backup, pb)
+        finally:
+            pb.finished()
+
+
+class cmd_assert_fail(Command):
+    """Test reporting of assertion failures"""
+    hidden = True
+    def run(self):
+        assert False, "always fails"
+
+
+class cmd_help(Command):
+    """Show help on a command or other topic.
+
+    For a list of all available commands, say 'bzr help commands'."""
+    takes_options = [Option('long', 'show help on all commands')]
+    takes_args = ['topic?']
+    aliases = ['?', '--help', '-?', '-h']
+    
+    @display_command
+    def run(self, topic=None, long=False):
+        import help
+        if topic is None and long:
+            topic = "commands"
+        help.help(topic)
+
+
+class cmd_shell_complete(Command):
+    """Show appropriate completions for context.
+
+    For a list of all available commands, say 'bzr shell-complete'."""
+    takes_args = ['context?']
+    aliases = ['s-c']
+    hidden = True
+    
+    @display_command
+    def run(self, context=None):
+        import shellcomplete
+        shellcomplete.shellcomplete(context)
+
+
+class cmd_fetch(Command):
+    """Copy in history from another branch but don't merge it.
+
+    This is an internal method used for pull and merge."""
+    hidden = True
+    takes_args = ['from_branch', 'to_branch']
+    def run(self, from_branch, to_branch):
+        from bzrlib.fetch import Fetcher
+        from bzrlib.branch import Branch
+        from_b = Branch.open(from_branch)
+        to_b = Branch.open(to_branch)
+        Fetcher(to_b, from_b)
+
+
+class cmd_missing(Command):
+    """Show unmerged/unpulled revisions between two branches.
+
+    OTHER_BRANCH may be local or remote."""
+    takes_args = ['other_branch?']
+    takes_options = [Option('reverse', 'Reverse the order of revisions'),
+                     Option('mine-only', 
+                            'Display changes in the local branch only'),
+                     Option('theirs-only', 
+                            'Display changes in the remote branch only'), 
+                     'log-format',
+                     'line',
+                     'long', 
+                     'short',
+                     'show-ids',
+                     'verbose'
+                     ]
+
+    def run(self, other_branch=None, reverse=False, mine_only=False,
+            theirs_only=False, log_format=None, long=False, short=False, line=False, 
+            show_ids=False, verbose=False):
+        from bzrlib.missing import find_unmerged, iter_log_data
+        from bzrlib.log import log_formatter
+        local_branch = bzrlib.branch.Branch.open_containing(u".")[0]
+        parent = local_branch.get_parent()
+        if other_branch is None:
+            other_branch = parent
+            if other_branch is None:
+                raise BzrCommandError("No missing location known or specified.")
+            print "Using last location: " + local_branch.get_parent()
+        remote_branch = bzrlib.branch.Branch.open(other_branch)
+        if remote_branch.base == local_branch.base:
+            remote_branch = local_branch
+        local_branch.lock_read()
+        try:
+            remote_branch.lock_read()
+            try:
+                local_extra, remote_extra = find_unmerged(local_branch, remote_branch)
+                if (log_format == None):
+                    default = bzrlib.config.BranchConfig(local_branch).log_format()
+                    log_format = get_log_format(long=long, short=short, line=line, default=default)
+                lf = log_formatter(log_format, sys.stdout,
+                                   show_ids=show_ids,
+                                   show_timezone='original')
+                if reverse is False:
+                    local_extra.reverse()
+                    remote_extra.reverse()
+                if local_extra and not theirs_only:
+                    print "You have %d extra revision(s):" % len(local_extra)
+                    for data in iter_log_data(local_extra, local_branch.repository,
+                                              verbose):
+                        lf.show(*data)
+                    printed_local = True
+                else:
+                    printed_local = False
+                if remote_extra and not mine_only:
+                    if printed_local is True:
+                        print "\n\n"
+                    print "You are missing %d revision(s):" % len(remote_extra)
+                    for data in iter_log_data(remote_extra, remote_branch.repository, 
+                                              verbose):
+                        lf.show(*data)
+                if not remote_extra and not local_extra:
+                    status_code = 0
+                    print "Branches are up to date."
+                else:
+                    status_code = 1
+            finally:
+                remote_branch.unlock()
+        finally:
+            local_branch.unlock()
+        if not status_code and parent is None and other_branch is not None:
+            local_branch.lock_write()
+            try:
+                # handle race conditions - a parent might be set while we run.
+                if local_branch.get_parent() is None:
+                    local_branch.set_parent(other_branch)
+            finally:
+                local_branch.unlock()
+        return status_code
+
+
+class cmd_plugins(Command):
+    """List plugins"""
+    hidden = True
+    @display_command
+    def run(self):
+        import bzrlib.plugin
+        from inspect import getdoc
+        for name, plugin in bzrlib.plugin.all_plugins().items():
+            if hasattr(plugin, '__path__'):
+                print plugin.__path__[0]
+            elif hasattr(plugin, '__file__'):
+                print plugin.__file__
+            else:
+                print `plugin`
+                
+            d = getdoc(plugin)
+            if d:
+                print '\t', d.split('\n')[0]
+
+
+class cmd_testament(Command):
+    """Show testament (signing-form) of a revision."""
+    takes_options = ['revision', 'long']
+    takes_args = ['branch?']
+    @display_command
+    def run(self, branch=u'.', revision=None, long=False):
+        from bzrlib.testament import Testament
+        b = WorkingTree.open_containing(branch)[0].branch
+        b.lock_read()
+        try:
+            if revision is None:
+                rev_id = b.last_revision()
+            else:
+                rev_id = revision[0].in_history(b).rev_id
+            t = Testament.from_revision(b.repository, rev_id)
+            if long:
+                sys.stdout.writelines(t.as_text_lines())
+            else:
+                sys.stdout.write(t.as_short_text())
+        finally:
+            b.unlock()
+
+
+class cmd_annotate(Command):
+    """Show the origin of each line in a file.
+
+    This prints out the given file with an annotation on the left side
+    indicating which revision, author and date introduced the change.
+
+    If the origin is the same for a run of consecutive lines, it is 
+    shown only at the top, unless the --all option is given.
+    """
+    # TODO: annotate directories; showing when each file was last changed
+    # TODO: annotate a previous version of a file
+    # TODO: if the working copy is modified, show annotations on that 
+    #       with new uncommitted lines marked
+    aliases = ['blame', 'praise']
+    takes_args = ['filename']
+    takes_options = [Option('all', help='show annotations on all lines'),
+                     Option('long', help='show date in annotations'),
+                     ]
+
+    @display_command
+    def run(self, filename, all=False, long=False):
+        from bzrlib.annotate import annotate_file
+        tree, relpath = WorkingTree.open_containing(filename)
+        branch = tree.branch
+        branch.lock_read()
+        try:
+            file_id = tree.inventory.path2id(relpath)
+            tree = branch.repository.revision_tree(branch.last_revision())
+            file_version = tree.inventory[file_id].revision
+            annotate_file(branch, file_version, file_id, long, all, sys.stdout)
+        finally:
+            branch.unlock()
+
+
+class cmd_re_sign(Command):
+    """Create a digital signature for an existing revision."""
+    # TODO be able to replace existing ones.
+
+    hidden = True # is this right ?
+    takes_args = ['revision_id*']
+    takes_options = ['revision']
+    
+    def run(self, revision_id_list=None, revision=None):
+        import bzrlib.config as config
+        import bzrlib.gpg as gpg
+        if revision_id_list is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id_list is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = WorkingTree.open_containing(u'.')[0].branch
+        gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
+        if revision_id_list is not None:
+            for revision_id in revision_id_list:
+                b.repository.sign_revision(revision_id, gpg_strategy)
+        elif revision is not None:
+            if len(revision) == 1:
+                revno, rev_id = revision[0].in_history(b)
+                b.repository.sign_revision(rev_id, gpg_strategy)
+            elif len(revision) == 2:
+                # are they both on rh- if so we can walk between them
+                # might be nice to have a range helper for arbitrary
+                # revision paths. hmm.
+                from_revno, from_revid = revision[0].in_history(b)
+                to_revno, to_revid = revision[1].in_history(b)
+                if to_revid is None:
+                    to_revno = b.revno()
+                if from_revno is None or to_revno is None:
+                    raise BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                for revno in range(from_revno, to_revno + 1):
+                    b.repository.sign_revision(b.get_rev_id(revno), 
+                                               gpg_strategy)
+            else:
+                raise BzrCommandError('Please supply either one revision, or a range.')
+
+
+class cmd_bind(Command):
+    """Bind the current branch to a master branch.
+
+    After binding, commits must succeed on the master branch
+    before they are executed on the local one.
+    """
+
+    takes_args = ['location']
+    takes_options = []
+
+    def run(self, location=None):
+        b, relpath = Branch.open_containing(u'.')
+        b_other = Branch.open(location)
+        try:
+            b.bind(b_other)
+        except DivergedBranches:
+            raise BzrCommandError('These branches have diverged.'
+                                  ' Try merging, and then bind again.')
+
+
+class cmd_unbind(Command):
+    """Bind the current branch to its parent.
+
+    After unbinding, the local branch is considered independent.
+    """
+
+    takes_args = []
+    takes_options = []
+
+    def run(self):
+        b, relpath = Branch.open_containing(u'.')
+        if not b.unbind():
+            raise BzrCommandError('Local branch is not bound')
+
+
+class cmd_uncommit(bzrlib.commands.Command):
+    """Remove the last committed revision.
+
+    By supplying the --all flag, it will not only remove the entry 
+    from revision_history, but also remove all of the entries in the
+    stores.
+
+    --verbose will print out what is being removed.
+    --dry-run will go through all the motions, but not actually
+    remove anything.
+    
+    In the future, uncommit will create a changeset, which can then
+    be re-applied.
+    """
+
+    # TODO: jam 20060108 Add an option to allow uncommit to remove
+    # unreferenced information in 'branch-as-repostory' branches.
+    # TODO: jam 20060108 Add the ability for uncommit to remove unreferenced
+    # information in shared branches as well.
+    takes_options = ['verbose', 'revision',
+                    Option('dry-run', help='Don\'t actually make changes'),
+                    Option('force', help='Say yes to all questions.')]
+    takes_args = ['location?']
+    aliases = []
+
+    def run(self, location=None, 
+            dry_run=False, verbose=False,
+            revision=None, force=False):
+        from bzrlib.branch import Branch
+        from bzrlib.log import log_formatter
+        import sys
+        from bzrlib.uncommit import uncommit
+
+        if location is None:
+            location = u'.'
+        control, relpath = bzrdir.BzrDir.open_containing(location)
+        try:
+            tree = control.open_workingtree()
+            b = tree.branch
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            tree = None
+            b = control.open_branch()
+
+        if revision is None:
+            revno = b.revno()
+            rev_id = b.last_revision()
+        else:
+            revno, rev_id = revision[0].in_history(b)
+        if rev_id is None:
+            print 'No revisions to uncommit.'
+
+        for r in range(revno, b.revno()+1):
+            rev_id = b.get_rev_id(r)
+            lf = log_formatter('short', to_file=sys.stdout,show_timezone='original')
+            lf.show(r, b.repository.get_revision(rev_id), None)
+
+        if dry_run:
+            print 'Dry-run, pretending to remove the above revisions.'
+            if not force:
+                val = raw_input('Press <enter> to continue')
+        else:
+            print 'The above revision(s) will be removed.'
+            if not force:
+                val = raw_input('Are you sure [y/N]? ')
+                if val.lower() not in ('y', 'yes'):
+                    print 'Canceled'
+                    return 0
+
+        uncommit(b, tree=tree, dry_run=dry_run, verbose=verbose,
+                revno=revno)
+
+
+class cmd_break_lock(Command):
+    """Break a dead lock on a repository, branch or working directory.
+
+    CAUTION: Locks should only be broken when you are sure that the process
+    holding the lock has been stopped.
+    
+    example:
+        bzr break-lock .
+    """
+    takes_args = ['location']
+    takes_options = [Option('show',
+                            help="just show information on the lock, " \
+                                 "don't break it"),
+                    ]
+    def run(self, location, show=False):
+        raise NotImplementedError("sorry, break-lock is not complete yet; "
+                "you can remove the 'held' directory manually to break the lock")
+
+
+# command-line interpretation helper for merge-related commands
+def merge(other_revision, base_revision,
+          check_clean=True, ignore_zero=False,
+          this_dir=None, backup_files=False, merge_type=Merge3Merger,
+          file_list=None, show_base=False, reprocess=False,
+          pb=DummyProgress()):
+    """Merge changes into a tree.
+
+    base_revision
+        list(path, revno) Base for three-way merge.  
+        If [None, None] then a base will be automatically determined.
+    other_revision
+        list(path, revno) Other revision for three-way merge.
+    this_dir
+        Directory to merge changes into; '.' by default.
+    check_clean
+        If true, this_dir must have no uncommitted changes before the
+        merge begins.
+    ignore_zero - If true, suppress the "zero conflicts" message when 
+        there are no conflicts; should be set when doing something we expect
+        to complete perfectly.
+    file_list - If supplied, merge only changes to selected files.
+
+    All available ancestors of other_revision and base_revision are
+    automatically pulled into the branch.
+
+    The revno may be -1 to indicate the last revision on the branch, which is
+    the typical case.
+
+    This function is intended for use from the command line; programmatic
+    clients might prefer to call merge.merge_inner(), which has less magic 
+    behavior.
+    """
+    from bzrlib.merge import Merger
+    if this_dir is None:
+        this_dir = u'.'
+    this_tree = WorkingTree.open_containing(this_dir)[0]
+    if show_base and not merge_type is Merge3Merger:
+        raise BzrCommandError("Show-base is not supported for this merge"
+                              " type. %s" % merge_type)
+    if reprocess and not merge_type.supports_reprocess:
+        raise BzrCommandError("Conflict reduction is not supported for merge"
+                              " type %s." % merge_type)
+    if reprocess and show_base:
+        raise BzrCommandError("Cannot do conflict reduction and show base.")
+    try:
+        merger = Merger(this_tree.branch, this_tree=this_tree, pb=pb)
+        merger.pp = ProgressPhase("Merge phase", 5, pb)
+        merger.pp.next_phase()
+        merger.check_basis(check_clean)
+        merger.set_other(other_revision)
+        merger.pp.next_phase()
+        merger.set_base(base_revision)
+        if merger.base_rev_id == merger.other_rev_id:
+            note('Nothing to do.')
+            return 0
+        merger.backup_files = backup_files
+        merger.merge_type = merge_type 
+        merger.set_interesting_files(file_list)
+        merger.show_base = show_base 
+        merger.reprocess = reprocess
+        conflicts = merger.do_merge()
+        if file_list is None:
+            merger.set_pending()
+    finally:
+        pb.clear()
+    return conflicts
+
+
+# these get imported and then picked up by the scan for cmd_*
+# TODO: Some more consistent way to split command definitions across files;
+# we do need to load at least some information about them to know of 
+# aliases.  ideally we would avoid loading the implementation until the
+# details were needed.
+from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore
+from bzrlib.sign_my_commits import cmd_sign_my_commits
+from bzrlib.weave_commands import cmd_weave_list, cmd_weave_join, \
+        cmd_weave_plan_merge, cmd_weave_merge_text

=== added file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/bzrdir.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,1731 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""BzrDir logic. The BzrDir is the basic control directory used by bzr.
+
+At format 7 this was split out into Branch, Repository and Checkout control
+directories.
+"""
+
+from copy import deepcopy
+import os
+from cStringIO import StringIO
+from unittest import TestSuite
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
+from bzrlib.osutils import safe_unicode
+from bzrlib.osutils import (
+                            abspath,
+                            pathjoin,
+                            safe_unicode,
+                            sha_strings,
+                            sha_string,
+                            )
+from bzrlib.store.revision.text import TextRevisionStore
+from bzrlib.store.text import TextStore
+from bzrlib.store.versioned import WeaveStore
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter
+from bzrlib.transactions import WriteTransaction
+from bzrlib.transport import get_transport, urlunescape
+from bzrlib.transport.local import LocalTransport
+from bzrlib.weave import Weave
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
+
+
+class BzrDir(object):
+    """A .bzr control diretory.
+    
+    BzrDir instances let you create or open any of the things that can be
+    found within .bzr - checkouts, branches and repositories.
+    
+    transport
+        the transport which this bzr dir is rooted at (i.e. file:///.../.bzr/)
+    root_transport
+        a transport connected to the directory this bzr was opened from.
+    """
+
+    def can_convert_format(self):
+        """Return true if this bzrdir is one whose format we can convert from."""
+        return True
+
+    @staticmethod
+    def _check_supported(format, allow_unsupported):
+        """Check whether format is a supported format.
+
+        If allow_unsupported is True, this is a no-op.
+        """
+        if not allow_unsupported and not format.is_supported():
+            # see open_downlevel to open legacy branches.
+            raise errors.UnsupportedFormatError(
+                    'sorry, format %s not supported' % format,
+                    ['use a different bzr version',
+                     'or remove the .bzr directory'
+                     ' and "bzr init" again'])
+
+    def clone(self, url, revision_id=None, basis=None, force_new_repo=False):
+        """Clone this bzrdir and its contents to url verbatim.
+
+        If urls last component does not exist, it will be created.
+
+        if revision_id is not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target 
+                               even if one is available.
+        """
+        self._make_tail(url)
+        basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
+        result = self._format.initialize(url)
+        try:
+            local_repo = self.find_repository()
+        except errors.NoRepositoryPresent:
+            local_repo = None
+        if local_repo:
+            # may need to copy content in
+            if force_new_repo:
+                local_repo.clone(result, revision_id=revision_id, basis=basis_repo)
+            else:
+                try:
+                    result_repo = result.find_repository()
+                    # fetch content this dir needs.
+                    if basis_repo:
+                        # XXX FIXME RBC 20060214 need tests for this when the basis
+                        # is incomplete
+                        result_repo.fetch(basis_repo, revision_id=revision_id)
+                    result_repo.fetch(local_repo, revision_id=revision_id)
+                except errors.NoRepositoryPresent:
+                    # needed to make one anyway.
+                    local_repo.clone(result, revision_id=revision_id, basis=basis_repo)
+        # 1 if there is a branch present
+        #   make sure its content is available in the target repository
+        #   clone it.
+        try:
+            self.open_branch().clone(result, revision_id=revision_id)
+        except errors.NotBranchError:
+            pass
+        try:
+            self.open_workingtree().clone(result, basis=basis_tree)
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            pass
+        return result
+
+    def _get_basis_components(self, basis):
+        """Retrieve the basis components that are available at basis."""
+        if basis is None:
+            return None, None, None
+        try:
+            basis_tree = basis.open_workingtree()
+            basis_branch = basis_tree.branch
+            basis_repo = basis_branch.repository
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            basis_tree = None
+            try:
+                basis_branch = basis.open_branch()
+                basis_repo = basis_branch.repository
+            except errors.NotBranchError:
+                basis_branch = None
+                try:
+                    basis_repo = basis.open_repository()
+                except errors.NoRepositoryPresent:
+                    basis_repo = None
+        return basis_repo, basis_branch, basis_tree
+
+    def _make_tail(self, url):
+        segments = url.split('/')
+        if segments and segments[-1] not in ('', '.'):
+            parent = '/'.join(segments[:-1])
+            t = bzrlib.transport.get_transport(parent)
+            try:
+                t.mkdir(segments[-1])
+            except errors.FileExists:
+                pass
+
+    @classmethod
+    def create(cls, base):
+        """Create a new BzrDir at the url 'base'.
+        
+        This will call the current default formats initialize with base
+        as the only parameter.
+
+        If you need a specific format, consider creating an instance
+        of that and calling initialize().
+        """
+        if cls is not BzrDir:
+            raise AssertionError("BzrDir.create always creates the default format, "
+                    "not one of %r" % cls)
+        segments = base.split('/')
+        if segments and segments[-1] not in ('', '.'):
+            parent = '/'.join(segments[:-1])
+            t = bzrlib.transport.get_transport(parent)
+            try:
+                t.mkdir(segments[-1])
+            except errors.FileExists:
+                pass
+        return BzrDirFormat.get_default_format().initialize(safe_unicode(base))
+
+    def create_branch(self):
+        """Create a branch in this BzrDir.
+
+        The bzrdirs format will control what branch format is created.
+        For more control see BranchFormatXX.create(a_bzrdir).
+        """
+        raise NotImplementedError(self.create_branch)
+
+    @staticmethod
+    def create_branch_and_repo(base, force_new_repo=False):
+        """Create a new BzrDir, Branch and Repository at the url 'base'.
+
+        This will use the current default BzrDirFormat, and use whatever 
+        repository format that that uses via bzrdir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially.
+
+        The created Branch object is returned.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        """
+        bzrdir = BzrDir.create(base)
+        bzrdir._find_or_create_repository(force_new_repo)
+        return bzrdir.create_branch()
+
+    def _find_or_create_repository(self, force_new_repo):
+        """Create a new repository if needed, returning the repository."""
+        if force_new_repo:
+            return self.create_repository()
+        try:
+            return self.find_repository()
+        except errors.NoRepositoryPresent:
+            return self.create_repository()
+        
+    @staticmethod
+    def create_branch_convenience(base, force_new_repo=False,
+                                  force_new_tree=None, format=None):
+        """Create a new BzrDir, Branch and Repository at the url 'base'.
+
+        This is a convenience function - it will use an existing repository
+        if possible, can be told explicitly whether to create a working tree or
+        not.
+
+        This will use the current default BzrDirFormat, and use whatever 
+        repository format that that uses via bzrdir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially. Whatever repository is used, its tree creation policy
+        is followed.
+
+        The created Branch object is returned.
+        If a working tree cannot be made due to base not being a file:// url,
+        no error is raised unless force_new_tree is True, in which case no 
+        data is created on disk and NotLocalUrl is raised.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        :param force_new_tree: If True or False force creation of a tree or 
+                               prevent such creation respectively.
+        :param format: Override for the for the bzrdir format to create
+        """
+        if force_new_tree:
+            # check for non local urls
+            t = get_transport(safe_unicode(base))
+            if not isinstance(t, LocalTransport):
+                raise errors.NotLocalUrl(base)
+        if format is None:
+            bzrdir = BzrDir.create(base)
+        else:
+            bzrdir = format.initialize(base)
+        repo = bzrdir._find_or_create_repository(force_new_repo)
+        result = bzrdir.create_branch()
+        if force_new_tree or (repo.make_working_trees() and 
+                              force_new_tree is None):
+            try:
+                bzrdir.create_workingtree()
+            except errors.NotLocalUrl:
+                pass
+        return result
+        
+    @staticmethod
+    def create_repository(base, shared=False):
+        """Create a new BzrDir and Repository at the url 'base'.
+
+        This will use the current default BzrDirFormat, and use whatever 
+        repository format that that uses for bzrdirformat.create_repository.
+
+        ;param shared: Create a shared repository rather than a standalone
+                       repository.
+        The Repository object is returned.
+
+        This must be overridden as an instance method in child classes, where
+        it should take no parameters and construct whatever repository format
+        that child class desires.
+        """
+        bzrdir = BzrDir.create(base)
+        return bzrdir.create_repository()
+
+    @staticmethod
+    def create_standalone_workingtree(base):
+        """Create a new BzrDir, WorkingTree, Branch and Repository at 'base'.
+
+        'base' must be a local path or a file:// url.
+
+        This will use the current default BzrDirFormat, and use whatever 
+        repository format that that uses for bzrdirformat.create_workingtree,
+        create_branch and create_repository.
+
+        The WorkingTree object is returned.
+        """
+        t = get_transport(safe_unicode(base))
+        if not isinstance(t, LocalTransport):
+            raise errors.NotLocalUrl(base)
+        bzrdir = BzrDir.create_branch_and_repo(safe_unicode(base),
+                                               force_new_repo=True).bzrdir
+        return bzrdir.create_workingtree()
+
+    def create_workingtree(self, revision_id=None):
+        """Create a working tree at this BzrDir.
+        
+        revision_id: create it as of this revision id.
+        """
+        raise NotImplementedError(self.create_workingtree)
+
+    def find_repository(self):
+        """Find the repository that should be used for a_bzrdir.
+
+        This does not require a branch as we use it to find the repo for
+        new branches as well as to hook existing branches up to their
+        repository.
+        """
+        try:
+            return self.open_repository()
+        except errors.NoRepositoryPresent:
+            pass
+        next_transport = self.root_transport.clone('..')
+        while True:
+            try:
+                found_bzrdir = BzrDir.open_containing_from_transport(
+                    next_transport)[0]
+            except errors.NotBranchError:
+                raise errors.NoRepositoryPresent(self)
+            try:
+                repository = found_bzrdir.open_repository()
+            except errors.NoRepositoryPresent:
+                next_transport = found_bzrdir.root_transport.clone('..')
+                continue
+            if ((found_bzrdir.root_transport.base == 
+                 self.root_transport.base) or repository.is_shared()):
+                return repository
+            else:
+                raise errors.NoRepositoryPresent(self)
+        raise errors.NoRepositoryPresent(self)
+
+    def get_branch_transport(self, branch_format):
+        """Get the transport for use by branch format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the branch format they are given has
+        a format string, and vice verca.
+
+        If branch_format is None, the transport is returned with no 
+        checking. if it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_branch_transport)
+        
+    def get_repository_transport(self, repository_format):
+        """Get the transport for use by repository format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the repository format they are given has
+        a format string, and vice verca.
+
+        If repository_format is None, the transport is returned with no 
+        checking. if it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_repository_transport)
+        
+    def get_workingtree_transport(self, tree_format):
+        """Get the transport for use by workingtree format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the workingtree format they are given has
+        a format string, and vice verca.
+
+        If workingtree_format is None, the transport is returned with no 
+        checking. if it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_workingtree_transport)
+        
+    def __init__(self, _transport, _format):
+        """Initialize a Bzr control dir object.
+        
+        Only really common logic should reside here, concrete classes should be
+        made with varying behaviours.
+
+        :param _format: the format that is creating this BzrDir instance.
+        :param _transport: the transport this dir is based at.
+        """
+        self._format = _format
+        self.transport = _transport.clone('.bzr')
+        self.root_transport = _transport
+
+    def needs_format_conversion(self, format=None):
+        """Return true if this bzrdir needs convert_format run on it.
+        
+        For instance, if the repository format is out of date but the 
+        branch and working tree are not, this should return True.
+
+        :param format: Optional parameter indicating a specific desired
+                       format we plan to arrive at.
+        """
+        raise NotImplementedError(self.needs_format_conversion)
+
+    @staticmethod
+    def open_unsupported(base):
+        """Open a branch which is not supported."""
+        return BzrDir.open(base, _unsupported=True)
+        
+    @staticmethod
+    def open(base, _unsupported=False):
+        """Open an existing bzrdir, rooted at 'base' (url)
+        
+        _unsupported is a private parameter to the BzrDir class.
+        """
+        t = get_transport(base)
+        mutter("trying to open %r with transport %r", base, t)
+        format = BzrDirFormat.find_format(t)
+        BzrDir._check_supported(format, _unsupported)
+        return format.open(t, _found=True)
+
+    def open_branch(self, unsupported=False):
+        """Open the branch object at this BzrDir if one is present.
+
+        If unsupported is True, then no longer supported branch formats can
+        still be opened.
+        
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_branch)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        :param url: url to search from.
+        See open_containing_from_transport for more detail.
+        """
+        return BzrDir.open_containing_from_transport(get_transport(url))
+    
+    @staticmethod
+    def open_containing_from_transport(a_transport):
+        """Open an existing branch which contains a_transport.base
+
+        This probes for a branch at a_transport, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one and it is either an unrecognised format or an unsupported 
+        format, UnknownFormatError or UnsupportedFormatError are raised.
+        If there is one, it is returned, along with the unused portion of url.
+        """
+        # this gets the normalised url back. I.e. '.' -> the full path.
+        url = a_transport.base
+        while True:
+            try:
+                format = BzrDirFormat.find_format(a_transport)
+                BzrDir._check_supported(format, False)
+                return format.open(a_transport), a_transport.relpath(url)
+            except errors.NotBranchError, e:
+                mutter('not a branch in: %r %s', a_transport.base, e)
+            new_t = a_transport.clone('..')
+            if new_t.base == a_transport.base:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            a_transport = new_t
+
+    def open_repository(self, _unsupported=False):
+        """Open the repository object at this BzrDir if one is present.
+
+        This will not follow the Branch object pointer - its strictly a direct
+        open facility. Most client code should use open_branch().repository to
+        get at a repository.
+
+        _unsupported is a private parameter, not part of the api.
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_repository)
+
+    def open_workingtree(self, _unsupported=False):
+        """Open the workingtree object at this BzrDir if one is present.
+        
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_workingtree)
+
+    def sprout(self, url, revision_id=None, basis=None, force_new_repo=False):
+        """Create a copy of this bzrdir prepared for use as a new line of
+        development.
+
+        If urls last component does not exist, it will be created.
+
+        Attributes related to the identity of the source branch like
+        branch nickname will be cleaned, a working tree is created
+        whether one existed before or not; and a local branch is always
+        created.
+
+        if revision_id is not None, then the clone operation may tune
+            itself to download less data.
+        """
+        self._make_tail(url)
+        result = self._format.initialize(url)
+        basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
+        try:
+            source_branch = self.open_branch()
+            source_repository = source_branch.repository
+        except errors.NotBranchError:
+            source_branch = None
+            try:
+                source_repository = self.open_repository()
+            except errors.NoRepositoryPresent:
+                # copy the entire basis one if there is one
+                # but there is no repository.
+                source_repository = basis_repo
+        if force_new_repo:
+            result_repo = None
+        else:
+            try:
+                result_repo = result.find_repository()
+            except errors.NoRepositoryPresent:
+                result_repo = None
+        if source_repository is None and result_repo is not None:
+            pass
+        elif source_repository is None and result_repo is None:
+            # no repo available, make a new one
+            result.create_repository()
+        elif source_repository is not None and result_repo is None:
+            # have soure, and want to make a new target repo
+            source_repository.clone(result,
+                                    revision_id=revision_id,
+                                    basis=basis_repo)
+        else:
+            # fetch needed content into target.
+            if basis_repo:
+                # XXX FIXME RBC 20060214 need tests for this when the basis
+                # is incomplete
+                result_repo.fetch(basis_repo, revision_id=revision_id)
+            result_repo.fetch(source_repository, revision_id=revision_id)
+        if source_branch is not None:
+            source_branch.sprout(result, revision_id=revision_id)
+        else:
+            result.create_branch()
+        if result_repo is None or result_repo.make_working_trees():
+            result.create_workingtree()
+        return result
+
+
+class BzrDirPreSplitOut(BzrDir):
+    """A common class for the all-in-one formats."""
+
+    def __init__(self, _transport, _format):
+        """See BzrDir.__init__."""
+        super(BzrDirPreSplitOut, self).__init__(_transport, _format)
+        assert self._format._lock_class == TransportLock
+        assert self._format._lock_file_name == 'branch-lock'
+        self._control_files = LockableFiles(self.get_branch_transport(None),
+                                            self._format._lock_file_name,
+                                            self._format._lock_class)
+
+    def clone(self, url, revision_id=None, basis=None, force_new_repo=False):
+        """See BzrDir.clone()."""
+        from bzrlib.workingtree import WorkingTreeFormat2
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
+        self.open_repository().clone(result, revision_id=revision_id, basis=basis_repo)
+        self.open_branch().clone(result, revision_id=revision_id)
+        try:
+            self.open_workingtree().clone(result, basis=basis_tree)
+        except errors.NotLocalUrl:
+            # make a new one, this format always has to have one.
+            try:
+                WorkingTreeFormat2().initialize(result)
+            except errors.NotLocalUrl:
+                # but we canot do it for remote trees.
+                pass
+        return result
+
+    def create_branch(self):
+        """See BzrDir.create_branch."""
+        return self.open_branch()
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        if shared:
+            raise errors.IncompatibleFormat('shared repository', self._format)
+        return self.open_repository()
+
+    def create_workingtree(self, revision_id=None):
+        """See BzrDir.create_workingtree."""
+        # this looks buggy but is not -really-
+        # clone and sprout will have set the revision_id
+        # and that will have set it for us, its only
+        # specific uses of create_workingtree in isolation
+        # that can do wonky stuff here, and that only
+        # happens for creating checkouts, which cannot be 
+        # done on this format anyway. So - acceptable wart.
+        result = self.open_workingtree()
+        if revision_id is not None:
+            result.set_last_revision(revision_id)
+        return result
+
+    def get_branch_transport(self, branch_format):
+        """See BzrDir.get_branch_transport()."""
+        if branch_format is None:
+            return self.transport
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(branch_format, self._format)
+
+    def get_repository_transport(self, repository_format):
+        """See BzrDir.get_repository_transport()."""
+        if repository_format is None:
+            return self.transport
+        try:
+            repository_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(repository_format, self._format)
+
+    def get_workingtree_transport(self, workingtree_format):
+        """See BzrDir.get_workingtree_transport()."""
+        if workingtree_format is None:
+            return self.transport
+        try:
+            workingtree_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(workingtree_format, self._format)
+
+    def needs_format_conversion(self, format=None):
+        """See BzrDir.needs_format_conversion()."""
+        # if the format is not the same as the system default,
+        # an upgrade is needed.
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        return not isinstance(self._format, format.__class__)
+
+    def open_branch(self, unsupported=False):
+        """See BzrDir.open_branch."""
+        from bzrlib.branch import BzrBranchFormat4
+        format = BzrBranchFormat4()
+        self._check_supported(format, unsupported)
+        return format.open(self, _found=True)
+
+    def sprout(self, url, revision_id=None, basis=None):
+        """See BzrDir.sprout()."""
+        from bzrlib.workingtree import WorkingTreeFormat2
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
+        try:
+            self.open_repository().clone(result, revision_id=revision_id, basis=basis_repo)
+        except errors.NoRepositoryPresent:
+            pass
+        try:
+            self.open_branch().sprout(result, revision_id=revision_id)
+        except errors.NotBranchError:
+            pass
+        # we always want a working tree
+        WorkingTreeFormat2().initialize(result)
+        return result
+
+
+class BzrDir4(BzrDirPreSplitOut):
+    """A .bzr version 4 control object.
+    
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        return self._format.repository_format.initialize(self, shared)
+
+    def needs_format_conversion(self, format=None):
+        """Format 4 dirs are always in need of conversion."""
+        return True
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat4
+        return RepositoryFormat4().open(self, _found=True)
+
+
+class BzrDir5(BzrDirPreSplitOut):
+    """A .bzr version 5 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat5
+        return RepositoryFormat5().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False):
+        """See BzrDir.create_workingtree."""
+        from bzrlib.workingtree import WorkingTreeFormat2
+        return WorkingTreeFormat2().open(self, _found=True)
+
+
+class BzrDir6(BzrDirPreSplitOut):
+    """A .bzr version 6 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat6
+        return RepositoryFormat6().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False):
+        """See BzrDir.create_workingtree."""
+        from bzrlib.workingtree import WorkingTreeFormat2
+        return WorkingTreeFormat2().open(self, _found=True)
+
+
+class BzrDirMeta1(BzrDir):
+    """A .bzr meta version 1 control object.
+    
+    This is the first control object where the 
+    individual aspects are really split out: there are separate repository,
+    workingtree and branch subdirectories and any subset of the three can be
+    present within a BzrDir.
+    """
+
+    def can_convert_format(self):
+        """See BzrDir.can_convert_format()."""
+        return True
+
+    def create_branch(self):
+        """See BzrDir.create_branch."""
+        from bzrlib.branch import BranchFormat
+        return BranchFormat.get_default_format().initialize(self)
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        return self._format.repository_format.initialize(self, shared)
+
+    def create_workingtree(self, revision_id=None):
+        """See BzrDir.create_workingtree."""
+        from bzrlib.workingtree import WorkingTreeFormat
+        return WorkingTreeFormat.get_default_format().initialize(self, revision_id)
+
+    def _get_mkdir_mode(self):
+        """Figure out the mode to use when creating a bzrdir subdir."""
+        temp_control = LockableFiles(self.transport, '', TransportLock)
+        return temp_control._dir_mode
+
+    def get_branch_transport(self, branch_format):
+        """See BzrDir.get_branch_transport()."""
+        if branch_format is None:
+            return self.transport.clone('branch')
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            raise errors.IncompatibleFormat(branch_format, self._format)
+        try:
+            self.transport.mkdir('branch', mode=self._get_mkdir_mode())
+        except errors.FileExists:
+            pass
+        return self.transport.clone('branch')
+
+    def get_repository_transport(self, repository_format):
+        """See BzrDir.get_repository_transport()."""
+        if repository_format is None:
+            return self.transport.clone('repository')
+        try:
+            repository_format.get_format_string()
+        except NotImplementedError:
+            raise errors.IncompatibleFormat(repository_format, self._format)
+        try:
+            self.transport.mkdir('repository', mode=self._get_mkdir_mode())
+        except errors.FileExists:
+            pass
+        return self.transport.clone('repository')
+
+    def get_workingtree_transport(self, workingtree_format):
+        """See BzrDir.get_workingtree_transport()."""
+        if workingtree_format is None:
+            return self.transport.clone('checkout')
+        try:
+            workingtree_format.get_format_string()
+        except NotImplementedError:
+            raise errors.IncompatibleFormat(workingtree_format, self._format)
+        try:
+            self.transport.mkdir('checkout', mode=self._get_mkdir_mode())
+        except errors.FileExists:
+            pass
+        return self.transport.clone('checkout')
+
+    def needs_format_conversion(self, format=None):
+        """See BzrDir.needs_format_conversion()."""
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        if not isinstance(self._format, format.__class__):
+            # it is not a meta dir format, conversion is needed.
+            return True
+        # we might want to push this down to the repository?
+        try:
+            if not isinstance(self.open_repository()._format,
+                              format.repository_format.__class__):
+                # the repository needs an upgrade.
+                return True
+        except errors.NoRepositoryPresent:
+            pass
+        # currently there are no other possible conversions for meta1 formats.
+        return False
+
+    def open_branch(self, unsupported=False):
+        """See BzrDir.open_branch."""
+        from bzrlib.branch import BranchFormat
+        format = BranchFormat.find_format(self)
+        self._check_supported(format, unsupported)
+        return format.open(self, _found=True)
+
+    def open_repository(self, unsupported=False):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat
+        format = RepositoryFormat.find_format(self)
+        self._check_supported(format, unsupported)
+        return format.open(self, _found=True)
+
+    def open_workingtree(self, unsupported=False):
+        """See BzrDir.open_workingtree."""
+        from bzrlib.workingtree import WorkingTreeFormat
+        format = WorkingTreeFormat.find_format(self)
+        self._check_supported(format, unsupported)
+        return format.open(self, _found=True)
+
+
+class BzrDirFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in an dict by their format string for reference 
+    during bzrdir opening. These should be subclasses of BzrDirFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every system load.
+    """
+
+    _default_format = None
+    """The default format used for new .bzr dirs."""
+
+    _formats = {}
+    """The known formats."""
+
+    _lock_file_name = 'branch-lock'
+
+    # _lock_class must be set in subclasses to the lock type, typ.
+    # TransportLock or LockDir
+
+    @classmethod
+    def find_format(klass, transport):
+        """Return the format registered for URL."""
+        try:
+            format_string = transport.get(".bzr/branch-format").read()
+            return klass._formats[format_string]
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+        except KeyError:
+            raise errors.UnknownFormatError(format_string)
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def get_format_description(self):
+        """Return the short description for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def get_converter(self, format=None):
+        """Return the converter to use to convert bzrdirs needing converts.
+
+        This returns a bzrlib.bzrdir.Converter object.
+
+        This should return the best upgrader to step this format towards the
+        current default format. In the case of plugins we can/shouold provide
+        some means for them to extend the range of returnable converters.
+
+        :param format: Optional format to override the default foramt of the 
+                       library.
+        """
+        raise NotImplementedError(self.get_converter)
+
+    def initialize(self, url):
+        """Create a bzr control dir at this url and return an opened copy.
+        
+        Subclasses should typically override initialize_on_transport
+        instead of this method.
+        """
+        return self.initialize_on_transport(get_transport(url))
+
+    def initialize_on_transport(self, transport):
+        """Initialize a new bzrdir in the base directory of a Transport."""
+        # Since we don'transport have a .bzr directory, inherit the
+        # mode from the root directory
+        temp_control = LockableFiles(transport, '', TransportLock)
+        temp_control._transport.mkdir('.bzr',
+                                      # FIXME: RBC 20060121 dont peek under
+                                      # the covers
+                                      mode=temp_control._dir_mode)
+        file_mode = temp_control._file_mode
+        del temp_control
+        mutter('created control directory in ' + transport.base)
+        control = transport.clone('.bzr')
+        utf8_files = [('README', 
+                       "This is a Bazaar-NG control directory.\n"
+                       "Do not change any files in this directory.\n"),
+                      ('branch-format', self.get_format_string()),
+                      ]
+        # NB: no need to escape relative paths that are url safe.
+        control_files = LockableFiles(control, self._lock_file_name, 
+                                      self._lock_class)
+        control_files.create_lock()
+        control_files.lock_write()
+        try:
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(transport, _found=True)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def open(self, transport, _found=False):
+        """Return an instance of this format for the dir transport points at.
+        
+        _found is a private parameter, do not use it.
+        """
+        if not _found:
+            assert isinstance(BzrDirFormat.find_format(transport),
+                              self.__class__)
+        return self._open(transport)
+
+    def _open(self, transport):
+        """Template method helper for opening BzrDirectories.
+
+        This performs the actual open and any additional logic or parameter
+        passing.
+        """
+        raise NotImplementedError(self._open)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    def __str__(self):
+        return self.get_format_string()[:-1]
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+
+class BzrDirFormat4(BzrDirFormat):
+    """Bzr dir format 4.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 1 working trees [always]
+     - Format 4 branches [always]
+     - Format 4 repositories [always]
+
+    This format is deprecated: it indexes texts using a text it which is
+    removed in format 5; write support for this format has been removed.
+    """
+
+    _lock_class = TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 0.0.4\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 4"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir4To5()
+        
+    def initialize_on_transport(self, transport):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not 
+        feasible.
+        """
+        return False
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir4(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.repository import RepositoryFormat4
+        return RepositoryFormat4(self)
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirFormat5(BzrDirFormat):
+    """Bzr control format 5.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always] 
+     - Format 4 branches [always] 
+     - Format 5 repositories [always]
+       Unhashed stores in the repository.
+    """
+
+    _lock_class = TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 5\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 5"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir5To6()
+
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+        
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 5 dirs always have working tree, branch and repository.
+        
+        Except when they are being cloned.
+        """
+        from bzrlib.branch import BzrBranchFormat4
+        from bzrlib.repository import RepositoryFormat5
+        from bzrlib.workingtree import WorkingTreeFormat2
+        result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
+        RepositoryFormat5().initialize(result, _internal=True)
+        if not _cloning:
+            BzrBranchFormat4().initialize(result)
+            WorkingTreeFormat2().initialize(result)
+        return result
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir5(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.repository import RepositoryFormat5
+        return RepositoryFormat5(self)
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirFormat6(BzrDirFormat):
+    """Bzr control format 6.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always] 
+     - Format 4 branches [always] 
+     - Format 6 repositories [always]
+    """
+
+    _lock_class = TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 6\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 6"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir6ToMeta()
+        
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 6 dirs always have working tree, branch and repository.
+        
+        Except when they are being cloned.
+        """
+        from bzrlib.branch import BzrBranchFormat4
+        from bzrlib.repository import RepositoryFormat6
+        from bzrlib.workingtree import WorkingTreeFormat2
+        result = super(BzrDirFormat6, self).initialize_on_transport(transport)
+        RepositoryFormat6().initialize(result, _internal=True)
+        if not _cloning:
+            BzrBranchFormat4().initialize(result)
+            try:
+                WorkingTreeFormat2().initialize(result)
+            except errors.NotLocalUrl:
+                # emulate pre-check behaviour for working tree and silently 
+                # fail.
+                pass
+        return result
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir6(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.repository import RepositoryFormat6
+        return RepositoryFormat6(self)
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirMetaFormat1(BzrDirFormat):
+    """Bzr meta control format 1
+
+    This is the first format with split out working tree, branch and repository
+    disk storage.
+    It has:
+     - Format 3 working trees [optional]
+     - Format 5 branches [optional]
+     - Format 7 repositories [optional]
+    """
+
+    _lock_class = LockDir
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        if not isinstance(self, format.__class__):
+            # converting away from metadir is not implemented
+            raise NotImplementedError(self.get_converter)
+        return ConvertMetaToMeta(format)
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG meta directory, format 1\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "Meta directory format 1"
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDirMeta1(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        if getattr(self, '_repository_format', None):
+            return self._repository_format
+        from bzrlib.repository import RepositoryFormat
+        return RepositoryFormat.get_default_format()
+
+    def __set_repository_format(self, value):
+        """Allow changint the repository format for metadir formats."""
+        self._repository_format = value
+
+    repository_format = property(__return_repository_format, __set_repository_format)
+
+
+BzrDirFormat.register_format(BzrDirFormat4())
+BzrDirFormat.register_format(BzrDirFormat5())
+BzrDirFormat.register_format(BzrDirFormat6())
+__default_format = BzrDirMetaFormat1()
+BzrDirFormat.register_format(__default_format)
+BzrDirFormat.set_default_format(__default_format)
+
+
+class BzrDirTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple bzrdir formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and bzrdir_format
+    classes into each copy. Each copy is also given a new id() to make it
+    easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+
+class ScratchDir(BzrDir6):
+    """Special test class: a bzrdir that cleans up itself..
+
+    >>> d = ScratchDir()
+    >>> base = d.transport.base
+    >>> isdir(base)
+    True
+    >>> b.transport.__del__()
+    >>> isdir(base)
+    False
+    """
+
+    def __init__(self, files=[], dirs=[], transport=None):
+        """Make a test branch.
+
+        This creates a temporary directory and runs init-tree in it.
+
+        If any files are listed, they are created in the working copy.
+        """
+        if transport is None:
+            transport = bzrlib.transport.local.ScratchTransport()
+            # local import for scope restriction
+            BzrDirFormat6().initialize(transport.base)
+            super(ScratchDir, self).__init__(transport, BzrDirFormat6())
+            self.create_repository()
+            self.create_branch()
+            self.create_workingtree()
+        else:
+            super(ScratchDir, self).__init__(transport, BzrDirFormat6())
+
+        # BzrBranch creates a clone to .bzr and then forgets about the
+        # original transport. A ScratchTransport() deletes itself and
+        # everything underneath it when it goes away, so we need to
+        # grab a local copy to prevent that from happening
+        self._transport = transport
+
+        for d in dirs:
+            self._transport.mkdir(d)
+            
+        for f in files:
+            self._transport.put(f, 'content of %s' % f)
+
+    def clone(self):
+        """
+        >>> orig = ScratchDir(files=["file1", "file2"])
+        >>> os.listdir(orig.base)
+        [u'.bzr', u'file1', u'file2']
+        >>> clone = orig.clone()
+        >>> if os.name != 'nt':
+        ...   os.path.samefile(orig.base, clone.base)
+        ... else:
+        ...   orig.base == clone.base
+        ...
+        False
+        >>> os.listdir(clone.base)
+        [u'.bzr', u'file1', u'file2']
+        """
+        from shutil import copytree
+        from bzrlib.osutils import mkdtemp
+        base = mkdtemp()
+        os.rmdir(base)
+        copytree(self.base, base, symlinks=True)
+        return ScratchDir(
+            transport=bzrlib.transport.local.ScratchTransport(base))
+
+
+class Converter(object):
+    """Converts a disk format object from one format to another."""
+
+    def convert(self, to_convert, pb):
+        """Perform the conversion of to_convert, giving feedback via pb.
+
+        :param to_convert: The disk object to convert.
+        :param pb: a progress bar to use for progress information.
+        """
+
+    def step(self, message):
+        """Update the pb by a step."""
+        self.count +=1
+        self.pb.update(message, self.count, self.total)
+
+
+class ConvertBzrDir4To5(Converter):
+    """Converts format 4 bzr dirs to format 5."""
+
+    def __init__(self):
+        super(ConvertBzrDir4To5, self).__init__()
+        self.converted_revs = set()
+        self.absent_revisions = set()
+        self.text_count = 0
+        self.revisions = {}
+        
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        self.pb = pb
+        self.pb.note('starting upgrade from format 4 to 5')
+        if isinstance(self.bzrdir.transport, LocalTransport):
+            self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
+        self._convert_to_weaves()
+        return BzrDir.open(self.bzrdir.root_transport.base)
+
+    def _convert_to_weaves(self):
+        self.pb.note('note: upgrade may be faster if all store files are ungzipped first')
+        try:
+            # TODO permissions
+            stat = self.bzrdir.transport.stat('weaves')
+            if not S_ISDIR(stat.st_mode):
+                self.bzrdir.transport.delete('weaves')
+                self.bzrdir.transport.mkdir('weaves')
+        except errors.NoSuchFile:
+            self.bzrdir.transport.mkdir('weaves')
+        # deliberately not a WeaveFile as we want to build it up slowly.
+        self.inv_weave = Weave('inventory')
+        # holds in-memory weaves for all files
+        self.text_weaves = {}
+        self.bzrdir.transport.delete('branch-format')
+        self.branch = self.bzrdir.open_branch()
+        self._convert_working_inv()
+        rev_history = self.branch.revision_history()
+        # to_read is a stack holding the revisions we still need to process;
+        # appending to it adds new highest-priority revisions
+        self.known_revisions = set(rev_history)
+        self.to_read = rev_history[-1:]
+        while self.to_read:
+            rev_id = self.to_read.pop()
+            if (rev_id not in self.revisions
+                and rev_id not in self.absent_revisions):
+                self._load_one_rev(rev_id)
+        self.pb.clear()
+        to_import = self._make_order()
+        for i, rev_id in enumerate(to_import):
+            self.pb.update('converting revision', i, len(to_import))
+            self._convert_one_rev(rev_id)
+        self.pb.clear()
+        self._write_all_weaves()
+        self._write_all_revs()
+        self.pb.note('upgraded to weaves:')
+        self.pb.note('  %6d revisions and inventories', len(self.revisions))
+        self.pb.note('  %6d revisions not present', len(self.absent_revisions))
+        self.pb.note('  %6d texts', self.text_count)
+        self._cleanup_spare_files_after_format4()
+        self.branch.control_files.put_utf8('branch-format', BzrDirFormat5().get_format_string())
+
+    def _cleanup_spare_files_after_format4(self):
+        # FIXME working tree upgrade foo.
+        for n in 'merged-patches', 'pending-merged-patches':
+            try:
+                ## assert os.path.getsize(p) == 0
+                self.bzrdir.transport.delete(n)
+            except errors.NoSuchFile:
+                pass
+        self.bzrdir.transport.delete_tree('inventory-store')
+        self.bzrdir.transport.delete_tree('text-store')
+
+    def _convert_working_inv(self):
+        inv = serializer_v4.read_inventory(self.branch.control_files.get('inventory'))
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        # FIXME inventory is a working tree change.
+        self.branch.control_files.put('inventory', new_inv_xml)
+
+    def _write_all_weaves(self):
+        controlweaves = WeaveStore(self.bzrdir.transport, prefixed=False)
+        weave_transport = self.bzrdir.transport.clone('weaves')
+        weaves = WeaveStore(weave_transport, prefixed=False)
+        transaction = WriteTransaction()
+
+        try:
+            i = 0
+            for file_id, file_weave in self.text_weaves.items():
+                self.pb.update('writing weave', i, len(self.text_weaves))
+                weaves._put_weave(file_id, file_weave, transaction)
+                i += 1
+            self.pb.update('inventory', 0, 1)
+            controlweaves._put_weave('inventory', self.inv_weave, transaction)
+            self.pb.update('inventory', 1, 1)
+        finally:
+            self.pb.clear()
+
+    def _write_all_revs(self):
+        """Write all revisions out in new form."""
+        self.bzrdir.transport.delete_tree('revision-store')
+        self.bzrdir.transport.mkdir('revision-store')
+        revision_transport = self.bzrdir.transport.clone('revision-store')
+        # TODO permissions
+        _revision_store = TextRevisionStore(TextStore(revision_transport,
+                                                      prefixed=False,
+                                                      compressed=True))
+        try:
+            transaction = bzrlib.transactions.WriteTransaction()
+            for i, rev_id in enumerate(self.converted_revs):
+                self.pb.update('write revision', i, len(self.converted_revs))
+                _revision_store.add_revision(self.revisions[rev_id], transaction)
+        finally:
+            self.pb.clear()
+            
+    def _load_one_rev(self, rev_id):
+        """Load a revision object into memory.
+
+        Any parents not either loaded or abandoned get queued to be
+        loaded."""
+        self.pb.update('loading revision',
+                       len(self.revisions),
+                       len(self.known_revisions))
+        if not self.branch.repository.has_revision(rev_id):
+            self.pb.clear()
+            self.pb.note('revision {%s} not present in branch; '
+                         'will be converted as a ghost',
+                         rev_id)
+            self.absent_revisions.add(rev_id)
+        else:
+            rev = self.branch.repository._revision_store.get_revision(rev_id,
+                self.branch.repository.get_transaction())
+            for parent_id in rev.parent_ids:
+                self.known_revisions.add(parent_id)
+                self.to_read.append(parent_id)
+            self.revisions[rev_id] = rev
+
+    def _load_old_inventory(self, rev_id):
+        assert rev_id not in self.converted_revs
+        old_inv_xml = self.branch.repository.inventory_store.get(rev_id).read()
+        inv = serializer_v4.read_inventory_from_string(old_inv_xml)
+        rev = self.revisions[rev_id]
+        if rev.inventory_sha1:
+            assert rev.inventory_sha1 == sha_string(old_inv_xml), \
+                'inventory sha mismatch for {%s}' % rev_id
+        return inv
+
+    def _load_updated_inventory(self, rev_id):
+        assert rev_id in self.converted_revs
+        inv_xml = self.inv_weave.get_text(rev_id)
+        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        return inv
+
+    def _convert_one_rev(self, rev_id):
+        """Convert revision and all referenced objects to new format."""
+        rev = self.revisions[rev_id]
+        inv = self._load_old_inventory(rev_id)
+        present_parents = [p for p in rev.parent_ids
+                           if p not in self.absent_revisions]
+        self._convert_revision_contents(rev, inv, present_parents)
+        self._store_new_weave(rev, inv, present_parents)
+        self.converted_revs.add(rev_id)
+
+    def _store_new_weave(self, rev, inv, present_parents):
+        # the XML is now updated with text versions
+        if __debug__:
+            for file_id in inv:
+                ie = inv[file_id]
+                if ie.kind == 'root_directory':
+                    continue
+                assert hasattr(ie, 'revision'), \
+                    'no revision on {%s} in {%s}' % \
+                    (file_id, rev.revision_id)
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        new_inv_sha1 = sha_string(new_inv_xml)
+        self.inv_weave.add_lines(rev.revision_id, 
+                                 present_parents,
+                                 new_inv_xml.splitlines(True))
+        rev.inventory_sha1 = new_inv_sha1
+
+    def _convert_revision_contents(self, rev, inv, present_parents):
+        """Convert all the files within a revision.
+
+        Also upgrade the inventory to refer to the text revision ids."""
+        rev_id = rev.revision_id
+        mutter('converting texts of revision {%s}',
+               rev_id)
+        parent_invs = map(self._load_updated_inventory, present_parents)
+        for file_id in inv:
+            ie = inv[file_id]
+            self._convert_file_version(rev, ie, parent_invs)
+
+    def _convert_file_version(self, rev, ie, parent_invs):
+        """Convert one version of one file.
+
+        The file needs to be added into the weave if it is a merge
+        of >=2 parents or if it's changed from its parent.
+        """
+        if ie.kind == 'root_directory':
+            return
+        file_id = ie.file_id
+        rev_id = rev.revision_id
+        w = self.text_weaves.get(file_id)
+        if w is None:
+            w = Weave(file_id)
+            self.text_weaves[file_id] = w
+        text_changed = False
+        previous_entries = ie.find_previous_heads(parent_invs,
+                                                  None,
+                                                  None,
+                                                  entry_vf=w)
+        for old_revision in previous_entries:
+                # if this fails, its a ghost ?
+                assert old_revision in self.converted_revs 
+        self.snapshot_ie(previous_entries, ie, w, rev_id)
+        del ie.text_id
+        assert getattr(ie, 'revision', None) is not None
+
+    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
+        # TODO: convert this logic, which is ~= snapshot to
+        # a call to:. This needs the path figured out. rather than a work_tree
+        # a v4 revision_tree can be given, or something that looks enough like
+        # one to give the file content to the entry if it needs it.
+        # and we need something that looks like a weave store for snapshot to 
+        # save against.
+        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
+        if len(previous_revisions) == 1:
+            previous_ie = previous_revisions.values()[0]
+            if ie._unchanged(previous_ie):
+                ie.revision = previous_ie.revision
+                return
+        if ie.has_text():
+            text = self.branch.repository.text_store.get(ie.text_id)
+            file_lines = text.readlines()
+            assert sha_strings(file_lines) == ie.text_sha1
+            assert sum(map(len, file_lines)) == ie.text_size
+            w.add_lines(rev_id, previous_revisions, file_lines)
+            self.text_count += 1
+        else:
+            w.add_lines(rev_id, previous_revisions, [])
+        ie.revision = rev_id
+
+    def _make_order(self):
+        """Return a suitable order for importing revisions.
+
+        The order must be such that an revision is imported after all
+        its (present) parents.
+        """
+        todo = set(self.revisions.keys())
+        done = self.absent_revisions.copy()
+        order = []
+        while todo:
+            # scan through looking for a revision whose parents
+            # are all done
+            for rev_id in sorted(list(todo)):
+                rev = self.revisions[rev_id]
+                parent_ids = set(rev.parent_ids)
+                if parent_ids.issubset(done):
+                    # can take this one now
+                    order.append(rev_id)
+                    todo.remove(rev_id)
+                    done.add(rev_id)
+        return order
+
+
+class ConvertBzrDir5To6(Converter):
+    """Converts format 5 bzr dirs to format 6."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        self.pb = pb
+        self.pb.note('starting upgrade from format 5 to 6')
+        self._convert_to_prefixed()
+        return BzrDir.open(self.bzrdir.root_transport.base)
+
+    def _convert_to_prefixed(self):
+        from bzrlib.store import TransportStore
+        self.bzrdir.transport.delete('branch-format')
+        for store_name in ["weaves", "revision-store"]:
+            self.pb.note("adding prefixes to %s" % store_name)
+            store_transport = self.bzrdir.transport.clone(store_name)
+            store = TransportStore(store_transport, prefixed=True)
+            for urlfilename in store_transport.list_dir('.'):
+                filename = urlunescape(urlfilename)
+                if (filename.endswith(".weave") or
+                    filename.endswith(".gz") or
+                    filename.endswith(".sig")):
+                    file_id = os.path.splitext(filename)[0]
+                else:
+                    file_id = filename
+                prefix_dir = store.hash_prefix(file_id)
+                # FIXME keep track of the dirs made RBC 20060121
+                try:
+                    store_transport.move(filename, prefix_dir + '/' + filename)
+                except errors.NoSuchFile: # catches missing dirs strangely enough
+                    store_transport.mkdir(prefix_dir)
+                    store_transport.move(filename, prefix_dir + '/' + filename)
+        self.bzrdir._control_files.put_utf8('branch-format', BzrDirFormat6().get_format_string())
+
+
+class ConvertBzrDir6ToMeta(Converter):
+    """Converts format 6 bzr dirs to metadirs."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        self.pb = pb
+        self.count = 0
+        self.total = 20 # the steps we know about
+        self.garbage_inventories = []
+
+        self.pb.note('starting upgrade from format 6 to metadir')
+        self.bzrdir._control_files.put_utf8('branch-format', "Converting to format 6")
+        # its faster to move specific files around than to open and use the apis...
+        # first off, nuke ancestry.weave, it was never used.
+        try:
+            self.step('Removing ancestry.weave')
+            self.bzrdir.transport.delete('ancestry.weave')
+        except errors.NoSuchFile:
+            pass
+        # find out whats there
+        self.step('Finding branch files')
+        last_revision = self.bzrdir.open_branch().last_revision()
+        bzrcontents = self.bzrdir.transport.list_dir('.')
+        for name in bzrcontents:
+            if name.startswith('basis-inventory.'):
+                self.garbage_inventories.append(name)
+        # create new directories for repository, working tree and branch
+        self.dir_mode = self.bzrdir._control_files._dir_mode
+        self.file_mode = self.bzrdir._control_files._file_mode
+        repository_names = [('inventory.weave', True),
+                            ('revision-store', True),
+                            ('weaves', True)]
+        self.step('Upgrading repository  ')
+        self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
+        self.make_lock('repository')
+        # we hard code the formats here because we are converting into
+        # the meta format. The meta format upgrader can take this to a 
+        # future format within each component.
+        self.put_format('repository', bzrlib.repository.RepositoryFormat7())
+        for entry in repository_names:
+            self.move_entry('repository', entry)
+
+        self.step('Upgrading branch      ')
+        self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
+        self.make_lock('branch')
+        self.put_format('branch', bzrlib.branch.BzrBranchFormat5())
+        branch_files = [('revision-history', True),
+                        ('branch-name', True),
+                        ('parent', False)]
+        for entry in branch_files:
+            self.move_entry('branch', entry)
+
+        self.step('Upgrading working tree')
+        self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
+        self.make_lock('checkout')
+        self.put_format('checkout', bzrlib.workingtree.WorkingTreeFormat3())
+        self.bzrdir.transport.delete_multi(self.garbage_inventories, self.pb)
+        checkout_files = [('pending-merges', True),
+                          ('inventory', True),
+                          ('stat-cache', False)]
+        for entry in checkout_files:
+            self.move_entry('checkout', entry)
+        if last_revision is not None:
+            self.bzrdir._control_files.put_utf8('checkout/last-revision',
+                                                last_revision)
+        self.bzrdir._control_files.put_utf8('branch-format', BzrDirMetaFormat1().get_format_string())
+        return BzrDir.open(self.bzrdir.root_transport.base)
+
+    def make_lock(self, name):
+        """Make a lock for the new control dir name."""
+        self.step('Make %s lock' % name)
+        ld = LockDir(self.bzrdir.transport, 
+                     '%s/lock' % name,
+                     file_modebits=self.file_mode,
+                     dir_modebits=self.dir_mode)
+        ld.create()
+
+    def move_entry(self, new_dir, entry):
+        """Move then entry name into new_dir."""
+        name = entry[0]
+        mandatory = entry[1]
+        self.step('Moving %s' % name)
+        try:
+            self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
+        except errors.NoSuchFile:
+            if mandatory:
+                raise
+
+    def put_format(self, dirname, format):
+        self.bzrdir._control_files.put_utf8('%s/format' % dirname, format.get_format_string())
+
+
+class ConvertMetaToMeta(Converter):
+    """Converts the components of metadirs."""
+
+    def __init__(self, target_format):
+        """Create a metadir to metadir converter.
+
+        :param target_format: The final metadir format that is desired.
+        """
+        self.target_format = target_format
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        self.pb = pb
+        self.count = 0
+        self.total = 1
+        self.step('checking repository format')
+        try:
+            repo = self.bzrdir.open_repository()
+        except errors.NoRepositoryPresent:
+            pass
+        else:
+            if not isinstance(repo._format, self.target_format.repository_format.__class__):
+                from bzrlib.repository import CopyConverter
+                self.pb.note('starting repository conversion')
+                converter = CopyConverter(self.target_format.repository_format)
+                converter.convert(repo, pb)
+        return to_convert

=== removed file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-08-17 08:32:56 +0000
+++ bzrlib/changeset.py	1970-01-01 00:00:00 +0000
@@ -1,1523 +0,0 @@
-# Copyright (C) 2004 Aaron Bentley <aaron.bentley@utoronto.ca>
-#
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
-#
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-import os.path
-import errno
-import patch
-import stat
-from bzrlib.trace import mutter
-
-# XXX: mbp: I'm not totally convinced that we should handle conflicts
-# as part of changeset application, rather than only in the merge
-# operation.
-
-"""Represent and apply a changeset
-
-Conflicts in applying a changeset are represented as exceptions.
-"""
-
-__docformat__ = "restructuredtext"
-
-NULL_ID = "!NULL"
-
-class OldFailedTreeOp(Exception):
-    def __init__(self):
-        Exception.__init__(self, "bzr-tree-change contains files from a"
-                           " previous failed merge operation.")
-def invert_dict(dict):
-    newdict = {}
-    for (key,value) in dict.iteritems():
-        newdict[value] = key
-    return newdict
-
-
-       
-class ChangeUnixPermissions(object):
-    """This is two-way change, suitable for file modification, creation,
-    deletion"""
-    def __init__(self, old_mode, new_mode):
-        self.old_mode = old_mode
-        self.new_mode = new_mode
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            from_mode = self.old_mode
-            to_mode = self.new_mode
-        else:
-            from_mode = self.new_mode
-            to_mode = self.old_mode
-        try:
-            current_mode = os.stat(filename).st_mode &0777
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                if conflict_handler.missing_for_chmod(filename) == "skip":
-                    return
-                else:
-                    current_mode = from_mode
-
-        if from_mode is not None and current_mode != from_mode:
-            if conflict_handler.wrong_old_perms(filename, from_mode, 
-                                                current_mode) != "continue":
-                return
-
-        if to_mode is not None:
-            try:
-                os.chmod(filename, to_mode)
-            except IOError, e:
-                if e.errno == errno.ENOENT:
-                    conflict_handler.missing_for_chmod(filename)
-
-    def __eq__(self, other):
-        if not isinstance(other, ChangeUnixPermissions):
-            return False
-        elif self.old_mode != other.old_mode:
-            return False
-        elif self.new_mode != other.new_mode:
-            return False
-        else:
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-def dir_create(filename, conflict_handler, reverse):
-    """Creates the directory, or deletes it if reverse is true.  Intended to be
-    used with ReplaceContents.
-
-    :param filename: The name of the directory to create
-    :type filename: str
-    :param reverse: If true, delete the directory, instead
-    :type reverse: bool
-    """
-    if not reverse:
-        try:
-            os.mkdir(filename)
-        except OSError, e:
-            if e.errno != errno.EEXIST:
-                raise
-            if conflict_handler.dir_exists(filename) == "continue":
-                os.mkdir(filename)
-        except IOError, e:
-            if e.errno == errno.ENOENT:
-                if conflict_handler.missing_parent(filename)=="continue":
-                    file(filename, "wb").write(self.contents)
-    else:
-        try:
-            os.rmdir(filename)
-        except OSError, e:
-            if e.errno != 39:
-                raise
-            if conflict_handler.rmdir_non_empty(filename) == "skip":
-                return
-            os.rmdir(filename)
-
-                
-            
-
-class SymlinkCreate(object):
-    """Creates or deletes a symlink (for use with ReplaceContents)"""
-    def __init__(self, contents):
-        """Constructor.
-
-        :param contents: The filename of the target the symlink should point to
-        :type contents: str
-        """
-        self.target = contents
-
-    def __call__(self, filename, conflict_handler, reverse):
-        """Creates or destroys the symlink.
-
-        :param filename: The name of the symlink to create
-        :type filename: str
-        """
-        if reverse:
-            assert(os.readlink(filename) == self.target)
-            os.unlink(filename)
-        else:
-            try:
-                os.symlink(self.target, filename)
-            except OSError, e:
-                if e.errno != errno.EEXIST:
-                    raise
-                if conflict_handler.link_name_exists(filename) == "continue":
-                    os.symlink(self.target, filename)
-
-    def __eq__(self, other):
-        if not isinstance(other, SymlinkCreate):
-            return False
-        elif self.target != other.target:
-            return False
-        else:
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-class FileCreate(object):
-    """Create or delete a file (for use with ReplaceContents)"""
-    def __init__(self, contents):
-        """Constructor
-
-        :param contents: The contents of the file to write
-        :type contents: str
-        """
-        self.contents = contents
-
-    def __repr__(self):
-        return "FileCreate(%i b)" % len(self.contents)
-
-    def __eq__(self, other):
-        if not isinstance(other, FileCreate):
-            return False
-        elif self.contents != other.contents:
-            return False
-        else:
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def __call__(self, filename, conflict_handler, reverse):
-        """Create or delete a file
-
-        :param filename: The name of the file to create
-        :type filename: str
-        :param reverse: Delete the file instead of creating it
-        :type reverse: bool
-        """
-        if not reverse:
-            try:
-                file(filename, "wb").write(self.contents)
-            except IOError, e:
-                if e.errno == errno.ENOENT:
-                    if conflict_handler.missing_parent(filename)=="continue":
-                        file(filename, "wb").write(self.contents)
-                else:
-                    raise
-
-        else:
-            try:
-                if (file(filename, "rb").read() != self.contents):
-                    direction = conflict_handler.wrong_old_contents(filename,
-                                                                    self.contents)
-                    if  direction != "continue":
-                        return
-                os.unlink(filename)
-            except IOError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-                if conflict_handler.missing_for_rm(filename, undo) == "skip":
-                    return
-
-                    
-
-def reversed(sequence):
-    max = len(sequence) - 1
-    for i in range(len(sequence)):
-        yield sequence[max - i]
-
-class ReplaceContents(object):
-    """A contents-replacement framework.  It allows a file/directory/symlink to
-    be created, deleted, or replaced with another file/directory/symlink.
-    Arguments must be callable with (filename, reverse).
-    """
-    def __init__(self, old_contents, new_contents):
-        """Constructor.
-
-        :param old_contents: The change to reverse apply (e.g. a deletion), \
-        when going forwards.
-        :type old_contents: `dir_create`, `SymlinkCreate`, `FileCreate`, \
-        NoneType, etc.
-        :param new_contents: The second change to apply (e.g. a creation), \
-        when going forwards.
-        :type new_contents: `dir_create`, `SymlinkCreate`, `FileCreate`, \
-        NoneType, etc.
-        """
-        self.old_contents=old_contents
-        self.new_contents=new_contents
-
-    def __repr__(self):
-        return "ReplaceContents(%r -> %r)" % (self.old_contents,
-                                              self.new_contents)
-
-    def __eq__(self, other):
-        if not isinstance(other, ReplaceContents):
-            return False
-        elif self.old_contents != other.old_contents:
-            return False
-        elif self.new_contents != other.new_contents:
-            return False
-        else:
-            return True
-    def __ne__(self, other):
-        return not (self == other)
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        """Applies the FileReplacement to the specified filename
-
-        :param filename: The name of the file to apply changes to
-        :type filename: str
-        :param reverse: If true, apply the change in reverse
-        :type reverse: bool
-        """
-        if not reverse:
-            undo = self.old_contents
-            perform = self.new_contents
-        else:
-            undo = self.new_contents
-            perform = self.old_contents
-        mode = None
-        if undo is not None:
-            try:
-                mode = os.lstat(filename).st_mode
-                if stat.S_ISLNK(mode):
-                    mode = None
-            except OSError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-                if conflict_handler.missing_for_rm(filename, undo) == "skip":
-                    return
-            undo(filename, conflict_handler, reverse=True)
-        if perform is not None:
-            perform(filename, conflict_handler, reverse=False)
-            if mode is not None:
-                os.chmod(filename, mode)
-
-class ApplySequence(object):
-    def __init__(self, changes=None):
-        self.changes = []
-        if changes is not None:
-            self.changes.extend(changes)
-
-    def __eq__(self, other):
-        if not isinstance(other, ApplySequence):
-            return False
-        elif len(other.changes) != len(self.changes):
-            return False
-        else:
-            for i in range(len(self.changes)):
-                if self.changes[i] != other.changes[i]:
-                    return False
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            iter = self.changes
-        else:
-            iter = reversed(self.changes)
-        for change in iter:
-            change.apply(filename, conflict_handler, reverse)
-
-
-class Diff3Merge(object):
-    def __init__(self, file_id, base, other):
-        self.file_id = file_id
-        self.base = base
-        self.other = other
-
-    def __eq__(self, other):
-        if not isinstance(other, Diff3Merge):
-            return False
-        return (self.base == other.base and 
-                self.other == other.other and self.file_id == other.file_id)
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new"
-        base_file = self.base.readonly_path(self.file_id)
-        other_file = self.other.readonly_path(self.file_id)
-        if not reverse:
-            base = base_file
-            other = other_file
-        else:
-            base = other_file
-            other = base_file
-        status = patch.diff3(new_file, filename, base, other)
-        if status == 0:
-            os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
-            return
-        else:
-            assert(status == 1)
-            def get_lines(filename):
-                my_file = file(base, "rb")
-                lines = my_file.readlines()
-                my_file.close()
-            base_lines = get_lines(base)
-            other_lines = get_lines(other)
-            conflict_handler.merge_conflict(new_file, filename, base_lines, 
-                                            other_lines)
-
-
-def CreateDir():
-    """Convenience function to create a directory.
-
-    :return: A ReplaceContents that will create a directory
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(None, dir_create)
-
-def DeleteDir():
-    """Convenience function to delete a directory.
-
-    :return: A ReplaceContents that will delete a directory
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(dir_create, None)
-
-def CreateFile(contents):
-    """Convenience fucntion to create a file.
-    
-    :param contents: The contents of the file to create 
-    :type contents: str
-    :return: A ReplaceContents that will create a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(None, FileCreate(contents))
-
-def DeleteFile(contents):
-    """Convenience fucntion to delete a file.
-    
-    :param contents: The contents of the file to delete
-    :type contents: str
-    :return: A ReplaceContents that will delete a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(FileCreate(contents), None)
-
-def ReplaceFileContents(old_contents, new_contents):
-    """Convenience fucntion to replace the contents of a file.
-    
-    :param old_contents: The contents of the file to replace 
-    :type old_contents: str
-    :param new_contents: The contents to replace the file with
-    :type new_contents: str
-    :return: A ReplaceContents that will replace the contents of a file a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(FileCreate(old_contents), FileCreate(new_contents))
-
-def CreateSymlink(target):
-    """Convenience fucntion to create a symlink.
-    
-    :param target: The path the link should point to
-    :type target: str
-    :return: A ReplaceContents that will delete a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(None, SymlinkCreate(target))
-
-def DeleteSymlink(target):
-    """Convenience fucntion to delete a symlink.
-    
-    :param target: The path the link should point to
-    :type target: str
-    :return: A ReplaceContents that will delete a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(SymlinkCreate(target), None)
-
-def ChangeTarget(old_target, new_target):
-    """Convenience fucntion to change the target of a symlink.
-    
-    :param old_target: The current link target
-    :type old_target: str
-    :param new_target: The new link target to use
-    :type new_target: str
-    :return: A ReplaceContents that will delete a file 
-    :rtype: `ReplaceContents`
-    """
-    return ReplaceContents(SymlinkCreate(old_target), SymlinkCreate(new_target))
-
-
-class InvalidEntry(Exception):
-    """Raise when a ChangesetEntry is invalid in some way"""
-    def __init__(self, entry, problem):
-        """Constructor.
-
-        :param entry: The invalid ChangesetEntry
-        :type entry: `ChangesetEntry`
-        :param problem: The problem with the entry
-        :type problem: str
-        """
-        msg = "Changeset entry for %s (%s) is invalid.\n%s" % (entry.id, 
-                                                               entry.path, 
-                                                               problem)
-        Exception.__init__(self, msg)
-        self.entry = entry
-
-
-class SourceRootHasName(InvalidEntry):
-    """This changeset entry has a name other than "", but its parent is !NULL"""
-    def __init__(self, entry, name):
-        """Constructor.
-
-        :param entry: The invalid ChangesetEntry
-        :type entry: `ChangesetEntry`
-        :param name: The name of the entry
-        :type name: str
-        """
-        msg = 'Child of !NULL is named "%s", not "./.".' % name
-        InvalidEntry.__init__(self, entry, msg)
-
-class NullIDAssigned(InvalidEntry):
-    """The id !NULL was assigned to a real entry"""
-    def __init__(self, entry):
-        """Constructor.
-
-        :param entry: The invalid ChangesetEntry
-        :type entry: `ChangesetEntry`
-        """
-        msg = '"!NULL" id assigned to a file "%s".' % entry.path
-        InvalidEntry.__init__(self, entry, msg)
-
-class ParentIDIsSelf(InvalidEntry):
-    """An entry is marked as its own parent"""
-    def __init__(self, entry):
-        """Constructor.
-
-        :param entry: The invalid ChangesetEntry
-        :type entry: `ChangesetEntry`
-        """
-        msg = 'file %s has "%s" id for both self id and parent id.' % \
-            (entry.path, entry.id)
-        InvalidEntry.__init__(self, entry, msg)
-
-class ChangesetEntry(object):
-    """An entry the changeset"""
-    def __init__(self, id, parent, path):
-        """Constructor. Sets parent and name assuming it was not
-        renamed/created/deleted.
-        :param id: The id associated with the entry
-        :param parent: The id of the parent of this entry (or !NULL if no
-        parent)
-        :param path: The file path relative to the tree root of this entry
-        """
-        self.id = id
-        self.path = path 
-        self.new_path = path
-        self.parent = parent
-        self.new_parent = parent
-        self.contents_change = None
-        self.metadata_change = None
-        if parent == NULL_ID and path !='./.':
-            raise SourceRootHasName(self, path)
-        if self.id == NULL_ID:
-            raise NullIDAssigned(self)
-        if self.id  == self.parent:
-            raise ParentIDIsSelf(self)
-
-    def __str__(self):
-        return "ChangesetEntry(%s)" % self.id
-
-    def __get_dir(self):
-        if self.path is None:
-            return None
-        return os.path.dirname(self.path)
-
-    def __set_dir(self, dir):
-        self.path = os.path.join(dir, os.path.basename(self.path))
-
-    dir = property(__get_dir, __set_dir)
-    
-    def __get_name(self):
-        if self.path is None:
-            return None
-        return os.path.basename(self.path)
-
-    def __set_name(self, name):
-        self.path = os.path.join(os.path.dirname(self.path), name)
-
-    name = property(__get_name, __set_name)
-
-    def __get_new_dir(self):
-        if self.new_path is None:
-            return None
-        return os.path.dirname(self.new_path)
-
-    def __set_new_dir(self, dir):
-        self.new_path = os.path.join(dir, os.path.basename(self.new_path))
-
-    new_dir = property(__get_new_dir, __set_new_dir)
-
-    def __get_new_name(self):
-        if self.new_path is None:
-            return None
-        return os.path.basename(self.new_path)
-
-    def __set_new_name(self, name):
-        self.new_path = os.path.join(os.path.dirname(self.new_path), name)
-
-    new_name = property(__get_new_name, __set_new_name)
-
-    def needs_rename(self):
-        """Determines whether the entry requires renaming.
-
-        :rtype: bool
-        """
-
-        return (self.parent != self.new_parent or self.name != self.new_name)
-
-    def is_deletion(self, reverse):
-        """Return true if applying the entry would delete a file/directory.
-
-        :param reverse: if true, the changeset is being applied in reverse
-        :rtype: bool
-        """
-        return ((self.new_parent is None and not reverse) or 
-                (self.parent is None and reverse))
-
-    def is_creation(self, reverse):
-        """Return true if applying the entry would create a file/directory.
-
-        :param reverse: if true, the changeset is being applied in reverse
-        :rtype: bool
-        """
-        return ((self.parent is None and not reverse) or 
-                (self.new_parent is None and reverse))
-
-    def is_creation_or_deletion(self):
-        """Return true if applying the entry would create or delete a 
-        file/directory.
-
-        :rtype: bool
-        """
-        return self.parent is None or self.new_parent is None
-
-    def get_cset_path(self, mod=False):
-        """Determine the path of the entry according to the changeset.
-
-        :param changeset: The changeset to derive the path from
-        :type changeset: `Changeset`
-        :param mod: If true, generate the MOD path.  Otherwise, generate the \
-        ORIG path.
-        :return: the path of the entry, or None if it did not exist in the \
-        requested tree.
-        :rtype: str or NoneType
-        """
-        if mod:
-            if self.new_parent == NULL_ID:
-                return "./."
-            elif self.new_parent is None:
-                return None
-            return self.new_path
-        else:
-            if self.parent == NULL_ID:
-                return "./."
-            elif self.parent is None:
-                return None
-            return self.path
-
-    def summarize_name(self, changeset, reverse=False):
-        """Produce a one-line summary of the filename.  Indicates renames as
-        old => new, indicates creation as None => new, indicates deletion as
-        old => None.
-
-        :param changeset: The changeset to get paths from
-        :type changeset: `Changeset`
-        :param reverse: If true, reverse the names in the output
-        :type reverse: bool
-        :rtype: str
-        """
-        orig_path = self.get_cset_path(False)
-        mod_path = self.get_cset_path(True)
-        if orig_path is not None:
-            orig_path = orig_path[2:]
-        if mod_path is not None:
-            mod_path = mod_path[2:]
-        if orig_path == mod_path:
-            return orig_path
-        else:
-            if not reverse:
-                return "%s => %s" % (orig_path, mod_path)
-            else:
-                return "%s => %s" % (mod_path, orig_path)
-
-
-    def get_new_path(self, id_map, changeset, reverse=False):
-        """Determine the full pathname to rename to
-
-        :param id_map: The map of ids to filenames for the tree
-        :type id_map: Dictionary
-        :param changeset: The changeset to get data from
-        :type changeset: `Changeset`
-        :param reverse: If true, we're applying the changeset in reverse
-        :type reverse: bool
-        :rtype: str
-        """
-        mutter("Finding new path for %s" % self.summarize_name(changeset))
-        if reverse:
-            parent = self.parent
-            to_dir = self.dir
-            from_dir = self.new_dir
-            to_name = self.name
-            from_name = self.new_name
-        else:
-            parent = self.new_parent
-            to_dir = self.new_dir
-            from_dir = self.dir
-            to_name = self.new_name
-            from_name = self.name
-
-        if to_name is None:
-            return None
-
-        if parent == NULL_ID or parent is None:
-            if to_name != '.':
-                raise SourceRootHasName(self, to_name)
-            else:
-                return '.'
-        if from_dir == to_dir:
-            dir = os.path.dirname(id_map[self.id])
-        else:
-            mutter("path, new_path: %r %r" % (self.path, self.new_path))
-            parent_entry = changeset.entries[parent]
-            dir = parent_entry.get_new_path(id_map, changeset, reverse)
-        if from_name == to_name:
-            name = os.path.basename(id_map[self.id])
-        else:
-            name = to_name
-            assert(from_name is None or from_name == os.path.basename(id_map[self.id]))
-        return os.path.join(dir, name)
-
-    def is_boring(self):
-        """Determines whether the entry does nothing
-        
-        :return: True if the entry does no renames or content changes
-        :rtype: bool
-        """
-        if self.contents_change is not None:
-            return False
-        elif self.metadata_change is not None:
-            return False
-        elif self.parent != self.new_parent:
-            return False
-        elif self.name != self.new_name:
-            return False
-        else:
-            return True
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        """Applies the file content and/or metadata changes.
-
-        :param filename: the filename of the entry
-        :type filename: str
-        :param reverse: If true, apply the changes in reverse
-        :type reverse: bool
-        """
-        if self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
-        if self.contents_change is not None:
-            self.contents_change.apply(filename, conflict_handler, reverse)
-        if not self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
-
-class IDPresent(Exception):
-    def __init__(self, id):
-        msg = "Cannot add entry because that id has already been used:\n%s" %\
-            id
-        Exception.__init__(self, msg)
-        self.id = id
-
-class Changeset(object):
-    """A set of changes to apply"""
-    def __init__(self):
-        self.entries = {}
-
-    def add_entry(self, entry):
-        """Add an entry to the list of entries"""
-        if self.entries.has_key(entry.id):
-            raise IDPresent(entry.id)
-        self.entries[entry.id] = entry
-
-def my_sort(sequence, key, reverse=False):
-    """A sort function that supports supplying a key for comparison
-    
-    :param sequence: The sequence to sort
-    :param key: A callable object that returns the values to be compared
-    :param reverse: If true, sort in reverse order
-    :type reverse: bool
-    """
-    def cmp_by_key(entry_a, entry_b):
-        if reverse:
-            tmp=entry_a
-            entry_a = entry_b
-            entry_b = tmp
-        return cmp(key(entry_a), key(entry_b))
-    sequence.sort(cmp_by_key)
-
-def get_rename_entries(changeset, inventory, reverse):
-    """Return a list of entries that will be renamed.  Entries are sorted from
-    longest to shortest source path and from shortest to longest target path.
-
-    :param changeset: The changeset to look in
-    :type changeset: `Changeset`
-    :param inventory: The source of current tree paths for the given ids
-    :type inventory: Dictionary
-    :param reverse: If true, the changeset is being applied in reverse
-    :type reverse: bool
-    :return: source entries and target entries as a tuple
-    :rtype: (List, List)
-    """
-    source_entries = [x for x in changeset.entries.itervalues() 
-                      if x.needs_rename()]
-    # these are done from longest path to shortest, to avoid deleting a
-    # parent before its children are deleted/renamed 
-    def longest_to_shortest(entry):
-        path = inventory.get(entry.id)
-        if path is None:
-            return 0
-        else:
-            return len(path)
-    my_sort(source_entries, longest_to_shortest, reverse=True)
-
-    target_entries = source_entries[:]
-    # These are done from shortest to longest path, to avoid creating a
-    # child before its parent has been created/renamed
-    def shortest_to_longest(entry):
-        path = entry.get_new_path(inventory, changeset, reverse)
-        if path is None:
-            return 0
-        else:
-            return len(path)
-    my_sort(target_entries, shortest_to_longest)
-    return (source_entries, target_entries)
-
-def rename_to_temp_delete(source_entries, inventory, dir, temp_dir, 
-                          conflict_handler, reverse):
-    """Delete and rename entries as appropriate.  Entries are renamed to temp
-    names.  A map of id -> temp name (or None, for deletions) is returned.
-
-    :param source_entries: The entries to rename and delete
-    :type source_entries: List of `ChangesetEntry`
-    :param inventory: The map of id -> filename in the current tree
-    :type inventory: Dictionary
-    :param dir: The directory to apply changes to
-    :type dir: str
-    :param reverse: Apply changes in reverse
-    :type reverse: bool
-    :return: a mapping of id to temporary name
-    :rtype: Dictionary
-    """
-    temp_name = {}
-    for i in range(len(source_entries)):
-        entry = source_entries[i]
-        if entry.is_deletion(reverse):
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
-            temp_name[entry.id] = None
-
-        else:
-            to_name = os.path.join(temp_dir, str(i))
-            src_path = inventory.get(entry.id)
-            if src_path is not None:
-                src_path = os.path.join(dir, src_path)
-                try:
-                    os.rename(src_path, to_name)
-                    temp_name[entry.id] = to_name
-                except OSError, e:
-                    if e.errno != errno.ENOENT:
-                        raise
-                    if conflict_handler.missing_for_rename(src_path) == "skip":
-                        continue
-
-    return temp_name
-
-
-def rename_to_new_create(changed_inventory, target_entries, inventory, 
-                         changeset, dir, conflict_handler, reverse):
-    """Rename entries with temp names to their final names, create new files.
-
-    :param changed_inventory: A mapping of id to temporary name
-    :type changed_inventory: Dictionary
-    :param target_entries: The entries to apply changes to
-    :type target_entries: List of `ChangesetEntry`
-    :param changeset: The changeset to apply
-    :type changeset: `Changeset`
-    :param dir: The directory to apply changes to
-    :type dir: str
-    :param reverse: If true, apply changes in reverse
-    :type reverse: bool
-    """
-    for entry in target_entries:
-        new_tree_path = entry.get_new_path(inventory, changeset, reverse)
-        if new_tree_path is None:
-            continue
-        new_path = os.path.join(dir, new_tree_path)
-        old_path = changed_inventory.get(entry.id)
-        if os.path.exists(new_path):
-            if conflict_handler.target_exists(entry, new_path, old_path) == \
-                "skip":
-                continue
-        if entry.is_creation(reverse):
-            entry.apply(new_path, conflict_handler, reverse)
-            changed_inventory[entry.id] = new_tree_path
-        else:
-            if old_path is None:
-                continue
-            try:
-                os.rename(old_path, new_path)
-                changed_inventory[entry.id] = new_tree_path
-            except OSError, e:
-                raise Exception ("%s is missing" % new_path)
-
-class TargetExists(Exception):
-    def __init__(self, entry, target):
-        msg = "The path %s already exists" % target
-        Exception.__init__(self, msg)
-        self.entry = entry
-        self.target = target
-
-class RenameConflict(Exception):
-    def __init__(self, id, this_name, base_name, other_name):
-        msg = """Trees all have different names for a file
- this: %s
- base: %s
-other: %s
-   id: %s""" % (this_name, base_name, other_name, id)
-        Exception.__init__(self, msg)
-        self.this_name = this_name
-        self.base_name = base_name
-        self_other_name = other_name
-
-class MoveConflict(Exception):
-    def __init__(self, id, this_parent, base_parent, other_parent):
-        msg = """The file is in different directories in every tree
- this: %s
- base: %s
-other: %s
-   id: %s""" % (this_parent, base_parent, other_parent, id)
-        Exception.__init__(self, msg)
-        self.this_parent = this_parent
-        self.base_parent = base_parent
-        self_other_parent = other_parent
-
-class MergeConflict(Exception):
-    def __init__(self, this_path):
-        Exception.__init__(self, "Conflict applying changes to %s" % this_path)
-        self.this_path = this_path
-
-class MergePermissionConflict(Exception):
-    def __init__(self, this_path, base_path, other_path):
-        this_perms = os.stat(this_path).st_mode & 0755
-        base_perms = os.stat(base_path).st_mode & 0755
-        other_perms = os.stat(other_path).st_mode & 0755
-        msg = """Conflicting permission for %s
-this: %o
-base: %o
-other: %o
-        """ % (this_path, this_perms, base_perms, other_perms)
-        self.this_path = this_path
-        self.base_path = base_path
-        self.other_path = other_path
-        Exception.__init__(self, msg)
-
-class WrongOldContents(Exception):
-    def __init__(self, filename):
-        msg = "Contents mismatch deleting %s" % filename
-        self.filename = filename
-        Exception.__init__(self, msg)
-
-class WrongOldPermissions(Exception):
-    def __init__(self, filename, old_perms, new_perms):
-        msg = "Permission missmatch on %s:\n" \
-        "Expected 0%o, got 0%o." % (filename, old_perms, new_perms)
-        self.filename = filename
-        Exception.__init__(self, msg)
-
-class RemoveContentsConflict(Exception):
-    def __init__(self, filename):
-        msg = "Conflict deleting %s, which has different contents in BASE"\
-            " and THIS" % filename
-        self.filename = filename
-        Exception.__init__(self, msg)
-
-class DeletingNonEmptyDirectory(Exception):
-    def __init__(self, filename):
-        msg = "Trying to remove dir %s while it still had files" % filename
-        self.filename = filename
-        Exception.__init__(self, msg)
-
-
-class PatchTargetMissing(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to patch %s, which does not exist" % filename
-        Exception.__init__(self, msg)
-        self.filename = filename
-
-class MissingPermsFile(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to change permissions on  %s, which does not exist" %\
-            filename
-        Exception.__init__(self, msg)
-        self.filename = filename
-
-class MissingForRm(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to remove missing path %s" % filename
-        Exception.__init__(self, msg)
-        self.filename = filename
-
-
-class MissingForRename(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to move missing path %s" % (filename)
-        Exception.__init__(self, msg)
-        self.filename = filename
-
-class NewContentsConflict(Exception):
-    def __init__(self, filename):
-        msg = "Conflicting contents for new file %s" % (filename)
-        Exception.__init__(self, msg)
-
-
-class MissingForMerge(Exception):
-    def __init__(self, filename):
-        msg = "The file %s was modified, but does not exist in this tree"\
-            % (filename)
-        Exception.__init__(self, msg)
-
-
-class ExceptionConflictHandler(object):
-    """Default handler for merge exceptions.
-
-    This throws an error on any kind of conflict.  Conflict handlers can
-    descend from this class if they have a better way to handle some or
-    all types of conflict.
-    """
-    def __init__(self, dir):
-        self.dir = dir
-    
-    def missing_parent(self, pathname):
-        parent = os.path.dirname(pathname)
-        raise Exception("Parent directory missing for %s" % pathname)
-
-    def dir_exists(self, pathname):
-        raise Exception("Directory already exists for %s" % pathname)
-
-    def failed_hunks(self, pathname):
-        raise Exception("Failed to apply some hunks for %s" % pathname)
-
-    def target_exists(self, entry, target, old_path):
-        raise TargetExists(entry, target)
-
-    def rename_conflict(self, id, this_name, base_name, other_name):
-        raise RenameConflict(id, this_name, base_name, other_name)
-
-    def move_conflict(self, id, this_dir, base_dir, other_dir):
-        raise MoveConflict(id, this_dir, base_dir, other_dir)
-
-    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
-        os.unlink(new_file)
-        raise MergeConflict(this_path)
-
-    def permission_conflict(self, this_path, base_path, other_path):
-        raise MergePermissionConflict(this_path, base_path, other_path)
-
-    def wrong_old_contents(self, filename, expected_contents):
-        raise WrongOldContents(filename)
-
-    def rem_contents_conflict(self, filename, this_contents, base_contents):
-        raise RemoveContentsConflict(filename)
-
-    def wrong_old_perms(self, filename, old_perms, new_perms):
-        raise WrongOldPermissions(filename, old_perms, new_perms)
-
-    def rmdir_non_empty(self, filename):
-        raise DeletingNonEmptyDirectory(filename)
-
-    def link_name_exists(self, filename):
-        raise TargetExists(filename)
-
-    def patch_target_missing(self, filename, contents):
-        raise PatchTargetMissing(filename)
-
-    def missing_for_chmod(self, filename):
-        raise MissingPermsFile(filename)
-
-    def missing_for_rm(self, filename, change):
-        raise MissingForRm(filename)
-
-    def missing_for_rename(self, filename):
-        raise MissingForRename(filename)
-
-    def missing_for_merge(self, file_id, other_path):
-        raise MissingForMerge(other_path)
-
-    def new_contents_conflict(self, filename, other_contents):
-        raise NewContentsConflict(filename)
-
-    def finalize():
-        pass
-
-def apply_changeset(changeset, inventory, dir, conflict_handler=None, 
-                    reverse=False):
-    """Apply a changeset to a directory.
-
-    :param changeset: The changes to perform
-    :type changeset: `Changeset`
-    :param inventory: The mapping of id to filename for the directory
-    :type inventory: Dictionary
-    :param dir: The path of the directory to apply the changes to
-    :type dir: str
-    :param reverse: If true, apply the changes in reverse
-    :type reverse: bool
-    :return: The mapping of the changed entries
-    :rtype: Dictionary
-    """
-    if conflict_handler is None:
-        conflict_handler = ExceptionConflictHandler(dir)
-    temp_dir = os.path.join(dir, "bzr-tree-change")
-    try:
-        os.mkdir(temp_dir)
-    except OSError, e:
-        if e.errno == errno.EEXIST:
-            try:
-                os.rmdir(temp_dir)
-            except OSError, e:
-                if e.errno == errno.ENOTEMPTY:
-                    raise OldFailedTreeOp()
-            os.mkdir(temp_dir)
-        else:
-            raise
-    
-    #apply changes that don't affect filenames
-    for entry in changeset.entries.itervalues():
-        if not entry.is_creation_or_deletion():
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
-
-    # Apply renames in stages, to minimize conflicts:
-    # Only files whose name or parent change are interesting, because their
-    # target name may exist in the source tree.  If a directory's name changes,
-    # that doesn't make its children interesting.
-    (source_entries, target_entries) = get_rename_entries(changeset, inventory,
-                                                          reverse)
-
-    changed_inventory = rename_to_temp_delete(source_entries, inventory, dir,
-                                              temp_dir, conflict_handler,
-                                              reverse)
-
-    rename_to_new_create(changed_inventory, target_entries, inventory,
-                         changeset, dir, conflict_handler, reverse)
-    os.rmdir(temp_dir)
-    return changed_inventory
-
-
-def apply_changeset_tree(cset, tree, reverse=False):
-    r_inventory = {}
-    for entry in tree.source_inventory().itervalues():
-        inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.root,
-                                    reverse=reverse)
-    new_entries, remove_entries = \
-        get_inventory_change(inventory, new_inventory, cset, reverse)
-    tree.update_source_inventory(new_entries, remove_entries)
-
-
-def get_inventory_change(inventory, new_inventory, cset, reverse=False):
-    new_entries = {}
-    remove_entries = []
-    for entry in cset.entries.itervalues():
-        if entry.needs_rename():
-            new_path = entry.get_new_path(inventory, cset)
-            if new_path is None:
-                remove_entries.append(entry.id)
-            else:
-                new_entries[new_path] = entry.id
-    return new_entries, remove_entries
-
-
-def print_changeset(cset):
-    """Print all non-boring changeset entries
-    
-    :param cset: The changeset to print
-    :type cset: `Changeset`
-    """
-    for entry in cset.entries.itervalues():
-        if entry.is_boring():
-            continue
-        print entry.id
-        print entry.summarize_name(cset)
-
-class CompositionFailure(Exception):
-    def __init__(self, old_entry, new_entry, problem):
-        msg = "Unable to conpose entries.\n %s" % problem
-        Exception.__init__(self, msg)
-
-class IDMismatch(CompositionFailure):
-    def __init__(self, old_entry, new_entry):
-        problem = "Attempt to compose entries with different ids: %s and %s" %\
-            (old_entry.id, new_entry.id)
-        CompositionFailure.__init__(self, old_entry, new_entry, problem)
-
-def compose_changesets(old_cset, new_cset):
-    """Combine two changesets into one.  This works well for exact patching.
-    Otherwise, not so well.
-
-    :param old_cset: The first changeset that would be applied
-    :type old_cset: `Changeset`
-    :param new_cset: The second changeset that would be applied
-    :type new_cset: `Changeset`
-    :return: A changeset that combines the changes in both changesets
-    :rtype: `Changeset`
-    """
-    composed = Changeset()
-    for old_entry in old_cset.entries.itervalues():
-        new_entry = new_cset.entries.get(old_entry.id)
-        if new_entry is None:
-            composed.add_entry(old_entry)
-        else:
-            composed_entry = compose_entries(old_entry, new_entry)
-            if composed_entry.parent is not None or\
-                composed_entry.new_parent is not None:
-                composed.add_entry(composed_entry)
-    for new_entry in new_cset.entries.itervalues():
-        if not old_cset.entries.has_key(new_entry.id):
-            composed.add_entry(new_entry)
-    return composed
-
-def compose_entries(old_entry, new_entry):
-    """Combine two entries into one.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: ChangesetEntry
-    :param old_entry: The second entry that would be applied
-    :type old_entry: ChangesetEntry
-    :return: A changeset entry combining both entries
-    :rtype: `ChangesetEntry`
-    """
-    if old_entry.id != new_entry.id:
-        raise IDMismatch(old_entry, new_entry)
-    output = ChangesetEntry(old_entry.id, old_entry.parent, old_entry.path)
-
-    if (old_entry.parent != old_entry.new_parent or 
-        new_entry.parent != new_entry.new_parent):
-        output.new_parent = new_entry.new_parent
-
-    if (old_entry.path != old_entry.new_path or 
-        new_entry.path != new_entry.new_path):
-        output.new_path = new_entry.new_path
-
-    output.contents_change = compose_contents(old_entry, new_entry)
-    output.metadata_change = compose_metadata(old_entry, new_entry)
-    return output
-
-def compose_contents(old_entry, new_entry):
-    """Combine the contents of two changeset entries.  Entries are combined
-    intelligently where possible, but the fallback behavior returns an 
-    ApplySequence.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: `ChangesetEntry`
-    :param new_entry: The second entry that would be applied
-    :type new_entry: `ChangesetEntry`
-    :return: A combined contents change
-    :rtype: anything supporting the apply(reverse=False) method
-    """
-    old_contents = old_entry.contents_change
-    new_contents = new_entry.contents_change
-    if old_entry.contents_change is None:
-        return new_entry.contents_change
-    elif new_entry.contents_change is None:
-        return old_entry.contents_change
-    elif isinstance(old_contents, ReplaceContents) and \
-        isinstance(new_contents, ReplaceContents):
-        if old_contents.old_contents == new_contents.new_contents:
-            return None
-        else:
-            return ReplaceContents(old_contents.old_contents,
-                                   new_contents.new_contents)
-    elif isinstance(old_contents, ApplySequence):
-        output = ApplySequence(old_contents.changes)
-        if isinstance(new_contents, ApplySequence):
-            output.changes.extend(new_contents.changes)
-        else:
-            output.changes.append(new_contents)
-        return output
-    elif isinstance(new_contents, ApplySequence):
-        output = ApplySequence((old_contents.changes,))
-        output.extend(new_contents.changes)
-        return output
-    else:
-        return ApplySequence((old_contents, new_contents))
-
-def compose_metadata(old_entry, new_entry):
-    old_meta = old_entry.metadata_change
-    new_meta = new_entry.metadata_change
-    if old_meta is None:
-        return new_meta
-    elif new_meta is None:
-        return old_meta
-    elif isinstance(old_meta, ChangeUnixPermissions) and \
-        isinstance(new_meta, ChangeUnixPermissions):
-        return ChangeUnixPermissions(old_meta.old_mode, new_meta.new_mode)
-    else:
-        return ApplySequence(old_meta, new_meta)
-
-
-def changeset_is_null(changeset):
-    for entry in changeset.entries.itervalues():
-        if not entry.is_boring():
-            return False
-    return True
-
-class UnsuppportedFiletype(Exception):
-    def __init__(self, full_path, stat_result):
-        msg = "The file \"%s\" is not a supported filetype." % full_path
-        Exception.__init__(self, msg)
-        self.full_path = full_path
-        self.stat_result = stat_result
-
-def generate_changeset(tree_a, tree_b, interesting_ids=None):
-    return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
-
-class ChangesetGenerator(object):
-    def __init__(self, tree_a, tree_b, interesting_ids=None):
-        object.__init__(self)
-        self.tree_a = tree_a
-        self.tree_b = tree_b
-        self._interesting_ids = interesting_ids
-
-    def iter_both_tree_ids(self):
-        for file_id in self.tree_a:
-            yield file_id
-        for file_id in self.tree_b:
-            if file_id not in self.tree_a:
-                yield file_id
-
-    def __call__(self):
-        cset = Changeset()
-        for file_id in self.iter_both_tree_ids():
-            cs_entry = self.make_entry(file_id)
-            if cs_entry is not None and not cs_entry.is_boring():
-                cset.add_entry(cs_entry)
-
-        for entry in list(cset.entries.itervalues()):
-            if entry.parent != entry.new_parent:
-                if not cset.entries.has_key(entry.parent) and\
-                    entry.parent != NULL_ID and entry.parent is not None:
-                    parent_entry = self.make_boring_entry(entry.parent)
-                    cset.add_entry(parent_entry)
-                if not cset.entries.has_key(entry.new_parent) and\
-                    entry.new_parent != NULL_ID and \
-                    entry.new_parent is not None:
-                    parent_entry = self.make_boring_entry(entry.new_parent)
-                    cset.add_entry(parent_entry)
-        return cset
-
-    def iter_inventory(self, tree):
-        for file_id in tree:
-            yield self.get_entry(file_id, tree)
-
-    def get_entry(self, file_id, tree):
-        if file_id not in tree:
-            return None
-        return tree.tree.inventory[file_id]
-
-    def get_entry_parent(self, entry):
-        if entry is None:
-            return None
-        return entry.parent_id
-
-    def get_path(self, file_id, tree):
-        if not tree.has_id(file_id):
-            return None
-        path = tree.id2path(file_id)
-        if path == '':
-            return './.'
-        else:
-            return path
-
-    def make_basic_entry(self, file_id, only_interesting):
-        entry_a = self.get_entry(file_id, self.tree_a)
-        entry_b = self.get_entry(file_id, self.tree_b)
-        if only_interesting and not self.is_interesting(entry_a, entry_b):
-            return None
-        parent = self.get_entry_parent(entry_a)
-        path = self.get_path(file_id, self.tree_a)
-        cs_entry = ChangesetEntry(file_id, parent, path)
-        new_parent = self.get_entry_parent(entry_b)
-
-        new_path = self.get_path(file_id, self.tree_b)
-
-        cs_entry.new_path = new_path
-        cs_entry.new_parent = new_parent
-        return cs_entry
-
-    def is_interesting(self, entry_a, entry_b):
-        if self._interesting_ids is None:
-            return True
-        if entry_a is not None:
-            file_id = entry_a.file_id
-        elif entry_b is not None:
-            file_id = entry_b.file_id
-        else:
-            return False
-        return file_id in self._interesting_ids
-
-    def make_boring_entry(self, id):
-        cs_entry = self.make_basic_entry(id, only_interesting=False)
-        if cs_entry.is_creation_or_deletion():
-            return self.make_entry(id, only_interesting=False)
-        else:
-            return cs_entry
-        
-
-    def make_entry(self, id, only_interesting=True):
-        cs_entry = self.make_basic_entry(id, only_interesting)
-
-        if cs_entry is None:
-            return None
-        if id in self.tree_a and id in self.tree_b:
-            a_sha1 = self.tree_a.get_file_sha1(id)
-            b_sha1 = self.tree_b.get_file_sha1(id)
-            if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
-                return cs_entry
-
-        full_path_a = self.tree_a.readonly_path(id)
-        full_path_b = self.tree_b.readonly_path(id)
-        stat_a = self.lstat(full_path_a)
-        stat_b = self.lstat(full_path_b)
-        if stat_b is None:
-            cs_entry.new_parent = None
-            cs_entry.new_path = None
-        
-        cs_entry.metadata_change = self.make_mode_change(stat_a, stat_b)
-        cs_entry.contents_change = self.make_contents_change(full_path_a,
-                                                             stat_a, 
-                                                             full_path_b, 
-                                                             stat_b)
-        return cs_entry
-
-    def make_mode_change(self, stat_a, stat_b):
-        mode_a = None
-        if stat_a is not None and not stat.S_ISLNK(stat_a.st_mode):
-            mode_a = stat_a.st_mode & 0777
-        mode_b = None
-        if stat_b is not None and not stat.S_ISLNK(stat_b.st_mode):
-            mode_b = stat_b.st_mode & 0777
-        if mode_a == mode_b:
-            return None
-        return ChangeUnixPermissions(mode_a, mode_b)
-
-    def make_contents_change(self, full_path_a, stat_a, full_path_b, stat_b):
-        if stat_a is None and stat_b is None:
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISDIR(stat_a.st_mode) and\
-            stat.S_ISDIR(stat_b.st_mode):
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISREG(stat_a.st_mode) and\
-            stat.S_ISREG(stat_b.st_mode):
-            if stat_a.st_ino == stat_b.st_ino and \
-                stat_a.st_dev == stat_b.st_dev:
-                return None
-
-        a_contents = self.get_contents(stat_a, full_path_a)
-        b_contents = self.get_contents(stat_b, full_path_b)
-        if a_contents == b_contents:
-            return None
-        return ReplaceContents(a_contents, b_contents)
-
-    def get_contents(self, stat_result, full_path):
-        if stat_result is None:
-            return None
-        elif stat.S_ISREG(stat_result.st_mode):
-            return FileCreate(file(full_path, "rb").read())
-        elif stat.S_ISDIR(stat_result.st_mode):
-            return dir_create
-        elif stat.S_ISLNK(stat_result.st_mode):
-            return SymlinkCreate(os.readlink(full_path))
-        else:
-            raise UnsupportedFiletype(full_path, stat_result)
-
-    def lstat(self, full_path):
-        stat_result = None
-        if full_path is not None:
-            try:
-                stat_result = os.lstat(full_path)
-            except OSError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-        return stat_result
-
-
-def full_path(entry, tree):
-    return os.path.join(tree.root, entry.path)
-
-def new_delete_entry(entry, tree, inventory, delete):
-    if entry.path == "":
-        parent = NULL_ID
-    else:
-        parent = inventory[dirname(entry.path)].id
-    cs_entry = ChangesetEntry(parent, entry.path)
-    if delete:
-        cs_entry.new_path = None
-        cs_entry.new_parent = None
-    else:
-        cs_entry.path = None
-        cs_entry.parent = None
-    full_path = full_path(entry, tree)
-    status = os.lstat(full_path)
-    if stat.S_ISDIR(file_stat.st_mode):
-        action = dir_create
-    
-
-
-        
-# XXX: Can't we unify this with the regular inventory object
-class Inventory(object):
-    def __init__(self, inventory):
-        self.inventory = inventory
-        self.rinventory = None
-
-    def get_rinventory(self):
-        if self.rinventory is None:
-            self.rinventory  = invert_dict(self.inventory)
-        return self.rinventory
-
-    def get_path(self, id):
-        return self.inventory.get(id)
-
-    def get_name(self, id):
-        path = self.get_path(id)
-        if path is None:
-            return None
-        else:
-            return os.path.basename(path)
-
-    def get_dir(self, id):
-        path = self.get_path(id)
-        if path == "":
-            return None
-        if path is None:
-            return None
-        return os.path.dirname(path)
-
-    def get_parent(self, id):
-        if self.get_path(id) is None:
-            return None
-        directory = self.get_dir(id)
-        if directory == '.':
-            directory = './.'
-        if directory is None:
-            return NULL_ID
-        return self.get_rinventory().get(directory)

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-08-04 22:12:40 +0000
+++ bzrlib/check.py	2006-03-20 22:32:41 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,222 +14,225 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-def _update_store_entry(obj, obj_id, branch, store_name, store):
-    """This is just a meta-function, which handles both revision entries
-    and inventory entries.
-    """
-    from bzrlib.trace import mutter
-    import tempfile, os, errno
-    from osutils import rename
-    obj_tmp = tempfile.TemporaryFile()
-    obj.write_xml(obj_tmp)
-    obj_tmp.seek(0)
-
-    tmpfd, tmp_path = tempfile.mkstemp(prefix=obj_id, suffix='.gz',
-        dir=branch.controlfilename(store_name))
-    os.close(tmpfd)
-    try:
-        orig_obj_path = branch.controlfilename([store_name, obj_id+'.gz'])
-        # Remove the old entry out of the way
-        rename(orig_obj_path, tmp_path)
-        try:
-            # TODO: We may need to handle the case where the old
-            # entry was not compressed (and thus did not end with .gz)
-
-            store.add(obj_tmp, obj_id) # Add the new one
-            os.remove(tmp_path) # Remove the old name
-            mutter('    Updated %s entry {%s}' % (store_name, obj_id))
-        except:
-            # On any exception, restore the old entry
-            rename(tmp_path, orig_obj_path)
-            raise
-    finally:
-        if os.path.exists(tmp_path):
-            # Unfortunately, the next command might throw
-            # an exception, which will mask a previous exception.
-            os.remove(tmp_path)
-        obj_tmp.close()
-
-def _update_revision_entry(rev, branch):
-    """After updating the values in a revision, make sure to
-    write out the data, but try to do it in an atomic manner.
-
-    :param rev:    The Revision object to store
-    :param branch: The Branch object where this Revision is to be stored.
-    """
-    _update_store_entry(rev, rev.revision_id, branch,
-            'revision-store', branch.revision_store)
-
-def _update_inventory_entry(inv, inv_id, branch):
-    """When an inventory has been modified (such as by adding a unique tree root)
-    this atomically re-generates the file.
-
-    :param inv:     The Inventory
-    :param inv_id:  The inventory id for this inventory
-    :param branch:  The Branch where this entry will be stored.
-    """
-    _update_store_entry(inv, inv_id, branch,
-            'inventory-store', branch.inventory_store)
-
-def check(branch):
-    """Run consistency checks on a branch.
-
-    TODO: Also check non-mainline revisions mentioned as parents.
-
-    TODO: Check for extra files in the control directory.
-    """
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
-    from bzrlib.osutils import fingerprint_file
-    from bzrlib.progress import ProgressBar
-    from bzrlib.inventory import ROOT_ID
-    from bzrlib.branch import gen_root_id
-
-    branch.lock_read()
-
-    try:
-        pb = ProgressBar(show_spinner=True)
+# TODO: Check ancestries are correct for every revision: includes
+# every committed so far, and in a reasonable order.
+
+# TODO: Also check non-mainline revisions mentioned as parents.
+
+# TODO: Check for extra files in the control directory.
+
+# TODO: Check revision, inventory and entry objects have all 
+# required fields.
+
+# TODO: Get every revision in the revision-store even if they're not
+# referenced by history and make sure they're all valid.
+
+# TODO: Perhaps have a way to record errors other than by raising exceptions;
+# would perhaps be enough to accumulate exception objects in a list without
+# raising them.  If there's more than one exception it'd be good to see them
+# all.
+
+import bzrlib.ui
+from bzrlib.trace import note, warning
+from bzrlib.osutils import rename, sha_string, fingerprint_file
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrCheckError, NoSuchRevision
+from bzrlib.inventory import ROOT_ID
+
+
+class Check(object):
+    """Check a branch"""
+
+    # The Check object interacts with InventoryEntry.check, etc.
+
+    def __init__(self, branch):
+        self.branch = branch
+        self.repository = branch.repository
+        self.checked_text_cnt = 0
+        self.checked_rev_cnt = 0
+        self.ghosts = []
+        self.repeated_text_cnt = 0
+        self.missing_parent_links = {}
+        self.missing_inventory_sha_cnt = 0
+        self.missing_revision_cnt = 0
+        # maps (file-id, version) -> sha1; used by InventoryFile._check
+        self.checked_texts = {}
+        self.checked_weaves = {}
+
+    def check(self):
+        self.branch.lock_read()
+        self.progress = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            self.progress.update('retrieving inventory', 0, 0)
+            # do not put in init, as it should be done with progess,
+            # and inside the lock.
+            self.inventory_weave = self.branch.repository.get_inventory_weave()
+            self.history = self.branch.revision_history()
+            if not len(self.history):
+                # nothing to see here
+                return
+            self.plan_revisions()
+            revno = 0
+            self.check_weaves()
+            while revno < len(self.planned_revisions):
+                rev_id = self.planned_revisions[revno]
+                self.progress.update('checking revision', revno,
+                                     len(self.planned_revisions))
+                revno += 1
+                self.check_one_rev(rev_id)
+        finally:
+            self.progress.finished()
+            self.branch.unlock()
+
+    def plan_revisions(self):
+        repository = self.branch.repository
+        self.planned_revisions = set(repository.all_revision_ids())
+        self.progress.clear()
+        inventoried = set(self.inventory_weave.versions())
+        awol = self.planned_revisions - inventoried
+        if len(awol) > 0:
+            raise BzrCheckError('Stored revisions missing from inventory'
+                '{%s}' % ','.join([f for f in awol]))
+        self.planned_revisions = list(self.planned_revisions)
+
+    def report_results(self, verbose):
+        note('checked branch %s format %s',
+             self.branch.base, 
+             self.branch._format)
+
+        note('%6d revisions', self.checked_rev_cnt)
+        note('%6d unique file texts', self.checked_text_cnt)
+        note('%6d repeated file texts', self.repeated_text_cnt)
+        note('%6d weaves', len(self.checked_weaves))
+        if self.missing_inventory_sha_cnt:
+            note('%6d revisions are missing inventory_sha1',
+                 self.missing_inventory_sha_cnt)
+        if self.missing_revision_cnt:
+            note('%6d revisions are mentioned but not present',
+                 self.missing_revision_cnt)
+        if len(self.ghosts):
+            note('%6d ghost revisions', len(self.ghosts))
+            if verbose:
+                for ghost in self.ghosts:
+                    note('      %s', ghost)
+        if len(self.missing_parent_links):
+            note('%6d revisions missing parents in ancestry', 
+                 len(self.missing_parent_links))
+            if verbose:
+                for link, linkers in self.missing_parent_links.items():
+                    note('      %s should be in the ancestry for:', link)
+                    for linker in linkers:
+                        note('       * %s', linker)
+
+    def check_one_rev(self, rev_id):
+        """Check one revision.
+
+        rev_id - the one to check
+
+        last_rev_id - the previous one on the mainline, if any.
+        """
+
+        # mutter('    revision {%s}', rev_id)
+        branch = self.branch
+        try:
+            rev_history_position = self.history.index(rev_id)
+        except ValueError:
+            rev_history_position = None
         last_rev_id = None
-
-        missing_inventory_sha_cnt = 0
-        missing_revision_sha_cnt = 0
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-        mismatch_inv_id = []
-
-        # for all texts checked, text_id -> sha1
-        checked_texts = {}
-
-        for rev_id in history:
-            revno += 1
-            pb.update('checking revision', revno, revcount)
-            mutter('    revision {%s}' % rev_id)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            # check the previous history entry is a parent of this entry
-            if rev.parents:
-                if last_rev_id is None:
-                    raise BzrCheckError("revision {%s} has %d parents, but is the "
-                                        "start of the branch"
-                                        % (rev_id, len(rev.parents)))
-                for prr in rev.parents:
-                    if prr.revision_id == last_rev_id:
+        if rev_history_position:
+            rev = branch.repository.get_revision(rev_id)
+            if rev_history_position > 0:
+                last_rev_id = self.history[rev_history_position - 1]
+        else:
+            rev = branch.repository.get_revision(rev_id)
+                
+        if rev.revision_id != rev_id:
+            raise BzrCheckError('wrong internal revision id in revision {%s}'
+                                % rev_id)
+
+        # check the previous history entry is a parent of this entry
+        if rev.parent_ids:
+            if last_rev_id is not None:
+                for parent_id in rev.parent_ids:
+                    if parent_id == last_rev_id:
                         break
                 else:
                     raise BzrCheckError("previous revision {%s} not listed among "
                                         "parents of {%s}"
                                         % (last_rev_id, rev_id))
-
-                for prr in rev.parents:
-                    if prr.revision_sha1 is None:
-                        missing_revision_sha_cnt += 1
-                        continue
-                    prid = prr.revision_id
-                    actual_sha = branch.get_revision_sha1(prid)
-                    if prr.revision_sha1 != actual_sha:
-                        raise BzrCheckError("mismatched revision sha1 for "
-                                            "parent {%s} of {%s}: %s vs %s"
-                                            % (prid, rev_id,
-                                               prr.revision_sha1, actual_sha))
-            elif last_rev_id:
-                raise BzrCheckError("revision {%s} has no parents listed but preceded "
-                                    "by {%s}"
-                                    % (rev_id, last_rev_id))
-
-            if rev.inventory_id != rev_id:
-                mismatch_inv_id.append(rev_id)
-
-            ## TODO: Check all the required fields are present on the revision.
-
-            if rev.inventory_sha1:
-                inv_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                if inv_sha1 != rev.inventory_sha1:
-                    raise BzrCheckError('Inventory sha1 hash doesn\'t match'
-                        ' value in revision {%s}' % rev_id)
-            else:
-                missing_inventory_sha_cnt += 1
-                mutter("no inventory_sha1 on revision {%s}" % rev_id)
-
-            inv = branch.get_inventory(rev.inventory_id)
-            seen_ids = {}
-            seen_names = {}
-
-            ## p('revision %d/%d file ids' % (revno, revcount))
-            for file_id in inv:
-                if file_id in seen_ids:
-                    raise BzrCheckError('duplicated file_id {%s} '
-                                        'in inventory for revision {%s}'
-                                        % (file_id, rev_id))
-                seen_ids[file_id] = True
-
-            i = 0
-            for file_id in inv:
-                i += 1
-                if i & 31 == 0:
-                    pb.tick()
-
-                ie = inv[file_id]
-
-                if ie.parent_id != None:
-                    if ie.parent_id not in seen_ids:
-                        raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
-                                % (ie.parent_id, rev_id))
-
-                if ie.kind == 'file':
-                    if ie.text_id in checked_texts:
-                        fp = checked_texts[ie.text_id]
+            for parent in rev.parent_ids:
+                if not parent in self.planned_revisions:
+                    missing_links = self.missing_parent_links.get(parent, [])
+                    missing_links.append(rev_id)
+                    self.missing_parent_links[parent] = missing_links
+                    # list based so somewhat slow,
+                    # TODO have a planned_revisions list and set.
+                    if self.branch.repository.has_revision(parent):
+                        missing_ancestry = self.repository.get_ancestry(parent)
+                        for missing in missing_ancestry:
+                            if (missing is not None 
+                                and missing not in self.planned_revisions):
+                                self.planned_revisions.append(missing)
                     else:
-                        if not ie.text_id in branch.text_store:
-                            raise BzrCheckError('text {%s} not in text_store' % ie.text_id)
-
-                        tf = branch.text_store[ie.text_id]
-                        fp = fingerprint_file(tf)
-                        checked_texts[ie.text_id] = fp
-
-                    if ie.text_size != fp['size']:
-                        raise BzrCheckError('text {%s} wrong size' % ie.text_id)
-                    if ie.text_sha1 != fp['sha1']:
-                        raise BzrCheckError('text {%s} wrong sha1' % ie.text_id)
-                elif ie.kind == 'directory':
-                    if ie.text_sha1 != None or ie.text_size != None or ie.text_id != None:
-                        raise BzrCheckError('directory {%s} has text in revision {%s}'
-                                % (file_id, rev_id))
-
-            pb.tick()
-            for path, ie in inv.iter_entries():
-                if path in seen_names:
-                    raise BzrCheckError('duplicated path %s '
-                                        'in inventory for revision {%s}'
-                                        % (path, rev_id))
+                        self.ghosts.append(rev_id)
+        elif last_rev_id:
+            raise BzrCheckError("revision {%s} has no parents listed "
+                                "but preceded by {%s}"
+                                % (rev_id, last_rev_id))
+
+        if rev.inventory_sha1:
+            inv_sha1 = branch.repository.get_inventory_sha1(rev_id)
+            if inv_sha1 != rev.inventory_sha1:
+                raise BzrCheckError('Inventory sha1 hash doesn\'t match'
+                    ' value in revision {%s}' % rev_id)
+        else:
+            missing_inventory_sha_cnt += 1
+            mutter("no inventory_sha1 on revision {%s}", rev_id)
+        self._check_revision_tree(rev_id)
+        self.checked_rev_cnt += 1
+
+    def check_weaves(self):
+        """Check all the weaves we can get our hands on.
+        """
+        n_weaves = 1
+        weave_ids = []
+        if self.branch.repository.weave_store.listable():
+            weave_ids = list(self.branch.repository.weave_store)
+            n_weaves = len(weave_ids)
+        self.progress.update('checking weave', 0, n_weaves)
+        self.inventory_weave.check(progress_bar=self.progress)
+        for i, weave_id in enumerate(weave_ids):
+            self.progress.update('checking weave', i, n_weaves)
+            w = self.branch.repository.weave_store.get_weave(weave_id,
+                    self.branch.repository.get_transaction())
+            # No progress here, because it looks ugly.
+            w.check()
+            self.checked_weaves[weave_id] = True
+
+    def _check_revision_tree(self, rev_id):
+        tree = self.branch.repository.revision_tree(rev_id)
+        inv = tree.inventory
+        seen_ids = {}
+        for file_id in inv:
+            if file_id in seen_ids:
+                raise BzrCheckError('duplicated file_id {%s} '
+                                    'in inventory for revision {%s}'
+                                    % (file_id, rev_id))
+            seen_ids[file_id] = True
+        for file_id in inv:
+            ie = inv[file_id]
+            ie.check(self, rev_id, inv, tree)
+        seen_names = {}
+        for path, ie in inv.iter_entries():
+            if path in seen_names:
+                raise BzrCheckError('duplicated path %s '
+                                    'in inventory for revision {%s}'
+                                    % (path, rev_id))
             seen_names[path] = True
-            last_rev_id = rev_id
-
-    finally:
-        branch.unlock()
-
-    pb.clear()
-
-    print 'checked %d revisions, %d file texts' % (revcount, len(checked_texts))
-    
-    if missing_inventory_sha_cnt:
-        print '%d revisions are missing inventory_sha1' % missing_inventory_sha_cnt
-
-    if missing_revision_sha_cnt:
-        print '%d parent links are missing revision_sha1' % missing_revision_sha_cnt
-
-    # stub this out for now because the main bzr branch has references
-    # to revisions that aren't present in the store -- mbp 20050804
-#    if (missing_inventory_sha_cnt
-#        or missing_revision_sha_cnt):
-#        print '  (use "bzr upgrade" to fix them)'
-
-    if mismatch_inv_id:
-        print '%d revisions have mismatched inventory ids:' % len(mismatch_inv_id)
-        for rev_id in mismatch_inv_id:
-            print '  ', rev_id
+
+
+def check(branch, verbose):
+    """Run consistency checks on a branch."""
+    checker = Check(branch)
+    checker.check()
+    checker.report_results(verbose)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-08-22 16:31:16 +0000
+++ bzrlib/commands.py	2006-04-17 13:20:55 +0000
@@ -1,40 +1,61 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
-
+# Copyright (C) 2006 by Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-# TODO: Split the command framework away from the actual commands.
-
 # TODO: probably should say which arguments are candidates for glob
 # expansion on windows and do that at the command level.
 
+# TODO: Define arguments by objects, rather than just using names.
+# Those objects can specify the expected type of the argument, which
+# would help with validation and shell completion.  They could also provide
+# help/explanation for that argument in a structured way.
+
+# TODO: Specific "examples" property on commands for consistent formatting.
+
+# TODO: "--profile=cum", to change sort order.  Is there any value in leaving
+# the profile output behind so it can be interactively examined?
+
 import sys
 import os
+from warnings import warn
+import errno
 
 import bzrlib
-from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
-from bzrlib.branch import find_branch
-from bzrlib import BZRDIR
-
+from bzrlib.errors import (BzrError,
+                           BzrCheckError,
+                           BzrCommandError,
+                           BzrOptionError,
+                           NotBranchError)
+from bzrlib.option import Option
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.symbol_versioning import *
+import bzrlib.trace
+from bzrlib.trace import mutter, note, log_error, warning, be_quiet
 
 plugin_cmds = {}
 
 
-def register_command(cmd):
-    "Utility function to help register a command"
+def register_command(cmd, decorate=False):
+    """Utility function to help register a command
+
+    :param cmd: Command subclass to register
+    :param decorate: If true, allow overriding an existing command
+        of the same name; the old command is returned by this function.
+        Otherwise it is an error to try to override an existing command.
+    """
     global plugin_cmds
     k = cmd.__name__
     if k.startswith("cmd_"):
@@ -43,6 +64,13 @@
         k_unsquished = k
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
+        mutter('registered plugin command %s', k_unsquished)
+        if decorate and k_unsquished in builtin_command_names():
+            return _builtin_commands()[k_unsquished]
+    elif decorate:
+        result = plugin_cmds[k_unsquished]
+        plugin_cmds[k_unsquished] = cmd
+        return result
     else:
         log_error('Two plugins defined the same command: %r' % k)
         log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
@@ -57,117 +85,31 @@
     return cmd[4:].replace('_','-')
 
 
-def _parse_revision_str(revstr):
-    """This handles a revision string -> revno.
-
-    This always returns a list.  The list will have one element for 
-
-    It supports integers directly, but everything else it
-    defers for passing to Branch.get_revision_info()
-
-    >>> _parse_revision_str('234')
-    [234]
-    >>> _parse_revision_str('234..567')
-    [234, 567]
-    >>> _parse_revision_str('..')
-    [None, None]
-    >>> _parse_revision_str('..234')
-    [None, 234]
-    >>> _parse_revision_str('234..')
-    [234, None]
-    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [234, 456, 789]
-    >>> _parse_revision_str('234....789') # Error?
-    [234, None, 789]
-    >>> _parse_revision_str('revid:test@other.com-234234')
-    ['revid:test@other.com-234234']
-    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    ['revid:test@other.com-234234', 'revid:test@other.com-234235']
-    >>> _parse_revision_str('revid:test@other.com-234234..23')
-    ['revid:test@other.com-234234', 23]
-    >>> _parse_revision_str('date:2005-04-12')
-    ['date:2005-04-12']
-    >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    ['date:2005-04-12 12:24:33']
-    >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    ['date:2005-04-12T12:24:33']
-    >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    ['date:2005-04-12,12:24:33']
-    >>> _parse_revision_str('-5..23')
-    [-5, 23]
-    >>> _parse_revision_str('-5')
-    [-5]
-    >>> _parse_revision_str('123a')
-    ['123a']
-    >>> _parse_revision_str('abc')
-    ['abc']
-    """
-    import re
-    old_format_re = re.compile('\d*:\d*')
-    m = old_format_re.match(revstr)
-    if m:
-        warning('Colon separator for revision numbers is deprecated.'
-                ' Use .. instead')
-        revs = []
-        for rev in revstr.split(':'):
-            if rev:
-                revs.append(int(rev))
-            else:
-                revs.append(None)
-        return revs
-    revs = []
-    for x in revstr.split('..'):
-        if not x:
-            revs.append(None)
-        else:
-            try:
-                revs.append(int(x))
-            except ValueError:
-                revs.append(x)
-    return revs
-
-
-def get_merge_type(typestring):
-    """Attempt to find the merge class/factory associated with a string."""
-    from merge import merge_types
-    try:
-        return merge_types[typestring][0]
-    except KeyError:
-        templ = '%s%%7s: %%s' % (' '*12)
-        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
-        type_list = '\n'.join(lines)
-        msg = "No known merge type %s. Supported types are:\n%s" %\
-            (typestring, type_list)
-        raise BzrCommandError(msg)
-    
-
-def get_merge_type(typestring):
-    """Attempt to find the merge class/factory associated with a string."""
-    from merge import merge_types
-    try:
-        return merge_types[typestring][0]
-    except KeyError:
-        templ = '%s%%7s: %%s' % (' '*12)
-        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
-        type_list = '\n'.join(lines)
-        msg = "No known merge type %s. Supported types are:\n%s" %\
-            (typestring, type_list)
-        raise BzrCommandError(msg)
-    
+def _builtin_commands():
+    import bzrlib.builtins
+    r = {}
+    builtins = bzrlib.builtins.__dict__
+    for name in builtins:
+        if name.startswith("cmd_"):
+            real_name = _unsquish_command_name(name)
+            r[real_name] = builtins[name]
+    return r
+            
+
+def builtin_command_names():
+    """Return list of builtin command names."""
+    return _builtin_commands().keys()
+    
+
+def plugin_command_names():
+    return plugin_cmds.keys()
 
 
 def _get_cmd_dict(plugins_override=True):
-    d = {}
-    for k, v in globals().iteritems():
-        if k.startswith("cmd_"):
-            d[_unsquish_command_name(k)] = v
-    # If we didn't load plugins, the plugin_cmds dict will be empty
+    """Return name->class mapping for all commands."""
+    d = _builtin_commands()
     if plugins_override:
         d.update(plugin_cmds)
-    else:
-        d2 = plugin_cmds.copy()
-        d2.update(d)
-        d = d2
     return d
 
     
@@ -177,1254 +119,155 @@
         yield k,v
 
 
-def get_cmd_class(cmd, plugins_override=True):
+def get_cmd_object(cmd_name, plugins_override=True):
     """Return the canonical name and command class for a command.
+
+    plugins_override
+        If true, plugin commands can override builtins.
     """
-    cmd = str(cmd)                      # not unicode
+    from bzrlib.externalcommand import ExternalCommand
+
+    cmd_name = str(cmd_name)            # not unicode
 
     # first look up this command under the specified name
     cmds = _get_cmd_dict(plugins_override=plugins_override)
     try:
-        return cmd, cmds[cmd]
+        return cmds[cmd_name]()
     except KeyError:
         pass
 
     # look for any command which claims this as an alias
-    for cmdname, cmdclass in cmds.iteritems():
-        if cmd in cmdclass.aliases:
-            return cmdname, cmdclass
-
-    cmdclass = ExternalCommand.find_command(cmd)
-    if cmdclass:
-        return cmd, cmdclass
-
-    raise BzrCommandError("unknown command %r" % cmd)
+    for real_cmd_name, cmd_class in cmds.iteritems():
+        if cmd_name in cmd_class.aliases:
+            return cmd_class()
+
+    cmd_obj = ExternalCommand.find_command(cmd_name)
+    if cmd_obj:
+        return cmd_obj
+
+    raise BzrCommandError("unknown command %r" % cmd_name)
 
 
 class Command(object):
     """Base class for commands.
 
+    Commands are the heart of the command-line bzr interface.
+
+    The command object mostly handles the mapping of command-line
+    parameters into one or more bzrlib operations, and of the results
+    into textual output.
+
+    Commands normally don't have any state.  All their arguments are
+    passed in to the run method.  (Subclasses may take a different
+    policy if the behaviour of the instance needs to depend on e.g. a
+    shell plugin and not just its Python class.)
+
     The docstring for an actual command should give a single-line
     summary, then a complete description of the command.  A grammar
     description will be inserted.
 
+    aliases
+        Other accepted names for this command.
+
     takes_args
         List of argument forms, marked with whether they are optional,
         repeated, etc.
 
+                Examples:
+
+                ['to_location', 'from_branch?', 'file*']
+
+                'to_location' is required
+                'from_branch' is optional
+                'file' can be specified 0 or more times
+
     takes_options
-        List of options that may be given for this command.
+        List of options that may be given for this command.  These can
+        be either strings, referring to globally-defined options,
+        or option objects.  Retrieve through options().
 
     hidden
-        If true, this command isn't advertised.
+        If true, this command isn't advertised.  This is typically
+        for commands intended for expert users.
     """
     aliases = []
-    
     takes_args = []
     takes_options = []
 
     hidden = False
     
-    def __init__(self, options, arguments):
-        """Construct and run the command.
-
-        Sets self.status to the return value of run()."""
-        assert isinstance(options, dict)
-        assert isinstance(arguments, dict)
-        cmdargs = options.copy()
-        cmdargs.update(arguments)
+    def __init__(self):
+        """Construct an instance of this command."""
         if self.__doc__ == Command.__doc__:
-            from warnings import warn
             warn("No help message set for %r" % self)
-        self.status = self.run(**cmdargs)
-        if self.status is None:
-            self.status = 0
-
+
+    def options(self):
+        """Return dict of valid options for this command.
+
+        Maps from long option name to option object."""
+        r = dict()
+        r['help'] = Option.OPTIONS['help']
+        for o in self.takes_options:
+            if not isinstance(o, Option):
+                o = Option.OPTIONS[o]
+            r[o.name] = o
+        return r
+
+    @deprecated_method(zero_eight)
+    def run_argv(self, argv):
+        """Parse command line and run.
+        
+        See run_argv_aliases for the 0.8 and beyond api.
+        """
+        return self.run_argv_aliases(argv)
+
+    def run_argv_aliases(self, argv, alias_argv=None):
+        """Parse the command line and run with extra aliases in alias_argv."""
+        args, opts = parse_args(self, argv, alias_argv)
+        if 'help' in opts:  # e.g. bzr add --help
+            from bzrlib.help import help_on_command
+            help_on_command(self.name())
+            return 0
+        # XXX: This should be handled by the parser
+        allowed_names = self.options().keys()
+        for oname in opts:
+            if oname not in allowed_names:
+                raise BzrCommandError("option '--%s' is not allowed for"
+                                      " command %r" % (oname, self.name()))
+        # mix arguments and options into one dictionary
+        cmdargs = _match_argform(self.name(), self.takes_args, args)
+        cmdopts = {}
+        for k, v in opts.items():
+            cmdopts[k.replace('-', '_')] = v
+
+        all_cmd_args = cmdargs.copy()
+        all_cmd_args.update(cmdopts)
+
+        return self.run(**all_cmd_args)
     
     def run(self):
-        """Override this in sub-classes.
+        """Actually run the command.
 
         This is invoked with the options and arguments bound to
         keyword parameters.
 
-        Return 0 or None if the command was successful, or a shell
-        error code if not.
+        Return 0 or None if the command was successful, or a non-zero
+        shell error code if not.  It's OK for this method to allow
+        an exception to raise up.
         """
-        return 0
-
-
-class ExternalCommand(Command):
-    """Class to wrap external commands.
-
-    We cheat a little here, when get_cmd_class() calls us we actually
-    give it back an object we construct that has the appropriate path,
-    help, options etc for the specified command.
-
-    When run_bzr() tries to instantiate that 'class' it gets caught by
-    the __call__ method, which we override to call the Command.__init__
-    method. That then calls our run method which is pretty straight
-    forward.
-
-    The only wrinkle is that we have to map bzr's dictionary of options
-    and arguments back into command line options and arguments for the
-    script.
-    """
-
-    def find_command(cls, cmd):
-        import os.path
-        bzrpath = os.environ.get('BZRPATH', '')
-
-        for dir in bzrpath.split(os.pathsep):
-            path = os.path.join(dir, cmd)
-            if os.path.isfile(path):
-                return ExternalCommand(path)
-
-        return None
-
-    find_command = classmethod(find_command)
-
-    def __init__(self, path):
-        self.path = path
-
-        pipe = os.popen('%s --bzr-usage' % path, 'r')
-        self.takes_options = pipe.readline().split()
-
-        for opt in self.takes_options:
-            if not opt in OPTIONS:
-                raise BzrError("Unknown option '%s' returned by external command %s"
-                               % (opt, path))
-
-        # TODO: Is there any way to check takes_args is valid here?
-        self.takes_args = pipe.readline().split()
-
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-usage'" % path)
-
-        pipe = os.popen('%s --bzr-help' % path, 'r')
-        self.__doc__ = pipe.read()
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-help'" % path)
-
-    def __call__(self, options, arguments):
-        Command.__init__(self, options, arguments)
-        return self
-
-    def run(self, **kargs):
-        opts = []
-        args = []
-
-        keys = kargs.keys()
-        keys.sort()
-        for name in keys:
-            optname = name.replace('_','-')
-            value = kargs[name]
-            if OPTIONS.has_key(optname):
-                # it's an option
-                opts.append('--%s' % optname)
-                if value is not None and value is not True:
-                    opts.append(str(value))
-            else:
-                # it's an arg, or arg list
-                if type(value) is not list:
-                    value = [value]
-                for v in value:
-                    if v is not None:
-                        args.append(str(v))
-
-        self.status = os.spawnv(os.P_WAIT, self.path, [self.path] + opts + args)
-        return self.status
-
-
-class cmd_status(Command):
-    """Display status summary.
-
-    This reports on versioned and unknown files, reporting them
-    grouped by state.  Possible states are:
-
-    added
-        Versioned in the working copy but not in the previous revision.
-
-    removed
-        Versioned in the previous revision but removed or deleted
-        in the working copy.
-
-    renamed
-        Path of this file changed from the previous revision;
-        the text may also have changed.  This includes files whose
-        parent directory was renamed.
-
-    modified
-        Text has changed since the previous revision.
-
-    unchanged
-        Nothing about this file has changed since the previous revision.
-        Only shown with --all.
-
-    unknown
-        Not versioned and not matching an ignore pattern.
-
-    To see ignored files use 'bzr ignored'.  For details in the
-    changes to file texts, use 'bzr diff'.
-
-    If no arguments are specified, the status of the entire working
-    directory is shown.  Otherwise, only the status of the specified
-    files or directories is reported.  If a directory is given, status
-    is reported for everything inside that directory.
-
-    If a revision is specified, the changes since that revision are shown.
-    """
-    takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
-    aliases = ['st', 'stat']
-    
-    def run(self, all=False, show_ids=False, file_list=None):
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
-                file_list = None
-        else:
-            b = find_branch('.')
-            
-        from bzrlib.status import show_status
-        show_status(b, show_unchanged=all, show_ids=show_ids,
-                    specific_files=file_list)
-
-
-class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
-
-    hidden = True
-    takes_args = ['revision_id']
-    
-    def run(self, revision_id):
-        from bzrlib.xml import pack_xml
-        pack_xml(find_branch('.').get_revision(revision_id), sys.stdout)
-
-
-class cmd_revno(Command):
-    """Show current revision number.
-
-    This is equal to the number of revisions on this branch."""
-    def run(self):
-        print find_branch('.').revno()
-
-class cmd_revision_info(Command):
-    """Show revision number and revision id for a given revision identifier.
-    """
-    hidden = True
-    takes_args = ['revision_info*']
-    takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
-
-        revs = []
-        if revision is not None:
-            revs.extend(revision)
-        if revision_info_list is not None:
-            revs.extend(revision_info_list)
-        if len(revs) == 0:
-            raise BzrCommandError('You must supply a revision identifier')
-
-        b = find_branch('.')
-
-        for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
-
-    
-class cmd_add(Command):
-    """Add specified files or directories.
-
-    In non-recursive mode, all the named items are added, regardless
-    of whether they were previously ignored.  A warning is given if
-    any of the named files are already versioned.
-
-    In recursive mode (the default), files are treated the same way
-    but the behaviour for directories is different.  Directories that
-    are already versioned do not give a warning.  All directories,
-    whether already versioned or not, are searched for files or
-    subdirectories that are neither versioned or ignored, and these
-    are added.  This search proceeds recursively into versioned
-    directories.  If no names are given '.' is assumed.
-
-    Therefore simply saying 'bzr add' will version all files that
-    are currently unknown.
-
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
-    """
-    takes_args = ['file*']
-    takes_options = ['verbose', 'no-recurse']
-    
-    def run(self, file_list, verbose=False, no_recurse=False):
-        from bzrlib.add import smart_add
-        smart_add(file_list, verbose, not no_recurse)
-
-
-
-class cmd_mkdir(Command):
-    """Create a new versioned directory.
-
-    This is equivalent to creating the directory and then adding it.
-    """
-    takes_args = ['dir+']
-
-    def run(self, dir_list):
-        b = None
-        
-        for d in dir_list:
-            os.mkdir(d)
-            if not b:
-                b = find_branch(d)
-            b.add([d], verbose=True)
-
-
-class cmd_relpath(Command):
-    """Show path of a file relative to root"""
-    takes_args = ['filename']
-    hidden = True
-    
-    def run(self, filename):
-        print find_branch(filename).relpath(filename)
-
-
-
-class cmd_inventory(Command):
-    """Show inventory of the current working copy or a revision."""
-    takes_options = ['revision', 'show-ids']
-    
-    def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
-            inv = b.read_working_inventory()
-        else:
-            if len(revision) > 1:
-                raise BzrCommandError('bzr inventory --revision takes'
-                    ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
-
-        for path, entry in inv.entries():
-            if show_ids:
-                print '%-50s %s' % (path, entry.file_id)
-            else:
-                print path
-
-
-class cmd_move(Command):
-    """Move files to a different directory.
-
-    examples:
-        bzr move *.txt doc
-
-    The destination must be a versioned directory in the same branch.
-    """
-    takes_args = ['source$', 'dest']
-    def run(self, source_list, dest):
-        b = find_branch('.')
-
-        # TODO: glob expansion on windows?
-        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
-
-
-class cmd_rename(Command):
-    """Change the name of an entry.
-
-    examples:
-      bzr rename frob.c frobber.c
-      bzr rename src/frob.c lib/frob.c
-
-    It is an error if the destination name exists.
-
-    See also the 'move' command, which moves files into a different
-    directory without changing their name.
-
-    TODO: Some way to rename multiple files without invoking bzr for each
-    one?"""
-    takes_args = ['from_name', 'to_name']
-    
-    def run(self, from_name, to_name):
-        b = find_branch('.')
-        b.rename_one(b.relpath(from_name), b.relpath(to_name))
-
-
-
-class cmd_mv(Command):
-    """Move or rename a file.
-
-    usage:
-        bzr mv OLDNAME NEWNAME
-        bzr mv SOURCE... DESTINATION
-
-    If the last argument is a versioned directory, all the other names
-    are moved into it.  Otherwise, there must be exactly two arguments
-    and the file is changed to a new name, which must not already exist.
-
-    Files cannot be moved between branches.
-    """
-    takes_args = ['names*']
-    def run(self, names_list):
-        if len(names_list) < 2:
-            raise BzrCommandError("missing file argument")
-        b = find_branch(names_list[0])
-
-        rel_names = [b.relpath(x) for x in names_list]
-        
-        if os.path.isdir(names_list[-1]):
-            # move into existing directory
-            b.move(rel_names[:-1], rel_names[-1])
-        else:
-            if len(names_list) != 2:
-                raise BzrCommandError('to mv multiple files the destination '
-                                      'must be a versioned directory')
-            b.move(rel_names[0], rel_names[1])
-            
-    
-
-
-class cmd_pull(Command):
-    """Pull any changes from another branch into the current one.
-
-    If the location is omitted, the last-used location will be used.
-    Both the revision history and the working directory will be
-    updated.
-
-    This command only works on branches that have not diverged.  Branches are
-    considered diverged if both branches have had commits without first
-    pulling from the other.
-
-    If branches have diverged, you can use 'bzr merge' to pull the text changes
-    from one into the other.
-    """
-    takes_args = ['location?']
-
-    def run(self, location=None):
-        from bzrlib.merge import merge
-        import tempfile
-        from shutil import rmtree
-        import errno
-        
-        br_to = find_branch('.')
-        stored_loc = None
-        try:
-            stored_loc = br_to.controlfile("x-pull", "rb").read().rstrip('\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        if location is None:
-            if stored_loc is None:
-                raise BzrCommandError("No pull location known or specified.")
-            else:
-                print "Using last location: %s" % stored_loc
-                location = stored_loc
-        cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
-        location = pull_loc(br_from)
-        old_revno = br_to.revno()
-        try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
-            location = pull_loc(br_from)
-            old_revno = br_to.revno()
-            try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-                
-            merge(('.', -1), ('.', old_revno), check_clean=False)
-            if location != stored_loc:
-                br_to.controlfile("x-pull", "wb").write(location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-
-class cmd_branch(Command):
-    """Create a new copy of a branch.
-
-    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
-    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
-
-    To retrieve the branch as of a particular revision, supply the --revision
-    parameter, as in "branch foo/bar -r 5".
-    """
-    takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision']
-    aliases = ['get', 'clone']
-
-    def run(self, from_location, to_location=None, revision=None):
-        import errno
-        from bzrlib.merge import merge
-        from bzrlib.branch import DivergedBranches, \
-             find_cached_branch, Branch
-        from shutil import rmtree
-        from meta_store import CachedStore
-        import tempfile
-        cache_root = tempfile.mkdtemp()
-
-        if revision is None:
-            revision = [None]
-        elif len(revision) > 1:
-            raise BzrCommandError('bzr branch --revision takes exactly 1 revision value')
-
-        try:
-            try:
-                br_from = find_cached_branch(from_location, cache_root)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Source location "%s" does not'
-                                          ' exist.' % to_location)
-                else:
-                    raise
-
-            if to_location is None:
-                to_location = os.path.basename(from_location.rstrip("/\\"))
-
-            try:
-                os.mkdir(to_location)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    raise BzrCommandError('Target directory "%s" already'
-                                          ' exists.' % to_location)
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Parent of "%s" does not exist.' %
-                                          to_location)
-                else:
-                    raise
-            br_to = Branch(to_location, init=True)
-
-            br_to.set_root_id(br_from.get_root_id())
-
-            if revision:
-                if revision[0] is None:
-                    revno = br_from.revno()
-                else:
-                    revno, rev_id = br_from.get_revision_info(revision[0])
-                try:
-                    br_to.update_revisions(br_from, stop_revision=revno)
-                except bzrlib.errors.NoSuchRevision:
-                    rmtree(to_location)
-                    msg = "The branch %s has no revision %d." % (from_location,
-                                                                 revno)
-                    raise BzrCommandError(msg)
-            
-            merge((to_location, -1), (to_location, 0), this_dir=to_location,
-                  check_clean=False, ignore_zero=True)
-            from_location = pull_loc(br_from)
-            br_to.controlfile("x-pull", "wb").write(from_location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-def pull_loc(branch):
-    # TODO: Should perhaps just make attribute be 'base' in
-    # RemoteBranch and Branch?
-    if hasattr(branch, "baseurl"):
-        return branch.baseurl
-    else:
-        return branch.base
-
-
-
-class cmd_renames(Command):
-    """Show list of renamed files.
-
-    TODO: Option to show renames between two historical versions.
-
-    TODO: Only show renames under dir, rather than in the whole branch.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        b = find_branch(dir)
-        old_inv = b.basis_tree().inventory
-        new_inv = b.read_working_inventory()
-
-        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
-        renames.sort()
-        for old_name, new_name in renames:
-            print "%s => %s" % (old_name, new_name)        
-
-
-class cmd_info(Command):
-    """Show statistical information about a branch."""
-    takes_args = ['branch?']
-    
-    def run(self, branch=None):
-        import info
-
-        b = find_branch(branch)
-        info.show_info(b)
-
-
-class cmd_remove(Command):
-    """Make a file unversioned.
-
-    This makes bzr stop tracking changes to a versioned file.  It does
-    not delete the working copy.
-    """
-    takes_args = ['file+']
-    takes_options = ['verbose']
-    
-    def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
-        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
-
-
-class cmd_file_id(Command):
-    """Print file_id of a particular file or directory.
-
-    The file_id is assigned when the file is first added and remains the
-    same through all revisions where the file exists, even when it is
-    moved or renamed.
-    """
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        i = b.inventory.path2id(b.relpath(filename))
-        if i == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        else:
-            print i
-
-
-class cmd_file_path(Command):
-    """Print path of file_ids to a file or directory.
-
-    This prints one line for each directory down to the target,
-    starting at the branch root."""
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.inventory
-        fid = inv.path2id(b.relpath(filename))
-        if fid == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        for fip in inv.get_idpath(fid):
-            print fip
-
-
-class cmd_revision_history(Command):
-    """Display list of revision ids on this branch."""
-    hidden = True
-    def run(self):
-        for patchid in find_branch('.').revision_history():
-            print patchid
-
-
-class cmd_directories(Command):
-    """Display list of versioned directories in this branch."""
-    def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
-            if name == '':
-                print '.'
-            else:
-                print name
-
-
-class cmd_init(Command):
-    """Make a directory into a versioned branch.
-
-    Use this to create an empty branch, or before importing an
-    existing project.
-
-    Recipe for importing a tree of files:
-        cd ~/project
-        bzr init
-        bzr add -v .
-        bzr status
-        bzr commit -m 'imported project'
-    """
-    def run(self):
-        from bzrlib.branch import Branch
-        Branch('.', init=True)
-
-
-class cmd_diff(Command):
-    """Show differences in working tree.
-    
-    If files are listed, only the changes in those files are listed.
-    Otherwise, all changes for the tree are listed.
-
-    TODO: Allow diff across branches.
-
-    TODO: Option to use external diff command; could be GNU diff, wdiff,
-          or a graphical diff.
-
-    TODO: Python difflib is not exactly the same as unidiff; should
-          either fix it up or prefer to use an external diff.
-
-    TODO: If a directory is given, diff everything under that.
-
-    TODO: Selected-file diff is inefficient and doesn't show you
-          deleted files.
-
-    TODO: This probably handles non-Unix newlines poorly.
-
-    examples:
-        bzr diff
-        bzr diff -r1
-        bzr diff -r1:2
-    """
-    
-    takes_args = ['file*']
-    takes_options = ['revision', 'diff-options']
-    aliases = ['di', 'dif']
-
-    def run(self, revision=None, file_list=None, diff_options=None):
-        from bzrlib.diff import show_diff
-
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
-        if revision is not None:
-            if len(revision) == 1:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options)
-            elif len(revision) == 2:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options,
-                          revision2=revision[1])
-            else:
-                raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
-        else:
-            show_diff(b, None, specific_files=file_list,
-                      external_diff_options=diff_options)
-
-        
-
-
-class cmd_deleted(Command):
-    """List files deleted in the working tree.
-
-    TODO: Show files deleted since a previous revision, or between two revisions.
-    """
-    def run(self, show_ids=False):
-        b = find_branch('.')
-        old = b.basis_tree()
-        new = b.working_tree()
-
-        ## TODO: Much more efficient way to do this: read in new
-        ## directories with readdir, rather than stating each one.  Same
-        ## level of effort but possibly much less IO.  (Or possibly not,
-        ## if the directories are very large...)
-
-        for path, ie in old.inventory.iter_entries():
-            if not new.has_id(ie.file_id):
-                if show_ids:
-                    print '%-50s %s' % (path, ie.file_id)
-                else:
-                    print path
-
-
-class cmd_modified(Command):
-    """List files modified in working tree."""
-    hidden = True
-    def run(self):
-        from bzrlib.delta import compare_trees
-
-        b = find_branch('.')
-        td = compare_trees(b.basis_tree(), b.working_tree())
-
-        for path, id, kind in td.modified:
-            print path
-
-
-
-class cmd_added(Command):
-    """List files added in working tree."""
-    hidden = True
-    def run(self):
-        b = find_branch('.')
-        wt = b.working_tree()
-        basis_inv = b.basis_tree().inventory
-        inv = wt.inventory
-        for file_id in inv:
-            if file_id in basis_inv:
-                continue
-            path = inv.id2path(file_id)
-            if not os.access(b.abspath(path), os.F_OK):
-                continue
-            print path
-                
-        
-
-class cmd_root(Command):
-    """Show the tree root directory.
-
-    The root is the nearest enclosing directory with a .bzr control
-    directory."""
-    takes_args = ['filename?']
-    def run(self, filename=None):
-        """Print the branch root."""
-        b = find_branch(filename)
-        print getattr(b, 'base', None) or getattr(b, 'baseurl')
-
-
-class cmd_log(Command):
-    """Show log of this branch.
-
-    To request a range of logs, you can use the command -r begin:end
-    -r revision requests a specific revision, -r :end or -r begin: are
-    also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
-
-    TODO: Make --revision support uuid: and hash: [future tag:] notation.
-  
-    """
-
-    takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
-                     'long', 'message', 'short',]
-    
-    def run(self, filename=None, timezone='original',
-            verbose=False,
-            show_ids=False,
-            forward=False,
-            revision=None,
-            message=None,
-            long=False,
-            short=False):
-        from bzrlib.branch import find_branch
-        from bzrlib.log import log_formatter, show_log
-        import codecs
-
-        direction = (forward and 'forward') or 'reverse'
-        
-        if filename:
-            b = find_branch(filename)
-            fp = b.relpath(filename)
-            if fp:
-                file_id = b.read_working_inventory().path2id(fp)
-            else:
-                file_id = None  # points to branch root
-        else:
-            b = find_branch('.')
-            file_id = None
-
-        if revision is None:
-            rev1 = None
-            rev2 = None
-        elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
-        elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
-        else:
-            raise BzrCommandError('bzr log --revision takes one or two values.')
-
-        if rev1 == 0:
-            rev1 = None
-        if rev2 == 0:
-            rev2 = None
-
-        mutter('encoding log as %r' % bzrlib.user_encoding)
-
-        # use 'replace' so that we don't abort if trying to write out
-        # in e.g. the default C locale.
-        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
-
-        if not short:
-            log_format = 'long'
-        else:
-            log_format = 'short'
-        lf = log_formatter(log_format,
-                           show_ids=show_ids,
-                           to_file=outf,
-                           show_timezone=timezone)
-
-        show_log(b,
-                 lf,
-                 file_id,
-                 verbose=verbose,
-                 direction=direction,
-                 start_revision=rev1,
-                 end_revision=rev2,
-                 search=message)
-
-
-
-class cmd_touching_revisions(Command):
-    """Return revision-ids which affected a particular file.
-
-    A more user-friendly interface is "bzr log FILE"."""
-    hidden = True
-    takes_args = ["filename"]
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.read_working_inventory()
-        file_id = inv.path2id(b.relpath(filename))
-        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
-            print "%6d %s" % (revno, what)
-
-
-class cmd_ls(Command):
-    """List files in a tree.
-
-    TODO: Take a revision or remote path and list that tree instead.
-    """
-    hidden = True
-    def run(self, revision=None, verbose=False):
-        b = find_branch('.')
-        if revision == None:
-            tree = b.working_tree()
-        else:
-            tree = b.revision_tree(b.lookup_revision(revision))
-
-        for fp, fc, kind, fid in tree.list_files():
-            if verbose:
-                if kind == 'directory':
-                    kindch = '/'
-                elif kind == 'file':
-                    kindch = ''
-                else:
-                    kindch = '???'
-
-                print '%-8s %s%s' % (fc, fp, kindch)
-            else:
-                print fp
-
-
-
-class cmd_unknowns(Command):
-    """List unknown files."""
-    def run(self):
-        from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
-            print quotefn(f)
-
-
-
-class cmd_ignore(Command):
-    """Ignore a command or pattern.
-
-    To remove patterns from the ignore list, edit the .bzrignore file.
-
-    If the pattern contains a slash, it is compared to the whole path
-    from the branch root.  Otherwise, it is comapred to only the last
-    component of the path.
-
-    Ignore patterns are case-insensitive on case-insensitive systems.
-
-    Note: wildcards must be quoted from the shell on Unix.
-
-    examples:
-        bzr ignore ./Makefile
-        bzr ignore '*.class'
-    """
-    takes_args = ['name_pattern']
-    
-    def run(self, name_pattern):
-        from bzrlib.atomicfile import AtomicFile
-        import os.path
-
-        b = find_branch('.')
-        ifn = b.abspath('.bzrignore')
-
-        if os.path.exists(ifn):
-            f = open(ifn, 'rt')
-            try:
-                igns = f.read().decode('utf-8')
-            finally:
-                f.close()
-        else:
-            igns = ''
-
-        # TODO: If the file already uses crlf-style termination, maybe
-        # we should use that for the newly added lines?
-
-        if igns and igns[-1] != '\n':
-            igns += '\n'
-        igns += name_pattern + '\n'
-
-        try:
-            f = AtomicFile(ifn, 'wt')
-            f.write(igns.encode('utf-8'))
-            f.commit()
-        finally:
-            f.close()
-
-        inv = b.working_tree().inventory
-        if inv.path2id('.bzrignore'):
-            mutter('.bzrignore is already versioned')
-        else:
-            mutter('need to make new .bzrignore file versioned')
-            b.add(['.bzrignore'])
-
-
-
-class cmd_ignored(Command):
-    """List ignored files and the patterns that matched them.
-
-    See also: bzr ignore"""
-    def run(self):
-        tree = find_branch('.').working_tree()
-        for path, file_class, kind, file_id in tree.list_files():
-            if file_class != 'I':
-                continue
-            ## XXX: Slightly inefficient since this was already calculated
-            pat = tree.is_ignored(path)
-            print '%-50s %s' % (path, pat)
-
-
-class cmd_lookup_revision(Command):
-    """Lookup the revision-id from a revision-number
-
-    example:
-        bzr lookup-revision 33
-    """
-    hidden = True
-    takes_args = ['revno']
-    
-    def run(self, revno):
-        try:
-            revno = int(revno)
-        except ValueError:
-            raise BzrCommandError("not a valid revision-number: %r" % revno)
-
-        print find_branch('.').lookup_revision(revno)
-
-
-class cmd_export(Command):
-    """Export past revision to destination directory.
-
-    If no revision is specified this exports the last committed revision.
-
-    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
-    given, try to find the format with the extension. If no extension
-    is found exports to a directory (equivalent to --format=dir).
-
-    Root may be the top directory for tar, tgz and tbz2 formats. If none
-    is given, the top directory will be the root name of the file."""
-    # TODO: list known exporters
-    takes_args = ['dest']
-    takes_options = ['revision', 'format', 'root']
-    def run(self, dest, revision=None, format=None, root=None):
-        import os.path
-        b = find_branch('.')
-        if revision is None:
-            rev_id = b.last_patch()
-        else:
-            if len(revision) != 1:
-                raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
-        t = b.revision_tree(rev_id)
-        root, ext = os.path.splitext(dest)
-        if not format:
-            if ext in (".tar",):
-                format = "tar"
-            elif ext in (".gz", ".tgz"):
-                format = "tgz"
-            elif ext in (".bz2", ".tbz2"):
-                format = "tbz2"
-            else:
-                format = "dir"
-        t.export(dest, format, root)
-
-
-class cmd_cat(Command):
-    """Write a file's text from a previous revision."""
-
-    takes_options = ['revision']
-    takes_args = ['filename']
-
-    def run(self, filename, revision=None):
-        if revision == None:
-            raise BzrCommandError("bzr cat requires a revision number")
-        elif len(revision) != 1:
-            raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
-
-
-class cmd_local_time_offset(Command):
-    """Show the offset in seconds from GMT to local time."""
-    hidden = True    
-    def run(self):
-        print bzrlib.osutils.local_time_offset()
-
-
-
-class cmd_commit(Command):
-    """Commit changes into a new revision.
-    
-    If no arguments are given, the entire tree is committed.
-
-    If selected files are specified, only changes to those files are
-    committed.  If a directory is specified then the directory and everything 
-    within it is committed.
-
-    A selected-file commit may fail in some cases where the committed
-    tree would be invalid, such as trying to commit a file in a
-    newly-added directory that is not itself committed.
-
-    TODO: Run hooks on tree to-be-committed, and after commit.
-
-    TODO: Strict commit that fails if there are unknown or deleted files.
-    """
-    takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
-    aliases = ['ci', 'checkin']
-
-    # TODO: Give better message for -s, --summary, used by tla people
-    
-    def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit
-        from bzrlib.osutils import get_text_message
-
-        ## Warning: shadows builtin file()
-        if not message and not file:
-            # FIXME: Ugly; change status code to send to a provided function?
-            
-            import cStringIO
-            stdout = sys.stdout
-            catcher = cStringIO.StringIO()
-            sys.stdout = catcher
-            cmd_status({"file_list":selected_list}, {})
-            info = catcher.getvalue()
-            sys.stdout = stdout
-            message = get_text_message(info)
-            
-            if message is None:
-                raise BzrCommandError("please specify a commit message",
-                                      ["use either --message or --file"])
-        elif message and file:
-            raise BzrCommandError("please specify either --message or --file")
-        
-        if file:
-            import codecs
-            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
-
-        b = find_branch('.')
-        if selected_list:
-            selected_list = [b.relpath(s) for s in selected_list]
-            
-        try:
-            b.commit(message, verbose=verbose,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
-        except PointlessCommit:
-            # FIXME: This should really happen before the file is read in;
-            # perhaps prepare the commit; get the message; then actually commit
-            raise BzrCommandError("no changes to commit",
-                                  ["use --unchanged to commit anyhow"])
-
-
-class cmd_check(Command):
-    """Validate consistency of branch history.
-
-    This command checks various invariants about the branch storage to
-    detect data corruption or bzr bugs.
-
-    If given the --update flag, it will update some optional fields
-    to help ensure data consistency.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.check import check
-        check(find_branch(dir))
-
-
-
-class cmd_scan_cache(Command):
-    hidden = True
-    def run(self):
-        from bzrlib.hashcache import HashCache
-        import os
-
-        c = HashCache('.')
-        c.read()
-        c.scan()
-            
-        print '%6d stats' % c.stat_count
-        print '%6d in hashcache' % len(c._cache)
-        print '%6d files removed from cache' % c.removed_count
-        print '%6d hashes updated' % c.update_count
-        print '%6d files changed too recently to cache' % c.danger_count
-
-        if c.needs_write:
-            c.write()
-            
-
-
-class cmd_upgrade(Command):
-    """Upgrade branch storage to current format.
-
-    This should normally be used only after the check command tells
-    you to run it.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
-
-
-
-class cmd_whoami(Command):
-    """Show bzr user id."""
-    takes_options = ['email']
-    
-    def run(self, email=False):
-        try:
-            b = bzrlib.branch.find_branch('.')
-        except:
-            b = None
-        
-        if email:
-            print bzrlib.osutils.user_email(b)
-        else:
-            print bzrlib.osutils.username(b)
-
-
-class cmd_selftest(Command):
-    """Run internal test suite"""
-    hidden = True
-    takes_options = ['verbose']
-    def run(self, verbose=False):
-        from bzrlib.selftest import selftest
-        return int(not selftest(verbose=verbose))
-
-
-class cmd_version(Command):
-    """Show version of bzr."""
-    def run(self):
-        show_version()
-
-def show_version():
-    print "bzr (bazaar-ng) %s" % bzrlib.__version__
-    # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
-    print bzrlib.__copyright__
-    print "http://bazaar-ng.org/"
-    print
-    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
-    print "you may use, modify and redistribute it under the terms of the GNU"
-    print "General Public License version 2 or later."
-
-
-class cmd_rocks(Command):
-    """Statement of optimism."""
-    hidden = True
-    def run(self):
-        print "it sure does!"
+        raise NotImplementedError('no implementation of command %r' 
+                                  % self.name())
+
+    def help(self):
+        """Return help message for this class."""
+        from inspect import getdoc
+        if self.__doc__ is Command.__doc__:
+            return None
+        return getdoc(self)
+
+    def name(self):
+        return _unsquish_command_name(self.__class__.__name__)
+
 
 def parse_spec(spec):
     """
@@ -1457,307 +300,119 @@
         parsed = [spec, None]
     return parsed
 
-
-
-class cmd_merge(Command):
-    """Perform a three-way merge.
-    
-    The branch is the branch you will merge from.  By default, it will merge
-    the latest revision.  If you specify a revision, that revision will be
-    merged.  If you specify two revisions, the first will be used as a BASE, 
-    and the second one as OTHER.  Revision numbers are always relative to the
-    specified branch.
-    
-    Examples:
-
-    To merge the latest revision from bzr.dev
-    bzr merge ../bzr.dev
-
-    To merge changes up to and including revision 82 from bzr.dev
-    bzr merge -r 82 ../bzr.dev
-
-    To merge the changes introduced by 82, without previous changes:
-    bzr merge -r 81..82 ../bzr.dev
-    
-    merge refuses to run if there are any uncommitted changes, unless
-    --force is given.
-    """
-    takes_args = ['branch?']
-    takes_options = ['revision', 'force', 'merge-type']
-
-    def run(self, branch='.', revision=None, force=False, 
-            merge_type=None):
-        from bzrlib.merge import merge
-        from bzrlib.merge_core import ApplyMerge3
-        if merge_type is None:
-            merge_type = ApplyMerge3
-
-        if revision is None or len(revision) < 1:
-            base = (None, None)
-            other = (branch, -1)
-        else:
-            if len(revision) == 1:
-                other = (branch, revision[0])
-                base = (None, None)
-            else:
-                assert len(revision) == 2
-                if None in revision:
-                    raise BzrCommandError(
-                        "Merge doesn't permit that revision specifier.")
-                base = (branch, revision[0])
-                other = (branch, revision[1])
-            
-        merge(other, base, check_clean=(not force), merge_type=merge_type)
-
-
-class cmd_revert(Command):
-    """Reverse all changes since the last commit.
-
-    Only versioned files are affected.  Specify filenames to revert only 
-    those files.  By default, any files that are changed will be backed up
-    first.  Backup files have a '~' appended to their name.
-    """
-    takes_options = ['revision', 'no-backup']
-    takes_args = ['file*']
-    aliases = ['merge-revert']
-
-    def run(self, revision=None, no_backup=False, file_list=None):
-        from bzrlib.merge import merge
-        if file_list is not None:
-            if len(file_list) == 0:
-                raise BzrCommandError("No files specified")
-        if revision is None:
-            revision = [-1]
-        elif len(revision) != 1:
-            raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
-              check_clean=False,
-              ignore_zero=True,
-              backup_files=not no_backup,
-              file_list=file_list)
-
-
-class cmd_assert_fail(Command):
-    """Test reporting of assertion failures"""
-    hidden = True
-    def run(self):
-        assert False, "always fails"
-
-
-class cmd_help(Command):
-    """Show help on a command or other topic.
-
-    For a list of all available commands, say 'bzr help commands'."""
-    takes_args = ['topic?']
-    aliases = ['?']
-    
-    def run(self, topic=None):
-        import help
-        help.help(topic)
-
-
-class cmd_shell_complete(Command):
-    """Show appropriate completions for context.
-
-    For a list of all available commands, say 'bzr shell-complete'."""
-    takes_args = ['context?']
-    aliases = ['s-c']
-    hidden = True
-    
-    def run(self, context=None):
-        import shellcomplete
-        shellcomplete.shellcomplete(context)
-
-
-class cmd_missing(Command):
-    """What is missing in this branch relative to other branch.
-    """
-    takes_args = ['remote?']
-    aliases = ['mis', 'miss']
-    # We don't have to add quiet to the list, because 
-    # unknown options are parsed as booleans
-    takes_options = ['verbose', 'quiet']
-
-    def run(self, remote=None, verbose=False, quiet=False):
-        from bzrlib.branch import find_branch, DivergedBranches
-        from bzrlib.errors import BzrCommandError
-        from bzrlib.missing import get_parent, show_missing
-
-        if verbose and quiet:
-            raise BzrCommandError('Cannot pass both quiet and verbose')
-
-        b = find_branch('.')
-        parent = get_parent(b)
-        if remote is None:
-            if parent is None:
-                raise BzrCommandError("No missing location known or specified.")
-            else:
-                if not quiet:
-                    print "Using last location: %s" % parent
-                remote = parent
-        elif parent is None:
-            # We only update x-pull if it did not exist, missing should not change the parent
-            b.controlfile('x-pull', 'wb').write(remote + '\n')
-        br_remote = find_branch(remote)
-
-        return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
-
-
-class cmd_plugins(Command):
-    """List plugins"""
-    hidden = True
-    def run(self):
-        import bzrlib.plugin
-        from inspect import getdoc
-        from pprint import pprint
-        for plugin in bzrlib.plugin.all_plugins:
-            print plugin.__path__[0]
-            d = getdoc(plugin)
-            if d:
-                print '\t', d.split('\n')[0]
-
-        #pprint(bzrlib.plugin.all_plugins)
-
-
-
-# list of all available options; the rhs can be either None for an
-# option that takes no argument, or a constructor function that checks
-# the type.
-OPTIONS = {
-    'all':                    None,
-    'diff-options':           str,
-    'help':                   None,
-    'file':                   unicode,
-    'force':                  None,
-    'format':                 unicode,
-    'forward':                None,
-    'message':                unicode,
-    'no-recurse':             None,
-    'profile':                None,
-    'revision':               _parse_revision_str,
-    'short':                  None,
-    'show-ids':               None,
-    'timezone':               str,
-    'verbose':                None,
-    'version':                None,
-    'email':                  None,
-    'unchanged':              None,
-    'update':                 None,
-    'long':                   None,
-    'root':                   str,
-    'no-backup':              None,
-    'merge-type':             get_merge_type,
-    }
-
-SHORT_OPTIONS = {
-    'F':                      'file', 
-    'h':                      'help',
-    'm':                      'message',
-    'r':                      'revision',
-    'v':                      'verbose',
-    'l':                      'long',
-}
-
-
-def parse_args(argv):
+def parse_args(command, argv, alias_argv=None):
     """Parse command line.
     
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
-
-    >>> parse_args('--help'.split())
-    ([], {'help': True})
-    >>> parse_args('--version'.split())
-    ([], {'version': True})
-    >>> parse_args('status --all'.split())
-    (['status'], {'all': True})
-    >>> parse_args('commit --message=biter'.split())
-    (['commit'], {'message': u'biter'})
-    >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [500]})
-    >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [500, 600]})
-    >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [500, 600]})
-    >>> parse_args('log -rv500..600'.split()) #the r takes an argument
-    (['log'], {'revision': ['v500', 600]})
     """
+    # TODO: chop up this beast; make it a method of the Command
     args = []
     opts = {}
-
-    # TODO: Maybe handle '--' to end options?
-
-    while argv:
-        a = argv.pop(0)
-        if a[0] == '-':
-            # option names must not be unicode
-            a = str(a)
-            optarg = None
-            if a[1] == '-':
-                mutter("  got option %r" % a)
-                if '=' in a:
-                    optname, optarg = a[2:].split('=', 1)
-                else:
-                    optname = a[2:]
-                if optname not in OPTIONS:
-                    raise BzrError('unknown long option %r' % a)
-            else:
-                shortopt = a[1:]
-                if shortopt in SHORT_OPTIONS:
-                    # Multi-character options must have a space to delimit
-                    # their value
-                    optname = SHORT_OPTIONS[shortopt]
-                else:
-                    # Single character short options, can be chained,
-                    # and have their value appended to their name
-                    shortopt = a[1:2]
-                    if shortopt not in SHORT_OPTIONS:
-                        # We didn't find the multi-character name, and we
-                        # didn't find the single char name
-                        raise BzrError('unknown short option %r' % a)
-                    optname = SHORT_OPTIONS[shortopt]
-
-                    if a[2:]:
-                        # There are extra things on this option
-                        # see if it is the value, or if it is another
-                        # short option
-                        optargfn = OPTIONS[optname]
-                        if optargfn is None:
-                            # This option does not take an argument, so the
-                            # next entry is another short option, pack it back
-                            # into the list
-                            argv.insert(0, '-' + a[2:])
+    alias_opts = {}
+
+    cmd_options = command.options()
+    argsover = False
+    proc_aliasarg = True # Are we processing alias_argv now?
+    for proc_argv in alias_argv, argv:
+        while proc_argv:
+            a = proc_argv.pop(0)
+            if argsover:
+                args.append(a)
+                continue
+            elif a == '--':
+                # We've received a standalone -- No more flags
+                argsover = True
+                continue
+            if a[0] == '-':
+                # option names must not be unicode
+                a = str(a)
+                optarg = None
+                if a[1] == '-':
+                    mutter("  got option %r", a)
+                    if '=' in a:
+                        optname, optarg = a[2:].split('=', 1)
+                    else:
+                        optname = a[2:]
+                    if optname not in cmd_options:
+                        raise BzrOptionError('unknown long option %r for'
+                                             ' command %s' % 
+                                             (a, command.name()))
+                else:
+                    shortopt = a[1:]
+                    if shortopt in Option.SHORT_OPTIONS:
+                        # Multi-character options must have a space to delimit
+                        # their value
+                        # ^^^ what does this mean? mbp 20051014
+                        optname = Option.SHORT_OPTIONS[shortopt].name
+                    else:
+                        # Single character short options, can be chained,
+                        # and have their value appended to their name
+                        shortopt = a[1:2]
+                        if shortopt not in Option.SHORT_OPTIONS:
+                            # We didn't find the multi-character name, and we
+                            # didn't find the single char name
+                            raise BzrError('unknown short option %r' % a)
+                        optname = Option.SHORT_OPTIONS[shortopt].name
+
+                        if a[2:]:
+                            # There are extra things on this option
+                            # see if it is the value, or if it is another
+                            # short option
+                            optargfn = Option.OPTIONS[optname].type
+                            if optargfn is None:
+                                # This option does not take an argument, so the
+                                # next entry is another short option, pack it
+                                # back into the list
+                                proc_argv.insert(0, '-' + a[2:])
+                            else:
+                                # This option takes an argument, so pack it
+                                # into the array
+                                optarg = a[2:]
+                
+                    if optname not in cmd_options:
+                        raise BzrOptionError('unknown short option %r for'
+                                             ' command %s' % 
+                                             (shortopt, command.name()))
+                if optname in opts:
+                    # XXX: Do we ever want to support this, e.g. for -r?
+                    if proc_aliasarg:
+                        raise BzrError('repeated option %r' % a)
+                    elif optname in alias_opts:
+                        # Replace what's in the alias with what's in the real
+                        # argument
+                        del alias_opts[optname]
+                        del opts[optname]
+                        proc_argv.insert(0, a)
+                        continue
+                    else:
+                        raise BzrError('repeated option %r' % a)
+                    
+                option_obj = cmd_options[optname]
+                optargfn = option_obj.type
+                if optargfn:
+                    if optarg == None:
+                        if not proc_argv:
+                            raise BzrError('option %r needs an argument' % a)
                         else:
-                            # This option takes an argument, so pack it
-                            # into the array
-                            optarg = a[2:]
-            
-            if optname in opts:
-                # XXX: Do we ever want to support this, e.g. for -r?
-                raise BzrError('repeated option %r' % a)
-                
-            optargfn = OPTIONS[optname]
-            if optargfn:
-                if optarg == None:
-                    if not argv:
-                        raise BzrError('option %r needs an argument' % a)
-                    else:
-                        optarg = argv.pop(0)
-                opts[optname] = optargfn(optarg)
+                            optarg = proc_argv.pop(0)
+                    opts[optname] = optargfn(optarg)
+                    if proc_aliasarg:
+                        alias_opts[optname] = optargfn(optarg)
+                else:
+                    if optarg != None:
+                        raise BzrError('option %r takes no argument' % optname)
+                    opts[optname] = True
+                    if proc_aliasarg:
+                        alias_opts[optname] = True
             else:
-                if optarg != None:
-                    raise BzrError('option %r takes no argument' % optname)
-                opts[optname] = True
-        else:
-            args.append(a)
-
+                args.append(a)
+        proc_aliasarg = False # Done with alias argv
     return args, opts
 
 
-
-
 def _match_argform(cmd, takes_args, args):
     argdict = {}
 
@@ -1803,6 +458,52 @@
 
 
 
+def apply_profiled(the_callable, *args, **kwargs):
+    import hotshot
+    import tempfile
+    import hotshot.stats
+    pffileno, pfname = tempfile.mkstemp()
+    try:
+        prof = hotshot.Profile(pfname)
+        try:
+            ret = prof.runcall(the_callable, *args, **kwargs) or 0
+        finally:
+            prof.close()
+        stats = hotshot.stats.load(pfname)
+        stats.strip_dirs()
+        stats.sort_stats('cum')   # 'time'
+        ## XXX: Might like to write to stderr or the trace file instead but
+        ## print_stats seems hardcoded to stdout
+        stats.print_stats(20)
+        return ret
+    finally:
+        os.close(pffileno)
+        os.remove(pfname)
+
+
+def apply_lsprofiled(filename, the_callable, *args, **kwargs):
+    from bzrlib.lsprof import profile
+    import cPickle
+    ret, stats = profile(the_callable, *args, **kwargs)
+    stats.sort()
+    if filename is None:
+        stats.pprint()
+    else:
+        stats.freeze()
+        cPickle.dump(stats, open(filename, 'w'), 2)
+        print 'Profile data written to %r.' % filename
+    return ret
+
+
+def get_alias(cmd):
+    """Return an expanded alias, or None if no alias exists"""
+    import bzrlib.config
+    alias = bzrlib.config.GlobalConfig().get_alias(cmd)
+    if (alias):
+        return alias.split(' ')
+    return None
+
+
 def run_bzr(argv):
     """Execute a command.
 
@@ -1820,160 +521,152 @@
     --no-plugins
         Do not load plugin modules at all
 
+    --no-aliases
+        Do not allow aliases
+
     --builtin
         Only use builtin commands.  (Plugins are still allowed to change
         other behaviour.)
 
     --profile
-        Run under the Python profiler.
+        Run under the Python hotshot profiler.
+
+    --lsprof
+        Run under the Python lsprof profiler.
     """
-    
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
-    opt_profile = opt_no_plugins = opt_builtin = False
+    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin =  \
+                opt_no_aliases = False
+    opt_lsprof_file = None
 
     # --no-plugins is handled specially at a very early stage. We need
     # to load plugins before doing other command parsing so that they
     # can override commands, but this needs to happen first.
 
-    for a in argv[:]:
+    argv_copy = []
+    i = 0
+    while i < len(argv):
+        a = argv[i]
         if a == '--profile':
             opt_profile = True
+        elif a == '--lsprof':
+            opt_lsprof = True
+        elif a == '--lsprof-file':
+            opt_lsprof_file = argv[i + 1]
+            i += 1
         elif a == '--no-plugins':
             opt_no_plugins = True
+        elif a == '--no-aliases':
+            opt_no_aliases = True
         elif a == '--builtin':
             opt_builtin = True
+        elif a in ('--quiet', '-q'):
+            be_quiet()
         else:
-            break
-        argv.remove(a)
-
+            argv_copy.append(a)
+        i += 1
+
+    argv = argv_copy
+    if (not argv):
+        from bzrlib.builtins import cmd_help
+        cmd_help().run_argv_aliases([])
+        return 0
+
+    if argv[0] == '--version':
+        from bzrlib.builtins import show_version
+        show_version()
+        return 0
+        
     if not opt_no_plugins:
         from bzrlib.plugin import load_plugins
         load_plugins()
-
-    args, opts = parse_args(argv)
-
-    if 'help' in opts:
-        from bzrlib.help import help
-        if args:
-            help(args[0])
+    else:
+        from bzrlib.plugin import disable_plugins
+        disable_plugins()
+
+    alias_argv = None
+
+    if not opt_no_aliases:
+        alias_argv = get_alias(argv[0])
+        if alias_argv:
+            alias_argv = [a.decode(bzrlib.user_encoding) for a in alias_argv]
+            argv[0] = alias_argv.pop(0)
+
+    cmd = str(argv.pop(0))
+
+    cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
+    if not getattr(cmd_obj.run_argv, 'is_deprecated', False):
+        run = cmd_obj.run_argv
+        run_argv = [argv]
+    else:
+        run = cmd_obj.run_argv_aliases
+        run_argv = [argv, alias_argv]
+
+    try:
+        if opt_lsprof:
+            ret = apply_lsprofiled(opt_lsprof_file, run, *run_argv)
+        elif opt_profile:
+            ret = apply_profiled(run, *run_argv)
         else:
-            help()
-        return 0            
-        
-    if 'version' in opts:
-        show_version()
-        return 0
-    
-    if not args:
-        print >>sys.stderr, "please try 'bzr help' for help"
-        return 1
-    
-    cmd = str(args.pop(0))
-
-    canonical_cmd, cmd_class = \
-                   get_cmd_class(cmd, plugins_override=not opt_builtin)
-
-    # check options are reasonable
-    allowed = cmd_class.takes_options
-    for oname in opts:
-        if oname not in allowed:
-            raise BzrCommandError("option '--%s' is not allowed for command %r"
-                                  % (oname, cmd))
-
-    # mix arguments and options into one dictionary
-    cmdargs = _match_argform(cmd, cmd_class.takes_args, args)
-    cmdopts = {}
-    for k, v in opts.items():
-        cmdopts[k.replace('-', '_')] = v
-
-    if opt_profile:
-        import hotshot, tempfile
-        pffileno, pfname = tempfile.mkstemp()
+            ret = run(*run_argv)
+        return ret or 0
+    finally:
+        # reset, in case we may do other commands later within the same process
+        be_quiet(False)
+
+def display_command(func):
+    """Decorator that suppresses pipe/interrupt errors."""
+    def ignore_pipe(*args, **kwargs):
         try:
-            prof = hotshot.Profile(pfname)
-            ret = prof.runcall(cmd_class, cmdopts, cmdargs) or 0
-            prof.close()
-
-            import hotshot.stats
-            stats = hotshot.stats.load(pfname)
-            #stats.strip_dirs()
-            stats.sort_stats('time')
-            ## XXX: Might like to write to stderr or the trace file instead but
-            ## print_stats seems hardcoded to stdout
-            stats.print_stats(20)
-            
-            return ret.status
-
-        finally:
-            os.close(pffileno)
-            os.remove(pfname)
-    else:
-        return cmd_class(cmdopts, cmdargs).status 
-
-
-def _report_exception(summary, quiet=False):
-    import traceback
-    
-    log_error('bzr: ' + summary)
-    bzrlib.trace.log_exception()
-
-    if os.environ.get('BZR_DEBUG'):
-        traceback.print_exc()
-
-    if not quiet:
-        sys.stderr.write('\n')
-        tb = sys.exc_info()[2]
-        exinfo = traceback.extract_tb(tb)
-        if exinfo:
-            sys.stderr.write('  at %s:%d in %s()\n' % exinfo[-1][:3])
-        sys.stderr.write('  see ~/.bzr.log for debug information\n')
-
+            result = func(*args, **kwargs)
+            sys.stdout.flush()
+            return result
+        except IOError, e:
+            if not hasattr(e, 'errno'):
+                raise
+            if e.errno != errno.EPIPE:
+                raise
+            pass
+        except KeyboardInterrupt:
+            pass
+    return ignore_pipe
 
 
 def main(argv):
-    
-    bzrlib.trace.open_tracefile(argv)
-
+    import bzrlib.ui
+    from bzrlib.ui.text import TextUIFactory
+    ## bzrlib.trace.enable_default_logging()
+    bzrlib.trace.log_startup(argv)
+    bzrlib.ui.ui_factory = TextUIFactory()
+    ret = run_bzr_catch_errors(argv[1:])
+    mutter("return code %d", ret)
+    return ret
+
+
+def run_bzr_catch_errors(argv):
     try:
         try:
-            try:
-                return run_bzr(argv[1:])
-            finally:
-                # do this here inside the exception wrappers to catch EPIPE
-                sys.stdout.flush()
-        except BzrError, e:
-            quiet = isinstance(e, (BzrCommandError))
-            _report_exception('error: ' + str(e), quiet=quiet)
-            if len(e.args) > 1:
-                for h in e.args[1]:
-                    # some explanation or hints
-                    log_error('  ' + h)
-            return 1
-        except AssertionError, e:
-            msg = 'assertion failed'
-            if str(e):
-                msg += ': ' + str(e)
-            _report_exception(msg)
-            return 2
-        except KeyboardInterrupt, e:
-            _report_exception('interrupted', quiet=True)
-            return 2
-        except Exception, e:
-            import errno
-            quiet = False
-            if (isinstance(e, IOError) 
-                and hasattr(e, 'errno')
-                and e.errno == errno.EPIPE):
-                quiet = True
-                msg = 'broken pipe'
-            else:
-                msg = str(e).rstrip('\n')
-            _report_exception(msg, quiet)
-            return 2
-    finally:
-        bzrlib.trace.close_trace()
-
+            return run_bzr(argv)
+        finally:
+            # do this here inside the exception wrappers to catch EPIPE
+            sys.stdout.flush()
+    except Exception, e:
+        # used to handle AssertionError and KeyboardInterrupt
+        # specially here, but hopefully they're handled ok by the logger now
+        import errno
+        if (isinstance(e, IOError) 
+            and hasattr(e, 'errno')
+            and e.errno == errno.EPIPE):
+            bzrlib.trace.note('broken pipe')
+            return 3
+        else:
+            bzrlib.trace.log_exception()
+            if os.environ.get('BZR_PDB'):
+                print '**** entering debugger'
+                import pdb
+                pdb.post_mortem(sys.exc_traceback)
+            return 3
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-08-18 04:25:42 +0000
+++ bzrlib/commit.py	2006-04-09 16:32:50 +0000
@@ -1,311 +1,573 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-# FIXME: "bzr commit doc/format" commits doc/format.txt!
-
-def commit(branch, message,
-           timestamp=None,
-           timezone=None,
-           committer=None,
-           verbose=True,
-           specific_files=None,
-           rev_id=None,
-           allow_pointless=True):
-    """Commit working copy as a new revision.
-
-    The basic approach is to add all the file texts into the
-    store, then the inventory, then make a new revision pointing
-    to that inventory and store that.
-
-    This is not quite safe if the working copy changes during the
-    commit; for the moment that is simply not allowed.  A better
-    approach is to make a temporary copy of the files before
-    computing their hashes, and then add those hashes in turn to
-    the inventory.  This should mean at least that there are no
-    broken hash pointers.  There is no way we can get a snapshot
-    of the whole directory at an instant.  This would also have to
-    be robust against files disappearing, moving, etc.  So the
-    whole thing is a bit hard.
-
-    This raises PointlessCommit if there are no changes, no new merges,
-    and allow_pointless  is false.
-
-    timestamp -- if not None, seconds-since-epoch for a
-         postdated/predated commit.
-
-    specific_files
-        If true, commit only those files.
-
-    rev_id
-        If set, use this as the new revision id.
-        Useful for test or import commands that need to tightly
-        control what revisions are assigned.  If you duplicate
-        a revision id that exists elsewhere it is your own fault.
-        If null (default), a time/random revision id is generated.
-    """
-
-    import time, tempfile
-
-    from bzrlib.osutils import local_time_offset, username
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError, PointlessCommit
-    from bzrlib.revision import Revision, RevisionReference
-    from bzrlib.trace import mutter, note
-    from bzrlib.xml import pack_xml
-
-    branch.lock_write()
-
-    try:
-        # First walk over the working inventory; and both update that
-        # and also build a new revision inventory.  The revision
-        # inventory needs to hold the text-id, sha1 and size of the
-        # actual file versions committed in the revision.  (These are
-        # not present in the working inventory.)  We also need to
-        # detect missing/deleted files, and remove them from the
-        # working inventory.
-
-        work_tree = branch.working_tree()
-        work_inv = work_tree.inventory
-        basis = branch.basis_tree()
-        basis_inv = basis.inventory
-
-        if verbose:
-            note('looking for changes...')
-
-        pending_merges = branch.pending_merges()
-
-        missing_ids, new_inv, any_changes = \
-                     _gather_commit(branch,
-                                    work_tree,
-                                    work_inv,
-                                    basis_inv,
-                                    specific_files,
-                                    verbose)
-
-        if not (any_changes or allow_pointless or pending_merges):
-            raise PointlessCommit()
-
-        for file_id in missing_ids:
-            # Any files that have been deleted are now removed from the
-            # working inventory.  Files that were not selected for commit
-            # are left as they were in the working inventory and ommitted
-            # from the revision inventory.
-
-            # have to do this later so we don't mess up the iterator.
-            # since parents may be removed before their children we
-            # have to test.
-
-            # FIXME: There's probably a better way to do this; perhaps
-            # the workingtree should know how to filter itbranch.
-            if work_inv.has_id(file_id):
-                del work_inv[file_id]
-
-        if rev_id is None:
-            rev_id = _gen_revision_id(branch, time.time())
-        inv_id = rev_id
-
-        inv_tmp = tempfile.TemporaryFile()
-        pack_xml(new_inv, inv_tmp)
-        inv_tmp.seek(0)
-        branch.inventory_store.add(inv_tmp, inv_id)
-        mutter('new inventory_id is {%s}' % inv_id)
-
-        # We could also just sha hash the inv_tmp file
-        # however, in the case that branch.inventory_store.add()
-        # ever actually does anything special
-        inv_sha1 = branch.get_inventory_sha1(inv_id)
-
-        branch._write_inventory(work_inv)
-
-        if timestamp == None:
-            timestamp = time.time()
-
-        if committer == None:
-            committer = username(branch)
-
-        if timezone == None:
-            timezone = local_time_offset()
-
-        mutter("building commit log message")
-        rev = Revision(timestamp=timestamp,
-                       timezone=timezone,
-                       committer=committer,
-                       message = message,
-                       inventory_id=inv_id,
-                       inventory_sha1=inv_sha1,
-                       revision_id=rev_id)
-
-        rev.parents = []
-        precursor_id = branch.last_patch()
+# XXX: Can we do any better about making interrupted commits change
+# nothing?  Perhaps the best approach is to integrate commit of
+# AtomicFiles with releasing the lock on the Branch.
+
+# TODO: Separate 'prepare' phase where we find a list of potentially
+# committed files.  We then can then pause the commit to prompt for a
+# commit message, knowing the summary will be the same as what's
+# actually used for the commit.  (But perhaps simpler to simply get
+# the tree status, then use that for a selective commit?)
+
+# The newly committed revision is going to have a shape corresponding
+# to that of the working inventory.  Files that are not in the
+# working tree and that were in the predecessor are reported as
+# removed --- this can include files that were either removed from the
+# inventory or deleted in the working tree.  If they were only
+# deleted from disk, they are removed from the working inventory.
+
+# We then consider the remaining entries, which will be in the new
+# version.  Directory entries are simply copied across.  File entries
+# must be checked to see if a new version of the file should be
+# recorded.  For each parent revision inventory, we check to see what
+# version of the file was present.  If the file was present in at
+# least one tree, and if it was the same version in all the trees,
+# then we can just refer to that version.  Otherwise, a new version
+# representing the merger of the file versions must be added.
+
+# TODO: Update hashcache before and after - or does the WorkingTree
+# look after that?
+
+# TODO: Rather than mashing together the ancestry and storing it back,
+# perhaps the weave should have single method which does it all in one
+# go, avoiding a lot of redundant work.
+
+# TODO: Perhaps give a warning if one of the revisions marked as
+# merged is already in the ancestry, and then don't record it as a
+# distinct parent.
+
+# TODO: If the file is newly merged but unchanged from the version it
+# merges from, then it should still be reported as newly added
+# relative to the basis revision.
+
+# TODO: Do checks that the tree can be committed *before* running the 
+# editor; this should include checks for a pointless commit and for 
+# unknown or missing files.
+
+# TODO: If commit fails, leave the message in a file somewhere.
+
+
+import os
+import re
+import sys
+import time
+import pdb
+
+from binascii import hexlify
+from cStringIO import StringIO
+
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.osutils import (local_time_offset,
+                            rand_bytes, compact_date,
+                            kind_marker, is_inside_any, quotefn,
+                            sha_file, isdir, isfile,
+                            split_lines)
+import bzrlib.config
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, PointlessCommit,
+                           HistoryMissing,
+                           ConflictsInTree,
+                           StrictCommitFailed
+                           )
+from bzrlib.revision import Revision
+from bzrlib.testament import Testament
+from bzrlib.trace import mutter, note, warning
+from bzrlib.xml5 import serializer_v5
+from bzrlib.inventory import Inventory, ROOT_ID
+from bzrlib.symbol_versioning import *
+from bzrlib.workingtree import WorkingTree
+
+
+@deprecated_function(zero_seven)
+def commit(*args, **kwargs):
+    """Commit a new revision to a branch.
+
+    Function-style interface for convenience of old callers.
+
+    New code should use the Commit class instead.
+    """
+    ## XXX: Remove this in favor of Branch.commit?
+    Commit().commit(*args, **kwargs)
+
+
+class NullCommitReporter(object):
+    """I report on progress of a commit."""
+
+    def snapshot_change(self, change, path):
+        pass
+
+    def completed(self, revno, rev_id):
+        pass
+
+    def deleted(self, file_id):
+        pass
+
+    def escaped(self, escape_count, message):
+        pass
+
+    def missing(self, path):
+        pass
+
+
+class ReportCommitToLog(NullCommitReporter):
+
+    def snapshot_change(self, change, path):
+        if change == 'unchanged':
+            return
+        note("%s %s", change, path)
+
+    def completed(self, revno, rev_id):
+        note('Committed revision %d.', revno)
+    
+    def deleted(self, file_id):
+        note('deleted %s', file_id)
+
+    def escaped(self, escape_count, message):
+        note("replaced %d control characters in message", escape_count)
+
+    def missing(self, path):
+        note('missing %s', path)
+
+
+class Commit(object):
+    """Task of committing a new revision.
+
+    This is a MethodObject: it accumulates state as the commit is
+    prepared, and then it is discarded.  It doesn't represent
+    historical revisions, just the act of recording a new one.
+
+            missing_ids
+            Modified to hold a list of files that have been deleted from
+            the working directory; these should be removed from the
+            working inventory.
+    """
+    def __init__(self,
+                 reporter=None,
+                 config=None):
+        if reporter is not None:
+            self.reporter = reporter
+        else:
+            self.reporter = NullCommitReporter()
+        if config is not None:
+            self.config = config
+        else:
+            self.config = None
+        
+    def commit(self,
+               branch=DEPRECATED_PARAMETER, message=None,
+               timestamp=None,
+               timezone=None,
+               committer=None,
+               specific_files=None,
+               rev_id=None,
+               allow_pointless=True,
+               strict=False,
+               verbose=False,
+               revprops=None,
+               working_tree=None,
+               local=False,
+               reporter=None,
+               config=None):
+        """Commit working copy as a new revision.
+
+        branch -- the deprecated branch to commit to. New callers should pass in 
+                  working_tree instead
+
+        message -- the commit message, a mandatory parameter
+
+        timestamp -- if not None, seconds-since-epoch for a
+             postdated/predated commit.
+
+        specific_files -- If true, commit only those files.
+
+        rev_id -- If set, use this as the new revision id.
+            Useful for test or import commands that need to tightly
+            control what revisions are assigned.  If you duplicate
+            a revision id that exists elsewhere it is your own fault.
+            If null (default), a time/random revision id is generated.
+
+        allow_pointless -- If true (default), commit even if nothing
+            has changed and no merges are recorded.
+
+        strict -- If true, don't allow a commit if the working tree
+            contains unknown files.
+
+        revprops -- Properties for new revision
+        :param local: Perform a local only commit.
+        """
+        mutter('preparing to commit')
+
+        if deprecated_passed(branch):
+            warn("Commit.commit (branch, ...): The branch parameter is "
+                 "deprecated as of bzr 0.8. Please use working_tree= instead.",
+                 DeprecationWarning, stacklevel=2)
+            self.branch = branch
+            self.work_tree = self.branch.bzrdir.open_workingtree()
+        elif working_tree is None:
+            raise BzrError("One of branch and working_tree must be passed into commit().")
+        else:
+            self.work_tree = working_tree
+            self.branch = self.work_tree.branch
+        if message is None:
+            raise BzrError("The message keyword parameter is required for commit().")
+
+        self.weave_store = self.branch.repository.weave_store
+        self.bound_branch = None
+        self.local = local
+        self.master_branch = None
+        self.master_locked = False
+        self.rev_id = rev_id
+        self.specific_files = specific_files
+        self.allow_pointless = allow_pointless
+        self.revprops = {}
+        if revprops is not None:
+            self.revprops.update(revprops)
+
+        if reporter is None and self.reporter is None:
+            self.reporter = NullCommitReporter()
+        elif reporter is not None:
+            self.reporter = reporter
+
+        self.work_tree.lock_write()
+        try:
+            # setup the bound branch variables as needed.
+            self._check_bound_branch()
+
+            # check for out of date working trees
+            # if we are bound, then self.branch is the master branch and this
+            # test is thus all we need.
+            if self.work_tree.last_revision() != self.master_branch.last_revision():
+                raise errors.OutOfDateTree(self.work_tree)
+    
+            if strict:
+                # raise an exception as soon as we find a single unknown.
+                for unknown in self.work_tree.unknowns():
+                    raise StrictCommitFailed()
+    
+            if timestamp is None:
+                self.timestamp = time.time()
+            else:
+                self.timestamp = long(timestamp)
+                
+            if self.config is None:
+                self.config = bzrlib.config.BranchConfig(self.branch)
+    
+            if rev_id is None:
+                self.rev_id = _gen_revision_id(self.config, self.timestamp)
+            else:
+                self.rev_id = rev_id
+    
+            if committer is None:
+                self.committer = self.config.username()
+            else:
+                assert isinstance(committer, basestring), type(committer)
+                self.committer = committer
+    
+            if timezone is None:
+                self.timezone = local_time_offset()
+            else:
+                self.timezone = int(timezone)
+    
+            if isinstance(message, str):
+                message = message.decode(bzrlib.user_encoding)
+            assert isinstance(message, unicode), type(message)
+            self.message = message
+            self._escape_commit_message()
+
+            self.work_inv = self.work_tree.inventory
+            self.basis_tree = self.work_tree.basis_tree()
+            self.basis_inv = self.basis_tree.inventory
+
+            self._gather_parents()
+            if len(self.parents) > 1 and self.specific_files:
+                raise NotImplementedError('selected-file commit of merges is not supported yet')
+            self._check_parents_present()
+            
+            self._remove_deleted()
+            self._populate_new_inv()
+            self._store_snapshot()
+            self._report_deletes()
+
+            if not (self.allow_pointless
+                    or len(self.parents) > 1
+                    or self.new_inv != self.basis_inv):
+                raise PointlessCommit()
+
+            if len(self.work_tree.conflicts())>0:
+                raise ConflictsInTree
+
+            self.inv_sha1 = self.branch.repository.add_inventory(
+                self.rev_id,
+                self.new_inv,
+                self.present_parents
+                )
+            self._make_revision()
+            # revision data is in the local branch now.
+            
+            # upload revision data to the master.
+            # this will propogate merged revisions too if needed.
+            if self.bound_branch:
+                self.master_branch.repository.fetch(self.branch.repository,
+                                                    revision_id=self.rev_id)
+                # now the master has the revision data
+                # 'commit' to the master first so a timeout here causes the local
+                # branch to be out of date
+                self.master_branch.append_revision(self.rev_id)
+
+            # and now do the commit locally.
+            self.branch.append_revision(self.rev_id)
+
+            self.work_tree.set_pending_merges([])
+            self.work_tree.set_last_revision(self.rev_id)
+            # now the work tree is up to date with the branch
+            
+            self.reporter.completed(self.branch.revno(), self.rev_id)
+            if self.config.post_commit() is not None:
+                hooks = self.config.post_commit().split(' ')
+                # this would be nicer with twisted.python.reflect.namedAny
+                for hook in hooks:
+                    result = eval(hook + '(branch, rev_id)',
+                                  {'branch':self.branch,
+                                   'bzrlib':bzrlib,
+                                   'rev_id':self.rev_id})
+        finally:
+            self._cleanup_bound_branch()
+            self.work_tree.unlock()
+
+    def _check_bound_branch(self):
+        """Check to see if the local branch is bound.
+
+        If it is bound, then most of the commit will actually be
+        done using the remote branch as the target branch.
+        Only at the end will the local branch be updated.
+        """
+        if self.local and not self.branch.get_bound_location():
+            raise errors.LocalRequiresBoundBranch()
+
+        if not self.local:
+            self.master_branch = self.branch.get_master_branch()
+
+        if not self.master_branch:
+            # make this branch the reference branch for out of date checks.
+            self.master_branch = self.branch
+            return
+
+        # If the master branch is bound, we must fail
+        master_bound_location = self.master_branch.get_bound_location()
+        if master_bound_location:
+            raise errors.CommitToDoubleBoundBranch(self.branch,
+                    self.master_branch, master_bound_location)
+
+        # TODO: jam 20051230 We could automatically push local
+        #       commits to the remote branch if they would fit.
+        #       But for now, just require remote to be identical
+        #       to local.
+        
+        # Make sure the local branch is identical to the master
+        master_rh = self.master_branch.revision_history()
+        local_rh = self.branch.revision_history()
+        if local_rh != master_rh:
+            raise errors.BoundBranchOutOfDate(self.branch,
+                    self.master_branch)
+
+        # Now things are ready to change the master branch
+        # so grab the lock
+        self.bound_branch = self.branch
+        self.master_branch.lock_write()
+        self.master_locked = True
+####        
+####        # Check to see if we have any pending merges. If we do
+####        # those need to be pushed into the master branch
+####        pending_merges = self.work_tree.pending_merges()
+####        if pending_merges:
+####            for revision_id in pending_merges:
+####                self.master_branch.repository.fetch(self.bound_branch.repository,
+####                                                    revision_id=revision_id)
+
+    def _cleanup_bound_branch(self):
+        """Executed at the end of a try/finally to cleanup a bound branch.
+
+        If the branch wasn't bound, this is a no-op.
+        If it was, it resents self.branch to the local branch, instead
+        of being the master.
+        """
+        if not self.bound_branch:
+            return
+        if self.master_locked:
+            self.master_branch.unlock()
+
+    def _escape_commit_message(self):
+        """Replace xml-incompatible control characters."""
+        # Python strings can include characters that can't be
+        # represented in well-formed XML; escape characters that
+        # aren't listed in the XML specification
+        # (http://www.w3.org/TR/REC-xml/#NT-Char).
+        self.message, escape_count = re.subn(
+            u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
+            lambda match: match.group(0).encode('unicode_escape'),
+            self.message)
+        if escape_count:
+            self.reporter.escaped(escape_count, self.message)
+
+    def _gather_parents(self):
+        """Record the parents of a merge for merge detection."""
+        pending_merges = self.work_tree.pending_merges()
+        self.parents = []
+        self.parent_invs = []
+        self.present_parents = []
+        precursor_id = self.branch.last_revision()
         if precursor_id:
-            precursor_sha1 = branch.get_revision_sha1(precursor_id)
-            rev.parents.append(RevisionReference(precursor_id, precursor_sha1))
-        for merge_rev in pending_merges:
-            rev.parents.append(RevisionReference(merge_rev))            
-
-        rev_tmp = tempfile.TemporaryFile()
-        pack_xml(rev, rev_tmp)
-        rev_tmp.seek(0)
-        branch.revision_store.add(rev_tmp, rev_id)
-        mutter("new revision_id is {%s}" % rev_id)
-
-        ## XXX: Everything up to here can simply be orphaned if we abort
-        ## the commit; it will leave junk files behind but that doesn't
-        ## matter.
-
-        ## TODO: Read back the just-generated changeset, and make sure it
-        ## applies and recreates the right state.
-
-        ## TODO: Also calculate and store the inventory SHA1
-        mutter("committing patch r%d" % (branch.revno() + 1))
-
-        branch.append_revision(rev_id)
-
-        branch.set_pending_merges([])
-
-        if verbose:
-            note("commited r%d" % branch.revno())
-    finally:
-        branch.unlock()
-
-
-
-def _gen_revision_id(branch, when):
+            self.parents.append(precursor_id)
+        self.parents += pending_merges
+        for revision in self.parents:
+            if self.branch.repository.has_revision(revision):
+                inventory = self.branch.repository.get_inventory(revision)
+                self.parent_invs.append(inventory)
+                self.present_parents.append(revision)
+
+    def _check_parents_present(self):
+        for parent_id in self.parents:
+            mutter('commit parent revision {%s}', parent_id)
+            if not self.branch.repository.has_revision(parent_id):
+                if parent_id == self.branch.last_revision():
+                    warning("parent is missing %r", parent_id)
+                    raise HistoryMissing(self.branch, 'revision', parent_id)
+                else:
+                    mutter("commit will ghost revision %r", parent_id)
+            
+    def _make_revision(self):
+        """Record a new revision object for this commit."""
+        rev = Revision(timestamp=self.timestamp,
+                       timezone=self.timezone,
+                       committer=self.committer,
+                       message=self.message,
+                       inventory_sha1=self.inv_sha1,
+                       revision_id=self.rev_id,
+                       properties=self.revprops)
+        rev.parent_ids = self.parents
+        self.branch.repository.add_revision(self.rev_id, rev, self.new_inv, self.config)
+
+    def _remove_deleted(self):
+        """Remove deleted files from the working inventories.
+
+        This is done prior to taking the working inventory as the
+        basis for the new committed inventory.
+
+        This returns true if any files
+        *that existed in the basis inventory* were deleted.
+        Files that were added and deleted
+        in the working copy don't matter.
+        """
+        specific = self.specific_files
+        deleted_ids = []
+        for path, ie in self.work_inv.iter_entries():
+            if specific and not is_inside_any(specific, path):
+                continue
+            if not self.work_tree.has_filename(path):
+                self.reporter.missing(path)
+                deleted_ids.append((path, ie.file_id))
+        if deleted_ids:
+            deleted_ids.sort(reverse=True)
+            for path, file_id in deleted_ids:
+                del self.work_inv[file_id]
+            self.work_tree._write_inventory(self.work_inv)
+
+    def _store_snapshot(self):
+        """Pass over inventory and record a snapshot.
+
+        Entries get a new revision when they are modified in 
+        any way, which includes a merge with a new set of
+        parents that have the same entry. 
+        """
+        # XXX: Need to think more here about when the user has
+        # made a specific decision on a particular value -- c.f.
+        # mark-merge.  
+        for path, ie in self.new_inv.iter_entries():
+            previous_entries = ie.find_previous_heads(
+                self.parent_invs,
+                self.weave_store,
+                self.branch.repository.get_transaction())
+            if ie.revision is None:
+                change = ie.snapshot(self.rev_id, path, previous_entries,
+                                     self.work_tree, self.weave_store,
+                                     self.branch.get_transaction())
+            else:
+                change = "unchanged"
+            self.reporter.snapshot_change(change, path)
+
+    def _populate_new_inv(self):
+        """Build revision inventory.
+
+        This creates a new empty inventory. Depending on
+        which files are selected for commit, and what is present in the
+        current tree, the new inventory is populated. inventory entries 
+        which are candidates for modification have their revision set to
+        None; inventory entries that are carried over untouched have their
+        revision set to their prior value.
+        """
+        mutter("Selecting files for commit with filter %s", self.specific_files)
+        self.new_inv = Inventory(revision_id=self.rev_id)
+        for path, new_ie in self.work_inv.iter_entries():
+            file_id = new_ie.file_id
+            mutter('check %s {%s}', path, new_ie.file_id)
+            if self.specific_files:
+                if not is_inside_any(self.specific_files, path):
+                    mutter('%s not selected for commit', path)
+                    self._carry_entry(file_id)
+                    continue
+                else:
+                    # this is selected, ensure its parents are too.
+                    parent_id = new_ie.parent_id
+                    while parent_id != ROOT_ID:
+                        if not self.new_inv.has_id(parent_id):
+                            ie = self._select_entry(self.work_inv[parent_id])
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+
+                        ie = self.new_inv[parent_id]
+                        if ie.revision is not None:
+                            ie.revision = None
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+                        parent_id = ie.parent_id
+            mutter('%s selected for commit', path)
+            self._select_entry(new_ie)
+
+    def _select_entry(self, new_ie):
+        """Make new_ie be considered for committing."""
+        ie = new_ie.copy()
+        ie.revision = None
+        self.new_inv.add(ie)
+        return ie
+
+    def _carry_entry(self, file_id):
+        """Carry the file unchanged from the basis revision."""
+        if self.basis_inv.has_id(file_id):
+            self.new_inv.add(self.basis_inv[file_id].copy())
+
+    def _report_deletes(self):
+        for file_id in self.basis_inv:
+            if file_id not in self.new_inv:
+                self.reporter.deleted(self.basis_inv.id2path(file_id))
+
+def _gen_revision_id(config, when):
     """Return new revision-id."""
-    from binascii import hexlify
-    from bzrlib.osutils import rand_bytes, compact_date, user_email
-
-    s = '%s-%s-' % (user_email(branch), compact_date(when))
+    s = '%s-%s-' % (config.user_email(), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s
-
-
-def _gather_commit(branch, work_tree, work_inv, basis_inv, specific_files,
-                   verbose):
-    """Build inventory preparatory to commit.
-
-    Returns missing_ids, new_inv, any_changes.
-
-    This adds any changed files into the text store, and sets their
-    test-id, sha and size in the returned inventory appropriately.
-
-    missing_ids
-        Modified to hold a list of files that have been deleted from
-        the working directory; these should be removed from the
-        working inventory.
-    """
-    from bzrlib.inventory import Inventory
-    from bzrlib.osutils import isdir, isfile, sha_string, quotefn, \
-         local_time_offset, username, kind_marker, is_inside_any
-    
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError
-    from bzrlib.revision import Revision
-    from bzrlib.trace import mutter, note
-
-    any_changes = False
-    inv = Inventory(work_inv.root.file_id)
-    missing_ids = []
-    
-    for path, entry in work_inv.iter_entries():
-        ## TODO: Check that the file kind has not changed from the previous
-        ## revision of this file (if any).
-
-        p = branch.abspath(path)
-        file_id = entry.file_id
-        mutter('commit prep file %s, id %r ' % (p, file_id))
-
-        if specific_files and not is_inside_any(specific_files, path):
-            mutter('  skipping file excluded from commit')
-            if basis_inv.has_id(file_id):
-                # carry over with previous state
-                inv.add(basis_inv[file_id].copy())
-            else:
-                # omit this from committed inventory
-                pass
-            continue
-
-        if not work_tree.has_id(file_id):
-            if verbose:
-                print('deleted %s%s' % (path, kind_marker(entry.kind)))
-                any_changes = True
-            mutter("    file is missing, removing from inventory")
-            missing_ids.append(file_id)
-            continue
-
-        # this is present in the new inventory; may be new, modified or
-        # unchanged.
-        old_ie = basis_inv.has_id(file_id) and basis_inv[file_id]
-        
-        entry = entry.copy()
-        inv.add(entry)
-
-        if old_ie:
-            old_kind = old_ie.kind
-            if old_kind != entry.kind:
-                raise BzrError("entry %r changed kind from %r to %r"
-                        % (file_id, old_kind, entry.kind))
-
-        if entry.kind == 'directory':
-            if not isdir(p):
-                raise BzrError("%s is entered as directory but not a directory"
-                               % quotefn(p))
-        elif entry.kind == 'file':
-            if not isfile(p):
-                raise BzrError("%s is entered as file but is not a file" % quotefn(p))
-
-            new_sha1 = work_tree.get_file_sha1(file_id)
-
-            if (old_ie
-                and old_ie.text_sha1 == new_sha1):
-                ## assert content == basis.get_file(file_id).read()
-                entry.text_id = old_ie.text_id
-                entry.text_sha1 = new_sha1
-                entry.text_size = old_ie.text_size
-                mutter('    unchanged from previous text_id {%s}' %
-                       entry.text_id)
-            else:
-                content = file(p, 'rb').read()
-
-                # calculate the sha again, just in case the file contents
-                # changed since we updated the cache
-                entry.text_sha1 = sha_string(content)
-                entry.text_size = len(content)
-
-                entry.text_id = gen_file_id(entry.name)
-                branch.text_store.add(content, entry.text_id)
-                mutter('    stored with text_id {%s}' % entry.text_id)
-
-        if verbose:
-            marked = path + kind_marker(entry.kind)
-            if not old_ie:
-                print 'added', marked
-                any_changes = True
-            elif old_ie == entry:
-                pass                    # unchanged
-            elif (old_ie.name == entry.name
-                  and old_ie.parent_id == entry.parent_id):
-                print 'modified', marked
-                any_changes = True
-            else:
-                print 'renamed', marked
-                any_changes = True
-                        
-    return missing_ids, inv, any_changes
-
-

=== added file 'bzrlib/config.py'
--- bzrlib/config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/config.py	2006-03-31 00:43:21 +0000
@@ -0,0 +1,617 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Configuration that affects the behaviour of Bazaar.
+
+Currently this configuration resides in ~/.bazaar/bazaar.conf
+and ~/.bazaar/branches.conf, which is written to by bzr.
+
+In bazaar.conf the following options may be set:
+[DEFAULT]
+editor=name-of-program
+email=Your Name <your@email.address>
+check_signatures=require|ignore|check-available(default)
+create_signatures=always|never|when-required(default)
+gpg_signing_command=name-of-program
+log_format=name-of-format
+
+in branches.conf, you specify the url of a branch and options for it.
+Wildcards may be used - * and ? as normal in shell completion. Options
+set in both bazaar.conf and branches.conf are overriden by the branches.conf
+setting.
+[/home/robertc/source]
+recurse=False|True(default)
+email= as above
+check_signatures= as abive 
+create_signatures= as above.
+
+explanation of options
+----------------------
+editor - this option sets the pop up editor to use during commits.
+email - this option sets the user id bzr will use when committing.
+check_signatures - this option controls whether bzr will require good gpg
+                   signatures, ignore them, or check them if they are 
+                   present.
+create_signatures - this option controls whether bzr will always create 
+                    gpg signatures, never create them, or create them if the
+                    branch is configured to require them.
+                    NB: This option is planned, but not implemented yet.
+log_format - This options set the default log format.  Options are long, 
+             short, line, or a plugin can register new formats
+
+In bazaar.conf you can also define aliases in the ALIASES sections, example
+
+[ALIASES]
+lastlog=log --line -r-10..-1
+ll=log --line -r-10..-1
+h=help
+up=pull
+"""
+
+
+import errno
+import os
+import sys
+from fnmatch import fnmatch
+import re
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
+import bzrlib.util.configobj.configobj as configobj
+from StringIO import StringIO
+
+CHECK_IF_POSSIBLE=0
+CHECK_ALWAYS=1
+CHECK_NEVER=2
+
+
+class ConfigObj(configobj.ConfigObj):
+
+    def get_bool(self, section, key):
+        return self[section].as_bool(key)
+
+    def get_value(self, section, name):
+        # Try [] for the old DEFAULT section.
+        if section == "DEFAULT":
+            try:
+                return self[name]
+            except KeyError:
+                pass
+        return self[section][name]
+
+
+class Config(object):
+    """A configuration policy - what username, editor, gpg needs etc."""
+
+    def get_editor(self):
+        """Get the users pop up editor."""
+        raise NotImplementedError
+
+    def _get_signature_checking(self):
+        """Template method to override signature checking policy."""
+
+    def _get_user_option(self, option_name):
+        """Template method to provide a user option."""
+        return None
+
+    def get_user_option(self, option_name):
+        """Get a generic option - no special process, no default."""
+        return self._get_user_option(option_name)
+
+    def gpg_signing_command(self):
+        """What program should be used to sign signatures?"""
+        result = self._gpg_signing_command()
+        if result is None:
+            result = "gpg"
+        return result
+
+    def _gpg_signing_command(self):
+        """See gpg_signing_command()."""
+        return None
+
+    def log_format(self):
+        """What log format should be used"""
+        result = self._log_format()
+        if result is None:
+            result = "long"
+        return result
+
+    def _log_format(self):
+        """See log_format()."""
+        return None
+
+    def __init__(self):
+        super(Config, self).__init__()
+
+    def post_commit(self):
+        """An ordered list of python functions to call.
+
+        Each function takes branch, rev_id as parameters.
+        """
+        return self._post_commit()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return None
+
+    def user_email(self):
+        """Return just the email component of a username."""
+        return extract_email_address(self.username())
+
+    def username(self):
+        """Return email-style username.
+    
+        Something similar to 'Martin Pool <mbp@sourcefrog.net>'
+        
+        $BZREMAIL can be set to override this, then
+        the concrete policy type is checked, and finally
+        $EMAIL is examined.
+        If none is found, a reasonable default is (hopefully)
+        created.
+    
+        TODO: Check it's reasonably well-formed.
+        """
+        v = os.environ.get('BZREMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+    
+        v = self._get_user_id()
+        if v:
+            return v
+        
+        v = os.environ.get('EMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+
+        name, email = _auto_user_id()
+        if name:
+            return '%s <%s>' % (name, email)
+        else:
+            return email
+
+    def signature_checking(self):
+        """What is the current policy for signature checking?."""
+        policy = self._get_signature_checking()
+        if policy is not None:
+            return policy
+        return CHECK_IF_POSSIBLE
+
+    def signature_needed(self):
+        """Is a signature needed when committing ?."""
+        policy = self._get_signature_checking()
+        if policy == CHECK_ALWAYS:
+            return True
+        return False
+
+    def get_alias(self, value):
+        return self._get_alias(value)
+
+    def _get_alias(self, value):
+        pass
+
+
+class IniBasedConfig(Config):
+    """A configuration policy that draws from ini files."""
+
+    def _get_parser(self, file=None):
+        if self._parser is not None:
+            return self._parser
+        if file is None:
+            input = self._get_filename()
+        else:
+            input = file
+        try:
+            self._parser = ConfigObj(input, encoding='utf-8')
+        except configobj.ConfigObjError, e:
+            raise errors.ParseConfigError(e.errors, e.config.filename)
+        return self._parser
+
+    def _get_section(self):
+        """Override this to define the section used by the config."""
+        return "DEFAULT"
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        policy = self._get_user_option('check_signatures')
+        if policy:
+            return self._string_to_signature_policy(policy)
+
+    def _get_user_id(self):
+        """Get the user id from the 'email' key in the current section."""
+        return self._get_user_option('email')
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        try:
+            return self._get_parser().get_value(self._get_section(),
+                                                option_name)
+        except KeyError:
+            pass
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_user_option('gpg_signing_command')
+
+    def _log_format(self):
+        """See Config.log_format."""
+        return self._get_user_option('log_format')
+
+    def __init__(self, get_filename):
+        super(IniBasedConfig, self).__init__()
+        self._get_filename = get_filename
+        self._parser = None
+        
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_user_option('post_commit')
+
+    def _string_to_signature_policy(self, signature_string):
+        """Convert a string to a signing policy."""
+        if signature_string.lower() == 'check-available':
+            return CHECK_IF_POSSIBLE
+        if signature_string.lower() == 'ignore':
+            return CHECK_NEVER
+        if signature_string.lower() == 'require':
+            return CHECK_ALWAYS
+        raise errors.BzrError("Invalid signatures policy '%s'"
+                              % signature_string)
+
+    def _get_alias(self, value):
+        try:
+            return self._get_parser().get_value("ALIASES", 
+                                                value)
+        except KeyError:
+            pass
+
+
+class GlobalConfig(IniBasedConfig):
+    """The configuration that should be used for a specific location."""
+
+    def get_editor(self):
+        return self._get_user_option('editor')
+
+    def __init__(self):
+        super(GlobalConfig, self).__init__(config_filename)
+
+
+class LocationConfig(IniBasedConfig):
+    """A configuration object that gives the policy for a location."""
+
+    def __init__(self, location):
+        super(LocationConfig, self).__init__(branches_config_filename)
+        self._global_config = None
+        self.location = location
+
+    def _get_global_config(self):
+        if self._global_config is None:
+            self._global_config = GlobalConfig()
+        return self._global_config
+
+    def _get_section(self):
+        """Get the section we should look in for config items.
+
+        Returns None if none exists. 
+        TODO: perhaps return a NullSection that thunks through to the 
+              global config.
+        """
+        sections = self._get_parser()
+        location_names = self.location.split('/')
+        if self.location.endswith('/'):
+            del location_names[-1]
+        matches=[]
+        for section in sections:
+            section_names = section.split('/')
+            if section.endswith('/'):
+                del section_names[-1]
+            names = zip(location_names, section_names)
+            matched = True
+            for name in names:
+                if not fnmatch(name[0], name[1]):
+                    matched = False
+                    break
+            if not matched:
+                continue
+            # so, for the common prefix they matched.
+            # if section is longer, no match.
+            if len(section_names) > len(location_names):
+                continue
+            # if path is longer, and recurse is not true, no match
+            if len(section_names) < len(location_names):
+                try:
+                    if not self._get_parser()[section].as_bool('recurse'):
+                        continue
+                except KeyError:
+                    pass
+            matches.append((len(section_names), section))
+        if not len(matches):
+            return None
+        matches.sort(reverse=True)
+        return matches[0][1]
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        command = super(LocationConfig, self)._gpg_signing_command()
+        if command is not None:
+            return command
+        return self._get_global_config()._gpg_signing_command()
+
+    def _log_format(self):
+        """See Config.log_format."""
+        command = super(LocationConfig, self)._log_format()
+        if command is not None:
+            return command
+        return self._get_global_config()._log_format()
+
+    def _get_user_id(self):
+        user_id = super(LocationConfig, self)._get_user_id()
+        if user_id is not None:
+            return user_id
+        return self._get_global_config()._get_user_id()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        option_value = super(LocationConfig, 
+                             self)._get_user_option(option_name)
+        if option_value is not None:
+            return option_value
+        return self._get_global_config()._get_user_option(option_name)
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        check = super(LocationConfig, self)._get_signature_checking()
+        if check is not None:
+            return check
+        return self._get_global_config()._get_signature_checking()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        hook = self._get_user_option('post_commit')
+        if hook is not None:
+            return hook
+        return self._get_global_config()._post_commit()
+
+    def set_user_option(self, option, value):
+        """Save option and its value in the configuration."""
+        # FIXME: RBC 20051029 This should refresh the parser and also take a
+        # file lock on branches.conf.
+        conf_dir = os.path.dirname(self._get_filename())
+        ensure_config_dir_exists(conf_dir)
+        location = self.location
+        if location.endswith('/'):
+            location = location[:-1]
+        if (not location in self._get_parser() and
+            not location + '/' in self._get_parser()):
+            self._get_parser()[location]={}
+        elif location + '/' in self._get_parser():
+            location = location + '/'
+        self._get_parser()[location][option]=value
+        self._get_parser().write(file(self._get_filename(), 'wb'))
+
+
+class BranchConfig(Config):
+    """A configuration object giving the policy for a branch."""
+
+    def _get_location_config(self):
+        if self._location_config is None:
+            self._location_config = LocationConfig(self.branch.base)
+        return self._location_config
+
+    def _get_user_id(self):
+        """Return the full user id for the branch.
+    
+        e.g. "John Hacker <jhacker@foo.org>"
+        This is looked up in the email controlfile for the branch.
+        """
+        try:
+            return (self.branch.control_files.get_utf8("email") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except errors.NoSuchFile, e:
+            pass
+        
+        return self._get_location_config()._get_user_id()
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        return self._get_location_config()._get_signature_checking()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        return self._get_location_config()._get_user_option(option_name)
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_location_config()._gpg_signing_command()
+        
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self.branch = branch
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_location_config()._post_commit()
+
+    def _log_format(self):
+        """See Config.log_format."""
+        return self._get_location_config()._log_format()
+
+
+def ensure_config_dir_exists(path=None):
+    """Make sure a configuration directory exists.
+    This makes sure that the directory exists.
+    On windows, since configuration directories are 2 levels deep,
+    it makes sure both the directory and the parent directory exists.
+    """
+    if path is None:
+        path = config_dir()
+    if not os.path.isdir(path):
+        if sys.platform == 'win32':
+            parent_dir = os.path.dirname(path)
+            if not os.path.isdir(parent_dir):
+                mutter('creating config parent directory: %r', parent_dir)
+            os.mkdir(parent_dir)
+        mutter('creating config directory: %r', path)
+        os.mkdir(path)
+
+
+def config_dir():
+    """Return per-user configuration directory.
+
+    By default this is ~/.bazaar/
+    
+    TODO: Global option --config-dir to override this.
+    """
+    base = os.environ.get('BZR_HOME', None)
+    if sys.platform == 'win32':
+        if base is None:
+            base = os.environ.get('APPDATA', None)
+        if base is None:
+            base = os.environ.get('HOME', None)
+        if base is None:
+            raise BzrError('You must have one of BZR_HOME, APPDATA, or HOME set')
+        return pathjoin(base, 'bazaar', '2.0')
+    else:
+        # cygwin, linux, and darwin all have a $HOME directory
+        if base is None:
+            base = os.path.expanduser("~")
+        return pathjoin(base, ".bazaar")
+
+
+def config_filename():
+    """Return per-user configuration ini file filename."""
+    return pathjoin(config_dir(), 'bazaar.conf')
+
+
+def branches_config_filename():
+    """Return per-user configuration ini file filename."""
+    return pathjoin(config_dir(), 'branches.conf')
+
+
+def _auto_user_id():
+    """Calculate automatic user identification.
+
+    Returns (realname, email).
+
+    Only used when none is set in the environment or the id file.
+
+    This previously used the FQDN as the default domain, but that can
+    be very slow on machines where DNS is broken.  So now we simply
+    use the hostname.
+    """
+    import socket
+
+    # XXX: Any good way to get real user name on win32?
+
+    try:
+        import pwd
+        uid = os.getuid()
+        w = pwd.getpwuid(uid)
+
+        try:
+            gecos = w.pw_gecos.decode(bzrlib.user_encoding)
+            username = w.pw_name.decode(bzrlib.user_encoding)
+        except UnicodeDecodeError:
+            # We're using pwd, therefore we're on Unix, so /etc/passwd is ok.
+            raise errors.BzrError("Can't decode username in " \
+                    "/etc/passwd as %s." % bzrlib.user_encoding)
+
+        comma = gecos.find(',')
+        if comma == -1:
+            realname = gecos
+        else:
+            realname = gecos[:comma]
+        if not realname:
+            realname = username
+
+    except ImportError:
+        import getpass
+        try:
+            realname = username = getpass.getuser().decode(bzrlib.user_encoding)
+        except UnicodeDecodeError:
+            raise errors.BzrError("Can't decode username as %s." % \
+                    bzrlib.user_encoding)
+
+    return realname, (username + '@' + socket.gethostname())
+
+
+def extract_email_address(e):
+    """Return just the address part of an email string.
+    
+    That is just the user@domain part, nothing else. 
+    This part is required to contain only ascii characters.
+    If it can't be extracted, raises an error.
+    
+    >>> extract_email_address('Jane Tester <jane@test.com>')
+    "jane@test.com"
+    """
+    m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+    if not m:
+        raise errors.BzrError("%r doesn't seem to contain "
+                              "a reasonable email address" % e)
+    return m.group(0)
+
+class TreeConfig(object):
+    """Branch configuration data associated with its contents, not location"""
+    def __init__(self, branch):
+        self.branch = branch
+
+    def _get_config(self):
+        try:
+            obj = ConfigObj(self.branch.control_files.get('branch.conf'), 
+                            encoding='utf-8')
+        except errors.NoSuchFile:
+            obj = ConfigObj(encoding='utf=8')
+        return obj
+
+    def get_option(self, name, section=None, default=None):
+        self.branch.lock_read()
+        try:
+            obj = self._get_config()
+            try:
+                if section is not None:
+                    obj[section]
+                result = obj[name]
+            except KeyError:
+                result = default
+        finally:
+            self.branch.unlock()
+        return result
+
+    def set_option(self, value, name, section=None):
+        """Set a per-branch configuration option"""
+        self.branch.lock_write()
+        try:
+            cfg_obj = self._get_config()
+            if section is None:
+                obj = cfg_obj
+            else:
+                try:
+                    obj = cfg_obj[section]
+                except KeyError:
+                    cfg_obj[section] = {}
+                    obj = cfg_obj[section]
+            obj[name] = value
+            out_file = StringIO()
+            cfg_obj.write(out_file)
+            out_file.seek(0)
+            self.branch.control_files.put('branch.conf', out_file)
+        finally:
+            self.branch.unlock()

=== added file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/conflicts.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,457 @@
+# Copyright (C) 2005 by Aaron Bentley
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# TODO: Move this into builtins
+
+# TODO: 'bzr resolve' should accept a directory name and work from that 
+# point down
+
+import os
+import errno
+
+import bzrlib
+from bzrlib.commands import register_command
+from bzrlib.errors import BzrCommandError, NotConflicted, UnsupportedOperation
+from bzrlib.option import Option
+from bzrlib.osutils import rename, delete_any
+from bzrlib.rio import Stanza
+
+
+CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
+
+
+class cmd_conflicts(bzrlib.commands.Command):
+    """List files with conflicts.
+
+    Merge will do its best to combine the changes in two branches, but there
+    are some kinds of problems only a human can fix.  When it encounters those,
+    it will mark a conflict.  A conflict means that you need to fix something,
+    before you should commit.
+
+    Use bzr resolve when you have fixed a problem.
+
+    (conflicts are determined by the presence of .BASE .TREE, and .OTHER 
+    files.)
+
+    See also bzr resolve.
+    """
+    def run(self):
+        from bzrlib.workingtree import WorkingTree
+        wt = WorkingTree.open_containing(u'.')[0]
+        for conflict in wt.conflicts():
+            print conflict
+
+
+class cmd_resolve(bzrlib.commands.Command):
+    """Mark a conflict as resolved.
+
+    Merge will do its best to combine the changes in two branches, but there
+    are some kinds of problems only a human can fix.  When it encounters those,
+    it will mark a conflict.  A conflict means that you need to fix something,
+    before you should commit.
+
+    Once you have fixed a problem, use "bzr resolve FILE.." to mark
+    individual files as fixed, or "bzr resolve --all" to mark all conflicts as
+    resolved.
+
+    See also bzr conflicts.
+    """
+    aliases = ['resolved']
+    takes_args = ['file*']
+    takes_options = [Option('all', help='Resolve all conflicts in this tree')]
+    def run(self, file_list=None, all=False):
+        from bzrlib.workingtree import WorkingTree
+        if all:
+            if file_list:
+                raise BzrCommandError("If --all is specified, no FILE may be provided")
+            tree = WorkingTree.open_containing('.')[0]
+            resolve(tree)
+        else:
+            if file_list is None:
+                raise BzrCommandError("command 'resolve' needs one or more FILE, or --all")
+            tree = WorkingTree.open_containing(file_list[0])[0]
+            to_resolve = [tree.relpath(p) for p in file_list]
+            resolve(tree, to_resolve)
+
+
+def resolve(tree, paths=None, ignore_misses=False):
+    tree.lock_write()
+    try:
+        tree_conflicts = tree.conflicts()
+        if paths is None:
+            new_conflicts = ConflictList()
+            selected_conflicts = tree_conflicts
+        else:
+            new_conflicts, selected_conflicts = \
+                tree_conflicts.select_conflicts(tree, paths, ignore_misses)
+        try:
+            tree.set_conflicts(new_conflicts)
+        except UnsupportedOperation:
+            pass
+        selected_conflicts.remove_files(tree)
+    finally:
+        tree.unlock()
+
+
+def restore(filename):
+    """\
+    Restore a conflicted file to the state it was in before merging.
+    Only text restoration supported at present.
+    """
+    conflicted = False
+    try:
+        rename(filename + ".THIS", filename)
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    try:
+        os.unlink(filename + ".BASE")
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    try:
+        os.unlink(filename + ".OTHER")
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    if not conflicted:
+        raise NotConflicted(filename)
+
+
+class ConflictList(object):
+    """List of conflicts.
+
+    Typically obtained from WorkingTree.conflicts()
+
+    Can be instantiated from stanzas or from Conflict subclasses.
+    """
+
+    def __init__(self, conflicts=None):
+        object.__init__(self)
+        if conflicts is None:
+            self.__list = []
+        else:
+            self.__list = conflicts
+
+    def is_empty(self):
+        return len(self.__list) == 0
+
+    def __len__(self):
+        return len(self.__list)
+
+    def __iter__(self):
+        return iter(self.__list)
+
+    def __getitem__(self, key):
+        return self.__list[key]
+
+    def append(self, conflict):
+        return self.__list.append(conflict)
+
+    def __eq__(self, other_list):
+        return list(self) == list(other_list)
+
+    def __ne__(self, other_list):
+        return not (self == other_list)
+
+    def __repr__(self):
+        return "ConflictList(%r)" % self.__list
+
+    @staticmethod
+    def from_stanzas(stanzas):
+        """Produce a new ConflictList from an iterable of stanzas"""
+        conflicts = ConflictList()
+        for stanza in stanzas:
+            conflicts.append(Conflict.factory(**stanza.as_dict()))
+        return conflicts
+
+    def to_stanzas(self):
+        """Generator of stanzas"""
+        for conflict in self:
+            yield conflict.as_stanza()
+            
+    def to_strings(self):
+        """Generate strings for the provided conflicts"""
+        for conflict in self:
+            yield str(conflict)
+
+    def remove_files(self, tree):
+        """Remove the THIS, BASE and OTHER files for listed conflicts"""
+        for conflict in self:
+            if not conflict.has_files:
+                continue
+            for suffix in CONFLICT_SUFFIXES:
+                try:
+                    delete_any(tree.abspath(conflict.path+suffix))
+                except OSError, e:
+                    if e.errno != errno.ENOENT:
+                        raise
+
+    def select_conflicts(self, tree, paths, ignore_misses=False):
+        """Select the conflicts associated with paths in a tree.
+        
+        File-ids are also used for this.
+        """
+        path_set = set(paths)
+        ids = {}
+        selected_paths = set()
+        new_conflicts = ConflictList()
+        selected_conflicts = ConflictList()
+        for path in paths:
+            file_id = tree.path2id(path)
+            if file_id is not None:
+                ids[file_id] = path
+
+        for conflict in self:
+            selected = False
+            for key in ('path', 'conflict_path'):
+                cpath = getattr(conflict, key, None)
+                if cpath is None:
+                    continue
+                if cpath in path_set:
+                    selected = True
+                    selected_paths.add(cpath)
+            for key in ('file_id', 'conflict_file_id'):
+                cfile_id = getattr(conflict, key, None)
+                if cfile_id is None:
+                    continue
+                try:
+                    cpath = ids[cfile_id]
+                except KeyError:
+                    continue
+                selected = True
+                selected_paths.add(cpath)
+            if selected:
+                selected_conflicts.append(conflict)
+            else:
+                new_conflicts.append(conflict)
+        if ignore_misses is not True:
+            for path in [p for p in paths if p not in selected_paths]:
+                if not os.path.exists(tree.abspath(path)):
+                    print "%s does not exist" % path
+                else:
+                    print "%s is not conflicted" % path
+        return new_conflicts, selected_conflicts
+
+ 
+class Conflict(object):
+    """Base class for all types of conflict"""
+
+    has_files = False
+
+    def __init__(self, path, file_id=None):
+        self.path = path
+        self.file_id = file_id
+
+    def as_stanza(self):
+        s = Stanza(type=self.typestring, path=self.path)
+        if self.file_id is not None:
+            s.add('file_id', self.file_id)
+        return s
+
+    def _cmp_list(self):
+        return [type(self), self.path, self.file_id]
+
+    def __cmp__(self, other):
+        if getattr(other, "_cmp_list", None) is None:
+            return -1
+        return cmp(self._cmp_list(), other._cmp_list())
+
+    def __eq__(self, other):
+        return self.__cmp__(other) == 0
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __str__(self):
+        return self.format % self.__dict__
+
+    def __repr__(self):
+        rdict = dict(self.__dict__)
+        rdict['class'] = self.__class__.__name__
+        return self.rformat % rdict
+
+    @staticmethod
+    def factory(type, **kwargs):
+        global ctype
+        return ctype[type](**kwargs)
+
+    @staticmethod
+    def sort_key(conflict):
+        if conflict.path is not None:
+            return conflict.path, conflict.typestring
+        elif getattr(conflict, "conflict_path", None) is not None:
+            return conflict.conflict_path, conflict.typestring
+        else:
+            return None, conflict.typestring
+
+
+class PathConflict(Conflict):
+    """A conflict was encountered merging file paths"""
+
+    typestring = 'path conflict'
+
+    format = 'Path conflict: %(path)s / %(conflict_path)s'
+
+    rformat = '%(class)s(%(path)r, %(conflict_path)r, %(file_id)r)'
+    def __init__(self, path, conflict_path=None, file_id=None):
+        Conflict.__init__(self, path, file_id)
+        self.conflict_path = conflict_path
+
+    def as_stanza(self):
+        s = Conflict.as_stanza(self)
+        if self.conflict_path is not None:
+            s.add('conflict_path', self.conflict_path)
+        return s
+
+
+class ContentsConflict(PathConflict):
+    """The files are of different types, or not present"""
+
+    has_files = True
+
+    typestring = 'contents conflict'
+
+    format = 'Contents conflict in %(path)s'
+
+
+class TextConflict(PathConflict):
+    """The merge algorithm could not resolve all differences encountered."""
+
+    has_files = True
+
+    typestring = 'text conflict'
+
+    format = 'Text conflict in %(path)s'
+
+
+class HandledConflict(Conflict):
+    """A path problem that has been provisionally resolved.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(file_id)r)"
+    
+    def __init__(self, action, path, file_id=None):
+        Conflict.__init__(self, path, file_id)
+        self.action = action
+
+    def _cmp_list(self):
+        return Conflict._cmp_list(self) + [self.action]
+
+    def as_stanza(self):
+        s = Conflict.as_stanza(self)
+        s.add('action', self.action)
+        return s
+
+
+class HandledPathConflict(HandledConflict):
+    """A provisionally-resolved path problem involving two paths.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(conflict_path)r,"\
+        " %(file_id)r, %(conflict_file_id)r)"
+
+    def __init__(self, action, path, conflict_path, file_id=None,
+                 conflict_file_id=None):
+        HandledConflict.__init__(self, action, path, file_id)
+        self.conflict_path = conflict_path 
+        self.conflict_file_id = conflict_file_id
+        
+    def _cmp_list(self):
+        return HandledConflict._cmp_list(self) + [self.conflict_path, 
+                                                  self.conflict_file_id]
+
+    def as_stanza(self):
+        s = HandledConflict.as_stanza(self)
+        s.add('conflict_path', self.conflict_path)
+        if self.conflict_file_id is not None:
+            s.add('conflict_file_id', self.conflict_file_id)
+            
+        return s
+
+
+class DuplicateID(HandledPathConflict):
+    """Two files want the same file_id."""
+
+    typestring = 'duplicate id'
+
+    format = 'Conflict adding id to %(conflict_path)s.  %(action)s %(path)s.'
+
+
+class DuplicateEntry(HandledPathConflict):
+    """Two directory entries want to have the same name."""
+
+    typestring = 'duplicate'
+
+    format = 'Conflict adding file %(conflict_path)s.  %(action)s %(path)s.'
+
+
+class ParentLoop(HandledPathConflict):
+    """An attempt to create an infinitely-looping directory structure.
+    This is rare, but can be produced like so:
+
+    tree A:
+      mv foo/bar
+    tree B:
+      mv bar/foo
+    merge A and B
+    """
+
+    typestring = 'parent loop'
+
+    format = 'Conflict moving %(conflict_path)s into %(path)s.  %(action)s.'
+
+
+class UnversionedParent(HandledConflict):
+    """An attempt to version an file whose parent directory is not versioned.
+    Typically, the result of a merge where one tree unversioned the directory
+    and the other added a versioned file to it.
+    """
+
+    typestring = 'unversioned parent'
+
+    format = 'Conflict adding versioned files to %(path)s.  %(action)s.'
+
+
+class MissingParent(HandledConflict):
+    """An attempt to add files to a directory that is not present.
+    Typically, the result of a merge where one tree deleted the directory and
+    the other added a file to it.
+    """
+
+    typestring = 'missing parent'
+
+    format = 'Conflict adding files to %(path)s.  %(action)s.'
+
+
+
+ctype = {}
+
+
+def register_types(*conflict_types):
+    """Register a Conflict subclass for serialization purposes"""
+    global ctype
+    for conflict_type in conflict_types:
+        ctype[conflict_type.typestring] = conflict_type
+
+
+register_types(ContentsConflict, TextConflict, PathConflict, DuplicateID,
+               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,)

=== added file 'bzrlib/decorators.py'
--- bzrlib/decorators.py	1970-01-01 00:00:00 +0000
+++ bzrlib/decorators.py	2006-02-08 05:24:53 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+__all__ = ['needs_read_lock',
+           'needs_write_lock',
+           ]
+
+def needs_read_lock(unbound):
+    """Decorate unbound to take out and release a read lock.
+
+    This decorator can be applied to methods of any class with lock_read() and
+    unlock() methods.
+    
+    Typical usage:
+        
+    class Branch(...):
+        @needs_read_lock
+        def branch_method(self, ...):
+            stuff
+    """
+    def read_locked(self, *args, **kwargs):
+        self.lock_read()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    read_locked.__doc__ = unbound.__doc__
+    read_locked.__name__ = unbound.__name__
+    return read_locked
+
+
+def needs_write_lock(unbound):
+    """Decorate unbound to take out and release a write lock."""
+    def write_locked(self, *args, **kwargs):
+        self.lock_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    write_locked.__doc__ = unbound.__doc__
+    write_locked.__name__ = unbound.__name__
+    return write_locked
+

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-08-01 20:25:24 +0000
+++ bzrlib/delta.py	2006-04-13 05:37:24 +0000
@@ -1,5 +1,4 @@
-#! /usr/bin/env python
-# -*- coding: UTF-8 -*-
+# Copyright (C) 2005, 2006 Canonical
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter
 
 class TreeDelta(object):
@@ -27,16 +27,18 @@
     removed
         (path, id, kind)
     renamed
-        (oldpath, newpath, id, kind, text_modified)
+        (oldpath, newpath, id, kind, text_modified, meta_modified)
     modified
-        (path, id, kind)
+        (path, id, kind, text_modified, meta_modified)
     unchanged
         (path, id, kind)
 
     Each id is listed only once.
 
     Files that are both modified and renamed are listed only in
-    renamed, with the text_modified flag true.
+    renamed, with the text_modified flag true. The text_modified
+    applies either to the the content of the file or the target of the
+    symbolic link, depending of the kind of file.
 
     Files are only considered renamed if their name has changed or
     their parent directory has changed.  Renaming a directory
@@ -69,9 +71,10 @@
             self.modified, self.unchanged)
 
     def has_changed(self):
-        changes = len(self.added) + len(self.removed) + len(self.renamed)
-        changes += len(self.modified) 
-        return (changes != 0)
+        return bool(self.modified
+                    or self.added
+                    or self.removed
+                    or self.renamed)
 
     def touches_file_id(self, file_id):
         """Return True if file_id is modified by this delta."""
@@ -87,12 +90,17 @@
 
     def show(self, to_file, show_ids=False, show_unchanged=False):
         def show_list(files):
-            for path, fid, kind in files:
+            for item in files:
+                path, fid, kind = item[:3]
+
                 if kind == 'directory':
                     path += '/'
                 elif kind == 'symlink':
                     path += '@'
-                    
+
+                if len(item) == 5 and item[4]:
+                    path += '*'
+
                 if show_ids:
                     print >>to_file, '  %-30s %s' % (path, fid)
                 else:
@@ -106,17 +114,26 @@
             print >>to_file, 'added:'
             show_list(self.added)
 
+        extra_modified = []
+
         if self.renamed:
             print >>to_file, 'renamed:'
-            for oldpath, newpath, fid, kind, text_modified in self.renamed:
+            for (oldpath, newpath, fid, kind,
+                 text_modified, meta_modified) in self.renamed:
+                if text_modified or meta_modified:
+                    extra_modified.append((newpath, fid, kind,
+                                           text_modified, meta_modified))
+                if meta_modified:
+                    newpath += '*'
                 if show_ids:
                     print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
                 else:
                     print >>to_file, '  %s => %s' % (oldpath, newpath)
                     
-        if self.modified:
+        if self.modified or extra_modified:
             print >>to_file, 'modified:'
             show_list(self.modified)
+            show_list(extra_modified)
             
         if show_unchanged and self.unchanged:
             print >>to_file, 'unchanged:'
@@ -140,9 +157,24 @@
 
     specific_files
         If true, only check for changes to specified names or
-        files within them.
+        files within them.  Any unversioned files given have no effect
+        (but this might change in the future).
     """
 
+    old_tree.lock_read()
+    try:
+        new_tree.lock_read()
+        try:
+            return _compare_trees(old_tree, new_tree, want_unchanged,
+                                  specific_files)
+        finally:
+            new_tree.unlock()
+    finally:
+        old_tree.unlock()
+
+
+def _compare_trees(old_tree, new_tree, want_unchanged, specific_files):
+
     from osutils import is_inside_any
     
     old_inv = old_tree.inventory
@@ -150,8 +182,10 @@
     delta = TreeDelta()
     mutter('start compare_trees')
 
-    # TODO: match for specific files can be rather smarter by finding
-    # the IDs of those files up front and then considering only that.
+    # TODO: Rather than iterating over the whole tree and then filtering, we
+    # could diff just the specified files (if any) and their subtrees.  
+    # Perhaps should take a list of file-ids instead?   Need to indicate any
+    # ids or names which were not found in the trees.
 
     for file_id in old_tree:
         if file_id in new_tree:
@@ -161,7 +195,7 @@
             kind = old_ie.kind
             assert kind == new_ie.kind
             
-            assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
+            assert kind in InventoryEntry.known_kinds, \
                    'invalid file kind %r' % kind
 
             if kind == 'root_directory':
@@ -172,13 +206,13 @@
                     and not is_inside_any(specific_files, new_inv.id2path(file_id))):
                     continue
 
-            if kind == 'file':
-                old_sha1 = old_tree.get_file_sha1(file_id)
-                new_sha1 = new_tree.get_file_sha1(file_id)
-                text_modified = (old_sha1 != new_sha1)
-            else:
-                ## mutter("no text to check for %r %r" % (file_id, kind))
-                text_modified = False
+            # temporary hack until all entries are populated before clients 
+            # get them
+            old_path = old_inv.id2path(file_id)
+            new_path = new_inv.id2path(file_id)
+            old_ie._read_tree_state(old_path, old_tree)
+            new_ie._read_tree_state(new_path, new_tree)
+            text_modified, meta_modified = new_ie.detect_changes(old_ie)
 
             # TODO: Can possibly avoid calculating path strings if the
             # two files are unchanged and their names and parents are
@@ -187,14 +221,15 @@
             
             if (old_ie.name != new_ie.name
                 or old_ie.parent_id != new_ie.parent_id):
-                delta.renamed.append((old_inv.id2path(file_id),
-                                      new_inv.id2path(file_id),
+                delta.renamed.append((old_path,
+                                      new_path,
                                       file_id, kind,
-                                      text_modified))
-            elif text_modified:
-                delta.modified.append((new_inv.id2path(file_id), file_id, kind))
+                                      text_modified, meta_modified))
+            elif text_modified or meta_modified:
+                delta.modified.append((new_path, file_id, kind,
+                                       text_modified, meta_modified))
             elif want_unchanged:
-                delta.unchanged.append((new_inv.id2path(file_id), file_id, kind))
+                delta.unchanged.append((new_path, file_id, kind))
         else:
             kind = old_inv.get_file_kind(file_id)
             if kind == 'root_directory':
@@ -207,7 +242,7 @@
 
     mutter('start looking for new files')
     for file_id in new_inv:
-        if file_id in old_inv:
+        if file_id in old_inv or file_id not in new_tree:
             continue
         kind = new_inv.get_file_kind(file_id)
         if kind == 'root_directory':

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-08-04 22:30:50 +0000
+++ bzrlib/diff.py	2006-04-13 05:51:35 +0000
@@ -1,5 +1,4 @@
-#! /usr/bin/env python
-# -*- coding: UTF-8 -*-
+# Copyright (C) 2004, 2005, 2006 Canonical Ltd.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,15 +14,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib.delta import compare_trees
+from bzrlib.errors import BzrError
+import bzrlib.errors as errors
+from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
-from bzrlib.delta import compare_trees
 
 # TODO: Rather than building a changeset object, we should probably
 # invoke callbacks on an object.  That object can either accumulate a
 # list, write them out directly, etc etc.
 
-def internal_diff(old_label, oldlines, new_label, newlines, to_file):
+def internal_diff(old_filename, oldlines, new_filename, newlines, to_file):
     import difflib
     
     # FIXME: difflib is wrong if there is no trailing newline.
@@ -43,16 +44,19 @@
         return
 
     ud = difflib.unified_diff(oldlines, newlines,
-                              fromfile=old_label, tofile=new_label)
+                              fromfile=old_filename+'\t', 
+                              tofile=new_filename+'\t')
 
+    ud = list(ud)
     # work-around for difflib being too smart for its own good
     # if /dev/null is "1,0", patch won't recognize it as /dev/null
     if not oldlines:
-        ud = list(ud)
         ud[2] = ud[2].replace('-1,0', '-0,0')
     elif not newlines:
-        ud = list(ud)
         ud[2] = ud[2].replace('+1,0', '+0,0')
+    # work around for difflib emitting random spaces after the label
+    ud[0] = ud[0][:-2] + '\n'
+    ud[1] = ud[1][:-2] + '\n'
 
     for line in ud:
         to_file.write(line)
@@ -61,9 +65,7 @@
     print >>to_file
 
 
-
-
-def external_diff(old_label, oldlines, new_label, newlines, to_file,
+def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
     import sys
@@ -98,9 +100,9 @@
         if not diff_opts:
             diff_opts = []
         diffcmd = ['diff',
-                   '--label', old_label,
+                   '--label', old_filename+'\t',
                    oldtmpf.name,
-                   '--label', new_label,
+                   '--label', new_filename+'\t',
                    newtmpf.name]
 
         # diff only allows one style to be specified; they don't override.
@@ -140,37 +142,88 @@
     finally:
         oldtmpf.close()                 # and delete
         newtmpf.close()
-    
-
-
-def show_diff(b, revision, specific_files, external_diff_options=None,
-              revision2=None):
+
+
+@deprecated_function(zero_eight)
+def show_diff(b, from_spec, specific_files, external_diff_options=None,
+              revision2=None, output=None, b2=None):
     """Shortcut for showing the diff to the working tree.
 
+    Please use show_diff_trees instead.
+
     b
         Branch.
 
     revision
-        None for each, or otherwise the old revision to compare against.
+        None for 'basis tree', or otherwise the old revision to compare against.
+    
+    The more general form is show_diff_trees(), where the caller
+    supplies any two trees.
+    """
+    if output is None:
+        import sys
+        output = sys.stdout
+
+    if from_spec is None:
+        old_tree = b.bzrdir.open_workingtree()
+        if b2 is None:
+            old_tree = old_tree = old_tree.basis_tree()
+    else:
+        old_tree = b.repository.revision_tree(from_spec.in_history(b).rev_id)
+
+    if revision2 is None:
+        if b2 is None:
+            new_tree = b.bzrdir.open_workingtree()
+        else:
+            new_tree = b2.bzrdir.open_workingtree()
+    else:
+        new_tree = b.repository.revision_tree(revision2.in_history(b).rev_id)
+
+    return show_diff_trees(old_tree, new_tree, output, specific_files,
+                           external_diff_options)
+
+
+def diff_cmd_helper(tree, specific_files, external_diff_options, 
+                    old_revision_spec=None, new_revision_spec=None):
+    """Helper for cmd_diff.
+
+   tree 
+        A WorkingTree
+
+    specific_files
+        The specific files to compare, or None
+
+    external_diff_options
+        If non-None, run an external diff, and pass it these options
+
+    old_revision_spec
+        If None, use basis tree as old revision, otherwise use the tree for
+        the specified revision. 
+
+    new_revision_spec
+        If None, use working tree as new revision, otherwise use the tree for
+        the specified revision.
     
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
     import sys
-
-    if revision == None:
-        old_tree = b.basis_tree()
-    else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
-
-    if revision2 == None:
-        new_tree = b.working_tree()
-    else:
-        new_tree = b.revision_tree(b.lookup_revision(revision2))
-
-    show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
-                    external_diff_options)
-
+    output = sys.stdout
+    def spec_tree(spec):
+        revision_id = spec.in_store(tree.branch).rev_id
+        return tree.branch.repository.revision_tree(revision_id)
+    if old_revision_spec is None:
+        old_tree = tree.basis_tree()
+    else:
+        old_tree = spec_tree(old_revision_spec)
+
+    if new_revision_spec is None:
+        new_tree = tree
+    else:
+        new_tree = spec_tree(new_revision_spec)
+
+    return show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
+                           external_diff_options)
 
 
 def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
@@ -183,10 +236,25 @@
     external_diff_options
         If set, use an external GNU diff and pass these options.
     """
-
-    # TODO: Options to control putting on a prefix or suffix, perhaps as a format string
-    old_label = ''
-    new_label = ''
+    old_tree.lock_read()
+    try:
+        new_tree.lock_read()
+        try:
+            return _show_diff_trees(old_tree, new_tree, to_file,
+                                    specific_files, external_diff_options)
+        finally:
+            new_tree.unlock()
+    finally:
+        old_tree.unlock()
+
+
+def _show_diff_trees(old_tree, new_tree, to_file,
+                     specific_files, external_diff_options):
+
+    # TODO: Options to control putting on a prefix or suffix, perhaps
+    # as a format string?
+    old_label = 'a/'
+    new_label = 'b/'
 
     DEVNULL = '/dev/null'
     # Windows users, don't panic about this filename -- it is a
@@ -196,6 +264,8 @@
     # TODO: Generation of pseudo-diffs for added/deleted files could
     # be usefully made into a much faster special case.
 
+    _raise_if_doubly_unversioned(specific_files, old_tree, new_tree)
+
     if external_diff_options:
         assert isinstance(external_diff_options, basestring)
         opts = external_diff_options.split()
@@ -204,47 +274,67 @@
     else:
         diff_file = internal_diff
     
-
     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
                           specific_files=specific_files)
 
+    has_changes = 0
     for path, file_id, kind in delta.removed:
-        print >>to_file, '*** removed %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      DEVNULL, 
-                      [],
-                      to_file)
-
+        has_changes = 1
+        print >>to_file, '=== removed %s %r' % (kind, old_label + path)
+        old_tree.inventory[file_id].diff(diff_file, old_label + path, old_tree,
+                                         DEVNULL, None, None, to_file)
     for path, file_id, kind in delta.added:
-        print >>to_file, '*** added %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(DEVNULL,
-                      [],
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-        print >>to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
+        has_changes = 1
+        print >>to_file, '=== added %s %r' % (kind, new_label + path)
+        new_tree.inventory[file_id].diff(diff_file, new_label + path, new_tree,
+                                         DEVNULL, None, None, to_file, 
+                                         reverse=True)
+    for (old_path, new_path, file_id, kind,
+         text_modified, meta_modified) in delta.renamed:
+        has_changes = 1
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== renamed %s %r => %r%s' % (
+                    kind, old_label + old_path, new_label + new_path, prop_str)
+        _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                    new_label, new_path, new_tree,
+                                    text_modified, kind, to_file, diff_file)
+    for path, file_id, kind, text_modified, meta_modified in delta.modified:
+        has_changes = 1
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== modified %s %r%s' % (kind, old_label + path,
+                    prop_str)
         if text_modified:
-            diff_file(old_label + old_path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + new_path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for path, file_id, kind in delta.modified:
-        print >>to_file, '*** modified %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-
-
-
-
+            _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
+                                        new_label, path, new_tree,
+                                        True, kind, to_file, diff_file)
+
+    return has_changes
+
+
+def _raise_if_doubly_unversioned(specific_files, old_tree, new_tree):
+    """Complain if paths are not versioned in either tree."""
+    if not specific_files:
+        return
+    old_unversioned = old_tree.filter_unversioned_files(specific_files)
+    new_unversioned = new_tree.filter_unversioned_files(specific_files)
+    unversioned = old_unversioned.intersection(new_unversioned)
+    if unversioned:
+        raise errors.PathsNotVersionedError(sorted(unversioned))
+    
+
+def get_prop_change(meta_modified):
+    if meta_modified:
+        return " (properties changed)"
+    else:
+        return  ""
+
+
+def _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                new_label, new_path, new_tree, text_modified,
+                                kind, to_file, diff_file):
+    if text_modified:
+        new_entry = new_tree.inventory[file_id]
+        old_tree.inventory[file_id].diff(diff_file,
+                                         old_label + old_path, old_tree,
+                                         new_label + new_path, new_entry, 
+                                         new_tree, to_file)

=== added directory 'bzrlib/doc'
=== added file 'bzrlib/doc/__init__.py'
--- bzrlib/doc/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc/__init__.py	2005-12-24 02:20:45 +0000
@@ -0,0 +1,26 @@
+# (C) 2005 Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Documentation for bzrlib.
+
+See bzrlib.doc.api for api documentation and in the future bzrlib.doc.man
+for man page generation.
+"""
+
+    
+def test_suite():
+    import bzrlib.doc.api
+    return bzrlib.doc.api.test_suite()

=== added directory 'bzrlib/doc/api'
=== added file 'bzrlib/doc/api/__init__.py'
--- bzrlib/doc/api/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc/api/__init__.py	2005-12-24 02:20:45 +0000
@@ -0,0 +1,34 @@
+# (C) 2005 Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""API Documentation for bzrlib.
+
+This documentation is made up of doctest testable examples.
+
+Look for bzrlib/doc/api/*.txt to read it.
+
+This documentation documents the current best practice in using the library.
+For details on specific apis, see pydoc on the api, or read the source.
+"""
+
+import doctest
+import os
+    
+def test_suite():
+    candidates = os.listdir(os.path.dirname(__file__))
+    scripts = [candidate for candidate in candidates 
+               if candidate.endswith('.txt')]
+    return doctest.DocFileSuite(*scripts)

=== added file 'bzrlib/doc/api/branch.txt'
--- bzrlib/doc/api/branch.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/doc/api/branch.txt	2006-02-01 11:50:27 +0000
@@ -0,0 +1,29 @@
+The Branch API in bzrlib provides creation and management of Branches.
+
+A Branch represents a series of commits and merges carried out by a user.
+
+Branches are created by BzrDir's:
+
+   >>> from bzrlib.bzrdir import ScratchDir
+   >>> from bzrlib.branch import Branch
+   >>> dir = ScratchDir()
+   >>> new_branch = dir.create_branch()
+
+
+Existing Branches can be opened directly:
+
+   >>> another_instance = Branch.open(dir.transport.clone('..').base)
+
+or via the BzrDir:
+
+   >>> still_the_same_branch = dir.open_branch()
+
+
+A branch has a history of revisions on it:
+
+   >>> new_branch.revision_history()
+   []
+
+
+We need to write some more documentation, showing
+push and pull operations. Cloning might also be nice.

=== added file 'bzrlib/doc/api/transport.txt'
--- bzrlib/doc/api/transport.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/doc/api/transport.txt	2006-03-30 05:11:23 +0000
@@ -0,0 +1,19 @@
+The Transport API in bzrlib provides URL based access to network resources.
+
+   >>> import os
+   >>> from bzrlib.osutils import getcwd, dirname
+   >>> import bzrlib.transport as transport
+   >>> root = transport.get_transport("file:///")
+   >>>
+
+Each Transport instance represents a single logical directory.
+
+   >>> dir = transport.get_transport(".")
+   >>> dir.base == getcwd() + '/'
+   True
+
+You can change directories via the clone method:
+
+   >>> parent = dir.clone('..')
+   >>> parent.base == (dirname(getcwd()).rstrip('/') + '/')
+   True

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-08-04 21:39:23 +0000
+++ bzrlib/errors.py	2006-04-13 23:03:17 +0000
@@ -1,5 +1,4 @@
-#! /usr/bin/env python
-# -*- coding: UTF-8 -*-
+# Copyright (C) 2005, 2006 Canonical
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,73 +14,382 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
-######################################################################
-# exceptions 
+"""Exceptions for bzr, and reporting of them.
+
+Exceptions are caught at a high level to report errors to the user, and
+might also be caught inside the program.  Therefore it needs to be
+possible to convert them to a meaningful string, and also for them to be
+interrogated by the program.
+
+Exceptions are defined such that the arguments given to the constructor
+are stored in the object as properties of the same name.  When the
+object is printed as a string, the doc string of the class is used as
+a format string with the property dictionary available to it.
+
+This means that exceptions can used like this:
+
+>>> import sys
+>>> try:
+...   raise NotBranchError(path='/foo/bar')
+... except:
+...   print sys.exc_type
+...   print sys.exc_value
+...   path = getattr(sys.exc_value, 'path')
+...   if path is not None:
+...     print path
+bzrlib.errors.NotBranchError
+Not a branch: /foo/bar
+/foo/bar
+
+Therefore:
+
+ * create a new exception class for any class of error that can be
+   usefully distinguished.
+
+ * the printable form of an exception is generated by the base class
+   __str__ method
+
+Exception strings should start with a capital letter and not have a final
+fullstop.
+"""
+
+from warnings import warn
+
+# based on Scott James Remnant's hct error classes
+
+# TODO: is there any value in providing the .args field used by standard
+# python exceptions?   A list of values with no names seems less useful 
+# to me.
+
+# TODO: Perhaps convert the exception to a string at the moment it's 
+# constructed to make sure it will succeed.  But that says nothing about
+# exceptions that are never raised.
+
+# TODO: Convert all the other error classes here to BzrNewError, and eliminate
+# the old one.
+
+
 class BzrError(StandardError):
-    pass
-
-class BzrCheckError(BzrError):
-    pass
-
-
-class InvalidRevisionNumber(BzrError):
+    def __str__(self):
+        # XXX: Should we show the exception class in 
+        # exceptions that don't provide their own message?  
+        # maybe it should be done at a higher level
+        ## n = self.__class__.__name__ + ': '
+        n = ''
+        if len(self.args) == 1:
+            return str(self.args[0])
+        elif len(self.args) == 2:
+            # further explanation or suggestions
+            try:
+                return n + '\n  '.join([self.args[0]] + self.args[1])
+            except TypeError:
+                return n + "%r" % self
+        else:
+            return n + `self.args`
+
+
+class BzrNewError(BzrError):
+    """bzr error"""
+    # base classes should override the docstring with their human-
+    # readable explanation
+
+    def __init__(self, **kwds):
+        for key, value in kwds.items():
+            setattr(self, key, value)
+
+    def __str__(self):
+        try:
+            return self.__doc__ % self.__dict__
+        except (NameError, ValueError, KeyError), e:
+            return 'Unprintable exception %s: %s' \
+                % (self.__class__.__name__, str(e))
+
+
+class BzrCheckError(BzrNewError):
+    """Internal check failed: %(message)s"""
+
+    def __init__(self, message):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class InvalidEntryName(BzrNewError):
+    """Invalid entry name: %(name)s"""
+    def __init__(self, name):
+        BzrNewError.__init__(self)
+        self.name = name
+
+
+class InvalidRevisionNumber(BzrNewError):
+    """Invalid revision number %(revno)d"""
     def __init__(self, revno):
-        self.args = [revno]
-        
-    def __str__(self):
-        return 'invalid revision number: %r' % self.args[0]
-
-
-class InvalidRevisionId(BzrError):
-    pass
+        BzrNewError.__init__(self)
+        self.revno = revno
+
+
+class InvalidRevisionId(BzrNewError):
+    """Invalid revision-id {%(revision_id)s} in %(branch)s"""
+    def __init__(self, revision_id, branch):
+        BzrNewError.__init__(self)
+        self.revision_id = revision_id
+        self.branch = branch
+
+
+class NoWorkingTree(BzrNewError):
+    """No WorkingTree exists for %s(base)."""
+    
+    def __init__(self, base):
+        BzrNewError.__init__(self)
+        self.base = base
+
+
+class NotLocalUrl(BzrNewError):
+    """%s(url) is not a local path."""
+    
+    def __init__(self, url):
+        BzrNewError.__init__(self)
+        self.url = url
 
 
 class BzrCommandError(BzrError):
     # Error from malformed user command
-    pass
-
-
-class NotBranchError(BzrError):
-    """Specified path is not in a branch"""
-    pass
-
-
-class NotVersionedError(BzrError):
-    """Specified object is not versioned."""
+    # This is being misused as a generic exception
+    # pleae subclass. RBC 20051030
+    #
+    # I think it's a waste of effort to differentiate between errors that
+    # are not intended to be caught anyway.  UI code need not subclass
+    # BzrCommandError, and non-UI code should not throw a subclass of
+    # BzrCommandError.  ADHB 20051211
+    def __str__(self):
+        return self.args[0]
+
+
+class BzrOptionError(BzrCommandError):
+    """Some missing or otherwise incorrect option was supplied."""
+
+    
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
+
+
+class PathError(BzrNewError):
+    """Generic path error: %(path)r%(extra)s)"""
+
+    def __init__(self, path, extra=None):
+        BzrNewError.__init__(self)
+        self.path = path
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class NoSuchFile(PathError):
+    """No such file: %(path)r%(extra)s"""
+
+
+class FileExists(PathError):
+    """File exists: %(path)r%(extra)s"""
+
+
+class DirectoryNotEmpty(PathError):
+    """Directory not empty: %(path)r%(extra)s"""
+
+
+class ResourceBusy(PathError):
+    """Device or resource busy: %(path)r%(extra)s"""
+
+
+class PermissionDenied(PathError):
+    """Permission denied: %(path)r%(extra)s"""
+
+
+class PathNotChild(BzrNewError):
+    """Path %(path)r is not a child of path %(base)r%(extra)s"""
+    def __init__(self, path, base, extra=None):
+        BzrNewError.__init__(self)
+        self.path = path
+        self.base = base
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class NotBranchError(PathError):
+    """Not a branch: %(path)s"""
+
+
+class AlreadyBranchError(PathError):
+    """Already a branch: %(path)s. Use `bzr checkout` to build a working tree."""
+
+
+class NoRepositoryPresent(BzrNewError):
+    """Not repository present: %(path)r"""
+    def __init__(self, bzrdir):
+        BzrNewError.__init__(self)
+        self.path = bzrdir.transport.clone('..').base
+
+
+class FileInWrongBranch(BzrNewError):
+    """File %(path)s in not in branch %(branch_base)s."""
+
+    def __init__(self, branch, path):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.branch_base = branch.base
+        self.path = path
+
+
+class UnsupportedFormatError(BzrError):
+    """Specified path is a bzr branch that we recognize but cannot read."""
+    def __str__(self):
+        return 'unsupported branch format: %s' % self.args[0]
+
+
+class UnknownFormatError(BzrError):
+    """Specified path is a bzr branch whose format we do not recognize."""
+    def __str__(self):
+        return 'unknown branch format: %s' % self.args[0]
+
+
+class IncompatibleFormat(BzrNewError):
+    """Format %(format)s is not compatible with .bzr version %(bzrdir)s."""
+
+    def __init__(self, format, bzrdir_format):
+        BzrNewError.__init__(self)
+        self.format = format
+        self.bzrdir = bzrdir_format
+
+
+class NotVersionedError(BzrNewError):
+    """%(path)s is not versioned"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
+
+
+class PathsNotVersionedError(BzrNewError):
+    # used when reporting several paths are not versioned
+    """Path(s) are not versioned: %(paths_as_string)s"""
+
+    def __init__(self, paths):
+        from bzrlib.osutils import quotefn
+        BzrNewError.__init__(self)
+        self.paths = paths
+        self.paths_as_string = ' '.join([quotefn(p) for p in paths])
 
 
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 
     (For example a symlink or device file.)"""
-    pass
 
 
 class ForbiddenFileError(BzrError):
     """Cannot operate on a file because it is a control file."""
-    pass
-
-
-class LockError(Exception):
-    """All exceptions from the lock/unlock functions should be from
-    this exception class.  They will be translated as necessary. The
-    original exception is available as e.original_error
-    """
-    def __init__(self, e=None):
-        self.original_error = e
-        if e:
-            Exception.__init__(self, e)
-        else:
-            Exception.__init__(self)
-
-
-class PointlessCommit(Exception):
-    """Commit failed because nothing was changed."""
+
+
+class LockError(BzrNewError):
+    """Lock error: %(message)s"""
+    # All exceptions from the lock/unlock functions should be from
+    # this exception class.  They will be translated as necessary. The
+    # original exception is available as e.original_error
+    #
+    # New code should prefer to raise specific subclasses
+    def __init__(self, message):
+        self.message = message
+
+
+class CommitNotPossible(LockError):
+    """A commit was attempted but we do not have a write lock open."""
+    def __init__(self):
+        pass
+
+
+class AlreadyCommitted(LockError):
+    """A rollback was requested, but is not able to be accomplished."""
+    def __init__(self):
+        pass
+
+
+class ReadOnlyError(LockError):
+    """A write attempt was made in a read only transaction on %(obj)s"""
+    def __init__(self, obj):
+        self.obj = obj
+
+
+class OutSideTransaction(BzrNewError):
+    """A transaction related operation was attempted after the transaction finished."""
+
+
+class ObjectNotLocked(LockError):
+    """%(obj)r is not locked"""
+    # this can indicate that any particular object is not locked; see also
+    # LockNotHeld which means that a particular *lock* object is not held by
+    # the caller -- perhaps they should be unified.
+    def __init__(self, obj):
+        self.obj = obj
+
+
+class ReadOnlyObjectDirtiedError(ReadOnlyError):
+    """Cannot change object %(obj)r in read only transaction"""
+    def __init__(self, obj):
+        self.obj = obj
+
+
+class UnlockableTransport(LockError):
+    """Cannot lock: transport is read only: %(transport)s"""
+    def __init__(self, transport):
+        self.transport = transport
+
+
+class LockContention(LockError):
+    """Could not acquire lock %(lock)s"""
+    # TODO: show full url for lock, combining the transport and relative bits?
+    def __init__(self, lock):
+        self.lock = lock
+
+
+class LockBroken(LockError):
+    """Lock was broken while still open: %(lock)s - check storage consistency!"""
+    def __init__(self, lock):
+        self.lock = lock
+
+
+class LockBreakMismatch(LockError):
+    """Lock was released and re-acquired before being broken: %(lock)s: held by %(holder)r, wanted to break %(target)r"""
+    def __init__(self, lock, holder, target):
+        self.lock = lock
+        self.holder = holder
+        self.target = target
+
+
+class LockNotHeld(LockError):
+    """Lock not held: %(lock)s"""
+    def __init__(self, lock):
+        self.lock = lock
+
+
+class PointlessCommit(BzrNewError):
+    """No changes to commit"""
+
+
+class UpgradeReadonly(BzrNewError):
+    """Upgrade URL cannot work with readonly URL's."""
+
+
+class UpToDateFormat(BzrNewError):
+    """The branch format %(format)s is already at the most recent format."""
+
+    def __init__(self, format):
+        BzrNewError.__init__(self)
+        self.format = format
+
+
+
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
 
 
 class NoSuchRevision(BzrError):
@@ -92,3 +400,460 @@
         BzrError.__init__(self, msg)
 
 
+class HistoryMissing(BzrError):
+    def __init__(self, branch, object_type, object_id):
+        self.branch = branch
+        BzrError.__init__(self,
+                          '%s is missing %s {%s}'
+                          % (branch, object_type, object_id))
+
+
+class DivergedBranches(BzrError):
+
+    def __init__(self, branch1, branch2):
+        BzrError.__init__(self, "These branches have diverged.  Try merge.")
+        self.branch1 = branch1
+        self.branch2 = branch2
+
+
+class UnrelatedBranches(BzrCommandError):
+    def __init__(self):
+        msg = "Branches have no common ancestor, and no base revision"\
+            " specified."
+        BzrCommandError.__init__(self, msg)
+
+
+class NoCommonAncestor(BzrError):
+    def __init__(self, revision_a, revision_b):
+        msg = "Revisions have no common ancestor: %s %s." \
+            % (revision_a, revision_b) 
+        BzrError.__init__(self, msg)
+
+
+class NoCommonRoot(BzrError):
+    def __init__(self, revision_a, revision_b):
+        msg = "Revisions are not derived from the same root: %s %s." \
+            % (revision_a, revision_b) 
+        BzrError.__init__(self, msg)
+
+
+
+class NotAncestor(BzrError):
+    def __init__(self, rev_id, not_ancestor_id):
+        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
+                                                        rev_id)
+        BzrError.__init__(self, msg)
+        self.rev_id = rev_id
+        self.not_ancestor_id = not_ancestor_id
+
+
+class InstallFailed(BzrError):
+    def __init__(self, revisions):
+        msg = "Could not install revisions:\n%s" % " ,".join(revisions)
+        BzrError.__init__(self, msg)
+        self.revisions = revisions
+
+
+class AmbiguousBase(BzrError):
+    def __init__(self, bases):
+        warn("BzrError AmbiguousBase has been deprecated as of bzrlib 0.8.",
+                DeprecationWarning)
+        msg = "The correct base is unclear, becase %s are all equally close" %\
+            ", ".join(bases)
+        BzrError.__init__(self, msg)
+        self.bases = bases
+
+
+class NoCommits(BzrError):
+    def __init__(self, branch):
+        msg = "Branch %s has no commits." % branch
+        BzrError.__init__(self, msg)
+
+
+class UnlistableStore(BzrError):
+    def __init__(self, store):
+        BzrError.__init__(self, "Store %s is not listable" % store)
+
+
+
+class UnlistableBranch(BzrError):
+    def __init__(self, br):
+        BzrError.__init__(self, "Stores for branch %s are not listable" % br)
+
+
+class BoundBranchOutOfDate(BzrNewError):
+    """Bound branch %(branch)s is out of date with master branch %(master)s."""
+    def __init__(self, branch, master):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.master = master
+
+        
+class CommitToDoubleBoundBranch(BzrNewError):
+    """Cannot commit to branch %(branch)s. It is bound to %(master)s, which is bound to %(remote)s."""
+    def __init__(self, branch, master, remote):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.master = master
+        self.remote = remote
+
+
+class OverwriteBoundBranch(BzrNewError):
+    """Cannot pull --overwrite to a branch which is bound %(branch)s"""
+    def __init__(self, branch):
+        BzrNewError.__init__(self)
+        self.branch = branch
+
+
+class BoundBranchConnectionFailure(BzrNewError):
+    """Unable to connect to target of bound branch %(branch)s => %(target)s: %(error)s"""
+    def __init__(self, branch, target, error):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.target = target
+        self.error = error
+
+
+class WeaveError(BzrNewError):
+    """Error in processing weave: %(message)s"""
+
+    def __init__(self, message=None):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class WeaveRevisionAlreadyPresent(WeaveError):
+    """Revision {%(revision_id)s} already present in %(weave)s"""
+    def __init__(self, revision_id, weave):
+
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveRevisionNotPresent(WeaveError):
+    """Revision {%(revision_id)s} not present in %(weave)s"""
+
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveFormatError(WeaveError):
+    """Weave invariant violated: %(what)s"""
+
+    def __init__(self, what):
+        WeaveError.__init__(self)
+        self.what = what
+
+
+class WeaveParentMismatch(WeaveError):
+    """Parents are mismatched between two revisions."""
+    
+
+class WeaveInvalidChecksum(WeaveError):
+    """Text did not match it's checksum: %(message)s"""
+
+
+class WeaveTextDiffers(WeaveError):
+    """Weaves differ on text content. Revision: {%(revision_id)s}, %(weave_a)s, %(weave_b)s"""
+
+    def __init__(self, revision_id, weave_a, weave_b):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave_a = weave_a
+        self.weave_b = weave_b
+
+
+class WeaveTextDiffers(WeaveError):
+    """Weaves differ on text content. Revision: {%(revision_id)s}, %(weave_a)s, %(weave_b)s"""
+
+    def __init__(self, revision_id, weave_a, weave_b):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave_a = weave_a
+        self.weave_b = weave_b
+
+
+class VersionedFileError(BzrNewError):
+    """Versioned file error."""
+
+
+class RevisionNotPresent(VersionedFileError):
+    """Revision {%(revision_id)s} not present in %(file_id)s."""
+
+    def __init__(self, revision_id, file_id):
+        VersionedFileError.__init__(self)
+        self.revision_id = revision_id
+        self.file_id = file_id
+
+
+class RevisionAlreadyPresent(VersionedFileError):
+    """Revision {%(revision_id)s} already present in %(file_id)s."""
+
+    def __init__(self, revision_id, file_id):
+        VersionedFileError.__init__(self)
+        self.revision_id = revision_id
+        self.file_id = file_id
+
+
+class KnitError(BzrNewError):
+    """Knit error"""
+
+
+class KnitHeaderError(KnitError):
+    """Knit header error: %(badline)r unexpected"""
+
+    def __init__(self, badline):
+        KnitError.__init__(self)
+        self.badline = badline
+
+
+class KnitCorrupt(KnitError):
+    """Knit %(filename)s corrupt: %(how)s"""
+
+    def __init__(self, filename, how):
+        KnitError.__init__(self)
+        self.filename = filename
+        self.how = how
+
+
+class NoSuchExportFormat(BzrNewError):
+    """Export format %(format)r not supported"""
+    def __init__(self, format):
+        BzrNewError.__init__(self)
+        self.format = format
+
+
+class TransportError(BzrError):
+    """All errors thrown by Transport implementations should derive
+    from this class.
+    """
+    def __init__(self, msg=None, orig_error=None):
+        if msg is None and orig_error is not None:
+            msg = str(orig_error)
+        BzrError.__init__(self, msg)
+        self.msg = msg
+        self.orig_error = orig_error
+
+
+# A set of semi-meaningful errors which can be thrown
+class TransportNotPossible(TransportError):
+    """This is for transports where a specific function is explicitly not
+    possible. Such as pushing files to an HTTP server.
+    """
+    pass
+
+
+class ConnectionError(TransportError):
+    """A connection problem prevents file retrieval.
+    This does not indicate whether the file exists or not; it indicates that a
+    precondition for requesting the file was not met.
+    """
+    def __init__(self, msg=None, orig_error=None):
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+
+
+class ConnectionReset(TransportError):
+    """The connection has been closed."""
+    pass
+
+
+class ConflictsInTree(BzrError):
+    def __init__(self):
+        BzrError.__init__(self, "Working tree has conflicts.")
+
+
+class ParseConfigError(BzrError):
+    def __init__(self, errors, filename):
+        if filename is None:
+            filename = ""
+        message = "Error(s) parsing config file %s:\n%s" % \
+            (filename, ('\n'.join(e.message for e in errors)))
+        BzrError.__init__(self, message)
+
+
+class SigningFailed(BzrError):
+    def __init__(self, command_line):
+        BzrError.__init__(self, "Failed to gpg sign data with command '%s'"
+                               % command_line)
+
+
+class WorkingTreeNotRevision(BzrError):
+    def __init__(self, tree):
+        BzrError.__init__(self, "The working tree for %s has changed since"
+                          " last commit, but weave merge requires that it be"
+                          " unchanged." % tree.basedir)
+
+
+class CantReprocessAndShowBase(BzrNewError):
+    """Can't reprocess and show base.
+Reprocessing obscures relationship of conflicting lines to base."""
+
+
+class GraphCycleError(BzrNewError):
+    """Cycle in graph %(graph)r"""
+    def __init__(self, graph):
+        BzrNewError.__init__(self)
+        self.graph = graph
+
+
+class NotConflicted(BzrNewError):
+    """File %(filename)s is not conflicted."""
+
+    def __init__(self, filename):
+        BzrNewError.__init__(self)
+        self.filename = filename
+
+
+class MustUseDecorated(Exception):
+    """A decorating function has requested its original command be used.
+    
+    This should never escape bzr, so does not need to be printable.
+    """
+
+
+class MissingText(BzrNewError):
+    """Branch %(base)s is missing revision %(text_revision)s of %(file_id)s"""
+
+    def __init__(self, branch, text_revision, file_id):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.base = branch.base
+        self.text_revision = text_revision
+        self.file_id = file_id
+
+
+class DuplicateKey(BzrNewError):
+    """Key %(key)s is already present in map"""
+
+
+class MalformedTransform(BzrNewError):
+    """Tree transform is malformed %(conflicts)r"""
+
+
+class BzrBadParameter(BzrNewError):
+    """A bad parameter : %(param)s is not usable.
+    
+    This exception should never be thrown, but it is a base class for all
+    parameter-to-function errors.
+    """
+    def __init__(self, param):
+        BzrNewError.__init__(self)
+        self.param = param
+
+
+class BzrBadParameterNotUnicode(BzrBadParameter):
+    """Parameter %(param)s is neither unicode nor utf8."""
+
+
+class ReusingTransform(BzrNewError):
+    """Attempt to reuse a transform that has already been applied."""
+
+
+class CantMoveRoot(BzrNewError):
+    """Moving the root directory is not supported at this time"""
+
+
+class BzrBadParameterNotString(BzrBadParameter):
+    """Parameter %(param)s is not a string or unicode string."""
+
+
+class BzrBadParameterMissing(BzrBadParameter):
+    """Parameter $(param)s is required but not present."""
+
+
+class DependencyNotPresent(BzrNewError):
+    """Unable to import library "%(library)s": %(error)s"""
+
+    def __init__(self, library, error):
+        BzrNewError.__init__(self, library=library, error=error)
+
+
+class ParamikoNotPresent(DependencyNotPresent):
+    """Unable to import paramiko (required for sftp support): %(error)s"""
+
+    def __init__(self, error):
+        DependencyNotPresent.__init__(self, 'paramiko', error)
+
+
+class UninitializableFormat(BzrNewError):
+    """Format %(format)s cannot be initialised by this version of bzr."""
+
+    def __init__(self, format):
+        BzrNewError.__init__(self)
+        self.format = format
+
+
+class NoDiff3(BzrNewError):
+    """Diff3 is not installed on this machine."""
+
+
+class ExistingLimbo(BzrNewError):
+    """This tree contains left-over files from a failed operation.
+    Please examine %(limbo_dir)s to see if it contains any files you wish to
+    keep, and delete it when you are done.
+    """
+    def __init__(self, limbo_dir):
+       BzrNewError.__init__(self)
+       self.limbo_dir = limbo_dir
+
+
+class ImmortalLimbo(BzrNewError):
+    """Unable to delete transform temporary directory $(limbo_dir)s.
+    Please examine %(limbo_dir)s to see if it contains any files you wish to
+    keep, and delete it when you are done.
+    """
+    def __init__(self, limbo_dir):
+       BzrNewError.__init__(self)
+       self.limbo_dir = limbo_dir
+
+
+class OutOfDateTree(BzrNewError):
+    """Working tree is out of date, please run 'bzr update'."""
+
+    def __init__(self, tree):
+        BzrNewError.__init__(self)
+        self.tree = tree
+
+
+class MergeModifiedFormatError(BzrNewError):
+    """Error in merge modified format"""
+
+
+class ConflictFormatError(BzrNewError):
+    """Format error in conflict listings"""
+
+
+class CorruptRepository(BzrNewError):
+    """An error has been detected in the repository %(repo_path)s.
+Please run bzr reconcile on this repository."""
+
+    def __init__(self, repo):
+        BzrNewError.__init__(self)
+        self.repo_path = repo.bzrdir.root_transport.base
+
+
+class UpgradeRequired(BzrNewError):
+    """To use this feature you must upgrade your branch at %(path)s."""
+
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
+
+
+class LocalRequiresBoundBranch(BzrNewError):
+    """Cannot perform local-only commits on unbound branches."""
+
+
+class MissingProgressBarFinish(BzrNewError):
+    """A nested progress bar was not 'finished' correctly."""
+
+
+class UnsupportedOperation(BzrNewError):
+    """The method %(mname)s is not supported on objects of type %(tname)s."""
+    def __init__(self, method, method_self):
+        self.method = method
+        self.mname = method.__name__
+        self.tname = type(method_self).__name__

=== added directory 'bzrlib/export'
=== added file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/__init__.py	2005-11-15 00:09:39 +0000
@@ -0,0 +1,132 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export functionality, which can take a Tree and create a different representation.
+
+Such as non-controlled directories, tarfiles, zipfiles, etc.
+"""
+
+from bzrlib.trace import mutter
+import os
+import bzrlib.errors as errors
+
+# Maps format name => export function
+_exporters = {}
+# Maps filename extensions => export format name
+_exporter_extensions = {}
+
+def register_exporter(format, extensions, func, override=False):
+    """Register an exporter.
+
+    :param format: This is the name of the format, such as 'tgz' or 'zip'
+    :param extensions: Extensions which should be used in the case that a 
+                       format was not explicitly specified.
+    :type extensions: List
+    :param func: The function. It will be called with (tree, dest, root)
+    :param override: Whether to override an object which already exists.
+                     Frequently plugins will want to provide functionality
+                     until it shows up in mainline, so the default is False.
+    """
+    global _exporters, _exporter_extensions
+
+    if not _exporters.has_key(format) or override:
+        _exporters[format] = func
+
+    for ext in extensions:
+        if not _exporter_extensions.has_key(ext) or override:
+            _exporter_extensions[ext] = format
+
+
+def register_lazy_exporter(scheme, extensions, module, funcname):
+    """Register lazy-loaded exporter function.
+
+    When requesting a specific type of export, load the respective path.
+    """
+    def _loader(tree, dest, root):
+        mod = __import__(module, globals(), locals(), [funcname])
+        func = getattr(mod, funcname)
+        return func(tree, dest, root)
+    register_exporter(scheme, extensions, _loader)
+
+
+def export(tree, dest, format=None, root=None):
+    """Export the given Tree to the specific destination.
+
+    :param tree: A Tree (such as RevisionTree) to export
+    :param dest: The destination where the files,etc should be put
+    :param format: The format (dir, zip, etc), if None, it will check the
+                   extension on dest, looking for a match
+    :param root: The root location inside the format.
+                 It is common practise to have zipfiles and tarballs 
+                 extract into a subdirectory, rather than into the
+                 current working directory.
+                 If root is None, the default root will be
+                 selected as the destination without its
+                 extension.
+    """
+    global _exporters, _exporter_extensions
+
+    if format is None:
+        for ext in _exporter_extensions:
+            if dest.endswith(ext):
+                format = _exporter_extensions[ext]
+                break
+
+    # Most of the exporters will just have to call
+    # this function anyway, so why not do it for them
+    if root is None:
+        root = get_root_name(dest)
+
+    if not _exporters.has_key(format):
+        raise errors.NoSuchExportFormat(format)
+    return _exporters[format](tree, dest, root)
+
+
+def get_root_name(dest):
+    """Get just the root name for an export.
+
+    >>> get_root_name('mytar.tar')
+    'mytar'
+    >>> get_root_name('mytar.tar.bz2')
+    'mytar'
+    >>> get_root_name('tar.tar.tar.tgz')
+    'tar.tar.tar'
+    >>> get_root_name('bzr-0.0.5.tar.gz')
+    'bzr-0.0.5'
+    >>> get_root_name('bzr-0.0.5.zip')
+    'bzr-0.0.5'
+    >>> get_root_name('bzr-0.0.5')
+    'bzr-0.0.5'
+    >>> get_root_name('a/long/path/mytar.tgz')
+    'mytar'
+    >>> get_root_name('../parent/../dir/other.tbz2')
+    'other'
+    """
+    global _exporter_extensions
+    dest = os.path.basename(dest)
+    for ext in _exporter_extensions:
+        if dest.endswith(ext):
+            return dest[:-len(ext)]
+    return dest
+
+
+register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter', 'dir_exporter')
+register_lazy_exporter('dir', [], 'bzrlib.export.dir_exporter', 'dir_exporter')
+register_lazy_exporter('tar', ['.tar'], 'bzrlib.export.tar_exporter', 'tar_exporter')
+register_lazy_exporter('tgz', ['.tar.gz', '.tgz'], 'bzrlib.export.tar_exporter', 'tgz_exporter')
+register_lazy_exporter('tbz2', ['.tar.bz2', '.tbz2'], 'bzrlib.export.tar_exporter', 'tbz_exporter')
+register_lazy_exporter('zip', ['.zip'], 'bzrlib.export.zip_exporter', 'zip_exporter')
+

=== added file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/dir_exporter.py	2005-12-29 03:26:31 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+
+def dir_exporter(tree, dest, root):
+    """Export this tree to a new directory.
+
+    `dest` should not exist, and will be created holding the
+    contents of this tree.
+
+    TODO: To handle subdirectories we need to create the
+           directories first.
+
+    :note: If the export fails, the destination directory will be
+           left in a half-assed state.
+    """
+    import os
+    os.mkdir(dest)
+    mutter('export version %r', tree)
+    inv = tree.inventory
+    for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
+        ie.put_on_disk(dest, dp, tree)
+

=== added file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/tar_exporter.py	2005-12-29 03:26:31 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+import tarfile
+
+
+def tar_exporter(tree, dest, root, compression=None):
+    """Export this tree to a new tar file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
+    from time import time
+    now = time()
+    compression = str(compression or '')
+    if root is None:
+        root = get_root_name(dest)
+    try:
+        ball = tarfile.open(dest, 'w:' + compression)
+    except tarfile.CompressionError, e:
+        raise BzrError(str(e))
+    mutter('export version %r', tree)
+    inv = tree.inventory
+    for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
+        mutter("  export {%s} kind %s to %s", ie.file_id, ie.kind, dest)
+        item, fileobj = ie.get_tar_item(root, dp, now, tree)
+        ball.addfile(item, fileobj)
+    ball.close()
+
+
+def tgz_exporter(tree, dest, root):
+    tar_exporter(tree, dest, root, compression='gz')
+
+
+def tbz_exporter(tree, dest, root):
+    tar_exporter(tree, dest, root, compression='bz2')
+

=== added file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/zip_exporter.py	2006-01-02 21:20:36 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+import zipfile
+
+def zip_exporter(tree, dest, root):
+    """ Export this tree to a new zip file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be overwritten".
+    """
+    import time
+
+    now = time.localtime()[:6]
+    mutter('export version %r', tree)
+
+    compression = zipfile.ZIP_DEFLATED
+    zipf = zipfile.ZipFile(dest, "w", compression)
+
+    inv = tree.inventory
+
+    try:
+        for dp, ie in inv.iter_entries():
+            # .bzrignore has no meaning outside of a working tree
+            # so do not export it
+            if dp == ".bzrignore":
+                continue
+
+            file_id = ie.file_id
+            mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)
+
+            if ie.kind == "file": 
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp)),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo, tree.get_file_text(file_id))
+            elif ie.kind == "directory":
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp)+os.sep),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo,'')
+            elif ie.kind == "symlink":
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp+".lnk")),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo, ie.symlink_target)
+
+        zipf.close()
+
+    except UnicodeEncodeError:
+        zipf.close()
+        os.remove(dest)
+        from bzrlib.errors import BzrError
+        raise BzrError("Can't export non-ascii filenames to zip")
+

=== added file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	1970-01-01 00:00:00 +0000
+++ bzrlib/externalcommand.py	2006-02-23 15:47:19 +0000
@@ -0,0 +1,66 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# TODO: Perhaps rather than mapping options and arguments back and
+# forth, we should just pass in the whole argv, and allow
+# ExternalCommands to handle it differently to internal commands?
+
+
+import os
+import sys
+
+from bzrlib.commands import Command
+from bzrlib.osutils import pathjoin
+
+
+class ExternalCommand(Command):
+    """Class to wrap external commands."""
+
+    @classmethod
+    def find_command(cls, cmd):
+        import os.path
+        bzrpath = os.environ.get('BZRPATH', '')
+
+        for dir in bzrpath.split(os.pathsep):
+            ## Empty directories are not real paths
+            if not dir:
+                continue
+            # This needs to be os.path.join() or windows cannot
+            # find the batch file that you are wanting to execute
+            path = os.path.join(dir, cmd)
+            if os.path.isfile(path):
+                return ExternalCommand(path)
+
+        return None
+
+
+    def __init__(self, path):
+        self.path = path
+
+    def name(self):
+        return os.path.basename(self.path)
+
+    def run(self, *args, **kwargs):
+        raise NotImplementedError('should not be called on %r' % self)
+
+    def run_argv(self, argv, alias_argv=None):
+        return os.spawnv(os.P_WAIT, self.path, [self.path] + argv)
+
+    def help(self):
+        m = 'external command from %s\n\n' % self.path
+        pipe = os.popen('%s --help' % self.path)
+        return m + pipe.read()
+

=== added file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/fetch.py	2006-03-20 09:58:25 +0000
@@ -0,0 +1,285 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Copying of history from one branch to another.
+
+The basic plan is that every branch knows the history of everything
+that has merged into it.  As the first step of a merge, pull, or
+branch operation we copy history from the source into the destination
+branch.
+
+The copying is done in a slightly complicated order.  We don't want to
+add a revision to the store until everything it refers to is also
+stored, so that if a revision is present we can totally recreate it.
+However, we can't know what files are included in a revision until we
+read its inventory.  So we query the inventory store of the source for
+the ids we need, and then pull those ids and finally actually join
+the inventories.
+"""
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import (InstallFailed, NoSuchRevision,
+                           MissingText)
+from bzrlib.trace import mutter
+from bzrlib.progress import ProgressBar, ProgressPhase
+from bzrlib.reconcile import RepoReconciler
+from bzrlib.revision import NULL_REVISION
+from bzrlib.symbol_versioning import *
+
+
+# TODO: Avoid repeatedly opening weaves so many times.
+
+# XXX: This doesn't handle ghost (not present in branch) revisions at
+# all yet.  I'm not sure they really should be supported.
+
+# NOTE: This doesn't copy revisions which may be present but not
+# merged into the last revision.  I'm not sure we want to do that.
+
+# - get a list of revisions that need to be pulled in
+# - for each one, pull in that revision file
+#   and get the inventory, and store the inventory with right
+#   parents.
+# - and get the ancestry, and store that with right parents too
+# - and keep a note of all file ids and version seen
+# - then go through all files; for each one get the weave,
+#   and add in all file versions
+
+
+@deprecated_function(zero_eight)
+def greedy_fetch(to_branch, from_branch, revision=None, pb=None):
+    """Legacy API, please see branch.fetch(from_branch, last_revision, pb)."""
+    f = Fetcher(to_branch, from_branch, revision, pb)
+    return f.count_copied, f.failed_revisions
+
+fetch = greedy_fetch
+
+
+class RepoFetcher(object):
+    """Pull revisions and texts from one repository to another.
+
+    last_revision
+        if set, try to limit to the data this revision references.
+
+    after running:
+    count_copied -- number of revisions copied
+
+    This should not be used directory, its essential a object to encapsulate
+    the logic in InterRepository.fetch().
+    """
+    def __init__(self, to_repository, from_repository, last_revision=None, pb=None):
+        # result variables.
+        self.failed_revisions = []
+        self.count_copied = 0
+        if to_repository.control_files._transport.base == from_repository.control_files._transport.base:
+            # check that last_revision is in 'from' and then return a no-operation.
+            if last_revision not in (None, NULL_REVISION):
+                from_repository.get_revision(last_revision)
+            return
+        self.to_repository = to_repository
+        self.from_repository = from_repository
+        # must not mutate self._last_revision as its potentially a shared instance
+        self._last_revision = last_revision
+        if pb is None:
+            self.pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            self.nested_pb = self.pb
+        else:
+            self.pb = pb
+            self.nested_pb = None
+        self.from_repository.lock_read()
+        try:
+            self.to_repository.lock_write()
+            try:
+                self.__fetch()
+            finally:
+                if self.nested_pb is not None:
+                    self.nested_pb.finished()
+                self.to_repository.unlock()
+        finally:
+            self.from_repository.unlock()
+
+    def __fetch(self):
+        """Primary worker function.
+
+        This initialises all the needed variables, and then fetches the 
+        requested revisions, finally clearing the progress bar.
+        """
+        self.to_weaves = self.to_repository.weave_store
+        self.to_control = self.to_repository.control_weaves
+        self.from_weaves = self.from_repository.weave_store
+        self.from_control = self.from_repository.control_weaves
+        self.count_total = 0
+        self.file_ids_names = {}
+        pp = ProgressPhase('fetch phase', 4, self.pb)
+        try:
+            revs = self._revids_to_fetch()
+            # something to do ?
+            if revs:
+                pp.next_phase()
+                self._fetch_weave_texts(revs)
+                pp.next_phase()
+                self._fetch_inventory_weave(revs)
+                pp.next_phase()
+                self._fetch_revision_texts(revs)
+                self.count_copied += len(revs)
+        finally:
+            self.pb.clear()
+
+    def _revids_to_fetch(self):
+        mutter('fetch up to rev {%s}', self._last_revision)
+        if self._last_revision is NULL_REVISION:
+            # explicit limit of no revisions needed
+            return None
+        if (self._last_revision != None and
+            self.to_repository.has_revision(self._last_revision)):
+            return None
+            
+        try:
+            return self.to_repository.missing_revision_ids(self.from_repository,
+                                                           self._last_revision)
+        except errors.NoSuchRevision:
+            raise InstallFailed([self._last_revision])
+
+    def _fetch_weave_texts(self, revs):
+        texts_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            file_ids = self.from_repository.fileid_involved_by_set(revs)
+            count = 0
+            num_file_ids = len(file_ids)
+            for file_id in file_ids:
+                texts_pb.update("fetch texts", count, num_file_ids)
+                count +=1
+                try:
+                    to_weave = self.to_weaves.get_weave(file_id,
+                        self.to_repository.get_transaction())
+                except errors.NoSuchFile:
+                    # destination is empty, just copy it.
+                    # this copies all the texts, which is useful and 
+                    # on per-file basis quite cheap.
+                    self.to_weaves.copy_multi(
+                        self.from_weaves,
+                        [file_id],
+                        None,
+                        self.from_repository.get_transaction(),
+                        self.to_repository.get_transaction())
+                else:
+                    # destination has contents, must merge
+                    from_weave = self.from_weaves.get_weave(file_id,
+                        self.from_repository.get_transaction())
+                    # we fetch all the texts, because texts do
+                    # not reference anything, and its cheap enough
+                    to_weave.join(from_weave)
+        finally:
+            texts_pb.finished()
+
+    def _fetch_inventory_weave(self, revs):
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            pb.update("fetch inventory", 0, 2)
+            to_weave = self.to_control.get_weave('inventory',
+                    self.to_repository.get_transaction())
+    
+            child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            try:
+                # just merge, this is optimisable and its means we dont
+                # copy unreferenced data such as not-needed inventories.
+                pb.update("fetch inventory", 1, 3)
+                from_weave = self.from_repository.get_inventory_weave()
+                pb.update("fetch inventory", 2, 3)
+                # we fetch only the referenced inventories because we do not
+                # know for unselected inventories whether all their required
+                # texts are present in the other repository - it could be
+                # corrupt.
+                to_weave.join(from_weave, pb=child_pb, msg='merge inventory',
+                              version_ids=revs)
+            finally:
+                child_pb.finished()
+        finally:
+            pb.finished()
+
+
+class GenericRepoFetcher(RepoFetcher):
+    """This is a generic repo to repo fetcher.
+
+    This makes minimal assumptions about repo layout and contents.
+    It triggers a reconciliation after fetching to ensure integrity.
+    """
+
+    def _fetch_revision_texts(self, revs):
+        """Fetch revision object texts"""
+        rev_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            to_txn = self.to_transaction = self.to_repository.get_transaction()
+            count = 0
+            total = len(revs)
+            to_store = self.to_repository._revision_store
+            for rev in revs:
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    pb.update('copying revisions', count, total)
+                    try:
+                        sig_text = self.from_repository.get_signature_text(rev)
+                        to_store.add_revision_signature_text(rev, sig_text, to_txn)
+                    except errors.NoSuchRevision:
+                        # not signed.
+                        pass
+                    to_store.add_revision(self.from_repository.get_revision(rev),
+                                          to_txn)
+                    count += 1
+                finally:
+                    pb.finished()
+            # fixup inventory if needed: 
+            # this is expensive because we have no inverse index to current ghosts.
+            # but on local disk its a few seconds and sftp push is already insane.
+            # so we just-do-it.
+            # FIXME: repository should inform if this is needed.
+            self.to_repository.reconcile()
+        finally:
+            rev_pb.finished()
+    
+
+class KnitRepoFetcher(RepoFetcher):
+    """This is a knit format repository specific fetcher.
+
+    This differs from the GenericRepoFetcher by not doing a 
+    reconciliation after copying, and using knit joining to
+    copy revision texts.
+    """
+
+    def _fetch_revision_texts(self, revs):
+        # may need to be a InterRevisionStore call here.
+        from_transaction = self.from_repository.get_transaction()
+        to_transaction = self.to_repository.get_transaction()
+        to_sf = self.to_repository._revision_store.get_signature_file(
+            to_transaction)
+        from_sf = self.from_repository._revision_store.get_signature_file(
+            from_transaction)
+        to_sf.join(from_sf, version_ids=revs, ignore_missing=True)
+        to_rf = self.to_repository._revision_store.get_revision_file(
+            to_transaction)
+        from_rf = self.from_repository._revision_store.get_revision_file(
+            from_transaction)
+        to_rf.join(from_rf, version_ids=revs)
+
+
+class Fetcher(object):
+    """Backwards compatability glue for branch.fetch()."""
+
+    @deprecated_method(zero_eight)
+    def __init__(self, to_branch, from_branch, last_revision=None, pb=None):
+        """Please see branch.fetch()."""
+        to_branch.fetch(from_branch, last_revision, pb)

=== added file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/gpg.py	2006-02-15 15:18:44 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""GPG signing and checking logic."""
+
+import errno
+import subprocess
+
+import bzrlib.errors as errors
+
+class DisabledGPGStrategy(object):
+    """A GPG Strategy that makes everything fail."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        raise errors.SigningFailed('Signing is disabled.')
+
+
+class LoopbackGPGStrategy(object):
+    """A GPG Strategy that acts like 'cat' - data is just passed through."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        return content
+
+
+class GPGStrategy(object):
+    """GPG Signing and checking facilities."""
+        
+    def _command_line(self):
+        return [self._config.gpg_signing_command(), '--clearsign']
+
+    def __init__(self, config):
+        self._config = config
+
+    def sign(self, content):
+        try:
+            process = subprocess.Popen(self._command_line(),
+                                       stdin=subprocess.PIPE,
+                                       stdout=subprocess.PIPE)
+            try:
+                result = process.communicate(content)[0]
+                if process.returncode is None:
+                    process.wait()
+                if process.returncode != 0:
+                    raise errors.SigningFailed(self._command_line())
+                return result
+            except OSError, e:
+                if e.errno == errno.EPIPE:
+                    raise errors.SigningFailed(self._command_line())
+                else:
+                    raise
+        except ValueError:
+            # bad subprocess parameters, should never happen.
+            raise
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # gpg is not installed
+                raise errors.SigningFailed(self._command_line())
+            else:
+                raise

=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2006-03-08 14:16:00 +0000
@@ -0,0 +1,162 @@
+# (C) 2005 Canonical
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.tsort import topo_sort
+
+
+def max_distance(node, ancestors, distances, root_descendants):
+    """Calculate the max distance to an ancestor.  
+    Return None if not all possible ancestors have known distances"""
+    best = None
+    if node in distances:
+        best = distances[node]
+    for ancestor in ancestors[node]:
+        # skip ancestors we will never traverse:
+        if root_descendants is not None and ancestor not in root_descendants:
+            continue
+        # An ancestor which is not listed in ancestors will never be in
+        # distances, so we pretend it never existed.
+        if ancestor not in ancestors:
+            continue
+        if ancestor not in distances:
+            return None
+        if best is None or distances[ancestor]+1 > best:
+            best = distances[ancestor] + 1
+    return best
+
+    
+def node_distances(graph, ancestors, start, root_descendants=None):
+    """Produce a list of nodes, sorted by distance from a start node.
+    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
+    backwards seemed too complicated.
+
+    For each node, we walk its descendants.  If all the descendant's ancestors
+    have a max-distance-to-start, (excluding ones that can never reach start),
+    we calculate their max-distance-to-start, and schedule their descendants.
+
+    So when a node's last parent acquires a distance, it will acquire a
+    distance on the next iteration.
+
+    Once we know the max distances for all nodes, we can return a list sorted
+    by distance, farthest first.
+    """
+    distances = {start: 0}
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            line_descendants = graph[line]
+            assert line not in line_descendants, "%s refers to itself" % line
+            for descendant in line_descendants:
+                distance = max_distance(descendant, ancestors, distances,
+                                        root_descendants)
+                if distance is None:
+                    continue
+                distances[descendant] = distance
+                new_lines.add(descendant)
+        lines = new_lines
+    return distances
+
+def nodes_by_distance(distances):
+    """Return a list of nodes sorted by distance"""
+    def by_distance(n):
+        return distances[n],n
+
+    node_list = distances.keys()
+    node_list.sort(key=by_distance, reverse=True)
+    return node_list
+
+def select_farthest(distances, common):
+    """Return the farthest common node, or None if no node qualifies."""
+    node_list = nodes_by_distance(distances)
+    for node in node_list:
+        if node in common:
+            return node
+
+def all_descendants(descendants, start):
+    """Produce a set of all descendants of the start node.
+    The input is a map of node->list of descendants for a graph encompassing
+    start.
+    """
+    result = set()
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line not in descendants:
+                continue
+            for descendant in descendants[line]:
+                if descendant in result:
+                    continue
+                result.add(descendant)
+                new_lines.add(descendant)
+        lines = new_lines
+    return result
+
+
+class Graph(object):
+    """A graph object which can memoise and cache results for performance."""
+
+    def __init__(self):
+        super(Graph, self).__init__()
+        self.roots = set([])
+        self.ghosts = set([])
+        self._graph_ancestors = {}
+        self._graph_descendants = {}
+
+    def add_ghost(self, node_id):
+        """Add a ghost to the graph."""
+        self.ghosts.add(node_id)
+        self._ensure_descendant(node_id)
+
+    def add_node(self, node_id, parent_ids):
+        """Add node_id to the graph with parent_ids as its parents."""
+        if parent_ids == []:
+            self.roots.add(node_id)
+        self._graph_ancestors[node_id] = list(parent_ids)
+        self._ensure_descendant(node_id)
+        for parent in parent_ids:
+            self._ensure_descendant(parent)
+            self._graph_descendants[parent][node_id] = 1
+        
+    def _ensure_descendant(self, node_id):
+        """Ensure that a descendant lookup for node_id will work."""
+        if not node_id in self._graph_descendants:
+            self._graph_descendants[node_id] = {}
+
+    def get_ancestors(self):
+        """Return a dictionary of graph node:ancestor_list entries."""
+        return dict(self._graph_ancestors.items())
+
+    def get_ancestry(self, node_id):
+        """Return the inclusive ancestors of node_id in topological order."""
+        # maybe optimise this ?
+        result = {}
+        pending = set([node_id])
+        while len(pending):
+            current = pending.pop()
+            parents = self._graph_ancestors[current]
+            parents = [parent for parent in parents if parent not in self.ghosts]
+            result[current] = parents
+            for parent in parents:
+                if parent not in result and parent not in pending:
+                    pending.add(parent)
+        return topo_sort(result.items())
+
+    def get_descendants(self):
+        """Return a dictionary of graph node:child_node:distance entries."""
+        return dict(self._graph_descendants.items())

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-07-22 18:49:46 +0000
+++ bzrlib/hashcache.py	2006-03-20 18:34:33 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,16 +23,24 @@
 # TODO: Perhaps return more details on the file to avoid statting it
 # again: nonexistent, file type, size, etc
 
+# TODO: Perhaps use a Python pickle instead of a text file; might be faster.
+
 
 
 CACHE_HEADER = "### bzr hashcache v5\n"
 
 import os, stat, time
+import sha
 
-from bzrlib.osutils import sha_file
+from bzrlib.osutils import sha_file, pathjoin, safe_unicode
 from bzrlib.trace import mutter, warning
-
-
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.errors import BzrError
+
+
+FP_MTIME_COLUMN = 1
+FP_CTIME_COLUMN = 2
+FP_MODE_COLUMN = 5
 
 def _fingerprint(abspath):
     try:
@@ -47,7 +55,7 @@
     # we discard any high precision because it's not reliable; perhaps we
     # could do better on some systems?
     return (fs.st_size, long(fs.st_mtime),
-            long(fs.st_ctime), fs.st_ino, fs.st_dev)
+            long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)
 
 
 class HashCache(object):
@@ -86,8 +94,9 @@
     """
     needs_write = False
 
-    def __init__(self, basedir):
-        self.basedir = basedir
+    def __init__(self, root, cache_file_name, mode=None):
+        """Create a hash cache in base dir, and set the file mode to mode."""
+        self.root = safe_unicode(root)
         self.hit_count = 0
         self.miss_count = 0
         self.stat_count = 0
@@ -95,13 +104,11 @@
         self.removed_count = 0
         self.update_count = 0
         self._cache = {}
-
+        self._mode = mode
+        self._cache_file_name = safe_unicode(cache_file_name)
 
     def cache_file_name(self):
-        return os.sep.join([self.basedir, '.bzr', 'stat-cache'])
-
-
-
+        return self._cache_file_name
 
     def clear(self):
         """Discard all cached information.
@@ -111,18 +118,19 @@
             self.needs_write = True
             self._cache = {}
 
-
     def scan(self):
         """Scan all files and remove entries where the cache entry is obsolete.
         
         Obsolete entries are those where the file has been modified or deleted
         since the entry was inserted.        
         """
+        # FIXME optimisation opportunity, on linux [and check other oses]:
+        # rather than iteritems order, stat in inode order.
         prep = [(ce[1][3], path, ce) for (path, ce) in self._cache.iteritems()]
         prep.sort()
         
         for inum, path, cache_entry in prep:
-            abspath = os.sep.join([self.basedir, path])
+            abspath = pathjoin(self.root, path)
             fp = _fingerprint(abspath)
             self.stat_count += 1
             
@@ -135,11 +143,10 @@
                 del self._cache[path]
 
 
-
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
-        abspath = os.sep.join([self.basedir, path])
+        abspath = pathjoin(self.root, path)
         self.stat_count += 1
         file_fp = _fingerprint(abspath)
         
@@ -161,13 +168,29 @@
             return cache_sha1
         
         self.miss_count += 1
-        digest = sha_file(file(abspath, 'rb', buffering=65000))
+
+
+        mode = file_fp[FP_MODE_COLUMN]
+        if stat.S_ISREG(mode):
+            digest = sha_file(file(abspath, 'rb', buffering=65000))
+        elif stat.S_ISLNK(mode):
+            digest = sha.new(os.readlink(abspath)).hexdigest()
+        else:
+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
         now = int(time.time())
-        if file_fp[1] >= now or file_fp[2] >= now:
+        if file_fp[FP_MTIME_COLUMN] >= now or file_fp[FP_CTIME_COLUMN] >= now:
             # changed too recently; can't be cached.  we can
             # return the result and it could possibly be cached
             # next time.
+            #
+            # the point is that we only want to cache when we are sure that any
+            # subsequent modifications of the file can be detected.  If a
+            # modification neither changes the inode, the device, the size, nor
+            # the mode, then we can only distinguish it by time; therefore we
+            # need to let sufficient time elapse before we may cache this entry
+            # again.  If we didn't do this, then, for example, a very quick 1
+            # byte replacement in the file might go undetected.
             self.danger_count += 1 
             if cache_fp:
                 self.removed_count += 1
@@ -177,17 +200,11 @@
             self.update_count += 1
             self.needs_write = True
             self._cache[path] = (digest, file_fp)
-        
         return digest
         
-
-
-
     def write(self):
         """Write contents of cache to file."""
-        from atomicfile import AtomicFile
-
-        outf = AtomicFile(self.cache_file_name(), 'wb')
+        outf = AtomicFile(self.cache_file_name(), 'wb', new_mode=self._mode)
         try:
             print >>outf, CACHE_HEADER,
 
@@ -199,14 +216,15 @@
                 for fld in c[1]:
                     print >>outf, "%d" % fld,
                 print >>outf
-
             outf.commit()
             self.needs_write = False
+            mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
+                   self.cache_file_name(), self.hit_count, self.miss_count,
+                   self.stat_count,
+                   self.danger_count, self.update_count)
         finally:
             if not outf.closed:
                 outf.abort()
-        
-
 
     def read(self):
         """Reinstate cache from file.
@@ -221,14 +239,17 @@
         try:
             inf = file(fn, 'rb', buffering=65000)
         except IOError, e:
-            mutter("failed to open %s: %s" % (fn, e))
+            mutter("failed to open %s: %s", fn, e)
+            # better write it now so it is valid
+            self.needs_write = True
             return
 
 
         hdr = inf.readline()
         if hdr != CACHE_HEADER:
-            mutter('cache header marker not found at top of %s; discarding cache'
-                   % fn)
+            mutter('cache header marker not found at top of %s;'
+                   ' discarding cache', fn)
+            self.needs_write = True
             return
 
         for l in inf:
@@ -240,7 +261,7 @@
 
             pos += 3
             fields = l[pos:].split(' ')
-            if len(fields) != 6:
+            if len(fields) != 7:
                 warning("bad line in hashcache: %r" % l)
                 continue
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-08-03 14:16:04 +0000
+++ bzrlib/help.py	2006-02-10 00:39:25 +0000
@@ -14,6 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some way to get a list of external commands (defined by shell
+# scripts) so that they can be included in the help listing as well.
+# It should be enough to just list the plugin directory and look for
+# executable files with reasonable names.
+
+# TODO: `help commands --all` should show hidden commands
+import textwrap
+
 global_help = \
 """Bazaar-NG -- a free distributed version-control tool
 http://bazaar-ng.org/
@@ -23,24 +31,24 @@
 
 Basic commands:
 
-  bzr init      makes this branch versioned
-  bzr branch    make a copy of another branch
-
-  bzr add       make files or directories versioned
-  bzr ignore    ignore a file or pattern
-  bzr mv        move or rename a versioned file
-
-  bzr status    summarize changes in working copy
-  bzr diff      show detailed diffs
-
-  bzr merge     pull in changes from another branch
-  bzr commit    
-
-  bzr log       show history of changes
-  bzr check     validate storage
-
-Use e.g. 'bzr help init' for more details, or 'bzr help commands' for
-all commands.
+  bzr init           makes this directory a versioned branch
+  bzr branch         make a copy of another branch
+
+  bzr add            make files or directories versioned
+  bzr ignore         ignore a file or pattern
+  bzr mv             move or rename a versioned file
+
+  bzr status         summarize changes in working copy
+  bzr diff           show detailed diffs
+
+  bzr merge          pull in changes from another branch
+  bzr commit         save some or all changes
+
+  bzr log            show history of changes
+  bzr check          validate storage
+
+  bzr help init      more help on e.g. init command
+  bzr help commands  list all commands
 """
 
 
@@ -58,13 +66,13 @@
         help_on_command(topic, outfile = outfile)
 
 
-def command_usage(cmdname, cmdclass):
+def command_usage(cmd_object):
     """Return single-line grammar for command.
 
     Only describes arguments, not options.
     """
-    s = cmdname + ' '
-    for aname in cmdclass.takes_args:
+    s = 'bzr ' + cmd_object.name() + ' '
+    for aname in cmd_object.takes_args:
         aname = aname.upper()
         if aname[-1] in ['$', '+']:
             aname = aname[:-1] + '...'
@@ -80,70 +88,79 @@
     return s
 
 
-def help_on_command(cmdname, outfile = None):
+def help_on_command(cmdname, outfile=None):
+    from bzrlib.commands import get_cmd_object
+
     cmdname = str(cmdname)
 
     if outfile == None:
         outfile = sys.stdout
 
-    from inspect import getdoc
-    import commands
-    topic, cmdclass = commands.get_cmd_class(cmdname)
+    cmd_object = get_cmd_object(cmdname)
 
-    doc = getdoc(cmdclass)
+    doc = cmd_object.help()
     if doc == None:
         raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
 
-    outfile.write('usage: ' + command_usage(topic, cmdclass) + '\n')
-
-    if cmdclass.aliases:
-        outfile.write('aliases: ' + ', '.join(cmdclass.aliases) + '\n')
-    
+    print >>outfile, 'usage:', command_usage(cmd_object) 
+
+    if cmd_object.aliases:
+        print >>outfile, 'aliases:',
+        print >>outfile, ', '.join(cmd_object.aliases)
+
+    print >>outfile
+
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
-    
-    help_on_option(cmdclass.takes_options, outfile = None)
-
-
-def help_on_option(options, outfile = None):
-    import commands
-    
+    help_on_command_options(cmd_object, outfile)
+
+
+def help_on_command_options(cmd, outfile=None):
+    from bzrlib.option import Option
+    options = cmd.options()
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     outfile.write('\noptions:\n')
-    for on in options:
-        l = '    --' + on
-        for shortname, longname in commands.SHORT_OPTIONS.items():
-            if longname == on:
-                l += ', -' + shortname
-                break
-        outfile.write(l + '\n')
-
-
-def help_commands(outfile = None):
+    for option_name, option in sorted(options.items()):
+        l = '    --' + option_name
+        if option.type is not None:
+            l += ' ' + option.argname.upper()
+        short_name = option.short_name()
+        if short_name:
+            assert len(short_name) == 1
+            l += ', -' + short_name
+        l += (30 - len(l)) * ' ' + option.help
+        # TODO: split help over multiple lines with correct indenting and 
+        # wrapping
+        wrapped = textwrap.fill(l, initial_indent='', subsequent_indent=30*' ')
+        outfile.write(wrapped + '\n')
+
+
+def help_commands(outfile=None):
     """List all commands"""
-    import inspect
-    import commands
+    from bzrlib.commands import (builtin_command_names,
+                                 plugin_command_names,
+                                 get_cmd_object)
 
     if outfile == None:
         outfile = sys.stdout
-    
-    accu = []
-    for cmdname, cmdclass in commands.get_all_cmds():
-        accu.append((cmdname, cmdclass))
-    accu.sort()
-    for cmdname, cmdclass in accu:
-        if cmdclass.hidden:
+
+    names = set()                       # to eliminate duplicates
+    names.update(builtin_command_names())
+    names.update(plugin_command_names())
+    names = list(names)
+    names.sort()
+
+    for cmd_name in names:
+        cmd_object = get_cmd_object(cmd_name)
+        if cmd_object.hidden:
             continue
-        outfile.write(command_usage(cmdname, cmdclass) + '\n')
-        help = inspect.getdoc(cmdclass)
-        if help:
-            outfile.write("    " + help.split('\n', 1)[0] + '\n')
-
-            
-
+        print >>outfile, command_usage(cmd_object)
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            print >>outfile, '        ' + firstline
+        

=== added file 'bzrlib/identitymap.py'
--- bzrlib/identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/identitymap.py	2006-03-06 06:06:07 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides an IdentityMap."""
+
+
+import bzrlib.errors as errors
+
+
+class IdentityMap(object):
+    """An in memory map from object id to instance.
+    
+    An IdentityMap maps from keys to single instances of objects in memory.
+    We have explicit calls on the map for the root of each inheritance tree
+    that is store in the map. Look for find_CLASS and add_CLASS methods.
+    """
+
+    def add_revision_history(self, revision_history):
+        """Add a revision_history object to the map.
+
+        There can only be one!
+        """
+        if self._revision_history is not None:
+            raise errors.BzrError("A revision history (%s) is already "
+                                  "identity map" % self._revision_history)
+        self._revision_history = revision_history
+
+    def add_weave(self, id, weave):
+        """Add weave to the map with a given id."""
+        if self._weave_key(id) in self._map:
+            raise errors.BzrError('weave %s already in the identity map' % id)
+        self._map[self._weave_key(id)] = weave
+        self._reverse_map[weave] = self._weave_key(id)
+
+    def find_revision_history(self):
+        return self._revision_history
+
+    def find_weave(self, id):
+        """Return the weave for 'id', or None if it is not present."""
+        return self._map.get(self._weave_key(id), None)
+
+    def __init__(self):
+        super(IdentityMap, self).__init__()
+        self._map = {}
+        self._reverse_map = {}
+        self._revision_history = None
+
+    def remove_object(self, an_object):
+        """Remove object from map."""
+        if isinstance(an_object, list):
+            if self._revision_history is an_object:
+                self._revision_history = None
+            else:
+                raise KeyError('%r not in identity map' % an_object)
+        else:
+            self._map.pop(self._reverse_map[an_object])
+            self._reverse_map.pop(an_object)
+
+    def _weave_key(self, id):
+        """Return the key for a weaves id."""
+        return "weave-" + id
+
+        
+class NullIdentityMap(object):
+    """A pretend in memory map from object id to instance.
+    
+    A NullIdentityMap is an Identity map that does not store anything in it.
+    """
+
+    def add_weave(self, id, weave):
+        """See IdentityMap.add_weave."""
+
+    def add_revision_history(self, revision_history):
+        """See IdentityMap.add_revision_history."""
+
+    def find_weave(self, id):
+        """See IdentityMap.find_weave."""
+        return None
+
+    def find_revision_history(self):
+        """See IdentityMap.find_revision_history."""

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2005-05-19 08:31:06 +0000
+++ bzrlib/info.py	2006-04-11 01:28:20 +0000
@@ -16,9 +16,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+__all__ = ['show_bzrdir_info']
+
 import time
 
-from osutils import format_date
+
+import bzrlib.diff as diff
+from bzrlib.errors import (NoWorkingTree, NotBranchError,
+                           NoRepositoryPresent, NotLocalUrl)
+from bzrlib.missing import find_unmerged
+from bzrlib.osutils import format_date
+from bzrlib.symbol_versioning import *
 
 
 def _countiter(it):
@@ -29,29 +37,97 @@
     return i        
 
 
-
-def show_info(b):
-    import diff
-    
-    print 'branch format:', b.controlfile('branch-format', 'r').readline().rstrip('\n')
-
-    def plural(n, base='', pl=None):
-        if n == 1:
-            return base
-        elif pl != None:
-            return pl
+def plural(n, base='', pl=None):
+    if n == 1:
+        return base
+    elif pl != None:
+        return pl
+    else:
+        return 's'
+
+
+def _show_location_info(repository=None, branch=None, working=None):
+    """Show known locations for working, branch and repository."""
+    print 'Location:'
+    if working and branch and working.bzrdir != branch.bzrdir:
+        # Lightweight checkout
+        print '       checkout root: %s' % (
+            working.bzrdir.root_transport.base)
+        print '  checkout of branch: %s' % (
+            branch.bzrdir.root_transport.base)
+    elif branch:
+        # Standalone or bound branch (normal checkout)
+        print '         branch root: %s' % (
+            branch.bzrdir.root_transport.base)
+        if branch.get_bound_location():
+            print '     bound to branch: %s' % branch.get_bound_location()
+
+    if repository and (not branch or repository.bzrdir != branch.bzrdir):
+        if repository.is_shared():
+            print '   shared repository: %s' % (
+                repository.bzrdir.root_transport.base)
         else:
-            return 's'
-
-    count_version_dirs = 0
-
-    basis = b.basis_tree()
-    working = b.working_tree()
-    work_inv = working.inventory
-    delta = diff.compare_trees(basis, working, want_unchanged=True)
-    
-    print
-    print 'in the working tree:'
+            print '          repository: %s' % (
+                repository.bzrdir.root_transport.base)
+
+    if branch:
+        if branch.get_parent():
+            print '       parent branch: %s' % branch.get_parent()
+        if branch.get_push_location():
+            print '      push to branch: %s' % branch.get_push_location()
+
+
+def _show_format_info(control=None, repository=None, branch=None, working=None):
+    """Show known formats for control, working, branch and repository."""
+    print
+    print 'Format:'
+    if control:
+        print '       control: %s' % control._format.get_format_description()
+    if working:
+        print '  working tree: %s' % working._format.get_format_description()
+    if branch:
+        print '        branch: %s' % branch._format.get_format_description()
+    if repository:
+        print '    repository: %s' % repository._format.get_format_description()
+
+
+def _show_missing_revisions_branch(branch):
+    """Show missing master revisions in branch."""
+    # Try with inaccessible branch ?
+    master = branch.get_master_branch()
+    if master:
+        local_extra, remote_extra = find_unmerged(branch, master)
+        if remote_extra:
+            print
+            print 'Branch is out of date: missing %d revision%s.' % (
+                len(remote_extra), plural(len(remote_extra)))
+
+
+def _show_missing_revisions_working(working):
+    """Show missing revisions in working tree."""
+    branch = working.branch
+    basis = working.basis_tree()
+    work_inv = working.inventory
+    delta = diff.compare_trees(basis, working, want_unchanged=True)
+    history = branch.revision_history()
+    tree_last_id = working.last_revision()
+
+    if len(history) and tree_last_id != history[-1]:
+        tree_last_revno = branch.revision_id_to_revno(tree_last_id)
+        missing_count = len(history) - tree_last_revno
+        print
+        print 'Working tree is out of date: missing %d revision%s.' % (
+            missing_count, plural(missing_count))
+
+
+def _show_working_stats(working):
+    """Show statistics about a working tree."""
+    basis = working.basis_tree()
+    work_inv = working.inventory
+    delta = diff.compare_trees(basis, working, want_unchanged=True)
+
+    print
+    print 'In the working tree:'
     print '  %8s unchanged' % len(delta.unchanged)
     print '  %8d modified' % len(delta.modified)
     print '  %8d added' % len(delta.added)
@@ -64,7 +140,6 @@
             ignore_cnt += 1
         else:
             unknown_cnt += 1
-
     print '  %8d unknown' % unknown_cnt
     print '  %8d ignored' % ignore_cnt
 
@@ -76,42 +151,139 @@
           % (dir_cnt,
              plural(dir_cnt, 'subdirectory', 'subdirectories'))
 
+
+def _show_branch_stats(branch, verbose):
+    """Show statistics about a branch."""
+    repository = branch.repository
+    history = branch.revision_history()
+
     print
-    print 'branch history:'
-    history = b.revision_history()
+    print 'Branch history:'
     revno = len(history)
     print '  %8d revision%s' % (revno, plural(revno))
-    committers = {}
-    for rev in history:
-        committers[b.get_revision(rev).committer] = True
-    print '  %8d committer%s' % (len(committers), plural(len(committers)))
+    if verbose:
+        committers = {}
+        for rev in history:
+            committers[repository.get_revision(rev).committer] = True
+        print '  %8d committer%s' % (len(committers), plural(len(committers)))
     if revno > 0:
-        firstrev = b.get_revision(history[0])
+        firstrev = repository.get_revision(history[0])
         age = int((time.time() - firstrev.timestamp) / 3600 / 24)
         print '  %8d day%s old' % (age, plural(age))
         print '   first revision: %s' % format_date(firstrev.timestamp,
                                                     firstrev.timezone)
 
-        lastrev = b.get_revision(history[-1])
+        lastrev = repository.get_revision(history[-1])
         print '  latest revision: %s' % format_date(lastrev.timestamp,
                                                     lastrev.timezone)
 
-    print
-    print 'text store:'
-    c, t = b.text_store.total_size()
-    print '  %8d file texts' % c
-    print '  %8d kB' % (t/1024)
-
-    print
-    print 'revision store:'
-    c, t = b.revision_store.total_size()
-    print '  %8d revisions' % c
-    print '  %8d kB' % (t/1024)
-
-
-    print
-    print 'inventory store:'
-    c, t = b.inventory_store.total_size()
-    print '  %8d inventories' % c
-    print '  %8d kB' % (t/1024)
-
+#     print
+#     print 'Text store:'
+#     c, t = branch.text_store.total_size()
+#     print '  %8d file texts' % c
+#     print '  %8d KiB' % (t/1024)
+
+#     print
+#     print 'Inventory store:'
+#     c, t = branch.inventory_store.total_size()
+#     print '  %8d inventories' % c
+#     print '  %8d KiB' % (t/1024)
+
+
+def _show_repository_info(repository):
+    """Show settings of a repository."""
+    if repository.make_working_trees():
+        print
+        print 'Create working tree for new branches inside the repository.'
+
+
+def _show_repository_stats(repository):
+    """Show statistics about a repository."""
+    if repository.bzrdir.root_transport.listable():
+        print
+        print 'Revision store:'
+        c, t = repository._revision_store.total_size(repository.get_transaction())
+        print '  %8d revision%s' % (c, plural(c))
+        print '  %8d KiB' % (t/1024)
+
+
+@deprecated_function(zero_eight)
+def show_info(b):
+    """Please see show_bzrdir_info."""
+    return show_bzrdir_info(b.bzrdir)
+
+
+def show_bzrdir_info(a_bzrdir, verbose=False):
+    """Output to stdout the 'info' for a_bzrdir."""
+    try:
+        working = a_bzrdir.open_workingtree()
+        working.lock_read()
+        try:
+            show_tree_info(working, verbose)
+        finally:
+            working.unlock()
+        return
+    except (NoWorkingTree, NotLocalUrl):
+        pass
+
+    try:
+        branch = a_bzrdir.open_branch()
+        branch.lock_read()
+        try:
+            show_branch_info(branch, verbose)
+        finally:
+            branch.unlock()
+        return
+    except NotBranchError:
+        pass
+
+    try:
+        repository = a_bzrdir.open_repository()
+        repository.lock_read()
+        try:
+            show_repository_info(repository, verbose)
+        finally:
+            repository.unlock()
+        return
+    except NoRepositoryPresent:
+        pass
+
+    # Return silently, cmd_info returns NotBranchError if no bzrdir
+    # could be opened.
+
+
+def show_tree_info(working, verbose):
+    """Output to stdout the 'info' for working."""
+    branch = working.branch
+    repository = branch.repository
+    control = working.bzrdir
+
+    _show_location_info(repository, branch, working)
+    _show_format_info(control, repository, branch, working)
+    _show_missing_revisions_branch(branch)
+    _show_missing_revisions_working(working)
+    _show_working_stats(working)
+    _show_branch_stats(branch, verbose)
+    _show_repository_stats(repository)
+
+
+def show_branch_info(branch, verbose):
+    """Output to stdout the 'info' for branch."""
+    repository = branch.repository
+    control = branch.bzrdir
+
+    _show_location_info(repository, branch)
+    _show_format_info(control, repository, branch)
+    _show_missing_revisions_branch(branch)
+    _show_branch_stats(branch, verbose)
+    _show_repository_stats(repository)
+
+
+def show_repository_info(repository, verbose):
+    """Output to stdout the 'info' for branch."""
+    control = repository.bzrdir
+
+    _show_location_info(repository)
+    _show_format_info(control, repository)
+    _show_repository_info(repository)
+    _show_repository_stats(repository)

=== added file 'bzrlib/inter.py'
--- bzrlib/inter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inter.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Inter-object utility class."""
+
+
+class InterObject(object):
+    """This class represents operations taking place between two objects.
+
+    Its instances have methods like join or copy_content or fetch, and contain
+    references to the source and target objects these operations can be 
+    carried out between.
+
+    Often we will provide convenience methods on the objects which carry out
+    operations with another of similar type - they will always forward to
+    a subclass of InterObject - i.e. 
+    InterVersionedFile.get(other).method_name(parameters).
+    """
+
+    # _optimisers = set()
+    # Each concrete InterObject type should have its own optimisers set.
+
+    def __init__(self, source, target):
+        """Construct a default InterObject instance. Please use 'get'.
+        
+        Only subclasses of InterObject should call 
+        InterObject.__init__ - clients should call InterFOO.get where FOO
+        is the base type of the objects they are interacting between. I.e.
+        InterVersionedFile or InterRepository.
+        get() is a convenience class method which will create an optimised
+        InterFOO if possible.
+        """
+        self.source = source
+        self.target = target
+
+    @classmethod
+    def get(klass, source, target):
+        """Retrieve a Inter worker object for these objects.
+
+        :param source: the object to be the 'source' member of
+                       the InterObject instance.
+        :param target: the object to be the 'target' member of
+                       the InterObject instance.
+        If an optimised worker exists it will be used otherwise
+        a default Inter worker instance will be created.
+        """
+        for provider in klass._optimisers:
+            if provider.is_compatible(source, target):
+                return provider(source, target)
+        return klass(source, target)
+
+    @classmethod
+    def register_optimiser(klass, optimiser):
+        """Register an InterObject optimiser."""
+        klass._optimisers.add(optimiser)
+
+    @classmethod
+    def unregister_optimiser(klass, optimiser):
+        """Unregister an InterObject optimiser."""
+        klass._optimisers.remove(optimiser)

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2005-07-17 18:26:45 +0000
+++ bzrlib/intset.py	2005-12-19 17:48:25 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/python
-
 # Copyright (C) 2005 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
@@ -210,12 +208,22 @@
         if not self._val & m:
             raise KeyError(to_remove)
         self._val ^= m
-        
-        
-            
-    
-
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
-    
+
+    def set_remove(self, to_remove):
+        """Remove all values that exist in to_remove.
+
+        >>> a = IntSet(range(10))
+        >>> b = IntSet([2,3,4,7,12])
+        >>> a.set_remove(b)
+        >>> list(a)
+        [0, 1, 5, 6, 8, 9]
+        >>> a.set_remove([1,2,5])
+        >>> list(a)
+        [0, 6, 8, 9]
+        """
+        if not isinstance(to_remove, IntSet):
+            self.set_remove(IntSet(to_remove))
+            return
+        intersect = self._val & to_remove._val
+        self._val ^= intersect
+

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-08-18 05:52:29 +0000
+++ bzrlib/inventory.py	2006-04-06 10:21:12 +0000
@@ -1,34 +1,46 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# FIXME: This refactoring of the workingtree code doesn't seem to keep 
+# the WorkingTree's copy of the inventory in sync with the branch.  The
+# branch modifies its working inventory when it does a commit to make
+# missing files permanently removed.
+
+# TODO: Maybe also keep the full path of the entry, and the children?
+# But those depend on its position within a particular inventory, and
+# it would be nice not to need to hold the backpointer here.
 
 # This should really be an id randomly assigned when the tree is
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
 
-import sys, os.path, types, re
+import os.path
+import re
+import sys
+import tarfile
+import types
 
 import bzrlib
-from bzrlib.errors import BzrError, BzrCheckError
-
-from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
+from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
+                            pathjoin, sha_strings)
 from bzrlib.trace import mutter
-from bzrlib.errors import NotVersionedError
-        
+from bzrlib.errors import (NotVersionedError, InvalidEntryName,
+                           BzrError, BzrCheckError)
+
 
 class InventoryEntry(object):
     """Description of a versioned file.
@@ -36,47 +48,62 @@
     An InventoryEntry has the following fields, which are also
     present in the XML inventory-entry element:
 
-    * *file_id*
-    * *name*: (only the basename within the directory, must not
-      contain slashes)
-    * *kind*: "directory" or "file"
-    * *directory_id*: (if absent/null means the branch root directory)
-    * *text_sha1*: only for files
-    * *text_size*: in bytes, only for files 
-    * *text_id*: identifier for the text version, only for files
-
-    InventoryEntries can also exist inside a WorkingTree
-    inventory, in which case they are not yet bound to a
-    particular revision of the file.  In that case the text_sha1,
-    text_size and text_id are absent.
-
+    file_id
+
+    name
+        (within the parent directory)
+
+    parent_id
+        file_id of the parent directory, or ROOT_ID
+
+    revision
+        the revision_id in which this variation of this file was 
+        introduced.
+
+    executable
+        Indicates that this file should be executable on systems
+        that support it.
+
+    text_sha1
+        sha-1 of the text of the file
+        
+    text_size
+        size in bytes of the text of the file
+        
+    (reading a version 4 tree created a text_id field.)
 
     >>> i = Inventory()
     >>> i.path2id('')
     'TREE_ROOT'
-    >>> i.add(InventoryEntry('123', 'src', 'directory', ROOT_ID))
-    >>> i.add(InventoryEntry('2323', 'hello.c', 'file', parent_id='123'))
-    >>> for j in i.iter_entries():
-    ...   print j
+    >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
+    InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
+    >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
+    InventoryFile('2323', 'hello.c', parent_id='123')
+    >>> shouldbe = {0: 'src', 1: pathjoin('src','hello.c')}
+    >>> for ix, j in enumerate(i.iter_entries()):
+    ...   print (j[0] == shouldbe[ix], j[1])
     ... 
-    ('src', InventoryEntry('123', 'src', kind='directory', parent_id='TREE_ROOT'))
-    ('src/hello.c', InventoryEntry('2323', 'hello.c', kind='file', parent_id='123'))
-    >>> i.add(InventoryEntry('2323', 'bye.c', 'file', '123'))
+    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
+    (True, InventoryFile('2323', 'hello.c', parent_id='123'))
+    >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
-    >>> i.add(InventoryEntry('2324', 'bye.c', 'file', '123'))
-    >>> i.add(InventoryEntry('2325', 'wibble', 'directory', '123'))
+    >>> i.add(InventoryFile('2324', 'bye.c', '123'))
+    InventoryFile('2324', 'bye.c', parent_id='123')
+    >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
+    InventoryDirectory('2325', 'wibble', parent_id='123')
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
-    >>> i.add(InventoryEntry('2326', 'wibble.c', 'file', '2325'))
+    >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> i['2326']
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
-    >>> for j in i.iter_entries():
-    ...     print j[0]
-    ...     assert i.path2id(j[0])
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    >>> for path, entry in i.iter_entries():
+    ...     print path
+    ...     assert i.path2id(path)
     ... 
     src
     src/bye.c
@@ -85,144 +112,315 @@
     src/wibble/wibble.c
     >>> i.id2path('2326')
     'src/wibble/wibble.c'
-
-    TODO: Maybe also keep the full path of the entry, and the children?
-           But those depend on its position within a particular inventory, and
-           it would be nice not to need to hold the backpointer here.
     """
-
-    # TODO: split InventoryEntry into subclasses for files,
-    # directories, etc etc.
-
+    
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', ]
-
-    def __init__(self, file_id, name, kind, parent_id, text_id=None):
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision']
+
+    def _add_text_to_weave(self, new_lines, parents, weave_store, transaction):
+        versionedfile = weave_store.get_weave_or_empty(self.file_id,
+                                                       transaction)
+        versionedfile.add_lines(self.revision, parents, new_lines)
+
+    def detect_changes(self, old_entry):
+        """Return a (text_modified, meta_modified) from this to old_entry.
+        
+        _read_tree_state must have been called on self and old_entry prior to 
+        calling detect_changes.
+        """
+        return False, False
+
+    def diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff from this to to_entry.
+
+        text_diff will be used for textual difference calculation.
+        This is a template method, override _diff in child classes.
+        """
+        self._read_tree_state(tree.id2path(self.file_id), tree)
+        if to_entry:
+            # cannot diff from one kind to another - you must do a removal
+            # and an addif they do not match.
+            assert self.kind == to_entry.kind
+            to_entry._read_tree_state(to_tree.id2path(to_entry.file_id),
+                                      to_tree)
+        self._diff(text_diff, from_label, tree, to_label, to_entry, to_tree,
+                   output_to, reverse)
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff between two entries of the same kind."""
+
+    def find_previous_heads(self, previous_inventories,
+                            versioned_file_store,
+                            transaction,
+                            entry_vf=None):
+        """Return the revisions and entries that directly preceed this.
+
+        Returned as a map from revision to inventory entry.
+
+        This is a map containing the file revisions in all parents
+        for which the file exists, and its revision is not a parent of
+        any other. If the file is new, the set will be empty.
+
+        :param versioned_file_store: A store where ancestry data on this
+                                     file id can be queried.
+        :param transaction: The transaction that queries to the versioned 
+                            file store should be completed under.
+        :param entry_vf: The entry versioned file, if its already available.
+        """
+        def get_ancestors(weave, entry):
+            return set(weave.get_ancestry(entry.revision))
+        # revision:ie mapping for each ie found in previous_inventories.
+        candidates = {}
+        # revision:ie mapping with one revision for each head.
+        heads = {}
+        # revision: ancestor list for each head
+        head_ancestors = {}
+        # identify candidate head revision ids.
+        for inv in previous_inventories:
+            if self.file_id in inv:
+                ie = inv[self.file_id]
+                assert ie.file_id == self.file_id
+                if ie.revision in candidates:
+                    # same revision value in two different inventories:
+                    # correct possible inconsistencies:
+                    #     * there was a bug in revision updates with 'x' bit 
+                    #       support.
+                    try:
+                        if candidates[ie.revision].executable != ie.executable:
+                            candidates[ie.revision].executable = False
+                            ie.executable = False
+                    except AttributeError:
+                        pass
+                    # must now be the same.
+                    assert candidates[ie.revision] == ie
+                else:
+                    # add this revision as a candidate.
+                    candidates[ie.revision] = ie
+
+        # common case optimisation
+        if len(candidates) == 1:
+            # if there is only one candidate revision found
+            # then we can opening the versioned file to access ancestry:
+            # there cannot be any ancestors to eliminate when there is 
+            # only one revision available.
+            heads[ie.revision] = ie
+            return heads
+
+        # eliminate ancestors amongst the available candidates:
+        # heads are those that are not an ancestor of any other candidate
+        # - this provides convergence at a per-file level.
+        for ie in candidates.values():
+            # may be an ancestor of a known head:
+            already_present = 0 != len(
+                [head for head in heads 
+                 if ie.revision in head_ancestors[head]])
+            if already_present:
+                # an ancestor of an analyzed candidate.
+                continue
+            # not an ancestor of a known head:
+            # load the versioned file for this file id if needed
+            if entry_vf is None:
+                entry_vf = versioned_file_store.get_weave_or_empty(
+                    self.file_id, transaction)
+            ancestors = get_ancestors(entry_vf, ie)
+            # may knock something else out:
+            check_heads = list(heads.keys())
+            for head in check_heads:
+                if head in ancestors:
+                    # this previously discovered 'head' is not
+                    # really a head - its an ancestor of the newly 
+                    # found head,
+                    heads.pop(head)
+            head_ancestors[ie.revision] = ancestors
+            heads[ie.revision] = ie
+        return heads
+
+    def get_tar_item(self, root, dp, now, tree):
+        """Get a tarfile item and a file stream for its content."""
+        item = tarfile.TarInfo(pathjoin(root, dp))
+        # TODO: would be cool to actually set it to the timestamp of the
+        # revision it was last changed
+        item.mtime = now
+        fileobj = self._put_in_tar(item, tree)
+        return item, fileobj
+
+    def has_text(self):
+        """Return true if the object this entry represents has textual data.
+
+        Note that textual data includes binary content.
+
+        Also note that all entries get weave files created for them.
+        This attribute is primarily used when upgrading from old trees that
+        did not have the weave index for all inventory entries.
+        """
+        return False
+
+    def __init__(self, file_id, name, parent_id, text_id=None):
         """Create an InventoryEntry
         
         The filename must be a single component, relative to the
         parent directory; it cannot be a whole path or relative name.
 
-        >>> e = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'hello.c', ROOT_ID)
         >>> e.name
         'hello.c'
         >>> e.file_id
         '123'
-        >>> e = InventoryEntry('123', 'src/hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'src/hello.c', ROOT_ID)
         Traceback (most recent call last):
-        BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
+        InvalidEntryName: Invalid entry name: src/hello.c
         """
+        assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise BzrCheckError('InventoryEntry name %r is invalid' % name)
-        
+            raise InvalidEntryName(name=name)
+        self.executable = False
+        self.revision = None
         self.text_sha1 = None
         self.text_size = None
-    
         self.file_id = file_id
         self.name = name
-        self.kind = kind
         self.text_id = text_id
         self.parent_id = parent_id
-        if kind == 'directory':
-            self.children = {}
-        elif kind == 'file':
-            pass
-        else:
-            raise BzrError("unhandled entry kind %r" % kind)
-
-
+        self.symlink_target = None
+
+    def kind_character(self):
+        """Return a short kind indicator useful for appending to names."""
+        raise BzrError('unknown kind %r' % self.kind)
+
+    known_kinds = ('file', 'directory', 'symlink', 'root_directory')
+
+    def _put_in_tar(self, item, tree):
+        """populate item for stashing in a tar, and return the content stream.
+
+        If no content is available, return None.
+        """
+        raise BzrError("don't know how to export {%s} of kind %r" %
+                       (self.file_id, self.kind))
+
+    def put_on_disk(self, dest, dp, tree):
+        """Create a representation of self on disk in the prefix dest.
+        
+        This is a template method - implement _put_on_disk in subclasses.
+        """
+        fullpath = pathjoin(dest, dp)
+        self._put_on_disk(fullpath, tree)
+        mutter("  export {%s} kind %s to %s", self.file_id,
+                self.kind, fullpath)
+
+    def _put_on_disk(self, fullpath, tree):
+        """Put this entry onto disk at fullpath, from tree tree."""
+        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
 
     def sorted_children(self):
         l = self.children.items()
         l.sort()
         return l
 
+    @staticmethod
+    def versionable_kind(kind):
+        return kind in ('file', 'directory', 'symlink')
+
+    def check(self, checker, rev_id, inv, tree):
+        """Check this inventory entry is intact.
+
+        This is a template method, override _check for kind specific
+        tests.
+
+        :param checker: Check object providing context for the checks; 
+             can be used to find out what parts of the repository have already
+             been checked.
+        :param rev_id: Revision id from which this InventoryEntry was loaded.
+             Not necessarily the last-changed revision for this file.
+        :param inv: Inventory from which the entry was loaded.
+        :param tree: RevisionTree for this entry.
+        """
+        if self.parent_id != None:
+            if not inv.has_id(self.parent_id):
+                raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
+                        % (self.parent_id, rev_id))
+        self._check(checker, rev_id, tree)
+
+    def _check(self, checker, rev_id, tree):
+        """Check this inventory entry for kind specific errors."""
+        raise BzrCheckError('unknown entry kind %r in revision {%s}' % 
+                            (self.kind, rev_id))
+
 
     def copy(self):
-        other = InventoryEntry(self.file_id, self.name, self.kind,
-                               self.parent_id, text_id=self.text_id)
-        other.text_sha1 = self.text_sha1
-        other.text_size = self.text_size
-        # note that children are *not* copied; they're pulled across when
-        # others are added
-        return other
+        """Clone this inventory entry."""
+        raise NotImplementedError
 
+    def _get_snapshot_change(self, previous_entries):
+        if len(previous_entries) > 1:
+            return 'merged'
+        elif len(previous_entries) == 0:
+            return 'added'
+        else:
+            return 'modified/renamed/reparented'
 
     def __repr__(self):
-        return ("%s(%r, %r, kind=%r, parent_id=%r)"
+        return ("%s(%r, %r, parent_id=%r)"
                 % (self.__class__.__name__,
                    self.file_id,
                    self.name,
-                   self.kind,
                    self.parent_id))
 
-    
-    def to_element(self):
-        """Convert to XML element"""
-        from bzrlib.xml import Element
-        
-        e = Element('entry')
-
-        e.set('name', self.name)
-        e.set('file_id', self.file_id)
-        e.set('kind', self.kind)
-
-        if self.text_size != None:
-            e.set('text_size', '%d' % self.text_size)
-            
-        for f in ['text_id', 'text_sha1']:
-            v = getattr(self, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if self.parent_id != ROOT_ID:
-            assert isinstance(self.parent_id, basestring)
-            e.set('parent_id', self.parent_id)
-
-        e.tail = '\n'
-            
-        return e
-
-
-    def from_element(cls, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-        self.text_id = elt.get('text_id')
-        self.text_sha1 = elt.get('text_sha1')
-        
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        self.text_size = v and int(v)
-
-        return self
-            
-
-    from_element = classmethod(from_element)
+    def snapshot(self, revision, path, previous_entries,
+                 work_tree, weave_store, transaction):
+        """Make a snapshot of this entry which may or may not have changed.
+        
+        This means that all its fields are populated, that it has its
+        text stored in the text store or weave.
+        """
+        mutter('new parents of %s are %r', path, previous_entries)
+        self._read_tree_state(path, work_tree)
+        if len(previous_entries) == 1:
+            # cannot be unchanged unless there is only one parent file rev.
+            parent_ie = previous_entries.values()[0]
+            if self._unchanged(parent_ie):
+                mutter("found unchanged entry")
+                self.revision = parent_ie.revision
+                return "unchanged"
+        return self.snapshot_revision(revision, previous_entries, 
+                                      work_tree, weave_store, transaction)
+
+    def snapshot_revision(self, revision, previous_entries, work_tree,
+                          weave_store, transaction):
+        """Record this revision unconditionally."""
+        mutter('new revision for {%s}', self.file_id)
+        self.revision = revision
+        change = self._get_snapshot_change(previous_entries)
+        self._snapshot_text(previous_entries, work_tree, weave_store,
+                            transaction)
+        return change
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction): 
+        """Record the 'text' of this entry, whatever form that takes.
+        
+        This default implementation simply adds an empty text.
+        """
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        self._add_text_to_weave([], file_parents.keys(), weave_store, transaction)
 
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
 
-        return (self.file_id == other.file_id) \
-               and (self.name == other.name) \
-               and (self.text_sha1 == other.text_sha1) \
-               and (self.text_size == other.text_size) \
-               and (self.text_id == other.text_id) \
-               and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind)
-
+        return ((self.file_id == other.file_id)
+                and (self.name == other.name)
+                and (other.symlink_target == self.symlink_target)
+                and (self.text_sha1 == other.text_sha1)
+                and (self.text_size == other.text_size)
+                and (self.text_id == other.text_id)
+                and (self.parent_id == other.parent_id)
+                and (self.kind == other.kind)
+                and (self.revision == other.revision)
+                and (self.executable == other.executable)
+                )
 
     def __ne__(self, other):
         return not (self == other)
@@ -230,15 +428,46 @@
     def __hash__(self):
         raise ValueError('not hashable')
 
+    def _unchanged(self, previous_ie):
+        """Has this entry changed relative to previous_ie.
+
+        This method should be overriden in child classes.
+        """
+        compatible = True
+        # different inv parent
+        if previous_ie.parent_id != self.parent_id:
+            compatible = False
+        # renamed
+        elif previous_ie.name != self.name:
+            compatible = False
+        return compatible
+
+    def _read_tree_state(self, path, work_tree):
+        """Populate fields in the inventory entry from the given tree.
+        
+        Note that this should be modified to be a noop on virtual trees
+        as all entries created there are prepopulated.
+        """
+        # TODO: Rather than running this manually, we should check the 
+        # working sha1 and other expensive properties when they're
+        # first requested, or preload them if they're already known
+        pass            # nothing to do by default
+
+    def _forget_tree_state(self):
+        pass
 
 
 class RootEntry(InventoryEntry):
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+
     def __init__(self, file_id):
         self.file_id = file_id
         self.children = {}
         self.kind = 'root_directory'
         self.parent_id = None
-        self.name = ''
+        self.name = u''
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -248,6 +477,269 @@
                and (self.children == other.children)
 
 
+class InventoryDirectory(InventoryEntry):
+    """A directory in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('directory {%s} has text in revision {%s}'
+                                % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryDirectory(self.file_id, self.name, self.parent_id)
+        other.revision = self.revision
+        # note that children are *not* copied; they're pulled across when
+        # others are added
+        return other
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryDirectory, self).__init__(file_id, name, parent_id)
+        self.children = {}
+        self.kind = 'directory'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return '/'
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.DIRTYPE
+        fileobj = None
+        item.name += '/'
+        item.size = 0
+        item.mode = 0755
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        os.mkdir(fullpath)
+
+
+class InventoryFile(InventoryEntry):
+    """A file in an inventory."""
+
+    def _check(self, checker, tree_revision_id, tree):
+        """See InventoryEntry._check"""
+        t = (self.file_id, self.revision)
+        if t in checker.checked_texts:
+            prev_sha = checker.checked_texts[t]
+            if prev_sha != self.text_sha1:
+                raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
+                                    (self.file_id, tree_revision_id))
+            else:
+                checker.repeated_text_cnt += 1
+                return
+
+        if self.file_id not in checker.checked_weaves:
+            mutter('check weave {%s}', self.file_id)
+            w = tree.get_weave(self.file_id)
+            # Not passing a progress bar, because it creates a new
+            # progress, which overwrites the current progress,
+            # and doesn't look nice
+            w.check()
+            checker.checked_weaves[self.file_id] = True
+        else:
+            w = tree.get_weave(self.file_id)
+
+        mutter('check version {%s} of {%s}', tree_revision_id, self.file_id)
+        checker.checked_text_cnt += 1
+        # We can't check the length, because Weave doesn't store that
+        # information, and the whole point of looking at the weave's
+        # sha1sum is that we don't have to extract the text.
+        if self.text_sha1 != w.get_sha1(self.revision):
+            raise BzrCheckError('text {%s} version {%s} wrong sha1' 
+                                % (self.file_id, self.revision))
+        checker.checked_texts[t] = self.text_sha1
+
+    def copy(self):
+        other = InventoryFile(self.file_id, self.name, self.parent_id)
+        other.executable = self.executable
+        other.text_id = self.text_id
+        other.text_sha1 = self.text_sha1
+        other.text_size = self.text_size
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        assert self.text_sha1 != None
+        assert old_entry.text_sha1 != None
+        text_modified = (self.text_sha1 != old_entry.text_sha1)
+        meta_modified = (self.executable != old_entry.executable)
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = tree.get_file(self.file_id).readlines()
+        if to_entry:
+            to_text = to_tree.get_file(to_entry.file_id).readlines()
+        else:
+            to_text = []
+        if not reverse:
+            text_diff(from_label, from_text,
+                      to_label, to_text, output_to)
+        else:
+            text_diff(to_label, to_text,
+                      from_label, from_text, output_to)
+
+    def has_text(self):
+        """See InventoryEntry.has_text."""
+        return True
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.kind = 'file'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.REGTYPE
+        fileobj = tree.get_file(self.file_id)
+        item.size = self.text_size
+        if tree.is_executable(self.file_id):
+            item.mode = 0755
+        else:
+            item.mode = 0644
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        if tree.is_executable(self.file_id):
+            os.chmod(fullpath, 0755)
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.text_sha1 = work_tree.get_file_sha1(self.file_id)
+        self.executable = work_tree.is_executable(self.file_id)
+
+    def _forget_tree_state(self):
+        self.text_sha1 = None
+        self.executable = None
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction):
+        """See InventoryEntry._snapshot_text."""
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        # special case to avoid diffing on renames or 
+        # reparenting
+        if (len(file_parents) == 1
+            and self.text_sha1 == file_parents.values()[0].text_sha1
+            and self.text_size == file_parents.values()[0].text_size):
+            previous_ie = file_parents.values()[0]
+            versionedfile = weave_store.get_weave(self.file_id, transaction)
+            versionedfile.clone_text(self.revision, previous_ie.revision, file_parents.keys())
+        else:
+            new_lines = work_tree.get_file(self.file_id).readlines()
+            self._add_text_to_weave(new_lines, file_parents.keys(), weave_store,
+                                    transaction)
+            self.text_sha1 = sha_strings(new_lines)
+            self.text_size = sum(map(len, new_lines))
+
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryFile, self)._unchanged(previous_ie)
+        if self.text_sha1 != previous_ie.text_sha1:
+            compatible = False
+        else:
+            # FIXME: 20050930 probe for the text size when getting sha1
+            # in _read_tree_state
+            self.text_size = previous_ie.text_size
+        if self.executable != previous_ie.executable:
+            compatible = False
+        return compatible
+
+
+class InventoryLink(InventoryEntry):
+    """A file in an inventory."""
+
+    __slots__ = ['symlink_target']
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('symlink {%s} has text in revision {%s}'
+                    % (self.file_id, rev_id))
+        if self.symlink_target == None:
+            raise BzrCheckError('symlink {%s} has no target in revision {%s}'
+                    % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryLink(self.file_id, self.name, self.parent_id)
+        other.symlink_target = self.symlink_target
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        # FIXME: which _modified field should we use ? RBC 20051003
+        text_modified = (self.symlink_target != old_entry.symlink_target)
+        if text_modified:
+            mutter("    symlink target changed")
+        meta_modified = False
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = self.symlink_target
+        if to_entry is not None:
+            to_text = to_entry.symlink_target
+            if reverse:
+                temp = from_text
+                from_text = to_text
+                to_text = temp
+            print >>output_to, '=== target changed %r => %r' % (from_text, to_text)
+        else:
+            if not reverse:
+                print >>output_to, '=== target was %r' % self.symlink_target
+            else:
+                print >>output_to, '=== target is %r' % self.symlink_target
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.kind = 'symlink'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.SYMTYPE
+        fileobj = None
+        item.size = 0
+        item.mode = 0755
+        item.linkname = self.symlink_target
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        try:
+            os.symlink(self.symlink_target, fullpath)
+        except OSError,e:
+            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.symlink_target = work_tree.get_symlink_target(self.file_id)
+
+    def _forget_tree_state(self):
+        self.symlink_target = None
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryLink, self)._unchanged(previous_ie)
+        if self.symlink_target != previous_ie.symlink_target:
+            compatible = False
+        return compatible
+
 
 class Inventory(object):
     """Inventory of versioned files in a tree.
@@ -267,7 +759,8 @@
     inserted, other than through the Inventory API.
 
     >>> inv = Inventory()
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT')
     >>> inv['123-123'].name
     'hello.c'
 
@@ -283,9 +776,10 @@
     >>> [x[0] for x in inv.iter_entries()]
     ['hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
     """
-    def __init__(self, root_id=ROOT_ID):
+    def __init__(self, root_id=ROOT_ID, revision_id=None):
         """Create or read an inventory.
 
         If a working directory is specified, the inventory is read
@@ -295,14 +789,27 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch(init=True) create a unique inventory
+        # We are letting Branch.create() create a unique inventory
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
         self.root = RootEntry(root_id)
+        self.revision_id = revision_id
         self._byid = {self.root.file_id: self.root}
 
 
+    def copy(self):
+        # TODO: jam 20051218 Should copy also copy the revision_id?
+        other = Inventory(self.root.file_id)
+        # copy recursively so we know directories will be added before
+        # their children.  There are more efficient ways than this...
+        for path, entry in self.iter_entries():
+            if entry == self.root:
+                continue
+            other.add(entry.copy())
+        return other
+
+
     def __iter__(self):
         return iter(self._byid)
 
@@ -326,7 +833,7 @@
             yield name, ie
             if ie.kind == 'directory':
                 for cn, cie in self.iter_entries(from_dir=ie.file_id):
-                    yield os.path.join(name, cn), cie
+                    yield pathjoin(name, cn), cie
 
 
     def entries(self):
@@ -339,12 +846,12 @@
             kids = dir_ie.children.items()
             kids.sort()
             for name, ie in kids:
-                child_path = os.path.join(dir_path, name)
+                child_path = pathjoin(dir_path, name)
                 accum.append((child_path, ie))
                 if ie.kind == 'directory':
                     descend(ie, child_path)
 
-        descend(self.root, '')
+        descend(self.root, u'')
         return accum
 
 
@@ -359,9 +866,9 @@
             kids.sort()
 
             for name, child_ie in kids:
-                child_path = os.path.join(parent_path, name)
+                child_path = pathjoin(parent_path, name)
                 descend(child_ie, child_path)
-        descend(self.root, '')
+        descend(self.root, u'')
         return accum
         
 
@@ -370,7 +877,8 @@
         """True if this entry contains a file with given id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> '456' in inv
@@ -383,7 +891,8 @@
         """Return the entry for given file_id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123123', 'hello.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123123', 'hello.c', ROOT_ID))
+        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT')
         >>> inv['123123'].name
         'hello.c'
         """
@@ -407,7 +916,10 @@
         """Add entry to inventory.
 
         To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this."""
+        which calls this.
+
+        Returns the new entry object.
+        """
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
@@ -421,32 +933,43 @@
 
         if parent.children.has_key(entry.name):
             raise BzrError("%s is already versioned" %
-                    appendpath(self.id2path(parent.file_id), entry.name))
+                    pathjoin(self.id2path(parent.file_id), entry.name))
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
+        return entry
 
 
     def add_path(self, relpath, kind, file_id=None):
         """Add entry from a path.
 
-        The immediate parent must already be versioned"""
-        from bzrlib.branch import gen_file_id
+        The immediate parent must already be versioned.
+
+        Returns the new entry object."""
+        from bzrlib.workingtree import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
-        if len(parts) == 0:
-            raise BzrError("cannot re-add root of inventory")
 
         if file_id == None:
             file_id = gen_file_id(relpath)
 
-        parent_path = parts[:-1]
-        parent_id = self.path2id(parent_path)
-        if parent_id == None:
-            raise NotVersionedError(parent_path)
-
-        ie = InventoryEntry(file_id, parts[-1],
-                            kind=kind, parent_id=parent_id)
+        if len(parts) == 0:
+            self.root = RootEntry(file_id)
+            self._byid = {self.root.file_id: self.root}
+            return
+        else:
+            parent_path = parts[:-1]
+            parent_id = self.path2id(parent_path)
+            if parent_id == None:
+                raise NotVersionedError(path=parent_path)
+        if kind == 'directory':
+            ie = InventoryDirectory(file_id, parts[-1], parent_id)
+        elif kind == 'file':
+            ie = InventoryFile(file_id, parts[-1], parent_id)
+        elif kind == 'symlink':
+            ie = InventoryLink(file_id, parts[-1], parent_id)
+        else:
+            raise BzrError("unknown kind %r" % kind)
         return self.add(ie)
 
 
@@ -454,7 +977,8 @@
         """Remove entry by id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -463,54 +987,12 @@
         """
         ie = self[file_id]
 
-        assert self[ie.parent_id].children[ie.name] == ie
+        assert ie.parent_id is None or \
+            self[ie.parent_id].children[ie.name] == ie
         
-        # TODO: Test deleting all children; maybe hoist to a separate
-        # deltree method?
-        if ie.kind == 'directory':
-            for cie in ie.children.values():
-                del self[cie.file_id]
-            del ie.children
-
         del self._byid[file_id]
-        del self[ie.parent_id].children[ie.name]
-
-
-    def to_element(self):
-        """Convert to XML Element"""
-        from bzrlib.xml import Element
-        
-        e = Element('inventory')
-        e.text = '\n'
-        if self.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', self.root.file_id)
-        for path, ie in self.iter_entries():
-            e.append(ie.to_element())
-        return e
-    
-
-    def from_element(cls, elt):
-        """Construct from XML Element
-        
-        >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c', 'file', ROOT_ID))
-        >>> elt = inv.to_element()
-        >>> inv2 = Inventory.from_element(elt)
-        >>> inv2 == inv
-        True
-        """
-        # XXXX: doctest doesn't run this properly under python2.3
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        o = cls(root_id)
-        for e in elt:
-            ie = InventoryEntry.from_element(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            o.add(ie)
-        return o
-        
-    from_element = classmethod(from_element)
+        if ie.parent_id is not None:
+            del self[ie.parent_id].children[ie.name]
 
 
     def __eq__(self, other):
@@ -520,10 +1002,12 @@
         >>> i2 = Inventory()
         >>> i1 == i2
         True
-        >>> i1.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        >>> i1.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         False
-        >>> i2.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        >>> i2.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         True
         """
@@ -538,13 +1022,21 @@
 
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self.__eq__(other)
 
 
     def __hash__(self):
         raise ValueError('not hashable')
 
-
+    def _iter_file_id_parents(self, file_id):
+        """Yield the parents of file_id up to the root."""
+        while file_id != None:
+            try:
+                ie = self._byid[file_id]
+            except KeyError:
+                raise BzrError("file_id {%s} not found in inventory" % file_id)
+            yield ie
+            file_id = ie.parent_id
 
     def get_idpath(self, file_id):
         """Return a list of file_ids for the path to an entry.
@@ -555,25 +1047,24 @@
         root directory as depth 1.
         """
         p = []
-        while file_id != None:
-            try:
-                ie = self._byid[file_id]
-            except KeyError:
-                raise BzrError("file_id {%s} not found in inventory" % file_id)
-            p.insert(0, ie.file_id)
-            file_id = ie.parent_id
+        for parent in self._iter_file_id_parents(file_id):
+            p.insert(0, parent.file_id)
         return p
 
-
     def id2path(self, file_id):
-        """Return as a list the path to file_id."""
-
+        """Return as a string the path to file_id.
+        
+        >>> i = Inventory()
+        >>> e = i.add(InventoryDirectory('src-id', 'src', ROOT_ID))
+        >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
+        >>> print i.id2path('foo-id')
+        src/foo.c
+        """
         # get all names, skipping root
-        p = [self._byid[fid].name for fid in self.get_idpath(file_id)[1:]]
-        return os.sep.join(p)
+        return '/'.join(reversed(
+            [parent.name for parent in 
+             self._iter_file_id_parents(file_id)][:-1]))
             
-
-
     def path2id(self, name):
         """Walk down through directories to return entry of last component.
 

=== added file 'bzrlib/iterablefile.py'
--- bzrlib/iterablefile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/iterablefile.py	2006-01-25 01:43:34 +0000
@@ -0,0 +1,257 @@
+# Copyright (C) 2005 Aaron Bentley
+# <aaron.bentley@utoronto.ca>
+#
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import doctest
+
+class IterableFileBase(object):
+    """Create a file-like object from any iterable"""
+
+    def __init__(self, iterable):
+        object.__init__(self)
+        self._iter = iterable.__iter__()
+        self._buffer = ""
+        self.done = False
+
+    def read_n(self, length):
+        """
+        >>> IterableFileBase(['This ', 'is ', 'a ', 'test.']).read_n(8)
+        'This is '
+        """
+        def test_length(result):
+            if len(result) >= length:
+                return length
+            else:
+                return None
+        return self._read(test_length)
+
+    def read_to(self, sequence, length=None):
+        """
+        >>> f = IterableFileBase(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.read_to('\\n')
+        'Th\\n'
+        >>> f.read_to('\\n')
+        'is is \\n'
+        """
+        def test_contents(result):
+            if length is not None:
+                if len(result) >= length:
+                    return length
+            try:
+                return result.index(sequence)+len(sequence)
+            except ValueError:
+                return None
+        return self._read(test_contents)
+
+    def _read(self, result_length):
+        """
+        Read data until result satisfies the condition result_length.
+        result_length is a callable that returns None until the condition
+        is satisfied, and returns the length of the result to use when
+        the condition is satisfied.  (i.e. it returns the length of the
+        subset of the first condition match.)
+        """
+        result = self._buffer
+        while result_length(result) is None:
+            try:
+                result += self._iter.next()
+            except StopIteration:
+                self.done = True
+                self._buffer = ""
+                return result
+        output_length = result_length(result)
+        self._buffer = result[output_length:]
+        return result[:output_length]
+
+    def read_all(self):
+        """
+        >>> IterableFileBase(['This ', 'is ', 'a ', 'test.']).read_all()
+        'This is a test.'
+        """
+        def no_stop(result):
+            return None
+        return self._read(no_stop)
+
+
+    def push_back(self, contents):
+        """
+        >>> f = IterableFileBase(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.read_to('\\n')
+        'Th\\n'
+        >>> f.push_back("Sh")
+        >>> f.read_all()
+        'Shis is \\na te\\nst.'
+        """
+        self._buffer = contents + self._buffer
+
+
+class IterableFile(object):
+    """This class supplies all File methods that can be implemented cheaply."""
+    def __init__(self, iterable):
+        object.__init__(self)
+        self._file_base = IterableFileBase(iterable)
+        self._iter = self._make_iterator()
+        self._closed = False
+        self.softspace = 0
+
+    def _make_iterator(self):
+        while not self._file_base.done:
+            self._check_closed()
+            result = self._file_base.read_to('\n')
+            if result != '':
+                yield result
+
+    def _check_closed(self):
+        if self.closed:
+            raise ValueError("File is closed.")
+
+    def close(self):
+        """
+        >>> f = IterableFile(['This ', 'is ', 'a ', 'test.'])
+        >>> f.closed
+        False
+        >>> f.close()
+        >>> f.closed
+        True
+        """
+        self._file_base.done = True
+        self._closed = True
+
+    closed = property(lambda x: x._closed)
+
+    def flush(self):
+        """No-op for standard compliance.
+        >>> f = IterableFile([])
+        >>> f.close()
+        >>> f.flush()
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        self._check_closed()
+
+    def next(self):
+        """Implementation of the iterator protocol's next()
+
+        >>> f = IterableFile(['This \\n', 'is ', 'a ', 'test.'])
+        >>> f.next()
+        'This \\n'
+        >>> f.close()
+        >>> f.next()
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        >>> f = IterableFile(['This \\n', 'is ', 'a ', 'test.\\n'])
+        >>> f.next()
+        'This \\n'
+        >>> f.next()
+        'is a test.\\n'
+        >>> f.next()
+        Traceback (most recent call last):
+        StopIteration
+        """
+        self._check_closed()
+        return self._iter.next()
+
+    def __iter__(self):
+        """
+        >>> list(IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.']))
+        ['Th\\n', 'is is \\n', 'a te\\n', 'st.']
+        >>> f = IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.close()
+        >>> list(f)
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        return self
+
+    def read(self, length=None):
+        """
+        >>> IterableFile(['This ', 'is ', 'a ', 'test.']).read()
+        'This is a test.'
+        >>> f = IterableFile(['This ', 'is ', 'a ', 'test.'])
+        >>> f.read(10)
+        'This is a '
+        >>> f = IterableFile(['This ', 'is ', 'a ', 'test.'])
+        >>> f.close()
+        >>> f.read(10)
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        self._check_closed()
+        if length is None:
+            return self._file_base.read_all()
+        else:
+            return self._file_base.read_n(length)
+
+    def read_to(self, sequence, size=None):
+        """
+        Read characters until a sequence is found, with optional max size.
+        The specified sequence, if found, will be included in the result
+
+        >>> f = IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.read_to('i')
+        'Th\\ni'
+        >>> f.read_to('i')
+        's i'
+        >>> f.close()
+        >>> f.read_to('i')
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        self._check_closed()
+        return self._file_base.read_to(sequence, size)
+
+    def readline(self, size=None):
+        """
+        >>> f = IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.readline()
+        'Th\\n'
+        >>> f.readline(4)
+        'is i'
+        >>> f.close()
+        >>> f.readline()
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        return self.read_to('\n', size)
+
+    def readlines(self, sizehint=None):
+        """
+        >>> f = IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.readlines()
+        ['Th\\n', 'is is \\n', 'a te\\n', 'st.']
+        >>> f = IterableFile(['Th\\nis ', 'is \\n', 'a ', 'te\\nst.'])
+        >>> f.close()
+        >>> f.readlines()
+        Traceback (most recent call last):
+        ValueError: File is closed.
+        """
+        lines = []
+        while True:
+            line = self.readline()
+            if line == "":
+                return lines
+            if sizehint is None:
+                lines.append(line)
+            elif len(line) < sizehint:
+                lines.append(line)
+                sizehint -= len(line)
+            else:
+                self._file_base.push_back(line)
+                return lines
+
+        
+if __name__ == "__main__":
+    doctest.testmod()

=== added file 'bzrlib/knit.py'
--- bzrlib/knit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/knit.py	2006-04-19 02:50:56 +0000
@@ -0,0 +1,1607 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+# Written by Martin Pool.
+# Modified by Johan Rydberg <jrydberg@gnu.org>
+# Modified by Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Knit versionedfile implementation.
+
+A knit is a versioned file implementation that supports efficient append only
+updates.
+
+Knit file layout:
+lifeless: the data file is made up of "delta records".  each delta record has a delta header 
+that contains; (1) a version id, (2) the size of the delta (in lines), and (3)  the digest of 
+the -expanded data- (ie, the delta applied to the parent).  the delta also ends with a 
+end-marker; simply "end VERSION"
+
+delta can be line or full contents.a
+... the 8's there are the index number of the annotation.
+version robertc@robertcollins.net-20051003014215-ee2990904cc4c7ad 7 c7d23b2a5bd6ca00e8e266cec0ec228158ee9f9e
+59,59,3
+8
+8         if ie.executable:
+8             e.set('executable', 'yes')
+130,130,2
+8         if elt.get('executable') == 'yes':
+8             ie.executable = True
+end robertc@robertcollins.net-20051003014215-ee2990904cc4c7ad 
+
+
+whats in an index:
+09:33 < jrydberg> lifeless: each index is made up of a tuple of; version id, options, position, size, parents
+09:33 < jrydberg> lifeless: the parents are currently dictionary compressed
+09:33 < jrydberg> lifeless: (meaning it currently does not support ghosts)
+09:33 < lifeless> right
+09:33 < jrydberg> lifeless: the position and size is the range in the data file
+
+
+so the index sequence is the dictionary compressed sequence number used
+in the deltas to provide line annotation
+
+"""
+
+# TODOS:
+# 10:16 < lifeless> make partial index writes safe
+# 10:16 < lifeless> implement 'knit.check()' like weave.check()
+# 10:17 < lifeless> record known ghosts so we can detect when they are filled in rather than the current 'reweave 
+#                    always' approach.
+# move sha1 out of the content so that join is faster at verifying parents
+# record content length ?
+                  
+
+from copy import copy
+from cStringIO import StringIO
+import difflib
+from itertools import izip, chain
+import os
+import sys
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import FileExists, NoSuchFile, KnitError, \
+        InvalidRevisionId, KnitCorrupt, KnitHeaderError, \
+        RevisionNotPresent, RevisionAlreadyPresent
+from bzrlib.tuned_gzip import *
+from bzrlib.trace import mutter
+from bzrlib.osutils import contains_whitespace, contains_linebreaks, \
+     sha_strings
+from bzrlib.versionedfile import VersionedFile, InterVersionedFile
+from bzrlib.tsort import topo_sort
+
+
+# TODO: Split out code specific to this format into an associated object.
+
+# TODO: Can we put in some kind of value to check that the index and data
+# files belong together?
+
+# TODO: accomodate binaries, perhaps by storing a byte count
+
+# TODO: function to check whole file
+
+# TODO: atomically append data, then measure backwards from the cursor
+# position after writing to work out where it was located.  we may need to
+# bypass python file buffering.
+
+DATA_SUFFIX = '.knit'
+INDEX_SUFFIX = '.kndx'
+
+
+class KnitContent(object):
+    """Content of a knit version to which deltas can be applied."""
+
+    def __init__(self, lines):
+        self._lines = lines
+
+    def annotate_iter(self):
+        """Yield tuples of (origin, text) for each content line."""
+        for origin, text in self._lines:
+            yield origin, text
+
+    def annotate(self):
+        """Return a list of (origin, text) tuples."""
+        return list(self.annotate_iter())
+
+    def line_delta_iter(self, new_lines):
+        """Generate line-based delta from this content to new_lines."""
+        new_texts = [text for origin, text in new_lines._lines]
+        old_texts = [text for origin, text in self._lines]
+        s = SequenceMatcher(None, old_texts, new_texts)
+        for op in s.get_opcodes():
+            if op[0] == 'equal':
+                continue
+            #     ofrom   oto   length        data
+            yield (op[1], op[2], op[4]-op[3], new_lines._lines[op[3]:op[4]])
+
+    def line_delta(self, new_lines):
+        return list(self.line_delta_iter(new_lines))
+
+    def text(self):
+        return [text for origin, text in self._lines]
+
+
+class _KnitFactory(object):
+    """Base factory for creating content objects."""
+
+    def make(self, lines, version):
+        num_lines = len(lines)
+        return KnitContent(zip([version] * num_lines, lines))
+
+
+class KnitAnnotateFactory(_KnitFactory):
+    """Factory for creating annotated Content objects."""
+
+    annotated = True
+
+    def parse_fulltext(self, content, version):
+        """Convert fulltext to internal representation
+
+        fulltext content is of the format
+        revid(utf8) plaintext\n
+        internal representation is of the format:
+        (revid, plaintext)
+        """
+        lines = []
+        for line in content:
+            origin, text = line.split(' ', 1)
+            lines.append((origin.decode('utf-8'), text))
+        return KnitContent(lines)
+
+    def parse_line_delta_iter(self, lines):
+        for result_item in self.parse_line_delta[lines]:
+            yield result_item
+
+    def parse_line_delta(self, lines, version):
+        """Convert a line based delta into internal representation.
+
+        line delta is in the form of:
+        intstart intend intcount
+        1..count lines:
+        revid(utf8) newline\n
+        internal represnetation is
+        (start, end, count, [1..count tuples (revid, newline)])
+        """
+        result = []
+        lines = iter(lines)
+        next = lines.next
+        # walk through the lines parsing.
+        for header in lines:
+            start, end, count = [int(n) for n in header.split(',')]
+            contents = []
+            remaining = count
+            while remaining:
+                origin, text = next().split(' ', 1)
+                remaining -= 1
+                contents.append((origin.decode('utf-8'), text))
+            result.append((start, end, count, contents))
+        return result
+
+    def lower_fulltext(self, content):
+        """convert a fulltext content record into a serializable form.
+
+        see parse_fulltext which this inverts.
+        """
+        return ['%s %s' % (o.encode('utf-8'), t) for o, t in content._lines]
+
+    def lower_line_delta(self, delta):
+        """convert a delta into a serializable form.
+
+        See parse_line_delta which this inverts.
+        """
+        out = []
+        for start, end, c, lines in delta:
+            out.append('%d,%d,%d\n' % (start, end, c))
+            for origin, text in lines:
+                out.append('%s %s' % (origin.encode('utf-8'), text))
+        return out
+
+
+class KnitPlainFactory(_KnitFactory):
+    """Factory for creating plain Content objects."""
+
+    annotated = False
+
+    def parse_fulltext(self, content, version):
+        """This parses an unannotated fulltext.
+
+        Note that this is not a noop - the internal representation
+        has (versionid, line) - its just a constant versionid.
+        """
+        return self.make(content, version)
+
+    def parse_line_delta_iter(self, lines, version):
+        while lines:
+            header = lines.pop(0)
+            start, end, c = [int(n) for n in header.split(',')]
+            yield start, end, c, zip([version] * c, lines[:c])
+            del lines[:c]
+
+    def parse_line_delta(self, lines, version):
+        return list(self.parse_line_delta_iter(lines, version))
+    
+    def lower_fulltext(self, content):
+        return content.text()
+
+    def lower_line_delta(self, delta):
+        out = []
+        for start, end, c, lines in delta:
+            out.append('%d,%d,%d\n' % (start, end, c))
+            out.extend([text for origin, text in lines])
+        return out
+
+
+def make_empty_knit(transport, relpath):
+    """Construct a empty knit at the specified location."""
+    k = KnitVersionedFile(transport, relpath, 'w', KnitPlainFactory)
+    k._data._open_file()
+
+
+class KnitVersionedFile(VersionedFile):
+    """Weave-like structure with faster random access.
+
+    A knit stores a number of texts and a summary of the relationships
+    between them.  Texts are identified by a string version-id.  Texts
+    are normally stored and retrieved as a series of lines, but can
+    also be passed as single strings.
+
+    Lines are stored with the trailing newline (if any) included, to
+    avoid special cases for files with no final newline.  Lines are
+    composed of 8-bit characters, not unicode.  The combination of
+    these approaches should mean any 'binary' file can be safely
+    stored and retrieved.
+    """
+
+    def __init__(self, relpath, transport, file_mode=None, access_mode=None, factory=None,
+                 basis_knit=None, delta=True, create=False):
+        """Construct a knit at location specified by relpath.
+        
+        :param create: If not True, only open an existing knit.
+        """
+        if access_mode is None:
+            access_mode = 'w'
+        super(KnitVersionedFile, self).__init__(access_mode)
+        assert access_mode in ('r', 'w'), "invalid mode specified %r" % access_mode
+        assert not basis_knit or isinstance(basis_knit, KnitVersionedFile), \
+            type(basis_knit)
+
+        self.transport = transport
+        self.filename = relpath
+        self.basis_knit = basis_knit
+        self.factory = factory or KnitAnnotateFactory()
+        self.writable = (access_mode == 'w')
+        self.delta = delta
+
+        self._index = _KnitIndex(transport, relpath + INDEX_SUFFIX,
+            access_mode, create=create)
+        self._data = _KnitData(transport, relpath + DATA_SUFFIX,
+            access_mode, create=not len(self.versions()))
+
+    def _add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
+        """See VersionedFile._add_delta()."""
+        self._check_add(version_id, []) # should we check the lines ?
+        self._check_versions_present(parents)
+        present_parents = []
+        ghosts = []
+        parent_texts = {}
+        for parent in parents:
+            if not self.has_version(parent):
+                ghosts.append(parent)
+            else:
+                present_parents.append(parent)
+
+        if delta_parent is None:
+            # reconstitute as full text.
+            assert len(delta) == 1 or len(delta) == 0
+            if len(delta):
+                assert delta[0][0] == 0
+                assert delta[0][1] == 0, delta[0][1]
+            return super(KnitVersionedFile, self)._add_delta(version_id,
+                                                             parents,
+                                                             delta_parent,
+                                                             sha1,
+                                                             noeol,
+                                                             delta)
+
+        digest = sha1
+
+        options = []
+        if noeol:
+            options.append('no-eol')
+
+        if delta_parent is not None:
+            # determine the current delta chain length.
+            # To speed the extract of texts the delta chain is limited
+            # to a fixed number of deltas.  This should minimize both
+            # I/O and the time spend applying deltas.
+            count = 0
+            delta_parents = [delta_parent]
+            while count < 25:
+                parent = delta_parents[0]
+                method = self._index.get_method(parent)
+                if method == 'fulltext':
+                    break
+                delta_parents = self._index.get_parents(parent)
+                count = count + 1
+            if method == 'line-delta':
+                # did not find a fulltext in the delta limit.
+                # just do a normal insertion.
+                return super(KnitVersionedFile, self)._add_delta(version_id,
+                                                                 parents,
+                                                                 delta_parent,
+                                                                 sha1,
+                                                                 noeol,
+                                                                 delta)
+
+        options.append('line-delta')
+        store_lines = self.factory.lower_line_delta(delta)
+
+        where, size = self._data.add_record(version_id, digest, store_lines)
+        self._index.add_version(version_id, options, where, size, parents)
+
+    def clear_cache(self):
+        """Clear the data cache only."""
+        self._data.clear_cache()
+
+    def copy_to(self, name, transport):
+        """See VersionedFile.copy_to()."""
+        # copy the current index to a temp index to avoid racing with local
+        # writes
+        transport.put(name + INDEX_SUFFIX + '.tmp', self.transport.get(self._index._filename))
+        # copy the data file
+        transport.put(name + DATA_SUFFIX, self._data._open_file())
+        # rename the copied index into place
+        transport.rename(name + INDEX_SUFFIX + '.tmp', name + INDEX_SUFFIX)
+
+    def create_empty(self, name, transport, mode=None):
+        return KnitVersionedFile(name, transport, factory=self.factory, delta=self.delta, create=True)
+    
+    def _fix_parents(self, version, new_parents):
+        """Fix the parents list for version.
+        
+        This is done by appending a new version to the index
+        with identical data except for the parents list.
+        the parents list must be a superset of the current
+        list.
+        """
+        current_values = self._index._cache[version]
+        assert set(current_values[4]).difference(set(new_parents)) == set()
+        self._index.add_version(version,
+                                current_values[1], 
+                                current_values[2],
+                                current_values[3],
+                                new_parents)
+
+    def get_delta(self, version_id):
+        """Get a delta for constructing version from some other version."""
+        if not self.has_version(version_id):
+            raise RevisionNotPresent(version_id, self.filename)
+        
+        parents = self.get_parents(version_id)
+        if len(parents):
+            parent = parents[0]
+        else:
+            parent = None
+        data_pos, data_size = self._index.get_position(version_id)
+        data, sha1 = self._data.read_records(((version_id, data_pos, data_size),))[version_id]
+        version_idx = self._index.lookup(version_id)
+        noeol = 'no-eol' in self._index.get_options(version_id)
+        if 'fulltext' == self._index.get_method(version_id):
+            new_content = self.factory.parse_fulltext(data, version_idx)
+            if parent is not None:
+                reference_content = self._get_content(parent)
+                old_texts = reference_content.text()
+            else:
+                old_texts = []
+            new_texts = new_content.text()
+            delta_seq = SequenceMatcher(None, old_texts, new_texts)
+            return parent, sha1, noeol, self._make_line_delta(delta_seq, new_content)
+        else:
+            delta = self.factory.parse_line_delta(data, version_idx)
+            return parent, sha1, noeol, delta
+        
+    def get_graph_with_ghosts(self):
+        """See VersionedFile.get_graph_with_ghosts()."""
+        graph_items = self._index.get_graph()
+        return dict(graph_items)
+
+    @staticmethod
+    def get_suffixes():
+        """See VersionedFile.get_suffixes()."""
+        return [DATA_SUFFIX, INDEX_SUFFIX]
+
+    def has_ghost(self, version_id):
+        """True if there is a ghost reference in the file to version_id."""
+        # maybe we have it
+        if self.has_version(version_id):
+            return False
+        # optimisable if needed by memoising the _ghosts set.
+        items = self._index.get_graph()
+        for node, parents in items:
+            for parent in parents:
+                if parent not in self._index._cache:
+                    if parent == version_id:
+                        return True
+        return False
+
+    def versions(self):
+        """See VersionedFile.versions."""
+        return self._index.get_versions()
+
+    def has_version(self, version_id):
+        """See VersionedFile.has_version."""
+        return self._index.has_version(version_id)
+
+    __contains__ = has_version
+
+    def _merge_annotations(self, content, parents, parent_texts={},
+                           delta=None, annotated=None):
+        """Merge annotations for content.  This is done by comparing
+        the annotations based on changed to the text.
+        """
+        if annotated:
+            delta_seq = None
+            for parent_id in parents:
+                merge_content = self._get_content(parent_id, parent_texts)
+                seq = SequenceMatcher(None, merge_content.text(), content.text())
+                if delta_seq is None:
+                    # setup a delta seq to reuse.
+                    delta_seq = seq
+                for i, j, n in seq.get_matching_blocks():
+                    if n == 0:
+                        continue
+                    # this appears to copy (origin, text) pairs across to the new
+                    # content for any line that matches the last-checked parent.
+                    # FIXME: save the sequence control data for delta compression
+                    # against the most relevant parent rather than rediffing.
+                    content._lines[j:j+n] = merge_content._lines[i:i+n]
+        if delta:
+            if not annotated:
+                reference_content = self._get_content(parents[0], parent_texts)
+                new_texts = content.text()
+                old_texts = reference_content.text()
+                delta_seq = SequenceMatcher(None, old_texts, new_texts)
+            return self._make_line_delta(delta_seq, content)
+
+    def _make_line_delta(self, delta_seq, new_content):
+        """Generate a line delta from delta_seq and new_content."""
+        diff_hunks = []
+        for op in delta_seq.get_opcodes():
+            if op[0] == 'equal':
+                continue
+            diff_hunks.append((op[1], op[2], op[4]-op[3], new_content._lines[op[3]:op[4]]))
+        return diff_hunks
+
+    def _get_components(self, version_id):
+        """Return a list of (version_id, method, data) tuples that
+        makes up version specified by version_id of the knit.
+
+        The components should be applied in the order of the returned
+        list.
+
+        The basis knit will be used to the largest extent possible
+        since it is assumed that accesses to it is faster.
+        """
+        #profile notes:
+        # 4168 calls in 14912, 2289 internal
+        # 4168 in 9711 to read_records
+        # 52554 in 1250 to get_parents
+        # 170166 in 865 to list.append
+        
+        # needed_revisions holds a list of (method, version_id) of
+        # versions that is needed to be fetched to construct the final
+        # version of the file.
+        #
+        # basis_revisions is a list of versions that needs to be
+        # fetched but exists in the basis knit.
+
+        basis = self.basis_knit
+        needed_versions = []
+        basis_versions = []
+        cursor = version_id
+
+        while 1:
+            picked_knit = self
+            if basis and basis._index.has_version(cursor):
+                picked_knit = basis
+                basis_versions.append(cursor)
+            method = picked_knit._index.get_method(cursor)
+            needed_versions.append((method, cursor))
+            if method == 'fulltext':
+                break
+            cursor = picked_knit.get_parents(cursor)[0]
+
+        components = {}
+        if basis_versions:
+            records = []
+            for comp_id in basis_versions:
+                data_pos, data_size = basis._index.get_data_position(comp_id)
+                records.append((piece_id, data_pos, data_size))
+            components.update(basis._data.read_records(records))
+
+        records = []
+        for comp_id in [vid for method, vid in needed_versions
+                        if vid not in basis_versions]:
+            data_pos, data_size = self._index.get_position(comp_id)
+            records.append((comp_id, data_pos, data_size))
+        components.update(self._data.read_records(records))
+
+        # get_data_records returns a mapping with the version id as
+        # index and the value as data.  The order the components need
+        # to be applied is held by needed_versions (reversed).
+        out = []
+        for method, comp_id in reversed(needed_versions):
+            out.append((comp_id, method, components[comp_id]))
+
+        return out
+
+    def _get_content(self, version_id, parent_texts={}):
+        """Returns a content object that makes up the specified
+        version."""
+        if not self.has_version(version_id):
+            raise RevisionNotPresent(version_id, self.filename)
+
+        cached_version = parent_texts.get(version_id, None)
+        if cached_version is not None:
+            return cached_version
+
+        if self.basis_knit and version_id in self.basis_knit:
+            return self.basis_knit._get_content(version_id)
+
+        content = None
+        components = self._get_components(version_id)
+        for component_id, method, (data, digest) in components:
+            version_idx = self._index.lookup(component_id)
+            if method == 'fulltext':
+                assert content is None
+                content = self.factory.parse_fulltext(data, version_idx)
+            elif method == 'line-delta':
+                delta = self.factory.parse_line_delta(data, version_idx)
+                content._lines = self._apply_delta(content._lines, delta)
+
+        if 'no-eol' in self._index.get_options(version_id):
+            line = content._lines[-1][1].rstrip('\n')
+            content._lines[-1] = (content._lines[-1][0], line)
+
+        if sha_strings(content.text()) != digest:
+            import pdb;pdb.set_trace()
+            raise KnitCorrupt(self.filename, 'sha-1 does not match %s' % version_id)
+
+        return content
+
+    def _check_versions_present(self, version_ids):
+        """Check that all specified versions are present."""
+        version_ids = set(version_ids)
+        for r in list(version_ids):
+            if self._index.has_version(r):
+                version_ids.remove(r)
+        if version_ids:
+            raise RevisionNotPresent(list(version_ids)[0], self.filename)
+
+    def _add_lines_with_ghosts(self, version_id, parents, lines, parent_texts):
+        """See VersionedFile.add_lines_with_ghosts()."""
+        self._check_add(version_id, lines)
+        return self._add(version_id, lines[:], parents, self.delta, parent_texts)
+
+    def _add_lines(self, version_id, parents, lines, parent_texts):
+        """See VersionedFile.add_lines."""
+        self._check_add(version_id, lines)
+        self._check_versions_present(parents)
+        return self._add(version_id, lines[:], parents, self.delta, parent_texts)
+
+    def _check_add(self, version_id, lines):
+        """check that version_id and lines are safe to add."""
+        assert self.writable, "knit is not opened for write"
+        ### FIXME escape. RBC 20060228
+        if contains_whitespace(version_id):
+            raise InvalidRevisionId(version_id)
+        if self.has_version(version_id):
+            raise RevisionAlreadyPresent(version_id, self.filename)
+
+        if False or __debug__:
+            for l in lines:
+                assert '\n' not in l[:-1]
+
+    def _add(self, version_id, lines, parents, delta, parent_texts):
+        """Add a set of lines on top of version specified by parents.
+
+        If delta is true, compress the text as a line-delta against
+        the first parent.
+
+        Any versions not present will be converted into ghosts.
+        """
+        #  461    0   6546.0390     43.9100   bzrlib.knit:489(_add)
+        # +400    0    889.4890    418.9790   +bzrlib.knit:192(lower_fulltext)
+        # +461    0   1364.8070    108.8030   +bzrlib.knit:996(add_record)
+        # +461    0    193.3940     41.5720   +bzrlib.knit:898(add_version)
+        # +461    0    134.0590     18.3810   +bzrlib.osutils:361(sha_strings)
+        # +461    0     36.3420     15.4540   +bzrlib.knit:146(make)
+        # +1383   0      8.0370      8.0370   +<len>
+        # +61     0     13.5770      7.9190   +bzrlib.knit:199(lower_line_delta)
+        # +61     0    963.3470      7.8740   +bzrlib.knit:427(_get_content)
+        # +61     0    973.9950      5.2950   +bzrlib.knit:136(line_delta)
+        # +61     0   1918.1800      5.2640   +bzrlib.knit:359(_merge_annotations)
+
+        present_parents = []
+        ghosts = []
+        if parent_texts is None:
+            parent_texts = {}
+        for parent in parents:
+            if not self.has_version(parent):
+                ghosts.append(parent)
+            else:
+                present_parents.append(parent)
+
+        if delta and not len(present_parents):
+            delta = False
+
+        digest = sha_strings(lines)
+        options = []
+        if lines:
+            if lines[-1][-1] != '\n':
+                options.append('no-eol')
+                lines[-1] = lines[-1] + '\n'
+
+        if len(present_parents) and delta:
+            # To speed the extract of texts the delta chain is limited
+            # to a fixed number of deltas.  This should minimize both
+            # I/O and the time spend applying deltas.
+            count = 0
+            delta_parents = present_parents
+            while count < 25:
+                parent = delta_parents[0]
+                method = self._index.get_method(parent)
+                if method == 'fulltext':
+                    break
+                delta_parents = self._index.get_parents(parent)
+                count = count + 1
+            if method == 'line-delta':
+                delta = False
+
+        lines = self.factory.make(lines, version_id)
+        if delta or (self.factory.annotated and len(present_parents) > 0):
+            # Merge annotations from parent texts if so is needed.
+            delta_hunks = self._merge_annotations(lines, present_parents, parent_texts,
+                                                  delta, self.factory.annotated)
+
+        if delta:
+            options.append('line-delta')
+            store_lines = self.factory.lower_line_delta(delta_hunks)
+        else:
+            options.append('fulltext')
+            store_lines = self.factory.lower_fulltext(lines)
+
+        where, size = self._data.add_record(version_id, digest, store_lines)
+        self._index.add_version(version_id, options, where, size, parents)
+        return lines
+
+    def check(self, progress_bar=None):
+        """See VersionedFile.check()."""
+
+    def _clone_text(self, new_version_id, old_version_id, parents):
+        """See VersionedFile.clone_text()."""
+        # FIXME RBC 20060228 make fast by only inserting an index with null delta.
+        self.add_lines(new_version_id, parents, self.get_lines(old_version_id))
+
+    def get_lines(self, version_id):
+        """See VersionedFile.get_lines()."""
+        return self._get_content(version_id).text()
+
+    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+        """See VersionedFile.iter_lines_added_or_present_in_versions()."""
+        if version_ids is None:
+            version_ids = self.versions()
+        # we dont care about inclusions, the caller cares.
+        # but we need to setup a list of records to visit.
+        # we need version_id, position, length
+        version_id_records = []
+        requested_versions = list(version_ids)
+        # filter for available versions
+        for version_id in requested_versions:
+            if not self.has_version(version_id):
+                raise RevisionNotPresent(version_id, self.filename)
+        # get a in-component-order queue:
+        version_ids = []
+        for version_id in self.versions():
+            if version_id in requested_versions:
+                version_ids.append(version_id)
+                data_pos, length = self._index.get_position(version_id)
+                version_id_records.append((version_id, data_pos, length))
+
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        count = 0
+        total = len(version_id_records)
+        try:
+            pb.update('Walking content.', count, total)
+            for version_id, data, sha_value in \
+                self._data.read_records_iter(version_id_records):
+                pb.update('Walking content.', count, total)
+                method = self._index.get_method(version_id)
+                version_idx = self._index.lookup(version_id)
+                assert method in ('fulltext', 'line-delta')
+                if method == 'fulltext':
+                    content = self.factory.parse_fulltext(data, version_idx)
+                    for line in content.text():
+                        yield line
+                else:
+                    delta = self.factory.parse_line_delta(data, version_idx)
+                    for start, end, count, lines in delta:
+                        for origin, line in lines:
+                            yield line
+                count +=1
+            pb.update('Walking content.', total, total)
+            pb.finished()
+        except:
+            pb.update('Walking content.', total, total)
+            pb.finished()
+            raise
+        
+    def num_versions(self):
+        """See VersionedFile.num_versions()."""
+        return self._index.num_versions()
+
+    __len__ = num_versions
+
+    def annotate_iter(self, version_id):
+        """See VersionedFile.annotate_iter."""
+        content = self._get_content(version_id)
+        for origin, text in content.annotate_iter():
+            yield origin, text
+
+    def get_parents(self, version_id):
+        """See VersionedFile.get_parents."""
+        # perf notes:
+        # optimism counts!
+        # 52554 calls in 1264 872 internal down from 3674
+        try:
+            return self._index.get_parents(version_id)
+        except KeyError:
+            raise RevisionNotPresent(version_id, self.filename)
+
+    def get_parents_with_ghosts(self, version_id):
+        """See VersionedFile.get_parents."""
+        try:
+            return self._index.get_parents_with_ghosts(version_id)
+        except KeyError:
+            raise RevisionNotPresent(version_id, self.filename)
+
+    def get_ancestry(self, versions):
+        """See VersionedFile.get_ancestry."""
+        if isinstance(versions, basestring):
+            versions = [versions]
+        if not versions:
+            return []
+        self._check_versions_present(versions)
+        return self._index.get_ancestry(versions)
+
+    def get_ancestry_with_ghosts(self, versions):
+        """See VersionedFile.get_ancestry_with_ghosts."""
+        if isinstance(versions, basestring):
+            versions = [versions]
+        if not versions:
+            return []
+        self._check_versions_present(versions)
+        return self._index.get_ancestry_with_ghosts(versions)
+
+    #@deprecated_method(zero_eight)
+    def walk(self, version_ids):
+        """See VersionedFile.walk."""
+        # We take the short path here, and extract all relevant texts
+        # and put them in a weave and let that do all the work.  Far
+        # from optimal, but is much simpler.
+        # FIXME RB 20060228 this really is inefficient!
+        from bzrlib.weave import Weave
+
+        w = Weave(self.filename)
+        ancestry = self.get_ancestry(version_ids)
+        sorted_graph = topo_sort(self._index.get_graph())
+        version_list = [vid for vid in sorted_graph if vid in ancestry]
+        
+        for version_id in version_list:
+            lines = self.get_lines(version_id)
+            w.add_lines(version_id, self.get_parents(version_id), lines)
+
+        for lineno, insert_id, dset, line in w.walk(version_ids):
+            yield lineno, insert_id, dset, line
+
+    def plan_merge(self, ver_a, ver_b):
+        """See VersionedFile.plan_merge."""
+        ancestors_b = set(self.get_ancestry(ver_b))
+        def status_a(revision, text):
+            if revision in ancestors_b:
+                return 'killed-b', text
+            else:
+                return 'new-a', text
+        
+        ancestors_a = set(self.get_ancestry(ver_a))
+        def status_b(revision, text):
+            if revision in ancestors_a:
+                return 'killed-a', text
+            else:
+                return 'new-b', text
+
+        annotated_a = self.annotate(ver_a)
+        annotated_b = self.annotate(ver_b)
+        plain_a = [t for (a, t) in annotated_a]
+        plain_b = [t for (a, t) in annotated_b]
+        blocks = SequenceMatcher(None, plain_a, plain_b).get_matching_blocks()
+        a_cur = 0
+        b_cur = 0
+        for ai, bi, l in blocks:
+            # process all mismatched sections
+            # (last mismatched section is handled because blocks always
+            # includes a 0-length last block)
+            for revision, text in annotated_a[a_cur:ai]:
+                yield status_a(revision, text)
+            for revision, text in annotated_b[b_cur:bi]:
+                yield status_b(revision, text)
+
+            # and now the matched section
+            a_cur = ai + l
+            b_cur = bi + l
+            for text_a, text_b in zip(plain_a[ai:a_cur], plain_b[bi:b_cur]):
+                assert text_a == text_b
+                yield "unchanged", text_a
+
+
+class _KnitComponentFile(object):
+    """One of the files used to implement a knit database"""
+
+    def __init__(self, transport, filename, mode):
+        self._transport = transport
+        self._filename = filename
+        self._mode = mode
+
+    def write_header(self):
+        if self._transport.append(self._filename, StringIO(self.HEADER)):
+            raise KnitCorrupt(self._filename, 'misaligned after writing header')
+
+    def check_header(self, fp):
+        line = fp.readline()
+        if line != self.HEADER:
+            raise KnitHeaderError(badline=line)
+
+    def commit(self):
+        """Commit is a nop."""
+
+    def __repr__(self):
+        return '%s(%s)' % (self.__class__.__name__, self._filename)
+
+
+class _KnitIndex(_KnitComponentFile):
+    """Manages knit index file.
+
+    The index is already kept in memory and read on startup, to enable
+    fast lookups of revision information.  The cursor of the index
+    file is always pointing to the end, making it easy to append
+    entries.
+
+    _cache is a cache for fast mapping from version id to a Index
+    object.
+
+    _history is a cache for fast mapping from indexes to version ids.
+
+    The index data format is dictionary compressed when it comes to
+    parent references; a index entry may only have parents that with a
+    lover index number.  As a result, the index is topological sorted.
+
+    Duplicate entries may be written to the index for a single version id
+    if this is done then the latter one completely replaces the former:
+    this allows updates to correct version and parent information. 
+    Note that the two entries may share the delta, and that successive
+    annotations and references MUST point to the first entry.
+
+    The index file on disc contains a header, followed by one line per knit
+    record. The same revision can be present in an index file more than once.
+    The first occurence gets assigned a sequence number starting from 0. 
+    
+    The format of a single line is
+    REVISION_ID FLAGS BYTE_OFFSET LENGTH( PARENT_ID|PARENT_SEQUENCE_ID)* :\n
+    REVISION_ID is a utf8-encoded revision id
+    FLAGS is a comma separated list of flags about the record. Values include 
+        no-eol, line-delta, fulltext.
+    BYTE_OFFSET is the ascii representation of the byte offset in the data file
+        that the the compressed data starts at.
+    LENGTH is the ascii representation of the length of the data file.
+    PARENT_ID a utf-8 revision id prefixed by a '.' that is a parent of
+        REVISION_ID.
+    PARENT_SEQUENCE_ID the ascii representation of the sequence number of a
+        revision id already in the knit that is a parent of REVISION_ID.
+    The ' :' marker is the end of record marker.
+    
+    partial writes:
+    when a write is interrupted to the index file, it will result in a line that
+    does not end in ' :'. If the ' :' is not present at the end of a line, or at
+    the end of the file, then the record that is missing it will be ignored by
+    the parser.
+
+    When writing new records to the index file, the data is preceeded by '\n'
+    to ensure that records always start on new lines even if the last write was
+    interrupted. As a result its normal for the last line in the index to be
+    missing a trailing newline. One can be added with no harmful effects.
+    """
+
+    HEADER = "# bzr knit index 7\n"
+
+    # speed of knit parsing went from 280 ms to 280 ms with slots addition.
+    # __slots__ = ['_cache', '_history', '_transport', '_filename']
+
+    def _cache_version(self, version_id, options, pos, size, parents):
+        """Cache a version record in the history array and index cache.
+        
+        This is inlined into __init__ for performance. KEEP IN SYNC.
+        (It saves 60ms, 25% of the __init__ overhead on local 4000 record
+         indexes).
+        """
+        # only want the _history index to reference the 1st index entry
+        # for version_id
+        if version_id not in self._cache:
+            index = len(self._history)
+            self._history.append(version_id)
+        else:
+            index = self._cache[version_id][5]
+        self._cache[version_id] = (version_id, 
+                                   options,
+                                   pos,
+                                   size,
+                                   parents,
+                                   index)
+
+    def __init__(self, transport, filename, mode, create=False):
+        _KnitComponentFile.__init__(self, transport, filename, mode)
+        self._cache = {}
+        # position in _history is the 'official' index for a revision
+        # but the values may have come from a newer entry.
+        # so - wc -l of a knit index is != the number of uniqe names
+        # in the weave.
+        self._history = []
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            count = 0
+            total = 1
+            try:
+                pb.update('read knit index', count, total)
+                fp = self._transport.get(self._filename)
+                self.check_header(fp)
+                # readlines reads the whole file at once:
+                # bad for transports like http, good for local disk
+                # we save 60 ms doing this one change (
+                # from calling readline each time to calling
+                # readlines once.
+                # probably what we want for nice behaviour on
+                # http is a incremental readlines that yields, or
+                # a check for local vs non local indexes,
+                for l in fp.readlines():
+                    rec = l.split()
+                    if len(rec) < 5 or rec[-1] != ':':
+                        # corrupt line.
+                        # FIXME: in the future we should determine if its a
+                        # short write - and ignore it 
+                        # or a different failure, and raise. RBC 20060407
+                        continue
+                    count += 1
+                    total += 1
+                    #pb.update('read knit index', count, total)
+                    # See self._parse_parents
+                    parents = []
+                    for value in rec[4:-1]:
+                        if '.' == value[0]:
+                            # uncompressed reference
+                            parents.append(value[1:])
+                        else:
+                            # this is 15/4000ms faster than isinstance,
+                            # (in lsprof)
+                            # this function is called thousands of times a 
+                            # second so small variations add up.
+                            assert value.__class__ is str
+                            parents.append(self._history[int(value)])
+                    # end self._parse_parents
+                    # self._cache_version(rec[0], 
+                    #                     rec[1].split(','),
+                    #                     int(rec[2]),
+                    #                     int(rec[3]),
+                    #                     parents)
+                    # --- self._cache_version
+                    # only want the _history index to reference the 1st 
+                    # index entry for version_id
+                    version_id = rec[0]
+                    if version_id not in self._cache:
+                        index = len(self._history)
+                        self._history.append(version_id)
+                    else:
+                        index = self._cache[version_id][5]
+                    self._cache[version_id] = (version_id,
+                                               rec[1].split(','),
+                                               int(rec[2]),
+                                               int(rec[3]),
+                                               parents,
+                                               index)
+                    # --- self._cache_version 
+            except NoSuchFile, e:
+                if mode != 'w' or not create:
+                    raise
+                self.write_header()
+        finally:
+            pb.update('read knit index', total, total)
+            pb.finished()
+
+    def _parse_parents(self, compressed_parents):
+        """convert a list of string parent values into version ids.
+
+        ints are looked up in the index.
+        .FOO values are ghosts and converted in to FOO.
+
+        NOTE: the function is retained here for clarity, and for possible
+              use in partial index reads. However bulk processing now has
+              it inlined in __init__ for inner-loop optimisation.
+        """
+        result = []
+        for value in compressed_parents:
+            if value[-1] == '.':
+                # uncompressed reference
+                result.append(value[1:])
+            else:
+                # this is 15/4000ms faster than isinstance,
+                # this function is called thousands of times a 
+                # second so small variations add up.
+                assert value.__class__ is str
+                result.append(self._history[int(value)])
+        return result
+
+    def get_graph(self):
+        graph = []
+        for version_id, index in self._cache.iteritems():
+            graph.append((version_id, index[4]))
+        return graph
+
+    def get_ancestry(self, versions):
+        """See VersionedFile.get_ancestry."""
+        # get a graph of all the mentioned versions:
+        graph = {}
+        pending = set(versions)
+        while len(pending):
+            version = pending.pop()
+            parents = self._cache[version][4]
+            # got the parents ok
+            # trim ghosts
+            parents = [parent for parent in parents if parent in self._cache]
+            for parent in parents:
+                # if not completed and not a ghost
+                if parent not in graph:
+                    pending.add(parent)
+            graph[version] = parents
+        return topo_sort(graph.items())
+
+    def get_ancestry_with_ghosts(self, versions):
+        """See VersionedFile.get_ancestry_with_ghosts."""
+        # get a graph of all the mentioned versions:
+        graph = {}
+        pending = set(versions)
+        while len(pending):
+            version = pending.pop()
+            try:
+                parents = self._cache[version][4]
+            except KeyError:
+                # ghost, fake it
+                graph[version] = []
+                pass
+            else:
+                # got the parents ok
+                for parent in parents:
+                    if parent not in graph:
+                        pending.add(parent)
+                graph[version] = parents
+        return topo_sort(graph.items())
+
+    def num_versions(self):
+        return len(self._history)
+
+    __len__ = num_versions
+
+    def get_versions(self):
+        return self._history
+
+    def idx_to_name(self, idx):
+        return self._history[idx]
+
+    def lookup(self, version_id):
+        assert version_id in self._cache
+        return self._cache[version_id][5]
+
+    def _version_list_to_index(self, versions):
+        result_list = []
+        for version in versions:
+            if version in self._cache:
+                # -- inlined lookup() --
+                result_list.append(str(self._cache[version][5]))
+                # -- end lookup () --
+            else:
+                result_list.append('.' + version.encode('utf-8'))
+        return ' '.join(result_list)
+
+    def add_version(self, version_id, options, pos, size, parents):
+        """Add a version record to the index."""
+        self._cache_version(version_id, options, pos, size, parents)
+
+        content = "\n%s %s %s %s %s :" % (version_id.encode('utf-8'),
+                                        ','.join(options),
+                                        pos,
+                                        size,
+                                        self._version_list_to_index(parents))
+        assert isinstance(content, str), 'content must be utf-8 encoded'
+        self._transport.append(self._filename, StringIO(content))
+
+    def has_version(self, version_id):
+        """True if the version is in the index."""
+        return self._cache.has_key(version_id)
+
+    def get_position(self, version_id):
+        """Return data position and size of specified version."""
+        return (self._cache[version_id][2], \
+                self._cache[version_id][3])
+
+    def get_method(self, version_id):
+        """Return compression method of specified version."""
+        options = self._cache[version_id][1]
+        if 'fulltext' in options:
+            return 'fulltext'
+        else:
+            assert 'line-delta' in options
+            return 'line-delta'
+
+    def get_options(self, version_id):
+        return self._cache[version_id][1]
+
+    def get_parents(self, version_id):
+        """Return parents of specified version ignoring ghosts."""
+        return [parent for parent in self._cache[version_id][4] 
+                if parent in self._cache]
+
+    def get_parents_with_ghosts(self, version_id):
+        """Return parents of specified version wth ghosts."""
+        return self._cache[version_id][4] 
+
+    def check_versions_present(self, version_ids):
+        """Check that all specified versions are present."""
+        version_ids = set(version_ids)
+        for version_id in list(version_ids):
+            if version_id in self._cache:
+                version_ids.remove(version_id)
+        if version_ids:
+            raise RevisionNotPresent(list(version_ids)[0], self.filename)
+
+
+class _KnitData(_KnitComponentFile):
+    """Contents of the knit data file"""
+
+    HEADER = "# bzr knit data 7\n"
+
+    def __init__(self, transport, filename, mode, create=False):
+        _KnitComponentFile.__init__(self, transport, filename, mode)
+        self._file = None
+        self._checked = False
+        if create:
+            self._transport.put(self._filename, StringIO(''))
+        self._records = {}
+
+    def clear_cache(self):
+        """Clear the record cache."""
+        self._records = {}
+
+    def _open_file(self):
+        if self._file is None:
+            try:
+                self._file = self._transport.get(self._filename)
+            except NoSuchFile:
+                pass
+        return self._file
+
+    def _record_to_data(self, version_id, digest, lines):
+        """Convert version_id, digest, lines into a raw data block.
+        
+        :return: (len, a StringIO instance with the raw data ready to read.)
+        """
+        sio = StringIO()
+        data_file = GzipFile(None, mode='wb', fileobj=sio)
+        data_file.writelines(chain(
+            ["version %s %d %s\n" % (version_id.encode('utf-8'), 
+                                     len(lines),
+                                     digest)],
+            lines,
+            ["end %s\n" % version_id.encode('utf-8')]))
+        data_file.close()
+        length= sio.tell()
+
+        sio.seek(0)
+        return length, sio
+
+    def add_raw_record(self, raw_data):
+        """Append a prepared record to the data file."""
+        assert isinstance(raw_data, str), 'data must be plain bytes'
+        start_pos = self._transport.append(self._filename, StringIO(raw_data))
+        return start_pos, len(raw_data)
+        
+    def add_record(self, version_id, digest, lines):
+        """Write new text record to disk.  Returns the position in the
+        file where it was written."""
+        size, sio = self._record_to_data(version_id, digest, lines)
+        # cache
+        self._records[version_id] = (digest, lines)
+        # write to disk
+        start_pos = self._transport.append(self._filename, sio)
+        return start_pos, size
+
+    def _parse_record_header(self, version_id, raw_data):
+        """Parse a record header for consistency.
+
+        :return: the header and the decompressor stream.
+                 as (stream, header_record)
+        """
+        df = GzipFile(mode='rb', fileobj=StringIO(raw_data))
+        rec = df.readline().split()
+        if len(rec) != 4:
+            raise KnitCorrupt(self._filename, 'unexpected number of elements in record header')
+        if rec[1].decode('utf-8')!= version_id:
+            raise KnitCorrupt(self._filename, 
+                              'unexpected version, wanted %r, got %r' % (
+                                version_id, rec[1]))
+        return df, rec
+
+    def _parse_record(self, version_id, data):
+        # profiling notes:
+        # 4168 calls in 2880 217 internal
+        # 4168 calls to _parse_record_header in 2121
+        # 4168 calls to readlines in 330
+        df, rec = self._parse_record_header(version_id, data)
+        record_contents = df.readlines()
+        l = record_contents.pop()
+        assert len(record_contents) == int(rec[2])
+        if l.decode('utf-8') != 'end %s\n' % version_id:
+            raise KnitCorrupt(self._filename, 'unexpected version end line %r, wanted %r' 
+                        % (l, version_id))
+        df.close()
+        return record_contents, rec[3]
+
+    def read_records_iter_raw(self, records):
+        """Read text records from data file and yield raw data.
+
+        This unpacks enough of the text record to validate the id is
+        as expected but thats all.
+
+        It will actively recompress currently cached records on the
+        basis that that is cheaper than I/O activity.
+        """
+        needed_records = []
+        for version_id, pos, size in records:
+            if version_id not in self._records:
+                needed_records.append((version_id, pos, size))
+
+        # setup an iterator of the external records:
+        # uses readv so nice and fast we hope.
+        if len(needed_records):
+            # grab the disk data needed.
+            raw_records = self._transport.readv(self._filename,
+                [(pos, size) for version_id, pos, size in needed_records])
+
+        for version_id, pos, size in records:
+            if version_id in self._records:
+                # compress a new version
+                size, sio = self._record_to_data(version_id,
+                                                 self._records[version_id][0],
+                                                 self._records[version_id][1])
+                yield version_id, sio.getvalue()
+            else:
+                pos, data = raw_records.next()
+                # validate the header
+                df, rec = self._parse_record_header(version_id, data)
+                df.close()
+                yield version_id, data
+
+
+    def read_records_iter(self, records):
+        """Read text records from data file and yield result.
+
+        Each passed record is a tuple of (version_id, pos, len) and
+        will be read in the given order.  Yields (version_id,
+        contents, digest).
+        """
+        # profiling notes:
+        # 60890  calls for 4168 extractions in 5045, 683 internal.
+        # 4168   calls to readv              in 1411
+        # 4168   calls to parse_record       in 2880
+
+        needed_records = []
+        for version_id, pos, size in records:
+            if version_id not in self._records:
+                needed_records.append((version_id, pos, size))
+
+        if len(needed_records):
+            # We take it that the transport optimizes the fetching as good
+            # as possible (ie, reads continous ranges.)
+            response = self._transport.readv(self._filename,
+                [(pos, size) for version_id, pos, size in needed_records])
+
+            for (record_id, pos, size), (pos, data) in izip(iter(needed_records), response):
+                content, digest = self._parse_record(record_id, data)
+                self._records[record_id] = (digest, content)
+    
+        for version_id, pos, size in records:
+            yield version_id, list(self._records[version_id][1]), self._records[version_id][0]
+
+    def read_records(self, records):
+        """Read records into a dictionary."""
+        components = {}
+        for record_id, content, digest in self.read_records_iter(records):
+            components[record_id] = (content, digest)
+        return components
+
+
+class InterKnit(InterVersionedFile):
+    """Optimised code paths for knit to knit operations."""
+    
+    _matching_file_factory = KnitVersionedFile
+    
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with knits.  """
+        try:
+            return (isinstance(source, KnitVersionedFile) and
+                    isinstance(target, KnitVersionedFile))
+        except AttributeError:
+            return False
+
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
+        """See InterVersionedFile.join."""
+        assert isinstance(self.source, KnitVersionedFile)
+        assert isinstance(self.target, KnitVersionedFile)
+
+        if version_ids is None:
+            version_ids = self.source.versions()
+        else:
+            if not ignore_missing:
+                self.source._check_versions_present(version_ids)
+            else:
+                version_ids = set(self.source.versions()).intersection(
+                    set(version_ids))
+
+        if not version_ids:
+            return 0
+
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            version_ids = list(version_ids)
+            if None in version_ids:
+                version_ids.remove(None)
+    
+            self.source_ancestry = set(self.source.get_ancestry(version_ids))
+            this_versions = set(self.target._index.get_versions())
+            needed_versions = self.source_ancestry - this_versions
+            cross_check_versions = self.source_ancestry.intersection(this_versions)
+            mismatched_versions = set()
+            for version in cross_check_versions:
+                # scan to include needed parents.
+                n1 = set(self.target.get_parents_with_ghosts(version))
+                n2 = set(self.source.get_parents_with_ghosts(version))
+                if n1 != n2:
+                    # FIXME TEST this check for cycles being introduced works
+                    # the logic is we have a cycle if in our graph we are an
+                    # ancestor of any of the n2 revisions.
+                    for parent in n2:
+                        if parent in n1:
+                            # safe
+                            continue
+                        else:
+                            parent_ancestors = self.source.get_ancestry(parent)
+                            if version in parent_ancestors:
+                                raise errors.GraphCycleError([parent, version])
+                    # ensure this parent will be available later.
+                    new_parents = n2.difference(n1)
+                    needed_versions.update(new_parents.difference(this_versions))
+                    mismatched_versions.add(version)
+    
+            if not needed_versions and not cross_check_versions:
+                return 0
+            full_list = topo_sort(self.source.get_graph())
+    
+            version_list = [i for i in full_list if (not self.target.has_version(i)
+                            and i in needed_versions)]
+    
+            # plan the join:
+            copy_queue = []
+            copy_queue_records = []
+            copy_set = set()
+            for version_id in version_list:
+                options = self.source._index.get_options(version_id)
+                parents = self.source._index.get_parents_with_ghosts(version_id)
+                # check that its will be a consistent copy:
+                for parent in parents:
+                    # if source has the parent, we must :
+                    # * already have it or
+                    # * have it scheduled already
+                    # otherwise we dont care
+                    assert (self.target.has_version(parent) or
+                            parent in copy_set or
+                            not self.source.has_version(parent))
+                data_pos, data_size = self.source._index.get_position(version_id)
+                copy_queue_records.append((version_id, data_pos, data_size))
+                copy_queue.append((version_id, options, parents))
+                copy_set.add(version_id)
+
+            # data suck the join:
+            count = 0
+            total = len(version_list)
+            # we want the raw gzip for bulk copying, but the record validated
+            # just enough to be sure its the right one.
+            # TODO: consider writev or write combining to reduce 
+            # death of a thousand cuts feeling.
+            for (version_id, raw_data), \
+                (version_id2, options, parents) in \
+                izip(self.source._data.read_records_iter_raw(copy_queue_records),
+                     copy_queue):
+                assert version_id == version_id2, 'logic error, inconsistent results'
+                count = count + 1
+                pb.update("Joining knit", count, total)
+                pos, size = self.target._data.add_raw_record(raw_data)
+                self.target._index.add_version(version_id, options, pos, size, parents)
+
+            for version in mismatched_versions:
+                # FIXME RBC 20060309 is this needed?
+                n1 = set(self.target.get_parents_with_ghosts(version))
+                n2 = set(self.source.get_parents_with_ghosts(version))
+                # write a combined record to our history preserving the current 
+                # parents as first in the list
+                new_parents = self.target.get_parents_with_ghosts(version) + list(n2.difference(n1))
+                self.target.fix_parents(version, new_parents)
+            return count
+        finally:
+            pb.finished()
+
+
+InterVersionedFile.register_optimiser(InterKnit)
+
+
+class SequenceMatcher(difflib.SequenceMatcher):
+    """Knit tuned sequence matcher.
+
+    This is based on profiling of difflib which indicated some improvements
+    for our usage pattern.
+    """
+
+    def find_longest_match(self, alo, ahi, blo, bhi):
+        """Find longest matching block in a[alo:ahi] and b[blo:bhi].
+
+        If isjunk is not defined:
+
+        Return (i,j,k) such that a[i:i+k] is equal to b[j:j+k], where
+            alo <= i <= i+k <= ahi
+            blo <= j <= j+k <= bhi
+        and for all (i',j',k') meeting those conditions,
+            k >= k'
+            i <= i'
+            and if i == i', j <= j'
+
+        In other words, of all maximal matching blocks, return one that
+        starts earliest in a, and of all those maximal matching blocks that
+        start earliest in a, return the one that starts earliest in b.
+
+        >>> s = SequenceMatcher(None, " abcd", "abcd abcd")
+        >>> s.find_longest_match(0, 5, 0, 9)
+        (0, 4, 5)
+
+        If isjunk is defined, first the longest matching block is
+        determined as above, but with the additional restriction that no
+        junk element appears in the block.  Then that block is extended as
+        far as possible by matching (only) junk elements on both sides.  So
+        the resulting block never matches on junk except as identical junk
+        happens to be adjacent to an "interesting" match.
+
+        Here's the same example as before, but considering blanks to be
+        junk.  That prevents " abcd" from matching the " abcd" at the tail
+        end of the second sequence directly.  Instead only the "abcd" can
+        match, and matches the leftmost "abcd" in the second sequence:
+
+        >>> s = SequenceMatcher(lambda x: x==" ", " abcd", "abcd abcd")
+        >>> s.find_longest_match(0, 5, 0, 9)
+        (1, 0, 4)
+
+        If no blocks match, return (alo, blo, 0).
+
+        >>> s = SequenceMatcher(None, "ab", "c")
+        >>> s.find_longest_match(0, 2, 0, 1)
+        (0, 0, 0)
+        """
+
+        # CAUTION:  stripping common prefix or suffix would be incorrect.
+        # E.g.,
+        #    ab
+        #    acab
+        # Longest matching block is "ab", but if common prefix is
+        # stripped, it's "a" (tied with "b").  UNIX(tm) diff does so
+        # strip, so ends up claiming that ab is changed to acab by
+        # inserting "ca" in the middle.  That's minimal but unintuitive:
+        # "it's obvious" that someone inserted "ac" at the front.
+        # Windiff ends up at the same place as diff, but by pairing up
+        # the unique 'b's and then matching the first two 'a's.
+
+        a, b, b2j, isbjunk = self.a, self.b, self.b2j, self.isbjunk
+        besti, bestj, bestsize = alo, blo, 0
+        # find longest junk-free match
+        # during an iteration of the loop, j2len[j] = length of longest
+        # junk-free match ending with a[i-1] and b[j]
+        j2len = {}
+        # nothing = []
+        b2jget = b2j.get
+        for i in xrange(alo, ahi):
+            # look at all instances of a[i] in b; note that because
+            # b2j has no junk keys, the loop is skipped if a[i] is junk
+            j2lenget = j2len.get
+            newj2len = {}
+            
+            # changing b2j.get(a[i], nothing) to a try:Keyerror pair produced the
+            # following improvement
+            #     704  0   4650.5320   2620.7410   bzrlib.knit:1336(find_longest_match)
+            # +326674  0   1655.1210   1655.1210   +<method 'get' of 'dict' objects>
+            #  +76519  0    374.6700    374.6700   +<method 'has_key' of 'dict' objects>
+            # to 
+            #     704  0   3733.2820   2209.6520   bzrlib.knit:1336(find_longest_match)
+            #  +211400 0   1147.3520   1147.3520   +<method 'get' of 'dict' objects>
+            #  +76519  0    376.2780    376.2780   +<method 'has_key' of 'dict' objects>
+
+            try:
+                js = b2j[a[i]]
+            except KeyError:
+                pass
+            else:
+                for j in js:
+                    # a[i] matches b[j]
+                    if j >= blo:
+                        if j >= bhi:
+                            break
+                        k = newj2len[j] = 1 + j2lenget(-1 + j, 0)
+                        if k > bestsize:
+                            besti, bestj, bestsize = 1 + i-k, 1 + j-k, k
+            j2len = newj2len
+
+        # Extend the best by non-junk elements on each end.  In particular,
+        # "popular" non-junk elements aren't in b2j, which greatly speeds
+        # the inner loop above, but also means "the best" match so far
+        # doesn't contain any junk *or* popular non-junk elements.
+        while besti > alo and bestj > blo and \
+              not isbjunk(b[bestj-1]) and \
+              a[besti-1] == b[bestj-1]:
+            besti, bestj, bestsize = besti-1, bestj-1, bestsize+1
+        while besti+bestsize < ahi and bestj+bestsize < bhi and \
+              not isbjunk(b[bestj+bestsize]) and \
+              a[besti+bestsize] == b[bestj+bestsize]:
+            bestsize += 1
+
+        # Now that we have a wholly interesting match (albeit possibly
+        # empty!), we may as well suck up the matching junk on each
+        # side of it too.  Can't think of a good reason not to, and it
+        # saves post-processing the (possibly considerable) expense of
+        # figuring out what to do with it.  In the case of an empty
+        # interesting match, this is clearly the right thing to do,
+        # because no other kind of match is possible in the regions.
+        while besti > alo and bestj > blo and \
+              isbjunk(b[bestj-1]) and \
+              a[besti-1] == b[bestj-1]:
+            besti, bestj, bestsize = besti-1, bestj-1, bestsize+1
+        while besti+bestsize < ahi and bestj+bestsize < bhi and \
+              isbjunk(b[bestj+bestsize]) and \
+              a[besti+bestsize] == b[bestj+bestsize]:
+            bestsize = bestsize + 1
+
+        return besti, bestj, bestsize
+

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2005-07-21 12:01:14 +0000
+++ bzrlib/lock.py	2006-03-03 07:31:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,35 +15,34 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-"""Locking wrappers.
+"""Locking using OS file locks or file existence.
 
-This only does local locking using OS locks for now.
+Note: This method of locking is generally deprecated in favour of LockDir, but
+is used to lock local WorkingTrees, and by some old formats.  It's accessed
+through Transport.lock_read(), etc.
 
 This module causes two methods, lock() and unlock() to be defined in
 any way that works on the current platform.
 
 It is not specified whether these locks are reentrant (i.e. can be
 taken repeatedly by a single process) or whether they exclude
-different threads in a single process.  
-
-Eventually we may need to use some kind of lock representation that
-will work on a dumb filesystem without actual locking primitives.
+different threads in a single process.  That reentrancy is provided by 
+LockableFiles.
 
 This defines two classes: ReadLock and WriteLock, which can be
 implemented in different ways on different platforms.  Both have an
 unlock() method.
 """
 
-
+import errno
+import os
 import sys
-import os
 
 from bzrlib.trace import mutter, note, warning
 from bzrlib.errors import LockError
 
 class _base_Lock(object):
     def _open(self, filename, filemode):
-        import errno
         try:
             self.f = open(filename, filemode)
             return self.f
@@ -52,19 +51,17 @@
                 raise
 
             # maybe this is an old branch (before may 2005)
-            mutter("trying to create missing branch lock %r" % filename)
+            mutter("trying to create missing branch lock %r", filename)
             
-            self.f = open(filename, 'wb')
+            self.f = open(filename, 'wb+')
             return self.f
 
-
     def __del__(self):
         if self.f:
             from warnings import warn
             warn("lock on %r not released" % self.f)
             self.unlock()
             
-
     def unlock(self):
         raise NotImplementedError()
 
@@ -84,37 +81,45 @@
         f = None
 
         def unlock(self):
-            fcntl.flock(self.f, fcntl.LOCK_UN)
+            fcntl.lockf(self.f, fcntl.LOCK_UN)
             self.f.close()
             del self.f 
 
-
     class _fcntl_WriteLock(_fcntl_FileLock):
         def __init__(self, filename):
+            # standard IO errors get exposed directly.
+            self._open(filename, 'wb')
             try:
-                fcntl.flock(self._open(filename, 'wb'), fcntl.LOCK_EX)
-            except Exception, e:
+                fcntl.lockf(self.f, fcntl.LOCK_EX)
+            except IOError, e:
+                # we should be more precise about whats a locking
+                # error and whats a random-other error
                 raise LockError(e)
 
-
     class _fcntl_ReadLock(_fcntl_FileLock):
+
         def __init__(self, filename):
+            # standard IO errors get exposed directly.
+            self._open(filename, 'rb')
             try:
-                fcntl.flock(self._open(filename, 'rb'), fcntl.LOCK_SH)
-            except Exception, e:
+                fcntl.lockf(self.f, fcntl.LOCK_SH)
+            except IOError, e:
+                # we should be more precise about whats a locking
+                # error and whats a random-other error
                 raise LockError(e)
 
     WriteLock = _fcntl_WriteLock
     ReadLock = _fcntl_ReadLock
 
+
 except ImportError:
     try:
         import win32con, win32file, pywintypes
 
 
-        #LOCK_SH = 0 # the default
-        #LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
-        #LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
+        LOCK_SH = 0 # the default
+        LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
+        LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
 
         class _w32c_FileLock(_base_Lock):
             def _lock(self, filename, openmode, lockmode):
@@ -139,12 +144,13 @@
 
         class _w32c_ReadLock(_w32c_FileLock):
             def __init__(self, filename):
-                _w32c_FileLock._lock(self, filename, 'rb', 0)
+                _w32c_FileLock._lock(self, filename, 'rb',
+                                     LOCK_NB)
 
         class _w32c_WriteLock(_w32c_FileLock):
             def __init__(self, filename):
                 _w32c_FileLock._lock(self, filename, 'wb',
-                                     win32con.LOCKFILE_EXCLUSIVE_LOCK)
+                                     LOCK_EX + LOCK_NB)
 
 
 
@@ -168,6 +174,8 @@
                 LOCK_NB = 4
                 def unlock(self):
                     _msvc_unlock(self.f)
+                    self.f.close()
+                    self.f = None
 
 
             class _msvc_ReadLock(_msvc_FileLock):
@@ -242,10 +250,3 @@
         except ImportError:
             raise NotImplementedError("please write a locking method "
                                       "for platform %r" % sys.platform)
-
-
-
-
-
-
-

=== added file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lockable_files.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,313 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import codecs
+#import traceback
+
+import bzrlib
+from bzrlib.decorators import *
+import bzrlib.errors as errors
+from bzrlib.errors import LockError, ReadOnlyError
+from bzrlib.osutils import file_iterator, safe_unicode
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter, note
+import bzrlib.transactions as transactions
+
+# XXX: The tracking here of lock counts and whether the lock is held is
+# somewhat redundant with what's done in LockDir; the main difference is that
+# LockableFiles permits reentrancy.
+
+class LockableFiles(object):
+    """Object representing a set of related files locked within the same scope.
+
+    These files are used by a WorkingTree, Repository or Branch, and should
+    generally only be touched by that object.
+
+    LockableFiles also provides some policy on top of Transport for encoding
+    control files as utf-8.
+
+    LockableFiles manage a lock count and can be locked repeatedly by
+    a single caller.  (The underlying lock implementation generally does not
+    support this.)
+
+    Instances of this class are often called control_files.
+    
+    This object builds on top of a Transport, which is used to actually write
+    the files to disk, and an OSLock or LockDir, which controls how access to
+    the files is controlled.  The particular type of locking used is set when
+    the object is constructed.  In older formats OSLocks are used everywhere.
+    in newer formats a LockDir is used for Repositories and Branches, and 
+    OSLocks for the local filesystem.
+    """
+
+    # _lock_mode: None, or 'r' or 'w'
+
+    # _lock_count: If _lock_mode is true, a positive count of the number of
+    # times the lock has been taken *by this process*.   
+    
+    # If set to False (by a plugin, etc) BzrBranch will not set the
+    # mode on created files or directories
+    _set_file_mode = True
+    _set_dir_mode = True
+
+    def __init__(self, transport, lock_name, lock_class):
+        """Create a LockableFiles group
+
+        :param transport: Transport pointing to the directory holding the 
+            control files and lock.
+        :param lock_name: Name of the lock guarding these files.
+        :param lock_class: Class of lock strategy to use: typically
+            either LockDir or TransportLock.
+        """
+        object.__init__(self)
+        self._transport = transport
+        self.lock_name = lock_name
+        self._transaction = None
+        self._find_modes()
+        self._lock_mode = None
+        self._lock_count = 0
+        esc_name = self._escape(lock_name)
+        self._lock = lock_class(transport, esc_name,
+                                file_modebits=self._file_mode,
+                                dir_modebits=self._dir_mode)
+
+    def create_lock(self):
+        """Create the lock.
+
+        This should normally be called only when the LockableFiles directory
+        is first created on disk.
+        """
+        self._lock.create(mode=self._dir_mode)
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__,
+                           self._transport)
+    def __str__(self):
+        return 'LockableFiles(%s, %s)' % (self.lock_name, self._transport.base)
+
+    def __del__(self):
+        if self.is_locked():
+            # XXX: This should show something every time, and be suitable for
+            # headless operation and embedding
+            from warnings import warn
+            warn("file group %r was not explicitly unlocked" % self)
+            self._lock.unlock()
+
+    def _escape(self, file_or_path):
+        if not isinstance(file_or_path, basestring):
+            file_or_path = '/'.join(file_or_path)
+        if file_or_path == '':
+            return u''
+        return bzrlib.transport.urlescape(safe_unicode(file_or_path))
+
+    def _find_modes(self):
+        """Determine the appropriate modes for files and directories."""
+        try:
+            st = self._transport.stat('.')
+        except errors.TransportNotPossible:
+            self._dir_mode = 0755
+            self._file_mode = 0644
+        else:
+            self._dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            self._file_mode = self._dir_mode & ~07111
+        if not self._set_dir_mode:
+            self._dir_mode = None
+        if not self._set_file_mode:
+            self._file_mode = None
+
+    def controlfilename(self, file_or_path):
+        """Return location relative to branch."""
+        return self._transport.abspath(self._escape(file_or_path))
+
+    @deprecated_method(zero_eight)
+    def controlfile(self, file_or_path, mode='r'):
+        """Open a control file for this branch.
+
+        There are two classes of file in a lockable directory: text
+        and binary.  binary files are untranslated byte streams.  Text
+        control files are stored with Unix newlines and in UTF-8, even
+        if the platform or locale defaults are different.
+
+        Such files are not openable in write mode : they are managed via
+        put and put_utf8 which atomically replace old versions using
+        atomicfile.
+        """
+
+        relpath = self._escape(file_or_path)
+        # TODO: codecs.open() buffers linewise, so it was overloaded with
+        # a much larger buffer, do we need to do the same for getreader/getwriter?
+        if mode == 'rb': 
+            return self.get(relpath)
+        elif mode == 'wb':
+            raise BzrError("Branch.controlfile(mode='wb') is not supported, use put[_utf8]")
+        elif mode == 'r':
+            return self.get_utf8(relpath)
+        elif mode == 'w':
+            raise BzrError("Branch.controlfile(mode='w') is not supported, use put[_utf8]")
+        else:
+            raise BzrError("invalid controlfile mode %r" % mode)
+
+    @needs_read_lock
+    def get(self, relpath):
+        """Get a file as a bytestream."""
+        relpath = self._escape(relpath)
+        return self._transport.get(relpath)
+
+    @needs_read_lock
+    def get_utf8(self, relpath):
+        """Get a file as a unicode stream."""
+        relpath = self._escape(relpath)
+        # DO NOT introduce an errors=replace here.
+        return codecs.getreader('utf-8')(self._transport.get(relpath))
+
+    @needs_write_lock
+    def put(self, path, file):
+        """Write a file.
+        
+        :param path: The path to put the file, relative to the .bzr control
+                     directory
+        :param f: A file-like or string object whose contents should be copied.
+        """
+        self._transport.put(self._escape(path), file, mode=self._file_mode)
+
+    @needs_write_lock
+    def put_utf8(self, path, a_string):
+        """Write a string, encoding as utf-8.
+
+        :param path: The path to put the string, relative to the transport root.
+        :param string: A file-like or string object whose contents should be copied.
+        """
+        # IterableFile would not be needed if Transport.put took iterables
+        # instead of files.  ADHB 2005-12-25
+        # RBC 20060103 surely its not needed anyway, with codecs transcode
+        # file support ?
+        # JAM 20060103 We definitely don't want encode(..., 'replace')
+        # these are valuable files which should have exact contents.
+        if not isinstance(a_string, basestring):
+            raise errors.BzrBadParameterNotString(a_string)
+        self.put(path, StringIO(a_string.encode('utf-8')))
+
+    def lock_write(self):
+        # mutter("lock write: %s (%s)", self, self._lock_count)
+        # TODO: Upgrade locking to support using a Transport,
+        # and potentially a remote locking protocol
+        if self._lock_mode:
+            if self._lock_mode != 'w' or not self.get_transaction().writeable():
+                raise ReadOnlyError(self)
+            self._lock_count += 1
+        else:
+            self._lock.lock_write()
+            #note('write locking %s', self)
+            #traceback.print_stack()
+            self._lock_mode = 'w'
+            self._lock_count = 1
+            self._set_transaction(transactions.WriteTransaction())
+
+    def lock_read(self):
+        # mutter("lock read: %s (%s)", self, self._lock_count)
+        if self._lock_mode:
+            assert self._lock_mode in ('r', 'w'), \
+                   "invalid lock mode %r" % self._lock_mode
+            self._lock_count += 1
+        else:
+            self._lock.lock_read()
+            #note('read locking %s', self)
+            #traceback.print_stack()
+            self._lock_mode = 'r'
+            self._lock_count = 1
+            self._set_transaction(transactions.ReadOnlyTransaction())
+            # 5K may be excessive, but hey, its a knob.
+            self.get_transaction().set_cache_size(5000)
+                        
+    def unlock(self):
+        # mutter("unlock: %s (%s)", self, self._lock_count)
+        if not self._lock_mode:
+            raise errors.LockNotHeld(self)
+        if self._lock_count > 1:
+            self._lock_count -= 1
+        else:
+            #note('unlocking %s', self)
+            #traceback.print_stack()
+            self._finish_transaction()
+            self._lock.unlock()
+            self._lock_mode = self._lock_count = None
+
+    def is_locked(self):
+        """Return true if this LockableFiles group is locked"""
+        return self._lock_count >= 1
+
+    def get_transaction(self):
+        """Return the current active transaction.
+
+        If no transaction is active, this returns a passthrough object
+        for which all data is immediately flushed and no caching happens.
+        """
+        if self._transaction is None:
+            return transactions.PassThroughTransaction()
+        else:
+            return self._transaction
+
+    def _set_transaction(self, new_transaction):
+        """Set a new active transaction."""
+        if self._transaction is not None:
+            raise errors.LockError('Branch %s is in a transaction already.' %
+                                   self)
+        self._transaction = new_transaction
+
+    def _finish_transaction(self):
+        """Exit the current transaction."""
+        if self._transaction is None:
+            raise errors.LockError('Branch %s is not in a transaction' %
+                                   self)
+        transaction = self._transaction
+        self._transaction = None
+        transaction.finish()
+
+
+class TransportLock(object):
+    """Locking method which uses transport-dependent locks.
+
+    On the local filesystem these transform into OS-managed locks.
+
+    These do not guard against concurrent access via different
+    transports.
+
+    This is suitable for use only in WorkingTrees (which are at present
+    always local).
+    """
+    def __init__(self, transport, escaped_name, file_modebits, dir_modebits):
+        self._transport = transport
+        self._escaped_name = escaped_name
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
+
+    def lock_write(self):
+        self._lock = self._transport.lock_write(self._escaped_name)
+
+    def lock_read(self):
+        self._lock = self._transport.lock_read(self._escaped_name)
+
+    def unlock(self):
+        self._lock.unlock()
+        self._lock = None
+
+    def create(self, mode=None):
+        """Create lock mechanism"""
+        # for old-style locks, create the file now
+        self._transport.put(self._escaped_name, StringIO(), 
+                            mode=self._file_modebits)

=== added file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lockdir.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,378 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""On-disk mutex protecting a resource
+
+bzr on-disk objects are locked by the existence of a directory with a
+particular name within the control directory.  We use this rather than OS
+internal locks (such as flock etc) because they can be seen across all
+transports, including http.
+
+Objects can be read if there is only physical read access; therefore 
+readers can never be required to create a lock, though they will
+check whether a writer is using the lock.  Writers can't detect
+whether anyone else is reading from the resource as they write.
+This works because of ordering constraints that make sure readers
+see a consistent view of existing data.
+
+Waiting for a lock must be done by polling; this can be aborted after
+a timeout.
+
+Locks must always be explicitly released, typically from a try/finally
+block -- they are not released from a finalizer or when Python
+exits.
+
+Locks may fail to be released if the process is abruptly terminated
+(machine stop, SIGKILL) or if a remote transport becomes permanently
+disconnected.  There is therefore a method to break an existing lock.
+This should rarely be used, and generally only with user approval.
+Locks contain some information on when the lock was taken and by who
+which may guide in deciding whether it can safely be broken.  (This is
+similar to the messages displayed by emacs and vim.) Note that if the
+lock holder is still alive they will get no notification that the lock
+has been broken and will continue their work -- so it is important to be
+sure they are actually dead.
+
+A lock is represented on disk by a directory of a particular name,
+containing an information file.  Taking a lock is done by renaming a
+temporary directory into place.  We use temporary directories because
+for all known transports and filesystems we believe that exactly one
+attempt to claim the lock will succeed and the others will fail.  (Files
+won't do because some filesystems or transports only have
+rename-and-overwrite, making it hard to tell who won.)
+
+The desired characteristics are:
+
+* Locks are not reentrant.  (That is, a client that tries to take a 
+  lock it already holds may deadlock or fail.)
+* Stale locks can be guessed at by a heuristic
+* Lost locks can be broken by any client
+* Failed lock operations leave little or no mess
+* Deadlocks are avoided by having a timeout always in use, clients
+  desiring indefinite waits can retry or set a silly big timeout.
+
+Storage formats use the locks, and also need to consider concurrency
+issues underneath the lock.  A format may choose not to use a lock
+at all for some operations.
+
+LockDirs always operate over a Transport.  The transport may be readonly, in
+which case the lock can be queried but not acquired.
+
+Locks are identified by a path name, relative to a base transport.
+
+Calling code will typically want to make sure there is exactly one LockDir
+object per actual lock on disk.  This module does nothing to prevent aliasing
+and deadlocks will likely occur if the locks are aliased.
+
+In the future we may add a "freshen" method which can be called
+by a lock holder to check that their lock has not been broken, and to 
+update the timestamp within it.
+
+Example usage:
+
+>>> from bzrlib.transport.memory import MemoryTransport
+>>> # typically will be obtained from a BzrDir, Branch, etc
+>>> t = MemoryTransport()
+>>> l = LockDir(t, 'sample-lock')
+>>> l.create()
+>>> l.wait_lock()
+>>> # do something here
+>>> l.unlock()
+
+"""
+
+import os
+import time
+from warnings import warn
+from StringIO import StringIO
+
+import bzrlib.config
+from bzrlib.errors import (
+        DirectoryNotEmpty,
+        FileExists,
+        LockBreakMismatch,
+        LockBroken,
+        LockContention,
+        LockError,
+        LockNotHeld,
+        NoSuchFile,
+        ResourceBusy,
+        UnlockableTransport,
+        )
+from bzrlib.transport import Transport
+from bzrlib.osutils import rand_chars
+from bzrlib.rio import RioWriter, read_stanza, Stanza
+
+# XXX: At the moment there is no consideration of thread safety on LockDir
+# objects.  This should perhaps be updated - e.g. if two threads try to take a
+# lock at the same time they should *both* get it.  But then that's unlikely
+# to be a good idea.
+
+# TODO: Transport could offer a simpler put() method that avoids the
+# rename-into-place for cases like creating the lock template, where there is
+# no chance that the file already exists.
+
+# TODO: Perhaps store some kind of note like the bzr command line in the lock
+# info?
+
+# TODO: Some kind of callback run while polling a lock to show progress
+# indicators.
+
+# TODO: Make sure to pass the right file and directory mode bits to all
+# files/dirs created.
+
+_DEFAULT_TIMEOUT_SECONDS = 300
+_DEFAULT_POLL_SECONDS = 0.5
+
+class LockDir(object):
+    """Write-lock guarding access to data."""
+
+    __INFO_NAME = '/info'
+
+    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755):
+        """Create a new LockDir object.
+
+        The LockDir is initially unlocked - this just creates the object.
+
+        :param transport: Transport which will contain the lock
+
+        :param path: Path to the lock within the base directory of the 
+            transport.
+        """
+        assert isinstance(transport, Transport), \
+            ("not a transport: %r" % transport)
+        self.transport = transport
+        self.path = path
+        self._lock_held = False
+        self._fake_read_lock = False
+        self._held_dir = path + '/held'
+        self._held_info_path = self._held_dir + self.__INFO_NAME
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
+        self.nonce = rand_chars(20)
+
+    def __repr__(self):
+        return '%s(%s%s)' % (self.__class__.__name__,
+                             self.transport.base,
+                             self.path)
+
+    is_held = property(lambda self: self._lock_held)
+
+    def create(self, mode=None):
+        """Create the on-disk lock.
+
+        This is typically only called when the object/directory containing the 
+        directory is first created.  The lock is not held when it's created.
+        """
+        if self.transport.is_readonly():
+            raise UnlockableTransport(self.transport)
+        self.transport.mkdir(self.path, mode=mode)
+
+    def attempt_lock(self):
+        """Take the lock; fail if it's already held.
+        
+        If you wish to block until the lock can be obtained, call wait_lock()
+        instead.
+        """
+        if self._fake_read_lock:
+            raise LockContention(self)
+        if self.transport.is_readonly():
+            raise UnlockableTransport(self.transport)
+        try:
+            tmpname = '%s/pending.%s.tmp' % (self.path, rand_chars(20))
+            self.transport.mkdir(tmpname)
+            sio = StringIO()
+            self._prepare_info(sio)
+            sio.seek(0)
+            self.transport.put(tmpname + self.__INFO_NAME, sio)
+            self.transport.rename(tmpname, self._held_dir)
+            self._lock_held = True
+            self.confirm()
+            return
+        except (DirectoryNotEmpty, FileExists, ResourceBusy), e:
+            pass
+        # fall through to here on contention
+        raise LockContention(self)
+
+    def unlock(self):
+        """Release a held lock
+        """
+        if self._fake_read_lock:
+            self._fake_read_lock = False
+            return
+        if not self._lock_held:
+            raise LockNotHeld(self)
+        # rename before deleting, because we can't atomically remove the whole
+        # tree
+        tmpname = '%s/releasing.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
+        self._lock_held = False
+        self.transport.delete(tmpname + self.__INFO_NAME)
+        self.transport.rmdir(tmpname)
+
+    def force_break(self, dead_holder_info):
+        """Release a lock held by another process.
+
+        WARNING: This should only be used when the other process is dead; if
+        it still thinks it has the lock there will be two concurrent writers.
+        In general the user's approval should be sought for lock breaks.
+
+        dead_holder_info must be the result of a previous LockDir.peek() call;
+        this is used to check that it's still held by the same process that
+        the user decided was dead.  If this is not the current holder,
+        LockBreakMismatch is raised.
+
+        After the lock is broken it will not be held by any process.
+        It is possible that another process may sneak in and take the 
+        lock before the breaking process acquires it.
+        """
+        if not isinstance(dead_holder_info, dict):
+            raise ValueError("dead_holder_info: %r" % dead_holder_info)
+        if self._lock_held:
+            raise AssertionError("can't break own lock: %r" % self)
+        current_info = self.peek()
+        if current_info is None:
+            # must have been recently released
+            return
+        if current_info != dead_holder_info:
+            raise LockBreakMismatch(self, current_info, dead_holder_info)
+        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
+        # check that we actually broke the right lock, not someone else;
+        # there's a small race window between checking it and doing the 
+        # rename.
+        broken_info_path = tmpname + self.__INFO_NAME
+        broken_info = self._read_info_file(broken_info_path)
+        if broken_info != dead_holder_info:
+            raise LockBreakMismatch(self, broken_info, dead_holder_info)
+        self.transport.delete(broken_info_path)
+        self.transport.rmdir(tmpname)
+
+    def confirm(self):
+        """Make sure that the lock is still held by this locker.
+
+        This should only fail if the lock was broken by user intervention,
+        or if the lock has been affected by a bug.
+
+        If the lock is not thought to be held, raises LockNotHeld.  If
+        the lock is thought to be held but has been broken, raises 
+        LockBroken.
+        """
+        if not self._lock_held:
+            raise LockNotHeld(self)
+        info = self.peek()
+        if info is None:
+            # no lock there anymore!
+            raise LockBroken(self)
+        if info.get('nonce') != self.nonce:
+            # there is a lock, but not ours
+            raise LockBroken(self)
+        
+    def _read_info_file(self, path):
+        """Read one given info file.
+
+        peek() reads the info file of the lock holder, if any.
+        """
+        return self._parse_info(self.transport.get(path))
+
+    def peek(self):
+        """Check if the lock is held by anyone.
+        
+        If it is held, this returns the lock info structure as a rio Stanza,
+        which contains some information about the current lock holder.
+        Otherwise returns None.
+        """
+        try:
+            info = self._read_info_file(self._held_info_path)
+            assert isinstance(info, dict), \
+                    "bad parse result %r" % info
+            return info
+        except NoSuchFile, e:
+            return None
+
+    def _prepare_info(self, outf):
+        """Write information about a pending lock to a temporary file.
+        """
+        import socket
+        # XXX: is creating this here inefficient?
+        config = bzrlib.config.GlobalConfig()
+        s = Stanza(hostname=socket.gethostname(),
+                   pid=str(os.getpid()),
+                   start_time=str(int(time.time())),
+                   nonce=self.nonce,
+                   user=config.user_email(),
+                   )
+        RioWriter(outf).write_stanza(s)
+
+    def _parse_info(self, info_file):
+        return read_stanza(info_file.readlines()).as_dict()
+
+    def wait_lock(self, timeout=_DEFAULT_TIMEOUT_SECONDS,
+                  poll=_DEFAULT_POLL_SECONDS):
+        """Wait a certain period for a lock.
+
+        If the lock can be acquired within the bounded time, it
+        is taken and this returns.  Otherwise, LockContention
+        is raised.  Either way, this function should return within
+        approximately `timeout` seconds.  (It may be a bit more if
+        a transport operation takes a long time to complete.)
+        """
+        # XXX: the transport interface doesn't let us guard 
+        # against operations there taking a long time.
+        deadline = time.time() + timeout
+        while True:
+            try:
+                self.attempt_lock()
+                return
+            except LockContention:
+                pass
+            if time.time() + poll < deadline:
+                time.sleep(poll)
+            else:
+                raise LockContention(self)
+
+    def lock_write(self):
+        """Wait for and acquire the lock."""
+        self.attempt_lock()
+
+    def lock_read(self):
+        """Compatability-mode shared lock.
+
+        LockDir doesn't support shared read-only locks, so this 
+        just pretends that the lock is taken but really does nothing.
+        """
+        # At the moment Branches are commonly locked for read, but 
+        # we can't rely on that remotely.  Once this is cleaned up,
+        # reenable this warning to prevent it coming back in 
+        # -- mbp 20060303
+        ## warn("LockDir.lock_read falls back to write lock")
+        if self._lock_held or self._fake_read_lock:
+            raise LockContention(self)
+        self._fake_read_lock = True
+
+    def wait(self, timeout=20, poll=0.5):
+        """Wait a certain period for a lock to be released."""
+        # XXX: the transport interface doesn't let us guard 
+        # against operations there taking a long time.
+        deadline = time.time() + timeout
+        while True:
+            if self.peek():
+                return
+            if time.time() + poll < deadline:
+                time.sleep(poll)
+            else:
+                raise LockContention(self)
+

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-08-01 20:10:24 +0000
+++ bzrlib/log.py	2006-03-28 14:29:13 +0000
@@ -50,10 +50,14 @@
 """
 
 
-from bzrlib.tree import EmptyTree
+# TODO: option to show delta summaries for merged-in revisions
+import re
+
 from bzrlib.delta import compare_trees
+import bzrlib.errors as errors
 from bzrlib.trace import mutter
-from bzrlib.errors import InvalidRevisionNumber
+from bzrlib.tree import EmptyTree
+from bzrlib.tsort import merge_sort
 
 
 def find_touching_revisions(branch, file_id):
@@ -71,7 +75,7 @@
     last_path = None
     revno = 1
     for revision_id in branch.revision_history():
-        this_inv = branch.get_revision_inventory(revision_id)
+        this_inv = branch.repository.get_revision_inventory(revision_id)
         if file_id in this_inv:
             this_ie = this_inv[file_id]
             this_path = this_inv.id2path(file_id)
@@ -110,6 +114,17 @@
     return rh
 
 
+def _get_revision_delta(branch, revno):
+    """Return the delta for a mainline revision.
+    
+    This is used to show summaries in verbose logs, and also for finding 
+    revisions which touch a given file."""
+    # XXX: What are we supposed to do when showing a summary for something 
+    # other than a mainline revision.  The delta to it's first parent, or
+    # (more useful) the delta to a nominated other revision.
+    return branch.get_revision_delta(revno)
+
+
 def show_log(branch,
              lf,
              specific_fileid=None,
@@ -140,6 +155,22 @@
     end_revision
         If not None, only show revisions <= end_revision
     """
+    branch.lock_read()
+    try:
+        _show_log(branch, lf, specific_fileid, verbose, direction,
+                  start_revision, end_revision, search)
+    finally:
+        branch.unlock()
+    
+def _show_log(branch,
+             lf,
+             specific_fileid=None,
+             verbose=False,
+             direction='reverse',
+             start_revision=None,
+             end_revision=None,
+             search=None):
+    """Worker function for show_log - see show_log."""
     from bzrlib.osutils import format_date
     from bzrlib.errors import BzrCheckError
     from bzrlib.textui import show_status
@@ -150,7 +181,7 @@
         warn("not a LogFormatter instance: %r" % lf)
 
     if specific_fileid:
-        mutter('get log for file_id %r' % specific_fileid)
+        mutter('get log for file_id %r', specific_fileid)
 
     if search is not None:
         import re
@@ -162,44 +193,69 @@
     
     if start_revision is None:
         start_revision = 1
-    elif start_revision < 1 or start_revision >= len(which_revs):
-        raise InvalidRevisionNumber(start_revision)
+    else:
+        branch.check_real_revno(start_revision)
     
     if end_revision is None:
         end_revision = len(which_revs)
-    elif end_revision < 1 or end_revision >= len(which_revs):
-        raise InvalidRevisionNumber(end_revision)
+    else:
+        branch.check_real_revno(end_revision)
 
     # list indexes are 0-based; revisions are 1-based
     cut_revs = which_revs[(start_revision-1):(end_revision)]
+    if not cut_revs:
+        return
+    # override the mainline to look like the revision history.
+    mainline_revs = [revision_id for index, revision_id in cut_revs]
+    if cut_revs[0][0] == 1:
+        mainline_revs.insert(0, None)
+    else:
+        mainline_revs.insert(0, which_revs[start_revision-2][1])
+
+    merge_sorted_revisions = merge_sort(
+        branch.repository.get_revision_graph(mainline_revs[-1]),
+        mainline_revs[-1],
+        mainline_revs)
 
     if direction == 'reverse':
         cut_revs.reverse()
     elif direction == 'forward':
-        pass
+        # forward means oldest first.
+        merge_sorted_revisions.reverse()
     else:
         raise ValueError('invalid direction %r' % direction)
 
-    for revno, rev_id in cut_revs:
-        if verbose or specific_fileid:
-            delta = branch.get_revision_delta(revno)
-            
-        if specific_fileid:
-            if not delta.touches_file_id(specific_fileid):
-                continue
-
-        if not verbose:
-            # although we calculated it, throw it away without display
-            delta = None
-
-        rev = branch.get_revision(rev_id)
+    revision_history = branch.revision_history()
+
+    # convert the revision history to a dictionary:
+    rev_nos = {}
+    for index, rev_id in cut_revs:
+        rev_nos[rev_id] = index
+
+    # now we just print all the revisions
+    for sequence, rev_id, merge_depth, end_of_merge in merge_sorted_revisions:
+        rev = branch.repository.get_revision(rev_id)
 
         if searchRE:
             if not searchRE.search(rev.message):
                 continue
 
-        lf.show(revno, rev, delta)
+        if merge_depth == 0:
+            # a mainline revision.
+            if verbose or specific_fileid:
+                delta = _get_revision_delta(branch, rev_nos[rev_id])
+                
+            if specific_fileid:
+                if not delta.touches_file_id(specific_fileid):
+                    continue
+    
+            if not verbose:
+                # although we calculated it, throw it away without display
+                delta = None
 
+            lf.show(rev_nos[rev_id], rev, delta)
+        elif hasattr(lf, 'show_merge'):
+            lf.show_merge(rev, merge_depth)
 
 
 def deltas_for_log_dummy(branch, which_revs):
@@ -294,80 +350,188 @@
 
     def show(self, revno, rev, delta):
         raise NotImplementedError('not implemented in abstract base')
-        
-
-
-
-
-
+
+    def short_committer(self, rev):
+        return re.sub('<.*@.*>', '', rev.committer).strip(' ')
+    
+    
 class LongLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
-        from osutils import format_date
-
+        return self._show_helper(revno=revno, rev=rev, delta=delta)
+
+    def show_merge(self, rev, merge_depth):
+        return self._show_helper(rev=rev, indent='    '*merge_depth, merged=True, delta=None)
+
+    def _show_helper(self, rev=None, revno=None, indent='', merged=False, delta=None):
+        """Show a revision, either merged or not."""
+        from bzrlib.osutils import format_date
         to_file = self.to_file
-
-        print >>to_file,  '-' * 60
-        print >>to_file,  'revno:', revno
+        print >>to_file,  indent+'-' * 60
+        if revno is not None:
+            print >>to_file,  'revno:', revno
+        if merged:
+            print >>to_file,  indent+'merged:', rev.revision_id
+        elif self.show_ids:
+            print >>to_file,  indent+'revision-id:', rev.revision_id
         if self.show_ids:
-            print >>to_file,  'revision-id:', rev.revision_id
-        print >>to_file,  'committer:', rev.committer
-
+            for parent_id in rev.parent_ids:
+                print >>to_file, indent+'parent:', parent_id
+        print >>to_file,  indent+'committer:', rev.committer
+        try:
+            print >>to_file, indent+'branch nick: %s' % \
+                rev.properties['branch-nick']
+        except KeyError:
+            pass
         date_str = format_date(rev.timestamp,
                                rev.timezone or 0,
                                self.show_timezone)
-        print >>to_file,  'timestamp: %s' % date_str
+        print >>to_file,  indent+'timestamp: %s' % date_str
 
-        print >>to_file,  'message:'
+        print >>to_file,  indent+'message:'
         if not rev.message:
-            print >>to_file,  '  (no message)'
+            print >>to_file,  indent+'  (no message)'
         else:
-            for l in rev.message.split('\n'):
-                print >>to_file,  '  ' + l
-
+            message = rev.message.rstrip('\r\n')
+            for l in message.split('\n'):
+                print >>to_file,  indent+'  ' + l
         if delta != None:
             delta.show(to_file, self.show_ids)
 
 
-
 class ShortLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
         from bzrlib.osutils import format_date
 
         to_file = self.to_file
-
-        print >>to_file, "%5d %s\t%s" % (revno, rev.committer,
+        date_str = format_date(rev.timestamp, rev.timezone or 0,
+                            self.show_timezone)
+        print >>to_file, "%5d %s\t%s" % (revno, self.short_committer(rev),
                 format_date(rev.timestamp, rev.timezone or 0,
-                            self.show_timezone))
+                            self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False))
         if self.show_ids:
             print >>to_file,  '      revision-id:', rev.revision_id
         if not rev.message:
             print >>to_file,  '      (no message)'
         else:
-            for l in rev.message.split('\n'):
+            message = rev.message.rstrip('\r\n')
+            for l in message.split('\n'):
                 print >>to_file,  '      ' + l
 
         # TODO: Why not show the modified files in a shorter form as
         # well? rewrap them single lines of appropriate length
         if delta != None:
             delta.show(to_file, self.show_ids)
-        print
-
-
-
-FORMATTERS = {'long': LongLogFormatter,
+        print >>to_file, ''
+
+class LineLogFormatter(LogFormatter):
+    def truncate(self, str, max_len):
+        if len(str) <= max_len:
+            return str
+        return str[:max_len-3]+'...'
+
+    def date_string(self, rev):
+        from bzrlib.osutils import format_date
+        return format_date(rev.timestamp, rev.timezone or 0, 
+                           self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False)
+
+    def message(self, rev):
+        if not rev.message:
+            return '(no message)'
+        else:
+            return rev.message
+
+    def show(self, revno, rev, delta):
+        print >> self.to_file, self.log_string(rev, 79) 
+
+    def log_string(self, rev, max_chars):
+        out = [self.truncate(self.short_committer(rev), 20)]
+        out.append(self.date_string(rev))
+        out.append(self.message(rev).replace('\n', ' '))
+        return self.truncate(" ".join(out).rstrip('\n'), max_chars)
+
+def line_log(rev, max_chars):
+    lf = LineLogFormatter(None)
+    return lf.log_string(rev, max_chars)
+
+FORMATTERS = {
+              'long': LongLogFormatter,
               'short': ShortLogFormatter,
+              'line': LineLogFormatter,
               }
 
+def register_formatter(name, formatter):
+    FORMATTERS[name] = formatter
 
 def log_formatter(name, *args, **kwargs):
+    """Construct a formatter from arguments.
+
+    name -- Name of the formatter to construct; currently 'long', 'short' and
+        'line' are supported.
+    """
     from bzrlib.errors import BzrCommandError
-    
     try:
         return FORMATTERS[name](*args, **kwargs)
-    except IndexError:
+    except KeyError:
         raise BzrCommandError("unknown log formatter: %r" % name)
 
 def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
     # deprecated; for compatability
     lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
     lf.show(revno, rev, delta)
+
+def show_changed_revisions(branch, old_rh, new_rh, to_file=None, log_format='long'):
+    """Show the change in revision history comparing the old revision history to the new one.
+
+    :param branch: The branch where the revisions exist
+    :param old_rh: The old revision history
+    :param new_rh: The new revision history
+    :param to_file: A file to write the results to. If None, stdout will be used
+    """
+    if to_file is None:
+        import sys
+        import codecs
+        import bzrlib
+        to_file = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+    lf = log_formatter(log_format,
+                       show_ids=False,
+                       to_file=to_file,
+                       show_timezone='original')
+
+    # This is the first index which is different between
+    # old and new
+    base_idx = None
+    for i in xrange(max(len(new_rh),
+                        len(old_rh))):
+        if (len(new_rh) <= i
+            or len(old_rh) <= i
+            or new_rh[i] != old_rh[i]):
+            base_idx = i
+            break
+
+    if base_idx is None:
+        to_file.write('Nothing seems to have changed\n')
+        return
+    ## TODO: It might be nice to do something like show_log
+    ##       and show the merged entries. But since this is the
+    ##       removed revisions, it shouldn't be as important
+    if base_idx < len(old_rh):
+        to_file.write('*'*60)
+        to_file.write('\nRemoved Revisions:\n')
+        for i in range(base_idx, len(old_rh)):
+            rev = branch.repository.get_revision(old_rh[i])
+            lf.show(i+1, rev, None)
+        to_file.write('*'*60)
+        to_file.write('\n\n')
+    if base_idx < len(new_rh):
+        to_file.write('Added Revisions:\n')
+        show_log(branch,
+                 lf,
+                 None,
+                 verbose=True,
+                 direction='forward',
+                 start_revision=base_idx+1,
+                 end_revision=len(new_rh),
+                 search=None)
+

=== added file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lsprof.py	2005-12-08 07:10:37 +0000
@@ -0,0 +1,108 @@
+# this is copied from the lsprof distro because somehow
+# it is not installed by distutils
+# I made one modification to profile so that it returns a pair
+# instead of just the Stats object
+
+import sys
+from _lsprof import Profiler, profiler_entry, profiler_subentry
+
+__all__ = ['profile', 'Stats']
+
+def profile(f, *args, **kwds):
+    """XXX docstring"""
+    p = Profiler()
+    p.enable(subcalls=True)
+    try:
+        ret = f(*args, **kwds)
+    finally:
+        p.disable()
+    return ret,Stats(p.getstats())
+
+
+class Stats(object):
+    """XXX docstring"""
+
+    def __init__(self, data):
+        self.data = data
+
+    def sort(self, crit="inlinetime"):
+        """XXX docstring"""
+        if crit not in profiler_entry.__dict__:
+            raise ValueError, "Can't sort by %s" % crit
+        self.data.sort(lambda b, a: cmp(getattr(a, crit),
+                                        getattr(b, crit)))
+        for e in self.data:
+            if e.calls:
+                e.calls.sort(lambda b, a: cmp(getattr(a, crit),
+                                              getattr(b, crit)))
+
+    def pprint(self, top=None, file=None):
+        """XXX docstring"""
+        if file is None:
+            file = sys.stdout
+        d = self.data
+        if top is not None:
+            d = d[:top]
+        cols = "% 12s %12s %11.4f %11.4f   %s\n"
+        hcols = "% 12s %12s %12s %12s %s\n"
+        cols2 = "+%12s %12s %11.4f %11.4f +  %s\n"
+        file.write(hcols % ("CallCount", "Recursive", "Total(ms)",
+                            "Inline(ms)", "module:lineno(function)"))
+        for e in d:
+            file.write(cols % (e.callcount, e.reccallcount, e.totaltime,
+                               e.inlinetime, label(e.code)))
+            if e.calls:
+                for se in e.calls:
+                    file.write(cols % ("+%s" % se.callcount, se.reccallcount,
+                                       se.totaltime, se.inlinetime,
+                                       "+%s" % label(se.code)))
+
+    def freeze(self):
+        """Replace all references to code objects with string
+        descriptions; this makes it possible to pickle the instance."""
+
+        # this code is probably rather ickier than it needs to be!
+        for i in range(len(self.data)):
+            e = self.data[i]
+            if not isinstance(e.code, str):
+                self.data[i] = type(e)((label(e.code),) + e[1:])
+                if e.calls:
+                    for j in range(len(e.calls)):
+                        se = e.calls[j]
+                        if not isinstance(se.code, str):
+                            e.calls[j] = type(se)((label(se.code),) + se[1:])
+
+_fn2mod = {}
+
+def label(code):
+    if isinstance(code, str):
+        return code
+    try:
+        mname = _fn2mod[code.co_filename]
+    except KeyError:
+        for k, v in sys.modules.iteritems():
+            if v is None:
+                continue
+            if not hasattr(v, '__file__'):
+                continue
+            if not isinstance(v.__file__, str):
+                continue
+            if v.__file__.startswith(code.co_filename):
+                mname = _fn2mod[code.co_filename] = k
+                break
+        else:
+            mname = _fn2mod[code.co_filename] = '<%s>'%code.co_filename
+    
+    return '%s:%d(%s)' % (mname, code.co_firstlineno, code.co_name)
+
+
+if __name__ == '__main__':
+    import os
+    sys.argv = sys.argv[1:]
+    if not sys.argv:
+        print >> sys.stderr, "usage: lsprof.py <script> <arguments...>"
+        sys.exit(2)
+    sys.path.insert(0, os.path.abspath(os.path.dirname(sys.argv[0])))
+    stats = profile(execfile, sys.argv[0], globals(), locals())
+    stats.sort()
+    stats.pprint()

=== removed file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-08-11 18:02:01 +0000
+++ bzrlib/mdiff.py	1970-01-01 00:00:00 +0000
@@ -1,163 +0,0 @@
-# (C) 2005 Matt Mackall
-# (C) 2005 Canonical Ltd
-
-# based on code by Matt Mackall, hacked by Martin Pool
-
-# mm's code works line-by-line; this just works on byte strings.
-# Possibly slower; possibly gives better results for code not
-# regularly separated by newlines and anyhow a bit simpler.
-
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-# TODO: maybe work on files not strings?
-
-# FIXME: doesn't work properly on files without trailing newlines
-
-import unittest
-import difflib, sys, struct
-from cStringIO import StringIO
-
-def linesplit(a):
-    """Split into two lists: content and line positions.
-
-    This returns (al, ap).
-
-    al[i] is the string content of line i of the file, including its
-    newline (if any).
-
-    ap[i] is the byte position in the file where that line starts.
-
-    ap[-1] is the byte position of the end of the file (i.e. the
-    length of the file.)
-
-    This transformation allows us to do a line-based diff and then map
-    back to byte positions.
-    """
-
-    al, ap = [], []
-    last = 0
-
-    n = a.find("\n") + 1
-    while n > 0:
-        ap.append(last)
-        al.append(a[last:n])
-        last = n
-        n = a.find("\n", n) + 1
-
-    if last < len(a):
-        al.append(a[last:])
-        ap.append(last)
-
-    # position at the end
-    ap.append(len(a))
-
-    return (al, ap)
-
-
-def diff(a, b):
-    # TODO: Use different splits, perhaps rsync-like, for binary files?
-    
-    (al, ap) = linesplit(a)
-    (bl, bp) = linesplit(b)
-
-    d = difflib.SequenceMatcher(None, al, bl)
-    
-    ## sys.stderr.write('  ~ real_quick_ratio: %.4f\n' % d.real_quick_ratio())
-    
-    for o, m, n, s, t in d.get_opcodes():
-        if o == 'equal': continue
-        # a[m:n] should be replaced by b[s:t]
-        if s == t:
-            yield ap[m], ap[n], ''
-        else:
-            yield ap[m], ap[n], ''.join(bl[s:t])
-
-
-def tobinary(ops):
-    b = StringIO()
-    for f in ops:
-        b.write(struct.pack(">III", f[0], f[1], len(f[2])))
-        b.write(f[2])
-    return b.getvalue()
-
-
-def bdiff(a, b):
-    return tobinary(diff(a, b))
-
-
-def patch(t, ops):
-    last = 0
-    b = StringIO()
-
-    for m, n, r in ops:
-        b.write(t[last:m])
-        if r:
-            b.write(r)
-        last = n
-        
-    b.write(t[last:])
-    return b.getvalue()
-
-
-def frombinary(b):
-    bin = StringIO(b)
-    while True:
-        p = bin.read(12)
-        if not p:
-            break
-
-        m, n, l = struct.unpack(">III", p)
-        
-        if l == 0:
-            r = ''
-        else:
-            r = bin.read(l)
-            if len(r) != l:
-                raise Exception("truncated patch data")
-            
-        yield m, n, r
-
-
-def bpatch(t, b):
-    return patch(t, frombinary(b))
-
-
-
-
-class TestDiffPatch(unittest.TestCase):
-    def doDiffPatch(self, old, new):
-        diff = bdiff(old, new)
-        result = bpatch(old, diff)
-        self.assertEquals(new, result)
-
-
-    def testSimpleDiff(self):
-        """Simply add a line at the end"""
-        self.doDiffPatch('a\nb\n', 'a\nb\nc\n')
-        
-
-        
-    def testTrailingLine(self):
-        """Test diff that adds an unterminated line.
-
-        (Old versions didn't do this properly.)"""
-        self.doDiffPatch('a\nb\nc\n',
-                         'a\nb\nc\nd')
-
-
-if __name__ == '__main__':
-    unittest.main()

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-08-17 08:32:56 +0000
+++ bzrlib/merge.py	2006-04-19 08:31:08 +0000
@@ -1,360 +1,909 @@
-from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
-from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
-from bzrlib.changeset import Inventory, Diff3Merge
-from bzrlib import find_branch
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+import errno
+from shutil import rmtree
+from tempfile import mkdtemp
+
+import bzrlib
+from bzrlib.branch import Branch
+from bzrlib.conflicts import ConflictList, Conflict
+from bzrlib.delta import compare_trees
+from bzrlib.errors import (BzrCommandError,
+                           BzrError,
+                           NoCommonAncestor,
+                           NoCommits,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           NotBranchError,
+                           NotVersionedError,
+                           UnrelatedBranches,
+                           UnsupportedOperation,
+                           WorkingTreeNotRevision,
+                           )
+from bzrlib.merge3 import Merge3
 import bzrlib.osutils
-from bzrlib.errors import BzrCommandError
-from bzrlib.delta import compare_trees
-from trace import mutter, warning
-import os.path
-import tempfile
-import shutil
-import errno
-
-
-# comments from abentley on irc: merge happens in two stages, each
-# of which generates a changeset object
-
-# stage 1: generate OLD->OTHER,
-# stage 2: use MINE and OLD->OTHER to generate MINE -> RESULT
-
-class UnrelatedBranches(BzrCommandError):
-    def __init__(self):
-        msg = "Branches have no common ancestor, and no base revision"\
-            " specified."
-        BzrCommandError.__init__(self, msg)
-
-
-class MergeConflictHandler(ExceptionConflictHandler):
-    """Handle conflicts encountered while merging.
-
-    This subclasses ExceptionConflictHandler, so that any types of
-    conflict that are not explicitly handled cause an exception and
-    terminate the merge.
-    """
-    def __init__(self, dir, ignore_zero=False):
-        ExceptionConflictHandler.__init__(self, dir)
-        self.conflicts = 0
-        self.ignore_zero = ignore_zero
-
-    def copy(self, source, dest):
-        """Copy the text and mode of a file
-        :param source: The path of the file to copy
-        :param dest: The distination file to create
-        """
-        s_file = file(source, "rb")
-        d_file = file(dest, "wb")
-        for line in s_file:
-            d_file.write(line)
-        os.chmod(dest, 0777 & os.stat(source).st_mode)
-
-    def dump(self, lines, dest):
-        """Copy the text and mode of a file
-        :param source: The path of the file to copy
-        :param dest: The distination file to create
-        """
-        d_file = file(dest, "wb")
-        for line in lines:
-            d_file.write(line)
-
-    def add_suffix(self, name, suffix, last_new_name=None):
-        """Rename a file to append a suffix.  If the new name exists, the
-        suffix is added repeatedly until a non-existant name is found
-
-        :param name: The path of the file
-        :param suffix: The suffix to append
-        :param last_new_name: (used for recursive calls) the last name tried
-        """
-        if last_new_name is None:
-            last_new_name = name
-        new_name = last_new_name+suffix
-        try:
-            os.rename(name, new_name)
-            return new_name
-        except OSError, e:
-            if e.errno != errno.EEXIST and e.errno != errno.ENOTEMPTY:
-                raise
-            return self.add_suffix(name, suffix, last_new_name=new_name)
-
-    def conflict(self, text):
-        warning(text)
-        self.conflicts += 1
-        
-
-    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
-        """
-        Handle diff3 conflicts by producing a .THIS, .BASE and .OTHER.  The
-        main file will be a version with diff3 conflicts.
-        :param new_file: Path to the output file with diff3 markers
-        :param this_path: Path to the file text for the THIS tree
-        :param base_path: Path to the file text for the BASE tree
-        :param other_path: Path to the file text for the OTHER tree
-        """
-        self.add_suffix(this_path, ".THIS")
-        self.dump(base_lines, this_path+".BASE")
-        self.dump(other_lines, this_path+".OTHER")
-        os.rename(new_file, this_path)
-        self.conflict("Diff3 conflict encountered in %s" % this_path)
-
-    def new_contents_conflict(self, filename, other_contents):
-        """Conflicting contents for newly added file."""
-        self.copy(other_contents, filename + ".OTHER")
-        self.conflict("Conflict in newly added file %s" % filename)
-    
-
-    def target_exists(self, entry, target, old_path):
-        """Handle the case when the target file or dir exists"""
-        moved_path = self.add_suffix(target, ".moved")
-        self.conflict("Moved existing %s to %s" % (target, moved_path))
-
-    def rmdir_non_empty(self, filename):
-        """Handle the case where the dir to be removed still has contents"""
-        self.conflict("Directory %s not removed because it is not empty"\
-            % filename)
-        return "skip"
-
-    def finalize(self):
-        if not self.ignore_zero:
-            print "%d conflicts encountered.\n" % self.conflicts
-            
-def get_tree(treespec, temp_root, label):
+from bzrlib.osutils import rename, pathjoin
+from progress import DummyProgress, ProgressPhase
+from bzrlib.revision import common_ancestor, is_ancestor, NULL_REVISION
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter, warning, note
+from bzrlib.transform import (TreeTransform, resolve_conflicts, cook_conflicts,
+                              FinalPaths, create_by_entry, unique_add)
+from bzrlib.versionedfile import WeaveMerge
+import bzrlib.ui
+
+# TODO: Report back as changes are merged in
+
+def _get_tree(treespec, local_branch=None):
     location, revno = treespec
-    branch = find_branch(location)
+    branch = Branch.open_containing(location)[0]
     if revno is None:
-        base_tree = branch.working_tree()
+        revision = None
     elif revno == -1:
-        base_tree = branch.basis_tree()
-    else:
-        base_tree = branch.revision_tree(branch.lookup_revision(revno))
-    temp_path = os.path.join(temp_root, label)
-    os.mkdir(temp_path)
-    return branch, MergeTree(base_tree, temp_path)
-
-
-def file_exists(tree, file_id):
-    return tree.has_filename(tree.id2path(file_id))
-    
-
-class MergeTree(object):
-    def __init__(self, tree, tempdir):
+        revision = branch.last_revision()
+    else:
+        revision = branch.get_rev_id(revno)
+        if revision is None:
+            revision = NULL_REVISION
+    return branch, _get_revid_tree(branch, revision, local_branch)
+
+
+def _get_revid_tree(branch, revision, local_branch):
+    if revision is None:
+        base_tree = branch.bzrdir.open_workingtree()
+    else:
+        if local_branch is not None:
+            if local_branch.base != branch.base:
+                local_branch.fetch(branch, revision)
+            base_tree = local_branch.repository.revision_tree(revision)
+        else:
+            base_tree = branch.repository.revision_tree(revision)
+    return base_tree
+
+
+def transform_tree(from_tree, to_tree, interesting_ids=None):
+    merge_inner(from_tree.branch, to_tree, from_tree, ignore_zero=True,
+                interesting_ids=interesting_ids, this_tree=from_tree)
+
+
+class Merger(object):
+    def __init__(self, this_branch, other_tree=None, base_tree=None, 
+                 this_tree=None, pb=DummyProgress()):
         object.__init__(self)
-        if hasattr(tree, "basedir"):
-            self.root = tree.basedir
-        else:
-            self.root = None
-        self.tree = tree
-        self.tempdir = tempdir
-        os.mkdir(os.path.join(self.tempdir, "texts"))
-        self.cached = {}
-
-    def __iter__(self):
-        return self.tree.__iter__()
-
-    def __contains__(self, file_id):
-        return file_id in self.tree
-
-    def get_file(self, file_id):
-        return self.tree.get_file(file_id)
-
-    def get_file_sha1(self, id):
-        return self.tree.get_file_sha1(id)
-
-    def id2path(self, file_id):
-        return self.tree.id2path(file_id)
-
-    def has_id(self, file_id):
-        return self.tree.has_id(file_id)
-
-    def readonly_path(self, id):
-        if id not in self.tree:
-            return None
-        if self.root is not None:
-            return self.tree.abspath(self.tree.id2path(id))
-        else:
-            if self.tree.inventory[id].kind in ("directory", "root_directory"):
-                return self.tempdir
-            if not self.cached.has_key(id):
-                path = os.path.join(self.tempdir, "texts", id)
-                outfile = file(path, "wb")
-                outfile.write(self.tree.get_file(id).read())
-                assert(os.path.exists(path))
-                self.cached[id] = path
-            return self.cached[id]
-
-
-
-def merge(other_revision, base_revision,
-          check_clean=True, ignore_zero=False,
-          this_dir=None, backup_files=False, merge_type=ApplyMerge3,
-          file_list=None):
-    """Merge changes into a tree.
-
-    base_revision
-        Base for three-way merge.
-    other_revision
-        Other revision for three-way merge.
-    this_dir
-        Directory to merge changes into; '.' by default.
-    check_clean
-        If true, this_dir must have no uncommitted changes before the
-        merge begins.
-    """
-    tempdir = tempfile.mkdtemp(prefix="bzr-")
-    try:
-        if this_dir is None:
-            this_dir = '.'
-        this_branch = find_branch(this_dir)
+        assert this_tree is not None, "this_tree is required"
+        self.this_branch = this_branch
+        self.this_basis = this_branch.last_revision()
+        self.this_rev_id = None
+        self.this_tree = this_tree
+        self.this_revision_tree = None
+        self.this_basis_tree = None
+        self.other_tree = other_tree
+        self.base_tree = base_tree
+        self.ignore_zero = False
+        self.backup_files = False
+        self.interesting_ids = None
+        self.show_base = False
+        self.reprocess = False
+        self._pb = pb 
+        self.pp = None
+
+
+    def revision_tree(self, revision_id):
+        return self.this_branch.repository.revision_tree(revision_id)
+
+    def ensure_revision_trees(self):
+        if self.this_revision_tree is None:
+            self.this_basis_tree = self.this_branch.repository.revision_tree(
+                self.this_basis)
+            if self.this_basis == self.this_rev_id:
+                self.this_revision_tree = self.this_basis_tree
+
+        if self.other_rev_id is None:
+            other_basis_tree = self.revision_tree(self.other_basis)
+            changes = compare_trees(self.other_tree, other_basis_tree)
+            if changes.has_changed():
+                raise WorkingTreeNotRevision(self.this_tree)
+            other_rev_id = other_basis
+            self.other_tree = other_basis_tree
+
+    def file_revisions(self, file_id):
+        self.ensure_revision_trees()
+        def get_id(tree, file_id):
+            revision_id = tree.inventory[file_id].revision
+            assert revision_id is not None
+            return revision_id
+        if self.this_rev_id is None:
+            if self.this_basis_tree.get_file_sha1(file_id) != \
+                self.this_tree.get_file_sha1(file_id):
+                raise WorkingTreeNotRevision(self.this_tree)
+
+        trees = (self.this_basis_tree, self.other_tree)
+        return [get_id(tree, file_id) for tree in trees]
+
+    def check_basis(self, check_clean):
+        if self.this_basis is None:
+            raise BzrCommandError("This branch has no commits")
         if check_clean:
-            changes = compare_trees(this_branch.working_tree(), 
-                                    this_branch.basis_tree(), False)
-            if changes.has_changed():
+            self.compare_basis()
+            if self.this_basis != self.this_rev_id:
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other")
+
+    def compare_basis(self):
+        changes = compare_trees(self.this_tree, 
+                                self.this_tree.basis_tree(), False)
+        if not changes.has_changed():
+            self.this_rev_id = self.this_basis
+
+    def set_interesting_files(self, file_list):
+        try:
+            self._set_interesting_files(file_list)
+        except NotVersionedError, e:
+            raise BzrCommandError("%s is not a source file in any"
+                                      " tree." % e.path)
+
+    def _set_interesting_files(self, file_list):
+        """Set the list of interesting ids from a list of files."""
+        if file_list is None:
+            self.interesting_ids = None
+            return
+
+        interesting_ids = set()
+        for path in file_list:
+            found_id = False
+            for tree in (self.this_tree, self.base_tree, self.other_tree):
+                file_id = tree.inventory.path2id(path)
+                if file_id is not None:
+                    interesting_ids.add(file_id)
+                    found_id = True
+            if not found_id:
+                raise NotVersionedError(path=path)
+        self.interesting_ids = interesting_ids
+
+    def set_pending(self):
+        if not self.base_is_ancestor:
+            return
+        if self.other_rev_id is None:
+            return
+        ancestry = self.this_branch.repository.get_ancestry(self.this_basis)
+        if self.other_rev_id in ancestry:
+            return
+        self.this_tree.add_pending_merge(self.other_rev_id)
+
+    def set_other(self, other_revision):
+        other_branch, self.other_tree = _get_tree(other_revision, 
+                                                  self.this_branch)
+        if other_revision[1] == -1:
+            self.other_rev_id = other_branch.last_revision()
+            if self.other_rev_id is None:
+                raise NoCommits(other_branch)
+            self.other_basis = self.other_rev_id
+        elif other_revision[1] is not None:
+            self.other_rev_id = other_branch.get_rev_id(other_revision[1])
+            self.other_basis = self.other_rev_id
+        else:
+            self.other_rev_id = None
+            self.other_basis = other_branch.last_revision()
+            if self.other_basis is None:
+                raise NoCommits(other_branch)
+        if other_branch.base != self.this_branch.base:
+            self.this_branch.fetch(other_branch, last_revision=self.other_basis)
+
+    def set_base(self, base_revision):
+        mutter("doing merge() with no base_revision specified")
         if base_revision == [None, None]:
-            if other_revision[1] == -1:
-                o_revno = None
-            else:
-                o_revno = other_revision[1]
-            base_revno = this_branch.common_ancestor(other_branch, 
-                                                     other_revno=o_revno)[0]
-            if base_revno is None:
+            try:
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    this_repo = self.this_branch.repository
+                    self.base_rev_id = common_ancestor(self.this_basis, 
+                                                       self.other_basis, 
+                                                       this_repo, pb)
+                finally:
+                    pb.finished()
+            except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_revision = ['.', base_revno]
-        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
-        if file_list is None:
-            interesting_ids = None
-        else:
-            interesting_ids = set()
-            this_tree = this_branch.working_tree()
-            for fname in file_list:
-                path = this_branch.relpath(fname)
-                found_id = False
-                for tree in (this_tree, base_tree.tree, other_tree.tree):
-                    file_id = tree.inventory.path2id(path)
+            self.base_tree = _get_revid_tree(self.this_branch, self.base_rev_id,
+                                            None)
+            self.base_is_ancestor = True
+        else:
+            base_branch, self.base_tree = _get_tree(base_revision)
+            if base_revision[1] == -1:
+                self.base_rev_id = base_branch.last_revision()
+            elif base_revision[1] is None:
+                self.base_rev_id = None
+            else:
+                self.base_rev_id = base_branch.get_rev_id(base_revision[1])
+            if self.this_branch.base != base_branch.base:
+                self.this_branch.fetch(base_branch)
+            self.base_is_ancestor = is_ancestor(self.this_basis, 
+                                                self.base_rev_id,
+                                                self.this_branch)
+
+    def do_merge(self):
+        kwargs = {'working_tree':self.this_tree, 'this_tree': self.this_tree, 
+                  'other_tree': self.other_tree, 
+                  'interesting_ids': self.interesting_ids,
+                  'pp': self.pp}
+        if self.merge_type.requires_base:
+            kwargs['base_tree'] = self.base_tree
+        if self.merge_type.supports_reprocess:
+            kwargs['reprocess'] = self.reprocess
+        elif self.reprocess:
+            raise BzrError("Conflict reduction is not supported for merge"
+                                  " type %s." % self.merge_type)
+        if self.merge_type.supports_show_base:
+            kwargs['show_base'] = self.show_base
+        elif self.show_base:
+            raise BzrError("Showing base is not supported for this"
+                                  " merge type. %s" % self.merge_type)
+        merge = self.merge_type(pb=self._pb, **kwargs)
+        if len(merge.cooked_conflicts) == 0:
+            if not self.ignore_zero:
+                note("All changes applied successfully.")
+        else:
+            note("%d conflicts encountered." % len(merge.cooked_conflicts))
+
+        return len(merge.cooked_conflicts)
+
+    def regen_inventory(self, new_entries):
+        old_entries = self.this_tree.read_working_inventory()
+        new_inventory = {}
+        by_path = {}
+        new_entries_map = {} 
+        for path, file_id in new_entries:
+            if path is None:
+                continue
+            new_entries_map[file_id] = path
+
+        def id2path(file_id):
+            path = new_entries_map.get(file_id)
+            if path is not None:
+                return path
+            entry = old_entries[file_id]
+            if entry.parent_id is None:
+                return entry.name
+            return pathjoin(id2path(entry.parent_id), entry.name)
+            
+        for file_id in old_entries:
+            entry = old_entries[file_id]
+            path = id2path(file_id)
+            if file_id in self.base_tree.inventory:
+                executable = getattr(self.base_tree.inventory[file_id], 'executable', False)
+            else:
+                executable = getattr(entry, 'executable', False)
+            new_inventory[file_id] = (path, file_id, entry.parent_id, 
+                                      entry.kind, executable)
+                                      
+            by_path[path] = file_id
+        
+        deletions = 0
+        insertions = 0
+        new_path_list = []
+        for path, file_id in new_entries:
+            if path is None:
+                del new_inventory[file_id]
+                deletions += 1
+            else:
+                new_path_list.append((path, file_id))
+                if file_id not in old_entries:
+                    insertions += 1
+        # Ensure no file is added before its parent
+        new_path_list.sort()
+        for path, file_id in new_path_list:
+            if path == '':
+                parent = None
+            else:
+                parent = by_path[os.path.dirname(path)]
+            abspath = pathjoin(self.this_tree.basedir, path)
+            kind = bzrlib.osutils.file_kind(abspath)
+            if file_id in self.base_tree.inventory:
+                executable = getattr(self.base_tree.inventory[file_id], 'executable', False)
+            else:
+                executable = False
+            new_inventory[file_id] = (path, file_id, parent, kind, executable)
+            by_path[path] = file_id 
+
+        # Get a list in insertion order
+        new_inventory_list = new_inventory.values()
+        mutter ("""Inventory regeneration:
+    old length: %i insertions: %i deletions: %i new_length: %i"""\
+            % (len(old_entries), insertions, deletions, 
+               len(new_inventory_list)))
+        assert len(new_inventory_list) == len(old_entries) + insertions\
+            - deletions
+        new_inventory_list.sort()
+        return new_inventory_list
+
+
+class Merge3Merger(object):
+    """Three-way merger that uses the merge3 text merger"""
+    requires_base = True
+    supports_reprocess = True
+    supports_show_base = True
+    history_based = False
+
+    def __init__(self, working_tree, this_tree, base_tree, other_tree, 
+                 interesting_ids=None, reprocess=False, show_base=False,
+                 pb=DummyProgress(), pp=None):
+        """Initialize the merger object and perform the merge."""
+        object.__init__(self)
+        self.this_tree = working_tree
+        self.base_tree = base_tree
+        self.other_tree = other_tree
+        self._raw_conflicts = []
+        self.cooked_conflicts = []
+        self.reprocess = reprocess
+        self.show_base = show_base
+        self.pb = pb
+        self.pp = pp
+        if self.pp is None:
+            self.pp = ProgressPhase("Merge phase", 3, self.pb)
+
+        if interesting_ids is not None:
+            all_ids = interesting_ids
+        else:
+            all_ids = set(base_tree)
+            all_ids.update(other_tree)
+        working_tree.lock_write()
+        self.tt = TreeTransform(working_tree, self.pb)
+        try:
+            self.pp.next_phase()
+            child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            try:
+                for num, file_id in enumerate(all_ids):
+                    child_pb.update('Preparing file merge', num, len(all_ids))
+                    self.merge_names(file_id)
+                    file_status = self.merge_contents(file_id)
+                    self.merge_executable(file_id, file_status)
+            finally:
+                child_pb.finished()
+                
+            self.pp.next_phase()
+            child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            try:
+                fs_conflicts = resolve_conflicts(self.tt, child_pb)
+            finally:
+                child_pb.finished()
+            self.cook_conflicts(fs_conflicts)
+            for conflict in self.cooked_conflicts:
+                warning(conflict)
+            self.pp.next_phase()
+            results = self.tt.apply()
+            self.write_modified(results)
+            try:
+                working_tree.set_conflicts(ConflictList(self.cooked_conflicts))
+            except UnsupportedOperation:
+                pass
+        finally:
+            try:
+                self.tt.finalize()
+            except:
+                pass
+            working_tree.unlock()
+            self.pb.clear()
+
+    def write_modified(self, results):
+        modified_hashes = {}
+        for path in results.modified_paths:
+            file_id = self.this_tree.path2id(self.this_tree.relpath(path))
+            if file_id is None:
+                continue
+            hash = self.this_tree.get_file_sha1(file_id)
+            if hash is None:
+                continue
+            modified_hashes[file_id] = hash
+        self.this_tree.set_merge_modified(modified_hashes)
+
+    @staticmethod
+    def parent(entry, file_id):
+        """Determine the parent for a file_id (used as a key method)"""
+        if entry is None:
+            return None
+        return entry.parent_id
+
+    @staticmethod
+    def name(entry, file_id):
+        """Determine the name for a file_id (used as a key method)"""
+        if entry is None:
+            return None
+        return entry.name
+    
+    @staticmethod
+    def contents_sha1(tree, file_id):
+        """Determine the sha1 of the file contents (used as a key method)."""
+        if file_id not in tree:
+            return None
+        return tree.get_file_sha1(file_id)
+
+    @staticmethod
+    def executable(tree, file_id):
+        """Determine the executability of a file-id (used as a key method)."""
+        if file_id not in tree:
+            return None
+        if tree.kind(file_id) != "file":
+            return False
+        return tree.is_executable(file_id)
+
+    @staticmethod
+    def kind(tree, file_id):
+        """Determine the kind of a file-id (used as a key method)."""
+        if file_id not in tree:
+            return None
+        return tree.kind(file_id)
+
+    @staticmethod
+    def scalar_three_way(this_tree, base_tree, other_tree, file_id, key):
+        """Do a three-way test on a scalar.
+        Return "this", "other" or "conflict", depending whether a value wins.
+        """
+        key_base = key(base_tree, file_id)
+        key_other = key(other_tree, file_id)
+        #if base == other, either they all agree, or only THIS has changed.
+        if key_base == key_other:
+            return "this"
+        key_this = key(this_tree, file_id)
+        if key_this not in (key_base, key_other):
+            return "conflict"
+        # "Ambiguous clean merge"
+        elif key_this == key_other:
+            return "this"
+        else:
+            assert key_this == key_base
+            return "other"
+
+    def merge_names(self, file_id):
+        """Perform a merge on file_id names and parents"""
+        def get_entry(tree):
+            if file_id in tree.inventory:
+                return tree.inventory[file_id]
+            else:
+                return None
+        this_entry = get_entry(self.this_tree)
+        other_entry = get_entry(self.other_tree)
+        base_entry = get_entry(self.base_tree)
+        name_winner = self.scalar_three_way(this_entry, base_entry, 
+                                            other_entry, file_id, self.name)
+        parent_id_winner = self.scalar_three_way(this_entry, base_entry, 
+                                                 other_entry, file_id, 
+                                                 self.parent)
+        if this_entry is None:
+            if name_winner == "this":
+                name_winner = "other"
+            if parent_id_winner == "this":
+                parent_id_winner = "other"
+        if name_winner == "this" and parent_id_winner == "this":
+            return
+        if name_winner == "conflict":
+            trans_id = self.tt.trans_id_file_id(file_id)
+            self._raw_conflicts.append(('name conflict', trans_id, 
+                                        self.name(this_entry, file_id), 
+                                        self.name(other_entry, file_id)))
+        if parent_id_winner == "conflict":
+            trans_id = self.tt.trans_id_file_id(file_id)
+            self._raw_conflicts.append(('parent conflict', trans_id, 
+                                        self.parent(this_entry, file_id), 
+                                        self.parent(other_entry, file_id)))
+        if other_entry is None:
+            # it doesn't matter whether the result was 'other' or 
+            # 'conflict'-- if there's no 'other', we leave it alone.
+            return
+        # if we get here, name_winner and parent_winner are set to safe values.
+        winner_entry = {"this": this_entry, "other": other_entry, 
+                        "conflict": other_entry}
+        trans_id = self.tt.trans_id_file_id(file_id)
+        parent_id = winner_entry[parent_id_winner].parent_id
+        parent_trans_id = self.tt.trans_id_file_id(parent_id)
+        self.tt.adjust_path(winner_entry[name_winner].name, parent_trans_id,
+                            trans_id)
+
+    def merge_contents(self, file_id):
+        """Performa a merge on file_id contents."""
+        def contents_pair(tree):
+            if file_id not in tree:
+                return (None, None)
+            kind = tree.kind(file_id)
+            if kind == "root_directory":
+                kind = "directory"
+            if kind == "file":
+                contents = tree.get_file_sha1(file_id)
+            elif kind == "symlink":
+                contents = tree.get_symlink_target(file_id)
+            else:
+                contents = None
+            return kind, contents
+        # See SPOT run.  run, SPOT, run.
+        # So we're not QUITE repeating ourselves; we do tricky things with
+        # file kind...
+        base_pair = contents_pair(self.base_tree)
+        other_pair = contents_pair(self.other_tree)
+        if base_pair == other_pair:
+            # OTHER introduced no changes
+            return "unmodified"
+        this_pair = contents_pair(self.this_tree)
+        if this_pair == other_pair:
+            # THIS and OTHER introduced the same changes
+            return "unmodified"
+        else:
+            trans_id = self.tt.trans_id_file_id(file_id)
+            if this_pair == base_pair:
+                # only OTHER introduced changes
+                if file_id in self.this_tree:
+                    # Remove any existing contents
+                    self.tt.delete_contents(trans_id)
+                if file_id in self.other_tree:
+                    # OTHER changed the file
+                    create_by_entry(self.tt, 
+                                    self.other_tree.inventory[file_id], 
+                                    self.other_tree, trans_id)
+                    if file_id not in self.this_tree.inventory:
+                        self.tt.version_file(file_id, trans_id)
+                    return "modified"
+                elif file_id in self.this_tree.inventory:
+                    # OTHER deleted the file
+                    self.tt.unversion_file(trans_id)
+                    return "deleted"
+            #BOTH THIS and OTHER introduced changes; scalar conflict
+            elif this_pair[0] == "file" and other_pair[0] == "file":
+                # THIS and OTHER are both files, so text merge.  Either
+                # BASE is a file, or both converted to files, so at least we
+                # have agreement that output should be a file.
+                if file_id not in self.this_tree.inventory:
+                    self.tt.version_file(file_id, trans_id)
+                self.text_merge(file_id, trans_id)
+                try:
+                    self.tt.tree_kind(trans_id)
+                    self.tt.delete_contents(trans_id)
+                except NoSuchFile:
+                    pass
+                return "modified"
+            else:
+                # Scalar conflict, can't text merge.  Dump conflicts
+                trans_id = self.tt.trans_id_file_id(file_id)
+                name = self.tt.final_name(trans_id)
+                parent_id = self.tt.final_parent(trans_id)
+                if file_id in self.this_tree.inventory:
+                    self.tt.unversion_file(trans_id)
+                    self.tt.delete_contents(trans_id)
+                file_group = self._dump_conflicts(name, parent_id, file_id, 
+                                                  set_version=True)
+                self._raw_conflicts.append(('contents conflict', file_group))
+
+    def get_lines(self, tree, file_id):
+        """Return the lines in a file, or an empty list."""
+        if file_id in tree:
+            return tree.get_file(file_id).readlines()
+        else:
+            return []
+
+    def text_merge(self, file_id, trans_id):
+        """Perform a three-way text merge on a file_id"""
+        # it's possible that we got here with base as a different type.
+        # if so, we just want two-way text conflicts.
+        if file_id in self.base_tree and \
+            self.base_tree.kind(file_id) == "file":
+            base_lines = self.get_lines(self.base_tree, file_id)
+        else:
+            base_lines = []
+        other_lines = self.get_lines(self.other_tree, file_id)
+        this_lines = self.get_lines(self.this_tree, file_id)
+        m3 = Merge3(base_lines, this_lines, other_lines)
+        start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
+        if self.show_base is True:
+            base_marker = '|' * 7
+        else:
+            base_marker = None
+
+        def iter_merge3(retval):
+            retval["text_conflicts"] = False
+            for line in m3.merge_lines(name_a = "TREE", 
+                                       name_b = "MERGE-SOURCE", 
+                                       name_base = "BASE-REVISION",
+                                       start_marker=start_marker, 
+                                       base_marker=base_marker,
+                                       reprocess=self.reprocess):
+                if line.startswith(start_marker):
+                    retval["text_conflicts"] = True
+                    yield line.replace(start_marker, '<' * 7)
+                else:
+                    yield line
+        retval = {}
+        merge3_iterator = iter_merge3(retval)
+        self.tt.create_file(merge3_iterator, trans_id)
+        if retval["text_conflicts"] is True:
+            self._raw_conflicts.append(('text conflict', trans_id))
+            name = self.tt.final_name(trans_id)
+            parent_id = self.tt.final_parent(trans_id)
+            file_group = self._dump_conflicts(name, parent_id, file_id, 
+                                              this_lines, base_lines,
+                                              other_lines)
+            file_group.append(trans_id)
+
+    def _dump_conflicts(self, name, parent_id, file_id, this_lines=None, 
+                        base_lines=None, other_lines=None, set_version=False,
+                        no_base=False):
+        """Emit conflict files.
+        If this_lines, base_lines, or other_lines are omitted, they will be
+        determined automatically.  If set_version is true, the .OTHER, .THIS
+        or .BASE (in that order) will be created as versioned files.
+        """
+        data = [('OTHER', self.other_tree, other_lines), 
+                ('THIS', self.this_tree, this_lines)]
+        if not no_base:
+            data.append(('BASE', self.base_tree, base_lines))
+        versioned = False
+        file_group = []
+        for suffix, tree, lines in data:
+            if file_id in tree:
+                trans_id = self._conflict_file(name, parent_id, tree, file_id,
+                                               suffix, lines)
+                file_group.append(trans_id)
+                if set_version and not versioned:
+                    self.tt.version_file(file_id, trans_id)
+                    versioned = True
+        return file_group
+           
+    def _conflict_file(self, name, parent_id, tree, file_id, suffix, 
+                       lines=None):
+        """Emit a single conflict file."""
+        name = name + '.' + suffix
+        trans_id = self.tt.create_path(name, parent_id)
+        entry = tree.inventory[file_id]
+        create_by_entry(self.tt, entry, tree, trans_id, lines)
+        return trans_id
+
+    def merge_executable(self, file_id, file_status):
+        """Perform a merge on the execute bit."""
+        if file_status == "deleted":
+            return
+        trans_id = self.tt.trans_id_file_id(file_id)
+        try:
+            if self.tt.final_kind(trans_id) != "file":
+                return
+        except NoSuchFile:
+            return
+        winner = self.scalar_three_way(self.this_tree, self.base_tree, 
+                                       self.other_tree, file_id, 
+                                       self.executable)
+        if winner == "conflict":
+        # There must be a None in here, if we have a conflict, but we
+        # need executability since file status was not deleted.
+            if self.other_tree.is_executable(file_id) is None:
+                winner = "this"
+            else:
+                winner = "other"
+        if winner == "this":
+            if file_status == "modified":
+                executability = self.this_tree.is_executable(file_id)
+                if executability is not None:
+                    trans_id = self.tt.trans_id_file_id(file_id)
+                    self.tt.set_executability(executability, trans_id)
+        else:
+            assert winner == "other"
+            if file_id in self.other_tree:
+                executability = self.other_tree.is_executable(file_id)
+            elif file_id in self.this_tree:
+                executability = self.this_tree.is_executable(file_id)
+            elif file_id in self.base_tree:
+                executability = self.base_tree.is_executable(file_id)
+            if executability is not None:
+                trans_id = self.tt.trans_id_file_id(file_id)
+                self.tt.set_executability(executability, trans_id)
+
+    def cook_conflicts(self, fs_conflicts):
+        """Convert all conflicts into a form that doesn't depend on trans_id"""
+        from conflicts import Conflict
+        name_conflicts = {}
+        self.cooked_conflicts.extend(cook_conflicts(fs_conflicts, self.tt))
+        fp = FinalPaths(self.tt)
+        for conflict in self._raw_conflicts:
+            conflict_type = conflict[0]
+            if conflict_type in ('name conflict', 'parent conflict'):
+                trans_id = conflict[1]
+                conflict_args = conflict[2:]
+                if trans_id not in name_conflicts:
+                    name_conflicts[trans_id] = {}
+                unique_add(name_conflicts[trans_id], conflict_type, 
+                           conflict_args)
+            if conflict_type == 'contents conflict':
+                for trans_id in conflict[1]:
+                    file_id = self.tt.final_file_id(trans_id)
                     if file_id is not None:
-                        interesting_ids.add(file_id)
-                        found_id = True
-                if not found_id:
-                    raise BzrCommandError("%s is not a source file in any"
-                                          " tree." % fname)
-        merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type, interesting_ids=interesting_ids)
-    finally:
-        shutil.rmtree(tempdir)
-
-
-def set_interesting(inventory_a, inventory_b, interesting_ids):
-    """Mark files whose ids are in interesting_ids as interesting
-    """
-    for inventory in (inventory_a, inventory_b):
-        for path, source_file in inventory.iteritems():
-             source_file.interesting = source_file.id in interesting_ids
-
-
-def generate_cset_optimized(tree_a, tree_b, interesting_ids=None):
-    """Generate a changeset.  If interesting_ids is supplied, only changes
-    to those files will be shown.  Metadata changes are stripped.
-    """ 
-    cset =  generate_changeset(tree_a, tree_b, interesting_ids)
-    for entry in cset.entries.itervalues():
-        entry.metadata_change = None
-    return cset
-
-
-def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
-                interesting_ids=None):
-
-    def merge_factory(file_id, base, other):
-        contents_change = merge_type(file_id, base, other)
-        if backup_files:
-            contents_change = BackupBeforeChange(contents_change)
-        return contents_change
-
-    this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
-
-    def get_inventory(tree):
-        return tree.tree.inventory
-
-    inv_changes = merge_flex(this_tree, base_tree, other_tree,
-                             generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(base_tree.root,
-                                                  ignore_zero=ignore_zero),
-                             merge_factory=merge_factory, 
-                             interesting_ids=interesting_ids)
-
-    adjust_ids = []
-    for id, path in inv_changes.iteritems():
-        if path is not None:
-            if path == '.':
-                path = ''
+                        break
+                path = fp.get_path(trans_id)
+                for suffix in ('.BASE', '.THIS', '.OTHER'):
+                    if path.endswith(suffix):
+                        path = path[:-len(suffix)]
+                        break
+                c = Conflict.factory(conflict_type, path=path, file_id=file_id)
+                self.cooked_conflicts.append(c)
+            if conflict_type == 'text conflict':
+                trans_id = conflict[1]
+                path = fp.get_path(trans_id)
+                file_id = self.tt.final_file_id(trans_id)
+                c = Conflict.factory(conflict_type, path=path, file_id=file_id)
+                self.cooked_conflicts.append(c)
+
+        for trans_id, conflicts in name_conflicts.iteritems():
+            try:
+                this_parent, other_parent = conflicts['parent conflict']
+                assert this_parent != other_parent
+            except KeyError:
+                this_parent = other_parent = \
+                    self.tt.final_file_id(self.tt.final_parent(trans_id))
+            try:
+                this_name, other_name = conflicts['name conflict']
+                assert this_name != other_name
+            except KeyError:
+                this_name = other_name = self.tt.final_name(trans_id)
+            other_path = fp.get_path(trans_id)
+            if this_parent is not None:
+                this_parent_path = \
+                    fp.get_path(self.tt.trans_id_file_id(this_parent))
+                this_path = pathjoin(this_parent_path, this_name)
             else:
-                assert path.startswith('./'), "path is %s" % path
-            path = path[2:]
-        adjust_ids.append((path, id))
-    if len(adjust_ids) > 0:
-        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
-                                                  adjust_ids))
-
-
-def regen_inventory(this_branch, root, new_entries):
-    old_entries = this_branch.read_working_inventory()
-    new_inventory = {}
-    by_path = {}
-    new_entries_map = {} 
-    for path, file_id in new_entries:
-        if path is None:
-            continue
-        new_entries_map[file_id] = path
-
-    def id2path(file_id):
-        path = new_entries_map.get(file_id)
-        if path is not None:
-            return path
-        entry = old_entries[file_id]
-        if entry.parent_id is None:
-            return entry.name
-        return os.path.join(id2path(entry.parent_id), entry.name)
-        
-    for file_id in old_entries:
-        entry = old_entries[file_id]
-        path = id2path(file_id)
-        new_inventory[file_id] = (path, file_id, entry.parent_id, entry.kind)
-        by_path[path] = file_id
-    
-    deletions = 0
-    insertions = 0
-    new_path_list = []
-    for path, file_id in new_entries:
-        if path is None:
-            del new_inventory[file_id]
-            deletions += 1
-        else:
-            new_path_list.append((path, file_id))
-            if file_id not in old_entries:
-                insertions += 1
-    # Ensure no file is added before its parent
-    new_path_list.sort()
-    for path, file_id in new_path_list:
-        if path == '':
-            parent = None
-        else:
-            parent = by_path[os.path.dirname(path)]
-        kind = bzrlib.osutils.file_kind(os.path.join(root, path))
-        new_inventory[file_id] = (path, file_id, parent, kind)
-        by_path[path] = file_id 
-
-    # Get a list in insertion order
-    new_inventory_list = new_inventory.values()
-    mutter ("""Inventory regeneration:
-old length: %i insertions: %i deletions: %i new_length: %i"""\
-        % (len(old_entries), insertions, deletions, len(new_inventory_list)))
-    assert len(new_inventory_list) == len(old_entries) + insertions - deletions
-    new_inventory_list.sort()
-    return new_inventory_list
-
-merge_types = {     "merge3": (ApplyMerge3, "Native diff3-style merge"), 
-                     "diff3": (Diff3Merge,  "Merge using external diff3")
+                this_path = "<deleted>"
+            file_id = self.tt.final_file_id(trans_id)
+            c = Conflict.factory('path conflict', path=this_path,
+                                 conflict_path=other_path, file_id=file_id)
+            self.cooked_conflicts.append(c)
+        self.cooked_conflicts.sort(key=Conflict.sort_key)
+
+
+class WeaveMerger(Merge3Merger):
+    """Three-way tree merger, text weave merger."""
+    supports_reprocess = True
+    supports_show_base = False
+
+    def __init__(self, working_tree, this_tree, base_tree, other_tree, 
+                 interesting_ids=None, pb=DummyProgress(), pp=None,
+                 reprocess=False):
+        self.this_revision_tree = self._get_revision_tree(this_tree)
+        self.other_revision_tree = self._get_revision_tree(other_tree)
+        super(WeaveMerger, self).__init__(working_tree, this_tree, 
+                                          base_tree, other_tree, 
+                                          interesting_ids=interesting_ids, 
+                                          pb=pb, pp=pp, reprocess=reprocess)
+
+    def _get_revision_tree(self, tree):
+        """Return a revision tree releated to this tree.
+        If the tree is a WorkingTree, the basis will be returned.
+        """
+        if getattr(tree, 'get_weave', False) is False:
+            # If we have a WorkingTree, try using the basis
+            return tree.branch.basis_tree()
+        else:
+            return tree
+
+    def _check_file(self, file_id):
+        """Check that the revision tree's version of the file matches."""
+        for tree, rt in ((self.this_tree, self.this_revision_tree), 
+                         (self.other_tree, self.other_revision_tree)):
+            if rt is tree:
+                continue
+            if tree.get_file_sha1(file_id) != rt.get_file_sha1(file_id):
+                raise WorkingTreeNotRevision(self.this_tree)
+
+    def _merged_lines(self, file_id):
+        """Generate the merged lines.
+        There is no distinction between lines that are meant to contain <<<<<<<
+        and conflicts.
+        """
+        weave = self.this_revision_tree.get_weave(file_id)
+        this_revision_id = self.this_revision_tree.inventory[file_id].revision
+        other_revision_id = \
+            self.other_revision_tree.inventory[file_id].revision
+        wm = WeaveMerge(weave, this_revision_id, other_revision_id, 
+                        '<<<<<<< TREE\n', '>>>>>>> MERGE-SOURCE\n')
+        return wm.merge_lines(self.reprocess)
+
+    def text_merge(self, file_id, trans_id):
+        """Perform a (weave) text merge for a given file and file-id.
+        If conflicts are encountered, .THIS and .OTHER files will be emitted,
+        and a conflict will be noted.
+        """
+        self._check_file(file_id)
+        lines, conflicts = self._merged_lines(file_id)
+        self.tt.create_file(lines, trans_id)
+        if conflicts:
+            self._raw_conflicts.append(('text conflict', trans_id))
+            name = self.tt.final_name(trans_id)
+            parent_id = self.tt.final_parent(trans_id)
+            file_group = self._dump_conflicts(name, parent_id, file_id, 
+                                              no_base=True)
+            file_group.append(trans_id)
+
+
+class Diff3Merger(Merge3Merger):
+    """Three-way merger using external diff3 for text merging"""
+    def dump_file(self, temp_dir, name, tree, file_id):
+        out_path = pathjoin(temp_dir, name)
+        out_file = file(out_path, "wb")
+        in_file = tree.get_file(file_id)
+        for line in in_file:
+            out_file.write(line)
+        return out_path
+
+    def text_merge(self, file_id, trans_id):
+        """Perform a diff3 merge using a specified file-id and trans-id.
+        If conflicts are encountered, .BASE, .THIS. and .OTHER conflict files
+        will be dumped, and a will be conflict noted.
+        """
+        import bzrlib.patch
+        temp_dir = mkdtemp(prefix="bzr-")
+        try:
+            new_file = pathjoin(temp_dir, "new")
+            this = self.dump_file(temp_dir, "this", self.this_tree, file_id)
+            base = self.dump_file(temp_dir, "base", self.base_tree, file_id)
+            other = self.dump_file(temp_dir, "other", self.other_tree, file_id)
+            status = bzrlib.patch.diff3(new_file, this, base, other)
+            if status not in (0, 1):
+                raise BzrError("Unhandled diff3 exit code")
+            self.tt.create_file(file(new_file, "rb"), trans_id)
+            if status == 1:
+                name = self.tt.final_name(trans_id)
+                parent_id = self.tt.final_parent(trans_id)
+                self._dump_conflicts(name, parent_id, file_id)
+            self._raw_conflicts.append(('text conflict', trans_id))
+        finally:
+            rmtree(temp_dir)
+
+
+def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
+                backup_files=False, 
+                merge_type=Merge3Merger, 
+                interesting_ids=None, 
+                show_base=False, 
+                reprocess=False, 
+                other_rev_id=None,
+                interesting_files=None,
+                this_tree=None,
+                pb=DummyProgress()):
+    """Primary interface for merging. 
+
+        typical use is probably 
+        'merge_inner(branch, branch.get_revision_tree(other_revision),
+                     branch.get_revision_tree(base_revision))'
+        """
+    if this_tree is None:
+        warn("bzrlib.merge.merge_inner requires a this_tree parameter as of "
+             "bzrlib version 0.8.",
+             DeprecationWarning,
+             stacklevel=2)
+        this_tree = this_branch.bzrdir.open_workingtree()
+    merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree, 
+                    pb=pb)
+    merger.backup_files = backup_files
+    merger.merge_type = merge_type
+    merger.interesting_ids = interesting_ids
+    merger.ignore_zero = ignore_zero
+    if interesting_files:
+        assert not interesting_ids, ('Only supply interesting_ids'
+                                     ' or interesting_files')
+        merger._set_interesting_files(interesting_files)
+    merger.show_base = show_base 
+    merger.reprocess = reprocess
+    merger.other_rev_id = other_rev_id
+    merger.other_basis = other_rev_id
+    return merger.do_merge()
+
+
+merge_types = {     "merge3": (Merge3Merger, "Native diff3-style merge"), 
+                     "diff3": (Diff3Merger,  "Merge using external diff3"),
+                     'weave': (WeaveMerger, "Weave-based merge")
               }
-

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-07-06 04:09:02 +0000
+++ bzrlib/merge3.py	2005-11-29 07:39:19 +0000
@@ -19,6 +19,8 @@
 # s: "i hate that."
 
 
+from difflib import SequenceMatcher
+from bzrlib.errors import CantReprocessAndShowBase
 
 def intersect(ra, rb):
     """Given two ranges return the range where they intersect or None.
@@ -67,27 +69,32 @@
         self.base = base
         self.a = a
         self.b = b
-        from difflib import SequenceMatcher
-        self.a_ops = SequenceMatcher(None, base, a).get_opcodes()
-        self.b_ops = SequenceMatcher(None, base, b).get_opcodes()
 
 
 
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
-                    start_marker='<<<<<<<<',
-                    mid_marker='========',
-                    end_marker='>>>>>>>>',
-                    show_base=False):
+                    name_base=None,
+                    start_marker='<<<<<<<',
+                    mid_marker='=======',
+                    end_marker='>>>>>>>',
+                    base_marker=None,
+                    reprocess=False):
         """Return merge in cvs-like form.
         """
+        if base_marker and reprocess:
+            raise CantReprocessAndShowBase()
         if name_a:
             start_marker = start_marker + ' ' + name_a
         if name_b:
             end_marker = end_marker + ' ' + name_b
-            
-        for t in self.merge_regions():
+        if name_base and base_marker:
+            base_marker = base_marker + ' ' + name_base
+        merge_regions = self.merge_regions()
+        if reprocess is True:
+            merge_regions = self.reprocess_merge_regions(merge_regions)
+        for t in merge_regions:
             what = t[0]
             if what == 'unchanged':
                 for i in range(t[1], t[2]):
@@ -102,6 +109,10 @@
                 yield start_marker + '\n'
                 for i in range(t[3], t[4]):
                     yield self.a[i]
+                if base_marker is not None:
+                    yield base_marker + '\n'
+                    for i in range(t[1], t[2]):
+                        yield self.base[i]
                 yield mid_marker + '\n'
                 for i in range(t[5], t[6]):
                     yield self.b[i]
@@ -263,16 +274,52 @@
                 iz = zend
                 ia = aend
                 ib = bend
-        
-
-        
+    
+
+    def reprocess_merge_regions(self, merge_regions):
+        """Where there are conflict regions, remove the agreed lines.
+
+        Lines where both A and B have made the same changes are 
+        eliminated.
+        """
+        for region in merge_regions:
+            if region[0] != "conflict":
+                yield region
+                continue
+            type, iz, zmatch, ia, amatch, ib, bmatch = region
+            a_region = self.a[ia:amatch]
+            b_region = self.b[ib:bmatch]
+            matches = SequenceMatcher(None, a_region, 
+                                      b_region).get_matching_blocks()
+            next_a = ia
+            next_b = ib
+            for region_ia, region_ib, region_len in matches[:-1]:
+                region_ia += ia
+                region_ib += ib
+                reg = self.mismatch_region(next_a, region_ia, next_b,
+                                           region_ib)
+                if reg is not None:
+                    yield reg
+                yield 'same', region_ia, region_len+region_ia
+                next_a = region_ia + region_len
+                next_b = region_ib + region_len
+            reg = self.mismatch_region(next_a, amatch, next_b, bmatch)
+            if reg is not None:
+                yield reg
+
+
+    @staticmethod
+    def mismatch_region(next_a, region_ia,  next_b, region_ib):
+        if next_a < region_ia or next_b < region_ib:
+            return 'conflict', None, None, next_a, region_ia, next_b, region_ib
+            
+
     def find_sync_regions(self):
         """Return a list of sync regions, where both descendents match the base.
 
         Generates a list of (base1, base2, a1, a2, b1, b2).  There is
         always a zero-length sync region at the end of all the files.
         """
-        from difflib import SequenceMatcher
 
         ia = ib = 0
         amatches = SequenceMatcher(None, self.base, self.a).get_matching_blocks()
@@ -332,7 +379,6 @@
 
     def find_unconflicted(self):
         """Return a list of ranges in base that are not conflicted."""
-        from difflib import SequenceMatcher
 
         import re
 

=== removed file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-08-17 08:32:56 +0000
+++ bzrlib/merge_core.py	1970-01-01 00:00:00 +0000
@@ -1,732 +0,0 @@
-import changeset
-from changeset import Inventory, apply_changeset, invert_dict
-import os.path
-from osutils import backup_file
-from merge3 import Merge3
-
-class ApplyMerge3:
-    """Contents-change wrapper around merge3.Merge3"""
-    def __init__(self, file_id, base, other):
-        self.file_id = file_id
-        self.base = base
-        self.other = other
- 
-    def __eq__(self, other):
-        if not isinstance(other, ApplyMerge3):
-            return False
-        return (self.base == other.base and 
-                self.other == other.other and self.file_id == other.file_id)
-
-    def __ne__(self, other):
-        return not (self == other)
-
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new" 
-        if not reverse:
-            base = self.base
-            other = self.other
-        else:
-            base = self.other
-            other = self.base
-        def get_lines(tree):
-            if self.file_id not in tree:
-                raise Exception("%s not in tree" % self.file_id)
-                return ()
-            return tree.get_file(self.file_id).readlines()
-        base_lines = get_lines(base)
-        other_lines = get_lines(other)
-        m3 = Merge3(base_lines, file(filename, "rb").readlines(), other_lines)
-
-        new_conflicts = False
-        output_file = file(new_file, "wb")
-        start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
-        for line in m3.merge_lines(name_a = "TREE", name_b = "MERGE-SOURCE", 
-                       start_marker=start_marker):
-            if line.startswith(start_marker):
-                new_conflicts = True
-                output_file.write(line.replace(start_marker, '<<<<<<<<'))
-            else:
-                output_file.write(line)
-        output_file.close()
-        if not new_conflicts:
-            os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
-            return
-        else:
-            conflict_handler.merge_conflict(new_file, filename, base_lines,
-                                            other_lines)
-
-
-class BackupBeforeChange:
-    """Contents-change wrapper to back up file first"""
-    def __init__(self, contents_change):
-        self.contents_change = contents_change
- 
-    def __eq__(self, other):
-        if not isinstance(other, BackupBeforeChange):
-            return False
-        return (self.contents_change == other.contents_change)
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        backup_file(filename)
-        self.contents_change.apply(filename, conflict_handler, reverse)
-
-
-def invert_invent(inventory):
-    invert_invent = {}
-    for file_id in inventory:
-        path = inventory.id2path(file_id)
-        if path == '':
-            path = './.'
-        else:
-            path = './' + path
-        invert_invent[file_id] = path
-    return invert_invent
-
-
-def merge_flex(this, base, other, changeset_function, inventory_function,
-               conflict_handler, merge_factory, interesting_ids):
-    cset = changeset_function(base, other, interesting_ids)
-    new_cset = make_merge_changeset(cset, this, base, other, 
-                                    conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
-                             this.root, conflict_handler, False)
-    conflict_handler.finalize()
-    return result
-
-    
-
-def make_merge_changeset(cset, this, base, other, 
-                         conflict_handler, merge_factory):
-    new_cset = changeset.Changeset()
-    def get_this_contents(id):
-        path = this.readonly_path(id)
-        if os.path.isdir(path):
-            return changeset.dir_create
-        else:
-            return changeset.FileCreate(file(path, "rb").read())
-
-    for entry in cset.entries.itervalues():
-        if entry.is_boring():
-            new_cset.add_entry(entry)
-        else:
-            new_entry = make_merged_entry(entry, this, base, other, 
-                                          conflict_handler)
-            new_contents = make_merged_contents(entry, this, base, other, 
-                                                conflict_handler,
-                                                merge_factory)
-            new_entry.contents_change = new_contents
-            new_entry.metadata_change = make_merged_metadata(entry, base, other)
-            new_cset.add_entry(new_entry)
-
-    return new_cset
-
-class ThreeWayConflict(Exception):
-    def __init__(self, this, base, other):
-        self.this = this
-        self.base = base
-        self.other = other
-        msg = "Conflict merging %s %s and %s" % (this, base, other)
-        Exception.__init__(self, msg)
-
-def threeway_select(this, base, other):
-    """Returns a value selected by the three-way algorithm.
-    Raises ThreewayConflict if the algorithm yields a conflict"""
-    if base == other:
-        return this
-    elif base == this:
-        return other
-    elif other == this:
-        return this
-    else:
-        raise ThreeWayConflict(this, base, other)
-
-
-def make_merged_entry(entry, this, base, other, conflict_handler):
-    from bzrlib.trace import mutter
-    def entry_data(file_id, tree):
-        assert hasattr(tree, "__contains__"), "%s" % tree
-        if file_id not in tree:
-            return (None, None, "")
-        entry = tree.tree.inventory[file_id]
-        my_dir = tree.id2path(entry.parent_id)
-        if my_dir is None:
-            my_dir = ""
-        return entry.name, entry.parent_id, my_dir 
-    this_name, this_parent, this_dir = entry_data(entry.id, this)
-    base_name, base_parent, base_dir = entry_data(entry.id, base)
-    other_name, other_parent, other_dir = entry_data(entry.id, other)
-    mutter("Dirs: this, base, other %r %r %r" % (this_dir, base_dir, other_dir))
-    mutter("Names: this, base, other %r %r %r" % (this_name, base_name, other_name))
-    old_name = this_name
-    try:
-        new_name = threeway_select(this_name, base_name, other_name)
-    except ThreeWayConflict:
-        new_name = conflict_handler.rename_conflict(entry.id, this_name, 
-                                                    base_name, other_name)
-
-    old_parent = this_parent
-    try:
-        new_parent = threeway_select(this_parent, base_parent, other_parent)
-    except ThreeWayConflict:
-        new_parent = conflict_handler.move_conflict(entry.id, this_dir,
-                                                    base_dir, other_dir)
-    def get_path(name, parent):
-        if name is not None and parent is not None:
-            parent_dir = {this_parent: this_dir, other_parent: other_dir, 
-                          base_parent: base_dir}
-            directory = parent_dir[parent]
-            return os.path.join(directory, name)
-        else:
-            assert name is None and parent is None
-            return None
-
-    old_path = get_path(old_name, old_parent)
-        
-    new_entry = changeset.ChangesetEntry(entry.id, old_parent, old_path)
-    new_entry.new_path = get_path(new_name, new_parent)
-    new_entry.new_parent = new_parent
-    mutter(repr(new_entry))
-    return new_entry
-
-
-def make_merged_contents(entry, this, base, other, conflict_handler,
-                         merge_factory):
-    contents = entry.contents_change
-    if contents is None:
-        return None
-    this_path = this.readonly_path(entry.id)
-    def make_merge():
-        if this_path is None:
-            return conflict_handler.missing_for_merge(entry.id, 
-                                                      other.id2path(entry.id))
-        return merge_factory(entry.id, base, other)
-
-    if isinstance(contents, changeset.ReplaceContents):
-        if contents.old_contents is None and contents.new_contents is None:
-            return None
-        if contents.new_contents is None:
-            if this_path is not None and os.path.exists(this_path):
-                return contents
-            else:
-                return None
-        elif contents.old_contents is None:
-            if this_path is None or not os.path.exists(this_path):
-                return contents
-            else:
-                this_contents = changeset.FileCreate(file(this_path, 
-                                                     "rb").read())
-                if this_contents == contents.new_contents:
-                    return None
-                else:
-                    other_path = other.readonly_path(entry.id)    
-                    conflict_handler.new_contents_conflict(this_path, 
-                                                           other_path)
-        elif isinstance(contents.old_contents, changeset.FileCreate) and \
-            isinstance(contents.new_contents, changeset.FileCreate):
-            return make_merge()
-        else:
-            raise Exception("Unhandled merge scenario")
-
-def make_merged_metadata(entry, base, other):
-    if entry.metadata_change is not None:
-        base_path = base.readonly_path(entry.id)
-        other_path = other.readonly_path(entry.id)    
-        return PermissionsMerge(base_path, other_path)
-    
-
-class PermissionsMerge(object):
-    def __init__(self, base_path, other_path):
-        self.base_path = base_path
-        self.other_path = other_path
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            base = self.base_path
-            other = self.other_path
-        else:
-            base = self.other_path
-            other = self.base_path
-        base_stat = os.stat(base).st_mode
-        other_stat = os.stat(other).st_mode
-        this_stat = os.stat(filename).st_mode
-        if base_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == base_stat &0777:
-            os.chmod(filename, other_stat)
-        else:
-            conflict_handler.permission_conflict(filename, base, other)
-
-
-import unittest
-import tempfile
-import shutil
-from bzrlib.inventory import InventoryEntry, RootEntry
-from osutils import file_kind
-class FalseTree(object):
-    def __init__(self, realtree):
-        self._realtree = realtree
-        self.inventory = self
-
-    def __getitem__(self, file_id):
-        entry = self.make_inventory_entry(file_id)
-        if entry is None:
-            raise KeyError(file_id)
-        return entry
-        
-    def make_inventory_entry(self, file_id):
-        path = self._realtree.inventory.get(file_id)
-        if path is None:
-            return None
-        if path == "":
-            return RootEntry(file_id)
-        dir, name = os.path.split(path)
-        kind = file_kind(self._realtree.abs_path(path))
-        for parent_id, path in self._realtree.inventory.iteritems():
-            if path == dir:
-                break
-        if path != dir:
-            raise Exception("Can't find parent for %s" % name)
-        return InventoryEntry(file_id, name, kind, parent_id)
-
-
-class MergeTree(object):
-    def __init__(self, dir):
-        self.dir = dir;
-        os.mkdir(dir)
-        self.inventory = {'0': ""}
-        self.tree = FalseTree(self)
-    
-    def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        file(full_path, "wb").write(contents)
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def remove_file(self, id):
-        os.unlink(self.full_path(id))
-        del self.inventory[id]
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        os.mkdir(self.abs_path(path))
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def abs_path(self, path):
-        return os.path.join(self.dir, path)
-
-    def full_path(self, id):
-        try:
-            tree_path = self.inventory[id]
-        except KeyError:
-            return None
-        return self.abs_path(tree_path)
-
-    def readonly_path(self, id):
-        return self.full_path(id)
-
-    def __contains__(self, file_id):
-        return file_id in self.inventory
-
-    def get_file(self, file_id):
-        path = self.readonly_path(file_id)
-        return file(path, "rb")
-
-    def id2path(self, file_id):
-        return self.inventory[file_id]
-
-    def change_path(self, id, path):
-        new = os.path.join(self.dir, self.inventory[id])
-        os.rename(self.abs_path(self.inventory[id]), self.abs_path(path))
-        self.inventory[id] = path
-
-class MergeBuilder(object):
-    def __init__(self):
-        self.dir = tempfile.mkdtemp(prefix="BaZing")
-        self.base = MergeTree(os.path.join(self.dir, "base"))
-        self.this = MergeTree(os.path.join(self.dir, "this"))
-        self.other = MergeTree(os.path.join(self.dir, "other"))
-        
-        self.cset = changeset.Changeset()
-        self.cset.add_entry(changeset.ChangesetEntry("0", 
-                                                     changeset.NULL_ID, "./."))
-    def get_cset_path(self, parent, name):
-        if name is None:
-            assert (parent is None)
-            return None
-        return os.path.join(self.cset.entries[parent].path, name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        self.base.add_file(id, parent, name, contents, mode)
-        self.this.add_file(id, parent, name, contents, mode)
-        self.other.add_file(id, parent, name, contents, mode)
-        path = self.get_cset_path(parent, name)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-
-    def remove_file(self, id, base=False, this=False, other=False):
-        for option, tree in ((base, self.base), (this, self.this), 
-                             (other, self.other)):
-            if option:
-                tree.remove_file(id)
-            if other or base:
-                change = self.cset.entries[id].contents_change
-                if change is None:
-                    change = changeset.ReplaceContents(None, None)
-                    self.cset.entries[id].contents_change = change
-                    def create_file(tree):
-                        return changeset.FileCreate(tree.get_file(id).read())
-                    if not other:
-                        change.new_contents = create_file(self.other)
-                    if not base:
-                        change.old_contents = create_file(self.base)
-                else:
-                    assert isinstance(change, changeset.ReplaceContents)
-                if other:
-                    change.new_contents=None
-                if base:
-                    change.old_contents=None
-                if change.old_contents is None and change.new_contents is None:
-                    change = None
-
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.get_cset_path(parent, name)
-        self.base.add_dir(id, parent, name, mode)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-        self.this.add_dir(id, parent, name, mode)
-        self.other.add_dir(id, parent, name, mode)
-
-
-    def change_name(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_name_tree(id, self.base, base)
-            self.cset.entries[id].name = base
-
-        if this is not None:
-            self.change_name_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_name_tree(id, self.other, other)
-            self.cset.entries[id].new_name = other
-
-    def change_parent(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_parent_tree(id, self.base, base)
-            self.cset.entries[id].parent = base
-            self.cset.entries[id].dir = self.cset.entries[base].path
-
-        if this is not None:
-            self.change_parent_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_parent_tree(id, self.other, other)
-            self.cset.entries[id].new_parent = other
-            self.cset.entries[id].new_dir = \
-                self.cset.entries[other].new_path
-
-    def change_contents(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_contents_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_contents_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_contents_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_contents = file(self.base.full_path(id)).read()
-            new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
-            self.cset.entries[id].contents_change = contents
-
-    def change_perms(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_perms_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_perms_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_perms_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_perms = os.stat(self.base.full_path(id)).st_mode &077
-            new_perms = os.stat(self.other.full_path(id)).st_mode &077
-            contents = changeset.ChangeUnixPermissions(old_perms, 
-                                                       new_perms)
-            self.cset.entries[id].metadata_change = contents
-
-    def change_name_tree(self, id, tree, name):
-        new_path = tree.child_path(self.cset.entries[id].parent, name)
-        tree.change_path(id, new_path)
-
-    def change_parent_tree(self, id, tree, parent):
-        new_path = tree.child_path(parent, self.cset.entries[id].name)
-        tree.change_path(id, new_path)
-
-    def change_contents_tree(self, id, tree, contents):
-        path = tree.full_path(id)
-        mode = os.stat(path).st_mode
-        file(path, "w").write(contents)
-        os.chmod(path, mode)
-
-    def change_perms_tree(self, id, tree, mode):
-        os.chmod(tree.full_path(id), mode)
-
-    def merge_changeset(self, merge_factory):
-        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
-        return make_merge_changeset(self.cset, self.this, self.base,
-                                    self.other, conflict_handler,
-                                    merge_factory)
-
-    def apply_inv_change(self, inventory_change, orig_inventory):
-        orig_inventory_by_path = {}
-        for file_id, path in orig_inventory.iteritems():
-            orig_inventory_by_path[path] = file_id
-
-        def parent_id(file_id):
-            try:
-                parent_dir = os.path.dirname(orig_inventory[file_id])
-            except:
-                print file_id
-                raise
-            if parent_dir == "":
-                return None
-            return orig_inventory_by_path[parent_dir]
-        
-        def new_path(file_id):
-            if inventory_change.has_key(file_id):
-                return inventory_change[file_id]
-            else:
-                parent = parent_id(file_id)
-                if parent is None:
-                    return orig_inventory[file_id]
-                dirname = new_path(parent)
-                return os.path.join(dirname, orig_inventory[file_id])
-
-        new_inventory = {}
-        for file_id in orig_inventory.iterkeys():
-            path = new_path(file_id)
-            if path is None:
-                continue
-            new_inventory[file_id] = path
-
-        for file_id, path in inventory_change.iteritems():
-            if orig_inventory.has_key(file_id):
-                continue
-            new_inventory[file_id] = path
-        return new_inventory
-
-        
-
-    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
-        inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
-                                                     self.this.dir,
-                                                     conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
-
-                    
-        
-
-        
-    def cleanup(self):
-        shutil.rmtree(self.dir)
-
-
-class MergeTest(unittest.TestCase):
-    def test_change_name(self):
-        """Test renames"""
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0755)
-        builder.change_name("1", other="name2")
-        builder.add_file("2", "0", "name3", "hello2", 0755)
-        builder.change_name("2", base="name4")
-        builder.add_file("3", "0", "name5", "hello3", 0755)
-        builder.change_name("3", this="name6")
-        cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["2"].is_boring())
-        assert(cset.entries["1"].name == "name1")
-        assert(cset.entries["1"].new_name == "name2")
-        assert(cset.entries["3"].is_boring())
-        for tree in (builder.this, builder.other, builder.base):
-            assert(tree.dir != builder.dir and 
-                   tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
-                fullpath = tree.abs_path(path)
-                assert(fullpath.startswith(tree.dir))
-                assert(not path.startswith(tree.dir))
-                assert os.path.exists(fullpath)
-        builder.apply_changeset(cset)
-        builder.cleanup()
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0644)
-        builder.change_name("1", other="name2", this="name3")
-        self.assertRaises(changeset.RenameConflict, 
-                          builder.merge_changeset, ApplyMerge3)
-        builder.cleanup()
-        
-    def test_file_moves(self):
-        """Test moves"""
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_file("3", "1", "file1", "hello1", 0644)
-        builder.add_file("4", "1", "file2", "hello2", 0644)
-        builder.add_file("5", "1", "file3", "hello3", 0644)
-        builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
-        builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
-        builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
-        cset = builder.merge_changeset(ApplyMerge3)
-        for id in ("1", "2", "4", "5"):
-            assert(cset.entries[id].is_boring())
-        assert(cset.entries["3"].parent == "1")
-        assert(cset.entries["3"].new_parent == "2")
-        builder.apply_changeset(cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_dir("3", "0", "dir3", 0755)
-        builder.add_file("4", "1", "file1", "hello1", 0644)
-        builder.change_parent("4", other="2", this="3")
-        self.assertRaises(changeset.MoveConflict, 
-                          builder.merge_changeset, ApplyMerge3)
-        builder.cleanup()
-
-    def test_contents_merge(self):
-        """Test merge3 merging"""
-        self.do_contents_test(ApplyMerge3)
-
-    def test_contents_merge2(self):
-        """Test diff3 merging"""
-        self.do_contents_test(changeset.Diff3Merge)
-
-    def test_contents_merge3(self):
-        """Test diff3 merging"""
-        def backup_merge(file_id, base, other):
-            return BackupBeforeChange(ApplyMerge3(file_id, base, other))
-        builder = self.contents_test_success(backup_merge)
-        def backup_exists(file_id):
-            return os.path.exists(builder.this.full_path(file_id)+"~")
-        assert backup_exists("1")
-        assert backup_exists("2")
-        assert not backup_exists("3")
-        builder.cleanup()
-
-    def do_contents_test(self, merge_factory):
-        """Test merging with specified ContentsChange factory"""
-        builder = self.contents_test_success(merge_factory)
-        builder.cleanup()
-        self.contents_test_conflicts(merge_factory)
-
-    def contents_test_success(self, merge_factory):
-        from inspect import isclass
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4")
-        builder.add_file("2", "0", "name3", "text2", 0655)
-        builder.change_contents("2", base="text5")
-        builder.add_file("3", "0", "name5", "text3", 0744)
-        builder.add_file("4", "0", "name6", "text4", 0744)
-        builder.remove_file("4", base=True)
-        assert not builder.cset.entries["4"].is_boring()
-        builder.change_contents("3", this="text6")
-        cset = builder.merge_changeset(merge_factory)
-        assert(cset.entries["1"].contents_change is not None)
-        if isclass(merge_factory):
-            assert(isinstance(cset.entries["1"].contents_change,
-                          merge_factory))
-            assert(isinstance(cset.entries["2"].contents_change,
-                          merge_factory))
-        assert(cset.entries["3"].is_boring())
-        assert(cset.entries["4"].is_boring())
-        builder.apply_changeset(cset)
-        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
-        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
-        return builder
-
-    def contents_test_conflicts(self, merge_factory):
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", this="text3")
-        cset = builder.merge_changeset(merge_factory)
-        self.assertRaises(changeset.MergeConflict, builder.apply_changeset,
-                          cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset, merge_factory)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
-                          merge_factory)
-        builder.cleanup()
-
-    def test_perms_merge(self):
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655)
-        builder.add_file("2", "0", "name2", "text2", 0755)
-        builder.change_perms("2", base=0655)
-        builder.add_file("3", "0", "name3", "text3", 0755)
-        builder.change_perms("3", this=0655)
-        cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["1"].metadata_change is not None)
-        assert(isinstance(cset.entries["1"].metadata_change,
-                          PermissionsMerge))
-        assert(isinstance(cset.entries["2"].metadata_change,
-                          PermissionsMerge))
-        assert(cset.entries["3"].is_boring())
-        builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
-        builder.cleanup();
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655, base=0555)
-        cset = builder.merge_changeset(ApplyMerge3)
-        self.assertRaises(changeset.MergePermissionConflict, 
-                     builder.apply_changeset, cset)
-        builder.cleanup()
-
-def test():        
-    changeset_suite = unittest.makeSuite(MergeTest, 'test_')
-    runner = unittest.TextTestRunner()
-    runner.run(changeset_suite)
-        
-if __name__ == "__main__":
-    test()

=== removed file 'bzrlib/meta_store.py'
--- bzrlib/meta_store.py	2005-06-27 01:36:22 +0000
+++ bzrlib/meta_store.py	1970-01-01 00:00:00 +0000
@@ -1,41 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-from trace import mutter
-from bzrlib.store import ImmutableStore
-
-class CachedStore:
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-    def __init__(self, store, cache_dir):
-        self.source_store = store
-        self.cache_store = ImmutableStore(cache_dir)
-
-    def __getitem__(self, id):
-        mutter("Cache add %s" % id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store[id], id)
-        return self.cache_store[id]
-
-    def prefetch(self, ids):
-        """Copy a series of ids into the cache, before they are used.
-        For remote stores that support pipelining or async downloads, this can
-        increase speed considerably.
-        """
-        mutter("Prefetch of ids %s" % ",".join(ids))
-        self.cache_store.copy_multi(self.source_store, ids)

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2005-08-12 15:37:01 +0000
+++ bzrlib/missing.py	2006-03-07 09:03:44 +0000
@@ -1,104 +1,104 @@
-#!/usr/bin/env python
-"""\
-A plugin for displaying what revisions are in 'other' but not in local.
-"""
-
-import bzrlib
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-
-def get_parent(branch):
-    """Get the last pull/push location.
-
-    TODO: This should probably be part of a Branch object.
-    """
-    import errno
-    _locs = ['parent', 'pull', 'x-pull']
-    loc = None
-    for l in _locs:
-        try:
-            stored_loc = branch.controlfile(l, 'rb').read().strip('\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        else:
-            return stored_loc
-
-
-def show_missing(br_local, br_remote, verbose=False, quiet=False):
-    """Show the revisions which exist in br_remote, that 
-    do not exist in br_local.
-    """
-    from bzrlib.log import show_one_log
-    import sys
-    local_history = br_local.revision_history()
-    remote_history = br_remote.revision_history()
-    if local_history == remote_history:
-        if not quiet:
-            print 'Trees are identical.'
-        return 0
-    if local_history[:len(remote_history)] == remote_history:
-        # Local is not missing anything from remote, so consider it
-        # up-to-date
-        if not quiet:
-            print 'Local tree has all of remote revisions (remote is missing local)'
-        return 0
-    if quiet:
-        return 1
-
-    # Check for divergence
-    common_idx = min(len(local_history), len(remote_history)) - 1
-    if common_idx >= 0 and local_history[common_idx] != remote_history[common_idx]:
-        print 'Trees have diverged'
-
-    local_rev_set = set(local_history)
-
-    # Find the last common revision between the two trees
-    revno = 0
-    for revno, (local_rev, remote_rev) in enumerate(zip(local_history, remote_history)):
-        if local_rev != remote_rev:
-            break
-
-    missing_remote = []
-    for rno, rev_id in enumerate(remote_history[revno:]):
-        # This assumes that you can have a revision in the
-        # local history, which does not have the same ancestry
-        # as the remote ancestry.
-        # This may or may not be possible.
-        # In the future this should also checked for merged revisions.
-        if rev_id not in local_rev_set:
-            missing_remote.append((rno+revno+1, rev_id))
-
-    print 'Missing %d revisions' %  len(missing_remote)
-    print
-
-    if verbose:
-        from bzrlib.diff import compare_trees
-        from bzrlib.tree import EmptyTree
-        show_ids = True
-        last_tree = EmptyTree
-        last_rev_id = None
-    else:
-        show_ids = False
-    for revno, rev_id in missing_remote:
-        rev = br_remote.get_revision(rev_id)
+"""Display what revisions are missing in 'other' from 'this' and vice verca."""
+
+import bzrlib.ui as ui
+
+
+def iter_log_data(revisions, revision_source, verbose):
+    from bzrlib.diff import compare_trees
+    from bzrlib.tree import EmptyTree
+    last_tree = EmptyTree
+    last_rev_id = None
+    for revno, rev_id in revisions:
+        rev = revision_source.get_revision(rev_id)
         if verbose:
-            parent_rev_id = rev.parents[0].revision_id
+            remote_tree = revision_source.revision_tree(rev_id)
+            parent_rev_id = rev.parent_ids[0]
             if last_rev_id == parent_rev_id:
                 parent_tree = last_tree
             else:
-                parent_tree = br_remote.revision_tree(parent_rev_id)
-            revision_tree = br_remote.revision_tree(rev_id)
+                parent_tree = revision_source.revision_tree(parent_rev_id)
+            revision_tree = revision_source.revision_tree(rev_id)
             last_rev_id = rev_id
             last_tree = revision_tree
             delta = compare_trees(revision_tree, parent_tree)
         else:
             delta = None
-
-        show_one_log(revno, rev, delta, verbose, sys.stdout, 'original')
-    return 1
-
+        yield revno, rev, delta
+
+
+def find_unmerged(local_branch, remote_branch):
+    progress = ui.ui_factory.nested_progress_bar()
+    local_branch.lock_read()
+    try:
+        remote_branch.lock_read()
+        try:
+            local_rev_history, local_rev_history_map = \
+                _get_history(local_branch, progress, "local", 0)
+            remote_rev_history, remote_rev_history_map = \
+                _get_history(remote_branch, progress, "remote", 1)
+            result = _shortcut(local_rev_history, remote_rev_history)
+            if result is not None:
+                local_extra, remote_extra = result
+                local_extra = sorted_revisions(local_extra, 
+                                               local_rev_history_map)
+                remote_extra = sorted_revisions(remote_extra, 
+                                                remote_rev_history_map)
+                return local_extra, remote_extra
+
+            local_ancestry = _get_ancestry(local_branch.repository, progress, 
+                                           "local", 2, local_rev_history)
+            remote_ancestry = _get_ancestry(remote_branch.repository, progress,
+                                            "remote", 3, remote_rev_history)
+            progress.update('pondering', 4, 5)
+            extras = local_ancestry.symmetric_difference(remote_ancestry) 
+            local_extra = extras.intersection(set(local_rev_history))
+            remote_extra = extras.intersection(set(remote_rev_history))
+            local_extra = sorted_revisions(local_extra, local_rev_history_map)
+            remote_extra = sorted_revisions(remote_extra, 
+                                            remote_rev_history_map)
+                    
+        finally:
+            remote_branch.unlock()
+    finally:
+        local_branch.unlock()
+        progress.finished()
+    return (local_extra, remote_extra)
+
+def _shortcut(local_rev_history, remote_rev_history):
+    local_history = set(local_rev_history)
+    remote_history = set(remote_rev_history)
+    if len(local_rev_history) == 0:
+        return set(), remote_history
+    elif len(remote_rev_history) == 0:
+        return local_history, set()
+    elif local_rev_history[-1] in remote_history:
+        return set(), _after(remote_rev_history, local_rev_history)
+    elif remote_rev_history[-1] in local_history:
+        return _after(local_rev_history, remote_rev_history), set()
+    else:
+        return None
+
+def _after(larger_history, smaller_history):
+    return set(larger_history[larger_history.index(smaller_history[-1])+1:])
+
+def _get_history(branch, progress, label, step):
+    progress.update('%s history' % label, step, 5)
+    rev_history = branch.revision_history()
+    rev_history_map = dict(
+        [(rev, rev_history.index(rev) + 1)
+         for rev in rev_history])
+    return rev_history, rev_history_map
+
+def _get_ancestry(repository, progress, label, step, rev_history):
+    progress.update('%s ancestry' % label, step, 5)
+    if len(rev_history) > 0:
+        ancestry = set(repository.get_ancestry(rev_history[-1]))
+    else:
+        ancestry = set()
+    return ancestry
+    
+
+def sorted_revisions(revisions, history_map):
+    revisions = [(history_map[r],r) for r in revisions]
+    revisions.sort()
+    return revisions

=== added file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	1970-01-01 00:00:00 +0000
+++ bzrlib/msgeditor.py	2006-03-09 00:24:48 +0000
@@ -0,0 +1,156 @@
+# Bazaar-NG -- distributed version control
+
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Commit message editor support."""
+
+import os
+import errno
+from subprocess import call
+
+import bzrlib.config as config
+from bzrlib.errors import BzrError
+
+def _get_editor():
+    """Return a sequence of possible editor binaries for the current platform"""
+    try:
+        yield os.environ["BZR_EDITOR"]
+    except KeyError:
+        pass
+
+    e = config.GlobalConfig().get_editor()
+    if e is not None:
+        yield e
+        
+    try:
+        yield os.environ["EDITOR"]
+    except KeyError:
+        pass
+
+    if os.name == "nt":
+        yield "notepad.exe"
+    elif os.name == "posix":
+        yield "/usr/bin/vi"
+
+
+def _run_editor(filename):
+    """Try to execute an editor to edit the commit message."""
+    for e in _get_editor():
+        edargs = e.split(' ')
+        try:
+            x = call(edargs + [filename])
+        except OSError, e:
+           # ENOENT means no such editor
+           if e.errno == errno.ENOENT:
+               continue
+           raise
+        if x == 0:
+            return True
+        elif x == 127:
+            continue
+        else:
+            break
+    raise BzrError("Could not start any editor. "
+                   "Please specify $EDITOR or use ~/.bzr.conf/editor")
+
+
+DEFAULT_IGNORE_LINE = "%(bar)s %(msg)s %(bar)s" % \
+    { 'bar' : '-' * 14, 'msg' : 'This line and the following will be ignored' }
+
+
+def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE):
+    """Let the user edit a commit message in a temp file.
+
+    This is run if they don't give a message or
+    message-containing file on the command line.
+
+    infotext:
+        Text to be displayed at bottom of message for
+        the user's reference; currently similar to
+        'bzr status'.
+    """
+    import tempfile
+
+    try:
+        tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.', dir=u'.')
+        msgfile = os.close(tmp_fileno)
+        if infotext is not None and infotext != "":
+            hasinfo = True
+            msgfile = file(msgfilename, "w")
+            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
+            msgfile.close()
+        else:
+            hasinfo = False
+
+        if not _run_editor(msgfilename):
+            return None
+        
+        started = False
+        msg = []
+        lastline, nlines = 0, 0
+        for line in file(msgfilename, "r"):
+            stripped_line = line.strip()
+            # strip empty line before the log message starts
+            if not started:
+                if stripped_line != "":
+                    started = True
+                else:
+                    continue
+            # check for the ignore line only if there
+            # is additional information at the end
+            if hasinfo and stripped_line == ignoreline:
+                break
+            nlines += 1
+            # keep track of the last line that had some content
+            if stripped_line != "":
+                lastline = nlines
+            msg.append(line)
+            
+        if len(msg) == 0:
+            return ""
+        # delete empty lines at the end
+        del msg[lastline:]
+        # add a newline at the end, if needed
+        if not msg[-1].endswith("\n"):
+            return "%s%s" % ("".join(msg), "\n")
+        else:
+            return "".join(msg)
+    finally:
+        # delete the msg file in any case
+        try: os.unlink(msgfilename)
+        except IOError: pass
+
+
+def make_commit_message_template(working_tree, specific_files):
+    """Prepare a template file for a commit into a branch.
+
+    Returns a unicode string containing the template.
+    """
+    # TODO: Should probably be given the WorkingTree not the branch
+    #
+    # TODO: make provision for this to be overridden or modified by a hook
+    #
+    # TODO: Rather than running the status command, should prepare a draft of
+    # the revision to be committed, then pause and ask the user to
+    # confirm/write a message.
+    from StringIO import StringIO       # must be unicode-safe
+    from bzrlib.status import show_tree_status
+    status_tmp = StringIO()
+    show_tree_status(working_tree, specific_files=specific_files, 
+                     to_file=status_tmp)
+    return status_tmp.getvalue()

=== added file 'bzrlib/option.py'
--- bzrlib/option.py	1970-01-01 00:00:00 +0000
+++ bzrlib/option.py	2006-04-18 00:48:02 +0000
@@ -0,0 +1,216 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import re
+
+import bzrlib.commands
+from bzrlib.trace import warning, mutter
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.errors import BzrCommandError
+
+
+def _parse_revision_str(revstr):
+    """This handles a revision string -> revno.
+
+    This always returns a list.  The list will have one element for
+    each revision specifier supplied.
+
+    >>> _parse_revision_str('234')
+    [<RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..567')
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
+    >>> _parse_revision_str('..')
+    [<RevisionSpec None>, <RevisionSpec None>]
+    >>> _parse_revision_str('..234')
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..')
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
+    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('234....789') #Error ?
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('revid:test@other.com-234234')
+    [<RevisionSpec_revid revid:test@other.com-234234>]
+    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
+    >>> _parse_revision_str('revid:test@other.com-234234..23')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('date:2005-04-12')
+    [<RevisionSpec_date date:2005-04-12>]
+    >>> _parse_revision_str('date:2005-04-12 12:24:33')
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12T12:24:33')
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12,12:24:33')
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
+    >>> _parse_revision_str('-5..23')
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('-5')
+    [<RevisionSpec_int -5>]
+    >>> _parse_revision_str('123a')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
+    >>> _parse_revision_str('abc')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
+    >>> _parse_revision_str('branch:../branch2')
+    [<RevisionSpec_branch branch:../branch2>]
+    >>> _parse_revision_str('branch:../../branch2')
+    [<RevisionSpec_branch branch:../../branch2>]
+    >>> _parse_revision_str('branch:../../branch2..23')
+    [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_int 23>]
+    """
+    # TODO: Maybe move this into revisionspec.py
+    old_format_re = re.compile('\d*:\d*')
+    m = old_format_re.match(revstr)
+    revs = []
+    if m:
+        warning('Colon separator for revision numbers is deprecated.'
+                ' Use .. instead')
+        for rev in revstr.split(':'):
+            if rev:
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        sep = re.compile("\\.\\.(?!/)")
+        for x in sep.split(revstr):
+            revs.append(RevisionSpec(x or None))
+    return revs
+
+
+def _parse_merge_type(typestring):
+    return get_merge_type(typestring)
+
+def get_merge_type(typestring):
+    """Attempt to find the merge class/factory associated with a string."""
+    from merge import merge_types
+    try:
+        return merge_types[typestring][0]
+    except KeyError:
+        templ = '%s%%7s: %%s' % (' '*12)
+        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
+        type_list = '\n'.join(lines)
+        msg = "No known merge type %s. Supported types are:\n%s" %\
+            (typestring, type_list)
+        raise BzrCommandError(msg)
+
+class Option(object):
+    """Description of a command line option"""
+    # TODO: Some way to show in help a description of the option argument
+
+    OPTIONS = {}
+    SHORT_OPTIONS = {}
+
+    def __init__(self, name, help='', type=None, argname=None):
+        """Make a new command option.
+
+        name -- regular name of the command, used in the double-dash
+            form and also as the parameter to the command's run() 
+            method.
+
+        help -- help message displayed in command help
+
+        type -- function called to parse the option argument, or 
+            None (default) if this option doesn't take an argument.
+
+        argname -- name of option argument, if any
+        """
+        # TODO: perhaps a subclass that automatically does 
+        # --option, --no-option for reversable booleans
+        self.name = name
+        self.help = help
+        self.type = type
+        if type is None:
+            assert argname is None
+        elif argname is None:
+            argname = 'ARG'
+        self.argname = argname
+
+    def short_name(self):
+        """Return the single character option for this command, if any.
+
+        Short options are globally registered.
+        """
+        for short, option in Option.SHORT_OPTIONS.iteritems():
+            if option is self:
+                return short
+
+
+def _global_option(name, **kwargs):
+    """Register o as a global option."""
+    Option.OPTIONS[name] = Option(name, **kwargs)
+
+_global_option('all')
+_global_option('overwrite', help='Ignore differences between branches and '
+               'overwrite unconditionally')
+_global_option('basis', type=str)
+_global_option('bound')
+_global_option('diff-options', type=str)
+_global_option('help',
+               help='show help message')
+_global_option('file', type=unicode)
+_global_option('force')
+_global_option('format', type=unicode)
+_global_option('forward')
+_global_option('message', type=unicode)
+_global_option('no-recurse')
+_global_option('profile',
+               help='show performance profiling information')
+_global_option('revision', type=_parse_revision_str)
+_global_option('show-ids', 
+               help='show internal object ids')
+_global_option('timezone', 
+               type=str,
+               help='display timezone as local, original, or utc')
+_global_option('unbound')
+_global_option('verbose',
+               help='display more information')
+_global_option('version')
+_global_option('email')
+_global_option('update')
+_global_option('log-format', type=str, help="Use this log format")
+_global_option('long', help='Use detailed log format. Same as --log-format long')
+_global_option('short', help='Use moderately short log format. Same as --log-format short')
+_global_option('line', help='Use log format with one line per revision. Same as --log-format line')
+_global_option('root', type=str)
+_global_option('no-backup')
+_global_option('merge-type', type=_parse_merge_type)
+_global_option('pattern', type=str)
+_global_option('quiet')
+_global_option('remember', help='Remember the specified location as a'
+               ' default.')
+_global_option('reprocess', help='Reprocess to reduce spurious conflicts')
+_global_option('kind', type=str)
+_global_option('dry-run',
+               help="show what would be done, but don't actually do anything")
+
+
+def _global_short(short_name, long_name):
+    assert short_name not in Option.SHORT_OPTIONS
+    Option.SHORT_OPTIONS[short_name] = Option.OPTIONS[long_name]
+    
+
+Option.SHORT_OPTIONS['F'] = Option.OPTIONS['file']
+Option.SHORT_OPTIONS['h'] = Option.OPTIONS['help']
+Option.SHORT_OPTIONS['m'] = Option.OPTIONS['message']
+Option.SHORT_OPTIONS['r'] = Option.OPTIONS['revision']
+Option.SHORT_OPTIONS['v'] = Option.OPTIONS['verbose']
+Option.SHORT_OPTIONS['l'] = Option.OPTIONS['long']
+Option.SHORT_OPTIONS['q'] = Option.OPTIONS['quiet']

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-08-18 05:52:29 +0000
+++ bzrlib/osutils.py	2006-04-19 02:50:56 +0000
@@ -1,31 +1,46 @@
 # Bazaar-NG -- distributed version control
-
+#
 # Copyright (C) 2005 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import os, types, re, time, errno, sys
-from stat import S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE
+from shutil import copyfile
+from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
+                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+from cStringIO import StringIO
+import errno
+import os
+import re
+import sha
+import string
+import sys
+import time
+import types
+import tempfile
 
-from bzrlib.errors import BzrError
+import bzrlib
+from bzrlib.errors import (BzrError,
+                           BzrBadParameterNotUnicode,
+                           NoSuchFile,
+                           PathNotChild,
+                           )
 from bzrlib.trace import mutter
-import bzrlib
+
 
 def make_readonly(filename):
     """Make a filename read-only."""
-    # TODO: probably needs to be fixed for windows
     mod = os.stat(filename).st_mode
     mod = mod & 0777555
     os.chmod(filename, mod)
@@ -48,7 +63,7 @@
     # TODO: I'm not really sure this is the best format either.x
     global _QUOTE_RE
     if _QUOTE_RE == None:
-        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/\\_~-])')
         
     if _QUOTE_RE.search(f):
         return '"' + f + '"'
@@ -64,8 +79,16 @@
         return 'directory'
     elif S_ISLNK(mode):
         return 'symlink'
+    elif S_ISCHR(mode):
+        return 'chardev'
+    elif S_ISBLK(mode):
+        return 'block'
+    elif S_ISFIFO(mode):
+        return 'fifo'
+    elif S_ISSOCK(mode):
+        return 'socket'
     else:
-        raise BzrError("can't handle file kind with mode %o of %r" % (mode, f))
+        return 'unknown'
 
 
 def kind_marker(kind):
@@ -78,6 +101,133 @@
     else:
         raise BzrError('invalid file kind %r' % kind)
 
+def lexists(f):
+    if hasattr(os.path, 'lexists'):
+        return os.path.lexists(f)
+    try:
+        if hasattr(os, 'lstat'):
+            os.lstat(f)
+        else:
+            os.stat(f)
+        return True
+    except OSError,e:
+        if e.errno == errno.ENOENT:
+            return False;
+        else:
+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
+
+def fancy_rename(old, new, rename_func, unlink_func):
+    """A fancy rename, when you don't have atomic rename.
+    
+    :param old: The old path, to rename from
+    :param new: The new path, to rename to
+    :param rename_func: The potentially non-atomic rename function
+    :param unlink_func: A way to delete the target file if the full rename succeeds
+    """
+
+    # sftp rename doesn't allow overwriting, so play tricks:
+    import random
+    base = os.path.basename(new)
+    dirname = os.path.dirname(new)
+    tmp_name = u'tmp.%s.%.9f.%d.%s' % (base, time.time(), os.getpid(), rand_chars(10))
+    tmp_name = pathjoin(dirname, tmp_name)
+
+    # Rename the file out of the way, but keep track if it didn't exist
+    # We don't want to grab just any exception
+    # something like EACCES should prevent us from continuing
+    # The downside is that the rename_func has to throw an exception
+    # with an errno = ENOENT, or NoSuchFile
+    file_existed = False
+    try:
+        rename_func(new, tmp_name)
+    except (NoSuchFile,), e:
+        pass
+    except IOError, e:
+        # RBC 20060103 abstraction leakage: the paramiko SFTP clients rename
+        # function raises an IOError with errno == None when a rename fails.
+        # This then gets caught here.
+        if e.errno not in (None, errno.ENOENT, errno.ENOTDIR):
+            raise
+    except Exception, e:
+        if (not hasattr(e, 'errno') 
+            or e.errno not in (errno.ENOENT, errno.ENOTDIR)):
+            raise
+    else:
+        file_existed = True
+
+    success = False
+    try:
+        # This may throw an exception, in which case success will
+        # not be set.
+        rename_func(old, new)
+        success = True
+    finally:
+        if file_existed:
+            # If the file used to exist, rename it back into place
+            # otherwise just delete it from the tmp location
+            if success:
+                unlink_func(tmp_name)
+            else:
+                rename_func(tmp_name, new)
+
+# Default is to just use the python builtins
+abspath = os.path.abspath
+realpath = os.path.realpath
+pathjoin = os.path.join
+normpath = os.path.normpath
+getcwd = os.getcwdu
+mkdtemp = tempfile.mkdtemp
+rename = os.rename
+dirname = os.path.dirname
+basename = os.path.basename
+
+if os.name == "posix":
+    # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
+    # choke on a Unicode string containing a relative path if
+    # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
+    # string.
+    _fs_enc = sys.getfilesystemencoding()
+    def abspath(path):
+        return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
+
+    def realpath(path):
+        return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
+
+if sys.platform == 'win32':
+    # We need to use the Unicode-aware os.path.abspath and
+    # os.path.realpath on Windows systems.
+    def abspath(path):
+        return os.path.abspath(path).replace('\\', '/')
+
+    def realpath(path):
+        return os.path.realpath(path).replace('\\', '/')
+
+    def pathjoin(*args):
+        return os.path.join(*args).replace('\\', '/')
+
+    def normpath(path):
+        return os.path.normpath(path).replace('\\', '/')
+
+    def getcwd():
+        return os.getcwdu().replace('\\', '/')
+
+    def mkdtemp(*args, **kwargs):
+        return tempfile.mkdtemp(*args, **kwargs).replace('\\', '/')
+
+    def rename(old, new):
+        fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
+
+
+def normalizepath(f):
+    if hasattr(os.path, 'realpath'):
+        F = realpath
+    else:
+        F = abspath
+    [p,e] = os.path.split(f)
+    if e == "" or e == "." or e == "..":
+        return F(f)
+    else:
+        return pathjoin(F(p), e)
 
 
 def backup_file(fn):
@@ -87,11 +237,14 @@
 
     If the file is already a backup, it's not copied.
     """
-    import os
     if fn[-1] == '~':
         return
     bfn = fn + '~'
 
+    if has_symlinks() and os.path.islink(fn):
+        target = os.readlink(fn)
+        os.symlink(target, bfn)
+        return
     inf = file(fn, 'rb')
     try:
         content = inf.read()
@@ -104,19 +257,6 @@
     finally:
         outf.close()
 
-def rename(path_from, path_to):
-    """Basically the same as os.rename() just special for win32"""
-    if sys.platform == 'win32':
-        try:
-            os.remove(path_to)
-        except OSError, e:
-            if e.errno != e.ENOENT:
-                raise
-    os.rename(path_from, path_to)
-
-
-
-
 
 def isdir(f):
     """True if f is an accessible directory."""
@@ -126,7 +266,6 @@
         return False
 
 
-
 def isfile(f):
     """True if f is a regular file."""
     try:
@@ -134,22 +273,28 @@
     except OSError:
         return False
 
+def islink(f):
+    """True if f is a symlink."""
+    try:
+        return S_ISLNK(os.lstat(f)[ST_MODE])
+    except OSError:
+        return False
 
 def is_inside(dir, fname):
     """True if fname is inside dir.
     
-    The parameters should typically be passed to os.path.normpath first, so
+    The parameters should typically be passed to osutils.normpath first, so
     that . and .. and repeated slashes are eliminated, and the separators
     are canonical for the platform.
     
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
     
-    >>> is_inside('src', 'src/foo.c')
+    >>> is_inside('src', pathjoin('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', 'src/a/a/a/foo.c')
+    >>> is_inside('src', pathjoin('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
@@ -165,10 +310,10 @@
     
     if dir == '':
         return True
-    
-    if dir[-1] != os.sep:
-        dir += os.sep
-    
+
+    if dir[-1] != '/':
+        dir += '/'
+
     return fname.startswith(dir)
 
 
@@ -183,19 +328,23 @@
 
 def pumpfile(fromfile, tofile):
     """Copy contents of one file to another."""
-    tofile.write(fromfile.read())
-
-
-def uuid():
-    """Return a new UUID"""
-    try:
-        return file('/proc/sys/kernel/random/uuid').readline().rstrip('\n')
-    except IOError:
-        return chomp(os.popen('uuidgen').readline())
+    BUFSIZE = 32768
+    while True:
+        b = fromfile.read(BUFSIZE)
+        if not b:
+            break
+        tofile.write(b)
+
+
+def file_iterator(input_file, readsize=32768):
+    while True:
+        b = input_file.read(readsize)
+        if len(b) == 0:
+            break
+        yield b
 
 
 def sha_file(f):
-    import sha
     if hasattr(f, 'tell'):
         assert f.tell() == 0
     s = sha.new()
@@ -208,16 +357,21 @@
     return s.hexdigest()
 
 
+
+def sha_strings(strings):
+    """Return the sha-1 of concatenation of strings"""
+    s = sha.new()
+    map(s.update, strings)
+    return s.hexdigest()
+
+
 def sha_string(f):
-    import sha
     s = sha.new()
     s.update(f)
     return s.hexdigest()
 
 
-
 def fingerprint_file(f):
-    import sha
     s = sha.new()
     b = f.read()
     s.update(b)
@@ -226,130 +380,6 @@
             'sha1': s.hexdigest()}
 
 
-def config_dir():
-    """Return per-user configuration directory.
-
-    By default this is ~/.bzr.conf/
-    
-    TODO: Global option --config-dir to override this.
-    """
-    return os.path.expanduser("~/.bzr.conf")
-
-
-def _auto_user_id():
-    """Calculate automatic user identification.
-
-    Returns (realname, email).
-
-    Only used when none is set in the environment or the id file.
-
-    This previously used the FQDN as the default domain, but that can
-    be very slow on machines where DNS is broken.  So now we simply
-    use the hostname.
-    """
-    import socket
-
-    # XXX: Any good way to get real user name on win32?
-
-    try:
-        import pwd
-        uid = os.getuid()
-        w = pwd.getpwuid(uid)
-        gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-        username = w.pw_name.decode(bzrlib.user_encoding)
-        comma = gecos.find(',')
-        if comma == -1:
-            realname = gecos
-        else:
-            realname = gecos[:comma]
-        if not realname:
-            realname = username
-
-    except ImportError:
-        import getpass
-        realname = username = getpass.getuser().decode(bzrlib.user_encoding)
-
-    return realname, (username + '@' + socket.gethostname())
-
-
-def _get_user_id(branch):
-    """Return the full user id from a file or environment variable.
-
-    e.g. "John Hacker <jhacker@foo.org>"
-
-    branch
-        A branch to use for a per-branch configuration, or None.
-
-    The following are searched in order:
-
-    1. $BZREMAIL
-    2. .bzr/email for this branch.
-    3. ~/.bzr.conf/email
-    4. $EMAIL
-    """
-    v = os.environ.get('BZREMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-
-    if branch:
-        try:
-            return (branch.controlfile("email", "r") 
-                    .read()
-                    .decode(bzrlib.user_encoding)
-                    .rstrip("\r\n"))
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        except BzrError, e:
-            pass
-    
-    try:
-        return (open(os.path.join(config_dir(), "email"))
-                .read()
-                .decode(bzrlib.user_encoding)
-                .rstrip("\r\n"))
-    except IOError, e:
-        if e.errno != errno.ENOENT:
-            raise e
-
-    v = os.environ.get('EMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-    else:    
-        return None
-
-
-def username(branch):
-    """Return email-style username.
-
-    Something similar to 'Martin Pool <mbp@sourcefrog.net>'
-
-    TODO: Check it's reasonably well-formed.
-    """
-    v = _get_user_id(branch)
-    if v:
-        return v
-    
-    name, email = _auto_user_id()
-    if name:
-        return '%s <%s>' % (name, email)
-    else:
-        return email
-
-
-def user_email(branch):
-    """Return just the email component of a username."""
-    e = _get_user_id(branch)
-    if e:
-        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
-        if not m:
-            raise BzrError("%r doesn't seem to contain a reasonable email address" % e)
-        return m.group(0)
-
-    return _auto_user_id()[1]
-    
-
-
 def compare_files(a, b):
     """Returns true if equal in contents"""
     BUFSIZE = 4096
@@ -362,7 +392,6 @@
             return True
 
 
-
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
     # python2.3 localtime() can't take None
@@ -375,7 +404,8 @@
         return -time.timezone
 
     
-def format_date(t, offset=0, timezone='original'):
+def format_date(t, offset=0, timezone='original', date_fmt=None, 
+                show_offset=True):
     ## TODO: Perhaps a global option to use either universal or local time?
     ## Or perhaps just let people set $TZ?
     assert isinstance(t, float)
@@ -393,9 +423,13 @@
     else:
         raise BzrError("unsupported timezone format %r" % timezone,
                        ['options are "utc", "original", "local"'])
-
-    return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
-            + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
+    if date_fmt is None:
+        date_fmt = "%a %Y-%m-%d %H:%M:%S"
+    if show_offset:
+        offset_str = ' %+03d%02d' % (offset / 3600, (offset / 60) % 60)
+    else:
+        offset_str = ''
+    return (time.strftime(date_fmt, tt) +  offset_str)
 
 
 def compact_date(when):
@@ -408,19 +442,40 @@
     return os.fstat(f.fileno())[ST_SIZE]
 
 
-if hasattr(os, 'urandom'): # python 2.4 and later
+# Define rand_bytes based on platform.
+try:
+    # Python 2.4 and later have os.urandom,
+    # but it doesn't work on some arches
+    os.urandom(1)
     rand_bytes = os.urandom
-elif sys.platform == 'linux2':
-    rand_bytes = file('/dev/urandom', 'rb').read
-else:
-    # not well seeded, but better than nothing
-    def rand_bytes(n):
-        import random
-        s = ''
-        while n:
-            s += chr(random.randint(0, 255))
-            n -= 1
-        return s
+except (NotImplementedError, AttributeError):
+    # If python doesn't have os.urandom, or it doesn't work,
+    # then try to first pull random data from /dev/urandom
+    if os.path.exists("/dev/urandom"):
+        rand_bytes = file('/dev/urandom', 'rb').read
+    # Otherwise, use this hack as a last resort
+    else:
+        # not well seeded, but better than nothing
+        def rand_bytes(n):
+            import random
+            s = ''
+            while n:
+                s += chr(random.randint(0, 255))
+                n -= 1
+            return s
+
+
+ALNUM = '0123456789abcdefghijklmnopqrstuvwxyz'
+def rand_chars(num):
+    """Return a random string of num alphanumeric characters
+    
+    The result only contains lowercase chars because it may be used on 
+    case-insensitive filesystems.
+    """
+    s = ''
+    for raw_byte in rand_bytes(num):
+        s += ALNUM[ord(raw_byte) % 36]
+    return s
 
 
 ## TODO: We could later have path objects that remember their list
@@ -463,116 +518,138 @@
     for f in p:
         if (f == '..') or (f == None) or (f == ''):
             raise BzrError("sorry, %r not allowed in path" % f)
-    return os.path.join(*p)
+    return pathjoin(*p)
 
 
 def appendpath(p1, p2):
     if p1 == '':
         return p2
     else:
-        return os.path.join(p1, p2)
+        return pathjoin(p1, p2)
     
 
-def extern_command(cmd, ignore_errors = False):
-    mutter('external command: %s' % `cmd`)
-    if os.system(cmd):
-        if not ignore_errors:
-            raise BzrError('command failed')
-
-
-def _read_config_value(name):
-    """Read a config value from the file ~/.bzr.conf/<name>
-    Return None if the file does not exist"""
-    try:
-        f = file(os.path.join(config_dir(), name), "r")
-        return f.read().decode(bzrlib.user_encoding).rstrip("\r\n")
-    except IOError, e:
-        if e.errno == errno.ENOENT:
-            return None
-        raise
-
-
-def _get_editor():
-    """Return a sequence of possible editor binaries for the current platform"""
-    e = _read_config_value("editor")
-    if e is not None:
-        yield e
+def split_lines(s):
+    """Split s into lines, but without removing the newline characters."""
+    return StringIO(s).readlines()
+
+
+def hardlinks_good():
+    return sys.platform not in ('win32', 'cygwin', 'darwin')
+
+
+def link_or_copy(src, dest):
+    """Hardlink a file, or copy it if it can't be hardlinked."""
+    if not hardlinks_good():
+        copyfile(src, dest)
+        return
+    try:
+        os.link(src, dest)
+    except (OSError, IOError), e:
+        if e.errno != errno.EXDEV:
+            raise
+        copyfile(src, dest)
+
+def delete_any(full_path):
+    """Delete a file or directory."""
+    try:
+        os.unlink(full_path)
+    except OSError, e:
+    # We may be renaming a dangling inventory id
+        if e.errno not in (errno.EISDIR, errno.EACCES, errno.EPERM):
+            raise
+        os.rmdir(full_path)
+
+
+def has_symlinks():
+    if hasattr(os, 'symlink'):
+        return True
+    else:
+        return False
         
-    if os.name == "windows":
-        yield "notepad.exe"
-    elif os.name == "posix":
-        try:
-            yield os.environ["EDITOR"]
-        except KeyError:
-            yield "/usr/bin/vi"
-
-
-def _run_editor(filename):
-    """Try to execute an editor to edit the commit message. Returns True on success,
-    False on failure"""
-    for e in _get_editor():
-        x = os.spawnvp(os.P_WAIT, e, (e, filename))
-        if x == 0:
-            return True
-        elif x == 127:
-            continue
-        else:
+
+def contains_whitespace(s):
+    """True if there are any whitespace characters in s."""
+    for ch in string.whitespace:
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def contains_linebreaks(s):
+    """True if there is any vertical whitespace in s."""
+    for ch in '\f\n\r':
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def relpath(base, path):
+    """Return path relative to base, or raise exception.
+
+    The path may be either an absolute path or a path relative to the
+    current working directory.
+
+    os.path.commonprefix (python2.4) has a bad bug that it works just
+    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
+    avoids that problem.
+    """
+    if sys.platform != "win32":
+        minlength = 1
+    else:
+        minlength = 3
+    assert len(base) >= minlength, ('Length of base must be equal or exceed the'
+        ' platform minimum length (which is %d)' % minlength)
+    rp = abspath(path)
+
+    s = []
+    head = rp
+    while len(head) >= len(base):
+        if head == base:
             break
-    raise BzrError("Could not start any editor. Please specify $EDITOR or use ~/.bzr.conf/editor")
-    return False
-                          
-
-def get_text_message(infotext, ignoreline = "default"):
-    import tempfile
-    
-    if ignoreline == "default":
-        ignoreline = "-- This line and the following will be ignored --"
-        
-    try:
-        tmp_fileno, msgfilename = tempfile.mkstemp()
-        msgfile = os.close(tmp_fileno)
-        if infotext is not None and infotext != "":
-            hasinfo = True
-            msgfile = file(msgfilename, "w")
-            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
-            msgfile.close()
-        else:
-            hasinfo = False
-
-        if not _run_editor(msgfilename):
-            return None
-        
-        started = False
-        msg = []
-        lastline, nlines = 0, 0
-        for line in file(msgfilename, "r"):
-            stripped_line = line.strip()
-            # strip empty line before the log message starts
-            if not started:
-                if stripped_line != "":
-                    started = True
-                else:
-                    continue
-            # check for the ignore line only if there
-            # is additional information at the end
-            if hasinfo and stripped_line == ignoreline:
-                break
-            nlines += 1
-            # keep track of the last line that had some content
-            if stripped_line != "":
-                lastline = nlines
-            msg.append(line)
-            
-        if len(msg) == 0:
-            return None
-        # delete empty lines at the end
-        del msg[lastline:]
-        # add a newline at the end, if needed
-        if not msg[-1].endswith("\n"):
-            return "%s%s" % ("".join(msg), "\n")
-        else:
-            return "".join(msg)
-    finally:
-        # delete the msg file in any case
-        try: os.unlink(msgfilename)
-        except IOError: pass
+        head, tail = os.path.split(head)
+        if tail:
+            s.insert(0, tail)
+    else:
+        # XXX This should raise a NotChildPath exception, as its not tied
+        # to branch anymore.
+        raise PathNotChild(rp, base)
+
+    if s:
+        return pathjoin(*s)
+    else:
+        return ''
+
+
+def safe_unicode(unicode_or_utf8_string):
+    """Coerce unicode_or_utf8_string into unicode.
+
+    If it is unicode, it is returned.
+    Otherwise it is decoded from utf-8. If a decoding error
+    occurs, it is wrapped as a If the decoding fails, the exception is wrapped 
+    as a BzrBadParameter exception.
+    """
+    if isinstance(unicode_or_utf8_string, unicode):
+        return unicode_or_utf8_string
+    try:
+        return unicode_or_utf8_string.decode('utf8')
+    except UnicodeDecodeError:
+        raise BzrBadParameterNotUnicode(unicode_or_utf8_string)
+
+
+def terminal_width():
+    """Return estimated terminal width."""
+
+    # TODO: Do something smart on Windows?
+
+    # TODO: Is there anything that gets a better update when the window
+    # is resized while the program is running? We could use the Python termcap
+    # library.
+    try:
+        return int(os.environ['COLUMNS'])
+    except (IndexError, KeyError, ValueError):
+        return 80
+
+def supports_executable():
+    return sys.platform != "win32"

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2005-08-17 08:32:56 +0000
+++ bzrlib/patch.py	2006-02-08 22:00:43 +0000
@@ -1,13 +1,21 @@
+import errno
 import os
 from subprocess import Popen, PIPE
+
+from bzrlib.errors import NoDiff3
 """
 Diff and patch functionality
 """
 __docformat__ = "restructuredtext"
 
 def write_to_cmd(args, input=""):
-    process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
-                    stderr=PIPE, close_fds=True)
+    if os.name != 'nt':
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE, close_fds=True)
+    else:
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE)
+
     stdout, stderr = process.communicate(input)
     status = process.wait()
     if status < 0:
@@ -48,7 +56,13 @@
     add_label(args, "ANCESTOR")
     add_label(args, "MERGE-SOURCE")
     args.extend((mine_path, older_path, yours_path))
-    output, stderr, status = write_to_cmd(args)
+    try:
+        output, stderr, status = write_to_cmd(args)
+    except OSError, e:
+        if e.errno == errno.ENOENT:
+            raise NoDiff3
+        else:
+            raise
     if status not in (0, 1):
         raise Exception(stderr)
     file(out_file, "wb").write(output)

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-08-17 02:06:19 +0000
+++ bzrlib/plugin.py	2006-02-13 01:28:03 +0000
@@ -15,55 +15,102 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-# This module implements plug-in support.
-# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-# update any bzrlib registries it wants to extend; for example, to add new
-# commands, import bzrlib.commands and add your new command to the
-# plugin_cmds variable.
-
-
+"""bzr python plugin support
+
+Any python module in $BZR_PLUGIN_PATH will be imported upon initialization of
+bzrlib. The module will be imported as 'bzrlib.plugins.$BASENAME(PLUGIN)'.
+In the plugin's main body, it should update any bzrlib registries it wants to
+extend; for example, to add new commands, import bzrlib.commands and add your
+new command to the plugin_cmds variable.
+"""
+
+# TODO: Refactor this to make it more testable.  The main problem at the
+# moment is that loading plugins affects the global process state -- for bzr
+# in general use it's a reasonable assumption that all plugins are loaded at
+# startup and then stay loaded, but this is less good for testing.
+# 
+# Several specific issues:
+#  - plugins can't be unloaded and will continue to effect later tests
+#  - load_plugins does nothing if called a second time
+#  - plugin hooks can't be removed
+#
+# Our options are either to remove these restrictions, or work around them by
+# loading the plugins into a different space than the one running the tests.
+# That could be either a separate Python interpreter or perhaps a new
+# namespace inside this interpreter.
+
+import imp
 import os
-from bzrlib.osutils import config_dir
-DEFAULT_PLUGIN_PATH = os.path.join(config_dir(), 'plugins')
-
-all_plugins = []
+import sys
+import types
+
+import bzrlib
+from bzrlib.config import config_dir
+from bzrlib.trace import log_error, mutter, log_exception, warning, \
+        log_exception_quietly
+from bzrlib.errors import BzrError
+from bzrlib import plugins
+from bzrlib.osutils import pathjoin
+
+DEFAULT_PLUGIN_PATH = pathjoin(config_dir(), 'plugins')
+
 _loaded = False
 
 
+def all_plugins():
+    """Return a dictionary of the plugins."""
+    result = {}
+    for name, plugin in bzrlib.plugins.__dict__.items():
+        if isinstance(plugin, types.ModuleType):
+            result[name] = plugin
+    return result
+
+
+def disable_plugins():
+    """Disable loading plugins.
+
+    Future calls to load_plugins() will be ignored.
+    """
+    # TODO: jam 20060131 This should probably also disable
+    #       load_from_dirs()
+    global _loaded
+    _loaded = True
+
+
 def load_plugins():
-    """
-    Find all python plugins and load them.
-
-    Loading a plugin means importing it into the python interpreter.
-    The plugin is expected to make calls to register commands when
-    it's loaded (or perhaps access other hooks in future.)
-
-    A list of plugs is stored in bzrlib.plugin.all_plugins for future
-    reference.
+    """Load bzrlib plugins.
 
     The environment variable BZR_PLUGIN_PATH is considered a delimited
     set of paths to look through. Each entry is searched for *.py
     files (and whatever other extensions are used in the platform,
     such as *.pyd).
+
+    load_from_dirs() provides the underlying mechanism and is called with
+    the default directory list to provide the normal behaviour.
     """
-
-    global all_plugins, _loaded
+    global _loaded
     if _loaded:
         # People can make sure plugins are loaded, they just won't be twice
         return
         #raise BzrError("plugins already initialized")
     _loaded = True
 
-    import sys, os, imp
-    
-    from bzrlib.trace import log_error, mutter, log_exception
-    from bzrlib.errors import BzrError
-    from bzrlib import plugins
-
-    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(":")
+    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(os.pathsep)
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
+    load_from_dirs(dirs)
+
+
+def load_from_dirs(dirs):
+    """Load bzrlib plugins found in each dir in dirs.
+
+    Loading a plugin means importing it into the python interpreter.
+    The plugin is expected to make calls to register commands when
+    it's loaded (or perhaps access other hooks in future.)
+
+    Plugins are loaded into bzrlib.plugins.NAME, and can be found there
+    for future reference.
+    """
     # The problem with imp.get_suffixes() is that it doesn't include
     # .pyo which is technically valid
     # It also means that "testmodule.so" will show up as both test and testmodule
@@ -73,21 +120,19 @@
     suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
     package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
     for d in dirs:
-        # going through them one by one allows different plugins with the same
-        # filename in different directories in the path
-        mutter('looking for plugins in %s' % d)
         if not d:
             continue
+        mutter('looking for plugins in %s', d)
         plugin_names = set()
         if not os.path.isdir(d):
             continue
         for f in os.listdir(d):
-            path = os.path.join(d, f)
+            path = pathjoin(d, f)
             if os.path.isdir(path):
                 for entry in package_entries:
                     # This directory should be a package, and thus added to
                     # the list
-                    if os.path.isfile(os.path.join(path, entry)):
+                    if os.path.isfile(pathjoin(path, entry)):
                         break
                 else: # This directory is not a package
                     continue
@@ -100,24 +145,30 @@
                         break
                 else:
                     continue
-            mutter('add plugin name %s' % f)
-            plugin_names.add(f)
+            if getattr(bzrlib.plugins, f, None):
+                mutter('Plugin name %s already loaded', f)
+            else:
+                mutter('add plugin name %s', f)
+                plugin_names.add(f)
 
         plugin_names = list(plugin_names)
         plugin_names.sort()
         for name in plugin_names:
             try:
                 plugin_info = imp.find_module(name, [d])
-                mutter('load plugin %r' % (plugin_info,))
+                mutter('load plugin %r', plugin_info)
                 try:
-                    plugin = imp.load_module('bzrlib.plugin.' + name,
+                    plugin = imp.load_module('bzrlib.plugins.' + name,
                                              *plugin_info)
-                    all_plugins.append(plugin)
+                    setattr(bzrlib.plugins, name, plugin)
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()
+
+                mutter('loaded succesfully')
+            except KeyboardInterrupt:
+                raise
             except Exception, e:
-                log_error('Unable to load plugin %r from %r' % (name, d))
-                log_error(str(e))
-                log_exception()
-
+                ## import pdb; pdb.set_trace()
+                warning('Unable to load plugin %r from %r' % (name, d))
+                log_exception_quietly()

=== modified file 'bzrlib/plugins/__init__.py'
--- bzrlib/plugins/__init__.py	2005-08-17 02:06:19 +0000
+++ bzrlib/plugins/__init__.py	2005-09-19 06:05:19 +0000
@@ -0,0 +1,1 @@
+"""Null placeholder plugin"""

=== removed file 'bzrlib/plugins/checkperms'
--- bzrlib/plugins/checkperms	2005-06-27 05:14:15 +0000
+++ bzrlib/plugins/checkperms	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-#! /bin/sh -ex
-
-root=`bzr root`
-
-find $root \! -perm -0444 
\ No newline at end of file

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2005-07-22 23:06:25 +0000
+++ bzrlib/progress.py	2006-04-07 04:46:31 +0000
@@ -1,5 +1,5 @@
 # Copyright (C) 2005 Aaron Bentley <aaron.bentley@utoronto.ca>
-# Copyright (C) 2005 Canonical <canonical.com>
+# Copyright (C) 2005, 2006 Canonical <canonical.com>
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -39,20 +39,11 @@
 import sys
 import time
 import os
-
-
-def _width():
-    """Return estimated terminal width.
-
-    TODO: Do something smart on Windows?
-
-    TODO: Is there anything that gets a better update when the window
-          is resized while the program is running?
-    """
-    try:
-        return int(os.environ['COLUMNS'])
-    except (IndexError, KeyError, ValueError):
-        return 80
+from collections import deque
+
+
+import bzrlib.errors as errors
+from bzrlib.trace import mutter 
 
 
 def _supports_progress(f):
@@ -74,18 +65,80 @@
     else:
         return DotsProgressBar(to_file=to_file, **kwargs)
     
-    
+ 
+class ProgressBarStack(object):
+    """A stack of progress bars."""
+
+    def __init__(self,
+                 to_file=sys.stderr,
+                 show_pct=False,
+                 show_spinner=True,
+                 show_eta=False,
+                 show_bar=True,
+                 show_count=True,
+                 to_messages_file=sys.stdout,
+                 klass=None):
+        """Setup the stack with the parameters the progress bars should have."""
+        self._to_file = to_file
+        self._show_pct = show_pct
+        self._show_spinner = show_spinner
+        self._show_eta = show_eta
+        self._show_bar = show_bar
+        self._show_count = show_count
+        self._to_messages_file = to_messages_file
+        self._stack = []
+        self._klass = klass or TTYProgressBar
+
+    def top(self):
+        if len(self._stack) != 0:
+            return self._stack[-1]
+        else:
+            return None
+
+    def bottom(self):
+        if len(self._stack) != 0:
+            return self._stack[0]
+        else:
+            return None
+
+    def get_nested(self):
+        """Return a nested progress bar."""
+        if len(self._stack) == 0:
+            func = self._klass
+        else:
+            func = self.top().child_progress
+        new_bar = func(to_file=self._to_file,
+                       show_pct=self._show_pct,
+                       show_spinner=self._show_spinner,
+                       show_eta=self._show_eta,
+                       show_bar=self._show_bar,
+                       show_count=self._show_count,
+                       to_messages_file=self._to_messages_file,
+                       _stack=self)
+        self._stack.append(new_bar)
+        return new_bar
+
+    def return_pb(self, bar):
+        """Return bar after its been used."""
+        if bar is not self._stack[-1]:
+            raise errors.MissingProgressBarFinish()
+        self._stack.pop()
+
+ 
 class _BaseProgressBar(object):
+
     def __init__(self,
                  to_file=sys.stderr,
                  show_pct=False,
                  show_spinner=False,
                  show_eta=True,
                  show_bar=True,
-                 show_count=True):
+                 show_count=True,
+                 to_messages_file=sys.stdout,
+                 _stack=None):
         object.__init__(self)
         self.to_file = to_file
- 
+        self.to_messages_file = to_messages_file
         self.last_msg = None
         self.last_cnt = None
         self.last_total = None
@@ -94,10 +147,56 @@
         self.show_eta = show_eta
         self.show_bar = show_bar
         self.show_count = show_count
-        
-        
-    
+        self._stack = _stack
+        # seed throttler
+        self.MIN_PAUSE = 0.1 # seconds
+        now = time.clock()
+        # starting now
+        self.start_time = now
+        # next update should not throttle
+        self.last_update = now - self.MIN_PAUSE - 1
+
+    def finished(self):
+        """Return this bar to its progress stack."""
+        self.clear()
+        assert self._stack is not None
+        self._stack.return_pb(self)
+
+    def note(self, fmt_string, *args, **kwargs):
+        """Record a note without disrupting the progress bar."""
+        self.clear()
+        self.to_messages_file.write(fmt_string % args)
+        self.to_messages_file.write('\n')
+
+    def child_progress(self, **kwargs):
+        return ChildProgress(**kwargs)
+
+
+class DummyProgress(_BaseProgressBar):
+    """Progress-bar standin that does nothing.
+
+    This can be used as the default argument for methods that
+    take an optional progress indicator."""
+    def tick(self):
+        pass
+
+    def update(self, msg=None, current=None, total=None):
+        pass
+
+    def child_update(self, message, current, total):
+        pass
+
+    def clear(self):
+        pass
+        
+    def note(self, fmt_string, *args, **kwargs):
+        """See _BaseProgressBar.note()."""
+
+    def child_progress(self, **kwargs):
+        return DummyProgress(**kwargs)
+
 class DotsProgressBar(_BaseProgressBar):
+
     def __init__(self, **kwargs):
         _BaseProgressBar.__init__(self, **kwargs)
         self.last_msg = None
@@ -120,6 +219,8 @@
         if self.need_nl:
             self.to_file.write('\n')
         
+    def child_update(self, message, current, total):
+        self.tick()
     
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
@@ -142,55 +243,95 @@
     The output file should be in line-buffered or unbuffered mode.
     """
     SPIN_CHARS = r'/-\|'
-    MIN_PAUSE = 0.1 # seconds
 
 
     def __init__(self, **kwargs):
+        from bzrlib.osutils import terminal_width
         _BaseProgressBar.__init__(self, **kwargs)
         self.spin_pos = 0
-        self.width = _width()
+        self.width = terminal_width()
         self.start_time = None
-        self.last_update = None
+        self.last_updates = deque()
+        self.child_fraction = 0
     
 
     def throttle(self):
         """Return True if the bar was updated too recently"""
-        now = time.time()
-        if self.start_time is None:
-            self.start_time = self.last_update = now
-            return False
-        else:
-            interval = now - self.last_update
-            if interval > 0 and interval < self.MIN_PAUSE:
-                return True
+        # time.time consistently takes 40/4000 ms = 0.01 ms.
+        # but every single update to the pb invokes it.
+        # so we use time.clock which takes 20/4000 ms = 0.005ms
+        # on the downside, time.clock() appears to have approximately
+        # 10ms granularity, so we treat a zero-time change as 'throttled.'
+        
+        now = time.clock()
+        interval = now - self.last_update
+        # if interval > 0
+        if interval < self.MIN_PAUSE:
+            return True
 
+        self.last_updates.append(now - self.last_update)
         self.last_update = now
         return False
         
 
     def tick(self):
-        self.update(self.last_msg, self.last_cnt, self.last_total)
-                 
-
-
-    def update(self, msg, current_cnt=None, total_cnt=None):
+        self.update(self.last_msg, self.last_cnt, self.last_total, 
+                    self.child_fraction)
+
+    def child_update(self, message, current, total):
+        if current is not None and total != 0:
+            child_fraction = float(current) / total
+            if self.last_cnt is None:
+                pass
+            elif self.last_cnt + child_fraction <= self.last_total:
+                self.child_fraction = child_fraction
+            else:
+                mutter('not updating child fraction')
+        if self.last_msg is None:
+            self.last_msg = ''
+        self.tick()
+
+
+    def update(self, msg, current_cnt=None, total_cnt=None, 
+               child_fraction=0):
         """Update and redraw progress bar."""
 
+        if current_cnt < 0:
+            current_cnt = 0
+            
+        if current_cnt > total_cnt:
+            total_cnt = current_cnt
+        
+        ## # optional corner case optimisation 
+        ## # currently does not seem to fire so costs more than saved.
+        ## # trivial optimal case:
+        ## # NB if callers are doing a clear and restore with
+        ## # the saved values, this will prevent that:
+        ## # in that case add a restore method that calls
+        ## # _do_update or some such
+        ## if (self.last_msg == msg and
+        ##     self.last_cnt == current_cnt and
+        ##     self.last_total == total_cnt and
+        ##     self.child_fraction == child_fraction):
+        ##     return
+
+        old_msg = self.last_msg
         # save these for the tick() function
         self.last_msg = msg
         self.last_cnt = current_cnt
         self.last_total = total_cnt
-            
-        if self.throttle():
-            return 
-        
-        if total_cnt:
-            assert current_cnt <= total_cnt
-        if current_cnt:
-            assert current_cnt >= 0
-        
-        if self.show_eta and self.start_time and total_cnt:
-            eta = get_eta(self.start_time, current_cnt, total_cnt)
+        self.child_fraction = child_fraction
+
+        # each function call takes 20ms/4000 = 0.005 ms, 
+        # but multiple that by 4000 calls -> starts to cost.
+        # so anything to make this function call faster
+        # will improve base 'diff' time by up to 0.1 seconds.
+        if old_msg == self.last_msg and self.throttle():
+            return
+
+        if self.show_eta and self.start_time and self.last_total:
+            eta = get_eta(self.start_time, self.last_cnt + self.child_fraction, 
+                    self.last_total, last_updates = self.last_updates)
             eta_str = " " + str_tdelta(eta)
         else:
             eta_str = ""
@@ -203,32 +344,33 @@
         # always update this; it's also used for the bar
         self.spin_pos += 1
 
-        if self.show_pct and total_cnt and current_cnt:
-            pct = 100.0 * current_cnt / total_cnt
+        if self.show_pct and self.last_total and self.last_cnt:
+            pct = 100.0 * ((self.last_cnt + self.child_fraction) / self.last_total)
             pct_str = ' (%5.1f%%)' % pct
         else:
             pct_str = ''
 
         if not self.show_count:
             count_str = ''
-        elif current_cnt is None:
+        elif self.last_cnt is None:
             count_str = ''
-        elif total_cnt is None:
-            count_str = ' %i' % (current_cnt)
+        elif self.last_total is None:
+            count_str = ' %i' % (self.last_cnt)
         else:
             # make both fields the same size
-            t = '%i' % (total_cnt)
-            c = '%*i' % (len(t), current_cnt)
+            t = '%i' % (self.last_total)
+            c = '%*i' % (len(t), self.last_cnt)
             count_str = ' ' + c + '/' + t 
 
         if self.show_bar:
             # progress bar, if present, soaks up all remaining space
-            cols = self.width - 1 - len(msg) - len(spin_str) - len(pct_str) \
+            cols = self.width - 1 - len(self.last_msg) - len(spin_str) - len(pct_str) \
                    - len(eta_str) - len(count_str) - 3
 
-            if total_cnt:
+            if self.last_total:
                 # number of markers highlighted in bar
-                markers = int(round(float(cols) * current_cnt / total_cnt))
+                markers = int(round(float(cols) * 
+                              (self.last_cnt + self.child_fraction) / self.last_total))
                 bar_str = '[' + ('=' * markers).ljust(cols) + '] '
             elif False:
                 # don't know total, so can't show completion.
@@ -242,19 +384,59 @@
         else:
             bar_str = ''
 
-        m = spin_str + bar_str + msg + count_str + pct_str + eta_str
+        m = spin_str + bar_str + self.last_msg + count_str + pct_str + eta_str
 
         assert len(m) < self.width
         self.to_file.write('\r' + m.ljust(self.width - 1))
         #self.to_file.flush()
             
-
     def clear(self):        
         self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
         #self.to_file.flush()        
-    
-
-        
+
+
+class ChildProgress(_BaseProgressBar):
+    """A progress indicator that pushes its data to the parent"""
+    def __init__(self, _stack, **kwargs):
+        _BaseProgressBar.__init__(self, _stack=_stack, **kwargs)
+        self.parent = _stack.top()
+        self.current = None
+        self.total = None
+        self.child_fraction = 0
+        self.message = None
+
+    def update(self, msg, current_cnt=None, total_cnt=None):
+        self.current = current_cnt
+        self.total = total_cnt
+        self.message = msg
+        self.child_fraction = 0
+        self.tick()
+
+    def child_update(self, message, current, total):
+        if current is None or total == 0:
+            self.child_fraction = 0
+        else:
+            self.child_fraction = float(current) / total
+        self.tick()
+
+    def tick(self):
+        if self.current is None:
+            count = None
+        else:
+            count = self.current+self.child_fraction
+            if count > self.total:
+                if __debug__:
+                    mutter('clamping count of %d to %d' % (count, self.total))
+                count = self.total
+        self.parent.child_update(self.message, count, self.total)
+
+    def clear(self):
+        pass
+
+    def note(self, *args, **kwargs):
+        self.parent.note(*args, **kwargs)
+
+ 
 def str_tdelta(delt):
     if delt is None:
         return "-:--:--"
@@ -264,7 +446,7 @@
                              delt % 60)
 
 
-def get_eta(start_time, current, total, enough_samples=3):
+def get_eta(start_time, current, total, enough_samples=3, last_updates=None, n_recent=10):
     if start_time is None:
         return None
 
@@ -277,7 +459,7 @@
     if current > total:
         return None                     # wtf?
 
-    elapsed = time.time() - start_time
+    elapsed = time.clock() - start_time
 
     if elapsed < 2.0:                   # not enough time to estimate
         return None
@@ -286,9 +468,38 @@
 
     assert total_duration >= elapsed
 
+    if last_updates and len(last_updates) >= n_recent:
+        while len(last_updates) > n_recent:
+            last_updates.popleft()
+        avg = sum(last_updates) / float(len(last_updates))
+        time_left = avg * (total - current)
+
+        old_time_left = total_duration - elapsed
+
+        # We could return the average, or some other value here
+        return (time_left + old_time_left) / 2
+
     return total_duration - elapsed
 
 
+class ProgressPhase(object):
+    """Update progress object with the current phase"""
+    def __init__(self, message, total, pb):
+        object.__init__(self)
+        self.pb = pb
+        self.message = message
+        self.total = total
+        self.cur_phase = None
+
+    def next_phase(self):
+        if self.cur_phase is None:
+            self.cur_phase = 0
+        else:
+            self.cur_phase += 1
+        assert self.cur_phase < self.total 
+        self.pb.update(self.message, self.cur_phase, self.total)
+
+
 def run_tests():
     import doctest
     result = doctest.testmod()

=== added file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/reconcile.py	2006-03-22 13:21:14 +0000
@@ -0,0 +1,352 @@
+# (C) 2005, 2006 Canonical Limited.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Reconcilers are able to fix some potential data errors in a branch."""
+
+
+__all__ = ['reconcile', 'Reconciler', 'RepoReconciler']
+
+
+import bzrlib.branch
+import bzrlib.errors as errors
+import bzrlib.progress
+from bzrlib.trace import mutter
+from bzrlib.tsort import TopoSorter
+import bzrlib.ui as ui
+
+
+def reconcile(dir):
+    """Reconcile the data in dir.
+
+    Currently this is limited to a inventory 'reweave'.
+
+    This is a convenience method, for using a Reconciler object.
+
+    Directly using Reconciler is recommended for library users that
+    desire fine grained control or analysis of the found issues.
+    """
+    reconciler = Reconciler(dir)
+    reconciler.reconcile()
+
+
+class Reconciler(object):
+    """Reconcilers are used to reconcile existing data."""
+
+    def __init__(self, dir):
+        self.bzrdir = dir
+
+    def reconcile(self):
+        """Perform reconciliation.
+        
+        After reconciliation the following attributes document found issues:
+        inconsistent_parents: The number of revisions in the repository whose
+                              ancestry was being reported incorrectly.
+        garbage_inventories: The number of inventory objects without revisions
+                             that were garbage collected.
+        """
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            self._reconcile()
+        finally:
+            self.pb.finished()
+
+    def _reconcile(self):
+        """Helper function for performing reconciliation."""
+        self.repo = self.bzrdir.find_repository()
+        self.pb.note('Reconciling repository %s',
+                     self.repo.bzrdir.root_transport.base)
+        repo_reconciler = RepoReconciler(self.repo)
+        repo_reconciler.reconcile()
+        self.inconsistent_parents = repo_reconciler.inconsistent_parents
+        self.garbage_inventories = repo_reconciler.garbage_inventories
+        self.pb.note('Reconciliation complete.')
+
+
+class RepoReconciler(object):
+    """Reconciler that reconciles a repository.
+
+    Currently this consists of an inventory reweave with revision cross-checks.
+    """
+
+    def __init__(self, repo):
+        self.repo = repo
+
+    def reconcile(self):
+        """Perform reconciliation.
+        
+        After reconciliation the following attributes document found issues:
+        inconsistent_parents: The number of revisions in the repository whose
+                              ancestry was being reported incorrectly.
+        garbage_inventories: The number of inventory objects without revisions
+                             that were garbage collected.
+        """
+        self.repo.lock_write()
+        try:
+            self.pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self._reconcile_steps()
+            finally:
+                self.pb.finished()
+        finally:
+            self.repo.unlock()
+
+    def _reconcile_steps(self):
+        """Perform the steps to reconcile this repository."""
+        self._reweave_inventory()
+
+    def _reweave_inventory(self):
+        """Regenerate the inventory weave for the repository from scratch."""
+        # local because its really a wart we want to hide
+        from bzrlib.weave import WeaveFile, Weave
+        transaction = self.repo.get_transaction()
+        self.pb.update('Reading inventory data.')
+        self.inventory = self.repo.get_inventory_weave()
+        # the total set of revisions to process
+        self.pending = set([rev_id for rev_id in self.repo._revision_store.all_revision_ids(transaction)])
+
+        # mapping from revision_id to parents
+        self._rev_graph = {}
+        # errors that we detect
+        self.inconsistent_parents = 0
+        # we need the revision id of each revision and its available parents list
+        self._setup_steps(len(self.pending))
+        for rev_id in self.pending:
+            # put a revision into the graph.
+            self._graph_revision(rev_id)
+        self._check_garbage_inventories()
+        if not self.inconsistent_parents and not self.garbage_inventories:
+            self.pb.note('Inventory ok.')
+            return
+        self.pb.update('Backing up inventory...', 0, 0)
+        self.repo.control_weaves.copy(self.inventory, 'inventory.backup', self.repo.get_transaction())
+        self.pb.note('Backup Inventory created.')
+        # asking for '' should never return a non-empty weave
+        new_inventory_vf = self.repo.control_weaves.get_empty('inventory.new',
+            self.repo.get_transaction())
+
+        # we have topological order of revisions and non ghost parents ready.
+        self._setup_steps(len(self._rev_graph))
+        for rev_id in TopoSorter(self._rev_graph.items()).iter_topo_order():
+            parents = self._rev_graph[rev_id]
+            # double check this really is in topological order.
+            unavailable = [p for p in parents if p not in new_inventory_vf]
+            assert len(unavailable) == 0
+            # this entry has all the non ghost parents in the inventory
+            # file already.
+            self._reweave_step('adding inventories')
+            if isinstance(new_inventory_vf, WeaveFile):
+                # It's really a WeaveFile, but we call straight into the
+                # Weave's add method to disable the auto-write-out behaviour.
+                # This is done to avoid a revision_count * time-to-write additional overhead on 
+                # reconcile.
+                new_inventory_vf._check_write_ok()
+                Weave._add_lines(new_inventory_vf, rev_id, parents, self.inventory.get_lines(rev_id),
+                                 None)
+            else:
+                new_inventory_vf.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
+
+        if isinstance(new_inventory_vf, WeaveFile):
+            new_inventory_vf._save()
+        # if this worked, the set of new_inventory_vf.names should equal
+        # self.pending
+        assert set(new_inventory_vf.versions()) == self.pending
+        self.pb.update('Writing weave')
+        self.repo.control_weaves.copy(new_inventory_vf, 'inventory', self.repo.get_transaction())
+        self.repo.control_weaves.delete('inventory.new', self.repo.get_transaction())
+        self.inventory = None
+        self.pb.note('Inventory regenerated.')
+
+    def _setup_steps(self, new_total):
+        """Setup the markers we need to control the progress bar."""
+        self.total = new_total
+        self.count = 0
+
+    def _graph_revision(self, rev_id):
+        """Load a revision into the revision graph."""
+        # pick a random revision
+        # analyse revision id rev_id and put it in the stack.
+        self._reweave_step('loading revisions')
+        rev = self.repo.get_revision_reconcile(rev_id)
+        assert rev.revision_id == rev_id
+        parents = []
+        for parent in rev.parent_ids:
+            if self._parent_is_available(parent):
+                parents.append(parent)
+            else:
+                mutter('found ghost %s', parent)
+        self._rev_graph[rev_id] = parents   
+        if set(self.inventory.get_parents(rev_id)) != set(parents):
+            self.inconsistent_parents += 1
+            mutter('Inconsistent inventory parents: id {%s} '
+                   'inventory claims %r, '
+                   'available parents are %r, '
+                   'unavailable parents are %r',
+                   rev_id, 
+                   set(self.inventory.get_parents(rev_id)),
+                   set(parents),
+                   set(rev.parent_ids).difference(set(parents)))
+
+    def _check_garbage_inventories(self):
+        """Check for garbage inventories which we cannot trust
+
+        We cant trust them because their pre-requisite file data may not
+        be present - all we know is that their revision was not installed.
+        """
+        inventories = set(self.inventory.versions())
+        revisions = set(self._rev_graph.keys())
+        garbage = inventories.difference(revisions)
+        self.garbage_inventories = len(garbage)
+        for revision_id in garbage:
+            mutter('Garbage inventory {%s} found.', revision_id)
+
+    def _parent_is_available(self, parent):
+        """True if parent is a fully available revision
+
+        A fully available revision has a inventory and a revision object in the
+        repository.
+        """
+        return (parent in self._rev_graph or 
+                (parent in self.inventory and self.repo.has_revision(parent)))
+
+    def _reweave_step(self, message):
+        """Mark a single step of regeneration complete."""
+        self.pb.update(message, self.count, self.total)
+        self.count += 1
+
+
+class KnitReconciler(RepoReconciler):
+    """Reconciler that reconciles a knit format repository.
+
+    This will detect garbage inventories and remove them.
+
+    Inconsistent parentage is checked for in the revision weave.
+    """
+
+    def _reconcile_steps(self):
+        """Perform the steps to reconcile this repository."""
+        self._load_indexes()
+        # knits never suffer this
+        self.inconsistent_parents = 0
+        self._gc_inventory()
+
+    def _load_indexes(self):
+        """Load indexes for the reconciliation."""
+        self.transaction = self.repo.get_transaction()
+        self.pb.update('Reading indexes.', 0, 2)
+        self.inventory = self.repo.get_inventory_weave()
+        self.pb.update('Reading indexes.', 1, 2)
+        self.revisions = self.repo._revision_store.get_revision_file(self.transaction)
+        self.pb.update('Reading indexes.', 2, 2)
+
+    def _gc_inventory(self):
+        """Remove inventories that are not referenced from the revision store."""
+        self.pb.update('Checking unused inventories.', 0, 1)
+        self._check_garbage_inventories()
+        self.pb.update('Checking unused inventories.', 1, 3)
+        if not self.garbage_inventories:
+            self.pb.note('Inventory ok.')
+            return
+        self.pb.update('Backing up inventory...', 0, 0)
+        self.repo.control_weaves.copy(self.inventory, 'inventory.backup', self.transaction)
+        self.pb.note('Backup Inventory created.')
+        # asking for '' should never return a non-empty weave
+        new_inventory_vf = self.repo.control_weaves.get_empty('inventory.new',
+            self.transaction)
+
+        # we have topological order of revisions and non ghost parents ready.
+        self._setup_steps(len(self.revisions))
+        for rev_id in TopoSorter(self.revisions.get_graph().items()).iter_topo_order():
+            parents = self.revisions.get_parents(rev_id)
+            # double check this really is in topological order.
+            unavailable = [p for p in parents if p not in new_inventory_vf]
+            assert len(unavailable) == 0
+            # this entry has all the non ghost parents in the inventory
+            # file already.
+            self._reweave_step('adding inventories')
+            # ugly but needed, weaves are just way tooooo slow else.
+            new_inventory_vf.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
+
+        # if this worked, the set of new_inventory_vf.names should equal
+        # self.pending
+        assert set(new_inventory_vf.versions()) == set(self.revisions.versions())
+        self.pb.update('Writing weave')
+        self.repo.control_weaves.copy(new_inventory_vf, 'inventory', self.transaction)
+        self.repo.control_weaves.delete('inventory.new', self.transaction)
+        self.inventory = None
+        self.pb.note('Inventory regenerated.')
+
+    def _reinsert_revisions(self):
+        """Correct the revision history for revisions in the revision knit."""
+        # the total set of revisions to process
+        self.pending = set(self.revisions.versions())
+
+        # mapping from revision_id to parents
+        self._rev_graph = {}
+        # errors that we detect
+        self.inconsistent_parents = 0
+        # we need the revision id of each revision and its available parents list
+        self._setup_steps(len(self.pending))
+        for rev_id in self.pending:
+            # put a revision into the graph.
+            self._graph_revision(rev_id)
+
+        if not self.inconsistent_parents:
+            self.pb.note('Revision history accurate.')
+            return
+        self._setup_steps(len(self._rev_graph))
+        for rev_id, parents in self._rev_graph.items():
+            if parents != self.revisions.get_parents(rev_id):
+                self.revisions.fix_parents(rev_id, parents)
+            self._reweave_step('Fixing parents')
+        self.pb.note('Ancestry corrected.')
+
+    def _graph_revision(self, rev_id):
+        """Load a revision into the revision graph."""
+        # pick a random revision
+        # analyse revision id rev_id and put it in the stack.
+        self._reweave_step('loading revisions')
+        rev = self.repo._revision_store.get_revision(rev_id, self.transaction)
+        assert rev.revision_id == rev_id
+        parents = []
+        for parent in rev.parent_ids:
+            if self.revisions.has_version(parent):
+                parents.append(parent)
+            else:
+                mutter('found ghost %s', parent)
+        self._rev_graph[rev_id] = parents   
+        if set(self.inventory.get_parents(rev_id)) != set(parents):
+            self.inconsistent_parents += 1
+            mutter('Inconsistent inventory parents: id {%s} '
+                   'inventory claims %r, '
+                   'available parents are %r, '
+                   'unavailable parents are %r',
+                   rev_id, 
+                   set(self.inventory.get_parents(rev_id)),
+                   set(parents),
+                   set(rev.parent_ids).difference(set(parents)))
+
+    def _check_garbage_inventories(self):
+        """Check for garbage inventories which we cannot trust
+
+        We cant trust them because their pre-requisite file data may not
+        be present - all we know is that their revision was not installed.
+        """
+        inventories = set(self.inventory.versions())
+        revisions = set(self.revisions.versions())
+        garbage = inventories.difference(revisions)
+        self.garbage_inventories = len(garbage)
+        for revision_id in garbage:
+            mutter('Garbage inventory {%s} found.', revision_id)

=== removed file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-08-17 02:06:19 +0000
+++ bzrlib/remotebranch.py	1970-01-01 00:00:00 +0000
@@ -1,185 +0,0 @@
-#! /usr/bin/env python
-
-# Copyright (C) 2005 Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Proxy object for access to remote branches.
-
-At the moment remote branches are only for HTTP and only for read
-access.
-"""
-
-
-import gzip
-from cStringIO import StringIO
-import urllib2
-
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT
-from trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
-ENABLE_URLGRABBER = True
-
-from bzrlib.errors import BzrError
-
-class GetFailed(BzrError):
-    def __init__(self, url, status):
-        BzrError.__init__(self, "Get %s failed with status %s" % (url, status))
-        self.url = url
-        self.status = status
-
-if ENABLE_URLGRABBER:
-    import util.urlgrabber
-    import util.urlgrabber.keepalive
-    util.urlgrabber.keepalive.DEBUG = 0
-    def get_url(path, compressed=False):
-        try:
-            url = path
-            if compressed:
-                url += '.gz'
-            mutter("grab url %s" % url)
-            url_f = util.urlgrabber.urlopen(url, keepalive=1, close_connection=0)
-            if url_f.status != 200:
-                raise GetFailed(url, url_f.status)
-            if not compressed:
-                return url_f
-            else:
-                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        except urllib2.URLError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-else:
-    def get_url(url, compressed=False):
-        import urllib2
-        if compressed:
-            url += '.gz'
-        mutter("get_url %s" % url)
-        url_f = urllib2.urlopen(url)
-        if compressed:
-            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        else:
-            return url_f
-
-
-
-def _find_remote_root(url):
-    """Return the prefix URL that corresponds to the branch root."""
-    orig_url = url
-    while True:
-        try:
-            ff = get_url(url + '/.bzr/branch-format')
-
-            fmt = ff.read()
-            ff.close()
-
-            fmt = fmt.rstrip('\r\n')
-            if fmt != BZR_BRANCH_FORMAT.rstrip('\r\n'):
-                raise BzrError("sorry, branch format %r not supported at url %s"
-                               % (fmt, url))
-            
-            return url
-        except urllib2.URLError:
-            pass
-
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
-        
-
-
-class RemoteBranch(Branch):
-    def __init__(self, baseurl, find_root=True):
-        """Create new proxy for a remote branch."""
-        if find_root:
-            self.baseurl = _find_remote_root(baseurl)
-        else:
-            self.baseurl = baseurl
-            self._check_format()
-
-        self.inventory_store = RemoteStore(baseurl + '/.bzr/inventory-store/')
-        self.text_store = RemoteStore(baseurl + '/.bzr/text-store/')
-        self.revision_store = RemoteStore(baseurl + '/.bzr/revision-store/')
-
-    def __str__(self):
-        b = getattr(self, 'baseurl', 'undefined')
-        return '%s(%r)' % (self.__class__.__name__, b)
-
-    __repr__ = __str__
-
-    def controlfile(self, filename, mode):
-        if mode not in ('rb', 'rt', 'r'):
-            raise BzrError("file mode %r not supported for remote branches" % mode)
-        return get_url(self.baseurl + '/.bzr/' + filename, False)
-
-
-    def lock_read(self):
-        # no locking for remote branches yet
-        pass
-
-    def lock_write(self):
-        from errors import LockError
-        raise LockError("write lock not supported for remote branch %s"
-                        % self.baseurl)
-
-    def unlock(self):
-        pass
-    
-
-    def relpath(self, path):
-        if not path.startswith(self.baseurl):
-            raise BzrError('path %r is not under base URL %r'
-                           % (path, self.baseurl))
-        pl = len(self.baseurl)
-        return path[pl:].lstrip('/')
-
-
-    def get_revision(self, revision_id):
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
-        revf = self.revision_store[revision_id]
-        r = unpack_xml(Revision, revf)
-        if r.revision_id != revision_id:
-            raise BzrCheckError('revision stored as {%s} actually contains {%s}'
-                                % (revision_id, r.revision_id))
-        return r
-
-
-class RemoteStore(object):
-    def __init__(self, baseurl):
-        self._baseurl = baseurl
-        
-
-    def _path(self, name):
-        if '/' in name:
-            raise ValueError('invalid store id', name)
-        return self._baseurl + '/' + name
-        
-    def __getitem__(self, fileid):
-        p = self._path(fileid)
-        try:
-            return get_url(p, compressed=True)
-        except:
-            raise KeyError(fileid)
-    
-
-    

=== added file 'bzrlib/repository.py'
--- bzrlib/repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/repository.py	2006-04-13 04:25:59 +0000
@@ -0,0 +1,1832 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from copy import deepcopy
+from cStringIO import StringIO
+from unittest import TestSuite
+
+import bzrlib.bzrdir as bzrdir
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+import bzrlib.errors as errors
+from bzrlib.errors import InvalidRevisionId
+import bzrlib.gpg as gpg
+from bzrlib.graph import Graph
+from bzrlib.inter import InterObject
+from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
+from bzrlib.osutils import safe_unicode
+from bzrlib.revision import NULL_REVISION
+from bzrlib.store.versioned import VersionedFileStore, WeaveStore
+from bzrlib.store.text import TextStore
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter, note
+from bzrlib.tree import RevisionTree
+from bzrlib.tsort import topo_sort
+from bzrlib.testament import Testament
+from bzrlib.tree import EmptyTree
+import bzrlib.ui
+from bzrlib.weave import WeaveFile
+import bzrlib.xml5
+
+
+class Repository(object):
+    """Repository holding history for one or more branches.
+
+    The repository holds and retrieves historical information including
+    revisions and file history.  It's normally accessed only by the Branch,
+    which views a particular line of development through that history.
+
+    The Repository builds on top of Stores and a Transport, which respectively 
+    describe the disk data format and the way of accessing the (possibly 
+    remote) disk.
+    """
+
+    @needs_write_lock
+    def add_inventory(self, revid, inv, parents):
+        """Add the inventory inv to the repository as revid.
+        
+        :param parents: The revision ids of the parents that revid
+                        is known to have and are in the repository already.
+
+        returns the sha1 of the serialized inventory.
+        """
+        inv_text = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
+        inv_sha1 = bzrlib.osutils.sha_string(inv_text)
+        inv_vf = self.control_weaves.get_weave('inventory',
+                                               self.get_transaction())
+        inv_vf.add_lines(revid, parents, bzrlib.osutils.split_lines(inv_text))
+        return inv_sha1
+
+    @needs_write_lock
+    def add_revision(self, rev_id, rev, inv=None, config=None):
+        """Add rev to the revision store as rev_id.
+
+        :param rev_id: the revision id to use.
+        :param rev: The revision object.
+        :param inv: The inventory for the revision. if None, it will be looked
+                    up in the inventory storer
+        :param config: If None no digital signature will be created.
+                       If supplied its signature_needed method will be used
+                       to determine if a signature should be made.
+        """
+        if config is not None and config.signature_needed():
+            if inv is None:
+                inv = self.get_inventory(rev_id)
+            plaintext = Testament(rev, inv).as_short_text()
+            self.store_revision_signature(
+                gpg.GPGStrategy(config), plaintext, rev_id)
+        if not rev_id in self.get_inventory_weave():
+            if inv is None:
+                raise errors.WeaveRevisionNotPresent(rev_id,
+                                                     self.get_inventory_weave())
+            else:
+                # yes, this is not suitable for adding with ghosts.
+                self.add_inventory(rev_id, inv, rev.parent_ids)
+        self._revision_store.add_revision(rev, self.get_transaction())
+
+    @needs_read_lock
+    def _all_possible_ids(self):
+        """Return all the possible revisions that we could find."""
+        return self.get_inventory_weave().versions()
+
+    @needs_read_lock
+    def all_revision_ids(self):
+        """Returns a list of all the revision ids in the repository. 
+
+        These are in as much topological order as the underlying store can 
+        present: for weaves ghosts may lead to a lack of correctness until
+        the reweave updates the parents list.
+        """
+        if self._revision_store.text_store.listable():
+            return self._revision_store.all_revision_ids(self.get_transaction())
+        result = self._all_possible_ids()
+        return self._eliminate_revisions_not_present(result)
+
+    @needs_read_lock
+    def _eliminate_revisions_not_present(self, revision_ids):
+        """Check every revision id in revision_ids to see if we have it.
+
+        Returns a set of the present revisions.
+        """
+        result = []
+        for id in revision_ids:
+            if self.has_revision(id):
+               result.append(id)
+        return result
+
+    @staticmethod
+    def create(a_bzrdir):
+        """Construct the current default format repository in a_bzrdir."""
+        return RepositoryFormat.get_default_format().initialize(a_bzrdir)
+
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store, control_store, text_store):
+        """instantiate a Repository.
+
+        :param _format: The format of the repository on disk.
+        :param a_bzrdir: The BzrDir of the repository.
+
+        In the future we will have a single api for all stores for
+        getting file texts, inventories and revisions, then
+        this construct will accept instances of those things.
+        """
+        super(Repository, self).__init__()
+        self._format = _format
+        # the following are part of the public API for Repository:
+        self.bzrdir = a_bzrdir
+        self.control_files = control_files
+        self._revision_store = _revision_store
+        self.text_store = text_store
+        # backwards compatability
+        self.weave_store = text_store
+        # not right yet - should be more semantically clear ? 
+        # 
+        self.control_store = control_store
+        self.control_weaves = control_store
+        # TODO: make sure to construct the right store classes, etc, depending
+        # on whether escaping is required.
+
+    def lock_write(self):
+        self.control_files.lock_write()
+
+    def lock_read(self):
+        self.control_files.lock_read()
+
+    def is_locked(self):
+        return self.control_files.is_locked()
+
+    @needs_read_lock
+    def missing_revision_ids(self, other, revision_id=None):
+        """Return the revision ids that other has that this does not.
+        
+        These are returned in topological order.
+
+        revision_id: only return revision ids included by revision_id.
+        """
+        return InterRepository.get(other, self).missing_revision_ids(revision_id)
+
+    @staticmethod
+    def open(base):
+        """Open the repository rooted at base.
+
+        For instance, if the repository is at URL/.bzr/repository,
+        Repository.open(URL) -> a Repository instance.
+        """
+        control = bzrlib.bzrdir.BzrDir.open(base)
+        return control.open_repository()
+
+    def copy_content_into(self, destination, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+        """
+        return InterRepository.get(self, destination).copy_content(revision_id, basis)
+
+    def fetch(self, source, revision_id=None, pb=None):
+        """Fetch the content required to construct revision_id from source.
+
+        If revision_id is None all content is copied.
+        """
+        return InterRepository.get(source, self).fetch(revision_id=revision_id,
+                                                       pb=pb)
+
+    def unlock(self):
+        self.control_files.unlock()
+
+    @needs_read_lock
+    def clone(self, a_bzrdir, revision_id=None, basis=None):
+        """Clone this repository into a_bzrdir using the current format.
+
+        Currently no check is made that the format of this repository and
+        the bzrdir format are compatible. FIXME RBC 20060201.
+        """
+        if not isinstance(a_bzrdir._format, self.bzrdir._format.__class__):
+            # use target default format.
+            result = a_bzrdir.create_repository()
+        # FIXME RBC 20060209 split out the repository type to avoid this check ?
+        elif isinstance(a_bzrdir._format,
+                      (bzrlib.bzrdir.BzrDirFormat4,
+                       bzrlib.bzrdir.BzrDirFormat5,
+                       bzrlib.bzrdir.BzrDirFormat6)):
+            result = a_bzrdir.open_repository()
+        else:
+            result = self._format.initialize(a_bzrdir, shared=self.is_shared())
+        self.copy_content_into(result, revision_id, basis)
+        return result
+
+    @needs_read_lock
+    def has_revision(self, revision_id):
+        """True if this repository has a copy of the revision."""
+        return self._revision_store.has_revision_id(revision_id,
+                                                    self.get_transaction())
+
+    @needs_read_lock
+    def get_revision_reconcile(self, revision_id):
+        """'reconcile' helper routine that allows access to a revision always.
+        
+        This variant of get_revision does not cross check the weave graph
+        against the revision one as get_revision does: but it should only
+        be used by reconcile, or reconcile-alike commands that are correcting
+        or testing the revision graph.
+        """
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise InvalidRevisionId(revision_id=revision_id, branch=self)
+        return self._revision_store.get_revision(revision_id,
+                                                 self.get_transaction())
+
+    @needs_read_lock
+    def get_revision_xml(self, revision_id):
+        rev = self.get_revision(revision_id) 
+        rev_tmp = StringIO()
+        # the current serializer..
+        self._revision_store._serializer.write_revision(rev, rev_tmp)
+        rev_tmp.seek(0)
+        return rev_tmp.getvalue()
+
+    @needs_read_lock
+    def get_revision(self, revision_id):
+        """Return the Revision object for a named revision"""
+        r = self.get_revision_reconcile(revision_id)
+        # weave corruption can lead to absent revision markers that should be
+        # present.
+        # the following test is reasonably cheap (it needs a single weave read)
+        # and the weave is cached in read transactions. In write transactions
+        # it is not cached but typically we only read a small number of
+        # revisions. For knits when they are introduced we will probably want
+        # to ensure that caching write transactions are in use.
+        inv = self.get_inventory_weave()
+        self._check_revision_parents(r, inv)
+        return r
+
+    def _check_revision_parents(self, revision, inventory):
+        """Private to Repository and Fetch.
+        
+        This checks the parentage of revision in an inventory weave for 
+        consistency and is only applicable to inventory-weave-for-ancestry
+        using repository formats & fetchers.
+        """
+        weave_parents = inventory.get_parents(revision.revision_id)
+        weave_names = inventory.versions()
+        for parent_id in revision.parent_ids:
+            if parent_id in weave_names:
+                # this parent must not be a ghost.
+                if not parent_id in weave_parents:
+                    # but it is a ghost
+                    raise errors.CorruptRepository(self)
+
+    @needs_write_lock
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        signature = gpg_strategy.sign(plaintext)
+        self._revision_store.add_revision_signature_text(revision_id,
+                                                         signature,
+                                                         self.get_transaction())
+
+    def fileid_involved_between_revs(self, from_revid, to_revid):
+        """Find file_id(s) which are involved in the changes between revisions.
+
+        This determines the set of revisions which are involved, and then
+        finds all file ids affected by those revisions.
+        """
+        w = self.get_inventory_weave()
+        from_set = set(w.get_ancestry(from_revid))
+        to_set = set(w.get_ancestry(to_revid))
+        changed = to_set.difference(from_set)
+        return self._fileid_involved_by_set(changed)
+
+    def fileid_involved(self, last_revid=None):
+        """Find all file_ids modified in the ancestry of last_revid.
+
+        :param last_revid: If None, last_revision() will be used.
+        """
+        w = self.get_inventory_weave()
+        if not last_revid:
+            changed = set(w.versions())
+        else:
+            changed = set(w.get_ancestry(last_revid))
+        return self._fileid_involved_by_set(changed)
+
+    def fileid_involved_by_set(self, changes):
+        """Find all file_ids modified by the set of revisions passed in.
+
+        :param changes: A set() of revision ids
+        """
+        # TODO: jam 20060119 This line does *nothing*, remove it.
+        #       or better yet, change _fileid_involved_by_set so
+        #       that it takes the inventory weave, rather than
+        #       pulling it out by itself.
+        return self._fileid_involved_by_set(changes)
+
+    def _fileid_involved_by_set(self, changes):
+        """Find the set of file-ids affected by the set of revisions.
+
+        :param changes: A set() of revision ids.
+        :return: A set() of file ids.
+        
+        This peaks at the Weave, interpreting each line, looking to
+        see if it mentions one of the revisions. And if so, includes
+        the file id mentioned.
+        This expects both the Weave format, and the serialization
+        to have a single line per file/directory, and to have
+        fileid="" and revision="" on that line.
+        """
+        assert isinstance(self._format, (RepositoryFormat5,
+                                         RepositoryFormat6,
+                                         RepositoryFormat7,
+                                         RepositoryFormatKnit1)), \
+            "fileid_involved only supported for branches which store inventory as unnested xml"
+
+        w = self.get_inventory_weave()
+        file_ids = set()
+
+        # this code needs to read every line in every inventory for the
+        # inventories [changes]. Seeing a line twice is ok. Seeing a line
+        # not pesent in one of those inventories is unnecessary and not 
+        # harmful because we are filtering by the revision id marker in the
+        # inventory lines to only select file ids altered in one of those  
+        # revisions. We dont need to see all lines in the inventory because
+        # only those added in an inventory in rev X can contain a revision=X
+        # line.
+        for line in w.iter_lines_added_or_present_in_versions(changes):
+            start = line.find('file_id="')+9
+            if start < 9: continue
+            end = line.find('"', start)
+            assert end>= 0
+            file_id = _unescape_xml(line[start:end])
+
+            # check if file_id is already present
+            if file_id in file_ids: continue
+
+            start = line.find('revision="')+10
+            if start < 10: continue
+            end = line.find('"', start)
+            assert end>= 0
+            revision_id = _unescape_xml(line[start:end])
+            if revision_id in changes:
+                file_ids.add(file_id)
+        return file_ids
+
+    @needs_read_lock
+    def get_inventory_weave(self):
+        return self.control_weaves.get_weave('inventory',
+            self.get_transaction())
+
+    @needs_read_lock
+    def get_inventory(self, revision_id):
+        """Get Inventory object by hash."""
+        xml = self.get_inventory_xml(revision_id)
+        return bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+
+    @needs_read_lock
+    def get_inventory_xml(self, revision_id):
+        """Get inventory XML as a file object."""
+        try:
+            assert isinstance(revision_id, basestring), type(revision_id)
+            iw = self.get_inventory_weave()
+            return iw.get_text(revision_id)
+        except IndexError:
+            raise bzrlib.errors.HistoryMissing(self, 'inventory', revision_id)
+
+    @needs_read_lock
+    def get_inventory_sha1(self, revision_id):
+        """Return the sha1 hash of the inventory entry
+        """
+        return self.get_revision(revision_id).inventory_sha1
+
+    @needs_read_lock
+    def get_revision_graph(self, revision_id=None):
+        """Return a dictionary containing the revision graph.
+        
+        :return: a dictionary of revision_id->revision_parents_list.
+        """
+        weave = self.get_inventory_weave()
+        all_revisions = self._eliminate_revisions_not_present(weave.versions())
+        entire_graph = dict([(node, weave.get_parents(node)) for 
+                             node in all_revisions])
+        if revision_id is None:
+            return entire_graph
+        elif revision_id not in entire_graph:
+            raise errors.NoSuchRevision(self, revision_id)
+        else:
+            # add what can be reached from revision_id
+            result = {}
+            pending = set([revision_id])
+            while len(pending) > 0:
+                node = pending.pop()
+                result[node] = entire_graph[node]
+                for revision_id in result[node]:
+                    if revision_id not in result:
+                        pending.add(revision_id)
+            return result
+
+    @needs_read_lock
+    def get_revision_graph_with_ghosts(self, revision_ids=None):
+        """Return a graph of the revisions with ghosts marked as applicable.
+
+        :param revision_ids: an iterable of revisions to graph or None for all.
+        :return: a Graph object with the graph reachable from revision_ids.
+        """
+        result = Graph()
+        if not revision_ids:
+            pending = set(self.all_revision_ids())
+            required = set([])
+        else:
+            pending = set(revision_ids)
+            required = set(revision_ids)
+        done = set([])
+        while len(pending):
+            revision_id = pending.pop()
+            try:
+                rev = self.get_revision(revision_id)
+            except errors.NoSuchRevision:
+                if revision_id in required:
+                    raise
+                # a ghost
+                result.add_ghost(revision_id)
+                continue
+            for parent_id in rev.parent_ids:
+                # is this queued or done ?
+                if (parent_id not in pending and
+                    parent_id not in done):
+                    # no, queue it.
+                    pending.add(parent_id)
+            result.add_node(revision_id, rev.parent_ids)
+            done.add(revision_id)
+        return result
+
+    @needs_read_lock
+    def get_revision_inventory(self, revision_id):
+        """Return inventory of a past revision."""
+        # TODO: Unify this with get_inventory()
+        # bzr 0.0.6 and later imposes the constraint that the inventory_id
+        # must be the same as its revision, so this is trivial.
+        if revision_id is None:
+            # This does not make sense: if there is no revision,
+            # then it is the current tree inventory surely ?!
+            # and thus get_root_id() is something that looks at the last
+            # commit on the branch, and the get_root_id is an inventory check.
+            raise NotImplementedError
+            # return Inventory(self.get_root_id())
+        else:
+            return self.get_inventory(revision_id)
+
+    @needs_read_lock
+    def is_shared(self):
+        """Return True if this repository is flagged as a shared repository."""
+        raise NotImplementedError(self.is_shared)
+
+    @needs_write_lock
+    def reconcile(self):
+        """Reconcile this repository."""
+        from bzrlib.reconcile import RepoReconciler
+        reconciler = RepoReconciler(self)
+        reconciler.reconcile()
+        return reconciler
+    
+    @needs_read_lock
+    def revision_tree(self, revision_id):
+        """Return Tree for a revision on this branch.
+
+        `revision_id` may be None for the null revision, in which case
+        an `EmptyTree` is returned."""
+        # TODO: refactor this to use an existing revision object
+        # so we don't need to read it in twice.
+        if revision_id is None or revision_id == NULL_REVISION:
+            return EmptyTree()
+        else:
+            inv = self.get_revision_inventory(revision_id)
+            return RevisionTree(self, inv, revision_id)
+
+    @needs_read_lock
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This is topologically sorted.
+        """
+        if revision_id is None:
+            return [None]
+        if not self.has_revision(revision_id):
+            raise errors.NoSuchRevision(self, revision_id)
+        w = self.get_inventory_weave()
+        candidates = w.get_ancestry(revision_id)
+        return [None] + candidates # self._eliminate_revisions_not_present(candidates)
+
+    @needs_read_lock
+    def print_file(self, file, revision_id):
+        """Print `file` to stdout.
+        
+        FIXME RBC 20060125 as John Meinel points out this is a bad api
+        - it writes to stdout, it assumes that that is valid etc. Fix
+        by creating a new more flexible convenience function.
+        """
+        tree = self.revision_tree(revision_id)
+        # use inventory as it was in that revision
+        file_id = tree.inventory.path2id(file)
+        if not file_id:
+            raise BzrError("%r is not present in revision %s" % (file, revno))
+            try:
+                revno = self.revision_id_to_revno(revision_id)
+            except errors.NoSuchRevision:
+                # TODO: This should not be BzrError,
+                # but NoSuchFile doesn't fit either
+                raise BzrError('%r is not present in revision %s' 
+                                % (file, revision_id))
+            else:
+                raise BzrError('%r is not present in revision %s'
+                                % (file, revno))
+        tree.print_file(file_id)
+
+    def get_transaction(self):
+        return self.control_files.get_transaction()
+
+    def revision_parents(self, revid):
+        return self.get_inventory_weave().parent_names(revid)
+
+    @needs_write_lock
+    def set_make_working_trees(self, new_value):
+        """Set the policy flag for making working trees when creating branches.
+
+        This only applies to branches that use this repository.
+
+        The default is 'True'.
+        :param new_value: True to restore the default, False to disable making
+                          working trees.
+        """
+        raise NotImplementedError(self.set_make_working_trees)
+    
+    def make_working_trees(self):
+        """Returns the policy for making working trees on new branches."""
+        raise NotImplementedError(self.make_working_trees)
+
+    @needs_write_lock
+    def sign_revision(self, revision_id, gpg_strategy):
+        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+    @needs_read_lock
+    def has_signature_for_revision_id(self, revision_id):
+        """Query for a revision signature for revision_id in the repository."""
+        return self._revision_store.has_signature(revision_id,
+                                                  self.get_transaction())
+
+    @needs_read_lock
+    def get_signature_text(self, revision_id):
+        """Return the text for a signature."""
+        return self._revision_store.get_signature_text(revision_id,
+                                                       self.get_transaction())
+
+
+class AllInOneRepository(Repository):
+    """Legacy support - the repository behaviour for all-in-one branches."""
+
+    def __init__(self, _format, a_bzrdir, _revision_store, control_store, text_store):
+        # we reuse one control files instance.
+        dir_mode = a_bzrdir._control_files._dir_mode
+        file_mode = a_bzrdir._control_files._file_mode
+
+        def get_store(name, compressed=True, prefixed=False):
+            # FIXME: This approach of assuming stores are all entirely compressed
+            # or entirely uncompressed is tidy, but breaks upgrade from 
+            # some existing branches where there's a mixture; we probably 
+            # still want the option to look for both.
+            relpath = a_bzrdir._control_files._escape(name)
+            store = TextStore(a_bzrdir._control_files._transport.clone(relpath),
+                              prefixed=prefixed, compressed=compressed,
+                              dir_mode=dir_mode,
+                              file_mode=file_mode)
+            #if self._transport.should_cache():
+            #    cache_path = os.path.join(self.cache_root, name)
+            #    os.mkdir(cache_path)
+            #    store = bzrlib.store.CachedStore(store, cache_path)
+            return store
+
+        # not broken out yet because the controlweaves|inventory_store
+        # and text_store | weave_store bits are still different.
+        if isinstance(_format, RepositoryFormat4):
+            # cannot remove these - there is still no consistent api 
+            # which allows access to this old info.
+            self.inventory_store = get_store('inventory-store')
+            text_store = get_store('text-store')
+        super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, _revision_store, control_store, text_store)
+
+    @needs_read_lock
+    def is_shared(self):
+        """AllInOne repositories cannot be shared."""
+        return False
+
+    @needs_write_lock
+    def set_make_working_trees(self, new_value):
+        """Set the policy flag for making working trees when creating branches.
+
+        This only applies to branches that use this repository.
+
+        The default is 'True'.
+        :param new_value: True to restore the default, False to disable making
+                          working trees.
+        """
+        raise NotImplementedError(self.set_make_working_trees)
+    
+    def make_working_trees(self):
+        """Returns the policy for making working trees on new branches."""
+        return True
+
+
+class MetaDirRepository(Repository):
+    """Repositories in the new meta-dir layout."""
+
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store, control_store, text_store):
+        super(MetaDirRepository, self).__init__(_format,
+                                                a_bzrdir,
+                                                control_files,
+                                                _revision_store,
+                                                control_store,
+                                                text_store)
+
+        dir_mode = self.control_files._dir_mode
+        file_mode = self.control_files._file_mode
+
+    @needs_read_lock
+    def is_shared(self):
+        """Return True if this repository is flagged as a shared repository."""
+        return self.control_files._transport.has('shared-storage')
+
+    @needs_write_lock
+    def set_make_working_trees(self, new_value):
+        """Set the policy flag for making working trees when creating branches.
+
+        This only applies to branches that use this repository.
+
+        The default is 'True'.
+        :param new_value: True to restore the default, False to disable making
+                          working trees.
+        """
+        if new_value:
+            try:
+                self.control_files._transport.delete('no-working-trees')
+            except errors.NoSuchFile:
+                pass
+        else:
+            self.control_files.put_utf8('no-working-trees', '')
+    
+    def make_working_trees(self):
+        """Returns the policy for making working trees on new branches."""
+        return not self.control_files._transport.has('no-working-trees')
+
+
+class KnitRepository(MetaDirRepository):
+    """Knit format repository."""
+
+    @needs_read_lock
+    def all_revision_ids(self):
+        """See Repository.all_revision_ids()."""
+        return self._revision_store.all_revision_ids(self.get_transaction())
+
+    def fileid_involved_between_revs(self, from_revid, to_revid):
+        """Find file_id(s) which are involved in the changes between revisions.
+
+        This determines the set of revisions which are involved, and then
+        finds all file ids affected by those revisions.
+        """
+        vf = self._get_revision_vf()
+        from_set = set(vf.get_ancestry(from_revid))
+        to_set = set(vf.get_ancestry(to_revid))
+        changed = to_set.difference(from_set)
+        return self._fileid_involved_by_set(changed)
+
+    def fileid_involved(self, last_revid=None):
+        """Find all file_ids modified in the ancestry of last_revid.
+
+        :param last_revid: If None, last_revision() will be used.
+        """
+        if not last_revid:
+            changed = set(self.all_revision_ids())
+        else:
+            changed = set(self.get_ancestry(last_revid))
+        if None in changed:
+            changed.remove(None)
+        return self._fileid_involved_by_set(changed)
+
+    @needs_read_lock
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This is topologically sorted.
+        """
+        if revision_id is None:
+            return [None]
+        vf = self._get_revision_vf()
+        try:
+            return [None] + vf.get_ancestry(revision_id)
+        except errors.RevisionNotPresent:
+            raise errors.NoSuchRevision(self, revision_id)
+
+    @needs_read_lock
+    def get_revision(self, revision_id):
+        """Return the Revision object for a named revision"""
+        return self.get_revision_reconcile(revision_id)
+
+    @needs_read_lock
+    def get_revision_graph(self, revision_id=None):
+        """Return a dictionary containing the revision graph.
+        
+        :return: a dictionary of revision_id->revision_parents_list.
+        """
+        weave = self._get_revision_vf()
+        entire_graph = weave.get_graph()
+        if revision_id is None:
+            return weave.get_graph()
+        elif revision_id not in weave:
+            raise errors.NoSuchRevision(self, revision_id)
+        else:
+            # add what can be reached from revision_id
+            result = {}
+            pending = set([revision_id])
+            while len(pending) > 0:
+                node = pending.pop()
+                result[node] = weave.get_parents(node)
+                for revision_id in result[node]:
+                    if revision_id not in result:
+                        pending.add(revision_id)
+            return result
+
+    @needs_read_lock
+    def get_revision_graph_with_ghosts(self, revision_ids=None):
+        """Return a graph of the revisions with ghosts marked as applicable.
+
+        :param revision_ids: an iterable of revisions to graph or None for all.
+        :return: a Graph object with the graph reachable from revision_ids.
+        """
+        result = Graph()
+        vf = self._get_revision_vf()
+        versions = set(vf.versions())
+        if not revision_ids:
+            pending = set(self.all_revision_ids())
+            required = set([])
+        else:
+            pending = set(revision_ids)
+            required = set(revision_ids)
+        done = set([])
+        while len(pending):
+            revision_id = pending.pop()
+            if not revision_id in versions:
+                if revision_id in required:
+                    raise errors.NoSuchRevision(self, revision_id)
+                # a ghost
+                result.add_ghost(revision_id)
+                # mark it as done so we dont try for it again.
+                done.add(revision_id)
+                continue
+            parent_ids = vf.get_parents_with_ghosts(revision_id)
+            for parent_id in parent_ids:
+                # is this queued or done ?
+                if (parent_id not in pending and
+                    parent_id not in done):
+                    # no, queue it.
+                    pending.add(parent_id)
+            result.add_node(revision_id, parent_ids)
+            done.add(revision_id)
+        return result
+
+    def _get_revision_vf(self):
+        """:return: a versioned file containing the revisions."""
+        vf = self._revision_store.get_revision_file(self.get_transaction())
+        return vf
+
+    @needs_write_lock
+    def reconcile(self):
+        """Reconcile this repository."""
+        from bzrlib.reconcile import KnitReconciler
+        reconciler = KnitReconciler(self)
+        reconciler.reconcile()
+        return reconciler
+    
+    def revision_parents(self, revid):
+        return self._get_revision_vf().get_parents(rev_id)
+
+class RepositoryFormat(object):
+    """A repository format.
+
+    Formats provide three things:
+     * An initialization routine to construct repository data on disk.
+     * a format string which is used when the BzrDir supports versioned
+       children.
+     * an open routine which returns a Repository instance.
+
+    Formats are placed in an dict by their format string for reference 
+    during opening. These should be subclasses of RepositoryFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every system load.
+
+    Common instance attributes:
+    _matchingbzrdir - the bzrdir format that the repository format was
+    originally written to work with. This can be used if manually
+    constructing a bzrdir and repository, or more commonly for test suite
+    parameterisation.
+    """
+
+    _default_format = None
+    """The default format used for new repositories."""
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def find_format(klass, a_bzrdir):
+        """Return the format for the repository object in a_bzrdir."""
+        try:
+            transport = a_bzrdir.get_repository_transport(None)
+            format_string = transport.get("format").read()
+            return klass._formats[format_string]
+        except errors.NoSuchFile:
+            raise errors.NoRepositoryPresent(a_bzrdir)
+        except KeyError:
+            raise errors.UnknownFormatError(format_string)
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        raise NotImplementedError(self._get_control_store)
+    
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format.
+        
+        Note that in pre format ?? repositories the format string is 
+        not permitted nor written to disk.
+        """
+        raise NotImplementedError(self.get_format_string)
+
+    def get_format_description(self):
+        """Return the short desciption for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """Return the revision store object for this a_bzrdir."""
+        raise NotImplementedError(self._get_revision_store)
+
+    def _get_text_rev_store(self,
+                            transport,
+                            control_files,
+                            name,
+                            compressed=True,
+                            prefixed=False,
+                            serializer=None):
+        """Common logic for getting a revision store for a repository.
+        
+        see self._get_revision_store for the subclass-overridable method to 
+        get the store for a repository.
+        """
+        from bzrlib.store.revision.text import TextRevisionStore
+        dir_mode = control_files._dir_mode
+        file_mode = control_files._file_mode
+        text_store =TextStore(transport.clone(name),
+                              prefixed=prefixed,
+                              compressed=compressed,
+                              dir_mode=dir_mode,
+                              file_mode=file_mode)
+        _revision_store = TextRevisionStore(text_store, serializer)
+        return _revision_store
+
+    def _get_versioned_file_store(self,
+                                  name,
+                                  transport,
+                                  control_files,
+                                  prefixed=True,
+                                  versionedfile_class=WeaveFile,
+                                  escaped=False):
+        weave_transport = control_files._transport.clone(name)
+        dir_mode = control_files._dir_mode
+        file_mode = control_files._file_mode
+        return VersionedFileStore(weave_transport, prefixed=prefixed,
+                                  dir_mode=dir_mode,
+                                  file_mode=file_mode,
+                                  versionedfile_class=versionedfile_class,
+                                  escaped=escaped)
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Initialize a repository of this format in a_bzrdir.
+
+        :param a_bzrdir: The bzrdir to put the new repository in it.
+        :param shared: The repository should be initialized as a sharable one.
+
+        This may raise UninitializableFormat if shared repository are not
+        compatible the a_bzrdir.
+        """
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def open(self, a_bzrdir, _found=False):
+        """Return an instance of this format for the bzrdir a_bzrdir.
+        
+        _found is a private parameter, do not use it.
+        """
+        raise NotImplementedError(self.open)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+
+class PreSplitOutRepositoryFormat(RepositoryFormat):
+    """Base class for the pre split out repository formats."""
+
+    def initialize(self, a_bzrdir, shared=False, _internal=False):
+        """Create a weave repository.
+        
+        TODO: when creating split out bzr branch formats, move this to a common
+        base for Format5, Format6. or something like that.
+        """
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
+
+        if shared:
+            raise errors.IncompatibleFormat(self, a_bzrdir._format)
+
+        if not _internal:
+            # always initialized when the bzrdir is.
+            return self.open(a_bzrdir, _found=True)
+        
+        # Create an empty weave
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'weaves']
+        files = [('inventory.weave', StringIO(empty_weave)),
+                 ]
+        
+        # FIXME: RBC 20060125 dont peek under the covers
+        # NB: no need to escape relative paths that are url safe.
+        control_files = LockableFiles(a_bzrdir.transport, 'branch-lock',
+                                      TransportLock)
+        control_files.create_lock()
+        control_files.lock_write()
+        control_files._transport.mkdir_multi(dirs,
+                mode=control_files._dir_mode)
+        try:
+            for file, content in files:
+                control_files.put(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(a_bzrdir, _found=True)
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
+    def _get_text_store(self, transport, control_files):
+        """Get a store for file texts for this format."""
+        raise NotImplementedError(self._get_text_store)
+
+    def open(self, a_bzrdir, _found=False):
+        """See RepositoryFormat.open()."""
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+
+        repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = a_bzrdir._control_files
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return AllInOneRepository(_format=self,
+                                  a_bzrdir=a_bzrdir,
+                                  _revision_store=_revision_store,
+                                  control_store=control_store,
+                                  text_store=text_store)
+
+
+class RepositoryFormat4(PreSplitOutRepositoryFormat):
+    """Bzr repository format 4.
+
+    This repository format has:
+     - flat stores
+     - TextStores for texts, inventories,revisions.
+
+    This format is deprecated: it indexes texts using a text id which is
+    removed in format 5; initializationa and write support for this format
+    has been removed.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat4, self).__init__()
+        self._matchingbzrdir = bzrlib.bzrdir.BzrDirFormat4()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Repository format 4"
+
+    def initialize(self, url, shared=False, _internal=False):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not 
+        feasible.
+        """
+        return False
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Format 4 repositories have no formal control store at this point.
+        
+        This will cause any control-file-needing apis to fail - this is desired.
+        """
+        return None
+    
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        from bzrlib.xml4 import serializer_v4
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        serializer=serializer_v4)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+
+
+class RepositoryFormat5(PreSplitOutRepositoryFormat):
+    """Bzr control format 5.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - flat stores
+     - TextStores for revisions and signatures.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat5, self).__init__()
+        self._matchingbzrdir = bzrlib.bzrdir.BzrDirFormat5()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 5"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        """Return the revision store object for this a_bzrdir."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves', transport, control_files, prefixed=False)
+
+
+class RepositoryFormat6(PreSplitOutRepositoryFormat):
+    """Bzr control format 6.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat6, self).__init__()
+        self._matchingbzrdir = bzrlib.bzrdir.BzrDirFormat6()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 6"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False,
+                                        prefixed=True)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves', transport, control_files)
+
+
+class MetaDirRepositoryFormat(RepositoryFormat):
+    """Common base class for the new repositories using the metadir layour."""
+
+    def __init__(self):
+        super(MetaDirRepositoryFormat, self).__init__()
+        self._matchingbzrdir = bzrlib.bzrdir.BzrDirMetaFormat1()
+
+    def _create_control_files(self, a_bzrdir):
+        """Create the required files and the initial control_files object."""
+        # FIXME: RBC 20060125 dont peek under the covers
+        # NB: no need to escape relative paths that are url safe.
+        repository_transport = a_bzrdir.get_repository_transport(self)
+        control_files = LockableFiles(repository_transport, 'lock', LockDir)
+        control_files.create_lock()
+        return control_files
+
+    def _upload_blank_content(self, a_bzrdir, dirs, files, utf8_files, shared):
+        """Upload the initial blank content."""
+        control_files = self._create_control_files(a_bzrdir)
+        control_files.lock_write()
+        try:
+            control_files._transport.mkdir_multi(dirs,
+                    mode=control_files._dir_mode)
+            for file, content in files:
+                control_files.put(file, content)
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+            if shared == True:
+                control_files.put_utf8('shared-storage', '')
+        finally:
+            control_files.unlock()
+
+
+class RepositoryFormat7(MetaDirRepositoryFormat):
+    """Bzr repository 7.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+    """
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Repository format 7"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 7"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False,
+                                        prefixed=True,
+                                        )
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves',
+                                              transport,
+                                              control_files)
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Create a weave repository.
+
+        :param shared: If true the repository will be initialized as a shared
+                       repository.
+        """
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
+
+        # Create an empty weave
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'weaves']
+        files = [('inventory.weave', StringIO(empty_weave)), 
+                 ]
+        utf8_files = [('format', self.get_format_string())]
+ 
+        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
+        return self.open(a_bzrdir=a_bzrdir, _found=True)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return MetaDirRepository(_format=self,
+                                 a_bzrdir=a_bzrdir,
+                                 control_files=control_files,
+                                 _revision_store=_revision_store,
+                                 control_store=control_store,
+                                 text_store=text_store)
+
+
+class RepositoryFormatKnit1(MetaDirRepositoryFormat):
+    """Bzr repository knit format 1.
+
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+
+    This format was introduced in bzr 0.8.
+    """
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return VersionedFileStore(
+            repo_transport,
+            prefixed=False,
+            file_mode=control_files._file_mode,
+            versionedfile_class=KnitVersionedFile,
+            versionedfile_kwargs={'factory':KnitPlainFactory()},
+            )
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Knit Repository Format 1"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 1"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        from bzrlib.store.revision.knit import KnitRevisionStore
+        versioned_file_store = VersionedFileStore(
+            repo_transport,
+            file_mode=control_files._file_mode,
+            prefixed=False,
+            precious=True,
+            versionedfile_class=KnitVersionedFile,
+            versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory()},
+            escaped=True,
+            )
+        return KnitRevisionStore(versioned_file_store)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('knits',
+                                              transport,
+                                              control_files,
+                                              versionedfile_class=KnitVersionedFile,
+                                              escaped=True)
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Create a knit format 1 repository.
+
+        :param a_bzrdir: bzrdir to contain the new repository; must already
+            be initialized.
+        :param shared: If true the repository will be initialized as a shared
+                       repository.
+        """
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'knits']
+        files = []
+        utf8_files = [('format', self.get_format_string())]
+        
+        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
+        repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        control_store = self._get_control_store(repo_transport, control_files)
+        transaction = bzrlib.transactions.WriteTransaction()
+        # trigger a write of the inventory store.
+        control_store.get_weave_or_empty('inventory', transaction)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        _revision_store.has_revision_id('A', transaction)
+        _revision_store.get_signature_file(transaction)
+        return self.open(a_bzrdir=a_bzrdir, _found=True)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository(_format=self,
+                              a_bzrdir=a_bzrdir,
+                              control_files=control_files,
+                              _revision_store=_revision_store,
+                              control_store=control_store,
+                              text_store=text_store)
+
+
+# formats which have no format string are not discoverable
+# and not independently creatable, so are not registered.
+_default_format = RepositoryFormat7()
+RepositoryFormat.register_format(_default_format)
+RepositoryFormat.register_format(RepositoryFormatKnit1())
+RepositoryFormat.set_default_format(_default_format)
+_legacy_formats = [RepositoryFormat4(),
+                   RepositoryFormat5(),
+                   RepositoryFormat6()]
+
+
+class InterRepository(InterObject):
+    """This class represents operations taking place between two repositories.
+
+    Its instances have methods like copy_content and fetch, and contain
+    references to the source and target repositories these operations can be 
+    carried out on.
+
+    Often we will provide convenience methods on 'repository' which carry out
+    operations with another repository - they will always forward to
+    InterRepository.get(other).method_name(parameters).
+    """
+
+    _optimisers = set()
+    """The available optimised InterRepository types."""
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+
+        :param revision_id: Only copy the content needed to construct
+                            revision_id and its parents.
+        :param basis: Copy the needed data preferentially from basis.
+        """
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except NotImplementedError:
+            pass
+        # grab the basis available data
+        if basis is not None:
+            self.target.fetch(basis, revision_id=revision_id)
+        # but dont bother fetching if we have the needed data now.
+        if (revision_id not in (None, NULL_REVISION) and 
+            self.target.has_revision(revision_id)):
+            return
+        self.target.fetch(self.source, revision_id=revision_id)
+
+    def _double_lock(self, lock_source, lock_target):
+        """Take out too locks, rolling back the first if the second throws."""
+        lock_source()
+        try:
+            lock_target()
+        except Exception:
+            # we want to ensure that we don't leave source locked by mistake.
+            # and any error on target should not confuse source.
+            self.source.unlock()
+            raise
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """Fetch the content required to construct revision_id.
+
+        The content is copied from source to target.
+
+        :param revision_id: if None all content is copied, if NULL_REVISION no
+                            content is copied.
+        :param pb: optional progress bar to use for progress reports. If not
+                   provided a default one will be created.
+
+        Returns the copied revision count and the failed revisions in a tuple:
+        (copied, failures).
+        """
+        from bzrlib.fetch import GenericRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, self.target._format)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    def lock_read(self):
+        """Take out a logical read lock.
+
+        This will lock the source branch and the target branch. The source gets
+        a read lock and the target a read lock.
+        """
+        self._double_lock(self.source.lock_read, self.target.lock_read)
+
+    def lock_write(self):
+        """Take out a logical write lock.
+
+        This will lock the source branch and the target branch. The source gets
+        a read lock and the target a write lock.
+        """
+        self._double_lock(self.source.lock_read, self.target.lock_write)
+
+    @needs_read_lock
+    def missing_revision_ids(self, revision_id=None):
+        """Return the revision ids that source has that target does not.
+        
+        These are returned in topological order.
+
+        :param revision_id: only return revision ids included by this
+                            revision_id.
+        """
+        # generic, possibly worst case, slow code path.
+        target_ids = set(self.target.all_revision_ids())
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            assert source_ids.pop(0) == None
+        else:
+            source_ids = self.source.all_revision_ids()
+        result_set = set(source_ids).difference(target_ids)
+        # this may look like a no-op: its not. It preserves the ordering
+        # other_ids had while only returning the members from other_ids
+        # that we've decided we need.
+        return [rev_id for rev_id in source_ids if rev_id in result_set]
+
+    def unlock(self):
+        """Release the locks on source and target."""
+        try:
+            self.target.unlock()
+        finally:
+            self.source.unlock()
+
+
+class InterWeaveRepo(InterRepository):
+    """Optimised code paths between Weave based repositories."""
+
+    _matching_repo_format = _default_format
+    """Repository format for testing with."""
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Weave formats.
+        
+        We dont test for the stores being of specific types becase that
+        could lead to confusing results, and there is no need to be 
+        overly general.
+        """
+        try:
+            return (isinstance(source._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)) and
+                    isinstance(target._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)))
+        except AttributeError:
+            return False
+    
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """See InterRepository.copy_content()."""
+        # weave specific optimised path:
+        if basis is not None:
+            # copy the basis in, then fetch remaining data.
+            basis.copy_content_into(self.target, revision_id)
+            # the basis copy_content_into could misset this.
+            try:
+                self.target.set_make_working_trees(self.source.make_working_trees())
+            except NotImplementedError:
+                pass
+            self.target.fetch(self.source, revision_id=revision_id)
+        else:
+            try:
+                self.target.set_make_working_trees(self.source.make_working_trees())
+            except NotImplementedError:
+                pass
+            # FIXME do not peek!
+            if self.source.control_files._transport.listable():
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    self.target.weave_store.copy_all_ids(
+                        self.source.weave_store,
+                        pb=pb,
+                        from_transaction=self.source.get_transaction(),
+                        to_transaction=self.target.get_transaction())
+                    pb.update('copying inventory', 0, 1)
+                    self.target.control_weaves.copy_multi(
+                        self.source.control_weaves, ['inventory'],
+                        from_transaction=self.source.get_transaction(),
+                        to_transaction=self.target.get_transaction())
+                    self.target._revision_store.text_store.copy_all_ids(
+                        self.source._revision_store.text_store,
+                        pb=pb)
+                finally:
+                    pb.finished()
+            else:
+                self.target.fetch(self.source, revision_id=revision_id)
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import GenericRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, self.target._format)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    @needs_read_lock
+    def missing_revision_ids(self, revision_id=None):
+        """See InterRepository.missing_revision_ids()."""
+        # we want all revisions to satisfy revision_id in source.
+        # but we dont want to stat every file here and there.
+        # we want then, all revisions other needs to satisfy revision_id 
+        # checked, but not those that we have locally.
+        # so the first thing is to get a subset of the revisions to 
+        # satisfy revision_id in source, and then eliminate those that
+        # we do already have. 
+        # this is slow on high latency connection to self, but as as this
+        # disk format scales terribly for push anyway due to rewriting 
+        # inventory.weave, this is considered acceptable.
+        # - RBC 20060209
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            assert source_ids.pop(0) == None
+        else:
+            source_ids = self.source._all_possible_ids()
+        source_ids_set = set(source_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but dont try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target._all_possible_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        required_topo_revisions = [rev_id for rev_id in source_ids if rev_id in required_revisions]
+        if revision_id is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            return required_topo_revisions
+        else:
+            # if we just grabbed the possibly available ids, then 
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            return self.source._eliminate_revisions_not_present(required_topo_revisions)
+
+
+class InterKnitRepo(InterRepository):
+    """Optimised code paths between Knit based repositories."""
+
+    _matching_repo_format = RepositoryFormatKnit1()
+    """Repository format for testing with."""
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Knit formats.
+        
+        We dont test for the stores being of specific types becase that
+        could lead to confusing results, and there is no need to be 
+        overly general.
+        """
+        try:
+            return (isinstance(source._format, (RepositoryFormatKnit1)) and
+                    isinstance(target._format, (RepositoryFormatKnit1)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import KnitRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, self.target._format)
+        f = KnitRepoFetcher(to_repository=self.target,
+                            from_repository=self.source,
+                            last_revision=revision_id,
+                            pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    @needs_read_lock
+    def missing_revision_ids(self, revision_id=None):
+        """See InterRepository.missing_revision_ids()."""
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            assert source_ids.pop(0) == None
+        else:
+            source_ids = self.source._all_possible_ids()
+        source_ids_set = set(source_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but dont try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target._all_possible_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        required_topo_revisions = [rev_id for rev_id in source_ids if rev_id in required_revisions]
+        if revision_id is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            return required_topo_revisions
+        else:
+            # if we just grabbed the possibly available ids, then 
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            return self.source._eliminate_revisions_not_present(required_topo_revisions)
+
+InterRepository.register_optimiser(InterWeaveRepo)
+InterRepository.register_optimiser(InterKnitRepo)
+
+
+class RepositoryTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple repository formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and bzrdir_format and
+    repository_format classes into each copy. Each copy is also given a new id()
+    to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for repository_format, bzrdir_format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = bzrdir_format
+            new_test.repository_format = repository_format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), repository_format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+
+class InterRepositoryTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple inter repository formats.
+
+    This is done by copying the test once for each interrepo provider and injecting
+    the transport_server, transport_readonly_server, repository_format and 
+    repository_to_format classes into each copy.
+    Each copy is also given a new id() to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for interrepo_class, repository_format, repository_format_to in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.interrepo_class = interrepo_class
+            new_test.repository_format = repository_format
+            new_test.repository_format_to = repository_format_to
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), interrepo_class.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    @staticmethod
+    def default_test_list():
+        """Generate the default list of interrepo permutations to test."""
+        result = []
+        # test the default InterRepository between format 6 and the current 
+        # default format.
+        # XXX: robertc 20060220 reinstate this when there are two supported
+        # formats which do not have an optimal code path between them.
+        result.append((InterRepository,
+                       RepositoryFormat6(),
+                       RepositoryFormatKnit1()))
+        for optimiser in InterRepository._optimisers:
+            result.append((optimiser,
+                           optimiser._matching_repo_format,
+                           optimiser._matching_repo_format
+                           ))
+        # if there are specific combinations we want to use, we can add them 
+        # here.
+        return result
+
+
+class CopyConverter(object):
+    """A repository conversion tool which just performs a copy of the content.
+    
+    This is slow but quite reliable.
+    """
+
+    def __init__(self, target_format):
+        """Create a CopyConverter.
+
+        :param target_format: The format the resulting repository should be.
+        """
+        self.target_format = target_format
+        
+    def convert(self, repo, pb):
+        """Perform the conversion of to_convert, giving feedback via pb.
+
+        :param to_convert: The disk object to convert.
+        :param pb: a progress bar to use for progress information.
+        """
+        self.pb = pb
+        self.count = 0
+        self.total = 4
+        # this is only useful with metadir layouts - separated repo content.
+        # trigger an assertion if not such
+        repo._format.get_format_string()
+        self.repo_dir = repo.bzrdir
+        self.step('Moving repository to repository.backup')
+        self.repo_dir.transport.move('repository', 'repository.backup')
+        backup_transport =  self.repo_dir.transport.clone('repository.backup')
+        self.source_repo = repo._format.open(self.repo_dir,
+            _found=True,
+            _override_transport=backup_transport)
+        self.step('Creating new repository')
+        converted = self.target_format.initialize(self.repo_dir,
+                                                  self.source_repo.is_shared())
+        converted.lock_write()
+        try:
+            self.step('Copying content into repository.')
+            self.source_repo.copy_content_into(converted)
+        finally:
+            converted.unlock()
+        self.step('Deleting old repository content.')
+        self.repo_dir.transport.delete_tree('repository.backup')
+        self.pb.note('repository converted')
+
+    def step(self, message):
+        """Update the pb by a step."""
+        self.count +=1
+        self.pb.update(message, self.count, self.total)
+
+
+# Copied from xml.sax.saxutils
+def _unescape_xml(data):
+    """Unescape &amp;, &lt;, and &gt; in a string of data.
+    """
+    data = data.replace("&lt;", "<")
+    data = data.replace("&gt;", ">")
+    # must do ampersand last
+    return data.replace("&amp;", "&")

=== removed file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-08-11 18:09:13 +0000
+++ bzrlib/revfile.py	1970-01-01 00:00:00 +0000
@@ -1,574 +0,0 @@
-#! /usr/bin/env python
-
-# (C) 2005 Canonical Ltd
-
-# based on an idea by Matt Mackall
-# modified to squish into bzr by Martin Pool
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Packed file revision storage.
-
-A Revfile holds the text history of a particular source file, such
-as Makefile.  It can represent a tree of text versions for that
-file, allowing for microbranches within a single repository.
-
-This is stored on disk as two files: an index file, and a data file.
-The index file is short and always read completely into memory; the
-data file is much longer and only the relevant bits of it,
-identified by the index file, need to be read.
-
-Each text version is identified by the SHA-1 of the full text of
-that version.  It also has a sequence number within the file.
-
-The index file has a short header and then a sequence of fixed-length
-records:
-
-* byte[20]    SHA-1 of text (as binary, not hex)
-* uint32      sequence number this is based on, or -1 for full text
-* uint32      flags: 1=zlib compressed
-* uint32      offset in text file of start
-* uint32      length of compressed delta in text file
-* uint32[3]   reserved
-
-total 48 bytes.
-
-The header is also 48 bytes for tidyness and easy calculation.
-
-Both the index and the text are only ever appended to; a consequence
-is that sequence numbers are stable references.  But not every
-repository in the world will assign the same sequence numbers,
-therefore the SHA-1 is the only universally unique reference.
-
-This is meant to scale to hold 100,000 revisions of a single file, by
-which time the index file will be ~4.8MB and a bit big to read
-sequentially.
-
-Some of the reserved fields could be used to implement a (semi?)
-balanced tree indexed by SHA1 so we can much more efficiently find the
-index associated with a particular hash.  For 100,000 revs we would be
-able to find it in about 17 random reads, which is not too bad.
-
-This performs pretty well except when trying to calculate deltas of
-really large files.  For that the main thing would be to plug in
-something faster than difflib, which is after all pure Python.
-Another approach is to just store the gzipped full text of big files,
-though perhaps that's too perverse?
-
-The iter method here will generally read through the whole index file
-in one go.  With readahead in the kernel and python/libc (typically
-128kB) this means that there should be no seeks and often only one
-read() call to get everything into memory.
-"""
- 
-
-# TODO: Something like pread() would make this slightly simpler and
-# perhaps more efficient.
-
-# TODO: Could also try to mmap things...  Might be faster for the
-# index in particular?
-
-# TODO: Some kind of faster lookup of SHAs?  The bad thing is that probably means
-# rewriting existing records, which is not so nice.
-
-# TODO: Something to check that regions identified in the index file
-# completely butt up and do not overlap.  Strictly it's not a problem
-# if there are gaps and that can happen if we're interrupted while
-# writing to the datafile.  Overlapping would be very bad though.
-
-# TODO: Shouldn't need to lock if we always write in append mode and
-# then ftell after writing to see where it went.  In any case we
-# assume the whole branch is protected by a lock.
-
-import sys, zlib, struct, mdiff, stat, os, sha
-from binascii import hexlify, unhexlify
-
-_RECORDSIZE = 48
-
-_HEADER = "bzr revfile v1\n"
-_HEADER = _HEADER + ('\xff' * (_RECORDSIZE - len(_HEADER)))
-_NO_RECORD = 0xFFFFFFFFL
-
-# fields in the index record
-I_SHA = 0
-I_BASE = 1
-I_FLAGS = 2
-I_OFFSET = 3
-I_LEN = 4
-
-FL_GZIP = 1
-
-# maximum number of patches in a row before recording a whole text.
-CHAIN_LIMIT = 10
-
-
-class RevfileError(Exception):
-    pass
-
-class LimitHitException(Exception):
-    pass
-
-class Revfile(object):
-    def __init__(self, basename, mode):
-        # TODO: Lock file  while open
-
-        # TODO: advise of random access
-
-        self.basename = basename
-
-        if mode not in ['r', 'w']:
-            raise RevfileError("invalid open mode %r" % mode)
-        self.mode = mode
-        
-        idxname = basename + '.irev'
-        dataname = basename + '.drev'
-
-        idx_exists = os.path.exists(idxname)
-        data_exists = os.path.exists(dataname)
-
-        if idx_exists != data_exists:
-            raise RevfileError("half-assed revfile")
-        
-        if not idx_exists:
-            if mode == 'r':
-                raise RevfileError("Revfile %r does not exist" % basename)
-            
-            self.idxfile = open(idxname, 'w+b')
-            self.datafile = open(dataname, 'w+b')
-            
-            self.idxfile.write(_HEADER)
-            self.idxfile.flush()
-        else:
-            if mode == 'r':
-                diskmode = 'rb'
-            else:
-                diskmode = 'r+b'
-                
-            self.idxfile = open(idxname, diskmode)
-            self.datafile = open(dataname, diskmode)
-            
-            h = self.idxfile.read(_RECORDSIZE)
-            if h != _HEADER:
-                raise RevfileError("bad header %r in index of %r"
-                                   % (h, self.basename))
-
-
-    def _check_index(self, idx):
-        if idx < 0 or idx > len(self):
-            raise RevfileError("invalid index %r" % idx)
-
-    def _check_write(self):
-        if self.mode != 'w':
-            raise RevfileError("%r is open readonly" % self.basename)
-
-
-    def find_sha(self, s):
-        assert isinstance(s, str)
-        assert len(s) == 20
-        
-        for idx, idxrec in enumerate(self):
-            if idxrec[I_SHA] == s:
-                return idx
-        else:
-            return _NO_RECORD
-
-
-
-    def _add_compressed(self, text_sha, data, base, compress):
-        # well, maybe compress
-        flags = 0
-        if compress:
-            data_len = len(data)
-            if data_len > 50:
-                # don't do compression if it's too small; it's unlikely to win
-                # enough to be worthwhile
-                compr_data = zlib.compress(data)
-                compr_len = len(compr_data)
-                if compr_len < data_len:
-                    data = compr_data
-                    flags = FL_GZIP
-                    ##print '- compressed %d -> %d, %.1f%%' \
-                    ##      % (data_len, compr_len, float(compr_len)/float(data_len) * 100.0)
-        return self._add_raw(text_sha, data, base, flags)
-        
-
-
-    def _add_raw(self, text_sha, data, base, flags):
-        """Add pre-processed data, can be either full text or delta.
-
-        This does the compression if that makes sense."""
-        idx = len(self)
-        self.datafile.seek(0, 2)        # to end
-        self.idxfile.seek(0, 2)
-        assert self.idxfile.tell() == _RECORDSIZE * (idx + 1)
-        data_offset = self.datafile.tell()
-
-        assert isinstance(data, str) # not unicode or anything weird
-
-        self.datafile.write(data)
-        self.datafile.flush()
-
-        assert isinstance(text_sha, str)
-        entry = text_sha
-        entry += struct.pack(">IIII12x", base, flags, data_offset, len(data))
-        assert len(entry) == _RECORDSIZE
-
-        self.idxfile.write(entry)
-        self.idxfile.flush()
-
-        return idx
-        
-
-
-    def _add_full_text(self, text, text_sha, compress):
-        """Add a full text to the file.
-
-        This is not compressed against any reference version.
-
-        Returns the index for that text."""
-        return self._add_compressed(text_sha, text, _NO_RECORD, compress)
-
-
-    # NOT USED
-    def _choose_base(self, seed, base):
-        while seed & 3 == 3:
-            if base == _NO_RECORD:
-                return _NO_RECORD
-            idxrec = self[base]
-            if idxrec[I_BASE] == _NO_RECORD:
-                return base
-
-            base = idxrec[I_BASE]
-            seed >>= 2
-                
-        return base        # relative to this full text
-        
-
-
-    def _add_delta(self, text, text_sha, base, compress):
-        """Add a text stored relative to a previous text."""
-        self._check_index(base)
-
-        try:
-            base_text = self.get(base, CHAIN_LIMIT)
-        except LimitHitException:
-            return self._add_full_text(text, text_sha, compress)
-        
-        data = mdiff.bdiff(base_text, text)
-
-
-        if True: # paranoid early check for bad diff
-            result = mdiff.bpatch(base_text, data)
-            assert result == text
-            
-        
-        # If the delta is larger than the text, we might as well just
-        # store the text.  (OK, the delta might be more compressible,
-        # but the overhead of applying it probably still makes it
-        # bad, and I don't want to compress both of them to find out.)
-        if len(data) >= len(text):
-            return self._add_full_text(text, text_sha, compress)
-        else:
-            return self._add_compressed(text_sha, data, base, compress)
-
-
-    def add(self, text, base=None, compress=True):
-        """Add a new text to the revfile.
-
-        If the text is already present them its existing id is
-        returned and the file is not changed.
-
-        If compress is true then gzip compression will be used if it
-        reduces the size.
-
-        If a base index is specified, that text *may* be used for
-        delta compression of the new text.  Delta compression will
-        only be used if it would be a size win and if the existing
-        base is not at too long of a delta chain already.
-        """
-        if base == None:
-            base = _NO_RECORD
-        
-        self._check_write()
-        
-        text_sha = sha.new(text).digest()
-
-        idx = self.find_sha(text_sha)
-        if idx != _NO_RECORD:
-            # TODO: Optional paranoid mode where we read out that record and make sure
-            # it's the same, in case someone ever breaks SHA-1.
-            return idx                  # already present
-        
-        # base = self._choose_base(ord(text_sha[0]), base)
-
-        if base == _NO_RECORD:
-            return self._add_full_text(text, text_sha, compress)
-        else:
-            return self._add_delta(text, text_sha, base, compress)
-
-
-
-    def get(self, idx, recursion_limit=None):
-        """Retrieve text of a previous revision.
-
-        If recursion_limit is an integer then walk back at most that
-        many revisions and then raise LimitHitException, indicating
-        that we ought to record a new file text instead of another
-        delta.  Don't use this when trying to get out an existing
-        revision."""
-        
-        idxrec = self[idx]
-        base = idxrec[I_BASE]
-        if base == _NO_RECORD:
-            text = self._get_full_text(idx, idxrec)
-        else:
-            text = self._get_patched(idx, idxrec, recursion_limit)
-
-        if sha.new(text).digest() != idxrec[I_SHA]:
-            raise RevfileError("corrupt SHA-1 digest on record %d in %s"
-                               % (idx, self.basename))
-
-        return text
-
-
-
-    def _get_raw(self, idx, idxrec):
-        flags = idxrec[I_FLAGS]
-        if flags & ~FL_GZIP:
-            raise RevfileError("unsupported index flags %#x on index %d"
-                               % (flags, idx))
-        
-        l = idxrec[I_LEN]
-        if l == 0:
-            return ''
-
-        self.datafile.seek(idxrec[I_OFFSET])
-
-        data = self.datafile.read(l)
-        if len(data) != l:
-            raise RevfileError("short read %d of %d "
-                               "getting text for record %d in %r"
-                               % (len(data), l, idx, self.basename))
-
-        if flags & FL_GZIP:
-            data = zlib.decompress(data)
-
-        return data
-        
-
-    def _get_full_text(self, idx, idxrec):
-        assert idxrec[I_BASE] == _NO_RECORD
-
-        text = self._get_raw(idx, idxrec)
-
-        return text
-
-
-    def _get_patched(self, idx, idxrec, recursion_limit):
-        base = idxrec[I_BASE]
-        assert base >= 0
-        assert base < idx    # no loops!
-
-        if recursion_limit == None:
-            sub_limit = None
-        else:
-            sub_limit = recursion_limit - 1
-            if sub_limit < 0:
-                raise LimitHitException()
-            
-        base_text = self.get(base, sub_limit)
-        patch = self._get_raw(idx, idxrec)
-
-        text = mdiff.bpatch(base_text, patch)
-
-        return text
-
-
-
-    def __len__(self):
-        """Return number of revisions."""
-        l = os.fstat(self.idxfile.fileno())[stat.ST_SIZE]
-        if l % _RECORDSIZE:
-            raise RevfileError("bad length %d on index of %r" % (l, self.basename))
-        if l < _RECORDSIZE:
-            raise RevfileError("no header present in index of %r" % (self.basename))
-        return int(l / _RECORDSIZE) - 1
-
-
-    def __getitem__(self, idx):
-        """Index by sequence id returns the index field"""
-        ## TODO: Can avoid seek if we just moved there...
-        self._seek_index(idx)
-        idxrec = self._read_next_index()
-        if idxrec == None:
-            raise IndexError("no index %d" % idx)
-        else:
-            return idxrec
-
-
-    def _seek_index(self, idx):
-        if idx < 0:
-            raise RevfileError("invalid index %r" % idx)
-        self.idxfile.seek((idx + 1) * _RECORDSIZE)
-
-
-
-    def __iter__(self):
-        """Read back all index records.
-
-        Do not seek the index file while this is underway!"""
-        ## sys.stderr.write(" ** iter called ** \n")
-        self._seek_index(0)
-        while True:
-            idxrec = self._read_next_index()
-            if not idxrec:
-                break
-            yield idxrec
-        
-
-    def _read_next_index(self):
-        rec = self.idxfile.read(_RECORDSIZE)
-        if not rec:
-            return None
-        elif len(rec) != _RECORDSIZE:
-            raise RevfileError("short read of %d bytes getting index %d from %r"
-                               % (len(rec), idx, self.basename))
-        
-        return struct.unpack(">20sIIII12x", rec)
-
-        
-    def dump(self, f=sys.stdout):
-        f.write('%-8s %-40s %-8s %-8s %-8s %-8s\n' 
-                % tuple('idx sha1 base flags offset len'.split()))
-        f.write('-------- ---------------------------------------- ')
-        f.write('-------- -------- -------- --------\n')
-
-        for i, rec in enumerate(self):
-            f.write("#%-7d %40s " % (i, hexlify(rec[0])))
-            if rec[1] == _NO_RECORD:
-                f.write("(none)   ")
-            else:
-                f.write("#%-7d " % rec[1])
-                
-            f.write("%8x %8d %8d\n" % (rec[2], rec[3], rec[4]))
-
-
-    def total_text_size(self):
-        """Return the sum of sizes of all file texts.
-
-        This is how much space they would occupy if they were stored without
-        delta and gzip compression.
-
-        As a side effect this completely validates the Revfile, checking that all
-        texts can be reproduced with the correct SHA-1."""
-        t = 0L
-        for idx in range(len(self)):
-            t += len(self.get(idx))
-        return t
-
-
-    def check(self, pb=None):
-        """Extract every version and check its hash."""
-        total = len(self)
-        for i in range(total):
-            if pb:
-                pb.update("check revision", i, total)
-            # the get method implicitly checks the SHA-1
-            self.get(i)
-        if pb:
-            pb.clear()
-        
-
-
-def main(argv):
-    try:
-        cmd = argv[1]
-        filename = argv[2]
-    except IndexError:
-        sys.stderr.write("usage: revfile dump REVFILE\n"
-                         "       revfile add REVFILE < INPUT\n"
-                         "       revfile add-delta REVFILE BASE < INPUT\n"
-                         "       revfile add-series REVFILE BASE FILE...\n"
-                         "       revfile get REVFILE IDX\n"
-                         "       revfile find-sha REVFILE HEX\n"
-                         "       revfile total-text-size REVFILE\n"
-                         "       revfile last REVFILE\n")
-        return 1
-
-    if filename.endswith('.drev') or filename.endswith('.irev'):
-        filename = filename[:-5]
-
-    def rw():
-        return Revfile(filename, 'w')
-
-    def ro():
-        return Revfile(filename, 'r')
-
-    if cmd == 'add':
-        print rw().add(sys.stdin.read())
-    elif cmd == 'add-delta':
-        print rw().add(sys.stdin.read(), int(argv[3]))
-    elif cmd == 'add-series':
-        r = rw()
-        rev = int(argv[3])
-        for fn in argv[4:]:
-            print rev
-            rev = r.add(file(fn).read(), rev)
-    elif cmd == 'dump':
-        ro().dump()
-    elif cmd == 'get':
-        try:
-            idx = int(argv[3])
-        except IndexError:
-            sys.stderr.write("usage: revfile get FILE IDX\n")
-            return 1
-
-        r = ro()
-
-        if idx < 0 or idx >= len(r):
-            sys.stderr.write("invalid index %r\n" % idx)
-            return 1
-
-        sys.stdout.write(r.get(idx))
-    elif cmd == 'find-sha':
-        try:
-            s = unhexlify(argv[3])
-        except IndexError:
-            sys.stderr.write("usage: revfile find-sha FILE HEX\n")
-            return 1
-
-        idx = ro().find_sha(s)
-        if idx == _NO_RECORD:
-            sys.stderr.write("no such record\n")
-            return 1
-        else:
-            print idx
-    elif cmd == 'total-text-size':
-        print ro().total_text_size()
-    elif cmd == 'last':
-        print len(ro())-1
-    elif cmd == 'check':
-        import bzrlib.progress
-        pb = bzrlib.progress.ProgressBar()
-        ro().check(pb)
-    else:
-        sys.stderr.write("unknown command %r\n" % cmd)
-        return 1
-    
-
-if __name__ == '__main__':
-    import sys
-    sys.exit(main(sys.argv) or 0)

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-08-05 19:23:12 +0000
+++ bzrlib/revision.py	2006-04-12 04:57:34 +0000
@@ -14,33 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some kind of command-line display of revision properties: 
+# perhaps show them in log -v and allow them as options to the commit command.
+
 
 import bzrlib.errors
-
-
-class RevisionReference(object):
-    """
-    Reference to a stored revision.
-
-    Includes the revision_id and revision_sha1.
-    """
-    revision_id = None
-    revision_sha1 = None
-    def __init__(self, revision_id, revision_sha1=None):
-        if revision_id == None \
-           or isinstance(revision_id, basestring):
-            self.revision_id = revision_id
-        else:
-            raise ValueError('bad revision_id %r' % revision_id)
-
-        if revision_sha1 != None:
-            if isinstance(revision_sha1, basestring) \
-               and len(revision_sha1) == 40:
-                self.revision_sha1 = revision_sha1
-            else:
-                raise ValueError('bad revision_sha1 %r' % revision_sha1)
-                
-
+import bzrlib.errors as errors
+from bzrlib.graph import node_distances, select_farthest, all_descendants, Graph
+from bzrlib.osutils import contains_whitespace
+from bzrlib.progress import DummyProgress
+from bzrlib.symbol_versioning import *
+
+NULL_REVISION="null:"
 
 class Revision(object):
     """Single revision on a branch.
@@ -51,137 +36,92 @@
 
     After bzr 0.0.5 revisions are allowed to have multiple parents.
 
-    parents
-        List of parent revisions, each is a RevisionReference.
+    parent_ids
+        List of parent revision_ids
+
+    properties
+        Dictionary of revision properties.  These are attached to the
+        revision as extra metadata.  The name must be a single 
+        word; the value can be an arbitrary string.
     """
-    inventory_id = None
-    inventory_sha1 = None
-    revision_id = None
-    timestamp = None
-    message = None
-    timezone = None
-    committer = None
     
-    def __init__(self, **args):
+    def __init__(self, revision_id, properties=None, **args):
+        self.revision_id = revision_id
+        self.properties = properties or {}
+        self._check_properties()
+        self.parent_ids = []
+        self.parent_sha1s = []
         self.__dict__.update(args)
-        self.parents = []
-
 
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
 
-        
-    def to_element(self):
-        from bzrlib.xml import Element, SubElement
-        
-        root = Element('revision',
-                       committer = self.committer,
-                       timestamp = '%.9f' % self.timestamp,
-                       revision_id = self.revision_id,
-                       inventory_id = self.inventory_id,
-                       inventory_sha1 = self.inventory_sha1,
-                       )
-        if self.timezone:
-            root.set('timezone', str(self.timezone))
-        root.text = '\n'
-        
-        msg = SubElement(root, 'message')
-        msg.text = self.message
-        msg.tail = '\n'
-
-        if self.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in self.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-
-    def from_element(cls, elt):
-        return unpack_revision(elt)
-
-    from_element = classmethod(from_element)
-
-
-
-def unpack_revision(elt):
-    """Convert XML element into Revision object."""
-    # <changeset> is deprecated...
-    if elt.tag not in ('revision', 'changeset'):
-        raise bzrlib.errors.BzrError("unexpected tag in revision file: %r" % elt)
-
-    rev = Revision(committer = elt.get('committer'),
-                   timestamp = float(elt.get('timestamp')),
-                   revision_id = elt.get('revision_id'),
-                   inventory_id = elt.get('inventory_id'),
-                   inventory_sha1 = elt.get('inventory_sha1')
-                   )
-
-    precursor = elt.get('precursor')
-    precursor_sha1 = elt.get('precursor_sha1')
-
-    pelts = elt.find('parents')
-
-    if pelts:
-        for p in pelts:
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'),
-                                        p.get('revision_sha1'))
-            rev.parents.append(rev_ref)
-
-        if precursor:
-            # must be consistent
-            prec_parent = rev.parents[0].revision_id
-            assert prec_parent == precursor
-    elif precursor:
-        # revisions written prior to 0.0.5 have a single precursor
-        # give as an attribute
-        rev_ref = RevisionReference(precursor, precursor_sha1)
-        rev.parents.append(rev_ref)
-
-    v = elt.get('timezone')
-    rev.timezone = v and int(v)
-
-    rev.message = elt.findtext('message') # text of <message>
-    return rev
-
-
-
-REVISION_ID_RE = None
-
-def validate_revision_id(rid):
-    """Check rid is syntactically valid for a revision id."""
-    global REVISION_ID_RE
-    if not REVISION_ID_RE:
-        import re
-        REVISION_ID_RE = re.compile('[\w.-]+@[\w.-]+--?\d+--?[0-9a-f]+\Z')
-
-    if not REVISION_ID_RE.match(rid):
-        raise ValueError("malformed revision-id %r" % rid)
-
-def is_ancestor(revision_id, candidate_id, revision_source):
+    def __eq__(self, other):
+        if not isinstance(other, Revision):
+            return False
+        # FIXME: rbc 20050930 parent_ids are not being compared
+        return (
+                self.inventory_sha1 == other.inventory_sha1
+                and self.revision_id == other.revision_id
+                and self.timestamp == other.timestamp
+                and self.message == other.message
+                and self.timezone == other.timezone
+                and self.committer == other.committer
+                and self.properties == other.properties)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def _check_properties(self):
+        """Verify that all revision properties are OK.
+        """
+        for name, value in self.properties.iteritems():
+            if not isinstance(name, basestring) or contains_whitespace(name):
+                raise ValueError("invalid property name %r" % name)
+            if not isinstance(value, basestring):
+                raise ValueError("invalid property value %r for %r" % 
+                                 (name, value))
+
+    def get_history(self, repository):
+        """Return the canonical line-of-history for this revision.
+
+        If ghosts are present this may differ in result from a ghost-free
+        repository.
+        """
+        current_revision = self
+        reversed_result = []
+        while current_revision is not None:
+            reversed_result.append(current_revision.revision_id)
+            if not len (current_revision.parent_ids):
+                reversed_result.append(None)
+                current_revision = None
+            else:
+                next_revision_id = current_revision.parent_ids[0]
+                current_revision = repository.get_revision(next_revision_id)
+        reversed_result.reverse()
+        return reversed_result
+
+
+def is_ancestor(revision_id, candidate_id, branch):
     """Return true if candidate_id is an ancestor of revision_id.
+
     A false negative will be returned if any intermediate descendent of
     candidate_id is not present in any of the revision_sources.
     
     revisions_source is an object supporting a get_revision operation that
     behaves like Branch's.
     """
-
+    return candidate_id in branch.repository.get_ancestry(revision_id)
+
+
+def iter_ancestors(revision_id, revision_source, only_present=False):
     ancestors = (revision_id,)
+    distance = 0
     while len(ancestors) > 0:
         new_ancestors = []
         for ancestor in ancestors:
-            if ancestor == candidate_id:
-                return True
+            if not only_present:
+                yield ancestor, distance
             try:
                 revision = revision_source.get_revision(ancestor)
             except bzrlib.errors.NoSuchRevision, e:
@@ -189,16 +129,173 @@
                     raise 
                 else:
                     continue
-            new_ancestors.extend([p.revision_id for p in revision.parents])
+            if only_present:
+                yield ancestor, distance
+            new_ancestors.extend(revision.parent_ids)
         ancestors = new_ancestors
+        distance += 1
+
+
+def find_present_ancestors(revision_id, revision_source):
+    """Return the ancestors of a revision present in a branch.
+
+    It's possible that a branch won't have the complete ancestry of
+    one of its revisions.  
+
+    """
+    found_ancestors = {}
+    anc_iter = enumerate(iter_ancestors(revision_id, revision_source,
+                         only_present=True))
+    for anc_order, (anc_id, anc_distance) in anc_iter:
+        if not found_ancestors.has_key(anc_id):
+            found_ancestors[anc_id] = (anc_order, anc_distance)
+    return found_ancestors
+    
+
+def __get_closest(intersection):
+    intersection.sort()
+    matches = [] 
+    for entry in intersection:
+        if entry[0] == intersection[0][0]:
+            matches.append(entry[2])
+    return matches
+
+
+def revision_graph(revision, revision_source):
+    """Produce a graph of the ancestry of the specified revision.
+    
+    :return: root, ancestors map, descendants map
+    """
+    revision_source.lock_read()
+    try:
+        return _revision_graph(revision, revision_source)
+    finally:
+        revision_source.unlock()
+
+
+def _revision_graph(revision, revision_source):
+    """See revision_graph."""
+    from bzrlib.tsort import topo_sort
+    graph = revision_source.get_revision_graph(revision)
+    # mark all no-parent revisions as being NULL_REVISION parentage.
+    for node, parents in graph.items():
+        if len(parents) == 0:
+            graph[node] = [NULL_REVISION]
+    # add NULL_REVISION to the graph
+    graph[NULL_REVISION] = []
+
+    # pick a root. If there are multiple roots
+    # this could pick a random one.
+    topo_order = topo_sort(graph.items())
+    root = topo_order[0]
+
+    ancestors = {}
+    descendants = {}
+
+    # map the descendants of the graph.
+    # and setup our set based return graph.
+    for node in graph.keys():
+        descendants[node] = {}
+    for node, parents in graph.items():
+        for parent in parents:
+            descendants[parent][node] = 1
+        ancestors[node] = set(parents)
+
+    assert root not in descendants[root]
+    assert root not in ancestors[root]
+    return root, ancestors, descendants
+
+
+def combined_graph(revision_a, revision_b, revision_source):
+    """Produce a combined ancestry graph.
+    Return graph root, ancestors map, descendants map, set of common nodes"""
+    root, ancestors, descendants = revision_graph(
+        revision_a, revision_source)
+    root_b, ancestors_b, descendants_b = revision_graph(
+        revision_b, revision_source)
+    if root != root_b:
+        raise bzrlib.errors.NoCommonRoot(revision_a, revision_b)
+    common = set()
+    for node, node_anc in ancestors_b.iteritems():
+        if node in ancestors:
+            common.add(node)
+        else:
+            ancestors[node] = set()
+        ancestors[node].update(node_anc)
+    for node, node_dec in descendants_b.iteritems():
+        if node not in descendants:
+            descendants[node] = {}
+        descendants[node].update(node_dec)
+    return root, ancestors, descendants, common
+
+
+def common_ancestor(revision_a, revision_b, revision_source, 
+                    pb=DummyProgress()):
+    if None in (revision_a, revision_b):
+        return None
+    # trivial optimisation
+    if revision_a == revision_b:
+        return revision_a
+    try:
+        try:
+            pb.update('Picking ancestor', 1, 3)
+            graph = revision_source.get_revision_graph_with_ghosts(
+                [revision_a, revision_b])
+            # convert to a NULL_REVISION based graph.
+            ancestors = graph.get_ancestors()
+            descendants = graph.get_descendants()
+            common = set(graph.get_ancestry(revision_a)).intersection(
+                     set(graph.get_ancestry(revision_b)))
+            descendants[NULL_REVISION] = {}
+            ancestors[NULL_REVISION] = []
+            for root in graph.roots:
+                descendants[NULL_REVISION][root] = 1
+                ancestors[root].append(NULL_REVISION)
+            for ghost in graph.ghosts:
+                # ghosts act as roots for the purpose of finding 
+                # the longest paths from the root: any ghost *might*
+                # be directly attached to the root, so we treat them
+                # as being such.
+                # ghost now descends from NULL
+                descendants[NULL_REVISION][ghost] = 1
+                # that is it has an ancestor of NULL
+                ancestors[ghost] = [NULL_REVISION]
+                # ghost is common if any of ghosts descendants are common:
+                for ghost_descendant in descendants[ghost]:
+                    if ghost_descendant in common:
+                        common.add(ghost)
+                
+            root = NULL_REVISION
+            common.add(NULL_REVISION)
+        except bzrlib.errors.NoCommonRoot:
+            raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+            
+        pb.update('Picking ancestor', 2, 3)
+        distances = node_distances (descendants, ancestors, root)
+        pb.update('Picking ancestor', 3, 2)
+        farthest = select_farthest(distances, common)
+        if farthest is None or farthest == NULL_REVISION:
+            raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    finally:
+        pb.clear()
+    return farthest
 
 
 class MultipleRevisionSources(object):
+    """Proxy that looks in multiple branches for revisions."""
     def __init__(self, *args):
         object.__init__(self)
         assert len(args) != 0
         self._revision_sources = args
 
+    def revision_parents(self, revision_id):
+        for source in self._revision_sources:
+            try:
+                return source.revision_parents(revision_id)
+            except (errors.WeaveRevisionNotPresent, errors.NoSuchRevision), e:
+                pass
+        raise e
+
     def get_revision(self, revision_id):
         for source in self._revision_sources:
             try:
@@ -206,3 +303,144 @@
             except bzrlib.errors.NoSuchRevision, e:
                 pass
         raise e
+
+    def get_revision_graph(self, revision_id):
+        # we could probe incrementally until the pending
+        # ghosts list stop growing, but its cheaper for now
+        # to just ask for the complete graph for each repository.
+        graphs = []
+        for source in self._revision_sources:
+            ghost_graph = source.get_revision_graph_with_ghosts()
+            graphs.append(ghost_graph)
+        absent = 0
+        for graph in graphs:
+            if not revision_id in graph.get_ancestors():
+                absent += 1
+        if absent == len(graphs):
+            raise errors.NoSuchRevision(self._revision_sources[0], revision_id)
+
+        # combine the graphs
+        result = {}
+        pending = set([revision_id])
+        def find_parents(node_id):
+            """find the parents for node_id."""
+            for graph in graphs:
+                ancestors = graph.get_ancestors()
+                try:
+                    return ancestors[node_id]
+                except KeyError:
+                    pass
+            raise errors.NoSuchRevision(self._revision_sources[0], node_id)
+        while len(pending):
+            # all the graphs should have identical parent lists
+            node_id = pending.pop()
+            try:
+                result[node_id] = find_parents(node_id)
+                for parent_node in result[node_id]:
+                    if not parent_node in result:
+                        pending.add(parent_node)
+            except errors.NoSuchRevision:
+                # ghost, ignore it.
+                pass
+        return result
+
+    def get_revision_graph_with_ghosts(self, revision_ids):
+        # query all the sources for their entire graphs 
+        # and then build a combined graph for just
+        # revision_ids.
+        graphs = []
+        for source in self._revision_sources:
+            ghost_graph = source.get_revision_graph_with_ghosts()
+            graphs.append(ghost_graph.get_ancestors())
+        for revision_id in revision_ids:
+            absent = 0
+            for graph in graphs:
+                    if not revision_id in graph:
+                        absent += 1
+            if absent == len(graphs):
+                raise errors.NoSuchRevision(self._revision_sources[0],
+                                            revision_id)
+
+        # combine the graphs
+        result = Graph()
+        pending = set(revision_ids)
+        done = set()
+        def find_parents(node_id):
+            """find the parents for node_id."""
+            for graph in graphs:
+                try:
+                    return graph[node_id]
+                except KeyError:
+                    pass
+            raise errors.NoSuchRevision(self._revision_sources[0], node_id)
+        while len(pending):
+            # all the graphs should have identical parent lists
+            node_id = pending.pop()
+            try:
+                parents = find_parents(node_id)
+                for parent_node in parents:
+                    # queued or done? 
+                    if (parent_node not in pending and
+                        parent_node not in done):
+                        # no, queue
+                        pending.add(parent_node)
+                result.add_node(node_id, parents)
+                done.add(node_id)
+            except errors.NoSuchRevision:
+                # ghost
+                result.add_ghost(node_id)
+                continue
+        return result
+
+    def lock_read(self):
+        for source in self._revision_sources:
+            source.lock_read()
+
+    def unlock(self):
+        for source in self._revision_sources:
+            source.unlock()
+
+
+@deprecated_method(zero_eight)
+def get_intervening_revisions(ancestor_id, rev_id, rev_source, 
+                              revision_history=None):
+    """Find the longest line of descent from maybe_ancestor to revision.
+    Revision history is followed where possible.
+
+    If ancestor_id == rev_id, list will be empty.
+    Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
+    If ancestor_id is not an ancestor, NotAncestor will be thrown
+    """
+    root, ancestors, descendants = revision_graph(rev_id, rev_source)
+    if len(descendants) == 0:
+        raise NoSuchRevision(rev_source, rev_id)
+    if ancestor_id not in descendants:
+        rev_source.get_revision(ancestor_id)
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    root_descendants = all_descendants(descendants, ancestor_id)
+    root_descendants.add(ancestor_id)
+    if rev_id not in root_descendants:
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    distances = node_distances(descendants, ancestors, ancestor_id,
+                               root_descendants=root_descendants)
+
+    def best_ancestor(rev_id):
+        best = None
+        for anc_id in ancestors[rev_id]:
+            try:
+                distance = distances[anc_id]
+            except KeyError:
+                continue
+            if revision_history is not None and anc_id in revision_history:
+                return anc_id
+            elif best is None or distance > best[1]:
+                best = (anc_id, distance)
+        return best[0]
+
+    next = rev_id
+    path = []
+    while next != ancestor_id:
+        path.append(next)
+        next = best_ancestor(next)
+    path.reverse()
+    return path

=== added file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revisionspec.py	2006-02-20 09:50:27 +0000
@@ -0,0 +1,354 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import datetime
+import re
+from bzrlib.errors import BzrError, NoSuchRevision, NoCommits
+
+_marker = []
+
+class RevisionInfo(object):
+    """The results of applying a revision specification to a branch.
+
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    This class exists mostly to be the return value of a RevisionSpec,
+    so that you can access the member you're interested in (number or id)
+    or treat the result as a tuple.
+    """
+
+    def __init__(self, branch, revno, rev_id=_marker):
+        self.branch = branch
+        self.revno = revno
+        if rev_id is _marker:
+            # allow caller to be lazy
+            if self.revno is None:
+                self.rev_id = None
+            else:
+                self.rev_id = branch.get_rev_id(self.revno)
+        else:
+            self.rev_id = rev_id
+
+    def __nonzero__(self):
+        # first the easy ones...
+        if self.rev_id is None:
+            return False
+        if self.revno is not None:
+            return True
+        # TODO: otherwise, it should depend on how I was built -
+        # if it's in_history(branch), then check revision_history(),
+        # if it's in_store(branch), do the check below
+        return self.branch.repository.has_revision(self.rev_id)
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.repository.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
+
+# classes in this list should have a "prefix" attribute, against which
+# string specs are matched
+SPEC_TYPES = []
+
+class RevisionSpec(object):
+    """A parsed revision specification.
+
+    A revision specification can be an integer, in which case it is
+    assumed to be a revno (though this will translate negative values
+    into positive ones); or it can be a string, in which case it is
+    parsed for something like 'date:' or 'revid:' etc.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
+
+    (Equivalent to the old Branch method get_revision_info())
+    """
+
+    prefix = None
+
+    def __new__(cls, spec, foo=_marker):
+        """Parse a revision specifier.
+        """
+        if spec is None:
+            return object.__new__(RevisionSpec, spec)
+
+        try:
+            spec = int(spec)
+        except ValueError:
+            pass
+
+        if isinstance(spec, int):
+            return object.__new__(RevisionSpec_int, spec)
+        elif isinstance(spec, basestring):
+            for spectype in SPEC_TYPES:
+                if spec.startswith(spectype.prefix):
+                    return object.__new__(spectype, spec)
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+    def __init__(self, spec):
+        if self.prefix and spec.startswith(self.prefix):
+            spec = spec[len(self.prefix):]
+        self.spec = spec
+
+    def _match_on(self, branch, revs):
+        return RevisionInfo(branch, 0, None)
+
+    def _match_on_and_check(self, branch, revs):
+        info = self._match_on(branch, revs)
+        if info:
+            return info
+        elif info == (0, None):
+            # special case - the empty tree
+            return info
+        elif self.prefix:
+            raise NoSuchRevision(branch, self.prefix + str(self.spec))
+        else:
+            raise NoSuchRevision(branch, str(self.spec))
+
+    def in_history(self, branch):
+        revs = branch.revision_history()
+        return self._match_on_and_check(branch, revs)
+
+        # FIXME: in_history is somewhat broken,
+        # it will return non-history revisions in many
+        # circumstances. The expected facility is that
+        # in_history only returns revision-history revs,
+        # in_store returns any rev. RBC 20051010
+    # aliases for now, when we fix the core logic, then they
+    # will do what you expect.
+    in_store = in_history
+    in_branch = in_store
+        
+    def __repr__(self):
+        # this is mostly for helping with testing
+        return '<%s %s%s>' % (self.__class__.__name__,
+                              self.prefix or '',
+                              self.spec)
+
+
+# private API
+
+class RevisionSpec_int(RevisionSpec):
+    """Spec is a number.  Special case."""
+    def __init__(self, spec):
+        self.spec = int(spec)
+
+    def _match_on(self, branch, revs):
+        if self.spec < 0:
+            revno = len(revs) + self.spec + 1
+        else:
+            revno = self.spec
+        rev_id = branch.get_rev_id(revno, revs)
+        return RevisionInfo(branch, revno, rev_id)
+
+
+class RevisionSpec_revno(RevisionSpec):
+    prefix = 'revno:'
+
+    def _match_on(self, branch, revs):
+        """Lookup a revision by revision number"""
+        try:
+            return RevisionInfo(branch, int(self.spec))
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revno)
+
+
+class RevisionSpec_revid(RevisionSpec):
+    prefix = 'revid:'
+
+    def _match_on(self, branch, revs):
+        try:
+            return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None, self.spec)
+
+SPEC_TYPES.append(RevisionSpec_revid)
+
+
+class RevisionSpec_last(RevisionSpec):
+
+    prefix = 'last:'
+
+    def _match_on(self, branch, revs):
+        try:
+            offset = int(self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+        else:
+            if offset <= 0:
+                raise BzrError('You must supply a positive value for --revision last:XXX')
+            return RevisionInfo(branch, len(revs) - offset + 1)
+
+SPEC_TYPES.append(RevisionSpec_last)
+
+
+class RevisionSpec_before(RevisionSpec):
+
+    prefix = 'before:'
+    
+    def _match_on(self, branch, revs):
+        r = RevisionSpec(self.spec)._match_on(branch, revs)
+        if (r.revno is None) or (r.revno == 0):
+            return r
+        return RevisionInfo(branch, r.revno - 1)
+
+SPEC_TYPES.append(RevisionSpec_before)
+
+
+class RevisionSpec_tag(RevisionSpec):
+    prefix = 'tag:'
+
+    def _match_on(self, branch, revs):
+        raise BzrError('tag: namespace registered, but not implemented.')
+
+SPEC_TYPES.append(RevisionSpec_tag)
+
+
+class RevisionSpec_date(RevisionSpec):
+    prefix = 'date:'
+    _date_re = re.compile(
+            r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+            r'(,|T)?\s*'
+            r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+        )
+
+    def _match_on(self, branch, revs):
+        """
+        Spec for date revisions:
+          date:value
+          value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+          matches the first entry after a given date (either at midnight or
+          at a specified time).
+
+          So the proper way of saying 'give me all entries for today' is:
+              -r date:today..date:tomorrow
+        """
+        today = datetime.datetime.fromordinal(datetime.date.today().toordinal())
+        if self.spec.lower() == 'yesterday':
+            dt = today - datetime.timedelta(days=1)
+        elif self.spec.lower() == 'today':
+            dt = today
+        elif self.spec.lower() == 'tomorrow':
+            dt = today + datetime.timedelta(days=1)
+        else:
+            m = self._date_re.match(self.spec)
+            if not m or (not m.group('date') and not m.group('time')):
+                raise BzrError('Invalid revision date %r' % self.spec)
+
+            if m.group('date'):
+                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
+            else:
+                year, month, day = today.year, today.month, today.day
+            if m.group('time'):
+                hour = int(m.group('hour'))
+                minute = int(m.group('minute'))
+                if m.group('second'):
+                    second = int(m.group('second'))
+                else:
+                    second = 0
+            else:
+                hour, minute, second = 0,0,0
+
+            dt = datetime.datetime(year=year, month=month, day=day,
+                    hour=hour, minute=minute, second=second)
+        first = dt
+        for i in range(len(revs)):
+            r = branch.repository.get_revision(revs[i])
+            # TODO: Handle timezone.
+            dt = datetime.datetime.fromtimestamp(r.timestamp)
+            if first <= dt:
+                return RevisionInfo(branch, i+1)
+        return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_date)
+
+
+class RevisionSpec_ancestor(RevisionSpec):
+    prefix = 'ancestor:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from revision import common_ancestor, MultipleRevisionSources
+        other_branch = Branch.open_containing(self.spec)[0]
+        revision_a = branch.last_revision()
+        revision_b = other_branch.last_revision()
+        for r, b in ((revision_a, branch), (revision_b, other_branch)):
+            if r is None:
+                raise NoCommits(b)
+        revision_source = MultipleRevisionSources(branch.repository,
+                                                  other_branch.repository)
+        rev_id = common_ancestor(revision_a, revision_b, revision_source)
+        try:
+            revno = branch.revision_id_to_revno(rev_id)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, rev_id)
+        
+SPEC_TYPES.append(RevisionSpec_ancestor)
+
+class RevisionSpec_branch(RevisionSpec):
+    """A branch: revision specifier.
+
+    This takes the path to a branch and returns its tip revision id.
+    """
+    prefix = 'branch:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        other_branch = Branch.open_containing(self.spec)[0]
+        revision_b = other_branch.last_revision()
+        if revision_b is None:
+            raise NoCommits(other_branch)
+        # pull in the remote revisions so we can diff
+        branch.fetch(other_branch, revision_b)
+        try:
+            revno = branch.revision_id_to_revno(revision_b)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, revision_b)
+        
+SPEC_TYPES.append(RevisionSpec_branch)

=== added file 'bzrlib/rio.py'
--- bzrlib/rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/rio.py	2006-03-09 18:47:40 +0000
@@ -0,0 +1,257 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# Distributed under the GNU General Public Licence v2
+
+# \subsection{\emph{rio} - simple text metaformat}
+# 
+# \emph{r} stands for `restricted', `reproducible', or `rfc822-like'.
+# 
+# The stored data consists of a series of \emph{stanzas}, each of which contains
+# \emph{fields} identified by an ascii name, with Unicode or string contents.
+# The field tag is constrained to alphanumeric characters.  
+# There may be more than one field in a stanza with the same name.
+# 
+# The format itself does not deal with character encoding issues, though
+# the result will normally be written in Unicode.
+# 
+# The format is intended to be simple enough that there is exactly one character
+# stream representation of an object and vice versa, and that this relation
+# will continue to hold for future versions of bzr.
+
+import re
+
+from bzrlib.iterablefile import IterableFile
+
+# XXX: some redundancy is allowing to write stanzas in isolation as well as
+# through a writer object.  
+
+class RioWriter(object):
+    def __init__(self, to_file):
+        self._soft_nl = False
+        self._to_file = to_file
+
+    def write_stanza(self, stanza):
+        if self._soft_nl:
+            print >>self._to_file
+        stanza.write(self._to_file)
+        self._soft_nl = True
+
+
+class RioReader(object):
+    """Read stanzas from a file as a sequence
+    
+    to_file can be anything that can be enumerated as a sequence of 
+    lines (with newlines.)
+    """
+    def __init__(self, from_file):
+        self._from_file = from_file
+
+    def __iter__(self):
+        while True:
+            s = read_stanza(self._from_file)
+            if s is None:
+                break
+            else:
+                yield s
+
+
+def rio_file(stanzas, header=None):
+    """Produce a rio IterableFile from an iterable of stanzas"""
+    def str_iter():
+        if header is not None:
+            yield header + '\n'
+        first_stanza = True
+        for s in stanzas:
+            if first_stanza is not True:
+                yield '\n'
+            for line in s.to_lines():
+                yield line
+            first_stanza = False
+    return IterableFile(str_iter())
+
+
+def read_stanzas(from_file):
+    while True:
+        s = read_stanza(from_file)
+        if s is None:
+            break
+        else:
+            yield s
+
+class Stanza(object):
+    """One stanza for rio.
+
+    Each stanza contains a set of named fields.  
+    
+    Names must be non-empty ascii alphanumeric plus _.  Names can be repeated
+    within a stanza.  Names are case-sensitive.  The ordering of fields is
+    preserved.
+
+    Each field value must be either an int or a string.
+    """
+
+    __slots__ = ['items']
+
+    def __init__(self, **kwargs):
+        """Construct a new Stanza.
+
+        The keyword arguments, if any, are added in sorted order to the stanza.
+        """
+        self.items = []
+        if kwargs:
+            for tag, value in sorted(kwargs.items()):
+                self.add(tag, value)
+
+    def add(self, tag, value):
+        """Append a name and value to the stanza."""
+        assert valid_tag(tag), \
+            ("invalid tag %r" % tag)
+        if isinstance(value, str):
+            value = unicode(value)
+        elif isinstance(value, unicode):
+            pass
+        ## elif isinstance(value, (int, long)):
+        ##    value = str(value)           # XXX: python2.4 without L-suffix
+        else:
+            raise TypeError("invalid type for rio value: %r of type %s"
+                            % (value, type(value)))
+        self.items.append((tag, value))
+        
+    def __contains__(self, find_tag):
+        """True if there is any field in this stanza with the given tag."""
+        for tag, value in self.items:
+            if tag == find_tag:
+                return True
+        return False
+
+    def __len__(self):
+        """Return number of pairs in the stanza."""
+        return len(self.items)
+
+    def __eq__(self, other):
+        if not isinstance(other, Stanza):
+            return False
+        return self.items == other.items
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __repr__(self):
+        return "Stanza(%r)" % self.items
+
+    def iter_pairs(self):
+        """Return iterator of tag, value pairs."""
+        return iter(self.items)
+
+    def to_lines(self):
+        """Generate sequence of lines for external version of this file.
+        
+        The lines are always utf-8 encoded strings.
+        """
+        if not self.items:
+            # max() complains if sequence is empty
+            return []
+        result = []
+        for tag, value in self.items:
+            assert isinstance(tag, str), type(tag)
+            assert isinstance(value, unicode)
+            if value == '':
+                result.append(tag + ': \n')
+            elif '\n' in value:
+                # don't want splitlines behaviour on empty lines
+                val_lines = value.split('\n')
+                result.append(tag + ': ' + val_lines[0].encode('utf-8') + '\n')
+                for line in val_lines[1:]:
+                    result.append('\t' + line.encode('utf-8') + '\n')
+            else:
+                result.append(tag + ': ' + value.encode('utf-8') + '\n')
+        return result
+
+    def to_string(self):
+        """Return stanza as a single string"""
+        return ''.join(self.to_lines())
+
+    def write(self, to_file):
+        """Write stanza to a file"""
+        to_file.writelines(self.to_lines())
+
+    def get(self, tag):
+        """Return the value for a field wih given tag.
+
+        If there is more than one value, only the first is returned.  If the
+        tag is not present, KeyError is raised.
+        """
+        for t, v in self.items:
+            if t == tag:
+                return v
+        else:
+            raise KeyError(tag)
+
+    __getitem__ = get
+
+    def get_all(self, tag):
+        r = []
+        for t, v in self.items:
+            if t == tag:
+                r.append(v)
+        return r
+
+    def as_dict(self):
+        """Return a dict containing the unique values of the stanza.
+        """
+        d = {}
+        for tag, value in self.items:
+            assert tag not in d
+            d[tag] = value
+        return d
+         
+_tag_re = re.compile(r'^[-a-zA-Z0-9_]+$')
+def valid_tag(tag):
+    return bool(_tag_re.match(tag))
+
+
+def read_stanza(line_iter):
+    """Return new Stanza read from list of lines or a file
+    
+    Returns one Stanza that was read, or returns None at end of file.  If a
+    blank line follows the stanza, it is consumed.  It's not an error for
+    there to be no blank at end of file.  If there is a blank file at the
+    start of the input this is really an empty stanza and that is returned. 
+
+    Only the stanza lines and the trailing blank (if any) are consumed
+    from the line_iter.
+
+    The raw lines must be in utf-8 encoding.
+    """
+    items = []
+    stanza = Stanza()
+    tag = None
+    accum_value = None
+    for line in line_iter:
+        if line == None or line == '':
+            break       # end of file
+        if line == '\n':
+            break       # end of stanza
+        line = line.decode('utf-8')
+        assert line[-1] == '\n'
+        real_l = line
+        if line[0] == '\t': # continues previous value
+            if tag is None:
+                raise ValueError('invalid continuation line %r' % real_l)
+            accum_value += '\n' + line[1:-1]
+        else: # new tag:value line
+            if tag is not None:
+                stanza.add(tag, accum_value)
+            try:
+                colon_index = line.index(': ')
+            except ValueError:
+                raise ValueError('tag/value separator not found in line %r' % real_l)
+            tag = str(line[:colon_index])
+            assert valid_tag(tag), \
+                    "invalid rio tag %r" % tag
+            accum_value = line[colon_index+2:-1]
+    if tag is not None: # add last tag-value
+        stanza.add(tag, accum_value)
+        return stanza
+    else:     # didn't see any content
+        return None    

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-08-22 16:31:16 +0000
+++ bzrlib/shellcomplete.py	2005-11-18 08:10:07 +0000
@@ -17,28 +17,25 @@
 
     from inspect import getdoc
     import commands
-    context, cmdclass = commands.get_cmd_class(cmdname)
+    cmdobj = commands.get_cmd_object(cmdname)
 
-    doc = getdoc(cmdclass)
+    doc = getdoc(cmdobj)
     if doc == None:
         raise NotImplementedError("sorry, no detailed shellcomplete yet for %r" % cmdname)
 
-    shellcomplete_on_option(cmdclass.takes_options, outfile = None)
-    for aname in cmdclass.takes_args:
-	    outfile.write(aname + '\n')
-
-
-def shellcomplete_on_option(options, outfile = None):
-    import commands
-    
+    shellcomplete_on_option(cmdobj.takes_options, outfile = None)
+    for aname in cmdobj.takes_args:
+        outfile.write(aname + '\n')
+
+
+def shellcomplete_on_option(options, outfile=None):
+    from bzrlib.option import Option
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     for on in options:
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in Option.SHORT_OPTIONS.items():
             if longname == on:
                 l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
                 break

=== added file 'bzrlib/sign_my_commits.py'
--- bzrlib/sign_my_commits.py	1970-01-01 00:00:00 +0000
+++ bzrlib/sign_my_commits.py	2006-03-06 09:38:37 +0000
@@ -0,0 +1,78 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Command which looks for unsigned commits by the current user, and signs them.
+"""
+
+from bzrlib.commands import Command
+import bzrlib.config
+import bzrlib.errors as errors
+import bzrlib.gpg
+from bzrlib.option import Option
+
+
+class cmd_sign_my_commits(Command):
+    """Sign all commits by a given committer.
+
+    If location is not specified the local tree is used.
+    If committer is not specified the default committer is used.
+
+    This does not sign commits that already have signatures.
+    """
+
+    takes_options = [Option('dry-run'
+                            , help='Don\'t actually sign anything, just print'
+                                   ' the revisions that would be signed')
+                    ]
+    takes_args = ['location?', 'committer?']
+
+    def run(self, location=None, committer=None, dry_run=False):
+        if location is None:
+            from bzrlib.workingtree import WorkingTree
+            # Open the containing directory
+            wt = WorkingTree.open_containing('.')[0]
+            b = wt.branch
+        else:
+            # Passed in locations should be exact
+            from bzrlib.branch import Branch
+            b = Branch.open(location)
+        repo = getattr(b, 'repository', b)
+
+        config = bzrlib.config.BranchConfig(b)
+
+        if committer is None:
+            committer = config.username()
+
+        gpg_strategy = bzrlib.gpg.GPGStrategy(config)
+
+        count = 0
+        # return in partial topological order for the sake of reproducibility
+        for rev_id in repo.all_revision_ids():
+            if repo.has_signature_for_revision_id(rev_id):
+                continue
+            
+            rev = repo.get_revision(rev_id)
+            if rev.committer != committer:
+                continue
+
+            # We have a revision without a signature who has a 
+            # matching committer, start signing
+            print rev_id
+            count += 1
+            if not dry_run:
+                repo.sign_revision(rev_id, gpg_strategy)
+        print 'Signed %d revisions' % (count,)
+
+

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-07-29 22:21:25 +0000
+++ bzrlib/status.py	2006-04-05 20:37:37 +0000
@@ -1,67 +1,186 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-
+import sys
+
+from bzrlib.delta import compare_trees
+from bzrlib.errors import NoSuchRevision
+from bzrlib.log import line_log
+from bzrlib.osutils import is_inside_any
+from bzrlib.symbol_versioning import *
+
+# TODO: when showing single-line logs, truncate to the width of the terminal
+# if known, but only if really going to the terminal (not into a file)
+
+
+@deprecated_function(zero_eight)
 def show_status(branch, show_unchanged=False,
                 specific_files=None,
                 show_ids=False,
-                to_file=None):
-    """Display single-line status for non-ignored working files.
-
-    show_all
-        If true, show unmodified files too.
-
-    specific_files
-        If set, only show the status of files in this list.
-
-    to_file
-        If set, write to this file (default stdout.)
-    """
-    import sys
-    from bzrlib.delta import compare_trees
-
+                to_file=None,
+                show_pending=True,
+                revision=None):
+    """Display summary of changes.
+
+    Please use show_tree_status instead.
+
+    By default this compares the working tree to a previous revision. 
+    If the revision argument is given, summarizes changes between the 
+    working tree and another, or between two revisions.
+
+    The result is written out as Unicode and to_file should be able 
+    to encode that.
+
+    show_unchanged
+        If set, includes unchanged files.
+
+    specific_files
+        If set, only show the status of files in this list.
+
+    show_ids
+        If set, includes each file's id.
+
+    to_file
+        If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
+
+    revision
+        If None the compare latest revision with working tree
+        If one revision show compared it with working tree.
+        If two revisions show status between first and second.
+    """
+    show_tree_status(branch.bzrdir.open_workingtree(), show_unchanged, 
+                     specific_files, show_ids, to_file, show_pending, revision)
+
+
+def show_tree_status(wt, show_unchanged=False,
+                     specific_files=None,
+                     show_ids=False,
+                     to_file=None,
+                     show_pending=True,
+                     revision=None):
+    """Display summary of changes.
+
+    By default this compares the working tree to a previous revision. 
+    If the revision argument is given, summarizes changes between the 
+    working tree and another, or between two revisions.
+
+    The result is written out as Unicode and to_file should be able 
+    to encode that.
+
+    show_unchanged
+        If set, includes unchanged files.
+
+    specific_files
+        If set, only show the status of files in this list.
+
+    show_ids
+        If set, includes each file's id.
+
+    to_file
+        If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
+
+    revision
+        If None the compare latest revision with working tree
+        If one revision show compared it with working tree.
+        If two revisions show status between first and second.
+    """
     if to_file == None:
         to_file = sys.stdout
     
-    branch.lock_read()
+    wt.lock_read()
     try:
-
-        old = branch.basis_tree()
-        new = branch.working_tree()
-
+        new_is_working_tree = True
+        if revision is None:
+            new = wt
+            old = new.basis_tree()
+        elif len(revision) > 0:
+            try:
+                rev_id = revision[0].in_history(wt.branch).rev_id
+                old = wt.branch.repository.revision_tree(rev_id)
+            except NoSuchRevision, e:
+                raise BzrCommandError(str(e))
+            if (len(revision) > 1) and (revision[1].spec is not None):
+                try:
+                    rev_id = revision[1].in_history(wt.branch).rev_id
+                    new = wt.branch.repository.revision_tree(rev_id)
+                    new_is_working_tree = False
+                except NoSuchRevision, e:
+                    raise BzrCommandError(str(e))
+            else:
+                new = wt
+                
         delta = compare_trees(old, new, want_unchanged=show_unchanged,
                               specific_files=specific_files)
-
         delta.show(to_file,
                    show_ids=show_ids,
                    show_unchanged=show_unchanged)
 
-        unknowns = new.unknowns()
-        done_header = False
-        for path in unknowns:
-            # FIXME: Should also match if the unknown file is within a
-            # specified directory.
-            if specific_files:
-                if path not in specific_files:
+        if new_is_working_tree:
+            list_paths('unknown', new.unknowns(), specific_files, to_file)
+            conflict_title = False
+            for conflict in wt.conflicts():
+                if conflict_title is False:
+                    print >> to_file, "conflicts:"
+                    conflict_title = True
+                print >> to_file, "  %s" % conflict
+        if new_is_working_tree and show_pending:
+            show_pending_merges(new, to_file)
+    finally:
+        wt.unlock()
+
+def show_pending_merges(new, to_file):
+    """Write out a display of pending merges in a working tree."""
+    pending = new.pending_merges()
+    branch = new.branch
+    if len(pending) == 0:
+        return
+    print >>to_file, 'pending merges:'
+    last_revision = branch.last_revision()
+    if last_revision is not None:
+        ignore = set(branch.repository.get_ancestry(last_revision))
+    else:
+        ignore = set()
+    for merge in new.pending_merges():
+        ignore.add(merge)
+        try:
+            m_revision = branch.repository.get_revision(merge)
+            print >> to_file, ' ', line_log(m_revision, 77)
+            inner_merges = branch.repository.get_ancestry(merge)
+            inner_merges.reverse()
+            for mmerge in inner_merges:
+                if mmerge in ignore:
                     continue
-            if not done_header:
-                print >>to_file, 'unknown:'
-                done_header = True
-            print >>to_file, ' ', path
-    finally:
-        branch.unlock()
+                mm_revision = branch.repository.get_revision(mmerge)
+                print >> to_file, '   ', line_log(mm_revision, 75)
+                ignore.add(mmerge)
+        except NoSuchRevision:
+            print >> to_file, ' ', merge 
         
+def list_paths(header, paths, specific_files, to_file):
+    done_header = False
+    for path in paths:
+        if specific_files and not is_inside_any(specific_files, path):
+            continue
+        if not done_header:
+            print >>to_file, '%s:' % header
+            done_header = True
+        print >>to_file, ' ', path

=== added directory 'bzrlib/store'
=== renamed file 'bzrlib/store.py' => 'bzrlib/store/__init__.py'
--- bzrlib/store.py	2005-08-02 23:14:41 +0000
+++ bzrlib/store/__init__.py	2006-04-12 07:02:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Development Ltd
+# Copyright (C) 2005, 2006 by Canonical Development Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,6 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Could remember a bias towards whether a particular store is typically
+# compressed or not.
+
 """
 Stores are the main data-storage mechanism for Bazaar-NG.
 
@@ -21,10 +24,18 @@
 unique ID.
 """
 
-import os, tempfile, types, osutils, gzip, errno
-from stat import ST_SIZE
-from StringIO import StringIO
-from trace import mutter
+import os
+from cStringIO import StringIO
+import urllib
+from zlib import adler32
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter
+from bzrlib.transport import Transport, urlescape
+from bzrlib.transport.local import LocalTransport
 
 ######################################################################
 # stores
@@ -33,163 +44,309 @@
     pass
 
 
-class ImmutableStore(object):
-    """Store that holds files indexed by unique names.
-
+class Store(object):
+    """This class represents the abstract storage layout for saving information.
+    
     Files can be added, but not modified once they are in.  Typically
     the hash is used as the name, or something else known to be unique,
     such as a UUID.
-
-    >>> st = ImmutableScratchStore()
-
-    >>> st.add(StringIO('hello'), 'aa')
-    >>> 'aa' in st
-    True
-    >>> 'foo' in st
-    False
-
-    You are not allowed to add an id that is already present.
-
-    Entries can be retrieved as files, which may then be read.
-
-    >>> st.add(StringIO('goodbye'), '123123')
-    >>> st['123123'].read()
-    'goodbye'
-
-    TODO: Atomic add by writing to a temporary file and renaming.
-
-    In bzr 0.0.5 and earlier, files within the store were marked
-    readonly on disk.  This is no longer done but existing stores need
-    to be accomodated.
     """
 
-    def __init__(self, basedir):
-        self._basedir = basedir
-
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
-
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self._basedir)
-
-    def add(self, f, fileid, compressed=True):
-        """Add contents of a file into the store.
-
-        f -- An open file, or file-like object."""
-        # FIXME: Only works on files that will fit in memory
-        
-        from bzrlib.atomicfile import AtomicFile
-        
-        mutter("add store entry %r" % (fileid))
-        if isinstance(f, types.StringTypes):
-            content = f
-        else:
-            content = f.read()
-            
-        p = self._path(fileid)
-        if os.access(p, os.F_OK) or os.access(p + '.gz', os.F_OK):
-            from bzrlib.errors import bailout
-            raise BzrError("store %r already contains id %r" % (self._basedir, fileid))
-
-        fn = p
-        if compressed:
-            fn = fn + '.gz'
-            
-        af = AtomicFile(fn, 'wb')
-        try:
-            if compressed:
-                gf = gzip.GzipFile(mode='wb', fileobj=af)
-                gf.write(content)
-                gf.close()
-            else:
-                af.write(content)
-            af.commit()
-        finally:
-            af.close()
-
-
-    def copy_multi(self, other, ids):
+    def __len__(self):
+        raise NotImplementedError('Children should define their length')
+
+    def get(self, fileid, suffix=None):
+        """Returns a file reading from a particular entry.
+        
+        If suffix is present, retrieve the named suffix for fileid.
+        """
+        raise NotImplementedError
+
+    def __getitem__(self, fileid):
+        """DEPRECATED. Please use .get(fileid) instead."""
+        raise NotImplementedError
+
+    #def __contains__(self, fileid):
+    #    """Deprecated, please use has_id"""
+    #    raise NotImplementedError
+
+    def __iter__(self):
+        raise NotImplementedError
+
+    def add(self, f, fileid):
+        """Add a file object f to the store accessible from the given fileid"""
+        raise NotImplementedError('Children of Store must define their method of adding entries.')
+
+    def has_id(self, fileid, suffix=None):
+        """Return True or false for the presence of fileid in the store.
+        
+        suffix, if present, is a per file suffix, i.e. for digital signature 
+        data."""
+        raise NotImplementedError
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return hasattr(self, "__iter__")
+
+    def copy_all_ids(self, store_from, pb=None):
+        """Copy all the file ids from store_from into self."""
+        if not store_from.listable():
+            raise UnlistableStore(store_from)
+        ids = []
+        for count, file_id in enumerate(store_from):
+            if pb:
+                pb.update('listing files', count, count)
+            ids.append(file_id)
+        if pb:
+            pb.clear()
+        mutter('copy_all ids: %r', ids)
+        self.copy_multi(store_from, ids, pb=pb)
+
+    def copy_multi(self, other, ids, pb=None, permit_failure=False):
         """Copy texts for ids from other into self.
 
         If an id is present in self, it is skipped.  A count of copied
         ids is returned, which may be less than len(ids).
+
+        :param other: Another Store object
+        :param ids: A list of entry ids to be copied
+        :param pb: A ProgressBar object, if none is given, the default will be created.
+        :param permit_failure: Allow missing entries to be ignored
+        :return: (n_copied, [failed]) The number of entries copied successfully,
+            followed by a list of entries which could not be copied (because they
+            were missing)
         """
-        from bzrlib.progress import ProgressBar
-        pb = ProgressBar()
-        pb.update('preparing to copy')
-        to_copy = [id for id in ids if id not in self]
-        if isinstance(other, ImmutableStore):
-            return self.copy_multi_immutable(other, to_copy, pb)
+        if pb:
+            pb.update('preparing to copy')
+        failed = set()
         count = 0
-        for id in to_copy:
+        ids = list(ids) # get the list for showing a length.
+        for fileid in ids:
             count += 1
-            pb.update('copy', count, len(to_copy))
-            self.add(other[id], id)
-        assert count == len(to_copy)
-        pb.clear()
-        return count
-
-
-    def copy_multi_immutable(self, other, to_copy, pb):
-        from shutil import copyfile
-        count = 0
-        for id in to_copy:
-            p = self._path(id)
-            other_p = other._path(id)
+            if self.has_id(fileid):
+                continue
             try:
-                copyfile(other_p, p)
-            except IOError, e:
-                if e.errno == errno.ENOENT:
-                    copyfile(other_p+".gz", p+".gz")
+                self._copy_one(fileid, None, other, pb)
+                for suffix in self._suffixes:
+                    try:
+                        self._copy_one(fileid, suffix, other, pb)
+                    except KeyError:
+                        pass
+                if pb:
+                    pb.update('copy', count, len(ids))
+            except KeyError:
+                if permit_failure:
+                    failed.add(fileid)
                 else:
                     raise
-            
-            count += 1
-            pb.update('copy', count, len(to_copy))
-        assert count == len(to_copy)
-        pb.clear()
-        return count
-    
-
-    def __contains__(self, fileid):
-        """"""
-        p = self._path(fileid)
-        return (os.access(p, os.R_OK)
-                or os.access(p + '.gz', os.R_OK))
-
-    # TODO: Guard against the same thing being stored twice, compressed and uncompresse
+        assert count == len(ids)
+        if pb:
+            pb.clear()
+        return count, failed
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        """Most generic copy-one object routine.
+        
+        Subclasses can override this to provide an optimised
+        copy between their own instances. Such overriden routines
+        should call this if they have no optimised facility for a 
+        specific 'other'.
+        """
+        mutter('Store._copy_one: %r', fileid)
+        f = other.get(fileid, suffix)
+        self.add(f, fileid, suffix)
+
+
+class TransportStore(Store):
+    """A TransportStore is a Store superclass for Stores that use Transports."""
+
+    def add(self, f, fileid, suffix=None):
+        """Add contents of a file into the store.
+
+        f -- A file-like object, or string
+        """
+        mutter("add store entry %r", fileid)
+        
+        names = self._id_to_names(fileid, suffix)
+        if self._transport.has_any(names):
+            raise BzrError("store %r already contains id %r" 
+                           % (self._transport.base, fileid))
+
+        # Most of the time, just adding the file will work
+        # if we find a time where it fails, (because the dir
+        # doesn't exist), then create the dir, and try again
+        self._add(names[0], f)
+
+
+    def _add(self, relpath, f):
+        """Actually add the file to the given location.
+        This should be overridden by children.
+        """
+        raise NotImplementedError('children need to implement this function.')
+
+    def _check_fileid(self, fileid):
+        if not isinstance(fileid, basestring):
+            raise TypeError('Fileids should be a string type: %s %r' % (type(fileid), fileid))
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+
+    def _id_to_names(self, fileid, suffix):
+        """Return the names in the expected order"""
+        if suffix is not None:
+            fn = self._relpath(fileid, [suffix])
+        else:
+            fn = self._relpath(fileid)
+
+        # FIXME RBC 20051128 this belongs in TextStore.
+        fn_gz = fn + '.gz'
+        if self._compressed:
+            return fn_gz, fn
+        else:
+            return fn, fn_gz
+
+    def has_id(self, fileid, suffix=None):
+        """See Store.has_id."""
+        return self._transport.has_any(self._id_to_names(fileid, suffix))
+
+    def _get_name(self, fileid, suffix=None):
+        """A special check, which returns the name of an existing file.
+        
+        This is similar in spirit to 'has_id', but it is designed
+        to return information about which file the store has.
+        """
+        for name in self._id_to_names(fileid, suffix=suffix):
+            if self._transport.has(name):
+                return name
+        return None
+
+    def _get(self, filename):
+        """Return an vanilla file stream for clients to read from.
+
+        This is the body of a template method on 'get', and should be 
+        implemented by subclasses.
+        """
+        raise NotImplementedError
+
+    def get(self, fileid, suffix=None):
+        """See Store.get()."""
+        names = self._id_to_names(fileid, suffix)
+        for name in names:
+            try:
+                return self._get(name)
+            except errors.NoSuchFile:
+                pass
+        raise KeyError(fileid)
+
+    def __init__(self, a_transport, prefixed=False, compressed=False,
+                 dir_mode=None, file_mode=None,
+                 escaped=False):
+        assert isinstance(a_transport, Transport)
+        super(TransportStore, self).__init__()
+        self._transport = a_transport
+        self._prefixed = prefixed
+        # FIXME RBC 20051128 this belongs in TextStore.
+        self._compressed = compressed
+        self._suffixes = set()
+        self._escaped = escaped
+
+        # It is okay for these to be None, it just means they
+        # will just use the filesystem defaults
+        self._dir_mode = dir_mode
+        self._file_mode = file_mode
+
+    def _unescape(self, file_id):
+        """If filename escaping is enabled for this store, unescape and return the filename."""
+        if self._escaped:
+            return urllib.unquote(file_id)
+        else:
+            return file_id
+
+    def _iter_files_recursive(self):
+        """Iterate through the files in the transport."""
+        for quoted_relpath in self._transport.iter_files_recursive():
+            # transport iterator always returns quoted paths, regardless of
+            # escaping
+            yield urllib.unquote(quoted_relpath)
 
     def __iter__(self):
-        for f in os.listdir(self._basedir):
-            if f[-3:] == '.gz':
-                # TODO: case-insensitive?
-                yield f[:-3]
-            else:
-                yield f
+        for relpath in self._iter_files_recursive():
+            # worst case is one of each suffix.
+            name = os.path.basename(relpath)
+            if name.endswith('.gz'):
+                name = name[:-3]
+            skip = False
+            for count in range(len(self._suffixes)):
+                for suffix in self._suffixes:
+                    if name.endswith('.' + suffix):
+                        skip = True
+            if not skip:
+                yield self._unescape(name)
 
     def __len__(self):
-        return len(os.listdir(self._basedir))
-
-
-    def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
-        p = self._path(fileid)
-        try:
-            return gzip.GzipFile(p + '.gz', 'rb')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-
-        try:
-            return file(p, 'rb')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-
-        raise IndexError(fileid)
-
+        return len(list(self.__iter__()))
+
+    def _relpath(self, fileid, suffixes=None):
+        self._check_fileid(fileid)
+        if suffixes:
+            for suffix in suffixes:
+                if not suffix in self._suffixes:
+                    raise ValueError("Unregistered suffix %r" % suffix)
+                self._check_fileid(suffix)
+        else:
+            suffixes = []
+        if self._prefixed:
+            # hash_prefix adds the '/' separator
+            prefix = self.hash_prefix(fileid)
+        else:
+            prefix = ''
+        fileid = self._escape_file_id(fileid)
+        path = prefix + fileid
+        full_path = u'.'.join([path] + suffixes)
+        return urlescape(full_path)
+
+    def _escape_file_id(self, file_id):
+        """Turn a file id into a filesystem safe string.
+
+        This is similar to a plain urllib.quote, except
+        it uses specific safe characters, so that it doesn't
+        have to translate a lot of valid file ids.
+        """
+        if not self._escaped:
+            return file_id
+        if isinstance(file_id, unicode):
+            file_id = file_id.encode('utf-8')
+        # @ does not get escaped. This is because it is a valid
+        # filesystem character we use all the time, and it looks
+        # a lot better than seeing %40 all the time.
+        safe = "abcdefghijklmnopqrstuvwxyz0123456789-_@,."
+        r = [((c in safe) and c or ('%%%02x' % ord(c)))
+             for c in file_id]
+        return ''.join(r)
+
+    def hash_prefix(self, fileid):
+        # fileid should be unescaped
+        if self._escaped:
+            fileid = self._escape_file_id(fileid)
+        return "%02x/" % (adler32(fileid) & 0xff)
+
+    def __repr__(self):
+        if self._transport is None:
+            return "%s(None)" % (self.__class__.__name__)
+        else:
+            return "%s(%r)" % (self.__class__.__name__, self._transport.base)
+
+    __str__ = __repr__
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return self._transport.listable()
+
+    def register_suffix(self, suffix):
+        """Register a suffix as being expected in this store."""
+        self._check_fileid(suffix)
+        if suffix == 'gz':
+            raise ValueError('You cannot register the "gz" suffix.')
+        self._suffixes.add(suffix)
 
     def total_size(self):
         """Return (count, bytes)
@@ -198,33 +355,14 @@
         the content."""
         total = 0
         count = 0
-        for fid in self:
+        for relpath in self._transport.iter_files_recursive():
             count += 1
-            p = self._path(fid)
-            try:
-                total += os.stat(p)[ST_SIZE]
-            except OSError:
-                total += os.stat(p + '.gz')[ST_SIZE]
+            total += self._transport.stat(relpath).st_size
                 
         return count, total
 
 
-
-
-class ImmutableScratchStore(ImmutableStore):
-    """Self-destructing test subclass of ImmutableStore.
-
-    The Store only exists for the lifetime of the Python object.
- Obviously you should not put anything precious in it.
-    """
-    def __init__(self):
-        ImmutableStore.__init__(self, tempfile.mkdtemp())
-
-    def __del__(self):
-        for f in os.listdir(self._basedir):
-            fpath = os.path.join(self._basedir, f)
-            # needed on windows, and maybe some other filesystems
-            os.chmod(fpath, 0600)
-            os.remove(fpath)
-        os.rmdir(self._basedir)
-        mutter("%r destroyed" % self)
+@deprecated_function(zero_eight)
+def copy_all(store_from, store_to, pb=None):
+    """Copy all ids from one store to another."""
+    store_to.copy_all_ids(store_from, pb)

=== added directory 'bzrlib/store/revision'
=== added file 'bzrlib/store/revision/__init__.py'
--- bzrlib/store/revision/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/revision/__init__.py	2006-03-06 10:17:26 +0000
@@ -0,0 +1,152 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Revision stores.
+
+Revision stores are responsible for storing a group of revisions
+and returning some interesting data about them such as ancestors
+and ghosts information.
+"""
+
+
+from copy import deepcopy
+from cStringIO import StringIO
+from unittest import TestSuite
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.trace import mutter
+
+
+class RevisionStoreTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple repository stores.
+
+    This is done by copying the test once for each repository store
+    and injecting the transport_server, transport_readonly_server,
+    and revision-store-factory into each copy.
+    Each copy is also given a new id() to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, factories):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._factories = factories
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for factory in self._factories:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.store_factory = factory
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), factory)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    @staticmethod
+    def default_test_list():
+        """Generate the default list of revision store permutations to test."""
+        from bzrlib.store.revision.text import TextRevisionStoreTestFactory
+        from bzrlib.store.revision.knit import KnitRevisionStoreFactory
+        result = []
+        # test the fallback InterVersionedFile from weave to annotated knits
+        result.append(TextRevisionStoreTestFactory())
+        result.append(KnitRevisionStoreFactory())
+        return result
+
+
+class RevisionStore(object):
+    """A revision store stores revisions."""
+
+    def __init__(self, serializer=None):
+        if serializer is None:
+            serializer = bzrlib.xml5.serializer_v5
+        self._serializer = serializer
+
+    def add_revision(self, revision, transaction):
+        """Add revision to the revision store.
+
+        :param rev: The revision object.
+        """
+        # serialisation : common to all at the moment.
+        rev_tmp = StringIO()
+        self._serializer.write_revision(revision, rev_tmp)
+        rev_tmp.seek(0)
+        self._add_revision(revision, rev_tmp, transaction)
+        mutter('added revision_id {%s}', revision.revision_id)
+
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        raise NotImplementedError(self._add_revision)
+
+    def add_revision_signature_text(self, revision_id, signature_text, transaction):
+        """Add signature_text as a signature for revision_id."""
+        raise NotImplementedError(self.add_revision_signature_text)
+
+    def all_revision_ids(self, transaction):
+        """Returns a list of all the revision ids in the revision store. 
+
+        :return: list of revision_ids in topological order.
+        """
+        raise NotImplementedError(self.all_revision_ids)
+
+    def get_revision(self, revision_id, transaction):
+        """Return the Revision object for a named revision."""
+        raise NotImplementedError(self.get_revision)
+
+    def get_signature_text(self, revision_id, transaction):
+        """Get the signature text for the digital signature of revision_id.
+        
+        :return: a signature text.
+        """
+        self._guard_revision(revision_id, transaction)
+        return self._get_signature_text(revision_id, transaction)
+
+    def _get_signature_text(self, revision_id, transaction):
+        """Helper method for get_signature_text to return the text itself."""
+        raise NotImplementedError(self.get_signature_text)
+
+    def _guard_revision(self, revision_id, transaction):
+        """Guard method for testing the presence of a revision."""
+        if not self.has_revision_id(revision_id, transaction):
+            raise errors.NoSuchRevision(self, revision_id)
+
+    def has_revision_id(self, revision_id, transaction):
+        """True if the store contains revision_id."""
+        raise NotImplementedError(self.has_revision_id)
+
+    def has_signature(self, revision_id, transaction):
+        """True if the store has a signature for revision_id."""
+        self._guard_revision(revision_id, transaction)
+        return self._has_signature(revision_id, transaction)
+
+    def _has_signature(self, revision_id, transaction):
+        """Return the presence of a signature for revision_id.
+
+        A worker memrthod for has_signature, this can assume the
+        revision is present.
+        """
+        return NotImplementedError(self._has_signature)
+        
+    def total_size(self, transaction):
+        """How big is the store?
+
+        :return: (count, bytes) tuple.
+        """
+        raise NotImplementedError(self.total_size)

=== added file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/revision/knit.py	2006-03-31 08:25:41 +0000
@@ -0,0 +1,124 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""VersionedFile based revision store.
+
+This stores revisions as individual entries in a knit, and signatures in a 
+parallel knit.
+"""
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
+from bzrlib.store.revision import RevisionStore
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.transport import get_transport
+
+
+class KnitRevisionStoreFactory(object):
+    """Factory to create a KnitRevisionStore for testing."""
+
+    def create(self, url):
+        """Create a revision store at url."""
+        t = get_transport(url)
+        t.mkdir('revision-store')
+        versioned_file_store = VersionedFileStore(
+            t.clone('revision-store'),
+            precious=True,
+            versionedfile_class=KnitVersionedFile,
+            versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory()})
+        return KnitRevisionStore(versioned_file_store)
+
+    def __str__(self):
+        return "KnitRevisionStore"
+
+
+class KnitRevisionStore(RevisionStore):
+    """A RevisionStore layering on a VersionedFileStore."""
+
+    def __init__(self, versioned_file_store):
+        """Create a KnitRevisionStore object.
+
+        :param versioned_file_store: the text store to use for storing 
+                                     revisions and signatures.
+        """
+        super(KnitRevisionStore, self).__init__()
+        self.versioned_file_store = versioned_file_store
+
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        # FIXME: make this ghost aware at the knit level
+        rf = self.get_revision_file(transaction)
+        self.get_revision_file(transaction).add_lines_with_ghosts(
+            revision.revision_id,
+            revision.parent_ids,
+            revision_as_file.readlines())
+
+    def add_revision_signature_text(self, revision_id, signature_text, transaction):
+        """See RevisionStore.add_revision_signature_text()."""
+        self.get_signature_file(transaction).add_lines(
+            revision_id, [], bzrlib.osutils.split_lines(signature_text))
+
+    def all_revision_ids(self, transaction):
+        """See RevisionStore.all_revision_ids()."""
+        rev_file = self.get_revision_file(transaction)
+        return rev_file.get_ancestry(rev_file.versions())
+
+    def get_revision(self, revision_id, transaction):
+        """See RevisionStore.get_revision()."""
+        xml = self._get_revision_xml(revision_id, transaction)
+        try:
+            r = self._serializer.read_revision_from_string(xml)
+        except SyntaxError, e:
+            raise errors.BzrError('failed to unpack revision_xml',
+                                   [revision_id,
+                                   str(e)])
+        assert r.revision_id == revision_id
+        return r
+
+    def _get_revision_xml(self, revision_id, transaction):
+        try:
+            return self.get_revision_file(transaction).get_text(revision_id)
+        except (errors.RevisionNotPresent):
+            raise errors.NoSuchRevision(self, revision_id)
+
+    def get_revision_file(self, transaction):
+        """Get the revision versioned file object."""
+        return self.versioned_file_store.get_weave_or_empty('revisions', transaction)
+
+    def get_signature_file(self, transaction):
+        """Get the signature text versioned file object."""
+        return self.versioned_file_store.get_weave_or_empty('signatures', transaction)
+
+    def _get_signature_text(self, revision_id, transaction):
+        """See RevisionStore._get_signature_text()."""
+        try:
+            return self.get_signature_file(transaction).get_text(revision_id)
+        except errors.RevisionNotPresent:
+            raise errors.NoSuchRevision(self, revision_id)
+
+    def has_revision_id(self, revision_id, transaction):
+        """True if the store contains revision_id."""
+        return (revision_id is None
+                or self.get_revision_file(transaction).has_version(revision_id))
+        
+    def _has_signature(self, revision_id, transaction):
+        """See RevisionStore._has_signature()."""
+        return self.get_signature_file(transaction).has_version(revision_id)
+
+    def total_size(self, transaction):
+        """ See RevisionStore.total_size()."""
+        return self.versioned_file_store.total_size()

=== added file 'bzrlib/store/revision/text.py'
--- bzrlib/store/revision/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/revision/text.py	2006-03-08 00:20:04 +0000
@@ -0,0 +1,125 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""TextStore based revision store.
+
+This stores revisions as individual text entries in a TextStore and 
+requires access to a inventory weave to produce object graphs.
+"""
+
+
+from cStringIO import StringIO
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.store.revision import RevisionStore
+from bzrlib.store.text import TextStore
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.transport import get_transport
+from bzrlib.tsort import topo_sort
+
+
+class TextRevisionStoreTestFactory(object):
+    """Factory to create a TextRevisionStore for testing.
+
+    This creates a inventory weave and hooks it into the revision store
+    """
+
+    def create(self, url):
+        """Create a revision store at url."""
+        t = get_transport(url)
+        t.mkdir('revstore')
+        text_store = TextStore(t.clone('revstore'))
+        return TextRevisionStore(text_store)
+
+    def __str__(self):
+        return "TextRevisionStore"
+
+
+class TextRevisionStore(RevisionStore):
+    """A RevisionStore layering on a TextStore and Inventory weave store."""
+
+    def __init__(self, text_store, serializer=None):
+        """Create a TextRevisionStore object.
+
+        :param text_store: the text store to put serialised revisions into.
+        """
+        super(TextRevisionStore, self).__init__(serializer)
+        self.text_store = text_store
+        self.text_store.register_suffix('sig')
+
+    def _add_revision(self, revision, revision_as_file, transaction):
+        """Template method helper to store revision in this store."""
+        self.text_store.add(revision_as_file, revision.revision_id)
+
+    def add_revision_signature_text(self, revision_id, signature_text, transaction):
+        """See RevisionStore.add_revision_signature_text()."""
+        self.text_store.add(StringIO(signature_text), revision_id, "sig")
+
+    def all_revision_ids(self, transaction):
+        """See RevisionStore.all_revision_ids()."""
+        # for TextRevisionStores, this is only functional
+        # on listable transports.
+        assert self.text_store.listable()
+        result_graph = {}
+        for rev_id in self.text_store:
+            rev = self.get_revision(rev_id, transaction)
+            result_graph[rev_id] = rev.parent_ids
+        # remove ghosts
+        for rev_id, parents in result_graph.items():
+            for parent in list(parents):
+                if not parent in result_graph:
+                    del parents[parents.index(parent)]
+        return topo_sort(result_graph.items())
+
+    def get_revision(self, revision_id, transaction):
+        """See RevisionStore.get_revision()."""
+        xml_file = self._get_revision_xml_file(revision_id)
+        try:
+            r = self._serializer.read_revision(xml_file)
+        except SyntaxError, e:
+            raise errors.BzrError('failed to unpack revision_xml',
+                                   [revision_id,
+                                   str(e)])
+            
+        assert r.revision_id == revision_id
+        return r
+
+    def _get_revision_xml_file(self, revision_id):
+        try:
+            return self.text_store.get(revision_id)
+        except (IndexError, KeyError):
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
+    def _get_signature_text(self, revision_id, transaction):
+        """See RevisionStore._get_signature_text()."""
+        try:
+            return self.text_store.get(revision_id, suffix='sig').read()
+        except (IndexError, KeyError):
+            raise errors.NoSuchRevision(self, revision_id)
+
+    def has_revision_id(self, revision_id, transaction):
+        """True if the store contains revision_id."""
+        return (revision_id is None
+                or self.text_store.has_id(revision_id))
+ 
+    def _has_signature(self, revision_id, transaction):
+        """See RevisionStore._has_signature()."""
+        return self.text_store.has_id(revision_id, suffix='sig')
+
+    def total_size(self, transaction):
+        """ See RevisionStore.total_size()."""
+        return self.text_store.total_size()

=== added file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/text.py	2006-04-04 10:20:54 +0000
@@ -0,0 +1,126 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A store that keeps the full text of every version.
+
+This store keeps uncompressed versions of the full text. It does not
+do any sort of delta compression.
+"""
+
+import os
+import bzrlib.store
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError, NoSuchFile, FileExists
+
+import gzip
+from cStringIO import StringIO
+
+
+class TextStore(bzrlib.store.TransportStore):
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    Files are stored uncompressed, with no delta compression.
+    """
+
+    def _add_compressed(self, fn, f):
+        from cStringIO import StringIO
+        from bzrlib.osutils import pumpfile
+        
+        if isinstance(f, basestring):
+            f = StringIO(f)
+            
+        sio = StringIO()
+        gf = gzip.GzipFile(mode='wb', fileobj=sio)
+        # if pumpfile handles files that don't fit in ram,
+        # so will this function
+        pumpfile(f, gf)
+        gf.close()
+        sio.seek(0)
+        self._try_put(fn, sio)
+
+    def _add(self, fn, f):
+        if self._compressed:
+            self._add_compressed(fn, f)
+        else:
+            self._try_put(fn, f)
+
+    def _try_put(self, fn, f):
+        try:
+            self._transport.put(fn, f, mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
+            except FileExists:
+                pass
+            self._transport.put(fn, f, mode=self._file_mode)
+
+    def _get(self, fn):
+        if fn.endswith('.gz'):
+            return self._get_compressed(fn)
+        else:
+            return self._transport.get(fn)
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        # TODO: Once the copy_to interface is improved to allow a source
+        #       and destination targets, then we can always do the copy
+        #       as long as other is a TextStore
+        if not (isinstance(other, TextStore)
+            and other._prefixed == self._prefixed):
+            return super(TextStore, self)._copy_one(fileid, suffix, other, pb)
+
+        mutter('_copy_one: %r, %r', fileid, suffix)
+        path = other._get_name(fileid, suffix)
+        if path is None:
+            raise KeyError(fileid + '-' + str(suffix))
+
+        try:
+            result = other._transport.copy_to([path], self._transport,
+                                              mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(self.hash_prefix(fileid)[:-1], mode=self._dir_mode)
+            except FileExists:
+                pass
+            result = other._transport.copy_to([path], self._transport,
+                                              mode=self._file_mode)
+
+        if result != 1:
+            raise BzrError('Unable to copy file: %r' % (path,))
+
+    def _get_compressed(self, filename):
+        """Returns a file reading from a particular entry."""
+        f = self._transport.get(filename)
+        # gzip.GzipFile.read() requires a tell() function
+        # but some transports return objects that cannot seek
+        # so buffer them in a StringIO instead
+        if hasattr(f, 'tell'):
+            return gzip.GzipFile(mode='rb', fileobj=f)
+        else:
+            from cStringIO import StringIO
+            sio = StringIO(f.read())
+            return gzip.GzipFile(mode='rb', fileobj=sio)
+
+
+def ScratchTextStore():
+    return TextStore(ScratchTransport())

=== added directory 'bzrlib/store/versioned'
=== added file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/versioned/__init__.py	2006-04-17 10:08:27 +0000
@@ -0,0 +1,287 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# XXX: Some consideration of the problems that might occur if there are
+# files whose id differs only in case.  That should probably be forbidden.
+
+
+import errno
+import os
+from cStringIO import StringIO
+import urllib
+
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.weave import WeaveFile, Weave
+from bzrlib.store import TransportStore
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.errors import NoSuchFile, FileExists
+from bzrlib.symbol_versioning import *
+from bzrlib.trace import mutter
+import bzrlib.ui
+
+
+class VersionedFileStore(TransportStore):
+    """Collection of many versioned files in a transport."""
+
+    # TODO: Rather than passing versionedfile_kwargs, perhaps pass in a
+    # transport factory callable?
+    def __init__(self, transport, prefixed=False, precious=False,
+                 dir_mode=None, file_mode=None,
+                 versionedfile_class=WeaveFile,
+                 versionedfile_kwargs={},
+                 escaped=False):
+        super(VersionedFileStore, self).__init__(transport,
+                dir_mode=dir_mode, file_mode=file_mode,
+                prefixed=prefixed, compressed=False, escaped=escaped)
+        self._precious = precious
+        self._versionedfile_class = versionedfile_class
+        self._versionedfile_kwargs = versionedfile_kwargs
+
+    def _clear_cache_id(self, file_id, transaction):
+        """WARNING may lead to inconsistent object references for file_id.
+
+        Remove file_id from the transaction map. 
+
+        NOT in the transaction api because theres no reliable way to clear
+        callers. So its here for very specialised use rather than having an
+        'api' that isn't.
+        """
+        weave = transaction.map.find_weave(file_id)
+        if weave is not None:
+            mutter("old data in transaction in %s for %s", self, file_id)
+            # FIXME abstraction violation - transaction now has stale data.
+            transaction.map.remove_object(weave)
+
+    def filename(self, file_id):
+        """Return the path relative to the transport root."""
+        return self._relpath(file_id)
+
+    def __iter__(self):
+        suffixes = self._versionedfile_class.get_suffixes()
+        ids = set()
+        for relpath in self._iter_files_recursive():
+            for suffix in suffixes:
+                if relpath.endswith(suffix):
+                    # TODO: use standard remove_suffix function
+                    escaped_id = os.path.basename(relpath[:-len(suffix)])
+                    file_id = self._unescape(escaped_id)
+                    if file_id not in ids:
+                        ids.add(file_id)
+                        yield file_id
+                    break # only one suffix can match
+
+    def has_id(self, fileid):
+        suffixes = self._versionedfile_class.get_suffixes()
+        filename = self.filename(fileid)
+        for suffix in suffixes:
+            if not self._transport.has(filename + suffix):
+                return False
+        return True
+
+    def get_empty(self, file_id, transaction):
+        """Get an empty weave, which implies deleting the existing one first."""
+        if self.has_id(file_id):
+            self.delete(file_id, transaction)
+        return self.get_weave_or_empty(file_id, transaction)
+
+    def delete(self, file_id, transaction):
+        """Remove file_id from the store."""
+        suffixes = self._versionedfile_class.get_suffixes()
+        filename = self.filename(file_id)
+        for suffix in suffixes:
+            self._transport.delete(filename + suffix)
+        self._clear_cache_id(file_id, transaction)
+
+    def _get(self, file_id):
+        return self._transport.get(self.filename(file_id))
+
+    def _put(self, file_id, f):
+        fn = self.filename(file_id)
+        try:
+            return self._transport.put(fn, f, mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
+            return self._transport.put(fn, f, mode=self._file_mode)
+
+    def get_weave(self, file_id, transaction):
+        weave = transaction.map.find_weave(file_id)
+        if weave is not None:
+            #mutter("cache hit in %s for %s", self, file_id)
+            return weave
+        if transaction.writeable():
+            w = self._versionedfile_class(self.filename(file_id), self._transport, self._file_mode,
+                                          **self._versionedfile_kwargs)
+            transaction.map.add_weave(file_id, w)
+            transaction.register_dirty(w)
+        else:
+            w = self._versionedfile_class(self.filename(file_id),
+                                          self._transport,
+                                          self._file_mode,
+                                          create=False,
+                                          access_mode='r',
+                                          **self._versionedfile_kwargs)
+            transaction.map.add_weave(file_id, w)
+            transaction.register_clean(w, precious=self._precious)
+        return w
+
+    @deprecated_method(zero_eight)
+    def get_lines(self, file_id, rev_id, transaction):
+        """Return text from a particular version of a weave.
+
+        Returned as a list of lines.
+        """
+        w = self.get_weave(file_id, transaction)
+        return w.get_lines(rev_id)
+    
+    def _new_weave(self, file_id, transaction):
+        """Make a new weave for file_id and return it."""
+        weave = self._make_new_versionedfile(file_id, transaction)
+        transaction.map.add_weave(file_id, weave)
+        # has to be dirty - its able to mutate on its own.
+        transaction.register_dirty(weave)
+        return weave
+
+    def _make_new_versionedfile(self, file_id, transaction):
+        if self.has_id(file_id):
+            self.delete(file_id, transaction)
+        try:
+            weave = self._versionedfile_class(self.filename(file_id), self._transport, self._file_mode, create=True,
+                                              **self._versionedfile_kwargs)
+        except NoSuchFile:
+            if not self._prefixed:
+                # unexpected error - NoSuchFile is raised on a missing dir only and that
+                # only occurs when we are prefixed.
+                raise
+            self._transport.mkdir(self.hash_prefix(file_id), mode=self._dir_mode)
+            weave = self._versionedfile_class(self.filename(file_id), self._transport, 
+                                              self._file_mode, create=True,
+                                              **self._versionedfile_kwargs)
+        return weave
+
+    def get_weave_or_empty(self, file_id, transaction):
+        """Return a weave, or an empty one if it doesn't exist.""" 
+        try:
+            return self.get_weave(file_id, transaction)
+        except NoSuchFile:
+            return self._new_weave(file_id, transaction)
+
+    @deprecated_method(zero_eight)
+    def put_weave(self, file_id, weave, transaction):
+        """This is a deprecated API: It writes an entire collection of ids out.
+        
+        This became inappropriate when we made a versioned file api which
+        tracks the state of the collection of versions for a single id.
+        
+        Its maintained for backwards compatability but will only work on
+        weave stores - pre 0.8 repositories.
+        """
+        self._put_weave(file_id, weave, transaction)
+
+    def _put_weave(self, file_id, weave, transaction):
+        """Preserved here for upgrades-to-weaves to use."""
+        myweave = self._make_new_versionedfile(file_id, transaction)
+        myweave.join(weave)
+
+    @deprecated_method(zero_eight)
+    def add_text(self, file_id, rev_id, new_lines, parents, transaction):
+        """This method was a shorthand for 
+
+        vfile = self.get_weave_or_empty(file_id, transaction)
+        vfile.add_lines(rev_id, parents, new_lines)
+        """
+        vfile = self.get_weave_or_empty(file_id, transaction)
+        vfile.add_lines(rev_id, parents, new_lines)
+        
+    @deprecated_method(zero_eight)
+    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents,
+                           transaction):
+        """This method was a shorthand for
+
+        vfile = self.get_weave_or_empty(file_id, transaction)
+        vfile.clone_text(new_rev_id, old_rev_id, parents)
+        """
+        vfile = self.get_weave_or_empty(file_id, transaction)
+        vfile.clone_text(new_rev_id, old_rev_id, parents)
+ 
+    def copy(self, source, result_id, transaction):
+        """Copy the source versioned file to result_id in this store."""
+        self._clear_cache_id(result_id, transaction)
+        source.copy_to(self.filename(result_id), self._transport)
+ 
+    def copy_all_ids(self, store_from, pb=None, from_transaction=None,
+                     to_transaction=None):
+        """Copy all the file ids from store_from into self."""
+        if from_transaction is None:
+            warn("Please pass from_transaction into "
+                 "versioned_store.copy_all_ids.", stacklevel=2)
+        if to_transaction is None:
+            warn("Please pass to_transaction into "
+                 "versioned_store.copy_all_ids.", stacklevel=2)
+        if not store_from.listable():
+            raise UnlistableStore(store_from)
+        ids = []
+        for count, file_id in enumerate(store_from):
+            if pb:
+                pb.update('listing files', count, count)
+            ids.append(file_id)
+        if pb:
+            pb.clear()
+        mutter('copy_all ids: %r', ids)
+        self.copy_multi(store_from, ids, pb=pb,
+                        from_transaction=from_transaction,
+                        to_transaction=to_transaction)
+
+    def copy_multi(self, from_store, file_ids, pb=None, from_transaction=None,
+                   to_transaction=None):
+        """Copy all the versions for multiple file_ids from from_store.
+        
+        :param from_transaction: required current transaction in from_store.
+        """
+        from bzrlib.transactions import PassThroughTransaction
+        assert isinstance(from_store, WeaveStore)
+        if from_transaction is None:
+            warn("WeaveStore.copy_multi without a from_transaction parameter "
+                 "is deprecated. Please provide a from_transaction.",
+                 DeprecationWarning,
+                 stacklevel=2)
+            # we are reading one object - caching is irrelevant.
+            from_transaction = PassThroughTransaction()
+        if to_transaction is None:
+            warn("WeaveStore.copy_multi without a to_transaction parameter "
+                 "is deprecated. Please provide a to_transaction.",
+                 DeprecationWarning,
+                 stacklevel=2)
+            # we are copying single objects, and there may be open tranasactions
+            # so again with the passthrough
+            to_transaction = PassThroughTransaction()
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        for count, f in enumerate(file_ids):
+            mutter("copy weave {%s} into %s", f, self)
+            pb.update('copy', count, len(file_ids))
+            # if we have it in cache, its faster.
+            # joining is fast with knits, and bearable for weaves -
+            # indeed the new case can be optimised if needed.
+            target = self._make_new_versionedfile(f, to_transaction)
+            target.join(from_store.get_weave(f, from_transaction))
+        pb.finished()
+
+    def total_size(self):
+        count, bytes =  super(VersionedFileStore, self).total_size()
+        return (count / len(self._versionedfile_class.get_suffixes())), bytes
+
+WeaveStore = VersionedFileStore

=== added file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/symbol_versioning.py	2006-02-28 04:31:50 +0000
@@ -0,0 +1,142 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning
+
+The methods here allow for api symbol versioning.
+"""
+
+__all__ = ['deprecated_function',
+           'deprecated_method',
+           'DEPRECATED_PARAMETER',
+           'deprecated_passed',
+           'warn', 'set_warning_method', 'zero_seven',
+           'zero_eight',
+           ]
+
+from warnings import warn
+
+
+DEPRECATED_PARAMETER = "A deprecated parameter marker."
+zero_seven = "%s was deprecated in version 0.7."
+zero_eight = "%s was deprecated in version 0.8."
+
+
+def set_warning_method(method):
+    """Set the warning method to be used by this module.
+
+    It should take a message and a warning category as warnings.warn does.
+    """
+    global warn
+    warn = method
+
+
+# TODO - maybe this would be easier to use as one 'smart' method that
+# guess if it is a method or a class or an attribute ? If so, we can
+# add that on top of the primitives, once we have all three written
+# - RBC 20050105
+
+def deprecated_function(deprecation_version):
+    """Decorate a function so that use of it will trigger a warning."""
+
+    def function_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_function(*args, **kwargs):
+            """This is the decorated function."""
+            symbol = "%s.%s" % (callable.__module__, 
+                                callable.__name__
+                                )
+            warn(deprecation_version % symbol, DeprecationWarning, stacklevel=2)
+            return callable(*args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "function",
+                            decorated_function)
+        return decorated_function
+    return function_decorator
+
+
+def deprecated_method(deprecation_version):
+    """Decorate a method so that use of it will trigger a warning.
+    
+    To deprecate an entire class, decorate __init__.
+    """
+
+    def method_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_method(self, *args, **kwargs):
+            """This is the decorated method."""
+            symbol = "%s.%s.%s" % (self.__class__.__module__, 
+                                   self.__class__.__name__,
+                                   callable.__name__
+                                   )
+            warn(deprecation_version % symbol, DeprecationWarning, stacklevel=2)
+            return callable(self, *args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "method",
+                            decorated_method)
+        return decorated_method
+    return method_decorator
+
+
+def deprecated_passed(parameter_value):
+    """Return True if parameter_value was used."""
+    # FIXME: it might be nice to have a parameter deprecation decorator. 
+    # it would need to handle positional and *args and **kwargs parameters,
+    # which means some mechanism to describe how the parameter was being
+    # passed before deprecation, and some way to deprecate parameters that
+    # were not at the end of the arg list. Thats needed for __init__ where
+    # we cannot just forward to a new method name.I.e. in the following
+    # examples we would want to have callers that pass any value to 'bad' be
+    # given a warning - because we have applied:
+    # @deprecated_parameter('bad', zero_seven)
+    #
+    # def __init__(self, bad=None)
+    # def __init__(self, bad, other)
+    # def __init__(self, **kwargs)
+    # RBC 20060116
+    return not parameter_value is DEPRECATED_PARAMETER
+
+
+def _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable):
+    docstring_lines = callable.__doc__.split('\n')
+    if len(docstring_lines) == 0:
+        decorated_callable.__doc__ = deprecation_version % ("This " + label)
+    elif len(docstring_lines) == 1:
+        decorated_callable.__doc__ = (callable.__doc__ 
+                                    + "\n"
+                                    + "\n"
+                                    + deprecation_version % ("This " + label)
+                                    + "\n")
+    else:
+        spaces = len(docstring_lines[-1])
+        new_doc = callable.__doc__
+        new_doc += "\n" + " " * spaces
+        new_doc += deprecation_version % ("This " + label)
+        new_doc += "\n" + " " * spaces
+        decorated_callable.__doc__ = new_doc
+
+
+def _populate_decorated(callable, deprecation_version, label,
+                        decorated_callable):
+    """Populate attributes like __name__ and __doc__ on the decorated callable.
+    """
+    _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable)
+    decorated_callable.__module__ = callable.__module__
+    decorated_callable.__name__ = callable.__name__
+    decorated_callable.is_deprecated = True

=== added file 'bzrlib/testament.py'
--- bzrlib/testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/testament.py	2006-02-12 22:40:24 +0000
@@ -0,0 +1,186 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Testament - a summary of a revision for signing.
+
+A testament can be defined as "something that serves as tangible 
+proof or evidence."  In bzr we use them to allow people to certify
+particular revisions as authentic.  
+
+The goal is that if two revisions are semantically equal, then they will
+have a byte-for-byte equal testament.  We can define different versions of
+"semantically equal" by using different testament classes; e.g. one that
+includes or ignores file-ids.
+
+We sign a testament rather than the revision XML itself for several reasons.
+The most important is that the form in which the revision is stored
+internally is designed for that purpose, and contains information which need
+not be attested to by the signer.  For example the inventory contains the
+last-changed revision for a file, but this is not necessarily something the
+user cares to sign.
+
+Having unnecessary fields signed makes the signatures brittle when the same
+revision is stored in different branches or when the format is upgraded.
+
+Handling upgrades is another motivation for using testaments separate from
+the stored revision.  We would like to be able to compare a signature
+generated from an old-format tree to newer tree, or vice versa.  This could
+be done by comparing the revisions but that makes it unclear about exactly
+what is being compared or not.
+
+Different signing keys might indicate different levels of trust; we can in
+the future extend this to allow signatures indicating not just that a
+particular version is authentic but that it has other properties.
+
+The signature can be applied to either the full testament or to just a
+hash of it.
+
+Testament format 1
+~~~~~~~~~~~~~~~~~~
+
+* timestamps are given as integers to avoid rounding errors
+* parents given in lexicographical order
+* indented-text form similar to log; intended to be human readable
+* paths are given with forward slashes
+* files are named using paths for ease of comparison/debugging
+* the testament uses unix line-endings (\n)
+"""
+
+# XXX: At the moment, clients trust that the graph described in a weave
+# is accurate, but that's not covered by the testament.  Perhaps the best
+# fix is when verifying a revision to make sure that every file mentioned 
+# in the revision has compatible ancestry links.
+
+# TODO: perhaps write timestamp in a more readable form
+
+# TODO: Perhaps these should just be different formats in which inventories/
+# revisions can be serialized.
+
+from copy import copy
+from cStringIO import StringIO
+import string
+from sha import sha
+
+from bzrlib.osutils import contains_whitespace, contains_linebreaks
+
+class Testament(object):
+    """Reduced summary of a revision.
+
+    Testaments can be 
+
+      - produced from a revision
+      - written to a stream
+      - loaded from a stream
+      - compared to a revision
+    """
+
+    @classmethod
+    def from_revision(cls, repository, revision_id):
+        """Produce a new testament from a historical revision"""
+        rev = repository.get_revision(revision_id)
+        inventory = repository.get_inventory(revision_id)
+        return cls(rev, inventory)
+
+    def __init__(self, rev, inventory):
+        """Create a new testament for rev using inventory."""
+        self.revision_id = str(rev.revision_id)
+        self.committer = rev.committer
+        self.timezone = rev.timezone or 0
+        self.timestamp = rev.timestamp
+        self.message = rev.message
+        self.parent_ids = rev.parent_ids[:]
+        self.inventory = inventory
+        self.revprops = copy(rev.properties)
+        assert not contains_whitespace(self.revision_id)
+        assert not contains_linebreaks(self.committer)
+
+    def as_text_lines(self):
+        """Yield text form as a sequence of lines.
+
+        The result is returned in utf-8, because it should be signed or
+        hashed in that encoding.
+        """
+        r = []
+        a = r.append
+        a('bazaar-ng testament version 1\n')
+        a('revision-id: %s\n' % self.revision_id)
+        a('committer: %s\n' % self.committer)
+        a('timestamp: %d\n' % self.timestamp)
+        a('timezone: %d\n' % self.timezone)
+        # inventory length contains the root, which is not shown here
+        a('parents:\n')
+        for parent_id in sorted(self.parent_ids):
+            assert not contains_whitespace(parent_id)
+            a('  %s\n' % parent_id)
+        a('message:\n')
+        for l in self.message.splitlines():
+            a('  %s\n' % l)
+        a('inventory:\n')
+        for path, ie in self.inventory.iter_entries():
+            a(self._entry_to_line(path, ie))
+        r.extend(self._revprops_to_lines())
+        if __debug__:
+            for l in r:
+                assert isinstance(l, basestring), \
+                    '%r of type %s is not a plain string' % (l, type(l))
+        return [line.encode('utf-8') for line in r]
+
+    def _escape_path(self, path):
+        assert not contains_linebreaks(path)
+        return unicode(path.replace('\\', '/').replace(' ', '\ ')).encode('utf-8')
+
+    def _entry_to_line(self, path, ie):
+        """Turn an inventory entry into a testament line"""
+        l = '  ' + str(ie.kind)
+        l += ' ' + self._escape_path(path)
+        assert not contains_whitespace(ie.file_id)
+        l += ' ' + unicode(ie.file_id).encode('utf-8')
+        if ie.kind == 'file':
+            # TODO: avoid switching on kind
+            assert ie.text_sha1
+            l += ' ' + ie.text_sha1
+        elif ie.kind == 'symlink':
+            assert ie.symlink_target
+            l += ' ' + self._escape_path(ie.symlink_target)
+        l += '\n'
+        return l
+
+    def as_text(self):
+        return ''.join(self.as_text_lines())
+
+    def as_short_text(self):
+        """Return short digest-based testament."""
+        s = sha()
+        map(s.update, self.as_text_lines())
+        return ('bazaar-ng testament short form 1\n'
+                'revision-id: %s\n'
+                'sha1: %s\n'
+                % (self.revision_id, s.hexdigest()))
+
+    def _revprops_to_lines(self):
+        """Pack up revision properties."""
+        if not self.revprops:
+            return []
+        r = ['properties:\n']
+        for name, value in sorted(self.revprops.items()):
+            assert isinstance(name, str)
+            assert not contains_whitespace(name)
+            r.append('  %s:\n' % name)
+            for line in value.splitlines():
+                if not isinstance(line, str):
+                    line = line.encode('utf-8')
+                r.append('    %s\n' % line)
+        return r

=== renamed directory 'bzrlib/selftest' => 'bzrlib/tests'
=== added file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2006-02-10 11:03:38 +0000
@@ -0,0 +1,33 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+import bzrlib
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestCaseWithWebserver(TestCaseWithTransport):
+    """A support class that provides readonly urls that are http://.
+
+    This is done by forcing the readonly server to be an http one. This 
+    will current fail if the primary transport is not backed by regular disk
+    files.
+    """
+
+    def setUp(self):
+        super(TestCaseWithWebserver, self).setUp()
+        self.transport_readonly_server = bzrlib.transport.http.HttpServer

=== added file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/TestUtil.py	2005-09-19 06:05:19 +0000
@@ -0,0 +1,80 @@
+# Copyright (c) 2004 Canonical Limited
+#       Author: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import sys
+import logging
+import unittest
+
+
+class LogCollector(logging.Handler):
+    def __init__(self):
+        logging.Handler.__init__(self)
+        self.records=[]
+    def emit(self, record):
+        self.records.append(record.getMessage())
+
+
+def makeCollectingLogger():
+    """I make a logger instance that collects its logs for programmatic analysis
+    -> (logger, collector)"""
+    logger=logging.Logger("collector")
+    handler=LogCollector()
+    handler.setFormatter(logging.Formatter("%(levelname)s: %(message)s"))
+    logger.addHandler(handler)
+    return logger, handler
+
+
+def visitTests(suite, visitor):
+    """A foreign method for visiting the tests in a test suite."""
+    for test in suite._tests:
+        #Abusing types to avoid monkey patching unittest.TestCase. 
+        # Maybe that would be better?
+        try:
+            test.visit(visitor)
+        except AttributeError:
+            if isinstance(test, unittest.TestCase):
+                visitor.visitCase(test)
+            elif isinstance(test, unittest.TestSuite):
+                visitor.visitSuite(test)
+                visitTests(test, visitor)
+            else:
+                print "unvisitable non-unittest.TestCase element %r (%r)" % (test, test.__class__)
+    
+
+class TestSuite(unittest.TestSuite):
+    """I am an extended TestSuite with a visitor interface.
+    This is primarily to allow filtering of tests - and suites or
+    more in the future. An iterator of just tests wouldn't scale..."""
+
+    def visit(self, visitor):
+        """visit the composite. Visiting is depth-first.
+        current callbacks are visitSuite and visitCase."""
+        visitor.visitSuite(self)
+        visitTests(self, visitor)
+
+
+class TestLoader(unittest.TestLoader):
+    """Custome TestLoader to set the right TestSuite class."""
+    suiteClass = TestSuite
+
+class TestVisitor(object):
+    """A visitor for Tests"""
+    def visitSuite(self, aTestSuite):
+        pass
+    def visitCase(self, aTestCase):
+        pass

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/selftest/__init__.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/__init__.py	2006-04-19 08:31:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,81 +15,1045 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from testsweet import TestCase, run_suite, InTempDir
+# TODO: Perhaps there should be an API to find out if bzr running under the
+# test suite -- some plugins might want to avoid making intrusive changes if
+# this is the case.  However, we want behaviour under to test to diverge as
+# little as possible, so this should be used rarely if it's added at all.
+# (Suggestion from j-a-meinel, 2005-11-24)
+
+# NOTE: Some classes in here use camelCaseNaming() rather than
+# underscore_naming().  That's for consistency with unittest; it's not the
+# general style of bzrlib.  Please continue that consistency when adding e.g.
+# new assertFoo() methods.
+
+import codecs
+from cStringIO import StringIO
+import difflib
+import errno
+import logging
+import os
+import re
+import shutil
+import stat
+import sys
+import tempfile
+import unittest
+import time
+
+
+import bzrlib.branch
+import bzrlib.bzrdir as bzrdir
 import bzrlib.commands
+import bzrlib.errors as errors
+import bzrlib.inventory
+import bzrlib.iterablefile
+import bzrlib.lockdir
+from bzrlib.merge import merge_inner
+import bzrlib.merge3
+import bzrlib.osutils
+import bzrlib.osutils as osutils
+import bzrlib.plugin
+from bzrlib.revision import common_ancestor
+import bzrlib.store
+import bzrlib.trace
+from bzrlib.transport import urlescape, get_transport
+import bzrlib.transport
+from bzrlib.transport.local import LocalRelpathServer
+from bzrlib.transport.readonly import ReadonlyServer
+from bzrlib.trace import mutter
+from bzrlib.tests.TestUtil import TestLoader, TestSuite
+from bzrlib.tests.treeshape import build_tree_contents
+from bzrlib.workingtree import WorkingTree, WorkingTreeFormat2
+
+default_transport = LocalRelpathServer
 
 MODULES_TO_TEST = []
-MODULES_TO_DOCTEST = []
-
-
-class BzrTestBase(InTempDir):
-    """bzr-specific test base class"""
+MODULES_TO_DOCTEST = [
+                      bzrlib.branch,
+                      bzrlib.commands,
+                      bzrlib.errors,
+                      bzrlib.inventory,
+                      bzrlib.iterablefile,
+                      bzrlib.lockdir,
+                      bzrlib.merge3,
+                      bzrlib.option,
+                      bzrlib.osutils,
+                      bzrlib.store
+                      ]
+def packages_to_test():
+    """Return a list of packages to test.
+
+    The packages are not globally imported so that import failures are
+    triggered when running selftest, not when importing the command.
+    """
+    import bzrlib.doc
+    import bzrlib.tests.blackbox
+    import bzrlib.tests.branch_implementations
+    import bzrlib.tests.bzrdir_implementations
+    import bzrlib.tests.interrepository_implementations
+    import bzrlib.tests.interversionedfile_implementations
+    import bzrlib.tests.repository_implementations
+    import bzrlib.tests.revisionstore_implementations
+    import bzrlib.tests.workingtree_implementations
+    return [
+            bzrlib.doc,
+            bzrlib.tests.blackbox,
+            bzrlib.tests.branch_implementations,
+            bzrlib.tests.bzrdir_implementations,
+            bzrlib.tests.interrepository_implementations,
+            bzrlib.tests.interversionedfile_implementations,
+            bzrlib.tests.repository_implementations,
+            bzrlib.tests.revisionstore_implementations,
+            bzrlib.tests.workingtree_implementations,
+            ]
+
+
+class _MyResult(unittest._TextTestResult):
+    """Custom TestResult.
+
+    Shows output in a different format, including displaying runtime for tests.
+    """
+    stop_early = False
+
+    def _elapsedTime(self):
+        return "%5dms" % (1000 * (time.time() - self._start_time))
+
+    def startTest(self, test):
+        unittest.TestResult.startTest(self, test)
+        # In a short description, the important words are in
+        # the beginning, but in an id, the important words are
+        # at the end
+        SHOW_DESCRIPTIONS = False
+        if self.showAll:
+            width = osutils.terminal_width()
+            name_width = width - 15
+            what = None
+            if SHOW_DESCRIPTIONS:
+                what = test.shortDescription()
+                if what:
+                    if len(what) > name_width:
+                        what = what[:name_width-3] + '...'
+            if what is None:
+                what = test.id()
+                if what.startswith('bzrlib.tests.'):
+                    what = what[13:]
+                if len(what) > name_width:
+                    what = '...' + what[3-name_width:]
+            what = what.ljust(name_width)
+            self.stream.write(what)
+        self.stream.flush()
+        self._start_time = time.time()
+
+    def addError(self, test, err):
+        if isinstance(err[1], TestSkipped):
+            return self.addSkipped(test, err)    
+        unittest.TestResult.addError(self, test, err)
+        if self.showAll:
+            self.stream.writeln("ERROR %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('E')
+        self.stream.flush()
+        if self.stop_early:
+            self.stop()
+
+    def addFailure(self, test, err):
+        unittest.TestResult.addFailure(self, test, err)
+        if self.showAll:
+            self.stream.writeln(" FAIL %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('F')
+        self.stream.flush()
+        if self.stop_early:
+            self.stop()
+
+    def addSuccess(self, test):
+        if self.showAll:
+            self.stream.writeln('   OK %s' % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('~')
+        self.stream.flush()
+        unittest.TestResult.addSuccess(self, test)
+
+    def addSkipped(self, test, skip_excinfo):
+        if self.showAll:
+            print >>self.stream, ' SKIP %s' % self._elapsedTime()
+            print >>self.stream, '     %s' % skip_excinfo[1]
+        elif self.dots:
+            self.stream.write('S')
+        self.stream.flush()
+        # seems best to treat this as success from point-of-view of unittest
+        # -- it actually does nothing so it barely matters :)
+        unittest.TestResult.addSuccess(self, test)
+
+    def printErrorList(self, flavour, errors):
+        for test, err in errors:
+            self.stream.writeln(self.separator1)
+            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
+            if getattr(test, '_get_log', None) is not None:
+                print >>self.stream
+                print >>self.stream, \
+                        ('vvvv[log from %s]' % test.id()).ljust(78,'-')
+                print >>self.stream, test._get_log()
+                print >>self.stream, \
+                        ('^^^^[log from %s]' % test.id()).ljust(78,'-')
+            self.stream.writeln(self.separator2)
+            self.stream.writeln("%s" % err)
+
+
+class TextTestRunner(unittest.TextTestRunner):
+    stop_on_failure = False
+
+    def _makeResult(self):
+        result = _MyResult(self.stream, self.descriptions, self.verbosity)
+        result.stop_early = self.stop_on_failure
+        return result
+
+
+def iter_suite_tests(suite):
+    """Return all tests in a suite, recursing through nested suites"""
+    for item in suite._tests:
+        if isinstance(item, unittest.TestCase):
+            yield item
+        elif isinstance(item, unittest.TestSuite):
+            for r in iter_suite_tests(item):
+                yield r
+        else:
+            raise Exception('unknown object %r inside test suite %r'
+                            % (item, suite))
+
+
+class TestSkipped(Exception):
+    """Indicates that a test was intentionally skipped, rather than failing."""
+    # XXX: Not used yet
+
+
+class CommandFailed(Exception):
+    pass
+
+class TestCase(unittest.TestCase):
+    """Base class for bzr unit tests.
+    
+    Tests that need access to disk resources should subclass 
+    TestCaseInTempDir not TestCase.
+
+    Error and debug log messages are redirected from their usual
+    location into a temporary file, the contents of which can be
+    retrieved by _get_log().  We use a real OS file, not an in-memory object,
+    so that it can also capture file IO.  When the test completes this file
+    is read into memory and removed from disk.
+       
+    There are also convenience functions to invoke bzr's command-line
+    routine, and to build and check bzr trees.
+   
+    In addition to the usual method of overriding tearDown(), this class also
+    allows subclasses to register functions into the _cleanups list, which is
+    run in order as the object is torn down.  It's less likely this will be
+    accidentally overlooked.
+    """
+
+    BZRPATH = 'bzr'
+    _log_file_name = None
+    _log_contents = ''
+
+    def __init__(self, methodName='testMethod'):
+        super(TestCase, self).__init__(methodName)
+        self._cleanups = []
+
+    def setUp(self):
+        unittest.TestCase.setUp(self)
+        self._cleanEnvironment()
+        bzrlib.trace.disable_default_logging()
+        self._startLogFile()
+
+    def _ndiff_strings(self, a, b):
+        """Return ndiff between two strings containing lines.
+        
+        A trailing newline is added if missing to make the strings
+        print properly."""
+        if b and b[-1] != '\n':
+            b += '\n'
+        if a and a[-1] != '\n':
+            a += '\n'
+        difflines = difflib.ndiff(a.splitlines(True),
+                                  b.splitlines(True),
+                                  linejunk=lambda x: False,
+                                  charjunk=lambda x: False)
+        return ''.join(difflines)
+
+    def assertEqualDiff(self, a, b, message=None):
+        """Assert two texts are equal, if not raise an exception.
+        
+        This is intended for use with multi-line strings where it can 
+        be hard to find the differences by eye.
+        """
+        # TODO: perhaps override assertEquals to call this for strings?
+        if a == b:
+            return
+        if message is None:
+            message = "texts not equal:\n"
+        raise AssertionError(message + 
+                             self._ndiff_strings(a, b))      
+        
+    def assertEqualMode(self, mode, mode_test):
+        self.assertEqual(mode, mode_test,
+                         'mode mismatch %o != %o' % (mode, mode_test))
+
+    def assertStartsWith(self, s, prefix):
+        if not s.startswith(prefix):
+            raise AssertionError('string %r does not start with %r' % (s, prefix))
+
+    def assertEndsWith(self, s, suffix):
+        if not s.endswith(prefix):
+            raise AssertionError('string %r does not end with %r' % (s, suffix))
+
+    def assertContainsRe(self, haystack, needle_re):
+        """Assert that a contains something matching a regular expression."""
+        if not re.search(needle_re, haystack):
+            raise AssertionError('pattern "%s" not found in "%s"'
+                    % (needle_re, haystack))
+
+    def assertSubset(self, sublist, superlist):
+        """Assert that every entry in sublist is present in superlist."""
+        missing = []
+        for entry in sublist:
+            if entry not in superlist:
+                missing.append(entry)
+        if len(missing) > 0:
+            raise AssertionError("value(s) %r not present in container %r" % 
+                                 (missing, superlist))
+
+    def assertIs(self, left, right):
+        if not (left is right):
+            raise AssertionError("%r is not %r." % (left, right))
+
+    def assertTransportMode(self, transport, path, mode):
+        """Fail if a path does not have mode mode.
+        
+        If modes are not supported on this transport, the assertion is ignored.
+        """
+        if not transport._can_roundtrip_unix_modebits():
+            return
+        path_stat = transport.stat(path)
+        actual_mode = stat.S_IMODE(path_stat.st_mode)
+        self.assertEqual(mode, actual_mode,
+            'mode of %r incorrect (%o != %o)' % (path, mode, actual_mode))
+
+    def assertIsInstance(self, obj, kls):
+        """Fail if obj is not an instance of kls"""
+        if not isinstance(obj, kls):
+            self.fail("%r is not an instance of %s" % (obj, kls))
+
+    def _startLogFile(self):
+        """Send bzr and test log messages to a temporary file.
+
+        The file is removed as the test is torn down.
+        """
+        fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
+        encoder, decoder, stream_reader, stream_writer = codecs.lookup('UTF-8')
+        self._log_file = stream_writer(os.fdopen(fileno, 'w+'))
+        self._log_nonce = bzrlib.trace.enable_test_log(self._log_file)
+        self._log_file_name = name
+        self.addCleanup(self._finishLogFile)
+
+    def _finishLogFile(self):
+        """Finished with the log file.
+
+        Read contents into memory, close, and delete.
+        """
+        bzrlib.trace.disable_test_log(self._log_nonce)
+        self._log_file.seek(0)
+        self._log_contents = self._log_file.read()
+        self._log_file.close()
+        os.remove(self._log_file_name)
+        self._log_file = self._log_file_name = None
+
+    def addCleanup(self, callable):
+        """Arrange to run a callable when this case is torn down.
+
+        Callables are run in the reverse of the order they are registered, 
+        ie last-in first-out.
+        """
+        if callable in self._cleanups:
+            raise ValueError("cleanup function %r already registered on %s" 
+                    % (callable, self))
+        self._cleanups.append(callable)
+
+    def _cleanEnvironment(self):
+        new_env = {
+            'HOME': os.getcwd(),
+            'APPDATA': os.getcwd(),
+            'BZREMAIL': None,
+            'EMAIL': None,
+        }
+        self.__old_env = {}
+        self.addCleanup(self._restoreEnvironment)
+        for name, value in new_env.iteritems():
+            self._captureVar(name, value)
+
+
+    def _captureVar(self, name, newvalue):
+        """Set an environment variable, preparing it to be reset when finished."""
+        self.__old_env[name] = os.environ.get(name, None)
+        if newvalue is None:
+            if name in os.environ:
+                del os.environ[name]
+        else:
+            os.environ[name] = newvalue
+
+    @staticmethod
+    def _restoreVar(name, value):
+        if value is None:
+            if name in os.environ:
+                del os.environ[name]
+        else:
+            os.environ[name] = value
+
+    def _restoreEnvironment(self):
+        for name, value in self.__old_env.iteritems():
+            self._restoreVar(name, value)
+
+    def tearDown(self):
+        self._runCleanups()
+        unittest.TestCase.tearDown(self)
+
+    def _runCleanups(self):
+        """Run registered cleanup functions. 
+
+        This should only be called from TestCase.tearDown.
+        """
+        # TODO: Perhaps this should keep running cleanups even if 
+        # one of them fails?
+        for cleanup_fn in reversed(self._cleanups):
+            cleanup_fn()
+
+    def log(self, *args):
+        mutter(*args)
+
+    def _get_log(self):
+        """Return as a string the log for this test"""
+        if self._log_file_name:
+            return open(self._log_file_name).read()
+        else:
+            return self._log_contents
+        # TODO: Delete the log after it's been read in
+
+    def capture(self, cmd, retcode=0):
+        """Shortcut that splits cmd into words, runs, and returns stdout"""
+        return self.run_bzr_captured(cmd.split(), retcode=retcode)[0]
+
+    def run_bzr_captured(self, argv, retcode=0):
+        """Invoke bzr and return (stdout, stderr).
+
+        Useful for code that wants to check the contents of the
+        output, the way error messages are presented, etc.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        Much of the old code runs bzr by forking a new copy of Python, but
+        that is slower, harder to debug, and generally not necessary.
+
+        This runs bzr through the interface that catches and reports
+        errors, and with logging set to something approximating the
+        default, so that error reporting can be checked.
+
+        argv -- arguments to invoke bzr
+        retcode -- expected return code, or None for don't-care.
+        """
+        stdout = StringIO()
+        stderr = StringIO()
+        self.log('run bzr: %s', ' '.join(argv))
+        # FIXME: don't call into logging here
+        handler = logging.StreamHandler(stderr)
+        handler.setFormatter(bzrlib.trace.QuietFormatter())
+        handler.setLevel(logging.INFO)
+        logger = logging.getLogger('')
+        logger.addHandler(handler)
+        try:
+            result = self.apply_redirected(None, stdout, stderr,
+                                           bzrlib.commands.run_bzr_catch_errors,
+                                           argv)
+        finally:
+            logger.removeHandler(handler)
+        out = stdout.getvalue()
+        err = stderr.getvalue()
+        if out:
+            self.log('output:\n%s', out)
+        if err:
+            self.log('errors:\n%s', err)
+        if retcode is not None:
+            self.assertEquals(result, retcode)
+        return out, err
+
     def run_bzr(self, *args, **kwargs):
-        retcode = kwargs.get('retcode', 0)
-        self.assertEquals(bzrlib.commands.run_bzr(args), retcode)
-        
-
-def selftest(verbose=False):
-    from unittest import TestLoader, TestSuite
-    import bzrlib, bzrlib.store, bzrlib.inventory, bzrlib.branch
-    import bzrlib.osutils, bzrlib.commands, bzrlib.merge3, bzrlib.plugin
+        """Invoke bzr, as if it were run from the command line.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        This sends the stdout/stderr results into the test's log,
+        where it may be useful for debugging.  See also run_captured.
+        """
+        retcode = kwargs.pop('retcode', 0)
+        return self.run_bzr_captured(args, retcode)
+
+    def check_inventory_shape(self, inv, shape):
+        """Compare an inventory to a list of expected names.
+
+        Fail if they are not precisely equal.
+        """
+        extras = []
+        shape = list(shape)             # copy
+        for path, ie in inv.entries():
+            name = path.replace('\\', '/')
+            if ie.kind == 'dir':
+                name = name + '/'
+            if name in shape:
+                shape.remove(name)
+            else:
+                extras.append(name)
+        if shape:
+            self.fail("expected paths not found in inventory: %r" % shape)
+        if extras:
+            self.fail("unexpected paths found in inventory: %r" % extras)
+
+    def apply_redirected(self, stdin=None, stdout=None, stderr=None,
+                         a_callable=None, *args, **kwargs):
+        """Call callable with redirected std io pipes.
+
+        Returns the return code."""
+        if not callable(a_callable):
+            raise ValueError("a_callable must be callable.")
+        if stdin is None:
+            stdin = StringIO("")
+        if stdout is None:
+            if getattr(self, "_log_file", None) is not None:
+                stdout = self._log_file
+            else:
+                stdout = StringIO()
+        if stderr is None:
+            if getattr(self, "_log_file", None is not None):
+                stderr = self._log_file
+            else:
+                stderr = StringIO()
+        real_stdin = sys.stdin
+        real_stdout = sys.stdout
+        real_stderr = sys.stderr
+        try:
+            sys.stdout = stdout
+            sys.stderr = stderr
+            sys.stdin = stdin
+            return a_callable(*args, **kwargs)
+        finally:
+            sys.stdout = real_stdout
+            sys.stderr = real_stderr
+            sys.stdin = real_stdin
+
+    def merge(self, branch_from, wt_to):
+        """A helper for tests to do a ui-less merge.
+
+        This should move to the main library when someone has time to integrate
+        it in.
+        """
+        # minimal ui-less merge.
+        wt_to.branch.fetch(branch_from)
+        base_rev = common_ancestor(branch_from.last_revision(),
+                                   wt_to.branch.last_revision(),
+                                   wt_to.branch.repository)
+        merge_inner(wt_to.branch, branch_from.basis_tree(), 
+                    wt_to.branch.repository.revision_tree(base_rev),
+                    this_tree=wt_to)
+        wt_to.add_pending_merge(branch_from.last_revision())
+
+
+BzrTestBase = TestCase
+
+     
+class TestCaseInTempDir(TestCase):
+    """Derived class that runs a test within a temporary directory.
+
+    This is useful for tests that need to create a branch, etc.
+
+    The directory is created in a slightly complex way: for each
+    Python invocation, a new temporary top-level directory is created.
+    All test cases create their own directory within that.  If the
+    tests complete successfully, the directory is removed.
+
+    InTempDir is an old alias for FunctionalTestCase.
+    """
+
+    TEST_ROOT = None
+    _TEST_NAME = 'test'
+    OVERRIDE_PYTHON = 'python'
+
+    def check_file_contents(self, filename, expect):
+        self.log("check contents of file %s" % filename)
+        contents = file(filename, 'r').read()
+        if contents != expect:
+            self.log("expected: %r" % expect)
+            self.log("actually: %r" % contents)
+            self.fail("contents of %s not as expected" % filename)
+
+    def _make_test_root(self):
+        if TestCaseInTempDir.TEST_ROOT is not None:
+            return
+        i = 0
+        while True:
+            root = u'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseInTempDir.TEST_ROOT = osutils.abspath(root)
+            break
+        # make a fake bzr directory there to prevent any tests propagating
+        # up onto the source directory's real branch
+        bzrdir.BzrDir.create_standalone_workingtree(TestCaseInTempDir.TEST_ROOT)
+
+    def setUp(self):
+        super(TestCaseInTempDir, self).setUp()
+        self._make_test_root()
+        _currentdir = os.getcwdu()
+        # shorten the name, to avoid test failures due to path length
+        short_id = self.id().replace('bzrlib.tests.', '') \
+                   .replace('__main__.', '')[-100:]
+        # it's possible the same test class is run several times for
+        # parameterized tests, so make sure the names don't collide.  
+        i = 0
+        while True:
+            if i > 0:
+                candidate_dir = '%s/%s.%d' % (self.TEST_ROOT, short_id, i)
+            else:
+                candidate_dir = '%s/%s' % (self.TEST_ROOT, short_id)
+            if os.path.exists(candidate_dir):
+                i = i + 1
+                continue
+            else:
+                self.test_dir = candidate_dir
+                os.mkdir(self.test_dir)
+                os.chdir(self.test_dir)
+                break
+        os.environ['HOME'] = self.test_dir
+        os.environ['APPDATA'] = self.test_dir
+        def _leaveDirectory():
+            os.chdir(_currentdir)
+        self.addCleanup(_leaveDirectory)
+        
+    def build_tree(self, shape, line_endings='native', transport=None):
+        """Build a test tree according to a pattern.
+
+        shape is a sequence of file specifications.  If the final
+        character is '/', a directory is created.
+
+        This doesn't add anything to a branch.
+        :param line_endings: Either 'binary' or 'native'
+                             in binary mode, exact contents are written
+                             in native mode, the line endings match the
+                             default platform endings.
+
+        :param transport: A transport to write to, for building trees on 
+                          VFS's. If the transport is readonly or None,
+                          "." is opened automatically.
+        """
+        # XXX: It's OK to just create them using forward slashes on windows?
+        if transport is None or transport.is_readonly():
+            transport = get_transport(".")
+        for name in shape:
+            self.assert_(isinstance(name, basestring))
+            if name[-1] == '/':
+                transport.mkdir(urlescape(name[:-1]))
+            else:
+                if line_endings == 'binary':
+                    end = '\n'
+                elif line_endings == 'native':
+                    end = os.linesep
+                else:
+                    raise errors.BzrError('Invalid line ending request %r' % (line_endings,))
+                content = "contents of %s%s" % (name, end)
+                transport.put(urlescape(name), StringIO(content))
+
+    def build_tree_contents(self, shape):
+        build_tree_contents(shape)
+
+    def failUnlessExists(self, path):
+        """Fail unless path, which may be abs or relative, exists."""
+        self.failUnless(osutils.lexists(path))
+
+    def failIfExists(self, path):
+        """Fail if path, which may be abs or relative, exists."""
+        self.failIf(osutils.lexists(path))
+        
+    def assertFileEqual(self, content, path):
+        """Fail if path does not contain 'content'."""
+        self.failUnless(osutils.lexists(path))
+        self.assertEqualDiff(content, open(path, 'r').read())
+
+
+class TestCaseWithTransport(TestCaseInTempDir):
+    """A test case that provides get_url and get_readonly_url facilities.
+
+    These back onto two transport servers, one for readonly access and one for
+    read write access.
+
+    If no explicit class is provided for readonly access, a
+    ReadonlyTransportDecorator is used instead which allows the use of non disk
+    based read write transports.
+
+    If an explicit class is provided for readonly access, that server and the 
+    readwrite one must both define get_url() as resolving to os.getcwd().
+    """
+
+    def __init__(self, methodName='testMethod'):
+        super(TestCaseWithTransport, self).__init__(methodName)
+        self.__readonly_server = None
+        self.__server = None
+        self.transport_server = default_transport
+        self.transport_readonly_server = None
+
+    def get_readonly_url(self, relpath=None):
+        """Get a URL for the readonly transport.
+
+        This will either be backed by '.' or a decorator to the transport 
+        used by self.get_url()
+        relpath provides for clients to get a path relative to the base url.
+        These should only be downwards relative, not upwards.
+        """
+        base = self.get_readonly_server().get_url()
+        if relpath is not None:
+            if not base.endswith('/'):
+                base = base + '/'
+            base = base + relpath
+        return base
+
+    def get_readonly_server(self):
+        """Get the server instance for the readonly transport
+
+        This is useful for some tests with specific servers to do diagnostics.
+        """
+        if self.__readonly_server is None:
+            if self.transport_readonly_server is None:
+                # readonly decorator requested
+                # bring up the server
+                self.get_url()
+                self.__readonly_server = ReadonlyServer()
+                self.__readonly_server.setUp(self.__server)
+            else:
+                self.__readonly_server = self.transport_readonly_server()
+                self.__readonly_server.setUp()
+            self.addCleanup(self.__readonly_server.tearDown)
+        return self.__readonly_server
+
+    def get_server(self):
+        """Get the read/write server instance.
+
+        This is useful for some tests with specific servers that need
+        diagnostics.
+        """
+        if self.__server is None:
+            self.__server = self.transport_server()
+            self.__server.setUp()
+            self.addCleanup(self.__server.tearDown)
+        return self.__server
+
+    def get_url(self, relpath=None):
+        """Get a URL for the readwrite transport.
+
+        This will either be backed by '.' or to an equivalent non-file based
+        facility.
+        relpath provides for clients to get a path relative to the base url.
+        These should only be downwards relative, not upwards.
+        """
+        base = self.get_server().get_url()
+        if relpath is not None and relpath != '.':
+            if not base.endswith('/'):
+                base = base + '/'
+            base = base + relpath
+        return base
+
+    def get_transport(self):
+        """Return a writeable transport for the test scratch space"""
+        t = get_transport(self.get_url())
+        self.assertFalse(t.is_readonly())
+        return t
+
+    def get_readonly_transport(self):
+        """Return a readonly transport for the test scratch space
+        
+        This can be used to test that operations which should only need
+        readonly access in fact do not try to write.
+        """
+        t = get_transport(self.get_readonly_url())
+        self.assertTrue(t.is_readonly())
+        return t
+
+    def make_branch(self, relpath, format=None):
+        """Create a branch on the transport at relpath."""
+        repo = self.make_repository(relpath, format=format)
+        return repo.bzrdir.create_branch()
+
+    def make_bzrdir(self, relpath, format=None):
+        try:
+            url = self.get_url(relpath)
+            segments = relpath.split('/')
+            if segments and segments[-1] not in ('', '.'):
+                parent = self.get_url('/'.join(segments[:-1]))
+                t = get_transport(parent)
+                try:
+                    t.mkdir(segments[-1])
+                except errors.FileExists:
+                    pass
+            if format is None:
+                format=bzrlib.bzrdir.BzrDirFormat.get_default_format()
+            # FIXME: make this use a single transport someday. RBC 20060418
+            return format.initialize_on_transport(get_transport(relpath))
+        except errors.UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable.")
+
+    def make_repository(self, relpath, shared=False, format=None):
+        """Create a repository on our default transport at relpath."""
+        made_control = self.make_bzrdir(relpath, format=format)
+        return made_control.create_repository(shared=shared)
+
+    def make_branch_and_tree(self, relpath, format=None):
+        """Create a branch on the transport and a tree locally.
+
+        Returns the tree.
+        """
+        # TODO: always use the local disk path for the working tree,
+        # this obviously requires a format that supports branch references
+        # so check for that by checking bzrdir.BzrDirFormat.get_default_format()
+        # RBC 20060208
+        b = self.make_branch(relpath, format=format)
+        try:
+            return b.bzrdir.create_workingtree()
+        except errors.NotLocalUrl:
+            # new formats - catch No tree error and create
+            # a branch reference and a checkout.
+            # old formats at that point - raise TestSkipped.
+            # TODO: rbc 20060208
+            return WorkingTreeFormat2().initialize(bzrdir.BzrDir.open(relpath))
+
+    def assertIsDirectory(self, relpath, transport):
+        """Assert that relpath within transport is a directory.
+
+        This may not be possible on all transports; in that case it propagates
+        a TransportNotPossible.
+        """
+        try:
+            mode = transport.stat(relpath).st_mode
+        except errors.NoSuchFile:
+            self.fail("path %s is not a directory; no such file"
+                      % (relpath))
+        if not stat.S_ISDIR(mode):
+            self.fail("path %s is not a directory; has mode %#o"
+                      % (relpath, mode))
+
+
+class ChrootedTestCase(TestCaseWithTransport):
+    """A support class that provides readonly urls outside the local namespace.
+
+    This is done by checking if self.transport_server is a MemoryServer. if it
+    is then we are chrooted already, if it is not then an HttpServer is used
+    for readonly urls.
+
+    TODO RBC 20060127: make this an option to TestCaseWithTransport so it can
+                       be used without needed to redo it when a different 
+                       subclass is in use ?
+    """
+
+    def setUp(self):
+        super(ChrootedTestCase, self).setUp()
+        if not self.transport_server == bzrlib.transport.memory.MemoryServer:
+            self.transport_readonly_server = bzrlib.transport.http.HttpServer
+
+
+def filter_suite_by_re(suite, pattern):
+    result = TestSuite()
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.search(test.id()):
+            result.addTest(test)
+    return result
+
+
+def run_suite(suite, name='test', verbose=False, pattern=".*",
+              stop_on_failure=False, keep_output=False,
+              transport=None):
+    TestCaseInTempDir._TEST_NAME = name
+    if verbose:
+        verbosity = 2
+    else:
+        verbosity = 1
+    runner = TextTestRunner(stream=sys.stdout,
+                            descriptions=0,
+                            verbosity=verbosity)
+    runner.stop_on_failure=stop_on_failure
+    if pattern != '.*':
+        suite = filter_suite_by_re(suite, pattern)
+    result = runner.run(suite)
+    # This is still a little bogus, 
+    # but only a little. Folk not using our testrunner will
+    # have to delete their temp directories themselves.
+    test_root = TestCaseInTempDir.TEST_ROOT
+    if result.wasSuccessful() or not keep_output:
+        if test_root is not None:
+            print 'Deleting test root %s...' % test_root
+            try:
+                shutil.rmtree(test_root)
+            finally:
+                print
+    else:
+        print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT
+    return result.wasSuccessful()
+
+
+def selftest(verbose=False, pattern=".*", stop_on_failure=True,
+             keep_output=False,
+             transport=None):
+    """Run the whole test suite under the enhanced runner"""
+    global default_transport
+    if transport is None:
+        transport = default_transport
+    old_transport = default_transport
+    default_transport = transport
+    suite = test_suite()
+    try:
+        return run_suite(suite, 'testbzr', verbose=verbose, pattern=pattern,
+                     stop_on_failure=stop_on_failure, keep_output=keep_output,
+                     transport=transport)
+    finally:
+        default_transport = old_transport
+
+
+
+def test_suite():
+    """Build and return TestSuite for the whole program."""
     from doctest import DocTestSuite
-    import os
-    import shutil
-    import time
-    import sys
-    import unittest
-
-    global MODULES_TO_TEST, MODULES_TO_DOCTEST
-
-    testmod_names = \
-                  ['bzrlib.selftest.whitebox',
-                   'bzrlib.selftest.versioning',
-                   'bzrlib.selftest.testinv',
-                   'bzrlib.selftest.testmerge3',
-                   'bzrlib.selftest.testhashcache',
-                   'bzrlib.selftest.teststatus',
-                   'bzrlib.selftest.testlog',
-                   'bzrlib.selftest.blackbox',
-                   'bzrlib.selftest.testrevisionnamespaces',
-                   'bzrlib.selftest.testbranch',
-                   'bzrlib.selftest.testrevision',
-                   'bzrlib.merge_core',
-                   'bzrlib.selftest.testdiff',
+
+    global MODULES_TO_DOCTEST
+
+    testmod_names = [ \
+                   'bzrlib.tests.test_ancestry',
+                   'bzrlib.tests.test_annotate',
+                   'bzrlib.tests.test_api',
+                   'bzrlib.tests.test_bad_files',
+                   'bzrlib.tests.test_branch',
+                   'bzrlib.tests.test_bzrdir',
+                   'bzrlib.tests.test_command',
+                   'bzrlib.tests.test_commit',
+                   'bzrlib.tests.test_commit_merge',
+                   'bzrlib.tests.test_config',
+                   'bzrlib.tests.test_conflicts',
+                   'bzrlib.tests.test_decorators',
+                   'bzrlib.tests.test_diff',
+                   'bzrlib.tests.test_doc_generate',
+                   'bzrlib.tests.test_errors',
+                   'bzrlib.tests.test_escaped_store',
+                   'bzrlib.tests.test_fetch',
+                   'bzrlib.tests.test_gpg',
+                   'bzrlib.tests.test_graph',
+                   'bzrlib.tests.test_hashcache',
+                   'bzrlib.tests.test_http',
+                   'bzrlib.tests.test_identitymap',
+                   'bzrlib.tests.test_inv',
+                   'bzrlib.tests.test_knit',
+                   'bzrlib.tests.test_lockdir',
+                   'bzrlib.tests.test_lockable_files',
+                   'bzrlib.tests.test_log',
+                   'bzrlib.tests.test_merge',
+                   'bzrlib.tests.test_merge3',
+                   'bzrlib.tests.test_merge_core',
+                   'bzrlib.tests.test_missing',
+                   'bzrlib.tests.test_msgeditor',
+                   'bzrlib.tests.test_nonascii',
+                   'bzrlib.tests.test_options',
+                   'bzrlib.tests.test_osutils',
+                   'bzrlib.tests.test_permissions',
+                   'bzrlib.tests.test_plugins',
+                   'bzrlib.tests.test_progress',
+                   'bzrlib.tests.test_reconcile',
+                   'bzrlib.tests.test_repository',
+                   'bzrlib.tests.test_revision',
+                   'bzrlib.tests.test_revisionnamespaces',
+                   'bzrlib.tests.test_revprops',
+                   'bzrlib.tests.test_rio',
+                   'bzrlib.tests.test_sampler',
+                   'bzrlib.tests.test_selftest',
+                   'bzrlib.tests.test_setup',
+                   'bzrlib.tests.test_sftp_transport',
+                   'bzrlib.tests.test_smart_add',
+                   'bzrlib.tests.test_source',
+                   'bzrlib.tests.test_store',
+                   'bzrlib.tests.test_symbol_versioning',
+                   'bzrlib.tests.test_testament',
+                   'bzrlib.tests.test_textmerge',
+                   'bzrlib.tests.test_trace',
+                   'bzrlib.tests.test_transactions',
+                   'bzrlib.tests.test_transform',
+                   'bzrlib.tests.test_transport',
+                   'bzrlib.tests.test_tsort',
+                   'bzrlib.tests.test_tuned_gzip',
+                   'bzrlib.tests.test_ui',
+                   'bzrlib.tests.test_upgrade',
+                   'bzrlib.tests.test_versionedfile',
+                   'bzrlib.tests.test_weave',
+                   'bzrlib.tests.test_whitebox',
+                   'bzrlib.tests.test_workingtree',
+                   'bzrlib.tests.test_xml',
                    ]
-
-    # XXX: should also test bzrlib.merge_core, but they seem to be out
-    # of date with the code.
-
-    for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
-              bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
-        if m not in MODULES_TO_DOCTEST:
-            MODULES_TO_DOCTEST.append(m)
-
-    
-    TestCase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%-30s %s' % ('bzr binary', TestCase.BZRPATH)
-
+    test_transport_implementations = [
+        'bzrlib.tests.test_transport_implementations']
+
+    TestCase.BZRPATH = osutils.pathjoin(
+            osutils.realpath(osutils.dirname(bzrlib.__path__[0])), 'bzr')
+    print '%10s: %s' % ('bzr', osutils.realpath(sys.argv[0]))
+    print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
     print
-
     suite = TestSuite()
-
-    suite.addTest(TestLoader().loadTestsFromNames(testmod_names))
-
+    # python2.4's TestLoader.loadTestsFromNames gives very poor 
+    # errors if it fails to load a named module - no indication of what's
+    # actually wrong, just "no such module".  We should probably override that
+    # class, but for the moment just load them ourselves. (mbp 20051202)
+    loader = TestLoader()
+    from bzrlib.transport import TransportTestProviderAdapter
+    adapter = TransportTestProviderAdapter()
+    adapt_modules(test_transport_implementations, adapter, loader, suite)
+    for mod_name in testmod_names:
+        mod = _load_module_by_name(mod_name)
+        suite.addTest(loader.loadTestsFromModule(mod))
+    for package in packages_to_test():
+        suite.addTest(package.test_suite())
     for m in MODULES_TO_TEST:
-         suite.addTest(TestLoader().loadTestsFromModule(m))
-
+        suite.addTest(loader.loadTestsFromModule(m))
     for m in (MODULES_TO_DOCTEST):
         suite.addTest(DocTestSuite(m))
-
-    for p in bzrlib.plugin.all_plugins:
-        if hasattr(p, 'test_suite'):
-            suite.addTest(p.test_suite())
-
-    import bzrlib.merge_core
-    suite.addTest(unittest.makeSuite(bzrlib.merge_core.MergeTest, 'test_'))
-
-    return run_suite(suite, 'testbzr', verbose=verbose)
-
-
-
+    for name, plugin in bzrlib.plugin.all_plugins().items():
+        if getattr(plugin, 'test_suite', None) is not None:
+            suite.addTest(plugin.test_suite())
+    return suite
+
+
+def adapt_modules(mods_list, adapter, loader, suite):
+    """Adapt the modules in mods_list using adapter and add to suite."""
+    for mod_name in mods_list:
+        mod = _load_module_by_name(mod_name)
+        for test in iter_suite_tests(loader.loadTestsFromModule(mod)):
+            suite.addTests(adapter.adapt(test))
+
+
+def _load_module_by_name(mod_name):
+    parts = mod_name.split('.')
+    module = __import__(mod_name)
+    del parts[0]
+    # for historical reasons python returns the top-level module even though
+    # it loads the submodule; we need to walk down to get the one we want.
+    while parts:
+        module = getattr(module, parts.pop(0))
+    return module

=== added directory 'bzrlib/tests/blackbox'
=== added file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-04-18 02:24:14 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr.
+
+These check that it behaves properly when it's invoked through the regular
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
+"""
+
+from bzrlib.tests import (
+                          _load_module_by_name,
+                          TestCaseWithTransport,
+                          TestSuite,
+                          TestLoader,
+                          )
+import bzrlib.ui as ui
+
+
+def test_suite():
+    testmod_names = [
+                     'bzrlib.tests.blackbox.test_added',
+                     'bzrlib.tests.blackbox.test_aliases',
+                     'bzrlib.tests.blackbox.test_ancestry',
+                     'bzrlib.tests.blackbox.test_break_lock',
+                     'bzrlib.tests.blackbox.test_bound_branches',
+                     'bzrlib.tests.blackbox.test_cat',
+                     'bzrlib.tests.blackbox.test_checkout',
+                     'bzrlib.tests.blackbox.test_commit',
+                     'bzrlib.tests.blackbox.test_conflicts',
+                     'bzrlib.tests.blackbox.test_diff',
+                     'bzrlib.tests.blackbox.test_export',
+                     'bzrlib.tests.blackbox.test_find_merge_base',
+                     'bzrlib.tests.blackbox.test_help',
+                     'bzrlib.tests.blackbox.test_info',
+                     'bzrlib.tests.blackbox.test_init',
+                     'bzrlib.tests.blackbox.test_log',
+                     'bzrlib.tests.blackbox.test_logformats',
+                     'bzrlib.tests.blackbox.test_merge',
+                     'bzrlib.tests.blackbox.test_missing',
+                     'bzrlib.tests.blackbox.test_outside_wt',
+                     'bzrlib.tests.blackbox.test_pull',
+                     'bzrlib.tests.blackbox.test_push',
+                     'bzrlib.tests.blackbox.test_reconcile',
+                     'bzrlib.tests.blackbox.test_re_sign',
+                     'bzrlib.tests.blackbox.test_revert',
+                     'bzrlib.tests.blackbox.test_revno',
+                     'bzrlib.tests.blackbox.test_revision_info',
+                     'bzrlib.tests.blackbox.test_selftest',
+                     'bzrlib.tests.blackbox.test_shared_repository',
+                     'bzrlib.tests.blackbox.test_sign_my_commits',
+                     'bzrlib.tests.blackbox.test_status',
+                     'bzrlib.tests.blackbox.test_too_much',
+                     'bzrlib.tests.blackbox.test_uncommit',
+                     'bzrlib.tests.blackbox.test_update',
+                     'bzrlib.tests.blackbox.test_upgrade',
+                     'bzrlib.tests.blackbox.test_versioning',
+                     ]
+
+    suite = TestSuite()
+    loader = TestLoader()
+    for mod_name in testmod_names:
+        mod = _load_module_by_name(mod_name)
+        suite.addTest(loader.loadTestsFromModule(mod))
+    return suite
+
+
+class ExternalBase(TestCaseWithTransport):
+
+    def runbzr(self, args, retcode=0, backtick=False):
+        if isinstance(args, basestring):
+            args = args.split()
+        if backtick:
+            return self.run_bzr_captured(args, retcode=retcode)[0]
+        else:
+            return self.run_bzr_captured(args, retcode=retcode)
+
+
+class TestUIFactory(ui.UIFactory):
+    """A UI Factory for testing - hide the progress bar but emit note()s."""
+
+    def clear(self):
+        """See progress.ProgressBar.clear()."""
+
+    def finished(self):
+        """See progress.ProgressBar.finished()."""
+
+    def note(self, fmt_string, *args, **kwargs):
+        """See progress.ProgressBar.note()."""
+        print fmt_string % args
+
+    def progress_bar(self):
+        return self
+    
+    def nested_progress_bar(self):
+        return self
+
+    def update(self, message, count=None, total=None):
+        """See progress.ProgressBar.update()."""

=== added file 'bzrlib/tests/blackbox/test_added.py'
--- bzrlib/tests/blackbox/test_added.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_added.py	2006-01-19 09:11:39 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2006 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for 'bzr added', which shows newly-added files.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests.treeshape import build_tree_contents
+
+class TestAdded(TestCaseInTempDir):
+
+    def test_added(self):
+        """Test that 'added' command reports added files"""
+
+        def check_added(expected):
+            out, err = self.run_bzr_captured(['added'])
+            self.assertEquals(out, expected)
+            self.assertEquals(err, '')
+
+        def bzr(*args):
+            self.run_bzr(*args)
+
+        # in empty directory, nothing added
+        bzr('init')
+        check_added('')
+
+        # with unknown file, still nothing added
+        build_tree_contents([('a', 'contents of a\n')])
+        check_added('')
+
+        # after add, shows up in list
+        # bug report 20060119 by Nathan McCallum -- 'bzr added' causes
+        # NameError
+        bzr('add', 'a')
+        check_added('a\n')
+
+        # after commit, now no longer listed
+        bzr('commit', '-m', 'add a')
+        check_added('')

=== added file 'bzrlib/tests/blackbox/test_aliases.py'
--- bzrlib/tests/blackbox/test_aliases.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_aliases.py	2006-03-12 20:42:58 +0000
@@ -0,0 +1,67 @@
+"""Black-box tests for bzr aliases.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.trace import mutter
+from bzrlib.config import (ensure_config_dir_exists, config_filename)
+
+
+class TestAliases(TestCaseInTempDir):
+
+    def test_aliases(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        def bzr_catch_error(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[1]
+
+
+        if os.path.isfile(config_filename()):
+            # Something is wrong in environment, 
+            # we risk overwriting users config 
+            self.assert_(config_filename() + "exists, abort")
+        
+        ensure_config_dir_exists()
+        CONFIG=("[ALIASES]\n"
+                "c=cat\n"
+                "c1=cat -r 1\n"
+                "c2=cat -r 1 -r2\n")
+
+        open(config_filename(),'wb').write(CONFIG)
+
+
+        str1 = 'foo\n'
+        str2 = 'bar\n'
+
+        bzr('init')
+        open('a', 'wb').write(str1)
+        bzr('add', 'a')
+
+        bzr('commit', '-m', '1')
+
+        self.assertEquals(bzr('c', 'a'), str1)
+
+        open('a', 'wb').write(str2)
+        bzr('commit', '-m', '2')
+
+        self.assertEquals(bzr('c', 'a'), str2)
+        self.assertEquals(bzr('c1', 'a'), str1)
+        self.assertEquals(bzr('c1', '--revision', '2', 'a'), str2)
+
+        # If --no-aliases isn't working, we will not get retcode=3
+        bzr('--no-aliases', 'c', 'a', retcode=3)
+
+        # If --no-aliases breaks all of bzr, we also get retcode=3
+        # So we need to catch the output as well
+        self.assertEquals(bzr_catch_error('--no-aliases', 'c', 'a', 
+                                          retcode=None), 
+                          "bzr: ERROR: unknown command 'c'\n")
+
+        bzr('c', '-r1', '-r2', retcode=3)
+        bzr('c1', '-r1', '-r2', retcode=3)
+        bzr('c2', retcode=3)
+        bzr('c2', '-r1', retcode=3)

=== added file 'bzrlib/tests/blackbox/test_ancestry.py'
--- bzrlib/tests/blackbox/test_ancestry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_ancestry.py	2006-02-14 17:19:40 +0000
@@ -0,0 +1,40 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.builtins import merge
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.workingtree import WorkingTree
+
+
+class TestAncestry(TestCaseWithTransport):
+
+    def test_ancestry(self):
+        """Tests 'ancestry' command"""
+        a_wt = self.make_branch_and_tree('A')
+        open('A/foo', 'wb').write('1111\n')
+        a_wt.add('foo')
+        a_wt.commit('added foo',rev_id='A1')
+        self.run_bzr_captured(['branch', 'A', 'B'])
+        b_wt = WorkingTree.open('B')
+        open('B/foo','wb').write('1111\n22\n')
+        b_wt.commit('modified B/foo',rev_id='B1')
+        open('A/foo', 'wb').write('000\n1111\n')
+        a_wt.commit('modified A/foo',rev_id='A2')
+        merge(['B',-1],['B',1],this_dir='A')
+        a_wt.commit('merged B into A',rev_id='A3')
+        self.run_bzr('ancestry')

=== added file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2006-04-19 05:22:38 +0000
@@ -0,0 +1,390 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests of bound branches (binding, unbinding, commit, etc) command.
+"""
+
+import os
+from cStringIO import StringIO
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import (BzrDir,
+                           BzrDirFormat,
+                           BzrDirFormat6,
+                           BzrDirMetaFormat1,
+                           )
+from bzrlib.osutils import getcwd
+from bzrlib.workingtree import WorkingTree
+
+
+class TestLegacyFormats(TestCaseWithTransport):
+    
+    def setUp(self):
+        super(TestLegacyFormats, self).setUp()
+        self.build_tree(['master/', 'child/'])
+        self.run_bzr('init', 'master')
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirFormat6())
+        try:
+            self.run_bzr('init', 'child')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+        os.chdir('child')
+    
+    def test_bind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('bind', '../master', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % getcwd(), err)
+    
+    def test_unbind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('unbind', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % getcwd(), err)
+
+
+class TestBoundBranches(TestCaseWithTransport):
+
+    def create_branches(self):
+        bzr = self.run_bzr
+        self.build_tree(['base/', 'base/a', 'base/b'])
+
+        branch = self.init_meta_branch('base')
+        tree = branch.bzrdir.open_workingtree()
+        tree.lock_write()
+        tree.add(['a', 'b'])
+        tree.commit('init')
+        tree.unlock()
+
+        self.run_bzr('checkout', 'base', 'child')
+
+        self.check_revno(1, 'child')
+        d = BzrDir.open('child')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+
+    def check_revno(self, val, loc='.'):
+        self.assertEqual(
+            val, len(BzrDir.open(loc).open_branch().revision_history()))
+
+    def test_simple_binding(self):
+        self.build_tree(['base/', 'base/a', 'base/b'])
+
+        self.init_meta_branch('base')
+        self.run_bzr('add', 'base')
+        self.run_bzr('commit', '-m', 'init', 'base')
+
+        self.run_bzr('branch', 'base', 'child')
+
+        os.chdir('child')
+        self.run_bzr('bind', '../base')
+
+        d = BzrDir.open('')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+
+        self.run_bzr('unbind')
+        self.assertEqual(None, d.open_branch().get_master_branch())
+
+        self.run_bzr('unbind', retcode=3)
+
+    def init_meta_branch(self, path):
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            return BzrDir.create_branch_convenience(
+                path, BzrDirMetaFormat1())
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+
+    def test_bound_commit(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        open('a', 'wb').write('new contents\n')
+        bzr('commit', '-m', 'child')
+
+        self.check_revno(2)
+
+        # Make sure it committed on the parent
+        self.check_revno(2, '../base')
+
+    def test_bound_fail(self):
+        # Make sure commit fails if out of date.
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('base')
+        open('a', 'wb').write('new base contents\n')
+        bzr('commit', '-m', 'base')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        self.check_revno(1)
+        open('b', 'wb').write('new b child contents\n')
+        bzr('commit', '-m', 'child', retcode=3)
+        self.check_revno(1)
+
+        bzr('update')
+        self.check_revno(2)
+
+        bzr('commit', '-m', 'child')
+        self.check_revno(3)
+        self.check_revno(3, '../base')
+
+    def test_double_binding(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        bzr('branch', 'child', 'child2')
+        os.chdir('child2')
+
+        # Double binding succeeds, but committing to child2 should fail
+        bzr('bind', '../child')
+
+        bzr('commit', '-m', 'child2', '--unchanged', retcode=3)
+
+    def test_unbinding(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('base')
+        open('a', 'wb').write('new base contents\n')
+        bzr('commit', '-m', 'base')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        open('b', 'wb').write('new b child contents\n')
+        self.check_revno(1)
+        bzr('commit', '-m', 'child', retcode=3)
+        self.check_revno(1)
+        bzr('unbind')
+        bzr('commit', '-m', 'child')
+        self.check_revno(2)
+
+        bzr('bind', retcode=3)
+
+    def test_commit_remote_bound(self):
+        # It is not possible to commit to a branch
+        # which is bound to a branch which is bound
+        bzr = self.run_bzr
+        self.create_branches()
+        bzr('branch', 'base', 'newbase')
+        os.chdir('base')
+        
+        # There is no way to know that B has already
+        # been bound by someone else, otherwise it
+        # might be nice if this would fail
+        bzr('bind', '../newbase')
+
+        os.chdir('../child')
+        bzr('commit', '-m', 'failure', '--unchanged', retcode=3)
+
+    def test_pull_updates_both(self):
+        bzr = self.run_bzr
+        self.create_branches()
+        bzr('branch', 'base', 'newchild')
+        os.chdir('newchild')
+        open('b', 'wb').write('newchild b contents\n')
+        bzr('commit', '-m', 'newchild')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        # The pull should succeed, and update
+        # the bound parent branch
+        bzr('pull', '../newchild')
+        self.check_revno(2)
+
+        self.check_revno(2, '../base')
+
+    def test_bind_diverged(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        bzr('unbind')
+
+        bzr('commit', '-m', 'child', '--unchanged')
+        self.check_revno(2)
+
+        os.chdir('../base')
+        self.check_revno(1)
+        bzr('commit', '-m', 'base', '--unchanged')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        # These branches have diverged
+        bzr('bind', '../base', retcode=3)
+
+        # TODO: In the future, this might require actual changes
+        # to have occurred, rather than just a new revision entry
+        bzr('merge', '../base')
+        bzr('commit', '-m', 'merged')
+        self.check_revno(3)
+
+        # After a merge, trying to bind again should succeed
+        # by pushing the new change to base
+        bzr('bind', '../base')
+        self.check_revno(3)
+        self.check_revno(3, '../base')
+
+        # After binding, the revision history should be identical
+        child_rh = bzr('revision-history')[0]
+        os.chdir('../base')
+        base_rh = bzr('revision-history')[0]
+        self.assertEquals(child_rh, base_rh)
+
+    def test_bind_parent_ahead(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        bzr('unbind')
+
+        os.chdir('../base')
+        bzr('commit', '-m', 'base', '--unchanged')
+
+        os.chdir('../child')
+        self.check_revno(1)
+        bzr('bind', '../base')
+
+        self.check_revno(2)
+        bzr('unbind')
+
+        # Check and make sure it also works if parent is ahead multiple
+        os.chdir('../base')
+        bzr('commit', '-m', 'base 3', '--unchanged')
+        bzr('commit', '-m', 'base 4', '--unchanged')
+        bzr('commit', '-m', 'base 5', '--unchanged')
+        self.check_revno(5)
+
+        os.chdir('../child')
+        self.check_revno(2)
+        bzr('bind', '../base')
+        self.check_revno(5)
+
+    def test_bind_child_ahead(self):
+        # test binding when the master branches history is a prefix of the 
+        # childs
+        bzr = self.run_bzr
+        self.create_branches()
+        return
+
+        os.chdir('child')
+        bzr('unbind')
+        bzr('commit', '-m', 'child', '--unchanged')
+        self.check_revno(2)
+        self.check_revno(1, '../base')
+
+        bzr('bind', '../base')
+        self.check_revno(2, '../base')
+
+        # Check and make sure it also works if child is ahead multiple
+        bzr('unbind')
+        bzr('commit', '-m', 'child 3', '--unchanged')
+        bzr('commit', '-m', 'child 4', '--unchanged')
+        bzr('commit', '-m', 'child 5', '--unchanged')
+        self.check_revno(5)
+
+        self.check_revno(2, '../base')
+        bzr('bind', '../base')
+        self.check_revno(5, '../base')
+
+    def test_commit_after_merge(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        # We want merge to be able to be a local only
+        # operation, because it can be without violating
+        # the binding invariants.
+        # But we can't fail afterwards
+
+        bzr('branch', 'child', 'other')
+
+        os.chdir('other')
+        open('c', 'wb').write('file c\n')
+        bzr('add', 'c')
+        bzr('commit', '-m', 'adding c')
+        new_rev_id = bzr('revision-history')[0].strip().split('\n')[-1]
+
+        os.chdir('../child')
+        bzr('merge', '../other')
+
+        self.failUnlessExists('c')
+        tree = WorkingTree.open('.')
+        self.assertEqual([new_rev_id], tree.pending_merges())
+
+        # Make sure the local branch has the installed revision
+        bzr('cat-revision', new_rev_id)
+        
+        # And make sure that the base tree does not
+        os.chdir('../base')
+        bzr('cat-revision', new_rev_id, retcode=3)
+
+        # Commit should succeed, and cause merged revisions to
+        # be pulled into base
+        os.chdir('../child')
+        bzr('commit', '-m', 'merge other')
+
+        self.check_revno(2)
+
+        os.chdir('../base')
+        self.check_revno(2)
+
+        bzr('cat-revision', new_rev_id)
+
+    def test_pull_overwrite_fails(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        bzr('branch', 'child', 'other')
+        
+        os.chdir('other')
+        open('a', 'wb').write('new contents\n')
+        bzr('commit', '-m', 'changed a')
+        self.check_revno(2)
+        open('a', 'ab').write('and then some\n')
+        bzr('commit', '-m', 'another a')
+        self.check_revno(3)
+        open('a', 'ab').write('and some more\n')
+        bzr('commit', '-m', 'yet another a')
+        self.check_revno(4)
+
+        os.chdir('../child')
+        open('a', 'wb').write('also changed a\n')
+        bzr('commit', '-m', 'child modified a')
+
+        self.check_revno(2)
+        self.check_revno(2, '../base')
+
+        # It might be possible that we want pull --overwrite to
+        # actually succeed.
+        # If we want it, just change this test to make sure that 
+        # both base and child are updated properly
+        bzr('pull', '--overwrite', '../other', retcode=3)
+
+        # It should fail without changing the local revision
+        self.check_revno(2)
+        self.check_revno(2, '../base')
+
+    # TODO: jam 20051230 Test that commit & pull fail when the branch we 
+    #       are bound to is not available
+
+

=== added file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2006-03-03 02:32:53 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for lock-breaking user interface"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.tests import TestCaseInTempDir
+
+class TestBreakLock(TestCaseInTempDir):
+    def test_break_lock_help(self):
+        self.run_bzr('break-lock', '--help')
+        # shouldn't fail
+
+    def test_show_no_lock(self):
+        wt = BzrDir.create_standalone_workingtree('.')
+        # out, err = self.run_bzr('break-lock', '--show', '.', retcode=3)
+        # shouldn't see any information
+        # self.assertContainsRe(err, 'not locked')

=== added file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2005-12-01 16:42:05 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr cat.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestCat(TestCaseInTempDir):
+
+    def test_cat(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        bzr('init')
+        open('a', 'wb').write('foo\n')
+        bzr('add', 'a')
+
+        # 'bzr cat' without an option should cat the last revision
+        bzr('cat', 'a', retcode=3)
+
+        bzr('commit', '-m', '1')
+        open('a', 'wb').write('baz\n')
+
+        self.assertEquals(bzr('cat', 'a'), 'foo\n')
+
+        bzr('commit', '-m', '2')
+        self.assertEquals(bzr('cat', 'a'), 'baz\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '1'), 'foo\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '-1'), 'baz\n')
+
+        rev_id = bzr('revision-history').strip().split('\n')[-1]
+
+        self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
+

=== added file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2006-04-03 04:20:11 +0000
@@ -0,0 +1,101 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the 'checkout' CLI command."""
+
+from cStringIO import StringIO
+import os
+import re
+import shutil
+import sys
+
+import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestCheckout(ExternalBase):
+    
+    def setUp(self):
+        super(TestCheckout, self).setUp()
+        tree = bzrdir.BzrDir.create_standalone_workingtree('branch')
+        tree.commit('1', rev_id='1', allow_pointless=True)
+        self.build_tree(['branch/added_in_2'])
+        tree.add('added_in_2')
+        tree.commit('2', rev_id='2')
+
+    def test_checkout_makes_bound_branch(self):
+        self.runbzr('checkout branch checkout')
+        # if we have a checkout, the branch base should be 'branch'
+        source = bzrdir.BzrDir.open('branch')
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual(source.open_branch().bzrdir.root_transport.base,
+                         result.open_branch().get_bound_location())
+
+    def test_checkout_light_makes_checkout(self):
+        self.runbzr('checkout --lightweight branch checkout')
+        # if we have a checkout, the branch base should be 'branch'
+        source = bzrdir.BzrDir.open('branch')
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual(source.open_branch().bzrdir.root_transport.base,
+                         result.open_branch().bzrdir.root_transport.base)
+
+    def test_checkout_dash_r(self):
+        self.runbzr('checkout -r -2 branch checkout')
+        # the working tree should now be at revision '1' with the content
+        # from 1.
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual('1', result.open_workingtree().last_revision())
+        self.failIfExists('checkout/added_in_2')
+
+    def test_checkout_light_dash_r(self):
+        self.runbzr('checkout --lightweight -r -2 branch checkout')
+        # the working tree should now be at revision '1' with the content
+        # from 1.
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual('1', result.open_workingtree().last_revision())
+        self.failIfExists('checkout/added_in_2')
+
+    def test_checkout_reconstitutes_working_trees(self):
+        # doing a 'bzr checkout' in the directory of a branch with no tree
+        # or a 'bzr checkout path' with path the name of a directory with
+        # a branch with no tree will reconsistute the tree.
+        os.mkdir('treeless-branch')
+        branch = bzrdir.BzrDir.create_branch_convenience(
+            'treeless-branch',
+            force_new_tree=False,
+            format=bzrdir.BzrDirMetaFormat1())
+        # check no tree was created
+        self.assertRaises(errors.NoWorkingTree, branch.bzrdir.open_workingtree)
+        out, err = self.run_bzr('checkout', 'treeless-branch')
+        # we should have a tree now
+        branch.bzrdir.open_workingtree()
+        # with no diff
+        out, err = self.run_bzr('diff', 'treeless-branch')
+
+        # now test with no parameters
+        branch = bzrdir.BzrDir.create_branch_convenience(
+            '.',
+            force_new_tree=False,
+            format=bzrdir.BzrDirMetaFormat1())
+        # check no tree was created
+        self.assertRaises(errors.NoWorkingTree, branch.bzrdir.open_workingtree)
+        out, err = self.run_bzr('checkout')
+        # we should have a tree now
+        branch.bzrdir.open_workingtree()
+        # with no diff
+        out, err = self.run_bzr('diff')

=== added file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-03-20 20:02:21 +0000
@@ -0,0 +1,127 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for the commit CLI of bzr."""
+
+from cStringIO import StringIO
+import os
+import re
+import shutil
+import sys
+
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.errors import BzrCommandError
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestCommit(ExternalBase):
+
+    def test_05_empty_commit(self):
+        """Commit of tree with no versioned files should fail"""
+        # If forced, it should succeed, but this is not tested here.
+        self.runbzr("init")
+        self.build_tree(['hello.txt'])
+        self.runbzr("commit -m empty", retcode=3)
+
+    def test_10_verbose_commit(self):
+        """Add one file and examine verbose commit output"""
+        self.runbzr("init")
+        self.build_tree(['hello.txt'])
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 1.\n',
+                         err)
+
+    def test_15_verbose_commit_with_unknown(self):
+        """Unknown files should not be listed by default in verbose output"""
+        # Is that really the best policy?
+        self.runbzr("init")
+        self.build_tree(['hello.txt', 'extra.txt'])
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 1.\n',
+                         err)
+
+    def test_16_verbose_commit_with_unchanged(self):
+        """Unchanged files should not be listed by default in verbose output"""
+        self.runbzr("init")
+        self.build_tree(['hello.txt', 'unchanged.txt'])
+        self.runbzr('add unchanged.txt')
+        self.runbzr('commit -m unchanged unchanged.txt')
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 2.\n',
+                         err)
+
+    def test_empty_commit_message(self):
+        self.runbzr("init")
+        file('foo.c', 'wt').write('int main() {}')
+        self.runbzr(['add', 'foo.c'])
+        self.runbzr(["commit", "-m", ""] , retcode=3)
+
+    def test_other_branch_commit(self):
+        # this branch is to ensure consistent behaviour, whether we're run
+        # inside a branch, or not.
+        os.mkdir('empty_branch')
+        os.chdir('empty_branch')
+        self.runbzr('init')
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.runbzr('init')
+        file('foo.c', 'wt').write('int main() {}')
+        file('bar.c', 'wt').write('int main() {}')
+        os.chdir('..')
+        self.runbzr(['add', 'branch/foo.c'])
+        self.runbzr(['add', 'branch'])
+        # can't commit files in different trees; sane error
+        self.runbzr('commit -m newstuff branch/foo.c .', retcode=3)
+        self.runbzr('commit -m newstuff branch/foo.c')
+        self.runbzr('commit -m newstuff branch')
+        self.runbzr('commit -m newstuff branch', retcode=3)
+
+    def test_out_of_date_tree_commit(self):
+        # check we get an error code and a clear message committing with an out
+        # of date checkout
+        self.make_branch_and_tree('branch')
+        # make a checkout
+        self.runbzr('checkout --lightweight branch checkout')
+        # commit to the original branch to make the checkout out of date
+        self.runbzr('commit --unchanged -m message branch')
+        # now commit to the checkout should emit
+        # ERROR: Out of date with the branch, 'bzr update' is suggested
+        output = self.runbzr('commit --unchanged -m checkout_message '
+                             'checkout', retcode=3)
+        self.assertEqual(output,
+                         ('',
+                          "bzr: ERROR: Working tree is out of date, please run "
+                          "'bzr update'.\n"))
+
+    def test_local_commit_unbound(self):
+        # a --local commit on an unbound branch is an error
+        self.make_branch_and_tree('.')
+        out, err = self.run_bzr('commit', '--local', retcode=3)
+        self.assertEqualDiff('', out)
+        self.assertEqualDiff('bzr: ERROR: Cannot perform local-only commits '
+                             'on unbound branches.\n', err)

=== added file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2006-04-12 04:52:02 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.workingtree import WorkingTree
+from bzrlib.tests.blackbox import ExternalBase
+
+class TestConflicts(ExternalBase):
+    def setUp(self):
+        super(ExternalBase, self).setUp()
+        try:
+            os.mkdir('a')
+        except:
+            raise os.getcwd()
+        os.chdir('a')
+        self.runbzr('init')
+        file('myfile', 'wb').write('contentsa\n')
+        file('my_other_file', 'wb').write('contentsa\n')
+        self.runbzr('add')
+        self.runbzr('commit -m new')
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        file('myfile', 'wb').write('contentsb\n')
+        file('my_other_file', 'wb').write('contentsb\n')
+        self.runbzr('commit -m change')
+        os.chdir('../a')
+        file('myfile', 'wb').write('contentsa2\n')
+        file('my_other_file', 'wb').write('contentsa2\n')
+        self.runbzr('commit -m change')
+        self.runbzr('merge ../b', retcode=1)
+
+    def test_conflicts(self):
+        conflicts = self.runbzr('conflicts', backtick=True)
+        self.assertEqual(len(conflicts.splitlines()), 2)
+
+    def test_resolve(self):
+        self.runbzr('resolve', retcode=3)
+        self.runbzr('resolve myfile')
+        conflicts = self.runbzr('conflicts', backtick=True)
+        self.assertEqual(len(conflicts.splitlines()), 1)
+        self.runbzr('resolve my_other_file')
+        conflicts = self.runbzr('conflicts', backtick=True)
+        self.assertEqual(len(conflicts.splitlines()), 0)
+
+    def test_resolve_all(self):
+        self.runbzr('resolve --all')
+        conflicts = self.runbzr('conflicts', backtick=True)
+        self.assertEqual(len(conflicts.splitlines()), 0)
+
+    def test_resolve_in_subdir(self):
+        """resolve when run from subdirectory should handle relative paths"""
+        orig_dir = os.getcwdu()
+        try:
+            os.mkdir("subdir")
+            os.chdir("subdir")
+            self.runbzr("resolve ../myfile")
+            os.chdir("../../b")
+            self.runbzr("resolve ../a/myfile")
+            wt = WorkingTree.open_containing('.')[0]
+            conflicts = wt.conflicts()
+            if not conflicts.is_empty():
+                self.fail("tree still contains conflicts: %r" % conflicts)
+        finally:
+            os.chdir(orig_dir)

=== added file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-04-13 05:51:35 +0000
@@ -0,0 +1,166 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr diff.
+"""
+
+import os
+
+import bzrlib
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestDiff(ExternalBase):
+
+    def make_example_branch(test):
+        # FIXME: copied from test_too_much -- share elsewhere?
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_diff(self):
+        self.make_example_branch()
+        file('hello', 'wt').write('hello world!')
+        self.runbzr('commit -m fixing hello')
+        output = self.runbzr('diff -r 2..3', backtick=1, retcode=1)
+        self.assert_('\n+hello world!' in output)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1, retcode=1)
+        self.assert_('\n+baz' in output)
+        file('moo', 'wb').write('moo')
+        self.runbzr('add moo')
+        os.unlink('moo')
+        self.runbzr('diff')
+
+    def test_diff_nonexistent(self):
+        # Get an error from a file that does not exist at all
+        # (Malone #3619)
+        self.make_example_branch()
+        out, err = self.runbzr('diff does-not-exist', retcode=3)
+        self.assertContainsRe(err, 'not versioned.*does-not-exist')
+
+    def test_diff_unversioned(self):
+        # Get an error when diffing a non-versioned file.
+        # (Malone #3619)
+        self.make_example_branch()
+        self.build_tree(['unversioned-file'])
+        out, err = self.runbzr('diff unversioned-file', retcode=3)
+        self.assertContainsRe(err, 'not versioned.*unversioned-file')
+
+    # TODO: What should diff say for a file deleted in working tree?
+
+    def example_branches(self):
+        self.build_tree(['branch1/', 'branch1/file'], line_endings='binary')
+        self.capture('init branch1')
+        self.capture('add branch1/file')
+        self.run_bzr_captured(['commit', '-m', 'add file', 'branch1'])
+        self.capture('branch branch1 branch2')
+        print >> open('branch2/file', 'wb'), 'new content'
+        self.run_bzr_captured(['commit', '-m', 'update file', 'branch2'])
+
+    def test_diff_branches(self):
+        self.example_branches()
+        # should open branch1 and diff against branch2, 
+        output = self.run_bzr_captured(['diff', '-r', 'branch:branch2', 
+                                        'branch1'],
+                                       retcode=1)
+        self.assertEquals(("=== modified file 'a/file'\n"
+                           "--- a/file\t\n"
+                           "+++ b/file\t\n"
+                           "@@ -1,1 +1,1 @@\n"
+                           "-new content\n"
+                           "+contents of branch1/file\n"
+                           "\n", ''), output)
+        output = self.run_bzr_captured(['diff', 'branch2', 'branch1'],
+                                       retcode=1)
+        self.assertEqualDiff(("=== modified file 'a/file'\n"
+                              "--- a/file\t\n"
+                              "+++ b/file\t\n"
+                              "@@ -1,1 +1,1 @@\n"
+                              "-new content\n"
+                              "+contents of branch1/file\n"
+                              "\n", ''), output)
+
+    def example_branch2(self):
+        self.build_tree(['branch1/', 'branch1/file1'], line_endings='binary')
+        self.capture('init branch1')
+        self.capture('add branch1/file1')
+        print >> open('branch1/file1', 'wb'), 'original line'
+        self.run_bzr_captured(['commit', '-m', 'first commit', 'branch1'])
+        
+        print >> open('branch1/file1', 'wb'), 'repo line'
+        self.run_bzr_captured(['commit', '-m', 'second commit', 'branch1'])
+
+    def test_diff_to_working_tree(self):
+        self.example_branch2()
+        
+        print >> open('branch1/file1', 'wb'), 'new line'
+        output = self.run_bzr_captured(['diff', '-r', '1..', 'branch1'], retcode=1)
+        self.assertTrue('\n-original line\n+new line\n' in output[0])
+
+
+class TestCheckoutDiff(TestDiff):
+
+    def make_example_branch(self):
+        super(TestCheckoutDiff, self).make_example_branch()
+        self.runbzr('checkout . checkout')
+        os.chdir('checkout')
+
+    def example_branch2(self):
+        super(TestCheckoutDiff, self).example_branch2()
+        os.mkdir('checkouts')
+        self.runbzr('checkout branch1 checkouts/branch1')
+        os.chdir('checkouts')
+
+    def example_branches(self):
+        super(TestCheckoutDiff, self).example_branches()
+        os.mkdir('checkouts')
+        self.runbzr('checkout branch1 checkouts/branch1')
+        self.runbzr('checkout branch2 checkouts/branch2')
+        os.chdir('checkouts')
+
+
+class TestDiffLabels(TestDiff):
+
+    def test_diff_label_removed(self):
+        super(TestDiffLabels, self).make_example_branch()
+        self.runbzr('remove hello')
+        diff = self.run_bzr_captured(['diff'], retcode=1)
+        self.assertTrue("=== removed file 'a/hello'" in diff[0])
+
+    def test_diff_label_added(self):
+        super(TestDiffLabels, self).make_example_branch()
+        file('barbar', 'wt').write('barbar')
+        self.runbzr('add barbar')
+        diff = self.run_bzr_captured(['diff'], retcode=1)
+        self.assertTrue("=== added file 'b/barbar'" in diff[0])
+
+    def test_diff_label_modified(self):
+        super(TestDiffLabels, self).make_example_branch()
+        file('hello', 'wt').write('barbar')
+        diff = self.run_bzr_captured(['diff'], retcode=1)
+        self.assertTrue("=== modified file 'a/hello'" in diff[0])
+
+    def test_diff_label_renamed(self):
+        super(TestDiffLabels, self).make_example_branch()
+        self.runbzr('rename hello gruezi')
+        diff = self.run_bzr_captured(['diff'], retcode=1)
+        self.assertTrue("=== renamed file 'a/hello' => 'b/gruezi'" in diff[0])

=== added file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export.py	2005-12-30 03:25:20 +0000
@@ -0,0 +1,74 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr export.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+import tarfile, zipfile
+
+class TestExport(ExternalBase):
+
+    def test_tar_export(self):
+
+        os.mkdir('tar')
+        os.chdir('tar')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.tar.gz')
+        ball = tarfile.open('test.tar.gz')
+        for m in ball.getnames():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'tar export contains .bzrignore')
+
+    def test_zip_export(self):
+
+        os.mkdir('zip')
+        os.chdir('zip')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.zip')
+        ball = zipfile.ZipFile('test.zip')
+        for m in ball.namelist():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'zip export contains .bzrignore')
+
+    def test_dir_export(self):
+
+        os.mkdir('dir')
+        os.chdir('dir')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export direxport')
+
+        files = sorted(os.listdir('direxport'))
+        self.assertEqual(['a'], files, 'dir contains .bzrignore')

=== added file 'bzrlib/tests/blackbox/test_find_merge_base.py'
--- bzrlib/tests/blackbox/test_find_merge_base.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_find_merge_base.py	2006-02-13 01:28:03 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+from bzrlib.tests.blackbox import ExternalBase
+import os
+
+
+class TestFindMergeBase(ExternalBase):
+    def test_find_merge_base(self):
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m foo --unchanged')
+        self.runbzr('branch . ../b')
+        q = self.run_bzr('find-merge-base', '.', '../b', backtick=True)
+        self.runbzr('commit -m bar --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m baz --unchanged')
+        r = self.run_bzr('find-merge-base', '.', '../a', backtick=True)
+        self.assertEqual(q, r)
+        

=== added file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_help.py	2006-03-23 19:00:53 +0000
@@ -0,0 +1,59 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr help.
+"""
+
+
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestHelp(ExternalBase):
+
+    def test_help_basic(self):
+        for cmd in ['--help', 'help', '-h', '-?']:
+            output = self.runbzr(cmd)[0]
+            line1 = output.split('\n')[0]
+            if not line1.startswith('Bazaar-NG'):
+                self.fail("bad output from bzr %s:\n%r" % (cmd, output))
+        # see https://launchpad.net/products/bzr/+bug/35940, -h doesn't work
+
+    def test_help_commands(self):
+        dash_help  = self.runbzr('--help commands')[0]
+        commands   = self.runbzr('help commands')[0]
+        long_help  = self.runbzr('help --long')[0]
+        qmark_long = self.runbzr('? --long')[0]
+        qmark_cmds = self.runbzr('? commands')[0]
+        self.assertEquals(dash_help, commands)
+        self.assertEquals(dash_help, long_help)
+        self.assertEquals(dash_help, qmark_long)
+        self.assertEquals(dash_help, qmark_cmds)
+
+    def test_help_detail(self):
+        dash_h  = self.runbzr('commit -h')[0]
+        help_x  = self.runbzr('help commit')[0]
+        qmark_x = self.runbzr('help commit')[0]
+        self.assertEquals(dash_h, help_x)
+        self.assertEquals(dash_h, qmark_x)
+
+    def test_help_help(self):
+        help = self.runbzr('help help')[0]
+        qmark = self.runbzr('? ?')[0]
+        self.assertEquals(help, qmark)
+        for line in help.split('\n'):
+            if '--long' in line:
+                self.assertTrue('show help on all commands' in line)

=== added file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-04-11 01:28:20 +0000
@@ -0,0 +1,950 @@
+# Copyright (C) 2006 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for the info command of bzr."""
+
+
+import bzrlib
+
+
+from bzrlib.osutils import format_date
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestInfo(ExternalBase):
+
+    def test_info_standalone(self):
+        transport = self.get_transport()
+
+        # Create initial standalone branch
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirFormat6())
+        tree1 = self.make_branch_and_tree('standalone')
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        self.build_tree(['standalone/a'])
+        tree1.add('a')
+        branch1 = tree1.branch
+        out, err = self.runbzr('info standalone')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+In the working tree:
+         0 unchanged
+         0 modified
+         1 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % branch1.bzrdir.root_transport.base, out)
+        self.assertEqual('', err)
+        tree1.commit('commit one')
+        rev = branch1.repository.get_revision(branch1.revision_history()[0])
+        datestring_first = format_date(rev.timestamp, rev.timezone)
+
+        # Branch standalone with push location
+        branch2 = branch1.bzrdir.sprout('branch').open_branch()
+        branch2.set_push_location(branch1.bzrdir.root_transport.base)
+        out, err = self.runbzr('info branch --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+       parent branch: %s
+      push to branch: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch2.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Branch and bind to standalone, needs upgrade to metadir
+        # (creates backup as unknown)
+        # XXX: I can't get this to work through API
+        self.runbzr('branch standalone bound')
+        #branch3 = branch1.bzrdir.sprout('bound').open_branch()
+        self.runbzr('upgrade --format=metadir bound')
+        #bzrlib.upgrade.upgrade('bound', 'metadir')
+        branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
+        branch3.bind(branch1)
+        out, err = self.runbzr('info bound')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+       parent branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         1 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Checkout standalone (same as above, but does not have parent set)
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout')
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch4.bind(branch1)
+        branch4.bzrdir.open_workingtree().update()
+        out, err = self.runbzr('info checkout --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch4.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Lightweight checkout (same as above, different branch and repository)
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        transport.mkdir('lightcheckout')
+        dir5 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('lightcheckout')
+        bzrlib.branch.BranchReferenceFormat().initialize(dir5, branch1)
+        dir5.create_workingtree()
+        tree5 = dir5.open_workingtree()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch5 = tree5.branch
+        out, err = self.runbzr('info lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+       checkout root: %s
+  checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (tree5.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Update initial standalone branch
+        self.build_tree(['standalone/b'])
+        tree1.add('b')
+        tree1.commit('commit two')
+        rev = branch1.repository.get_revision(branch1.revision_history()[-1])
+        datestring_last = format_date(rev.timestamp, rev.timezone)
+
+        # Out of date branched standalone branch will not be detected
+        out, err = self.runbzr('info branch')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+       parent branch: %s
+      push to branch: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch2.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Out of date bound branch
+        out, err = self.runbzr('info bound')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+       parent branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Branch is out of date: missing 1 revision.
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         1 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Out of date checkout
+        out, err = self.runbzr('info checkout')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Branch is out of date: missing 1 revision.
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch4.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Out of date lightweight checkout
+        out, err = self.runbzr('info lightcheckout --verbose')
+        self.assertEqualDiff(
+"""Location:
+       checkout root: %s
+  checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+Working tree is out of date: missing 1 revision.
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         2 revisions
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         2 revisions
+         0 KiB
+""" % (tree5.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_last), out)
+        self.assertEqual('', err)
+
+    def test_info_shared_repository(self):
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        transport = self.get_transport()
+
+        # Create shared repository
+        repo = self.make_repository('repo', shared=True)
+        repo.set_make_working_trees(False)
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+    repository: Weave repository format 7
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % repo.bzrdir.root_transport.base, out)
+        self.assertEqual('', err)
+
+        # Create branch inside shared repository
+        repo.bzrdir.root_transport.mkdir('branch')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch')
+        out, err = self.runbzr('info repo/branch')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Create lightweight checkout
+        transport.mkdir('tree')
+        transport.mkdir('tree/lightcheckout')
+        dir2 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
+        bzrlib.branch.BranchReferenceFormat().initialize(dir2, branch1)
+        dir2.create_workingtree()
+        tree2 = dir2.open_workingtree()
+        branch2 = tree2.branch
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+       checkout root: %s
+  checkout of branch: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (tree2.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Create normal checkout
+        branch3 = bzrlib.bzrdir.BzrDir.create_branch_convenience('tree/checkout')
+        branch3.bind(branch1)
+        tree3 = branch3.bzrdir.open_workingtree()
+        tree3.update()
+        out, err = self.runbzr('info tree/checkout --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+         0 committers
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (branch3.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Update lightweight checkout
+        self.build_tree(['tree/lightcheckout/a'])
+        tree2.add('a')
+        tree2.commit('commit one')
+        rev = repo.get_revision(branch2.revision_history()[0])
+        datestring_first = format_date(rev.timestamp, rev.timezone)
+        out, err = self.runbzr('info tree/lightcheckout --verbose')
+        self.assertEqualDiff(
+"""Location:
+       checkout root: %s
+  checkout of branch: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (tree2.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Out of date checkout
+        out, err = self.runbzr('info tree/checkout')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Branch is out of date: missing 1 revision.
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (tree3.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Update checkout
+        tree3.update()
+        self.build_tree(['tree/checkout/b'])
+        tree3.add('b')
+        out, err = self.runbzr('info tree/checkout --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+     bound to branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         1 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (tree3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+        tree3.commit('commit two')
+
+        # Out of date lightweight checkout
+        rev = repo.get_revision(branch1.revision_history()[-1])
+        datestring_last = format_date(rev.timestamp, rev.timezone)
+        out, err = self.runbzr('info tree/lightcheckout --verbose')
+        self.assertEqualDiff(
+"""Location:
+       checkout root: %s
+  checkout of branch: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Working tree is out of date: missing 1 revision.
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         2 revisions
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         2 revisions
+         0 KiB
+""" % (tree2.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base,
+       datestring_first, datestring_last), out)
+        self.assertEqual('', err)
+
+        # Show info about shared branch
+        out, err = self.runbzr('info repo/branch --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+Branch history:
+         2 revisions
+         1 committer
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         2 revisions
+         0 KiB
+""" % (branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base,
+       datestring_first, datestring_last), out)
+        self.assertEqual('', err)
+
+        # Show info about repository with revisions
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+    repository: Weave repository format 7
+
+Revision store:
+         2 revisions
+         0 KiB
+""" % repo.bzrdir.root_transport.base, out)
+        self.assertEqual('', err)
+
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_info_shared_repository_with_trees(self):
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        transport = self.get_transport()
+
+        # Create shared repository with working trees
+        repo = self.make_repository('repo', shared=True)
+        repo.set_make_working_trees(True)
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+    repository: Weave repository format 7
+
+Create working tree for new branches inside the repository.
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % repo.bzrdir.root_transport.base, out)
+        self.assertEqual('', err)
+
+        # Create two branches
+        repo.bzrdir.root_transport.mkdir('branch1')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1')
+        branch2 = branch1.bzrdir.sprout('repo/branch2').open_branch()
+
+        # Empty first branch
+        out, err = self.runbzr('info repo/branch1 --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+         0 committers
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (branch1.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Update first branch
+        self.build_tree(['repo/branch1/a'])
+        tree1 = branch1.bzrdir.open_workingtree()
+        tree1.add('a')
+        tree1.commit('commit one')
+        rev = repo.get_revision(branch1.revision_history()[0])
+        datestring_first = format_date(rev.timestamp, rev.timezone)
+        out, err = self.runbzr('info repo/branch1')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch1.bzrdir.root_transport.base, repo.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Out of date second branch
+        out, err = self.runbzr('info repo/branch2 --verbose')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+       parent branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+         0 committers
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch2.bzrdir.root_transport.base, repo.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base), out)
+        self.assertEqual('', err)
+
+        # Update second branch
+        tree2 = branch2.bzrdir.open_workingtree()
+        tree2.pull(branch1)
+        out, err = self.runbzr('info repo/branch2')
+        self.assertEqualDiff(
+"""Location:
+         branch root: %s
+   shared repository: %s
+       parent branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Weave repository format 7
+
+In the working tree:
+         1 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         1 revision
+         0 days old
+   first revision: %s
+  latest revision: %s
+
+Revision store:
+         1 revision
+         0 KiB
+""" % (branch2.bzrdir.root_transport.base,
+       repo.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first), out)
+        self.assertEqual('', err)
+
+        # Show info about repository with revisions
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+   shared repository: %s
+
+Format:
+       control: Meta directory format 1
+    repository: Weave repository format 7
+
+Create working tree for new branches inside the repository.
+
+Revision store:
+         1 revision
+         0 KiB
+""" % repo.bzrdir.root_transport.base, out)
+        self.assertEqual('', err)
+
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)

=== added file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_init.py	2006-04-18 05:08:30 +0000
@@ -0,0 +1,83 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Test "bzr init"""
+
+import os
+
+from bzrlib.bzrdir import BzrDirMetaFormat1
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestInit(ExternalBase):
+
+    def test_init_with_format(self):
+        # Verify bzr init --format constructs something plausible
+        t = self.get_transport()
+        self.runbzr('init --format metadir')
+        self.assertIsDirectory('.bzr', t)
+        self.assertIsDirectory('.bzr/checkout', t)
+        self.assertIsDirectory('.bzr/checkout/lock', t)
+
+    def test_init_weave(self):
+        # --format=weave should be accepted to allow interoperation with
+        # old releases when desired.
+        out, err = self.run_bzr('init', '--format=weave')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+    def test_init_at_repository_root(self):
+        # bzr init at the root of a repository should create a branch
+        # and working tree even when creation of working trees is disabled.
+        t = self.get_transport()
+        t.mkdir('repo')
+        format = BzrDirMetaFormat1()
+        newdir = format.initialize(t.abspath('repo'))
+        repo = newdir.create_repository(shared=True)
+        repo.set_make_working_trees(False)
+        out, err = self.run_bzr('init', 'repo')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+        newdir.open_branch()
+        newdir.open_workingtree()
+        
+    def test_init_branch(self):
+        out, err = self.run_bzr('init')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+        # Can it handle subdirectories of branches too ?
+        out, err = self.run_bzr('init', 'subdir1')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+        WorkingTree.open('subdir1')
+        
+        out, err = self.run_bzr('init', 'subdir2/nothere', retcode=3)
+        self.assertEqual('', out)
+        self.failUnless(err.startswith(
+            'bzr: ERROR: exceptions.OSError: '
+            '[Errno 2] No such file or directory: '))
+        
+        os.mkdir('subdir2')
+        out, err = self.run_bzr('init', 'subdir2')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+        # init an existing branch.
+        out, err = self.run_bzr('init', 'subdir2', retcode=3)
+        self.assertEqual('', out)
+        self.failUnless(err.startswith('bzr: ERROR: Already a branch:'))

=== added file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_log.py	2006-03-28 14:29:13 +0000
@@ -0,0 +1,153 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr log."""
+
+import os
+
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestLog(ExternalBase):
+
+    def _prepare(self):
+        self.runbzr("init")
+        self.build_tree(['hello.txt', 'goodbye.txt', 'meep.txt'])
+        self.runbzr("add hello.txt")
+        self.runbzr("commit -m message1 hello.txt")
+        self.runbzr("add goodbye.txt")
+        self.runbzr("commit -m message2 goodbye.txt")
+        self.runbzr("add meep.txt")
+        self.runbzr("commit -m message3 meep.txt")
+        self.full_log = self.runbzr("log")[0]
+
+    def test_log_null_end_revspec(self):
+        self._prepare()
+        self.assertTrue('revno: 1\n' in self.full_log)
+        self.assertTrue('revno: 2\n' in self.full_log)
+        self.assertTrue('revno: 3\n' in self.full_log)
+        self.assertTrue('message:\n  message1\n' in self.full_log)
+        self.assertTrue('message:\n  message2\n' in self.full_log)
+        self.assertTrue('message:\n  message3\n' in self.full_log)
+
+        log = self.runbzr("log -r 1..")[0]
+        self.assertEquals(log, self.full_log)
+
+    def test_log_null_begin_revspec(self):
+        self._prepare()
+        log = self.runbzr("log -r ..3")[0]
+        self.assertEquals(self.full_log, log)
+
+    def test_log_null_both_revspecs(self):
+        self._prepare()
+        log = self.runbzr("log -r ..")[0]
+        self.assertEquals(self.full_log, log)
+
+    def test_log_negative_begin_revspec_full_log(self):
+        self._prepare()
+        log = self.runbzr("log -r -3..")[0]
+        self.assertEquals(self.full_log, log)
+
+    def test_log_negative_both_revspec_full_log(self):
+        self._prepare()
+        log = self.runbzr("log -r -3..-1")[0]
+        self.assertEquals(self.full_log, log)
+
+    def test_log_negative_both_revspec_partial(self):
+        self._prepare()
+        log = self.runbzr("log -r -3..-2")[0]
+        self.assertTrue('revno: 1\n' in log)
+        self.assertTrue('revno: 2\n' in log)
+        self.assertTrue('revno: 3\n' not in log)
+
+    def test_log_negative_begin_revspec(self):
+        self._prepare()
+        log = self.runbzr("log -r -2..")[0]
+        self.assertTrue('revno: 1\n' not in log)
+        self.assertTrue('revno: 2\n' in log)
+        self.assertTrue('revno: 3\n' in log)
+
+    def test_log_postive_revspecs(self):
+        self._prepare()
+        log = self.runbzr("log -r 1..3")[0]
+        self.assertEquals(self.full_log, log)
+
+
+class TestLogMerges(ExternalBase):
+
+    def test_merges_are_indented_by_level(self):
+        self.build_tree(['parent/'])
+        self.run_bzr('init', 'parent')
+        self.run_bzr('commit', '-m', 'first post', '--unchanged', 'parent')
+        self.run_bzr('branch', 'parent', 'child')
+        self.run_bzr('commit', '-m', 'branch 1', '--unchanged', 'child')
+        self.run_bzr('branch', 'child', 'smallerchild')
+        self.run_bzr('commit', '-m', 'branch 2', '--unchanged', 'smallerchild')
+        os.chdir('child')
+        self.run_bzr('merge', '../smallerchild')
+        self.run_bzr('commit', '-m', 'merge branch 2')
+        os.chdir('../parent')
+        self.run_bzr('merge', '../child')
+        self.run_bzr('commit', '-m', 'merge branch 1')
+        out,err = self.run_bzr('log')
+        # the log will look something like:
+#        self.assertEqual("""\
+#------------------------------------------------------------
+#revno: 2
+#committer: Robert Collins <foo@example.com>
+#branch nick: parent
+#timestamp: Tue 2006-03-28 22:31:40 +1100
+#message:
+#  merge branch 1
+#    ------------------------------------------------------------
+#    merged: foo@example.com-20060328113140-91f43cfb46dc2863
+#    committer: Robert Collins <foo@example.com>
+#    branch nick: child
+#    timestamp: Tue 2006-03-28 22:31:40 +1100
+#    message:
+#      merge branch 2
+#        ------------------------------------------------------------
+#        merged: foo@example.com-20060328113140-1ba24f850a0ef573
+#        committer: Robert Collins <foo@example.com>
+#        branch nick: smallerchild
+#        timestamp: Tue 2006-03-28 22:31:40 +1100
+#        message:
+#          branch 2
+#    ------------------------------------------------------------
+#    merged: foo@example.com-20060328113140-5749a4757a8ac792
+#    committer: Robert Collins <foo@example.com>
+#    branch nick: child
+#    timestamp: Tue 2006-03-28 22:31:40 +1100
+#    message:
+#      branch 1
+#------------------------------------------------------------
+#revno: 1
+#committer: Robert Collins <foo@example.com>
+#branch nick: parent
+#timestamp: Tue 2006-03-28 22:31:39 +1100
+#message:
+#  first post
+#""", out)
+        # but we dont have a nice pattern matcher hooked up yet, so:
+        # we check for the indenting of the commit message:
+        self.assertTrue('  merge branch 1' in out)
+        self.assertTrue('      merge branch 2' in out)
+        self.assertTrue('          branch 2' in out)
+        self.assertTrue('      branch 1' in out)
+        self.assertTrue('  first post' in out)
+        self.assertEqual('', err)

=== added file 'bzrlib/tests/blackbox/test_logformats.py'
--- bzrlib/tests/blackbox/test_logformats.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_logformats.py	2006-03-12 20:42:58 +0000
@@ -0,0 +1,111 @@
+"""Black-box tests for default log_formats/log_formatters
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.config import (ensure_config_dir_exists, config_filename)
+
+
+class TestLogFormats(TestCaseInTempDir):
+
+    def bzr(self, *args, **kwargs):
+        return self.run_bzr(*args, **kwargs)[0]
+
+    def test_log_default_format(self):
+        setup_config()
+
+        self.bzr('init')
+        open('a', 'wb').write('foo\n')
+        self.bzr('add', 'a')
+
+        self.bzr('commit', '-m', '1')
+        open('a', 'wb').write('baz\n')
+
+        self.bzr('commit', '-m', '2')
+
+        # only the lines formatter is this short
+        self.assertEquals(3, len(self.bzr('log').split('\n')))
+
+    def test_log_format_arg(self):
+        self.bzr('init')
+        open('a', 'wb').write('foo\n')
+        self.bzr('add', 'a')
+
+        self.bzr('commit', '-m', '1')
+        open('a', 'wb').write('baz\n')
+
+        self.bzr('commit', '-m', '2')
+
+        # only the lines formatter is this short
+        self.assertEquals(7, len(self.bzr('log', '--log-format', 'short').split('\n')))
+
+    def test_missing_default_format(self):
+        setup_config()
+
+        os.mkdir('a')
+        os.chdir('a')
+        self.bzr('init')
+
+        open('a', 'wb').write('foo\n')
+        self.bzr('add', 'a')
+        self.bzr('commit', '-m', '1')
+
+        os.chdir('..')
+        self.bzr('branch', 'a', 'b')
+        os.chdir('a')
+
+        open('a', 'wb').write('bar\n')
+        self.bzr('commit', '-m', '2')
+
+        open('a', 'wb').write('baz\n')
+        self.bzr('commit', '-m', '3')
+
+        os.chdir('../b')
+        
+        self.assertEquals(5, len(self.bzr('missing', retcode=1).split('\n')))
+        
+        os.chdir('..')
+
+    def test_missing_format_arg(self):
+        setup_config()
+
+        os.mkdir('a')
+        os.chdir('a')
+        self.bzr('init')
+
+        open('a', 'wb').write('foo\n')
+        self.bzr('add', 'a')
+        self.bzr('commit', '-m', '1')
+
+        os.chdir('..')
+        self.bzr('branch', 'a', 'b')
+        os.chdir('a')
+
+        open('a', 'wb').write('bar\n')
+        self.bzr('commit', '-m', '2')
+
+        open('a', 'wb').write('baz\n')
+        self.bzr('commit', '-m', '3')
+
+        os.chdir('../b')
+        
+        self.assertEquals(9, len(self.bzr('missing', '--log-format', 'short', retcode=1).split('\n')))
+        
+        os.chdir('..')
+
+
+def setup_config():
+        if os.path.isfile(config_filename()):
+                # Something is wrong in environment, 
+                # we risk overwriting users config 
+                self.assert_(config_filename() + "exists, abort")
+            
+        ensure_config_dir_exists()
+        CONFIG=("[DEFAULT]\n"
+                "email=Joe Foo <joe@foo.com>\n"
+                "log_format=line\n")
+
+        open(config_filename(),'wb').write(CONFIG)
+

=== added file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2006-04-18 02:24:14 +0000
@@ -0,0 +1,169 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+# Author: Aaron Bentley <aaron.bentley@utoronto.ca>
+
+"""Black-box tests for bzr merge.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.osutils import abspath
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestMerge(ExternalBase):
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_merge_reprocess(self):
+        d = BzrDir.create_standalone_workingtree('.')
+        d.commit('h')
+        self.run_bzr('merge', '.', '--reprocess', '--merge-type', 'weave')
+
+    def test_merge(self):
+        from bzrlib.branch import Branch
+        
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        file('goodbye', 'wt').write('quux')
+        self.runbzr(['commit',  '-m',  "more u's are always good"])
+
+        os.chdir('../a')
+        file('hello', 'wt').write('quuux')
+        # We can't merge when there are in-tree changes
+        self.runbzr('merge ../b', retcode=3)
+        self.runbzr(['commit', '-m', "Like an epidemic of u's"])
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type blooof',
+                    retcode=3)
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type merge3')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type weave')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --reprocess')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1')
+        self.check_file_contents('goodbye', 'quux')
+        # Merging a branch pulls its revision into the tree
+        a = WorkingTree.open('.')
+        b = Branch.open('../b')
+        a.branch.repository.get_revision_xml(b.last_revision())
+        self.log('pending merges: %s', a.pending_merges())
+        self.assertEquals(a.pending_merges(),
+                          [b.last_revision()])
+        self.runbzr('commit -m merged')
+        self.runbzr('merge ../b -r last:1')
+        self.assertEqual(a.pending_merges(), [])
+
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.THIS'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
+        os.chdir('../a')
+        self.runbzr('merge ../b/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.OTHER'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
+
+    def test_merge_remember(self):
+        """Merge changes from one branch to another and test parent location."""
+        tree_a = self.make_branch_and_tree('branch_a')
+        branch_a = tree_a.branch
+        self.build_tree(['branch_a/a'])
+        tree_a.add('a')
+        tree_a.commit('commit a')
+        branch_b = branch_a.bzrdir.sprout('branch_b').open_branch()
+        tree_b = branch_b.bzrdir.open_workingtree()
+        branch_c = branch_a.bzrdir.sprout('branch_c').open_branch()
+        tree_c = branch_c.bzrdir.open_workingtree()
+        self.build_tree(['branch_a/b'])
+        tree_a.add('b')
+        tree_a.commit('commit b')
+        self.build_tree(['branch_c/c'])
+        tree_c.add('c')
+        tree_c.commit('commit c')
+        # reset parent
+        parent = branch_b.get_parent()
+        branch_b.set_parent(None)
+        self.assertEqual(None, branch_b.get_parent())
+        # test merge for failure without parent set
+        os.chdir('branch_b')
+        out = self.runbzr('merge', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: No merge branch known or specified.\n'))
+        # test implicit --remember when no parent set, this merge conflicts
+        self.build_tree(['d'])
+        tree_b.add('d')
+        out = self.runbzr('merge ../branch_a', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: Working tree has uncommitted changes.\n'))
+        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        # test implicit --remember after resolving conflict
+        tree_b.commit('commit d')
+        out, err = self.runbzr('merge')
+        self.assertEquals(out, 'Using saved branch: ../branch_a\n')
+        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        # re-open tree as external runbzr modified it
+        tree_b = branch_b.bzrdir.open_workingtree()
+        tree_b.commit('merge branch_a')
+        # test explicit --remember
+        out, err = self.runbzr('merge ../branch_c --remember')
+        self.assertEquals(out, '')
+        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(abspath(branch_b.get_parent()),
+                          abspath(branch_c.bzrdir.root_transport.base))
+        # re-open tree as external runbzr modified it
+        tree_b = branch_b.bzrdir.open_workingtree()
+        tree_b.commit('merge branch_c')

=== added file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2006-03-25 09:40:57 +0000
@@ -0,0 +1,101 @@
+"""Black-box tests for bzr missing.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+
+class TestMissing(TestCaseInTempDir):
+
+    def test_missing(self):
+        missing = "You are missing 1 revision(s):"
+
+        # create a source branch
+        os.mkdir('a')
+        os.chdir('a')
+        self.capture('init')
+        open('a', 'wb').write('initial\n')
+        self.capture('add a')
+        self.capture('commit -m inital')
+
+        # clone and add a differing revision
+        self.capture('branch . ../b')
+        os.chdir('../b')
+        open('a', 'ab').write('more\n')
+        self.capture('commit -m more')
+
+        # run missing in a against b
+        os.chdir('../a')
+        # this should not require missing to take out a write lock on a 
+        # or b. So we take a write lock on both to test that at the same
+        # time. This may let the test pass while the default branch is an
+        # os-locking branch, but it will trigger failures with lockdir based
+        # branches.
+        branch_a = Branch.open('.')
+        branch_a.lock_write()
+        branch_b = Branch.open('../b')
+        branch_b.lock_write()
+        out,err = self.run_bzr('missing', '../b', retcode=1)
+        lines = out.splitlines()
+        # we're missing the extra revision here
+        self.assertEqual(missing, lines[0])
+        # and we expect 8 lines of output which we trust at the moment to be
+        # good.
+        self.assertEqual(8, len(lines))
+        # we do not expect any error output.
+        self.assertEqual('', err)
+        # unlock the branches for the rest of the test
+        branch_a.unlock()
+        branch_b.unlock()
+
+        # get extra revision from b
+        self.capture('merge ../b')
+        self.capture('commit -m merge')
+
+        # compare again, but now we have the 'merge' commit extra
+        lines = self.capture('missing ../b', retcode=1).splitlines()
+        self.assertEqual("You have 1 extra revision(s):", lines[0])
+        self.assertEqual(8, len(lines))
+        lines2 = self.capture('missing ../b --mine-only', retcode=1)
+        lines2 = lines2.splitlines()
+        self.assertEqual(lines, lines2)
+        lines3 = self.capture('missing ../b --theirs-only', retcode=1)
+        lines3 = lines3.splitlines()
+        self.assertEqual(0, len(lines3))
+
+        # relative to a, missing the 'merge' commit 
+        os.chdir('../b')
+        lines = self.capture('missing ../a', retcode=1).splitlines()
+        self.assertEqual(missing, lines[0])
+        self.assertEqual(8, len(lines))
+        lines2 = self.capture('missing ../a --theirs-only', retcode=1)
+        lines2 = lines2.splitlines()
+        self.assertEqual(lines, lines2)
+        lines3 = self.capture('missing ../a --mine-only', retcode=1)
+        lines3 = lines3.splitlines()
+        self.assertEqual(0, len(lines3))
+        lines4 = self.capture('missing ../a --short', retcode=1)
+        lines4 = lines4.splitlines()
+        self.assertEqual(4, len(lines4))
+        lines5 = self.capture('missing ../a --line', retcode=1)
+        lines5 = lines5.splitlines()
+        self.assertEqual(2, len(lines5))
+        lines6 = self.capture('missing ../a --reverse', retcode=1)
+        lines6 = lines6.splitlines()
+        self.assertEqual(lines6, lines)
+        lines7 = self.capture('missing ../a --show-ids', retcode=1)
+        lines7 = lines7.splitlines()
+        self.assertEqual(11, len(lines7))
+        lines8 = self.capture('missing ../a --verbose', retcode=1)
+        lines8 = lines8.splitlines()
+        self.assertEqual("modified:", lines8[-2])
+        self.assertEqual("  a", lines8[-1])
+
+        
+        # after a pull we're back on track
+        self.capture('pull')
+        self.assertEqual("Branches are up to date.\n", 
+                         self.capture('missing ../a'))
+

=== added file 'bzrlib/tests/blackbox/test_outside_wt.py'
--- bzrlib/tests/blackbox/test_outside_wt.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_outside_wt.py	2006-03-12 19:57:25 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2006 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for running bzr outside of a working tree."""
+
+import os
+import tempfile
+
+from bzrlib.tests import ChrootedTestCase
+from bzrlib.osutils import getcwd
+
+
+class TestOutsideWT(ChrootedTestCase):
+    """Test that bzr gives proper errors outside of a working tree."""
+
+    def test_cwd_log(self):
+        os.chdir(tempfile.mkdtemp())
+        cwd = getcwd()
+        out, err = self.run_bzr('log', retcode=3)
+
+        self.assertEqual(u'bzr: ERROR: Not a branch: %s/\n' % (cwd,),
+                         err)
+
+    def test_url_log(self):
+        url = self.get_readonly_url() + 'subdir/'
+        out, err = self.run_bzr('log', 
+                                url, retcode=3)
+        self.assertEqual(u'bzr: ERROR: Not a branch:'
+                         u' %s\n' % url, err)

=== added file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2006-04-19 05:22:38 +0000
@@ -0,0 +1,261 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr pull."""
+
+import os
+import sys
+
+from bzrlib.branch import Branch
+from bzrlib.osutils import abspath
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.uncommit import uncommit
+
+
+class TestPull(ExternalBase):
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        self.runbzr('pull', retcode=3)
+        self.runbzr('missing', retcode=3)
+        self.runbzr('missing .')
+        self.runbzr('missing')
+        # this will work on windows because we check for the same branch
+        # in pull - if it fails, it is a regression
+        self.runbzr('pull')
+        self.runbzr('pull /', retcode=3)
+        if sys.platform not in ('win32', 'cygwin'):
+            self.runbzr('pull')
+
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('pull')
+        os.mkdir('subdir')
+        self.runbzr('add subdir')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        self.assertEquals(a.revision_history(), b.revision_history()[:-1])
+        self.runbzr('pull ../b')
+        self.assertEquals(a.revision_history(), b.revision_history())
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        # no overwrite
+        self.runbzr('pull ../a', retcode=3)
+        os.chdir('..')
+        self.runbzr('branch b overwriteme')
+        os.chdir('overwriteme')
+        self.runbzr('pull --overwrite ../a')
+        overwritten = Branch.open('.')
+        self.assertEqual(overwritten.revision_history(),
+                         a.revision_history())
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b/subdir')
+        self.runbzr('pull ../../a')
+        self.assertEquals(a.revision_history()[-1], b.revision_history()[-1])
+        self.runbzr('commit -m blah5 --unchanged')
+        self.runbzr('commit -m blah6 --unchanged')
+        os.chdir('..')
+        self.runbzr('pull ../a')
+        os.chdir('../a')
+        self.runbzr('commit -m blah7 --unchanged')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah8 --unchanged')
+        self.runbzr('pull ../b')
+        self.runbzr('pull ../b')
+
+    def test_pull_revision(self):
+        """Pull some changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        file('hello2', 'wt').write('foo')
+        self.runbzr('add hello2')
+        self.runbzr('commit -m setup hello2')
+        file('goodbye2', 'wt').write('baz')
+        self.runbzr('add goodbye2')
+        self.runbzr('commit -m setup goodbye2')
+
+        os.chdir('..')
+        self.runbzr('branch -r 1 a b')
+        os.chdir('b')
+        self.runbzr('pull -r 2')
+        a = Branch.open('../a')
+        b = Branch.open('.')
+        self.assertEquals(a.revno(),4)
+        self.assertEquals(b.revno(),2)
+        self.runbzr('pull -r 3')
+        self.assertEquals(b.revno(),3)
+        self.runbzr('pull -r 4')
+        self.assertEquals(a.revision_history(), b.revision_history())
+
+
+    def test_overwrite_uptodate(self):
+        # Make sure pull --overwrite overwrites
+        # even if the target branch has merged
+        # everything already.
+        bzr = self.run_bzr
+
+        def get_rh(expected_len):
+            rh = self.capture('revision-history')
+            # Make sure we don't have trailing empty revisions
+            rh = rh.strip().split('\n')
+            self.assertEqual(len(rh), expected_len)
+            return rh
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        open('foo', 'wb').write('original\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'initial commit')
+
+        os.chdir('..')
+        bzr('branch', 'a', 'b')
+
+        os.chdir('a')
+        open('foo', 'wb').write('changed\n')
+        bzr('commit', '-m', 'later change')
+
+        open('foo', 'wb').write('another\n')
+        bzr('commit', '-m', 'a third change')
+
+        rev_history_a = get_rh(3)
+
+        os.chdir('../b')
+        bzr('merge', '../a')
+        bzr('commit', '-m', 'merge')
+
+        rev_history_b = get_rh(2)
+
+        bzr('pull', '--overwrite', '../a')
+        rev_history_b = get_rh(3)
+
+        self.assertEqual(rev_history_b, rev_history_a)
+
+    def test_overwrite_children(self):
+        # Make sure pull --overwrite sets the revision-history
+        # to be identical to the pull source, even if we have convergence
+        bzr = self.run_bzr
+
+        def get_rh(expected_len):
+            rh = self.capture('revision-history')
+            # Make sure we don't have trailing empty revisions
+            rh = rh.strip().split('\n')
+            self.assertEqual(len(rh), expected_len)
+            return rh
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        open('foo', 'wb').write('original\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'initial commit')
+
+        os.chdir('..')
+        bzr('branch', 'a', 'b')
+
+        os.chdir('a')
+        open('foo', 'wb').write('changed\n')
+        bzr('commit', '-m', 'later change')
+
+        open('foo', 'wb').write('another\n')
+        bzr('commit', '-m', 'a third change')
+
+        rev_history_a = get_rh(3)
+
+        os.chdir('../b')
+        bzr('merge', '../a')
+        bzr('commit', '-m', 'merge')
+
+        rev_history_b = get_rh(2)
+
+        os.chdir('../a')
+        open('foo', 'wb').write('a fourth change\n')
+        bzr('commit', '-m', 'a fourth change')
+
+        rev_history_a = get_rh(4)
+
+        # With convergence, we could just pull over the
+        # new change, but with --overwrite, we want to switch our history
+        os.chdir('../b')
+        bzr('pull', '--overwrite', '../a')
+        rev_history_b = get_rh(4)
+
+        self.assertEqual(rev_history_b, rev_history_a)
+
+    def test_pull_remember(self):
+        """Pull changes from one branch to another and test parent location."""
+        transport = self.get_transport()
+        tree_a = self.make_branch_and_tree('branch_a')
+        branch_a = tree_a.branch
+        self.build_tree(['branch_a/a'])
+        tree_a.add('a')
+        tree_a.commit('commit a')
+        tree_b = branch_a.bzrdir.sprout('branch_b').open_workingtree()
+        branch_b = tree_b.branch
+        tree_c = branch_a.bzrdir.sprout('branch_c').open_workingtree()
+        branch_c = tree_c.branch
+        self.build_tree(['branch_a/b'])
+        tree_a.add('b')
+        tree_a.commit('commit b')
+        # reset parent
+        parent = branch_b.get_parent()
+        branch_b.set_parent(None)
+        self.assertEqual(None, branch_b.get_parent())
+        # test pull for failure without parent set
+        os.chdir('branch_b')
+        out = self.runbzr('pull', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: No pull location known or specified.\n'))
+        # test implicit --remember when no parent set, this pull conflicts
+        self.build_tree(['d'])
+        tree_b.add('d')
+        tree_b.commit('commit d')
+        out = self.runbzr('pull ../branch_a', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: These branches have diverged.  Try merge.\n'))
+        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        # test implicit --remember after resolving previous failure
+        uncommit(branch=branch_b, tree=tree_b)
+        transport.delete('branch_b/d')
+        self.runbzr('pull')
+        self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
+        # test explicit --remember
+        self.runbzr('pull ../branch_c --remember')
+        self.assertEquals(abspath(branch_b.get_parent()),
+                          abspath(branch_c.bzrdir.root_transport.base))

=== added file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_push.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,73 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr push.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.osutils import abspath
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.uncommit import uncommit
+
+
+class TestPush(ExternalBase):
+
+    def test_push_remember(self):
+        """Push changes from one branch to another and test push location."""
+        transport = self.get_transport()
+        tree_a = self.make_branch_and_tree('branch_a')
+        branch_a = tree_a.branch
+        self.build_tree(['branch_a/a'])
+        tree_a.add('a')
+        tree_a.commit('commit a')
+        tree_b = branch_a.bzrdir.sprout('branch_b').open_workingtree()
+        branch_b = tree_b.branch
+        tree_c = branch_a.bzrdir.sprout('branch_c').open_workingtree()
+        branch_c = tree_c.branch
+        self.build_tree(['branch_a/b'])
+        tree_a.add('b')
+        tree_a.commit('commit b')
+        self.build_tree(['branch_b/c'])
+        tree_b.add('c')
+        tree_b.commit('commit c')
+        # initial push location must be empty
+        self.assertEqual(None, branch_b.get_push_location())
+        # test push for failure without push location set
+        os.chdir('branch_a')
+        out = self.runbzr('push', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: No push location known or specified.\n'))
+        # test implicit --remember when no push location set, push fails
+        out = self.runbzr('push ../branch_b', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: These branches have diverged.  '
+                    'Try a merge then push with overwrite.\n'))
+        self.assertEquals(abspath(branch_a.get_push_location()),
+                          abspath(branch_b.bzrdir.root_transport.base))
+        # test implicit --remember after resolving previous failure
+        uncommit(branch=branch_b, tree=tree_b)
+        transport.delete('branch_b/c')
+        self.runbzr('push')
+        self.assertEquals(abspath(branch_a.get_push_location()),
+                          abspath(branch_b.bzrdir.root_transport.base))
+        # test explicit --remember
+        self.runbzr('push ../branch_c --remember')
+        self.assertEquals(abspath(branch_a.get_push_location()),
+                          abspath(branch_c.bzrdir.root_transport.base))

=== added file 'bzrlib/tests/blackbox/test_re_sign.py'
--- bzrlib/tests/blackbox/test_re_sign.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_re_sign.py	2006-03-06 12:11:25 +0000
@@ -0,0 +1,93 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr re-sign.
+"""
+
+import os
+
+import bzrlib.gpg
+from bzrlib.bzrdir import BzrDir
+from bzrlib.testament import Testament
+from bzrlib.tests import TestCaseInTempDir
+
+
+class ReSign(TestCaseInTempDir):
+
+    def monkey_patch_gpg(self):
+        """Monkey patch the gpg signing strategy to be a loopback.
+
+        This also registers the cleanup, so that we will revert to
+        the original gpg strategy when done.
+        """
+        self._oldstrategy = bzrlib.gpg.GPGStrategy
+
+        # monkey patch gpg signing mechanism
+        bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+
+        self.addCleanup(self._fix_gpg_strategy)
+
+    def _fix_gpg_strategy(self):
+        bzrlib.gpg.GPGStrategy = self._oldstrategy
+
+    def setup_tree(self):
+        wt = BzrDir.create_standalone_workingtree('.')
+        wt.commit("base A", allow_pointless=True, rev_id='A')
+        wt.commit("base B", allow_pointless=True, rev_id='B')
+        wt.commit("base C", allow_pointless=True, rev_id='C')
+
+        return wt
+
+    def assertEqualSignature(self, repo, revision_id):
+        """Assert a signature is stored correctly in repository."""
+        self.assertEqual(
+            Testament.from_revision(repo, revision_id).as_short_text(),
+            repo.get_signature_text(revision_id))
+
+    def test_resign(self):
+        #Test re signing of data.
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign', '-r', 'revid:A')
+
+        self.assertEqualSignature(repo, 'A')
+
+        self.run_bzr('re-sign', 'B')
+        self.assertEqualSignature(repo, 'B')
+            
+    def test_resign_range(self):
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign', '-r', '1..')
+        self.assertEqualSignature(repo, 'A')
+        self.assertEqualSignature(repo, 'B')
+        self.assertEqualSignature(repo, 'C')
+
+    def test_resign_multiple(self):
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign', 'A', 'B', 'C')
+        self.assertEqualSignature(repo, 'A')
+        self.assertEqualSignature(repo, 'B')
+        self.assertEqualSignature(repo, 'C')

=== added file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2006-02-25 00:32:59 +0000
@@ -0,0 +1,63 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black box tests for the reconcile command."""
+
+
+import bzrlib
+import bzrlib.bzrdir as bzrdir
+import bzrlib.repository as repository
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.blackbox import TestUIFactory
+from bzrlib.transport import get_transport
+import bzrlib.ui as ui
+
+
+class TrivialTest(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TrivialTest, self).setUp()
+        self.old_format = bzrdir.BzrDirFormat.get_default_format()
+        self.old_ui_factory = ui.ui_factory
+        self.addCleanup(self.restoreDefaults)
+        ui.ui_factory = TestUIFactory()
+
+    def restoreDefaults(self):
+        ui.ui_factory = self.old_ui_factory
+
+    def test_trivial_reconcile(self):
+        t = bzrdir.BzrDir.create_standalone_workingtree('.')
+        (out, err) = self.run_bzr_captured(['reconcile'])
+        self.assertEqualDiff(out, "Reconciling repository %s\n"
+                                  "Inventory ok.\n"
+                                  "Reconciliation complete.\n" %
+                                  t.bzrdir.root_transport.base)
+        self.assertEqualDiff(err, "")
+
+    def test_does_something_reconcile(self):
+        t = bzrdir.BzrDir.create_standalone_workingtree('.')
+        # an empty inventory with no revision will trigger reconciliation.
+        repo = t.branch.repository
+        inv = bzrlib.tree.EmptyTree().inventory
+        repo.add_inventory('missing', inv, [])
+        (out, err) = self.run_bzr_captured(['reconcile'])
+        self.assertEqualDiff(out, "Reconciling repository %s\n"
+                                  "Backup Inventory created.\n"
+                                  "Inventory regenerated.\n"
+                                  "Reconciliation complete.\n" %
+                                  t.bzrdir.root_transport.base)
+        self.assertEqualDiff(err, "")
+

=== added file 'bzrlib/tests/blackbox/test_revert.py'
--- bzrlib/tests/blackbox/test_revert.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_revert.py	2006-03-09 14:40:12 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for bzr revert."""
+
+import os
+
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.trace import mutter
+
+
+class TestRevert(ExternalBase):
+
+    def _prepare_tree(self):
+        self.runbzr('init')
+        self.runbzr('mkdir dir')
+
+        f = file('dir/file', 'wb')
+        f.write('spam')
+        f.close()
+        self.runbzr('add dir/file')
+
+        self.runbzr('commit -m1')
+
+        # modify file
+        f = file('dir/file', 'wb')
+        f.write('eggs')
+        f.close()
+
+        # check status
+        self.assertEquals('modified:\n  dir/file\n', self.capture('status'))
+
+    def helper(self, param=''):
+        self._prepare_tree()
+        # change dir
+        # revert to default revision for file in subdir does work
+        os.chdir('dir')
+        mutter('cd dir\n')
+
+        self.assertEquals('1\n', self.capture('revno'))
+        self.runbzr('revert %s file' % param)
+        self.assertEquals('spam', open('file', 'rb').read())
+
+    def test_revert_in_subdir(self):
+        self.helper()
+
+    def test_revert_to_revision_in_subdir(self):
+        # test case for bug #29424:
+        # revert to specific revision for file in subdir does not work
+        self.helper('-r 1')
+
+    def test_revert_in_checkout(self):
+        os.mkdir('brach')
+        os.chdir('brach')
+        self._prepare_tree()
+        self.runbzr('checkout --lightweight . ../sprach')
+        self.runbzr('commit -m more')
+        os.chdir('../sprach')
+        self.assertEqual('', self.capture('status'))
+        self.runbzr('revert')
+        self.assertEqual('', self.capture('status'))

=== added file 'bzrlib/tests/blackbox/test_revision_info.py'
--- bzrlib/tests/blackbox/test_revision_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_revision_info.py	2006-02-10 11:03:38 +0000
@@ -0,0 +1,112 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.errors import BzrCommandError, NoSuchRevision
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.workingtree import WorkingTree
+
+
+class TestRevisionInfo(TestCaseWithTransport):
+    
+    def check_error(self, output, *args):
+        """Verify that the expected error matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertContainsRe(self.run_bzr_captured(args, retcode=3)[1], output)
+
+    def check_output(self, output, *args):
+        """Verify that the expected output matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args)[0], output)
+
+    def test_revision_info(self):
+        """Test that 'bzr revision-info' reports the correct thing."""
+        wt = self.make_branch_and_tree('.')
+
+        wt.commit('Commit one', rev_id='a@r-0-1')
+        wt.commit('Commit two', rev_id='a@r-0-2')
+        wt.commit('Commit three', rev_id='a@r-0-3')
+
+        # Make sure revision-info without any arguments throws an exception
+        self.check_error('bzr: ERROR: '
+                         'You must supply a revision identifier\n',
+                         'revision-info')
+
+        values = {
+            1:'   1 a@r-0-1\n',
+            2:'   2 a@r-0-2\n',
+            3:'   3 a@r-0-3\n'
+        }
+
+        # Check the results of just specifying a numeric revision
+        self.check_output(values[1], 'revision-info', '1')
+        self.check_output(values[2], 'revision-info', '2')
+        self.check_output(values[3], 'revision-info', '3')
+        self.check_output(values[1]+values[2], 'revision-info', '1', '2')
+        self.check_output(values[1]+values[2]+values[3], 'revision-info', '1', '2', '3')
+        self.check_output(values[2]+values[1], 'revision-info', '2', '1')
+
+        # Check as above, only using the '--revision' syntax
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', '1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', '2')
+        self.check_output('   3 a@r-0-3\n', 'revision-info', '-r', '3')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n', 'revision-info', '-r', '1..2')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n   3 a@r-0-3\n'
+                , 'revision-info', '-r', '1..2..3')
+        self.check_output('   2 a@r-0-2\n   1 a@r-0-1\n', 'revision-info', '-r', '2..1')
+
+        # Now try some more advanced revision specifications
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', 'revid:a@r-0-1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', 'revid:a@r-0-2')
+
+    def test_cat_revision(self):
+        """Test bzr cat-revision.
+        """
+        wt = self.make_branch_and_tree('.')
+        r = wt.branch.repository
+
+        wt.commit('Commit one', rev_id='a@r-0-1')
+        wt.commit('Commit two', rev_id='a@r-0-2')
+        wt.commit('Commit three', rev_id='a@r-0-3')
+
+        revs = {
+            1:r.get_revision_xml('a@r-0-1'),
+            2:r.get_revision_xml('a@r-0-2'),
+            3:r.get_revision_xml('a@r-0-3'),
+        }
+
+        self.check_output(revs[1], 'cat-revision', 'a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', 'a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', 'a@r-0-3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', '1')
+        self.check_output(revs[2], 'cat-revision', '-r', '2')
+        self.check_output(revs[3], 'cat-revision', '-r', '3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', 'revid:a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', '-r', 'revid:a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', '-r', 'revid:a@r-0-3')
+

=== added file 'bzrlib/tests/blackbox/test_revno.py'
--- bzrlib/tests/blackbox/test_revno.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_revno.py	2005-12-04 21:45:39 +0000
@@ -0,0 +1,53 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr revno.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestRevno(TestCaseInTempDir):
+
+    def test_revno(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        self.assertEquals(int(bzr('revno')), 0)
+
+        open('foo', 'wb').write('foo\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'foo')
+        self.assertEquals(int(bzr('revno')), 1)
+
+        os.mkdir('baz')
+        bzr('add', 'baz')
+        bzr('commit', '-m', 'baz')
+        self.assertEquals(int(bzr('revno')), 2)
+
+        os.chdir('..')
+        self.assertEquals(int(bzr('revno', 'a')), 2)
+        self.assertEquals(int(bzr('revno', 'a/baz')), 2)
+
+

=== added file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-03-12 20:20:13 +0000
@@ -0,0 +1,75 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""UI tests for the test framework."""
+
+import bzrlib
+from bzrlib.errors import ParamikoNotPresent
+from bzrlib.tests import (
+                          TestCase,
+                          TestCaseInTempDir,
+                          TestSkipped,
+                          )
+
+
+class TestOptions(TestCase):
+
+    current_test = None
+
+    def test_transport_set_to_sftp(self):
+        # test the --transport option has taken effect from within the
+        # test_transport test
+        try:
+            import bzrlib.transport.sftp
+        except ParamikoNotPresent:
+            raise TestSkipped("Paramiko not present")
+        if TestOptions.current_test != "test_transport_set_to_sftp":
+            return
+        self.assertEqual(bzrlib.transport.sftp.SFTPAbsoluteServer,
+                         bzrlib.tests.default_transport)
+
+    def test_transport_set_to_memory(self):
+        # test the --transport option has taken effect from within the
+        # test_transport test
+        import bzrlib.transport.memory
+        if TestOptions.current_test != "test_transport_set_to_memory":
+            return
+        self.assertEqual(bzrlib.transport.memory.MemoryServer,
+                         bzrlib.tests.default_transport)
+
+    def test_transport(self):
+        # test that --transport=sftp works
+        try:
+            import bzrlib.transport.sftp
+        except ParamikoNotPresent:
+            raise TestSkipped("Paramiko not present")
+        old_transport = bzrlib.tests.default_transport
+        old_root = TestCaseInTempDir.TEST_ROOT
+        TestCaseInTempDir.TEST_ROOT = None
+        try:
+            TestOptions.current_test = "test_transport_set_to_sftp"
+            stdout = self.capture('selftest --transport=sftp test_transport_set_to_sftp')
+            
+            self.assertContainsRe(stdout, 'Ran 1 test')
+            self.assertEqual(old_transport, bzrlib.tests.default_transport)
+
+            TestOptions.current_test = "test_transport_set_to_memory"
+            stdout = self.capture('selftest --transport=memory test_transport_set_to_memory')
+            self.assertContainsRe(stdout, 'Ran 1 test')
+            self.assertEqual(old_transport, bzrlib.tests.default_transport)
+        finally:
+            bzrlib.tests.default_transport = old_transport
+            TestOptions.current_test = None
+            TestCaseInTempDir.TEST_ROOT = old_root

=== added file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2006-04-13 03:48:05 +0000
@@ -0,0 +1,80 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for repositories with shared branches"""
+
+import os
+
+from bzrlib.tests import TestCaseInTempDir
+import bzrlib.bzrdir
+from bzrlib.bzrdir import BzrDir
+import bzrlib.errors as errors
+
+class TestSharedRepo(TestCaseInTempDir):
+
+    def test_make_repository(self):
+        out, err = self.run_bzr("init-repository", "a")
+        self.assertEqual(out, "")
+        self.assertEqual(err, "")
+        dir = bzrlib.bzrdir.BzrDir.open('a')
+        self.assertIs(dir.open_repository().is_shared(), True)
+        self.assertRaises(errors.NotBranchError, dir.open_branch)
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)        
+
+    def test_init_repo_existing_dir(self):
+        """Make repo in existing directory.
+        
+        (Malone #38331)
+        """
+        out, err = self.run_bzr("init-repository", ".")
+        dir = BzrDir.open('.')
+        self.assertTrue(dir.open_repository())
+
+    def test_init(self):
+        self.run_bzr("init-repo", "a")
+        self.run_bzr("init", "--format=metadir", "a/b")
+        dir = bzrlib.bzrdir.BzrDir.open('a')
+        self.assertIs(dir.open_repository().is_shared(), True)
+        self.assertRaises(errors.NotBranchError, dir.open_branch)
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
+        bdir = bzrlib.bzrdir.BzrDir.open('a/b')
+        bdir.open_branch()
+        self.assertRaises(errors.NoRepositoryPresent, bdir.open_repository)
+        self.assertRaises(errors.NoWorkingTree, bdir.open_workingtree)
+
+    def test_branch(self):
+        self.run_bzr("init-repo", "a")
+        self.run_bzr("init", "--format=metadir", "a/b")
+        self.run_bzr('branch', 'a/b', 'a/c')
+        cdir = bzrlib.bzrdir.BzrDir.open('a/c')
+        cdir.open_branch()
+        self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
+        self.assertRaises(errors.NoWorkingTree, cdir.open_workingtree)
+
+    def test_branch_tree(self):
+        self.run_bzr("init-repo", "--trees", "a")
+        self.run_bzr("init", "--format=metadir", "b")
+        file('b/hello', 'wt').write('bar')
+        self.run_bzr("add", "b/hello")
+        self.run_bzr("commit", "-m", "bar", "b/hello")
+
+        self.run_bzr('branch', 'b', 'a/c')
+        cdir = bzrlib.bzrdir.BzrDir.open('a/c')
+        cdir.open_branch()
+        self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
+        self.failUnlessExists('a/c/hello')
+        cdir.open_workingtree()
+

=== added file 'bzrlib/tests/blackbox/test_sign_my_commits.py'
--- bzrlib/tests/blackbox/test_sign_my_commits.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_sign_my_commits.py	2006-03-06 12:11:25 +0000
@@ -0,0 +1,121 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for bzr sign-my-commits."""
+
+import os
+
+import bzrlib.gpg
+from bzrlib.testament import Testament
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.workingtree import WorkingTree
+
+
+class SignMyCommits(TestCaseWithTransport):
+
+    def monkey_patch_gpg(self):
+        """Monkey patch the gpg signing strategy to be a loopback.
+
+        This also registers the cleanup, so that we will revert to
+        the original gpg strategy when done.
+        """
+        self._oldstrategy = bzrlib.gpg.GPGStrategy
+
+        # monkey patch gpg signing mechanism
+        bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+
+        self.addCleanup(self._fix_gpg_strategy)
+
+    def _fix_gpg_strategy(self):
+        bzrlib.gpg.GPGStrategy = self._oldstrategy
+
+    def setup_tree(self, location='.'):
+        wt = self.make_branch_and_tree(location)
+        wt.commit("base A", allow_pointless=True, rev_id='A')
+        wt.commit("base B", allow_pointless=True, rev_id='B')
+        wt.commit("base C", allow_pointless=True, rev_id='C')
+        wt.commit("base D", allow_pointless=True, rev_id='D',
+                committer='Alternate <alt@foo.com>')
+
+        return wt
+
+    def assertUnsigned(self, repo, revision_id):
+        """Assert that revision_id is not signed in repo."""
+        self.assertFalse(repo.has_signature_for_revision_id(revision_id))
+
+    def assertSigned(self, repo, revision_id):
+        """Assert that revision_id is signed in repo."""
+        self.assertTrue(repo.has_signature_for_revision_id(revision_id))
+
+    def test_sign_my_commits(self):
+        #Test re signing of data.
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
+
+        self.run_bzr('sign-my-commits')
+
+        self.assertSigned(repo, 'A')
+        self.assertSigned(repo, 'B')
+        self.assertSigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
+            
+    def test_sign_my_commits_location(self):
+        wt = self.setup_tree('other')
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+
+        self.run_bzr('sign-my-commits', 'other')
+
+        self.assertSigned(repo, 'A')
+        self.assertSigned(repo, 'B')
+        self.assertSigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
+
+    def test_sign_diff_committer(self):
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+
+        self.run_bzr('sign-my-commits', '.', 'Alternate <alt@foo.com>')
+
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertSigned(repo, 'D')
+
+    def test_sign_dry_run(self):
+        wt = self.setup_tree()
+        repo = wt.branch.repository
+
+        self.monkey_patch_gpg()
+
+        out = self.run_bzr('sign-my-commits', '--dry-run')[0]
+
+        self.assertEquals('A\nB\nC\nSigned 3 revisions\n', out)
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')

=== renamed file 'bzrlib/selftest/teststatus.py' => 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/selftest/teststatus.py	2005-07-12 01:44:23 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-02-23 01:34:20 +0000
@@ -18,32 +18,199 @@
 """Tests of status command.
 
 Most of these depend on the particular formatting used.
+As such they really are blackbox tests even though some of the 
+tests are not using self.capture. If we add tests for the programmatic
+interface later, they will be non blackbox tests.
 """
 
 
-from bzrlib.selftest import InTempDir
-
-class BranchStatus(InTempDir):
-    def runTest(self): 
-        """Basic 'bzr mkdir' operation"""
-        from cStringIO import StringIO
-        from bzrlib.status import show_status
-        from bzrlib.branch import Branch
-        
-        b = Branch('.', init=True)
+from cStringIO import StringIO
+from os import mkdir, chdir
+from tempfile import TemporaryFile
+import codecs
+
+import bzrlib.branch
+from bzrlib.builtins import merge
+import bzrlib.bzrdir as bzrdir
+from bzrlib.osutils import pathjoin
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.status import show_tree_status
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.workingtree import WorkingTree
+
+
+class BranchStatus(TestCaseWithTransport):
+    
+    def test_branch_status(self): 
+        """Test basic branch status"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
 
         # status with nothing
         tof = StringIO()
-        show_status(b, to_file=tof)
+        show_tree_status(wt, to_file=tof)
         self.assertEquals(tof.getvalue(), "")
 
         tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
-        show_status(b, to_file=tof)
+        wt.add_pending_merge('pending@pending-0-0')
+        show_tree_status(wt, to_file=tof)
         tof.seek(0)
         self.assertEquals(tof.readlines(),
                           ['unknown:\n',
                            '  bye.c\n',
                            '  hello.c\n',
-                           ])
+                           'pending merges:\n',
+                           '  pending@pending-0-0\n'
+                           ])
+
+    def test_branch_status_revisions(self):
+        """Tests branch status with revisions"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        tof = StringIO()
+        self.build_tree(['hello.c', 'bye.c'])
+        wt.add('hello.c')
+        wt.add('bye.c')
+        wt.commit('Test message')
+
+        tof = StringIO()
+        revs =[]
+        revs.append(RevisionSpec(0))
+        
+        show_tree_status(wt, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+        self.build_tree(['more.c'])
+        wt.add('more.c')
+        wt.commit('Another test message')
+        
+        tof = StringIO()
+        revs.append(RevisionSpec(1))
+        
+        show_tree_status(wt, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+    def status_string(self, wt):
+        # use a real file rather than StringIO because it doesn't handle
+        # Unicode very well.
+        tof = codecs.getwriter('utf-8')(TemporaryFile())
+        show_tree_status(wt, to_file=tof)
+        tof.seek(0)
+        return tof.read().decode('utf-8')
+
+    def test_pending(self):
+        """Pending merges display works, including Unicode"""
+        mkdir("./branch")
+        wt = self.make_branch_and_tree('branch')
+        b = wt.branch
+        wt.commit("Empty commit 1")
+        b_2_dir = b.bzrdir.sprout('./copy')
+        b_2 = b_2_dir.open_branch()
+        wt2 = b_2_dir.open_workingtree()
+        wt.commit(u"\N{TIBETAN DIGIT TWO} Empty commit 2")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(wt2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_(message.endswith("Empty commit 2\n")) 
+        wt2.commit("merged")
+        # must be long to make sure we see elipsis at the end
+        wt.commit("Empty commit 3 " + 
+                   "blah blah blah blah " * 10)
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(wt2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_("Empty commit 3" in message)
+        self.assert_(message.endswith("...\n")) 
+
+    def test_branch_status_specific_files(self): 
+        """Tests branch status with given specific files"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
+        wt.add('directory')
+        wt.add('test.c')
+        wt.commit('testing')
+        
+        tof = StringIO()
+        show_tree_status(wt, to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n',
+                           '  dir2\n',
+                           '  directory/hello.c\n'
+                           ])
+
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['bye.c','test.c','absent.c'], 
+                         to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n'
+                           ])
+        
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['directory'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  directory/hello.c\n'
+                           ])
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['dir2'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  dir2\n'
+                           ])
+
+class CheckoutStatus(BranchStatus):
+
+    def setUp(self):
+        super(CheckoutStatus, self).setUp()
+        mkdir('codir')
+        chdir('codir')
+        
+    def make_branch_and_tree(self, relpath):
+        source = self.make_branch(pathjoin('..', relpath))
+        checkout = bzrdir.BzrDirMetaFormat1().initialize(relpath)
+        bzrlib.branch.BranchReferenceFormat().initialize(checkout, source)
+        return checkout.create_workingtree()
+
+
+class TestStatus(TestCaseWithTransport):
+
+    def test_status(self):
+        self.run_bzr("init")
+        self.build_tree(['hello.txt'])
+        result = self.run_bzr("status")[0]
+        self.assert_("unknown:\n  hello.txt\n" in result, result)
+        self.run_bzr("add", "hello.txt")
+        result = self.run_bzr("status")[0]
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.run_bzr("commit", "-m", "added")
+        result = self.run_bzr("status", "-r", "0..1")[0]
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.build_tree(['world.txt'])
+        result = self.run_bzr("status", "-r", "0")[0]
+        self.assert_("added:\n  hello.txt\n" \
+                     "unknown:\n  world.txt\n" in result, result)
+
+        result2 = self.run_bzr("status", "-r", "0..")[0]
+        self.assertEquals(result2, result)
+
 

=== renamed file 'bzrlib/selftest/blackbox.py' => 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/selftest/blackbox.py	2005-08-23 06:17:55 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-04-19 08:31:08 +0000
@@ -15,112 +15,45 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# Mr. Smoketoomuch: I'm sorry?
+# Mr. Bounder: You'd better cut down a little then.
+# Mr. Smoketoomuch: Oh, I see! Smoke too much so I'd better cut down a little
+#                   then!
 
 """Black-box tests for bzr.
 
 These check that it behaves properly when it's invoked through the regular
-command-line interface.
-
-This always reinvokes bzr through a new Python interpreter, which is a
-bit inefficient but arguably tests in a way more representative of how
-it's normally invoked.
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
 """
 
+
+# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
+# Note: Please don't add new tests here, it's too big and bulky.  Instead add
+# them into small suites in bzrlib.tests.blackbox.test_FOO for the particular
+# UI command/aspect that is being tested.
+
+
+from cStringIO import StringIO
+import os
+import re
+import shutil
 import sys
 
-from bzrlib.selftest import InTempDir, BzrTestBase
-
-
-
-class ExternalBase(InTempDir):
-    def runbzr(self, args, retcode=0,backtick=False):
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-        if isinstance(args, basestring):
-            args = args.split()
-
-        if backtick:
-            return self.backtick(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
-        else:
-            return self.runcmd(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
-
-
-
-class MvCommand(BzrTestBase):
-    def runbzr(self):
-        """Test two modes of operation for mv"""
-        b = Branch('.', init=True)
-        self.build_tree(['a', 'c', 'subdir/'])
-        self.run_bzr('mv', 'a', 'b')
-        self.run_bzr('mv', 'b', 'subdir')
-        self.run_bzr('mv', 'subdir/b', 'a')
-        self.run_bzr('mv', 'a', 'b', 'subdir')
-        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
-
-
-
-class TestVersion(BzrTestBase):
-    """Check output from version command and master option is reasonable"""
-    def runTest(self):
-        # output is intentionally passed through to stdout so that we
-        # can see the version being tested
-        from cStringIO import StringIO
-        save_out = sys.stdout
-        try:
-            sys.stdout = tmp_out = StringIO()
-            
-            self.run_bzr('version')
-        finally:
-            sys.stdout = save_out
-
-        output = tmp_out.getvalue()
-        self.log('bzr version output:')
-        self.log(output)
-        
-        self.assert_(output.startswith('bzr (bazaar-ng) '))
-        self.assertNotEqual(output.index('Canonical'), -1)
-
-        # make sure --version is consistent
-        try:
-            sys.stdout = tmp_out = StringIO()
-            
-            self.run_bzr('--version')
-        finally:
-            sys.stdout = save_out
-
-        self.log('bzr --version output:')
-        self.log(tmp_out.getvalue())
-
-        self.assertEquals(output, tmp_out.getvalue())
-
-
-        
-
-
-class HelpCommands(ExternalBase):
-    def runTest(self):
-        self.runbzr('--help')
-        self.runbzr('help')
-        self.runbzr('help commands')
-        self.runbzr('help help')
-        self.runbzr('commit -h')
-
-
-class InitBranch(ExternalBase):
-    def runTest(self):
-        import os
-        self.runbzr(['init'])
-
-
-class UserIdentity(ExternalBase):
-    def runTest(self):
+import bzrlib
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks, pathjoin
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestCommands(ExternalBase):
+
+    def test_whoami(self):
         # this should always identify something, if only "john@localhost"
         self.runbzr("whoami")
         self.runbzr("whoami --email")
@@ -128,82 +61,754 @@
         self.assertEquals(self.runbzr("whoami --email",
                                       backtick=True).count('@'), 1)
         
-class UserIdentityBranch(ExternalBase):
-    def runTest(self):
-        # tests branch specific user identity
+    def test_whoami_branch(self):
+        """branch specific user identity works."""
         self.runbzr('init')
-        f = file('.bzr/email', 'wt')
-        f.write('Branch Identity <branch@identi.ty>')
-        f.close()
+        b = bzrlib.branch.Branch.open('.')
+        b.control_files.put_utf8('email', 'Branch Identity <branch@identi.ty>')
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
         whoami = self.runbzr("whoami",backtick=True)
         whoami_email = self.runbzr("whoami --email",backtick=True)
         self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
         self.assertTrue(whoami_email.startswith('branch@identi.ty'))
-
-
-class InvalidCommands(ExternalBase):
-    def runTest(self):
-        self.runbzr("pants", retcode=1)
-        self.runbzr("--pants off", retcode=1)
-        self.runbzr("diff --message foo", retcode=1)
-
-
-
-class EmptyCommit(ExternalBase):
-    def runTest(self):
+        # Verify that the environment variable overrides the value 
+        # in the file
+        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
+        self.assertTrue(whoami_email.startswith('other@environ.ment'))
+        if bzr_email is not None:
+            os.environ['BZREMAIL'] = bzr_email
+
+    def test_nick_command(self):
+        """bzr nick for viewing, setting nicknames"""
+        os.mkdir('me.dev')
+        os.chdir('me.dev')
+        self.runbzr('init')
+        nick = self.runbzr("nick",backtick=True)
+        self.assertEqual(nick, 'me.dev\n')
+        nick = self.runbzr("nick moo")
+        nick = self.runbzr("nick",backtick=True)
+        self.assertEqual(nick, 'moo\n')
+
+    def test_invalid_commands(self):
+        self.runbzr("pants", retcode=3)
+        self.runbzr("--pants off", retcode=3)
+        self.runbzr("diff --message foo", retcode=3)
+
+    def test_remove_deleted(self):
         self.runbzr("init")
-        self.build_tree(['hello.txt'])
-        self.runbzr("commit -m empty", retcode=1)
-        self.runbzr("add hello.txt")
-        self.runbzr("commit -m added")
-
-
-
-class IgnorePatterns(ExternalBase):
-    def runTest(self):
-        from bzrlib.branch import Branch
-        
-        b = Branch('.', init=True)
-        self.assertEquals(list(b.unknowns()), [])
+        self.build_tree(['a'])
+        self.runbzr(['add', 'a'])
+        self.runbzr(['commit', '-m', 'added a'])
+        os.unlink('a')
+        self.runbzr(['remove', 'a'])
+
+    def test_ignore_patterns(self):
+        self.runbzr('init')
+        self.assertEquals(self.capture('unknowns'), '')
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
-        self.assertEquals(list(b.unknowns()), [])
-        assert self.backtick('bzr unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         file('foo.c', 'wt').write('int main() {}')
-        self.assertEquals(list(b.unknowns()), ['foo.c'])
-        assert self.backtick('bzr unknowns') == 'foo.c\n'
+        self.assertEquals(self.capture('unknowns'), 'foo.c\n')
 
         self.runbzr(['add', 'foo.c'])
-        assert self.backtick('bzr unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         # 'ignore' works when creating the .bzignore file
         file('foo.blah', 'wt').write('blah')
-        self.assertEquals(list(b.unknowns()), ['foo.blah'])
+        self.assertEquals(self.capture('unknowns'), 'foo.blah\n')
         self.runbzr('ignore *.blah')
-        self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\n'
+        self.assertEquals(self.capture('unknowns'), '')
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\n')
 
         # 'ignore' works when then .bzrignore file already exists
         file('garh', 'wt').write('garh')
-        self.assertEquals(list(b.unknowns()), ['garh'])
-        assert self.backtick('bzr unknowns') == 'garh\n'
+        self.assertEquals(self.capture('unknowns'), 'garh\n')
         self.runbzr('ignore garh')
-        self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
-        
-
+        self.assertEquals(self.capture('unknowns'), '')
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\ngarh\n')
+
+    def test_revert(self):
+        self.runbzr('init')
+
+        file('hello', 'wt').write('foo')
+        self.runbzr('add hello')
+        self.runbzr('commit -m setup hello')
+
+        file('goodbye', 'wt').write('baz')
+        self.runbzr('add goodbye')
+        self.runbzr('commit -m setup goodbye')
+
+        file('hello', 'wt').write('bar')
+        file('goodbye', 'wt').write('qux')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'foo')
+        self.check_file_contents('goodbye', 'qux')
+        self.runbzr('revert')
+        self.check_file_contents('goodbye', 'baz')
+
+        os.mkdir('revertdir')
+        self.runbzr('add revertdir')
+        self.runbzr('commit -m f')
+        os.rmdir('revertdir')
+        self.runbzr('revert')
+
+        if has_symlinks():
+            os.symlink('/unlikely/to/exist', 'symlink')
+            self.runbzr('add symlink')
+            self.runbzr('commit -m f')
+            os.unlink('symlink')
+            self.runbzr('revert')
+            self.failUnlessExists('symlink')
+            os.unlink('symlink')
+            os.symlink('a-different-path', 'symlink')
+            self.runbzr('revert')
+            self.assertEqual('/unlikely/to/exist',
+                             os.readlink('symlink'))
+        else:
+            self.log("skipping revert symlink tests")
+        
+        file('hello', 'wt').write('xyz')
+        self.runbzr('commit -m xyz hello')
+        self.runbzr('revert -r 1 hello')
+        self.check_file_contents('hello', 'foo')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'xyz')
+        os.chdir('revertdir')
+        self.runbzr('revert')
+        os.chdir('..')
+
+    def test_mv_modes(self):
+        """Test two modes of operation for mv"""
+        self.runbzr('init')
+        self.build_tree(['a', 'c', 'subdir/'])
+        self.run_bzr_captured(['add', self.test_dir])
+        self.run_bzr_captured(['mv', 'a', 'b'])
+        self.run_bzr_captured(['mv', 'b', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
+        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
+
+    def test_main_version(self):
+        """Check output from version command and master option is reasonable"""
+        # output is intentionally passed through to stdout so that we
+        # can see the version being tested
+        output = self.runbzr('version', backtick=1)
+        self.log('bzr version output:')
+        self.log(output)
+        self.assert_(output.startswith('bzr (bazaar-ng) '))
+        self.assertNotEqual(output.index('Canonical'), -1)
+        # make sure --version is consistent
+        tmp_output = self.runbzr('--version', backtick=1)
+        self.log('bzr --version output:')
+        self.log(tmp_output)
+        self.assertEquals(output, tmp_output)
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_export(self):
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.example_branch()
+        self.runbzr('export ../latest')
+        self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
+        self.runbzr('export ../first -r 1')
+        self.assert_(not os.path.exists('../first/goodbye'))
+        self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.gz -r 1')
+        self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.bz2 -r 1')
+        self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
+
+        from tarfile import TarFile
+        self.runbzr('export ../first.tar -r 1')
+        self.assert_(os.path.isfile('../first.tar'))
+        tf = TarFile('../first.tar')
+        self.assert_('first/hello' in tf.getnames(), tf.getnames())
+        self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
+        self.runbzr('export ../first.tar.gz -r 1')
+        self.assert_(os.path.isfile('../first.tar.gz'))
+        self.runbzr('export ../first.tbz2 -r 1')
+        self.assert_(os.path.isfile('../first.tbz2'))
+        self.runbzr('export ../first.tar.bz2 -r 1')
+        self.assert_(os.path.isfile('../first.tar.bz2'))
+        self.runbzr('export ../first.tar.tbz2 -r 1')
+        self.assert_(os.path.isfile('../first.tar.tbz2'))
+
+        from bz2 import BZ2File
+        tf = TarFile('../first.tar.tbz2', 
+                     fileobj=BZ2File('../first.tar.tbz2', 'r'))
+        self.assert_('first.tar/hello' in tf.getnames(), tf.getnames())
+        self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
+        self.runbzr('export ../first2.tar -r 1 --root pizza')
+        tf = TarFile('../first2.tar')
+        self.assert_('pizza/hello' in tf.getnames(), tf.getnames())
+
+        from zipfile import ZipFile
+        self.runbzr('export ../first.zip -r 1')
+        self.failUnlessExists('../first.zip')
+        zf = ZipFile('../first.zip')
+        self.assert_('first/hello' in zf.namelist(), zf.namelist())
+        self.assertEqual(zf.read('first/hello'), 'foo')
+
+        self.runbzr('export ../first2.zip -r 1 --root pizza')
+        zf = ZipFile('../first2.zip')
+        self.assert_('pizza/hello' in zf.namelist(), zf.namelist())
+        
+        self.runbzr('export ../first-zip --format=zip -r 1')
+        zf = ZipFile('../first-zip')
+        self.assert_('first-zip/hello' in zf.namelist(), zf.namelist())
+
+    def test_branch(self):
+        """Branch from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        b = bzrlib.branch.Branch.open('b')
+        self.assertEqual('b\n', b.control_files.get_utf8('branch-name').read())
+        self.runbzr('branch a c -r 1')
+        os.chdir('b')
+        self.runbzr('commit -m foo --unchanged')
+        os.chdir('..')
+
+    def test_branch_basis(self):
+        # ensure that basis really does grab from the basis by having incomplete source
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch_and_tree('source')
+        # this gives us an incomplete repository
+        tree.bzrdir.open_repository().copy_content_into(source.branch.repository)
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        tree.bzrdir.open_branch().copy_content_into(source.branch)
+        tree.copy_content_into(source)
+        self.assertFalse(source.branch.repository.has_revision('2'))
+        dir = source.bzrdir
+        self.runbzr('branch source target --basis commit_tree')
+        target = bzrdir.BzrDir.open('target')
+        self.assertEqual('2', target.open_branch().last_revision())
+        self.assertEqual('2', target.open_workingtree().last_revision())
+        self.assertTrue(target.open_branch().repository.has_revision('2'))
+
+    def test_inventory(self):
+        bzr = self.runbzr
+        def output_equals(value, *args):
+            out = self.runbzr(['inventory'] + list(args), backtick=True)
+            self.assertEquals(out, value)
+
+        bzr('init')
+        open('a', 'wb').write('hello\n')
+        os.mkdir('b')
+
+        bzr('add a b')
+        bzr('commit -m add')
+
+        output_equals('a\n', '--kind', 'file')
+        output_equals('b\n', '--kind', 'directory')        
+
+    def test_ls(self):
+        """Test the abilities of 'bzr ls'"""
+        bzr = self.runbzr
+        def bzrout(*args, **kwargs):
+            kwargs['backtick'] = True
+            return self.runbzr(*args, **kwargs)
+
+        def ls_equals(value, *args):
+            out = self.runbzr(['ls'] + list(args), backtick=True)
+            self.assertEquals(out, value)
+
+        bzr('init')
+        open('a', 'wb').write('hello\n')
+
+        # Can't supply both
+        bzr('ls --verbose --null', retcode=3)
+
+        ls_equals('a\n')
+        ls_equals('?        a\n', '--verbose')
+        ls_equals('a\n', '--unknown')
+        ls_equals('', '--ignored')
+        ls_equals('', '--versioned')
+        ls_equals('a\n', '--unknown', '--ignored', '--versioned')
+        ls_equals('', '--ignored', '--versioned')
+        ls_equals('a\0', '--null')
+
+        bzr('add a')
+        ls_equals('V        a\n', '--verbose')
+        bzr('commit -m add')
+        
+        os.mkdir('subdir')
+        ls_equals('V        a\n'
+                  '?        subdir/\n'
+                  , '--verbose')
+        open('subdir/b', 'wb').write('b\n')
+        bzr('add')
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        bzr('commit -m subdir')
+
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--non-recursive')
+
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  , '--verbose', '--non-recursive')
+
+        # Check what happens in a sub-directory
+        os.chdir('subdir')
+        ls_equals('b\n')
+        ls_equals('b\0'
+                  , '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--from-root')
+        ls_equals('a\0'
+                  'subdir\0'
+                  'subdir/b\0'
+                  , '--from-root', '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--from-root', '--non-recursive')
+
+        os.chdir('..')
+
+        # Check what happens when we supply a specific revision
+        ls_equals('a\n', '--revision', '1')
+        ls_equals('V        a\n'
+                  , '--verbose', '--revision', '1')
+
+        os.chdir('subdir')
+        ls_equals('', '--revision', '1')
+
+        # Now try to do ignored files.
+        os.chdir('..')
+        open('blah.py', 'wb').write('unknown\n')
+        open('blah.pyo', 'wb').write('ignored\n')
+        ls_equals('a\n'
+                  'blah.py\n'
+                  'blah.pyo\n'
+                  'subdir\n'
+                  'subdir/b\n')
+        ls_equals('V        a\n'
+                  '?        blah.py\n'
+                  'I        blah.pyo\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        ls_equals('blah.pyo\n'
+                  , '--ignored')
+        ls_equals('blah.py\n'
+                  , '--unknown')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--versioned')
+
+    def test_cat(self):
+        self.runbzr('init')
+        file("myfile", "wb").write("My contents\n")
+        self.runbzr('add')
+        self.runbzr('commit -m myfile')
+        self.run_bzr_captured('cat -r 1 myfile'.split(' '))
+
+    def test_pull_verbose(self):
+        """Pull changes from one branch to another and watch the output."""
+
+        os.mkdir('a')
+        os.chdir('a')
+
+        bzr = self.runbzr
+        self.example_branch()
+
+        os.chdir('..')
+        bzr('branch a b')
+        os.chdir('b')
+        open('b', 'wb').write('else\n')
+        bzr('add b')
+        bzr(['commit', '-m', 'added b'])
+
+        os.chdir('../a')
+        out = bzr('pull --verbose ../b', backtick=True)
+        self.failIfEqual(out.find('Added Revisions:'), -1)
+        self.failIfEqual(out.find('message:\n  added b'), -1)
+        self.failIfEqual(out.find('added b'), -1)
+
+        # Check that --overwrite --verbose prints out the removed entries
+        bzr('commit -m foo --unchanged')
+        os.chdir('../b')
+        bzr('commit -m baz --unchanged')
+        bzr('pull ../a', retcode=3)
+        out = bzr('pull --overwrite --verbose ../a', backtick=1)
+
+        remove_loc = out.find('Removed Revisions:')
+        self.failIfEqual(remove_loc, -1)
+        added_loc = out.find('Added Revisions:')
+        self.failIfEqual(added_loc, -1)
+
+        removed_message = out.find('message:\n  baz')
+        self.failIfEqual(removed_message, -1)
+        self.failUnless(remove_loc < removed_message < added_loc)
+
+        added_message = out.find('message:\n  foo')
+        self.failIfEqual(added_message, -1)
+        self.failUnless(added_loc < added_message)
+        
+    def test_locations(self):
+        """Using and remembering different locations"""
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m unchanged --unchanged')
+        self.runbzr('pull', retcode=3)
+        self.runbzr('merge', retcode=3)
+        self.runbzr('branch . ../b')
+        os.chdir('../b')
+        self.runbzr('pull')
+        self.runbzr('branch . ../c')
+        self.runbzr('pull ../c')
+        self.runbzr('merge')
+        os.chdir('../a')
+        self.runbzr('pull ../b')
+        self.runbzr('pull')
+        self.runbzr('pull ../c')
+        self.runbzr('branch ../c ../d')
+        shutil.rmtree('../c')
+        self.runbzr('pull')
+        os.chdir('../b')
+        self.runbzr('pull')
+        os.chdir('../d')
+        self.runbzr('pull', retcode=3)
+        self.runbzr('pull ../a --remember')
+        self.runbzr('pull')
+        
+    def test_add_reports(self):
+        """add command prints the names of added files."""
+        self.runbzr('init')
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
+        out = self.run_bzr_captured(['add'], retcode=0)[0]
+        # the ordering is not defined at the moment
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'added dir',
+                           'added dir/sub.txt',
+                           'added top.txt',
+                           'ignored 1 file(s) matching "CVS"'],
+                          results)
+        out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'ignored CVS matching "CVS"'],
+                          results)
+
+    def test_add_quiet_is(self):
+        """add -q does not print the names of added files."""
+        self.runbzr('init')
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+        out = self.run_bzr_captured(['add', '-q'], retcode=0)[0]
+        # the ordering is not defined at the moment
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals([''], results)
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        self.runbzr('init')
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
+        self.run_bzr('add', 'inertiatic/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        self.assertEquals(self.capture('unknowns'), 'veil\n')
+        self.run_bzr('add', 'veil/cerpin/taxt')
+        self.assertEquals(self.capture('unknowns'), '')
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        self.assertEquals(self.capture('unknowns'), 'cicatriz\n')
+        self.run_bzr('add', 'inertiatic/../cicatriz/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        self.runbzr('init')
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
+        self.run_bzr('add', '--no-recurse', 'inertiatic')
+        self.assertEquals(self.capture('unknowns'), 'inertiatic/esp\n')
+        self.run_bzr('add', 'inertiatic/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+    def test_subdir_add(self):
+        """Add in subdirectory should add only things from there down"""
+        from bzrlib.workingtree import WorkingTree
+        
+        eq = self.assertEqual
+        ass = self.assert_
+        chdir = os.chdir
+        
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        self.build_tree(['src/', 'README'])
+        
+        eq(sorted(t.unknowns()),
+           ['README', 'src'])
+        
+        self.run_bzr('add', 'src')
+        
+        self.build_tree(['src/foo.c'])
+        
+        chdir('src')
+        self.run_bzr('add')
+        
+        self.assertEquals(self.capture('unknowns'), 'README\n')
+        eq(len(t.read_working_inventory()), 3)
+                
+        chdir('..')
+        self.run_bzr('add')
+        self.assertEquals(self.capture('unknowns'), '')
+        self.run_bzr('check')
+
+    def test_unknown_command(self):
+        """Handling of unknown command."""
+        out, err = self.run_bzr_captured(['fluffy-badger'],
+                                         retcode=3)
+        self.assertEquals(out, '')
+        err.index('unknown command')
+
+    def create_conflicts(self):
+        """Create a conflicted tree"""
+        os.mkdir('base')
+        os.chdir('base')
+        file('hello', 'wb').write("hi world")
+        file('answer', 'wb').write("42")
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr('commit -m base')
+        self.runbzr('branch . ../other')
+        self.runbzr('branch . ../this')
+        os.chdir('../other')
+        file('hello', 'wb').write("Hello.")
+        file('answer', 'wb').write("Is anyone there?")
+        self.runbzr('commit -m other')
+        os.chdir('../this')
+        file('hello', 'wb').write("Hello, world")
+        self.runbzr('mv answer question')
+        file('question', 'wb').write("What do you get when you multiply six"
+                                   "times nine?")
+        self.runbzr('commit -m this')
+
+    def test_remerge(self):
+        """Remerge command works as expected"""
+        self.create_conflicts()
+        self.runbzr('merge ../other --show-base', retcode=1)
+        conflict_text = file('hello').read()
+        assert '|||||||' in conflict_text
+        assert 'hi world' in conflict_text
+        self.runbzr('remerge', retcode=1)
+        conflict_text = file('hello').read()
+        assert '|||||||' not in conflict_text
+        assert 'hi world' not in conflict_text
+        os.unlink('hello.OTHER')
+        os.unlink('question.OTHER')
+        self.runbzr('remerge jello --merge-type weave', retcode=3)
+        self.runbzr('remerge hello --merge-type weave', retcode=1)
+        assert os.path.exists('hello.OTHER')
+        self.assertIs(False, os.path.exists('question.OTHER'))
+        file_id = self.runbzr('file-id hello')
+        file_id = self.runbzr('file-id hello.THIS', retcode=3)
+        self.runbzr('remerge --merge-type weave', retcode=1)
+        assert os.path.exists('hello.OTHER')
+        assert not os.path.exists('hello.BASE')
+        assert '|||||||' not in conflict_text
+        assert 'hi world' not in conflict_text
+        self.runbzr('remerge . --merge-type weave --show-base', retcode=3)
+        self.runbzr('remerge . --show-base --reprocess', retcode=3)
+        self.runbzr('remerge . --merge-type weave --reprocess', retcode=1)
+        self.runbzr('remerge hello --show-base', retcode=1)
+        self.runbzr('remerge hello --reprocess', retcode=1)
+        self.runbzr('resolve --all')
+        self.runbzr('commit -m done',)
+        self.runbzr('remerge', retcode=3)
+
+    def test_status(self):
+        os.mkdir('branch1')
+        os.chdir('branch1')
+        self.runbzr('init')
+        self.runbzr('commit --unchanged --message f')
+        self.runbzr('branch . ../branch2')
+        self.runbzr('branch . ../branch3')
+        self.runbzr('commit --unchanged --message peter')
+        os.chdir('../branch2')
+        self.runbzr('merge ../branch1')
+        self.runbzr('commit --unchanged --message pumpkin')
+        os.chdir('../branch3')
+        self.runbzr('merge ../branch2')
+        message = self.capture('status')
+
+
+    def test_conflicts(self):
+        """Handling of merge conflicts"""
+        self.create_conflicts()
+        self.runbzr('merge ../other --show-base', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('<<<<<<<' in conflict_text)
+        self.assert_('>>>>>>>' in conflict_text)
+        self.assert_('=======' in conflict_text)
+        self.assert_('|||||||' in conflict_text)
+        self.assert_('hi world' in conflict_text)
+        self.runbzr('revert')
+        self.runbzr('resolve --all')
+        self.runbzr('merge ../other', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('|||||||' not in conflict_text)
+        self.assert_('hi world' not in conflict_text)
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "Text conflict in hello\nText conflict in"
+                                  " question\n")
+        result = self.runbzr('status', backtick=1)
+        self.assert_("conflicts:\n  Text conflict in hello\n"
+                     "  Text conflict in question\n" in result, result)
+        self.runbzr('resolve hello')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "Text conflict in question\n")
+        self.runbzr('commit -m conflicts', retcode=3)
+        self.runbzr('resolve --all')
+        result = self.runbzr('conflicts', backtick=1)
+        self.runbzr('commit -m conflicts')
+        self.assertEquals(result, "")
+
+    def test_push(self):
+        # create a source branch
+        os.mkdir('my-branch')
+        os.chdir('my-branch')
+        self.example_branch()
+
+        # with no push target, fail
+        self.runbzr('push', retcode=3)
+        # with an explicit target work
+        self.runbzr('push ../output-branch')
+        # with an implicit target work
+        self.runbzr('push')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        # advance this branch
+        self.runbzr('commit --unchanged -m unchanged')
+
+        os.chdir('../output-branch')
+        # There is no longer a difference as long as we have
+        # access to the working tree
+        self.runbzr('diff')
+
+        # But we should be missing a revision
+        self.runbzr('missing ../my-branch', retcode=1)
+
+        # diverge the branches
+        self.runbzr('commit --unchanged -m unchanged')
+        os.chdir('../my-branch')
+        # cannot push now
+        self.runbzr('push', retcode=3)
+        # and there are difference
+        self.runbzr('missing ../output-branch', retcode=1)
+        self.runbzr('missing --verbose ../output-branch', retcode=1)
+        # but we can force a push
+        self.runbzr('push --overwrite')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        
+        # pushing to a new dir with no parent should fail
+        self.runbzr('push ../missing/new-branch', retcode=3)
+        # unless we provide --create-prefix
+        self.runbzr('push --create-prefix ../missing/new-branch')
+        # nothing missing
+        self.runbzr('missing ../missing/new-branch')
+
+    def test_external_command(self):
+        """Test that external commands can be run by setting the path
+        """
+        # We don't at present run bzr in a subprocess for blackbox tests, and so 
+        # don't really capture stdout, only the internal python stream.
+        # Therefore we don't use a subcommand that produces any output or does
+        # anything -- we just check that it can be run successfully.  
+        cmd_name = 'test-command'
+        if sys.platform == 'win32':
+            cmd_name += '.bat'
+        oldpath = os.environ.get('BZRPATH', None)
+        bzr = self.capture
+        try:
+            if os.environ.has_key('BZRPATH'):
+                del os.environ['BZRPATH']
+
+            f = file(cmd_name, 'wb')
+            if sys.platform == 'win32':
+                f.write('@echo off\n')
+            else:
+                f.write('#!/bin/sh\n')
+            # f.write('echo Hello from test-command')
+            f.close()
+            os.chmod(cmd_name, 0755)
+
+            # It should not find the command in the local 
+            # directory by default, since it is not in my path
+            bzr(cmd_name, retcode=3)
+
+            # Now put it into my path
+            os.environ['BZRPATH'] = '.'
+
+            bzr(cmd_name)
+
+            # Make sure empty path elements are ignored
+            os.environ['BZRPATH'] = os.pathsep
+
+            bzr(cmd_name, retcode=3)
+
+        finally:
+            if oldpath:
+                os.environ['BZRPATH'] = oldpath
+
+
+def listdir_sorted(dir):
+    L = os.listdir(dir)
+    L.sort()
+    return L
 
 
 class OldTests(ExternalBase):
-    # old tests moved from ./testbzr
-    def runTest(self):
+    """old tests moved from ./testbzr."""
+
+    def test_bzr(self):
         from os import chdir, mkdir
         from os.path import exists
-        import os
 
         runbzr = self.runbzr
-        backtick = self.backtick
+        capture = self.capture
         progress = self.log
 
         progress("basic branch creation")
@@ -211,8 +816,8 @@
         chdir('branch1')
         runbzr('init')
 
-        self.assertEquals(backtick('bzr root').rstrip(),
-                          os.path.join(self.test_dir, 'branch1'))
+        self.assertEquals(capture('root').rstrip(),
+                          pathjoin(self.test_dir, 'branch1'))
 
         progress("status of new file")
 
@@ -220,143 +825,144 @@
         f.write('hello world!\n')
         f.close()
 
-        out = backtick("bzr unknowns")
-        self.assertEquals(out, 'test.txt\n')
-
-        out = backtick("bzr status")
-        assert out == 'unknown:\n  test.txt\n'
-
-        out = backtick("bzr status --all")
-        assert out == "unknown:\n  test.txt\n"
-
-        out = backtick("bzr status test.txt --all")
-        assert out == "unknown:\n  test.txt\n"
+        self.assertEquals(capture('unknowns'), 'test.txt\n')
+
+        out = capture("status")
+        self.assertEquals(out, 'unknown:\n  test.txt\n')
+
+        out = capture("status --all")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
+
+        out = capture("status test.txt --all")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
 
-        out = backtick("bzr status test.txt")
-        assert out == "unknown:\n  test.txt\n"
+        out = capture("status test.txt")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
-        out = backtick("bzr status")
-        assert out == ("unknown:\n"
-                       "  test.txt\n"
-                       "  test2.txt\n")
+        out = capture("status")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n" "  test2.txt\n"))
 
         os.unlink('test2.txt')
 
         progress("command aliases")
-        out = backtick("bzr st --all")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        out = capture("st --all")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
-        out = backtick("bzr stat")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        out = capture("stat")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
         progress("command help")
         runbzr("help st")
         runbzr("help")
         runbzr("help commands")
-        runbzr("help slartibartfast", 1)
+        runbzr("help slartibartfast", 3)
 
-        out = backtick("bzr help ci")
+        out = capture("help ci")
         out.index('aliases: ')
 
         progress("can't rename unversioned file")
-        runbzr("rename test.txt new-test.txt", 1)
+        runbzr("rename test.txt new-test.txt", 3)
 
         progress("adding a file")
 
         runbzr("add test.txt")
-        assert backtick("bzr unknowns") == ''
-        assert backtick("bzr status --all") == ("added:\n"
-                                                "  test.txt\n")
+        self.assertEquals(capture("unknowns"), '')
+        self.assertEquals(capture("status --all"), ("added:\n" "  test.txt\n"))
 
         progress("rename newly-added file")
         runbzr("rename test.txt hello.txt")
-        assert os.path.exists("hello.txt")
-        assert not os.path.exists("test.txt")
+        self.assert_(os.path.exists("hello.txt"))
+        self.assert_(not os.path.exists("test.txt"))
 
-        assert backtick("bzr revno") == '0\n'
+        self.assertEquals(capture("revno"), '0\n')
 
         progress("add first revision")
         runbzr(['commit', '-m', 'add first revision'])
 
         progress("more complex renames")
         os.mkdir("sub1")
-        runbzr("rename hello.txt sub1", 1)
-        runbzr("rename hello.txt sub1/hello.txt", 1)
-        runbzr("move hello.txt sub1", 1)
+        runbzr("rename hello.txt sub1", 3)
+        runbzr("rename hello.txt sub1/hello.txt", 3)
+        runbzr("move hello.txt sub1", 3)
 
         runbzr("add sub1")
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
-        assert backtick("bzr relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
+        self.assertEqual(capture("relpath sub2/hello.txt"),
+                         pathjoin("sub2", "hello.txt\n"))
 
-        assert exists("sub2")
-        assert exists("sub2/hello.txt")
-        assert not exists("sub1")
-        assert not exists("hello.txt")
+        self.assert_(exists("sub2"))
+        self.assert_(exists("sub2/hello.txt"))
+        self.assert_(not exists("sub1"))
+        self.assert_(not exists("hello.txt"))
 
         runbzr(['commit', '-m', 'commit with some things moved to subdirs'])
 
         mkdir("sub1")
         runbzr('add sub1')
         runbzr('move sub2/hello.txt sub1')
-        assert not exists('sub2/hello.txt')
-        assert exists('sub1/hello.txt')
+        self.assert_(not exists('sub2/hello.txt'))
+        self.assert_(exists('sub1/hello.txt'))
         runbzr('move sub2 sub1')
-        assert not exists('sub2')
-        assert exists('sub1/sub2')
+        self.assert_(not exists('sub2'))
+        self.assert_(exists('sub1/sub2'))
 
         runbzr(['commit', '-m', 'rename nested subdirectories'])
 
         chdir('sub1/sub2')
-        self.assertEquals(backtick('bzr root')[:-1],
-                          os.path.join(self.test_dir, 'branch1'))
+        self.assertEquals(capture('root')[:-1],
+                          pathjoin(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
-        assert exists('./hello.txt')
-        assert backtick('bzr relpath hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
-        assert backtick('bzr relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assert_(exists('./hello.txt'))
+        self.assertEquals(capture('relpath hello.txt'),
+                          pathjoin('sub1', 'sub2', 'hello.txt') + '\n')
+        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        assert backtick('bzr relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assertEquals(capture('relpath sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
 
         runbzr('move sub2/hello.txt .')
-        assert exists('hello.txt')
+        self.assert_(exists('hello.txt'))
 
         f = file('hello.txt', 'wt')
         f.write('some nice new content\n')
         f.close()
 
         f = file('msg.tmp', 'wt')
-        f.write('this is my new commit\n')
+        f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
 
-        assert backtick('bzr revno') == '5\n'
+        self.assertEquals(capture('revno'), '5\n')
         runbzr('export -r 5 export-5.tmp')
         runbzr('export export.tmp')
 
         runbzr('log')
         runbzr('log -v')
         runbzr('log -v --forward')
-        runbzr('log -m', retcode=1)
-        log_out = backtick('bzr log -m commit')
-        assert "this is my new commit" in log_out
-        assert "rename nested" not in log_out
-        assert 'revision-id' not in log_out
-        assert 'revision-id' in backtick('bzr log --show-ids -m commit')
+        runbzr('log -m', retcode=3)
+        log_out = capture('log -m commit')
+        self.assert_("this is my new commit\n  and" in log_out)
+        self.assert_("rename nested" not in log_out)
+        self.assert_('revision-id' not in log_out)
+        self.assert_('revision-id' in capture('log --show-ids -m commit'))
+
+        log_out = capture('log --line')
+        for line in log_out.splitlines():
+            self.assert_(len(line) <= 79, len(line))
+        self.assert_("this is my new commit and" in log_out)
 
 
         progress("file with spaces in name")
         mkdir('sub directory')
         file('sub directory/file with spaces ', 'wt').write('see how this works\n')
         runbzr('add .')
-        runbzr('diff')
+        runbzr('diff', retcode=1)
         runbzr('commit -m add-spaces')
         runbzr('check')
 
@@ -365,28 +971,172 @@
 
         runbzr('info')
 
-
-
-
-
-
-class RevertCommand(ExternalBase):
-    def runTest(self):
-        self.runbzr('init')
-
+        if has_symlinks():
+            progress("symlinks")
+            mkdir('symlinks')
+            chdir('symlinks')
+            runbzr('init')
+            os.symlink("NOWHERE1", "link1")
+            runbzr('add link1')
+            self.assertEquals(self.capture('unknowns'), '')
+            runbzr(['commit', '-m', '1: added symlink link1'])
+    
+            mkdir('d1')
+            runbzr('add d1')
+            self.assertEquals(self.capture('unknowns'), '')
+            os.symlink("NOWHERE2", "d1/link2")
+            self.assertEquals(self.capture('unknowns'), 'd1/link2\n')
+            # is d1/link2 found when adding d1
+            runbzr('add d1')
+            self.assertEquals(self.capture('unknowns'), '')
+            os.symlink("NOWHERE3", "d1/link3")
+            self.assertEquals(self.capture('unknowns'), 'd1/link3\n')
+            runbzr(['commit', '-m', '2: added dir, symlink'])
+    
+            runbzr('rename d1 d2')
+            runbzr('move d2/link2 .')
+            runbzr('move link1 d2')
+            self.assertEquals(os.readlink("./link2"), "NOWHERE2")
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
+            runbzr('add d2/link3')
+            runbzr('diff', retcode=1)
+            runbzr(['commit', '-m', '3: rename of dir, move symlinks, add link3'])
+    
+            os.unlink("link2")
+            os.symlink("TARGET 2", "link2")
+            os.unlink("d2/link1")
+            os.symlink("TARGET 1", "d2/link1")
+            runbzr('diff', retcode=1)
+            self.assertEquals(self.capture("relpath d2/link1"), "d2/link1\n")
+            runbzr(['commit', '-m', '4: retarget of two links'])
+    
+            runbzr('remove d2/link1')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '5: remove d2/link1'])
+            # try with the rm alias
+            runbzr('add d2/link1')
+            runbzr(['commit', '-m', '6: add d2/link1'])
+            runbzr('rm d2/link1')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '7: remove d2/link1'])
+    
+            os.mkdir("d1")
+            runbzr('add d1')
+            runbzr('rename d2/link3 d1/link3new')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '8: remove d2/link1, move/rename link3'])
+            
+            runbzr(['check'])
+            
+            runbzr(['export', '-r', '1', 'exp1.tmp'])
+            chdir("exp1.tmp")
+            self.assertEquals(listdir_sorted("."), [ "link1" ])
+            self.assertEquals(os.readlink("link1"), "NOWHERE1")
+            chdir("..")
+            
+            runbzr(['export', '-r', '2', 'exp2.tmp'])
+            chdir("exp2.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d1", "link1" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '3', 'exp3.tmp'])
+            chdir("exp3.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
+            self.assertEquals(os.readlink("link2")   , "NOWHERE2")
+            chdir("..")
+            
+            runbzr(['export', '-r', '4', 'exp4.tmp'])
+            chdir("exp4.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(os.readlink("d2/link1"), "TARGET 1")
+            self.assertEquals(os.readlink("link2")   , "TARGET 2")
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '5', 'exp5.tmp'])
+            chdir("exp5.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assert_(os.path.islink("link2"))
+            self.assert_(listdir_sorted("d2")== [ "link3" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '8', 'exp6.tmp'])
+            chdir("exp6.tmp")
+            self.assertEqual(listdir_sorted("."), [ "d1", "d2", "link2"])
+            self.assertEquals(listdir_sorted("d1"), [ "link3new" ])
+            self.assertEquals(listdir_sorted("d2"), [])
+            self.assertEquals(os.readlink("d1/link3new"), "NOWHERE3")
+            chdir("..")
+        else:
+            progress("skipping symlink tests")
+
+
+class RemoteTests(object):
+    """Test bzr ui commands against remote branches."""
+
+    def test_branch(self):
+        os.mkdir('from')
+        wt = self.make_branch_and_tree('from')
+        branch = wt.branch
+        wt.commit('empty commit for nonsense', allow_pointless=True)
+        url = self.get_readonly_url('from')
+        self.run_bzr('branch', url, 'to')
+        branch = Branch.open('to')
+        self.assertEqual(1, len(branch.revision_history()))
+        # the branch should be set in to to from
+        self.assertEqual(url + '/', branch.get_parent())
+
+    def test_log(self):
+        self.build_tree(['branch/', 'branch/file'])
+        self.capture('init branch')
+        self.capture('add branch/file')
+        self.capture('commit -m foo branch')
+        url = self.get_readonly_url('branch/file')
+        output = self.capture('log %s' % url)
+        self.assertEqual(8, len(output.split('\n')))
+        
+    def test_check(self):
+        self.build_tree(['branch/', 'branch/file'])
+        self.capture('init branch')
+        self.capture('add branch/file')
+        self.capture('commit -m foo branch')
+        url = self.get_readonly_url('branch/')
+        self.run_bzr('check', url)
+    
+    def test_push(self):
+        # create a source branch
+        os.mkdir('my-branch')
+        os.chdir('my-branch')
+        self.run_bzr('init')
         file('hello', 'wt').write('foo')
-        self.runbzr('add hello')
-        self.runbzr('commit -m setup hello')
-
-        file('goodbye', 'wt').write('baz')
-        self.runbzr('add goodbye')
-        self.runbzr('commit -m setup goodbye')
-        
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-        self.check_file_contents('goodbye', 'qux')
-        self.runbzr('revert')
-        self.check_file_contents('goodbye', 'baz')
-
+        self.run_bzr('add', 'hello')
+        self.run_bzr('commit', '-m', 'setup')
+
+        # with an explicit target work
+        self.run_bzr('push', self.get_url('output-branch'))
+
+    
+class HTTPTests(TestCaseWithWebserver, RemoteTests):
+    """Test various commands against a HTTP server."""
+    
+    
+class SFTPTestsAbsolute(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using abs paths."""
+
+    
+class SFTPTestsAbsoluteSibling(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using abs paths."""
+
+    def setUp(self):
+        super(SFTPTestsAbsoluteSibling, self).setUp()
+        self._override_home = '/dev/noone/runs/tests/here'
+
+    
+class SFTPTestsRelative(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using homedir rel paths."""
+
+    def setUp(self):
+        super(SFTPTestsRelative, self).setUp()
+        self._get_remote_is_absolute = False

=== added file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2006-04-02 02:38:13 +0000
@@ -0,0 +1,86 @@
+"""\
+Test the uncommit command.
+"""
+
+import os
+
+from bzrlib.bzrdir import BzrDirMetaFormat1
+from bzrlib.errors import BzrError, BoundBranchOutOfDate
+from bzrlib.uncommit import uncommit
+from bzrlib.tests import TestCaseInTempDir
+
+class TestUncommit(TestCaseInTempDir):
+    def test_uncommit(self):
+        """Test uncommit functionality."""
+        bzr = self.capture 
+        os.mkdir('branch')
+        os.chdir('branch')
+        bzr('init')
+        self.build_tree(['a', 'b', 'c'])
+
+        bzr('add')
+        bzr('commit -m initial')
+
+        self.assertEquals(bzr('revno'), '1\n')
+
+        open('a', 'wb').write('new contents of a\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+        bzr('commit -m second')
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --dry-run --force')
+        self.failIfEqual(txt.find('Dry-run'), -1)
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --force')
+
+        self.assertEquals(bzr('revno'), '1\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+        
+        bzr('checkout . ../checkout')
+        os.chdir('../checkout')
+        self.assertEquals("", bzr('status'))
+        self.assertEquals(bzr('revno'), '1\n')
+
+        open('a', 'wb').write('new contents of a\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+        bzr('commit -m second')
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --dry-run --force')
+        self.failIfEqual(txt.find('Dry-run'), -1)
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --force')
+
+        self.assertEquals(bzr('revno'), '1\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+
+    def test_uncommit_bound(self):
+        os.mkdir('a')
+        a = BzrDirMetaFormat1().initialize('a')
+        a.create_repository()
+        a.create_branch()
+        t = a.create_workingtree()
+        t.commit('commit 1')
+        t.commit('commit 2')
+        t.commit('commit 3')
+        b = t.bzrdir.sprout('b').open_branch()
+        b.bind(t.branch)
+        uncommit(b)
+        t.set_last_revision(t.branch.last_revision())
+        self.assertEqual(len(b.revision_history()), 2)
+        self.assertEqual(len(t.branch.revision_history()), 2)
+        t.commit('commit 3b')
+        self.assertRaises(BoundBranchOutOfDate, uncommit, b)
+        b.pull(t.branch)
+        uncommit(b)
+

=== added file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_update.py	2006-03-05 11:48:54 +0000
@@ -0,0 +1,132 @@
+# Copyright (C) 2006 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for the update command of bzr."""
+
+
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestUpdate(ExternalBase):
+
+    def test_update_standalone_trivial(self):
+        self.runbzr("init")
+        out, err = self.runbzr('update')
+        self.assertEqual('Tree is up to date.\n', err)
+        self.assertEqual('', out)
+
+    def test_update_up_to_date_light_checkout(self):
+        self.make_branch_and_tree('branch')
+        self.runbzr('checkout --lightweight branch checkout')
+        out, err = self.runbzr('update checkout')
+        self.assertEqual('Tree is up to date.\n', err)
+        self.assertEqual('', out)
+
+    def test_update_out_of_date_standalone_tree(self):
+        # FIXME the default format has to change for this to pass
+        # because it currently uses the branch last-revision marker.
+        raise TestSkipped('default format too old')
+        self.make_branch_and_tree('branch')
+        # make a checkout
+        self.runbzr('checkout --lightweight branch checkout')
+        self.build_tree(['checkout/file'])
+        self.runbzr('add checkout/file')
+        self.runbzr('commit -m add-file checkout')
+        # now branch should be out of date
+        out,err = self.runbzr('update branch')
+        self.assertEqual('Updated to revision 1.\n', out)
+        self.assertEqual('', err)
+        self.failUnlessExists('branch/file')
+
+    def test_update_out_of_date_light_checkout(self):
+        self.make_branch_and_tree('branch')
+        # make two checkouts
+        self.runbzr('checkout --lightweight branch checkout')
+        self.runbzr('checkout --lightweight branch checkout2')
+        self.build_tree(['checkout/file'])
+        self.runbzr('add checkout/file')
+        self.runbzr('commit -m add-file checkout')
+        # now checkout2 should be out of date
+        out,err = self.runbzr('update checkout2')
+        self.assertEqual('All changes applied successfully.\n'
+                         'Updated to revision 1.\n',
+                         err)
+        self.assertEqual('', out)
+
+    def test_update_conflicts_returns_2(self):
+        self.make_branch_and_tree('branch')
+        # make two checkouts
+        self.runbzr('checkout --lightweight branch checkout')
+        self.build_tree(['checkout/file'])
+        self.runbzr('add checkout/file')
+        self.runbzr('commit -m add-file checkout')
+        self.runbzr('checkout --lightweight branch checkout2')
+        # now alter file in checkout
+        a_file = file('checkout/file', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.runbzr('commit -m checnge-file checkout')
+        # now checkout2 should be out of date
+        # make a local change to file
+        a_file = file('checkout2/file', 'wt')
+        a_file.write('Bar')
+        a_file.close()
+        out,err = self.runbzr('update checkout2', retcode=1)
+        self.assertEqual(['1 conflicts encountered.',
+                          'Updated to revision 2.'],
+                         err.split('\n')[1:3])
+        self.assertContainsRe(err, 'Text conflict in file\n')
+        self.assertEqual('', out)
+
+    def test_smoke_update_checkout_bound_branch_local_commits(self):
+        # smoke test for doing an update of a checkout of a bound
+        # branch with local commits.
+        self.make_branch_and_tree('master')
+        # make a bound branch
+        self.run_bzr('checkout', 'master', 'child')
+        # check that out
+        self.run_bzr('checkout', '--lightweight', 'child', 'checkout')
+        # change master
+        a_file = file('master/file', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'master')
+        self.run_bzr('commit', '-m', 'add file', 'master')
+        # change child
+        a_file = file('child/file_b', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'child')
+        self.run_bzr('commit', '--local', '-m', 'add file_b', 'child')
+        # check checkout
+        a_file = file('checkout/file_c', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'checkout')
+
+        # now, update checkout ->
+        # get all three files and a pending merge.
+        self.run_bzr('update', 'checkout')
+        wt = WorkingTree.open('checkout')
+        self.assertNotEqual([], wt.pending_merges())
+        self.failUnlessExists('checkout/file')
+        self.failUnlessExists('checkout/file_b')
+        self.failUnlessExists('checkout/file_c')
+        self.assertTrue(wt.has_filename('file_c'))

=== added file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2006-04-18 05:08:30 +0000
@@ -0,0 +1,183 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black box tests for the upgrade ui."""
+
+import os
+
+import bzrlib.bzrdir as bzrdir
+import bzrlib.repository as repository
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.blackbox import TestUIFactory
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.transport import get_transport
+import bzrlib.ui as ui
+
+
+class TestWithUpgradableBranches(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestWithUpgradableBranches, self).setUp()
+        self.old_format = bzrdir.BzrDirFormat.get_default_format()
+        self.old_ui_factory = ui.ui_factory
+        self.addCleanup(self.restoreDefaults)
+
+        ui.ui_factory = TestUIFactory()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        # FIXME RBC 20060120 we should be able to do this via ui calls only.
+        # setup a format 5 branch we can upgrade from.
+        t = get_transport(self.get_url())
+        t.mkdir('format_5_branch')
+        bzrdir.BzrDirFormat5().initialize(self.get_url('format_5_branch'))
+        bzrdir.BzrDir.create_standalone_workingtree('current_format_branch')
+        d = bzrdir.BzrDir.create('metadir_weave_branch')
+        d.create_repository()
+        d.create_branch()
+        d.create_workingtree()
+        self.run_bzr('checkout',
+                     '--lightweight',
+                     self.get_url('current_format_branch'),
+                     'current_format_checkout')
+
+    def restoreDefaults(self):
+        bzrdir.BzrDirFormat.set_default_format(self.old_format)
+        ui.ui_factory = self.old_ui_factory
+
+    def test_readonly_url_error(self):
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', self.get_readonly_url('format_5_branch')], 3)
+        self.assertEqual(out, "")
+        self.assertEqual(err, "bzr: ERROR: Upgrade URL cannot work with readonly URL's.\n")
+
+    def test_upgrade_up_to_date(self):
+        # when up to date we should get a message to that effect
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', 'current_format_branch'], 3)
+        self.assertEqual("", out)
+        self.assertEqualDiff("bzr: ERROR: The branch format Bazaar-NG meta "
+                             "directory, format 1 is already at the most "
+                             "recent format.\n", err)
+
+    def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
+        # when upgrading a checkout, the branch location and a suggestion
+        # to upgrade it should be emitted even if the checkout is up to 
+        # date
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', 'current_format_checkout'], 3)
+        self.assertEqual("This is a checkout. The branch (%s) needs to be "
+                         "upgraded separately.\n" 
+                         % get_transport(self.get_url('current_format_branch')).base,
+                         out)
+        self.assertEqualDiff("bzr: ERROR: The branch format Bazaar-NG meta "
+                             "directory, format 1 is already at the most "
+                             "recent format.\n", err)
+
+    def test_upgrade_checkout(self):
+        # upgrading a checkout should work
+        pass
+
+    def test_upgrade_repository_scans_branches(self):
+        # we should get individual upgrade notes for each branch even the 
+        # anonymous branch
+        pass
+
+    def test_ugrade_branch_in_repo(self):
+        # upgrading a branch in a repo should warn about not upgrading the repo
+        pass
+
+    def test_upgrade_explicit_metaformat(self):
+        # users can force an upgrade to metadir format.
+        url = get_transport(self.get_url('format_5_branch')).base
+        # check --format takes effect
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', '--format=metadir', url])
+        self.assertEqualDiff("""starting upgrade of %s
+making backup of tree history
+%s.bzr has been backed up to %s.bzr.backup
+if conversion fails, you can move this directory back to .bzr
+if it succeeds, you can remove this directory if you wish
+starting upgrade from format 5 to 6
+adding prefixes to weaves
+adding prefixes to revision-store
+starting upgrade from format 6 to metadir
+finished
+""" % (url, url, url), out)
+        self.assertEqualDiff("", err)
+        self.assertTrue(isinstance(
+            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDirMetaFormat1))
+
+    def test_upgrade_explicit_knit(self):
+        # users can force an upgrade to knit format from a metadir weave 
+        # branch
+        url = get_transport(self.get_url('metadir_weave_branch')).base
+        # check --format takes effect
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', '--format=knit', url])
+        self.assertEqualDiff("""starting upgrade of %s
+making backup of tree history
+%s.bzr has been backed up to %s.bzr.backup
+if conversion fails, you can move this directory back to .bzr
+if it succeeds, you can remove this directory if you wish
+starting repository conversion
+repository converted
+finished
+""" % (url, url, url), out)
+        self.assertEqualDiff("", err)
+        converted_dir = bzrdir.BzrDir.open(self.get_url('metadir_weave_branch'))
+        self.assertTrue(isinstance(converted_dir._format,
+                                   bzrdir.BzrDirMetaFormat1))
+        self.assertTrue(isinstance(converted_dir.open_repository()._format,
+                                   repository.RepositoryFormatKnit1))
+
+    def test_upgrade_repo(self):
+        self.run_bzr('init-repository', '--format=metadir', 'repo')
+        self.run_bzr('upgrade', '--format=knit', 'repo')
+
+
+class SFTPTests(TestCaseWithSFTPServer):
+    """Tests for upgrade over sftp."""
+
+    def setUp(self):
+        super(SFTPTests, self).setUp()
+        self.old_ui_factory = ui.ui_factory
+        self.addCleanup(self.restoreDefaults)
+
+        ui.ui_factory = TestUIFactory()
+
+    def restoreDefaults(self):
+        ui.ui_factory = self.old_ui_factory
+
+    def test_upgrade_url(self):
+        self.run_bzr('init', '--format=weave')
+        t = get_transport(self.get_url())
+        url = t.base
+        out, err = self.run_bzr('upgrade', '--format=knit', url)
+        self.assertEqualDiff("""starting upgrade of %s
+making backup of tree history
+%s.bzr has been backed up to %s.bzr.backup
+if conversion fails, you can move this directory back to .bzr
+if it succeeds, you can remove this directory if you wish
+starting upgrade from format 6 to metadir
+starting repository conversion
+repository converted
+finished
+""" % (url, url, url), out)
+        self.assertEqual('', err)

=== renamed file 'bzrlib/selftest/versioning.py' => 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/selftest/versioning.py	2005-08-01 20:39:06 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2006-02-11 11:58:06 +0000
@@ -17,31 +17,33 @@
 
 """Tests of simple versioning operations"""
 
-# TODO: test adding a file whose directory is not versioned
 # TODO: test trying to commit within a directory that is not yet added
 
 
 import os
-from bzrlib.selftest import InTempDir, BzrTestBase
+
+from bzrlib.tests import BzrTestBase, TestCaseInTempDir
 from bzrlib.branch import Branch
-
-
-class Mkdir(InTempDir):
-    def runTest(self): 
+from bzrlib.trace import mutter
+from bzrlib.osutils import pathjoin
+from bzrlib.workingtree import WorkingTree
+
+
+class TestVersioning(TestCaseInTempDir):
+    
+    def test_mkdir(self): 
         """Basic 'bzr mkdir' operation"""
-        from bzrlib.commands import run_bzr
 
-        run_bzr(['init'])
-        run_bzr(['mkdir', 'foo'])
+        self.run_bzr('init')
+        self.run_bzr('mkdir', 'foo')
         self.assert_(os.path.isdir('foo'))
 
-        self.assertRaises(OSError, run_bzr, ['mkdir', 'foo'])
+        self.run_bzr('mkdir', 'foo', retcode=3)
 
         from bzrlib.diff import compare_trees
-        from bzrlib.branch import Branch
-        b = Branch('.')
+        wt = WorkingTree.open('.')
         
-        delta = compare_trees(b.basis_tree(), b.working_tree())
+        delta = compare_trees(wt.basis_tree(), wt)
 
         self.log('delta.added = %r' % delta.added)
 
@@ -49,30 +51,83 @@
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-
-
-class AddInUnversioned(InTempDir):
-    def runTest(self):
-        """Try to add a file in an unversioned directory.
-
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
-        """
-        from bzrlib.branch import Branch
-        from bzrlib.errors import NotVersionedError
-
-        b = Branch('.', init=True)
-
-        self.build_tree(['foo/',
-                         'foo/hello'])
-
-        self.assertRaises(NotVersionedError,
-                          b.add,
-                          'foo/hello')
-        
-        
-class SubdirCommit(BzrTestBase):
-    def runTest(self):
+    def test_mkdir_in_subdir(self):
+        """'bzr mkdir' operation in subdirectory"""
+
+        self.run_bzr('init')
+        self.run_bzr('mkdir', 'dir')
+        self.assert_(os.path.isdir('dir'))
+
+        os.chdir('dir')
+        self.log('Run mkdir in subdir')
+        self.run_bzr('mkdir', 'subdir')
+        self.assert_(os.path.isdir('subdir'))
+        os.chdir('..')
+
+        from bzrlib.diff import compare_trees
+        wt = WorkingTree.open('.')
+        
+        delta = compare_trees(wt.basis_tree(), wt)
+
+        self.log('delta.added = %r' % delta.added)
+
+        self.assertEquals(len(delta.added), 2)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.assertEquals(delta.added[1][0], pathjoin('dir','subdir'))
+        self.failIf(delta.modified)
+
+    def test_mkdir_w_nested_trees(self):
+        """'bzr mkdir' with nested trees"""
+
+        self.run_bzr('init')
+        os.mkdir('a')
+        os.chdir('a')
+        self.run_bzr('init')
+        os.mkdir('b')
+        os.chdir('b')
+        self.run_bzr('init')
+        os.chdir('../..')
+
+        self.run_bzr('mkdir', 'dir', 'a/dir', 'a/b/dir')
+        self.failUnless(os.path.isdir('dir'))
+        self.failUnless(os.path.isdir('a/dir'))
+        self.failUnless(os.path.isdir('a/b/dir'))
+
+        from bzrlib.diff import compare_trees
+        wt = WorkingTree.open('.')
+        wt_a = WorkingTree.open('a')
+        wt_b = WorkingTree.open('a/b')
+        
+        delta = compare_trees(wt.basis_tree(), wt)
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+        delta = compare_trees(wt_a.basis_tree(), wt_a)
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+        delta = compare_trees(wt_b.basis_tree(), wt_b)
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+    def check_branch(self):
+        """After all the above changes, run the check and upgrade commands.
+
+        The upgrade should be a no-op."""
+        b = Branch.open(u'.')
+        mutter('branch has %d revisions', b.revno())
+        
+        mutter('check branch...')
+        from bzrlib.check import check
+        check(b, False)
+
+
+class SubdirCommit(TestCaseInTempDir):
+
+    def test_subdir_commit(self):
         """Test committing a subdirectory, and committing within a directory."""
         run_bzr = self.run_bzr
         eq = self.assertEqual
@@ -80,7 +135,7 @@
         self.build_tree(['a/', 'b/'])
         
         run_bzr('init')
-        b = Branch('.')
+        b = Branch.open(u'.')
         
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('old contents')
@@ -91,10 +146,11 @@
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('new contents')
             
+        mutter('start selective subdir commit')
         run_bzr('commit', 'a', '-m', 'commit a only')
         
-        old = b.revision_tree(b.lookup_revision(1))
-        new = b.revision_tree(b.lookup_revision(2))
+        old = b.repository.revision_tree(b.get_rev_id(1))
+        new = b.repository.revision_tree(b.get_rev_id(2))
         
         eq(new.get_file_by_path('b/two').read(), 'old contents')
         eq(new.get_file_by_path('top').read(), 'old contents')
@@ -103,50 +159,21 @@
         os.chdir('a')
         # commit from here should do nothing
         run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
-        v3 = b.revision_tree(b.lookup_revision(3))
+        v3 = b.repository.revision_tree(b.get_rev_id(3))
         eq(v3.get_file_by_path('b/two').read(), 'old contents')
         eq(v3.get_file_by_path('top').read(), 'old contents')
         eq(v3.get_file_by_path('a/one').read(), 'new contents')
                 
         # commit in subdirectory commits whole tree
         run_bzr('commit', '-m', 'commit whole tree from subdir')
-        v4 = b.revision_tree(b.lookup_revision(4))
+        v4 = b.repository.revision_tree(b.get_rev_id(4))
         eq(v4.get_file_by_path('b/two').read(), 'new contents')        
         eq(v4.get_file_by_path('top').read(), 'new contents')
         
         # TODO: factor out some kind of assert_tree_state() method
         
-        
-        
-class SubdirAdd(InTempDir):
-    def runTest(self):
-        """Add in subdirectory should add only things from there down"""
-        
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        
-        eq = self.assertEqual
-        ass = self.assert_
-        chdir = os.chdir
-        
-        b = Branch('.', init=True)
-        self.build_tree(['src/', 'README'])
-        
-        eq(sorted(b.unknowns()),
-           ['README', 'src'])
-        
-        eq(run_bzr(['add', 'src']), 0)
-        
-        self.build_tree(['src/foo.c'])
-        
-        chdir('src')
-        eq(run_bzr(['add']), 0)
-        
-        eq(sorted(b.unknowns()), 
-           ['README'])
-        eq(len(b.inventory), 3)
-                
-        chdir('..')
-        eq(run_bzr(['add']), 0)
-        eq(list(b.unknowns()), [])
-        
+
+if __name__ == '__main__':
+    import unittest
+    unittest.main()
+    

=== added directory 'bzrlib/tests/branch_implementations'
=== added file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2006-04-12 04:57:34 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Branch implementation tests for bzr.
+
+These test the conformance of all the branch variations to the expected API.
+Specific tests for individual formats are in the tests/test_branch file 
+rather than in tests/branch_implementations/*.py.
+"""
+
+from bzrlib.branch import (BranchFormat,
+                           BranchTestProviderAdapter,
+                           _legacy_formats,
+                           )
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_branch_implementations = [
+        'bzrlib.tests.branch_implementations.test_bound_sftp',
+        'bzrlib.tests.branch_implementations.test_branch',
+        'bzrlib.tests.branch_implementations.test_parent',
+        'bzrlib.tests.branch_implementations.test_permissions',
+        'bzrlib.tests.branch_implementations.test_pull',
+        'bzrlib.tests.branch_implementations.test_update',
+        ]
+    adapter = BranchTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        [(format, format._matchingbzrdir) for format in 
+         BranchFormat._formats.values() + _legacy_formats])
+    loader = TestLoader()
+    adapt_modules(test_branch_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/branch_implementations/test_bound_sftp.py'
--- bzrlib/tests/branch_implementations/test_bound_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-04-19 05:22:38 +0000
@@ -0,0 +1,371 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branches bound to an sftp branch."""
+
+
+import os
+
+import bzrlib
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import (BzrDir,
+                           BzrDirFormat,
+                           BzrDirFormat6,
+                           BzrDirMetaFormat1,
+                           )
+import bzrlib.errors as errors
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer, paramiko_loaded
+
+
+class BoundSFTPBranch(TestCaseWithSFTPServer):
+
+    def create_branches(self):
+        self.build_tree(['base/', 'base/a', 'base/b'])
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            wt_base = BzrDir.create_standalone_workingtree('base')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+    
+        b_base = wt_base.branch
+
+        wt_base.add('a')
+        wt_base.add('b')
+        wt_base.commit('first', rev_id='r@b-1')
+
+        wt_child = b_base.bzrdir.sprout('child').open_workingtree()
+        self.sftp_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(self.sftp_base)
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+
+        return b_base, wt_child
+
+    def tearDown(self):
+        self.sftp_base = None
+        bzrlib.transport.sftp.clear_connection_cache()
+        super(BoundSFTPBranch, self).tearDown()
+
+    def test_simple_binding(self):
+        self.build_tree(['base/', 'base/a', 'base/b', 'child/'])
+        wt_base = BzrDir.create_standalone_workingtree('base')
+
+        wt_base.add('a')
+        wt_base.add('b')
+        wt_base.commit('first', rev_id='r@b-1')
+
+        b_base = wt_base.branch
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            b_child = BzrDir.create_branch_convenience('child')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+        self.assertEqual(None, b_child.get_bound_location())
+        self.assertEqual(None, b_child.get_master_branch())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        b_child.bind(sftp_b_base)
+        self.assertEqual(sftp_b_base.base, b_child.get_bound_location())
+        # this line is more of a working tree test line, but - what the hey.
+        b_child.bzrdir.open_workingtree().update()
+        self.failUnlessExists('child/a')
+        self.failUnlessExists('child/b')
+
+        b_child.unbind()
+        self.assertEqual(None, b_child.get_bound_location())
+
+    def test_bound_commit(self):
+        b_base, wt_child = self.create_branches()
+
+        open('child/a', 'wb').write('new contents\n')
+        wt_child.commit('modified a', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+    def test_bound_fail(self):
+        # Make sure commit fails if out of date.
+        b_base, wt_child = self.create_branches()
+
+        open('base/a', 'wb').write('new base contents\n')
+        b_base.bzrdir.open_workingtree().commit('base', rev_id='r@b-2')
+
+        open('child/b', 'wb').write('new b child contents\n')
+        self.assertRaises(errors.BoundBranchOutOfDate,
+                wt_child.commit, 'child', rev_id='r@c-2')
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+
+        # This is all that cmd_update does
+        wt_child.pull(sftp_b_base, overwrite=False)
+
+        wt_child.commit('child', rev_id='r@c-3')
+
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                b_base.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                sftp_b_base.revision_history())
+
+    def test_double_binding(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child2 = wt_child.bzrdir.sprout('child2').open_workingtree()
+
+        wt_child2.branch.bind(wt_child.branch)
+
+        open('child2/a', 'wb').write('new contents\n')
+        self.assertRaises(errors.CommitToDoubleBoundBranch,
+                wt_child2.commit, 'child2', rev_id='r@d-2')
+
+    def test_unbinding(self):
+        from bzrlib.transport import get_transport
+        b_base, wt_child = self.create_branches()
+
+        # TestCaseWithSFTPServer only allows you to connect one time
+        # to the SFTP server. So we have to create a connection and
+        # keep it around, so that it can be reused
+        __unused_t = get_transport(self.get_url('.'))
+
+        wt_base = b_base.bzrdir.open_workingtree()
+        open('base/a', 'wb').write('new base contents\n')
+        wt_base.commit('base', rev_id='r@b-2')
+
+        open('child/b', 'wb').write('new b child contents\n')
+        self.assertRaises(errors.BoundBranchOutOfDate,
+                wt_child.commit, 'child', rev_id='r@c-2')
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        wt_child.branch.unbind()
+        wt_child.commit('child', rev_id='r@c-2')
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        self.assertRaises(errors.DivergedBranches,
+                wt_child.branch.bind, sftp_b_base)
+
+    def test_commit_remote_bound(self):
+        # Make sure it is detected if the current base
+        # suddenly is bound when child goes to commit
+        b_base, wt_child = self.create_branches()
+
+        b_base.bzrdir.sprout('newbase')
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        sftp_b_newbase = Branch.open(self.get_url('newbase'))
+
+        sftp_b_base.bind(sftp_b_newbase)
+
+        open('child/a', 'wb').write('new contents\n')
+        self.assertRaises(errors.CommitToDoubleBoundBranch,
+                wt_child.commit, 'failure', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], sftp_b_newbase.revision_history())
+
+    def test_pull_updates_both(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_newchild = b_base.bzrdir.sprout('newchild').open_workingtree()
+        open('newchild/b', 'wb').write('newchild b contents\n')
+        wt_newchild.commit('newchild', rev_id='r@d-2')
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_newchild.branch.revision_history())
+
+        wt_child.pull(wt_newchild.branch)
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@d-2'], b_base.revision_history())
+
+    def test_bind_diverged(self):
+        from bzrlib.builtins import merge
+
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+        open('child/a', 'ab').write('child contents\n')
+        wt_child.commit('child', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+
+        open('base/b', 'ab').write('base contents\n')
+        b_base.bzrdir.open_workingtree().commit('base', rev_id='r@b-2')
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+
+        self.assertRaises(errors.DivergedBranches,
+                wt_child.branch.bind, sftp_b_base)
+
+        # TODO: jam 20051230 merge_inner doesn't set pending merges
+        #       Is this on purpose?
+        #       merge_inner also doesn't fetch any missing revisions
+        #merge_inner(wt_child.branch, sftp_b_base.revision_tree('r@b-2'), 
+        #        wt_child.branch.revision_tree('r@b-1'))
+        # TODO: jam 20051230 merge(..., (None, None), ...) seems to
+        #       cause an infinite loop of some sort. It definitely doesn't
+        #       work, you have to use list notation
+        merge((sftp_b_base.base, 2), [None, None], this_dir=wt_child.branch.base)
+
+        self.assertEqual(['r@b-2'], wt_child.pending_merges())
+        wt_child.commit('merged', rev_id='r@c-3')
+
+        # After a merge, trying to bind again should succeed
+        # by pushing the new change to base
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
+                b_base.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
+                wt_child.branch.revision_history())
+
+    def test_bind_parent_ahead(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+
+        open('a', 'ab').write('base changes\n')
+        wt_base = b_base.bzrdir.open_workingtree()
+        wt_base.commit('base', rev_id='r@b-2')
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+
+        wt_child.branch.unbind()
+
+        # Check and make sure it also works if parent is ahead multiple
+        wt_base.commit('base 3', rev_id='r@b-3', allow_pointless=True)
+        wt_base.commit('base 4', rev_id='r@b-4', allow_pointless=True)
+        wt_base.commit('base 5', rev_id='r@b-5', allow_pointless=True)
+
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
+                b_base.revision_history())
+
+        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+
+        wt_child.branch.bind(sftp_b_base)
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
+                wt_child.branch.revision_history())
+
+    def test_bind_child_ahead(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+
+        wt_child.commit('child', rev_id='r@c-2', allow_pointless=True)
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+        # Check and make sure it also works if child is ahead multiple
+        wt_child.branch.unbind()
+        wt_child.commit('child 3', rev_id='r@c-3', allow_pointless=True)
+        wt_child.commit('child 4', rev_id='r@c-4', allow_pointless=True)
+        wt_child.commit('child 5', rev_id='r@c-5', allow_pointless=True)
+
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
+                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+        wt_child.branch.bind(sftp_b_base)
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
+                b_base.revision_history())
+
+    def test_commit_after_merge(self):
+        from bzrlib.builtins import merge
+
+        b_base, wt_child = self.create_branches()
+
+        # We want merge to be able to be a local only
+        # operation, because it does not alter the branch data.
+
+        # But we can't fail afterwards
+
+        wt_other = wt_child.bzrdir.sprout('other').open_workingtree()
+
+        open('other/c', 'wb').write('file c\n')
+        wt_other.add('c')
+        wt_other.commit('adding c', rev_id='r@d-2')
+
+        self.failIf(wt_child.branch.repository.has_revision('r@d-2'))
+        self.failIf(b_base.repository.has_revision('r@d-2'))
+
+        # TODO: jam 20051230 merge_inner doesn't set pending merges
+        #       Is this on purpose?
+        #       merge_inner also doesn't fetch any missing revisions
+        #merge_inner(wt_child.branch, b_other.revision_tree('r@d-2'),
+        #        wt_child.branch.revision_tree('r@b-1'))
+        merge((wt_other.branch.base, 2), [None, None], this_dir=wt_child.branch.base)
+
+        self.failUnlessExists('child/c')
+        self.assertEqual(['r@d-2'], wt_child.pending_merges())
+        self.failUnless(wt_child.branch.repository.has_revision('r@d-2'))
+        self.failIf(b_base.repository.has_revision('r@d-2'))
+
+        # Commit should succeed, and cause merged revisions to
+        # be pulled into base
+        wt_child.commit('merge other', rev_id='r@c-2')
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+        self.failUnless(b_base.repository.has_revision('r@d-2'))
+
+    def test_commit_fails(self):
+        b_base, wt_child = self.create_branches()
+
+        open('a', 'ab').write('child adds some text\n')
+
+        del b_base
+        os.rename('base', 'hidden_base')
+
+        self.assertRaises(errors.BoundBranchConnectionFailure,
+                wt_child.commit, 'added text', rev_id='r@c-2')
+
+    def test_pull_fails(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_other = wt_child.bzrdir.sprout('other').open_workingtree()
+        open('other/a', 'wb').write('new contents\n')
+        wt_other.commit('changed a', rev_id='r@d-2')
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_other.branch.revision_history())
+
+        del b_base
+        os.rename('base', 'hidden_base')
+
+        self.assertRaises(errors.BoundBranchConnectionFailure,
+                wt_child.pull, wt_other.branch)
+
+    # TODO: jam 20051231 We need invasive failure tests, so that we can show
+    #       performance even when something fails.
+
+
+if not paramiko_loaded:
+    del BoundSFTPBranch
+

=== renamed file 'bzrlib/selftest/testbranch.py' => 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/selftest/testbranch.py	2005-07-11 07:05:34 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-04-18 22:41:16 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,22 +14,526 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
-
-
-
-class TestAppendRevisions(InTempDir):
-    """Test appending more than one revision"""
-    def runTest(self):
-        from bzrlib.branch import Branch
-        br = Branch(".", init=True)
+"""Tests for branch implementations - tests a branch format."""
+
+import os
+import sys
+
+import bzrlib.branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.gpg
+from bzrlib.osutils import getcwd
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import HttpServer
+from bzrlib.transport.memory import MemoryServer
+from bzrlib.upgrade import upgrade
+from bzrlib.workingtree import WorkingTree
+
+
+# TODO: Make a branch using basis branch, and check that it 
+# doesn't request any files that could have been avoided, by 
+# hooking into the Transport.
+
+
+class TestCaseWithBranch(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithBranch, self).setUp()
+        self.branch = None
+
+    def get_branch(self):
+        if self.branch is None:
+            self.branch = self.make_branch('')
+        return self.branch
+
+    def make_branch(self, relpath, format=None):
+        repo = self.make_repository(relpath, format=format)
+        # fixme RBC 20060210 this isnt necessarily a fixable thing,
+        # Skipped is the wrong exception to raise.
+        try:
+            return self.branch_format.initialize(repo.bzrdir)
+        except errors.UninitializableFormat:
+            raise TestSkipped('Uninitializable branch format')
+
+    def make_repository(self, relpath, shared=False, format=None):
+        made_control = self.make_bzrdir(relpath, format=format)
+        return made_control.create_repository(shared=shared)
+
+
+class TestBranch(TestCaseWithBranch):
+
+    def test_append_revisions(self):
+        """Test appending more than one revision"""
+        br = self.get_branch()
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
-        
-
-
-TEST_CLASSES = [
-    TestAppendRevisions,
-    ]
+
+    def test_fetch_revisions(self):
+        """Test fetch-revision operation."""
+        get_transport(self.get_url()).mkdir('b1')
+        get_transport(self.get_url()).mkdir('b2')
+        wt = self.make_branch_and_tree('b1')
+        b1 = wt.branch
+        b2 = self.make_branch('b2')
+        file('b1/foo', 'w').write('hello')
+        wt.add(['foo'], ['foo-id'])
+        wt.commit('lala!', rev_id='revision-1', allow_pointless=False)
+
+        mutter('start fetch')
+        self.assertEqual((1, []), b2.fetch(b1))
+
+        rev = b2.repository.get_revision('revision-1')
+        tree = b2.repository.revision_tree('revision-1')
+        self.assertEqual(tree.get_file_text('foo-id'), 'hello')
+
+    def get_unbalanced_tree_pair(self):
+        """Return two branches, a and b, with one file in a."""
+        get_transport(self.get_url()).mkdir('a')
+        tree_a = self.make_branch_and_tree('a')
+        file('a/b', 'wb').write('b')
+        tree_a.add('b')
+        tree_a.commit("silly commit", rev_id='A')
+
+        get_transport(self.get_url()).mkdir('b')
+        tree_b = self.make_branch_and_tree('b')
+        return tree_a, tree_b
+
+    def get_balanced_branch_pair(self):
+        """Returns br_a, br_b as with one commit in a, and b has a's stores."""
+        tree_a, tree_b = self.get_unbalanced_tree_pair()
+        tree_b.branch.repository.fetch(tree_a.branch.repository)
+        return tree_a, tree_b
+
+    def test_clone_branch(self):
+        """Copy the stores from one branch to another"""
+        tree_a, tree_b = self.get_balanced_branch_pair()
+        tree_b.commit("silly commit")
+        os.mkdir('c')
+        # this fails to test that the history from a was not used.
+        dir_c = tree_a.bzrdir.clone('c', basis=tree_b.bzrdir)
+        self.assertEqual(tree_a.branch.revision_history(),
+                         dir_c.open_branch().revision_history())
+
+    def test_clone_partial(self):
+        """Copy only part of the history of a branch."""
+        # TODO: RBC 20060208 test with a revision not on revision-history.
+        #       what should that behaviour be ? Emailed the list.
+        wt_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/one'])
+        wt_a.add(['one'])
+        wt_a.commit('commit one', rev_id='1')
+        self.build_tree(['a/two'])
+        wt_a.add(['two'])
+        wt_a.commit('commit two', rev_id='2')
+        repo_b = self.make_repository('b')
+        wt_a.bzrdir.open_repository().copy_content_into(repo_b)
+        br_b = wt_a.bzrdir.open_branch().clone(repo_b.bzrdir, revision_id='1')
+        self.assertEqual(br_b.last_revision(), '1')
+
+    def test_sprout_partial(self):
+        # test sprouting with a prefix of the revision-history.
+        # also needs not-on-revision-history behaviour defined.
+        wt_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/one'])
+        wt_a.add(['one'])
+        wt_a.commit('commit one', rev_id='1')
+        self.build_tree(['a/two'])
+        wt_a.add(['two'])
+        wt_a.commit('commit two', rev_id='2')
+        repo_b = self.make_repository('b')
+        wt_a.bzrdir.open_repository().copy_content_into(repo_b)
+        br_b = wt_a.bzrdir.open_branch().sprout(repo_b.bzrdir, revision_id='1')
+        self.assertEqual(br_b.last_revision(), '1')
+
+    def test_clone_branch_nickname(self):
+        # test the nick name is preserved always
+        raise TestSkipped('XXX branch cloning is not yet tested..')
+
+    def test_clone_branch_parent(self):
+        # test the parent is preserved always
+        raise TestSkipped('XXX branch cloning is not yet tested..')
+        
+    def test_sprout_branch_nickname(self):
+        # test the nick name is reset always
+        raise TestSkipped('XXX branch sprouting is not yet tested..')
+
+    def test_sprout_branch_parent(self):
+        source = self.make_branch('source')
+        target = source.bzrdir.sprout(self.get_url('target')).open_branch()
+        self.assertEqual(source.bzrdir.root_transport.base, target.get_parent())
+        
+    def test_record_initial_ghost_merge(self):
+        """A pending merge with no revision present is still a merge."""
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        wt.add_pending_merge('non:existent@rev--ision--0--2')
+        wt.commit('pretend to merge nonexistent-revision', rev_id='first')
+        rev = branch.repository.get_revision(branch.last_revision())
+        self.assertEqual(len(rev.parent_ids), 1)
+        # parent_sha1s is not populated now, WTF. rbc 20051003
+        self.assertEqual(len(rev.parent_sha1s), 0)
+        self.assertEqual(rev.parent_ids[0], 'non:existent@rev--ision--0--2')
+
+    def test_bad_revision(self):
+        self.assertRaises(errors.InvalidRevisionId,
+                          self.get_branch().repository.get_revision,
+                          None)
+
+# TODO 20051003 RBC:
+# compare the gpg-to-sign info for a commit with a ghost and 
+#     an identical tree without a ghost
+# fetch missing should rewrite the TOC of weaves to list newly available parents.
+        
+    def test_pending_merges(self):
+        """Tracking pending-merged revisions."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.assertEquals(wt.pending_merges(), [])
+        wt.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(wt.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        wt.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(wt.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        wt.add_pending_merge('wibble@fofof--20050401--1928390812')
+        self.assertEquals(wt.pending_merges(),
+                          ['foo@azkhazan-123123-abcabc',
+                           'wibble@fofof--20050401--1928390812'])
+        wt.commit("commit from base with two merges")
+        rev = b.repository.get_revision(b.revision_history()[0])
+        self.assertEquals(len(rev.parent_ids), 2)
+        self.assertEquals(rev.parent_ids[0],
+                          'foo@azkhazan-123123-abcabc')
+        self.assertEquals(rev.parent_ids[1],
+                           'wibble@fofof--20050401--1928390812')
+        # list should be cleared when we do a commit
+        self.assertEquals(wt.pending_merges(), [])
+
+    def test_sign_existing_revision(self):
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        from bzrlib.testament import Testament
+        strategy = bzrlib.gpg.LoopbackGPGStrategy(None)
+        branch.repository.sign_revision('A', strategy)
+        self.assertEqual(Testament.from_revision(branch.repository, 
+                         'A').as_short_text(),
+                         branch.repository.get_signature_text('A'))
+
+    def test_store_signature(self):
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        branch.repository.store_revision_signature(
+            bzrlib.gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+        self.assertRaises(errors.NoSuchRevision,
+                          branch.repository.has_signature_for_revision_id,
+                          'A')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        self.assertEqual('FOO', 
+                         branch.repository.get_signature_text('A'))
+
+    def test_branch_keeps_signatures(self):
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        wt.branch.repository.sign_revision('A',
+            bzrlib.gpg.LoopbackGPGStrategy(None))
+        #FIXME: clone should work to urls,
+        # wt.clone should work to disks.
+        self.build_tree(['target/'])
+        d2 = wt.bzrdir.clone('target')
+        self.assertEqual(wt.branch.repository.get_signature_text('A'),
+                         d2.open_repository().get_signature_text('A'))
+
+    def test_nicks(self):
+        """Branch nicknames"""
+        t = get_transport(self.get_url())
+        t.mkdir('bzr.dev')
+        branch = self.make_branch('bzr.dev')
+        self.assertEqual(branch.nick, 'bzr.dev')
+        t.move('bzr.dev', 'bzr.ab')
+        branch = Branch.open(self.get_url('bzr.ab'))
+        self.assertEqual(branch.nick, 'bzr.ab')
+        branch.nick = "Aaron's branch"
+        branch.nick = "Aaron's branch"
+        self.failUnless(
+            t.has(
+                t.relpath(
+                    branch.control_files.controlfilename("branch.conf")
+                    )
+                )
+            )
+        self.assertEqual(branch.nick, "Aaron's branch")
+        t.move('bzr.ab', 'integration')
+        branch = Branch.open(self.get_url('integration'))
+        self.assertEqual(branch.nick, "Aaron's branch")
+        branch.nick = u"\u1234"
+        self.assertEqual(branch.nick, u"\u1234")
+
+    def test_commit_nicks(self):
+        """Nicknames are committed to the revision"""
+        get_transport(self.get_url()).mkdir('bzr.dev')
+        wt = self.make_branch_and_tree('bzr.dev')
+        branch = wt.branch
+        branch.nick = "My happy branch"
+        wt.commit('My commit respect da nick.')
+        committed = branch.repository.get_revision(branch.last_revision())
+        self.assertEqual(committed.properties["branch-nick"], 
+                         "My happy branch")
+
+    def test_create_open_branch_uses_repository(self):
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        repo.bzrdir.root_transport.mkdir('child')
+        child_dir = self.bzrdir_format.initialize('child')
+        try:
+            child_branch = self.branch_format.initialize(child_dir)
+        except errors.UninitializableFormat:
+            # branch references are not default init'able.
+            return
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         child_branch.repository.bzrdir.root_transport.base)
+        child_branch = bzrlib.branch.Branch.open(self.get_url('child'))
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         child_branch.repository.bzrdir.root_transport.base)
+
+    def test_format_description(self):
+        tree = self.make_branch_and_tree('tree')
+        text = tree.branch._format.get_format_description()
+        self.failUnless(len(text))
+
+
+class ChrootedTests(TestCaseWithBranch):
+    """A support class that provides readonly urls outside the local namespace.
+
+    This is done by checking if self.transport_server is a MemoryServer. if it
+    is then we are chrooted already, if it is not then an HttpServer is used
+    for readonly urls.
+    """
+
+    def setUp(self):
+        super(ChrootedTests, self).setUp()
+        if not self.transport_server == MemoryServer:
+            self.transport_readonly_server = HttpServer
+
+    def test_open_containing(self):
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_readonly_url(''))
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_readonly_url('g/p/q'))
+        branch = self.make_branch('.')
+        branch, relpath = Branch.open_containing(self.get_readonly_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = Branch.open_containing(self.get_readonly_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
+        
+# TODO: rewrite this as a regular unittest, without relying on the displayed output        
+#         >>> from bzrlib.commit import commit
+#         >>> bzrlib.trace.silent = True
+#         >>> br1 = ScratchBranch(files=['foo', 'bar'])
+#         >>> br1.working_tree().add('foo')
+#         >>> br1.working_tree().add('bar')
+#         >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
+#         >>> br2 = ScratchBranch()
+#         >>> br2.update_revisions(br1)
+#         Added 2 texts.
+#         Added 1 inventories.
+#         Added 1 revisions.
+#         >>> br2.revision_history()
+#         [u'REVISION-ID-1']
+#         >>> br2.update_revisions(br1)
+#         Added 0 revisions.
+#         >>> br1.text_store.total_size() == br2.text_store.total_size()
+#         True
+
+class InstrumentedTransaction(object):
+
+    def finish(self):
+        self.calls.append('finish')
+
+    def __init__(self):
+        self.calls = []
+
+
+class TestDecorator(object):
+
+    def __init__(self):
+        self._calls = []
+
+    def lock_read(self):
+        self._calls.append('lr')
+
+    def lock_write(self):
+        self._calls.append('lw')
+
+    def unlock(self):
+        self._calls.append('ul')
+
+    @needs_read_lock
+    def do_with_read(self):
+        return 1
+
+    @needs_read_lock
+    def except_with_read(self):
+        raise RuntimeError
+
+    @needs_write_lock
+    def do_with_write(self):
+        return 2
+
+    @needs_write_lock
+    def except_with_write(self):
+        raise RuntimeError
+
+
+class TestDecorators(TestCase):
+
+    def test_needs_read_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(1, branch.do_with_read())
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_excepts_in_read_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_read)
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_needs_write_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(2, branch.do_with_write())
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+    def test_excepts_in_write_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_write)
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+
+class TestBranchTransaction(TestCaseWithBranch):
+
+    def setUp(self):
+        super(TestBranchTransaction, self).setUp()
+        self.branch = None
+        
+    def test_default_get_transaction(self):
+        """branch.get_transaction on a new branch should give a PassThrough."""
+        self.failUnless(isinstance(self.get_branch().get_transaction(),
+                                   transactions.PassThroughTransaction))
+
+    def test__set_new_transaction(self):
+        self.get_branch()._set_transaction(transactions.ReadOnlyTransaction())
+
+    def test__set_over_existing_transaction_raises(self):
+        self.get_branch()._set_transaction(transactions.ReadOnlyTransaction())
+        self.assertRaises(errors.LockError,
+                          self.get_branch()._set_transaction,
+                          transactions.ReadOnlyTransaction())
+
+    def test_finish_no_transaction_raises(self):
+        self.assertRaises(errors.LockError, self.get_branch()._finish_transaction)
+
+    def test_finish_readonly_transaction_works(self):
+        self.get_branch()._set_transaction(transactions.ReadOnlyTransaction())
+        self.get_branch()._finish_transaction()
+        self.assertEqual(None, self.get_branch().control_files._transaction)
+
+    def test_unlock_calls_finish(self):
+        self.get_branch().lock_read()
+        transaction = InstrumentedTransaction()
+        self.get_branch().control_files._transaction = transaction
+        self.get_branch().unlock()
+        self.assertEqual(['finish'], transaction.calls)
+
+    def test_lock_read_acquires_ro_transaction(self):
+        self.get_branch().lock_read()
+        self.failUnless(isinstance(self.get_branch().get_transaction(),
+                                   transactions.ReadOnlyTransaction))
+        self.get_branch().unlock()
+        
+    def test_lock_write_acquires_write_transaction(self):
+        self.get_branch().lock_write()
+        # cannot use get_transaction as its magic
+        self.failUnless(isinstance(self.get_branch().control_files._transaction,
+                                   transactions.WriteTransaction))
+        self.get_branch().unlock()
+
+
+class TestBranchPushLocations(TestCaseWithBranch):
+
+    def test_get_push_location_unset(self):
+        self.assertEqual(None, self.get_branch().get_push_location())
+
+    def test_get_push_location_exact(self):
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
+        print >> open(fn, 'wt'), ("[%s]\n"
+                                  "push_location=foo" %
+                                  self.get_branch().base[:-1])
+        self.assertEqual("foo", self.get_branch().get_push_location())
+
+    def test_set_push_location(self):
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
+        self.get_branch().set_push_location('foo')
+        self.assertFileEqual("[%s]\n"
+                             "push_location = foo" % self.get_branch().base[:-1],
+                             fn)
+
+    # TODO RBC 20051029 test getting a push location from a branch in a 
+    # recursive section - that is, it appends the branch name.
+
+
+class TestFormat(TestCaseWithBranch):
+    """Tests for the format itself."""
+
+    def test_format_initialize_find_open(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.branch_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        t = get_transport(self.get_url())
+        readonly_t = get_transport(self.get_readonly_url())
+        made_branch = self.make_branch('.')
+        self.failUnless(isinstance(made_branch, bzrlib.branch.Branch))
+
+        # find it via bzrdir opening:
+        opened_control = bzrdir.BzrDir.open(readonly_t.base)
+        direct_opened_branch = opened_control.open_branch()
+        self.assertEqual(direct_opened_branch.__class__, made_branch.__class__)
+        self.assertEqual(opened_control, direct_opened_branch.bzrdir)
+        self.failUnless(isinstance(direct_opened_branch._format,
+                        self.branch_format.__class__))
+
+        # find it via Branch.open
+        opened_branch = bzrlib.branch.Branch.open(readonly_t.base)
+        self.failUnless(isinstance(opened_branch, made_branch.__class__))
+        self.assertEqual(made_branch._format.__class__,
+                         opened_branch._format.__class__)
+        # if it has a unique id string, can we probe for it ?
+        try:
+            self.branch_format.get_format_string()
+        except NotImplementedError:
+            return
+        self.assertEqual(self.branch_format,
+                         bzrlib.branch.BranchFormat.find_format(opened_control))

=== added file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2006-04-02 02:35:12 +0000
@@ -0,0 +1,42 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.osutils import abspath, realpath
+from bzrlib.tests import TestCaseWithTransport
+
+
+"""Tests for Branch parent URL"""
+
+
+class TestParent(TestCaseWithTransport):
+
+    def test_no_default_parent(self):
+        """Branches should have no parent by default"""
+        b = self.make_branch('.')
+        self.assertEquals(b.get_parent(), None)
+        
+    def test_set_get_parent(self):
+        """Set, re-get and reset the parent"""
+        b = self.make_branch('.')
+        url = 'http://bazaar-ng.org/bzr/bzr.dev'
+        b.set_parent(url)
+        self.assertEquals(b.get_parent(), url)
+        b.set_parent(None)
+        self.assertEquals(b.get_parent(), None)

=== added file 'bzrlib/tests/branch_implementations/test_permissions.py'
--- bzrlib/tests/branch_implementations/test_permissions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_permissions.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,112 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for bzr setting permissions.
+
+Files in the branch control directory (.bzr or .bzr/branch) should inherit
+the .bzr directory permissions.
+So if the directory is group writable, the files and subdirs should be as well.
+"""
+
+# TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+#                    defined by the local umask. This isn't terrible, is it
+#                    the truly desired behavior?
+ 
+import os
+import sys
+import stat
+from StringIO import StringIO
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.lockable_files import LockableFiles
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.transport import get_transport
+from bzrlib.workingtree import WorkingTree
+
+# TODO RBC consolidate the helper methods here and in tests/test_permissions.py
+
+def chmod_r(base, file_mode, dir_mode):
+    """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            os.chmod(p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            os.chmod(p, file_mode)
+
+
+def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
+    """Check that all permissions match
+
+    :param test: The TestCase being run
+    :param base: The path to the root directory to check
+    :param file_mode: The mode for all files
+    :param dir_mode: The mode for all directories
+    :param include_base: If false, only check the subdirectories
+    """
+    assert os.path.isdir(base)
+    t = get_transport(".")
+    if include_base:
+        test.assertTransportMode(t, base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            test.assertTransportMode(t, p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            test.assertTransportMode(t, p, file_mode)
+
+
+class TestPermissions(TestCaseWithTransport):
+
+    def test_new_branch(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        # also, these are BzrBranch format specific things..
+        os.mkdir('a')
+        mode = stat.S_IMODE(os.stat('a').st_mode)
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        self.assertEqualMode(mode, b.control_files._dir_mode)
+        self.assertEqualMode(mode & ~07111, b.control_files._file_mode)
+
+        os.mkdir('b')
+        os.chmod('b', 02777)
+        b = self.make_branch('b')
+        self.assertEqualMode(02777, b.control_files._dir_mode)
+        self.assertEqualMode(00666, b.control_files._file_mode)
+        check_mode_r(self, b.control_files._transport.base, 00666, 02777)
+
+        os.mkdir('c')
+        os.chmod('c', 02750)
+        b = self.make_branch('c')
+        self.assertEqualMode(02750, b.control_files._dir_mode)
+        self.assertEqualMode(00640, b.control_files._file_mode)
+        check_mode_r(self, b.control_files._transport.base, 00640, 02750)
+
+        os.mkdir('d')
+        os.chmod('d', 0700)
+        b = self.make_branch('d')
+        self.assertEqualMode(0700, b.control_files._dir_mode)
+        self.assertEqualMode(0600, b.control_files._file_mode)
+        check_mode_r(self, b.control_files._transport.base, 00600, 00700)

=== added file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2006-04-12 04:57:34 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branch.pull behaviour."""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.osutils import abspath, realpath
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestPull(TestCaseWithTransport):
+
+    def test_pull_convergence_simple(self):
+        # when revisions are pulled, the left-most accessible parents must 
+        # become the revision-history.
+        parent = self.make_branch_and_tree('parent')
+        parent.commit('1st post', rev_id='P1', allow_pointless=True)
+        mine = parent.bzrdir.sprout('mine').open_workingtree()
+        mine.commit('my change', rev_id='M1', allow_pointless=True)
+        self.merge(mine.branch, parent)
+        parent.commit('merge my change', rev_id='P2')
+        mine.pull(parent.branch)
+        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+
+    def test_pull_merged_indirect(self):
+        # it should be possible to do a pull from one branch into another
+        # when the tip of the target was merged into the source branch
+        # via a third branch - so its buried in the ancestry and is not
+        # directly accessible.
+        parent = self.make_branch_and_tree('parent')
+        parent.commit('1st post', rev_id='P1', allow_pointless=True)
+        mine = parent.bzrdir.sprout('mine').open_workingtree()
+        mine.commit('my change', rev_id='M1', allow_pointless=True)
+        other = parent.bzrdir.sprout('other').open_workingtree()
+        self.merge(mine.branch, other)
+        other.commit('merge my change', rev_id='O2')
+        self.merge(other.branch, parent)
+        parent.commit('merge other', rev_id='P2')
+        mine.pull(parent.branch)
+        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())

=== added file 'bzrlib/tests/branch_implementations/test_update.py'
--- bzrlib/tests/branch_implementations/test_update.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_update.py	2006-03-05 02:20:07 +0000
@@ -0,0 +1,65 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import bzrlib.errors as errors
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+
+
+"""Tests for branch.update()"""
+
+
+class TestUpdate(TestCaseWithBranch):
+
+    def test_update_unbound_works(self):
+        b = self.make_branch('.')
+        b.update()
+        self.assertEqual(None, b.last_revision())
+
+    def test_update_prefix_returns_none(self):
+        # update in a branch when its a prefix of the master should
+        # indicate that no local changes were present.
+        master_tree = self.make_branch_and_tree('master')
+        child_tree = self.make_branch_and_tree('child')
+        try:
+            child_tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            return
+        # commit to the child to make the last rev not-None.
+        child_tree.commit('foo', rev_id='foo', allow_pointless=True)
+        # update the master so we can commit there.
+        master_tree.update()
+        # commit to the master making the child tree out of date and a prefix.
+        master_tree.commit('bar', rev_id='bar', allow_pointless=True)
+        self.assertEqual(None, child_tree.branch.update())
+
+    def test_update_local_commits_returns_old_tip(self):
+        # update in a branch when its not a prefix of the master should
+        # return the previous tip and reset the revision history.
+        master_tree = self.make_branch_and_tree('master')
+        child_tree = self.make_branch_and_tree('child')
+        try:
+            child_tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            return
+        # commit to the child to make the last rev not-None and skew it from master.
+        child_tree.commit('foo', rev_id='foo', allow_pointless=True, local=True)
+        # commit to the master making the child tree out of date and not a prefix.
+        master_tree.commit('bar', rev_id='bar', allow_pointless=True)
+        self.assertEqual('foo', child_tree.branch.update())
+        self.assertEqual(['bar'], child_tree.branch.revision_history())

=== added directory 'bzrlib/tests/bzrdir_implementations'
=== added file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2006-01-31 07:08:09 +0000
@@ -0,0 +1,49 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the bzrdir variations to the expected API.
+Specific tests for individual formats are in the tests/test_bzrdir.py file 
+rather than in tests/branch_implementations/*.py.
+"""
+
+from bzrlib.bzrdir import BzrDirTestProviderAdapter, BzrDirFormat
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_bzrdir_implementations = [
+        'bzrlib.tests.bzrdir_implementations.test_bzrdir',
+        ]
+    adapter = BzrDirTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        BzrDirFormat._formats.values())
+    loader = TestLoader()
+    adapt_modules(test_bzrdir_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,1103 @@
+# (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import os
+from stat import *
+import sys
+
+import bzrlib.branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.check import check
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.tests import (
+                          ChrootedTestCase,
+                          TestCase,
+                          TestCaseWithTransport,
+                          TestSkipped,
+                          )
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+import bzrlib.transport as transport
+from bzrlib.transport import get_transport
+import bzrlib.ui as ui
+from bzrlib.upgrade import upgrade
+import bzrlib.workingtree as workingtree
+
+
+class TestCaseWithBzrDir(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestCaseWithBzrDir, self).setUp()
+        self.bzrdir = None
+
+    def get_bzrdir(self):
+        if self.bzrdir is None:
+            self.bzrdir = self.make_bzrdir(None)
+        return self.bzrdir
+
+    def make_bzrdir(self, relpath, format=None):
+        return super(TestCaseWithBzrDir, self).make_bzrdir(
+            relpath, format=self.bzrdir_format)
+
+
+
+class TestBzrDir(TestCaseWithBzrDir):
+    # Many of these tests test for disk equality rather than checking
+    # for semantic equivalence. This works well for some tests but
+    # is not good at handling changes in representation or the addition
+    # or removal of control data. It would be nice to for instance:
+    # sprout a new branch, check that the nickname has been reset by hand
+    # and then set the nickname to match the source branch, at which point
+    # a semantic equivalence should pass
+
+    def assertDirectoriesEqual(self, source, target, ignore_list=[]):
+        """Assert that the content of source and target are identical.
+
+        paths in ignore list will be completely ignored.
+        """
+        files = []
+        directories = ['.']
+        while directories:
+            dir = directories.pop()
+            for path in source.list_dir(dir):
+                path = dir + '/' + path
+                if path in ignore_list:
+                    continue
+                stat = source.stat(path)
+                if S_ISDIR(stat.st_mode):
+                    self.assertTrue(S_ISDIR(target.stat(path).st_mode))
+                    directories.append(path)
+                else:
+                    self.assertEqualDiff(source.get(path).read(),
+                                         target.get(path).read(),
+                                         "text for file %r differs:\n" % path)
+
+    def test_clone_bzrdir_empty(self):
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+    
+    def test_clone_bzrdir_empty_force_new_ignored(self):
+        # the force_new_repo parameter should have no effect on an empty
+        # bzrdir's clone logic
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+    
+    def test_clone_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_clone_bzrdir_repository_under_shared(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.clone(self.get_url('target/child'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
+
+    def test_clone_bzrdir_repository_branch_both_under_shared(self):
+        try:
+            shared_repo = self.make_repository('shared', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        tree.bzrdir.open_repository().copy_content_into(shared_repo)
+        dir = self.make_bzrdir('shared/source')
+        dir.create_branch()
+        target = dir.clone(self.get_url('shared/target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
+        self.assertTrue(shared_repo.has_revision('1'))
+        
+    def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_clone_bzrdir_repository_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a repository with some revisions,
+        # and clone it with a revision limit.
+        # 
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        source = self.make_repository('source')
+        tree.bzrdir.open_repository().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'), revision_id='2')
+        raise TestSkipped('revision limiting not strict yet')
+
+    def test_clone_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+
+    def test_clone_bzrdir_branch_and_repo_into_shared_repo(self):
+        # by default cloning into a shared repo uses the shared repo.
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target/child'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
+        self.assertEqual(source.revision_history(),
+                         target.open_branch().revision_history())
+
+    def test_clone_bzrdir_branch_and_repo_into_shared_repo_force_new_repo(self):
+        # by default cloning into a shared repo uses the shared repo.
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        target.open_repository()
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_clone_bzrdir_branch_reference(self):
+        # cloning should preserve the reference status of the branch in a bzrdir
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_clone_bzrdir_branch_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a branch with some revisions,
+        # and clone it with a revision limit.
+        # 
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'), revision_id='1')
+        self.assertEqual('1', target.open_branch().last_revision())
+        
+    def test_clone_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('sourcce')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+        target.open_workingtree().revert([])
+
+    def test_revert_inventory(self):
+        tree = self.make_branch_and_tree('sourcce')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+        target.open_workingtree().revert([])
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+
+    def test_clone_bzrdir_tree_branch_reference(self):
+        # a tree with a branch reference (aka a checkout) 
+        # should stay a checkout on clone.
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        dir.create_workingtree()
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+
+    def test_clone_bzrdir_tree_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a tree with a revision with a last-revision
+        # and clone it with a revision limit.
+        # This smoke test just checks the revision-id is right. Tree specific
+        # tests will check corner cases.
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'), revision_id='1')
+        self.assertEqual('1', target.open_workingtree().last_revision())
+
+    def test_clone_bzrdir_incomplete_source_with_basis(self):
+        # ensure that basis really does grab from the basis by having incomplete source
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch_and_tree('source')
+        # this gives us an incomplete repository
+        tree.bzrdir.open_repository().copy_content_into(source.branch.repository)
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        tree.bzrdir.open_branch().copy_content_into(source.branch)
+        tree.copy_content_into(source)
+        self.assertFalse(source.branch.repository.has_revision('2'))
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'), basis=tree.bzrdir)
+        self.assertEqual('2', target.open_branch().last_revision())
+        self.assertEqual('2', target.open_workingtree().last_revision())
+        self.assertTrue(target.open_branch().repository.has_revision('2'))
+
+    def test_sprout_bzrdir_empty(self):
+        dir = self.make_bzrdir('source')
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # creates a new repository branch and tree
+        target.open_repository()
+        target.open_branch()
+        target.open_workingtree()
+
+    def test_sprout_bzrdir_empty_under_shared_repo(self):
+        # sprouting an empty dir into a repo uses the repo
+        dir = self.make_bzrdir('source')
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'))
+        self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
+        target.open_branch()
+        target.open_workingtree()
+
+    def test_sprout_bzrdir_empty_under_shared_repo(self):
+        # the force_new_repo parameter should force use of a new repo in an empty
+        # bzrdir's sprout logic
+        dir = self.make_bzrdir('source')
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        target.open_repository()
+        target.open_branch()
+        target.open_workingtree()
+    
+    def test_sprout_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_sprout_bzrdir_with_repository_to_shared(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        source = self.make_repository('source')
+        tree.bzrdir.open_repository().copy_content_into(source)
+        dir = source.bzrdir
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertTrue(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_repository_branch_both_under_shared(self):
+        try:
+            shared_repo = self.make_repository('shared', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        tree.bzrdir.open_repository().copy_content_into(shared_repo)
+        dir = self.make_bzrdir('shared/source')
+        dir.create_branch()
+        target = dir.sprout(self.get_url('shared/target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
+        self.assertTrue(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_repository_under_shared_force_new_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        source = self.make_repository('source')
+        tree.bzrdir.open_repository().copy_content_into(source)
+        dir = source.bzrdir
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertFalse(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_repository_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a repository with some revisions,
+        # and sprout it with a revision limit.
+        # 
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.bzrdir.open_branch().set_revision_history([])
+        tree.set_last_revision(None)
+        tree.commit('revision 2', rev_id='2')
+        source = self.make_repository('source')
+        tree.bzrdir.open_repository().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'), revision_id='2')
+        raise TestSkipped('revision limiting not strict yet')
+
+    def test_sprout_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/checkout/inventory',
+                                     ])
+
+    def test_sprout_bzrdir_branch_and_repo_shared(self):
+        # sprouting a branch with a repo into a shared repo uses the shared
+        # repo
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'))
+        self.assertTrue(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_branch_and_repo_shared_force_new_repo(self):
+        # sprouting a branch with a repo into a shared repo uses the shared
+        # repo
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertFalse(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_branch_reference(self):
+        # sprouting should create a repository if needed and a sprouted branch.
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # we want target to have a branch that is in-place.
+        self.assertEqual(target, target.open_branch().bzrdir)
+        # and as we dont support repositories being detached yet, a repo in 
+        # place
+        target.open_repository()
+
+    def test_sprout_bzrdir_branch_reference_shared(self):
+        # sprouting should create a repository if needed and a sprouted branch.
+        referenced_tree = self.make_branch_and_tree('referenced')
+        referenced_tree.commit('1', rev_id='1', allow_pointless=True)
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_tree.branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # we want target to have a branch that is in-place.
+        self.assertEqual(target, target.open_branch().bzrdir)
+        # and we want no repository as the target is shared
+        self.assertRaises(errors.NoRepositoryPresent, 
+                          target.open_repository)
+        # and we want revision '1' in the shared repo
+        self.assertTrue(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_branch_reference_shared_force_new_repo(self):
+        # sprouting should create a repository if needed and a sprouted branch.
+        referenced_tree = self.make_branch_and_tree('referenced')
+        referenced_tree.commit('1', rev_id='1', allow_pointless=True)
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_tree.branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
+        try:
+            shared_repo = self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # we want target to have a branch that is in-place.
+        self.assertEqual(target, target.open_branch().bzrdir)
+        # and we want revision '1' in the new repo
+        self.assertTrue(target.open_repository().has_revision('1'))
+        # but not the shared one
+        self.assertFalse(shared_repo.has_revision('1'))
+
+    def test_sprout_bzrdir_branch_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a repository with some revisions,
+        # and sprout it with a revision limit.
+        # 
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        source = self.make_branch('source')
+        tree.bzrdir.open_repository().copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'), revision_id='1')
+        self.assertEqual('1', target.open_branch().last_revision())
+        
+    def test_sprout_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('sourcce')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/checkout/inventory',
+                                     ])
+
+    def test_sprout_bzrdir_tree_branch_reference(self):
+        # sprouting should create a repository if needed and a sprouted branch.
+        # the tree state should not be copied.
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
+        tree = dir.create_workingtree()
+        tree.bzrdir.root_transport.mkdir('subdir')
+        tree.add('subdir')
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # we want target to have a branch that is in-place.
+        self.assertEqual(target, target.open_branch().bzrdir)
+        # and as we dont support repositories being detached yet, a repo in 
+        # place
+        target.open_repository()
+        result_tree = target.open_workingtree()
+        self.assertFalse(result_tree.has_filename('subdir'))
+
+    def test_sprout_bzrdir_tree_branch_reference_revision(self):
+        # sprouting should create a repository if needed and a sprouted branch.
+        # the tree state should not be copied but the revision changed,
+        # and the likewise the new branch should be truncated too
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
+        tree = dir.create_workingtree()
+        self.build_tree(['foo'], transport=dir.root_transport)
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        target = dir.sprout(self.get_url('target'), revision_id='1')
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # we want target to have a branch that is in-place.
+        self.assertEqual(target, target.open_branch().bzrdir)
+        # and as we dont support repositories being detached yet, a repo in 
+        # place
+        target.open_repository()
+        # we trust that the working tree sprouting works via the other tests.
+        self.assertEqual('1', target.open_workingtree().last_revision())
+        self.assertEqual('1', target.open_branch().last_revision())
+
+    def test_sprout_bzrdir_tree_revision(self):
+        # test for revision limiting, [smoke test, not corner case checks].
+        # make a tree with a revision with a last-revision
+        # and sprout it with a revision limit.
+        # This smoke test just checks the revision-id is right. Tree specific
+        # tests will check corner cases.
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['foo'], transport=tree.bzrdir.root_transport)
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        dir = tree.bzrdir
+        target = dir.sprout(self.get_url('target'), revision_id='1')
+        self.assertEqual('1', target.open_workingtree().last_revision())
+
+    def test_sprout_bzrdir_incomplete_source_with_basis(self):
+        # ensure that basis really does grab from the basis by having incomplete source
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.root_transport)
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch_and_tree('source')
+        # this gives us an incomplete repository
+        tree.bzrdir.open_repository().copy_content_into(source.branch.repository)
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        tree.bzrdir.open_branch().copy_content_into(source.branch)
+        tree.copy_content_into(source)
+        self.assertFalse(source.branch.repository.has_revision('2'))
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'), basis=tree.bzrdir)
+        self.assertEqual('2', target.open_branch().last_revision())
+        self.assertEqual('2', target.open_workingtree().last_revision())
+        self.assertTrue(target.open_branch().repository.has_revision('2'))
+
+    def test_format_initialize_find_open(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        t = get_transport(self.get_url())
+        readonly_t = get_transport(self.get_readonly_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        self.failUnless(isinstance(made_control, bzrdir.BzrDir))
+        self.assertEqual(self.bzrdir_format,
+                         bzrdir.BzrDirFormat.find_format(readonly_t))
+        direct_opened_dir = self.bzrdir_format.open(readonly_t)
+        opened_dir = bzrdir.BzrDir.open(t.base)
+        self.assertEqual(made_control._format,
+                         opened_dir._format)
+        self.assertEqual(direct_opened_dir._format,
+                         opened_dir._format)
+        self.failUnless(isinstance(opened_dir, bzrdir.BzrDir))
+
+    def test_open_not_bzrdir(self):
+        # test the formats specific behaviour for no-content or similar dirs.
+        self.assertRaises(NotBranchError,
+                          self.bzrdir_format.open,
+                          get_transport(self.get_readonly_url()))
+
+    def test_create_branch(self):
+        # a bzrdir can construct a branch and repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        made_branch = made_control.create_branch()
+        self.failUnless(isinstance(made_branch, bzrlib.branch.Branch))
+        self.assertEqual(made_control, made_branch.bzrdir)
+        
+    def test_open_branch(self):
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        made_branch = made_control.create_branch()
+        opened_branch = made_control.open_branch()
+        self.assertEqual(made_control, opened_branch.bzrdir)
+        self.failUnless(isinstance(opened_branch, made_branch.__class__))
+        self.failUnless(isinstance(opened_branch._format, made_branch._format.__class__))
+
+    def test_create_repository(self):
+        # a bzrdir can construct a repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+        
+    def test_open_repository(self):
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        opened_repo = made_control.open_repository()
+        self.assertEqual(made_control, opened_repo.bzrdir)
+        self.failUnless(isinstance(opened_repo, made_repo.__class__))
+        self.failUnless(isinstance(opened_repo._format, made_repo._format.__class__))
+
+    def test_create_workingtree(self):
+        # a bzrdir can construct a working tree for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # this has to be tested with local access as we still support creating 
+        # format 6 bzrdirs
+        t = get_transport('.')
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        made_branch = made_control.create_branch()
+        made_tree = made_control.create_workingtree()
+        self.failUnless(isinstance(made_tree, workingtree.WorkingTree))
+        self.assertEqual(made_control, made_tree.bzrdir)
+        
+    def test_create_workingtree_revision(self):
+        # a bzrdir can construct a working tree for itself @ a specific revision.
+        source = self.make_branch_and_tree('source')
+        source.commit('a', rev_id='a', allow_pointless=True)
+        source.commit('b', rev_id='b', allow_pointless=True)
+        self.build_tree(['new/'])
+        made_control = self.bzrdir_format.initialize('new')
+        source.branch.repository.clone(made_control)
+        source.branch.clone(made_control)
+        made_tree = made_control.create_workingtree(revision_id='a')
+        self.assertEqual('a', made_tree.last_revision())
+        
+    def test_open_workingtree(self):
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # this has to be tested with local access as we still support creating 
+        # format 6 bzrdirs
+        t = get_transport('.')
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        made_branch = made_control.create_branch()
+        made_tree = made_control.create_workingtree()
+        opened_tree = made_control.open_workingtree()
+        self.assertEqual(made_control, opened_tree.bzrdir)
+        self.failUnless(isinstance(opened_tree, made_tree.__class__))
+        self.failUnless(isinstance(opened_tree._format, made_tree._format.__class__))
+
+    def test_get_branch_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_branch_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_branch_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # branches, or it supports anonymous  branch formats, but not both.
+        anonymous_format = bzrlib.branch.BzrBranchFormat4()
+        identifiable_format = bzrlib.branch.BzrBranchFormat5()
+        try:
+            found_transport = dir.get_branch_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_branch_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_branch_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must be statable.
+        found_transport.stat('.')
+
+    def test_get_repository_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_repository_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_repository_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # repositoryes, or it supports anonymous  repository formats, but not both.
+        anonymous_format = repository.RepositoryFormat6()
+        identifiable_format = repository.RepositoryFormat7()
+        try:
+            found_transport = dir.get_repository_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_repository_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_repository_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must be statable.
+        found_transport.stat('.')
+
+    def test_get_workingtree_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_workingtree_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_workingtree_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # trees, or it supports anonymous tree formats, but not both.
+        anonymous_format = workingtree.WorkingTreeFormat2()
+        identifiable_format = workingtree.WorkingTreeFormat3()
+        try:
+            found_transport = dir.get_workingtree_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_workingtree_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_workingtree_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must be statable.
+        found_transport.stat('.')
+
+    def test_root_transport(self):
+        dir = self.make_bzrdir('.')
+        self.assertEqual(dir.root_transport.base,
+                         get_transport(self.get_url('.')).base)
+
+    def test_find_repository_no_repo_under_standalone_branch(self):
+        # finding a repo stops at standalone branches even if there is a
+        # higher repository available.
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            # need a shared repository to test this.
+            return
+        url = self.get_url('intermediate')
+        get_transport(self.get_url()).mkdir('intermediate')
+        get_transport(self.get_url()).mkdir('intermediate/child')
+        made_control = self.bzrdir_format.initialize(url)
+        made_control.create_repository()
+        innermost_control = self.bzrdir_format.initialize(
+            self.get_url('intermediate/child'))
+        try:
+            child_repo = innermost_control.open_repository()
+            # if there is a repository, then the format cannot ever hit this 
+            # code path.
+            return
+        except errors.NoRepositoryPresent:
+            pass
+        self.assertRaises(errors.NoRepositoryPresent,
+                          innermost_control.find_repository)
+
+    def test_find_repository_containing_shared_repository(self):
+        # find repo inside a shared repo with an empty control dir
+        # returns the shared repo.
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            # need a shared repository to test this.
+            return
+        url = self.get_url('childbzrdir')
+        get_transport(self.get_url()).mkdir('childbzrdir')
+        made_control = self.bzrdir_format.initialize(url)
+        try:
+            child_repo = made_control.open_repository()
+            # if there is a repository, then the format cannot ever hit this 
+            # code path.
+            return
+        except errors.NoRepositoryPresent:
+            pass
+        found_repo = made_control.find_repository()
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         found_repo.bzrdir.root_transport.base)
+        
+    def test_find_repository_standalone_with_containing_shared_repository(self):
+        # find repo inside a standalone repo inside a shared repo finds the standalone repo
+        try:
+            containing_repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            # need a shared repository to test this.
+            return
+        child_repo = self.make_repository('childrepo')
+        opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
+        found_repo = opened_control.find_repository()
+        self.assertEqual(child_repo.bzrdir.root_transport.base,
+                         found_repo.bzrdir.root_transport.base)
+
+    def test_find_repository_shared_within_shared_repository(self):
+        # find repo at a shared repo inside a shared repo finds the inner repo
+        try:
+            containing_repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            # need a shared repository to test this.
+            return
+        url = self.get_url('childrepo')
+        get_transport(self.get_url()).mkdir('childrepo')
+        child_control = self.bzrdir_format.initialize(url)
+        child_repo = child_control.create_repository(shared=True)
+        opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
+        found_repo = opened_control.find_repository()
+        self.assertEqual(child_repo.bzrdir.root_transport.base,
+                         found_repo.bzrdir.root_transport.base)
+        self.assertNotEqual(child_repo.bzrdir.root_transport.base,
+                            containing_repo.bzrdir.root_transport.base)
+
+    def test_find_repository_with_nested_dirs_works(self):
+        # find repo inside a bzrdir inside a bzrdir inside a shared repo 
+        # finds the outer shared repo.
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            # need a shared repository to test this.
+            return
+        url = self.get_url('intermediate')
+        get_transport(self.get_url()).mkdir('intermediate')
+        get_transport(self.get_url()).mkdir('intermediate/child')
+        made_control = self.bzrdir_format.initialize(url)
+        try:
+            child_repo = made_control.open_repository()
+            # if there is a repository, then the format cannot ever hit this 
+            # code path.
+            return
+        except errors.NoRepositoryPresent:
+            pass
+        innermost_control = self.bzrdir_format.initialize(
+            self.get_url('intermediate/child'))
+        try:
+            child_repo = innermost_control.open_repository()
+            # if there is a repository, then the format cannot ever hit this 
+            # code path.
+            return
+        except errors.NoRepositoryPresent:
+            pass
+        found_repo = innermost_control.find_repository()
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         found_repo.bzrdir.root_transport.base)
+        
+    def test_can_and_needs_format_conversion(self):
+        # check that we can ask an instance if its upgradable
+        dir = self.make_bzrdir('.')
+        if dir.can_convert_format():
+            # if its default updatable there must be an updater 
+            # (we change the default to match the lastest known format
+            # as downgrades may not be available
+            old_format = bzrdir.BzrDirFormat.get_default_format()
+            bzrdir.BzrDirFormat.set_default_format(dir._format)
+            try:
+                self.assertTrue(isinstance(dir._format.get_converter(),
+                                           bzrdir.Converter))
+            finally:
+                bzrdir.BzrDirFormat.set_default_format(old_format)
+        dir.needs_format_conversion(None)
+
+    def test_upgrade_new_instance(self):
+        """Does an available updater work ?."""
+        dir = self.make_bzrdir('.')
+        # for now, check is not ready for partial bzrdirs.
+        dir.create_repository()
+        dir.create_branch()
+        dir.create_workingtree()
+        if dir.can_convert_format():
+            # if its default updatable there must be an updater 
+            # (we change the default to match the lastest known format
+            # as downgrades may not be available
+            old_format = bzrdir.BzrDirFormat.get_default_format()
+            bzrdir.BzrDirFormat.set_default_format(dir._format)
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                dir._format.get_converter(None).convert(dir, pb)
+            finally:
+                bzrdir.BzrDirFormat.set_default_format(old_format)
+                pb.finished()
+            # and it should pass 'check' now.
+            check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)
+
+    def test_format_description(self):
+        dir = self.make_bzrdir('.')
+        text = dir._format.get_format_description()
+        self.failUnless(len(text))
+
+
+class ChrootedBzrDirTests(ChrootedTestCase):
+
+    def test_find_repository_no_repository(self):
+        # loopback test to check the current format fails to find a 
+        # share repository correctly.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        url = self.get_url('subdir')
+        get_transport(self.get_url()).mkdir('subdir')
+        made_control = self.bzrdir_format.initialize(url)
+        try:
+            repo = made_control.open_repository()
+            # if there is a repository, then the format cannot ever hit this 
+            # code path.
+            return
+        except errors.NoRepositoryPresent:
+            pass
+        opened_control = bzrdir.BzrDir.open(self.get_readonly_url('subdir'))
+        self.assertRaises(errors.NoRepositoryPresent,
+                          opened_control.find_repository)
+

=== added directory 'bzrlib/tests/interrepository_implementations'
=== added file 'bzrlib/tests/interrepository_implementations/__init__.py'
--- bzrlib/tests/interrepository_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interrepository_implementations/__init__.py	2006-02-20 07:01:10 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""InterRepository implementation tests for bzr.
+
+These test the conformance of all the interrepository variations to the
+expected API including generally applicable corner cases.
+Specific tests for individual formats are in the tests/test_repository.py file 
+rather than in tests/interrepository_implementations/*.py.
+"""
+
+from bzrlib.repository import (_legacy_formats,
+                               InterRepositoryTestProviderAdapter,
+                               )
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_interrepository_implementations = [
+        'bzrlib.tests.interrepository_implementations.test_interrepository',
+        ]
+    adapter = InterRepositoryTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        InterRepositoryTestProviderAdapter.default_test_list()
+        )
+    loader = TestLoader()
+    adapt_modules(test_interrepository_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,214 @@
+# (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for InterRepository implementastions."""
+
+
+import bzrlib
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.transport import get_transport
+
+
+class TestCaseWithInterRepository(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithInterRepository, self).setUp()
+
+    def make_branch(self, relpath, format=None):
+        repo = self.make_repository(relpath, format=format)
+        return repo.bzrdir.create_branch()
+
+    def make_bzrdir(self, relpath, format=None):
+        try:
+            url = self.get_url(relpath)
+            segments = url.split('/')
+            if segments and segments[-1] not in ('', '.'):
+                parent = '/'.join(segments[:-1])
+                t = get_transport(parent)
+                try:
+                    t.mkdir(segments[-1])
+                except FileExists:
+                    pass
+            if format is None:
+                format = self.repository_format._matchingbzrdir
+            return format.initialize(url)
+        except UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable.")
+
+    def make_repository(self, relpath, format=None):
+        made_control = self.make_bzrdir(relpath, format=format)
+        return self.repository_format.initialize(made_control)
+
+    def make_to_repository(self, relpath):
+        made_control = self.make_bzrdir(relpath,
+            self.repository_format_to._matchingbzrdir)
+        return self.repository_format_to.initialize(made_control)
+
+
+class TestInterRepository(TestCaseWithInterRepository):
+
+    def test_interrepository_get_returns_correct_optimiser(self):
+        # we assume the optimising code paths are triggered
+        # by the type of the repo not the transport - at this point.
+        # we may need to update this test if this changes.
+        source_repo = self.make_repository("source")
+        target_repo = self.make_to_repository("target")
+        interrepo = repository.InterRepository.get(source_repo, target_repo)
+        self.assertEqual(self.interrepo_class, interrepo.__class__)
+
+    def test_fetch(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        def check_push_rev1(repo):
+            # ensure the revision is missing.
+            self.assertRaises(NoSuchRevision, repo.get_revision, 'rev1')
+            # fetch with a limit of NULL_REVISION and an explicit progress bar.
+            repo.fetch(tree_a.branch.repository,
+                       revision_id=NULL_REVISION,
+                       pb=bzrlib.progress.DummyProgress())
+            # nothing should have been pushed
+            self.assertFalse(repo.has_revision('rev1'))
+            # fetch with a default limit (grab everything)
+            repo.fetch(tree_a.branch.repository)
+            # check that b now has all the data from a's first commit.
+            rev = repo.get_revision('rev1')
+            tree = repo.revision_tree('rev1')
+            tree.get_file_text('file1')
+            for file_id in tree:
+                if tree.inventory[file_id].kind == "file":
+                    tree.get_file(file_id).read()
+
+        # makes a target version repo 
+        repo_b = self.make_to_repository('b')
+        check_push_rev1(repo_b)
+        
+    def test_fetch_missing_revision_same_location_fails(self):
+        repo_a = self.make_repository('.')
+        repo_b = repository.Repository.open('.')
+        self.assertRaises(errors.NoSuchRevision, repo_b.fetch, repo_a, revision_id='XXX')
+
+    def test_fetch_same_location_trivial_works(self):
+        repo_a = self.make_repository('.')
+        repo_b = repository.Repository.open('.')
+        repo_a.fetch(repo_b)
+
+
+class TestCaseWithComplexRepository(TestCaseWithInterRepository):
+
+    def setUp(self):
+        super(TestCaseWithComplexRepository, self).setUp()
+        tree_a = self.make_branch_and_tree('a')
+        self.bzrdir = tree_a.branch.bzrdir
+        # add a corrupt inventory 'orphan'
+        inv_file = tree_a.branch.repository.control_weaves.get_weave(
+            'inventory', 
+            tree_a.branch.repository.get_transaction())
+        inv_file.add_lines('orphan', [], [])
+        # add a real revision 'rev1'
+        tree_a.commit('rev1', rev_id='rev1', allow_pointless=True)
+        # add a real revision 'rev2' based on rev1
+        tree_a.commit('rev2', rev_id='rev2', allow_pointless=True)
+        # and sign 'rev2'
+        tree_a.branch.repository.sign_revision('rev2',
+            bzrlib.gpg.LoopbackGPGStrategy(None))
+
+    def test_missing_revision_ids(self):
+        # revision ids in repository A but not B are returned, fake ones
+        # are stripped. (fake meaning no revision object, but an inventory 
+        # as some formats keyed off inventory data in the past.
+        # make a repository to compare against that claims to have rev1
+        repo_b = self.make_to_repository('rev1_only')
+        repo_a = self.bzrdir.open_repository()
+        repo_b.fetch(repo_a, 'rev1')
+        # check the test will be valid
+        self.assertFalse(repo_b.has_revision('rev2'))
+        self.assertEqual(['rev2'],
+                         repo_b.missing_revision_ids(repo_a))
+
+    def test_missing_revision_ids_revision_limited(self):
+        # revision ids in repository A that are not referenced by the
+        # requested revision are not returned.
+        # make a repository to compare against that is empty
+        repo_b = self.make_to_repository('empty')
+        repo_a = self.bzrdir.open_repository()
+        self.assertEqual(['rev1'],
+                         repo_b.missing_revision_ids(repo_a, revision_id='rev1'))
+        
+    def test_fetch_preserves_signatures(self):
+        from_repo = self.bzrdir.open_repository()
+        from_signature = from_repo.get_signature_text('rev2')
+        to_repo = self.make_to_repository('target')
+        to_repo.fetch(from_repo)
+        to_signature = to_repo.get_signature_text('rev2')
+        self.assertEqual(from_signature, to_signature)
+
+
+class TestCaseWithGhosts(TestCaseWithInterRepository):
+
+    def setUp(self):
+        super(TestCaseWithGhosts, self).setUp()
+        # we want two repositories at this point
+        # one with a revision that is a ghost in the other
+        # repository.
+
+        # 'ghost' is a ghost in missing_ghost and not in with_ghost_rev
+        inv = bzrlib.tree.EmptyTree().inventory
+        repo = self.make_repository('with_ghost_rev')
+        sha1 = repo.add_inventory('ghost', inv, [])
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       inventory_sha1=sha1,
+                                       revision_id='ghost')
+        rev.parent_ids = []
+        repo.add_revision('ghost', rev)
+         
+        repo = self.make_to_repository('missing_ghost')
+        sha1 = repo.add_inventory('with_ghost', inv, [])
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       inventory_sha1=sha1,
+                                       revision_id='with_ghost')
+        rev.parent_ids = ['ghost']
+        repo.add_revision('with_ghost', rev)
+
+    def test_fetch_all_fixes_up_ghost(self):
+        # fetching from a repo with a current ghost unghosts it in referencing
+        # revisions.
+        repo = repository.Repository.open('missing_ghost')
+        rev = repo.get_revision('with_ghost')
+        from_repo = repository.Repository.open('with_ghost_rev')
+        repo.fetch(from_repo)
+        # rev must not be corrupt now
+        rev = repo.get_revision('with_ghost')
+        self.assertEqual([None, 'ghost', 'with_ghost'], repo.get_ancestry('with_ghost'))

=== added directory 'bzrlib/tests/interversionedfile_implementations'
=== added file 'bzrlib/tests/interversionedfile_implementations/__init__.py'
--- bzrlib/tests/interversionedfile_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interversionedfile_implementations/__init__.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""InterVersioned implementation tests for bzr.
+
+These test the conformance of all the interversionedfile variations to the
+expected API including generally applicable corner cases.
+Specific tests for individual cases are in the tests/test_versionedfile.py file 
+rather than in tests/interversionedfile_implementations/*.py.
+"""
+
+from bzrlib.versionedfile import InterVersionedFileTestProviderAdapter
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_interversionedfile_implementations = [
+        'bzrlib.tests.interversionedfile_implementations.test_join',
+        ]
+    adapter = InterVersionedFileTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        InterVersionedFileTestProviderAdapter.default_test_list()
+        )
+    loader = TestLoader()
+    adapt_modules(test_interversionedfile_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/interversionedfile_implementations/test_join.py'
--- bzrlib/tests/interversionedfile_implementations/test_join.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interversionedfile_implementations/test_join.py	2006-03-10 02:04:26 +0000
@@ -0,0 +1,308 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for join between versioned files."""
+
+
+import bzrlib.errors as errors
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.transport import get_transport
+import bzrlib.versionedfile as versionedfile
+
+
+class TestJoin(TestCaseWithTransport):
+    #Tests have self.versionedfile_factory and self.versionedfile_factory_to
+    #available to create source and target versioned files respectively.
+
+    def get_source(self, name='source'):
+        """Get a versioned file we will be joining from."""
+        return self.versionedfile_factory(name,
+                                          get_transport(self.get_url()),
+                                          create=True)
+
+    def get_target(self, name='target'):
+        """"Get an empty versioned file to join into."""
+        return self.versionedfile_factory_to(name,
+                                             get_transport(self.get_url()),
+                                             create=True)
+
+    def test_join(self):
+        f1 = self.get_source()
+        f1.add_lines('r0', [], ['a\n', 'b\n'])
+        f1.add_lines('r1', ['r0'], ['c\n', 'b\n'])
+        f2 = self.get_target()
+        f2.join(f1, None)
+        def verify_file(f):
+            self.assertTrue(f.has_version('r0'))
+            self.assertTrue(f.has_version('r1'))
+        verify_file(f2)
+        verify_file(self.get_target())
+
+        self.assertRaises(errors.RevisionNotPresent,
+            f2.join, f1, version_ids=['r3'])
+
+        #f3 = self.get_file('1')
+        #f3.add_lines('r0', ['a\n', 'b\n'], [])
+        #f3.add_lines('r1', ['c\n', 'b\n'], ['r0'])
+        #f4 = self.get_file('2')
+        #f4.join(f3, ['r0'])
+        #self.assertTrue(f4.has_version('r0'))
+        #self.assertFalse(f4.has_version('r1'))
+
+    def test_gets_expected_inter_worker(self):
+        source = self.get_source()
+        target = self.get_target()
+        inter = versionedfile.InterVersionedFile.get(source, target)
+        self.assertTrue(isinstance(inter, self.interversionedfile_class))
+
+    def test_join_versions_joins_ancestors_not_siblings(self):
+        # joining with a version list should bring in ancestors of the
+        # named versions but not siblings thereof.
+        target = self.get_target()
+        target.add_lines('base', [], [])
+        source = self.get_source()
+        source.add_lines('base', [], [])
+        source.add_lines('sibling', ['base'], [])
+        source.add_lines('ancestorleft', ['base'], [])
+        source.add_lines('ancestorright', ['base'], [])
+        source.add_lines('namedleft', ['ancestorleft'], [])
+        source.add_lines('namedright', ['ancestorright'], [])
+        target.join(source, version_ids=['namedleft', 'namedright'])
+        self.assertFalse(target.has_version('sibling'))
+        self.assertTrue(target.has_version('ancestorleft'))
+        self.assertTrue(target.has_version('ancestorright'))
+        self.assertTrue(target.has_version('namedleft'))
+        self.assertTrue(target.has_version('namedright'))
+
+    def test_join_add_parents(self):
+        """Join inserting new parents into existing versions
+        
+        The new version must have the right parent list and must identify
+        lines originating in another parent.
+        """
+        w1 = self.get_target('w1')
+        w2 = self.get_source('w2')
+        w1.add_lines('v-1', [], ['line 1\n'])
+        w2.add_lines('v-2', [], ['line 2\n'])
+        w1.add_lines('v-3', ['v-1'], ['line 1\n'])
+        w2.add_lines('v-3', ['v-2'], ['line 1\n'])
+        w1.join(w2)
+        self.assertEqual(sorted(w1.versions()),
+                         'v-1 v-2 v-3'.split())
+        self.assertEqualDiff(w1.get_text('v-3'),
+                'line 1\n')
+        self.assertEqual(sorted(w1.get_parents('v-3')),
+                ['v-1', 'v-2'])
+        ann = list(w1.annotate('v-3'))
+        self.assertEqual(len(ann), 1)
+        self.assertEqual(ann[0][0], 'v-1')
+        self.assertEqual(ann[0][1], 'line 1\n')
+        
+    def build_weave1(self):
+        weave1 = self.get_source()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        weave1.add_lines('v1', [], self.lines1)
+        weave1.add_lines('v2', ['v1'], ['hello\n', 'world\n'])
+        weave1.add_lines('v3', ['v2'], self.lines3)
+        return weave1
+        
+    def test_join_with_empty(self):
+        """Reweave adding empty weave"""
+        wb = self.get_target()
+        w1 = self.build_weave1()
+        w1.join(wb)
+        self.verify_weave1(w1)
+
+    def verify_weave1(self, w1):
+        self.assertEqual(sorted(w1.versions()), ['v1', 'v2', 'v3'])
+        self.assertEqual(w1.get_lines('v1'), ['hello\n'])
+        self.assertEqual([], w1.get_parents('v1'))
+        self.assertEqual(w1.get_lines('v2'), ['hello\n', 'world\n'])
+        self.assertEqual(['v1'], w1.get_parents('v2'))
+        self.assertEqual(w1.get_lines('v3'), ['hello\n', 'cruel\n', 'world\n'])
+        self.assertEqual(['v2'], w1.get_parents('v3'))
+
+    def test_join_source_has_less_parents_preserves_parents(self):
+        # when the target has a text with more parent info, join 
+        # preserves that.
+        s = self.get_source()
+        s.add_lines('base', [], [])
+        s.add_lines('text', [], [])
+        t = self.get_target()
+        t.add_lines('base', [], [])
+        t.add_lines('text', ['base'], [])
+        t.join(s)
+        self.assertEqual(['base'], t.get_parents('text'))
+
+    def test_join_with_ghosts_merges_parents(self):
+        """Join combined parent lists"""
+        wa = self.build_weave1()
+        wb = self.get_target()
+        wb.add_lines('x1', [], ['line from x1\n'])
+        wb.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        self.assertEqual(['v1','x1'], wa.get_parents('v2'))
+
+    def test_join_with_ghosts(self):
+        """Join that inserts parents of an existing revision.
+
+        This can happen when merging from another branch who
+        knows about revisions the destination does not.  In 
+        this test the second weave knows of an additional parent of 
+        v2.  Any revisions which are in common still have to have the 
+        same text.
+        """
+        w1 = self.build_weave1()
+        wb = self.get_target()
+        wb.add_lines('x1', [], ['line from x1\n'])
+        wb.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        w1.join(wb)
+        eq = self.assertEquals
+        eq(sorted(w1.versions()), ['v1', 'v2', 'v3', 'x1',])
+        eq(w1.get_text('x1'), 'line from x1\n')
+        eq(w1.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(w1.get_parents('v2'), ['v1', 'x1'])
+
+    def test_join_with_ignore_missing_versions(self):
+        # test that ignore_missing=True makes a listed but absent version id
+        # be ignored, and that unlisted version_ids are not integrated.
+        w1 = self.build_weave1()
+        wb = self.get_target()
+        wb.add_lines('x1', [], ['line from x1\n'])
+        wb.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        w1.join(wb, version_ids=['x1', 'z1'], ignore_missing=True)
+        eq = self.assertEquals
+        eq(sorted(w1.versions()), ['v1', 'v2', 'v3', 'x1'])
+        eq(w1.get_text('x1'), 'line from x1\n')
+        eq(w1.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(w1.get_parents('v2'), ['v1'])
+    
+    def build_source_weave(self, name, *pattern):
+        w = self.get_source(name)
+        for version, parents in pattern:
+            w.add_lines(version, parents, [])
+        return w
+
+    def build_target_weave(self, name, *pattern):
+        w = self.get_target(name)
+        for version, parents in pattern:
+            w.add_lines(version, parents, [])
+        return w
+
+    def test_join_reorder(self):
+        """Reweave requiring reordering of versions.
+
+        Weaves must be stored such that parents come before children.  When
+        reweaving, we may add new parents to some children, but it is required
+        that there must be *some* valid order that can be found, otherwise the
+        ancestries are contradictory.  (For the specific case of inserting
+        ghost revisions there will be no disagreement, only partial knowledge
+        of the history.)
+
+        Note that the weaves are only partially ordered: when there are two
+        versions where neither is an ancestor of the other the order in which
+        they occur is unconstrained.  When we join those versions into
+        another weave, they may become more constrained and it may be
+        necessary to change their order.
+
+        One simple case of this is 
+
+        w1: (c[], a[], b[a])
+        w2: (b[], c[b], a[])
+        
+        We need to recognize that the final weave must show the ordering
+        a[], b[a], c[b].  The version that must be first in the result is 
+        not first in either of the input weaves.
+        """
+        w1 = self.build_target_weave('1', ('c', []), ('a', []), ('b', ['a']))
+        w2 = self.build_source_weave('2', ('b', []), ('c', ['b']), ('a', []))
+        w1.join(w2)
+        self.assertEqual([], w1.get_parents('a'))
+        self.assertEqual(['a'], w1.get_parents('b'))
+        self.assertEqual(['b'], w1.get_parents('c'))
+        
+    def test_joining_ghosts(self):
+        # some versioned file formats allow lines to be added with parent
+        # information that is > than that in the format. Formats that do
+        # not support this need to raise NotImplementedError on the
+        # add_lines_with_ghosts api.
+        # files with ghost information when joined into a file which
+        # supports that must preserve it, when joined into a file which
+        # does not must discard it, and when filling a ghost for a listed
+        # ghost must reconcile it
+        source = self.get_source()
+        try:
+            source.has_ghost('a')
+            source_ghosts = True
+        except NotImplementedError:
+            source_ghosts = False
+        target = self.get_target()
+        try:
+            target.has_ghost('a')
+            target_ghosts = True
+        except NotImplementedError:
+            target_ghosts = False
+
+        if not source_ghosts and not target_ghosts:
+            # nothing to do
+            return
+        if source_ghosts and not target_ghosts:
+            # switch source and target so source is ghostless
+            t = source
+            source = target
+            target = source
+            source_ghosts = False
+            target_ghosts = True
+        # now target always supports ghosts.
+
+        # try filling target with ghosts and filling in reverse -  
+        target.add_lines_with_ghosts('notbase', ['base'], [])
+        source.join(target)
+        # legacy apis should behave
+        self.assertEqual(['notbase'], source.get_ancestry(['notbase']))
+        self.assertEqual([], source.get_parents('notbase'))
+        self.assertEqual({'notbase':[]}, source.get_graph())
+        self.assertFalse(source.has_version('base'))
+        if source_ghosts:
+            # ghost data should have been preserved
+            self.assertEqual(['base', 'notbase'], source.get_ancestry_with_ghosts(['notbase']))
+            self.assertEqual(['base'], source.get_parents_with_ghosts('notbase'))
+            self.assertEqual({'notbase':['base']}, source.get_graph_with_ghosts())
+            self.assertTrue(source.has_ghost('base'))
+
+        # if we add something that is fills out what is a ghost, then 
+        # when joining into a ghost aware join it should flesh out the ghosts.
+        source.add_lines('base', [], [])
+        target.join(source, version_ids=['base']) 
+        self.assertEqual(['base', 'notbase'], target.get_ancestry(['notbase']))
+        self.assertEqual(['base'], target.get_parents('notbase'))
+        self.assertEqual({'base':[],
+                          'notbase':['base'],
+                          },
+                         target.get_graph())
+        self.assertTrue(target.has_version('base'))
+        # we have _with_ghost apis to give us ghost information.
+        self.assertEqual(['base', 'notbase'], target.get_ancestry_with_ghosts(['notbase']))
+        self.assertEqual(['base'], target.get_parents_with_ghosts('notbase'))
+        self.assertEqual({'base':[],
+                          'notbase':['base'],
+                          },
+                         target.get_graph_with_ghosts())
+        self.assertFalse(target.has_ghost('base'))

=== added directory 'bzrlib/tests/repository_implementations'
=== added file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2006-02-23 04:08:56 +0000
@@ -0,0 +1,56 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the repository variations to the expected API.
+Specific tests for individual formats are in the tests/test_repository.py file 
+rather than in tests/branch_implementations/*.py.
+"""
+
+from bzrlib.repository import (_legacy_formats,
+                               RepositoryFormat,
+                               RepositoryTestProviderAdapter,
+                               )
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_repository_implementations = [
+        'bzrlib.tests.repository_implementations.test_fileid_involved',
+        'bzrlib.tests.repository_implementations.test_reconcile',
+        'bzrlib.tests.repository_implementations.test_repository',
+        ]
+    adapter = RepositoryTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        [(format, format._matchingbzrdir) for format in 
+         RepositoryFormat._formats.values() + _legacy_formats])
+    loader = TestLoader()
+    adapt_modules(test_repository_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/repository_implementations/test_fileid_involved.py'
--- bzrlib/tests/repository_implementations/test_fileid_involved.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-04-12 04:57:34 +0000
@@ -0,0 +1,214 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.add import smart_add
+from bzrlib.builtins import merge
+from bzrlib.delta import compare_trees
+from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
+from bzrlib.workingtree import WorkingTree
+
+
+class FileIdInvolvedBase(TestCaseWithRepository):
+
+    def touch(self,filename):
+        f = file(filename,"a")
+        f.write("appended line\n")
+        f.close( )
+
+    def compare_tree_fileids(self, branch, old_rev, new_rev):
+        old_tree = self.branch.repository.revision_tree(old_rev)
+        new_tree = self.branch.repository.revision_tree(new_rev)
+        delta = compare_trees(old_tree, new_tree)
+
+        l2 = [id for path, id, kind in delta.added] + \
+             [id for oldpath, newpath, id, kind, text_modified, \
+                meta_modified in delta.renamed] + \
+             [id for path, id, kind, text_modified, meta_modified in \
+                delta.modified]
+        return set(l2)
+
+    
+class TestFileIdInvolved(FileIdInvolvedBase):
+
+    def setUp(self):
+        super(TestFileIdInvolved, self).setUp()
+        # create three branches, and merge it
+        #
+        #           /-->J ------>K                (branch2)
+        #          /              \
+        #  A ---> B --->C ---->D->G               (main)
+        #  \           /      /
+        #   \---> E---/----> F                 (branch1)
+
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id> quiji%bo'])
+        main_wt.commit("Commit one", rev_id="rev-A")
+        #-------- end A -----------
+
+        d1 = main_branch.bzrdir.clone('branch1')
+        b1 = d1.open_branch()
+        self.build_tree(["branch1/d"])
+        bt1 = d1.open_workingtree()
+        bt1.add('d')
+        bt1.commit("branch1, Commit one", rev_id="rev-E")
+
+        #-------- end E -----------
+
+        self.touch("main/a")
+        main_wt.commit("Commit two", rev_id="rev-B")
+
+        #-------- end B -----------
+
+        d2 = main_branch.bzrdir.clone('branch2')
+        branch2_branch = d2.open_branch()
+        bt2 = d2.open_workingtree()
+        os.chmod("branch2/b",0770)
+        bt2.commit("branch2, Commit one", rev_id="rev-J")
+
+        #-------- end J -----------
+
+        self.merge(b1, main_wt)
+        main_wt.commit("merge branch1, rev-11", rev_id="rev-C")
+
+        #-------- end C -----------
+
+        bt1.rename_one("d","e")
+        bt1.commit("branch1, commit two", rev_id="rev-F")
+
+        #-------- end F -----------
+
+        self.touch("branch2/c")
+        bt2.commit("branch2, commit two", rev_id="rev-K")
+
+        #-------- end K -----------
+
+        self.touch("main/b")
+        self.merge(b1, main_wt)
+        # D gets some funky characters to make sure the unescaping works
+        main_wt.commit("merge branch1, rev-12", rev_id="rev-<D>")
+
+        # end D
+
+        self.merge(branch2_branch, main_wt)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+
+    def test_fileid_involved_all_revs(self):
+
+        l = self.branch.repository.fileid_involved( )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["a","b","c","d"])
+
+    def test_fileid_involved_one_rev(self):
+
+        l = self.branch.repository.fileid_involved("rev-B" )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["a","b","c"])
+
+    def test_fileid_involved_two_revs(self):
+
+        l = self.branch.repository.fileid_involved_between_revs("rev-B","rev-K" )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["b","c"])
+
+        l = self.branch.repository.fileid_involved_between_revs("rev-C","rev-<D>" )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["b","d"])
+
+        l = self.branch.repository.fileid_involved_between_revs("rev-C","rev-G" )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["b","c","d"])
+
+        l = self.branch.repository.fileid_involved_between_revs("rev-E","rev-G" )
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["a", "b","c","d"])
+
+    def test_fileid_involved_sets(self):
+
+        l = self.branch.repository.fileid_involved_by_set(set(["rev-B"]))
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["a"])
+
+        l = self.branch.repository.fileid_involved_by_set(set(["rev-<D>"]))
+        self.assertEquals( sorted(map( lambda x: x[0], l )), ["b"])
+
+    def test_fileid_involved_compare(self):
+
+        l1 = self.branch.repository.fileid_involved_between_revs("rev-E", "rev-<D>")
+        l2 = self.branch.repository.fileid_involved_by_set(set(["rev-<D>","rev-F","rev-C","rev-B"]))
+        self.assertEquals( l1, l2 )
+
+        l1 = self.branch.repository.fileid_involved_between_revs("rev-C", "rev-G")
+        l2 = self.branch.repository.fileid_involved_by_set(
+            set(["rev-G","rev-<D>","rev-F","rev-K","rev-J"]))
+        self.assertEquals( l1, l2 )
+
+    def test_fileid_involved_full_compare(self):
+        pp=[]
+        history = self.branch.revision_history( )
+
+        if len(history) < 2: return
+
+        for start in range(0,len(history)-1):
+            start_id = history[start]
+            for end in range(start+1,len(history)):
+                end_id = history[end]
+                l1 = self.branch.repository.fileid_involved_between_revs(
+                    start_id, end_id)
+                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
+                self.assertEquals(l1, l2)
+
+
+class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
+
+    def setUp(self):
+        super(TestFileIdInvolvedSuperset, self).setUp()
+
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id> quiji%bo'])
+        main_wt.commit("Commit one", rev_id="rev-A")
+
+        branch2_bzrdir = main_branch.bzrdir.sprout("branch2")
+        branch2_branch = branch2_bzrdir.open_branch()
+        branch2_wt = branch2_bzrdir.open_workingtree()
+        os.chmod("branch2/b",0770)
+        branch2_wt.commit("branch2, Commit one", rev_id="rev-J")
+
+        self.merge(branch2_branch, main_wt)
+        os.chmod("main/b",0660)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileid_involved_full_compare2(self):
+        history = self.branch.revision_history()
+        old_rev = history[0]
+        new_rev = history[1]
+
+        l1 = self.branch.repository.fileid_involved_between_revs(old_rev, new_rev)
+
+        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
+        self.assertNotEqual(l2, l1)
+        self.assertSubset(l2, l1)

=== added file 'bzrlib/tests/repository_implementations/test_reconcile.py'
--- bzrlib/tests/repository_implementations/test_reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_reconcile.py	2006-03-08 11:08:06 +0000
@@ -0,0 +1,187 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for reconiliation of repositories."""
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.reconcile import reconcile, Reconciler
+from bzrlib.revision import Revision
+from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
+from bzrlib.transport import get_transport
+from bzrlib.tree import EmptyTree
+from bzrlib.workingtree import WorkingTree
+
+
+class TestsNeedingReweave(TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestsNeedingReweave, self).setUp()
+        
+        t = get_transport(self.get_url())
+        # an empty inventory with no revision for testing with.
+        # this is referenced by 'references_missing' to let us test
+        # that all the cached data is correctly converted into ghost links
+        # and the referenced inventory still cleaned.
+        repo = self.make_repository('inventory_without_revision')
+        inv = EmptyTree().inventory
+        repo.add_inventory('missing', inv, [])
+        sha1 = repo.add_inventory('references_missing', inv, ['missing'])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='references_missing')
+        rev.parent_ids = ['missing']
+        repo.add_revision('references_missing', rev)
+
+        # a inventory with no parents and the revision has parents..
+        # i.e. a ghost.
+        repo = self.make_repository('inventory_one_ghost')
+        sha1 = repo.add_inventory('ghost', inv, [])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='ghost')
+        rev.parent_ids = ['the_ghost']
+        repo.add_revision('ghost', rev)
+         
+        # a inventory with a ghost that can be corrected now.
+        t.copy_tree('inventory_one_ghost', 'inventory_ghost_present')
+        repo = bzrlib.repository.Repository.open('inventory_ghost_present')
+        sha1 = repo.add_inventory('the_ghost', inv, [])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='the_ghost')
+        rev.parent_ids = []
+        repo.add_revision('the_ghost', rev)
+
+    def test_reweave_empty(self):
+        self.make_repository('empty')
+        d = bzrlib.bzrdir.BzrDir.open('empty')
+        # calling on a empty repository should do nothing
+        reconciler = d.find_repository().reconcile()
+        # no inconsistent parents should have been found
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # and no backup weave should have been needed/made.
+        repo = d.open_repository()
+        self.assertRaises(errors.NoSuchFile,
+                          repo.control_weaves.get_weave,
+                          'inventory.backup',
+                          repo.get_transaction())
+
+    def test_reweave_inventory_without_revision_reconcile(self):
+        # smoke test for the all in one ui tool
+        d = bzrlib.bzrdir.BzrDir.open('inventory_without_revision')
+        reconcile(d)
+        # now the backup should have it but not the current inventory
+        repo = d.open_repository()
+        backup = repo.control_weaves.get_weave('inventory.backup',
+                                               repo.get_transaction())
+        self.assertTrue('missing' in backup.versions())
+        self.assertRaises(errors.RevisionNotPresent,
+                          repo.get_inventory, 'missing')
+
+    def test_reweave_inventory_without_revision_reconciler(self):
+        # smoke test for the all in one Reconciler class,
+        # other tests use the lower level repo.reconcile()
+        d = bzrlib.bzrdir.BzrDir.open('inventory_without_revision')
+        reconciler = Reconciler(d)
+        reconciler.reconcile()
+        # no inconsistent parents should have been found
+        self.assertEqual(1, reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(1, reconciler.garbage_inventories)
+        # now the backup should have it but not the current inventory
+        repo = d.open_repository()
+        backup = repo.control_weaves.get_weave('inventory.backup',
+                                               repo.get_transaction())
+        self.assertTrue('missing' in backup.versions())
+        self.assertRaises(errors.RevisionNotPresent,
+                          repo.get_inventory, 'missing')
+
+    def test_reweave_inventory_without_revision(self):
+        # actual low level test.
+        d = bzrlib.bzrdir.BzrDir.open('inventory_without_revision')
+        repo = d.open_repository()
+        if ([None, 'missing', 'references_missing'] 
+            != repo.get_ancestry('references_missing')):
+            # the repo handles ghosts without corruption, so reconcile has
+            # nothing to do here
+            expected_inconsistent_parents = 0
+        else:
+            expected_inconsistent_parents = 1
+        reconciler = repo.reconcile()
+        # some number of inconsistent parents should have been found
+        self.assertEqual(expected_inconsistent_parents,
+                         reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(1, reconciler.garbage_inventories)
+        # now the backup should have it but not the current inventory
+        repo = d.open_repository()
+        backup = repo.control_weaves.get_weave('inventory.backup',
+                                               repo.get_transaction())
+        self.assertTrue('missing' in backup.versions())
+        self.assertRaises(errors.RevisionNotPresent,
+                          repo.get_inventory, 'missing')
+        # and the parent list for 'references_missing' should have that
+        # revision a ghost now.
+        self.assertEqual([None, 'references_missing'],
+                         repo.get_ancestry('references_missing'))
+
+    def test_reweave_inventory_preserves_a_revision_with_ghosts(self):
+        d = bzrlib.bzrdir.BzrDir.open('inventory_one_ghost')
+        reconciler = d.open_repository().reconcile()
+        # no inconsistent parents should have been found: 
+        # the lack of a parent for ghost is normal
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # now the current inventory should still have 'ghost'
+        repo = d.open_repository()
+        repo.get_inventory('ghost')
+        self.assertEqual([None, 'ghost'], repo.get_ancestry('ghost'))
+        
+    def test_reweave_inventory_fixes_ancestryfor_a_present_ghost(self):
+        d = bzrlib.bzrdir.BzrDir.open('inventory_ghost_present')
+        repo = d.open_repository()
+        ghost_ancestry = repo.get_ancestry('ghost')
+        if ghost_ancestry == [None, 'the_ghost', 'ghost']:
+            # the repo handles ghosts without corruption, so reconcile has
+            # nothing to do
+            return
+        self.assertEqual([None, 'ghost'], ghost_ancestry)
+        reconciler = repo.reconcile()
+        # one inconsistent parents should have been found : the
+        # available but not reference parent for ghost.
+        self.assertEqual(1, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # now the current inventory should still have 'ghost'
+        repo = d.open_repository()
+        repo.get_inventory('ghost')
+        repo.get_inventory('the_ghost')
+        self.assertEqual([None, 'the_ghost', 'ghost'], repo.get_ancestry('ghost'))
+        self.assertEqual([None, 'the_ghost'], repo.get_ancestry('the_ghost'))

=== added file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,371 @@
+# (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import os
+import sys
+
+import bzrlib
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.transport import get_transport
+from bzrlib.upgrade import upgrade
+from bzrlib.workingtree import WorkingTree
+
+
+class TestCaseWithRepository(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithRepository, self).setUp()
+
+    def make_branch(self, relpath, format=None):
+        repo = self.make_repository(relpath, format=None)
+        return repo.bzrdir.create_branch()
+
+    def make_repository(self, relpath, format=None):
+        made_control = self.make_bzrdir(relpath)
+        return self.repository_format.initialize(made_control)
+
+
+class TestRepository(TestCaseWithRepository):
+
+    def test_clone_to_default_format(self):
+        #TODO: Test that cloning a repository preserves all the information
+        # such as signatures[not tested yet] etc etc.
+        # when changing to the current default format.
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        bzrdirb = self.make_bzrdir('b')
+        repo_b = tree_a.branch.repository.clone(bzrdirb)
+        tree_b = repo_b.revision_tree('rev1')
+        tree_b.get_file_text('file1')
+        rev1 = repo_b.get_revision('rev1')
+
+    def test_clone_specific_format(self):
+        """todo"""
+
+    def test_format_initialize_find_open(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.repository_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        t = get_transport(self.get_url())
+        readonly_t = get_transport(self.get_readonly_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = self.repository_format.initialize(made_control)
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+
+        # find it via bzrdir opening:
+        opened_control = bzrdir.BzrDir.open(readonly_t.base)
+        direct_opened_repo = opened_control.open_repository()
+        self.assertEqual(direct_opened_repo.__class__, made_repo.__class__)
+        self.assertEqual(opened_control, direct_opened_repo.bzrdir)
+
+        self.failUnless(isinstance(direct_opened_repo._format,
+                        self.repository_format.__class__))
+        # find it via Repository.open
+        opened_repo = repository.Repository.open(readonly_t.base)
+        self.failUnless(isinstance(opened_repo, made_repo.__class__))
+        self.assertEqual(made_repo._format.__class__,
+                         opened_repo._format.__class__)
+        # if it has a unique id string, can we probe for it ?
+        try:
+            self.repository_format.get_format_string()
+        except NotImplementedError:
+            return
+        self.assertEqual(self.repository_format,
+                         repository.RepositoryFormat.find_format(opened_control))
+
+    def test_create_repository(self):
+        # bzrdir can construct a repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+        
+    def test_create_repository_shared(self):
+        # bzrdir can construct a shared repository.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        try:
+            made_repo = made_control.create_repository(shared=True)
+        except errors.IncompatibleFormat:
+            # not all repository formats understand being shared, or
+            # may only be shared in some circumstances.
+            return
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+        self.assertTrue(made_repo.is_shared())
+
+    def test_revision_tree(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit('lala!', rev_id='revision-1', allow_pointless=True)
+        tree = wt.branch.repository.revision_tree('revision-1')
+        self.assertEqual(list(tree.list_files()), [])
+        tree = wt.branch.repository.revision_tree(None)
+        self.assertEqual(len(tree.list_files()), 0)
+        tree = wt.branch.repository.revision_tree(NULL_REVISION)
+        self.assertEqual(len(tree.list_files()), 0)
+
+    def test_fetch(self):
+        # smoke test fetch to ensure that the convenience function works.
+        # it is defined as a convenience function with the underlying 
+        # functionality provided by an InterRepository
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        # fetch with a default limit (grab everything)
+        repo = bzrdir.BzrDir.create_repository(self.get_url('b'))
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+
+    def test_clone_bzrdir_repository_revision(self):
+        # make a repository with some revisions,
+        # and clone it, this should not have unreferenced revisions.
+        # also: test cloning with a revision id of NULL_REVISION -> empty repo.
+        raise TestSkipped('revision limiting is not implemented yet.')
+
+    def test_clone_repository_basis_revision(self):
+        raise TestSkipped('the use of a basis should not add noise data to the result.')
+
+    def test_clone_repository_incomplete_source_with_basis(self):
+        # ensure that basis really does grab from the basis by having incomplete source
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_repository('source')
+        # this gives us an incomplete repository
+        tree.bzrdir.open_repository().copy_content_into(source)
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        self.assertFalse(source.has_revision('2'))
+        target = source.bzrdir.clone(self.get_url('target'), basis=tree.bzrdir)
+        self.assertTrue(target.open_repository().has_revision('2'))
+
+    def test_clone_shared_no_tree(self):
+        # cloning a shared repository keeps it shared
+        # and preserves the make_working_tree setting.
+        made_control = self.make_bzrdir('source')
+        try:
+            made_repo = made_control.create_repository(shared=True)
+        except errors.IncompatibleFormat:
+            # not all repository formats understand being shared, or
+            # may only be shared in some circumstances.
+            return
+        made_repo.set_make_working_trees(False)
+        result = made_control.clone(self.get_url('target'))
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+        self.assertTrue(result.open_repository().is_shared())
+        self.assertFalse(result.open_repository().make_working_trees())
+
+    def test_upgrade_preserves_signatures(self):
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        wt.branch.repository.sign_revision('A',
+            bzrlib.gpg.LoopbackGPGStrategy(None))
+        old_signature = wt.branch.repository.get_signature_text('A')
+        try:
+            old_format = bzrdir.BzrDirFormat.get_default_format()
+            # This gives metadir branches something they can convert to.
+            # it would be nice to have a 'latest' vs 'default' concept.
+            bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+            try:
+                upgrade(wt.basedir)
+            finally:
+                bzrdir.BzrDirFormat.set_default_format(old_format)
+        except errors.UpToDateFormat:
+            # this is in the most current format already.
+            return
+        wt = WorkingTree.open(wt.basedir)
+        new_signature = wt.branch.repository.get_signature_text('A')
+        self.assertEqual(old_signature, new_signature)
+
+    def test_exposed_versioned_files_are_marked_dirty(self):
+        repo = self.make_repository('.')
+        repo.lock_write()
+        inv = repo.get_inventory_weave()
+        repo.unlock()
+        self.assertRaises(errors.OutSideTransaction, inv.add_lines, 'foo', [], [])
+
+    def test_format_description(self):
+        repo = self.make_repository('.')
+        text = repo._format.get_format_description()
+        self.failUnless(len(text))
+
+
+class TestCaseWithComplexRepository(TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestCaseWithComplexRepository, self).setUp()
+        tree_a = self.make_branch_and_tree('a')
+        self.bzrdir = tree_a.branch.bzrdir
+        # add a corrupt inventory 'orphan'
+        # this may need some generalising for knits.
+        inv_file = tree_a.branch.repository.control_weaves.get_weave(
+            'inventory', 
+            tree_a.branch.repository.get_transaction())
+        inv_file.add_lines('orphan', [], [])
+        # add a real revision 'rev1'
+        tree_a.commit('rev1', rev_id='rev1', allow_pointless=True)
+        # add a real revision 'rev2' based on rev1
+        tree_a.commit('rev2', rev_id='rev2', allow_pointless=True)
+        # add a reference to a ghost
+        tree_a.add_pending_merge('ghost1')
+        tree_a.commit('rev3', rev_id='rev3', allow_pointless=True)
+        # add another reference to a ghost, and a second ghost.
+        tree_a.add_pending_merge('ghost1')
+        tree_a.add_pending_merge('ghost2')
+        tree_a.commit('rev4', rev_id='rev4', allow_pointless=True)
+
+    def test_all_revision_ids(self):
+        # all_revision_ids -> all revisions
+        self.assertEqual(['rev1', 'rev2', 'rev3', 'rev4'],
+                         self.bzrdir.open_repository().all_revision_ids())
+
+    def test_get_ancestry_missing_revision(self):
+        # get_ancestry(revision that is in some data but not fully installed
+        # -> NoSuchRevision
+        self.assertRaises(errors.NoSuchRevision,
+                          self.bzrdir.open_repository().get_ancestry, 'orphan')
+
+    def test_get_revision_graph(self):
+        # we can get a mapping of id->parents for the entire revision graph or bits thereof.
+        self.assertEqual({'rev1':[],
+                          'rev2':['rev1'],
+                          'rev3':['rev2'],
+                          'rev4':['rev3'],
+                          },
+                         self.bzrdir.open_repository().get_revision_graph(None))
+        self.assertEqual({'rev1':[]},
+                         self.bzrdir.open_repository().get_revision_graph('rev1'))
+        self.assertEqual({'rev1':[],
+                          'rev2':['rev1']},
+                         self.bzrdir.open_repository().get_revision_graph('rev2'))
+        self.assertRaises(NoSuchRevision,
+                          self.bzrdir.open_repository().get_revision_graph,
+                          'orphan')
+        # and ghosts are not mentioned
+        self.assertEqual({'rev1':[],
+                          'rev2':['rev1'],
+                          'rev3':['rev2'],
+                          },
+                         self.bzrdir.open_repository().get_revision_graph('rev3'))
+
+    def test_get_revision_graph_with_ghosts(self):
+        # we can get a graph object with roots, ghosts, ancestors and
+        # descendants.
+        repo = self.bzrdir.open_repository()
+        graph = repo.get_revision_graph_with_ghosts([])
+        self.assertEqual(set(['rev1']), graph.roots)
+        self.assertEqual(set(['ghost1', 'ghost2']), graph.ghosts)
+        self.assertEqual({'rev1':[],
+                          'rev2':['rev1'],
+                          'rev3':['rev2', 'ghost1'],
+                          'rev4':['rev3', 'ghost1', 'ghost2'],
+                          },
+                          graph.get_ancestors())
+        self.assertEqual({'ghost1':{'rev3':1, 'rev4':1},
+                          'ghost2':{'rev4':1},
+                          'rev1':{'rev2':1},
+                          'rev2':{'rev3':1},
+                          'rev3':{'rev4':1},
+                          'rev4':{},
+                          },
+                          graph.get_descendants())
+
+
+class TestCaseWithCorruptRepository(TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestCaseWithCorruptRepository, self).setUp()
+        # a inventory with no parents and the revision has parents..
+        # i.e. a ghost.
+        repo = self.make_repository('inventory_with_unnecessary_ghost')
+        inv = bzrlib.tree.EmptyTree().inventory
+        sha1 = repo.add_inventory('ghost', inv, [])
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       inventory_sha1=sha1,
+                                       revision_id='ghost')
+        rev.parent_ids = ['the_ghost']
+        repo.add_revision('ghost', rev)
+         
+        sha1 = repo.add_inventory('the_ghost', inv, [])
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       inventory_sha1=sha1,
+                                       revision_id='the_ghost')
+        rev.parent_ids = []
+        repo.add_revision('the_ghost', rev)
+        # check its setup usefully
+        inv_weave = repo.get_inventory_weave()
+        self.assertEqual(['ghost'], inv_weave.get_ancestry(['ghost']))
+
+    def test_corrupt_revision_access_asserts_if_reported_wrong(self):
+        repo = repository.Repository.open('inventory_with_unnecessary_ghost')
+        reported_wrong = False
+        try:
+            if repo.get_ancestry('ghost') != [None, 'the_ghost', 'ghost']:
+                reported_wrong = True
+        except errors.CorruptRepository:
+            # caught the bad data:
+            return
+        if not reported_wrong:
+            return
+        self.assertRaises(errors.CorruptRepository, repo.get_revision, 'ghost')
+
+    def test_corrupt_revision_get_revision_reconcile(self):
+        repo = repository.Repository.open('inventory_with_unnecessary_ghost')
+        repo.get_revision_reconcile('ghost')

=== added directory 'bzrlib/tests/revisionstore_implementations'
=== added file 'bzrlib/tests/revisionstore_implementations/__init__.py'
--- bzrlib/tests/revisionstore_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/revisionstore_implementations/__init__.py	2006-03-03 02:09:49 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""RevisionStore implemnetation tests for bzr.
+
+These test the conformance of all the revision store implementations to the 
+expected API including generally applicable corner cases.
+Specific tests for individual cases are in the tests/test_revisionstore.py file 
+rather than in tests/revisionstore_implementations/*.py.
+"""
+
+from bzrlib.store.revision import RevisionStoreTestProviderAdapter
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_revisionstore_implementations = [
+        'bzrlib.tests.revisionstore_implementations.test_all',
+        ]
+    adapter = RevisionStoreTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        RevisionStoreTestProviderAdapter.default_test_list()
+        )
+    loader = TestLoader()
+    adapt_modules(test_revisionstore_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/revisionstore_implementations/test_all.py'
--- bzrlib/tests/revisionstore_implementations/test_all.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/revisionstore_implementations/test_all.py	2006-03-06 10:17:26 +0000
@@ -0,0 +1,119 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Revision store tests."""
+
+
+import bzrlib.errors as errors
+from bzrlib.revision import Revision
+from bzrlib.store.revision import RevisionStore
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.transactions import PassThroughTransaction
+from bzrlib.tree import EmptyTree
+
+
+class TestFactory(TestCaseWithTransport):
+
+    def test_factory_keeps_smoke_in(self):
+        s = self.store_factory.create(self.get_url('.'))
+        self.assertTrue(isinstance(s, RevisionStore))
+
+
+class TestAll(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestAll, self).setUp()
+        self.store = self.store_factory.create(self.get_url('.'))
+        self.transaction = PassThroughTransaction()
+
+    def test_add_has_get(self):
+        rev = self.add_sample_rev()
+        self.assertTrue(self.store.has_revision_id('A', self.transaction))
+        rev2 = self.store.get_revision('A', self.transaction)
+        self.assertEqual(rev, rev2)
+
+    def add_sample_rev(self):
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1='aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
+                       revision_id='A')
+        self.store.add_revision(rev, self.transaction)
+        return rev
+
+    def test_has_missing(self):
+        # has of a non present id -> False
+        self.assertFalse(self.store.has_revision_id('missing', self.transaction))
+
+    def test_has_None(self):
+        # has of None -> True
+        self.assertTrue(self.store.has_revision_id(None, self.transaction))
+
+    def test_get_revision_none(self):
+        # get_revision(None) -> raises NoSuchRevision
+        self.assertRaises(errors.NoSuchRevision,
+                          self.store.get_revision,
+                          'B',
+                          self.transaction)
+
+    def test_add_signature_text_missing(self):
+        # add of a text signature for a missing revision must work, to allow
+        # revisions to be added after the signature.
+        self.store.add_revision_signature_text('A', 'foo\nbar', self.transaction)
+        # but must not be visible
+        self.assertRaises(errors.NoSuchRevision,
+                          self.store.has_signature,
+                          'A',
+                          self.transaction)
+        # at all
+        self.assertRaises(errors.NoSuchRevision,
+                          self.store.get_signature_text,
+                          'A',
+                          self.transaction)
+        # until the revision is added
+        self.add_sample_rev()
+        self.assertTrue(self.store.has_signature('A', self.transaction))
+        self.assertEqual('foo\nbar',
+                         self.store.get_signature_text('A', self.transaction))
+    
+    def test_add_signature_text(self):
+        # add a signature to a existing revision works.
+        self.add_sample_rev()
+        self.assertFalse(self.store.has_signature('A', self.transaction))
+        self.assertRaises(errors.NoSuchRevision,
+                          self.store.get_signature_text,
+                          'A',
+                          self.transaction)
+        self.store.add_revision_signature_text('A', 'foo\nbar', self.transaction)
+        self.assertTrue(self.store.has_signature('A', self.transaction))
+        self.assertEqual('foo\nbar',
+                         self.store.get_signature_text('A', self.transaction))
+
+    def test_total_size(self):
+        # we get a revision count and a numeric size figure from total_size().
+        count, bytes = self.store.total_size(self.transaction)
+        self.assertEqual(0, count)
+        self.assertEqual(0, bytes)
+        self.add_sample_rev()
+        count, bytes = self.store.total_size(self.transaction)
+        self.assertEqual(1, count)
+        self.assertNotEqual(0, bytes)
+        
+    def test_all_revision_ids(self):
+        self.assertEqual([], self.store.all_revision_ids(self.transaction))
+        self.add_sample_rev()
+        self.assertEqual(['A'], self.store.all_revision_ids(self.transaction))

=== added file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/stub_sftp.py	2006-01-25 01:48:01 +0000
@@ -0,0 +1,189 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+A stub SFTP server for loopback SFTP testing.
+Adapted from the one in paramiko's unit tests.
+"""
+
+import os
+from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
+    SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
+
+
+class StubServer (ServerInterface):
+    def __init__(self, test_case):
+        ServerInterface.__init__(self)
+        self._test_case = test_case
+
+    def check_auth_password(self, username, password):
+        # all are allowed
+        self._test_case.log('sftpserver - authorizing: %s' % (username,))
+        return AUTH_SUCCESSFUL
+
+    def check_channel_request(self, kind, chanid):
+        self._test_case.log('sftpserver - channel request: %s, %s' % (kind, chanid))
+        return OPEN_SUCCEEDED
+
+
+class StubSFTPHandle (SFTPHandle):
+    def stat(self):
+        try:
+            return SFTPAttributes.from_stat(os.fstat(self.readfile.fileno()))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def chattr(self, attr):
+        # python doesn't have equivalents to fchown or fchmod, so we have to
+        # use the stored filename
+        mutter('Changing permissions on %s to %s', self.filename, attr)
+        try:
+            SFTPServer.set_file_attr(self.filename, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+
+class StubSFTPServer (SFTPServerInterface):
+    def __init__(self, server, root, home=None):
+        SFTPServerInterface.__init__(self, server)
+        self.root = root
+        if home is None:
+            self.home = self.root
+        else:
+            self.home = home[len(self.root):]
+        if (len(self.home) > 0) and (self.home[0] == '/'):
+            self.home = self.home[1:]
+        server._test_case.log('sftpserver - new connection')
+
+    def _realpath(self, path):
+        return self.root + self.canonicalize(path)
+
+    def canonicalize(self, path):
+        if os.path.isabs(path):
+            return os.path.normpath(path)
+        else:
+            return os.path.normpath('/' + os.path.join(self.home, path))
+
+    def chattr(self, path, attr):
+        try:
+            SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def list_folder(self, path):
+        path = self._realpath(path)
+        try:
+            out = [ ]
+            flist = os.listdir(path)
+            for fname in flist:
+                attr = SFTPAttributes.from_stat(os.stat(pathjoin(path, fname)))
+                attr.filename = fname
+                out.append(attr)
+            return out
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def stat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.stat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def lstat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.lstat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def open(self, path, flags, attr):
+        path = self._realpath(path)
+        try:
+            if hasattr(os, 'O_BINARY'):
+                flags |= os.O_BINARY
+            if getattr(attr, 'st_mode', None):
+                fd = os.open(path, flags, attr.st_mode)
+            else:
+                fd = os.open(path, flags)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        if (flags & os.O_CREAT) and (attr is not None):
+            attr._flags &= ~attr.FLAG_PERMISSIONS
+            SFTPServer.set_file_attr(path, attr)
+        if flags & os.O_WRONLY:
+            fstr = 'wb'
+        elif flags & os.O_RDWR:
+            fstr = 'rb+'
+        else:
+            # O_RDONLY (== 0)
+            fstr = 'rb'
+        try:
+            f = os.fdopen(fd, fstr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        fobj = StubSFTPHandle()
+        fobj.filename = path
+        fobj.readfile = f
+        fobj.writefile = f
+        return fobj
+
+    def remove(self, path):
+        path = self._realpath(path)
+        try:
+            os.remove(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rename(self, oldpath, newpath):
+        oldpath = self._realpath(oldpath)
+        newpath = self._realpath(newpath)
+        try:
+            os.rename(oldpath, newpath)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def mkdir(self, path, attr):
+        path = self._realpath(path)
+        try:
+            # Using getattr() in case st_mode is None or 0
+            # both evaluate to False
+            if getattr(attr, 'st_mode', None):
+                os.mkdir(path, attr.st_mode)
+            else:
+                os.mkdir(path)
+            if attr is not None:
+                attr._flags &= ~attr.FLAG_PERMISSIONS
+                SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rmdir(self, path):
+        path = self._realpath(path)
+        try:
+            os.rmdir(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    # removed: chattr, symlink, readlink
+    # (nothing in bzr's sftp transport uses those)

=== added file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_ancestry.py	2006-01-26 20:52:58 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from cStringIO import StringIO
+import os
+import sys
+
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.revision import is_ancestor
+
+
+class TestAncestry(TestCaseWithTransport):
+
+    def test_straightline_ancestry(self):
+        """Test ancestry file when just committing."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        wt.commit(message='one',
+                  allow_pointless=True,
+                  rev_id='tester@foo--1')
+
+        wt.commit(message='two',
+                  allow_pointless=True,
+                  rev_id='tester@foo--2')
+
+        ancs = b.repository.get_ancestry('tester@foo--2')
+        self.assertEqual([None, 'tester@foo--1', 'tester@foo--2'], ancs)
+        self.assertEqual([None, 'tester@foo--1'], 
+                         b.repository.get_ancestry('tester@foo--1'))
+
+    def test_none_is_always_an_ancestor(self):
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        # note this is tested before any commits are done.
+        self.assertEqual(True, is_ancestor(None, None, b))
+        wt.commit(message='one',
+                  allow_pointless=True,
+                  rev_id='tester@foo--1')
+        self.assertEqual(True, is_ancestor(None, None, b))
+        self.assertEqual(True, is_ancestor('tester@foo--1', None, b))
+        self.assertEqual(False, is_ancestor(None, 'tester@foo--1', b))
+
+
+# TODO: check that ancestry is updated to include indirectly merged revisions

=== added file 'bzrlib/tests/test_annotate.py'
--- bzrlib/tests/test_annotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_annotate.py	2006-01-26 20:52:58 +0000
@@ -0,0 +1,70 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr.
+
+These check that it behaves properly when it's invoked through the regular
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
+"""
+
+
+from cStringIO import StringIO
+import os
+import shutil
+import sys
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.annotate import annotate_file
+
+
+class TestAnnotate(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestAnnotate, self).setUp()
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.build_tree_contents([('hello.txt', 'my helicopter\n'),
+                                  ('nomail.txt', 'nomail\n')])
+        wt.add(['hello.txt'])
+        wt.commit('add hello', 
+                                committer='test@user')
+        wt.add(['nomail.txt'])
+        wt.commit('add nomail', committer='no mail')
+
+    def test_help_annotate(self):
+        """Annotate command exists"""
+        out, err = self.run_bzr_captured(['--no-plugins', 'annotate', '--help'])
+
+    def test_annotate_cmd(self):
+        out, err = self.run_bzr_captured(['annotate', 'hello.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    1 test@us | my helicopter
+''')
+
+    def test_no_mail(self):
+        out, err = self.run_bzr_captured(['annotate', 'nomail.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    2 no mail | nomail
+''')

=== added file 'bzrlib/tests/test_api.py'
--- bzrlib/tests/test_api.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_api.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for library API infrastructure
+
+This is specifically for things controlling the interface, such as versioning.  
+Tests for particular parts of the library interface should be in specific
+relevant test modules.
+"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.tests import TestCase
+
+class APITests(TestCase):
+
+    def test_library_version(self):
+        """Library API version is exposed"""
+        self.assert_(isinstance(bzrlib.__version__, str))
+        self.assert_(isinstance(bzrlib.version_string, str))
+        self.assert_(isinstance(bzrlib.version_info, tuple))
+        self.assertEqual(len(bzrlib.version_info), 5)

=== added file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_bad_files.py	2006-02-22 23:54:32 +0000
@@ -0,0 +1,94 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests being able to ignore mad filetypes.
+"""
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.errors import BadFileKindError
+import os
+
+def verify_status(tester, tree, value):
+    from bzrlib.status import show_tree_status
+    from cStringIO import StringIO
+
+    tof = StringIO()
+    show_tree_status(tree, to_file=tof)
+    tof.seek(0)
+    tester.assertEquals(tof.readlines(), value)
+
+
+class TestBadFiles(TestCaseWithTransport):
+    
+    def test_bad_files(self): 
+        """Test that bzr will ignore files it doesn't like"""
+        from bzrlib.add import smart_add_tree
+        from bzrlib.branch import Branch
+
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        files = ['one', 'two', 'three']
+        self.build_tree(files)
+        wt.add(files)
+        wt.commit("Commit one", rev_id="a@u-0-0")
+        self.build_tree(['four'])
+        wt.add(['four'])
+        wt.commit("Commit two", rev_id="a@u-0-1")
+        self.build_tree(['five'])
+        wt.add(['five'])
+        wt.commit("Commit three", rev_id="a@u-0-2")
+
+        # We should now have a few files, lets try to
+        # put some bogus stuff in the tree
+
+        # We can only continue if we have mkfifo
+        if not hasattr(os, 'mkfifo'):
+            return
+
+        # status with nothing
+        verify_status(self, wt, [])
+
+        os.mkfifo('a-fifo')
+        self.build_tree(['six'])
+
+        verify_status(self, wt,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           '  six\n'
+                           ])
+        
+        # Make sure smart_add can handle having a bogus
+        # file in the way
+        smart_add_tree(wt, '.')
+        verify_status(self, wt,
+                          ['added:\n',
+                           '  six\n',
+                           'unknown:\n',
+                           '  a-fifo\n',
+                           ])
+        wt.commit("Commit four", rev_id="a@u-0-3")
+
+        verify_status(self, wt,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           ])
+
+        # We should raise an error if we are adding a bogus file
+        # Is there a way to test the actual error that should be raised?
+        self.run_bzr('add', 'a-fifo', retcode=3)
+

=== added file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_branch.py	2006-04-13 03:12:21 +0000
@@ -0,0 +1,201 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the Branch facility that are not interface  tests.
+
+For interface tests see tests/branch_implementations/*.py.
+
+For concrete class tests see this file, and for meta-branch tests
+also see this file.
+"""
+
+from StringIO import StringIO
+
+import bzrlib.branch
+from bzrlib.branch import (BzrBranch5, 
+                           BzrBranchFormat5)
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import (BzrDirMetaFormat1, BzrDirMeta1, 
+                           BzrDir, BzrDirFormat)
+from bzrlib.errors import (NotBranchError,
+                           UnknownFormatError,
+                           UnsupportedFormatError,
+                           )
+
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.transport import get_transport
+
+class TestDefaultFormat(TestCase):
+
+    def test_get_set_default_format(self):
+        old_format = bzrlib.branch.BranchFormat.get_default_format()
+        # default is 5
+        self.assertTrue(isinstance(old_format, bzrlib.branch.BzrBranchFormat5))
+        bzrlib.branch.BranchFormat.set_default_format(SampleBranchFormat())
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = BzrDirMetaFormat1().initialize('memory:/')
+            result = dir.create_branch()
+            self.assertEqual(result, 'A branch')
+        finally:
+            bzrlib.branch.BranchFormat.set_default_format(old_format)
+        self.assertEqual(old_format, bzrlib.branch.BranchFormat.get_default_format())
+
+
+class TestBranchFormat5(TestCaseWithTransport):
+    """Tests specific to branch format 5"""
+
+    def test_branch_format_5_uses_lockdir(self):
+        url = self.get_url()
+        bzrdir = BzrDirMetaFormat1().initialize(url)
+        bzrdir.create_repository()
+        branch = bzrdir.create_branch()
+        t = self.get_transport()
+        self.log("branch instance is %r" % branch)
+        self.assert_(isinstance(branch, BzrBranch5))
+        self.assertIsDirectory('.', t)
+        self.assertIsDirectory('.bzr/branch', t)
+        self.assertIsDirectory('.bzr/branch/lock', t)
+        branch.lock_write()
+        try:
+            self.assertIsDirectory('.bzr/branch/lock/held', t)
+        finally:
+            branch.unlock()
+
+
+class TestBranchEscaping(TestCaseWithTransport):
+    """Test a branch can be correctly stored and used on a vfat-like transport
+    
+    Makes sure we have proper escaping of invalid characters, etc.
+
+    It'd be better to test all operations on the FakeVFATTransportDecorator,
+    but working trees go straight to the os not through the Transport layer.
+    Therefore we build some history first in the regular way and then 
+    check it's safe to access for vfat.
+    """
+
+    FOO_ID = 'foo<:>ID'
+    REV_ID = 'revid-1'
+
+    def setUp(self):
+        super(TestBranchEscaping, self).setUp()
+        from bzrlib.repository import RepositoryFormatKnit1
+        bzrdir = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormatKnit1().initialize(bzrdir)
+        branch = bzrdir.create_branch()
+        wt = bzrdir.create_workingtree()
+        self.build_tree_contents([("foo", "contents of foo")])
+        # add file with id containing wierd characters
+        wt.add(['foo'], [self.FOO_ID])
+        wt.commit('this is my new commit', rev_id=self.REV_ID)
+
+    def test_branch_on_vfat(self):
+        from bzrlib.transport.fakevfat import FakeVFATTransportDecorator
+        # now access over vfat; should be safe
+        transport = FakeVFATTransportDecorator('vfat+' + self.get_url())
+        bzrdir, junk = BzrDir.open_containing_from_transport(transport)
+        branch = bzrdir.open_branch()
+        revtree = branch.repository.revision_tree(self.REV_ID)
+        contents = revtree.get_file_text(self.FOO_ID)
+        self.assertEqual(contents, 'contents of foo')
+
+
+class SampleBranchFormat(bzrlib.branch.BranchFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open_downlevel routines.
+    """
+
+    def get_format_string(self):
+        """See BzrBranchFormat.get_format_string()."""
+        return "Sample branch format."
+
+    def initialize(self, a_bzrdir):
+        """Format 4 branches cannot be created."""
+        t = a_bzrdir.get_branch_transport(self)
+        t.put('format', StringIO(self.get_format_string()))
+        return 'A branch'
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport, _found=False):
+        return "opened branch."
+
+
+class TestBzrBranchFormat(TestCaseWithTransport):
+    """Tests for the BzrBranchFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a branch?
+        # create a branch with a few known format objects.
+        # this is not quite the same as 
+        self.build_tree(["foo/", "bar/"])
+        def check_format(format, url):
+            dir = format._matchingbzrdir.initialize(url)
+            dir.create_repository()
+            format.initialize(dir)
+            found_format = bzrlib.branch.BranchFormat.find_format(dir)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(bzrlib.branch.BzrBranchFormat5(), "bar")
+        
+    def test_find_format_not_branch(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        self.assertRaises(NotBranchError,
+                          bzrlib.branch.BranchFormat.find_format,
+                          dir)
+
+    def test_find_format_unknown_format(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        SampleBranchFormat().initialize(dir)
+        self.assertRaises(UnknownFormatError,
+                          bzrlib.branch.BranchFormat.find_format,
+                          dir)
+
+    def test_register_unregister_format(self):
+        format = SampleBranchFormat()
+        # make a control dir
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        # make a branch
+        format.initialize(dir)
+        # register a format for it.
+        bzrlib.branch.BranchFormat.register_format(format)
+        # which branch.Open will refuse (not supported)
+        self.assertRaises(UnsupportedFormatError, bzrlib.branch.Branch.open, self.get_url())
+        # but open_downlevel will work
+        self.assertEqual(format.open(dir), bzrdir.BzrDir.open(self.get_url()).open_branch(unsupported=True))
+        # unregister the format
+        bzrlib.branch.BranchFormat.unregister_format(format)
+
+
+class TestBranchReference(TestCaseWithTransport):
+    """Tests for the branch reference facility."""
+
+    def test_create_open_reference(self):
+        bzrdirformat = bzrdir.BzrDirMetaFormat1()
+        t = get_transport(self.get_url('.'))
+        t.mkdir('repo')
+        dir = bzrdirformat.initialize(self.get_url('repo'))
+        dir.create_repository()
+        target_branch = dir.create_branch()
+        t.mkdir('branch')
+        branch_dir = bzrdirformat.initialize(self.get_url('branch'))
+        made_branch = bzrlib.branch.BranchReferenceFormat().initialize(branch_dir, target_branch)
+        self.assertEqual(made_branch.base, target_branch.base)
+        opened_branch = branch_dir.open_branch()
+        self.assertEqual(opened_branch.base, target_branch.base)

=== added file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,515 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the BzrDir facility and any format specific tests.
+
+For interface contract tests, see tests/bzr_dir_implementations.
+"""
+
+from StringIO import StringIO
+
+import bzrlib.branch
+import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
+from bzrlib.errors import (NotBranchError,
+                           UnknownFormatError,
+                           UnsupportedFormatError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import HttpServer
+from bzrlib.transport.memory import MemoryServer
+import bzrlib.workingtree as workingtree
+
+
+class TestDefaultFormat(TestCase):
+
+    def test_get_set_default_format(self):
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        # default is BzrDirFormat6
+        self.failUnless(isinstance(old_format, bzrdir.BzrDirMetaFormat1))
+        bzrdir.BzrDirFormat.set_default_format(SampleBzrDirFormat())
+        # creating a bzr dir should now create an instrumented dir.
+        try:
+            result = bzrdir.BzrDir.create('memory:/')
+            self.failUnless(isinstance(result, SampleBzrDir))
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+        self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
+
+
+class SampleBranch(bzrlib.branch.Branch):
+    """A dummy branch for guess what, dummy use."""
+
+    def __init__(self, dir):
+        self.bzrdir = dir
+
+
+class SampleBzrDir(bzrdir.BzrDir):
+    """A sample BzrDir implementation to allow testing static methods."""
+
+    def create_repository(self):
+        """See BzrDir.create_repository."""
+        return "A repository"
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        return "A repository"
+
+    def create_branch(self):
+        """See BzrDir.create_branch."""
+        return SampleBranch(self)
+
+    def create_workingtree(self):
+        """See BzrDir.create_workingtree."""
+        return "A tree"
+
+
+class SampleBzrDirFormat(bzrdir.BzrDirFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open_downlevel routines.
+    """
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Sample .bzr dir format."
+
+    def initialize(self, url):
+        """Create a bzr dir."""
+        t = get_transport(url)
+        t.mkdir('.bzr')
+        t.put('.bzr/branch-format', StringIO(self.get_format_string()))
+        return SampleBzrDir(t, self)
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport, _found=None):
+        return "opened branch."
+
+
+class TestBzrDirFormat(TestCaseWithTransport):
+    """Tests for the BzrDirFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a branch?
+        # create a branch with a few known format objects.
+        # this is not quite the same as 
+        t = get_transport(self.get_url())
+        self.build_tree(["foo/", "bar/"], transport=t)
+        def check_format(format, url):
+            format.initialize(url)
+            t = get_transport(url)
+            found_format = bzrdir.BzrDirFormat.find_format(t)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(bzrdir.BzrDirFormat5(), "foo")
+        check_format(bzrdir.BzrDirFormat6(), "bar")
+        
+    def test_find_format_nothing_there(self):
+        self.assertRaises(NotBranchError,
+                          bzrdir.BzrDirFormat.find_format,
+                          get_transport('.'))
+
+    def test_find_format_unknown_format(self):
+        t = get_transport(self.get_url())
+        t.mkdir('.bzr')
+        t.put('.bzr/branch-format', StringIO())
+        self.assertRaises(UnknownFormatError,
+                          bzrdir.BzrDirFormat.find_format,
+                          get_transport('.'))
+
+    def test_register_unregister_format(self):
+        format = SampleBzrDirFormat()
+        url = self.get_url()
+        # make a bzrdir
+        format.initialize(url)
+        # register a format for it.
+        bzrdir.BzrDirFormat.register_format(format)
+        # which bzrdir.Open will refuse (not supported)
+        self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open, url)
+        # which bzrdir.open_containing will refuse (not supported)
+        self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open_containing, url)
+        # but open_downlevel will work
+        t = get_transport(url)
+        self.assertEqual(format.open(t), bzrdir.BzrDir.open_unsupported(url))
+        # unregister the format
+        bzrdir.BzrDirFormat.unregister_format(format)
+        # now open_downlevel should fail too.
+        self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
+
+    def test_create_repository(self):
+        format = SampleBzrDirFormat()
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(format)
+        try:
+            repo = bzrdir.BzrDir.create_repository(self.get_url())
+            self.assertEqual('A repository', repo)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_repository_under_shared(self):
+        # an explicit create_repository always does so.
+        # we trust the format is right from the 'create_repository test'
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            repo = bzrdir.BzrDir.create_repository(self.get_url('child'))
+            self.assertTrue(isinstance(repo, repository.Repository))
+            self.assertTrue(repo.bzrdir.root_transport.base.endswith('child/'))
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_and_repo_uses_default(self):
+        format = SampleBzrDirFormat()
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(format)
+        try:
+            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url())
+            self.assertTrue(isinstance(branch, SampleBranch))
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_and_repo_under_shared(self):
+        # creating a branch and repo in a shared repo uses the
+        # shared repository
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url('child'))
+            self.assertRaises(errors.NoRepositoryPresent,
+                              branch.bzrdir.open_repository)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_and_repo_under_shared_force_new(self):
+        # creating a branch and repo in a shared repo can be forced to 
+        # make a new repo
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url('child'),
+                                                          force_new_repo=True)
+            branch.bzrdir.open_repository()
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_standalone_working_tree(self):
+        format = SampleBzrDirFormat()
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(format)
+        try:
+            # note this is deliberately readonly, as this failure should 
+            # occur before any writes.
+            self.assertRaises(errors.NotLocalUrl,
+                              bzrdir.BzrDir.create_standalone_workingtree,
+                              self.get_readonly_url())
+            tree = bzrdir.BzrDir.create_standalone_workingtree('.')
+            self.assertEqual('A tree', tree)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_standalone_working_tree_under_shared_repo(self):
+        # create standalone working tree always makes a repo.
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            # note this is deliberately readonly, as this failure should 
+            # occur before any writes.
+            self.assertRaises(errors.NotLocalUrl,
+                              bzrdir.BzrDir.create_standalone_workingtree,
+                              self.get_readonly_url('child'))
+            tree = bzrdir.BzrDir.create_standalone_workingtree('child')
+            tree.bzrdir.open_repository()
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_convenience(self):
+        # outside a repo the default convenience output is a repo+branch_tree
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            branch = bzrdir.BzrDir.create_branch_convenience('.')
+            branch.bzrdir.open_workingtree()
+            branch.bzrdir.open_repository()
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_convenience_under_shared_repo(self):
+        # inside a repo the default convenience output is a branch+ follow the
+        # repo tree policy
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            branch = bzrdir.BzrDir.create_branch_convenience('child')
+            branch.bzrdir.open_workingtree()
+            self.assertRaises(errors.NoRepositoryPresent,
+                              branch.bzrdir.open_repository)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+            
+    def test_create_branch_convenience_under_shared_repo_force_no_tree(self):
+        # inside a repo the default convenience output is a branch+ follow the
+        # repo tree policy but we can override that
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            branch = bzrdir.BzrDir.create_branch_convenience('child',
+                force_new_tree=False)
+            self.assertRaises(errors.NoWorkingTree,
+                              branch.bzrdir.open_workingtree)
+            self.assertRaises(errors.NoRepositoryPresent,
+                              branch.bzrdir.open_repository)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+            
+    def test_create_branch_convenience_under_shared_repo_no_tree_policy(self):
+        # inside a repo the default convenience output is a branch+ follow the
+        # repo tree policy
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            repo = self.make_repository('.', shared=True)
+            repo.set_make_working_trees(False)
+            branch = bzrdir.BzrDir.create_branch_convenience('child')
+            self.assertRaises(errors.NoWorkingTree,
+                              branch.bzrdir.open_workingtree)
+            self.assertRaises(errors.NoRepositoryPresent,
+                              branch.bzrdir.open_repository)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_convenience_under_shared_repo_no_tree_policy_force_tree(self):
+        # inside a repo the default convenience output is a branch+ follow the
+        # repo tree policy but we can override that
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            repo = self.make_repository('.', shared=True)
+            repo.set_make_working_trees(False)
+            branch = bzrdir.BzrDir.create_branch_convenience('child',
+                force_new_tree=True)
+            branch.bzrdir.open_workingtree()
+            self.assertRaises(errors.NoRepositoryPresent,
+                              branch.bzrdir.open_repository)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_convenience_under_shared_repo_force_new_repo(self):
+        # inside a repo the default convenience output is overridable to give
+        # repo+branch+tree
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.make_repository('.', shared=True)
+            branch = bzrdir.BzrDir.create_branch_convenience('child',
+                force_new_repo=True)
+            branch.bzrdir.open_repository()
+            branch.bzrdir.open_workingtree()
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+
+class ChrootedTests(TestCaseWithTransport):
+    """A support class that provides readonly urls outside the local namespace.
+
+    This is done by checking if self.transport_server is a MemoryServer. if it
+    is then we are chrooted already, if it is not then an HttpServer is used
+    for readonly urls.
+    """
+
+    def setUp(self):
+        super(ChrootedTests, self).setUp()
+        if not self.transport_server == MemoryServer:
+            self.transport_readonly_server = HttpServer
+
+    def test_open_containing(self):
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing,
+                          self.get_readonly_url(''))
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing,
+                          self.get_readonly_url('g/p/q'))
+        control = bzrdir.BzrDir.create(self.get_url())
+        branch, relpath = bzrdir.BzrDir.open_containing(self.get_readonly_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = bzrdir.BzrDir.open_containing(self.get_readonly_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
+
+    def test_open_containing_from_transport(self):
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing_from_transport,
+                          get_transport(self.get_readonly_url('')))
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing_from_transport,
+                          get_transport(self.get_readonly_url('g/p/q')))
+        control = bzrdir.BzrDir.create(self.get_url())
+        branch, relpath = bzrdir.BzrDir.open_containing_from_transport(
+            get_transport(self.get_readonly_url('')))
+        self.assertEqual('', relpath)
+        branch, relpath = bzrdir.BzrDir.open_containing_from_transport(
+            get_transport(self.get_readonly_url('g/p/q')))
+        self.assertEqual('g/p/q', relpath)
+
+
+class TestMeta1DirFormat(TestCaseWithTransport):
+    """Tests specific to the meta1 dir format."""
+
+    def test_right_base_dirs(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        t = dir.transport
+        branch_base = t.clone('branch').base
+        self.assertEqual(branch_base, dir.get_branch_transport(None).base)
+        self.assertEqual(branch_base,
+                         dir.get_branch_transport(bzrlib.branch.BzrBranchFormat5()).base)
+        repository_base = t.clone('repository').base
+        self.assertEqual(repository_base, dir.get_repository_transport(None).base)
+        self.assertEqual(repository_base,
+                         dir.get_repository_transport(repository.RepositoryFormat7()).base)
+        checkout_base = t.clone('checkout').base
+        self.assertEqual(checkout_base, dir.get_workingtree_transport(None).base)
+        self.assertEqual(checkout_base,
+                         dir.get_workingtree_transport(workingtree.WorkingTreeFormat3()).base)
+
+    def test_meta1dir_uses_lockdir(self):
+        """Meta1 format uses a LockDir to guard the whole directory, not a file."""
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        t = dir.transport
+        self.assertIsDirectory('branch-lock', t)
+
+        
+class TestFormat5(TestCaseWithTransport):
+    """Tests specific to the version 5 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created 
+        # for format 5 objects
+        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+    
+    def test_can_convert(self):
+        # format 5 dirs are convertable
+        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+    
+    def test_needs_conversion(self):
+        # format 5 dirs need a conversion if they are not the default.
+        # and they start of not the default.
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        try:
+            dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
+            self.assertFalse(dir.needs_format_conversion())
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+        self.assertTrue(dir.needs_format_conversion())
+
+
+class TestFormat6(TestCaseWithTransport):
+    """Tests specific to the version 6 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created 
+        # for format 6 objects
+        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+    
+    def test_can_convert(self):
+        # format 6 dirs are convertable
+        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+    
+    def test_needs_conversion(self):
+        # format 6 dirs need an conversion if they are not the default.
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
+            self.assertTrue(dir.needs_format_conversion())
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+
+class NonLocalTests(TestCaseWithTransport):
+    """Tests for bzrdir static behaviour on non local paths."""
+
+    def setUp(self):
+        super(NonLocalTests, self).setUp()
+        self.transport_server = MemoryServer
+    
+    def test_create_branch_convenience(self):
+        # outside a repo the default convenience output is a repo+branch_tree
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            branch = bzrdir.BzrDir.create_branch_convenience(self.get_url('foo'))
+            self.assertRaises(errors.NoWorkingTree,
+                              branch.bzrdir.open_workingtree)
+            branch.bzrdir.open_repository()
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_create_branch_convenience_force_tree_not_local_fails(self):
+        # outside a repo the default convenience output is a repo+branch_tree
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            self.assertRaises(errors.NotLocalUrl,
+                bzrdir.BzrDir.create_branch_convenience,
+                self.get_url('foo'),
+                force_new_tree=True)
+            t = get_transport(self.get_url('.'))
+            self.assertFalse(t.has('foo'))
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_clone(self):
+        # clone into a nonlocal path works
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            branch = bzrdir.BzrDir.create_branch_convenience('local')
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch.bzrdir.open_workingtree()
+        result = branch.bzrdir.clone(self.get_url('remote'))
+        self.assertRaises(errors.NoWorkingTree,
+                          result.open_workingtree)
+        result.open_branch()
+        result.open_repository()
+

=== added file 'bzrlib/tests/test_command.py'
--- bzrlib/tests/test_command.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_command.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,35 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.tests import TestCase
+from bzrlib.commands import display_command
+import errno
+
+class TestCommands(TestCase):
+    def test_display_command(self):
+        """EPIPE message is selectively suppressed"""
+        def pipe_thrower():
+            raise IOError(errno.EPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, pipe_thrower)
+        @display_command
+        def non_thrower():
+            pipe_thrower()
+        non_thrower()
+        @display_command
+        def other_thrower():
+            raise IOError(errno.ESPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, other_thrower)
+

=== added file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_commit.py	2006-04-13 03:12:21 +0000
@@ -0,0 +1,394 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+import bzrlib
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
+from bzrlib.workingtree import WorkingTree
+from bzrlib.commit import Commit
+from bzrlib.config import BranchConfig
+from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed, 
+                           LockContention)
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class MustSignConfig(BranchConfig):
+
+    def signature_needed(self):
+        return True
+
+    def gpg_signing_command(self):
+        return ['cat', '-']
+
+
+class BranchWithHooks(BranchConfig):
+
+    def post_commit(self):
+        return "bzrlib.ahook bzrlib.ahook"
+
+
+class TestCommit(TestCaseWithTransport):
+
+    def test_simple_commit(self):
+        """Commit and check two versions of a single file."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        wt.commit(message='add hello')
+        file_id = wt.path2id('hello')
+
+        file('hello', 'w').write('version 2')
+        wt.commit(message='commit 2')
+
+        eq = self.assertEquals
+        eq(b.revno(), 2)
+        rh = b.revision_history()
+        rev = b.repository.get_revision(rh[0])
+        eq(rev.message, 'add hello')
+
+        tree1 = b.repository.revision_tree(rh[0])
+        text = tree1.get_file_text(file_id)
+        eq(text, 'hello world')
+
+        tree2 = b.repository.revision_tree(rh[1])
+        eq(tree2.get_file_text(file_id), 'version 2')
+
+    def test_delete_commit(self):
+        """Test a commit with a deleted file"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add(['hello'], ['hello-id'])
+        wt.commit(message='add hello')
+
+        os.remove('hello')
+        wt.commit('removed hello', rev_id='rev2')
+
+        tree = b.repository.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+    def test_pointless_commit(self):
+        """Commit refuses unless there are changes or it's forced."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello')
+        wt.add(['hello'])
+        wt.commit(message='add hello')
+        self.assertEquals(b.revno(), 1)
+        self.assertRaises(PointlessCommit,
+                          wt.commit,
+                          message='fails',
+                          allow_pointless=False)
+        self.assertEquals(b.revno(), 1)
+        
+    def test_commit_empty(self):
+        """Commiting an empty tree works."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        wt.commit(message='empty tree', allow_pointless=True)
+        self.assertRaises(PointlessCommit,
+                          wt.commit,
+                          message='empty tree',
+                          allow_pointless=False)
+        wt.commit(message='empty tree', allow_pointless=True)
+        self.assertEquals(b.revno(), 2)
+
+    def test_selective_delete(self):
+        """Selective commit in tree with deletions"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello')
+        file('buongia', 'w').write('buongia')
+        wt.add(['hello', 'buongia'],
+              ['hello-id', 'buongia-id'])
+        wt.commit(message='add files',
+                 rev_id='test@rev-1')
+        
+        os.remove('hello')
+        file('buongia', 'w').write('new text')
+        wt.commit(message='update text',
+                 specific_files=['buongia'],
+                 allow_pointless=False,
+                 rev_id='test@rev-2')
+
+        wt.commit(message='remove hello',
+                 specific_files=['hello'],
+                 allow_pointless=False,
+                 rev_id='test@rev-3')
+
+        eq = self.assertEquals
+        eq(b.revno(), 3)
+
+        tree2 = b.repository.revision_tree('test@rev-2')
+        self.assertTrue(tree2.has_filename('hello'))
+        self.assertEquals(tree2.get_file_text('hello-id'), 'hello')
+        self.assertEquals(tree2.get_file_text('buongia-id'), 'new text')
+        
+        tree3 = b.repository.revision_tree('test@rev-3')
+        self.assertFalse(tree3.has_filename('hello'))
+        self.assertEquals(tree3.get_file_text('buongia-id'), 'new text')
+
+    def test_commit_rename(self):
+        """Test commit of a revision where a file is renamed."""
+        tree = self.make_branch_and_tree('.')
+        b = tree.branch
+        self.build_tree(['hello'], line_endings='binary')
+        tree.add(['hello'], ['hello-id'])
+        tree.commit(message='one', rev_id='test@rev-1', allow_pointless=False)
+
+        tree.rename_one('hello', 'fruity')
+        tree.commit(message='renamed', rev_id='test@rev-2', allow_pointless=False)
+
+        eq = self.assertEquals
+        tree1 = b.repository.revision_tree('test@rev-1')
+        eq(tree1.id2path('hello-id'), 'hello')
+        eq(tree1.get_file_text('hello-id'), 'contents of hello\n')
+        self.assertFalse(tree1.has_filename('fruity'))
+        self.check_inventory_shape(tree1.inventory, ['hello'])
+        ie = tree1.inventory['hello-id']
+        eq(ie.revision, 'test@rev-1')
+
+        tree2 = b.repository.revision_tree('test@rev-2')
+        eq(tree2.id2path('hello-id'), 'fruity')
+        eq(tree2.get_file_text('hello-id'), 'contents of hello\n')
+        self.check_inventory_shape(tree2.inventory, ['fruity'])
+        ie = tree2.inventory['hello-id']
+        eq(ie.revision, 'test@rev-2')
+
+    def test_reused_rev_id(self):
+        """Test that a revision id cannot be reused in a branch"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        wt.commit('initial', rev_id='test@rev-1', allow_pointless=True)
+        self.assertRaises(Exception,
+                          wt.commit,
+                          message='reused id',
+                          rev_id='test@rev-1',
+                          allow_pointless=True)
+
+    def test_commit_move(self):
+        """Test commit of revisions with moved files and directories"""
+        eq = self.assertEquals
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        r1 = 'test@rev-1'
+        self.build_tree(['hello', 'a/', 'b/'])
+        wt.add(['hello', 'a', 'b'], ['hello-id', 'a-id', 'b-id'])
+        wt.commit('initial', rev_id=r1, allow_pointless=False)
+        wt.move(['hello'], 'a')
+        r2 = 'test@rev-2'
+        wt.commit('two', rev_id=r2, allow_pointless=False)
+        self.check_inventory_shape(wt.read_working_inventory(),
+                                   ['a', 'a/hello', 'b'])
+
+        wt.move(['b'], 'a')
+        r3 = 'test@rev-3'
+        wt.commit('three', rev_id=r3, allow_pointless=False)
+        self.check_inventory_shape(wt.read_working_inventory(),
+                                   ['a', 'a/hello', 'a/b'])
+        self.check_inventory_shape(b.repository.get_revision_inventory(r3),
+                                   ['a', 'a/hello', 'a/b'])
+
+        wt.move(['a/hello'], 'a/b')
+        r4 = 'test@rev-4'
+        wt.commit('four', rev_id=r4, allow_pointless=False)
+        self.check_inventory_shape(wt.read_working_inventory(),
+                                   ['a', 'a/b/hello', 'a/b'])
+
+        inv = b.repository.get_revision_inventory(r4)
+        eq(inv['hello-id'].revision, r4)
+        eq(inv['a-id'].revision, r1)
+        eq(inv['b-id'].revision, r3)
+        
+    def test_removed_commit(self):
+        """Commit with a removed file"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add(['hello'], ['hello-id'])
+        wt.commit(message='add hello')
+        wt.remove('hello')
+        wt.commit('removed hello', rev_id='rev2')
+
+        tree = b.repository.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+    def test_committed_ancestry(self):
+        """Test commit appends revisions to ancestry."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        rev_ids = []
+        for i in range(4):
+            file('hello', 'w').write((str(i) * 4) + '\n')
+            if i == 0:
+                wt.add(['hello'], ['hello-id'])
+            rev_id = 'test@rev-%d' % (i+1)
+            rev_ids.append(rev_id)
+            wt.commit(message='rev %d' % (i+1),
+                     rev_id=rev_id)
+        eq = self.assertEquals
+        eq(b.revision_history(), rev_ids)
+        for i in range(4):
+            anc = b.repository.get_ancestry(rev_ids[i])
+            eq(anc, [None] + rev_ids[:i+1])
+
+    def test_commit_new_subdir_child_selective(self):
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.build_tree(['dir/', 'dir/file1', 'dir/file2'])
+        wt.add(['dir', 'dir/file1', 'dir/file2'],
+              ['dirid', 'file1id', 'file2id'])
+        wt.commit('dir/file1', specific_files=['dir/file1'], rev_id='1')
+        inv = b.repository.get_inventory('1')
+        self.assertEqual('1', inv['dirid'].revision)
+        self.assertEqual('1', inv['file1id'].revision)
+        # FIXME: This should raise a KeyError I think, rbc20051006
+        self.assertRaises(BzrError, inv.__getitem__, 'file2id')
+
+    def test_strict_commit(self):
+        """Try and commit with unknown files and strict = True, should fail."""
+        from bzrlib.errors import StrictCommitFailed
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        self.assertRaises(StrictCommitFailed, wt.commit,
+            message='add hello but not goodbye', strict=True)
+
+    def test_strict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = True,
+        should work."""
+        from bzrlib.errors import StrictCommitFailed
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        wt.commit(message='add hello', strict=True)
+
+    def test_nonstrict_commit(self):
+        """Try and commit with unknown files and strict = False, should work."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        wt.commit(message='add hello but not goodbye', strict=False)
+
+    def test_nonstrict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = False,
+        should work."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        wt.commit(message='add hello', strict=False)
+
+    def test_signed_commit(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.repository.has_signature_for_revision_id('A'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            commit.Commit(config=MustSignConfig(branch)).commit(message="base",
+                                                      allow_pointless=True,
+                                                      rev_id='B',
+                                                      working_tree=wt)
+            self.assertEqual(Testament.from_revision(branch.repository,
+                             'B').as_short_text(),
+                             branch.repository.get_signature_text('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_failed_signature(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.repository.has_signature_for_revision_id('A'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
+            config = MustSignConfig(branch)
+            self.assertRaises(SigningFailed,
+                              commit.Commit(config=config).commit,
+                              message="base",
+                              allow_pointless=True,
+                              rev_id='B',
+                              working_tree=wt)
+            branch = Branch.open(self.get_url('.'))
+            self.assertEqual(branch.revision_history(), ['A'])
+            self.failIf(branch.repository.has_revision('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_invokes_hooks(self):
+        import bzrlib.commit as commit
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        calls = []
+        def called(branch, rev_id):
+            calls.append('called')
+        bzrlib.ahook = called
+        try:
+            config = BranchWithHooks(branch)
+            commit.Commit(config=config).commit(
+                            message = "base",
+                            allow_pointless=True,
+                            rev_id='A', working_tree = wt)
+            self.assertEqual(['called', 'called'], calls)
+        finally:
+            del bzrlib.ahook
+
+    def test_commit_object_doesnt_set_nick(self):
+        # using the Commit object directly does not set the branch nick.
+        wt = self.make_branch_and_tree('.')
+        c = Commit()
+        c.commit(working_tree=wt, message='empty tree', allow_pointless=True)
+        self.assertEquals(wt.branch.revno(), 1)
+        self.assertEqual({},
+                         wt.branch.repository.get_revision(
+                            wt.branch.last_revision()).properties)
+
+    def test_safe_master_lock(self):
+        os.mkdir('master')
+        master = BzrDirMetaFormat1().initialize('master')
+        master.create_repository()
+        master_branch = master.create_branch()
+        master.create_workingtree()
+        bound = master.sprout('bound')
+        wt = bound.open_workingtree()
+        wt.branch.set_bound_location(os.path.realpath('master'))
+        master_branch.lock_write()
+        try:
+            self.assertRaises(LockContention, wt.commit, 'silly')
+        finally:
+            master_branch.unlock()

=== added file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_commit_merge.py	2006-02-20 09:50:27 +0000
@@ -0,0 +1,97 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+import shutil
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.errors import PointlessCommit, BzrError, PointlessCommit
+from bzrlib.tests.test_revision import make_branches
+from bzrlib.check import check
+
+
+class TestCommitMerge(TestCaseWithTransport):
+    """Tests for committing the results of a merge.
+
+    These don't currently test the merge code, which is intentional to
+    reduce the scope of testing.  We just mark the revision as merged
+    without bothering about the contents much."""
+
+    def test_merge_commit_empty(self):
+        """Simple commit of two-way merge of empty trees."""
+        wtx = self.make_branch_and_tree('x')
+        bx = wtx.branch
+        wty = self.make_branch_and_tree('y')
+        by = wty.branch
+
+        wtx.commit('commit one', rev_id='x@u-0-1', allow_pointless=True)
+        wty.commit('commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        self.assertEqual((1, []), by.fetch(bx))
+        # just having the history there does nothing
+        self.assertRaises(PointlessCommit,
+                          wty.commit,
+                          'no changes yet', rev_id='y@u-0-2',
+                          allow_pointless=False)
+        wty.add_pending_merge('x@u-0-1')
+        wty.commit('merge from x', rev_id='y@u-0-2', allow_pointless=False)
+
+        self.assertEquals(by.revno(), 2)
+        self.assertEquals(list(by.revision_history()),
+                          ['y@u-0-1', 'y@u-0-2'])
+        rev = by.repository.get_revision('y@u-0-2')
+        self.assertEquals(rev.parent_ids,
+                          ['y@u-0-1', 'x@u-0-1'])
+
+    def test_merge_new_file(self):
+        """Commit merge of two trees with no overlapping files."""
+        wtx = self.make_branch_and_tree('x')
+        bx = wtx.branch
+        wty = self.make_branch_and_tree('y')
+        by = wty.branch
+
+        self.build_tree(['x/ecks', 'y/why'])
+
+        wtx.add(['ecks'], ['ecks-id'])
+        wty.add(['why'], ['why-id'])
+
+        wtx.commit('commit one', rev_id='x@u-0-1', allow_pointless=True)
+        wty.commit('commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        by.fetch(bx)
+        # we haven't merged the texts, but let's fake it
+        shutil.copyfile('x/ecks', 'y/ecks')
+        wty.add(['ecks'], ['ecks-id'])
+        wty.add_pending_merge('x@u-0-1')
+
+        # partial commit of merges is currently not allowed, because
+        # it would give different merge graphs for each file which
+        # might be complex.  it can be allowed in the future.
+        self.assertRaises(Exception,
+                          wty.commit,
+                          'partial commit', allow_pointless=False,
+                          specific_files=['ecks'])
+        
+        wty.commit('merge from x', rev_id='y@u-0-2', allow_pointless=False)
+        tree = by.repository.revision_tree('y@u-0-2')
+        inv = tree.inventory
+        self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
+        self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
+
+        check(bx, False)
+        check(by, False)

=== added file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_config.py	2006-03-31 00:43:21 +0000
@@ -0,0 +1,659 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for finding and reading the bzr config file[s]."""
+# import system imports here
+from bzrlib.util.configobj.configobj import ConfigObj, ConfigObjError
+from cStringIO import StringIO
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.config as config
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+sample_long_alias="log -r-15..-1 --line"
+sample_config_text = ("[DEFAULT]\n"
+                      u"email=Erik B\u00e5gfors <erik@bagfors.nu>\n"
+                      "editor=vim\n"
+                      "gpg_signing_command=gnome-gpg\n"
+                      "log_format=short\n"
+                      "user_global_option=something\n"
+                      "[ALIASES]\n"
+                      "h=help\n"
+                      "ll=" + sample_long_alias + "\n")
+
+
+sample_always_signatures = ("[DEFAULT]\n"
+                            "check_signatures=require\n")
+
+
+sample_ignore_signatures = ("[DEFAULT]\n"
+                            "check_signatures=ignore\n")
+
+
+sample_maybe_signatures = ("[DEFAULT]\n"
+                            "check_signatures=check-available\n")
+
+
+sample_branches_text = ("[http://www.example.com]\n"
+                        "# Top level policy\n"
+                        "email=Robert Collins <robertc@example.org>\n"
+                        "[http://www.example.com/useglobal]\n"
+                        "# different project, forces global lookup\n"
+                        "recurse=false\n"
+                        "[/b/]\n"
+                        "check_signatures=require\n"
+                        "# test trailing / matching with no children\n"
+                        "[/a/]\n"
+                        "check_signatures=check-available\n"
+                        "gpg_signing_command=false\n"
+                        "user_local_option=local\n"
+                        "# test trailing / matching\n"
+                        "[/a/*]\n"
+                        "#subdirs will match but not the parent\n"
+                        "recurse=False\n"
+                        "[/a/c]\n"
+                        "check_signatures=ignore\n"
+                        "post_commit=bzrlib.tests.test_config.post_commit\n"
+                        "#testing explicit beats globs\n")
+
+
+
+class InstrumentedConfigObj(object):
+    """A config obj look-enough-alike to record calls made to it."""
+
+    def __contains__(self, thing):
+        self._calls.append(('__contains__', thing))
+        return False
+
+    def __getitem__(self, key):
+        self._calls.append(('__getitem__', key))
+        return self
+
+    def __init__(self, input, encoding=None):
+        self._calls = [('__init__', input, encoding)]
+
+    def __setitem__(self, key, value):
+        self._calls.append(('__setitem__', key, value))
+
+    def write(self, arg):
+        self._calls.append(('write',))
+
+
+class FakeBranch(object):
+
+    def __init__(self):
+        self.base = "http://example.com/branches/demo"
+        self.control_files = FakeControlFiles()
+
+
+class FakeControlFiles(object):
+
+    def __init__(self):
+        self.email = 'Robert Collins <robertc@example.net>\n'
+
+    def get_utf8(self, filename):
+        if filename != 'email':
+            raise NotImplementedError
+        if self.email is not None:
+            return StringIO(self.email)
+        raise errors.NoSuchFile(filename)
+
+
+class InstrumentedConfig(config.Config):
+    """An instrumented config that supplies stubs for template methods."""
+    
+    def __init__(self):
+        super(InstrumentedConfig, self).__init__()
+        self._calls = []
+        self._signatures = config.CHECK_NEVER
+
+    def _get_user_id(self):
+        self._calls.append('_get_user_id')
+        return "Robert Collins <robert.collins@example.org>"
+
+    def _get_signature_checking(self):
+        self._calls.append('_get_signature_checking')
+        return self._signatures
+
+
+bool_config = """[DEFAULT]
+active = true
+inactive = false
+[UPPERCASE]
+active = True
+nonactive = False
+"""
+class TestConfigObj(TestCase):
+    def test_get_bool(self):
+        from bzrlib.config import ConfigObj
+        co = ConfigObj(StringIO(bool_config))
+        self.assertIs(co.get_bool('DEFAULT', 'active'), True)
+        self.assertIs(co.get_bool('DEFAULT', 'inactive'), False)
+        self.assertIs(co.get_bool('UPPERCASE', 'active'), True)
+        self.assertIs(co.get_bool('UPPERCASE', 'nonactive'), False)
+
+
+class TestConfig(TestCase):
+
+    def test_constructs(self):
+        config.Config()
+ 
+    def test_no_default_editor(self):
+        self.assertRaises(NotImplementedError, config.Config().get_editor)
+
+    def test_user_email(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('robert.collins@example.org', my_config.user_email())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_username(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('Robert Collins <robert.collins@example.org>',
+                         my_config.username())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_signatures_default(self):
+        my_config = config.Config()
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+
+    def test_signatures_template_method(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_signatures_template_method_none(self):
+        my_config = InstrumentedConfig()
+        my_config._signatures = None
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_gpg_signing_command_default(self):
+        my_config = config.Config()
+        self.assertEqual('gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_post_commit_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.post_commit())
+
+    def test_log_format_default(self):
+        my_config = config.Config()
+        self.assertEqual('long', my_config.log_format())
+
+
+class TestConfigPath(TestCase):
+
+    def setUp(self):
+        super(TestConfigPath, self).setUp()
+        self.old_home = os.environ.get('HOME', None)
+        self.old_appdata = os.environ.get('APPDATA', None)
+        os.environ['HOME'] = '/home/bogus'
+        os.environ['APPDATA'] = \
+            r'C:\Documents and Settings\bogus\Application Data'
+
+    def tearDown(self):
+        if self.old_home is None:
+            del os.environ['HOME']
+        else:
+            os.environ['HOME'] = self.old_home
+        if self.old_appdata is None:
+            del os.environ['APPDATA']
+        else:
+            os.environ['APPDATA'] = self.old_appdata
+        super(TestConfigPath, self).tearDown()
+    
+    def test_config_dir(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.config_dir(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0')
+        else:
+            self.assertEqual(config.config_dir(), '/home/bogus/.bazaar')
+
+    def test_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/bazaar.conf')
+        else:
+            self.assertEqual(config.config_filename(),
+                             '/home/bogus/.bazaar/bazaar.conf')
+
+    def test_branches_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.branches_config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/branches.conf')
+        else:
+            self.assertEqual(config.branches_config_filename(),
+                             '/home/bogus/.bazaar/branches.conf')
+
+class TestIniConfig(TestCase):
+
+    def test_contructs(self):
+        my_config = config.IniBasedConfig("nothing")
+
+    def test_from_fp(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        my_config = config.IniBasedConfig(None)
+        self.failUnless(
+            isinstance(my_config._get_parser(file=config_file),
+                        ConfigObj))
+
+    def test_cached(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        my_config = config.IniBasedConfig(None)
+        parser = my_config._get_parser(file=config_file)
+        self.failUnless(my_config._get_parser() is parser)
+
+
+class TestGetConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.GlobalConfig()
+
+    def test_calls_read_filenames(self):
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.GlobalConfig()
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls, [('__init__', config.config_filename(),
+                                          'utf-8')])
+
+
+class TestBranchConfig(TestCaseInTempDir):
+
+    def test_constructs(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertRaises(TypeError, config.BranchConfig)
+
+    def test_get_location_config(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        location_config = my_config._get_location_config()
+        self.assertEqual(branch.base, location_config.location)
+        self.failUnless(location_config is my_config._get_location_config())
+
+
+class TestGlobalConfigItems(TestCase):
+
+    def test_user_id(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
+                         my_config._get_user_id())
+
+    def test_absent_user_id(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(None, my_config._get_user_id())
+
+    def test_configured_editor(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("vim", my_config.get_editor())
+
+    def test_signatures_always(self):
+        config_file = StringIO(sample_always_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_ALWAYS,
+                         my_config.signature_checking())
+        self.assertEqual(True, my_config.signature_needed())
+
+    def test_signatures_if_possible(self):
+        config_file = StringIO(sample_maybe_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def test_signatures_ignore(self):
+        config_file = StringIO(sample_ignore_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_NEVER,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_sample_config(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_empty_config(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command_unset(self):
+        my_config = self._get_empty_config()
+        self.assertEqual("gpg", my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = self._get_empty_config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_get_user_option_global(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("something",
+                         my_config.get_user_option('user_global_option'))
+        
+    def test_post_commit_default(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(None, my_config.post_commit())
+
+    def test_configured_logformat(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("short", my_config.log_format())
+
+    def test_get_alias(self):
+        my_config = self._get_sample_config()
+        self.assertEqual('help', my_config.get_alias('h'))
+
+    def test_get_no_alias(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(None, my_config.get_alias('foo'))
+
+    def test_get_long_alias(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(sample_long_alias, my_config.get_alias('ll'))
+
+class TestLocationConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.LocationConfig('http://example.com')
+        self.assertRaises(TypeError, config.LocationConfig)
+
+    def test_branch_calls_read_filenames(self):
+        # This is testing the correct file names are provided.
+        # TODO: consolidate with the test for GlobalConfigs filename checks.
+        #
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.LocationConfig('http://www.example.com')
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls,
+                         [('__init__', config.branches_config_filename())])
+
+    def test_get_global_config(self):
+        my_config = config.LocationConfig('http://example.com')
+        global_config = my_config._get_global_config()
+        self.failUnless(isinstance(global_config, config.GlobalConfig))
+        self.failUnless(global_config is my_config._get_global_config())
+
+    def test__get_section_no_match(self):
+        self.get_location_config('/')
+        self.assertEqual(None, self.my_config._get_section())
+        
+    def test__get_section_exact(self):
+        self.get_location_config('http://www.example.com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+   
+    def test__get_section_suffix_does_not(self):
+        self.get_location_config('http://www.example.com-com')
+        self.assertEqual(None, self.my_config._get_section())
+
+    def test__get_section_subdir_recursive(self):
+        self.get_location_config('http://www.example.com/com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_matches(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('http://www.example.com/useglobal',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_nonrecursive(self):
+        self.get_location_config(
+            'http://www.example.com/useglobal/childbranch')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_trailing_slash(self):
+        self.get_location_config('/b')
+        self.assertEqual('/b/', self.my_config._get_section())
+
+    def test__get_section_subdir_child(self):
+        self.get_location_config('/a/foo')
+        self.assertEqual('/a/*', self.my_config._get_section())
+
+    def test__get_section_subdir_child_child(self):
+        self.get_location_config('/a/foo/bar')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_trailing_slash_with_children(self):
+        self.get_location_config('/a/')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_explicit_over_glob(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('/a/c', self.my_config._get_section())
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text)
+        else:
+            global_file = StringIO(global_config)
+        branches_file = StringIO(sample_branches_text)
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_location_without_username(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_location_not_listed(self):
+        self.get_location_config('/home/robertc/sources')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_overriding_location(self):
+        self.get_location_config('http://www.example.com/foo')
+        self.assertEqual('Robert Collins <robertc@example.org>',
+                         self.my_config.username())
+
+    def test_signatures_not_set(self):
+        self.get_location_config('http://www.example.com',
+                                 global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+
+    def test_signatures_never(self):
+        self.get_location_config('/a/c')
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_when_available(self):
+        self.get_location_config('/a/', global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_always(self):
+        self.get_location_config('/b')
+        self.assertEqual(config.CHECK_ALWAYS,
+                         self.my_config.signature_checking())
+        
+    def test_gpg_signing_command(self):
+        self.get_location_config('/b')
+        self.assertEqual("gnome-gpg", self.my_config.gpg_signing_command())
+
+    def test_gpg_signing_command_missing(self):
+        self.get_location_config('/a')
+        self.assertEqual("false", self.my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        self.get_location_config('/a')
+        self.assertEqual('something',
+                         self.my_config.get_user_option('user_global_option'))
+
+    def test_get_user_option_local(self):
+        self.get_location_config('/a')
+        self.assertEqual('local',
+                         self.my_config.get_user_option('user_local_option'))
+        
+    def test_post_commit_default(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('bzrlib.tests.test_config.post_commit',
+                         self.my_config.post_commit())
+
+
+class TestLocationConfig(TestCaseInTempDir):
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text.encode('utf-8'))
+        else:
+            global_file = StringIO(global_config.encode('utf-8'))
+        branches_file = StringIO(sample_branches_text.encode('utf-8'))
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_set_user_setting_sets_and_saves(self):
+        self.get_location_config('/a/c')
+        record = InstrumentedConfigObj("foo")
+        self.my_config._parser = record
+
+        real_mkdir = os.mkdir
+        self.created = False
+        def checked_mkdir(path, mode=0777):
+            self.log('making directory: %s', path)
+            real_mkdir(path, mode)
+            self.created = True
+
+        os.mkdir = checked_mkdir
+        try:
+            self.my_config.set_user_option('foo', 'bar')
+        finally:
+            os.mkdir = real_mkdir
+
+        self.failUnless(self.created, 'Failed to create ~/.bazaar')
+        self.assertEqual([('__contains__', '/a/c'),
+                          ('__contains__', '/a/c/'),
+                          ('__setitem__', '/a/c', {}),
+                          ('__getitem__', '/a/c'),
+                          ('__setitem__', 'foo', 'bar'),
+                          ('write',)],
+                         record._calls[1:])
+
+
+class TestBranchConfigItems(TestCase):
+
+    def test_user_id(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.net>",
+                         my_config._get_user_id())
+        branch.control_files.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_not_set_in_branch(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        branch.control_files.email = None
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
+                         my_config._get_user_id())
+        branch.control_files.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_BZREMAIL_OVERRIDES(self):
+        os.environ['BZREMAIL'] = "Robert Collins <robertc@example.org>"
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.org>",
+                         my_config.username())
+    
+    def test_signatures_forced(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_always_signatures)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())
+
+    def test_gpg_signing_command(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('something',
+                         my_config.get_user_option('user_global_option'))
+
+    def test_post_commit_default(self):
+        branch = FakeBranch()
+        branch.base='/a/c'
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        branch_file = StringIO(sample_branches_text)
+        my_config._get_location_config()._get_parser(branch_file)
+        self.assertEqual('bzrlib.tests.test_config.post_commit',
+                         my_config.post_commit())
+
+
+class TestMailAddressExtraction(TestCase):
+
+    def test_extract_email_address(self):
+        self.assertEqual('jane@test.com',
+                         config.extract_email_address('Jane <jane@test.com>'))
+        self.assertRaises(errors.BzrError,
+                          config.extract_email_address, 'Jane Tester')

=== added file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_conflicts.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,102 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport, TestCase
+from bzrlib.branch import Branch
+from bzrlib.conflicts import *
+from bzrlib.errors import NotConflicted
+
+
+# TODO: Test commit with some added, and added-but-missing files
+# RBC 20060124 is that not tested in test_commit.py ?
+
+# The order of 'path' here is important - do not let it
+# be a sorted list.
+example_conflicts = ConflictList([ 
+    MissingParent('Not deleting', 'pathg', 'idg'),
+    ContentsConflict('patha', 'ida'), 
+    TextConflict('patha'),
+    PathConflict('pathb', 'pathc', 'idb'),
+    DuplicateID('Unversioned existing file', 'pathc', 'pathc2', 'idc', 'idc'),
+    DuplicateEntry('Moved existing file to',  'pathdd.moved', 'pathd', 'idd', 
+                   None),
+    ParentLoop('Cancelled move', 'pathe', 'path2e', None, 'id2e'),
+    UnversionedParent('Versioned directory', 'pathf', 'idf'),
+])
+
+
+class TestConflicts(TestCaseWithTransport):
+
+    def test_conflicts(self):
+        """Conflicts are detected properly"""
+        tree = self.make_branch_and_tree('.',
+            format=bzrlib.bzrdir.BzrDirFormat6())
+        b = tree.branch
+        file('hello', 'w').write('hello world4')
+        file('hello.THIS', 'w').write('hello world2')
+        file('hello.BASE', 'w').write('hello world1')
+        file('hello.OTHER', 'w').write('hello world3')
+        file('hello.sploo.BASE', 'w').write('yellow world')
+        file('hello.sploo.OTHER', 'w').write('yellow world2')
+        self.assertEqual(len(list(tree.list_files())), 6)
+        conflicts = tree.conflicts()
+        self.assertEqual(len(conflicts), 2)
+        self.assert_('hello' in conflicts[0].path)
+        self.assert_('hello.sploo' in conflicts[1].path)
+        restore('hello')
+        restore('hello.sploo')
+        self.assertEqual(len(tree.conflicts()), 0)
+        self.assertFileEqual('hello world2', 'hello')
+        assert not os.path.lexists('hello.sploo')
+        self.assertRaises(NotConflicted, restore, 'hello')
+        self.assertRaises(NotConflicted, restore, 'hello.sploo')
+
+    def test_resolve_conflict_dir(self):
+        tree = self.make_branch_and_tree('.')
+        b = tree.branch
+        file('hello', 'w').write('hello world4')
+        tree.add('hello', 'q')
+        file('hello.THIS', 'w').write('hello world2')
+        file('hello.BASE', 'w').write('hello world1')
+        os.mkdir('hello.OTHER')
+        l = ConflictList([TextConflict('hello')])
+        l.remove_files(tree)
+
+
+class TestConflictStanzas(TestCase):
+
+    def test_stanza_roundtrip(self):
+        # write and read our example stanza.
+        stanza_iter = example_conflicts.to_stanzas()
+        processed = ConflictList.from_stanzas(stanza_iter)
+        for o,p in zip(processed, example_conflicts):
+            self.assertEqual(o, p)
+
+    def test_stanzification(self):
+        for stanza in example_conflicts.to_stanzas():
+            try:
+                self.assertStartsWith(stanza['file_id'], 'id')
+            except KeyError:
+                pass
+            self.assertStartsWith(stanza['path'], 'path')
+            try:
+                self.assertStartsWith(stanza['conflict_file_id'], 'id')
+                self.assertStartsWith(stanza['conflict_file_path'], 'path')
+            except KeyError:
+                pass

=== added file 'bzrlib/tests/test_decorators.py'
--- bzrlib/tests/test_decorators.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_decorators.py	2006-01-13 06:38:56 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for decorator functions"""
+
+from bzrlib.decorators import needs_read_lock
+from bzrlib.tests import TestCase
+
+
+class DecoratorSample(object):
+    """Sample class that uses decorators.
+
+    This doesn't actually work because the class doesn't have the 
+    requiste lock_read()/unlock() methods.
+    """
+
+    @needs_read_lock
+    def frob(self):
+        """Frob the sample object"""
+
+
+class TestDecoratorDocs(TestCase):
+    """Test method decorators"""
+
+    def test_read_lock_passthrough(self):
+        """@needs_read_lock exposes underlying name and doc."""
+        sam = DecoratorSample()
+        self.assertEqual(sam.frob.__name__, 'frob')
+        self.assertEqual(sam.frob.__doc__, 
+                'Frob the sample object')

=== renamed file 'bzrlib/selftest/testdiff.py' => 'bzrlib/tests/test_diff.py'
--- bzrlib/selftest/testdiff.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/test_diff.py	2005-11-23 15:44:24 +0000
@@ -1,4 +1,4 @@
-from bzrlib.selftest import TestCase
+from bzrlib.tests import TestCase
 from bzrlib.diff import internal_diff
 from cStringIO import StringIO
 def udiff_lines(old, new):
@@ -7,55 +7,43 @@
     output.seek(0, 0)
     return output.readlines()
 
-def check_patch(lines):
-    assert len(lines) > 1, \
-        "Not enough lines for a file header for patch:\n%s" % "".join(lines)
-    assert lines[0].startswith ('---'), \
-        'No orig line for patch:\n%s' % "".join(lines)
-    assert lines[1].startswith ('+++'), \
-        'No mod line for patch:\n%s' % "".join(lines)
-    assert len(lines) > 2, \
-        "No hunks for patch:\n%s" % "".join(lines)
-    assert lines[2].startswith('@@'),\
-        "No hunk header for patch:\n%s" % "".join(lines)
-    assert '@@' in lines[2][2:], \
-        "Unterminated hunk header for patch:\n%s" % "".join(lines)
-
-class AddNL(TestCase):
-    """
-    diff generates a valid diff for patches that add a newline
-    """
-    def runTest(self):
+class TestDiff(TestCase):
+    def test_add_nl(self):
+        """diff generates a valid diff for patches that add a newline"""
         lines = udiff_lines(['boo'], ['boo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
-
-
-class AddNL2(TestCase):
-    """
-    diff generates a valid diff for patches that change last line and add a
-    newline
-    """
-    def runTest(self):
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
+
+    def test_add_nl_2(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
         lines = udiff_lines(['boo'], ['goo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
 
-class RemoveNL(TestCase):
-    """
-    diff generates a valid diff for patches that change last line and add a
-    newline
-    """
-    def runTest(self):
+    def test_remove_nl(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
         lines = udiff_lines(['boo\n'], ['boo'])
-        check_patch(lines)
-        assert lines[5] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[5]
-
-TEST_CLASSES = [
-    AddNL, 
-    AddNL2, 
-    RemoveNL,
-]
+        self.check_patch(lines)
+        self.assertEquals(lines[5], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[5]
+
+    def check_patch(self, lines):
+        self.assert_(len(lines) > 1)
+            ## "Not enough lines for a file header for patch:\n%s" % "".join(lines)
+        self.assert_(lines[0].startswith ('---'))
+            ## 'No orig line for patch:\n%s' % "".join(lines)
+        self.assert_(lines[1].startswith ('+++'))
+            ## 'No mod line for patch:\n%s' % "".join(lines)
+        self.assert_(len(lines) > 2)
+            ## "No hunks for patch:\n%s" % "".join(lines)
+        self.assert_(lines[2].startswith('@@'))
+            ## "No hunk header for patch:\n%s" % "".join(lines)
+        self.assert_('@@' in lines[2][2:])
+            ## "Unterminated hunk header for patch:\n%s" % "".join(lines)
+

=== added file 'bzrlib/tests/test_doc_generate.py'
--- bzrlib/tests/test_doc_generate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_doc_generate.py	2006-03-16 14:55:45 +0000
@@ -0,0 +1,30 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.branch import Branch
+
+# TODO: perhaps doc_generate should be moved into bzrlib instead?
+
+class TestDocGenerate(TestCaseInTempDir):
+
+    def test_generate_manpage(self):
+        """Simple smoke test for doc_generate"""
+        try:
+            import tools.doc_generate
+        except ImportError, e:
+            raise TestSkipped("can't load doc_generate: %s" % e)
+        infogen_mod = tools.doc_generate.get_module("man")

=== added file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_errors.py	2006-02-26 07:54:02 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the formatting and construction of errors."""
+
+import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestErrors(TestCaseWithTransport):
+
+    def test_no_repo(self):
+        dir = bzrdir.BzrDir.create(self.get_url())
+        error = errors.NoRepositoryPresent(dir)
+        self.assertNotEqual(-1, str(error).find(repr(dir.transport.clone('..').base)))
+        self.assertEqual(-1, str(error).find(repr(dir.transport.base)))
+
+    def test_up_to_date(self):
+        error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())
+        self.assertEqualDiff("The branch format Bazaar-NG branch, "
+                             "format 0.0.4 is already at the most "
+                             "recent format.",
+                             str(error))
+
+    def test_corrupt_repository(self):
+        repo = self.make_repository('.')
+        error = errors.CorruptRepository(repo)
+        self.assertEqualDiff("An error has been detected in the repository %s.\n"
+                             "Please run bzr reconcile on this repository." %
+                             repo.bzrdir.root_transport.base,
+                             str(error))

=== added file 'bzrlib/tests/test_escaped_store.py'
--- bzrlib/tests/test_escaped_store.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_escaped_store.py	2006-04-18 06:49:41 +0000
@@ -0,0 +1,112 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test Escaped Stores."""
+
+from cStringIO import StringIO
+import os
+import gzip
+
+from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
+from bzrlib.store import copy_all
+from bzrlib.store.text import TextStore
+from bzrlib.tests import TestCaseWithTransport
+import bzrlib.transport
+
+
+class TestEscaped(TestCaseWithTransport):
+    """Mixin template class that provides some common tests for stores"""
+
+    def get_store(self, prefixed=False, escaped=True):
+        t = bzrlib.transport.get_transport(self.get_url())
+        return TextStore(t, prefixed=prefixed, escaped=escaped)
+
+    def test_paths(self):
+        text_store = self.get_store()
+
+        self.assertEqual('a', text_store._relpath('a'))
+        self.assertEqual('a', text_store._relpath(u'a'))
+        self.assertEqual('%2520', text_store._relpath(' '))
+        self.assertEqual('%40%253a%253c%253e', text_store._relpath('@:<>'))
+        self.assertEqual('%25c3%25a5', text_store._relpath(u'\xe5'))
+
+    def test_prefixed(self):
+        # Prefix should be determined by unescaped string
+        text_store = self.get_store(prefixed=True)
+
+        # hash_prefix() is not defined for unicode characters
+        # it is only defined for byte streams.
+        # so hash_prefix() needs to operate on *at most* utf-8
+        # encoded. However urlescape() does both encoding to utf-8
+        # and urllib quoting, so we will use the escaped form
+        # as the path passed to hash_prefix
+
+        self.assertEqual('62/a', text_store._relpath('a'))
+        self.assertEqual('88/%2520', text_store._relpath(' '))
+        self.assertEqual('72/%40%253a%253c%253e',
+                text_store._relpath('@:<>'))
+        self.assertEqual('77/%25c3%25a5', text_store._relpath(u'\xe5'))
+
+    def test_files(self):
+        text_store = self.get_store(prefixed=True)
+
+        text_store.add(StringIO('a'), 'a')
+        self.failUnlessExists('62/a')
+
+        text_store.add(StringIO('space'), ' ')
+        self.failUnlessExists('88/%20')
+        self.assertEquals('space', text_store.get(' ').read())
+
+        text_store.add(StringIO('surprise'), '@:<>')
+        self.failUnlessExists('72/@%3a%3c%3e')
+        self.assertEquals('surprise', text_store.get('@:<>').read())
+
+        text_store.add(StringIO('unicode'), u'\xe5')
+        self.failUnlessExists('77/%c3%a5')
+        self.assertEquals('unicode', text_store.get(u'\xe5').read())
+
+    def test_weave(self):
+        from bzrlib.store.versioned import WeaveStore
+        from bzrlib.transactions import PassThroughTransaction
+
+        trans = PassThroughTransaction()
+
+        t = bzrlib.transport.get_transport(self.get_url())
+        weave_store = WeaveStore(t, prefixed=True, escaped=True)
+        def add_text(file_id, rev_id, contents, parents, transaction):
+            vfile = weave_store.get_weave_or_empty(file_id, transaction)
+            vfile.add_lines(rev_id, parents, contents)
+
+        def check_text(file_id, revision_id, contents):
+            vfile = weave_store.get_weave(file_id, trans)
+            self.assertEqual(contents, vfile.get_lines(revision_id))
+
+        add_text('a', 'r', ['a'], [], trans)
+        self.failUnlessExists('62/a.weave')
+        check_text('a', 'r', ['a'])
+
+        add_text(' ', 'r', ['space'], [], trans)
+        self.failIfExists('21/ .weave')
+        self.failUnlessExists('88/%20.weave')
+        check_text(' ', 'r', ['space'])
+
+        add_text('@:<>', 'r', ['surprise'], [], trans)
+        self.failUnlessExists('72/@%3a%3c%3e.weave')
+        check_text('@:<>', 'r', ['surprise'])
+
+        add_text(u'\xe5', 'r', ['unicode'], [], trans)
+        self.failUnlessExists('77/%c3%a5.weave')
+        check_text(u'\xe5', 'r', ['unicode'])

=== added file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_fetch.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,247 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import sys
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.builtins import merge
+import bzrlib.errors
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests.test_revision import make_branches
+from bzrlib.trace import mutter
+from bzrlib.workingtree import WorkingTree
+
+
+def has_revision(branch, revision_id):
+    return branch.repository.has_revision(revision_id)
+
+def fetch_steps(self, br_a, br_b, writable_a):
+    """A foreign test method for testing fetch locally and remotely."""
+     
+    # TODO RBC 20060201 make this a repository test.
+    repo_b = br_b.repository
+    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
+    self.assertTrue(repo_b.has_revision(br_a.revision_history()[2]))
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertEquals(br_b.fetch(br_a, br_a.revision_history()[2])[0], 0)
+    # branch.fetch is not supposed to alter the revision history
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
+
+    # fetching the next revision up in sample data copies one revision
+    self.assertEquals(br_b.fetch(br_a, br_a.revision_history()[3])[0], 1)
+    self.assertTrue(repo_b.has_revision(br_a.revision_history()[3]))
+    self.assertFalse(has_revision(br_a, br_b.revision_history()[6]))
+    self.assertTrue(br_a.repository.has_revision(br_b.revision_history()[5]))
+
+    # When a non-branch ancestor is missing, it should be unlisted...
+    # as its not reference from the inventory weave.
+    br_b4 = self.make_branch('br_4')
+    count, failures = br_b4.fetch(br_b)
+    self.assertEqual(count, 7)
+    self.assertEqual(failures, [])
+
+    self.assertEqual(writable_a.fetch(br_b)[0], 1)
+    self.assertTrue(has_revision(br_a, br_b.revision_history()[3]))
+    self.assertTrue(has_revision(br_a, br_b.revision_history()[4]))
+        
+    br_b2 = self.make_branch('br_b2')
+    self.assertEquals(br_b2.fetch(br_b)[0], 7)
+    self.assertTrue(has_revision(br_b2, br_b.revision_history()[4]))
+    self.assertTrue(has_revision(br_b2, br_a.revision_history()[2]))
+    self.assertFalse(has_revision(br_b2, br_a.revision_history()[3]))
+
+    br_a2 = self.make_branch('br_a2')
+    self.assertEquals(br_a2.fetch(br_a)[0], 9)
+    self.assertTrue(has_revision(br_a2, br_b.revision_history()[4]))
+    self.assertTrue(has_revision(br_a2, br_a.revision_history()[3]))
+    self.assertTrue(has_revision(br_a2, br_a.revision_history()[2]))
+
+    br_a3 = self.make_branch('br_a3')
+    # pulling a branch with no revisions grabs nothing, regardless of 
+    # whats in the inventory.
+    self.assertEquals(br_a3.fetch(br_a2)[0], 0)
+    for revno in range(4):
+        self.assertFalse(
+            br_a3.repository.has_revision(br_a.revision_history()[revno]))
+    self.assertEqual(br_a3.fetch(br_a2, br_a.revision_history()[2])[0], 3)
+    # pull the 3 revisions introduced by a@u-0-3
+    fetched = br_a3.fetch(br_a2, br_a.revision_history()[3])[0]
+    self.assertEquals(fetched, 3, "fetched %d instead of 3" % fetched)
+    # InstallFailed should be raised if the branch is missing the revision
+    # that was requested.
+    self.assertRaises(bzrlib.errors.InstallFailed, br_a3.fetch, br_a2, 'pizza')
+    # InstallFailed should be raised if the branch is missing a revision
+    # from its own revision history
+    br_a2.append_revision('a-b-c')
+    self.assertRaises(bzrlib.errors.InstallFailed, br_a3.fetch, br_a2)
+
+    # TODO: jam 20051218 Branch should no longer allow append_revision for revisions
+    #       which don't exist. So this test needs to be rewritten
+    #       RBC 20060403 the way to do this is to uncommit the revision from the
+    #           repository after the commit
+
+    #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
+    # Note that this means - updating the weave when ghosts are filled in to 
+    # add the right parents.
+
+
+class TestFetch(TestCaseWithTransport):
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches(self)
+        fetch_steps(self, br_a, br_b, br_a)
+
+    def test_fetch_self(self):
+        wt = self.make_branch_and_tree('br')
+        self.assertEqual(wt.branch.fetch(wt.branch), (0, []))
+
+
+class TestMergeFetch(TestCaseWithTransport):
+
+    def test_merge_fetches_unrelated(self):
+        """Merge brings across history from unrelated source"""
+        wt1 = self.make_branch_and_tree('br1')
+        br1 = wt1.branch
+        wt1.commit(message='rev 1-1', rev_id='1-1')
+        wt1.commit(message='rev 1-2', rev_id='1-2')
+        wt2 = self.make_branch_and_tree('br2')
+        br2 = wt2.branch
+        wt2.commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=['br1', 0],
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def test_merge_fetches(self):
+        """Merge brings across history from source"""
+        wt1 = self.make_branch_and_tree('br1')
+        br1 = wt1.branch
+        wt1.commit(message='rev 1-1', rev_id='1-1')
+        dir_2 = br1.bzrdir.sprout('br2')
+        br2 = dir_2.open_branch()
+        wt1.commit(message='rev 1-2', rev_id='1-2')
+        dir_2.open_workingtree().commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def _check_revs_present(self, br2):
+        for rev_id in '1-1', '1-2', '2-1':
+            self.assertTrue(br2.repository.has_revision(rev_id))
+            rev = br2.repository.get_revision(rev_id)
+            self.assertEqual(rev.revision_id, rev_id)
+            self.assertTrue(br2.repository.get_inventory(rev_id))
+
+
+class TestMergeFileHistory(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestMergeFileHistory, self).setUp()
+        wt1 = self.make_branch_and_tree('br1')
+        br1 = wt1.branch
+        self.build_tree_contents([('br1/file', 'original contents\n')])
+        wt1.add('file', 'this-file-id')
+        wt1.commit(message='rev 1-1', rev_id='1-1')
+        dir_2 = br1.bzrdir.sprout('br2')
+        br2 = dir_2.open_branch()
+        wt2 = dir_2.open_workingtree()
+        self.build_tree_contents([('br1/file', 'original from 1\n')])
+        wt1.commit(message='rev 1-2', rev_id='1-2')
+        self.build_tree_contents([('br1/file', 'agreement\n')])
+        wt1.commit(message='rev 1-3', rev_id='1-3')
+        self.build_tree_contents([('br2/file', 'contents in 2\n')])
+        wt2.commit(message='rev 2-1', rev_id='2-1')
+        self.build_tree_contents([('br2/file', 'agreement\n')])
+        wt2.commit(message='rev 2-2', rev_id='2-2')
+
+    def test_merge_fetches_file_history(self):
+        """Merge brings across file histories"""
+        br2 = Branch.open('br2')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        for rev_id, text in [('1-2', 'original from 1\n'),
+                             ('1-3', 'agreement\n'),
+                             ('2-1', 'contents in 2\n'),
+                             ('2-2', 'agreement\n')]:
+            self.assertEqualDiff(
+                br2.repository.revision_tree(
+                    rev_id).get_file_text('this-file-id'), text)
+
+
+class TestHttpFetch(TestCaseWithWebserver):
+    # FIXME RBC 20060124 this really isn't web specific, perhaps an
+    # instrumented readonly transport? Can we do an instrumented
+    # adapter and use self.get_readonly_url ?
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches(self)
+        br_rem_a = Branch.open(self.get_readonly_url('branch1'))
+        fetch_steps(self, br_rem_a, br_b, br_a)
+
+    def _count_log_matches(self, target, logs):
+        """Count the number of times the target file pattern was fetched in an http log"""
+        log_pattern = '%s HTTP/1.1" 200 - "-" "bzr/%s' % \
+            (target, bzrlib.__version__)
+        c = 0
+        for line in logs:
+            # TODO: perhaps use a regexp instead so we can match more
+            # precisely?
+            if line.find(log_pattern) > -1:
+                c += 1
+        return c
+
+    def test_weaves_are_retrieved_once(self):
+        self.build_tree(("source/", "source/file", "target/"))
+        wt = self.make_branch_and_tree('source')
+        branch = wt.branch
+        wt.add(["file"], ["id"])
+        wt.commit("added file")
+        print >>open("source/file", 'w'), "blah"
+        wt.commit("changed file")
+        target = BzrDir.create_branch_and_repo("target/")
+        source = Branch.open(self.get_readonly_url("source/"))
+        self.assertEqual(target.fetch(source), (2, []))
+        log_pattern = '%%s HTTP/1.1" 200 - "-" "bzr/%s' % bzrlib.__version__
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        self.log("web server logs are:")
+        http_logs = self.get_readonly_server().logs
+        self.log('\n'.join(http_logs))
+        self.assertEqual(1, self._count_log_matches('weaves/ce/id.weave', http_logs))
+        self.assertEqual(1, self._count_log_matches('inventory.weave', http_logs))
+        # this r-h check test will prevent regressions, but it currently already 
+        # passes, before the patch to cache-rh is applied :[
+        self.assertEqual(1, self._count_log_matches('revision-history', http_logs))
+        # FIXME naughty poking in there.
+        self.get_readonly_server().logs = []
+        # check there is nothing more to fetch
+        source = Branch.open(self.get_readonly_url("source/"))
+        self.assertEqual(target.fetch(source), (0, []))
+        # should make just two requests
+        http_logs = self.get_readonly_server().logs
+        self.log("web server logs are:")
+        self.log('\n'.join(http_logs))
+        self.assertEqual(1, self._count_log_matches('branch-format', http_logs))
+        self.assertEqual(1, self._count_log_matches('branch/format', http_logs))
+        self.assertEqual(1, self._count_log_matches('repository/format', http_logs))
+        self.assertEqual(1, self._count_log_matches('revision-history', http_logs))
+        self.assertEqual(4, len(http_logs))

=== added file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_gpg.py	2006-02-15 15:18:44 +0000
@@ -0,0 +1,72 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for signing and verifying blobs of data via gpg."""
+
+# import system imports here
+import os
+import sys
+
+import bzrlib.errors as errors
+import bzrlib.gpg as gpg
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+class FakeConfig(object):
+
+    def gpg_signing_command(self):
+        return "false"
+        
+
+class TestCommandLine(TestCase):
+
+    def test_signing_command_line(self):
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual(['false',  '--clearsign'],
+                         my_gpg._command_line())
+
+    def test_checks_return_code(self):
+        # This test needs a unix like platform - one with 'false' to run.
+        # if you have one, please make this work :)
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertRaises(errors.SigningFailed, my_gpg.sign, 'content')
+
+    def test_returns_output(self):
+        # This test needs a 'cat' command or similar to work.
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        content = "some content\nwith newlines\n"
+
+        if sys.platform == 'win32':
+            # Windows doesn't come with cat, and we don't require it
+            # so lets try using python instead.
+            # But stupid windows and line-ending conversions. 
+            # It is too much work to make sys.stdout be in binary mode.
+            # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65443
+            my_gpg._command_line = lambda:[sys.executable, '-c',
+                    'import sys; sys.stdout.write(sys.stdin.read())']
+            new_content = content.replace('\n', '\r\n')
+
+            self.assertEqual(new_content, my_gpg.sign(content))
+        else:
+            my_gpg._command_line = lambda:['cat', '-']
+            self.assertEqual(content, my_gpg.sign(content))
+
+
+class TestDisabled(TestCase):
+
+    def test_sign(self):
+        self.assertRaises(errors.SigningFailed,
+                          gpg.DisabledGPGStrategy(None).sign, 'content')

=== added file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_graph.py	2006-03-22 16:26:09 +0000
@@ -0,0 +1,63 @@
+from bzrlib.tests import TestCase
+from bzrlib.graph import node_distances, nodes_by_distance, Graph
+
+class TestBase(TestCase):
+
+    def edge_add(self, *args):
+        for start, end in zip(args[:-1], args[1:]):
+            if start not in self.graph:
+                self.graph[start] = {}
+            if end not in self.graph:
+                self.graph[end] = {}
+            self.graph[start][end] = 1
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self.graph = {}
+        self.edge_add('A', 'B', 'C', 'D')
+        self.edge_add('A', 'E', 'F', 'C')
+        self.edge_add('A', 'G', 'H', 'I', 'B')
+        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
+        self.edge_add('O', 'N')
+
+    def node_descendants(self):
+        descendants = {'A':set()}
+        for node in self.graph:
+            for ancestor in self.graph[node]:
+                if ancestor not in descendants:
+                    descendants[ancestor] = set()
+                descendants[ancestor].add(node)
+        return descendants
+    
+    def test_distances(self):
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        nodes = nodes_by_distance(distances)
+        self.assertEqual(nodes[0], 'D')
+        self.assert_(nodes[1] in ('N', 'C'))
+        self.assert_(nodes[2] in ('N', 'C'))
+        self.assert_(nodes[3] in ('B', 'M'))
+        self.assert_(nodes[4] in ('B', 'M'))
+
+        #Ensure we don't shortcut through B when there's only a difference of
+        # 1 in distance
+        self.graph = {}
+        self.edge_add('A', 'B', 'C')
+        self.edge_add('A', 'D', 'E', 'C')
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        self.assertEqual(distances['C'], 3)
+
+
+class TestGraph(TestCase):
+
+    def test_get_descendants(self):
+        # Graph objects let you get a descendants graph in 
+        # node: {direct-children:distance} which contains
+        # known children, including ghost children
+        graph = Graph()
+        graph.add_ghost('ghost')
+        graph.add_node('rev1', ['ghost'])
+        # check the result contains ghosts:
+        self.assertEqual({'ghost': {'rev1': 1}, 'rev1': {}},
+                         graph.get_descendants())

=== renamed file 'bzrlib/selftest/testhashcache.py' => 'bzrlib/tests/test_hashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-07-08 06:54:58 +0000
+++ bzrlib/tests/test_hashcache.py	2006-03-31 00:43:21 +0000
@@ -14,34 +14,44 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
+import os
+import sha
+import sys
+import time
 
+from bzrlib.errors import BzrError
+from bzrlib.hashcache import HashCache
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
 
 
 def sha1(t):
-    import sha
     return sha.new(t).hexdigest()
 
 
 def pause():
-    import time
+    if False:
+        return
+    if sys.platform in ('win32', 'cygwin'):
+        time.sleep(3)
+        return
     # allow it to stabilize
     start = int(time.time())
     while int(time.time()) == start:
         time.sleep(0.2)
+
+
+class FixThisError(Exception):
+    pass
     
 
+class TestHashCache(TestCaseInTempDir):
 
-class TestHashCache(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
-        from bzrlib.hashcache import HashCache
-        import os
-        import time
+    def test_hashcache(self):
+        """Functional tests for hashcache"""
 
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
-        hc = HashCache('.')
+        hc = HashCache('.', '.bzr/stat-cache')
 
         file('foo', 'wb').write('hello')
         os.mkdir('subdir')
@@ -89,22 +99,19 @@
 
         self.assertEquals(hc.get_sha1('subdir'), None)
 
-        # it's likely neither are cached at the moment because they 
-        # changed recently, but we can't be sure
+        # pause briefly to make sure they're not treated as new uncacheable
+        # files
         pause()
 
-        # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
         hc.write()
         del hc
 
-        hc = HashCache('.')
+        hc = HashCache('.', '.bzr/stat-cache')
         hc.read()
 
         self.assertEquals(len(hc._cache), 2)
@@ -113,7 +120,26 @@
         self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
 
-        
-
-        
-
+    def test_hashcache_raise(self):
+        """check that hashcache can raise BzrError"""
+
+        os.mkdir('.bzr')
+        hc = HashCache('.', '.bzr/stat-cache')
+        ok = False
+
+        # make a best effort to create a weird kind of file
+        funcs = (getattr(os, 'mkfifo', None), getattr(os, 'mknod', None))
+        for func in funcs:
+            if func is None:
+                continue
+            try:
+                func('a')
+                ok = True
+                break
+            except FixThisError:
+                pass
+
+        if ok:
+            self.assertRaises(BzrError, hc.get_sha1, 'a')
+        else:
+            raise TestSkipped('No weird file type could be created')

=== added file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_http.py	2006-03-16 20:02:36 +0000
@@ -0,0 +1,161 @@
+# Copyright (C) 2005, 2006 Canonical
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# FIXME: This test should be repeated for each available http client
+# implementation; at the moment we have urllib and pycurl.
+
+# TODO: Should be renamed to bzrlib.transport.http.tests?
+
+import bzrlib
+from bzrlib.errors import DependencyNotPresent
+from bzrlib.tests import TestCase, TestSkipped
+from bzrlib.transport import Transport
+from bzrlib.transport.http import extract_auth
+from bzrlib.transport.http._urllib import HttpTransport_urllib
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+
+class FakeManager (object):
+    def __init__(self):
+        self.credentials = []
+        
+    def add_password(self, realm, host, username, password):
+        self.credentials.append([realm, host, username, password])
+
+
+class TestHttpUrls(TestCase):
+    def test_url_parsing(self):
+        f = FakeManager()
+        url = extract_auth('http://example.com', f)
+        self.assertEquals('http://example.com', url)
+        self.assertEquals(0, len(f.credentials))
+        url = extract_auth('http://user:pass@www.bazaar-ng.org/bzr/bzr.dev', f)
+        self.assertEquals('http://www.bazaar-ng.org/bzr/bzr.dev', url)
+        self.assertEquals(1, len(f.credentials))
+        self.assertEquals([None, 'www.bazaar-ng.org', 'user', 'pass'], f.credentials[0])
+        
+    def test_abs_url(self):
+        """Construction of absolute http URLs"""
+        t = HttpTransport_urllib('http://bazaar-ng.org/bzr/bzr.dev/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.'),
+           'http://bazaar-ng.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 
+           'http://bazaar-ng.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.bzr/1//2/./3'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr/1/2/3')
+
+    def test_invalid_http_urls(self):
+        """Trap invalid construction of urls"""
+        t = HttpTransport_urllib('http://bazaar-ng.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '.bzr/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '/.bzr')
+
+    def test_http_root_urls(self):
+        """Construction of URLs from server root"""
+        t = HttpTransport_urllib('http://bzr.ozlabs.org/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.bzr/tree-version'),
+           'http://bzr.ozlabs.org/.bzr/tree-version')
+
+    def test_http_impl_urls(self):
+        """There are servers which ask for particular clients to connect"""
+        try:
+            from bzrlib.transport.http._pycurl import HttpServer_PyCurl
+            server = HttpServer_PyCurl()
+            try:
+                server.setUp()
+                url = server.get_url()
+                self.assertTrue(url.startswith('http+pycurl://'))
+            finally:
+                server.tearDown()
+        except DependencyNotPresent:
+            raise TestSkipped('pycurl not present')
+
+class TestHttpMixins(object):
+
+    def _prep_tree(self):
+        self.build_tree(['xxx', 'foo/', 'foo/bar'], line_endings='binary',
+                        transport=self.get_transport())
+
+    def test_http_has(self):
+        server = self.get_readonly_server()
+        t = self._transport(server.get_url())
+        self.assertEqual(t.has('foo/bar'), True)
+        self.assertEqual(len(server.logs), 1)
+        self.assertContainsRe(server.logs[0], 
+            r'"HEAD /foo/bar HTTP/1.." (200|302) - "-" "bzr/')
+
+    def test_http_has_not_found(self):
+        server = self.get_readonly_server()
+        t = self._transport(server.get_url())
+        self.assertEqual(t.has('not-found'), False)
+        self.assertContainsRe(server.logs[1], 
+            r'"HEAD /not-found HTTP/1.." 404 - "-" "bzr/')
+
+    def test_http_get(self):
+        server = self.get_readonly_server()
+        t = self._transport(server.get_url())
+        fp = t.get('foo/bar')
+        self.assertEqualDiff(
+            fp.read(),
+            'contents of foo/bar\n')
+        self.assertEqual(len(server.logs), 1)
+        self.assertTrue(server.logs[0].find(
+            '"GET /foo/bar HTTP/1.1" 200 - "-" "bzr/%s' % bzrlib.__version__) > -1)
+
+
+class TestHttpConnections_urllib(TestCaseWithWebserver, TestHttpMixins):
+    _transport = HttpTransport_urllib
+
+    def setUp(self):
+        TestCaseWithWebserver.setUp(self)
+        self._prep_tree()
+
+
+
+class TestHttpConnections_pycurl(TestCaseWithWebserver, TestHttpMixins):
+
+    def _get_pycurl_maybe(self):
+        try:
+            from bzrlib.transport.http._pycurl import PyCurlTransport
+            return PyCurlTransport
+        except DependencyNotPresent:
+            raise TestSkipped('pycurl not present')
+
+    _transport = property(_get_pycurl_maybe)
+
+    def setUp(self):
+        TestCaseWithWebserver.setUp(self)
+        self._prep_tree()
+
+
+
+class TestHttpTransportRegistration(TestCase):
+    """Test registrations of various http implementations"""
+
+    def test_http_registered(self):
+        import bzrlib.transport.http._urllib
+        from bzrlib.transport import get_transport
+        # urlllib should always be present
+        t = get_transport('http+urllib://bzr.google.com/')
+        self.assertIsInstance(t, Transport)
+        self.assertIsInstance(t, bzrlib.transport.http._urllib.HttpTransport_urllib)

=== added file 'bzrlib/tests/test_identitymap.py'
--- bzrlib/tests/test_identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_identitymap.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,111 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the IdentityMap class."""
+
+# import system imports here
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase
+import bzrlib.identitymap as identitymap
+
+
+class TestIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import IdentityMap
+
+    def test_construct(self):
+        identitymap.IdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertRaises(errors.BzrError, map.add_weave, "id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+ 
+    def test_remove_object(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.remove_object(weave)
+        map.add_weave("id", weave)
+        rev_history = [1]
+        map.add_revision_history(rev_history)
+        map.remove_object(rev_history)
+
+    def test_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(rev_history, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        self.assertRaises(errors.BzrError,
+                          map.add_revision_history,
+                          revision_history)
+        self.assertEqual(revision_history, map.find_revision_history())
+
+ 
+class TestNullIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import NullIdentityMap
+
+    def test_construct(self):
+        identitymap.NullIdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+        
+    def test_null_identity_map_has_no_remove(self):
+        map = identitymap.NullIdentityMap()
+        self.assertEqual(None, getattr(map, 'remove_object', None))
+
+    def test_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(None, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        map.add_revision_history(revision_history)
+        self.assertEqual(None, map.find_revision_history())

=== renamed file 'bzrlib/selftest/testinv.py' => 'bzrlib/tests/test_inv.py'
--- bzrlib/selftest/testinv.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/test_inv.py	2006-04-18 22:41:16 +0000
@@ -14,17 +14,27 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import TestCase
-
-from bzrlib.inventory import Inventory, InventoryEntry
-
-
-class TestIsWithin(TestCase):
-    def runTest(self):
+from cStringIO import StringIO
+import os
+
+from bzrlib.branch import Branch
+import bzrlib.errors as errors
+from bzrlib.diff import internal_diff
+from bzrlib.inventory import Inventory, ROOT_ID
+import bzrlib.inventory as inventory
+from bzrlib.osutils import has_symlinks, rename, pathjoin
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.uncommit import uncommit
+
+
+class TestInventory(TestCase):
+
+    def test_is_within(self):
         from bzrlib.osutils import is_inside_any
-        
-        for dirs, fn in [(['src', 'doc'], 'src/foo.c'),
-                         (['src'], 'src/foo.c'),
+
+        SRC_FOO_C = pathjoin('src', 'foo.c')
+        for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
+                         (['src'], SRC_FOO_C),
                          (['src'], 'src'),
                          ]:
             self.assert_(is_inside_any(dirs, fn))
@@ -33,10 +43,7 @@
                          (['src'], 'srccontrol/foo')]:
             self.assertFalse(is_inside_any(dirs, fn))
             
-            
-            
-class TestInventoryIds(TestCase):
-    def runTest(self):
+    def test_ids(self):
         """Test detection of files within selected directories."""
         inv = Inventory()
         
@@ -51,3 +58,438 @@
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
         
         self.assert_('src-id' in inv)
+
+
+    def test_version(self):
+        """Inventory remembers the text's version."""
+        inv = Inventory()
+        ie = inv.add_path('foo.txt', 'file')
+        ## XXX
+
+
+class TestInventoryEntry(TestCase):
+
+    def test_file_kind_character(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.assertEqual(file.kind_character(), '')
+
+    def test_dir_kind_character(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '/')
+
+    def test_link_kind_character(self):
+        dir = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '')
+
+    def test_dir_detect_changes(self):
+        left = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='bar'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+
+    def test_file_detect_changes(self):
+        left = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        right = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 123
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.executable = True
+        self.assertEqual((False, True), left.detect_changes(right))
+        self.assertEqual((False, True), right.detect_changes(left))
+        right.text_sha1 = 321
+        self.assertEqual((True, True), left.detect_changes(right))
+        self.assertEqual((True, True), right.detect_changes(left))
+
+    def test_symlink_detect_changes(self):
+        left = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='foo'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.symlink_target = 'different'
+        self.assertEqual((True, False), left.detect_changes(right))
+        self.assertEqual((True, False), right.detect_changes(left))
+
+    def test_file_has_text(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.failUnless(file.has_text())
+
+    def test_directory_has_text(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.failIf(dir.has_text())
+
+    def test_link_has_text(self):
+        link = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.failIf(link.has_text())
+
+
+class TestEntryDiffing(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestEntryDiffing, self).setUp()
+        self.wt = self.make_branch_and_tree('.')
+        self.branch = self.wt.branch
+        print >> open('file', 'wb'), 'foo'
+        self.wt.add(['file'], ['fileid'])
+        if has_symlinks():
+            os.symlink('target1', 'symlink')
+            self.wt.add(['symlink'], ['linkid'])
+        self.wt.commit('message_1', rev_id = '1')
+        print >> open('file', 'wb'), 'bar'
+        if has_symlinks():
+            os.unlink('symlink')
+            os.symlink('target2', 'symlink')
+        self.tree_1 = self.branch.repository.revision_tree('1')
+        self.inv_1 = self.branch.repository.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.tree_2 = self.wt
+        self.inv_2 = self.tree_2.read_working_inventory()
+        self.file_2 = self.inv_2['fileid']
+        if has_symlinks():
+            self.link_1 = self.inv_1['linkid']
+            self.link_2 = self.inv_2['linkid']
+
+    def test_file_diff_deleted(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(), "--- old_label\t\n"
+                                            "+++ /dev/null\t\n"
+                                            "@@ -1,1 +0,0 @@\n"
+                                            "-foo\n"
+                                            "\n")
+
+    def test_file_diff_added(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(), "--- /dev/null\t\n"
+                                            "+++ new_label\t\n"
+                                            "@@ -0,0 +1,1 @@\n"
+                                            "+foo\n"
+                                            "\n")
+
+    def test_file_diff_changed(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.file_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(), "--- /dev/null\t\n"
+                                            "+++ new_label\t\n"
+                                            "@@ -1,1 +1,1 @@\n"
+                                            "-foo\n"
+                                            "+bar\n"
+                                            "\n")
+        
+    def test_link_diff_deleted(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target was 'target1'\n")
+
+    def test_link_diff_added(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(),
+                         "=== target is 'target1'\n")
+
+    def test_link_diff_changed(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.link_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target changed 'target1' => 'target2'\n")
+
+
+class TestSnapshot(TestCaseWithTransport):
+
+    def setUp(self):
+        # for full testing we'll need a branch
+        # with a subdir to test parent changes.
+        # and a file, link and dir under that.
+        # but right now I only need one attribute
+        # to change, and then test merge patterns
+        # with fake parent entries.
+        super(TestSnapshot, self).setUp()
+        self.wt = self.make_branch_and_tree('.')
+        self.branch = self.wt.branch
+        self.build_tree(['subdir/', 'subdir/file'], line_endings='binary')
+        self.wt.add(['subdir', 'subdir/file'],
+                                       ['dirid', 'fileid'])
+        if has_symlinks():
+            pass
+        self.wt.commit('message_1', rev_id = '1')
+        self.tree_1 = self.branch.repository.revision_tree('1')
+        self.inv_1 = self.branch.repository.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.file_active = self.wt.inventory['fileid']
+
+    def test_snapshot_new_revision(self):
+        # This tests that a simple commit with no parents makes a new
+        # revision value in the inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {}, self.wt, 
+                                  self.branch.repository.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2', and we can get
+        # its text of 'file contents' out of the weave.
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '2')
+        # this should be a separate test probably, but lets check it once..
+        lines = self.branch.repository.weave_store.get_weave(
+            'fileid', 
+            self.branch.get_transaction()).get_lines('2')
+        self.assertEqual(lines, ['contents of subdir/file\n'])
+
+    def test_snapshot_unchanged(self):
+        #This tests that a simple commit does not make a new entry for
+        # an unchanged inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {'1':self.file_1},
+                                  self.wt, 
+                                  self.branch.repository.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '1')
+        vf = self.branch.repository.weave_store.get_weave(
+            'fileid', 
+            self.branch.repository.get_transaction())
+        self.assertRaises(errors.RevisionNotPresent,
+                          vf.get_lines,
+                          '2')
+
+    def test_snapshot_merge_identical_different_revid(self):
+        # This tests that a commit with two identical parents, one of which has
+        # a different revision id, results in a new revision id in the entry.
+        # 1->other, commit a merge of other against 1, results in 2.
+        other_ie = inventory.InventoryFile('fileid', 'newname', self.file_1.parent_id)
+        other_ie = inventory.InventoryFile('fileid', 'file', self.file_1.parent_id)
+        other_ie.revision = '1'
+        other_ie.text_sha1 = self.file_1.text_sha1
+        other_ie.text_size = self.file_1.text_size
+        self.assertEqual(self.file_1, other_ie)
+        other_ie.revision = 'other'
+        self.assertNotEqual(self.file_1, other_ie)
+        versionfile = self.branch.repository.weave_store.get_weave(
+            'fileid', self.branch.repository.get_transaction())
+        versionfile.clone_text('other', '1', ['1'])
+        self.file_active.snapshot('2', 'subdir/file', 
+                                  {'1':self.file_1, 'other':other_ie},
+                                  self.wt, 
+                                  self.branch.repository.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_active.revision, '2')
+
+    def test_snapshot_changed(self):
+        # This tests that a commit with one different parent results in a new
+        # revision id in the entry.
+        self.file_active.name='newname'
+        rename('subdir/file', 'subdir/newname')
+        self.file_active.snapshot('2', 'subdir/newname', {'1':self.file_1}, 
+                                  self.wt,
+                                  self.branch.repository.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2'
+        self.assertEqual(self.file_active.revision, '2')
+
+
+class TestPreviousHeads(TestCaseWithTransport):
+
+    def setUp(self):
+        # we want several inventories, that respectively
+        # give use the following scenarios:
+        # A) fileid not in any inventory (A),
+        # B) fileid present in one inventory (B) and (A,B)
+        # C) fileid present in two inventories, and they
+        #   are not mutual descendents (B, C)
+        # D) fileid present in two inventories and one is
+        #   a descendent of the other. (B, D)
+        super(TestPreviousHeads, self).setUp()
+        self.wt = self.make_branch_and_tree('.')
+        self.branch = self.wt.branch
+        self.build_tree(['file'])
+        self.wt.commit('new branch', allow_pointless=True, rev_id='A')
+        self.inv_A = self.branch.repository.get_inventory('A')
+        self.wt.add(['file'], ['fileid'])
+        self.wt.commit('add file', rev_id='B')
+        self.inv_B = self.branch.repository.get_inventory('B')
+        uncommit(self.branch, tree=self.wt)
+        self.assertEqual(self.branch.revision_history(), ['A'])
+        self.wt.commit('another add of file', rev_id='C')
+        self.inv_C = self.branch.repository.get_inventory('C')
+        self.wt.add_pending_merge('B')
+        self.wt.commit('merge in B', rev_id='D')
+        self.inv_D = self.branch.repository.get_inventory('D')
+        self.file_active = self.wt.inventory['fileid']
+        self.weave = self.branch.repository.weave_store.get_weave('fileid',
+            self.branch.repository.get_transaction())
+        
+    def get_previous_heads(self, inventories):
+        return self.file_active.find_previous_heads(
+            inventories, 
+            self.branch.repository.weave_store,
+            self.branch.repository.get_transaction())
+        
+    def test_fileid_in_no_inventory(self):
+        self.assertEqual({}, self.get_previous_heads([self.inv_A]))
+
+    def test_fileid_in_one_inventory(self):
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_A, self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_A]))
+
+    def test_fileid_in_two_inventories_gives_both_entries(self):
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_B, self.inv_C]))
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_C, self.inv_B]))
+
+    def test_fileid_in_two_inventories_already_merged_gives_head(self):
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_D]))
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_D, self.inv_B]))
+
+    # TODO: test two inventories with the same file revision 
+
+
+class TestExecutable(TestCaseWithTransport):
+
+    def test_stays_executable(self):
+        a_id = "a-20051208024829-849e76f7968d7a86"
+        b_id = "b-20051208024829-849e76f7968d7a86"
+        wt = self.make_branch_and_tree('b1')
+        b = wt.branch
+        open('b1/a', 'wb').write('a test\n')
+        open('b1/b', 'wb').write('b test\n')
+        os.chmod('b1/a', 0755)
+        os.chmod('b1/b', 0644)
+        wt.add(['a', 'b'], [a_id, b_id])
+        wt.inventory[a_id].executable = True
+        self.failUnless(wt.is_executable(a_id), "'a' lost the execute bit")
+
+        # reopen the tree and ensure it stuck.
+        wt = wt.bzrdir.open_workingtree()
+        self.assertEqual(['a', 'b'], [cn for cn,ie in wt.inventory.iter_entries()])
+
+        self.failUnless(wt.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(wt.is_executable(b_id), "'b' gained an execute bit")
+
+        wt.commit('adding a,b', rev_id='r1')
+
+        rev_tree = b.repository.revision_tree('r1')
+        self.failUnless(rev_tree.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(rev_tree.is_executable(b_id), "'b' gained an execute bit")
+
+        self.failUnless(rev_tree.inventory[a_id].executable)
+        self.failIf(rev_tree.inventory[b_id].executable)
+
+        # Make sure the entries are gone
+        os.remove('b1/a')
+        os.remove('b1/b')
+        self.failIf(wt.has_id(a_id))
+        self.failIf(wt.has_filename('a'))
+        self.failIf(wt.has_id(b_id))
+        self.failIf(wt.has_filename('b'))
+
+        # Make sure that revert is able to bring them back,
+        # and sets 'a' back to being executable
+
+        wt.revert(['a', 'b'], rev_tree, backups=False)
+        self.assertEqual(['a', 'b'], [cn for cn,ie in wt.inventory.iter_entries()])
+
+        self.failUnless(wt.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(wt.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now remove them again, and make sure that after a
+        # commit, they are still marked correctly
+        os.remove('b1/a')
+        os.remove('b1/b')
+        wt.commit('removed', rev_id='r2')
+
+        self.assertEqual([], [cn for cn,ie in wt.inventory.iter_entries()])
+        self.failIf(wt.has_id(a_id))
+        self.failIf(wt.has_filename('a'))
+        self.failIf(wt.has_id(b_id))
+        self.failIf(wt.has_filename('b'))
+
+        # Now revert back to the previous commit
+        wt.revert([], rev_tree, backups=False)
+        self.assertEqual(['a', 'b'], [cn for cn,ie in wt.inventory.iter_entries()])
+
+        self.failUnless(wt.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(wt.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now make sure that 'bzr branch' also preserves the
+        # executable bit
+        # TODO: Maybe this should be a blackbox test
+        d2 = b.bzrdir.clone('b2', revision_id='r1')
+        t2 = d2.open_workingtree()
+        b2 = t2.branch
+        self.assertEquals('r1', b2.last_revision())
+
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")
+
+        # Make sure pull will delete the files
+        t2.pull(b)
+        self.assertEquals('r2', b2.last_revision())
+        self.assertEqual([], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        # Now commit the changes on the first branch
+        # so that the second branch can pull the changes
+        # and make sure that the executable bit has been copied
+        wt.commit('resurrected', rev_id='r3')
+
+        t2.pull(b)
+        self.assertEquals('r3', b2.last_revision())
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")
+
+class TestRevert(TestCaseWithTransport):
+    def test_dangling_id(self):
+        wt = self.make_branch_and_tree('b1')
+        self.assertEqual(len(wt.inventory), 1)
+        open('b1/a', 'wb').write('a test\n')
+        wt.add('a')
+        self.assertEqual(len(wt.inventory), 2)
+        os.unlink('b1/a')
+        wt.revert([])
+        self.assertEqual(len(wt.inventory), 1)
+
+

=== added file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_knit.py	2006-04-18 00:19:54 +0000
@@ -0,0 +1,465 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for Knit data structure"""
+
+
+import difflib
+
+
+from bzrlib.errors import KnitError, RevisionAlreadyPresent
+from bzrlib.knit import KnitVersionedFile, KnitPlainFactory, KnitAnnotateFactory
+from bzrlib.osutils import split_lines
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.transport import TransportLogger
+from bzrlib.transport.local import LocalTransport
+from bzrlib.transport.memory import MemoryTransport
+
+
+class KnitTests(TestCaseInTempDir):
+
+    def add_stock_one_and_one_a(self, k):
+        k.add_lines('text-1', [], split_lines(TEXT_1))
+        k.add_lines('text-1a', ['text-1'], split_lines(TEXT_1A))
+
+    def test_knit_constructor(self):
+        """Construct empty k"""
+        self.make_test_knit()
+
+    def make_test_knit(self, annotate=False):
+        if not annotate:
+            factory = KnitPlainFactory()
+        else:
+            factory = None
+        return KnitVersionedFile('test', LocalTransport('.'), access_mode='w', factory=factory, create=True)
+
+    def test_knit_add(self):
+        """Store one text in knit and retrieve"""
+        k = self.make_test_knit()
+        k.add_lines('text-1', [], split_lines(TEXT_1))
+        self.assertTrue(k.has_version('text-1'))
+        self.assertEqualDiff(''.join(k.get_lines('text-1')), TEXT_1)
+
+    def test_knit_reload(self):
+        # test that the content in a reloaded knit is correct
+        k = self.make_test_knit()
+        k.add_lines('text-1', [], split_lines(TEXT_1))
+        del k
+        k2 = KnitVersionedFile('test', LocalTransport('.'), access_mode='r', factory=KnitPlainFactory(), create=True)
+        self.assertTrue(k2.has_version('text-1'))
+        self.assertEqualDiff(''.join(k2.get_lines('text-1')), TEXT_1)
+
+    def test_knit_several(self):
+        """Store several texts in a knit"""
+        k = self.make_test_knit()
+        k.add_lines('text-1', [], split_lines(TEXT_1))
+        k.add_lines('text-2', [], split_lines(TEXT_2))
+        self.assertEqualDiff(''.join(k.get_lines('text-1')), TEXT_1)
+        self.assertEqualDiff(''.join(k.get_lines('text-2')), TEXT_2)
+        
+    def test_repeated_add(self):
+        """Knit traps attempt to replace existing version"""
+        k = self.make_test_knit()
+        k.add_lines('text-1', [], split_lines(TEXT_1))
+        self.assertRaises(RevisionAlreadyPresent, 
+                k.add_lines,
+                'text-1', [], split_lines(TEXT_1))
+
+    def test_empty(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-1', [], [])
+        self.assertEquals(k.get_lines('text-1'), [])
+
+    def test_incomplete(self):
+        """Test if texts without a ending line-end can be inserted and
+        extracted."""
+        k = KnitVersionedFile('test', LocalTransport('.'), delta=False, create=True)
+        k.add_lines('text-1', [], ['a\n',    'b'  ])
+        k.add_lines('text-2', ['text-1'], ['a\rb\n', 'b\n'])
+        self.assertEquals(k.get_lines('text-1'), ['a\n',    'b'  ])
+        self.assertEquals(k.get_lines('text-2'), ['a\rb\n', 'b\n'])
+
+    def test_delta(self):
+        """Expression of knit delta as lines"""
+        k = self.make_test_knit()
+        td = list(line_delta(TEXT_1.splitlines(True),
+                             TEXT_1A.splitlines(True)))
+        self.assertEqualDiff(''.join(td), delta_1_1a)
+        out = apply_line_delta(TEXT_1.splitlines(True), td)
+        self.assertEqualDiff(''.join(out), TEXT_1A)
+
+    def test_add_with_parents(self):
+        """Store in knit with parents"""
+        k = self.make_test_knit()
+        self.add_stock_one_and_one_a(k)
+        self.assertEquals(k.get_parents('text-1'), [])
+        self.assertEquals(k.get_parents('text-1a'), ['text-1'])
+
+    def test_ancestry(self):
+        """Store in knit with parents"""
+        k = self.make_test_knit()
+        self.add_stock_one_and_one_a(k)
+        self.assertEquals(set(k.get_ancestry(['text-1a'])), set(['text-1a', 'text-1']))
+
+    def test_add_delta(self):
+        """Store in knit with parents"""
+        k = KnitVersionedFile('test', LocalTransport('.'), factory=KnitPlainFactory(),
+            delta=True, create=True)
+        self.add_stock_one_and_one_a(k)
+        k.clear_cache()
+        self.assertEqualDiff(''.join(k.get_lines('text-1a')), TEXT_1A)
+
+    def test_annotate(self):
+        """Annotations"""
+        k = KnitVersionedFile('knit', LocalTransport('.'), factory=KnitAnnotateFactory(),
+            delta=True, create=True)
+        self.insert_and_test_small_annotate(k)
+
+    def insert_and_test_small_annotate(self, k):
+        """test annotation with k works correctly."""
+        k.add_lines('text-1', [], ['a\n', 'b\n'])
+        k.add_lines('text-2', ['text-1'], ['a\n', 'c\n'])
+
+        origins = k.annotate('text-2')
+        self.assertEquals(origins[0], ('text-1', 'a\n'))
+        self.assertEquals(origins[1], ('text-2', 'c\n'))
+
+    def test_annotate_fulltext(self):
+        """Annotations"""
+        k = KnitVersionedFile('knit', LocalTransport('.'), factory=KnitAnnotateFactory(),
+            delta=False, create=True)
+        self.insert_and_test_small_annotate(k)
+
+    def test_annotate_merge_1(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n'])
+        k.add_lines('text-a2', [], ['d\n', 'c\n'])
+        k.add_lines('text-am', ['text-a1', 'text-a2'], ['d\n', 'b\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-a2', 'd\n'))
+        self.assertEquals(origins[1], ('text-a1', 'b\n'))
+
+    def test_annotate_merge_2(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-a2', [], ['x\n', 'y\n', 'z\n'])
+        k.add_lines('text-am', ['text-a1', 'text-a2'], ['a\n', 'y\n', 'c\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-a1', 'a\n'))
+        self.assertEquals(origins[1], ('text-a2', 'y\n'))
+        self.assertEquals(origins[2], ('text-a1', 'c\n'))
+
+    def test_annotate_merge_9(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-a2', [], ['x\n', 'y\n', 'z\n'])
+        k.add_lines('text-am', ['text-a1', 'text-a2'], ['k\n', 'y\n', 'c\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-am', 'k\n'))
+        self.assertEquals(origins[1], ('text-a2', 'y\n'))
+        self.assertEquals(origins[2], ('text-a1', 'c\n'))
+
+    def test_annotate_merge_3(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-a2', [] ,['x\n', 'y\n', 'z\n'])
+        k.add_lines('text-am', ['text-a1', 'text-a2'], ['k\n', 'y\n', 'z\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-am', 'k\n'))
+        self.assertEquals(origins[1], ('text-a2', 'y\n'))
+        self.assertEquals(origins[2], ('text-a2', 'z\n'))
+
+    def test_annotate_merge_4(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-a2', [], ['x\n', 'y\n', 'z\n'])
+        k.add_lines('text-a3', ['text-a1'], ['a\n', 'b\n', 'p\n'])
+        k.add_lines('text-am', ['text-a2', 'text-a3'], ['a\n', 'b\n', 'z\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-a1', 'a\n'))
+        self.assertEquals(origins[1], ('text-a1', 'b\n'))
+        self.assertEquals(origins[2], ('text-a2', 'z\n'))
+
+    def test_annotate_merge_5(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-a1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-a2', [], ['d\n', 'e\n', 'f\n'])
+        k.add_lines('text-a3', [], ['x\n', 'y\n', 'z\n'])
+        k.add_lines('text-am',
+                    ['text-a1', 'text-a2', 'text-a3'],
+                    ['a\n', 'e\n', 'z\n'])
+        origins = k.annotate('text-am')
+        self.assertEquals(origins[0], ('text-a1', 'a\n'))
+        self.assertEquals(origins[1], ('text-a2', 'e\n'))
+        self.assertEquals(origins[2], ('text-a3', 'z\n'))
+
+    def test_annotate_file_cherry_pick(self):
+        k = self.make_test_knit(True)
+        k.add_lines('text-1', [], ['a\n', 'b\n', 'c\n'])
+        k.add_lines('text-2', ['text-1'], ['d\n', 'e\n', 'f\n'])
+        k.add_lines('text-3', ['text-2', 'text-1'], ['a\n', 'b\n', 'c\n'])
+        origins = k.annotate('text-3')
+        self.assertEquals(origins[0], ('text-1', 'a\n'))
+        self.assertEquals(origins[1], ('text-1', 'b\n'))
+        self.assertEquals(origins[2], ('text-1', 'c\n'))
+
+    def test_knit_join(self):
+        """Store in knit with parents"""
+        k1 = KnitVersionedFile('test1', LocalTransport('.'), factory=KnitPlainFactory(), create=True)
+        k1.add_lines('text-a', [], split_lines(TEXT_1))
+        k1.add_lines('text-b', ['text-a'], split_lines(TEXT_1))
+
+        k1.add_lines('text-c', [], split_lines(TEXT_1))
+        k1.add_lines('text-d', ['text-c'], split_lines(TEXT_1))
+
+        k1.add_lines('text-m', ['text-b', 'text-d'], split_lines(TEXT_1))
+
+        k2 = KnitVersionedFile('test2', LocalTransport('.'), factory=KnitPlainFactory(), create=True)
+        count = k2.join(k1, version_ids=['text-m'])
+        self.assertEquals(count, 5)
+        self.assertTrue(k2.has_version('text-a'))
+        self.assertTrue(k2.has_version('text-c'))
+
+    def test_reannotate(self):
+        k1 = KnitVersionedFile('knit1', LocalTransport('.'),
+                               factory=KnitAnnotateFactory(), create=True)
+        # 0
+        k1.add_lines('text-a', [], ['a\n', 'b\n'])
+        # 1
+        k1.add_lines('text-b', ['text-a'], ['a\n', 'c\n'])
+
+        k2 = KnitVersionedFile('test2', LocalTransport('.'),
+                               factory=KnitAnnotateFactory(), create=True)
+        k2.join(k1, version_ids=['text-b'])
+
+        # 2
+        k1.add_lines('text-X', ['text-b'], ['a\n', 'b\n'])
+        # 2
+        k2.add_lines('text-c', ['text-b'], ['z\n', 'c\n'])
+        # 3
+        k2.add_lines('text-Y', ['text-b'], ['b\n', 'c\n'])
+
+        # test-c will have index 3
+        k1.join(k2, version_ids=['text-c'])
+
+        lines = k1.get_lines('text-c')
+        self.assertEquals(lines, ['z\n', 'c\n'])
+
+        origins = k1.annotate('text-c')
+        self.assertEquals(origins[0], ('text-c', 'z\n'))
+        self.assertEquals(origins[1], ('text-b', 'c\n'))
+
+    def test_extraction_reads_components_once(self):
+        t = MemoryTransport()
+        instrumented_t = TransportLogger(t)
+        k1 = KnitVersionedFile('id', instrumented_t, create=True, delta=True)
+        # should read the index
+        self.assertEqual([('id.kndx',)], instrumented_t._calls)
+        instrumented_t._calls = []
+        # add a text       
+        k1.add_lines('base', [], ['text\n'])
+        # should not have read at all
+        self.assertEqual([], instrumented_t._calls)
+
+        # add a text
+        k1.add_lines('sub', ['base'], ['text\n', 'text2\n'])
+        # should not have read at all
+        self.assertEqual([], instrumented_t._calls)
+        
+        # read a text
+        k1.get_lines('sub')
+        # should not have read at all
+        self.assertEqual([], instrumented_t._calls)
+
+        # clear the cache
+        k1.clear_cache()
+
+        # read a text
+        k1.get_lines('base')
+        # should have read a component
+        # should not have read the first component only
+        self.assertEqual([('id.knit', [(0, 87)])], instrumented_t._calls)
+        instrumented_t._calls = []
+        # read again
+        k1.get_lines('base')
+        # should not have read at all
+        self.assertEqual([], instrumented_t._calls)
+        # and now read the other component
+        k1.get_lines('sub')
+        # should have read the second component
+        self.assertEqual([('id.knit', [(87, 93)])], instrumented_t._calls)
+        instrumented_t._calls = []
+
+        # clear the cache
+        k1.clear_cache()
+        # add a text cold 
+        k1.add_lines('sub2', ['base'], ['text\n', 'text3\n'])
+        # should read the first component only
+        self.assertEqual([('id.knit', [(0, 87)])], instrumented_t._calls)
+        
+    def test_iter_lines_reads_in_order(self):
+        t = MemoryTransport()
+        instrumented_t = TransportLogger(t)
+        k1 = KnitVersionedFile('id', instrumented_t, create=True, delta=True)
+        self.assertEqual([('id.kndx',)], instrumented_t._calls)
+        # add texts with no required ordering
+        k1.add_lines('base', [], ['text\n'])
+        k1.add_lines('base2', [], ['text2\n'])
+        k1.clear_cache()
+        instrumented_t._calls = []
+        # request a last-first iteration
+        results = list(k1.iter_lines_added_or_present_in_versions(['base2', 'base']))
+        self.assertEqual([('id.knit', [(0, 87), (87, 89)])], instrumented_t._calls)
+        self.assertEqual(['text\n', 'text2\n'], results)
+
+    def test_create_empty_annotated(self):
+        k1 = self.make_test_knit(True)
+        # 0
+        k1.add_lines('text-a', [], ['a\n', 'b\n'])
+        k2 = k1.create_empty('t', MemoryTransport())
+        self.assertTrue(isinstance(k2.factory, KnitAnnotateFactory))
+        self.assertEqual(k1.delta, k2.delta)
+        # the generic test checks for empty content and file class
+
+    def test_knit_format(self):
+        # this tests that a new knit index file has the expected content
+        # and that is writes the data we expect as records are added.
+        knit = self.make_test_knit(True)
+        self.assertFileEqual("# bzr knit index 7\n", 'test.kndx')
+        knit.add_lines_with_ghosts('revid', ['a_ghost'], ['a\n'])
+        self.assertFileEqual(
+            "# bzr knit index 7\n"
+            "\n"
+            "revid fulltext 0 84 .a_ghost :",
+            'test.kndx')
+        knit.add_lines_with_ghosts('revid2', ['revid'], ['a\n'])
+        self.assertFileEqual(
+            "# bzr knit index 7\n"
+            "\nrevid fulltext 0 84 .a_ghost :"
+            "\nrevid2 line-delta 84 82 0 :",
+            'test.kndx')
+        # we should be able to load this file again
+        knit = KnitVersionedFile('test', LocalTransport('.'), access_mode='r')
+        self.assertEqual(['revid', 'revid2'], knit.versions())
+        # write a short write to the file and ensure that its ignored
+        indexfile = file('test.kndx', 'at')
+        indexfile.write('\nrevid3 line-delta 166 82 1 2 3 4 5 .phwoar:demo ')
+        indexfile.close()
+        # we should be able to load this file again
+        knit = KnitVersionedFile('test', LocalTransport('.'), access_mode='w')
+        self.assertEqual(['revid', 'revid2'], knit.versions())
+        # and add a revision with the same id the failed write had
+        knit.add_lines('revid3', ['revid2'], ['a\n'])
+        # and when reading it revid3 should now appear.
+        knit = KnitVersionedFile('test', LocalTransport('.'), access_mode='r')
+        self.assertEqual(['revid', 'revid2', 'revid3'], knit.versions())
+        self.assertEqual(['revid2'], knit.get_parents('revid3'))
+
+    def test_plan_merge(self):
+        my_knit = self.make_test_knit(annotate=True)
+        my_knit.add_lines('text1', [], split_lines(TEXT_1))
+        my_knit.add_lines('text1a', ['text1'], split_lines(TEXT_1A))
+        my_knit.add_lines('text1b', ['text1'], split_lines(TEXT_1B))
+        plan = list(my_knit.plan_merge('text1a', 'text1b'))
+        for plan_line, expected_line in zip(plan, AB_MERGE):
+            self.assertEqual(plan_line, expected_line)
+
+
+TEXT_1 = """\
+Banana cup cakes:
+
+- bananas
+- eggs
+- broken tea cups
+"""
+
+TEXT_1A = """\
+Banana cup cake recipe
+(serves 6)
+
+- bananas
+- eggs
+- broken tea cups
+- self-raising flour
+"""
+
+TEXT_1B = """\
+Banana cup cake recipe
+
+- bananas (do not use plantains!!!)
+- broken tea cups
+- flour
+"""
+
+delta_1_1a = """\
+0,1,2
+Banana cup cake recipe
+(serves 6)
+5,5,1
+- self-raising flour
+"""
+
+TEXT_2 = """\
+Boeuf bourguignon
+
+- beef
+- red wine
+- small onions
+- carrot
+- mushrooms
+"""
+
+AB_MERGE_TEXT="""unchanged|Banana cup cake recipe
+new-a|(serves 6)
+unchanged|
+killed-b|- bananas
+killed-b|- eggs
+new-b|- bananas (do not use plantains!!!)
+unchanged|- broken tea cups
+new-a|- self-raising flour
+new-b|- flour
+"""
+AB_MERGE=[tuple(l.split('|')) for l in AB_MERGE_TEXT.splitlines(True)]
+
+
+def line_delta(from_lines, to_lines):
+    """Generate line-based delta from one text to another"""
+    s = difflib.SequenceMatcher(None, from_lines, to_lines)
+    for op in s.get_opcodes():
+        if op[0] == 'equal':
+            continue
+        yield '%d,%d,%d\n' % (op[1], op[2], op[4]-op[3])
+        for i in range(op[3], op[4]):
+            yield to_lines[i]
+
+
+def apply_line_delta(basis_lines, delta_lines):
+    """Apply a line-based perfect diff
+    
+    basis_lines -- text to apply the patch to
+    delta_lines -- diff instructions and content
+    """
+    out = basis_lines[:]
+    i = 0
+    offset = 0
+    while i < len(delta_lines):
+        l = delta_lines[i]
+        a, b, c = map(long, l.split(','))
+        i = i + 1
+        out[offset+a:offset+b] = delta_lines[i:i+c]
+        i = i + c
+        offset = offset + (b - a) + c
+    return out

=== added file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lockable_files.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,147 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from StringIO import StringIO
+
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests.test_transactions import DummyWeave
+from bzrlib.transactions import (PassThroughTransaction,
+                                 ReadOnlyTransaction,
+                                 WriteTransaction,
+                                 )
+from bzrlib.transport import get_transport
+
+
+# these tests are applied in each parameterized suite for LockableFiles
+class _TestLockableFiles_mixin(object):
+
+    def test_read_write(self):
+        self.assertRaises(NoSuchFile, self.lockable.get, 'foo')
+        self.assertRaises(NoSuchFile, self.lockable.get_utf8, 'foo')
+        self.lockable.lock_write()
+        try:
+            unicode_string = u'bar\u1234'
+            self.assertEqual(4, len(unicode_string))
+            byte_string = unicode_string.encode('utf-8')
+            self.assertEqual(6, len(byte_string))
+            self.assertRaises(UnicodeEncodeError, self.lockable.put, 'foo', 
+                              StringIO(unicode_string))
+            self.lockable.put('foo', StringIO(byte_string))
+            self.assertEqual(byte_string,
+                             self.lockable.get('foo').read())
+            self.assertEqual(unicode_string,
+                             self.lockable.get_utf8('foo').read())
+            self.assertRaises(BzrBadParameterNotString,
+                              self.lockable.put_utf8,
+                              'bar',
+                              StringIO(unicode_string)
+                              )
+            self.lockable.put_utf8('bar', unicode_string)
+            self.assertEqual(unicode_string, 
+                             self.lockable.get_utf8('bar').read())
+            self.assertEqual(byte_string, 
+                             self.lockable.get('bar').read())
+        finally:
+            self.lockable.unlock()
+
+    def test_locks(self):
+        self.lockable.lock_read()
+        try:
+            self.assertRaises(ReadOnlyError, self.lockable.put, 'foo', 
+                              StringIO('bar\u1234'))
+        finally:
+            self.lockable.unlock()
+
+    def test_transactions(self):
+        self.assertIs(self.lockable.get_transaction().__class__,
+                      PassThroughTransaction)
+        self.lockable.lock_read()
+        try:
+            self.assertIs(self.lockable.get_transaction().__class__,
+                          ReadOnlyTransaction)
+        finally:
+            self.lockable.unlock()
+        self.assertIs(self.lockable.get_transaction().__class__,
+                      PassThroughTransaction)
+        self.lockable.lock_write()
+        self.assertIs(self.lockable.get_transaction().__class__,
+                      WriteTransaction)
+        # check that finish is called:
+        vf = DummyWeave('a')
+        self.lockable.get_transaction().register_dirty(vf)
+        self.lockable.unlock()
+        self.assertTrue(vf.finished)
+
+    def test__escape(self):
+        self.assertEqual('%25', self.lockable._escape('%'))
+        
+    def test__escape_empty(self):
+        self.assertEqual('', self.lockable._escape(''))
+
+
+# This method of adapting tests to parameters is different to 
+# the TestProviderAdapters used elsewhere, but seems simpler for this 
+# case.  
+class TestLockableFiles_TransportLock(TestCaseInTempDir,
+                                      _TestLockableFiles_mixin):
+
+    def setUp(self):
+        super(TestLockableFiles_TransportLock, self).setUp()
+        transport = get_transport('.')
+        transport.mkdir('.bzr')
+        sub_transport = transport.clone('.bzr')
+        self.lockable = LockableFiles(sub_transport, 'my-lock', TransportLock)
+        self.lockable.create_lock()
+        
+
+class TestLockableFiles_LockDir(TestCaseInTempDir,
+                              _TestLockableFiles_mixin):
+    """LockableFile tests run with LockDir underneath"""
+
+    def setUp(self):
+        super(TestLockableFiles_LockDir, self).setUp()
+        self.transport = get_transport('.')
+        self.lockable = LockableFiles(self.transport, 'my-lock', LockDir)
+        # the lock creation here sets mode - test_permissions on branch 
+        # tests that implicitly, but it might be a good idea to factor 
+        # out the mode checking logic and have it applied to loackable files
+        # directly. RBC 20060418
+        self.lockable.create_lock()
+
+    def test_lock_is_lockdir(self):
+        """Created instance should use a LockDir.
+        
+        This primarily tests the mixin adapter works properly.
+        """
+        ## self.assertIsInstance(self.lockable, LockableFiles)
+        ## self.assertIsInstance(self.lockable._lock_strategy,
+                              ## LockDirStrategy)
+
+    def test_lock_created(self):
+        self.assertTrue(self.transport.has('my-lock'))
+        self.lockable.lock_write()
+        self.assertTrue(self.transport.has('my-lock/held/info'))
+        self.lockable.unlock()
+        self.assertFalse(self.transport.has('my-lock/held/info'))
+        self.assertTrue(self.transport.has('my-lock'))
+
+
+    # TODO: Test the lockdir inherits the right file and directory permissions
+    # from the LockableFiles.

=== added file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lockdir.py	2006-03-09 03:28:52 +0000
@@ -0,0 +1,320 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for LockDir"""
+
+from threading import Thread
+import time
+
+from bzrlib.errors import (
+        LockBreakMismatch,
+        LockContention, LockError, UnlockableTransport,
+        LockNotHeld, LockBroken
+        )
+from bzrlib.lockdir import LockDir
+from bzrlib.tests import TestCaseWithTransport
+
+# These tests sometimes use threads to test the behaviour of lock files with
+# concurrent actors.  This is not a typical (or necessarily supported) use;
+# they're really meant for guarding between processes.
+
+# These tests are run on the default transport provided by the test framework
+# (typically a local disk transport).  That can be changed by the --transport
+# option to bzr selftest.  The required properties of the transport
+# implementation are tested separately.  (The main requirement is just that
+# they don't allow overwriting nonempty directories.)
+
+class TestLockDir(TestCaseWithTransport):
+    """Test LockDir operations"""
+
+    def test_00_lock_creation(self):
+        """Creation of lock file on a transport"""
+        t = self.get_transport()
+        lf = LockDir(t, 'test_lock')
+        self.assertFalse(lf.is_held)
+
+    def test_01_lock_repr(self):
+        """Lock string representation"""
+        lf = LockDir(self.get_transport(), 'test_lock')
+        r = repr(lf)
+        self.assertContainsRe(r, r'^LockDir\(.*/test_lock\)$')
+
+    def test_02_unlocked_peek(self):
+        lf = LockDir(self.get_transport(), 'test_lock')
+        self.assertEqual(lf.peek(), None)
+
+    def test_03_readonly_peek(self):
+        lf = LockDir(self.get_readonly_transport(), 'test_lock')
+        self.assertEqual(lf.peek(), None)
+
+    def test_10_lock_uncontested(self):
+        """Acquire and release a lock"""
+        t = self.get_transport()
+        lf = LockDir(t, 'test_lock')
+        lf.create()
+        lf.attempt_lock()
+        try:
+            self.assertTrue(lf.is_held)
+        finally:
+            lf.unlock()
+            self.assertFalse(lf.is_held)
+
+    def test_11_create_readonly_transport(self):
+        """Fail to create lock on readonly transport"""
+        t = self.get_readonly_transport()
+        lf = LockDir(t, 'test_lock')
+        self.assertRaises(UnlockableTransport, lf.create)
+
+    def test_12_lock_readonly_transport(self):
+        """Fail to lock on readonly transport"""
+        lf = LockDir(self.get_transport(), 'test_lock')
+        lf.create()
+        lf = LockDir(self.get_readonly_transport(), 'test_lock')
+        self.assertRaises(UnlockableTransport, lf.attempt_lock)
+
+    def test_20_lock_contested(self):
+        """Contention to get a lock"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        lf2 = LockDir(t, 'test_lock')
+        try:
+            # locking is between LockDir instances; aliases within 
+            # a single process are not detected
+            lf2.attempt_lock()
+            self.fail('Failed to detect lock collision')
+        except LockContention, e:
+            self.assertEqual(e.lock, lf2)
+            self.assertContainsRe(str(e),
+                    r'^Could not acquire.*test_lock.*$')
+        lf1.unlock()
+
+    def test_20_lock_peek(self):
+        """Peek at the state of a lock"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        # lock is held, should get some info on it
+        info1 = lf1.peek()
+        self.assertEqual(set(info1.keys()),
+                         set(['user', 'nonce', 'hostname', 'pid', 'start_time']))
+        # should get the same info if we look at it through a different
+        # instance
+        info2 = LockDir(t, 'test_lock').peek()
+        self.assertEqual(info1, info2)
+        # locks which are never used should be not-held
+        self.assertEqual(LockDir(t, 'other_lock').peek(), None)
+
+    def test_21_peek_readonly(self):
+        """Peek over a readonly transport"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf2 = LockDir(self.get_readonly_transport(), 'test_lock')
+        self.assertEqual(lf2.peek(), None)
+        lf1.attempt_lock()
+        info2 = lf2.peek()
+        self.assertTrue(info2)
+        self.assertEqual(info2['nonce'], lf1.nonce)
+
+    def test_30_lock_wait_fail(self):
+        """Wait on a lock, then fail
+        
+        We ask to wait up to 400ms; this should fail within at most one
+        second.  (Longer times are more realistic but we don't want the test
+        suite to take too long, and this should do for now.)
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf2 = LockDir(t, 'test_lock')
+        lf1.attempt_lock()
+        try:
+            before = time.time()
+            self.assertRaises(LockContention, lf2.wait_lock,
+                              timeout=0.4, poll=0.1)
+            after = time.time()
+            self.assertTrue(after - before <= 1.0)
+        finally:
+            lf1.unlock()
+
+    def test_31_lock_wait_easy(self):
+        """Succeed when waiting on a lock with no contention.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        try:
+            before = time.time()
+            lf1.wait_lock(timeout=0.4, poll=0.1)
+            after = time.time()
+            self.assertTrue(after - before <= 1.0)
+        finally:
+            lf1.unlock()
+
+    def test_32_lock_wait_succeed(self):
+        """Succeed when trying to acquire a lock that gets released
+
+        One thread holds on a lock and then releases it; another 
+        tries to lock it.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+
+        def wait_and_unlock():
+            time.sleep(0.1)
+            lf1.unlock()
+        unlocker = Thread(target=wait_and_unlock)
+        unlocker.start()
+        try:
+            lf2 = LockDir(t, 'test_lock')
+            before = time.time()
+            # wait and then lock
+            lf2.wait_lock(timeout=0.4, poll=0.1)
+            after = time.time()
+            self.assertTrue(after - before <= 1.0)
+        finally:
+            unlocker.join()
+
+    def test_33_wait(self):
+        """Succeed when waiting on a lock that gets released
+
+        The difference from test_32_lock_wait_succeed is that the second 
+        caller does not actually acquire the lock, but just waits for it
+        to be released.  This is done over a readonly transport.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+
+        def wait_and_unlock():
+            time.sleep(0.1)
+            lf1.unlock()
+        unlocker = Thread(target=wait_and_unlock)
+        unlocker.start()
+        try:
+            lf2 = LockDir(self.get_readonly_transport(), 'test_lock')
+            before = time.time()
+            # wait but don't lock
+            lf2.wait(timeout=0.4, poll=0.1)
+            after = time.time()
+            self.assertTrue(after - before <= 1.0)
+        finally:
+            unlocker.join()
+
+    def test_40_confirm_easy(self):
+        """Confirm a lock that's already held"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        lf1.confirm()
+
+    def test_41_confirm_not_held(self):
+        """Confirm a lock that's already held"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        self.assertRaises(LockNotHeld, lf1.confirm)
+
+    def test_42_confirm_broken_manually(self):
+        """Confirm a lock broken by hand"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        t.move('test_lock', 'lock_gone_now')
+        self.assertRaises(LockBroken, lf1.confirm)
+
+    def test_43_break(self):
+        """Break a lock whose caller has forgotten it"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        # we incorrectly discard the lock object without unlocking it
+        del lf1
+        # someone else sees it's still locked
+        lf2 = LockDir(t, 'test_lock')
+        holder_info = lf2.peek()
+        self.assertTrue(holder_info)
+        lf2.force_break(holder_info)
+        # now we should be able to take it
+        lf2.attempt_lock()
+        lf2.confirm()
+
+    def test_44_break_already_released(self):
+        """Lock break races with regular release"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        # someone else sees it's still locked
+        lf2 = LockDir(t, 'test_lock')
+        holder_info = lf2.peek()
+        # in the interim the lock is released
+        lf1.unlock()
+        # break should succeed
+        lf2.force_break(holder_info)
+        # now we should be able to take it
+        lf2.attempt_lock()
+        lf2.confirm()
+
+    def test_45_break_mismatch(self):
+        """Lock break races with someone else acquiring it"""
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+        # someone else sees it's still locked
+        lf2 = LockDir(t, 'test_lock')
+        holder_info = lf2.peek()
+        # in the interim the lock is released
+        lf1.unlock()
+        lf3 = LockDir(t, 'test_lock')
+        lf3.attempt_lock()
+        # break should now *fail*
+        self.assertRaises(LockBreakMismatch, lf2.force_break,
+                          holder_info)
+        lf3.unlock()
+
+    def test_46_fake_read_lock(self):
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.lock_read()
+        lf1.unlock()
+
+    def test_50_lockdir_representation(self):
+        """Check the on-disk representation of LockDirs is as expected.
+
+        There should always be a top-level directory named by the lock.
+        When the lock is held, there should be a lockname/held directory 
+        containing an info file.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        self.assertTrue(t.has('test_lock'))
+        lf1.lock_write()
+        self.assertTrue(t.has('test_lock/held/info'))
+        lf1.unlock()
+        self.assertFalse(t.has('test_lock/held/info'))

=== renamed file 'bzrlib/selftest/testlog.py' => 'bzrlib/tests/test_log.py'
--- bzrlib/selftest/testlog.py	2005-07-29 13:18:36 +0000
+++ bzrlib/tests/test_log.py	2006-01-27 08:09:59 +0000
@@ -15,10 +15,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+from cStringIO import StringIO
 
-from bzrlib.selftest import BzrTestBase
-from bzrlib.log import LogFormatter, show_log, LongLogFormatter
+from bzrlib.tests import BzrTestBase, TestCaseWithTransport
+from bzrlib.log import LogFormatter, show_log, LongLogFormatter, ShortLogFormatter
 from bzrlib.branch import Branch
+from bzrlib.errors import InvalidRevisionNumber
 
 class _LogEntry(object):
     # should probably move into bzrlib.log?
@@ -47,34 +49,73 @@
         self.logs.append(le)
 
 
-class SimpleLogTest(BzrTestBase):
+class SimpleLogTest(TestCaseWithTransport):
+
     def checkDelta(self, delta, **kw):
         """Check the filenames touched by a delta are as expected."""
         for n in 'added', 'removed', 'renamed', 'modified', 'unchanged':
             expected = kw.get(n, [])
 
             # tests are written with unix paths; fix them up for windows
-            if os.sep != '/':
-                expected = [x.replace('/', os.sep) for x in expected]
+            #if os.sep != '/':
+            #    expected = [x.replace('/', os.sep) for x in expected]
 
             # strip out only the path components
             got = [x[0] for x in getattr(delta, n)]
             self.assertEquals(expected, got)
 
-    
-    def runTest(self):
+    def test_cur_revno(self):
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        lf = LogCatcher()
+        wt.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_cur_revno(self):
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        lf = LogCatcher()
+        wt.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_simple_log(self):
         eq = self.assertEquals
-        ass = self.assert_
         
-        b = Branch('.', init=True)
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
 
         lf = LogCatcher()
         show_log(b, lf)
         # no entries yet
         eq(lf.logs, [])
 
-
-        b.commit('empty commit')
+        wt.commit('empty commit')
         lf = LogCatcher()
         show_log(b, lf, verbose=True)
         eq(len(lf.logs), 1)
@@ -84,12 +125,16 @@
         self.log('log delta: %r' % d)
         self.checkDelta(d)
 
-
         self.build_tree(['hello'])
-        b.add('hello')
-        b.commit('add one file')
+        wt.add('hello')
+        wt.commit('add one file')
+
+        lf = StringIO()
         # log using regular thing
-        show_log(b, LongLogFormatter(self.TEST_LOG))
+        show_log(b, LongLogFormatter(lf))
+        lf.seek(0)
+        for l in lf.readlines():
+            self.log(l)
 
         # get log as data structure
         lf = LogCatcher()
@@ -107,3 +152,127 @@
         self.log('log 2 delta: %r' % d)
         # self.checkDelta(d, added=['hello'])
         
+        # commit a log message with control characters
+        msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
+        self.log("original commit message: %r", msg)
+        wt.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg != committed_msg)
+        self.assert_(len(committed_msg) > len(msg))
+
+        # Check that log message with only XML-valid characters isn't
+        # escaped.  As ElementTree apparently does some kind of
+        # newline conversion, neither LF (\x0A) nor CR (\x0D) are
+        # included in the test commit message, even though they are
+        # valid XML 1.0 characters.
+        msg = "\x09" + ''.join([unichr(x) for x in range(0x20, 256)])
+        self.log("original commit message: %r", msg)
+        wt.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg == committed_msg)
+
+    def test_trailing_newlines(self):
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        b.nick='test'
+        open('a', 'wb').write('hello moto\n')
+        wt.add('a')
+        wt.commit('simple log message', rev_id='a1'
+                , timestamp=1132586655.459960938, timezone=-6*3600
+                , committer='Joe Foo <joe@foo.com>')
+        open('b', 'wb').write('goodbye\n')
+        wt.add('b')
+        wt.commit('multiline\nlog\nmessage\n', rev_id='a2'
+                , timestamp=1132586842.411175966, timezone=-6*3600
+                , committer='Joe Foo <joe@foo.com>')
+
+        open('c', 'wb').write('just another manic monday\n')
+        wt.add('c')
+        wt.commit('single line with trailing newline\n', rev_id='a3'
+                , timestamp=1132587176.835228920, timezone=-6*3600
+                , committer = 'Joe Foo <joe@foo.com>')
+
+        sio = StringIO()
+        lf = ShortLogFormatter(to_file=sio)
+        show_log(b, lf)
+        self.assertEquals(sio.getvalue(), """\
+    3 Joe Foo\t2005-11-21
+      single line with trailing newline
+
+    2 Joe Foo\t2005-11-21
+      multiline
+      log
+      message
+
+    1 Joe Foo\t2005-11-21
+      simple log message
+
+""")
+
+        sio = StringIO()
+        lf = LongLogFormatter(to_file=sio)
+        show_log(b, lf)
+        self.assertEquals(sio.getvalue(), """\
+------------------------------------------------------------
+revno: 3
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:32:56 -0600
+message:
+  single line with trailing newline
+------------------------------------------------------------
+revno: 2
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:27:22 -0600
+message:
+  multiline
+  log
+  message
+------------------------------------------------------------
+revno: 1
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:24:15 -0600
+message:
+  simple log message
+""")
+        
+    def test_verbose_log(self):
+        """Verbose log includes changed files
+        
+        bug #4676
+        """
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.build_tree(['a'])
+        wt.add('a')
+        # XXX: why does a longer nick show up?
+        b.nick = 'test_verbose_log'
+        wt.commit(message='add a', 
+                  timestamp=1132711707, 
+                  timezone=36000,
+                  committer='Lorem Ipsum <test@example.com>')
+        logfile = file('out.tmp', 'w+')
+        formatter = LongLogFormatter(to_file=logfile)
+        show_log(b, formatter, verbose=True)
+        logfile.flush()
+        logfile.seek(0)
+        log_contents = logfile.read()
+        self.assertEqualDiff(log_contents, '''\
+------------------------------------------------------------
+revno: 1
+committer: Lorem Ipsum <test@example.com>
+branch nick: test_verbose_log
+timestamp: Wed 2005-11-23 12:08:27 +1000
+message:
+  add a
+added:
+  a
+''')

=== added file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_merge.py	2006-04-09 06:19:54 +0000
@@ -0,0 +1,126 @@
+import os
+from StringIO import StringIO
+
+from bzrlib.branch import Branch
+from bzrlib.builtins import merge
+from bzrlib.commit import commit
+from bzrlib.errors import UnrelatedBranches, NoCommits, BzrCommandError
+from bzrlib.merge import transform_tree, merge_inner
+from bzrlib.osutils import pathjoin
+from bzrlib.revision import common_ancestor
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.trace import (enable_test_log, disable_test_log)
+from bzrlib.workingtree import WorkingTree
+
+
+class TestMerge(TestCaseWithTransport):
+    """Test appending more than one revision"""
+
+    def test_pending(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("lala!")
+        self.assertEquals(len(wt.pending_merges()), 0)
+        merge([u'.', -1], [None, None])
+        self.assertEquals(len(wt.pending_merges()), 0)
+
+    def test_undo(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("lala!")
+        wt.commit("haha!")
+        wt.commit("blabla!")
+        merge([u'.', 2], [u'.', 1])
+
+    def test_nocommits(self):
+        self.test_pending()
+        wt2 = self.make_branch_and_tree('branch2')
+        self.assertRaises(NoCommits, merge, ['branch2', -1], 
+                          [None, None])
+        return wt2
+
+    def test_unrelated(self):
+        wt2 = self.test_nocommits()
+        wt2.commit("blah")
+        self.assertRaises(UnrelatedBranches, merge, ['branch2', -1], 
+                          [None, None])
+        return wt2
+
+    def test_merge_one(self):
+        wt1 = self.make_branch_and_tree('branch1')
+        wt1.commit('empty commit')
+        wt2 = self.make_branch_and_tree('branch2')
+        wt2.pull(wt1.branch)
+        file('branch1/foo', 'wb').write('foo')
+        file('branch1/bar', 'wb').write('bar')
+        wt1.add('foo')
+        wt1.add('bar')
+        wt1.commit('add foobar')
+        os.chdir('branch2')
+        self.run_bzr('merge', '../branch1/baz', retcode=3)
+        self.run_bzr('merge', '../branch1/foo')
+        self.failUnlessExists('foo')
+        self.failIfExists('bar')
+        wt2 = WorkingTree.open_containing('branch2')[0]
+        self.assertEqual(wt2.pending_merges(), [])
+
+    def test_pending_with_null(self):
+        """When base is forced to revno 0, pending_merges is set"""
+        wt2 = self.test_unrelated()
+        wt1 = WorkingTree.open('.')
+        br1 = wt1.branch
+        br1.fetch(wt2.branch)
+        # merge all of branch 2 into branch 1 even though they 
+        # are not related.
+        self.assertRaises(BzrCommandError, merge, ['branch2', -1], 
+                          ['branch2', 0], reprocess=True, show_base=True)
+        merge(['branch2', -1], ['branch2', 0], reprocess=True)
+        self.assertEquals(len(wt1.pending_merges()), 1)
+        return (wt1, wt2.branch)
+
+    def test_two_roots(self):
+        """Merge base is sane when two unrelated branches are merged"""
+        wt1, br2 = self.test_pending_with_null()
+        wt1.commit("blah")
+        last = wt1.branch.last_revision()
+        self.assertEquals(common_ancestor(last, last, wt1.branch.repository), last)
+
+    def test_create_rename(self):
+        """Rename an inventory entry while creating the file"""
+        tree =self.make_branch_and_tree('.')
+        file('name1', 'wb').write('Hello')
+        tree.add('name1')
+        tree.commit(message="hello")
+        tree.rename_one('name1', 'name2')
+        os.unlink('name2')
+        transform_tree(tree, tree.branch.basis_tree())
+
+    def test_layered_rename(self):
+        """Rename both child and parent at same time"""
+        tree =self.make_branch_and_tree('.')
+        os.mkdir('dirname1')
+        tree.add('dirname1')
+        filename = pathjoin('dirname1', 'name1')
+        file(filename, 'wb').write('Hello')
+        tree.add(filename)
+        tree.commit(message="hello")
+        filename2 = pathjoin('dirname1', 'name2')
+        tree.rename_one(filename, filename2)
+        tree.rename_one('dirname1', 'dirname2')
+        transform_tree(tree, tree.branch.basis_tree())
+
+    def test_ignore_zero_merge_inner(self):
+        # Test that merge_inner's ignore zero paramter is effective
+        tree_a =self.make_branch_and_tree('a')
+        tree_a.commit(message="hello")
+        dir_b = tree_a.bzrdir.sprout('b')
+        tree_b = dir_b.open_workingtree()
+        tree_a.commit(message="hello again")
+        log = StringIO()
+        merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
+                    this_tree=tree_b, ignore_zero=True)
+        log = self._get_log()
+        self.failUnless('All changes applied successfully.\n' not in log)
+        tree_b.revert([])
+        merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
+                    this_tree=tree_b, ignore_zero=False)
+        log = self._get_log()
+        self.failUnless('All changes applied successfully.\n' in log)

=== renamed file 'bzrlib/selftest/testmerge3.py' => 'bzrlib/tests/test_merge3.py'
--- bzrlib/selftest/testmerge3.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/test_merge3.py	2005-11-23 15:44:24 +0000
@@ -15,16 +15,86 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.selftest import InTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.merge3 import Merge3
-
-
-
-
-
-class NoChanges(TestCase):
-    """No conflicts because nothing changed"""
-    def runTest(self):
+from bzrlib.errors import CantReprocessAndShowBase
+
+def split_lines(t):
+    from cStringIO import StringIO
+    return StringIO(t).readlines()
+
+############################################################
+# test case data from the gnu diffutils manual
+# common base
+TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     They both may be called deep and profound.
+     Deeper and more profound,
+     The door of all subtleties!
+""")
+
+LAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+""")
+
+
+TAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+""")
+
+MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+<<<<<<< LAO
+=======
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+>>>>>>> TAO
+""")
+
+class TestMerge3(TestCase):
+
+    def test_no_changes(self):
+        """No conflicts because nothing changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', 'bbb'],
                     ['aaa', 'bbb'])
@@ -44,9 +114,7 @@
         self.assertEquals(list(m3.merge_groups()),
                           [('unchanged', ['aaa', 'bbb'])])
 
-
-class FrontInsert(TestCase):
-    def runTest(self):
+    def test_front_insert(self):
         m3 = Merge3(['zz'],
                     ['aaa', 'bbb', 'zz'],
                     ['zz'])
@@ -65,14 +133,10 @@
                           [('a', ['aaa', 'bbb']),
                            ('unchanged', ['zz'])])
         
-    
-
-class NullInsert(TestCase):
-    def runTest(self):
+    def test_null_insert(self):
         m3 = Merge3([],
                     ['aaa', 'bbb'],
                     [])
-
         # todo: should use a sentinal at end as from get_matching_blocks
         # to match without zz
         self.assertEquals(list(m3.find_sync_regions()),
@@ -83,12 +147,9 @@
 
         self.assertEquals(list(m3.merge_lines()),
                           ['aaa', 'bbb'])
-        
-    
 
-class NoConflicts(TestCase):
-    """No conflicts because only one side changed"""
-    def runTest(self):
+    def test_no_conflicts(self):
+        """No conflicts because only one side changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', 'bbb'])
@@ -106,10 +167,7 @@
                            ('a', 1, 2),
                            ('unchanged', 1, 2),])
 
-
-
-class AppendA(TestCase):
-    def runTest(self):
+    def test_append_a(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n'])
@@ -117,8 +175,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendB(TestCase):
-    def runTest(self):
+    def test_append_b(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -126,8 +183,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendAgreement(TestCase):
-    def runTest(self):
+    def test_append_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -135,8 +191,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendClash(TestCase):
-    def runTest(self):
+    def test_append_clash(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '333\n'])
@@ -157,9 +212,7 @@
 >> b
 ''')
 
-
-class InsertAgreement(TestCase):
-    def runTest(self):
+    def test_insert_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -169,15 +222,11 @@
                             start_marker='<<',
                             mid_marker='--',
                             end_marker='>>')
-        self.assertEquals(''.join(m3.merge_lines()),
-                          'aaa\n222\nbbb\n')
-
-
-
-
-class InsertClash(TestCase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+        self.assertEquals(''.join(ml), 'aaa\n222\nbbb\n')
+        
+
+    def test_insert_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '111\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -216,11 +265,8 @@
 bbb
 ''')
 
-
-
-class ReplaceClash(TestCase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+    def test_replace_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa', '000', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', '222', 'bbb'])
@@ -233,11 +279,8 @@
                            (2,3, 2,3, 2,3),
                            (3,3, 3,3, 3,3),])
 
-
-
-class ReplaceMulti(TestCase):
-    """Replacement with regions of different size."""
-    def runTest(self):
+    def test_replace_multi(self):
+        """Replacement with regions of different size."""
         m3 = Merge3(['aaa', '000', '000', 'bbb'],
                     ['aaa', '111', '111', '111', 'bbb'],
                     ['aaa', '222', '222', '222', '222', 'bbb'])
@@ -251,112 +294,33 @@
                            (3,4, 4,5, 5,6),
                            (4,4, 5,5, 6,6),])
 
-        
-        
-
-
-
-
-def split_lines(t):
-    from cStringIO import StringIO
-    return StringIO(t).readlines()
-
-
-
-############################################################
-# test case from the gnu diffutils manual
-
-# common base
-TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     They both may be called deep and profound.
-     Deeper and more profound,
-     The door of all subtleties!
-""")
-
-LAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-""")
-
-
-TAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
-""")
-
-MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-<<<<<<<< LAO
-========
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
->>>>>>>> TAO
-""")
-
-
-
-class MergePoem(TestCase):
-    """Test case from diff3 manual"""
-    def runTest(self):
+    def test_merge_poem(self):
+        """Test case from diff3 manual"""
         m3 = Merge3(TZU, LAO, TAO)
         ml = list(m3.merge_lines('LAO', 'TAO'))
         self.log('merge result:')
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
 
-
-
-TEST_CLASSES = [
-    NoChanges,
-    FrontInsert,
-    NullInsert,
-    NoConflicts,
-    AppendA,
-    AppendB,
-    AppendAgreement,
-    AppendClash,
-    InsertAgreement,
-    InsertClash,
-    ReplaceClash,
-    ReplaceMulti,
-    MergePoem,
-    ]
+    def test_minimal_conflicts(self):
+        """Reprocessing"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
+        merged_text = "".join(list(m_lines))
+        optimal_text = "a\n" * 10 + "<<<<<<< OTHER\nc\n=======\n>>>>>>> THIS"\
+            + "\n" + 8* "b\n" + "<<<<<<< OTHER\nc\n=======\nb\nb\n>>>>>>>"\
+            + " THIS\n"
+        self.assertEqualDiff(merged_text, optimal_text)
+
+    def test_reprocess_and_base(self):
+        """Reprocessing and showing base breaks correctly"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True, 
+                                 base_marker='|||||||')
+        self.assertRaises(CantReprocessAndShowBase, list, m_lines)

=== added file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_merge_core.py	2006-04-19 08:31:08 +0000
@@ -0,0 +1,658 @@
+import os
+import shutil
+import stat
+import sys
+
+import bzrlib
+from bzrlib.add import smart_add_tree
+from bzrlib.builtins import merge
+from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
+from bzrlib.errors import (NotBranchError, NotVersionedError,
+                           WorkingTreeNotRevision, BzrCommandError, NoDiff3)
+from bzrlib.inventory import RootEntry
+import bzrlib.inventory as inventory
+from bzrlib.merge import Merge3Merger, Diff3Merger, WeaveMerger
+from bzrlib.osutils import file_kind, rename, sha_file, pathjoin, mkdtemp
+from bzrlib.transform import TreeTransform
+from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
+from bzrlib.workingtree import WorkingTree
+
+
+class MergeBuilder(object):
+    def __init__(self):
+        self.dir = mkdtemp(prefix="merge-test")
+        def wt(name):
+           path = pathjoin(self.dir, name)
+           os.mkdir(path)
+           wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(path)
+           tt = TreeTransform(wt)
+           return wt, tt
+        self.base, self.base_tt = wt('base') 
+        self.this, self.this_tt = wt('this')
+        self.other, self.other_tt = wt('other')
+
+    def get_cset_path(self, parent, name):
+        if name is None:
+            assert (parent is None)
+            return None
+        return pathjoin(self.cset.entries[parent].path, name)
+
+    def add_file(self, id, parent, name, contents, executable, this=True, 
+                 base=True, other=True):
+        def new_file(tt):
+            parent_id = tt.trans_id_file_id(parent)
+            tt.new_file(name, parent_id, contents, id, executable)
+        for option, tt in self.selected_transforms(this, base, other):
+            if option is True:
+                new_file(tt)
+
+    def merge(self, merge_type=Merge3Merger, interesting_ids=None, **kwargs):
+        self.base_tt.apply()
+        self.base.commit('base commit')
+        for tt, wt in ((self.this_tt, self.this), (self.other_tt, self.other)):
+            wt.branch.pull(self.base.branch)
+            wt.set_last_revision(wt.branch.last_revision())
+            tt.apply()
+            wt.commit('branch commit')
+            assert len(wt.branch.revision_history()) == 2
+        self.this.branch.fetch(self.other.branch)
+        other_basis = self.other.branch.basis_tree()
+        merger = merge_type(self.this, self.this, self.base, other_basis, 
+                            interesting_ids=interesting_ids, **kwargs)
+        return merger.cooked_conflicts
+
+    def list_transforms(self):
+        return [self.this_tt, self.base_tt, self.other_tt]
+
+    def selected_transforms(self, this, base, other):
+        pairs = [(this, self.this_tt), (base, self.base_tt), 
+                 (other, self.other_tt)]
+        return [(v, tt) for (v, tt) in pairs if v is not None]
+
+    def add_symlink(self, id, parent, name, contents):
+        for tt in self.list_transforms():
+            parent_id = tt.trans_id_file_id(parent)
+            tt.new_symlink(name, parent_id, contents, id)
+
+    def remove_file(self, file_id, base=False, this=False, other=False):
+        for option, tt in self.selected_transforms(this, base, other):
+            if option is True:
+                trans_id = tt.trans_id_file_id(file_id)
+                tt.cancel_creation(trans_id)
+                tt.cancel_versioning(trans_id)
+                tt.set_executability(None, trans_id)
+
+    def add_dir(self, file_id, parent, name):
+        for tt in self.list_transforms():
+            parent_id = tt.trans_id_file_id(parent)
+            tt.new_directory(name, parent_id, file_id)
+
+    def change_name(self, id, base=None, this=None, other=None):
+        for val, tt in ((base, self.base_tt), (this, self.this_tt), 
+                        (other, self.other_tt)):
+            if val is None:
+                continue
+            trans_id = tt.trans_id_file_id(id)
+            parent_id = tt.final_parent(trans_id)
+            tt.adjust_path(val, parent_id, trans_id)
+
+    def change_parent(self, file_id, base=None, this=None, other=None):
+        for parent, tt in self.selected_transforms(this, base, other):
+            trans_id  = tt.trans_id_file_id(file_id)
+            parent_id = tt.trans_id_file_id(parent)
+            tt.adjust_path(tt.final_name(trans_id), parent_id, trans_id)
+
+    def change_contents(self, file_id, base=None, this=None, other=None):
+        for contents, tt in self.selected_transforms(this, base, other):
+            trans_id = tt.trans_id_file_id(file_id)
+            tt.cancel_creation(trans_id)
+            tt.create_file(contents, trans_id)
+
+    def change_target(self, id, base=None, this=None, other=None):
+        for target, tt in self.selected_transforms(this, base, other):
+            trans_id = tt.trans_id_file_id(id)
+            tt.cancel_creation(trans_id)
+            tt.create_symlink(target, trans_id)
+
+    def change_perms(self, id, base=None, this=None, other=None):
+        for executability, tt in self.selected_transforms(this, base, other):
+            trans_id = tt.trans_id_file_id(id)
+            tt.set_executability(None, trans_id)
+            tt.set_executability(executability, trans_id)
+
+    def change_perms_tree(self, id, tree, mode):
+        os.chmod(tree.full_path(id), mode)
+
+    def merge_changeset(self, merge_factory):
+        conflict_handler = changeset.ExceptionConflictHandler()
+        return make_merge_changeset(self.cset, self.this, self.base,
+                                    self.other, conflict_handler,
+                                    merge_factory)
+
+    def apply_inv_change(self, inventory_change, orig_inventory):
+        orig_inventory_by_path = {}
+        for file_id, path in orig_inventory.iteritems():
+            orig_inventory_by_path[path] = file_id
+
+        def parent_id(file_id):
+            try:
+                parent_dir = os.path.dirname(orig_inventory[file_id])
+            except:
+                print file_id
+                raise
+            if parent_dir == "":
+                return None
+            return orig_inventory_by_path[parent_dir]
+        
+        def new_path(file_id):
+            if inventory_change.has_key(file_id):
+                return inventory_change[file_id]
+            else:
+                parent = parent_id(file_id)
+                if parent is None:
+                    return orig_inventory[file_id]
+                dirname = new_path(parent)
+                return pathjoin(dirname, os.path.basename(orig_inventory[file_id]))
+
+        new_inventory = {}
+        for file_id in orig_inventory.iterkeys():
+            path = new_path(file_id)
+            if path is None:
+                continue
+            new_inventory[file_id] = path
+
+        for file_id, path in inventory_change.iteritems():
+            if orig_inventory.has_key(file_id):
+                continue
+            new_inventory[file_id] = path
+        return new_inventory
+
+    def apply_changeset(self, cset, conflict_handler=None):
+        inventory_change = changeset.apply_changeset(cset,
+                                                     self.this.inventory_dict,
+                                                     self.this.dir,
+                                                     conflict_handler)
+        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory_dict)
+
+    def cleanup(self):
+        shutil.rmtree(self.dir)
+
+
+class MergeTest(TestCase):
+
+    def test_change_name(self):
+        """Test renames"""
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.change_name("1", other="name2")
+        builder.add_file("2", "TREE_ROOT", "name3", "hello2", True)
+        builder.change_name("2", base="name4")
+        builder.add_file("3", "TREE_ROOT", "name5", "hello3", True)
+        builder.change_name("3", this="name6")
+        builder.merge()
+        builder.cleanup()
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "hello1", False)
+        builder.change_name("1", other="name2", this="name3")
+        conflicts = builder.merge()
+        self.assertEqual(conflicts, [PathConflict('name3', 'name2', '1')])
+        builder.cleanup()
+
+    def test_merge_one(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.change_contents("1", other="text4")
+        builder.add_file("2", "TREE_ROOT", "name2", "hello1", True)
+        builder.change_contents("2", other="text4")
+        builder.merge(interesting_ids=["1"])
+        self.assertEqual(builder.this.get_file("1").read(), "text4" )
+        self.assertEqual(builder.this.get_file("2").read(), "hello1" )
+        
+    def test_file_moves(self):
+        """Test moves"""
+        builder = MergeBuilder()
+        builder.add_dir("1", "TREE_ROOT", "dir1")
+        builder.add_dir("2", "TREE_ROOT", "dir2")
+        builder.add_file("3", "1", "file1", "hello1", True)
+        builder.add_file("4", "1", "file2", "hello2", True)
+        builder.add_file("5", "1", "file3", "hello3", True)
+        builder.change_parent("3", other="2")
+        builder.change_parent("4", this="2")
+        builder.change_parent("5", base="2")
+        builder.merge()
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_dir("1", "TREE_ROOT", "dir1")
+        builder.add_dir("2", "TREE_ROOT", "dir2")
+        builder.add_dir("3", "TREE_ROOT", "dir3")
+        builder.add_file("4", "1", "file1", "hello1", False)
+        builder.change_parent("4", other="2", this="3")
+        conflicts = builder.merge()
+        path2 = pathjoin('dir2', 'file1')
+        path3 = pathjoin('dir3', 'file1')
+        self.assertEqual(conflicts, [PathConflict(path3, path2, '4')])
+        builder.cleanup()
+
+    def test_contents_merge(self):
+        """Test merge3 merging"""
+        self.do_contents_test(Merge3Merger)
+
+    def test_contents_merge2(self):
+        """Test diff3 merging"""
+        try:
+            self.do_contents_test(Diff3Merger)
+        except NoDiff3:
+            raise TestSkipped("diff3 not available")
+
+    def test_contents_merge3(self):
+        """Test diff3 merging"""
+        self.do_contents_test(WeaveMerger)
+
+    def test_reprocess_weave(self):
+        # Reprocess works on weaves, and behaves as expected
+        builder = MergeBuilder()
+        builder.add_file('a', 'TREE_ROOT', 'blah', 'a', False)
+        builder.change_contents('a', this='b\nc\nd\ne\n', other='z\nc\nd\ny\n')
+        builder.merge(WeaveMerger, reprocess=True)
+        expected = """<<<<<<< TREE
+b
+=======
+z
+>>>>>>> MERGE-SOURCE
+c
+d
+<<<<<<< TREE
+e
+=======
+y
+>>>>>>> MERGE-SOURCE
+"""
+        self.assertEqualDiff(builder.this.get_file("a").read(), expected)
+
+ 
+
+    def do_contents_test(self, merge_factory):
+        """Test merging with specified ContentsChange factory"""
+        builder = self.contents_test_success(merge_factory)
+        builder.cleanup()
+        self.contents_test_conflicts(merge_factory)
+
+    def contents_test_success(self, merge_factory):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.change_contents("1", other="text4")
+        builder.add_file("2", "TREE_ROOT", "name3", "text2", False)
+        builder.change_contents("2", base="text5")
+        builder.add_file("3", "TREE_ROOT", "name5", "text3", True)
+        builder.add_file("4", "TREE_ROOT", "name6", "text4", True)
+        builder.remove_file("4", base=True)
+        builder.add_file("5", "TREE_ROOT", "name7", "a\nb\nc\nd\ne\nf\n", True)
+        builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n", 
+                                     this="a\nb\nc\nd\ne\nz\n")
+        builder.merge(merge_factory)
+        self.assertEqual(builder.this.get_file("1").read(), "text4" )
+        self.assertEqual(builder.this.get_file("2").read(), "text2" )
+        self.assertEqual(builder.this.get_file("5").read(), 
+                         "a\nz\nc\nd\ne\nz\n")
+        self.assertIs(builder.this.is_executable("1"), True)
+        self.assertIs(builder.this.is_executable("2"), False)
+        self.assertIs(builder.this.is_executable("3"), True)
+        return builder
+
+    def contents_test_conflicts(self, merge_factory):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.change_contents("1", other="text4", this="text3")
+        conflicts = builder.merge(merge_factory)
+        self.assertEqual(conflicts, [TextConflict('name1', file_id='1')])
+        builder.cleanup()
+
+    def test_symlink_conflicts(self):
+        if sys.platform != "win32":
+            builder = MergeBuilder()
+            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
+            builder.change_target("2", other="target4", base="text3")
+            conflicts = builder.merge()
+            self.assertEqual(conflicts, [ContentsConflict('name2', 
+                                                          file_id='2')])
+            builder.cleanup()
+
+    def test_symlink_merge(self):
+        if sys.platform != "win32":
+            builder = MergeBuilder()
+            builder.add_symlink("1", "TREE_ROOT", "name1", "target1")
+            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
+            builder.add_symlink("3", "TREE_ROOT", "name3", "target1")
+            builder.change_target("1", this="target2")
+            builder.change_target("2", base="target2")
+            builder.change_target("3", other="target2")
+            builder.merge()
+            self.assertEqual(builder.this.get_symlink_target("1"), "target2")
+            self.assertEqual(builder.this.get_symlink_target("2"), "target1")
+            self.assertEqual(builder.this.get_symlink_target("3"), "target2")
+            builder.cleanup()
+
+    def test_no_passive_add(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.remove_file("1", this=True)
+        builder.merge()
+        builder.cleanup()
+
+    def test_perms_merge(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.change_perms("1", other=False)
+        builder.add_file("2", "TREE_ROOT", "name2", "text2", True)
+        builder.change_perms("2", base=False)
+        builder.add_file("3", "TREE_ROOT", "name3", "text3", True)
+        builder.change_perms("3", this=False)
+        builder.add_file('4', 'TREE_ROOT', 'name4', 'text4', False)
+        builder.change_perms('4', this=True)
+        builder.remove_file('4', base=True)
+        builder.merge()
+        self.assertIs(builder.this.is_executable("1"), False)
+        self.assertIs(builder.this.is_executable("2"), True)
+        self.assertIs(builder.this.is_executable("3"), False)
+        builder.cleanup();
+
+    def test_new_suffix(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.change_contents("1", other="text3")
+        builder.add_file("2", "TREE_ROOT", "name1.new", "text2", True)
+        builder.merge()
+        os.lstat(builder.this.id2abspath("2"))
+        builder.cleanup()
+
+    def test_spurious_conflict(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "TREE_ROOT", "name1", "text1", False)
+        builder.remove_file("1", other=True)
+        builder.add_file("2", "TREE_ROOT", "name1", "text1", False, this=False, 
+                         base=False)
+        conflicts = builder.merge()
+        self.assertEqual(conflicts, []) 
+
+
+class FunctionalMergeTest(TestCaseWithTransport):
+
+    def test_trivial_star_merge(self):
+        """Test that merges in a star shape Just Work.""" 
+        # John starts a branch
+        self.build_tree(("original/", "original/file1", "original/file2"))
+        tree = self.make_branch_and_tree('original')
+        branch = tree.branch
+        smart_add_tree(tree, ["original"])
+        tree.commit("start branch.", verbose=False)
+        # Mary branches it.
+        self.build_tree(("mary/",))
+        branch.bzrdir.clone("mary")
+        # Now John commits a change
+        file = open("original/file1", "wt")
+        file.write("John\n")
+        file.close()
+        tree.commit("change file1")
+        # Mary does too
+        mary_tree = WorkingTree.open('mary')
+        mary_branch = mary_tree.branch
+        file = open("mary/file2", "wt")
+        file.write("Mary\n")
+        file.close()
+        mary_tree.commit("change file2")
+        # john should be able to merge with no conflicts.
+        merge_type = Merge3Merger
+        base = [None, None]
+        other = ("mary", -1)
+        self.assertRaises(BzrCommandError, merge, other, base, check_clean=True,
+                          merge_type=WeaveMerger, this_dir="original",
+                          show_base=True)
+        merge(other, base, check_clean=True, merge_type=merge_type,
+              this_dir="original")
+        self.assertEqual("John\n", open("original/file1", "rt").read())
+        self.assertEqual("Mary\n", open("original/file2", "rt").read())
+ 
+    def test_conflicts(self):
+        os.mkdir('a')
+        wta = self.make_branch_and_tree('a')
+        a = wta.branch
+        file('a/file', 'wb').write('contents\n')
+        wta.add('file')
+        wta.commit('base revision', allow_pointless=False)
+        d_b = a.bzrdir.clone('b')
+        b = d_b.open_branch()
+        file('a/file', 'wb').write('other contents\n')
+        wta.commit('other revision', allow_pointless=False)
+        file('b/file', 'wb').write('this contents contents\n')
+        wtb = d_b.open_workingtree()
+        wtb.commit('this revision', allow_pointless=False)
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b'), 1)
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+        self.assertRaises(WorkingTreeNotRevision, merge, ['a', -1], 
+                          [None, None], this_dir='b', check_clean=False,
+                          merge_type=WeaveMerger)
+        wtb.revert([])
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b', 
+                               check_clean=False, merge_type=WeaveMerger), 1)
+        self.assert_(os.path.lexists('b/file'))
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(not os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+
+    def test_merge_unrelated(self):
+        """Sucessfully merges unrelated branches with no common names"""
+        wta = self.make_branch_and_tree('a')
+        a = wta.branch
+        file('a/a_file', 'wb').write('contents\n')
+        wta.add('a_file')
+        wta.commit('a_revision', allow_pointless=False)
+        wtb = self.make_branch_and_tree('b')
+        b = wtb.branch
+        file('b/b_file', 'wb').write('contents\n')
+        wtb.add('b_file')
+        wtb.commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/b_file'))
+        self.assertEqual(wta.pending_merges(),
+                         [b.last_revision()]) 
+
+    def test_merge_unrelated_conflicting(self):
+        """Sucessfully merges unrelated branches with common names"""
+        wta = self.make_branch_and_tree('a')
+        a = wta.branch
+        file('a/file', 'wb').write('contents\n')
+        wta.add('file')
+        wta.commit('a_revision', allow_pointless=False)
+        wtb = self.make_branch_and_tree('b')
+        b = wtb.branch
+        file('b/file', 'wb').write('contents\n')
+        wtb.add('file')
+        wtb.commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/file'))
+        self.assert_(os.path.lexists('a/file.moved'))
+        self.assertEqual(wta.pending_merges(), [b.last_revision()])
+
+    def test_merge_deleted_conflicts(self):
+        wta = self.make_branch_and_tree('a')
+        file('a/file', 'wb').write('contents\n')
+        wta.add('file')
+        wta.commit('a_revision', allow_pointless=False)
+        self.run_bzr('branch', 'a', 'b')
+        os.remove('a/file')
+        wta.commit('removed file', allow_pointless=False)
+        file('b/file', 'wb').write('changed contents\n')
+        wtb = WorkingTree.open('b')
+        wtb.commit('changed file', allow_pointless=False)
+        merge(['a', -1], ['a', 1], this_dir='b')
+        self.failIf(os.path.lexists('b/file'))
+
+    def test_merge_metadata_vs_deletion(self):
+        """Conflict deletion vs metadata change"""
+        a_wt = self.make_branch_and_tree('a')
+        file('a/file', 'wb').write('contents\n')
+        a_wt.add('file')
+        a_wt.commit('r0')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        os.chmod('b/file', 0755)
+        os.remove('a/file')
+        a_wt.commit('removed a')
+        self.assertEqual(a_wt.branch.revno(), 2)
+        self.assertFalse(os.path.exists('a/file'))
+        b_wt.commit('exec a')
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.exists('a/file'))
+
+    def test_merge_swapping_renames(self):
+        a_wt = self.make_branch_and_tree('a')
+        file('a/un','wb').write('UN')
+        file('a/deux','wb').write('DEUX')
+        a_wt.add('un', 'un')
+        a_wt.add('deux', 'deux')
+        a_wt.commit('r0', rev_id='r0')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        b_wt.rename_one('un','tmp')
+        b_wt.rename_one('deux','un')
+        b_wt.rename_one('tmp','deux')
+        b_wt.commit('r1', rev_id='r1')
+        self.assertEqual(0, merge(['b', -1], ['b', 1], this_dir='a'))
+        self.failUnlessExists('a/un')
+        self.failUnless('a/deux')
+        self.assertFalse(os.path.exists('a/tmp'))
+        self.assertEqual(file('a/un').read(),'DEUX')
+        self.assertEqual(file('a/deux').read(),'UN')
+
+    def test_merge_delete_and_add_same(self):
+        a_wt = self.make_branch_and_tree('a')
+        file('a/file', 'wb').write('THIS')
+        a_wt.add('file')
+        a_wt.commit('r0')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        os.remove('b/file')
+        b_wt.commit('r1')
+        file('b/file', 'wb').write('THAT')
+        b_wt.add('file')
+        b_wt.commit('r2')
+        merge(['b', -1],['b', 1],this_dir='a')
+        self.assert_(os.path.exists('a/file'))
+        self.assertEqual(file('a/file').read(),'THAT')
+
+    def test_merge_rename_before_create(self):
+        """rename before create
+        
+        This case requires that you must not do creates
+        before move-into-place:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mv foo bar
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        """
+        a_wt = self.make_branch_and_tree('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        b_wt.rename_one('foo', 'bar')
+        file('b/foo', 'wb').write('B/FOO')
+        b_wt.add('foo')
+        b_wt.commit('moved foo to bar, added new foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_create_before_rename(self):
+        """create before rename, target parents before children
+
+        This case requires that you must not do move-into-place
+        before creates, and that you must not do children after
+        parents:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mkdir bar
+        $ bzr add bar
+        $ bzr mv foo bar/foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a_wt = self.make_branch_and_tree('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        os.mkdir('b/bar')
+        b_wt.add('bar')
+        b_wt.rename_one('foo', 'bar/foo')
+        b_wt.commit('created bar dir, moved foo into bar')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_rename_to_temp_before_delete(self):
+        """rename to temp before delete, source children before parents
+
+        This case requires that you must not do deletes before
+        move-out-of-the-way, and that you must not do children
+        after parents:
+        
+        $ mkdir foo
+        $ touch foo/bar
+        $ bzr add foo/bar
+        $ bzr commit
+        $ bzr mv foo/bar bar
+        $ rmdir foo
+        $ bzr commit
+        """
+        a_wt = self.make_branch_and_tree('a')
+        os.mkdir('a/foo')
+        file('a/foo/bar', 'wb').write('A/FOO/BAR')
+        a_wt.add('foo')
+        a_wt.add('foo/bar')
+        a_wt.commit('added foo/bar')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        b_wt.rename_one('foo/bar', 'bar')
+        os.rmdir('b/foo')
+        b_wt.remove('foo')
+        b_wt.commit('moved foo/bar to bar, deleted foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_delete_before_rename_to_temp(self):
+        """delete before rename to temp
+
+        This case requires that you must not do
+        move-out-of-the-way before deletes:
+        
+        $ touch foo
+        $ touch bar
+        $ bzr add foo bar
+        $ bzr commit
+        $ rm foo
+        $ bzr rm foo
+        $ bzr mv bar foo
+        $ bzr commit
+        """
+        a_wt = self.make_branch_and_tree('a')
+        file('a/foo', 'wb').write('A/FOO')
+        file('a/bar', 'wb').write('A/BAR')
+        a_wt.add('foo')
+        a_wt.add('bar')
+        a_wt.commit('added foo and bar')
+        self.run_bzr('branch', 'a', 'b')
+        b_wt = WorkingTree.open('b')
+        os.unlink('b/foo')
+        b_wt.remove('foo')
+        b_wt.rename_one('bar', 'foo')
+        b_wt.commit('deleted foo, renamed bar to foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+

=== added file 'bzrlib/tests/test_missing.py'
--- bzrlib/tests/test_missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_missing.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+
+from bzrlib.builtins import merge
+from bzrlib.missing import find_unmerged
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.workingtree import WorkingTree
+
+
+class TestMissing(TestCaseWithTransport):
+
+    def test_find_unmerged(self):
+        original_tree = self.make_branch_and_tree('original')
+        original = original_tree.branch
+        puller_tree = self.make_branch_and_tree('puller')
+        puller = puller_tree.branch
+        merger_tree = self.make_branch_and_tree('merger')
+        merger = merger_tree.branch
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        original_tree.commit('a', rev_id='a')
+        self.assertEqual(find_unmerged(original, puller), ([(1, u'a')], []))
+        puller_tree.pull(original)
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        merger_tree.pull(original)
+        original_tree.commit('b', rev_id='b')
+        original_tree.commit('c', rev_id='c')
+        self.assertEqual(find_unmerged(original, puller), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+
+        puller_tree.pull(original)
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        self.assertEqual(find_unmerged(original, merger), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+        merge(['original', -1], [None, None], this_dir='merger')
+        self.assertEqual(find_unmerged(original, merger), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+        merger_tree.commit('d', rev_id='d')
+        self.assertEqual(find_unmerged(original, merger), ([], [(2, 'd')]))

=== added file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_msgeditor.py	2006-01-24 04:44:33 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test commit message editor.
+"""
+
+import os
+import sys
+
+from bzrlib.branch import Branch
+from bzrlib.msgeditor import make_commit_message_template
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+
+
+class MsgEditorTest(TestCaseWithTransport):
+
+    def make_uncommitted_tree(self):
+        """Build a branch with uncommitted unicode named changes in the cwd."""
+        working_tree = self.make_branch_and_tree('.')
+        b = working_tree.branch
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        working_tree.add(filename)
+        return working_tree
+    
+    def test_commit_template(self):
+        """Test building a commit message template"""
+        working_tree = self.make_uncommitted_tree()
+        template = make_commit_message_template(working_tree, None)
+        self.assertEqualDiff(template,
+u"""\
+added:
+  hell\u00d8
+""")

=== added file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_nonascii.py	2006-01-24 04:44:33 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that various operations work in a non-ASCII environment."""
+
+import os
+
+from bzrlib.osutils import pathjoin
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.workingtree import WorkingTree
+
+
+class NonAsciiTest(TestCaseWithTransport):
+
+    def test_add_in_nonascii_branch(self):
+        """Test adding in a non-ASCII branch."""
+        br_dir = u"\u1234"
+        try:
+            wt = self.make_branch_and_tree(br_dir)
+        except UnicodeEncodeError:
+            raise TestSkipped("filesystem can't accomodate nonascii names")
+            return
+        file(pathjoin(br_dir, "a"), "w").write("hello")
+        wt.add(["a"], ["a-id"])

=== added file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_options.py	2006-03-22 17:48:49 +0000
@@ -0,0 +1,59 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+from bzrlib.tests import TestCase
+from bzrlib.commands import Command, parse_args
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+
+# TODO: might be nice to just parse them into a structured form and test
+# against that, rather than running the whole command.
+
+class OptionTests(TestCase):
+    """Command-line option tests"""
+
+    def test_parse_args(self):
+        """Option parser"""
+        eq = self.assertEquals
+        eq(parse_args(cmd_commit(), ['--help']),
+           ([], {'help': True}))
+        eq(parse_args(cmd_status(), ['--all']),
+           ([], {'all': True}))
+        eq(parse_args(cmd_commit(), ['--message=biter']),
+           ([], {'message': 'biter'}))
+        ## eq(parse_args(cmd_log(),  '-r 500'.split()),
+        ##   ([], {'revision': RevisionSpec_int(500)}))
+
+    def test_no_more_opts(self):
+        """Terminated options"""
+        self.assertEquals(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
+                          (['-file-with-dashes'], {}))
+
+    def test_option_help(self):
+        """Options have help strings."""
+        out, err = self.run_bzr_captured(['commit', '--help'])
+        self.assertContainsRe(out, r'--file.*file containing commit message')
+        self.assertContainsRe(out, r'--help.*-h')
+
+    def test_option_help_global(self):
+        """Global options have help strings."""
+        out, err = self.run_bzr_captured(['help', 'status'])
+        self.assertContainsRe(out, r'--show-ids.*show internal object')
+
+    def test_option_arg_help(self):
+        """Help message shows option arguments."""
+        out, err = self.run_bzr_captured(['help', 'commit'])
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, r'--file[ =]MSGFILE')
+
+    def test_unknown_short_opt(self):
+        out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
+        self.assertContainsRe(err, r'unknown short option')
+
+
+#     >>> parse_args('log -r 500'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>]})
+#     >>> parse_args('log -r500..600'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -vr500..600'.split())
+#     (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+#     (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})

=== added file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_osutils.py	2006-02-17 20:34:30 +0000
@@ -0,0 +1,92 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the osutils wrapper.
+"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.errors import BzrBadParameterNotUnicode
+import bzrlib.osutils as osutils
+from bzrlib.tests import TestCaseInTempDir, TestCase
+
+
+class TestOSUtils(TestCaseInTempDir):
+
+    def test_fancy_rename(self):
+        # This should work everywhere
+        def rename(a, b):
+            osutils.fancy_rename(a, b,
+                    rename_func=os.rename,
+                    unlink_func=os.unlink)
+
+        open('a', 'wb').write('something in a\n')
+        rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+    def test_rename(self):
+        # Rename should be semi-atomic on all platforms
+        open('a', 'wb').write('something in a\n')
+        osutils.rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        osutils.rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+    # TODO: test fancy_rename using a MemoryTransport
+
+    def test_01_rand_chars_empty(self):
+        result = osutils.rand_chars(0)
+        self.assertEqual(result, '')
+
+    def test_02_rand_chars_100(self):
+        result = osutils.rand_chars(100)
+        self.assertEqual(len(result), 100)
+        self.assertEqual(type(result), str)
+        self.assertContainsRe(result, r'^[a-z0-9]{100}$')
+
+
+class TestSafeUnicode(TestCase):
+
+    def test_from_ascii_string(self):
+        self.assertEqual(u'foobar', osutils.safe_unicode('foobar'))
+
+    def test_from_unicode_string_ascii_contents(self):
+        self.assertEqual(u'bargam', osutils.safe_unicode(u'bargam'))
+
+    def test_from_unicode_string_unicode_contents(self):
+        self.assertEqual(u'bargam\xae', osutils.safe_unicode(u'bargam\xae'))
+
+    def test_from_utf8_string(self):
+        self.assertEqual(u'foo\xae', osutils.safe_unicode('foo\xc2\xae'))
+
+    def test_bad_utf8_string(self):
+        self.assertRaises(BzrBadParameterNotUnicode,
+                          osutils.safe_unicode,
+                          '\xbb\xbb')

=== added file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_permissions.py	2006-03-06 05:01:35 +0000
@@ -0,0 +1,288 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for bzr setting permissions.
+
+Files which are created underneath .bzr/ should inherit its permissions.
+So if the directory is group writable, the files and subdirs should be as well.
+
+In the future, when we have Repository/Branch/Checkout information, the
+permissions should be inherited individually, rather than all be the same.
+"""
+
+# TODO: jam 20051215 There are no tests for ftp yet, because we have no ftp server
+# TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+#                    defined by the local umask. This isn't terrible, is it
+#                    the truly desired behavior?
+ 
+import os
+import sys
+import stat
+from StringIO import StringIO
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.transport import get_transport
+from bzrlib.workingtree import WorkingTree
+
+
+def chmod_r(base, file_mode, dir_mode):
+    """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            os.chmod(p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            os.chmod(p, file_mode)
+
+
+def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
+    """Check that all permissions match
+
+    :param test: The TestCase being run
+    :param base: The path to the root directory to check
+    :param file_mode: The mode for all files
+    :param dir_mode: The mode for all directories
+    :param include_base: If false, only check the subdirectories
+    """
+    assert os.path.isdir(base)
+    t = get_transport(".")
+    if include_base:
+        test.assertTransportMode(t, base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            test.assertTransportMode(t, p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            test.assertTransportMode(t, p, file_mode)
+
+
+class TestPermissions(TestCaseWithTransport):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('foo\n')
+        t.add('a')
+        t.commit('foo')
+
+        chmod_r('.bzr', 0644, 0755)
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # although we are modifying the filesystem
+        # underneath the objects, they are not locked, and thus it must
+        # be safe for most operations. But here we want to observe a 
+        # mode change in the control bits, which current do not refresh
+        # when a new lock is taken out.
+        t = WorkingTree.open('.')
+        b = t.branch
+        self.assertEqualMode(0755, b.control_files._dir_mode)
+        self.assertEqualMode(0644, b.control_files._file_mode)
+
+        # Modifying a file shouldn't break the permissions
+        open('a', 'wb').write('foo2\n')
+        t.commit('foo2')
+        # The mode should be maintained after commit
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # Adding a new file should maintain the permissions
+        open('b', 'wb').write('new b\n')
+        t.add('b')
+        t.commit('new b')
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 0775)
+        check_mode_r(self, '.bzr', 0664, 0775)
+        t = WorkingTree.open('.')
+        b = t.branch
+        self.assertEqualMode(0775, b.control_files._dir_mode)
+        self.assertEqualMode(0664, b.control_files._file_mode)
+
+        open('a', 'wb').write('foo3\n')
+        t.commit('foo3')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        open('c', 'wb').write('new c\n')
+        t.add('c')
+        t.commit('new c')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        # Test the group sticky bit
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 02775)
+        check_mode_r(self, '.bzr', 0664, 02775)
+        t = WorkingTree.open('.')
+        b = t.branch
+        self.assertEqualMode(02775, b.control_files._dir_mode)
+        self.assertEqualMode(0664, b.control_files._file_mode)
+
+        open('a', 'wb').write('foo4\n')
+        t.commit('foo4')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+        open('d', 'wb').write('new d\n')
+        t.add('d')
+        t.commit('new d')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+    def test_disable_set_mode(self):
+        # TODO: jam 20051215 Ultimately, this test should probably test that
+        #                    extra chmod calls aren't being made
+        try:
+            transport = get_transport(self.get_url())
+            transport.put('my-lock', StringIO(''))
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
+            self.assertNotEqual(None, lockable._dir_mode)
+            self.assertNotEqual(None, lockable._file_mode)
+
+            LockableFiles._set_dir_mode = False
+            transport = get_transport('.')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
+            self.assertEqual(None, lockable._dir_mode)
+            self.assertNotEqual(None, lockable._file_mode)
+
+            LockableFiles._set_file_mode = False
+            transport = get_transport('.')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
+            self.assertEqual(None, lockable._dir_mode)
+            self.assertEqual(None, lockable._file_mode)
+
+            LockableFiles._set_dir_mode = True
+            transport = get_transport('.')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
+            self.assertNotEqual(None, lockable._dir_mode)
+            self.assertEqual(None, lockable._file_mode)
+
+            LockableFiles._set_file_mode = True
+            transport = get_transport('.')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
+            self.assertNotEqual(None, lockable._dir_mode)
+            self.assertNotEqual(None, lockable._file_mode)
+        finally:
+            LockableFiles._set_dir_mode = True
+            LockableFiles._set_file_mode = True
+
+
+class TestSftpPermissions(TestCaseWithSFTPServer):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        # Though it would be nice to test that SFTP to a server
+        # which does support chmod has the right effect
+
+        # bodge around for stubsftpserver not letting use connect
+        # more than once
+        _t = get_transport(self.get_url())
+
+        os.mkdir('local')
+        t_local = self.make_branch_and_tree('local')
+        b_local = t_local.branch
+        open('local/a', 'wb').write('foo\n')
+        t_local.add('a')
+        t_local.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        chmod_r('local/.bzr', 0644, 0755)
+        check_mode_r(self, 'local/.bzr', 0644, 0755)
+
+        t = WorkingTree.open('local')
+        b_local = t.branch
+        self.assertEqualMode(0755, b_local.control_files._dir_mode)
+        self.assertEqualMode(0644, b_local.control_files._file_mode)
+
+        os.mkdir('sftp')
+        sftp_url = self.get_url('sftp')
+        b_sftp = BzrDir.create_branch_and_repo(sftp_url)
+
+        b_sftp.pull(b_local)
+        del b_sftp
+        chmod_r('sftp/.bzr', 0644, 0755)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        b_sftp = Branch.open(sftp_url)
+        self.assertEqualMode(0755, b_sftp.control_files._dir_mode)
+        self.assertEqualMode(0644, b_sftp.control_files._file_mode)
+
+        open('local/a', 'wb').write('foo2\n')
+        t_local.commit('foo2')
+        b_sftp.pull(b_local)
+        # The mode should be maintained after commit
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        open('local/b', 'wb').write('new b\n')
+        t_local.add('b')
+        t_local.commit('new b')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        del b_sftp
+        # Recursively update the modes of all files
+        chmod_r('sftp/.bzr', 0664, 0775)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        b_sftp = Branch.open(sftp_url)
+        self.assertEqualMode(0775, b_sftp.control_files._dir_mode)
+        self.assertEqualMode(0664, b_sftp.control_files._file_mode)
+
+        open('local/a', 'wb').write('foo3\n')
+        t_local.commit('foo3')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        open('local/c', 'wb').write('new c\n')
+        t_local.add('c')
+        t_local.commit('new c')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+    def test_sftp_server_modes(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        umask = 0022
+        original_umask = os.umask(umask)
+
+        try:
+            t = get_transport(self.get_url())
+            # Direct access should be masked by umask
+            t._sftp_open_exclusive('a', mode=0666).write('foo\n')
+            self.assertTransportMode(t, 'a', 0666 &~umask)
+
+            # but Transport overrides umask
+            t.put('b', 'txt', mode=0666)
+            self.assertTransportMode(t, 'b', 0666)
+
+            t._sftp.mkdir('c', mode=0777)
+            self.assertTransportMode(t, 'c', 0777 &~umask)
+
+            t.mkdir('d', mode=0777)
+            self.assertTransportMode(t, 'd', 0777)
+        finally:
+            os.umask(original_umask)

=== renamed file 'bzrlib/selftest/plugins.py' => 'bzrlib/tests/test_plugins.py'
--- bzrlib/selftest/plugins.py	2005-08-17 23:31:01 +0000
+++ bzrlib/tests/test_plugins.py	2005-12-16 03:43:14 +0000
@@ -17,51 +17,36 @@
 
 """Tests for plugins"""
 
-# NOT RUN YET
-
-
-
-
-
-
-
-from bzrlib.selftest import InTempDir
-
-
-def PluginTest(InTempDir):
+# XXX: There are no plugin tests at the moment because the plugin module
+# affects the global state of the process.  See bzrlib/plugins.py for more
+# comments.
+
+import os
+
+import bzrlib.plugin
+import bzrlib.plugins
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.osutils import pathjoin, abspath
+
+class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
-    def runTest(self):
-        import os
-        
-        orig_help = self.backtick('bzr help commands') # No plugins yet
-        os.mkdir('plugin_test')
-        f = open(os.path.join('plugin_test', 'myplug.py'), 'wt')
-        f.write(PLUGIN_TEXT)
-        f.close()
-
-        newhelp = backtick('bzr help commands')
-        assert newhelp.startswith('You have been overridden\n')
-        # We added a line, but the rest should work
-        assert newhelp[25:] == help
-
-        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
-
-        shutil.rmtree('plugin_test')
-
-
-
-
-#         PLUGIN_TEXT = \
-#         """import bzrlib, bzrlib.commands
-#         class cmd_myplug(bzrlib.commands.Command):
-#             '''Just a simple test plugin.'''
-#             aliases = ['mplg']
-#             def run(self):
-#                 print 'Hello from my plugin'
-#         """
-#         f.close()
-
-#         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
+#    def test_plugin_loading(self):
+#        orig_help = self.run_bzr_captured('bzr help commands')[0]
+#        os.mkdir('plugin_test')
+#        f = open(pathjoin('plugin_test', 'myplug.py'), 'wt')
+#        f.write(PLUGIN_TEXT)
+#        f.close()
+#        newhelp = self.run_bzr_captured('bzr help commands')[0]
+#        assert newhelp.startswith('You have been overridden\n')
+#        # We added a line, but the rest should work
+#        assert newhelp[25:] == help
+#
+#        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
+#
+#        shutil.rmtree('plugin_test')
+#
+
+#         os.environ['BZRPLUGINPATH'] = abspath('plugin_test')
 #         help = backtick('bzr help commands')
 #         assert help.find('myplug') != -1
 #         assert help.find('Just a simple test plugin.') != -1
@@ -70,7 +55,7 @@
 #         assert backtick('bzr myplug') == 'Hello from my plugin\n'
 #         assert backtick('bzr mplg') == 'Hello from my plugin\n'
 
-#         f = open(os.path.join('plugin_test', 'override.py'), 'wb')
+#         f = open(pathjoin('plugin_test', 'override.py'), 'wb')
 #         f.write("""import bzrlib, bzrlib.commands
 #     class cmd_commit(bzrlib.commands.cmd_commit):
 #         '''Commit changes into a new revision.'''
@@ -84,3 +69,71 @@
 #             bzrlib.commands.cmd_help.run(self, *args, **kwargs)
 
 #         """
+
+PLUGIN_TEXT = """\
+import bzrlib.commands
+class cmd_myplug(bzrlib.commands.Command):
+    '''Just a simple test plugin.'''
+    aliases = ['mplg']
+    def run(self):
+        print 'Hello from my plugin'
+"""
+
+# TODO: Write a test for plugin decoration of commands.
+
+class TestOneNamedPluginOnly(TestCaseInTempDir):
+
+    activeattributes = {}
+
+    def test_plugins_with_the_same_name_are_not_loaded(self):
+        # This test tests that having two plugins in different
+        # directories does not result in both being loaded.
+        # get a file name we can use which is also a valid attribute
+        # for accessing in activeattributes. - we cannot give import parameters.
+        tempattribute = "0"
+        self.failIf(tempattribute in self.activeattributes)
+        # set a place for the plugins to record their loading, and at the same
+        # time validate that the location the plugins should record to is
+        # valid and correct.
+        bzrlib.tests.test_plugins.TestOneNamedPluginOnly.activeattributes \
+            [tempattribute] = []
+        self.failUnless(tempattribute in self.activeattributes)
+        # create two plugin directories
+        os.mkdir('first')
+        os.mkdir('second')
+        # write a plugin that will record when its loaded in the 
+        # tempattribute list.
+        template = ("from bzrlib.tests.test_plugins import TestOneNamedPluginOnly\n"
+                    "TestOneNamedPluginOnly.activeattributes[%r].append('%s')\n")
+        print >> file(os.path.join('first', 'plugin.py'), 'w'), template % (tempattribute, 'first')
+        print >> file(os.path.join('second', 'plugin.py'), 'w'), template % (tempattribute, 'second')
+        try:
+            bzrlib.plugin.load_from_dirs(['first', 'second'])
+            self.assertEqual(['first'], self.activeattributes[tempattribute])
+        finally:
+            # remove the plugin 'plugin'
+            del self.activeattributes[tempattribute]
+            if getattr(bzrlib.plugins, 'plugin', None):
+                del bzrlib.plugins.plugin
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+
+
+class TestAllPlugins(TestCaseInTempDir):
+
+    def test_plugin_appears_in_all_plugins(self):
+        # This test tests a new plugin appears in bzrlib.plugin.all_plugins().
+        # check the plugin is not loaded already
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+        # write a plugin that _cannot_ fail to load.
+        print >> file('plugin.py', 'w'), ""
+        try:
+            bzrlib.plugin.load_from_dirs(['.'])
+            self.failUnless('plugin' in bzrlib.plugin.all_plugins())
+            self.failUnless(getattr(bzrlib.plugins, 'plugin', None))
+            self.assertEqual(bzrlib.plugin.all_plugins()['plugin'],
+                             bzrlib.plugins.plugin)
+        finally:
+            # remove the plugin 'plugin'
+            if getattr(bzrlib.plugins, 'plugin', None):
+                del bzrlib.plugins.plugin
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))

=== added file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_progress.py	2006-03-08 19:51:44 +0000
@@ -0,0 +1,92 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from StringIO import StringIO
+
+from bzrlib.progress import *
+from bzrlib.tests import TestCase
+
+class FakeStack:
+    def __init__(self, top):
+        self.__top = top
+
+    def top(self):
+        return self.__top
+
+class TestProgress(TestCase):
+    def setUp(self):
+        q = DummyProgress()
+        self.top = ChildProgress(_stack=FakeStack(q))
+
+    def test_propogation(self):
+        self.top.update('foobles', 1, 2)
+        self.assertEqual(self.top.message, 'foobles')
+        self.assertEqual(self.top.current, 1)
+        self.assertEqual(self.top.total, 2)
+        self.assertEqual(self.top.child_fraction, 0)
+        child = ChildProgress(_stack=FakeStack(self.top))
+        child.update('baubles', 2, 4)
+        self.assertEqual(self.top.message, 'foobles')
+        self.assertEqual(self.top.current, 1)
+        self.assertEqual(self.top.total, 2)
+        self.assertEqual(self.top.child_fraction, 0.5)
+        grandchild = ChildProgress(_stack=FakeStack(child))
+        grandchild.update('barbells', 1, 2)
+        self.assertEqual(self.top.child_fraction, 0.625)
+        self.assertEqual(child.child_fraction, 0.5)
+        child.update('baubles', 3, 4)
+        self.assertEqual(child.child_fraction, 0)
+        self.assertEqual(self.top.child_fraction, 0.75)
+        grandchild.update('barbells', 1, 2)
+        self.assertEqual(self.top.child_fraction, 0.875)
+        grandchild.update('barbells', 2, 2)
+        self.assertEqual(self.top.child_fraction, 1)
+        child.update('baubles', 4, 4)
+        self.assertEqual(self.top.child_fraction, 1)
+        #test clamping
+        grandchild.update('barbells', 2, 2)
+        self.assertEqual(self.top.child_fraction, 1)
+
+    def test_implementations(self):
+        for implementation in (TTYProgressBar, DotsProgressBar, 
+                               DummyProgress):
+            self.check_parent_handling(implementation)
+
+    def check_parent_handling(self, parentclass):
+        top = parentclass(to_file=StringIO())
+        top.update('foobles', 1, 2)
+        child = ChildProgress(_stack=FakeStack(top))
+        child.update('baubles', 4, 4)
+        top.update('lala', 2, 2)
+        child.update('baubles', 4, 4)
+
+    def test_stacking(self):
+        self.check_stack(TTYProgressBar, ChildProgress)
+        self.check_stack(DotsProgressBar, ChildProgress)
+        self.check_stack(DummyProgress, DummyProgress)
+
+    def check_stack(self, parent_class, child_class):
+        stack = ProgressBarStack(klass=parent_class, to_file=StringIO())
+        parent = stack.get_nested()
+        try:
+            self.assertIs(parent.__class__, parent_class)
+            child = stack.get_nested()
+            try:
+                self.assertIs(child.__class__, child_class)
+            finally:
+                child.finished()
+        finally:
+            parent.finished()

=== added file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_reconcile.py	2006-02-25 05:48:54 +0000
@@ -0,0 +1,47 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for reconiliation behaviour that is repository independent."""
+
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.reconcile import reconcile, Reconciler
+from bzrlib.revision import Revision
+from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
+from bzrlib.transport import get_transport
+from bzrlib.tree import EmptyTree
+from bzrlib.workingtree import WorkingTree
+
+
+class TestWorksWithSharedRepositories(TestCaseWithRepository):
+
+    def test_reweave_empty(self):
+        # we want a repo capable format
+        parent = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('.')
+        parent.create_repository(shared=True)
+        parent.root_transport.mkdir('child')
+        child = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('child')
+        self.assertRaises(errors.NoRepositoryPresent, child.open_repository)
+        reconciler = Reconciler(child)
+        reconciler.reconcile()
+        # smoke test for reconcile appears to work too.
+        reconcile(child)
+        # no inconsistent parents should have been found
+        # but the values should have been set.
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)

=== added file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_repository.py	2006-04-13 09:12:14 +0000
@@ -0,0 +1,431 @@
+# (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the Repository facility that are not interface tests.
+
+For interface tests see tests/repository_implementations/*.py.
+
+For concrete class tests see this file, and for storage formats tests
+also see this file.
+"""
+
+from stat import *
+from StringIO import StringIO
+
+import bzrlib
+import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
+from bzrlib.errors import (NotBranchError,
+                           NoSuchFile,
+                           UnknownFormatError,
+                           UnsupportedFormatError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import HttpServer
+from bzrlib.transport.memory import MemoryServer
+
+
+class TestDefaultFormat(TestCase):
+
+    def test_get_set_default_format(self):
+        old_format = repository.RepositoryFormat.get_default_format()
+        # default is None - we cannot create a Repository independently yet
+        self.assertTrue(isinstance(old_format, repository.RepositoryFormat7))
+        repository.RepositoryFormat.set_default_format(SampleRepositoryFormat())
+        # creating a repository should now create an instrumented dir.
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = bzrdir.BzrDirMetaFormat1().initialize('memory:/')
+            result = dir.create_repository()
+            self.assertEqual(result, 'A bzr repository dir')
+        finally:
+            repository.RepositoryFormat.set_default_format(old_format)
+        self.assertEqual(old_format, repository.RepositoryFormat.get_default_format())
+
+
+class SampleRepositoryFormat(repository.RepositoryFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open(unsupported=True) routines.
+    """
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Sample .bzr repository format."
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Initialize a repository in a BzrDir"""
+        t = a_bzrdir.get_repository_transport(self)
+        t.put('format', StringIO(self.get_format_string()))
+        return 'A bzr repository dir'
+
+    def is_supported(self):
+        return False
+
+    def open(self, a_bzrdir, _found=False):
+        return "opened repository."
+
+
+class TestRepositoryFormat(TestCaseWithTransport):
+    """Tests for the Repository format detection used by the bzr meta dir facility.BzrBranchFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a repository?
+        # create a branch with a few known format objects.
+        # this is not quite the same as 
+        self.build_tree(["foo/", "bar/"])
+        def check_format(format, url):
+            dir = format._matchingbzrdir.initialize(url)
+            format.initialize(dir)
+            t = get_transport(url)
+            found_format = repository.RepositoryFormat.find_format(dir)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(repository.RepositoryFormat7(), "bar")
+        
+    def test_find_format_no_repository(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        self.assertRaises(errors.NoRepositoryPresent,
+                          repository.RepositoryFormat.find_format,
+                          dir)
+
+    def test_find_format_unknown_format(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        SampleRepositoryFormat().initialize(dir)
+        self.assertRaises(UnknownFormatError,
+                          repository.RepositoryFormat.find_format,
+                          dir)
+
+    def test_register_unregister_format(self):
+        format = SampleRepositoryFormat()
+        # make a control dir
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        # make a repo
+        format.initialize(dir)
+        # register a format for it.
+        repository.RepositoryFormat.register_format(format)
+        # which repository.Open will refuse (not supported)
+        self.assertRaises(UnsupportedFormatError, repository.Repository.open, self.get_url())
+        # but open(unsupported) will work
+        self.assertEqual(format.open(dir), "opened repository.")
+        # unregister the format
+        repository.RepositoryFormat.unregister_format(format)
+
+
+class TestFormat6(TestCaseWithTransport):
+
+    def test_no_ancestry_weave(self):
+        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
+        repo = repository.RepositoryFormat6().initialize(control)
+        # We no longer need to create the ancestry.weave file
+        # since it is *never* used.
+        self.assertRaises(NoSuchFile,
+                          control.transport.get,
+                          'ancestry.weave')
+
+
+class TestFormat7(TestCaseWithTransport):
+    
+    def test_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormat7().initialize(control)
+        # in case of side effects of locking.
+        repo.lock_write()
+        repo.unlock()
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # lock ''
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+
+    def test_shared_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        # lock is not present when unlocked
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+        self.assertFalse(t.has('branch-lock'))
+
+    def test_creates_lockdir(self):
+        """Make sure it appears to be controlled by a LockDir existence"""
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        # TODO: Should check there is a 'lock' toplevel directory, 
+        # regardless of contents
+        self.assertFalse(t.has('lock/held/info'))
+        repo.lock_write()
+        try:
+            self.assertTrue(t.has('lock/held/info'))
+        finally:
+            # unlock so we don't get a warning about failing to do so
+            repo.unlock()
+
+    def test_uses_lockdir(self):
+        """repo format 7 actually locks on lockdir"""
+        base_url = self.get_url()
+        control = bzrdir.BzrDirMetaFormat1().initialize(base_url)
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        repo.lock_write()
+        repo.unlock()
+        del repo
+        # make sure the same lock is created by opening it
+        repo = repository.Repository.open(base_url)
+        repo.lock_write()
+        self.assertTrue(t.has('lock/held/info'))
+        repo.unlock()
+        self.assertFalse(t.has('lock/held/info'))
+
+    def test_shared_no_tree_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        repo.set_make_working_trees(False)
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # lock ''
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        ## self.assertEqualDiff('', t.get('lock').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertEqualDiff('', t.get('no-working-trees').read())
+        repo.set_make_working_trees(True)
+        self.assertFalse(t.has('no-working-trees'))
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+
+
+class TestFormatKnit1(TestCaseWithTransport):
+    
+    def test_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormatKnit1().initialize(control)
+        # in case of side effects of locking.
+        repo.lock_write()
+        repo.unlock()
+        # we want:
+        # format 'Bazaar-NG Knit Repository Format 1'
+        # lock: is a directory
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
+                             t.get('format').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
+        self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
+        self.check_knits(t)
+
+    def assertHasKnit(self, t, knit_name):
+        """Assert that knit_name exists on t."""
+        self.assertEqualDiff('# bzr knit index 7\n',
+                             t.get(knit_name + '.kndx').read())
+        # no default content
+        self.assertTrue(t.has(knit_name + '.knit'))
+
+    def check_knits(self, t):
+        """check knit content for a repository."""
+        self.assertHasKnit(t, 'inventory')
+        self.assertHasKnit(t, 'revisions')
+        self.assertHasKnit(t, 'signatures')
+
+    def test_shared_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormatKnit1().initialize(control, shared=True)
+        # we want:
+        # format 'Bazaar-NG Knit Repository Format 1'
+        # lock: is a directory
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
+                             t.get('format').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
+        self.check_knits(t)
+
+    def test_shared_no_tree_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormatKnit1().initialize(control, shared=True)
+        repo.set_make_working_trees(False)
+        # we want:
+        # format 'Bazaar-NG Knit Repository Format 1'
+        # lock ''
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
+                             t.get('format').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertEqualDiff('', t.get('no-working-trees').read())
+        repo.set_make_working_trees(True)
+        self.assertFalse(t.has('no-working-trees'))
+        self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
+        self.check_knits(t)
+
+
+class InterString(repository.InterRepository):
+    """An inter-repository optimised code path for strings.
+
+    This is for use during testing where we use strings as repositories
+    so that none of the default regsitered inter-repository classes will
+    match.
+    """
+
+    @staticmethod
+    def is_compatible(repo_source, repo_target):
+        """InterString is compatible with strings-as-repos."""
+        return isinstance(repo_source, str) and isinstance(repo_target, str)
+
+
+class TestInterRepository(TestCaseWithTransport):
+
+    def test_get_default_inter_repository(self):
+        # test that the InterRepository.get(repo_a, repo_b) probes
+        # for a inter_repo class where is_compatible(repo_a, repo_b) returns
+        # true and returns a default inter_repo otherwise.
+        # This also tests that the default registered optimised interrepository
+        # classes do not barf inappropriately when a surprising repository type
+        # is handed to them.
+        dummy_a = "Repository 1."
+        dummy_b = "Repository 2."
+        self.assertGetsDefaultInterRepository(dummy_a, dummy_b)
+
+    def assertGetsDefaultInterRepository(self, repo_a, repo_b):
+        """Asserts that InterRepository.get(repo_a, repo_b) -> the default."""
+        inter_repo = repository.InterRepository.get(repo_a, repo_b)
+        self.assertEqual(repository.InterRepository,
+                         inter_repo.__class__)
+        self.assertEqual(repo_a, inter_repo.source)
+        self.assertEqual(repo_b, inter_repo.target)
+
+    def test_register_inter_repository_class(self):
+        # test that a optimised code path provider - a
+        # InterRepository subclass can be registered and unregistered
+        # and that it is correctly selected when given a repository
+        # pair that it returns true on for the is_compatible static method
+        # check
+        dummy_a = "Repository 1."
+        dummy_b = "Repository 2."
+        repository.InterRepository.register_optimiser(InterString)
+        try:
+            # we should get the default for something InterString returns False
+            # to
+            self.assertFalse(InterString.is_compatible(dummy_a, None))
+            self.assertGetsDefaultInterRepository(dummy_a, None)
+            # and we should get an InterString for a pair it 'likes'
+            self.assertTrue(InterString.is_compatible(dummy_a, dummy_b))
+            inter_repo = repository.InterRepository.get(dummy_a, dummy_b)
+            self.assertEqual(InterString, inter_repo.__class__)
+            self.assertEqual(dummy_a, inter_repo.source)
+            self.assertEqual(dummy_b, inter_repo.target)
+        finally:
+            repository.InterRepository.unregister_optimiser(InterString)
+        # now we should get the default InterRepository object again.
+        self.assertGetsDefaultInterRepository(dummy_a, dummy_b)
+
+
+class TestInterWeaveRepo(TestCaseWithTransport):
+
+    def test_is_compatible_and_registered(self):
+        # InterWeaveRepo is compatible when either side
+        # is a format 5/6/7 branch
+        formats = [repository.RepositoryFormat5(),
+                   repository.RepositoryFormat6(),
+                   repository.RepositoryFormat7()]
+        incompatible_formats = [repository.RepositoryFormat4(),
+                                repository.RepositoryFormatKnit1(),
+                                ]
+        repo_a = self.make_repository('a')
+        repo_b = self.make_repository('b')
+        is_compatible = repository.InterWeaveRepo.is_compatible
+        for source in incompatible_formats:
+            # force incompatible left then right
+            repo_a._format = source
+            repo_b._format = formats[0]
+            self.assertFalse(is_compatible(repo_a, repo_b))
+            self.assertFalse(is_compatible(repo_b, repo_a))
+        for source in formats:
+            repo_a._format = source
+            for target in formats:
+                repo_b._format = target
+                self.assertTrue(is_compatible(repo_a, repo_b))
+        self.assertEqual(repository.InterWeaveRepo,
+                         repository.InterRepository.get(repo_a,
+                                                        repo_b).__class__)
+
+
+class TestRepositoryConverter(TestCaseWithTransport):
+
+    def test_convert_empty(self):
+        t = get_transport(self.get_url('.'))
+        t.mkdir('repository')
+        repo_dir = bzrdir.BzrDirMetaFormat1().initialize('repository')
+        repo = repository.RepositoryFormat7().initialize(repo_dir)
+        target_format = repository.RepositoryFormatKnit1()
+        converter = repository.CopyConverter(target_format)
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            converter.convert(repo, pb)
+        finally:
+            pb.finished()
+        repo = repo_dir.open_repository()
+        self.assertTrue(isinstance(target_format, repo._format.__class__))

=== renamed file 'bzrlib/selftest/testrevision.py' => 'bzrlib/tests/test_revision.py'
--- bzrlib/selftest/testrevision.py	2005-08-04 21:39:23 +0000
+++ bzrlib/tests/test_revision.py	2006-04-12 04:57:34 +0000
@@ -14,52 +14,318 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
-
-
-def make_branches():
-    from bzrlib.branch import Branch
-    from bzrlib.commit import commit
-    import os
-    os.mkdir("branch1")
-    br1 = Branch("branch1", init=True)
-    commit(br1, "Commit one")
-    commit(br1, "Commit two")
-    commit(br1, "Commit three")
-
-    os.mkdir("branch2")
-    br2 = Branch("branch2", init=True)
-    br2.update_revisions(br1)
-    commit(br2, "Commit four")
-    commit(br2, "Commit five")
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.errors import NoSuchRevision
+from bzrlib.commit import commit
+from bzrlib.graph import Graph
+from bzrlib.revision import (find_present_ancestors, combined_graph,
+                             common_ancestor,
+                             is_ancestor, MultipleRevisionSources)
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.trace import mutter
+from bzrlib.workingtree import WorkingTree
+
+# XXX: Make this a method of a merge base case
+def make_branches(self):
+    """Create two branches
+
+    branch 1 has 6 commits, branch 2 has 3 commits
+    commit 10 is a ghosted merge merge from branch 1
+
+    the object graph is
+    B:     A:
+    a..0   a..0 
+    a..1   a..1
+    a..2   a..2
+    b..3   a..3 merges b..4
+    b..4   a..4
+    b..5   a..5 merges b..5
+    b..6 merges a4
+
+    so A is missing b6 at the start
+    and B is missing a3, a4, a5
+    """
+    tree1 = self.make_branch_and_tree("branch1")
+    br1 = tree1.branch
+    
+    tree1.commit("Commit one", rev_id="a@u-0-0")
+    tree1.commit("Commit two", rev_id="a@u-0-1")
+    tree1.commit("Commit three", rev_id="a@u-0-2")
+
+    tree2 = tree1.bzrdir.clone("branch2").open_workingtree()
+    br2 = tree2.branch
+    tree2.commit("Commit four", rev_id="b@u-0-3")
+    tree2.commit("Commit five", rev_id="b@u-0-4")
     revisions_2 = br2.revision_history()
-    br1.add_pending_merge(revisions_2[4])
-    commit(br1, "Commit six")
+    
+    br1.fetch(br2)
+    tree1.add_pending_merge(revisions_2[4])
+    self.assertEquals(revisions_2[4], 'b@u-0-4')
+    tree1.commit("Commit six", rev_id="a@u-0-3")
+    tree1.commit("Commit seven", rev_id="a@u-0-4")
+    tree2.commit("Commit eight", rev_id="b@u-0-5")
+    
+    br1.fetch(br2)
+    tree1.add_pending_merge(br2.revision_history()[5])
+    tree1.commit("Commit nine", rev_id="a@u-0-5")
+    # DO NOT FETCH HERE - we WANT a GHOST.
+    # br2.fetch(br1)
+    tree2.add_pending_merge(br1.revision_history()[4])
+    tree2.commit("Commit ten - ghost merge", rev_id="b@u-0-6")
+    
     return br1, br2
 
 
-class TestIsAncestor(InTempDir):
+class TestIsAncestor(TestCaseWithTransport):
+
+    def test_recorded_ancestry(self):
+        """Test that commit records all ancestors"""
+        br1, br2 = make_branches(self)
+        d = [('a@u-0-0', ['a@u-0-0']),
+             ('a@u-0-1', ['a@u-0-0', 'a@u-0-1']),
+             ('a@u-0-2', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2']),
+             ('b@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3']),
+             ('b@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3',
+                          'b@u-0-4']),
+             ('a@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3']),
+             ('a@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3', 'a@u-0-4']),
+             ('b@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5']),
+             ('a@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'a@u-0-5']),
+             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'b@u-0-6']),
+             ]
+        br1_only = ('a@u-0-3', 'a@u-0-4', 'a@u-0-5')
+        br2_only = ('b@u-0-6',)
+        for branch in br1, br2:
+            for rev_id, anc in d:
+                if rev_id in br1_only and not branch is br1:
+                    continue
+                if rev_id in br2_only and not branch is br2:
+                    continue
+                mutter('ancestry of {%s}: %r',
+                       rev_id, branch.repository.get_ancestry(rev_id))
+                result = sorted(branch.repository.get_ancestry(rev_id))
+                self.assertEquals(result, [None] + sorted(anc))
+    
+    
+    def test_is_ancestor(self):
+        """Test checking whether a revision is an ancestor of another revision"""
+        br1, br2 = make_branches(self)
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = br1
+
+        self.assert_(is_ancestor(revisions[0], revisions[0], br1))
+        self.assert_(is_ancestor(revisions[1], revisions[0], sources))
+        self.assert_(not is_ancestor(revisions[0], revisions[1], sources))
+        self.assert_(is_ancestor(revisions_2[3], revisions[0], sources))
+        # disabled mbp 20050914, doesn't seem to happen anymore
+        ## self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
+        ##                  revisions[0], br1)        
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], sources))
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], br1))
+        self.assert_(is_ancestor(revisions[3], revisions_2[3], sources))
+        ## self.assert_(not is_ancestor(revisions[3], revisions_2[3], br1))
+
+
+class TestIntermediateRevisions(TestCaseWithTransport):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseWithTransport.setUp(self)
+        self.br1, self.br2 = make_branches(self)
+        wt1 = self.br1.bzrdir.open_workingtree()
+        wt2 = self.br2.bzrdir.open_workingtree()
+        wt2.commit("Commit eleven", rev_id="b@u-0-7")
+        wt2.commit("Commit twelve", rev_id="b@u-0-8")
+        wt2.commit("Commit thirtteen", rev_id="b@u-0-9")
+
+        self.br1.fetch(self.br2)
+        wt1.add_pending_merge(self.br2.revision_history()[6])
+        wt1.commit("Commit fourtten", rev_id="a@u-0-6")
+
+        self.br2.fetch(self.br1)
+        wt2.add_pending_merge(self.br1.revision_history()[6])
+        wt2.commit("Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1.repository,
+                                               self.br2.repository)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        self.assertEquals(len(self.intervene('a@u-0-0', 'a@u-0-0')), 0)
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
+
+class MockRevisionSource(object):
+    """A RevisionSource that takes a pregenerated graph.
+
+    This is useful for testing revision graph algorithms where
+    the actual branch existing is irrelevant.
+    """
+
+    def __init__(self, full_graph):
+        self._full_graph = full_graph
+
+    def get_revision_graph_with_ghosts(self, revision_ids):
+        # This is mocked out to just return a constant graph.
+        return self._full_graph
+
+
+class TestCommonAncestor(TestCaseWithTransport):
     """Test checking whether a revision is an ancestor of another revision"""
-    def runTest(self):
-        from bzrlib.revision import is_ancestor, MultipleRevisionSources
-        from bzrlib.errors import NoSuchRevision
-        br1, br2 = make_branches()
+
+    def test_common_ancestor(self):
+        """Pick a reasonable merge base"""
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
-        sources = MultipleRevisionSources(br1, br2)
-
-        assert is_ancestor(revisions[0], revisions[0], sources)
-        assert is_ancestor(revisions[1], revisions[0], sources)
-        assert not is_ancestor(revisions[0], revisions[1], sources)
-        assert is_ancestor(revisions_2[3], revisions[0], sources)
-        self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
-                          revisions[0], br1)        
-        assert is_ancestor(revisions[3], revisions_2[4], sources)
-        assert is_ancestor(revisions[3], revisions_2[4], br1)
-        assert is_ancestor(revisions[3], revisions_2[3], sources)
-        assert not is_ancestor(revisions[3], revisions_2[3], br1)
-
-
-TEST_CLASSES = [
-    TestIsAncestor,
-    ]
+        sources = MultipleRevisionSources(br1.repository, br2.repository)
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        self.assertEquals(len(expected_ancestors_list), len(ancestors_list))
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertTrue(common_ancestor(revisions[5], revisions_2[6], sources) in
+                        (revisions[4], revisions_2[5]))
+        self.assertTrue(common_ancestor(revisions_2[6], revisions[5], sources),
+                        (revisions[4], revisions_2[5]))
+        self.assertEqual(None, common_ancestor(None, revisions[5], sources))
+
+    def test_combined(self):
+        """combined_graph
+        Ensure it's not order-sensitive
+        """
+        br1, br2 = make_branches(self)
+        source = MultipleRevisionSources(br1.repository, br2.repository)
+        combined_1 = combined_graph(br1.last_revision(), 
+                                    br2.last_revision(), source)
+        combined_2 = combined_graph(br2.last_revision(),
+                                    br1.last_revision(), source)
+        self.assertEquals(combined_1[1], combined_2[1])
+        self.assertEquals(combined_1[2], combined_2[2])
+        self.assertEquals(combined_1[3], combined_2[3])
+        self.assertEquals(combined_1, combined_2)
+
+    def test_get_history(self):
+        # TODO: test ghosts on the left hand branch's impact
+        # TODO: test ghosts on all parents, we should get some
+        # indicator. i.e. NULL_REVISION
+        # RBC 20060608
+        tree = self.make_branch_and_tree('.')
+        tree.commit('1', rev_id = '1', allow_pointless=True)
+        tree.commit('2', rev_id = '2', allow_pointless=True)
+        tree.commit('3', rev_id = '3', allow_pointless=True)
+        rev = tree.branch.repository.get_revision('1')
+        history = rev.get_history(tree.branch.repository)
+        self.assertEqual([None, '1'], history)
+        rev = tree.branch.repository.get_revision('2')
+        history = rev.get_history(tree.branch.repository)
+        self.assertEqual([None, '1', '2'], history)
+        rev = tree.branch.repository.get_revision('3')
+        history = rev.get_history(tree.branch.repository)
+        self.assertEqual([None, '1', '2' ,'3'], history)
+
+    def test_common_ancestor_rootless_graph(self):
+        # common_ancestor on a graph with no reachable roots - only
+        # ghosts - should still return a useful value.
+        graph = Graph()
+        # add a ghost node which would be a root if it wasn't a ghost.
+        graph.add_ghost('a_ghost')
+        # add a normal commit on top of that
+        graph.add_node('rev1', ['a_ghost'])
+        # add a left-branch revision
+        graph.add_node('left', ['rev1'])
+        # add a right-branch revision
+        graph.add_node('right', ['rev1'])
+        source = MockRevisionSource(graph)
+        self.assertEqual('rev1', common_ancestor('left', 'right', source))
+
+
+class TestMultipleRevisionSources(TestCaseWithTransport):
+    """Tests for the MultipleRevisionSources adapter."""
+
+    def test_get_revision_graph_merges_ghosts(self):
+        # when we ask for the revision graph for B, which
+        # is in repo 1 with a ghost of A, and which is not
+        # in repo 2, which has A, the revision_graph()
+        # should return A and B both.
+        tree_1 = self.make_branch_and_tree('1')
+        tree_1.add_pending_merge('A')
+        tree_1.commit('foo', rev_id='B', allow_pointless=True)
+        tree_2 = self.make_branch_and_tree('2')
+        tree_2.commit('bar', rev_id='A', allow_pointless=True)
+        source = MultipleRevisionSources(tree_1.branch.repository,
+                                         tree_2.branch.repository)
+        self.assertEqual({'B':['A'],
+                          'A':[]},
+                         source.get_revision_graph('B'))

=== renamed file 'bzrlib/selftest/testrevisionnamespaces.py' => 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2006-02-10 11:03:38 +0000
@@ -14,33 +14,90 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-from bzrlib.selftest import InTempDir
-
-class TestRevisionNamespaces(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
-        from bzrlib.errors import BzrError
-        from bzrlib.branch import Branch
-        import os
-        import time
-
-        b = Branch('.', init=True)
-
-        b.commit('Commit one', rev_id='a@r-0-1')
-        b.commit('Commit two', rev_id='a@r-0-2')
-        b.commit('Commit three', rev_id='a@r-0-3')
-
-        self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(BzrError, b.get_revision_info, 'revid:a@r-0-0')
-
-        self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
-
-TEST_CLASSES = [
-    TestRevisionNamespaces
-    ]
+import os
+import time
+
+from bzrlib.builtins import merge
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.errors import NoCommonAncestor, NoCommits
+from bzrlib.errors import NoSuchRevision
+from bzrlib.revisionspec import RevisionSpec
+
+
+class TestRevisionNamespaces(TestCaseWithTransport):
+
+    def test_revision_namespaces(self):
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+
+        wt.commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
+        wt.commit('Commit two', rev_id='a@r-0-2')
+        wt.commit('Commit three', rev_id='a@r-0-3')
+
+        self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
+        self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revid:a@r-0-1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision,
+                          RevisionSpec('revid:a@r-0-0').in_history, b)
+        self.assertRaises(TypeError, RevisionSpec, object)
+
+        self.assertEquals(RevisionSpec('date:today').in_history(b),
+                          (2, 'a@r-0-2'))
+        self.assertEquals(RevisionSpec('date:yesterday').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('before:date:today').in_history(b),
+                          (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec('last:1').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('-1').in_history(b), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b).rev_id,
+                          'a@r-0-3')
+
+        os.mkdir('newbranch')
+        wt2 = self.make_branch_and_tree('newbranch')
+        b2 = wt2.branch
+        self.assertRaises(NoCommits, RevisionSpec('ancestor:.').in_history, b2)
+
+        d3 = b.bzrdir.sprout('copy')
+        b3 = d3.open_branch()
+        wt3 = d3.open_workingtree()
+        wt3.commit('Commit four', rev_id='b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'a@r-0-3')
+        merge(['copy', -1], [None, None])
+        wt.commit('Commit five', rev_id='a@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
+                          'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'b@r-0-4')
+
+        # This should be in the revision store, but not in revision-history
+        self.assertEquals((None, 'b@r-0-4'),
+                RevisionSpec('revid:b@r-0-4').in_history(b))
+
+    def test_branch_namespace(self):
+        """Ensure that the branch namespace pulls in the requisite content."""
+        self.build_tree(['branch1/', 'branch1/file', 'branch2/'])
+        wt = self.make_branch_and_tree('branch1')
+        branch = wt.branch
+        wt.add(['file'])
+        wt.commit('add file')
+        d2 = branch.bzrdir.sprout('branch2')
+        print >> open('branch2/file', 'w'), 'new content'
+        branch2 = d2.open_branch()
+        d2.open_workingtree().commit('update file', rev_id='A')
+        spec = RevisionSpec('branch:./branch2/.bzr/../')
+        rev_info = spec.in_history(branch)
+        self.assertEqual(rev_info, (None, 'A'))
+

=== added file 'bzrlib/tests/test_revprops.py'
--- bzrlib/tests/test_revprops.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_revprops.py	2006-01-26 20:52:58 +0000
@@ -0,0 +1,39 @@
+# (C) 2005 Canonical
+
+"""Tests for revision properties."""
+
+from bzrlib.tests import TestCaseWithTransport
+
+class TestRevProps(TestCaseWithTransport):
+
+    def test_simple_revprops(self):
+        """Simple revision properties"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        b.nick = 'Nicholas'
+        props = dict(flavor='choc-mint', 
+                     condiment='orange\n  mint\n\tcandy')
+        wt.commit(message='initial null commit', 
+                 revprops=props,
+                 allow_pointless=True,
+                 rev_id='test@user-1')
+        rev = b.repository.get_revision('test@user-1')
+        self.assertTrue('flavor' in rev.properties)
+        self.assertEquals(rev.properties['flavor'], 'choc-mint')
+        self.assertEquals(sorted(rev.properties.items()),
+                          [('branch-nick', 'Nicholas'), 
+                           ('condiment', 'orange\n  mint\n\tcandy'),
+                           ('flavor', 'choc-mint')])
+
+    def test_invalid_revprops(self):
+        """Invalid revision properties"""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.assertRaises(ValueError,
+                          wt.commit, 
+                          message='invalid',
+                          revprops={'what a silly property': 'fine'})
+        self.assertRaises(ValueError,
+                          wt.commit, 
+                          message='invalid',
+                          revprops=dict(number=13))

=== added file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_rio.py	2006-03-09 18:47:40 +0000
@@ -0,0 +1,333 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for rio serialization
+
+A simple, reproducible structured IO format.
+
+rio itself works in Unicode strings.  It is typically encoded to UTF-8,
+but this depends on the transport.
+"""
+
+import cStringIO
+import os
+import sys
+from tempfile import TemporaryFile
+
+from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.rio import (RioWriter, Stanza, read_stanza, read_stanzas, rio_file,
+                        RioReader)
+
+
+class TestRio(TestCase):
+
+    def test_stanza(self):
+        """Construct rio stanza in memory"""
+        s = Stanza(number='42', name="fred")
+        self.assertTrue('number' in s)
+        self.assertFalse('color' in s)
+        self.assertFalse('42' in s)
+        self.assertEquals(list(s.iter_pairs()),
+                [('name', 'fred'), ('number', '42')])
+        self.assertEquals(s.get('number'), '42')
+        self.assertEquals(s.get('name'), 'fred')
+
+    def test_value_checks(self):
+        """rio checks types on construction"""
+        # these aren't enforced at construction time
+        ## self.assertRaises(ValueError,
+        ##        Stanza, complex=42 + 3j)
+        ## self.assertRaises(ValueError, 
+        ##        Stanza, several=range(10))
+
+    def test_empty_value(self):
+        """Serialize stanza with empty field"""
+        s = Stanza(empty='')
+        self.assertEqualDiff(s.to_string(),
+                "empty: \n")
+
+    def test_to_lines(self):
+        """Write simple rio stanza to string"""
+        s = Stanza(number='42', name='fred')
+        self.assertEquals(list(s.to_lines()),
+                ['name: fred\n',
+                 'number: 42\n'])
+
+    def test_as_dict(self):
+        """Convert rio Stanza to dictionary"""
+        s = Stanza(number='42', name='fred')
+        sd = s.as_dict()
+        self.assertEquals(sd, dict(number='42', name='fred'))
+
+    def test_to_file(self):
+        """Write rio to file"""
+        tmpf = TemporaryFile()
+        s = Stanza(a_thing='something with "quotes like \\"this\\""', number='42', name='fred')
+        s.write(tmpf)
+        tmpf.seek(0)
+        self.assertEqualDiff(tmpf.read(), r'''
+a_thing: something with "quotes like \"this\""
+name: fred
+number: 42
+'''[1:])
+
+    def test_multiline_string(self):
+        tmpf = TemporaryFile()
+        s = Stanza(motto="war is peace\nfreedom is slavery\nignorance is strength")
+        s.write(tmpf)
+        tmpf.seek(0)
+        self.assertEqualDiff(tmpf.read(), '''\
+motto: war is peace
+\tfreedom is slavery
+\tignorance is strength
+''')
+        tmpf.seek(0)
+        s2 = read_stanza(tmpf)
+        self.assertEquals(s, s2)
+
+    def test_read_stanza(self):
+        """Load stanza from string"""
+        lines = """\
+revision: mbp@sourcefrog.net-123-abc
+timestamp: 1130653962
+timezone: 36000
+committer: Martin Pool <mbp@test.sourcefrog.net>
+""".splitlines(True)
+        s = read_stanza(lines)
+        self.assertTrue('revision' in s)
+        self.assertEqualDiff(s.get('revision'), 'mbp@sourcefrog.net-123-abc')
+        self.assertEquals(list(s.iter_pairs()),
+                [('revision', 'mbp@sourcefrog.net-123-abc'),
+                 ('timestamp', '1130653962'),
+                 ('timezone', '36000'),
+                 ('committer', "Martin Pool <mbp@test.sourcefrog.net>")])
+        self.assertEquals(len(s), 4)
+
+    def test_repeated_field(self):
+        """Repeated field in rio"""
+        s = Stanza()
+        for k, v in [('a', '10'), ('b', '20'), ('a', '100'), ('b', '200'), 
+                     ('a', '1000'), ('b', '2000')]:
+            s.add(k, v)
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+        self.assertEquals(s.get_all('a'), map(str, [10, 100, 1000]))
+        self.assertEquals(s.get_all('b'), map(str, [20, 200, 2000]))
+
+    def test_backslash(self):
+        s = Stanza(q='\\')
+        t = s.to_string()
+        self.assertEqualDiff(t, 'q: \\\n')
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_blank_line(self):
+        s = Stanza(none='', one='\n', two='\n\n')
+        self.assertEqualDiff(s.to_string(), """\
+none: 
+one: 
+\t
+two: 
+\t
+\t
+""")
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_whitespace_value(self):
+        s = Stanza(space=' ', tabs='\t\t\t', combo='\n\t\t\n')
+        self.assertEqualDiff(s.to_string(), """\
+combo: 
+\t\t\t
+\t
+space:  
+tabs: \t\t\t
+""")
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+        self.rio_file_stanzas([s])
+
+    def test_quoted(self):
+        """rio quoted string cases"""
+        s = Stanza(q1='"hello"', 
+                   q2=' "for', 
+                   q3='\n\n"for"\n',
+                   q4='for\n"\nfor',
+                   q5='\n',
+                   q6='"', 
+                   q7='""',
+                   q8='\\',
+                   q9='\\"\\"',
+                   )
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+        # apparent bug in read_stanza
+        # s3 = read_stanza(self.stanzas_to_str([s]))
+        # self.assertEquals(s, s3)
+
+    def test_read_empty(self):
+        """Detect end of rio file"""
+        s = read_stanza([])
+        self.assertEqual(s, None)
+        self.assertTrue(s is None)
+        
+    def test_read_iter(self):
+        """Read several stanzas from file"""
+        tmpf = TemporaryFile()
+        tmpf.write("""\
+version_header: 1
+
+name: foo
+val: 123
+
+name: bar
+val: 129319
+""")
+        tmpf.seek(0)
+        reader = read_stanzas(tmpf)
+        read_iter = iter(reader)
+        stuff = list(reader)
+        self.assertEqual(stuff, 
+                [ Stanza(version_header='1'),
+                  Stanza(name="foo", val='123'),
+                  Stanza(name="bar", val='129319'), ])
+
+    def test_read_several(self):
+        """Read several stanzas from file"""
+        tmpf = TemporaryFile()
+        tmpf.write("""\
+version_header: 1
+
+name: foo
+val: 123
+
+name: quoted
+address:   "Willowglen"
+\t  42 Wallaby Way
+\t  Sydney
+
+name: bar
+val: 129319
+""")
+        tmpf.seek(0)
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(version_header='1'))
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(name="foo", val='123'))
+        s = read_stanza(tmpf)
+        self.assertEqualDiff(s.get('name'), 'quoted')
+        self.assertEqualDiff(s.get('address'), '  "Willowglen"\n  42 Wallaby Way\n  Sydney')
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(name="bar", val='129319'))
+        s = read_stanza(tmpf)
+        self.assertEquals(s, None)
+        self.check_rio_file(tmpf)
+
+    def check_rio_file(self, real_file):
+        real_file.seek(0)
+        read_write = rio_file(RioReader(real_file)).read()
+        real_file.seek(0)
+        self.assertEquals(read_write, real_file.read())
+
+    @staticmethod
+    def stanzas_to_str(stanzas):
+        return rio_file(stanzas).read()
+
+    def rio_file_stanzas(self, stanzas):
+        new_stanzas = list(RioReader(rio_file(stanzas)))
+        self.assertEqual(new_stanzas, stanzas)
+
+    def test_tricky_quoted(self):
+        tmpf = TemporaryFile()
+        tmpf.write('''\
+s: "one"
+
+s: 
+\t"one"
+\t
+
+s: "
+
+s: ""
+
+s: """
+
+s: 
+\t
+
+s: \\
+
+s: 
+\t\\
+\t\\\\
+\t
+
+s: word\\
+
+s: quote"
+
+s: backslashes\\\\\\
+
+s: both\\\"
+
+''')
+        tmpf.seek(0)
+        expected_vals = ['"one"',
+            '\n"one"\n',
+            '"',
+            '""',
+            '"""',
+            '\n',
+            '\\',
+            '\n\\\n\\\\\n',
+            'word\\',
+            'quote\"',
+            'backslashes\\\\\\',
+            'both\\\"',
+            ]
+        for expected in expected_vals:
+            stanza = read_stanza(tmpf)
+            self.rio_file_stanzas([stanza])
+            self.assertEquals(len(stanza), 1)
+            self.assertEqualDiff(stanza.get('s'), expected)
+
+    def test_write_empty_stanza(self):
+        """Write empty stanza"""
+        l = list(Stanza().to_lines())
+        self.assertEquals(l, [])
+
+    def test_rio_raises_type_error(self):
+        """TypeError on adding invalid type to Stanza"""
+        s = Stanza()
+        self.assertRaises(TypeError, s.add, 'foo', {})
+
+    def test_rio_raises_type_error_key(self):
+        """TypeError on adding invalid type to Stanza"""
+        s = Stanza()
+        self.assertRaises(TypeError, s.add, 10, {})
+
+    def test_rio_unicode(self):
+        # intentionally use cStringIO which doesn't accomodate unencoded unicode objects
+        sio = cStringIO.StringIO()
+        uni_data = u'\N{KATAKANA LETTER O}'
+        s = Stanza(foo=uni_data)
+        self.assertEquals(s.get('foo'), uni_data)
+        raw_lines = s.to_lines()
+        self.assertEquals(raw_lines,
+                ['foo: ' + uni_data.encode('utf-8') + '\n'])
+        new_s = read_stanza(raw_lines)
+        self.assertEquals(new_s.get('foo'), uni_data)
+

=== added file 'bzrlib/tests/test_sampler.py'
--- bzrlib/tests/test_sampler.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_sampler.py	2006-01-05 10:49:26 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Document what this test file is expecting to test here.
+
+If you need more than one line, make the first line a good sentence on its
+own and add more explanation here, like this.
+
+Be sure to register your new test script in bzrlib/tests/__init__.py -
+search for sampler in there.
+"""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+from bzrlib.tests import TestCaseInTempDir
+
+
+# Now we need a test script:
+class DemoTest(TestCaseInTempDir):
+
+    def test_nothing(self):
+        self.assertEqual(1,1)

=== added file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_selftest.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,409 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the test framework
+"""
+
+import os
+import sys
+import unittest
+import warnings
+
+import bzrlib
+from bzrlib.tests import (
+                          _load_module_by_name,
+                          ChrootedTestCase,
+                          TestCase,
+                          TestCaseInTempDir,
+                          TestCaseWithTransport,
+                          TestSkipped,
+                          TextTestRunner,
+                          )
+import bzrlib.errors as errors
+
+
+class SelftestTests(TestCase):
+
+    def test_import_tests(self):
+        mod = _load_module_by_name('bzrlib.tests.test_selftest')
+        self.assertEqual(mod.SelftestTests, SelftestTests)
+
+    def test_import_test_failure(self):
+        self.assertRaises(ImportError,
+                          _load_module_by_name,
+                          'bzrlib.no-name-yet')
+
+
+class MetaTestLog(TestCase):
+
+    def test_logging(self):
+        """Test logs are captured when a test fails."""
+        self.log('a test message')
+        self._log_file.flush()
+        self.assertContainsRe(self._get_log(), 'a test message\n')
+
+
+class TestTreeShape(TestCaseInTempDir):
+
+    def test_unicode_paths(self):
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        self.failUnlessExists(filename)
+
+
+class TestSkippedTest(TestCase):
+    """Try running a test which is skipped, make sure it's reported properly."""
+
+    def test_skipped_test(self):
+        # must be hidden in here so it's not run as a real test
+        def skipping_test():
+            raise TestSkipped('test intentionally skipped')
+        runner = TextTestRunner(stream=self._log_file)
+        test = unittest.FunctionTestCase(skipping_test)
+        result = runner.run(test)
+        self.assertTrue(result.wasSuccessful())
+
+
+class TestTransportProviderAdapter(TestCase):
+    """A group of tests that test the transport implementation adaption core.
+
+    This is a meta test that the tests are applied to all available 
+    transports.
+
+    This will be generalised in the future which is why it is in this 
+    test file even though it is specific to transport tests at the moment.
+    """
+
+    def test_get_transport_permutations(self):
+        # this checks that we the module get_test_permutations call
+        # is made by the adapter get_transport_test_permitations method.
+        class MockModule(object):
+            def get_test_permutations(self):
+                return sample_permutation
+        sample_permutation = [(1,2), (3,4)]
+        from bzrlib.transport import TransportTestProviderAdapter
+        adapter = TransportTestProviderAdapter()
+        self.assertEqual(sample_permutation,
+                         adapter.get_transport_test_permutations(MockModule()))
+
+    def test_adapter_checks_all_modules(self):
+        # this checks that the adapter returns as many permurtations as
+        # there are in all the registered# transport modules for there
+        # - we assume if this matches its probably doing the right thing
+        # especially in combination with the tests for setting the right
+        # classes below.
+        from bzrlib.transport import (TransportTestProviderAdapter,
+                                      _get_transport_modules
+                                      )
+        modules = _get_transport_modules()
+        permutation_count = 0
+        for module in modules:
+            try:
+                permutation_count += len(reduce(getattr, 
+                    (module + ".get_test_permutations").split('.')[1:],
+                     __import__(module))())
+            except errors.DependencyNotPresent:
+                pass
+        input_test = TestTransportProviderAdapter(
+            "test_adapter_sets_transport_class")
+        adapter = TransportTestProviderAdapter()
+        self.assertEqual(permutation_count,
+                         len(list(iter(adapter.adapt(input_test)))))
+
+    def test_adapter_sets_transport_class(self):
+        # Check that the test adapter inserts a transport and server into the
+        # generated test.
+        #
+        # This test used to know about all the possible transports and the
+        # order they were returned but that seems overly brittle (mbp
+        # 20060307)
+        input_test = TestTransportProviderAdapter(
+            "test_adapter_sets_transport_class")
+        from bzrlib.transport import TransportTestProviderAdapter
+        suite = TransportTestProviderAdapter().adapt(input_test)
+        tests = list(iter(suite))
+        self.assertTrue(len(tests) > 6)
+        # there are at least that many builtin transports
+        one_test = tests[0]
+        self.assertTrue(issubclass(one_test.transport_class, 
+                                   bzrlib.transport.Transport))
+        self.assertTrue(issubclass(one_test.transport_server, 
+                                   bzrlib.transport.Server))
+
+
+class TestBranchProviderAdapter(TestCase):
+    """A group of tests that test the branch implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.branch import BranchTestProviderAdapter
+        input_test = TestBranchProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [("c", "C"), ("d", "D")]
+        adapter = BranchTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].branch_format, formats[0][0])
+        self.assertEqual(tests[0].bzrdir_format, formats[0][1])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].branch_format, formats[1][0])
+        self.assertEqual(tests[1].bzrdir_format, formats[1][1])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestBzrDirProviderAdapter(TestCase):
+    """A group of tests that test the bzr dir implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.bzrdir import BzrDirTestProviderAdapter
+        input_test = TestBzrDirProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = ["c", "d"]
+        adapter = BzrDirTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].bzrdir_format, formats[0])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].bzrdir_format, formats[1])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestRepositoryProviderAdapter(TestCase):
+    """A group of tests that test the repository implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.repository import RepositoryTestProviderAdapter
+        input_test = TestRepositoryProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [("c", "C"), ("d", "D")]
+        adapter = RepositoryTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].bzrdir_format, formats[0][1])
+        self.assertEqual(tests[0].repository_format, formats[0][0])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].bzrdir_format, formats[1][1])
+        self.assertEqual(tests[1].repository_format, formats[1][0])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestInterRepositoryProviderAdapter(TestCase):
+    """A group of tests that test the InterRepository test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.repository import InterRepositoryTestProviderAdapter
+        input_test = TestInterRepositoryProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [(str, "C1", "C2"), (int, "D1", "D2")]
+        adapter = InterRepositoryTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].interrepo_class, formats[0][0])
+        self.assertEqual(tests[0].repository_format, formats[0][1])
+        self.assertEqual(tests[0].repository_format_to, formats[0][2])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].interrepo_class, formats[1][0])
+        self.assertEqual(tests[1].repository_format, formats[1][1])
+        self.assertEqual(tests[1].repository_format_to, formats[1][2])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestInterVersionedFileProviderAdapter(TestCase):
+    """A group of tests that test the InterVersionedFile test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.versionedfile import InterVersionedFileTestProviderAdapter
+        input_test = TestInterRepositoryProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [(str, "C1", "C2"), (int, "D1", "D2")]
+        adapter = InterVersionedFileTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].interversionedfile_class, formats[0][0])
+        self.assertEqual(tests[0].versionedfile_factory, formats[0][1])
+        self.assertEqual(tests[0].versionedfile_factory_to, formats[0][2])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].interversionedfile_class, formats[1][0])
+        self.assertEqual(tests[1].versionedfile_factory, formats[1][1])
+        self.assertEqual(tests[1].versionedfile_factory_to, formats[1][2])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestRevisionStoreProviderAdapter(TestCase):
+    """A group of tests that test the RevisionStore test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.store.revision import RevisionStoreTestProviderAdapter
+        input_test = TestRevisionStoreProviderAdapter(
+            "test_adapted_tests")
+        # revision stores need a store factory - i.e. RevisionKnit
+        #, a readonly and rw transport 
+        # transport servers:
+        server1 = "a"
+        server2 = "b"
+        store_factories = ["c", "d"]
+        adapter = RevisionStoreTestProviderAdapter(server1, server2, store_factories)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].store_factory, store_factories[0][0])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].store_factory, store_factories[1][0])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestWorkingTreeProviderAdapter(TestCase):
+    """A group of tests that test the workingtree implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.workingtree import WorkingTreeTestProviderAdapter
+        input_test = TestWorkingTreeProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [("c", "C"), ("d", "D")]
+        adapter = WorkingTreeTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].workingtree_format, formats[0][0])
+        self.assertEqual(tests[0].bzrdir_format, formats[0][1])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].workingtree_format, formats[1][0])
+        self.assertEqual(tests[1].bzrdir_format, formats[1][1])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
+class TestTestCaseWithTransport(TestCaseWithTransport):
+    """Tests for the convenience functions TestCaseWithTransport introduces."""
+
+    def test_get_readonly_url_none(self):
+        from bzrlib.transport import get_transport
+        from bzrlib.transport.memory import MemoryServer
+        from bzrlib.transport.readonly import ReadonlyTransportDecorator
+        self.transport_server = MemoryServer
+        self.transport_readonly_server = None
+        # calling get_readonly_transport() constructs a decorator on the url
+        # for the server
+        url = self.get_readonly_url()
+        url2 = self.get_readonly_url('foo/bar')
+        t = get_transport(url)
+        t2 = get_transport(url2)
+        self.failUnless(isinstance(t, ReadonlyTransportDecorator))
+        self.failUnless(isinstance(t2, ReadonlyTransportDecorator))
+        self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
+
+    def test_get_readonly_url_http(self):
+        from bzrlib.transport import get_transport
+        from bzrlib.transport.local import LocalRelpathServer
+        from bzrlib.transport.http import HttpServer, HttpTransportBase
+        self.transport_server = LocalRelpathServer
+        self.transport_readonly_server = HttpServer
+        # calling get_readonly_transport() gives us a HTTP server instance.
+        url = self.get_readonly_url()
+        url2 = self.get_readonly_url('foo/bar')
+        # the transport returned may be any HttpTransportBase subclass
+        t = get_transport(url)
+        t2 = get_transport(url2)
+        self.failUnless(isinstance(t, HttpTransportBase))
+        self.failUnless(isinstance(t2, HttpTransportBase))
+        self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
+
+    def test_is_directory(self):
+        """Test assertIsDirectory assertion"""
+        t = self.get_transport()
+        self.build_tree(['a_dir/', 'a_file'], transport=t)
+        self.assertIsDirectory('a_dir', t)
+        self.assertRaises(AssertionError, self.assertIsDirectory, 'a_file', t)
+        self.assertRaises(AssertionError, self.assertIsDirectory, 'not_here', t)
+
+
+class TestChrootedTest(ChrootedTestCase):
+
+    def test_root_is_root(self):
+        from bzrlib.transport import get_transport
+        t = get_transport(self.get_readonly_url())
+        url = t.base
+        self.assertEqual(url, t.clone('..').base)
+
+
+class TestExtraAssertions(TestCase):
+    """Tests for new test assertions in bzrlib test suite"""
+
+    def test_assert_isinstance(self):
+        self.assertIsInstance(2, int)
+        self.assertIsInstance(u'', basestring)
+        self.assertRaises(AssertionError, self.assertIsInstance, None, int)
+        self.assertRaises(AssertionError, self.assertIsInstance, 23.3, int)
+
+
+class TestConvenienceMakers(TestCaseWithTransport):
+    """Test for the make_* convenience functions."""
+
+    def test_make_branch_and_tree_with_format(self):
+        # we should be able to supply a format to make_branch_and_tree
+        self.make_branch_and_tree('a', format=bzrlib.bzrdir.BzrDirMetaFormat1())
+        self.make_branch_and_tree('b', format=bzrlib.bzrdir.BzrDirFormat6())
+        self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('a')._format,
+                              bzrlib.bzrdir.BzrDirMetaFormat1)
+        self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('b')._format,
+                              bzrlib.bzrdir.BzrDirFormat6)
+        

=== added file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_setup.py	2006-03-12 15:58:22 +0000
@@ -0,0 +1,33 @@
+""" test for setup.py build process """
+
+import os
+import sys
+import subprocess
+import shutil
+from tempfile import TemporaryFile
+
+from bzrlib.tests import TestCase
+
+
+# TODO: ideally run this in a separate directory, so as not to clobber the
+# real build directory
+
+class TestSetup(TestCase):
+
+    def test_build(self):
+        """ test cmd `python setup.py build`
+        
+        This typically catches new subdirectories which weren't added to setup.py
+        """
+        self.log('test_build running in %s' % os.getcwd())
+        try:
+            # run setup.py build as subproces and catch return code
+            out_file = TemporaryFile()
+            err_file = TemporaryFile()
+            p = subprocess.Popen([sys.executable, 'setup.py', 'build'],
+                                 stdout=out_file, stderr=err_file)
+            s = p.communicate()
+            self.assertEqual(0, p.returncode, '`python setup.py build` fails')
+        finally:
+            if os.path.exists('build'):
+                shutil.rmtree(u'build')

=== added file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,234 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import socket
+import threading
+
+import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin, lexists
+from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
+import bzrlib.transport
+from bzrlib.workingtree import WorkingTree
+
+try:
+    import paramiko
+    paramiko_loaded = True
+except ImportError:
+    paramiko_loaded = False
+
+
+class TestCaseWithSFTPServer(TestCaseWithTransport):
+    """A test case base class that provides a sftp server on localhost."""
+
+    def setUp(self):
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+        super(TestCaseWithSFTPServer, self).setUp()
+        from bzrlib.transport.sftp import SFTPAbsoluteServer, SFTPHomeDirServer
+        if getattr(self, '_get_remote_is_absolute', None) is None:
+            self._get_remote_is_absolute = True
+        if self._get_remote_is_absolute:
+            self.transport_server = SFTPAbsoluteServer
+        else:
+            self.transport_server = SFTPHomeDirServer
+        self.transport_readonly_server = bzrlib.transport.http.HttpServer
+
+    def get_transport(self, path=None):
+        """Return a transport relative to self._test_root."""
+        return bzrlib.transport.get_transport(self.get_url(path))
+
+
+class SFTPLockTests (TestCaseWithSFTPServer):
+
+    def test_sftp_locks(self):
+        from bzrlib.errors import LockError
+        t = self.get_transport()
+
+        l = t.lock_write('bogus')
+        self.failUnlessExists('bogus.write-lock')
+
+        # Don't wait for the lock, locking an already locked
+        # file should raise an assert
+        self.assertRaises(LockError, t.lock_write, 'bogus')
+
+        l.unlock()
+        self.failIf(lexists('bogus.write-lock'))
+
+        open('something.write-lock', 'wb').write('fake lock\n')
+        self.assertRaises(LockError, t.lock_write, 'something')
+        os.remove('something.write-lock')
+
+        l = t.lock_write('something')
+
+        l2 = t.lock_write('bogus')
+
+        l.unlock()
+        l2.unlock()
+
+    def test_multiple_connections(self):
+        t = self.get_transport()
+        self.assertTrue('sftpserver - new connection' in self.get_server().logs)
+        self.get_server().logs = []
+        # The second request should reuse the first connection
+        # SingleListener only allows for a single connection,
+        # So the next line fails unless the connection is reused
+        t2 = self.get_transport()
+        self.assertEquals(self.get_server().logs, [])
+
+
+class SFTPTransportTestRelative(TestCaseWithSFTPServer):
+    """Test the SFTP transport with homedir based relative paths."""
+
+    def test__remote_path(self):
+        t = self.get_transport()
+        # try what is currently used:
+        # remote path = self._abspath(relpath)
+        self.assertEqual(self.test_dir + '/relative', t._remote_path('relative'))
+        # we dont os.path.join because windows gives us the wrong path
+        root_segments = self.test_dir.split('/')
+        root_parent = '/'.join(root_segments[:-1])
+        # .. should be honoured
+        self.assertEqual(root_parent + '/sibling', t._remote_path('../sibling'))
+        # /  should be illegal ?
+        ### FIXME decide and then test for all transports. RBC20051208
+
+
+class SFTPTransportTestRelative(TestCaseWithSFTPServer):
+    """Test the SFTP transport with homedir based relative paths."""
+
+    def setUp(self):
+        self._get_remote_is_absolute = False
+        super(SFTPTransportTestRelative, self).setUp()
+
+    def test__remote_path_relative_root(self):
+        # relative paths are preserved
+        t = self.get_transport('')
+        self.assertEqual('a', t._remote_path('a'))
+
+
+class FakeSFTPTransport (object):
+    _sftp = object()
+fake = FakeSFTPTransport()
+
+
+class SFTPNonServerTest(TestCase):
+    def setUp(self):
+        TestCase.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+
+    def test_parse_url(self):
+        from bzrlib.transport.sftp import SFTPTransport
+        s = SFTPTransport('sftp://simple.example.com/home/source', clone_from=fake)
+        self.assertEquals(s._host, 'simple.example.com')
+        self.assertEquals(s._port, None)
+        self.assertEquals(s._path, '/home/source')
+        self.failUnless(s._password is None)
+
+        self.assertEquals(s.base, 'sftp://simple.example.com/home/source/')
+
+        s = SFTPTransport('sftp://ro%62ey:h%40t@example.com:2222/~/relative', clone_from=fake)
+        self.assertEquals(s._host, 'example.com')
+        self.assertEquals(s._port, 2222)
+        self.assertEquals(s._username, 'robey')
+        self.assertEquals(s._password, 'h@t')
+        self.assertEquals(s._path, 'relative')
+
+        # Base should not keep track of the password
+        self.assertEquals(s.base, 'sftp://robey@example.com:2222/~/relative/')
+
+    def test_relpath(self):
+        from bzrlib.transport.sftp import SFTPTransport
+        from bzrlib.errors import PathNotChild
+
+        s = SFTPTransport('sftp://user@host.com/abs/path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://user@host.com/abs/path/sub'), 'sub')
+        # Can't test this one, because we actually get an AssertionError
+        # TODO: Consider raising an exception rather than an assert
+        #self.assertRaises(PathNotChild, s.relpath, 'http://user@host.com/abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user2@host.com/abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@otherhost.com/abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@host.com:33/abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@host.com/~/rel/path/sub')
+
+        # Make sure it works when we don't supply a username
+        s = SFTPTransport('sftp://host.com/abs/path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com/abs/path/sub'), 'sub')
+
+        # Make sure it works when parts of the path will be url encoded
+        # TODO: These may be incorrect, we might need to urllib.urlencode() before
+        # we pass the paths into the SFTPTransport constructor
+        s = SFTPTransport('sftp://host.com/dev/,path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com/dev/,path/sub'), 'sub')
+        s = SFTPTransport('sftp://host.com/dev/%path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com/dev/%path/sub'), 'sub')
+
+    def test_parse_invalid_url(self):
+        from bzrlib.transport.sftp import SFTPTransport, TransportError
+        try:
+            s = SFTPTransport('sftp://lilypond.org:~janneke/public_html/bzr/gub',
+                              clone_from=fake)
+            self.fail('expected exception not raised')
+        except TransportError, e:
+            self.assertEquals(str(e), 
+                    '~janneke: invalid port number')
+
+
+class SFTPBranchTest(TestCaseWithSFTPServer):
+    """Test some stuff when accessing a bzr Branch over sftp"""
+
+    def test_lock_file(self):
+        # old format branches use a special lock file on sftp.
+        b = self.make_branch('', format=bzrdir.BzrDirFormat6())
+        b = bzrlib.branch.Branch.open(self.get_url())
+        self.failUnlessExists('.bzr/')
+        self.failUnlessExists('.bzr/branch-format')
+        self.failUnlessExists('.bzr/branch-lock')
+
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+        b.lock_write()
+        self.failUnlessExists('.bzr/branch-lock.write-lock')
+        b.unlock()
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+
+    def test_push_support(self):
+        self.build_tree(['a/', 'a/foo'])
+        t = bzrdir.BzrDir.create_standalone_workingtree('a')
+        b = t.branch
+        t.add('foo')
+        t.commit('foo', rev_id='a1')
+
+        b2 = bzrdir.BzrDir.create_branch_and_repo(self.get_url('/b'))
+        b2.pull(b)
+
+        self.assertEquals(b2.revision_history(), ['a1'])
+
+        open('a/foo', 'wt').write('something new in foo\n')
+        t.commit('new', rev_id='a2')
+        b2.pull(b)
+
+        self.assertEquals(b2.revision_history(), ['a1', 'a2'])
+
+
+class SFTPFullHandshakingTest(TestCaseWithSFTPServer):
+    """Verify that a full-handshake (SSH over loopback TCP) sftp connection works."""
+    
+    def test_connection(self):
+        from bzrlib.transport.sftp import SFTPFullAbsoluteServer
+        self.transport_server = SFTPFullAbsoluteServer
+        self.get_transport()

=== added file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_add.py	2006-02-06 10:31:46 +0000
@@ -0,0 +1,217 @@
+import os
+import unittest
+
+from bzrlib.add import smart_add, smart_add_tree
+from bzrlib.tests import TestCaseWithTransport, TestCase
+from bzrlib.branch import Branch
+from bzrlib.errors import NotBranchError, NoSuchFile
+from bzrlib.inventory import InventoryFile, Inventory
+from bzrlib.workingtree import WorkingTree
+
+class TestSmartAdd(TestCaseWithTransport):
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        smart_add_tree(wt, (u".",))
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        os.chdir("original")
+        smart_add_tree(wt, (u".",))
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        wt = self.make_branch_and_tree('branch')
+        branch = wt.branch
+        smart_add_tree(wt, ("branch",))
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path",)
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        
+        self.build_tree(build_paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        child_tree = self.make_branch_and_tree('original/child')
+        smart_add_tree(wt, (u".",))
+        for path in paths:
+            self.assertNotEqual((path, wt.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, wt.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_tree.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        smart_add_tree(wt, paths)
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_dry_run(self):
+        """Test a dry run add, make sure nothing is added."""
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(wt.unknowns()), ['inertiatic'])
+        self.capture('add --dry-run .')
+        eq(list(wt.unknowns()), ['inertiatic'])
+
+    def test_add_non_existant(self):
+        """Test smart-adding a file that does not exist."""
+        from bzrlib.add import smart_add
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        self.assertRaises(NoSuchFile, smart_add_tree, wt, 'non-existant-file')
+
+    def test_returns(self):
+        """Correctly returns added/ignored files"""
+        from bzrlib.commands import run_bzr
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS', 
+                        'inertiatic/foo.pyc'])
+        added, ignored = smart_add_tree(wt, u'.')
+        self.assertSubset(('inertiatic', 'inertiatic/esp'), added)
+        self.assertSubset(('CVS', '*.py[oc]'), ignored)
+        self.assertSubset(('inertiatic/CVS',), ignored['CVS'])
+        self.assertSubset(('inertiatic/foo.pyc',), ignored['*.py[oc]'])
+
+
+class TestSmartAddBranch(TestCaseWithTransport):
+    """Test smart adds with a specified branch."""
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        smart_add_tree(wt, (u".",))
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add_tree
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        os.chdir("original")
+        smart_add_tree(wt, (u".",))
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add_tree
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        tree = self.make_branch_and_tree('branch')
+        branch = tree.branch
+        smart_add_tree(tree, ("branch",))
+        for path in paths:
+            self.assertNotEqual(tree.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add_tree
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path")
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        self.build_tree(build_paths)
+        tree = self.make_branch_and_tree('.')
+        branch = tree.branch
+        child_tree = self.make_branch_and_tree("original/child")
+        smart_add_tree(tree, (u".",))
+        for path in paths:
+            self.assertNotEqual((path, tree.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, tree.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_tree.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add_tree
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        smart_add_tree(wt, paths)
+        for path in paths:
+            self.assertNotEqual(wt.path2id(path), None)
+
+
+class TestAddActions(TestCase):
+
+    def test_null(self):
+        from bzrlib.add import add_action_null
+        self.run_action(add_action_null, "", False)
+
+    def test_add(self):
+        self.entry = InventoryFile("id", "name", None)
+        from bzrlib.add import add_action_add
+        self.run_action(add_action_add, "", True)
+
+    def test_add_and_print(self):
+        from bzrlib.add import add_action_add_and_print
+        self.run_action(add_action_add_and_print, "added path\n", True)
+
+    def test_print(self):
+        from bzrlib.add import add_action_print
+        self.run_action(add_action_print, "added path\n", False)
+
+    def run_action(self, action, output, should_add):
+        from StringIO import StringIO
+        inv = Inventory()
+        stdout = StringIO()
+
+        self.apply_redirected(None, stdout, None, action, inv, 'path', 'file')
+        self.assertEqual(stdout.getvalue(), output)
+
+        if should_add:
+            self.assertNotEqual(inv.path2id('path'), None)
+        else:
+            self.assertEqual(inv.path2id('path'), None)

=== added file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_source.py	2006-03-05 02:20:07 +0000
@@ -0,0 +1,75 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""These tests are tests about the source code of bzrlib itself.
+
+They are useful for testing code quality, checking coverage metric etc.
+"""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+from bzrlib.tests import TestCase
+import bzrlib.branch
+
+
+class TestApiUsage(TestCase):
+
+    def find_occurences(self, rule, filename):
+        """Find the number of occurences of rule in a file."""
+        occurences = 0
+        source = file(filename, 'r')
+        for line in source:
+            if line.find(rule) > -1:
+                occurences += 1
+        return occurences
+
+    def source_file_name(self, package):
+        """Return the path of the .py file for package."""
+        path = package.__file__
+        if path[-1] in 'co':
+            return path[:-1]
+        else:
+            return path
+
+    def test_branch_working_tree(self):
+        """Test that the number of uses of working_tree in branch is stable."""
+        occurences = self.find_occurences('self.working_tree()',
+                                          self.source_file_name(bzrlib.branch))
+        # do not even think of increasing this number. If you think you need to
+        # increase it, then you almost certainly are doing something wrong as
+        # the relationship from working_tree to branch is one way.
+        # Note that this is an exact equality so that when the number drops, 
+        #it is not given a buffer but rather has this test updated immediately.
+        self.assertEqual(0, occurences)
+
+    def test_branch_WorkingTree(self):
+        """Test that the number of uses of working_tree in branch is stable."""
+        occurences = self.find_occurences('WorkingTree',
+                                          self.source_file_name(bzrlib.branch))
+        # do not even think of increasing this number. If you think you need to
+        # increase it, then you almost certainly are doing something wrong as
+        # the relationship from working_tree to branch is one way.
+        # This number should be 4 (import NoWorkingTree and WorkingTree, 
+        # raise NoWorkingTree from working_tree(), and construct a working tree
+        # there) but a merge that regressed this was done before this test was
+        # written. Note that this is an exact equality so that when the number
+        # drops, it is not given a buffer but rather this test updated
+        # immediately.
+        self.assertEqual(4, occurences)

=== added file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_store.py	2006-04-12 07:02:00 +0000
@@ -0,0 +1,436 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test Store implementations."""
+
+from cStringIO import StringIO
+import os
+import gzip
+
+import bzrlib.errors as errors
+from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
+from bzrlib.transport.local import LocalTransport
+from bzrlib.store.text import TextStore
+from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
+import bzrlib.store as store
+import bzrlib.transactions as transactions
+import bzrlib.transport as transport
+from bzrlib.transport.memory import MemoryTransport
+
+
+class TestStores(object):
+    """Mixin template class that provides some common tests for stores"""
+
+    def check_content(self, store, fileid, value):
+        f = store.get(fileid)
+        self.assertEqual(f.read(), value)
+
+    def fill_store(self, store):
+        store.add(StringIO('hello'), 'a')
+        store.add(StringIO('other'), 'b')
+        store.add(StringIO('something'), 'c')
+        store.add(StringIO('goodbye'), '123123')
+
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = self.get_store('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = self.get_store('b')
+        store_b.copy_all_ids(store_a)
+        self.assertEqual(store_a.get('1').read(), 'foo')
+        self.assertEqual(store_b.get('1').read(), 'foo')
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+    def test_get(self):
+        store = self.get_store()
+        self.fill_store(store)
+    
+        self.check_content(store, 'a', 'hello')
+        self.check_content(store, 'b', 'other')
+        self.check_content(store, 'c', 'something')
+    
+        # Make sure that requesting a non-existing file fails
+        self.assertRaises(KeyError, self.check_content, store, 'd', None)
+
+    def test_multiple_add(self):
+        """Multiple add with same ID should raise a BzrError"""
+        store = self.get_store()
+        self.fill_store(store)
+        self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
+
+
+class TestCompressedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.'):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=True)
+
+    def test_total_size(self):
+        store = self.get_store(u'.')
+        store.register_suffix('dsc')
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123', 'dsc')
+        # these get gzipped - content should be stable
+        self.assertEqual(store.total_size(), (2, 55))
+        
+    def test__relpath_suffixed(self):
+        my_store = TextStore(MockTransport(),
+                             prefixed=True, compressed=True)
+        my_store.register_suffix('dsc')
+        self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
+
+
+class TestMemoryStore(TestCase):
+    
+    def get_store(self):
+        return TextStore(MemoryTransport())
+    
+    def test_add_and_retrieve(self):
+        store = self.get_store()
+        store.add(StringIO('hello'), 'aa')
+        self.assertNotEqual(store.get('aa'), None)
+        self.assertEqual(store.get('aa').read(), 'hello')
+        store.add(StringIO('hello world'), 'bb')
+        self.assertNotEqual(store.get('bb'), None)
+        self.assertEqual(store.get('bb').read(), 'hello world')
+
+    def test_missing_is_absent(self):
+        store = self.get_store()
+        self.failIf('aa' in store)
+
+    def test_adding_fails_when_present(self):
+        my_store = self.get_store()
+        my_store.add(StringIO('hello'), 'aa')
+        self.assertRaises(BzrError,
+                          my_store.add, StringIO('hello'), 'aa')
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.'):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=False)
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestMixedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.', compressed=True):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=compressed)
+
+    def test_get_mixed(self):
+        cs = self.get_store(u'.', compressed=True)
+        s = self.get_store(u'.', compressed=False)
+        cs.add(StringIO('hello there'), 'a')
+
+        self.failUnlessExists('a.gz')
+        self.failIf(os.path.lexists('a'))
+
+        self.assertEquals(gzip.GzipFile('a.gz').read(), 'hello there')
+
+        self.assertEquals(cs.has_id('a'), True)
+        self.assertEquals(s.has_id('a'), True)
+        self.assertEquals(cs.get('a').read(), 'hello there')
+        self.assertEquals(s.get('a').read(), 'hello there')
+        
+        self.assertRaises(BzrError, s.add, StringIO('goodbye'), 'a')
+
+        s.add(StringIO('goodbye'), 'b')
+        self.failUnlessExists('b')
+        self.failIf(os.path.lexists('b.gz'))
+        self.assertEquals(open('b').read(), 'goodbye')
+
+        self.assertEquals(cs.has_id('b'), True)
+        self.assertEquals(s.has_id('b'), True)
+        self.assertEquals(cs.get('b').read(), 'goodbye')
+        self.assertEquals(s.get('b').read(), 'goodbye')
+        
+        self.assertRaises(BzrError, cs.add, StringIO('again'), 'b')
+
+class MockTransport(transport.Transport):
+    """A fake transport for testing with."""
+
+    def has(self, filename):
+        return False
+
+    def __init__(self, url=None):
+        if url is None:
+            url = "http://example.com"
+        super(MockTransport, self).__init__(url)
+
+    def mkdir(self, filename):
+        return
+
+
+class InstrumentedTransportStore(store.TransportStore):
+    """An instrumented TransportStore.
+
+    Here we replace template method worker methods with calls that record the
+    expected results.
+    """
+
+    def _add(self, filename, file):
+        self._calls.append(("_add", filename, file))
+
+    def __init__(self, transport, prefixed=False):
+        super(InstrumentedTransportStore, self).__init__(transport, prefixed)
+        self._calls = []
+
+
+class TestInstrumentedTransportStore(TestCase):
+
+    def test__add_records(self):
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store._add("filename", "file")
+        self.assertEqual([("_add", "filename", "file")], my_store._calls)
+
+
+class TestMockTransport(TestCase):
+
+    def test_isinstance(self):
+        self.failUnless(isinstance(MockTransport(), transport.Transport))
+
+    def test_has(self):
+        self.assertEqual(False, MockTransport().has('foo'))
+
+    def test_mkdir(self):
+        MockTransport().mkdir('45')
+
+
+class TestTransportStore(TestCase):
+    
+    def test__relpath_invalid(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, '/foo')
+        self.assertRaises(ValueError, my_store._relpath, 'foo/')
+
+    def test_register_invalid_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store.register_suffix, '/')
+        self.assertRaises(ValueError, my_store.register_suffix, '.gz/bar')
+
+    def test__relpath_unregister_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['gz'])
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['dsc', 'gz'])
+
+    def test__relpath_simple(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertEqual("foo", my_store._relpath('foo'))
+
+    def test__relpath_prefixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        self.assertEqual('45/foo', my_store._relpath('foo'))
+
+    def test__relpath_simple_suffixed(self):
+        my_store = store.TransportStore(MockTransport())
+        my_store.register_suffix('bar')
+        my_store.register_suffix('baz')
+        self.assertEqual('foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('foo.bar.baz', my_store._relpath('foo', ['bar', 'baz']))
+
+    def test__relpath_prefixed_suffixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        my_store.register_suffix('bar')
+        my_store.register_suffix('baz')
+        self.assertEqual('45/foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('45/foo.bar.baz',
+                         my_store._relpath('foo', ['bar', 'baz']))
+
+    def test_add_simple(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "foo", stream)], my_store._calls)
+
+    def test_add_prefixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "45/foo", stream)], my_store._calls)
+
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "foo.dsc", stream)], my_store._calls)
+        
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "45/foo.dsc", stream)], my_store._calls)
+
+    def get_populated_store(self, prefixed=False,
+            store_class=TextStore, compressed=False):
+        my_store = store_class(MemoryTransport(), prefixed,
+                               compressed=compressed)
+        my_store.register_suffix('sig')
+        stream = StringIO("signature")
+        my_store.add(stream, "foo", 'sig')
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        stream = StringIO("signature for missing base")
+        my_store.add(stream, "missing", 'sig')
+        return my_store
+        
+    def test_has_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo'))
+
+    def test_has_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+
+    def test_has_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(False, my_store.has_id('missing'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(False, my_store.has_id('missing'))
+
+    def test_get_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('content', my_store.get('foo').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('content', my_store.get('foo').read())
+
+    def test_get_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+
+    def test_get_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+
+    def test___iter__no_suffix(self):
+        my_store = TextStore(MemoryTransport(),
+                             prefixed=False, compressed=False)
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        self.assertEqual(set(['foo']),
+                         set(my_store.__iter__()))
+
+    def test___iter__(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store().__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(True).__iter__()))
+
+    def test___iter__compressed(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             compressed=True).__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             True, compressed=True).__iter__()))
+
+    def test___len__(self):
+        self.assertEqual(1, len(self.get_populated_store()))
+
+    def test_copy_suffixes(self):
+        from_store = self.get_populated_store()
+        to_store = TextStore(MemoryTransport(),
+                             prefixed=True, compressed=True)
+        to_store.register_suffix('sig')
+        to_store.copy_all_ids(from_store)
+        self.assertEqual(1, len(to_store))
+        self.assertEqual(set(['foo']), set(to_store.__iter__()))
+        self.assertEqual('content', to_store.get('foo').read())
+        self.assertEqual('signature', to_store.get('foo', 'sig').read())
+        self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
+
+    def test_relpath_escaped(self):
+        my_store = store.TransportStore(MemoryTransport())
+        self.assertEqual('%25', my_store._relpath('%'))
+
+    def test_escaped_uppercase(self):
+        """Uppercase letters are escaped for safety on Windows"""
+        my_store = store.TransportStore(MemoryTransport(), escaped=True)
+        # a particularly perverse file-id! :-)
+        self.assertEquals(my_store._escape_file_id('C:<>'), '%43%3a%3c%3e')
+
+
+class TestVersionFileStore(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestVersionFileStore, self).setUp()
+        self.vfstore = store.versioned.VersionedFileStore(MemoryTransport())
+
+    def test_get_weave_registers_dirty_in_write(self):
+        transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', transaction)
+        transaction.finish()
+        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
+        transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave('id', transaction)
+        transaction.finish()
+        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
+
+    def test_get_weave_or_empty_readonly_fails(self):
+        transaction = transactions.ReadOnlyTransaction()
+        vf = self.assertRaises(errors.ReadOnlyError,
+                               self.vfstore.get_weave_or_empty,
+                               'id',
+                               transaction)
+
+    def test_get_weave_readonly_cant_write(self):
+        transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', transaction)
+        transaction.finish()
+        transaction = transactions.ReadOnlyTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', transaction)
+        self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'b', [], [])
+

=== added file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2006-02-28 04:31:50 +0000
@@ -0,0 +1,100 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning tests."""
+
+import bzrlib.symbol_versioning as symbol_versioning
+from bzrlib.tests import TestCase
+
+
+@symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
+def deprecated_function():
+    """Deprecated function docstring."""
+    return 1
+
+
+class TestDeprecationWarnings(TestCase):
+
+    def capture_warning(self, message, category, stacklevel=None):
+        self._warnings.append((message, category, stacklevel))
+
+    def setUp(self):
+        super(TestDeprecationWarnings, self).setUp()
+        self._warnings = []
+    
+    @symbol_versioning.deprecated_method(symbol_versioning.zero_seven)
+    def deprecated_method(self):
+        """Deprecated method docstring.
+        
+        This might explain stuff.
+        """
+        return 1
+
+    def test_deprecated_method(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning."
+            "TestDeprecationWarnings.deprecated_method "
+            "was deprecated in version 0.7.", DeprecationWarning, 2)
+        expected_docstring = ('Deprecated method docstring.\n'
+                              '        \n'
+                              '        This might explain stuff.\n'
+                              '        \n'
+                              '        This method was deprecated in version 0.7.\n'
+                              '        ')
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_method",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       self.deprecated_method)
+
+    def test_deprecated_function(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning.deprecated_function "
+            "was deprecated in version 0.7.", DeprecationWarning, 2)
+        expected_docstring = ('Deprecated function docstring.\n'
+                              '\n'
+                              'This function was deprecated in version 0.7.\n'
+                              )
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_function",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       deprecated_function)
+
+    def check_deprecated_callable(self, expected_warning, expected_docstring,
+                                  expected_name, expected_module,
+                                  deprecated_callable):
+        old_warning_method = symbol_versioning.warn
+        try:
+            symbol_versioning.set_warning_method(self.capture_warning)
+            self.assertEqual(1, deprecated_callable())
+            self.assertEqual([expected_warning], self._warnings)
+            deprecated_callable()
+            self.assertEqual([expected_warning, expected_warning],
+                             self._warnings)
+            self.assertEqualDiff(expected_docstring, deprecated_callable.__doc__)
+            self.assertEqualDiff(expected_name, deprecated_callable.__name__)
+            self.assertEqualDiff(expected_module, deprecated_callable.__module__)
+            self.assertTrue(deprecated_callable.is_deprecated)
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)
+    
+    def test_deprecated_passed(self):
+        self.assertEqual(True, symbol_versioning.deprecated_passed(None))
+        self.assertEqual(True, symbol_versioning.deprecated_passed(True))
+        self.assertEqual(True, symbol_versioning.deprecated_passed(False))
+        self.assertEqual(False,
+                         symbol_versioning.deprecated_passed(
+                            symbol_versioning.DEPRECATED_PARAMETER))

=== added file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_testament.py	2006-02-21 15:23:00 +0000
@@ -0,0 +1,253 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test testaments for gpg signing."""
+
+# TODO: Testaments with x-bits
+
+import os
+from sha import sha
+import sys
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.testament import Testament
+from bzrlib.trace import mutter
+from bzrlib.osutils import has_symlinks
+
+
+class TestamentTests(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestamentTests, self).setUp()
+        self.wt = self.make_branch_and_tree('.')
+        b = self.b = self.wt.branch
+        b.nick = "test branch"
+        self.wt.commit(message='initial null commit',
+                 committer='test@user',
+                 timestamp=1129025423, # 'Tue Oct 11 20:10:23 2005'
+                 timezone=0,
+                 rev_id='test@user-1')
+        self.build_tree_contents([('hello', 'contents of hello file'),
+                             ('src/', ),
+                             ('src/foo.c', 'int main()\n{\n}\n')])
+        self.wt.add(['hello', 'src', 'src/foo.c'],
+                             ['hello-id', 'src-id', 'foo.c-id'])
+        self.wt.commit(message='add files and directories',
+                 timestamp=1129025483,
+                 timezone=36000,
+                 rev_id='test@user-2',
+                 committer='test@user')
+
+    def test_null_testament(self):
+        """Testament for a revision with no contents."""
+        t = Testament.from_revision(self.b.repository, 'test@user-1')
+        ass = self.assertTrue
+        eq = self.assertEqual
+        ass(isinstance(t, Testament))
+        eq(t.revision_id, 'test@user-1')
+        eq(t.committer, 'test@user')
+        eq(t.timestamp, 1129025423)
+        eq(t.timezone, 0)
+
+    def test_testment_text_form(self):
+        """Conversion of testament to canonical text form."""
+        t = Testament.from_revision(self.b.repository, 'test@user-1')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqual(text_form, REV_1_TESTAMENT)
+
+    def test_testament_with_contents(self):
+        """Testament containing a file and a directory."""
+        t = Testament.from_revision(self.b.repository, 'test@user-2')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqualDiff(text_form, REV_2_TESTAMENT)
+        actual_short = t.as_short_text()
+        self.assertEqualDiff(actual_short, REV_2_SHORT)
+
+    def test_testament_command(self):
+        """Testament containing a file and a directory."""
+        out, err = self.run_bzr_captured(['testament', '--long'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_2_TESTAMENT)
+
+    def test_testament_command_2(self):
+        """Command getting short testament of previous version."""
+        out, err = self.run_bzr_captured(['testament', '-r1'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_1_SHORT)
+
+    def test_testament_symlinks(self):
+        """Testament containing symlink (where possible)"""
+        if not has_symlinks():
+            return
+        os.symlink('wibble/linktarget', 'link')
+        self.wt.add(['link'], ['link-id'])
+        self.wt.commit(message='add symlink',
+                 timestamp=1129025493,
+                 timezone=36000,
+                 rev_id='test@user-3',
+                 committer='test@user')
+        t = Testament.from_revision(self.b.repository, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_3_TESTAMENT)
+
+    def test_testament_revprops(self):
+        """Testament to revision with extra properties"""
+        props = dict(flavor='sour cherry\ncream cheese',
+                     size='medium')
+        self.wt.commit(message='revision with properties',
+                      timestamp=1129025493,
+                      timezone=36000,
+                      rev_id='test@user-3',
+                      committer='test@user',
+                      revprops=props)
+        t = Testament.from_revision(self.b.repository, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+
+    def test_testament_unicode_commit_message(self):
+        self.wt.commit(
+            message=u'non-ascii commit \N{COPYRIGHT SIGN} me',
+            timestamp=1129025493,
+            timezone=36000,
+            rev_id='test@user-3',
+            committer='test@user')
+        t = Testament.from_revision(self.b.repository, 'test@user-3')
+        self.assertEqualDiff(
+            SAMPLE_UNICODE_TESTAMENT.encode('utf-8'), t.as_text())
+
+    def test___init__(self):
+        revision = self.b.repository.get_revision('test@user-2')
+        inventory = self.b.repository.get_inventory('test@user-2')
+        testament_1 = Testament(revision, inventory).as_short_text()
+        testament_2 = Testament.from_revision(self.b.repository, 
+                                              'test@user-2').as_short_text()
+        self.assertEqual(testament_1, testament_2)
+                    
+
+REV_1_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-1
+committer: test@user
+timestamp: 1129025423
+timezone: 0
+parents:
+message:
+  initial null commit
+inventory:
+properties:
+  branch-nick:
+    test branch
+"""
+
+REV_1_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-1
+sha1: %s
+""" % sha(REV_1_TESTAMENT).hexdigest()
+
+
+REV_2_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-2
+committer: test@user
+timestamp: 1129025483
+timezone: 36000
+parents:
+  test@user-1
+message:
+  add files and directories
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+"""
+
+
+REV_2_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-2
+sha1: %s
+""" % sha(REV_2_TESTAMENT).hexdigest()
+
+
+REV_PROPS_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
+REV_3_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  add symlink
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  symlink link link-id wibble/linktarget
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+"""
+
+
+SAMPLE_UNICODE_TESTAMENT = u"""\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  non-ascii commit \N{COPYRIGHT SIGN} me
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+"""

=== added file 'bzrlib/tests/test_textmerge.py'
--- bzrlib/tests/test_textmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_textmerge.py	2006-04-08 23:57:33 +0000
@@ -0,0 +1,41 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+# Author: Aaron Bentley <aaron.bentley@utoronto.ca>
+from bzrlib.textmerge import Merge2
+from bzrlib.tests import TestCase
+class TestMerge2(TestCase):
+    def test_agreed(self):
+        lines = "a\nb\nc\nd\ne\nf\n".splitlines(True)
+        mlines = list(Merge2(lines, lines).merge_lines()[0])
+        self.assertEqualDiff(mlines, lines)
+
+    def test_conflict(self):
+        lines_a = "a\nb\nc\nd\ne\nf\ng\nh\n".splitlines(True)
+        lines_b = "z\nb\nx\nd\ne\ne\nf\ng\ny\n".splitlines(True)
+        expected = "<\na\n=\nz\n>\nb\n<\nc\n=\nx\n>\nd\n<\n=\ne\n>\ne\nf\n"\
+                   "g\n<\nh\n=\ny\n>\n"
+        m2 = Merge2(lines_a, lines_b, '<\n', '>\n', '=\n')
+        mlines= m2.merge_lines()[0]
+        self.assertEqualDiff(''.join(mlines), expected)
+        mlines= m2.merge_lines(reprocess=True)[0]
+        self.assertEqualDiff(''.join(mlines), expected)
+
+    def test_reprocess(self):
+        struct = [('a', 'b'), ('c',), ('def','geh'), ('i',)]
+        expect = [('a', 'b'), ('c',), ('d', 'g'), ('e',), ('f', 'h'), ('i',)]
+        result = Merge2.reprocess_struct(struct)
+        self.assertEqual(list(result), expect)

=== added file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_trace.py	2005-11-29 07:58:36 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# "weren't nothing promised to you.  do i look like i got a promise face?"
+
+"""Tests for trace library"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.trace import format_exception_short, mutter
+from bzrlib.errors import NotBranchError, BzrError, BzrNewError
+
+class TestTrace(TestCase):
+    def test_format_sys_exception(self):
+        """Short formatting of exceptions"""
+        try:
+            raise NotImplementedError, "time travel"
+        except NotImplementedError:
+            pass
+        error_lines = format_exception_short(sys.exc_info()).splitlines()
+        self.assertEqualDiff(error_lines[0], 
+                'exceptions.NotImplementedError: time travel')
+        self.assertContainsRe(error_lines[1], 
+                r'^  at .*trace\.py line \d+$')  
+        self.assertContainsRe(error_lines[2], 
+                r'^  in test_format_sys_exception$')
+
+    def test_format_exception(self):
+        """Short formatting of exceptions"""
+        try:
+            raise NotBranchError, 'wibble'
+        except NotBranchError:
+            pass
+        msg = format_exception_short(sys.exc_info())
+        self.assertEqualDiff(msg, 'Not a branch: wibble')
+
+    def test_format_old_exception(self):
+        # format a class that doesn't descend from BzrNewError; 
+        # remove this test when everything is unified there
+        self.assertFalse(issubclass(BzrError, BzrNewError))
+        try:
+            raise BzrError('some old error')
+        except BzrError:
+            pass
+        msg = format_exception_short(sys.exc_info())
+        self.assertEqualDiff(msg, 'some old error')
+
+    def test_trace_unicode(self):
+        """Write Unicode to trace log"""
+        self.log(u'the unicode character for benzene is \N{BENZENE RING}')
+        self.assertContainsRe('the unicode character',
+                self._get_log())

=== added file 'bzrlib/tests/test_transactions.py'
--- bzrlib/tests/test_transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transactions.py	2006-03-08 23:51:07 +0000
@@ -0,0 +1,317 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the behaviour of the Transaction concept in bzr."""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase, TestCaseInTempDir
+import bzrlib.transactions as transactions
+
+
+class DummyWeave(object):
+    """A class that can be instantiated and compared."""
+
+    def __init__(self, message):
+        self._message = message
+        self.finished = False
+
+    def __eq__(self, other):
+        if other is None:
+            return False
+        return self._message == other._message
+
+    def transaction_finished(self):
+        self.finished = True
+
+
+class TestSymbols(TestCase):
+
+    def test_public_symbols(self):
+        from bzrlib.transactions import ReadOnlyTransaction
+        from bzrlib.transactions import PassThroughTransaction
+
+
+class TestReadOnlyTransaction(TestCase):
+
+    def setUp(self):
+        self.transaction = transactions.ReadOnlyTransaction()
+        super(TestReadOnlyTransaction, self).setUp()
+
+    def test_register_clean(self):
+        self.transaction.register_clean("anobject")
+
+    def test_register_dirty_raises(self):
+        self.assertRaises(errors.ReadOnlyError, 
+                          self.transaction.register_dirty,"anobject")
+    
+    def test_map(self):
+        self.assertNotEqual(None, getattr(self.transaction, "map", None))
+    
+    def test_add_and_get(self):
+        weave = "a weave"
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+
+    def test_finish_returns(self):
+        self.transaction.finish()
+
+    def test_finish_does_not_tell_versioned_file_finished(self):
+        # read only transactions never write, so theres no
+        # need to inform versioned files about finishing
+        weave = DummyWeave('a weave')
+        self.transaction.finish()
+        self.assertFalse(weave.finished)
+
+    def test_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should fall right out if there are no references
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        # but if we have a reference it should stick around
+        weave = DummyWeave("another weave")
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        del weave
+        # its not a weakref system
+        self.assertEqual(DummyWeave("another weave"),
+                         self.transaction.map.find_weave("id"))
+        
+    def test_small_cache(self):
+        self.transaction.set_cache_size(1)
+        # add an object, should not fall right out if there are no references
+        #sys.getrefcounts(foo)
+        self.transaction.map.add_weave("id", DummyWeave("a weave"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave"),
+                         self.transaction.map.find_weave("id"))
+        self.transaction.map.add_weave("id2", DummyWeave("a weave also"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id2"))
+        # currently a fifo
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave also"),
+                         self.transaction.map.find_weave("id2"))
+
+    def test_small_cache_with_references(self):
+        # if we have a reference it should stick around
+        weave = "a weave"
+        weave2 = "another weave"
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.map.add_weave("id2", weave2)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        self.assertEqual(weave2, self.transaction.map.find_weave("id2"))
+        weave = None
+        # its not a weakref system
+        self.assertEqual("a weave", self.transaction.map.find_weave("id"))
+
+    def test_precious_with_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(self.transaction.map.find_weave("id"),
+                                        precious=True)
+        self.assertEqual(DummyWeave('a weave'),
+                         self.transaction.map.find_weave("id"))
+
+    def test_precious_revision_history(self):
+        """Disabled test until revision-history is a real object."""
+        print "Disabled: test_precious_revision_history"
+        return
+        self.transaction.set_cache_size(0)
+        history = []
+        self.transaction.map.add_revision_history(history)
+        self.assertEqual(history, self.transaction.map.find_revision_history())
+        history = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(
+            self.transaction.map.find_revision_history(), precious=True)
+        self.assertEqual([], self.transaction.map.find_revision_history())
+
+    def test_writable(self):
+        self.assertFalse(self.transaction.writeable())
+
+
+class TestPassThroughTransaction(TestCase):
+
+    def test_construct(self):
+        transactions.PassThroughTransaction()
+
+    def test_register_clean(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_clean("anobject")
+    
+    def test_register_dirty(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_dirty("anobject")
+    
+    def test_map(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertNotEqual(None, getattr(transaction, "map", None))
+    
+    def test_add_and_get(self):
+        transaction = transactions.PassThroughTransaction()
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))
+        
+    def test_finish_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.finish()
+
+    def test_finish_tells_versioned_file_finished(self):
+        # pass through transactions allow writes so they
+        # need to inform versioned files about finishing
+        weave = DummyWeave('a weave')
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_dirty(weave)
+        transaction.finish()
+        self.assertTrue(weave.finished)
+
+    def test_cache_is_ignored(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.set_cache_size(100)
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))
+
+    def test_writable(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertTrue(transaction.writeable())
+        
+
+class TestWriteTransaction(TestCase):
+
+    def setUp(self):
+        self.transaction = transactions.WriteTransaction()
+        super(TestWriteTransaction, self).setUp()
+
+    def test_register_clean(self):
+        self.transaction.register_clean("anobject")
+    
+    def test_register_dirty(self):
+        self.transaction.register_dirty("anobject")
+    
+    def test_map(self):
+        self.assertNotEqual(None, getattr(self.transaction, "map", None))
+    
+    def test_add_and_get(self):
+        weave = "a weave"
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        
+    def test_finish_returns(self):
+        self.transaction.finish()
+
+    def test_finish_tells_versioned_file_finished(self):
+        # write transactions allow writes so they
+        # need to inform versioned files about finishing
+        weave = DummyWeave('a weave')
+        self.transaction.register_dirty(weave)
+        self.transaction.finish()
+        self.assertTrue(weave.finished)
+
+    def test_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        # add an object, should fall right out if there are no references
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        # but if we have a reference to a clean object it should stick around
+        weave = DummyWeave("another weave")
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        del weave
+        # its not a weakref system
+        self.assertEqual(DummyWeave("another weave"),
+                         self.transaction.map.find_weave("id"))
+
+    def test_zero_size_cache_dirty_objects(self):
+        self.transaction.set_cache_size(0)
+        # add a dirty object, which should not fall right out.
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        self.transaction.register_dirty(self.transaction.map.find_weave("id"))
+        self.assertNotEqual(None, self.transaction.map.find_weave("id"))
+    
+    def test_clean_to_dirty(self):
+        # a clean object may become dirty.
+        weave = DummyWeave('A weave')
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.register_clean(weave)
+        self.transaction.register_dirty(weave)
+        self.assertTrue(self.transaction.is_dirty(weave))
+        self.assertFalse(self.transaction.is_clean(weave))
+
+    def test_small_cache(self):
+        self.transaction.set_cache_size(1)
+        # add an object, should not fall right out if there are no references
+        #sys.getrefcounts(foo)
+        self.transaction.map.add_weave("id", DummyWeave("a weave"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave"),
+                         self.transaction.map.find_weave("id"))
+        self.transaction.map.add_weave("id2", DummyWeave("a weave also"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id2"))
+        # currently a fifo
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave also"),
+                         self.transaction.map.find_weave("id2"))
+
+    def test_small_cache_with_references(self):
+        # if we have a reference it should stick around
+        weave = "a weave"
+        weave2 = "another weave"
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.map.add_weave("id2", weave2)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        self.assertEqual(weave2, self.transaction.map.find_weave("id2"))
+        weave = None
+        # its not a weakref system
+        self.assertEqual("a weave", self.transaction.map.find_weave("id"))
+
+    def test_precious_with_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(self.transaction.map.find_weave("id"),
+                                        precious=True)
+        self.assertEqual(DummyWeave('a weave'),
+                         self.transaction.map.find_weave("id"))
+
+    def test_writable(self):
+        self.assertTrue(self.transaction.writeable())

=== added file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transform.py	2006-04-12 05:50:57 +0000
@@ -0,0 +1,734 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.bzrdir import BzrDir
+from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
+                              UnversionedParent, ParentLoop)
+from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
+                           ReusingTransform, CantMoveRoot, NotVersionedError,
+                           ExistingLimbo, ImmortalLimbo, LockError)
+from bzrlib.osutils import file_kind, has_symlinks, pathjoin
+from bzrlib.merge import Merge3Merger
+from bzrlib.tests import TestCaseInTempDir, TestSkipped, TestCase
+from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths, 
+                              resolve_conflicts, cook_conflicts, 
+                              find_interesting, build_tree, get_backup_name)
+
+class TestTreeTransform(TestCaseInTempDir):
+    def setUp(self):
+        super(TestTreeTransform, self).setUp()
+        self.wt = BzrDir.create_standalone_workingtree('.')
+        os.chdir('..')
+
+    def get_transform(self):
+        transform = TreeTransform(self.wt)
+        #self.addCleanup(transform.finalize)
+        return transform, transform.trans_id_tree_file_id(self.wt.get_root_id())
+
+    def test_existing_limbo(self):
+        limbo_name = self.wt._control_files.controlfilename('limbo')
+        transform, root = self.get_transform()
+        os.mkdir(pathjoin(limbo_name, 'hehe'))
+        self.assertRaises(ImmortalLimbo, transform.apply)
+        self.assertRaises(LockError, self.wt.unlock)
+        self.assertRaises(ExistingLimbo, self.get_transform)
+        self.assertRaises(LockError, self.wt.unlock)
+        os.rmdir(pathjoin(limbo_name, 'hehe'))
+        os.rmdir(limbo_name)
+        transform, root = self.get_transform()
+        transform.apply()
+
+    def test_build(self):
+        transform, root = self.get_transform() 
+        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
+        imaginary_id = transform.trans_id_tree_path('imaginary')
+        self.assertEqual(transform.get_tree_parent(imaginary_id), root)
+        self.assertEqual(transform.final_kind(root), 'directory')
+        self.assertEqual(transform.final_file_id(root), self.wt.get_root_id())
+        trans_id = transform.create_path('name', root)
+        self.assertIs(transform.final_file_id(trans_id), None)
+        self.assertRaises(NoSuchFile, transform.final_kind, trans_id)
+        transform.create_file('contents', trans_id)
+        transform.set_executability(True, trans_id)
+        transform.version_file('my_pretties', trans_id)
+        self.assertRaises(DuplicateKey, transform.version_file,
+                          'my_pretties', trans_id)
+        self.assertEqual(transform.final_file_id(trans_id), 'my_pretties')
+        self.assertEqual(transform.final_parent(trans_id), root)
+        self.assertIs(transform.final_parent(root), ROOT_PARENT)
+        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
+        oz_id = transform.create_path('oz', root)
+        transform.create_directory(oz_id)
+        transform.version_file('ozzie', oz_id)
+        trans_id2 = transform.create_path('name2', root)
+        transform.create_file('contents', trans_id2)
+        transform.set_executability(False, trans_id2)
+        transform.version_file('my_pretties2', trans_id2)
+        modified_paths = transform.apply().modified_paths
+        self.assertEqual('contents', self.wt.get_file_byname('name').read())
+        self.assertEqual(self.wt.path2id('name'), 'my_pretties')
+        self.assertIs(self.wt.is_executable('my_pretties'), True)
+        self.assertIs(self.wt.is_executable('my_pretties2'), False)
+        self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
+        self.assertEqual(len(modified_paths), 3)
+        tree_mod_paths = [self.wt.id2abspath(f) for f in 
+                          ('ozzie', 'my_pretties', 'my_pretties2')]
+        self.assertSubset(tree_mod_paths, modified_paths)
+        # is it safe to finalize repeatedly?
+        transform.finalize()
+        transform.finalize()
+
+    def test_convenience(self):
+        transform, root = self.get_transform()
+        trans_id = transform.new_file('name', root, 'contents', 
+                                      'my_pretties', True)
+        oz = transform.new_directory('oz', root, 'oz-id')
+        dorothy = transform.new_directory('dorothy', oz, 'dorothy-id')
+        toto = transform.new_file('toto', dorothy, 'toto-contents', 
+                                  'toto-id', False)
+
+        self.assertEqual(len(transform.find_conflicts()), 0)
+        transform.apply()
+        self.assertRaises(ReusingTransform, transform.find_conflicts)
+        self.assertEqual('contents', file(self.wt.abspath('name')).read())
+        self.assertEqual(self.wt.path2id('name'), 'my_pretties')
+        self.assertIs(self.wt.is_executable('my_pretties'), True)
+        self.assertEqual(self.wt.path2id('oz'), 'oz-id')
+        self.assertEqual(self.wt.path2id('oz/dorothy'), 'dorothy-id')
+        self.assertEqual(self.wt.path2id('oz/dorothy/toto'), 'toto-id')
+
+        self.assertEqual('toto-contents', 
+                         self.wt.get_file_byname('oz/dorothy/toto').read())
+        self.assertIs(self.wt.is_executable('toto-id'), False)
+
+    def test_conflicts(self):
+        transform, root = self.get_transform()
+        trans_id = transform.new_file('name', root, 'contents', 
+                                      'my_pretties')
+        self.assertEqual(len(transform.find_conflicts()), 0)
+        trans_id2 = transform.new_file('name', root, 'Crontents', 'toto')
+        self.assertEqual(transform.find_conflicts(), 
+                         [('duplicate', trans_id, trans_id2, 'name')])
+        self.assertRaises(MalformedTransform, transform.apply)
+        transform.adjust_path('name', trans_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('non-directory parent', trans_id)])
+        tinman_id = transform.trans_id_tree_path('tinman')
+        transform.adjust_path('name', tinman_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('unversioned parent', tinman_id), 
+                          ('missing parent', tinman_id)])
+        lion_id = transform.create_path('lion', root)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('unversioned parent', tinman_id), 
+                          ('missing parent', tinman_id)])
+        transform.adjust_path('name', lion_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('unversioned parent', lion_id),
+                          ('missing parent', lion_id)])
+        transform.version_file("Courage", lion_id)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('missing parent', lion_id), 
+                          ('versioning no contents', lion_id)])
+        transform.adjust_path('name2', root, trans_id2)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('versioning no contents', lion_id)])
+        transform.create_file('Contents, okay?', lion_id)
+        transform.adjust_path('name2', trans_id2, trans_id2)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('parent loop', trans_id2), 
+                          ('non-directory parent', trans_id2)])
+        transform.adjust_path('name2', root, trans_id2)
+        oz_id = transform.new_directory('oz', root)
+        transform.set_executability(True, oz_id)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('unversioned executability', oz_id)])
+        transform.version_file('oz-id', oz_id)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('non-file executability', oz_id)])
+        transform.set_executability(None, oz_id)
+        tip_id = transform.new_file('tip', oz_id, 'ozma', 'tip-id')
+        transform.apply()
+        self.assertEqual(self.wt.path2id('name'), 'my_pretties')
+        self.assertEqual('contents', file(self.wt.abspath('name')).read())
+        transform2, root = self.get_transform()
+        oz_id = transform2.trans_id_tree_file_id('oz-id')
+        newtip = transform2.new_file('tip', oz_id, 'other', 'tip-id')
+        result = transform2.find_conflicts()
+        fp = FinalPaths(transform2)
+        self.assert_('oz/tip' in transform2._tree_path_ids)
+        self.assertEqual(fp.get_path(newtip), pathjoin('oz', 'tip'))
+        self.assertEqual(len(result), 2)
+        self.assertEqual((result[0][0], result[0][1]), 
+                         ('duplicate', newtip))
+        self.assertEqual((result[1][0], result[1][2]), 
+                         ('duplicate id', newtip))
+        transform2.finalize()
+        transform3 = TreeTransform(self.wt)
+        self.addCleanup(transform3.finalize)
+        oz_id = transform3.trans_id_tree_file_id('oz-id')
+        transform3.delete_contents(oz_id)
+        self.assertEqual(transform3.find_conflicts(), 
+                         [('missing parent', oz_id)])
+        root_id = transform3.trans_id_tree_file_id('TREE_ROOT')
+        tip_id = transform3.trans_id_tree_file_id('tip-id')
+        transform3.adjust_path('tip', root_id, tip_id)
+        transform3.apply()
+
+    def test_add_del(self):
+        start, root = self.get_transform()
+        start.new_directory('a', root, 'a')
+        start.apply()
+        transform, root = self.get_transform()
+        transform.delete_versioned(transform.trans_id_tree_file_id('a'))
+        transform.new_directory('a', root, 'a')
+        transform.apply()
+
+    def test_unversioning(self):
+        create_tree, root = self.get_transform()
+        parent_id = create_tree.new_directory('parent', root, 'parent-id')
+        create_tree.new_file('child', parent_id, 'child', 'child-id')
+        create_tree.apply()
+        unversion = TreeTransform(self.wt)
+        self.addCleanup(unversion.finalize)
+        parent = unversion.trans_id_tree_path('parent')
+        unversion.unversion_file(parent)
+        self.assertEqual(unversion.find_conflicts(), 
+                         [('unversioned parent', parent_id)])
+        file_id = unversion.trans_id_tree_file_id('child-id')
+        unversion.unversion_file(file_id)
+        unversion.apply()
+
+    def test_name_invariants(self):
+        create_tree, root = self.get_transform()
+        # prepare tree
+        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        create_tree.new_file('name1', root, 'hello1', 'name1')
+        create_tree.new_file('name2', root, 'hello2', 'name2')
+        ddir = create_tree.new_directory('dying_directory', root, 'ddir')
+        create_tree.new_file('dying_file', ddir, 'goodbye1', 'dfile')
+        create_tree.new_file('moving_file', ddir, 'later1', 'mfile')
+        create_tree.new_file('moving_file2', root, 'later2', 'mfile2')
+        create_tree.apply()
+
+        mangle_tree,root = self.get_transform()
+        root = mangle_tree.trans_id_tree_file_id('TREE_ROOT')
+        #swap names
+        name1 = mangle_tree.trans_id_tree_file_id('name1')
+        name2 = mangle_tree.trans_id_tree_file_id('name2')
+        mangle_tree.adjust_path('name2', root, name1)
+        mangle_tree.adjust_path('name1', root, name2)
+
+        #tests for deleting parent directories 
+        ddir = mangle_tree.trans_id_tree_file_id('ddir')
+        mangle_tree.delete_contents(ddir)
+        dfile = mangle_tree.trans_id_tree_file_id('dfile')
+        mangle_tree.delete_versioned(dfile)
+        mangle_tree.unversion_file(dfile)
+        mfile = mangle_tree.trans_id_tree_file_id('mfile')
+        mangle_tree.adjust_path('mfile', root, mfile)
+
+        #tests for adding parent directories
+        newdir = mangle_tree.new_directory('new_directory', root, 'newdir')
+        mfile2 = mangle_tree.trans_id_tree_file_id('mfile2')
+        mangle_tree.adjust_path('mfile2', newdir, mfile2)
+        mangle_tree.new_file('newfile', newdir, 'hello3', 'dfile')
+        self.assertEqual(mangle_tree.final_file_id(mfile2), 'mfile2')
+        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
+        self.assertEqual(mangle_tree.final_file_id(mfile2), 'mfile2')
+        mangle_tree.apply()
+        self.assertEqual(file(self.wt.abspath('name1')).read(), 'hello2')
+        self.assertEqual(file(self.wt.abspath('name2')).read(), 'hello1')
+        mfile2_path = self.wt.abspath(pathjoin('new_directory','mfile2'))
+        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
+        self.assertEqual(file(mfile2_path).read(), 'later2')
+        self.assertEqual(self.wt.id2path('mfile2'), 'new_directory/mfile2')
+        self.assertEqual(self.wt.path2id('new_directory/mfile2'), 'mfile2')
+        newfile_path = self.wt.abspath(pathjoin('new_directory','newfile'))
+        self.assertEqual(file(newfile_path).read(), 'hello3')
+        self.assertEqual(self.wt.path2id('dying_directory'), 'ddir')
+        self.assertIs(self.wt.path2id('dying_directory/dying_file'), None)
+        mfile2_path = self.wt.abspath(pathjoin('new_directory','mfile2'))
+
+    def test_both_rename(self):
+        create_tree,root = self.get_transform()
+        newdir = create_tree.new_directory('selftest', root, 'selftest-id')
+        create_tree.new_file('blackbox.py', newdir, 'hello1', 'blackbox-id')
+        create_tree.apply()        
+        mangle_tree,root = self.get_transform()
+        selftest = mangle_tree.trans_id_tree_file_id('selftest-id')
+        blackbox = mangle_tree.trans_id_tree_file_id('blackbox-id')
+        mangle_tree.adjust_path('test', root, selftest)
+        mangle_tree.adjust_path('test_too_much', root, selftest)
+        mangle_tree.set_executability(True, blackbox)
+        mangle_tree.apply()
+
+    def test_both_rename2(self):
+        create_tree,root = self.get_transform()
+        bzrlib = create_tree.new_directory('bzrlib', root, 'bzrlib-id')
+        tests = create_tree.new_directory('tests', bzrlib, 'tests-id')
+        blackbox = create_tree.new_directory('blackbox', tests, 'blackbox-id')
+        create_tree.new_file('test_too_much.py', blackbox, 'hello1', 
+                             'test_too_much-id')
+        create_tree.apply()        
+        mangle_tree,root = self.get_transform()
+        bzrlib = mangle_tree.trans_id_tree_file_id('bzrlib-id')
+        tests = mangle_tree.trans_id_tree_file_id('tests-id')
+        test_too_much = mangle_tree.trans_id_tree_file_id('test_too_much-id')
+        mangle_tree.adjust_path('selftest', bzrlib, tests)
+        mangle_tree.adjust_path('blackbox.py', tests, test_too_much) 
+        mangle_tree.set_executability(True, test_too_much)
+        mangle_tree.apply()
+
+    def test_both_rename3(self):
+        create_tree,root = self.get_transform()
+        tests = create_tree.new_directory('tests', root, 'tests-id')
+        create_tree.new_file('test_too_much.py', tests, 'hello1', 
+                             'test_too_much-id')
+        create_tree.apply()        
+        mangle_tree,root = self.get_transform()
+        tests = mangle_tree.trans_id_tree_file_id('tests-id')
+        test_too_much = mangle_tree.trans_id_tree_file_id('test_too_much-id')
+        mangle_tree.adjust_path('selftest', root, tests)
+        mangle_tree.adjust_path('blackbox.py', tests, test_too_much) 
+        mangle_tree.set_executability(True, test_too_much)
+        mangle_tree.apply()
+
+    def test_move_dangling_ie(self):
+        create_tree, root = self.get_transform()
+        # prepare tree
+        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        create_tree.new_file('name1', root, 'hello1', 'name1')
+        create_tree.apply()
+        delete_contents, root = self.get_transform()
+        file = delete_contents.trans_id_tree_file_id('name1')
+        delete_contents.delete_contents(file)
+        delete_contents.apply()
+        move_id, root = self.get_transform()
+        name1 = move_id.trans_id_tree_file_id('name1')
+        newdir = move_id.new_directory('dir', root, 'newdir')
+        move_id.adjust_path('name2', newdir, name1)
+        move_id.apply()
+        
+    def test_replace_dangling_ie(self):
+        create_tree, root = self.get_transform()
+        # prepare tree
+        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        create_tree.new_file('name1', root, 'hello1', 'name1')
+        create_tree.apply()
+        delete_contents = TreeTransform(self.wt)
+        self.addCleanup(delete_contents.finalize)
+        file = delete_contents.trans_id_tree_file_id('name1')
+        delete_contents.delete_contents(file)
+        delete_contents.apply()
+        delete_contents.finalize()
+        replace = TreeTransform(self.wt)
+        self.addCleanup(replace.finalize)
+        name2 = replace.new_file('name2', root, 'hello2', 'name1')
+        conflicts = replace.find_conflicts()
+        name1 = replace.trans_id_tree_file_id('name1')
+        self.assertEqual(conflicts, [('duplicate id', name1, name2)])
+        resolve_conflicts(replace)
+        replace.apply()
+
+    def test_symlinks(self):
+        if not has_symlinks():
+            raise TestSkipped('Symlinks are not supported on this platform')
+        transform,root = self.get_transform()
+        oz_id = transform.new_directory('oz', root, 'oz-id')
+        wizard = transform.new_symlink('wizard', oz_id, 'wizard-target', 
+                                       'wizard-id')
+        wiz_id = transform.create_path('wizard2', oz_id)
+        transform.create_symlink('behind_curtain', wiz_id)
+        transform.version_file('wiz-id2', wiz_id)            
+        transform.set_executability(True, wiz_id)
+        self.assertEqual(transform.find_conflicts(), 
+                         [('non-file executability', wiz_id)])
+        transform.set_executability(None, wiz_id)
+        transform.apply()
+        self.assertEqual(self.wt.path2id('oz/wizard'), 'wizard-id')
+        self.assertEqual(file_kind(self.wt.abspath('oz/wizard')), 'symlink')
+        self.assertEqual(os.readlink(self.wt.abspath('oz/wizard2')), 
+                         'behind_curtain')
+        self.assertEqual(os.readlink(self.wt.abspath('oz/wizard')),
+                         'wizard-target')
+
+
+    def get_conflicted(self):
+        create,root = self.get_transform()
+        create.new_file('dorothy', root, 'dorothy', 'dorothy-id')
+        oz = create.new_directory('oz', root, 'oz-id')
+        create.new_directory('emeraldcity', oz, 'emerald-id')
+        create.apply()
+        conflicts,root = self.get_transform()
+        # set up duplicate entry, duplicate id
+        new_dorothy = conflicts.new_file('dorothy', root, 'dorothy', 
+                                         'dorothy-id')
+        old_dorothy = conflicts.trans_id_tree_file_id('dorothy-id')
+        oz = conflicts.trans_id_tree_file_id('oz-id')
+        # set up missing, unversioned parent
+        conflicts.delete_versioned(oz)
+        emerald = conflicts.trans_id_tree_file_id('emerald-id')
+        # set up parent loop
+        conflicts.adjust_path('emeraldcity', emerald, emerald)
+        return conflicts, emerald, oz, old_dorothy, new_dorothy
+
+    def test_conflict_resolution(self):
+        conflicts, emerald, oz, old_dorothy, new_dorothy =\
+            self.get_conflicted()
+        resolve_conflicts(conflicts)
+        self.assertEqual(conflicts.final_name(old_dorothy), 'dorothy.moved')
+        self.assertIs(conflicts.final_file_id(old_dorothy), None)
+        self.assertEqual(conflicts.final_name(new_dorothy), 'dorothy')
+        self.assertEqual(conflicts.final_file_id(new_dorothy), 'dorothy-id')
+        self.assertEqual(conflicts.final_parent(emerald), oz)
+        conflicts.apply()
+
+    def test_cook_conflicts(self):
+        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        raw_conflicts = resolve_conflicts(tt)
+        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved', 
+                                   'dorothy', None, 'dorothy-id')
+        self.assertEqual(cooked_conflicts[0], duplicate)
+        duplicate_id = DuplicateID('Unversioned existing file', 
+                                   'dorothy.moved', 'dorothy', None,
+                                   'dorothy-id')
+        self.assertEqual(cooked_conflicts[1], duplicate_id)
+        missing_parent = MissingParent('Not deleting', 'oz', 'oz-id')
+        self.assertEqual(cooked_conflicts[2], missing_parent)
+        unversioned_parent = UnversionedParent('Versioned directory', 'oz',
+                                               'oz-id')
+        self.assertEqual(cooked_conflicts[3], unversioned_parent)
+        parent_loop = ParentLoop('Cancelled move', 'oz/emeraldcity', 
+                                 'oz/emeraldcity', 'emerald-id', 'emerald-id')
+        self.assertEqual(cooked_conflicts[4], parent_loop)
+        self.assertEqual(len(cooked_conflicts), 5)
+        tt.finalize()
+
+    def test_string_conflicts(self):
+        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        raw_conflicts = resolve_conflicts(tt)
+        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        tt.finalize()
+        conflicts_s = [str(c) for c in cooked_conflicts]
+        self.assertEqual(len(cooked_conflicts), len(conflicts_s))
+        self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
+                                         'Moved existing file to '
+                                         'dorothy.moved.')
+        self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
+                                         'Unversioned existing file '
+                                         'dorothy.moved.')
+        self.assertEqual(conflicts_s[2], 'Conflict adding files to oz.  '
+                                         'Not deleting.')
+        self.assertEqual(conflicts_s[3], 'Conflict adding versioned files to '
+                                         'oz.  Versioned directory.')
+        self.assertEqual(conflicts_s[4], 'Conflict moving oz/emeraldcity into'
+                                         ' oz/emeraldcity.  Cancelled move.')
+
+    def test_moving_versioned_directories(self):
+        create, root = self.get_transform()
+        kansas = create.new_directory('kansas', root, 'kansas-id')
+        create.new_directory('house', kansas, 'house-id')
+        create.new_directory('oz', root, 'oz-id')
+        create.apply()
+        cyclone, root = self.get_transform()
+        oz = cyclone.trans_id_tree_file_id('oz-id')
+        house = cyclone.trans_id_tree_file_id('house-id')
+        cyclone.adjust_path('house', oz, house)
+        cyclone.apply()
+
+    def test_moving_root(self):
+        create, root = self.get_transform()
+        fun = create.new_directory('fun', root, 'fun-id')
+        create.new_directory('sun', root, 'sun-id')
+        create.new_directory('moon', root, 'moon')
+        create.apply()
+        transform, root = self.get_transform()
+        transform.adjust_root_path('oldroot', fun)
+        new_root=transform.trans_id_tree_path('')
+        transform.version_file('new-root', new_root)
+        transform.apply()
+
+    def test_renames(self):
+        create, root = self.get_transform()
+        old = create.new_directory('old-parent', root, 'old-id')
+        intermediate = create.new_directory('intermediate', old, 'im-id')
+        myfile = create.new_file('myfile', intermediate, 'myfile-text',
+                                 'myfile-id')
+        create.apply()
+        rename, root = self.get_transform()
+        old = rename.trans_id_file_id('old-id')
+        rename.adjust_path('new', root, old)
+        myfile = rename.trans_id_file_id('myfile-id')
+        rename.set_executability(True, myfile)
+        rename.apply()
+
+    def test_find_interesting(self):
+        create, root = self.get_transform()
+        wt = create._tree
+        create.new_file('vfile', root, 'myfile-text', 'myfile-id')
+        create.new_file('uvfile', root, 'othertext')
+        create.apply()
+        self.assertEqual(find_interesting(wt, wt, ['vfile']),
+                         set(['myfile-id']))
+        self.assertRaises(NotVersionedError, find_interesting, wt, wt,
+                          ['uvfile'])
+
+
+class TransformGroup(object):
+    def __init__(self, dirname):
+        self.name = dirname
+        os.mkdir(dirname)
+        self.wt = BzrDir.create_standalone_workingtree(dirname)
+        self.b = self.wt.branch
+        self.tt = TreeTransform(self.wt)
+        self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
+
+def conflict_text(tree, merge):
+    template = '%s TREE\n%s%s\n%s%s MERGE-SOURCE\n'
+    return template % ('<' * 7, tree, '=' * 7, merge, '>' * 7)
+
+
+class TestTransformMerge(TestCaseInTempDir):
+    def test_text_merge(self):
+        base = TransformGroup("base")
+        base.tt.new_file('a', base.root, 'a\nb\nc\nd\be\n', 'a')
+        base.tt.new_file('b', base.root, 'b1', 'b')
+        base.tt.new_file('c', base.root, 'c', 'c')
+        base.tt.new_file('d', base.root, 'd', 'd')
+        base.tt.new_file('e', base.root, 'e', 'e')
+        base.tt.new_file('f', base.root, 'f', 'f')
+        base.tt.new_directory('g', base.root, 'g')
+        base.tt.new_directory('h', base.root, 'h')
+        base.tt.apply()
+        other = TransformGroup("other")
+        other.tt.new_file('a', other.root, 'y\nb\nc\nd\be\n', 'a')
+        other.tt.new_file('b', other.root, 'b2', 'b')
+        other.tt.new_file('c', other.root, 'c2', 'c')
+        other.tt.new_file('d', other.root, 'd', 'd')
+        other.tt.new_file('e', other.root, 'e2', 'e')
+        other.tt.new_file('f', other.root, 'f', 'f')
+        other.tt.new_file('g', other.root, 'g', 'g')
+        other.tt.new_file('h', other.root, 'h\ni\nj\nk\n', 'h')
+        other.tt.new_file('i', other.root, 'h\ni\nj\nk\n', 'i')
+        other.tt.apply()
+        this = TransformGroup("this")
+        this.tt.new_file('a', this.root, 'a\nb\nc\nd\bz\n', 'a')
+        this.tt.new_file('b', this.root, 'b', 'b')
+        this.tt.new_file('c', this.root, 'c', 'c')
+        this.tt.new_file('d', this.root, 'd2', 'd')
+        this.tt.new_file('e', this.root, 'e2', 'e')
+        this.tt.new_file('f', this.root, 'f', 'f')
+        this.tt.new_file('g', this.root, 'g', 'g')
+        this.tt.new_file('h', this.root, '1\n2\n3\n4\n', 'h')
+        this.tt.new_file('i', this.root, '1\n2\n3\n4\n', 'i')
+        this.tt.apply()
+        Merge3Merger(this.wt, this.wt, base.wt, other.wt)
+        # textual merge
+        self.assertEqual(this.wt.get_file('a').read(), 'y\nb\nc\nd\bz\n')
+        # three-way text conflict
+        self.assertEqual(this.wt.get_file('b').read(), 
+                         conflict_text('b', 'b2'))
+        # OTHER wins
+        self.assertEqual(this.wt.get_file('c').read(), 'c2')
+        # THIS wins
+        self.assertEqual(this.wt.get_file('d').read(), 'd2')
+        # Ambigious clean merge
+        self.assertEqual(this.wt.get_file('e').read(), 'e2')
+        # No change
+        self.assertEqual(this.wt.get_file('f').read(), 'f')
+        # Correct correct results when THIS == OTHER 
+        self.assertEqual(this.wt.get_file('g').read(), 'g')
+        # Text conflict when THIS & OTHER are text and BASE is dir
+        self.assertEqual(this.wt.get_file('h').read(), 
+                         conflict_text('1\n2\n3\n4\n', 'h\ni\nj\nk\n'))
+        self.assertEqual(this.wt.get_file_byname('h.THIS').read(),
+                         '1\n2\n3\n4\n')
+        self.assertEqual(this.wt.get_file_byname('h.OTHER').read(),
+                         'h\ni\nj\nk\n')
+        self.assertEqual(file_kind(this.wt.abspath('h.BASE')), 'directory')
+        self.assertEqual(this.wt.get_file('i').read(), 
+                         conflict_text('1\n2\n3\n4\n', 'h\ni\nj\nk\n'))
+        self.assertEqual(this.wt.get_file_byname('i.THIS').read(),
+                         '1\n2\n3\n4\n')
+        self.assertEqual(this.wt.get_file_byname('i.OTHER').read(),
+                         'h\ni\nj\nk\n')
+        self.assertEqual(os.path.exists(this.wt.abspath('i.BASE')), False)
+        modified = ['a', 'b', 'c', 'h', 'i']
+        merge_modified = this.wt.merge_modified()
+        self.assertSubset(merge_modified, modified)
+        self.assertEqual(len(merge_modified), len(modified))
+        file(this.wt.id2abspath('a'), 'wb').write('booga')
+        modified.pop(0)
+        merge_modified = this.wt.merge_modified()
+        self.assertSubset(merge_modified, modified)
+        self.assertEqual(len(merge_modified), len(modified))
+        this.wt.remove('b')
+        this.wt.revert([])
+
+    def test_file_merge(self):
+        if not has_symlinks():
+            raise TestSkipped('Symlinks are not supported on this platform')
+        base = TransformGroup("BASE")
+        this = TransformGroup("THIS")
+        other = TransformGroup("OTHER")
+        for tg in this, base, other:
+            tg.tt.new_directory('a', tg.root, 'a')
+            tg.tt.new_symlink('b', tg.root, 'b', 'b')
+            tg.tt.new_file('c', tg.root, 'c', 'c')
+            tg.tt.new_symlink('d', tg.root, tg.name, 'd')
+        targets = ((base, 'base-e', 'base-f', None, None), 
+                   (this, 'other-e', 'this-f', 'other-g', 'this-h'), 
+                   (other, 'other-e', None, 'other-g', 'other-h'))
+        for tg, e_target, f_target, g_target, h_target in targets:
+            for link, target in (('e', e_target), ('f', f_target), 
+                                 ('g', g_target), ('h', h_target)):
+                if target is not None:
+                    tg.tt.new_symlink(link, tg.root, target, link)
+
+        for tg in this, base, other:
+            tg.tt.apply()
+        Merge3Merger(this.wt, this.wt, base.wt, other.wt)
+        self.assertIs(os.path.isdir(this.wt.abspath('a')), True)
+        self.assertIs(os.path.islink(this.wt.abspath('b')), True)
+        self.assertIs(os.path.isfile(this.wt.abspath('c')), True)
+        for suffix in ('THIS', 'BASE', 'OTHER'):
+            self.assertEqual(os.readlink(this.wt.abspath('d.'+suffix)), suffix)
+        self.assertIs(os.path.lexists(this.wt.abspath('d')), False)
+        self.assertEqual(this.wt.id2path('d'), 'd.OTHER')
+        self.assertEqual(this.wt.id2path('f'), 'f.THIS')
+        self.assertEqual(os.readlink(this.wt.abspath('e')), 'other-e')
+        self.assertIs(os.path.lexists(this.wt.abspath('e.THIS')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('e.OTHER')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('e.BASE')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('g')), True)
+        self.assertIs(os.path.lexists(this.wt.abspath('g.BASE')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('h')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('h.BASE')), False)
+        self.assertIs(os.path.lexists(this.wt.abspath('h.THIS')), True)
+        self.assertIs(os.path.lexists(this.wt.abspath('h.OTHER')), True)
+
+    def test_filename_merge(self):
+        base = TransformGroup("BASE")
+        this = TransformGroup("THIS")
+        other = TransformGroup("OTHER")
+        base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
+                                   for t in [base, this, other]]
+        base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
+                                   for t in [base, this, other]]
+        base.tt.new_directory('c', base_a, 'c')
+        this.tt.new_directory('c1', this_a, 'c')
+        other.tt.new_directory('c', other_b, 'c')
+
+        base.tt.new_directory('d', base_a, 'd')
+        this.tt.new_directory('d1', this_b, 'd')
+        other.tt.new_directory('d', other_a, 'd')
+
+        base.tt.new_directory('e', base_a, 'e')
+        this.tt.new_directory('e', this_a, 'e')
+        other.tt.new_directory('e1', other_b, 'e')
+
+        base.tt.new_directory('f', base_a, 'f')
+        this.tt.new_directory('f1', this_b, 'f')
+        other.tt.new_directory('f1', other_b, 'f')
+
+        for tg in [this, base, other]:
+            tg.tt.apply()
+        Merge3Merger(this.wt, this.wt, base.wt, other.wt)
+        self.assertEqual(this.wt.id2path('c'), pathjoin('b/c1'))
+        self.assertEqual(this.wt.id2path('d'), pathjoin('b/d1'))
+        self.assertEqual(this.wt.id2path('e'), pathjoin('b/e1'))
+        self.assertEqual(this.wt.id2path('f'), pathjoin('b/f1'))
+
+    def test_filename_merge_conflicts(self):
+        base = TransformGroup("BASE")
+        this = TransformGroup("THIS")
+        other = TransformGroup("OTHER")
+        base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
+                                   for t in [base, this, other]]
+        base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
+                                   for t in [base, this, other]]
+
+        base.tt.new_file('g', base_a, 'g', 'g')
+        other.tt.new_file('g1', other_b, 'g1', 'g')
+
+        base.tt.new_file('h', base_a, 'h', 'h')
+        this.tt.new_file('h1', this_b, 'h1', 'h')
+
+        base.tt.new_file('i', base.root, 'i', 'i')
+        other.tt.new_directory('i1', this_b, 'i')
+
+        for tg in [this, base, other]:
+            tg.tt.apply()
+        Merge3Merger(this.wt, this.wt, base.wt, other.wt)
+
+        self.assertEqual(this.wt.id2path('g'), pathjoin('b/g1.OTHER'))
+        self.assertIs(os.path.lexists(this.wt.abspath('b/g1.BASE')), True)
+        self.assertIs(os.path.lexists(this.wt.abspath('b/g1.THIS')), False)
+        self.assertEqual(this.wt.id2path('h'), pathjoin('b/h1.THIS'))
+        self.assertIs(os.path.lexists(this.wt.abspath('b/h1.BASE')), True)
+        self.assertIs(os.path.lexists(this.wt.abspath('b/h1.OTHER')), False)
+        self.assertEqual(this.wt.id2path('i'), pathjoin('b/i1.OTHER'))
+
+class TestBuildTree(TestCaseInTempDir):
+    def test_build_tree(self):
+        if not has_symlinks():
+            raise TestSkipped('Test requires symlink support')
+        os.mkdir('a')
+        a = BzrDir.create_standalone_workingtree('a')
+        os.mkdir('a/foo')
+        file('a/foo/bar', 'wb').write('contents')
+        os.symlink('a/foo/bar', 'a/foo/baz')
+        a.add(['foo', 'foo/bar', 'foo/baz'])
+        a.commit('initial commit')
+        b = BzrDir.create_standalone_workingtree('b')
+        build_tree(a.basis_tree(), b)
+        self.assertIs(os.path.isdir('b/foo'), True)
+        self.assertEqual(file('b/foo/bar', 'rb').read(), "contents")
+        self.assertEqual(os.readlink('b/foo/baz'), 'a/foo/bar')
+        
+class MockTransform(object):
+
+    def has_named_child(self, by_parent, parent_id, name):
+        for child_id in by_parent[parent_id]:
+            if child_id == '0':
+                if name == "name~":
+                    return True
+            elif name == "name.~%s~" % child_id:
+                return True
+        return False
+
+class MockEntry(object):
+    def __init__(self):
+        object.__init__(self)
+        self.name = "name"
+
+class TestGetBackupName(TestCase):
+    def test_get_backup_name(self):
+        tt = MockTransform()
+        name = get_backup_name(MockEntry(), {'a':[]}, 'a', tt)
+        self.assertEqual(name, 'name.~1~')
+        name = get_backup_name(MockEntry(), {'a':['1']}, 'a', tt)
+        self.assertEqual(name, 'name.~2~')
+        name = get_backup_name(MockEntry(), {'a':['2']}, 'a', tt)
+        self.assertEqual(name, 'name.~1~')
+        name = get_backup_name(MockEntry(), {'a':['2'], 'b':[]}, 'b', tt)
+        self.assertEqual(name, 'name.~1~')
+        name = get_backup_name(MockEntry(), {'a':['1', '2', '3']}, 'a', tt)
+        self.assertEqual(name, 'name.~4~')

=== added file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transport.py	2006-04-12 04:52:20 +0000
@@ -0,0 +1,309 @@
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+import sys
+import stat
+from cStringIO import StringIO
+
+import bzrlib
+from bzrlib.errors import (NoSuchFile, FileExists,
+                           TransportNotPossible,
+                           ConnectionError,
+                           DependencyNotPresent,
+                           )
+from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.transport import (_get_protocol_handlers,
+                              _get_transport_modules,
+                              get_transport,
+                              register_lazy_transport,
+                              _set_protocol_handlers,
+                              urlescape,
+                              Transport,
+                              )
+from bzrlib.transport.memory import MemoryTransport
+from bzrlib.transport.local import LocalTransport
+
+
+class TestTransport(TestCase):
+    """Test the non transport-concrete class functionality."""
+
+    def test_urlescape(self):
+        self.assertEqual('%25', urlescape('%'))
+
+    def test__get_set_protocol_handlers(self):
+        handlers = _get_protocol_handlers()
+        self.assertNotEqual({}, handlers)
+        try:
+            _set_protocol_handlers({})
+            self.assertEqual({}, _get_protocol_handlers())
+        finally:
+            _set_protocol_handlers(handlers)
+
+    def test_get_transport_modules(self):
+        handlers = _get_protocol_handlers()
+        class SampleHandler(object):
+            """I exist, isnt that enough?"""
+        try:
+            my_handlers = {}
+            _set_protocol_handlers(my_handlers)
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
+            register_lazy_transport('bar', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
+            self.assertEqual([SampleHandler.__module__],
+                             _get_transport_modules())
+        finally:
+            _set_protocol_handlers(handlers)
+
+    def test_transport_dependency(self):
+        """Transport with missing dependency causes no error"""
+        saved_handlers = _get_protocol_handlers()
+        try:
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport',
+                    'BadTransportHandler')
+            t = get_transport('foo://fooserver/foo')
+            # because we failed to load the transport
+            self.assertTrue(isinstance(t, LocalTransport))
+        finally:
+            # restore original values
+            _set_protocol_handlers(saved_handlers)
+            
+    def test_transport_fallback(self):
+        """Transport with missing dependency causes no error"""
+        saved_handlers = _get_protocol_handlers()
+        try:
+            _set_protocol_handlers({})
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport',
+                    'BackupTransportHandler')
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport',
+                    'BadTransportHandler')
+            t = get_transport('foo://fooserver/foo')
+            self.assertTrue(isinstance(t, BackupTransportHandler))
+        finally:
+            _set_protocol_handlers(saved_handlers)
+            
+
+class TestMemoryTransport(TestCase):
+
+    def test_get_transport(self):
+        MemoryTransport()
+
+    def test_clone(self):
+        transport = MemoryTransport()
+        self.assertTrue(isinstance(transport, MemoryTransport))
+
+    def test_abspath(self):
+        transport = MemoryTransport()
+        self.assertEqual("memory:/relpath", transport.abspath('relpath'))
+
+    def test_relpath(self):
+        transport = MemoryTransport()
+
+    def test_append_and_get(self):
+        transport = MemoryTransport()
+        transport.append('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+        transport.append('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'contentcontent')
+
+    def test_put_and_get(self):
+        transport = MemoryTransport()
+        transport.put('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+        transport.put('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+
+    def test_append_without_dir_fails(self):
+        transport = MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.append, 'dir/path', StringIO('content'))
+
+    def test_put_without_dir_fails(self):
+        transport = MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.put, 'dir/path', StringIO('content'))
+
+    def test_get_missing(self):
+        transport = MemoryTransport()
+        self.assertRaises(NoSuchFile, transport.get, 'foo')
+
+    def test_has_missing(self):
+        transport = MemoryTransport()
+        self.assertEquals(False, transport.has('foo'))
+
+    def test_has_present(self):
+        transport = MemoryTransport()
+        transport.append('foo', StringIO('content'))
+        self.assertEquals(True, transport.has('foo'))
+
+    def test_mkdir(self):
+        transport = MemoryTransport()
+        transport.mkdir('dir')
+        transport.append('dir/path', StringIO('content'))
+        self.assertEqual(transport.get('dir/path').read(), 'content')
+
+    def test_mkdir_missing_parent(self):
+        transport = MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.mkdir, 'dir/dir')
+
+    def test_mkdir_twice(self):
+        transport = MemoryTransport()
+        transport.mkdir('dir')
+        self.assertRaises(FileExists, transport.mkdir, 'dir')
+
+    def test_parameters(self):
+        transport = MemoryTransport()
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+        self.assertEqual(False, transport.is_readonly())
+
+    def test_iter_files_recursive(self):
+        transport = MemoryTransport()
+        transport.mkdir('dir')
+        transport.put('dir/foo', StringIO('content'))
+        transport.put('dir/bar', StringIO('content'))
+        transport.put('bar', StringIO('content'))
+        paths = set(transport.iter_files_recursive())
+        self.assertEqual(set(['dir/foo', 'dir/bar', 'bar']), paths)
+
+    def test_stat(self):
+        transport = MemoryTransport()
+        transport.put('foo', StringIO('content'))
+        transport.put('bar', StringIO('phowar'))
+        self.assertEqual(7, transport.stat('foo').st_size)
+        self.assertEqual(6, transport.stat('bar').st_size)
+
+        
+class ReadonlyDecoratorTransportTest(TestCase):
+    """Readonly decoration specific tests."""
+
+    def test_local_parameters(self):
+        import bzrlib.transport.readonly as readonly
+        # connect to . in readonly mode
+        transport = readonly.ReadonlyTransportDecorator('readonly+.')
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+        self.assertEqual(True, transport.is_readonly())
+
+    def test_http_parameters(self):
+        import bzrlib.transport.readonly as readonly
+        from bzrlib.transport.http import HttpServer
+        # connect to . via http which is not listable
+        server = HttpServer()
+        server.setUp()
+        try:
+            transport = get_transport('readonly+' + server.get_url())
+            self.failUnless(isinstance(transport,
+                                       readonly.ReadonlyTransportDecorator))
+            self.assertEqual(False, transport.listable())
+            self.assertEqual(True, transport.should_cache())
+            self.assertEqual(True, transport.is_readonly())
+        finally:
+            server.tearDown()
+
+
+class FakeNFSDecoratorTests(TestCaseInTempDir):
+    """NFS decorator specific tests."""
+
+    def get_nfs_transport(self, url):
+        import bzrlib.transport.fakenfs as fakenfs
+        # connect to url with nfs decoration
+        return fakenfs.FakeNFSTransportDecorator('fakenfs+' + url)
+
+    def test_local_parameters(self):
+        # the listable, should_cache and is_readonly parameters
+        # are not changed by the fakenfs decorator
+        transport = self.get_nfs_transport('.')
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+        self.assertEqual(False, transport.is_readonly())
+
+    def test_http_parameters(self):
+        # the listable, should_cache and is_readonly parameters
+        # are not changed by the fakenfs decorator
+        from bzrlib.transport.http import HttpServer
+        # connect to . via http which is not listable
+        server = HttpServer()
+        server.setUp()
+        try:
+            transport = self.get_nfs_transport(server.get_url())
+            self.assertIsInstance(
+                transport, bzrlib.transport.fakenfs.FakeNFSTransportDecorator)
+            self.assertEqual(False, transport.listable())
+            self.assertEqual(True, transport.should_cache())
+            self.assertEqual(True, transport.is_readonly())
+        finally:
+            server.tearDown()
+
+    def test_fakenfs_server_default(self):
+        # a FakeNFSServer() should bring up a local relpath server for itself
+        import bzrlib.transport.fakenfs as fakenfs
+        server = fakenfs.FakeNFSServer()
+        server.setUp()
+        try:
+            # the server should be a relpath localhost server
+            self.assertEqual(server.get_url(), 'fakenfs+.')
+            # and we should be able to get a transport for it
+            transport = get_transport(server.get_url())
+            # which must be a FakeNFSTransportDecorator instance.
+            self.assertIsInstance(
+                transport, fakenfs.FakeNFSTransportDecorator)
+        finally:
+            server.tearDown()
+
+    def test_fakenfs_rename_semantics(self):
+        # a FakeNFS transport must mangle the way rename errors occur to
+        # look like NFS problems.
+        transport = self.get_nfs_transport('.')
+        self.build_tree(['from/', 'from/foo', 'to/', 'to/bar'],
+                        transport=transport)
+        self.assertRaises(bzrlib.errors.ResourceBusy,
+                          transport.rename, 'from', 'to')
+
+
+class FakeVFATDecoratorTests(TestCaseInTempDir):
+    """Tests for simulation of VFAT restrictions"""
+
+    def get_vfat_transport(self, url):
+        """Return vfat-backed transport for test directory"""
+        from bzrlib.transport.fakevfat import FakeVFATTransportDecorator
+        return FakeVFATTransportDecorator('vfat+' + url)
+
+    def test_transport_creation(self):
+        from bzrlib.transport.fakevfat import FakeVFATTransportDecorator
+        transport = self.get_vfat_transport('.')
+        self.assertIsInstance(transport, FakeVFATTransportDecorator)
+
+    def test_transport_mkdir(self):
+        transport = self.get_vfat_transport('.')
+        transport.mkdir('HELLO')
+        self.assertTrue(transport.has('hello'))
+        self.assertTrue(transport.has('Hello'))
+
+    def test_forbidden_chars(self):
+        transport = self.get_vfat_transport('.')
+        self.assertRaises(ValueError, transport.has, "<NU>")
+
+
+class BadTransportHandler(Transport):
+    def __init__(self, base_url):
+        raise DependencyNotPresent('some_lib', 'testing missing dependency')
+
+
+class BackupTransportHandler(Transport):
+    """Test transport that works as a backup for the BadTransportHandler"""
+    pass

=== added file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-04-12 04:45:32 +0000
@@ -0,0 +1,945 @@
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for Transport implementations.
+
+Transport implementations tested here are supplied by
+TransportTestProviderAdapter.
+"""
+
+import os
+from cStringIO import StringIO
+import stat
+import sys
+
+from bzrlib.errors import (DirectoryNotEmpty, NoSuchFile, FileExists,
+                           LockError,
+                           PathError,
+                           TransportNotPossible, ConnectionError)
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.transport import memory, urlescape
+import bzrlib.transport
+
+
+def _append(fn, txt):
+    """Append the given text (file-like object) to the supplied filename."""
+    f = open(fn, 'ab')
+    try:
+        f.write(txt.read())
+    finally:
+        f.close()
+
+
+class TestTransportImplementation(TestCaseInTempDir):
+    """Implementation verification for transports.
+    
+    To verify a transport we need a server factory, which is a callable
+    that accepts no parameters and returns an implementation of
+    bzrlib.transport.Server.
+    
+    That Server is then used to construct transport instances and test
+    the transport via loopback activity.
+
+    Currently this assumes that the Transport object is connected to the 
+    current working directory.  So that whatever is done 
+    through the transport, should show up in the working 
+    directory, and vice-versa. This is a bug, because its possible to have
+    URL schemes which provide access to something that may not be 
+    result in storage on the local disk, i.e. due to file system limits, or 
+    due to it being a database or some other non-filesystem tool.
+
+    This also tests to make sure that the functions work with both
+    generators and lists (assuming iter(list) is effectively a generator)
+    """
+    
+    def setUp(self):
+        super(TestTransportImplementation, self).setUp()
+        self._server = self.transport_server()
+        self._server.setUp()
+
+    def tearDown(self):
+        super(TestTransportImplementation, self).tearDown()
+        self._server.tearDown()
+        
+    def check_transport_contents(self, content, transport, relpath):
+        """Check that transport.get(relpath).read() == content."""
+        self.assertEqualDiff(content, transport.get(relpath).read())
+
+    def get_transport(self):
+        """Return a connected transport to the local directory."""
+        base_url = self._server.get_url()
+        t = bzrlib.transport.get_transport(base_url)
+        if not isinstance(t, self.transport_class):
+            # we want to make sure to construct one particular class, even if
+            # there are several available implementations of this transport;
+            # therefore construct it by hand rather than through the regular
+            # get_transport method
+            t = self.transport_class(base_url)
+        return t
+
+    def assertListRaises(self, excClass, func, *args, **kwargs):
+        """Fail unless excClass is raised when the iterator from func is used.
+        
+        Many transport functions can return generators this makes sure
+        to wrap them in a list() call to make sure the whole generator
+        is run, and that the proper exception is raised.
+        """
+        try:
+            list(func(*args, **kwargs))
+        except excClass:
+            return
+        else:
+            if hasattr(excClass,'__name__'): excName = excClass.__name__
+            else: excName = str(excClass)
+            raise self.failureException, "%s not raised" % excName
+
+    def test_has(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g', '%']
+        self.build_tree(files, transport=t)
+        self.assertEqual(True, t.has('a'))
+        self.assertEqual(False, t.has('c'))
+        self.assertEqual(True, t.has(urlescape('%')))
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(True, t.has_any(['a', 'b', 'c']))
+        self.assertEqual(False, t.has_any(['c', 'd', 'f', urlescape('%%')]))
+        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(False, t.has_any(['c', 'c', 'c']))
+        self.assertEqual(True, t.has_any(['b', 'b', 'b']))
+
+    def test_get(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        contents = ['contents of a\n',
+                    'contents of b\n',
+                    'contents of e\n',
+                    'contents of g\n',
+                    ]
+        self.build_tree(files, transport=t, line_endings='binary')
+        self.check_transport_contents('contents of a\n', t, 'a')
+        content_f = t.get_multi(files)
+        for content, f in zip(contents, content_f):
+            self.assertEqual(content, f.read())
+
+        content_f = t.get_multi(iter(files))
+        for content, f in zip(contents, content_f):
+            self.assertEqual(content, f.read())
+
+        self.assertRaises(NoSuchFile, t.get, 'c')
+        self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
+        self.assertListRaises(NoSuchFile, t.get_multi, iter(['a', 'b', 'c']))
+
+    def test_put(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'a', 'some text for a\n')
+            return
+
+        t.put('a', StringIO('some text for a\n'))
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+        # Make sure 'has' is updated
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, False, False])
+        # Put also replaces contents
+        self.assertEqual(t.put_multi([('a', StringIO('new\ncontents for\na\n')),
+                                      ('d', StringIO('contents\nfor d\n'))]),
+                         2)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, True, False])
+        self.check_transport_contents('new\ncontents for\na\n', t, 'a')
+        self.check_transport_contents('contents\nfor d\n', t, 'd')
+
+        self.assertEqual(
+            t.put_multi(iter([('a', StringIO('diff\ncontents for\na\n')),
+                              ('d', StringIO('another contents\nfor d\n'))])),
+                        2)
+        self.check_transport_contents('diff\ncontents for\na\n', t, 'a')
+        self.check_transport_contents('another contents\nfor d\n', t, 'd')
+
+        self.assertRaises(NoSuchFile,
+                          t.put, 'path/doesnt/exist/c', 'contents')
+
+    def test_put_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        t.put('mode644', StringIO('test text\n'), mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put('mode666', StringIO('test text\n'), mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put('mode600', StringIO('test text\n'), mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put a file such that it becomes readonly
+        t.put('mode400', StringIO('test text\n'), mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+        t.put_multi([('mmode644', StringIO('text\n'))], mode=0644)
+        self.assertTransportMode(t, 'mmode644', 0644)
+        
+    def test_mkdir(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            # cannot mkdir on readonly transports. We're not testing for 
+            # cache coherency because cache behaviour is not currently
+            # defined for the transport interface.
+            self.assertRaises(TransportNotPossible, t.mkdir, '.')
+            self.assertRaises(TransportNotPossible, t.mkdir, 'new_dir')
+            self.assertRaises(TransportNotPossible, t.mkdir_multi, ['new_dir'])
+            self.assertRaises(TransportNotPossible, t.mkdir, 'path/doesnt/exist')
+            return
+        # Test mkdir
+        t.mkdir('dir_a')
+        self.assertEqual(t.has('dir_a'), True)
+        self.assertEqual(t.has('dir_b'), False)
+
+        t.mkdir('dir_b')
+        self.assertEqual(t.has('dir_b'), True)
+
+        t.mkdir_multi(['dir_c', 'dir_d'])
+
+        t.mkdir_multi(iter(['dir_e', 'dir_f']))
+        self.assertEqual(list(t.has_multi(
+            ['dir_a', 'dir_b', 'dir_c', 'dir_q',
+             'dir_d', 'dir_e', 'dir_f', 'dir_b'])),
+            [True, True, True, False,
+             True, True, True, True])
+
+        # we were testing that a local mkdir followed by a transport
+        # mkdir failed thusly, but given that we * in one process * do not
+        # concurrently fiddle with disk dirs and then use transport to do 
+        # things, the win here seems marginal compared to the constraint on
+        # the interface. RBC 20051227
+        t.mkdir('dir_g')
+        self.assertRaises(FileExists, t.mkdir, 'dir_g')
+
+        # Test get/put in sub-directories
+        self.assertEqual(2, 
+            t.put_multi([('dir_a/a', StringIO('contents of dir_a/a')),
+                         ('dir_b/b', StringIO('contents of dir_b/b'))]))
+        self.check_transport_contents('contents of dir_a/a', t, 'dir_a/a')
+        self.check_transport_contents('contents of dir_b/b', t, 'dir_b/b')
+
+        # mkdir of a dir with an absent parent
+        self.assertRaises(NoSuchFile, t.mkdir, 'missing/dir')
+
+    def test_mkdir_permissions(self):
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # no sense testing on this transport
+            return
+        # Test mkdir with a mode
+        t.mkdir('dmode755', mode=0755)
+        self.assertTransportMode(t, 'dmode755', 0755)
+        t.mkdir('dmode555', mode=0555)
+        self.assertTransportMode(t, 'dmode555', 0555)
+        t.mkdir('dmode777', mode=0777)
+        self.assertTransportMode(t, 'dmode777', 0777)
+        t.mkdir('dmode700', mode=0700)
+        self.assertTransportMode(t, 'dmode700', 0700)
+        # TODO: jam 20051215 test mkdir_multi with a mode
+        t.mkdir_multi(['mdmode755'], mode=0755)
+        self.assertTransportMode(t, 'mdmode755', 0755)
+
+    def test_copy_to(self):
+        # FIXME: test:   same server to same server (partly done)
+        # same protocol two servers
+        # and    different protocols (done for now except for MemoryTransport.
+        # - RBC 20060122
+        from bzrlib.transport.memory import MemoryTransport
+
+        def simple_copy_files(transport_from, transport_to):
+            files = ['a', 'b', 'c', 'd']
+            self.build_tree(files, transport=transport_from)
+            self.assertEqual(4, transport_from.copy_to(files, transport_to))
+            for f in files:
+                self.check_transport_contents(transport_to.get(f).read(),
+                                              transport_from, f)
+
+        t = self.get_transport()
+        temp_transport = MemoryTransport('memory:/')
+        simple_copy_files(t, temp_transport)
+        if not t.is_readonly():
+            t.mkdir('copy_to_simple')
+            t2 = t.clone('copy_to_simple')
+            simple_copy_files(t, t2)
+
+
+        # Test that copying into a missing directory raises
+        # NoSuchFile
+        if t.is_readonly():
+            self.build_tree(['e/', 'e/f'])
+        else:
+            t.mkdir('e')
+            t.put('e/f', StringIO('contents of e'))
+        self.assertRaises(NoSuchFile, t.copy_to, ['e/f'], temp_transport)
+        temp_transport.mkdir('e')
+        t.copy_to(['e/f'], temp_transport)
+
+        del temp_transport
+        temp_transport = MemoryTransport('memory:/')
+
+        files = ['a', 'b', 'c', 'd']
+        t.copy_to(iter(files), temp_transport)
+        for f in files:
+            self.check_transport_contents(temp_transport.get(f).read(),
+                                          t, f)
+        del temp_transport
+
+        for mode in (0666, 0644, 0600, 0400):
+            temp_transport = MemoryTransport("memory:/")
+            t.copy_to(files, temp_transport, mode=mode)
+            for f in files:
+                self.assertTransportMode(temp_transport, f, mode)
+
+    def test_append(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('b', 'wb').write('contents\nfor b\n')
+        else:
+            t.put_multi([
+                    ('a', StringIO('diff\ncontents for\na\n')),
+                    ('b', StringIO('contents\nfor b\n'))
+                    ])
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
+            _append('a', StringIO('add\nsome\nmore\ncontents\n'))
+        else:
+            self.assertEqual(20,
+                t.append('a', StringIO('add\nsome\nmore\ncontents\n')))
+
+        self.check_transport_contents(
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
+            t, 'a')
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append_multi,
+                        [('a', 'and\nthen\nsome\nmore\n'),
+                         ('b', 'some\nmore\nfor\nb\n')])
+            _append('a', StringIO('and\nthen\nsome\nmore\n'))
+            _append('b', StringIO('some\nmore\nfor\nb\n'))
+        else:
+            self.assertEqual((43, 15), 
+                t.append_multi([('a', StringIO('and\nthen\nsome\nmore\n')),
+                                ('b', StringIO('some\nmore\nfor\nb\n'))]))
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n',
+            t, 'a')
+        self.check_transport_contents(
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n',
+                t, 'b')
+
+        if t.is_readonly():
+            _append('a', StringIO('a little bit more\n'))
+            _append('b', StringIO('from an iterator\n'))
+        else:
+            self.assertEqual((62, 31),
+                t.append_multi(iter([('a', StringIO('a little bit more\n')),
+                                     ('b', StringIO('from an iterator\n'))])))
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n',
+            t, 'a')
+        self.check_transport_contents(
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n'
+                'from an iterator\n',
+                t, 'b')
+
+        if t.is_readonly():
+            _append('c', StringIO('some text\nfor a missing file\n'))
+            _append('a', StringIO('some text in a\n'))
+            _append('d', StringIO('missing file r\n'))
+        else:
+            self.assertEqual(0,
+                t.append('c', StringIO('some text\nfor a missing file\n')))
+            self.assertEqual((80, 0),
+                t.append_multi([('a', StringIO('some text in a\n')),
+                                ('d', StringIO('missing file r\n'))]))
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n'
+            'some text in a\n',
+            t, 'a')
+        self.check_transport_contents('some text\nfor a missing file\n',
+                                      t, 'c')
+        self.check_transport_contents('missing file r\n', t, 'd')
+        
+        # a file with no parent should fail..
+        if not t.is_readonly():
+            self.assertRaises(NoSuchFile,
+                              t.append, 'missing/path', 
+                              StringIO('content'))
+
+    def test_append_file(self):
+        t = self.get_transport()
+
+        contents = [
+            ('f1', StringIO('this is a string\nand some more stuff\n')),
+            ('f2', StringIO('here is some text\nand a bit more\n')),
+            ('f3', StringIO('some text for the\nthird file created\n')),
+            ('f4', StringIO('this is a string\nand some more stuff\n')),
+            ('f5', StringIO('here is some text\nand a bit more\n')),
+            ('f6', StringIO('some text for the\nthird file created\n'))
+        ]
+        
+        if t.is_readonly():
+            for f, val in contents:
+                open(f, 'wb').write(val.read())
+        else:
+            t.put_multi(contents)
+
+        a1 = StringIO('appending to\none\n')
+        if t.is_readonly():
+            _append('f1', a1)
+        else:
+            t.append('f1', a1)
+
+        del a1
+
+        self.check_transport_contents(
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n',
+                t, 'f1')
+
+        a2 = StringIO('adding more\ntext to two\n')
+        a3 = StringIO('some garbage\nto put in three\n')
+
+        if t.is_readonly():
+            _append('f2', a2)
+            _append('f3', a3)
+        else:
+            t.append_multi([('f2', a2), ('f3', a3)])
+
+        del a2, a3
+
+        self.check_transport_contents(
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n',
+                t, 'f2')
+        self.check_transport_contents( 
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n',
+                t, 'f3')
+
+        # Test that an actual file object can be used with put
+        a4 = t.get('f1')
+        if t.is_readonly():
+            _append('f4', a4)
+        else:
+            t.append('f4', a4)
+
+        del a4
+
+        self.check_transport_contents(
+                'this is a string\nand some more stuff\n'
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n',
+                t, 'f4')
+
+        a5 = t.get('f2')
+        a6 = t.get('f3')
+        if t.is_readonly():
+            _append('f5', a5)
+            _append('f6', a6)
+        else:
+            t.append_multi([('f5', a5), ('f6', a6)])
+
+        del a5, a6
+
+        self.check_transport_contents(
+                'here is some text\nand a bit more\n'
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n',
+                t, 'f5')
+        self.check_transport_contents(
+                'some text for the\nthird file created\n'
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n',
+                t, 'f6')
+
+        a5 = t.get('f2')
+        a6 = t.get('f2')
+        a7 = t.get('f3')
+        if t.is_readonly():
+            _append('c', a5)
+            _append('a', a6)
+            _append('d', a7)
+        else:
+            t.append('c', a5)
+            t.append_multi([('a', a6), ('d', a7)])
+        del a5, a6, a7
+        self.check_transport_contents(t.get('f2').read(), t, 'c')
+        self.check_transport_contents(t.get('f3').read(), t, 'd')
+
+    def test_delete(self):
+        # TODO: Test Transport.delete
+        t = self.get_transport()
+
+        # Not much to do with a readonly transport
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible, t.delete, 'missing')
+            return
+
+        t.put('a', StringIO('a little bit of text\n'))
+        self.failUnless(t.has('a'))
+        t.delete('a')
+        self.failIf(t.has('a'))
+
+        self.assertRaises(NoSuchFile, t.delete, 'a')
+
+        t.put('a', StringIO('a text\n'))
+        t.put('b', StringIO('b text\n'))
+        t.put('c', StringIO('c text\n'))
+        self.assertEqual([True, True, True],
+                list(t.has_multi(['a', 'b', 'c'])))
+        t.delete_multi(['a', 'c'])
+        self.assertEqual([False, True, False],
+                list(t.has_multi(['a', 'b', 'c'])))
+        self.failIf(t.has('a'))
+        self.failUnless(t.has('b'))
+        self.failIf(t.has('c'))
+
+        self.assertRaises(NoSuchFile,
+                t.delete_multi, ['a', 'b', 'c'])
+
+        self.assertRaises(NoSuchFile,
+                t.delete_multi, iter(['a', 'b', 'c']))
+
+        t.put('a', StringIO('another a text\n'))
+        t.put('c', StringIO('another c text\n'))
+        t.delete_multi(iter(['a', 'b', 'c']))
+
+        # We should have deleted everything
+        # SftpServer creates control files in the
+        # working directory, so we can just do a
+        # plain "listdir".
+        # self.assertEqual([], os.listdir('.'))
+
+    def test_rmdir(self):
+        t = self.get_transport()
+        # Not much to do with a readonly transport
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible, t.rmdir, 'missing')
+            return
+        t.mkdir('adir')
+        t.mkdir('adir/bdir')
+        t.rmdir('adir/bdir')
+        self.assertRaises(NoSuchFile, t.stat, 'adir/bdir')
+        t.rmdir('adir')
+        self.assertRaises(NoSuchFile, t.stat, 'adir')
+
+    def test_rmdir_not_empty(self):
+        """Deleting a non-empty directory raises an exception
+        
+        sftp (and possibly others) don't give us a specific "directory not
+        empty" exception -- we can just see that the operation failed.
+        """
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+        t.mkdir('adir')
+        t.mkdir('adir/bdir')
+        self.assertRaises(PathError, t.rmdir, 'adir')
+
+    def test_rename_dir_succeeds(self):
+        t = self.get_transport()
+        if t.is_readonly():
+            raise TestSkipped("transport is readonly")
+        t.mkdir('adir')
+        t.mkdir('adir/asubdir')
+        t.rename('adir', 'bdir')
+        self.assertTrue(t.has('bdir/asubdir'))
+        self.assertFalse(t.has('adir'))
+
+    def test_rename_dir_nonempty(self):
+        """Attempting to replace a nonemtpy directory should fail"""
+        t = self.get_transport()
+        if t.is_readonly():
+            raise TestSkipped("transport is readonly")
+        t.mkdir('adir')
+        t.mkdir('adir/asubdir')
+        t.mkdir('bdir')
+        t.mkdir('bdir/bsubdir')
+        self.assertRaises(PathError, t.rename, 'bdir', 'adir')
+        # nothing was changed so it should still be as before
+        self.assertTrue(t.has('bdir/bsubdir'))
+        self.assertFalse(t.has('adir/bdir'))
+        self.assertFalse(t.has('adir/bsubdir'))
+
+    def test_delete_tree(self):
+        t = self.get_transport()
+
+        # Not much to do with a readonly transport
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible, t.delete_tree, 'missing')
+            return
+
+        # and does it like listing ?
+        t.mkdir('adir')
+        try:
+            t.delete_tree('adir')
+        except TransportNotPossible:
+            # ok, this transport does not support delete_tree
+            return
+        
+        # did it delete that trivial case?
+        self.assertRaises(NoSuchFile, t.stat, 'adir')
+
+        self.build_tree(['adir/',
+                         'adir/file', 
+                         'adir/subdir/', 
+                         'adir/subdir/file', 
+                         'adir/subdir2/',
+                         'adir/subdir2/file',
+                         ], transport=t)
+
+        t.delete_tree('adir')
+        # adir should be gone now.
+        self.assertRaises(NoSuchFile, t.stat, 'adir')
+
+    def test_move(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+
+        # TODO: I would like to use os.listdir() to
+        # make sure there are no extra files, but SftpServer
+        # creates control files in the working directory
+        # perhaps all of this could be done in a subdirectory
+
+        t.put('a', StringIO('a first file\n'))
+        self.assertEquals([True, False], list(t.has_multi(['a', 'b'])))
+
+        t.move('a', 'b')
+        self.failUnless(t.has('b'))
+        self.failIf(t.has('a'))
+
+        self.check_transport_contents('a first file\n', t, 'b')
+        self.assertEquals([False, True], list(t.has_multi(['a', 'b'])))
+
+        # Overwrite a file
+        t.put('c', StringIO('c this file\n'))
+        t.move('c', 'b')
+        self.failIf(t.has('c'))
+        self.check_transport_contents('c this file\n', t, 'b')
+
+        # TODO: Try to write a test for atomicity
+        # TODO: Test moving into a non-existant subdirectory
+        # TODO: Test Transport.move_multi
+
+    def test_copy(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+
+        t.put('a', StringIO('a file\n'))
+        t.copy('a', 'b')
+        self.check_transport_contents('a file\n', t, 'b')
+
+        self.assertRaises(NoSuchFile, t.copy, 'c', 'd')
+        os.mkdir('c')
+        # What should the assert be if you try to copy a
+        # file over a directory?
+        #self.assertRaises(Something, t.copy, 'a', 'c')
+        t.put('d', StringIO('text in d\n'))
+        t.copy('d', 'b')
+        self.check_transport_contents('text in d\n', t, 'b')
+
+        # TODO: test copy_multi
+
+    def test_connection_error(self):
+        """ConnectionError is raised when connection is impossible"""
+        try:
+            url = self._server.get_bogus_url()
+        except NotImplementedError:
+            raise TestSkipped("Transport %s has no bogus URL support." %
+                              self._server.__class__)
+        t = bzrlib.transport.get_transport(url)
+        try:
+            t.get('.bzr/branch')
+        except (ConnectionError, NoSuchFile), e:
+            pass
+        except (Exception), e:
+            self.failIf(True, 'Wrong exception thrown: %s' % e)
+        else:
+            self.failIf(True, 'Did not get the expected exception.')
+
+    def test_stat(self):
+        # TODO: Test stat, just try once, and if it throws, stop testing
+        from stat import S_ISDIR, S_ISREG
+
+        t = self.get_transport()
+
+        try:
+            st = t.stat('.')
+        except TransportNotPossible, e:
+            # This transport cannot stat
+            return
+
+        paths = ['a', 'b/', 'b/c', 'b/d/', 'b/d/e']
+        sizes = [14, 0, 16, 0, 18] 
+        self.build_tree(paths, transport=t, line_endings='binary')
+
+        for path, size in zip(paths, sizes):
+            st = t.stat(path)
+            if path.endswith('/'):
+                self.failUnless(S_ISDIR(st.st_mode))
+                # directory sizes are meaningless
+            else:
+                self.failUnless(S_ISREG(st.st_mode))
+                self.assertEqual(size, st.st_size)
+
+        remote_stats = list(t.stat_multi(paths))
+        remote_iter_stats = list(t.stat_multi(iter(paths)))
+
+        self.assertRaises(NoSuchFile, t.stat, 'q')
+        self.assertRaises(NoSuchFile, t.stat, 'b/a')
+
+        self.assertListRaises(NoSuchFile, t.stat_multi, ['a', 'c', 'd'])
+        self.assertListRaises(NoSuchFile, t.stat_multi, iter(['a', 'c', 'd']))
+        self.build_tree(['subdir/', 'subdir/file'], transport=t)
+        subdir = t.clone('subdir')
+        subdir.stat('./file')
+        subdir.stat('.')
+
+    def test_list_dir(self):
+        # TODO: Test list_dir, just try once, and if it throws, stop testing
+        t = self.get_transport()
+        
+        if not t.listable():
+            self.assertRaises(TransportNotPossible, t.list_dir, '.')
+            return
+
+        def sorted_list(d):
+            l = list(t.list_dir(d))
+            l.sort()
+            return l
+
+        # SftpServer creates control files in the working directory
+        # so lets move down a directory to avoid those.
+        if not t.is_readonly():
+            t.mkdir('wd')
+        else:
+            os.mkdir('wd')
+        t = t.clone('wd')
+
+        self.assertEqual([], sorted_list(u'.'))
+        # c2 is precisely one letter longer than c here to test that
+        # suffixing is not confused.
+        if not t.is_readonly():
+            self.build_tree(['a', 'b', 'c/', 'c/d', 'c/e', 'c2/'], transport=t)
+        else:
+            self.build_tree(['wd/a', 'wd/b', 'wd/c/', 'wd/c/d', 'wd/c/e', 'wd/c2/'])
+
+        self.assertEqual([u'a', u'b', u'c', u'c2'], sorted_list(u'.'))
+        self.assertEqual([u'd', u'e'], sorted_list(u'c'))
+
+        if not t.is_readonly():
+            t.delete('c/d')
+            t.delete('b')
+        else:
+            os.unlink('wd/c/d')
+            os.unlink('wd/b')
+            
+        self.assertEqual([u'a', u'c', u'c2'], sorted_list('.'))
+        self.assertEqual([u'e'], sorted_list(u'c'))
+
+        self.assertListRaises(NoSuchFile, t.list_dir, 'q')
+        self.assertListRaises(NoSuchFile, t.list_dir, 'c/f')
+        self.assertListRaises(NoSuchFile, t.list_dir, 'a')
+
+    def test_clone(self):
+        # TODO: Test that clone moves up and down the filesystem
+        t1 = self.get_transport()
+
+        self.build_tree(['a', 'b/', 'b/c'], transport=t1)
+
+        self.failUnless(t1.has('a'))
+        self.failUnless(t1.has('b/c'))
+        self.failIf(t1.has('c'))
+
+        t2 = t1.clone('b')
+        self.assertEqual(t1.base + 'b/', t2.base)
+
+        self.failUnless(t2.has('c'))
+        self.failIf(t2.has('a'))
+
+        t3 = t2.clone('..')
+        self.failUnless(t3.has('a'))
+        self.failIf(t3.has('c'))
+
+        self.failIf(t1.has('b/d'))
+        self.failIf(t2.has('d'))
+        self.failIf(t3.has('b/d'))
+
+        if t1.is_readonly():
+            open('b/d', 'wb').write('newfile\n')
+        else:
+            t2.put('d', StringIO('newfile\n'))
+
+        self.failUnless(t1.has('b/d'))
+        self.failUnless(t2.has('d'))
+        self.failUnless(t3.has('b/d'))
+
+    def test_relpath(self):
+        t = self.get_transport()
+        self.assertEqual('', t.relpath(t.base))
+        # base ends with /
+        self.assertEqual('', t.relpath(t.base[:-1]))
+        # subdirs which dont exist should still give relpaths.
+        self.assertEqual('foo', t.relpath(t.base + 'foo'))
+        # trailing slash should be the same.
+        self.assertEqual('foo', t.relpath(t.base + 'foo/'))
+
+    def test_relpath_at_root(self):
+        t = self.get_transport()
+        # clone all the way to the top
+        new_transport = t.clone('..')
+        while new_transport.base != t.base:
+            t = new_transport
+            new_transport = t.clone('..')
+        # we must be able to get a relpath below the root
+        self.assertEqual('', t.relpath(t.base))
+        # and a deeper one should work too
+        self.assertEqual('foo/bar', t.relpath(t.base + 'foo/bar'))
+
+    def test_abspath(self):
+        # smoke test for abspath. Corner cases for backends like unix fs's
+        # that have aliasing problems like symlinks should go in backend
+        # specific test cases.
+        transport = self.get_transport()
+        
+        # disabled because some transports might normalize urls in generating
+        # the abspath - eg http+pycurl-> just http -- mbp 20060308 
+        self.assertEqual(transport.base + 'relpath',
+                         transport.abspath('relpath'))
+
+    def test_abspath_at_root(self):
+        t = self.get_transport()
+        # clone all the way to the top
+        new_transport = t.clone('..')
+        while new_transport.base != t.base:
+            t = new_transport
+            new_transport = t.clone('..')
+        # we must be able to get a abspath of the root when we ask for
+        # t.abspath('..') - this due to our choice that clone('..')
+        # should return the root from the root, combined with the desire that
+        # the url from clone('..') and from abspath('..') should be the same.
+        self.assertEqual(t.base, t.abspath('..'))
+        # '' should give us the root
+        self.assertEqual(t.base, t.abspath(''))
+        # and a path should append to the url
+        self.assertEqual(t.base + 'foo', t.abspath('foo'))
+
+    def test_iter_files_recursive(self):
+        transport = self.get_transport()
+        if not transport.listable():
+            self.assertRaises(TransportNotPossible,
+                              transport.iter_files_recursive)
+            return
+        self.build_tree(['isolated/',
+                         'isolated/dir/',
+                         'isolated/dir/foo',
+                         'isolated/dir/bar',
+                         'isolated/bar'],
+                        transport=transport)
+        paths = set(transport.iter_files_recursive())
+        # nb the directories are not converted
+        self.assertEqual(paths,
+                    set(['isolated/dir/foo',
+                         'isolated/dir/bar',
+                         'isolated/bar']))
+        sub_transport = transport.clone('isolated')
+        paths = set(sub_transport.iter_files_recursive())
+        self.assertEqual(set(['dir/foo', 'dir/bar', 'bar']), paths)
+
+    def test_connect_twice_is_same_content(self):
+        # check that our server (whatever it is) is accessable reliably
+        # via get_transport and multiple connections share content.
+        transport = self.get_transport()
+        if transport.is_readonly():
+            return
+        transport.put('foo', StringIO('bar'))
+        transport2 = self.get_transport()
+        self.check_transport_contents('bar', transport2, 'foo')
+        # its base should be usable.
+        transport2 = bzrlib.transport.get_transport(transport.base)
+        self.check_transport_contents('bar', transport2, 'foo')
+
+        # now opening at a relative url should give use a sane result:
+        transport.mkdir('newdir')
+        transport2 = bzrlib.transport.get_transport(transport.base + "newdir")
+        transport2 = transport2.clone('..')
+        self.check_transport_contents('bar', transport2, 'foo')
+
+    def test_lock_write(self):
+        transport = self.get_transport()
+        if transport.is_readonly():
+            self.assertRaises(TransportNotPossible, transport.lock_write, 'foo')
+            return
+        transport.put('lock', StringIO())
+        lock = transport.lock_write('lock')
+        # TODO make this consistent on all platforms:
+        # self.assertRaises(LockError, transport.lock_write, 'lock')
+        lock.unlock()
+
+    def test_lock_read(self):
+        transport = self.get_transport()
+        if transport.is_readonly():
+            file('lock', 'w').close()
+        else:
+            transport.put('lock', StringIO())
+        lock = transport.lock_read('lock')
+        # TODO make this consistent on all platforms:
+        # self.assertRaises(LockError, transport.lock_read, 'lock')
+        lock.unlock()
+
+    def test_readv(self):
+        transport = self.get_transport()
+        if transport.is_readonly():
+            file('a', 'w').write('0123456789')
+        else:
+            transport.put('a', StringIO('01234567890'))
+
+        d = list(transport.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
+        self.assertEqual(d[0], (0, '0'))
+        self.assertEqual(d[1], (1, '1'))
+        self.assertEqual(d[2], (3, '34'))
+        self.assertEqual(d[3], (9, '9'))

=== added file 'bzrlib/tests/test_tsort.py'
--- bzrlib/tests/test_tsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_tsort.py	2006-03-28 14:29:13 +0000
@@ -0,0 +1,281 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for topological sort."""
+
+
+from bzrlib.tests import TestCase
+from bzrlib.tsort import topo_sort, TopoSorter, MergeSorter, merge_sort
+from bzrlib.errors import GraphCycleError
+
+
+class TopoSortTests(TestCase):
+
+    def assertSortAndIterate(self, graph, result_list):
+        """Check that sorting and iter_topo_order on graph works."""
+        self.assertEquals(result_list, topo_sort(graph))
+        self.assertEqual(result_list,
+                         list(TopoSorter(graph).iter_topo_order()))
+
+    def assertSortAndIterateRaise(self, exception_type, graph):
+        """Try both iterating and topo_sorting graph and expect an exception."""
+        self.assertRaises(exception_type, topo_sort, graph)
+        self.assertRaises(exception_type,
+                          list,
+                          TopoSorter(graph).iter_topo_order())
+
+    def test_tsort_empty(self):
+        """TopoSort empty list"""
+        self.assertSortAndIterate([], [])
+
+    def test_tsort_easy(self):
+        """TopoSort list with one node"""
+        self.assertSortAndIterate({0: []}.items(), [0])
+
+    def test_tsort_cycle(self):
+        """TopoSort traps graph with cycles"""
+        self.assertSortAndIterateRaise(GraphCycleError,
+                                       {0: [1], 
+                                        1: [0]}.items())
+
+    def test_tsort_cycle_2(self):
+        """TopoSort traps graph with longer cycle"""
+        self.assertSortAndIterateRaise(GraphCycleError,
+                                       {0: [1], 
+                                        1: [2], 
+                                        2: [0]}.items())
+                 
+    def test_tsort_1(self):
+        """TopoSort simple nontrivial graph"""
+        self.assertSortAndIterate({0: [3], 
+                                   1: [4],
+                                   2: [1, 4],
+                                   3: [], 
+                                   4: [0, 3]}.items(),
+                                  [3, 0, 4, 1, 2])
+
+    def test_tsort_partial(self):
+        """Topological sort with partial ordering.
+
+        If the graph does not give an order between two nodes, they are 
+        returned in lexicographical order.
+        """
+        self.assertSortAndIterate(([(0, []),
+                                   (1, [0]),
+                                   (2, [0]),
+                                   (3, [0]),
+                                   (4, [1, 2, 3]),
+                                   (5, [1, 2]),
+                                   (6, [1, 2]),
+                                   (7, [2, 3]),
+                                   (8, [0, 1, 4, 5, 6])]),
+                                  [0, 1, 2, 3, 4, 5, 6, 7, 8])
+
+
+class MergeSortTests(TestCase):
+
+    def assertSortAndIterate(self, graph, branch_tip, result_list,
+            mainline_revisions=None):
+        """Check that merge based sorting and iter_topo_order on graph works."""
+        self.assertEquals(result_list,
+            merge_sort(graph, branch_tip, mainline_revisions=mainline_revisions))
+        self.assertEqual(result_list,
+            list(MergeSorter(
+                graph,
+                branch_tip,
+                mainline_revisions=mainline_revisions).iter_topo_order()))
+
+    def test_merge_sort_empty(self):
+        # sorting of an emptygraph does not error
+        self.assertSortAndIterate({}, None, [])
+
+    def test_merge_sort_not_empty_no_tip(self):
+        # merge sorting of a branch starting with None should result
+        # in an empty list: no revisions are dragged in.
+        self.assertSortAndIterate({0: []}.items(), None, [])
+
+    def test_merge_sort_one_revision(self):
+        # sorting with one revision as the tip returns the correct fields:
+        # sequence - 0, revision id, merge depth - 0, end_of_merge
+        self.assertSortAndIterate({'id': []}.items(),
+                                  'id',
+                                  [(0, 'id', 0, True)])
+    
+    def test_sequence_numbers_increase_no_merges(self):
+        # emit a few revisions with no merges to check the sequence
+        # numbering works in trivial cases
+        self.assertSortAndIterate(
+            {'A': [],
+             'B': ['A'],
+             'C': ['B']}.items(),
+            'C',
+            [(0, 'C', 0, False),
+             (1, 'B', 0, False),
+             (2, 'A', 0, True),
+             ]
+            )
+
+    def test_sequence_numbers_increase_with_merges(self):
+        # test that sequence numbers increase across merges
+        self.assertSortAndIterate(
+            {'A': [],
+             'B': ['A'],
+             'C': ['A', 'B']}.items(),
+            'C',
+            [(0, 'C', 0, False),
+             (1, 'B', 1, True),
+             (2, 'A', 0, True),
+             ]
+            )
+        
+    def test_merge_depth_with_nested_merges(self):
+        # the merge depth marker should reflect the depth of the revision
+        # in terms of merges out from the mainline
+        # revid, depth, parents:
+        #  A 0   [D, B]   
+        #  B  1  [C, F]   
+        #  C  1  [H] 
+        #  D 0   [H, E]
+        #  E  1  [G, F]
+        #  F   2 [G]
+        #  G  1  [H]
+        #  H 0
+        self.assertSortAndIterate(
+            {'A': ['D', 'B'],
+             'B': ['C', 'F'],
+             'C': ['H'],
+             'D': ['H', 'E'],
+             'E': ['G', 'F'],
+             'F': ['G'],
+             'G': ['H'],
+             'H': []
+             }.items(),
+            'A',
+            [(0, 'A', 0, False),
+             (1, 'B', 1, False),
+             (2, 'C', 1, True),
+             (3, 'D', 0, False),
+             (4, 'E', 1, False),
+             (5, 'F', 2, True),
+             (6, 'G', 1, True),
+             (7, 'H', 0, True),
+             ]
+            )
+ 
+    def test_end_of_merge_not_last_revision_in_branch(self):
+        # within a branch only the last revision gets an
+        # end of merge marker.
+        self.assertSortAndIterate(
+            {'A': ['B'],
+             'B': [],
+             },
+            'A',
+            [(0, 'A', 0, False),
+             (1, 'B', 0, True)
+             ]
+            )
+
+    def test_end_of_merge_multiple_revisions_merged_at_once(self):
+        # when multiple branches are merged at once, both of their
+        # branch-endpoints should be listed as end-of-merge.
+        # Also, the order of the multiple merges should be 
+        # left-right shown top to bottom.
+        # * means end of merge
+        # A 0    [H, B, E] 
+        # B  1   [D, C]
+        # C   2  [D]       *
+        # D  1   [H]       *
+        # E  1   [G, F]
+        # F   2  [G]       *
+        # G  1   [H]       *
+        # H 0    []        *
+        self.assertSortAndIterate(
+            {'A': ['H', 'B', 'E'],
+             'B': ['D', 'C'],
+             'C': ['D'],
+             'D': ['H'],
+             'E': ['G', 'F'],
+             'F': ['G'],
+             'G': ['H'],
+             'H': [],
+             },
+            'A',
+            [(0, 'A', 0, False),
+             (1, 'B', 1, False),
+             (2, 'C', 2, True),
+             (3, 'D', 1, True),
+             (4, 'E', 1, False),
+             (5, 'F', 2, True),
+             (6, 'G', 1, True),
+             (7, 'H', 0, True),
+             ]
+            )
+
+    def test_mainline_revs_partial(self):
+        # when a mainline_revisions list is passed this must
+        # override the graphs idea of mainline, and must also
+        # truncate the output to the specified range, if needed.
+        # so we test both at once: a mainline_revisions list that
+        # disagrees with the graph about which revs are 'mainline'
+        # and also truncates the output.
+        # graph:
+        # A 0 [E, B]
+        # B 1 [D, C]
+        # C 2 [D]
+        # D 1 [E]
+        # E 0
+        # with a mainline of NONE,E,A (the inferred one) this will show the merge
+        # depths above.
+        # with a overriden mainline of NONE,E,D,B,A it should show:
+        # A 0
+        # B 0
+        # C 1
+        # D 0
+        # E 0
+        # and thus when truncated to D,B,A it should show
+        # A 0
+        # B 0
+        # C 1 
+        # because C is brought in by B in this view and D
+        # is the terminating revision id
+        self.assertSortAndIterate(
+            {'A': ['E', 'B'],
+             'B': ['D', 'C'],
+             'C': ['D'],
+             'D': ['E'],
+             'E': []
+             },
+            'A',
+            [(0, 'A', 0, False),
+             (1, 'B', 0, False),
+             (2, 'C', 1, True),
+             ],
+            mainline_revisions=['D', 'B', 'A']
+            )
+
+    def test_mainline_revs_with_none(self):
+        # a simple test to ensure that a mainline_revs
+        # list which goes all the way to None works
+        self.assertSortAndIterate(
+            {'A': [],
+             },
+            'A',
+            [(0, 'A', 0, True),
+             ],
+            mainline_revisions=[None, 'A']
+            )
+

=== added file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2006-04-18 04:22:52 +0000
@@ -0,0 +1,84 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for tuned_gzip."""
+
+
+# do not use bzrlib test cases here - this should be suitable for sending
+# upstream.
+from cStringIO import StringIO
+from unittest import TestCase
+import zlib
+
+
+from bzrlib.tuned_gzip import *
+
+
+class FakeDecompress(object):
+    """A fake decompressor for testing GzipFile."""
+
+    def __init__(self):
+        self.unused_data=''
+
+    def decompress(self, buf):
+        """Return an empty string as though we are at eof."""
+        self.unused_data += buf
+        return ''
+
+
+class TestFakeDecompress(TestCase):
+    """We use a fake decompressor to test GzipFile.
+
+    This class tests the behaviours we want from it.
+    """
+
+    def test_decompress(self):
+        # decompressing returns no data.
+        decompress = FakeDecompress()
+        self.assertEqual('', decompress.decompress('0'))
+
+    def test_unused_data(self):
+        # after decompressing, we have 1 unused byte.
+        # this is normally set by decompressors when they
+        # detect the end of a compressed stream.
+        decompress = FakeDecompress()
+        decompress.decompress('0')
+        self.assertEqual('0', decompress.unused_data)
+        # decompressing again (when the short read is read)
+        # will give us the full value in the unused_data
+        decompress.decompress('1234567')
+        self.assertEqual('01234567', decompress.unused_data)
+
+
+class TestGzip(TestCase):
+
+    def test__read_short_remainder(self):
+        # a _read call at the end of a compressed hunk should
+        # read more bytes if there is less than 8 bytes (the 
+        # gzip trailer) unread.
+        stream = StringIO('\0\0\0\0\0\0\0\0')
+        myfile = GzipFile(fileobj=stream)
+        # disable the _new_member check, we are microtesting.
+        myfile._new_member = False
+        myfile.crc = zlib.crc32('')
+        myfile.decompress = FakeDecompress()
+        myfile.size = 0
+        myfile._read(1)
+        # all the data should have been read now
+        self.assertEqual('', stream.read())
+        # and it should be new member time in the stream.
+        self.failUnless(myfile._new_member)

=== added file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_ui.py	2006-04-03 15:05:16 +0000
@@ -0,0 +1,109 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the bzrlib ui
+"""
+
+import os
+from StringIO import StringIO
+import sys
+
+
+import bzrlib.errors as errors
+from bzrlib.progress import TTYProgressBar, ProgressBarStack
+from bzrlib.tests import TestCase
+from bzrlib.ui import SilentUIFactory
+from bzrlib.ui.text import TextUIFactory
+
+class UITests(TestCase):
+
+    def test_silent_factory(self):
+
+        ui = SilentUIFactory()
+        pb = ui.nested_progress_bar()
+        try:
+            # TODO: Test that there is no output from SilentUIFactory
+    
+            self.assertEquals(ui.get_password(), None)
+            self.assertEquals(ui.get_password(u'Hello There \u1234 %(user)s',
+                                              user=u'some\u1234')
+                             , None)
+        finally:
+            pb.finished()
+
+    def test_text_factory(self):
+        ui = TextUIFactory()
+        pb = ui.nested_progress_bar()
+        pb.finished()
+        # TODO: Test the output from TextUIFactory, perhaps by overriding sys.stdout
+
+        # Unfortunately we can't actually test the ui.get_password() because 
+        # that would actually prompt the user for a password during the test suite
+        # This has been tested manually with both LANG=en_US.utf-8 and LANG=C
+        # print
+        # self.assertEquals(ui.get_password(u"%(user)s please type 'bogus'",
+        #                                   user=u'some\u1234')
+        #                  , 'bogus')
+
+
+    def test_progress_note(self):
+        stderr = StringIO()
+        stdout = StringIO()
+        ui_factory = TextUIFactory()
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            result = pb.note('t')
+            self.assertEqual(None, result)
+            self.assertEqual("t\n", stdout.getvalue())
+            # the exact contents will depend on the terminal width and we don't
+            # care about that right now - but you're probably running it on at
+            # least a 10-character wide terminal :)
+            self.assertContainsRe(stderr.getvalue(), r'^\r {10,}\r$')
+        finally:
+            pb.finished()
+
+    def test_progress_nested(self):
+        # test factory based nested and popping.
+        ui = TextUIFactory()
+        pb1 = ui.nested_progress_bar()
+        pb2 = ui.nested_progress_bar()
+        self.assertRaises(errors.MissingProgressBarFinish, pb1.finished)
+        pb2.finished()
+        pb1.finished()
+
+    def test_progress_stack(self):
+        # test the progress bar stack which the default text factory 
+        # uses.
+        stderr = StringIO()
+        stdout = StringIO()
+        # make a stack, which accepts parameters like a pb.
+        stack = ProgressBarStack(to_file=stderr, to_messages_file=stdout)
+        # but is not one
+        self.assertFalse(getattr(stack, 'note', False))
+        pb1 = stack.get_nested()
+        pb2 = stack.get_nested()
+        self.assertRaises(errors.MissingProgressBarFinish, pb1.finished)
+        pb2.finished()
+        pb1.finished()
+        # the text ui factory never actually removes the stack once its setup.
+        # we need to be able to nest again correctly from here.
+        pb1 = stack.get_nested()
+        pb2 = stack.get_nested()
+        self.assertRaises(errors.MissingProgressBarFinish, pb1.finished)
+        pb2.finished()
+        pb1.finished()

=== added file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_upgrade.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,294 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for upgrade of old trees.
+
+This file contains canned versions of some old trees, which are instantiated 
+and then upgraded to the new format."""
+
+# TODO queue for upgrade:
+# test the error message when upgrading an unknown BzrDir format.
+
+import base64
+import os
+import sys
+
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+import bzrlib.repository as repository
+from bzrlib.revision import is_ancestor
+from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.transport import get_transport
+from bzrlib.upgrade import upgrade
+import bzrlib.workingtree as workingtree
+
+
+class TestUpgrade(TestCaseInTempDir):
+    
+    def test_build_tree(self):
+        """Test tree-building test helper"""
+        self.build_tree_contents(_upgrade1_template)
+        self.failUnlessExists('foo')
+        self.failUnlessExists('.bzr/README')
+
+    def test_upgrade_simple(self):
+        """Upgrade simple v0.0.4 format to v6"""
+        eq = self.assertEquals
+        self.build_tree_contents(_upgrade1_template)
+        upgrade(u'.')
+        control = bzrdir.BzrDir.open('.')
+        b = control.open_branch()
+        # tsk, peeking under the covers.
+        self.failUnless(
+            isinstance(
+                control._format,
+                bzrdir.BzrDirFormat.get_default_format().__class__))
+        rh = b.revision_history()
+        eq(rh,
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+        rt = b.repository.revision_tree(rh[0])
+        foo_id = 'foo-20051004035605-91e788d1875603ae'
+        eq(rt.get_file_text(foo_id), 'initial contents\n')
+        rt = b.repository.revision_tree(rh[1])
+        eq(rt.get_file_text(foo_id), 'new contents\n')
+        # check a backup was made:
+        transport = get_transport(b.base)
+        transport.stat('.bzr.backup')
+        transport.stat('.bzr.backup/README')
+        transport.stat('.bzr.backup/branch-format')
+        transport.stat('.bzr.backup/revision-history')
+        transport.stat('.bzr.backup/merged-patches')
+        transport.stat('.bzr.backup/pending-merged-patches')
+        transport.stat('.bzr.backup/pending-merges')
+        transport.stat('.bzr.backup/branch-name')
+        transport.stat('.bzr.backup/branch-lock')
+        transport.stat('.bzr.backup/inventory')
+        transport.stat('.bzr.backup/stat-cache')
+        transport.stat('.bzr.backup/text-store')
+        transport.stat('.bzr.backup/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
+        transport.stat('.bzr.backup/text-store/foo-20051004035756-4081373d897c3453.gz')
+        transport.stat('.bzr.backup/inventory-store/')
+        transport.stat('.bzr.backup/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        transport.stat('.bzr.backup/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+        transport.stat('.bzr.backup/revision-store/')
+        transport.stat('.bzr.backup/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        transport.stat('.bzr.backup/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+
+    def test_upgrade_with_ghosts(self):
+        """Upgrade v0.0.4 tree containing ghost references.
+
+        That is, some of the parents of revisions mentioned in the branch
+        aren't present in the branch's storage. 
+
+        This shouldn't normally happen in branches created entirely in 
+        bzr, but can happen in branches imported from baz and arch, or from
+        other systems, where the importer knows about a revision but not 
+        its contents."""
+        eq = self.assertEquals
+        self.build_tree_contents(_ghost_template)
+        upgrade(u'.')
+        b = Branch.open(u'.')
+        revision_id = b.revision_history()[1]
+        rev = b.repository.get_revision(revision_id)
+        eq(len(rev.parent_ids), 2)
+        eq(rev.parent_ids[1], 'wibble@wobble-2')
+
+    def test_upgrade_makes_dir_weaves(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        upgrade('.')
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        repo = bzrlib.repository.Repository.open('.')
+        # it should have changed the format
+        self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
+        # and we should be able to read the names for the file id 
+        # 'dir-20051005095101-da1441ea3fa6917a'
+        self.assertNotEqual(
+            [],
+            repo.text_store.get_weave(
+                'dir-20051005095101-da1441ea3fa6917a',
+                repo.get_transaction()))
+
+    def test_upgrade_to_meta_sets_workingtree_last_revision(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        upgrade('.', bzrdir.BzrDirMetaFormat1())
+        tree = workingtree.WorkingTree.open('.')
+        self.assertEqual(tree.last_revision(),
+                         tree.branch.revision_history()[-1])
+
+
+_upgrade1_template = \
+     [
+     ('foo', 'new contents\n'),
+     ('.bzr/',),
+     ('.bzr/README',
+      'This is a Bazaar-NG control directory.\nDo not change any files in this directory.\n'),
+     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
+     ('.bzr/revision-history',
+      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
+      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
+     ('.bzr/merged-patches', ''),
+     ('.bzr/pending-merged-patches', ''),
+     ('.bzr/branch-name', ''),
+     ('.bzr/branch-lock', ''),
+     ('.bzr/pending-merges', ''),
+     ('.bzr/inventory',
+      '<inventory>\n'
+      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
+      '</inventory>\n'),
+     ('.bzr/stat-cache',
+      '### bzr hashcache v5\n'
+      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
+     ('.bzr/text-store/',),
+     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
+     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
+     ('.bzr/inventory-store/',),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
+     ('.bzr/revision-store/',),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
+
+
+_ghost_template = [
+    ( './foo',
+        'hello\n'
+    ),
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
+        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
+        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
+    ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
+        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
+        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
+        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
+    ),
+]
+
+_upgrade_dir_template = [
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
+        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
+        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
+        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
+        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
+    ),
+    ( './dir/', ),
+]

=== added file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_versionedfile.py	2006-04-18 23:42:34 +0000
@@ -0,0 +1,1081 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# Authors:
+#   Johan Rydberg <jrydberg@gnu.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from StringIO import StringIO
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import (
+                           RevisionNotPresent, 
+                           RevisionAlreadyPresent,
+                           WeaveParentMismatch
+                           )
+from bzrlib.knit import KnitVersionedFile, \
+     KnitAnnotateFactory
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+from bzrlib.transport.memory import MemoryTransport
+import bzrlib.versionedfile as versionedfile
+from bzrlib.weave import WeaveFile
+from bzrlib.weavefile import read_weave, write_weave
+
+
+class VersionedFileTestMixIn(object):
+    """A mixin test class for testing VersionedFiles.
+
+    This is not an adaptor-style test at this point because
+    theres no dynamic substitution of versioned file implementations,
+    they are strictly controlled by their owning repositories.
+    """
+
+    def test_add(self):
+        f = self.get_file()
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.add_lines('r1', ['r0'], ['b\n', 'c\n'])
+        def verify_file(f):
+            versions = f.versions()
+            self.assertTrue('r0' in versions)
+            self.assertTrue('r1' in versions)
+            self.assertEquals(f.get_lines('r0'), ['a\n', 'b\n'])
+            self.assertEquals(f.get_text('r0'), 'a\nb\n')
+            self.assertEquals(f.get_lines('r1'), ['b\n', 'c\n'])
+            self.assertEqual(2, len(f))
+            self.assertEqual(2, f.num_versions())
+    
+            self.assertRaises(RevisionNotPresent,
+                f.add_lines, 'r2', ['foo'], [])
+            self.assertRaises(RevisionAlreadyPresent,
+                f.add_lines, 'r1', [], [])
+        verify_file(f)
+        f = self.reopen_file()
+        verify_file(f)
+
+    def test_adds_with_parent_texts(self):
+        f = self.get_file()
+        parent_texts = {}
+        parent_texts['r0'] = f.add_lines('r0', [], ['a\n', 'b\n'])
+        try:
+            parent_texts['r1'] = f.add_lines_with_ghosts('r1',
+                                                         ['r0', 'ghost'], 
+                                                         ['b\n', 'c\n'],
+                                                         parent_texts=parent_texts)
+        except NotImplementedError:
+            # if the format doesn't support ghosts, just add normally.
+            parent_texts['r1'] = f.add_lines('r1',
+                                             ['r0'], 
+                                             ['b\n', 'c\n'],
+                                             parent_texts=parent_texts)
+        f.add_lines('r2', ['r1'], ['c\n', 'd\n'], parent_texts=parent_texts)
+        self.assertNotEqual(None, parent_texts['r0'])
+        self.assertNotEqual(None, parent_texts['r1'])
+        def verify_file(f):
+            versions = f.versions()
+            self.assertTrue('r0' in versions)
+            self.assertTrue('r1' in versions)
+            self.assertTrue('r2' in versions)
+            self.assertEquals(f.get_lines('r0'), ['a\n', 'b\n'])
+            self.assertEquals(f.get_lines('r1'), ['b\n', 'c\n'])
+            self.assertEquals(f.get_lines('r2'), ['c\n', 'd\n'])
+            self.assertEqual(3, f.num_versions())
+            origins = f.annotate('r1')
+            self.assertEquals(origins[0][0], 'r0')
+            self.assertEquals(origins[1][0], 'r1')
+            origins = f.annotate('r2')
+            self.assertEquals(origins[0][0], 'r1')
+            self.assertEquals(origins[1][0], 'r2')
+
+        verify_file(f)
+        f = self.reopen_file()
+        verify_file(f)
+
+    def test_get_delta(self):
+        f = self.get_file()
+        sha1s = self._setup_for_deltas(f)
+        expected_delta = (None, '6bfa09d82ce3e898ad4641ae13dd4fdb9cf0d76b', False, 
+                          [(0, 0, 1, [('base', 'line\n')])])
+        self.assertEqual(expected_delta, f.get_delta('base'))
+        next_parent = 'base'
+        text_name = 'chain1-'
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            expected_delta = (next_parent, sha1s[depth], 
+                              False,
+                              [(depth + 1, depth + 1, 1, [(new_version, 'line\n')])])
+            self.assertEqual(expected_delta, f.get_delta(new_version))
+            next_parent = new_version
+        next_parent = 'base'
+        text_name = 'chain2-'
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            expected_delta = (next_parent, sha1s[depth], False,
+                              [(depth + 1, depth + 1, 1, [(new_version, 'line\n')])])
+            self.assertEqual(expected_delta, f.get_delta(new_version))
+            next_parent = new_version
+        # smoke test for eol support
+        expected_delta = ('base', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True, [])
+        self.assertEqual(['line'], f.get_lines('noeol'))
+        self.assertEqual(expected_delta, f.get_delta('noeol'))
+
+    def test_get_deltas(self):
+        f = self.get_file()
+        sha1s = self._setup_for_deltas(f)
+        deltas = f.get_deltas(f.versions())
+        expected_delta = (None, '6bfa09d82ce3e898ad4641ae13dd4fdb9cf0d76b', False, 
+                          [(0, 0, 1, [('base', 'line\n')])])
+        self.assertEqual(expected_delta, deltas['base'])
+        next_parent = 'base'
+        text_name = 'chain1-'
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            expected_delta = (next_parent, sha1s[depth], 
+                              False,
+                              [(depth + 1, depth + 1, 1, [(new_version, 'line\n')])])
+            self.assertEqual(expected_delta, deltas[new_version])
+            next_parent = new_version
+        next_parent = 'base'
+        text_name = 'chain2-'
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            expected_delta = (next_parent, sha1s[depth], False,
+                              [(depth + 1, depth + 1, 1, [(new_version, 'line\n')])])
+            self.assertEqual(expected_delta, deltas[new_version])
+            next_parent = new_version
+        # smoke tests for eol support
+        expected_delta = ('base', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True, [])
+        self.assertEqual(['line'], f.get_lines('noeol'))
+        self.assertEqual(expected_delta, deltas['noeol'])
+        # smoke tests for eol support - two noeol in a row same content
+        expected_deltas = (('noeol', '3ad7ee82dbd8f29ecba073f96e43e414b3f70a4d', True, 
+                          [(0, 1, 2, [(u'noeolsecond', 'line\n'), (u'noeolsecond', 'line\n')])]),
+                          ('noeol', '3ad7ee82dbd8f29ecba073f96e43e414b3f70a4d', True, 
+                           [(0, 0, 1, [('noeolsecond', 'line\n')]), (1, 1, 0, [])]))
+        self.assertEqual(['line\n', 'line'], f.get_lines('noeolsecond'))
+        self.assertTrue(deltas['noeolsecond'] in expected_deltas)
+        # two no-eol in a row, different content
+        expected_delta = ('noeolsecond', '8bb553a84e019ef1149db082d65f3133b195223b', True, 
+                          [(1, 2, 1, [(u'noeolnotshared', 'phone\n')])])
+        self.assertEqual(['line\n', 'phone'], f.get_lines('noeolnotshared'))
+        self.assertEqual(expected_delta, deltas['noeolnotshared'])
+        # eol folling a no-eol with content change
+        expected_delta = ('noeol', 'a61f6fb6cfc4596e8d88c34a308d1e724caf8977', False, 
+                          [(0, 1, 1, [(u'eol', 'phone\n')])])
+        self.assertEqual(['phone\n'], f.get_lines('eol'))
+        self.assertEqual(expected_delta, deltas['eol'])
+        # eol folling a no-eol with content change
+        expected_delta = ('noeol', '6bfa09d82ce3e898ad4641ae13dd4fdb9cf0d76b', False, 
+                          [(0, 1, 1, [(u'eolline', 'line\n')])])
+        self.assertEqual(['line\n'], f.get_lines('eolline'))
+        self.assertEqual(expected_delta, deltas['eolline'])
+        # eol with no parents
+        expected_delta = (None, '264f39cab871e4cfd65b3a002f7255888bb5ed97', True, 
+                          [(0, 0, 1, [(u'noeolbase', 'line\n')])])
+        self.assertEqual(['line'], f.get_lines('noeolbase'))
+        self.assertEqual(expected_delta, deltas['noeolbase'])
+        # eol with two parents, in inverse insertion order
+        expected_deltas = (('noeolbase', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True,
+                            [(0, 1, 1, [(u'eolbeforefirstparent', 'line\n')])]),
+                           ('noeolbase', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True,
+                            [(0, 1, 1, [(u'eolbeforefirstparent', 'line\n')])]))
+        self.assertEqual(['line'], f.get_lines('eolbeforefirstparent'))
+        #self.assertTrue(deltas['eolbeforefirstparent'] in expected_deltas)
+
+    def _setup_for_deltas(self, f):
+        self.assertRaises(errors.RevisionNotPresent, f.get_delta, 'base')
+        # add texts that should trip the knit maximum delta chain threshold
+        # as well as doing parallel chains of data in knits.
+        # this is done by two chains of 25 insertions
+        f.add_lines('base', [], ['line\n'])
+        f.add_lines('noeol', ['base'], ['line'])
+        # detailed eol tests:
+        # shared last line with parent no-eol
+        f.add_lines('noeolsecond', ['noeol'], ['line\n', 'line'])
+        # differing last line with parent, both no-eol
+        f.add_lines('noeolnotshared', ['noeolsecond'], ['line\n', 'phone'])
+        # add eol following a noneol parent, change content
+        f.add_lines('eol', ['noeol'], ['phone\n'])
+        # add eol following a noneol parent, no change content
+        f.add_lines('eolline', ['noeol'], ['line\n'])
+        # noeol with no parents:
+        f.add_lines('noeolbase', [], ['line'])
+        # noeol preceeding its leftmost parent in the output:
+        # this is done by making it a merge of two parents with no common
+        # anestry: noeolbase and noeol with the 
+        # later-inserted parent the leftmost.
+        f.add_lines('eolbeforefirstparent', ['noeolbase', 'noeol'], ['line'])
+        # two identical eol texts
+        f.add_lines('noeoldup', ['noeol'], ['line'])
+        next_parent = 'base'
+        text_name = 'chain1-'
+        text = ['line\n']
+        sha1s = {0 :'da6d3141cb4a5e6f464bf6e0518042ddc7bfd079',
+                 1 :'45e21ea146a81ea44a821737acdb4f9791c8abe7',
+                 2 :'e1f11570edf3e2a070052366c582837a4fe4e9fa',
+                 3 :'26b4b8626da827088c514b8f9bbe4ebf181edda1',
+                 4 :'e28a5510be25ba84d31121cff00956f9970ae6f6',
+                 5 :'d63ec0ce22e11dcf65a931b69255d3ac747a318d',
+                 6 :'2c2888d288cb5e1d98009d822fedfe6019c6a4ea',
+                 7 :'95c14da9cafbf828e3e74a6f016d87926ba234ab',
+                 8 :'779e9a0b28f9f832528d4b21e17e168c67697272',
+                 9 :'1f8ff4e5c6ff78ac106fcfe6b1e8cb8740ff9a8f',
+                 10:'131a2ae712cf51ed62f143e3fbac3d4206c25a05',
+                 11:'c5a9d6f520d2515e1ec401a8f8a67e6c3c89f199',
+                 12:'31a2286267f24d8bedaa43355f8ad7129509ea85',
+                 13:'dc2a7fe80e8ec5cae920973973a8ee28b2da5e0a',
+                 14:'2c4b1736566b8ca6051e668de68650686a3922f2',
+                 15:'5912e4ecd9b0c07be4d013e7e2bdcf9323276cde',
+                 16:'b0d2e18d3559a00580f6b49804c23fea500feab3',
+                 17:'8e1d43ad72f7562d7cb8f57ee584e20eb1a69fc7',
+                 18:'5cf64a3459ae28efa60239e44b20312d25b253f3',
+                 19:'1ebed371807ba5935958ad0884595126e8c4e823',
+                 20:'2aa62a8b06fb3b3b892a3292a068ade69d5ee0d3',
+                 21:'01edc447978004f6e4e962b417a4ae1955b6fe5d',
+                 22:'d8d8dc49c4bf0bab401e0298bb5ad827768618bb',
+                 23:'c21f62b1c482862983a8ffb2b0c64b3451876e3f',
+                 24:'c0593fe795e00dff6b3c0fe857a074364d5f04fc',
+                 25:'dd1a1cf2ba9cc225c3aff729953e6364bf1d1855',
+                 }
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            text = text + ['line\n']
+            f.add_lines(new_version, [next_parent], text)
+            next_parent = new_version
+        next_parent = 'base'
+        text_name = 'chain2-'
+        text = ['line\n']
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            text = text + ['line\n']
+            f.add_lines(new_version, [next_parent], text)
+            next_parent = new_version
+        return sha1s
+
+    def test_add_delta(self):
+        # tests for the add-delta facility.
+        # at this point, optimising for speed, we assume no checks when deltas are inserted.
+        # this may need to be revisited.
+        source = self.get_file('source')
+        source.add_lines('base', [], ['line\n'])
+        next_parent = 'base'
+        text_name = 'chain1-'
+        text = ['line\n']
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            text = text + ['line\n']
+            source.add_lines(new_version, [next_parent], text)
+            next_parent = new_version
+        next_parent = 'base'
+        text_name = 'chain2-'
+        text = ['line\n']
+        for depth in range(26):
+            new_version = text_name + '%s' % depth
+            text = text + ['line\n']
+            source.add_lines(new_version, [next_parent], text)
+            next_parent = new_version
+        source.add_lines('noeol', ['base'], ['line'])
+        
+        target = self.get_file('target')
+        for version in source.versions():
+            parent, sha1, noeol, delta = source.get_delta(version)
+            target.add_delta(version,
+                             source.get_parents(version),
+                             parent,
+                             sha1,
+                             noeol,
+                             delta)
+        self.assertRaises(RevisionAlreadyPresent,
+                          target.add_delta, 'base', [], None, '', False, [])
+        for version in source.versions():
+            self.assertEqual(source.get_lines(version),
+                             target.get_lines(version))
+
+    def test_ancestry(self):
+        f = self.get_file()
+        self.assertEqual([], f.get_ancestry([]))
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.add_lines('r1', ['r0'], ['b\n', 'c\n'])
+        f.add_lines('r2', ['r0'], ['b\n', 'c\n'])
+        f.add_lines('r3', ['r2'], ['b\n', 'c\n'])
+        f.add_lines('rM', ['r1', 'r2'], ['b\n', 'c\n'])
+        self.assertEqual([], f.get_ancestry([]))
+        versions = f.get_ancestry(['rM'])
+        # there are some possibilities:
+        # r0 r1 r2 rM r3
+        # r0 r1 r2 r3 rM
+        # etc
+        # so we check indexes
+        r0 = versions.index('r0')
+        r1 = versions.index('r1')
+        r2 = versions.index('r2')
+        self.assertFalse('r3' in versions)
+        rM = versions.index('rM')
+        self.assertTrue(r0 < r1)
+        self.assertTrue(r0 < r2)
+        self.assertTrue(r1 < rM)
+        self.assertTrue(r2 < rM)
+
+        self.assertRaises(RevisionNotPresent,
+            f.get_ancestry, ['rM', 'rX'])
+
+    def test_mutate_after_finish(self):
+        f = self.get_file()
+        f.transaction_finished()
+        self.assertRaises(errors.OutSideTransaction, f.add_delta, '', [], '', '', False, [])
+        self.assertRaises(errors.OutSideTransaction, f.add_lines, '', [], [])
+        self.assertRaises(errors.OutSideTransaction, f.add_lines_with_ghosts, '', [], [])
+        self.assertRaises(errors.OutSideTransaction, f.fix_parents, '', [])
+        self.assertRaises(errors.OutSideTransaction, f.join, '')
+        self.assertRaises(errors.OutSideTransaction, f.clone_text, 'base', 'bar', ['foo'])
+        
+    def test_clear_cache(self):
+        f = self.get_file()
+        # on a new file it should not error
+        f.clear_cache()
+        # and after adding content, doing a clear_cache and a get should work.
+        f.add_lines('0', [], ['a'])
+        f.clear_cache()
+        self.assertEqual(['a'], f.get_lines('0'))
+
+    def test_clone_text(self):
+        f = self.get_file()
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.clone_text('r1', 'r0', ['r0'])
+        def verify_file(f):
+            self.assertEquals(f.get_lines('r1'), f.get_lines('r0'))
+            self.assertEquals(f.get_lines('r1'), ['a\n', 'b\n'])
+            self.assertEquals(f.get_parents('r1'), ['r0'])
+    
+            self.assertRaises(RevisionNotPresent,
+                f.clone_text, 'r2', 'rX', [])
+            self.assertRaises(RevisionAlreadyPresent,
+                f.clone_text, 'r1', 'r0', [])
+        verify_file(f)
+        verify_file(self.reopen_file())
+
+    def test_create_empty(self):
+        f = self.get_file()
+        f.add_lines('0', [], ['a\n'])
+        new_f = f.create_empty('t', MemoryTransport())
+        # smoke test, specific types should check it is honoured correctly for
+        # non type attributes
+        self.assertEqual([], new_f.versions())
+        self.assertTrue(isinstance(new_f, f.__class__))
+
+    def test_copy_to(self):
+        f = self.get_file()
+        f.add_lines('0', [], ['a\n'])
+        t = MemoryTransport()
+        f.copy_to('foo', t)
+        for suffix in f.__class__.get_suffixes():
+            self.assertTrue(t.has('foo' + suffix))
+
+    def test_get_suffixes(self):
+        f = self.get_file()
+        # should be the same
+        self.assertEqual(f.__class__.get_suffixes(), f.__class__.get_suffixes())
+        # and should be a list
+        self.assertTrue(isinstance(f.__class__.get_suffixes(), list))
+
+    def test_get_graph(self):
+        f = self.get_file()
+        f.add_lines('v1', [], ['hello\n'])
+        f.add_lines('v2', ['v1'], ['hello\n', 'world\n'])
+        f.add_lines('v3', ['v2'], ['hello\n', 'cruel\n', 'world\n'])
+        self.assertEqual({'v1': [],
+                          'v2': ['v1'],
+                          'v3': ['v2']},
+                         f.get_graph())
+
+    def test_get_parents(self):
+        f = self.get_file()
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.add_lines('r1', [], ['a\n', 'b\n'])
+        f.add_lines('r2', [], ['a\n', 'b\n'])
+        f.add_lines('r3', [], ['a\n', 'b\n'])
+        f.add_lines('m', ['r0', 'r1', 'r2', 'r3'], ['a\n', 'b\n'])
+        self.assertEquals(f.get_parents('m'), ['r0', 'r1', 'r2', 'r3'])
+
+        self.assertRaises(RevisionNotPresent,
+            f.get_parents, 'y')
+
+    def test_annotate(self):
+        f = self.get_file()
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.add_lines('r1', ['r0'], ['c\n', 'b\n'])
+        origins = f.annotate('r1')
+        self.assertEquals(origins[0][0], 'r1')
+        self.assertEquals(origins[1][0], 'r0')
+
+        self.assertRaises(RevisionNotPresent,
+            f.annotate, 'foo')
+
+    def test_walk(self):
+        # tests that walk returns all the inclusions for the requested
+        # revisions as well as the revisions changes themselves.
+        f = self.get_file('1')
+        f.add_lines('r0', [], ['a\n', 'b\n'])
+        f.add_lines('r1', ['r0'], ['c\n', 'b\n'])
+        f.add_lines('rX', ['r1'], ['d\n', 'b\n'])
+        f.add_lines('rY', ['r1'], ['c\n', 'e\n'])
+
+        lines = {}
+        for lineno, insert, dset, text in f.walk(['rX', 'rY']):
+            lines[text] = (insert, dset)
+
+        self.assertTrue(lines['a\n'], ('r0', set(['r1'])))
+        self.assertTrue(lines['b\n'], ('r0', set(['rY'])))
+        self.assertTrue(lines['c\n'], ('r1', set(['rX'])))
+        self.assertTrue(lines['d\n'], ('rX', set([])))
+        self.assertTrue(lines['e\n'], ('rY', set([])))
+
+    def test_detection(self):
+        # Test weaves detect corruption.
+        #
+        # Weaves contain a checksum of their texts.
+        # When a text is extracted, this checksum should be
+        # verified.
+
+        w = self.get_file_corrupted_text()
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        w = self.get_file_corrupted_checksum()
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+    def get_file_corrupted_text(self):
+        """Return a versioned file with corrupt text but valid metadata."""
+        raise NotImplementedError(self.get_file_corrupted_text)
+
+    def reopen_file(self, name='foo'):
+        """Open the versioned file from disk again."""
+        raise NotImplementedError(self.reopen_file)
+
+    def test_iter_lines_added_or_present_in_versions(self):
+        # test that we get at least an equalset of the lines added by
+        # versions in the weave 
+        # the ordering here is to make a tree so that dumb searches have
+        # more changes to muck up.
+        vf = self.get_file()
+        # add a base to get included
+        vf.add_lines('base', [], ['base\n'])
+        # add a ancestor to be included on one side
+        vf.add_lines('lancestor', [], ['lancestor\n'])
+        # add a ancestor to be included on the other side
+        vf.add_lines('rancestor', ['base'], ['rancestor\n'])
+        # add a child of rancestor with no eofile-nl
+        vf.add_lines('child', ['rancestor'], ['base\n', 'child\n'])
+        # add a child of lancestor and base to join the two roots
+        vf.add_lines('otherchild',
+                     ['lancestor', 'base'],
+                     ['base\n', 'lancestor\n', 'otherchild\n'])
+        def iter_with_versions(versions):
+            # now we need to see what lines are returned, and how often.
+            lines = {'base\n':0,
+                     'lancestor\n':0,
+                     'rancestor\n':0,
+                     'child\n':0,
+                     'otherchild\n':0,
+                     }
+            # iterate over the lines
+            for line in vf.iter_lines_added_or_present_in_versions(versions):
+                lines[line] += 1
+            return lines
+        lines = iter_with_versions(['child', 'otherchild'])
+        # we must see child and otherchild
+        self.assertTrue(lines['child\n'] > 0)
+        self.assertTrue(lines['otherchild\n'] > 0)
+        # we dont care if we got more than that.
+        
+        # test all lines
+        lines = iter_with_versions(None)
+        # all lines must be seen at least once
+        self.assertTrue(lines['base\n'] > 0)
+        self.assertTrue(lines['lancestor\n'] > 0)
+        self.assertTrue(lines['rancestor\n'] > 0)
+        self.assertTrue(lines['child\n'] > 0)
+        self.assertTrue(lines['otherchild\n'] > 0)
+
+    def test_fix_parents(self):
+        # some versioned files allow incorrect parents to be corrected after
+        # insertion - this may not fix ancestry..
+        # if they do not supported, they just do not implement it.
+        # we test this as an interface test to ensure that those that *do*
+        # implementent it get it right.
+        vf = self.get_file()
+        vf.add_lines('notbase', [], [])
+        vf.add_lines('base', [], [])
+        try:
+            vf.fix_parents('notbase', ['base'])
+        except NotImplementedError:
+            return
+        self.assertEqual(['base'], vf.get_parents('notbase'))
+        # open again, check it stuck.
+        vf = self.get_file()
+        self.assertEqual(['base'], vf.get_parents('notbase'))
+
+    def test_fix_parents_with_ghosts(self):
+        # when fixing parents, ghosts that are listed should not be ghosts
+        # anymore.
+        vf = self.get_file()
+
+        try:
+            vf.add_lines_with_ghosts('notbase', ['base', 'stillghost'], [])
+        except NotImplementedError:
+            return
+        vf.add_lines('base', [], [])
+        vf.fix_parents('notbase', ['base', 'stillghost'])
+        self.assertEqual(['base'], vf.get_parents('notbase'))
+        # open again, check it stuck.
+        vf = self.get_file()
+        self.assertEqual(['base'], vf.get_parents('notbase'))
+        # and check the ghosts
+        self.assertEqual(['base', 'stillghost'],
+                         vf.get_parents_with_ghosts('notbase'))
+
+    def test_add_lines_with_ghosts(self):
+        # some versioned file formats allow lines to be added with parent
+        # information that is > than that in the format. Formats that do
+        # not support this need to raise NotImplementedError on the
+        # add_lines_with_ghosts api.
+        vf = self.get_file()
+        # add a revision with ghost parents
+        try:
+            vf.add_lines_with_ghosts(u'notbxbfse', [u'b\xbfse'], [])
+        except NotImplementedError:
+            # check the other ghost apis are also not implemented
+            self.assertRaises(NotImplementedError, vf.has_ghost, 'foo')
+            self.assertRaises(NotImplementedError, vf.get_ancestry_with_ghosts, ['foo'])
+            self.assertRaises(NotImplementedError, vf.get_parents_with_ghosts, 'foo')
+            self.assertRaises(NotImplementedError, vf.get_graph_with_ghosts)
+            return
+        # test key graph related apis: getncestry, _graph, get_parents
+        # has_version
+        # - these are ghost unaware and must not be reflect ghosts
+        self.assertEqual([u'notbxbfse'], vf.get_ancestry(u'notbxbfse'))
+        self.assertEqual([], vf.get_parents(u'notbxbfse'))
+        self.assertEqual({u'notbxbfse':[]}, vf.get_graph())
+        self.assertFalse(vf.has_version(u'b\xbfse'))
+        # we have _with_ghost apis to give us ghost information.
+        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry_with_ghosts([u'notbxbfse']))
+        self.assertEqual([u'b\xbfse'], vf.get_parents_with_ghosts(u'notbxbfse'))
+        self.assertEqual({u'notbxbfse':[u'b\xbfse']}, vf.get_graph_with_ghosts())
+        self.assertTrue(vf.has_ghost(u'b\xbfse'))
+        # if we add something that is a ghost of another, it should correct the
+        # results of the prior apis
+        vf.add_lines(u'b\xbfse', [], [])
+        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry([u'notbxbfse']))
+        self.assertEqual([u'b\xbfse'], vf.get_parents(u'notbxbfse'))
+        self.assertEqual({u'b\xbfse':[],
+                          u'notbxbfse':[u'b\xbfse'],
+                          },
+                         vf.get_graph())
+        self.assertTrue(vf.has_version(u'b\xbfse'))
+        # we have _with_ghost apis to give us ghost information.
+        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry_with_ghosts([u'notbxbfse']))
+        self.assertEqual([u'b\xbfse'], vf.get_parents_with_ghosts(u'notbxbfse'))
+        self.assertEqual({u'b\xbfse':[],
+                          u'notbxbfse':[u'b\xbfse'],
+                          },
+                         vf.get_graph_with_ghosts())
+        self.assertFalse(vf.has_ghost(u'b\xbfse'))
+
+    def test_add_lines_with_ghosts_after_normal_revs(self):
+        # some versioned file formats allow lines to be added with parent
+        # information that is > than that in the format. Formats that do
+        # not support this need to raise NotImplementedError on the
+        # add_lines_with_ghosts api.
+        vf = self.get_file()
+        # probe for ghost support
+        try:
+            vf.has_ghost('hoo')
+        except NotImplementedError:
+            return
+        vf.add_lines_with_ghosts('base', [], ['line\n', 'line_b\n'])
+        vf.add_lines_with_ghosts('references_ghost',
+                                 ['base', 'a_ghost'],
+                                 ['line\n', 'line_b\n', 'line_c\n'])
+        origins = vf.annotate('references_ghost')
+        self.assertEquals(('base', 'line\n'), origins[0])
+        self.assertEquals(('base', 'line_b\n'), origins[1])
+        self.assertEquals(('references_ghost', 'line_c\n'), origins[2])
+
+    def test_readonly_mode(self):
+        transport = get_transport(self.get_url('.'))
+        factory = self.get_factory()
+        vf = factory('id', transport, 0777, create=True, access_mode='w')
+        vf = factory('id', transport, access_mode='r')
+        self.assertRaises(errors.ReadOnlyError, vf.add_delta, '', [], '', '', False, [])
+        self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'base', [], [])
+        self.assertRaises(errors.ReadOnlyError,
+                          vf.add_lines_with_ghosts,
+                          'base',
+                          [],
+                          [])
+        self.assertRaises(errors.ReadOnlyError, vf.fix_parents, 'base', [])
+        self.assertRaises(errors.ReadOnlyError, vf.join, 'base')
+        self.assertRaises(errors.ReadOnlyError, vf.clone_text, 'base', 'bar', ['foo'])
+        
+
+class TestWeave(TestCaseWithTransport, VersionedFileTestMixIn):
+
+    def get_file(self, name='foo'):
+        return WeaveFile(name, get_transport(self.get_url('.')), create=True)
+
+    def get_file_corrupted_text(self):
+        w = WeaveFile('foo', get_transport(self.get_url('.')), create=True)
+        w.add_lines('v1', [], ['hello\n'])
+        w.add_lines('v2', ['v1'], ['hello\n', 'there\n'])
+        
+        # We are going to invasively corrupt the text
+        # Make sure the internals of weave are the same
+        self.assertEqual([('{', 0)
+                        , 'hello\n'
+                        , ('}', None)
+                        , ('{', 1)
+                        , 'there\n'
+                        , ('}', None)
+                        ], w._weave)
+        
+        self.assertEqual(['f572d396fae9206628714fb2ce00f72e94f2258f'
+                        , '90f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+                        ], w._sha1s)
+        w.check()
+        
+        # Corrupted
+        w._weave[4] = 'There\n'
+        return w
+
+    def get_file_corrupted_checksum(self):
+        w = self.get_file_corrupted_text()
+        # Corrected
+        w._weave[4] = 'there\n'
+        self.assertEqual('hello\nthere\n', w.get_text('v2'))
+        
+        #Invalid checksum, first digit changed
+        w._sha1s[1] =  'f0f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+        return w
+
+    def reopen_file(self, name='foo'):
+        return WeaveFile(name, get_transport(self.get_url('.')))
+
+    def test_no_implicit_create(self):
+        self.assertRaises(errors.NoSuchFile,
+                          WeaveFile,
+                          'foo',
+                          get_transport(self.get_url('.')))
+
+    def get_factory(self):
+        return WeaveFile
+
+
+class TestKnit(TestCaseWithTransport, VersionedFileTestMixIn):
+
+    def get_file(self, name='foo'):
+        return KnitVersionedFile(name, get_transport(self.get_url('.')),
+                                 delta=True, create=True)
+
+    def get_factory(self):
+        return KnitVersionedFile
+
+    def get_file_corrupted_text(self):
+        knit = self.get_file()
+        knit.add_lines('v1', [], ['hello\n'])
+        knit.add_lines('v2', ['v1'], ['hello\n', 'there\n'])
+        return knit
+
+    def reopen_file(self, name='foo'):
+        return KnitVersionedFile(name, get_transport(self.get_url('.')), delta=True)
+
+    def test_detection(self):
+        print "TODO for merging: create a corrupted knit."
+        knit = self.get_file()
+        knit.check()
+
+    def test_no_implicit_create(self):
+        self.assertRaises(errors.NoSuchFile,
+                          KnitVersionedFile,
+                          'foo',
+                          get_transport(self.get_url('.')))
+
+
+class InterString(versionedfile.InterVersionedFile):
+    """An inter-versionedfile optimised code path for strings.
+
+    This is for use during testing where we use strings as versionedfiles
+    so that none of the default regsitered interversionedfile classes will
+    match - which lets us test the match logic.
+    """
+
+    @staticmethod
+    def is_compatible(source, target):
+        """InterString is compatible with strings-as-versionedfiles."""
+        return isinstance(source, str) and isinstance(target, str)
+
+
+# TODO this and the InterRepository core logic should be consolidatable
+# if we make the registry a separate class though we still need to 
+# test the behaviour in the active registry to catch failure-to-handle-
+# stange-objects
+class TestInterVersionedFile(TestCaseWithTransport):
+
+    def test_get_default_inter_versionedfile(self):
+        # test that the InterVersionedFile.get(a, b) probes
+        # for a class where is_compatible(a, b) returns
+        # true and returns a default interversionedfile otherwise.
+        # This also tests that the default registered optimised interversionedfile
+        # classes do not barf inappropriately when a surprising versionedfile type
+        # is handed to them.
+        dummy_a = "VersionedFile 1."
+        dummy_b = "VersionedFile 2."
+        self.assertGetsDefaultInterVersionedFile(dummy_a, dummy_b)
+
+    def assertGetsDefaultInterVersionedFile(self, a, b):
+        """Asserts that InterVersionedFile.get(a, b) -> the default."""
+        inter = versionedfile.InterVersionedFile.get(a, b)
+        self.assertEqual(versionedfile.InterVersionedFile,
+                         inter.__class__)
+        self.assertEqual(a, inter.source)
+        self.assertEqual(b, inter.target)
+
+    def test_register_inter_versionedfile_class(self):
+        # test that a optimised code path provider - a
+        # InterVersionedFile subclass can be registered and unregistered
+        # and that it is correctly selected when given a versionedfile
+        # pair that it returns true on for the is_compatible static method
+        # check
+        dummy_a = "VersionedFile 1."
+        dummy_b = "VersionedFile 2."
+        versionedfile.InterVersionedFile.register_optimiser(InterString)
+        try:
+            # we should get the default for something InterString returns False
+            # to
+            self.assertFalse(InterString.is_compatible(dummy_a, None))
+            self.assertGetsDefaultInterVersionedFile(dummy_a, None)
+            # and we should get an InterString for a pair it 'likes'
+            self.assertTrue(InterString.is_compatible(dummy_a, dummy_b))
+            inter = versionedfile.InterVersionedFile.get(dummy_a, dummy_b)
+            self.assertEqual(InterString, inter.__class__)
+            self.assertEqual(dummy_a, inter.source)
+            self.assertEqual(dummy_b, inter.target)
+        finally:
+            versionedfile.InterVersionedFile.unregister_optimiser(InterString)
+        # now we should get the default InterVersionedFile object again.
+        self.assertGetsDefaultInterVersionedFile(dummy_a, dummy_b)
+
+
+class TestReadonlyHttpMixin(object):
+
+    def test_readonly_http_works(self):
+        # we should be able to read from http with a versioned file.
+        vf = self.get_file()
+        vf.add_lines('1', [], ['a\n'])
+        vf.add_lines('2', ['1'], ['b\n', 'a\n'])
+        readonly_vf = self.get_factory()('foo', get_transport(self.get_readonly_url('.')))
+        for version in readonly_vf.versions():
+            readonly_vf.get_lines(version)
+
+
+class TestWeaveHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
+
+    def get_file(self):
+        return WeaveFile('foo', get_transport(self.get_url('.')), create=True)
+
+    def get_factory(self):
+        return WeaveFile
+
+
+class TestKnitHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
+
+    def get_file(self):
+        return KnitVersionedFile('foo', get_transport(self.get_url('.')),
+                                 delta=True, create=True)
+
+    def get_factory(self):
+        return KnitVersionedFile
+
+
+class MergeCasesMixin(object):
+
+    def doMerge(self, base, a, b, mp):
+        from cStringIO import StringIO
+        from textwrap import dedent
+
+        def addcrlf(x):
+            return x + '\n'
+        
+        w = self.get_file()
+        w.add_lines('text0', [], map(addcrlf, base))
+        w.add_lines('text1', ['text0'], map(addcrlf, a))
+        w.add_lines('text2', ['text0'], map(addcrlf, b))
+
+        self.log_contents(w)
+
+        self.log('merge plan:')
+        p = list(w.plan_merge('text1', 'text2'))
+        for state, line in p:
+            if line:
+                self.log('%12s | %s' % (state, line[:-1]))
+
+        self.log('merge:')
+        mt = StringIO()
+        mt.writelines(w.weave_merge(p))
+        mt.seek(0)
+        self.log(mt.getvalue())
+
+        mp = map(addcrlf, mp)
+        self.assertEqual(mt.readlines(), mp)
+        
+        
+    def testOneInsert(self):
+        self.doMerge([],
+                     ['aa'],
+                     [],
+                     ['aa'])
+
+    def testSeparateInserts(self):
+        self.doMerge(['aaa', 'bbb', 'ccc'],
+                     ['aaa', 'xxx', 'bbb', 'ccc'],
+                     ['aaa', 'bbb', 'yyy', 'ccc'],
+                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'])
+
+    def testSameInsert(self):
+        self.doMerge(['aaa', 'bbb', 'ccc'],
+                     ['aaa', 'xxx', 'bbb', 'ccc'],
+                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'],
+                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'])
+    overlappedInsertExpected = ['aaa', 'xxx', 'yyy', 'bbb']
+    def testOverlappedInsert(self):
+        self.doMerge(['aaa', 'bbb'],
+                     ['aaa', 'xxx', 'yyy', 'bbb'],
+                     ['aaa', 'xxx', 'bbb'], self.overlappedInsertExpected)
+
+        # really it ought to reduce this to 
+        # ['aaa', 'xxx', 'yyy', 'bbb']
+
+
+    def testClashReplace(self):
+        self.doMerge(['aaa'],
+                     ['xxx'],
+                     ['yyy', 'zzz'],
+                     ['<<<<<<< ', 'xxx', '=======', 'yyy', 'zzz', 
+                      '>>>>>>> '])
+
+    def testNonClashInsert1(self):
+        self.doMerge(['aaa'],
+                     ['xxx', 'aaa'],
+                     ['yyy', 'zzz'],
+                     ['<<<<<<< ', 'xxx', 'aaa', '=======', 'yyy', 'zzz', 
+                      '>>>>>>> '])
+
+    def testNonClashInsert2(self):
+        self.doMerge(['aaa'],
+                     ['aaa'],
+                     ['yyy', 'zzz'],
+                     ['yyy', 'zzz'])
+
+
+    def testDeleteAndModify(self):
+        """Clashing delete and modification.
+
+        If one side modifies a region and the other deletes it then
+        there should be a conflict with one side blank.
+        """
+
+        #######################################
+        # skippd, not working yet
+        return
+        
+        self.doMerge(['aaa', 'bbb', 'ccc'],
+                     ['aaa', 'ddd', 'ccc'],
+                     ['aaa', 'ccc'],
+                     ['<<<<<<<< ', 'aaa', '=======', '>>>>>>> ', 'ccc'])
+
+    def _test_merge_from_strings(self, base, a, b, expected):
+        w = self.get_file()
+        w.add_lines('text0', [], base.splitlines(True))
+        w.add_lines('text1', ['text0'], a.splitlines(True))
+        w.add_lines('text2', ['text0'], b.splitlines(True))
+        self.log('merge plan:')
+        p = list(w.plan_merge('text1', 'text2'))
+        for state, line in p:
+            if line:
+                self.log('%12s | %s' % (state, line[:-1]))
+        self.log('merge result:')
+        result_text = ''.join(w.weave_merge(p))
+        self.log(result_text)
+        self.assertEqualDiff(result_text, expected)
+
+    def test_weave_merge_conflicts(self):
+        # does weave merge properly handle plans that end with unchanged?
+        result = ''.join(self.get_file().weave_merge([('new-a', 'hello\n')]))
+        self.assertEqual(result, 'hello\n')
+
+    def test_deletion_extended(self):
+        """One side deletes, the other deletes more.
+        """
+        base = """\
+            line 1
+            line 2
+            line 3
+            """
+        a = """\
+            line 1
+            line 2
+            """
+        b = """\
+            line 1
+            """
+        result = """\
+            line 1
+            """
+        self._test_merge_from_strings(base, a, b, result)
+
+    def test_deletion_overlap(self):
+        """Delete overlapping regions with no other conflict.
+
+        Arguably it'd be better to treat these as agreement, rather than 
+        conflict, but for now conflict is safer.
+        """
+        base = """\
+            start context
+            int a() {}
+            int b() {}
+            int c() {}
+            end context
+            """
+        a = """\
+            start context
+            int a() {}
+            end context
+            """
+        b = """\
+            start context
+            int c() {}
+            end context
+            """
+        result = """\
+            start context
+<<<<<<< 
+            int a() {}
+=======
+            int c() {}
+>>>>>>> 
+            end context
+            """
+        self._test_merge_from_strings(base, a, b, result)
+
+    def test_agreement_deletion(self):
+        """Agree to delete some lines, without conflicts."""
+        base = """\
+            start context
+            base line 1
+            base line 2
+            end context
+            """
+        a = """\
+            start context
+            base line 1
+            end context
+            """
+        b = """\
+            start context
+            base line 1
+            end context
+            """
+        result = """\
+            start context
+            base line 1
+            end context
+            """
+        self._test_merge_from_strings(base, a, b, result)
+
+    def test_sync_on_deletion(self):
+        """Specific case of merge where we can synchronize incorrectly.
+        
+        A previous version of the weave merge concluded that the two versions
+        agreed on deleting line 2, and this could be a synchronization point.
+        Line 1 was then considered in isolation, and thought to be deleted on 
+        both sides.
+
+        It's better to consider the whole thing as a disagreement region.
+        """
+        base = """\
+            start context
+            base line 1
+            base line 2
+            end context
+            """
+        a = """\
+            start context
+            base line 1
+            a's replacement line 2
+            end context
+            """
+        b = """\
+            start context
+            b replaces
+            both lines
+            end context
+            """
+        result = """\
+            start context
+<<<<<<< 
+            base line 1
+            a's replacement line 2
+=======
+            b replaces
+            both lines
+>>>>>>> 
+            end context
+            """
+        self._test_merge_from_strings(base, a, b, result)
+
+
+class TestKnitMerge(TestCaseWithTransport, MergeCasesMixin):
+
+    def get_file(self, name='foo'):
+        return KnitVersionedFile(name, get_transport(self.get_url('.')),
+                                 delta=True, create=True)
+
+    def log_contents(self, w):
+        pass
+
+
+class TestWeaveMerge(TestCaseWithTransport, MergeCasesMixin):
+
+    def get_file(self, name='foo'):
+        return WeaveFile(name, get_transport(self.get_url('.')), create=True)
+
+    def log_contents(self, w):
+        self.log('weave is:')
+        tmpf = StringIO()
+        write_weave(w, tmpf)
+        self.log(tmpf.getvalue())
+
+    overlappedInsertExpected = ['aaa', '<<<<<<< ', 'xxx', 'yyy', '=======', 
+                                'xxx', '>>>>>>> ', 'bbb']

=== renamed file 'tools/testweave.py' => 'bzrlib/tests/test_weave.py'
--- tools/testweave.py	2005-08-18 07:51:58 +0000
+++ bzrlib/tests/test_weave.py	2006-04-18 23:42:34 +0000
@@ -18,27 +18,18 @@
 
 
 # TODO: tests regarding version names
-
-
+# TODO: rbc 20050108 test that join does not leave an inconsistent weave 
+#       if it fails.
 
 """test suite for weave algorithm"""
 
+from pprint import pformat
 
-import testsweet
-from bzrlib.weave import Weave, WeaveFormatError
+import bzrlib.errors as errors
+from bzrlib.weave import Weave, WeaveFormatError, WeaveError, reweave
 from bzrlib.weavefile import write_weave, read_weave
-from pprint import pformat
-
-
-try:
-    set
-    frozenset
-except NameError:
-    from sets import Set, ImmutableSet
-    set = Set
-    frozenset = ImmutableSet
-    del Set, ImmutableSet
-
+from bzrlib.tests import TestCase
+from bzrlib.osutils import sha_string
 
 
 # texts for use in testing
@@ -47,8 +38,7 @@
           "A second line"]
 
 
-
-class TestBase(testsweet.TestBase):
+class TestBase(TestCase):
     def check_read_write(self, k):
         """Check the weave k can be written & re-read."""
         from tempfile import TemporaryFile
@@ -68,11 +58,16 @@
             self.log('         %r' % k._parents)
             self.log('         %r' % k2._parents)
             self.log('')
-
-            
             self.fail('read/write check failed')
-        
-        
+
+
+class WeaveContains(TestBase):
+    """Weave __contains__ operator"""
+    def runTest(self):
+        k = Weave()
+        self.assertFalse('foo' in k)
+        k.add_lines('foo', [], TEXT_1)
+        self.assertTrue('foo' in k)
 
 
 class Easy(TestBase):
@@ -82,51 +77,86 @@
 
 class StoreText(TestBase):
     """Store and retrieve a simple text."""
-    def runTest(self):
+
+    def test_storing_text(self):
         k = Weave()
-        idx = k.add('text0', [], TEXT_0)
-        self.assertEqual(k.get(idx), TEXT_0)
+        idx = k.add_lines('text0', [], TEXT_0)
+        self.assertEqual(k.get_lines(idx), TEXT_0)
         self.assertEqual(idx, 0)
 
 
-
 class AnnotateOne(TestBase):
     def runTest(self):
         k = Weave()
-        k.add('text0', [], TEXT_0)
-        self.assertEqual(k.annotate(0),
-                         [(0, TEXT_0[0])])
+        k.add_lines('text0', [], TEXT_0)
+        self.assertEqual(k.annotate('text0'),
+                         [('text0', TEXT_0[0])])
 
 
 class StoreTwo(TestBase):
     def runTest(self):
         k = Weave()
 
-        idx = k.add('text0', [], TEXT_0)
+        idx = k.add_lines('text0', [], TEXT_0)
         self.assertEqual(idx, 0)
 
-        idx = k.add('text1', [], TEXT_1)
+        idx = k.add_lines('text1', [], TEXT_1)
         self.assertEqual(idx, 1)
 
-        self.assertEqual(k.get(0), TEXT_0)
-        self.assertEqual(k.get(1), TEXT_1)
-
-        k.dump(self.TEST_LOG)
-
-
+        self.assertEqual(k.get_lines(0), TEXT_0)
+        self.assertEqual(k.get_lines(1), TEXT_1)
+
+
+class GetSha1(TestBase):
+    def test_get_sha1(self):
+        k = Weave()
+        k.add_lines('text0', [], 'text0')
+        self.assertEqual('34dc0e430c642a26c3dd1c2beb7a8b4f4445eb79',
+                         k.get_sha1('text0'))
+        self.assertRaises(errors.RevisionNotPresent,
+                          k.get_sha1, 0)
+        self.assertRaises(errors.RevisionNotPresent,
+                          k.get_sha1, 'text1')
+                        
 
 class InvalidAdd(TestBase):
     """Try to use invalid version number during add."""
     def runTest(self):
         k = Weave()
 
-        self.assertRaises(IndexError,
-                          k.add,
+        self.assertRaises(errors.RevisionNotPresent,
+                          k.add_lines,
                           'text0',
-                          [69],
+                          ['69'],
                           ['new text!'])
 
 
+class RepeatedAdd(TestBase):
+    """Add the same version twice; harmless."""
+    def runTest(self):
+        k = Weave()
+        idx = k.add_lines('text0', [], TEXT_0)
+        idx2 = k.add_lines('text0', [], TEXT_0)
+        self.assertEqual(idx, idx2)
+
+
+class InvalidRepeatedAdd(TestBase):
+    def runTest(self):
+        k = Weave()
+        k.add_lines('basis', [], TEXT_0)
+        idx = k.add_lines('text0', [], TEXT_0)
+        self.assertRaises(errors.RevisionAlreadyPresent,
+                          k.add_lines,
+                          'text0',
+                          [],
+                          ['not the same text'])
+        self.assertRaises(errors.RevisionAlreadyPresent,
+                          k.add_lines,
+                          'text0',
+                          ['basis'],         # not the right parents
+                          TEXT_0)
+        
+
 class InsertLines(TestBase):
     """Store a revision that adds one line to the original.
 
@@ -135,53 +165,52 @@
     def runTest(self):
         k = Weave()
 
-        k.add('text0', [], ['line 1'])
-        k.add('text1', [0], ['line 1', 'line 2'])
-
-        self.assertEqual(k.annotate(0),
-                         [(0, 'line 1')])
-
-        self.assertEqual(k.get(1),
+        k.add_lines('text0', [], ['line 1'])
+        k.add_lines('text1', ['text0'], ['line 1', 'line 2'])
+
+        self.assertEqual(k.annotate('text0'),
+                         [('text0', 'line 1')])
+
+        self.assertEqual(k.get_lines(1),
                          ['line 1',
                           'line 2'])
 
-        self.assertEqual(k.annotate(1),
-                         [(0, 'line 1'),
-                          (1, 'line 2')])
-
-        k.add('text2', [0], ['line 1', 'diverged line'])
-
-        self.assertEqual(k.annotate(2),
-                         [(0, 'line 1'),
-                          (2, 'diverged line')])
+        self.assertEqual(k.annotate('text1'),
+                         [('text0', 'line 1'),
+                          ('text1', 'line 2')])
+
+        k.add_lines('text2', ['text0'], ['line 1', 'diverged line'])
+
+        self.assertEqual(k.annotate('text2'),
+                         [('text0', 'line 1'),
+                          ('text2', 'diverged line')])
 
         text3 = ['line 1', 'middle line', 'line 2']
-        k.add('text3',
-              [0, 1],
+        k.add_lines('text3',
+              ['text0', 'text1'],
               text3)
 
         # self.log("changes to text3: " + pformat(list(k._delta(set([0, 1]), text3))))
 
         self.log("k._weave=" + pformat(k._weave))
 
-        self.assertEqual(k.annotate(3),
-                         [(0, 'line 1'),
-                          (3, 'middle line'),
-                          (1, 'line 2')])
+        self.assertEqual(k.annotate('text3'),
+                         [('text0', 'line 1'),
+                          ('text3', 'middle line'),
+                          ('text1', 'line 2')])
 
         # now multiple insertions at different places
-        k.add('text4',
-              [0, 1, 3],
+        k.add_lines('text4',
+              ['text0', 'text1', 'text3'],
               ['line 1', 'aaa', 'middle line', 'bbb', 'line 2', 'ccc'])
 
-        self.assertEqual(k.annotate(4), 
-                         [(0, 'line 1'),
-                          (4, 'aaa'),
-                          (3, 'middle line'),
-                          (4, 'bbb'),
-                          (1, 'line 2'),
-                          (4, 'ccc')])
-
+        self.assertEqual(k.annotate('text4'), 
+                         [('text0', 'line 1'),
+                          ('text4', 'aaa'),
+                          ('text3', 'middle line'),
+                          ('text4', 'bbb'),
+                          ('text1', 'line 2'),
+                          ('text4', 'ccc')])
 
 
 class DeleteLines(TestBase):
@@ -193,7 +222,7 @@
 
         base_text = ['one', 'two', 'three', 'four']
 
-        k.add('text0', [], base_text)
+        k.add_lines('text0', [], base_text)
         
         texts = [['one', 'two', 'three'],
                  ['two', 'three', 'four'],
@@ -203,18 +232,16 @@
 
         i = 1
         for t in texts:
-            ver = k.add('text%d' % i,
-                        [0], t)
+            ver = k.add_lines('text%d' % i,
+                        ['text0'], t)
             i += 1
 
         self.log('final weave:')
         self.log('k._weave=' + pformat(k._weave))
 
         for i in range(len(texts)):
-            self.assertEqual(k.get(i+1),
+            self.assertEqual(k.get_lines(i+1),
                              texts[i])
-            
-
 
 
 class SuicideDelete(TestBase):
@@ -236,11 +263,10 @@
         return 
 
         self.assertRaises(WeaveFormatError,
-                          k.get,
+                          k.get_lines,
                           0)        
 
 
-
 class CannedDelete(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -257,20 +283,21 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linelast line')]
 
-        self.assertEqual(k.get(0),
+        self.assertEqual(k.get_lines(0),
                          ['first line',
                           'line to be deleted',
                           'last line',
                           ])
 
-        self.assertEqual(k.get(1),
+        self.assertEqual(k.get_lines(1),
                          ['first line',
                           'last line',
                           ])
 
 
-
 class CannedReplacement(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -290,21 +317,22 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linereplacement linelast line')]
 
-        self.assertEqual(k.get(0),
+        self.assertEqual(k.get_lines(0),
                          ['first line',
                           'line to be deleted',
                           'last line',
                           ])
 
-        self.assertEqual(k.get(1),
+        self.assertEqual(k.get_lines(1),
                          ['first line',
                           'replacement line',
                           'last line',
                           ])
 
 
-
 class BadWeave(TestBase):
     """Test that we trap an insert which should not occur."""
     def runTest(self):
@@ -390,22 +418,28 @@
                 '}',
                 ('}', 0)]
 
-        self.assertEqual(k.get(0),
+        k._sha1s = [sha_string('foo {}')
+                  , sha_string('foo {  added in version 1  also from v1}')
+                  , sha_string('foo {  added in v2}')
+                  , sha_string('foo {  added in version 1  added in v2  also from v1}')
+                  ]
+
+        self.assertEqual(k.get_lines(0),
                          ['foo {',
                           '}'])
 
-        self.assertEqual(k.get(1),
+        self.assertEqual(k.get_lines(1),
                          ['foo {',
                           '  added in version 1',
                           '  also from v1',
                           '}'])
                        
-        self.assertEqual(k.get(2),
+        self.assertEqual(k.get_lines(2),
                          ['foo {',
                           '  added in v2',
                           '}'])
 
-        self.assertEqual(k.get(3),
+        self.assertEqual(k.get_lines(3),
                          ['foo {',
                           '  added in version 1',
                           '  added in v2',
@@ -413,7 +447,6 @@
                           '}'])
                          
 
-
 class DeleteLines2(TestBase):
     """Test recording revisions that delete lines.
 
@@ -422,24 +455,23 @@
     def runTest(self):
         k = Weave()
 
-        k.add('text0', [], ["line the first",
+        k.add_lines('text0', [], ["line the first",
                    "line 2",
                    "line 3",
                    "fine"])
 
-        self.assertEqual(len(k.get(0)), 4)
+        self.assertEqual(len(k.get_lines(0)), 4)
 
-        k.add('text1', [0], ["line the first",
+        k.add_lines('text1', ['text0'], ["line the first",
                    "fine"])
 
-        self.assertEqual(k.get(1),
+        self.assertEqual(k.get_lines(1),
                          ["line the first",
                           "fine"])
 
-        self.assertEqual(k.annotate(1),
-                         [(0, "line the first"),
-                          (0, "fine")])
-
+        self.assertEqual(k.annotate('text1'),
+                         [('text0', "line the first"),
+                          ('text0', "fine")])
 
 
 class IncludeVersions(TestBase):
@@ -463,22 +495,26 @@
                 "second line",
                 ('}', 1)]
 
-        self.assertEqual(k.get(1),
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')]
+
+        self.assertEqual(k.get_lines(1),
                          ["first line",
                           "second line"])
 
-        self.assertEqual(k.get(0),
+        self.assertEqual(k.get_lines(0),
                          ["first line"])
 
-        k.dump(self.TEST_LOG)
-
 
 class DivergedIncludes(TestBase):
     """Weave with two diverged texts based on version 0.
     """
     def runTest(self):
+        # FIXME make the weave, dont poke at it.
         k = Weave()
 
+        k._names = ['0', '1', '2']
+        k._name_map = {'0':0, '1':1, '2':2}
         k._parents = [frozenset(),
                 frozenset([0]),
                 frozenset([0]),
@@ -494,20 +530,23 @@
                 ('}', 2),                
                 ]
 
-        self.assertEqual(k.get(0),
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')
+                  , sha_string('first linealternative second line')]
+
+        self.assertEqual(k.get_lines(0),
                          ["first line"])
 
-        self.assertEqual(k.get(1),
+        self.assertEqual(k.get_lines(1),
                          ["first line",
                           "second line"])
 
-        self.assertEqual(k.get(2),
+        self.assertEqual(k.get_lines('2'),
                          ["first line",
                           "alternative second line"])
 
-        self.assertEqual(list(k.inclusions([2])),
-                         [0, 2])
-
+        self.assertEqual(list(k.get_ancestry(['2'])),
+                         ['0', '2'])
 
 
 class ReplaceLine(TestBase):
@@ -517,14 +556,13 @@
         text0 = ['cheddar', 'stilton', 'gruyere']
         text1 = ['cheddar', 'blue vein', 'neufchatel', 'chevre']
         
-        k.add('text0', [], text0)
-        k.add('text1', [0], text1)
+        k.add_lines('text0', [], text0)
+        k.add_lines('text1', ['text0'], text1)
 
         self.log('k._weave=' + pformat(k._weave))
 
-        self.assertEqual(k.get(0), text0)
-        self.assertEqual(k.get(1), text1)
-
+        self.assertEqual(k.get_lines(0), text0)
+        self.assertEqual(k.get_lines(1), text1)
 
 
 class Merge(TestBase):
@@ -538,24 +576,24 @@
                  ['header', '', 'line from 1', 'fixup line', 'line from 2'],
                  ]
 
-        k.add('text0', [], texts[0])
-        k.add('text1', [0], texts[1])
-        k.add('text2', [0], texts[2])
-        k.add('merge', [0, 1, 2], texts[3])
+        k.add_lines('text0', [], texts[0])
+        k.add_lines('text1', ['text0'], texts[1])
+        k.add_lines('text2', ['text0'], texts[2])
+        k.add_lines('merge', ['text0', 'text1', 'text2'], texts[3])
 
         for i, t in enumerate(texts):
-            self.assertEqual(k.get(i), t)
+            self.assertEqual(k.get_lines(i), t)
 
-        self.assertEqual(k.annotate(3),
-                         [(0, 'header'),
-                          (1, ''),
-                          (1, 'line from 1'),
-                          (3, 'fixup line'),
-                          (2, 'line from 2'),
+        self.assertEqual(k.annotate('merge'),
+                         [('text0', 'header'),
+                          ('text1', ''),
+                          ('text1', 'line from 1'),
+                          ('merge', 'fixup line'),
+                          ('text2', 'line from 2'),
                           ])
 
-        self.assertEqual(list(k.inclusions([3])),
-                         [0, 1, 2, 3])
+        self.assertEqual(list(k.get_ancestry(['merge'])),
+                         ['text0', 'text1', 'text2', 'merge'])
 
         self.log('k._weave=' + pformat(k._weave))
 
@@ -572,9 +610,9 @@
         return  # NOT RUN
         k = Weave()
 
-        k.add([], ['aaa', 'bbb'])
-        k.add([0], ['aaa', '111', 'bbb'])
-        k.add([1], ['aaa', '222', 'bbb'])
+        k.add_lines([], ['aaa', 'bbb'])
+        k.add_lines([0], ['aaa', '111', 'bbb'])
+        k.add_lines([1], ['aaa', '222', 'bbb'])
 
         merged = k.merge([1, 2])
 
@@ -583,7 +621,6 @@
                            [['bbb']]])
 
 
-
 class NonConflict(TestBase):
     """Two descendants insert compatible changes.
 
@@ -592,42 +629,15 @@
         return  # NOT RUN
         k = Weave()
 
-        k.add([], ['aaa', 'bbb'])
-        k.add([0], ['111', 'aaa', 'ccc', 'bbb'])
-        k.add([1], ['aaa', 'ccc', 'bbb', '222'])
-
-    
-    
-
-
-class AutoMerge(TestBase):
-    def runTest(self):
-        k = Weave()
-
-        texts = [['header', 'aaa', 'bbb'],
-                 ['header', 'aaa', 'line from 1', 'bbb'],
-                 ['header', 'aaa', 'bbb', 'line from 2', 'more from 2'],
-                 ]
-
-        k.add('text0', [], texts[0])
-        k.add('text1', [0], texts[1])
-        k.add('text2', [0], texts[2])
-
-        self.log('k._weave=' + pformat(k._weave))
-
-        m = list(k.mash_iter([0, 1, 2]))
-
-        self.assertEqual(m,
-                         ['header', 'aaa',
-                          'line from 1',
-                          'bbb',
-                          'line from 2', 'more from 2'])
-        
+        k.add_lines([], ['aaa', 'bbb'])
+        k.add_lines([0], ['111', 'aaa', 'ccc', 'bbb'])
+        k.add_lines([1], ['aaa', 'ccc', 'bbb', '222'])
 
 
 class Khayyam(TestBase):
     """Test changes to multi-line texts, and read/write"""
-    def runTest(self):
+
+    def test_multi_line_merge(self):
         rawtexts = [
             """A Book of Verses underneath the Bough,
             A Jug of Wine, a Loaf of Bread, -- and Thou
@@ -659,131 +669,224 @@
         parents = set()
         i = 0
         for t in texts:
-            ver = k.add('text%d' % i,
+            ver = k.add_lines('text%d' % i,
                         list(parents), t)
-            parents.add(ver)
+            parents.add('text%d' % i)
             i += 1
 
         self.log("k._weave=" + pformat(k._weave))
 
         for i, t in enumerate(texts):
-            self.assertEqual(k.get(i), t)
+            self.assertEqual(k.get_lines(i), t)
 
         self.check_read_write(k)
 
 
-
-class MergeCases(TestBase):
-    def doMerge(self, base, a, b, mp):
+class JoinWeavesTests(TestBase):
+    def setUp(self):
+        super(JoinWeavesTests, self).setUp()
+        self.weave1 = Weave()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        self.weave1.add_lines('v1', [], self.lines1)
+        self.weave1.add_lines('v2', ['v1'], ['hello\n', 'world\n'])
+        self.weave1.add_lines('v3', ['v2'], self.lines3)
+        
+    def test_join_empty(self):
+        """Join two empty weaves."""
+        eq = self.assertEqual
+        w1 = Weave()
+        w2 = Weave()
+        w1.join(w2)
+        eq(len(w1), 0)
+        
+    def test_join_empty_to_nonempty(self):
+        """Join empty weave onto nonempty."""
+        self.weave1.join(Weave())
+        self.assertEqual(len(self.weave1), 3)
+
+    def test_join_unrelated(self):
+        """Join two weaves with no history in common."""
+        wb = Weave()
+        wb.add_lines('b1', [], ['line from b\n'])
+        w1 = self.weave1
+        w1.join(wb)
+        eq = self.assertEqual
+        eq(len(w1), 4)
+        eq(sorted(w1.versions()),
+           ['b1', 'v1', 'v2', 'v3'])
+
+    def test_join_related(self):
+        wa = self.weave1.copy()
+        wb = self.weave1.copy()
+        wa.add_lines('a1', ['v3'], ['hello\n', 'sweet\n', 'world\n'])
+        wb.add_lines('b1', ['v3'], ['hello\n', 'pale blue\n', 'world\n'])
+        eq = self.assertEquals
+        eq(len(wa), 4)
+        eq(len(wb), 4)
+        wa.join(wb)
+        eq(len(wa), 5)
+        eq(wa.get_lines('b1'),
+           ['hello\n', 'pale blue\n', 'world\n'])
+
+    def test_join_parent_disagreement(self):
+        #join reconciles differening parents into a union.
+        wa = Weave()
+        wb = Weave()
+        wa.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v0', [], [])
+        wb.add_lines('v1', ['v0'], ['hello\n'])
+        wa.join(wb)
+        self.assertEqual(['v0'], wa.get_parents('v1'))
+
+    def test_join_text_disagreement(self):
+        """Cannot join weaves with different texts for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v1', [], ['not\n', 'hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_unordered(self):
+        """Join weaves where indexes differ.
+        
+        The source weave contains a different version at index 0."""
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add_lines('x1', [], ['line from x1\n'])
+        wb.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v2', ['v1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        eq = self.assertEquals
+        eq(sorted(wa.versions()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wa.get_text('x1'), 'line from x1\n')
+
+    def test_written_detection(self):
+        # Test detection of weave file corruption.
+        #
+        # Make sure that we can detect if a weave file has
+        # been corrupted. This doesn't test all forms of corruption,
+        # but it at least helps verify the data you get, is what you want.
         from cStringIO import StringIO
-        from textwrap import dedent
 
-        def addcrlf(x):
-            return x + '\n'
-        
         w = Weave()
-        w.add('text0', [], map(addcrlf, base))
-        w.add('text1', [0], map(addcrlf, a))
-        w.add('text2', [0], map(addcrlf, b))
+        w.add_lines('v1', [], ['hello\n'])
+        w.add_lines('v2', ['v1'], ['hello\n', 'there\n'])
 
-        self.log('weave is:')
         tmpf = StringIO()
         write_weave(w, tmpf)
-        self.log(tmpf.getvalue())
-
-        self.log('merge plan:')
-        p = list(w.plan_merge(1, 2))
-        for state, line in p:
-            if line:
-                self.log('%12s | %s' % (state, line[:-1]))
-
-        self.log('merge:')
-        mt = StringIO()
-        mt.writelines(w.weave_merge(p))
-        mt.seek(0)
-        self.log(mt.getvalue())
-
-        mp = map(addcrlf, mp)
-        self.assertEqual(mt.readlines(), mp)
-        
-        
-    def testOneInsert(self):
-        self.doMerge([],
-                     ['aa'],
-                     [],
-                     ['aa'])
-
-    def testSeparateInserts(self):
-        self.doMerge(['aaa', 'bbb', 'ccc'],
-                     ['aaa', 'xxx', 'bbb', 'ccc'],
-                     ['aaa', 'bbb', 'yyy', 'ccc'],
-                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'])
-
-    def testSameInsert(self):
-        self.doMerge(['aaa', 'bbb', 'ccc'],
-                     ['aaa', 'xxx', 'bbb', 'ccc'],
-                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'],
-                     ['aaa', 'xxx', 'bbb', 'yyy', 'ccc'])
-
-    def testOverlappedInsert(self):
-        self.doMerge(['aaa', 'bbb'],
-                     ['aaa', 'xxx', 'yyy', 'bbb'],
-                     ['aaa', 'xxx', 'bbb'],
-                     ['aaa', '<<<<', 'xxx', 'yyy', '====', 'xxx', '>>>>', 'bbb'])
-
-        # really it ought to reduce this to 
-        # ['aaa', 'xxx', 'yyy', 'bbb']
-
-
-    def testClashReplace(self):
-        self.doMerge(['aaa'],
-                     ['xxx'],
-                     ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', '====', 'yyy', 'zzz', '>>>>'])
-
-    def testNonClashInsert(self):
-        self.doMerge(['aaa'],
-                     ['xxx', 'aaa'],
-                     ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', 'aaa', '====', 'yyy', 'zzz', '>>>>'])
-
-        self.doMerge(['aaa'],
-                     ['aaa'],
-                     ['yyy', 'zzz'],
-                     ['yyy', 'zzz'])
-
-
-    def testDeleteAndModify(self):
-        """Clashing delete and modification.
-
-        If one side modifies a region and the other deletes it then
-        there should be a conflict with one side blank.
-        """
-
-        #######################################
-        # skippd, not working yet
-        return
-        
-        self.doMerge(['aaa', 'bbb', 'ccc'],
-                     ['aaa', 'ddd', 'ccc'],
-                     ['aaa', 'ccc'],
-                     ['<<<<', 'aaa', '====', '>>>>', 'ccc'])
-    
-
-
-def testweave():
-    import testsweet
-    from unittest import TestSuite, TestLoader
-    import testweave
-
-    tl = TestLoader()
-    suite = TestSuite()
-    suite.addTest(tl.loadTestsFromModule(testweave))
-    
-    return int(not testsweet.run_suite(suite)) # for shell 0=true
-
-
-if __name__ == '__main__':
-    import sys
-    sys.exit(testweave())
-    
+
+        # Because we are corrupting, we need to make sure we have the exact text
+        self.assertEquals('# bzr weave file v5\n'
+                          'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                          'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                          'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n',
+                          tmpf.getvalue())
+
+        # Change a single letter
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. There\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        # Change the sha checksum
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 f0f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+
+class InstrumentedWeave(Weave):
+    """Keep track of how many times functions are called."""
+    
+    def __init__(self, weave_name=None):
+        self._extract_count = 0
+        Weave.__init__(self, weave_name=weave_name)
+
+    def _extract(self, versions):
+        self._extract_count += 1
+        return Weave._extract(self, versions)
+
+
+class JoinOptimization(TestCase):
+    """Test that Weave.join() doesn't extract all texts, only what must be done."""
+
+    def test_join(self):
+        w1 = InstrumentedWeave()
+        w2 = InstrumentedWeave()
+
+        txt0 = ['a\n']
+        txt1 = ['a\n', 'b\n']
+        txt2 = ['a\n', 'c\n']
+        txt3 = ['a\n', 'b\n', 'c\n']
+
+        w1.add_lines('txt0', [], txt0) # extract 1a
+        w2.add_lines('txt0', [], txt0) # extract 1b
+        w1.add_lines('txt1', ['txt0'], txt1)# extract 2a
+        w2.add_lines('txt2', ['txt0'], txt2)# extract 2b
+        w1.join(w2) # extract 3a to add txt2 
+        w2.join(w1) # extract 3b to add txt1 
+
+        w1.add_lines('txt3', ['txt1', 'txt2'], txt3) # extract 4a 
+        w2.add_lines('txt3', ['txt2', 'txt1'], txt3) # extract 4b
+        # These secretly have inverted parents
+
+        # This should not have to do any extractions
+        w1.join(w2) # NO extract, texts already present with same parents
+        w2.join(w1) # NO extract, texts already present with same parents
+
+        self.assertEqual(4, w1._extract_count)
+        self.assertEqual(4, w2._extract_count)
+
+    def test_double_parent(self):
+        # It should not be considered illegal to add
+        # a revision with the same parent twice
+        w1 = InstrumentedWeave()
+        w2 = InstrumentedWeave()
+
+        txt0 = ['a\n']
+        txt1 = ['a\n', 'b\n']
+        txt2 = ['a\n', 'c\n']
+        txt3 = ['a\n', 'b\n', 'c\n']
+
+        w1.add_lines('txt0', [], txt0)
+        w2.add_lines('txt0', [], txt0)
+        w1.add_lines('txt1', ['txt0'], txt1)
+        w2.add_lines('txt1', ['txt0', 'txt0'], txt1)
+        # Same text, effectively the same, because the
+        # parent is only repeated
+        w1.join(w2) # extract 3a to add txt2 
+        w2.join(w1) # extract 3b to add txt1 
+
+
+class TestNeedsReweave(TestCase):
+    """Internal corner cases for when reweave is needed."""
+
+    def test_compatible_parents(self):
+        w1 = Weave('a')
+        my_parents = set([1, 2, 3])
+        # subsets are ok
+        self.assertTrue(w1._compatible_parents(my_parents, set([3])))
+        # same sets
+        self.assertTrue(w1._compatible_parents(my_parents, set(my_parents)))
+        # same empty corner case
+        self.assertTrue(w1._compatible_parents(set(), set()))
+        # other cannot contain stuff my_parents does not
+        self.assertFalse(w1._compatible_parents(set(), set([1])))
+        self.assertFalse(w1._compatible_parents(my_parents, set([1, 2, 3, 4])))
+        self.assertFalse(w1._compatible_parents(my_parents, set([4])))

=== renamed file 'bzrlib/selftest/whitebox.py' => 'bzrlib/tests/test_whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-08-23 04:49:48 +0000
+++ bzrlib/tests/test_whitebox.py	2006-01-26 20:52:58 +0000
@@ -1,222 +1,49 @@
-#! /usr/bin/python
-
 import os
 import unittest
 
-from bzrlib.selftest import InTempDir, TestCase
+from bzrlib.tests import TestCaseWithTransport, TestCase
 from bzrlib.branch import ScratchBranch, Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
-
-
-class Unknowns(InTempDir):
-    def runTest(self):
-        b = Branch('.', init=True)
-
-        self.build_tree(['hello.txt',
-                         'hello.txt~'])
-
-        self.assertEquals(list(b.unknowns()),
-                          ['hello.txt'])
-
-
-
-class NoChanges(InTempDir):
-    def runTest(self):
-        from bzrlib.errors import PointlessCommit
-        
-        b = Branch('.', init=True)
-
-        self.build_tree(['hello.txt'])
-
-        self.assertRaises(PointlessCommit,
-                          b.commit,
-                          'commit without adding',
-                          allow_pointless=False)
-
-        b.commit('commit pointless tree',
-                 allow_pointless=True)
-
-        b.add('hello.txt')
-        
-        b.commit('commit first added file',
-                 allow_pointless=False)
-        
-        self.assertRaises(PointlessCommit,
-                          b.commit,
-                          'commit after adding file',
-                          allow_pointless=False)
-        
-        b.commit('commit pointless revision with one file',
-                 allow_pointless=True)
-
-        b.add_pending_merge('mbp@892739123-2005-123123')
-        b.commit('commit new merge with no text changes',
-                 allow_pointless=False)
-        
-
-
-class ValidateRevisionId(TestCase):
-    def runTest(self):
-        from bzrlib.revision import validate_revision_id
-        validate_revision_id('mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' asdkjas')
-
-
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe\n')
-
-
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'Martin Pool <mbp@sourcefrog.net>-20050311061123-96a255005c7c9dbe')
-
-
-
-class PendingMerges(InTempDir):
-    """Tracking pending-merged revisions."""
-    def runTest(self):
-        b = Branch('.', init=True)
-
-        self.assertEquals(b.pending_merges(), [])
-        
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+from bzrlib.errors import PathNotChild
+from bzrlib.osutils import relpath, pathjoin, abspath, realpath
+
+
+class MoreTests(TestCaseWithTransport):
+
+    def test_relpath(self):
+        """test for branch path lookups
     
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-
-        b.add_pending_merge('wibble@fofof--20050401--1928390812')
-        self.assertEquals(b.pending_merges(),
-                          ['foo@azkhazan-123123-abcabc',
-                           'wibble@fofof--20050401--1928390812'])
-
-        b.commit("commit from base with two merges")
-
-        rev = b.get_revision(b.revision_history()[0])
-        self.assertEquals(len(rev.parents), 2)
-        self.assertEquals(rev.parents[0].revision_id,
-                          'foo@azkhazan-123123-abcabc')
-        self.assertEquals(rev.parents[1].revision_id,
-                           'wibble@fofof--20050401--1928390812')
-
-        # list should be cleared when we do a commit
-        self.assertEquals(b.pending_merges(), [])
-        
-        
-        
-
-class Revert(InTempDir):
-    """Test selected-file revert"""
-    def runTest(self):
-        b = Branch('.', init=True)
-
-        self.build_tree(['hello.txt'])
-        file('hello.txt', 'w').write('initial hello')
-
-        self.assertRaises(NotVersionedError,
-                          b.revert, ['hello.txt'])
-        
-        b.add(['hello.txt'])
-        b.commit('create initial hello.txt')
-
-        self.check_file_contents('hello.txt', 'initial hello')
-        file('hello.txt', 'w').write('new hello')
-        self.check_file_contents('hello.txt', 'new hello')
-
-        # revert file modified since last revision
-        b.revert(['hello.txt'])
-        self.check_file_contents('hello.txt', 'initial hello')
-        self.check_file_contents('hello.txt~', 'new hello')
-
-        # reverting again clobbers the backup
-        b.revert(['hello.txt'])
-        self.check_file_contents('hello.txt', 'initial hello')
-        self.check_file_contents('hello.txt~', 'initial hello')
-
-
-
-class RenameDirs(InTempDir):
-    """Test renaming directories and the files within them."""
-    def runTest(self):
-        b = Branch('.', init=True)
-        self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
-        b.add(['dir', 'dir/sub', 'dir/sub/file'])
-
-        b.commit('create initial state')
-
-        # TODO: lift out to a test helper that checks the shape of
-        # an inventory
-        
-        revid = b.revision_history()[0]
-        self.log('first revision_id is {%s}' % revid)
-        
-        inv = b.get_revision_inventory(revid)
-        self.log('contents of inventory: %r' % inv.entries())
-
-        self.check_inventory_shape(inv,
-                                   ['dir', 'dir/sub', 'dir/sub/file'])
-
-        b.rename_one('dir', 'newdir')
-
-        self.check_inventory_shape(b.inventory,
-                                   ['newdir', 'newdir/sub', 'newdir/sub/file'])
-
-        b.rename_one('newdir/sub', 'newdir/newsub')
-        self.check_inventory_shape(b.inventory,
-                                   ['newdir', 'newdir/newsub',
-                                    'newdir/newsub/file'])
-
-        
-
-
-class BranchPathTestCase(TestCase):
-    """test for branch path lookups
-
-    Branch.relpath and bzrlib.branch._relpath do a simple but subtle
-    job: given a path (either relative to cwd or absolute), work out
-    if it is inside a branch and return the path relative to the base.
-    """
-
-    def runTest(self):
-        from bzrlib.branch import _relpath
+        bzrlib.osutils._relpath do a simple but subtle
+        job: given a path (either relative to cwd or absolute), work out
+        if it is inside a branch and return the path relative to the base.
+        """
         import tempfile, shutil
         
         savedir = os.getcwdu()
         dtmp = tempfile.mkdtemp()
         # On Mac OSX, /tmp actually expands to /private/tmp
-        dtmp = os.path.realpath(dtmp)
+        dtmp = realpath(dtmp)
 
         def rp(p):
-            return _relpath(dtmp, p)
+            return relpath(dtmp, p)
         
         try:
             # check paths inside dtmp while standing outside it
-            self.assertEqual(rp(os.path.join(dtmp, 'foo')), 'foo')
+            self.assertEqual(rp(pathjoin(dtmp, 'foo')), 'foo')
 
             # root = nothing
             self.assertEqual(rp(dtmp), '')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               '/etc')
 
             # now some near-miss operations -- note that
             # os.path.commonprefix gets these wrong!
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               dtmp.rstrip('\\/') + '2')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               dtmp.rstrip('\\/') + '2/foo')
 
@@ -230,22 +57,11 @@
 
             self.assertEqual(rp('./foo'), 'foo')
 
-            self.assertEqual(rp(os.path.abspath('foo')), 'foo')
+            self.assertEqual(rp(abspath('foo')), 'foo')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp, '../foo')
 
         finally:
             os.chdir(savedir)
             shutil.rmtree(dtmp)
-
-
-
-
-TEST_CLASSES = [Unknowns,
-                ValidateRevisionId,
-                PendingMerges,
-                Revert,
-                RenameDirs,
-                BranchPathTestCase,
-                ]

=== added file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_workingtree.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,236 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import os
+
+import bzrlib
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
+from bzrlib.conflicts import *
+import bzrlib.errors as errors
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.lockdir import LockDir
+from bzrlib.osutils import pathjoin, getcwd, has_symlinks
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+import bzrlib.workingtree as workingtree
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+class TestTreeDirectory(TestCaseWithTransport):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeDirectory().kind_character(), '/')
+
+
+class TestTreeEntry(TestCaseWithTransport):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeEntry().kind_character(), '???')
+
+
+class TestTreeFile(TestCaseWithTransport):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeFile().kind_character(), '')
+
+
+class TestTreeLink(TestCaseWithTransport):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeLink().kind_character(), '')
+
+
+class TestDefaultFormat(TestCaseWithTransport):
+
+    def test_get_set_default_format(self):
+        old_format = workingtree.WorkingTreeFormat.get_default_format()
+        # default is 3
+        self.assertTrue(isinstance(old_format, workingtree.WorkingTreeFormat3))
+        workingtree.WorkingTreeFormat.set_default_format(SampleTreeFormat())
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+            dir.create_repository()
+            dir.create_branch()
+            result = dir.create_workingtree()
+            self.assertEqual(result, 'A tree')
+        finally:
+            workingtree.WorkingTreeFormat.set_default_format(old_format)
+        self.assertEqual(old_format, workingtree.WorkingTreeFormat.get_default_format())
+
+
+class SampleTreeFormat(workingtree.WorkingTreeFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open_downlevel routines.
+    """
+
+    def get_format_string(self):
+        """See WorkingTreeFormat.get_format_string()."""
+        return "Sample tree format."
+
+    def initialize(self, a_bzrdir, revision_id=None):
+        """Sample branches cannot be created."""
+        t = a_bzrdir.get_workingtree_transport(self)
+        t.put('format', StringIO(self.get_format_string()))
+        return 'A tree'
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport, _found=False):
+        return "opened tree."
+
+
+class TestWorkingTreeFormat(TestCaseWithTransport):
+    """Tests for the WorkingTreeFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a working tree?
+        # create a branch with a few known format objects.
+        self.build_tree(["foo/", "bar/"])
+        def check_format(format, url):
+            dir = format._matchingbzrdir.initialize(url)
+            dir.create_repository()
+            dir.create_branch()
+            format.initialize(dir)
+            t = get_transport(url)
+            found_format = workingtree.WorkingTreeFormat.find_format(dir)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(workingtree.WorkingTreeFormat3(), "bar")
+        
+    def test_find_format_no_tree(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+        self.assertRaises(errors.NoWorkingTree,
+                          workingtree.WorkingTreeFormat.find_format,
+                          dir)
+
+    def test_find_format_unknown_format(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+        dir.create_repository()
+        dir.create_branch()
+        SampleTreeFormat().initialize(dir)
+        self.assertRaises(errors.UnknownFormatError,
+                          workingtree.WorkingTreeFormat.find_format,
+                          dir)
+
+    def test_register_unregister_format(self):
+        format = SampleTreeFormat()
+        # make a control dir
+        dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+        dir.create_repository()
+        dir.create_branch()
+        # make a branch
+        format.initialize(dir)
+        # register a format for it.
+        workingtree.WorkingTreeFormat.register_format(format)
+        # which branch.Open will refuse (not supported)
+        self.assertRaises(errors.UnsupportedFormatError, workingtree.WorkingTree.open, '.')
+        # but open_downlevel will work
+        self.assertEqual(format.open(dir), workingtree.WorkingTree.open_downlevel('.'))
+        # unregister the format
+        workingtree.WorkingTreeFormat.unregister_format(format)
+
+
+class TestWorkingTreeFormat3(TestCaseWithTransport):
+    """Tests specific to WorkingTreeFormat3."""
+
+    def test_disk_layout(self):
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        control.create_repository()
+        control.create_branch()
+        tree = workingtree.WorkingTreeFormat3().initialize(control)
+        # we want:
+        # format 'Bazaar-NG Working Tree format 3'
+        # inventory = blank inventory
+        # pending-merges = ''
+        # stat-cache = ??
+        # no inventory.basis yet
+        t = control.get_workingtree_transport(None)
+        self.assertEqualDiff('Bazaar-NG Working Tree format 3',
+                             t.get('format').read())
+        self.assertEqualDiff('<inventory format="5">\n'
+                             '</inventory>\n',
+                             t.get('inventory').read())
+        self.assertEqualDiff('### bzr hashcache v5\n',
+                             t.get('stat-cache').read())
+        self.assertFalse(t.has('inventory.basis'))
+        # no last-revision file means 'None' or 'NULLREVISION'
+        self.assertFalse(t.has('last-revision'))
+        # TODO RBC 20060210 do a commit, check the inventory.basis is created 
+        # correctly and last-revision file becomes present.
+
+    def test_uses_lockdir(self):
+        """WorkingTreeFormat3 uses its own LockDir:
+            
+            - lock is a directory
+            - when the WorkingTree is locked, LockDir can see that
+        """
+        t = self.get_transport()
+        url = self.get_url()
+        dir = bzrdir.BzrDirMetaFormat1().initialize(url)
+        repo = dir.create_repository()
+        branch = dir.create_branch()
+        try:
+            tree = workingtree.WorkingTreeFormat3().initialize(dir)
+        except errors.NotLocalUrl:
+            raise TestSkipped('Not a local URL')
+        self.assertIsDirectory('.bzr', t)
+        self.assertIsDirectory('.bzr/checkout', t)
+        self.assertIsDirectory('.bzr/checkout/lock', t)
+        our_lock = LockDir(t, '.bzr/checkout/lock')
+        self.assertEquals(our_lock.peek(), None)
+        tree.lock_write()
+        self.assertTrue(our_lock.peek())
+        tree.unlock()
+        self.assertEquals(our_lock.peek(), None)
+
+
+class TestFormat2WorkingTree(TestCaseWithTransport):
+    """Tests that are specific to format 2 trees."""
+
+    def create_format2_tree(self, url):
+        return self.make_branch_and_tree(
+            url, format=bzrlib.bzrdir.BzrDirFormat6())
+
+    def test_conflicts(self):
+        # test backwards compatability
+        tree = self.create_format2_tree('.')
+        self.assertRaises(errors.UnsupportedOperation, tree.set_conflicts,
+                          None)
+        file('lala.BASE', 'wb').write('labase')
+        expected = ContentsConflict('lala')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        file('lala', 'wb').write('la')
+        tree.add('lala', 'lala-id')
+        expected = ContentsConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        file('lala.THIS', 'wb').write('lathis')
+        file('lala.OTHER', 'wb').write('laother')
+        # When "text conflict"s happen, stem, THIS and OTHER are text
+        expected = TextConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        os.unlink('lala.OTHER')
+        os.mkdir('lala.OTHER')
+        expected = ContentsConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])

=== added file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_xml.py	2006-04-06 10:21:12 +0000
@@ -0,0 +1,173 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+
+from bzrlib.tests import TestCase
+from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+_revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
+_committed_inv_v5 = """<inventory>
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      revision="mbp@foo-123123"/>
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           revision="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      revision="mbp@foo-00"/>
+</inventory>
+"""
+
+_basis_inv_v5 = """<inventory revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92">
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      revision="mbp@foo-123123"/>
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           revision="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      revision="mbp@foo-00"/>
+</inventory>
+"""
+
+class TestSerializer(TestCase):
+    """Test XML serialization"""
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_revision_5(self):
+        """Test unpacking a canned revision v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.timezone, 36000)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.revision, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_unpack_basis_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_basis_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        eq(inv.revision_id, 'mbp@sourcefrog.net-20050905063503-43948f59fa127d92')
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.revision, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_repack_inventory_5(self):
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        serializer_v5.write_inventory(inv, outp)
+        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def test_repack_revision_5(self):
+        """Round-trip revision to XML v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        outfile_contents = outp.getvalue()
+        rev2 = serializer_v5.read_revision(StringIO(outfile_contents))
+        self.assertEqual(rev, rev2)
+
+    def test_pack_revision_5(self):
+        """Pack revision to XML v5"""
+        # fixed 20051025, revisions should have final newline
+        rev = serializer_v5.read_revision_from_string(_revision_v5)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        outfile_contents = outp.getvalue()
+        self.assertEqual(outfile_contents[-1], '\n')
+        self.assertEqualDiff(outfile_contents, serializer_v5.write_revision_to_string(rev))

=== added file 'bzrlib/tests/treeshape.py'
--- bzrlib/tests/treeshape.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/treeshape.py	2005-12-01 17:25:16 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Test helper for constructing and testing directories.
+
+This module transforms filesystem directories to and from Python lists.  
+As a Python list the descriptions can be stored in test cases, compared,
+etc.
+"""
+
+# TODO: Script to write a description of a directory for testing
+# TODO: Helper that compares two structures and raises a helpful error
+# where they differ.  Option to ignore some files or directories in the 
+# comparison.
+
+import os
+import stat
+
+from bzrlib.trace import warning
+from bzrlib.osutils import pathjoin
+
+def build_tree_contents(template):
+    """Reconstitute some files from a text description.
+
+    Each element of template is a tuple.  The first element is a filename,
+    with an optional ending character indicating the type.
+
+    The template is built relative to the Python process's current
+    working directory.
+    """
+    for tt in template:
+        name = tt[0]
+        if name[-1] == '/':
+            os.mkdir(name)
+        elif name[-1] == '@':
+            raise NotImplementedError('symlinks not handled yet')
+        else:
+            f = file(name, 'wb')
+            try:
+                f.write(tt[1])
+            finally:
+                f.close()
+
+
+def capture_tree_contents(top):
+    """Make a Python datastructure description of a tree.
+    
+    If top is an absolute path the descriptions will be absolute."""
+    for dirpath, dirnames, filenames in os.walk(top):
+        yield (dirpath + '/', )
+        filenames.sort()
+        for fn in filenames:
+            fullpath = pathjoin(dirpath, fn)
+            self.assertFalse(fullpath[-1] in '@/')
+            info = os.lstat(fullpath)
+            if stat.S_ISLNK(info.st_mode):
+                yield (fullpath + '@', os.readlink(fullpath))
+            elif stat.S_ISREG(info.st_mode):
+                yield (fullpath, file(fullpath, 'rb').read())
+            else:
+                warning("can't capture file %s with mode %#o",
+                        fullpath, info.st_mode)
+                pass

=== added directory 'bzrlib/tests/workingtree_implementations'
=== added file 'bzrlib/tests/workingtree_implementations/__init__.py'
--- bzrlib/tests/workingtree_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/__init__.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""WorkingTree implementation tests for bzr.
+
+These test the conformance of all the workingtre variations to the expected API.
+Specific tests for individual formats are in the tests/test_workingtree file 
+rather than in tests/workingtree_implementations/*.py.
+"""
+
+import bzrlib.errors as errors
+from bzrlib.transport import get_transport
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.workingtree import (WorkingTreeFormat,
+                                WorkingTreeTestProviderAdapter,
+                                _legacy_formats,
+                                )
+
+
+class TestCaseWithWorkingTree(TestCaseWithBzrDir):
+
+    def make_branch_and_tree(self, relpath, format=None):
+        made_control = self.make_bzrdir(relpath, format=format)
+        made_control.create_repository()
+        made_control.create_branch()
+        return self.workingtree_format.initialize(made_control)
+
+
+def test_suite():
+    result = TestSuite()
+    test_workingtree_implementations = [
+        'bzrlib.tests.workingtree_implementations.test_basis_inventory',
+        'bzrlib.tests.workingtree_implementations.test_is_control_filename',
+        'bzrlib.tests.workingtree_implementations.test_pull',
+        'bzrlib.tests.workingtree_implementations.test_workingtree',
+        ]
+    adapter = WorkingTreeTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        [(format, format._matchingbzrdir) for format in 
+         WorkingTreeFormat._formats.values() + _legacy_formats])
+    loader = TestLoader()
+    adapt_modules(test_workingtree_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/workingtree_implementations/test_basis_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_basis_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+from bzrlib.branch import Branch
+from bzrlib.xml5 import serializer_v5
+
+
+class TestBasisInventory(TestCaseWithWorkingTree):
+
+    def test_create(self):
+        # TODO: jam 20051218 this probably should add more than just
+        #                    a couple files to the inventory
+
+        # Make sure the basis file is created by a commit
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        t._control_files.get_utf8('basis-inventory')
+
+        basis_inv = t.basis_tree().inventory
+        self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.repository.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        t._control_files.get_utf8('basis-inventory')
+
+        basis_inv_txt = t.read_basis_inventory()
+        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        self.assertEquals('r2', basis_inv.revision_id)
+        store_inv = b.repository.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+

=== added file 'bzrlib/tests/workingtree_implementations/test_is_control_filename.py'
--- bzrlib/tests/workingtree_implementations/test_is_control_filename.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_is_control_filename.py	2006-02-16 02:53:00 +0000
@@ -0,0 +1,55 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import os
+
+import bzrlib
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
+import bzrlib.errors as errors
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.osutils import basename
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+import bzrlib.workingtree as workingtree
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+
+class TestIsControlFilename(TestCaseWithWorkingTree):
+
+    def validate_tree_is_controlfilename(self, tree):
+        """check that 'tree' obeys the contract for is_control_filename."""
+        bzrdirname = basename(tree.bzrdir.transport.base[:-1])
+        self.assertTrue(tree.is_control_filename(bzrdirname))
+        self.assertTrue(tree.is_control_filename(bzrdirname + '/subdir'))
+        self.assertFalse(tree.is_control_filename('dir/' + bzrdirname))
+        self.assertFalse(tree.is_control_filename('dir/' + bzrdirname + '/sub'))
+
+    def test_dotbzr_is_control_in_cwd(self):
+        tree = self.make_branch_and_tree('.')
+        self.validate_tree_is_controlfilename(tree)
+        
+    def test_dotbzr_is_control_in_subdir(self):
+        tree = self.make_branch_and_tree('subdir')
+        self.validate_tree_is_controlfilename(tree)
+        

=== added file 'bzrlib/tests/workingtree_implementations/test_pull.py'
--- bzrlib/tests/workingtree_implementations/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_pull.py	2006-02-22 05:45:47 +0000
@@ -0,0 +1,67 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import os
+
+import bzrlib
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
+import bzrlib.errors as errors
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.osutils import basename
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+import bzrlib.workingtree as workingtree
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+
+class TestPull(TestCaseWithWorkingTree):
+
+    def get_pullable_trees(self):
+        self.build_tree(['from/', 'from/file', 'to/'])
+        tree = self.make_branch_and_tree('from')
+        tree.add('file')
+        tree.commit('foo', rev_id='A')
+        tree_b = self.make_branch_and_tree('to')
+        return tree, tree_b
+ 
+    def test_pull(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.pull(tree_a.branch)
+        self.failUnless(tree_b.branch.repository.has_revision('A'))
+        self.assertEqual('A', tree_b.last_revision())
+
+    def test_pull_overwrites(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.commit('foo', rev_id='B')
+        self.assertEqual(['B'], tree_b.branch.revision_history())
+        tree_b.pull(tree_a.branch, overwrite=True)
+        self.failUnless(tree_b.branch.repository.has_revision('A'))
+        self.failUnless(tree_b.branch.repository.has_revision('B'))
+        self.assertEqual('A', tree_b.last_revision())
+
+    def test_pull_merges_tree_content(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.pull(tree_a.branch)
+        self.assertFileEqual('contents of from/file\n', 'to/file')
+

=== added file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-04-12 03:21:39 +0000
@@ -0,0 +1,592 @@
+# (C) 2005,2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import os
+
+import bzrlib
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
+import bzrlib.errors as errors
+from bzrlib.errors import (NotBranchError, NotVersionedError, 
+                           UnsupportedOperation)
+from bzrlib.osutils import pathjoin, getcwd, has_symlinks
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+from bzrlib.trace import mutter
+import bzrlib.workingtree as workingtree
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+
+class TestWorkingTree(TestCaseWithWorkingTree):
+
+    def test_listfiles(self):
+        tree = self.make_branch_and_tree('.')
+        os.mkdir('dir')
+        print >> open('file', 'w'), "content"
+        if has_symlinks():
+            os.symlink('target', 'symlink')
+        files = list(tree.list_files())
+        self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
+        self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
+        if has_symlinks():
+            self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+
+    def test_open_containing(self):
+        branch = self.make_branch_and_tree('.').branch
+        wt, relpath = WorkingTree.open_containing()
+        self.assertEqual('', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        wt, relpath = WorkingTree.open_containing(u'.')
+        self.assertEqual('', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        wt, relpath = WorkingTree.open_containing('./foo')
+        self.assertEqual('foo', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        wt, relpath = WorkingTree.open_containing('file://' + getcwd() + '/foo')
+        self.assertEqual('foo', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+
+    def test_basic_relpath(self):
+        # for comprehensive relpath tests, see whitebox.py.
+        tree = self.make_branch_and_tree('.')
+        self.assertEqual('child',
+                         tree.relpath(pathjoin(getcwd(), 'child')))
+
+    def test_lock_locks_branch(self):
+        tree = self.make_branch_and_tree('.')
+        tree.lock_read()
+        self.assertEqual('r', tree.branch.peek_lock_mode())
+        tree.unlock()
+        self.assertEqual(None, tree.branch.peek_lock_mode())
+        tree.lock_write()
+        self.assertEqual('w', tree.branch.peek_lock_mode())
+        tree.unlock()
+        self.assertEqual(None, tree.branch.peek_lock_mode())
+ 
+    def test_revert(self):
+        """Test selected-file revert"""
+        tree = self.make_branch_and_tree('.')
+
+        self.build_tree(['hello.txt'])
+        file('hello.txt', 'w').write('initial hello')
+
+        self.assertRaises(NotVersionedError,
+                          tree.revert, ['hello.txt'])
+        tree.add(['hello.txt'])
+        tree.commit('create initial hello.txt')
+
+        self.check_file_contents('hello.txt', 'initial hello')
+        file('hello.txt', 'w').write('new hello')
+        self.check_file_contents('hello.txt', 'new hello')
+
+        # revert file modified since last revision
+        tree.revert(['hello.txt'])
+        self.check_file_contents('hello.txt', 'initial hello')
+        self.check_file_contents('hello.txt.~1~', 'new hello')
+
+        # reverting again does not clobber the backup
+        tree.revert(['hello.txt'])
+        self.check_file_contents('hello.txt', 'initial hello')
+        self.check_file_contents('hello.txt.~1~', 'new hello')
+        
+        # backup files are numbered
+        file('hello.txt', 'w').write('new hello2')
+        tree.revert(['hello.txt'])
+        self.check_file_contents('hello.txt', 'initial hello')
+        self.check_file_contents('hello.txt.~1~', 'new hello')
+        self.check_file_contents('hello.txt.~2~', 'new hello2')
+
+    def test_revert_missing(self):
+        # Revert a file that has been deleted since last commit
+        tree = self.make_branch_and_tree('.')
+        file('hello.txt', 'w').write('initial hello')
+        tree.add('hello.txt')
+        tree.commit('added hello.txt')
+        os.unlink('hello.txt')
+        tree.remove('hello.txt')
+        tree.revert(['hello.txt'])
+        self.failUnlessExists('hello.txt')
+
+    def test_unknowns(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['hello.txt',
+                         'hello.txt.~1~'])
+        self.assertEquals(list(tree.unknowns()),
+                          ['hello.txt'])
+
+    def test_hashcache(self):
+        from bzrlib.tests.test_hashcache import pause
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['hello.txt',
+                         'hello.txt.~1~'])
+        tree.add('hello.txt')
+        pause()
+        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
+        self.assertEqual(1, tree._hashcache.miss_count)
+        tree2 = WorkingTree.open('.')
+        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
+        self.assertEqual(0, tree2._hashcache.miss_count)
+        self.assertEqual(1, tree2._hashcache.hit_count)
+
+    def test_initialize(self):
+        # initialize should create a working tree and branch in an existing dir
+        t = self.make_branch_and_tree('.')
+        b = Branch.open('.')
+        self.assertEqual(t.branch.base, b.base)
+        t2 = WorkingTree.open('.')
+        self.assertEqual(t.basedir, t2.basedir)
+        self.assertEqual(b.base, t2.branch.base)
+        # TODO maybe we should check the branch format? not sure if its
+        # appropriate here.
+
+    def test_rename_dirs(self):
+        """Test renaming directories and the files within them."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
+        wt.add(['dir', 'dir/sub', 'dir/sub/file'])
+
+        wt.commit('create initial state')
+
+        revid = b.revision_history()[0]
+        self.log('first revision_id is {%s}' % revid)
+        
+        inv = b.repository.get_revision_inventory(revid)
+        self.log('contents of inventory: %r' % inv.entries())
+
+        self.check_inventory_shape(inv,
+                                   ['dir', 'dir/sub', 'dir/sub/file'])
+
+        wt.rename_one('dir', 'newdir')
+
+        self.check_inventory_shape(wt.read_working_inventory(),
+                                   ['newdir', 'newdir/sub', 'newdir/sub/file'])
+
+        wt.rename_one('newdir/sub', 'newdir/newsub')
+        self.check_inventory_shape(wt.read_working_inventory(),
+                                   ['newdir', 'newdir/newsub',
+                                    'newdir/newsub/file'])
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" adds the parent as necessary, but simple working tree add
+        doesn't do that.
+        """
+        from bzrlib.errors import NotVersionedError
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo/',
+                         'foo/hello'])
+        self.assertRaises(NotVersionedError,
+                          wt.add,
+                          'foo/hello')
+
+    def test_add_missing(self):
+        # adding a msising file -> NoSuchFile
+        wt = self.make_branch_and_tree('.')
+        self.assertRaises(errors.NoSuchFile, wt.add, 'fpp')
+
+    def test_remove_verbose(self):
+        #FIXME the remove api should not print or otherwise depend on the
+        # text UI - RBC 20060124
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['hello'])
+        wt.add(['hello'])
+        wt.commit(message='add hello')
+        stdout = StringIO()
+        stderr = StringIO()
+        self.assertEqual(None, self.apply_redirected(None, stdout, stderr,
+                                                     wt.remove,
+                                                     ['hello'],
+                                                     verbose=True))
+        self.assertEqual('?       hello\n', stdout.getvalue())
+        self.assertEqual('', stderr.getvalue())
+
+    def test_clone_trivial(self):
+        wt = self.make_branch_and_tree('source')
+        cloned_dir = wt.bzrdir.clone('target')
+        cloned = cloned_dir.open_workingtree()
+        self.assertEqual(cloned.last_revision(), wt.last_revision())
+
+    def test_last_revision(self):
+        wt = self.make_branch_and_tree('source')
+        self.assertEqual(None, wt.last_revision())
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        self.assertEqual('A', wt.last_revision())
+
+    def test_set_last_revision(self):
+        wt = self.make_branch_and_tree('source')
+        self.assertEqual(None, wt.last_revision())
+        # cannot set the last revision to one not in the branch history.
+        self.assertRaises(errors.NoSuchRevision, wt.set_last_revision, 'A')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        self.assertEqual('A', wt.last_revision())
+        # None is aways in the branch
+        wt.set_last_revision(None)
+        self.assertEqual(None, wt.last_revision())
+        # and now we can set it to 'A'
+        # because some formats mutate the branch to set it on the tree
+        # we need to alter the branch to let this pass.
+        wt.branch.set_revision_history(['A', 'B'])
+        wt.set_last_revision('A')
+        self.assertEqual('A', wt.last_revision())
+
+    def test_set_last_revision_different_to_branch(self):
+        # working tree formats from the meta-dir format and newer support
+        # setting the last revision on a tree independently of that on the 
+        # branch. Its concievable that some future formats may want to 
+        # couple them again (i.e. because its really a smart server and
+        # the working tree will always match the branch). So we test
+        # that formats where initialising a branch does not initialise a 
+        # tree - and thus have separable entities - support skewing the 
+        # two things.
+        branch = self.make_branch('tree')
+        try:
+            # if there is a working tree now, this is not supported.
+            branch.bzrdir.open_workingtree()
+            return
+        except errors.NoWorkingTree:
+            pass
+        wt = branch.bzrdir.create_workingtree()
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        wt.set_last_revision(None)
+        self.assertEqual(None, wt.last_revision())
+        self.assertEqual('A', wt.branch.last_revision())
+        # and now we can set it back to 'A'
+        wt.set_last_revision('A')
+        self.assertEqual('A', wt.last_revision())
+        self.assertEqual('A', wt.branch.last_revision())
+
+    def test_clone_and_commit_preserves_last_revision(self):
+        wt = self.make_branch_and_tree('source')
+        cloned_dir = wt.bzrdir.clone('target')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        self.assertNotEqual(cloned_dir.open_workingtree().last_revision(),
+                            wt.last_revision())
+
+    def test_clone_preserves_content(self):
+        wt = self.make_branch_and_tree('source')
+        self.build_tree(['added', 'deleted', 'notadded'], transport=wt.bzrdir.transport.clone('..'))
+        wt.add('deleted', 'deleted')
+        wt.commit('add deleted')
+        wt.remove('deleted')
+        wt.add('added', 'added')
+        cloned_dir = wt.bzrdir.clone('target')
+        cloned = cloned_dir.open_workingtree()
+        cloned_transport = cloned.bzrdir.transport.clone('..')
+        self.assertFalse(cloned_transport.has('deleted'))
+        self.assertTrue(cloned_transport.has('added'))
+        self.assertFalse(cloned_transport.has('notadded'))
+        self.assertEqual('added', cloned.path2id('added'))
+        self.assertEqual(None, cloned.path2id('deleted'))
+        self.assertEqual(None, cloned.path2id('notadded'))
+        
+    def test_basis_tree_returns_last_revision(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo'])
+        wt.add('foo', 'foo-id')
+        wt.commit('A', rev_id='A')
+        wt.rename_one('foo', 'bar')
+        wt.commit('B', rev_id='B')
+        wt.set_last_revision('B')
+        tree = wt.basis_tree()
+        self.failUnless(tree.has_filename('bar'))
+        wt.set_last_revision('A')
+        tree = wt.basis_tree()
+        self.failUnless(tree.has_filename('foo'))
+
+    def test_clone_tree_revision(self):
+        # make a tree with a last-revision,
+        # and clone it with a different last-revision, this should switch
+        # do it.
+        #
+        # also test that the content is merged
+        # and conflicts recorded.
+        # This should merge between the trees - local edits should be preserved
+        # but other changes occured.
+        # we test this by having one file that does
+        # not change between two revisions, and another that does -
+        # if the changed one is not changed, fail,
+        # if the one that did not change has lost a local change, fail.
+        # 
+        raise TestSkipped('revision limiting is not implemented yet.')
+
+    def test_initialize_with_revision_id(self):
+        # a bzrdir can construct a working tree for itself @ a specific revision.
+        source = self.make_branch_and_tree('source')
+        source.commit('a', rev_id='a', allow_pointless=True)
+        source.commit('b', rev_id='b', allow_pointless=True)
+        self.build_tree(['new/'])
+        made_control = self.bzrdir_format.initialize('new')
+        source.branch.repository.clone(made_control)
+        source.branch.clone(made_control)
+        made_tree = self.workingtree_format.initialize(made_control, revision_id='a')
+        self.assertEqual('a', made_tree.last_revision())
+
+    def test_commit_sets_last_revision(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.commit('foo', rev_id='foo', allow_pointless=True)
+        self.assertEqual('foo', tree.last_revision())
+
+    def test_commit_local_unbound(self):
+        # using the library api to do a local commit on unbound branches is 
+        # also an error
+        tree = self.make_branch_and_tree('tree')
+        self.assertRaises(errors.LocalRequiresBoundBranch,
+                          tree.commit,
+                          'foo',
+                          local=True)
+ 
+    def test_local_commit_ignores_master(self):
+        # a --local commit does not require access to the master branch
+        # at all, or even for it to exist.
+        # we test this by setting up a bound branch and then corrupting
+        # the master.
+        master = self.make_branch('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # older format.
+            return
+        master.bzrdir.transport.put('branch-format', StringIO('garbage'))
+        del master
+        # check its corrupted.
+        self.assertRaises(errors.UnknownFormatError,
+                          bzrdir.BzrDir.open,
+                          'master')
+        tree.commit('foo', rev_id='foo', local=True)
+ 
+    def test_local_commit_does_not_push_to_master(self):
+        # a --local commit does not require access to the master branch
+        # at all, or even for it to exist.
+        # we test that even when its available it does not push to it.
+        master = self.make_branch('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # older format.
+            return
+        tree.commit('foo', rev_id='foo', local=True)
+        self.failIf(master.repository.has_revision('foo'))
+        self.assertEqual(None, master.last_revision())
+        
+    def test_update_sets_last_revision(self):
+        # working tree formats from the meta-dir format and newer support
+        # setting the last revision on a tree independently of that on the 
+        # branch. Its concievable that some future formats may want to 
+        # couple them again (i.e. because its really a smart server and
+        # the working tree will always match the branch). So we test
+        # that formats where initialising a branch does not initialise a 
+        # tree - and thus have separable entities - support skewing the 
+        # two things.
+        main_branch = self.make_branch('tree')
+        try:
+            # if there is a working tree now, this is not supported.
+            main_branch.bzrdir.open_workingtree()
+            return
+        except errors.NoWorkingTree:
+            pass
+        wt = main_branch.bzrdir.create_workingtree()
+        # create an out of date working tree by making a checkout in this
+        # current format
+        self.build_tree(['checkout/', 'tree/file'])
+        checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
+        bzrlib.branch.BranchReferenceFormat().initialize(checkout, main_branch)
+        old_tree = self.workingtree_format.initialize(checkout)
+        # now commit to 'tree'
+        wt.add('file')
+        wt.commit('A', rev_id='A')
+        # and update old_tree
+        self.assertEqual(0, old_tree.update())
+        self.failUnlessExists('checkout/file')
+        self.assertEqual('A', old_tree.last_revision())
+
+    def test_update_returns_conflict_count(self):
+        # working tree formats from the meta-dir format and newer support
+        # setting the last revision on a tree independently of that on the 
+        # branch. Its concievable that some future formats may want to 
+        # couple them again (i.e. because its really a smart server and
+        # the working tree will always match the branch). So we test
+        # that formats where initialising a branch does not initialise a 
+        # tree - and thus have separable entities - support skewing the 
+        # two things.
+        main_branch = self.make_branch('tree')
+        try:
+            # if there is a working tree now, this is not supported.
+            main_branch.bzrdir.open_workingtree()
+            return
+        except errors.NoWorkingTree:
+            pass
+        wt = main_branch.bzrdir.create_workingtree()
+        # create an out of date working tree by making a checkout in this
+        # current format
+        self.build_tree(['checkout/', 'tree/file'])
+        checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
+        bzrlib.branch.BranchReferenceFormat().initialize(checkout, main_branch)
+        old_tree = self.workingtree_format.initialize(checkout)
+        # now commit to 'tree'
+        wt.add('file')
+        wt.commit('A', rev_id='A')
+        # and add a file file to the checkout
+        self.build_tree(['checkout/file'])
+        old_tree.add('file')
+        # and update old_tree
+        self.assertEqual(1, old_tree.update())
+        self.assertEqual('A', old_tree.last_revision())
+
+    def test_merge_revert(self):
+        from bzrlib.merge import merge_inner
+        this = self.make_branch_and_tree('b1')
+        open('b1/a', 'wb').write('a test\n')
+        this.add('a')
+        open('b1/b', 'wb').write('b test\n')
+        this.add('b')
+        this.commit(message='')
+        base = this.bzrdir.clone('b2').open_workingtree()
+        open('b2/a', 'wb').write('b test\n')
+        other = this.bzrdir.clone('b3').open_workingtree()
+        open('b3/a', 'wb').write('c test\n')
+        open('b3/c', 'wb').write('c test\n')
+        other.add('c')
+
+        open('b1/b', 'wb').write('q test\n')
+        open('b1/d', 'wb').write('d test\n')
+        merge_inner(this.branch, other, base, this_tree=this)
+        self.assertNotEqual(open('b1/a', 'rb').read(), 'a test\n')
+        this.revert([])
+        self.assertEqual(open('b1/a', 'rb').read(), 'a test\n')
+        self.assertIs(os.path.exists('b1/b.~1~'), True)
+        self.assertIs(os.path.exists('b1/c'), False)
+        self.assertIs(os.path.exists('b1/a.~1~'), False)
+        self.assertIs(os.path.exists('b1/d'), True)
+
+    def test_update_updates_bound_branch_no_local_commits(self):
+        # doing an update in a tree updates the branch its bound to too.
+        master_tree = self.make_branch_and_tree('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # legacy branches cannot bind
+            return
+        master_tree.commit('foo', rev_id='foo', allow_pointless=True)
+        tree.update()
+        self.assertEqual('foo', tree.last_revision())
+        self.assertEqual('foo', tree.branch.last_revision())
+
+    def test_update_turns_local_commit_into_merge(self):
+        # doing an update with a few local commits and no master commits
+        # makes pending-merges. 
+        # this is done so that 'bzr update; bzr revert' will always produce
+        # an exact copy of the 'logical branch' - the referenced branch for
+        # a checkout, and the master for a bound branch.
+        # its possible that we should instead have 'bzr update' when there
+        # is nothing new on the master leave the current commits intact and
+        # alter 'revert' to revert to the master always. But for now, its
+        # good.
+        master_tree = self.make_branch_and_tree('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # legacy branches cannot bind
+            return
+        tree.commit('foo', rev_id='foo', allow_pointless=True, local=True)
+        tree.commit('bar', rev_id='bar', allow_pointless=True, local=True)
+        tree.update()
+        self.assertEqual(None, tree.last_revision())
+        self.assertEqual([], tree.branch.revision_history())
+        self.assertEqual(['bar'], tree.pending_merges())
+
+    def test_merge_modified(self):
+        tree = self.make_branch_and_tree('master')
+        tree._control_files.put('merge-hashes', StringIO('asdfasdf'))
+        self.assertRaises(errors.MergeModifiedFormatError, tree.merge_modified)
+
+    def test_conflicts(self):
+        from bzrlib.tests.test_conflicts import example_conflicts
+        tree = self.make_branch_and_tree('master')
+        try:
+            tree.set_conflicts(example_conflicts)
+        except UnsupportedOperation:
+            raise TestSkipped('set_conflicts not supported')
+            
+        tree2 = WorkingTree.open('master')
+        self.assertEqual(tree2.conflicts(), example_conflicts)
+        tree2._control_files.put('conflicts', StringIO(''))
+        self.assertRaises(errors.ConflictFormatError, 
+                          tree2.conflicts)
+        tree2._control_files.put('conflicts', StringIO('a'))
+        self.assertRaises(errors.ConflictFormatError, 
+                          tree2.conflicts)
+
+    def make_merge_conflicts(self):
+        from bzrlib.merge import merge_inner 
+        tree = self.make_branch_and_tree('mine')
+        file('mine/bloo', 'wb').write('one')
+        tree.add('bloo')
+        file('mine/blo', 'wb').write('on')
+        tree.add('blo')
+        tree.commit("blah", allow_pointless=False)
+        base = tree.basis_tree()
+        BzrDir.open("mine").sprout("other")
+        file('other/bloo', 'wb').write('two')
+        othertree = WorkingTree.open('other')
+        othertree.commit('blah', allow_pointless=False)
+        file('mine/bloo', 'wb').write('three')
+        tree.commit("blah", allow_pointless=False)
+        merge_inner(tree.branch, othertree, base, this_tree=tree)
+        return tree
+
+    def test_merge_conflicts(self):
+        tree = self.make_merge_conflicts()
+        self.assertEqual(len(tree.conflicts()), 1)
+
+    def test_clear_merge_conflicts(self):
+        from bzrlib.conflicts import ConflictList
+        tree = self.make_merge_conflicts()
+        self.assertEqual(len(tree.conflicts()), 1)
+        try:
+            tree.set_conflicts(ConflictList())
+        except UnsupportedOperation:
+            raise TestSkipped
+        self.assertEqual(tree.conflicts(), ConflictList())
+
+    def test_revert_clear_conflicts(self):
+        tree = self.make_merge_conflicts()
+        self.assertEqual(len(tree.conflicts()), 1)
+        tree.revert(["blo"])
+        self.assertEqual(len(tree.conflicts()), 1)
+        tree.revert(["bloo"])
+        self.assertEqual(len(tree.conflicts()), 0)
+
+    def test_revert_clear_conflicts2(self):
+        tree = self.make_merge_conflicts()
+        self.assertEqual(len(tree.conflicts()), 1)
+        tree.revert([])
+        self.assertEqual(len(tree.conflicts()), 0)
+
+    def test_format_description(self):
+        tree = self.make_branch_and_tree('tree')
+        text = tree._format.get_format_description()
+        self.failUnless(len(text))

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2005-04-11 02:44:45 +0000
+++ bzrlib/textinv.py	2005-10-03 05:54:35 +0000
@@ -15,8 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from errors import BzrError
-from inventory import InventoryEntry, Inventory
+from bzrlib.errors import BzrError
+from bzrlib.inventory import InventoryEntry, Inventory
 
 
 START_MARK = "# bzr inventory format 3\n"
@@ -69,8 +69,6 @@
     outf.write(END_MARK)
 
 
-
-
 def read_text_inventory(tf):
     """Return an inventory read in from tf"""
     if tf.readline() != START_MARK:

=== added file 'bzrlib/textmerge.py'
--- bzrlib/textmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/textmerge.py	2006-04-10 05:42:28 +0000
@@ -0,0 +1,145 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+# Author: Martin Pool <mbp@canonical.com> 
+#         Aaron Bentley <aaron.bentley@utoronto.ca>
+
+
+from difflib import SequenceMatcher
+
+
+class TextMerge(object):
+    """Base class for text-mergers
+    Subclasses must implement _merge_struct.
+
+    Many methods produce or consume structured merge information.
+    This is an iterable of tuples of lists of lines.
+    Each tuple may have a length of 1 - 3, depending on whether the region it
+    represents is conflicted.
+    
+    Unconflicted region tuples have length 1.
+    Conflicted region tuples have length 2 or 3.  Index 1 is text_a, e.g. THIS.
+    Index 1 is text_b, e.g. OTHER.  Index 2 is optional.  If present, it
+    represents BASE.
+    """
+    # TODO: Show some version information (e.g. author, date) on conflicted
+    # regions.
+    A_MARKER = '<<<<<<< \n'
+    B_MARKER = '>>>>>>> \n'
+    SPLIT_MARKER = '=======\n'
+    def __init__(self, a_marker=A_MARKER, b_marker=B_MARKER,
+                 split_marker=SPLIT_MARKER):
+        self.a_marker = a_marker
+        self.b_marker = b_marker
+        self.split_marker = split_marker
+
+    def _merge_struct(self):
+        """Return structured merge info.  Must be implemented by subclasses.
+        See TextMerge docstring for details on the format.
+        """
+        raise NotImplementedError('_merge_struct is abstract')
+
+    def struct_to_lines(self, struct_iter):
+        """Convert merge result tuples to lines"""
+        for lines in struct_iter:
+            if len(lines) == 1:
+                for line in lines[0]:
+                    yield line
+            else:
+                yield self.a_marker
+                for line in lines[0]: 
+                    yield line
+                yield self.split_marker
+                for line in lines[1]: 
+                    yield line
+                yield self.b_marker
+
+    def iter_useful(self, struct_iter):
+        """Iterate through input tuples, skipping empty ones."""
+        for group in struct_iter:
+            if len(group[0]) > 0:
+                yield group
+            elif len(group) > 1 and len(group[1]) > 0:
+                yield group
+
+    def merge_lines(self, reprocess=False):
+        """Produce an iterable of lines, suitable for writing to a file
+        Returns a tuple of (line iterable, conflict indicator)
+        If reprocess is True, a two-way merge will be performed on the
+        intermediate structure, to reduce conflict regions.
+        """
+        struct = []
+        conflicts = False
+        for group in self.merge_struct(reprocess):
+            struct.append(group)
+            if len(group) > 1:
+                conflicts = True
+        return self.struct_to_lines(struct), conflicts
+
+    def merge_struct(self, reprocess=False):
+        """Produce structured merge info"""
+        struct_iter = self.iter_useful(self._merge_struct())
+        if reprocess is True:
+            return self.reprocess_struct(struct_iter)
+        else:
+            return struct_iter
+
+    @staticmethod
+    def reprocess_struct(struct_iter):
+        """ Perform a two-way merge on structural merge info.
+        This reduces the size of conflict regions, but breaks the connection
+        between the BASE text and the conflict region.
+
+        This process may split a single conflict region into several smaller
+        ones, but will not introduce new conflicts.
+        """
+        for group in struct_iter:
+            if len(group) == 1:
+                yield group
+            else:
+                for newgroup in Merge2(group[0], group[1]).merge_struct():
+                    yield newgroup
+
+
+class Merge2(TextMerge):
+    """ Two-way merge.
+    In a two way merge, common regions are shown as unconflicting, and uncommon
+    regions produce conflicts.
+    """
+
+    def __init__(self, lines_a, lines_b, a_marker=TextMerge.A_MARKER, 
+                 b_marker=TextMerge.B_MARKER, 
+                 split_marker=TextMerge.SPLIT_MARKER):
+        TextMerge.__init__(self, a_marker, b_marker, split_marker)
+        self.lines_a = lines_a
+        self.lines_b = lines_b
+
+    def _merge_struct(self):
+        """Return structured merge info.  
+        See TextMerge docstring.
+        """
+        sm = SequenceMatcher(None, self.lines_a, self.lines_b)
+        pos_a = 0
+        pos_b = 0
+        for ai, bi, l in sm.get_matching_blocks():
+            # non-matching lines
+            yield(self.lines_a[pos_a:ai], self.lines_b[pos_b:bi])
+            # matching lines
+            yield(self.lines_a[ai:ai+l],)
+            pos_a = ai + l 
+            pos_b = bi + l
+        # final non-matching lines
+        yield(self.lines_a[pos_a:-1], self.lines_b[pos_b:-1])

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-07-20 17:31:25 +0000
+++ bzrlib/trace.py	2006-04-07 11:50:30 +0000
@@ -1,80 +1,110 @@
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-
-# TODO: Could probably replace this with something based on Python logging
-# module.
-
-
-
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
-"""Messages and logging for bazaar-ng
-
-Nothing is actually logged unless you call bzrlib.open_tracefile().
+# Copyright (C) 2005, Canonical Ltd
+
+"""Messages and logging for bazaar-ng.
+
+Messages are supplied by callers as a string-formatting template, plus values
+to be inserted into it.  The actual %-formatting is deferred to the log
+library so that it doesn't need to be done for messages that won't be emitted.
+
+Messages are classified by severity levels: critical, error, warning, info,
+and debug.
+
+They can be sent to two places: to stderr, and to ~/.bzr.log.  For purposes
+such as running the test suite, they can also be redirected away from both of
+those two places to another location.
+
+~/.bzr.log gets all messages, and full tracebacks for uncaught exceptions.
+This trace file is always in UTF-8, regardless of the user's default encoding,
+so that we can always rely on writing any message.
+
+Output to stderr depends on the mode chosen by the user.  By default, messages
+of info and above are sent out, which results in progress messages such as the
+list of files processed by add and commit.  In quiet mode, only warnings and
+above are shown.  In debug mode, stderr gets debug messages too.
+
+Errors that terminate an operation are generally passed back as exceptions;
+others may be just emitted as messages.
+
+Exceptions are reported in a brief form to stderr so as not to look scary.
+BzrErrors are required to be able to format themselves into a properly
+explanatory message.  This is not true for builtin excexceptions such as
+KeyError, which typically just str to "0".  They're printed in a different
+form.
 """
 
-
-import sys, os
-
-######################################################################
-# messages and logging
-
-global _tracefile, _starttime
-_tracefile = None
-
-# used to have % (os.environ['USER'], time.time(), os.getpid()), 'w')
-_starttime = None
-
-# If false, notes also go to stdout; should replace this with --silent
-# at some point.
-silent = False
-
-
-# fix this if we ever fork within python
-_mypid = os.getpid()
-_logprefix = '[%d] ' % _mypid
-
-
-def _write_trace(msg):
-    if 0:
-        if _tracefile:
-            _tracefile.write(_logprefix + msg + '\n')
-
-
-def warning(msg):
-    sys.stderr.write('bzr: warning: ' + msg + '\n')
-    _write_trace('warning: ' + msg)
-
-
-mutter = _write_trace
-
-
-def note(msg):
-    b = '* ' + str(msg) + '\n'
-    if not silent:
-        sys.stderr.write(b)
-    _write_trace('note: ' + msg)
-
-
-def log_error(msg):
-    sys.stderr.write(msg + '\n')
-    _write_trace(msg)
+# TODO: in debug mode, stderr should get full tracebacks and also
+# debug messages.  (Is this really needed?)
+
+# FIXME: Unfortunately it turns out that python's logging module
+# is quite expensive, even when the message is not printed by any handlers.
+# We should perhaps change back to just simply doing it here.
+
+
+import sys
+import os
+import logging
+
+import bzrlib
+from bzrlib.errors import BzrError, BzrNewError
+
+
+_file_handler = None
+_stderr_handler = None
+_stderr_quiet = False
+_trace_file = None
+_bzr_log_file = None
+
+
+class QuietFormatter(logging.Formatter):
+    """Formatter that supresses the details of errors.
+
+    This is used by default on stderr so as not to scare the user.
+    """
+    # At first I tried overriding formatException to suppress the
+    # exception details, but that has global effect: no loggers
+    # can get the exception details is we suppress them here.
+
+    def format(self, record):
+        if record.levelno >= logging.WARNING:
+            s = 'bzr: ' + record.levelname + ': '
+        else:
+            s = ''
+        s += record.getMessage()
+        if record.exc_info:
+            s += '\n' + format_exception_short(record.exc_info)
+        return s
+        
+# configure convenient aliases for output routines
+
+_bzr_logger = logging.getLogger('bzr')
+
+def note(*args, **kwargs):
+    import bzrlib.ui
+    bzrlib.ui.ui_factory.clear_term()
+    _bzr_logger.info(*args, **kwargs)
+
+def warning(*args, **kwargs):
+    import bzrlib.ui
+    bzrlib.ui.ui_factory.clear_term()
+    _bzr_logger.warning(*args, **kwargs)
+
+info = note
+log_error = _bzr_logger.error
+error =     _bzr_logger.error
+
+
+def mutter(fmt, *args):
+    if _trace_file is None:
+        return
+    if hasattr(_trace_file, 'closed') and _trace_file.closed:
+        return
+    if len(args) > 0:
+        out = fmt % args
+    else:
+        out = fmt
+    out += '\n'
+    _trace_file.write(out)
+debug = mutter
 
 
 def _rollover_trace_maybe(trace_fname):
@@ -84,68 +114,170 @@
         if size <= 4 << 20:
             return
         old_fname = trace_fname + '.old'
-
-        try:
-            # must remove before rename on windows
-            os.remove(old_fname)
-        except OSError:
-            pass
-
-        try:
-            # might fail if in use on windows
-            os.rename(trace_fname, old_fname)
-        except OSError:
-            pass
+        from osutils import rename
+        rename(trace_fname, old_fname)
     except OSError:
         return
 
 
-
-def open_tracefile(argv=[], tracefilename='~/.bzr.log'):
+def open_tracefile(tracefilename='~/.bzr.log'):
     # Messages are always written to here, so that we have some
     # information if something goes wrong.  In a future version this
     # file will be removed on successful completion.
-    global _starttime, _tracefile
+    global _file_handler, _bzr_log_file
     import stat, codecs
 
-    _starttime = os.times()[4]
-
     trace_fname = os.path.join(os.path.expanduser(tracefilename))
     _rollover_trace_maybe(trace_fname)
-
-    # buffering=1 means line buffered
     try:
-        _tracefile = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
-        t = _tracefile
-
-        if os.fstat(t.fileno())[stat.ST_SIZE] == 0:
-            t.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
-            t.write("you can delete or truncate this file, or include sections in\n")
-            t.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
-
-        import bzrlib
-        _write_trace('bzr %s invoked on python %s (%s)'
-                     % (bzrlib.__version__,
-                        '.'.join(map(str, sys.version_info)),
-                        sys.platform))
-
-        _write_trace('  arguments: %r' % argv)
-        _write_trace('  working dir: ' + os.getcwdu())
+        LINE_BUFFERED = 1
+        tf = codecs.open(trace_fname, 'at', 'utf8', buffering=LINE_BUFFERED)
+        _bzr_log_file = tf
+        if tf.tell() == 0:
+            tf.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
+            tf.write("you can delete or truncate this file, or include sections in\n")
+            tf.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
+        _file_handler = logging.StreamHandler(tf)
+        fmt = r'[%(process)5d] %(asctime)s.%(msecs)03d %(levelname)s: %(message)s'
+        datefmt = r'%a %H:%M:%S'
+        _file_handler.setFormatter(logging.Formatter(fmt, datefmt))
+        _file_handler.setLevel(logging.DEBUG)
+        logging.getLogger('').addHandler(_file_handler)
     except IOError, e:
         warning("failed to open trace file: %s" % (e))
 
-def close_trace():
-    times = os.times()
-    mutter("finished, %.3fu/%.3fs cpu, %.3fu/%.3fs cum, %.3f elapsed"
-           % (times[:4] + ((times[4] - _starttime),)))
-
-
-
-def log_exception():
-    """Log the last exception into the trace file."""
-    import cgitb
-    s = cgitb.text(sys.exc_info())
-    for l in s.split('\n'):
-        _write_trace(l)
-        
-    
+
+def log_startup(argv):
+    debug('\n\nbzr %s invoked on python %s (%s)',
+          bzrlib.__version__,
+          '.'.join(map(str, sys.version_info)),
+          sys.platform)
+    debug('  arguments: %r', argv)
+    debug('  working dir: %r', os.getcwdu())
+
+
+def log_exception(msg=None):
+    """Log the last exception to stderr and the trace file.
+
+    The exception string representation is used as the error
+    summary, unless msg is given.
+    """
+    if msg:
+        error(msg)
+    else:
+        exc_str = format_exception_short(sys.exc_info())
+        error(exc_str)
+    log_exception_quietly()
+
+
+def log_exception_quietly():
+    """Log the last exception to the trace file only.
+
+    Used for exceptions that occur internally and that may be 
+    interesting to developers but not to users.  For example, 
+    errors loading plugins.
+    """
+    import traceback
+    debug(traceback.format_exc())
+
+
+def enable_default_logging():
+    """Configure default logging to stderr and .bzr.log"""
+    # FIXME: if this is run twice, things get confused
+    global _stderr_handler, _file_handler, _trace_file, _bzr_log_file
+    _stderr_handler = logging.StreamHandler()
+    _stderr_handler.setFormatter(QuietFormatter())
+    logging.getLogger('').addHandler(_stderr_handler)
+    _stderr_handler.setLevel(logging.INFO)
+    if not _file_handler:
+        open_tracefile()
+    _trace_file = _bzr_log_file
+    if _file_handler:
+        _file_handler.setLevel(logging.DEBUG)
+    _bzr_logger.setLevel(logging.DEBUG) 
+
+
+
+def be_quiet(quiet=True):
+    global _stderr_handler, _stderr_quiet
+    
+    _stderr_quiet = quiet
+    if quiet:
+        _stderr_handler.setLevel(logging.WARNING)
+    else:
+        _stderr_handler.setLevel(logging.INFO)
+
+
+def is_quiet():
+    global _stderr_quiet
+    return _stderr_quiet
+
+
+def disable_default_logging():
+    """Turn off default log handlers.
+
+    This is intended to be used by the test framework, which doesn't
+    want leakage from the code-under-test into the main logs.
+    """
+
+    l = logging.getLogger('')
+    l.removeHandler(_stderr_handler)
+    if _file_handler:
+        l.removeHandler(_file_handler)
+    _trace_file = None
+
+
+def enable_test_log(to_file):
+    """Redirect logging to a temporary file for a test
+    
+    returns an opaque reference that should be passed to disable_test_log
+    after the test complete.
+    """
+    disable_default_logging()
+    global _trace_file
+    hdlr = logging.StreamHandler(to_file)
+    hdlr.setLevel(logging.DEBUG)
+    hdlr.setFormatter(logging.Formatter('%(levelname)8s  %(message)s'))
+    _bzr_logger.addHandler(hdlr)
+    _bzr_logger.setLevel(logging.DEBUG)
+    result = hdlr, _trace_file
+    _trace_file = to_file
+    return result
+
+
+def disable_test_log((test_log_hdlr, old_trace_file)):
+    _bzr_logger.removeHandler(test_log_hdlr)
+    test_log_hdlr.close()
+    _trace_file = old_trace_file
+    enable_default_logging()
+
+
+def format_exception_short(exc_info):
+    """Make a short string form of an exception.
+
+    This is used for display to stderr.  It specially handles exception
+    classes without useful string methods.
+
+    The result has no trailing newline.
+
+    exc_info - typically an exception from sys.exc_info()
+    """
+    exc_type, exc_object, exc_tb = exc_info
+    try:
+        if exc_type is None:
+            return '(no exception)'
+        if isinstance(exc_object, (BzrError, BzrNewError)):
+            return str(exc_object)
+        else:
+            import traceback
+            tb = traceback.extract_tb(exc_tb)
+            msg = '%s: %s' % (exc_type, exc_object)
+            if msg[-1] == '\n':
+                msg = msg[:-1]
+            if tb:
+                msg += '\n  at %s line %d\n  in %s' % (tb[-1][:3])
+            return msg
+    except Exception, formatting_exc:
+        # XXX: is this really better than just letting it run up?
+        return '(error formatting exception of type %s: %s)' \
+                % (exc_type, formatting_exc)

=== added file 'bzrlib/transactions.py'
--- bzrlib/transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transactions.py	2006-03-08 23:51:07 +0000
@@ -0,0 +1,196 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides a transactional facility.
+
+Transactions provide hooks to allow data objects (i.e. inventory weaves or
+the revision-history file) to be placed in a registry and retrieved later
+during the same transaction. Transactions in bzr are not atomic - they
+depend on data ordering of writes, so we do not have commit or rollback
+facilities at the transaction level.
+
+Read only transactions raise an assert when objects are listed as dirty
+against them - preventing unintended writes.
+
+Write transactions preserve dirty objects in the cache, though due to the
+write ordering approach we use for consistency 'dirty' is a misleading term.
+A dirty object is one we have modified.
+
+Both read and write transactions *may* flush unchanged objects out of 
+memory, unless they are marked as 'precious' which indicates that 
+repeated reads cannot be obtained if the object is ejected, or that
+the object is an expensive one for obtaining.
+"""
+
+import sys
+
+import bzrlib.errors as errors
+from bzrlib.identitymap import IdentityMap, NullIdentityMap
+from bzrlib.trace import mutter
+
+
+class ReadOnlyTransaction(object):
+    """A read only unit of work for data objects."""
+
+    def finish(self):
+        """Clean up this transaction."""
+
+    def __init__(self):
+        super(ReadOnlyTransaction, self).__init__()
+        self.map = IdentityMap()
+        self._clean_objects = set()
+        self._clean_queue = []
+        self._limit = -1
+        self._precious_objects = set()
+
+    def is_clean(self, an_object):
+        """Return True if an_object is clean."""
+        return an_object in self._clean_objects
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        If the precious hint is True, the object will not
+        be ejected from the object identity map ever.
+        """
+        self._clean_objects.add(an_object)
+        self._clean_queue.append(an_object)
+        if precious:
+            self._precious_objects.add(an_object)
+        self._trim()
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+        raise errors.ReadOnlyObjectDirtiedError(an_object)
+
+    def set_cache_size(self, size):
+        """Set a new cache size."""
+        assert -1 <= size
+        self._limit = size
+        self._trim()
+
+    def _trim(self):
+        """Trim the cache back if needed."""
+        if self._limit < 0 or self._limit - len(self._clean_objects) > 0:
+            return
+        needed = len(self._clean_objects) - self._limit
+        offset = 0
+        while needed and offset < len(self._clean_objects):
+            # references we know of:
+            # temp passed to getrefcount in our frame
+            # temp in getrefcount's frame
+            # the map forward
+            # the map backwards
+            # _clean_objects
+            # _clean_queue
+            # 1 missing ?
+            if (sys.getrefcount(self._clean_queue[offset]) <= 7 and
+                not self._clean_queue[offset] in self._precious_objects):
+                removed = self._clean_queue[offset]
+                self._clean_objects.remove(removed)
+                del self._clean_queue[offset]
+                self.map.remove_object(removed)
+                mutter('removed object %r', removed)
+                needed -= 1
+            else:
+                offset += 1
+
+    def writeable(self):
+        """Read only transactions do not allow writes."""
+
+
+class WriteTransaction(ReadOnlyTransaction):
+    """A write transaction
+
+    - caches domain objects
+    - clean objects can be removed from the cache
+    - dirty objects are retained.
+    """
+
+    def finish(self):
+        """Clean up this transaction."""
+        for thing in self._dirty_objects:
+            callback = getattr(thing, 'transaction_finished', None)
+            if callback is not None:
+                callback()
+
+    def __init__(self):
+        super(WriteTransaction, self).__init__()
+        self._dirty_objects = set()
+
+    def is_dirty(self, an_object):
+        """Return True if an_object is dirty."""
+        return an_object in self._dirty_objects
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty.
+        
+        Dirty objects are not ejected from the identity map
+        until the transaction finishes and get informed
+        when the transaction finishes.
+        """
+        self._dirty_objects.add(an_object)
+        if self.is_clean(an_object):
+            self._clean_objects.remove(an_object)
+            del self._clean_queue[self._clean_queue.index(an_object)]
+        self._trim()
+
+    def writeable(self):
+        """Write transactions allow writes."""
+        return True
+
+        
+class PassThroughTransaction(object):
+    """A pass through transaction
+    
+    - nothing is cached.
+    - nothing ever gets into the identity map.
+    """
+
+    def finish(self):
+        """Clean up this transaction."""
+        for thing in self._dirty_objects:
+            callback = getattr(thing, 'transaction_finished', None)
+            if callback is not None:
+                callback()
+
+    def __init__(self):
+        super(PassThroughTransaction, self).__init__()
+        self.map = NullIdentityMap()
+        self._dirty_objects = set()
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        Note that precious is only a hint, and PassThroughTransaction
+        ignores it.
+        """
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty.
+        
+        Dirty objects get informed
+        when the transaction finishes.
+        """
+        self._dirty_objects.add(an_object)
+
+    def set_cache_size(self, ignored):
+        """Do nothing, we are passing through."""
+
+    def writeable(self):
+        """Pass through transactions allow writes."""
+        return True

=== added file 'bzrlib/transform.py'
--- bzrlib/transform.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transform.py	2006-04-18 22:41:16 +0000
@@ -0,0 +1,1226 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import errno
+from stat import S_ISREG
+
+from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
+                           ReusingTransform, NotVersionedError, CantMoveRoot,
+                           ExistingLimbo, ImmortalLimbo)
+from bzrlib.inventory import InventoryEntry
+from bzrlib.osutils import (file_kind, supports_executable, pathjoin, lexists,
+                            delete_any)
+from bzrlib.progress import DummyProgress, ProgressPhase
+from bzrlib.trace import mutter, warning
+import bzrlib.ui 
+
+
+ROOT_PARENT = "root-parent"
+
+
+def unique_add(map, key, value):
+    if key in map:
+        raise DuplicateKey(key=key)
+    map[key] = value
+
+
+class _TransformResults(object):
+    def __init__(self, modified_paths):
+        object.__init__(self)
+        self.modified_paths = modified_paths
+
+
+class TreeTransform(object):
+    """Represent a tree transformation.
+    
+    This object is designed to support incremental generation of the transform,
+    in any order.  
+    
+    It is easy to produce malformed transforms, but they are generally
+    harmless.  Attempting to apply a malformed transform will cause an
+    exception to be raised before any modifications are made to the tree.  
+
+    Many kinds of malformed transforms can be corrected with the 
+    resolve_conflicts function.  The remaining ones indicate programming error,
+    such as trying to create a file with no path.
+
+    Two sets of file creation methods are supplied.  Convenience methods are:
+     * new_file
+     * new_directory
+     * new_symlink
+
+    These are composed of the low-level methods:
+     * create_path
+     * create_file or create_directory or create_symlink
+     * version_file
+     * set_executability
+    """
+    def __init__(self, tree, pb=DummyProgress()):
+        """Note: a write lock is taken on the tree.
+        
+        Use TreeTransform.finalize() to release the lock
+        """
+        object.__init__(self)
+        self._tree = tree
+        self._tree.lock_write()
+        try:
+            control_files = self._tree._control_files
+            self._limbodir = control_files.controlfilename('limbo')
+            try:
+                os.mkdir(self._limbodir)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    raise ExistingLimbo(self._limbodir)
+        except: 
+            self._tree.unlock()
+            raise
+
+        self._id_number = 0
+        self._new_name = {}
+        self._new_parent = {}
+        self._new_contents = {}
+        self._removed_contents = set()
+        self._new_executability = {}
+        self._new_id = {}
+        self._non_present_ids = {}
+        self._r_new_id = {}
+        self._removed_id = set()
+        self._tree_path_ids = {}
+        self._tree_id_paths = {}
+        self._new_root = self.trans_id_tree_file_id(tree.get_root_id())
+        self.__done = False
+        self._pb = pb
+
+    def __get_root(self):
+        return self._new_root
+
+    root = property(__get_root)
+
+    def finalize(self):
+        """Release the working tree lock, if held, clean up limbo dir."""
+        if self._tree is None:
+            return
+        try:
+            for trans_id, kind in self._new_contents.iteritems():
+                path = self._limbo_name(trans_id)
+                if kind == "directory":
+                    os.rmdir(path)
+                else:
+                    os.unlink(path)
+            try:
+                os.rmdir(self._limbodir)
+            except OSError:
+                # We don't especially care *why* the dir is immortal.
+                raise ImmortalLimbo(self._limbodir)
+        finally:
+            self._tree.unlock()
+            self._tree = None
+
+    def _assign_id(self):
+        """Produce a new tranform id"""
+        new_id = "new-%s" % self._id_number
+        self._id_number +=1
+        return new_id
+
+    def create_path(self, name, parent):
+        """Assign a transaction id to a new path"""
+        trans_id = self._assign_id()
+        unique_add(self._new_name, trans_id, name)
+        unique_add(self._new_parent, trans_id, parent)
+        return trans_id
+
+    def adjust_path(self, name, parent, trans_id):
+        """Change the path that is assigned to a transaction id."""
+        if trans_id == self._new_root:
+            raise CantMoveRoot
+        self._new_name[trans_id] = name
+        self._new_parent[trans_id] = parent
+
+    def adjust_root_path(self, name, parent):
+        """Emulate moving the root by moving all children, instead.
+        
+        We do this by undoing the association of root's transaction id with the
+        current tree.  This allows us to create a new directory with that
+        transaction id.  We unversion the root directory and version the 
+        physically new directory, and hope someone versions the tree root
+        later.
+        """
+        old_root = self._new_root
+        old_root_file_id = self.final_file_id(old_root)
+        # force moving all children of root
+        for child_id in self.iter_tree_children(old_root):
+            if child_id != parent:
+                self.adjust_path(self.final_name(child_id), 
+                                 self.final_parent(child_id), child_id)
+            file_id = self.final_file_id(child_id)
+            if file_id is not None:
+                self.unversion_file(child_id)
+            self.version_file(file_id, child_id)
+        
+        # the physical root needs a new transaction id
+        self._tree_path_ids.pop("")
+        self._tree_id_paths.pop(old_root)
+        self._new_root = self.trans_id_tree_file_id(self._tree.get_root_id())
+        if parent == old_root:
+            parent = self._new_root
+        self.adjust_path(name, parent, old_root)
+        self.create_directory(old_root)
+        self.version_file(old_root_file_id, old_root)
+        self.unversion_file(self._new_root)
+
+    def trans_id_tree_file_id(self, inventory_id):
+        """Determine the transaction id of a working tree file.
+        
+        This reflects only files that already exist, not ones that will be
+        added by transactions.
+        """
+        path = self._tree.inventory.id2path(inventory_id)
+        return self.trans_id_tree_path(path)
+
+    def trans_id_file_id(self, file_id):
+        """Determine or set the transaction id associated with a file ID.
+        A new id is only created for file_ids that were never present.  If
+        a transaction has been unversioned, it is deliberately still returned.
+        (this will likely lead to an unversioned parent conflict.)
+        """
+        if file_id in self._r_new_id and self._r_new_id[file_id] is not None:
+            return self._r_new_id[file_id]
+        elif file_id in self._tree.inventory:
+            return self.trans_id_tree_file_id(file_id)
+        elif file_id in self._non_present_ids:
+            return self._non_present_ids[file_id]
+        else:
+            trans_id = self._assign_id()
+            self._non_present_ids[file_id] = trans_id
+            return trans_id
+
+    def canonical_path(self, path):
+        """Get the canonical tree-relative path"""
+        # don't follow final symlinks
+        dirname, basename = os.path.split(self._tree.abspath(path))
+        dirname = os.path.realpath(dirname)
+        return self._tree.relpath(pathjoin(dirname, basename))
+
+    def trans_id_tree_path(self, path):
+        """Determine (and maybe set) the transaction ID for a tree path."""
+        path = self.canonical_path(path)
+        if path not in self._tree_path_ids:
+            self._tree_path_ids[path] = self._assign_id()
+            self._tree_id_paths[self._tree_path_ids[path]] = path
+        return self._tree_path_ids[path]
+
+    def get_tree_parent(self, trans_id):
+        """Determine id of the parent in the tree."""
+        path = self._tree_id_paths[trans_id]
+        if path == "":
+            return ROOT_PARENT
+        return self.trans_id_tree_path(os.path.dirname(path))
+
+    def create_file(self, contents, trans_id, mode_id=None):
+        """Schedule creation of a new file.
+
+        See also new_file.
+        
+        Contents is an iterator of strings, all of which will be written
+        to the target destination.
+
+        New file takes the permissions of any existing file with that id,
+        unless mode_id is specified.
+        """
+        f = file(self._limbo_name(trans_id), 'wb')
+        unique_add(self._new_contents, trans_id, 'file')
+        for segment in contents:
+            f.write(segment)
+        f.close()
+        self._set_mode(trans_id, mode_id, S_ISREG)
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        """Set the mode of new file contents.
+        The mode_id is the existing file to get the mode from (often the same
+        as trans_id).  The operation is only performed if there's a mode match
+        according to typefunc.
+        """
+        if mode_id is None:
+            mode_id = trans_id
+        try:
+            old_path = self._tree_id_paths[mode_id]
+        except KeyError:
+            return
+        try:
+            mode = os.stat(old_path).st_mode
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                return
+            else:
+                raise
+        if typefunc(mode):
+            os.chmod(self._limbo_name(trans_id), mode)
+
+    def create_directory(self, trans_id):
+        """Schedule creation of a new directory.
+        
+        See also new_directory.
+        """
+        os.mkdir(self._limbo_name(trans_id))
+        unique_add(self._new_contents, trans_id, 'directory')
+
+    def create_symlink(self, target, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        os.symlink(target, self._limbo_name(trans_id))
+        unique_add(self._new_contents, trans_id, 'symlink')
+
+    def cancel_creation(self, trans_id):
+        """Cancel the creation of new file contents."""
+        del self._new_contents[trans_id]
+        delete_any(self._limbo_name(trans_id))
+
+    def delete_contents(self, trans_id):
+        """Schedule the contents of a path entry for deletion"""
+        self.tree_kind(trans_id)
+        self._removed_contents.add(trans_id)
+
+    def cancel_deletion(self, trans_id):
+        """Cancel a scheduled deletion"""
+        self._removed_contents.remove(trans_id)
+
+    def unversion_file(self, trans_id):
+        """Schedule a path entry to become unversioned"""
+        self._removed_id.add(trans_id)
+
+    def delete_versioned(self, trans_id):
+        """Delete and unversion a versioned file"""
+        self.delete_contents(trans_id)
+        self.unversion_file(trans_id)
+
+    def set_executability(self, executability, trans_id):
+        """Schedule setting of the 'execute' bit
+        To unschedule, set to None
+        """
+        if executability is None:
+            del self._new_executability[trans_id]
+        else:
+            unique_add(self._new_executability, trans_id, executability)
+
+    def version_file(self, file_id, trans_id):
+        """Schedule a file to become versioned."""
+        assert file_id is not None
+        unique_add(self._new_id, trans_id, file_id)
+        unique_add(self._r_new_id, file_id, trans_id)
+
+    def cancel_versioning(self, trans_id):
+        """Undo a previous versioning of a file"""
+        file_id = self._new_id[trans_id]
+        del self._new_id[trans_id]
+        del self._r_new_id[file_id]
+
+    def new_paths(self):
+        """Determine the paths of all new and changed files"""
+        new_ids = set()
+        fp = FinalPaths(self)
+        for id_set in (self._new_name, self._new_parent, self._new_contents,
+                       self._new_id, self._new_executability):
+            new_ids.update(id_set)
+        new_paths = [(fp.get_path(t), t) for t in new_ids]
+        new_paths.sort()
+        return new_paths
+
+    def tree_kind(self, trans_id):
+        """Determine the file kind in the working tree.
+
+        Raises NoSuchFile if the file does not exist
+        """
+        path = self._tree_id_paths.get(trans_id)
+        if path is None:
+            raise NoSuchFile(None)
+        try:
+            return file_kind(self._tree.abspath(path))
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+            else:
+                raise NoSuchFile(path)
+
+    def final_kind(self, trans_id):
+        """Determine the final file kind, after any changes applied.
+        
+        Raises NoSuchFile if the file does not exist/has no contents.
+        (It is conceivable that a path would be created without the
+        corresponding contents insertion command)
+        """
+        if trans_id in self._new_contents:
+            return self._new_contents[trans_id]
+        elif trans_id in self._removed_contents:
+            raise NoSuchFile(None)
+        else:
+            return self.tree_kind(trans_id)
+
+    def tree_file_id(self, trans_id):
+        """Determine the file id associated with the trans_id in the tree"""
+        try:
+            path = self._tree_id_paths[trans_id]
+        except KeyError:
+            # the file is a new, unversioned file, or invalid trans_id
+            return None
+        # the file is old; the old id is still valid
+        if self._new_root == trans_id:
+            return self._tree.inventory.root.file_id
+        return self._tree.inventory.path2id(path)
+
+    def final_file_id(self, trans_id):
+        """Determine the file id after any changes are applied, or None.
+        
+        None indicates that the file will not be versioned after changes are
+        applied.
+        """
+        try:
+            # there is a new id for this file
+            assert self._new_id[trans_id] is not None
+            return self._new_id[trans_id]
+        except KeyError:
+            if trans_id in self._removed_id:
+                return None
+        return self.tree_file_id(trans_id)
+
+    def inactive_file_id(self, trans_id):
+        """Return the inactive file_id associated with a transaction id.
+        That is, the one in the tree or in non_present_ids.
+        The file_id may actually be active, too.
+        """
+        file_id = self.tree_file_id(trans_id)
+        if file_id is not None:
+            return file_id
+        for key, value in self._non_present_ids.iteritems():
+            if value == trans_id:
+                return key
+
+    def final_parent(self, trans_id):
+        """Determine the parent file_id, after any changes are applied.
+
+        ROOT_PARENT is returned for the tree root.
+        """
+        try:
+            return self._new_parent[trans_id]
+        except KeyError:
+            return self.get_tree_parent(trans_id)
+
+    def final_name(self, trans_id):
+        """Determine the final filename, after all changes are applied."""
+        try:
+            return self._new_name[trans_id]
+        except KeyError:
+            return os.path.basename(self._tree_id_paths[trans_id])
+
+    def by_parent(self):
+        """Return a map of parent: children for known parents.
+        
+        Only new paths and parents of tree files with assigned ids are used.
+        """
+        by_parent = {}
+        items = list(self._new_parent.iteritems())
+        items.extend((t, self.final_parent(t)) for t in 
+                      self._tree_id_paths.keys())
+        for trans_id, parent_id in items:
+            if parent_id not in by_parent:
+                by_parent[parent_id] = set()
+            by_parent[parent_id].add(trans_id)
+        return by_parent
+
+    def path_changed(self, trans_id):
+        """Return True if a trans_id's path has changed."""
+        return trans_id in self._new_name or trans_id in self._new_parent
+
+    def find_conflicts(self):
+        """Find any violations of inventory or filesystem invariants"""
+        if self.__done is True:
+            raise ReusingTransform()
+        conflicts = []
+        # ensure all children of all existent parents are known
+        # all children of non-existent parents are known, by definition.
+        self._add_tree_children()
+        by_parent = self.by_parent()
+        conflicts.extend(self._unversioned_parents(by_parent))
+        conflicts.extend(self._parent_loops())
+        conflicts.extend(self._duplicate_entries(by_parent))
+        conflicts.extend(self._duplicate_ids())
+        conflicts.extend(self._parent_type_conflicts(by_parent))
+        conflicts.extend(self._improper_versioning())
+        conflicts.extend(self._executability_conflicts())
+        conflicts.extend(self._overwrite_conflicts())
+        return conflicts
+
+    def _add_tree_children(self):
+        """Add all the children of all active parents to the known paths.
+
+        Active parents are those which gain children, and those which are
+        removed.  This is a necessary first step in detecting conflicts.
+        """
+        parents = self.by_parent().keys()
+        parents.extend([t for t in self._removed_contents if 
+                        self.tree_kind(t) == 'directory'])
+        for trans_id in self._removed_id:
+            file_id = self.tree_file_id(trans_id)
+            if self._tree.inventory[file_id].kind in ('directory', 
+                                                      'root_directory'):
+                parents.append(trans_id)
+
+        for parent_id in parents:
+            # ensure that all children are registered with the transaction
+            list(self.iter_tree_children(parent_id))
+
+    def iter_tree_children(self, parent_id):
+        """Iterate through the entry's tree children, if any"""
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return
+        try:
+            children = os.listdir(self._tree.abspath(path))
+        except OSError, e:
+            if e.errno != errno.ENOENT and e.errno != errno.ESRCH:
+                raise
+            return
+            
+        for child in children:
+            childpath = joinpath(path, child)
+            if self._tree.is_control_filename(childpath):
+                continue
+            yield self.trans_id_tree_path(childpath)
+
+    def has_named_child(self, by_parent, parent_id, name):
+        try:
+            children = by_parent[parent_id]
+        except KeyError:
+            children = []
+        for child in children:
+            if self.final_name(child) == name:
+                return True
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return False
+        childpath = joinpath(path, name)
+        child_id = self._tree_path_ids.get(childpath)
+        if child_id is None:
+            return lexists(self._tree.abspath(childpath))
+        else:
+            if tt.final_parent(child_id) != parent_id:
+                return False
+            if child_id in tt._removed_contents:
+                # XXX What about dangling file-ids?
+                return False
+            else:
+                return True
+
+    def _parent_loops(self):
+        """No entry should be its own ancestor"""
+        conflicts = []
+        for trans_id in self._new_parent:
+            seen = set()
+            parent_id = trans_id
+            while parent_id is not ROOT_PARENT:
+                seen.add(parent_id)
+                parent_id = self.final_parent(parent_id)
+                if parent_id == trans_id:
+                    conflicts.append(('parent loop', trans_id))
+                if parent_id in seen:
+                    break
+        return conflicts
+
+    def _unversioned_parents(self, by_parent):
+        """If parent directories are versioned, children must be versioned."""
+        conflicts = []
+        for parent_id, children in by_parent.iteritems():
+            if parent_id is ROOT_PARENT:
+                continue
+            if self.final_file_id(parent_id) is not None:
+                continue
+            for child_id in children:
+                if self.final_file_id(child_id) is not None:
+                    conflicts.append(('unversioned parent', parent_id))
+                    break;
+        return conflicts
+
+    def _improper_versioning(self):
+        """Cannot version a file with no contents, or a bad type.
+        
+        However, existing entries with no contents are okay.
+        """
+        conflicts = []
+        for trans_id in self._new_id.iterkeys():
+            try:
+                kind = self.final_kind(trans_id)
+            except NoSuchFile:
+                conflicts.append(('versioning no contents', trans_id))
+                continue
+            if not InventoryEntry.versionable_kind(kind):
+                conflicts.append(('versioning bad kind', trans_id, kind))
+        return conflicts
+
+    def _executability_conflicts(self):
+        """Check for bad executability changes.
+        
+        Only versioned files may have their executability set, because
+        1. only versioned entries can have executability under windows
+        2. only files can be executable.  (The execute bit on a directory
+           does not indicate searchability)
+        """
+        conflicts = []
+        for trans_id in self._new_executability:
+            if self.final_file_id(trans_id) is None:
+                conflicts.append(('unversioned executability', trans_id))
+            else:
+                try:
+                    non_file = self.final_kind(trans_id) != "file"
+                except NoSuchFile:
+                    non_file = True
+                if non_file is True:
+                    conflicts.append(('non-file executability', trans_id))
+        return conflicts
+
+    def _overwrite_conflicts(self):
+        """Check for overwrites (not permitted on Win32)"""
+        conflicts = []
+        for trans_id in self._new_contents:
+            try:
+                self.tree_kind(trans_id)
+            except NoSuchFile:
+                continue
+            if trans_id not in self._removed_contents:
+                conflicts.append(('overwrite', trans_id,
+                                 self.final_name(trans_id)))
+        return conflicts
+
+    def _duplicate_entries(self, by_parent):
+        """No directory may have two entries with the same name."""
+        conflicts = []
+        for children in by_parent.itervalues():
+            name_ids = [(self.final_name(t), t) for t in children]
+            name_ids.sort()
+            last_name = None
+            last_trans_id = None
+            for name, trans_id in name_ids:
+                if name == last_name:
+                    conflicts.append(('duplicate', last_trans_id, trans_id,
+                    name))
+                try:
+                    kind = self.final_kind(trans_id)
+                except NoSuchFile:
+                    kind = None
+                file_id = self.final_file_id(trans_id)
+                if kind is not None or file_id is not None:
+                    last_name = name
+                    last_trans_id = trans_id
+        return conflicts
+
+    def _duplicate_ids(self):
+        """Each inventory id may only be used once"""
+        conflicts = []
+        removed_tree_ids = set((self.tree_file_id(trans_id) for trans_id in
+                                self._removed_id))
+        active_tree_ids = set((f for f in self._tree.inventory if
+                               f not in removed_tree_ids))
+        for trans_id, file_id in self._new_id.iteritems():
+            if file_id in active_tree_ids:
+                old_trans_id = self.trans_id_tree_file_id(file_id)
+                conflicts.append(('duplicate id', old_trans_id, trans_id))
+        return conflicts
+
+    def _parent_type_conflicts(self, by_parent):
+        """parents must have directory 'contents'."""
+        conflicts = []
+        for parent_id, children in by_parent.iteritems():
+            if parent_id is ROOT_PARENT:
+                continue
+            if not self._any_contents(children):
+                continue
+            for child in children:
+                try:
+                    self.final_kind(child)
+                except NoSuchFile:
+                    continue
+            try:
+                kind = self.final_kind(parent_id)
+            except NoSuchFile:
+                kind = None
+            if kind is None:
+                conflicts.append(('missing parent', parent_id))
+            elif kind != "directory":
+                conflicts.append(('non-directory parent', parent_id))
+        return conflicts
+
+    def _any_contents(self, trans_ids):
+        """Return true if any of the trans_ids, will have contents."""
+        for trans_id in trans_ids:
+            try:
+                kind = self.final_kind(trans_id)
+            except NoSuchFile:
+                continue
+            return True
+        return False
+            
+    def apply(self):
+        """Apply all changes to the inventory and filesystem.
+        
+        If filesystem or inventory conflicts are present, MalformedTransform
+        will be thrown.
+        """
+        conflicts = self.find_conflicts()
+        if len(conflicts) != 0:
+            raise MalformedTransform(conflicts=conflicts)
+        limbo_inv = {}
+        inv = self._tree.inventory
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            child_pb.update('Apply phase', 0, 2)
+            self._apply_removals(inv, limbo_inv)
+            child_pb.update('Apply phase', 1, 2)
+            modified_paths = self._apply_insertions(inv, limbo_inv)
+        finally:
+            child_pb.finished()
+        self._tree._write_inventory(inv)
+        self.__done = True
+        self.finalize()
+        return _TransformResults(modified_paths)
+
+    def _limbo_name(self, trans_id):
+        """Generate the limbo name of a file"""
+        return pathjoin(self._limbodir, trans_id)
+
+    def _apply_removals(self, inv, limbo_inv):
+        """Perform tree operations that remove directory/inventory names.
+        
+        That is, delete files that are to be deleted, and put any files that
+        need renaming into limbo.  This must be done in strict child-to-parent
+        order.
+        """
+        tree_paths = list(self._tree_path_ids.iteritems())
+        tree_paths.sort(reverse=True)
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            for num, data in enumerate(tree_paths):
+                path, trans_id = data
+                child_pb.update('removing file', num, len(tree_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._removed_contents:
+                    delete_any(full_path)
+                elif trans_id in self._new_name or trans_id in \
+                    self._new_parent:
+                    try:
+                        os.rename(full_path, self._limbo_name(trans_id))
+                    except OSError, e:
+                        if e.errno != errno.ENOENT:
+                            raise
+                if trans_id in self._removed_id:
+                    if trans_id == self._new_root:
+                        file_id = self._tree.inventory.root.file_id
+                    else:
+                        file_id = self.tree_file_id(trans_id)
+                    del inv[file_id]
+                elif trans_id in self._new_name or trans_id in self._new_parent:
+                    file_id = self.tree_file_id(trans_id)
+                    if file_id is not None:
+                        limbo_inv[trans_id] = inv[file_id]
+                        del inv[file_id]
+        finally:
+            child_pb.finished()
+
+    def _apply_insertions(self, inv, limbo_inv):
+        """Perform tree operations that insert directory/inventory names.
+        
+        That is, create any files that need to be created, and restore from
+        limbo any files that needed renaming.  This must be done in strict
+        parent-to-child order.
+        """
+        new_paths = self.new_paths()
+        modified_paths = []
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            for num, (path, trans_id) in enumerate(new_paths):
+                child_pb.update('adding file', num, len(new_paths))
+                try:
+                    kind = self._new_contents[trans_id]
+                except KeyError:
+                    kind = contents = None
+                if trans_id in self._new_contents or \
+                    self.path_changed(trans_id):
+                    full_path = self._tree.abspath(path)
+                    try:
+                        os.rename(self._limbo_name(trans_id), full_path)
+                    except OSError, e:
+                        # We may be renaming a dangling inventory id
+                        if e.errno != errno.ENOENT:
+                            raise
+                    if trans_id in self._new_contents:
+                        modified_paths.append(full_path)
+                        del self._new_contents[trans_id]
+
+                if trans_id in self._new_id:
+                    if kind is None:
+                        kind = file_kind(self._tree.abspath(path))
+                    inv.add_path(path, kind, self._new_id[trans_id])
+                elif trans_id in self._new_name or trans_id in\
+                    self._new_parent:
+                    entry = limbo_inv.get(trans_id)
+                    if entry is not None:
+                        entry.name = self.final_name(trans_id)
+                        parent_path = os.path.dirname(path)
+                        entry.parent_id = \
+                            self._tree.inventory.path2id(parent_path)
+                        inv.add(entry)
+
+                # requires files and inventory entries to be in place
+                if trans_id in self._new_executability:
+                    self._set_executability(path, inv, trans_id)
+        finally:
+            child_pb.finished()
+        return modified_paths
+
+    def _set_executability(self, path, inv, trans_id):
+        """Set the executability of versioned files """
+        file_id = inv.path2id(path)
+        new_executability = self._new_executability[trans_id]
+        inv[file_id].executable = new_executability
+        if supports_executable():
+            abspath = self._tree.abspath(path)
+            current_mode = os.stat(abspath).st_mode
+            if new_executability:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0004:
+                    to_mode |= 0001 & ~umask
+                if current_mode & 0040:
+                    to_mode |= 0010 & ~umask
+            else:
+                to_mode = current_mode & ~0111
+            os.chmod(abspath, to_mode)
+
+    def _new_entry(self, name, parent_id, file_id):
+        """Helper function to create a new filesystem entry."""
+        trans_id = self.create_path(name, parent_id)
+        if file_id is not None:
+            self.version_file(file_id, trans_id)
+        return trans_id
+
+    def new_file(self, name, parent_id, contents, file_id=None, 
+                 executable=None):
+        """Convenience method to create files.
+        
+        name is the name of the file to create.
+        parent_id is the transaction id of the parent directory of the file.
+        contents is an iterator of bytestrings, which will be used to produce
+        the file.
+        file_id is the inventory ID of the file, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_file(contents, trans_id)
+        if executable is not None:
+            self.set_executability(executable, trans_id)
+        return trans_id
+
+    def new_directory(self, name, parent_id, file_id=None):
+        """Convenience method to create directories.
+
+        name is the name of the directory to create.
+        parent_id is the transaction id of the parent directory of the
+        directory.
+        file_id is the inventory ID of the directory, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_directory(trans_id)
+        return trans_id 
+
+    def new_symlink(self, name, parent_id, target, file_id=None):
+        """Convenience method to create symbolic link.
+        
+        name is the name of the symlink to create.
+        parent_id is the transaction id of the parent directory of the symlink.
+        target is a bytestring of the target of the symlink.
+        file_id is the inventory ID of the file, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_symlink(target, trans_id)
+        return trans_id
+
+def joinpath(parent, child):
+    """Join tree-relative paths, handling the tree root specially"""
+    if parent is None or parent == "":
+        return child
+    else:
+        return pathjoin(parent, child)
+
+
+class FinalPaths(object):
+    """Make path calculation cheap by memoizing paths.
+
+    The underlying tree must not be manipulated between calls, or else
+    the results will likely be incorrect.
+    """
+    def __init__(self, transform):
+        object.__init__(self)
+        self._known_paths = {}
+        self.transform = transform
+
+    def _determine_path(self, trans_id):
+        if trans_id == self.transform.root:
+            return ""
+        name = self.transform.final_name(trans_id)
+        parent_id = self.transform.final_parent(trans_id)
+        if parent_id == self.transform.root:
+            return name
+        else:
+            return pathjoin(self.get_path(parent_id), name)
+
+    def get_path(self, trans_id):
+        """Find the final path associated with a trans_id"""
+        if trans_id not in self._known_paths:
+            self._known_paths[trans_id] = self._determine_path(trans_id)
+        return self._known_paths[trans_id]
+
+def topology_sorted_ids(tree):
+    """Determine the topological order of the ids in a tree"""
+    file_ids = list(tree)
+    file_ids.sort(key=tree.id2path)
+    return file_ids
+
+def build_tree(tree, wt):
+    """Create working tree for a branch, using a Transaction."""
+    file_trans_id = {}
+    top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    pp = ProgressPhase("Build phase", 2, top_pb)
+    tt = TreeTransform(wt)
+    try:
+        pp.next_phase()
+        file_trans_id[wt.get_root_id()] = tt.trans_id_tree_file_id(wt.get_root_id())
+        file_ids = topology_sorted_ids(tree)
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            for num, file_id in enumerate(file_ids):
+                pb.update("Building tree", num, len(file_ids))
+                entry = tree.inventory[file_id]
+                if entry.parent_id is None:
+                    continue
+                if entry.parent_id not in file_trans_id:
+                    raise repr(entry.parent_id)
+                parent_id = file_trans_id[entry.parent_id]
+                file_trans_id[file_id] = new_by_entry(tt, entry, parent_id, 
+                                                      tree)
+        finally:
+            pb.finished()
+        pp.next_phase()
+        tt.apply()
+    finally:
+        tt.finalize()
+        top_pb.finished()
+
+def new_by_entry(tt, entry, parent_id, tree):
+    """Create a new file according to its inventory entry"""
+    name = entry.name
+    kind = entry.kind
+    if kind == 'file':
+        contents = tree.get_file(entry.file_id).readlines()
+        executable = tree.is_executable(entry.file_id)
+        return tt.new_file(name, parent_id, contents, entry.file_id, 
+                           executable)
+    elif kind == 'directory':
+        return tt.new_directory(name, parent_id, entry.file_id)
+    elif kind == 'symlink':
+        target = tree.get_symlink_target(entry.file_id)
+        return tt.new_symlink(name, parent_id, target, entry.file_id)
+
+def create_by_entry(tt, entry, tree, trans_id, lines=None, mode_id=None):
+    """Create new file contents according to an inventory entry."""
+    if entry.kind == "file":
+        if lines == None:
+            lines = tree.get_file(entry.file_id).readlines()
+        tt.create_file(lines, trans_id, mode_id=mode_id)
+    elif entry.kind == "symlink":
+        tt.create_symlink(tree.get_symlink_target(entry.file_id), trans_id)
+    elif entry.kind == "directory":
+        tt.create_directory(trans_id)
+
+def create_entry_executability(tt, entry, trans_id):
+    """Set the executability of a trans_id according to an inventory entry"""
+    if entry.kind == "file":
+        tt.set_executability(entry.executable, trans_id)
+
+
+def find_interesting(working_tree, target_tree, filenames):
+    """Find the ids corresponding to specified filenames."""
+    if not filenames:
+        interesting_ids = None
+    else:
+        interesting_ids = set()
+        for tree_path in filenames:
+            not_found = True
+            for tree in (working_tree, target_tree):
+                file_id = tree.inventory.path2id(tree_path)
+                if file_id is not None:
+                    interesting_ids.add(file_id)
+                    not_found = False
+            if not_found:
+                raise NotVersionedError(path=tree_path)
+    return interesting_ids
+
+
+def change_entry(tt, file_id, working_tree, target_tree, 
+                 trans_id_file_id, backups, trans_id, by_parent):
+    """Replace a file_id's contents with those from a target tree."""
+    e_trans_id = trans_id_file_id(file_id)
+    entry = target_tree.inventory[file_id]
+    has_contents, contents_mod, meta_mod, = _entry_changes(file_id, entry, 
+                                                           working_tree)
+    if contents_mod:
+        mode_id = e_trans_id
+        if has_contents:
+            if not backups:
+                tt.delete_contents(e_trans_id)
+            else:
+                parent_trans_id = trans_id_file_id(entry.parent_id)
+                backup_name = get_backup_name(entry, by_parent,
+                                              parent_trans_id, tt)
+                tt.adjust_path(backup_name, parent_trans_id, e_trans_id)
+                tt.unversion_file(e_trans_id)
+                e_trans_id = tt.create_path(entry.name, parent_trans_id)
+                tt.version_file(file_id, e_trans_id)
+                trans_id[file_id] = e_trans_id
+        create_by_entry(tt, entry, target_tree, e_trans_id, mode_id=mode_id)
+        create_entry_executability(tt, entry, e_trans_id)
+
+    elif meta_mod:
+        tt.set_executability(entry.executable, e_trans_id)
+    if tt.final_name(e_trans_id) != entry.name:
+        adjust_path  = True
+    else:
+        parent_id = tt.final_parent(e_trans_id)
+        parent_file_id = tt.final_file_id(parent_id)
+        if parent_file_id != entry.parent_id:
+            adjust_path = True
+        else:
+            adjust_path = False
+    if adjust_path:
+        parent_trans_id = trans_id_file_id(entry.parent_id)
+        tt.adjust_path(entry.name, parent_trans_id, e_trans_id)
+
+
+def get_backup_name(entry, by_parent, parent_trans_id, tt):
+    """Produce a backup-style name that appears to be available"""
+    def name_gen():
+        counter = 1
+        while True:
+            yield "%s.~%d~" % (entry.name, counter)
+            counter += 1
+    for name in name_gen():
+        if not tt.has_named_child(by_parent, parent_trans_id, name):
+            return name
+
+def _entry_changes(file_id, entry, working_tree):
+    """Determine in which ways the inventory entry has changed.
+
+    Returns booleans: has_contents, content_mod, meta_mod
+    has_contents means there are currently contents, but they differ
+    contents_mod means contents need to be modified
+    meta_mod means the metadata needs to be modified
+    """
+    cur_entry = working_tree.inventory[file_id]
+    try:
+        working_kind = working_tree.kind(file_id)
+        has_contents = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+        has_contents = False
+        contents_mod = True
+        meta_mod = False
+    if has_contents is True:
+        real_e_kind = entry.kind
+        if real_e_kind == 'root_directory':
+            real_e_kind = 'directory'
+        if real_e_kind != working_kind:
+            contents_mod, meta_mod = True, False
+        else:
+            cur_entry._read_tree_state(working_tree.id2path(file_id), 
+                                       working_tree)
+            contents_mod, meta_mod = entry.detect_changes(cur_entry)
+            cur_entry._forget_tree_state()
+    return has_contents, contents_mod, meta_mod
+
+
+def revert(working_tree, target_tree, filenames, backups=False, 
+           pb=DummyProgress()):
+    """Revert a working tree's contents to those of a target tree."""
+    interesting_ids = find_interesting(working_tree, target_tree, filenames)
+    def interesting(file_id):
+        return interesting_ids is None or file_id in interesting_ids
+
+    tt = TreeTransform(working_tree, pb)
+    try:
+        merge_modified = working_tree.merge_modified()
+        trans_id = {}
+        def trans_id_file_id(file_id):
+            try:
+                return trans_id[file_id]
+            except KeyError:
+                return tt.trans_id_tree_file_id(file_id)
+
+        pp = ProgressPhase("Revert phase", 4, pb)
+        pp.next_phase()
+        sorted_interesting = [i for i in topology_sorted_ids(target_tree) if
+                              interesting(i)]
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            by_parent = tt.by_parent()
+            for id_num, file_id in enumerate(sorted_interesting):
+                child_pb.update("Reverting file", id_num+1, 
+                                len(sorted_interesting))
+                if file_id not in working_tree.inventory:
+                    entry = target_tree.inventory[file_id]
+                    parent_id = trans_id_file_id(entry.parent_id)
+                    e_trans_id = new_by_entry(tt, entry, parent_id, target_tree)
+                    trans_id[file_id] = e_trans_id
+                else:
+                    backup_this = backups
+                    if file_id in merge_modified:
+                        backup_this = False
+                        del merge_modified[file_id]
+                    change_entry(tt, file_id, working_tree, target_tree, 
+                                 trans_id_file_id, backup_this, trans_id,
+                                 by_parent)
+        finally:
+            child_pb.finished()
+        pp.next_phase()
+        wt_interesting = [i for i in working_tree.inventory if interesting(i)]
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            for id_num, file_id in enumerate(wt_interesting):
+                child_pb.update("New file check", id_num+1, 
+                                len(sorted_interesting))
+                if file_id not in target_tree:
+                    trans_id = tt.trans_id_tree_file_id(file_id)
+                    tt.unversion_file(trans_id)
+                    if file_id in merge_modified:
+                        tt.delete_contents(trans_id)
+                        del merge_modified[file_id]
+        finally:
+            child_pb.finished()
+        pp.next_phase()
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            raw_conflicts = resolve_conflicts(tt, child_pb)
+        finally:
+            child_pb.finished()
+        conflicts = cook_conflicts(raw_conflicts, tt)
+        for conflict in conflicts:
+            warning(conflict)
+        pp.next_phase()
+        tt.apply()
+        working_tree.set_merge_modified({})
+    finally:
+        tt.finalize()
+        pb.clear()
+    return conflicts
+
+
+def resolve_conflicts(tt, pb=DummyProgress()):
+    """Make many conflict-resolution attempts, but die if they fail"""
+    new_conflicts = set()
+    try:
+        for n in range(10):
+            pb.update('Resolution pass', n+1, 10)
+            conflicts = tt.find_conflicts()
+            if len(conflicts) == 0:
+                return new_conflicts
+            new_conflicts.update(conflict_pass(tt, conflicts))
+        raise MalformedTransform(conflicts=conflicts)
+    finally:
+        pb.clear()
+
+
+def conflict_pass(tt, conflicts):
+    """Resolve some classes of conflicts."""
+    new_conflicts = set()
+    for c_type, conflict in ((c[0], c) for c in conflicts):
+        if c_type == 'duplicate id':
+            tt.unversion_file(conflict[1])
+            new_conflicts.add((c_type, 'Unversioned existing file',
+                               conflict[1], conflict[2], ))
+        elif c_type == 'duplicate':
+            # files that were renamed take precedence
+            new_name = tt.final_name(conflict[1])+'.moved'
+            final_parent = tt.final_parent(conflict[1])
+            if tt.path_changed(conflict[1]):
+                tt.adjust_path(new_name, final_parent, conflict[2])
+                new_conflicts.add((c_type, 'Moved existing file to', 
+                                   conflict[2], conflict[1]))
+            else:
+                tt.adjust_path(new_name, final_parent, conflict[1])
+                new_conflicts.add((c_type, 'Moved existing file to', 
+                                  conflict[1], conflict[2]))
+        elif c_type == 'parent loop':
+            # break the loop by undoing one of the ops that caused the loop
+            cur = conflict[1]
+            while not tt.path_changed(cur):
+                cur = tt.final_parent(cur)
+            new_conflicts.add((c_type, 'Cancelled move', cur,
+                               tt.final_parent(cur),))
+            tt.adjust_path(tt.final_name(cur), tt.get_tree_parent(cur), cur)
+            
+        elif c_type == 'missing parent':
+            trans_id = conflict[1]
+            try:
+                tt.cancel_deletion(trans_id)
+                new_conflicts.add((c_type, 'Not deleting', trans_id))
+            except KeyError:
+                tt.create_directory(trans_id)
+                new_conflicts.add((c_type, 'Created directory.', trans_id))
+        elif c_type == 'unversioned parent':
+            tt.version_file(tt.inactive_file_id(conflict[1]), conflict[1])
+            new_conflicts.add((c_type, 'Versioned directory', conflict[1]))
+    return new_conflicts
+
+
+def cook_conflicts(raw_conflicts, tt):
+    """Generate a list of cooked conflicts, sorted by file path"""
+    from bzrlib.conflicts import Conflict
+    conflict_iter = iter_cook_conflicts(raw_conflicts, tt)
+    return sorted(conflict_iter, key=Conflict.sort_key)
+
+
+def iter_cook_conflicts(raw_conflicts, tt):
+    from bzrlib.conflicts import Conflict
+    fp = FinalPaths(tt)
+    for conflict in raw_conflicts:
+        c_type = conflict[0]
+        action = conflict[1]
+        modified_path = fp.get_path(conflict[2])
+        modified_id = tt.final_file_id(conflict[2])
+        if len(conflict) == 3:
+            yield Conflict.factory(c_type, action=action, path=modified_path,
+                                     file_id=modified_id)
+             
+        else:
+            conflicting_path = fp.get_path(conflict[3])
+            conflicting_id = tt.final_file_id(conflict[3])
+            yield Conflict.factory(c_type, action=action, path=modified_path,
+                                   file_id=modified_id, 
+                                   conflict_path=conflicting_path,
+                                   conflict_file_id=conflicting_id)

=== added directory 'bzrlib/transport'
=== added file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/__init__.py	2006-04-12 04:45:32 +0000
@@ -0,0 +1,810 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Transport is an abstraction layer to handle file access.
+
+The abstraction is to allow access from the local filesystem, as well
+as remote (such as http or sftp).
+
+Transports are constructed from a string, being a URL or (as a degenerate
+case) a local filesystem path.  This is typically the top directory of
+a bzrdir, repository, or similar object we are interested in working with.
+The Transport returned has methods to read, write and manipulate files within
+it.
+"""
+
+import errno
+from collections import deque
+from copy import deepcopy
+from stat import *
+import sys
+from unittest import TestSuite
+import urllib
+import urlparse
+
+from bzrlib.trace import mutter, warning
+import bzrlib.errors as errors
+from bzrlib.errors import DependencyNotPresent
+from bzrlib.symbol_versioning import *
+
+# {prefix: [transport_classes]}
+# Transports are inserted onto the list LIFO and tried in order; as a result
+# transports provided by plugins are tried first, which is usually what we
+# want.
+_protocol_handlers = {
+}
+
+def register_transport(prefix, klass, override=DEPRECATED_PARAMETER):
+    """Register a transport that can be used to open URLs
+
+    Normally you should use register_lazy_transport, which defers loading the
+    implementation until it's actually used, and so avoids pulling in possibly
+    large implementation libraries.
+    """
+    # Note that this code runs very early in library setup -- trace may not be
+    # working, etc.
+    global _protocol_handlers
+    if deprecated_passed(override):
+        warn("register_transport(override) is deprecated")
+    _protocol_handlers.setdefault(prefix, []).insert(0, klass)
+
+
+def register_lazy_transport(scheme, module, classname):
+    """Register lazy-loaded transport class.
+
+    When opening a URL with the given scheme, load the module and then
+    instantiate the particular class.  
+
+    If the module raises DependencyNotPresent when it's imported, it is
+    skipped and another implementation of the protocol is tried.  This is
+    intended to be used when the implementation depends on an external
+    implementation that may not be present.  If any other error is raised, it
+    propagates up and the attempt to open the url fails.
+    """
+    # TODO: If no implementation of a protocol is available because of missing
+    # dependencies, we should perhaps show the message about what dependency
+    # was missing.
+    def _loader(base):
+        mod = __import__(module, globals(), locals(), [classname])
+        klass = getattr(mod, classname)
+        return klass(base)
+    _loader.module = module
+    register_transport(scheme, _loader)
+
+
+def _get_protocol_handlers():
+    """Return a dictionary of {urlprefix: [factory]}"""
+    return _protocol_handlers
+
+
+def _set_protocol_handlers(new_handlers):
+    """Replace the current protocol handlers dictionary.
+
+    WARNING this will remove all build in protocols. Use with care.
+    """
+    global _protocol_handlers
+    _protocol_handlers = new_handlers
+
+
+def _clear_protocol_handlers():
+    global _protocol_handlers
+    _protocol_handlers = {}
+
+
+def _get_transport_modules():
+    """Return a list of the modules providing transports."""
+    modules = set()
+    for prefix, factory_list in _protocol_handlers.items():
+        for factory in factory_list:
+            if factory.__module__ == "bzrlib.transport":
+                # this is a lazy load transport, because no real ones
+                # are directlry in bzrlib.transport
+                modules.add(factory.module)
+            else:
+                modules.add(factory.__module__)
+    result = list(modules)
+    result.sort()
+    return result
+
+
+def register_urlparse_netloc_protocol(protocol):
+    """Ensure that protocol is setup to be used with urlparse netloc parsing."""
+    if protocol not in urlparse.uses_netloc:
+        urlparse.uses_netloc.append(protocol)
+
+
+class Transport(object):
+    """This class encapsulates methods for retrieving or putting a file
+    from/to a storage location.
+
+    Most functions have a _multi variant, which allows you to queue up
+    multiple requests. They generally have a dumb base implementation 
+    which just iterates over the arguments, but smart Transport
+    implementations can do pipelining.
+    In general implementations should support having a generator or a list
+    as an argument (ie always iterate, never index)
+    """
+
+    def __init__(self, base):
+        super(Transport, self).__init__()
+        self.base = base
+
+    def _translate_error(self, e, path, raise_generic=True):
+        """Translate an IOError or OSError into an appropriate bzr error.
+
+        This handles things like ENOENT, ENOTDIR, EEXIST, and EACCESS
+        """
+        if hasattr(e, 'errno'):
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                raise errors.NoSuchFile(path, extra=e)
+            # I would rather use errno.EFOO, but there doesn't seem to be
+            # any matching for 267
+            # This is the error when doing a listdir on a file:
+            # WindowsError: [Errno 267] The directory name is invalid
+            if sys.platform == 'win32' and e.errno in (errno.ESRCH, 267):
+                raise errors.NoSuchFile(path, extra=e)
+            if e.errno == errno.EEXIST:
+                raise errors.FileExists(path, extra=e)
+            if e.errno == errno.EACCES:
+                raise errors.PermissionDenied(path, extra=e)
+            if e.errno == errno.ENOTEMPTY:
+                raise errors.DirectoryNotEmpty(path, extra=e)
+            if e.errno == errno.EBUSY:
+                raise errors.ResourceBusy(path, extra=e)
+        if raise_generic:
+            raise errors.TransportError(orig_error=e)
+
+    def clone(self, offset=None):
+        """Return a new Transport object, cloned from the current location,
+        using a subdirectory or parent directory. This allows connections 
+        to be pooled, rather than a new one needed for each subdir.
+        """
+        raise NotImplementedError(self.clone)
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return False
+
+    def _pump(self, from_file, to_file):
+        """Most children will need to copy from one file-like 
+        object or string to another one.
+        This just gives them something easy to call.
+        """
+        if isinstance(from_file, basestring):
+            to_file.write(from_file)
+        else:
+            from bzrlib.osutils import pumpfile
+            pumpfile(from_file, to_file)
+
+    def _get_total(self, multi):
+        """Try to figure out how many entries are in multi,
+        but if not possible, return None.
+        """
+        try:
+            return len(multi)
+        except TypeError: # We can't tell how many, because relpaths is a generator
+            return None
+
+    def _update_pb(self, pb, msg, count, total):
+        """Update the progress bar based on the current count
+        and total available, total may be None if it was
+        not possible to determine.
+        """
+        if pb is None:
+            return
+        if total is None:
+            pb.update(msg, count, count+1)
+        else:
+            pb.update(msg, count, total)
+
+    def _iterate_over(self, multi, func, pb, msg, expand=True):
+        """Iterate over all entries in multi, passing them to func,
+        and update the progress bar as you go along.
+
+        :param expand:  If True, the entries will be passed to the function
+                        by expanding the tuple. If False, it will be passed
+                        as a single parameter.
+        """
+        total = self._get_total(multi)
+        result = []
+        count = 0
+        for entry in multi:
+            self._update_pb(pb, msg, count, total)
+            if expand:
+                result.append(func(*entry))
+            else:
+                result.append(func(entry))
+            count += 1
+        return tuple(result)
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+
+        XXX: Robert Collins 20051016 - is this really needed in the public
+             interface ?
+        """
+        raise NotImplementedError(self.abspath)
+
+    def relpath(self, abspath):
+        """Return the local path portion from a given absolute path.
+
+        This default implementation is not suitable for filesystems with
+        aliasing, such as that given by symlinks, where a path may not 
+        start with our base, but still be a relpath once aliasing is 
+        resolved.
+        """
+        # TODO: This might want to use bzrlib.osutils.relpath
+        #       but we have to watch out because of the prefix issues
+        if not (abspath == self.base[:-1] or abspath.startswith(self.base)):
+            raise errors.PathNotChild(abspath, self.base)
+        pl = len(self.base)
+        return abspath[pl:].strip('/')
+
+    def has(self, relpath):
+        """Does the file relpath exist?
+        
+        Note that some transports MAY allow querying on directories, but this
+        is not part of the protocol.  In other words, the results of 
+        t.has("a_directory_name") are undefined."
+        """
+        raise NotImplementedError(self.has)
+
+    def has_multi(self, relpaths, pb=None):
+        """Return True/False for each entry in relpaths"""
+        total = self._get_total(relpaths)
+        count = 0
+        for relpath in relpaths:
+            self._update_pb(pb, 'has', count, total)
+            yield self.has(relpath)
+            count += 1
+
+    def has_any(self, relpaths):
+        """Return True if any of the paths exist."""
+        for relpath in relpaths:
+            if self.has(relpath):
+                return True
+        return False
+
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree.
+
+        *NOTE*: This only lists *files*, not subdirectories!
+        
+        As with other listing functions, only some transports implement this,.
+        you may check via is_listable to determine if it will.
+        """
+        raise errors.TransportNotPossible("This transport has not "
+                                          "implemented iter_files_recursive "
+                                          "(but must claim to be listable "
+                                          "to trigger this error).")
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        raise NotImplementedError(self.get)
+
+    def readv(self, relpath, offsets):
+        """Get parts of the file at the given relative path.
+
+        :offsets: A list of (offset, size) tuples.
+        :return: A list or generator of (offset, data) tuples
+        """
+        def do_combined_read(combined_offsets):
+            total_size = 0
+            for offset, size in combined_offsets:
+                total_size += size
+            mutter('readv coalesced %d reads.', len(combined_offsets))
+            offset = combined_offsets[0][0]
+            fp.seek(offset)
+            data = fp.read(total_size)
+            pos = 0
+            for offset, size in combined_offsets:
+                yield offset, data[pos:pos + size]
+                pos += size
+
+        if not len(offsets):
+            return
+        fp = self.get(relpath)
+        pending_offsets = deque(offsets)
+        combined_offsets = []
+        while len(pending_offsets):
+            offset, size = pending_offsets.popleft()
+            if not combined_offsets:
+                combined_offsets = [[offset, size]]
+            else:
+                if (len (combined_offsets) < 50 and
+                    combined_offsets[-1][0] + combined_offsets[-1][1] == offset):
+                    # combatible offset:
+                    combined_offsets.append([offset, size])
+                else:
+                    # incompatible, or over the threshold issue a read and yield
+                    pending_offsets.appendleft((offset, size))
+                    for result in do_combined_read(combined_offsets):
+                        yield result
+                    combined_offsets = []
+        # whatever is left is a single coalesced request
+        if len(combined_offsets):
+            for result in do_combined_read(combined_offsets):
+                yield result
+
+    def get_multi(self, relpaths, pb=None):
+        """Get a list of file-like objects, one for each entry in relpaths.
+
+        :param relpaths: A list of relative paths.
+        :param pb:  An optional ProgressBar for indicating percent done.
+        :return: A list or generator of file-like objects
+        """
+        # TODO: Consider having this actually buffer the requests,
+        # in the default mode, it probably won't give worse performance,
+        # and all children wouldn't have to implement buffering
+        total = self._get_total(relpaths)
+        count = 0
+        for relpath in relpaths:
+            self._update_pb(pb, 'get', count, total)
+            yield self.get(relpath)
+            count += 1
+
+    def put(self, relpath, f, mode=None):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        :param mode: The mode for the newly created file, 
+                     None means just use the default
+        """
+        raise NotImplementedError(self.put)
+
+    def put_multi(self, files, mode=None, pb=None):
+        """Put a set of files into the location.
+
+        :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
+        :param pb:  An optional ProgressBar for indicating percent done.
+        :param mode: The mode for the newly created files
+        :return: The number of files copied.
+        """
+        def put(path, f):
+            self.put(path, f, mode=mode)
+        return len(self._iterate_over(files, put, pb, 'put', expand=True))
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        raise NotImplementedError(self.mkdir)
+
+    def mkdir_multi(self, relpaths, mode=None, pb=None):
+        """Create a group of directories"""
+        def mkdir(path):
+            self.mkdir(path, mode=mode)
+        return len(self._iterate_over(relpaths, mkdir, pb, 'mkdir', expand=False))
+
+    def append(self, relpath, f):
+        """Append the text in the file-like or string object to 
+        the supplied location.
+
+        returns the length of f before the content was written to it.
+        """
+        raise NotImplementedError(self.append)
+
+    def append_multi(self, files, pb=None):
+        """Append the text in each file-like or string object to
+        the supplied location.
+
+        :param files: A set of (path, f) entries
+        :param pb:  An optional ProgressBar for indicating percent done.
+        """
+        return self._iterate_over(files, self.append, pb, 'append', expand=True)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to.
+        
+        Override this for efficiency if a specific transport can do it 
+        faster than this default implementation.
+        """
+        self.put(rel_to, self.get(rel_from))
+
+    def copy_multi(self, relpaths, pb=None):
+        """Copy a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from, to), (from, to),...]
+        """
+        # This is the non-pipelined implementation, so that
+        # implementors don't have to implement everything.
+        return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
+
+    def copy_to(self, relpaths, other, mode=None, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        :param mode: This is the target mode for the newly created files
+        TODO: This interface needs to be updated so that the target location
+              can be different from the source location.
+        """
+        # The dummy implementation just does a simple get + put
+        def copy_entry(path):
+            other.put(path, self.get(path), mode=mode)
+
+        return len(self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False))
+
+    def copy_tree(self, from_relpath, to_relpath):
+        """Copy a subtree from one relpath to another.
+
+        If a faster implementation is available, specific transports should 
+        implement it.
+        """
+        source = self.clone(from_relpath)
+        self.mkdir(to_relpath)
+        target = self.clone(to_relpath)
+        files = []
+        directories = ['.']
+        while directories:
+            dir = directories.pop()
+            if dir != '.':
+                target.mkdir(dir)
+            for path in source.list_dir(dir):
+                path = dir + '/' + path
+                stat = source.stat(path)
+                if S_ISDIR(stat.st_mode):
+                    directories.append(path)
+                else:
+                    files.append(path)
+        source.copy_to(files, target)
+
+    def rename(self, rel_from, rel_to):
+        """Rename a file or directory.
+
+        This *must* fail if the destination is a nonempty directory - it must
+        not automatically remove it.  It should raise DirectoryNotEmpty, or
+        some other PathError if the case can't be specifically detected.
+
+        If the destination is an empty directory or a file this function may
+        either fail or succeed, depending on the underlying transport.  It
+        should not attempt to remove the destination if overwriting is not the
+        native transport behaviour.  If at all possible the transport should
+        ensure that the rename either completes or not, without leaving the
+        destination deleted and the new file not moved in place.
+
+        This is intended mainly for use in implementing LockDir.
+        """
+        # transports may need to override this
+        raise NotImplementedError(self.rename)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to.
+
+        The destination is deleted if possible, even if it's a non-empty
+        directory tree.
+        
+        If a transport can directly implement this it is suggested that
+        it do so for efficiency.
+        """
+        if S_ISDIR(self.stat(rel_from).st_mode):
+            self.copy_tree(rel_from, rel_to)
+            self.delete_tree(rel_from)
+        else:
+            self.copy(rel_from, rel_to)
+            self.delete(rel_from)
+
+    def move_multi(self, relpaths, pb=None):
+        """Move a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from1, to1), (from2, to2),...]
+        """
+        return self._iterate_over(relpaths, self.move, pb, 'move', expand=True)
+
+    def move_multi_to(self, relpaths, rel_to):
+        """Move a bunch of entries to a single location.
+        This differs from move_multi in that you give a list of from, and
+        a single destination, rather than multiple destinations.
+
+        :param relpaths: A list of relative paths [from1, from2, from3, ...]
+        :param rel_to: A directory where each entry should be placed.
+        """
+        # This is not implemented, because you need to do special tricks to
+        # extract the basename, and add it to rel_to
+        raise NotImplementedError(self.move_multi_to)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise NotImplementedError(self.delete)
+
+    def delete_multi(self, relpaths, pb=None):
+        """Queue up a bunch of deletes to be done.
+        """
+        return self._iterate_over(relpaths, self.delete, pb, 'delete', expand=False)
+
+    def delete_tree(self, relpath):
+        """Delete an entire tree. This may require a listable transport."""
+        subtree = self.clone(relpath)
+        files = []
+        directories = ['.']
+        pending_rmdirs = []
+        while directories:
+            dir = directories.pop()
+            if dir != '.':
+                pending_rmdirs.append(dir)
+            for path in subtree.list_dir(dir):
+                path = dir + '/' + path
+                stat = subtree.stat(path)
+                if S_ISDIR(stat.st_mode):
+                    directories.append(path)
+                else:
+                    files.append(path)
+        subtree.delete_multi(files)
+        pending_rmdirs.reverse()
+        for dir in pending_rmdirs:
+            subtree.rmdir(dir)
+        self.rmdir(relpath)
+
+    def __repr__(self):
+        return "<%s.%s url=%s>" % (self.__module__, self.__class__.__name__, self.base)
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        WARNING: This may not be implementable for all protocols, so use
+        sparingly.
+        NOTE: This returns an object with fields such as 'st_size'. It MAY
+        or MAY NOT return the literal result of an os.stat() call, so all
+        access should be via named fields.
+        ALSO NOTE: Stats of directories may not be supported on some 
+        transports.
+        """
+        raise NotImplementedError(self.stat)
+
+    def rmdir(self, relpath):
+        """Remove a directory at the given path."""
+        raise NotImplementedError
+
+    def stat_multi(self, relpaths, pb=None):
+        """Stat multiple files and return the information.
+        """
+        #TODO:  Is it worth making this a generator instead of a
+        #       returning a list?
+        stats = []
+        def gather(path):
+            stats.append(self.stat(path))
+
+        count = self._iterate_over(relpaths, gather, pb, 'stat', expand=False)
+        return stats
+
+    def listable(self):
+        """Return True if this store supports listing."""
+        raise NotImplementedError(self.listable)
+
+    def list_dir(self, relpath):
+        """Return a list of all files at the given location.
+        WARNING: many transports do not support this, so trying avoid using
+        it if at all possible.
+        """
+        raise errors.TransportNotPossible("This transport has not "
+                                          "implemented list_dir "
+                                          "(but must claim to be listable "
+                                          "to trigger this error).")
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should contain an unlock() function.
+        """
+        raise NotImplementedError(self.lock_read)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should contain an unlock() function.
+        """
+        raise NotImplementedError(self.lock_write)
+
+    def is_readonly(self):
+        """Return true if this connection cannot be written to."""
+        return False
+
+    def _can_roundtrip_unix_modebits(self):
+        """Return true if this transport can store and retrieve unix modebits.
+
+        (For example, 0700 to make a directory owner-private.)
+        
+        Note: most callers will not want to switch on this, but should rather 
+        just try and set permissions and let them be either stored or not.
+        This is intended mainly for the use of the test suite.
+        
+        Warning: this is not guaranteed to be accurate as sometimes we can't 
+        be sure: for example with vfat mounted on unix, or a windows sftp
+        server."""
+        # TODO: Perhaps return a e.g. TransportCharacteristics that can answer
+        # several questions about the transport.
+        return False
+
+
+def get_transport(base):
+    """Open a transport to access a URL or directory.
+
+    base is either a URL or a directory name.  
+    """
+    # TODO: give a better error if base looks like a url but there's no
+    # handler for the scheme?
+    global _protocol_handlers
+    if base is None:
+        base = u'.'
+    else:
+        base = unicode(base)
+    for proto, factory_list in _protocol_handlers.iteritems():
+        if proto is not None and base.startswith(proto):
+            t = _try_transport_factories(base, factory_list)
+            if t:
+                return t
+    # The default handler is the filesystem handler, stored as protocol None
+    return _try_transport_factories(base, _protocol_handlers[None])
+
+
+def _try_transport_factories(base, factory_list):
+    for factory in factory_list:
+        try:
+            return factory(base)
+        except DependencyNotPresent, e:
+            mutter("failed to instantiate transport %r for %r: %r" %
+                    (factory, base, e))
+            continue
+    return None
+
+
+def urlescape(relpath):
+    """Escape relpath to be a valid url."""
+    if isinstance(relpath, unicode):
+        relpath = relpath.encode('utf-8')
+    return urllib.quote(relpath)
+
+
+def urlunescape(relpath):
+    """Unescape relpath from url format."""
+    return urllib.unquote(relpath)
+    # TODO de-utf8 it last. relpath = utf8relpath.decode('utf8')
+
+
+class Server(object):
+    """A Transport Server.
+    
+    The Server interface provides a server for a given transport. We use
+    these servers as loopback testing tools. For any given transport the
+    Servers it provides must either allow writing, or serve the contents
+    of os.getcwdu() at the time setUp is called.
+    
+    Note that these are real servers - they must implement all the things
+    that we want bzr transports to take advantage of.
+    """
+
+    def setUp(self):
+        """Setup the server to service requests."""
+
+    def tearDown(self):
+        """Remove the server and cleanup any resources it owns."""
+
+    def get_url(self):
+        """Return a url for this server.
+        
+        If the transport does not represent a disk directory (i.e. it is 
+        a database like svn, or a memory only transport, it should return
+        a connection to a newly established resource for this Server.
+        Otherwise it should return a url that will provide access to the path
+        that was os.getcwdu() when setUp() was called.
+        
+        Subsequent calls will return the same resource.
+        """
+        raise NotImplementedError
+
+    def get_bogus_url(self):
+        """Return a url for this protocol, that will fail to connect."""
+        raise NotImplementedError
+
+
+class TransportTestProviderAdapter(object):
+    """A tool to generate a suite testing all transports for a single test.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_class and transport_server classes into each copy. Each copy
+    is also given a new id() to make it easy to identify.
+    """
+
+    def adapt(self, test):
+        result = TestSuite()
+        for klass, server_factory in self._test_permutations():
+            new_test = deepcopy(test)
+            new_test.transport_class = klass
+            new_test.transport_server = server_factory
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), server_factory.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    def get_transport_test_permutations(self, module):
+        """Get the permutations module wants to have tested."""
+        if not hasattr(module, 'get_test_permutations'):
+            warning("transport module %s doesn't provide get_test_permutations()"
+                    % module.__name__)
+            return []
+        return module.get_test_permutations()
+
+    def _test_permutations(self):
+        """Return a list of the klass, server_factory pairs to test."""
+        result = []
+        for module in _get_transport_modules():
+            try:
+                result.extend(self.get_transport_test_permutations(reduce(getattr, 
+                    (module).split('.')[1:],
+                     __import__(module))))
+            except errors.DependencyNotPresent, e:
+                # Continue even if a dependency prevents us 
+                # from running this test
+                pass
+        return result
+
+
+class TransportLogger(object):
+    """Adapt a transport to get clear logging data on api calls.
+    
+    Feel free to extend to log whatever calls are of interest.
+    """
+
+    def __init__(self, adapted):
+        self._adapted = adapted
+        self._calls = []
+
+    def get(self, name):
+        self._calls.append((name,))
+        return self._adapted.get(name)
+
+    def __getattr__(self, name):
+        """Thunk all undefined access through to self._adapted."""
+        # raise AttributeError, name 
+        return getattr(self._adapted, name)
+
+    def readv(self, name, offsets):
+        self._calls.append((name, offsets))
+        return self._adapted.readv(name, offsets)
+        
+
+# None is the default transport, for things with no url scheme
+register_lazy_transport(None, 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('sftp://', 'bzrlib.transport.sftp', 'SFTPTransport')
+register_lazy_transport('http+urllib://', 'bzrlib.transport.http._urllib',
+                        'HttpTransport_urllib')
+register_lazy_transport('https+urllib://', 'bzrlib.transport.http._urllib',
+                        'HttpTransport_urllib')
+register_lazy_transport('http+pycurl://', 'bzrlib.transport.http._pycurl',
+                        'PyCurlTransport')
+register_lazy_transport('https+pycurl://', 'bzrlib.transport.http._pycurl',
+                        'PyCurlTransport')
+register_lazy_transport('http://', 'bzrlib.transport.http._urllib',
+                        'HttpTransport_urllib')
+register_lazy_transport('https://', 'bzrlib.transport.http._urllib',
+                        'HttpTransport_urllib')
+register_lazy_transport('http://', 'bzrlib.transport.http._pycurl', 'PyCurlTransport')
+register_lazy_transport('https://', 'bzrlib.transport.http._pycurl', 'PyCurlTransport')
+register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
+register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
+register_lazy_transport('memory:/', 'bzrlib.transport.memory', 'MemoryTransport')
+register_lazy_transport('readonly+', 'bzrlib.transport.readonly', 'ReadonlyTransportDecorator')
+register_lazy_transport('fakenfs+', 'bzrlib.transport.fakenfs', 'FakeNFSTransportDecorator')
+register_lazy_transport('vfat+', 
+                        'bzrlib.transport.fakevfat',
+                        'FakeVFATTransportDecorator')

=== added file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/decorator.py	2006-04-10 07:44:13 +0000
@@ -0,0 +1,189 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport that decorates another transport.
+
+This does not change the transport behaviour at all, but provides all the 
+stub functions to allow other decorators to be written easily.
+"""
+
+from bzrlib.transport import get_transport, Transport, Server
+
+
+class TransportDecorator(Transport):
+    """A no-change decorator for Transports.
+
+    Subclasses of this are new transports that are based on an
+    underlying transport and can override or intercept some 
+    behavior.  For example ReadonlyTransportDecorator prevents 
+    all write attempts, and FakeNFSTransportDecorator simulates 
+    some NFS quirks.
+
+    This decorator class is not directly usable as a decorator:
+    you must use a subclass which has overridden the _get_url_prefix() class
+    method to return the url prefix for the subclass.
+    """
+
+    def __init__(self, url, _decorated=None):
+        """Set the 'base' path where files will be stored.
+        
+        _decorated is a private parameter for cloning."""
+        prefix = self._get_url_prefix()
+        assert url.startswith(prefix), \
+                "url %r doesn't start with decorator prefix %r" % \
+                (url, prefix)
+        decorated_url = url[len(prefix):]
+        if _decorated is None:
+            self._decorated = get_transport(decorated_url)
+        else:
+            self._decorated = _decorated
+        super(TransportDecorator, self).__init__(
+            prefix + self._decorated.base)
+
+    def abspath(self, relpath):
+        """See Transport.abspath()."""
+        return self._get_url_prefix() + self._decorated.abspath(relpath)
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        return self._decorated.append(relpath, f)
+
+    def clone(self, offset=None):
+        """See Transport.clone()."""
+        decorated_clone = self._decorated.clone(offset)
+        return self.__class__(
+            self._get_url_prefix() + decorated_clone.base, decorated_clone)
+
+    def delete(self, relpath):
+        """See Transport.delete()."""
+        return self._decorated.delete(relpath)
+
+    def delete_tree(self, relpath):
+        """See Transport.delete_tree()."""
+        return self._decorated.delete_tree(relpath)
+
+    @classmethod
+    def _get_url_prefix(self):
+        """Return the URL prefix of this decorator."""
+        raise NotImplementedError(self._get_url_prefix)
+
+    def get(self, relpath):
+        """See Transport.get()."""
+        return self._decorated.get(relpath)
+
+    def has(self, relpath):
+        """See Transport.has()."""
+        return self._decorated.has(relpath)
+
+    def is_readonly(self):
+        """See Transport.is_readonly."""
+        return self._decorated.is_readonly()
+
+    def mkdir(self, relpath, mode=None):
+        """See Transport.mkdir()."""
+        return self._decorated.mkdir(relpath, mode)
+
+    def put(self, relpath, f, mode=None):
+        """See Transport.put()."""
+        return self._decorated.put(relpath, f, mode)
+    
+    def listable(self):
+        """See Transport.listable."""
+        return self._decorated.listable()
+
+    def iter_files_recursive(self):
+        """See Transport.iter_files_recursive()."""
+        return self._decorated.iter_files_recursive()
+    
+    def list_dir(self, relpath):
+        """See Transport.list_dir()."""
+        return self._decorated.list_dir(relpath)
+
+    def rename(self, rel_from, rel_to):
+        return self._decorated.rename(rel_from, rel_to)
+    
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        return self._decorated.rmdir(relpath)
+
+    def should_cache(self):
+        """See Transport.should_cache()."""
+        return self._decorated.should_cache()
+
+    def stat(self, relpath):
+        """See Transport.stat()."""
+        return self._decorated.stat(relpath)
+
+    def lock_read(self, relpath):
+        """See Transport.lock_read."""
+        return self._decorated.lock_read(relpath)
+
+    def lock_write(self, relpath):
+        """See Transport.lock_write."""
+        return self._decorated.lock_write(relpath)
+
+
+class DecoratorServer(Server):
+    """Server for the TransportDecorator for testing with.
+    
+    To use this when subclassing TransportDecorator, override override the
+    get_decorator_class method.
+    """
+
+    def setUp(self, server=None):
+        """See bzrlib.transport.Server.setUp.
+
+        :server: decorate the urls given by server. If not provided a
+        LocalServer is created.
+        """
+        if server is not None:
+            self._made_server = False
+            self._server = server
+        else:
+            from bzrlib.transport.local import LocalRelpathServer
+            self._made_server = True
+            self._server = LocalRelpathServer()
+            self._server.setUp()
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        if self._made_server:
+            self._server.tearDown()
+
+    def get_decorator_class(self):
+        """Return the class of the decorators we should be constructing."""
+        raise NotImplementedError(self.get_decorator_class)
+
+    def get_url_prefix(self):
+        """What URL prefix does this decorator produce?"""
+        return self.get_decorator_class()._get_url_prefix()
+
+    def get_bogus_url(self):
+        """See bzrlib.transport.Server.get_bogus_url."""
+        return self.get_url_prefix() + self._server.get_bogus_url()
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self.get_url_prefix() + self._server.get_url()
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing.
+    
+    The Decorator class is not directly usable, and testing it would not have
+    any benefit - its the concrete classes which need to be tested.
+    """
+    return []

=== added file 'bzrlib/transport/fakenfs.py'
--- bzrlib/transport/fakenfs.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/fakenfs.py	2006-04-02 22:42:19 +0000
@@ -0,0 +1,67 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Transport implementation that adapts another transport to look like NFS.
+
+Currently this means that the rename() call will raise ResourceBusy when a
+target path is a directory.
+
+To get a fake nfs transport use get_transport('fakenfs+' + real_url)
+"""
+
+from stat import *
+
+import bzrlib.errors as errors
+from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
+
+
+class FakeNFSTransportDecorator(TransportDecorator):
+    """A transport that behaves like NFS, for testing"""
+
+    @classmethod
+    def _get_url_prefix(self):
+        """FakeNFS transports are identified by 'fakenfs+'"""
+        return 'fakenfs+'
+
+    def rename(self, rel_from, rel_to):
+        """See Transport.rename().
+
+        This variation on rename converts DirectoryNotEmpty and FileExists
+        errors into ResourceBusy if the target is a directory.
+        """
+        try:
+            self._decorated.rename(rel_from, rel_to)
+        except (errors.DirectoryNotEmpty, errors.FileExists), e:
+            # if this is a directory rename, raise
+            # resourcebusy rather than DirectoryNotEmpty
+            stat = self._decorated.stat(rel_to)
+            if S_ISDIR(stat.st_mode):
+                raise errors.ResourceBusy(rel_to)
+            else:
+                raise
+
+
+class FakeNFSServer(DecoratorServer):
+    """Server for the FakeNFSTransportDecorator for testing with."""
+
+    def get_decorator_class(self):
+        return FakeNFSTransportDecorator
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(FakeNFSTransportDecorator, FakeNFSServer),
+            ]

=== added file 'bzrlib/transport/fakevfat.py'
--- bzrlib/transport/fakevfat.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/fakevfat.py	2006-04-10 07:43:09 +0000
@@ -0,0 +1,111 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Fake transport with some restrictions of Windows VFAT filesystems.
+
+VFAT on Windows has several restrictions that are not present on unix
+filesystems, which are imposed by this transport. 
+
+VFAT is strictly 8-bit using codepages to represent non-ascii characters. 
+This implementation currently doesn't model the codepage but just insists
+on only ascii characters being written.
+
+Restrictions imposed by this transport:
+
+ * filenames are squashed to lowercase
+ * filenames containing non-ascii characters are not allowed
+ * filenames containing the characters "@<>" are not allowed
+   (there should be more?)
+
+Some other restrictions are not implemented yet, but possibly could be:
+
+ * open files can't be deleted or renamed
+ * directories containing open files can't be renamed
+ * special device names (NUL, LPT, ...) are not allowed
+
+"""
+
+import re
+
+from bzrlib.errors import TransportNotPossible
+from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
+
+
+# TODO: It might be nice if these hooks were available in a more general way
+# on all paths passed in to the Transport, so that we didn't have to hook
+# every single method.
+
+# TODO: Perhaps don't inherit from TransportDecorator so that methods
+# which are not implemented here fail by default?
+    
+
+class FakeVFATTransportDecorator(TransportDecorator):
+    """A decorator that can convert any transport to be readonly.
+
+    This is requested via the 'vfat+' prefix to get_transport().
+
+    This is intended only for use in testing and doesn't implement every
+    method very well yet.
+
+    This transport is typically layered on a local or memory transport
+    which actually stored the files.
+    """
+
+    @classmethod
+    def _get_url_prefix(self):
+        """Readonly transport decorators are invoked via 'vfat+'"""
+        return 'vfat+'
+
+    def _squash_name(self, name):
+        """Return vfat-squashed filename.
+
+        The name is returned as it will be stored on disk.  This raises an
+        error if there are invalid characters in the name.
+        """
+        if re.search(r'[?*:;<>]', name):
+            raise ValueError("illegal characters for VFAT filename: %r" % name)
+        return name.lower()
+
+    def get(self, relpath):
+        return self._decorated.get(self._squash_name(relpath))
+
+    def mkdir(self, relpath, mode=None):
+        return self._decorated.mkdir(self._squash_name(relpath), 0755)
+
+    def has(self, relpath):
+        return self._decorated.has(self._squash_name(relpath))
+
+    def readv(self, relpath, offsets):
+        return self._decorated.readv(self._squash_name(relpath), offsets)
+
+    def put(self, relpath, f, mode=None):
+        return self._decorated.put(self._squash_name(relpath), f, mode)
+
+
+class FakeVFATServer(DecoratorServer):
+    """A server that suggests connections through FakeVFATTransportDecorator
+
+    For use in testing.
+    """
+
+    def get_decorator_class(self):
+        return FakeVFATTransportDecorator
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(FakeVFATTransportDecorator, FakeVFATServer),
+            ]

=== added file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/ftp.py	2006-04-13 02:33:36 +0000
@@ -0,0 +1,412 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport over ftp.
+
+Written by Daniel Silverstone <dsilvers@digital-scurf.org> with serious
+cargo-culting from the sftp transport and the http transport.
+
+It provides the ftp:// and aftp:// protocols where ftp:// is passive ftp
+and aftp:// is active ftp. Most people will want passive ftp for traversing
+NAT and other firewalls, so it's best to use it unless you explicitly want
+active, in which case aftp:// will be your friend.
+"""
+
+from cStringIO import StringIO
+import errno
+import ftplib
+import os
+import urllib
+import urlparse
+import stat
+import time
+import random
+from warnings import warn
+
+
+from bzrlib.transport import Transport
+from bzrlib.errors import (TransportNotPossible, TransportError,
+                           NoSuchFile, FileExists, DirectoryNotEmpty)
+from bzrlib.trace import mutter, warning
+
+
+_FTP_cache = {}
+def _find_FTP(hostname, username, password, is_active):
+    """Find an ftplib.FTP instance attached to this triplet."""
+    key = "%s|%s|%s|%s" % (hostname, username, password, is_active)
+    if key not in _FTP_cache:
+        mutter("Constructing FTP instance against %r" % key)
+        _FTP_cache[key] = ftplib.FTP(hostname, username, password)
+        _FTP_cache[key].set_pasv(not is_active)
+    return _FTP_cache[key]    
+
+
+class FtpTransportError(TransportError):
+    pass
+
+
+class FtpStatResult(object):
+    def __init__(self, f, relpath):
+        try:
+            self.st_size = f.size(relpath)
+            self.st_mode = stat.S_IFREG
+        except ftplib.error_perm:
+            pwd = f.pwd()
+            try:
+                f.cwd(relpath)
+                self.st_mode = stat.S_IFDIR
+            finally:
+                f.cwd(pwd)
+
+
+_number_of_retries = 2
+_sleep_between_retries = 5
+
+class FtpTransport(Transport):
+    """This is the transport agent for ftp:// access."""
+
+    def __init__(self, base, _provided_instance=None):
+        """Set the base path where files will be stored."""
+        assert base.startswith('ftp://') or base.startswith('aftp://')
+        super(FtpTransport, self).__init__(base)
+        self.is_active = base.startswith('aftp://')
+        if self.is_active:
+            base = base[1:]
+        (self._proto, self._host,
+            self._path, self._parameters,
+            self._query, self._fragment) = urlparse.urlparse(self.base)
+        self._FTP_instance = _provided_instance
+
+    def _get_FTP(self):
+        """Return the ftplib.FTP instance for this object."""
+        if self._FTP_instance is not None:
+            return self._FTP_instance
+        
+        try:
+            username = ''
+            password = ''
+            hostname = self._host
+            if '@' in hostname:
+                username, hostname = hostname.split("@", 1)
+            if ':' in username:
+                username, password = username.split(":", 1)
+
+            self._FTP_instance = _find_FTP(hostname, username, password,
+                                           self.is_active)
+            return self._FTP_instance
+        except ftplib.error_perm, e:
+            raise TransportError(msg="Error setting up connection: %s"
+                                    % str(e), orig_error=e)
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """Return a new FtpTransport with root at self.base + offset.
+        """
+        mutter("FTP clone")
+        if offset is None:
+            return FtpTransport(self.base, self._FTP_instance)
+        else:
+            return FtpTransport(self.abspath(offset), self._FTP_instance)
+
+    def _abspath(self, relpath):
+        assert isinstance(relpath, basestring)
+        relpath = urllib.unquote(relpath)
+        if isinstance(relpath, basestring):
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+        for p in relpath_parts:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.' or p == '':
+                continue # No-op
+            else:
+                basepath.append(p)
+        # Possibly, we could use urlparse.urljoin() here, but
+        # I'm concerned about when it chooses to strip the last
+        # portion of the path, and when it doesn't.
+        return '/'.join(basepath)
+    
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+        """
+        path = self._abspath(relpath)
+        return urlparse.urlunparse((self._proto,
+                self._host, path, '', '', ''))
+
+    def has(self, relpath):
+        """Does the target location exist?
+
+        XXX: I assume we're never asked has(dirname) and thus I use
+        the FTP size command and assume that if it doesn't raise,
+        all is good.
+        """
+        try:
+            f = self._get_FTP()
+            s = f.size(self._abspath(relpath))
+            mutter("FTP has: %s" % self._abspath(relpath))
+            return True
+        except ftplib.error_perm:
+            mutter("FTP has not: %s" % self._abspath(relpath))
+            return False
+
+    def get(self, relpath, decode=False, retries=0):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        :param retries: Number of retries after temporary failures so far
+                        for this operation.
+
+        We're meant to return a file-like object which bzr will
+        then read from. For now we do this via the magic of StringIO
+        """
+        # TODO: decode should be deprecated
+        try:
+            mutter("FTP get: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            ret = StringIO()
+            f.retrbinary('RETR '+self._abspath(relpath), ret.write, 8192)
+            ret.seek(0)
+            return ret
+        except ftplib.error_perm, e:
+            raise NoSuchFile(self.abspath(relpath), extra=str(e))
+        except ftplib.error_temp, e:
+            if retries > _number_of_retries:
+                raise TransportError(msg="FTP temporary error during GET %s. Aborting."
+                                     % self.abspath(relpath),
+                                     orig_error=e)
+            else:
+                warning("FTP temporary error: %s. Retrying." % str(e))
+                self._FTP_instance = None
+                return self.get(relpath, decode, retries+1)
+        except EOFError, e:
+            if retries > _number_of_retries:
+                raise TransportError("FTP control connection closed during GET %s."
+                                     % self.abspath(relpath),
+                                     orig_error=e)
+            else:
+                warning("FTP control connection closed. Trying to reopen.")
+                time.sleep(_sleep_between_retries)
+                self._FTP_instance = None
+                return self.get(relpath, decode, retries+1)
+
+    def put(self, relpath, fp, mode=None, retries=0):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param fp:       File-like or string object.
+        :param retries: Number of retries after temporary failures so far
+                        for this operation.
+
+        TODO: jam 20051215 ftp as a protocol seems to support chmod, but ftplib does not
+        """
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (self._abspath(relpath), time.time(),
+                        os.getpid(), random.randint(0,0x7FFFFFFF))
+        if not hasattr(fp, 'read'):
+            fp = StringIO(fp)
+        try:
+            mutter("FTP put: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            try:
+                f.storbinary('STOR '+tmp_abspath, fp)
+                f.rename(tmp_abspath, self._abspath(relpath))
+            except (ftplib.error_temp,EOFError), e:
+                warning("Failure during ftp PUT. Deleting temporary file.")
+                try:
+                    f.delete(tmp_abspath)
+                except:
+                    warning("Failed to delete temporary file on the server.\nFile: %s"
+                            % tmp_abspath)
+                    raise e
+                raise
+        except ftplib.error_perm, e:
+            if "no such file" in str(e).lower():
+                raise NoSuchFile("Error storing %s: %s"
+                                 % (self.abspath(relpath), str(e)), extra=e)
+            else:
+                raise FtpTransportError(orig_error=e)
+        except ftplib.error_temp, e:
+            if retries > _number_of_retries:
+                raise TransportError("FTP temporary error during PUT %s. Aborting."
+                                     % self.abspath(relpath), orig_error=e)
+            else:
+                warning("FTP temporary error: %s. Retrying." % str(e))
+                self._FTP_instance = None
+                self.put(relpath, fp, mode, retries+1)
+        except EOFError:
+            if retries > _number_of_retries:
+                raise TransportError("FTP control connection closed during PUT %s."
+                                     % self.abspath(relpath), orig_error=e)
+            else:
+                warning("FTP control connection closed. Trying to reopen.")
+                time.sleep(_sleep_between_retries)
+                self._FTP_instance = None
+                self.put(relpath, fp, mode, retries+1)
+
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        try:
+            mutter("FTP mkd: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            try:
+                f.mkd(self._abspath(relpath))
+            except ftplib.error_perm, e:
+                s = str(e)
+                if 'File exists' in s:
+                    raise FileExists(self.abspath(relpath), extra=s)
+                else:
+                    raise
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def rmdir(self, rel_path):
+        """Delete the directory at rel_path"""
+        try:
+            mutter("FTP rmd: %s" % self._abspath(rel_path))
+
+            f = self._get_FTP()
+            f.rmd(self._abspath(rel_path))
+        except ftplib.error_perm, e:
+            if str(e).endswith("Directory not empty"):
+                raise DirectoryNotEmpty(self._abspath(rel_path), extra=str(e))
+            else:
+                raise TransportError(msg="Cannot remove directory at %s" % \
+                        self._abspath(rel_path), extra=str(e))
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        raise TransportNotPossible('ftp does not support append()')
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('ftp does not (yet) support copy()')
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        try:
+            mutter("FTP mv: %s => %s" % (self._abspath(rel_from),
+                                         self._abspath(rel_to)))
+            f = self._get_FTP()
+            f.rename(self._abspath(rel_from), self._abspath(rel_to))
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    rename = move
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        try:
+            mutter("FTP rm: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            f.delete(self._abspath(relpath))
+        except ftplib.error_perm, e:
+            if str(e).endswith("No such file or directory"):
+                raise NoSuchFile(self._abspath(relpath), extra=str(e))
+            else:
+                raise TransportError(orig_error=e)
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def list_dir(self, relpath):
+        """See Transport.list_dir."""
+        try:
+            mutter("FTP nlst: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            basepath = self._abspath(relpath)
+            # FTP.nlst returns paths prefixed by relpath, strip 'em
+            the_list = f.nlst(basepath)
+            stripped = [path[len(basepath)+1:] for path in the_list]
+            # Remove . and .. if present, and return
+            return [path for path in stripped if path not in (".", "..")]
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def iter_files_recursive(self):
+        """See Transport.iter_files_recursive.
+
+        This is cargo-culted from the SFTP transport"""
+        mutter("FTP iter_files_recursive")
+        queue = list(self.list_dir("."))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+"/"+basename)
+            else:
+                yield relpath
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        try:
+            mutter("FTP stat: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            return FtpStatResult(f, self._abspath(relpath))
+        except ftplib.error_perm, e:
+            if "no such file" in str(e).lower():
+                raise NoSuchFile("Error storing %s: %s"
+                                 % (self.abspath(relpath), str(e)), extra=e)
+            else:
+                raise FtpTransportError(orig_error=e)
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # The old RemoteBranch ignore lock for reading, so we will
+        # continue that tradition and return a bogus lock object.
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        return self.lock_read(relpath)
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    warn("There are no FTP transport provider tests yet.")
+    return []

=== added directory 'bzrlib/transport/http'
=== added file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http/__init__.py	2006-04-05 05:43:06 +0000
@@ -0,0 +1,480 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Base implementation of Transport over http.
+
+There are separate implementation modules for each http client implementation.
+"""
+
+import errno
+import os
+from collections import deque
+from cStringIO import StringIO
+import re
+import urlparse
+import urllib
+from warnings import warn
+
+from bzrlib.transport import Transport, register_transport, Server
+from bzrlib.errors import (TransportNotPossible, NoSuchFile,
+                           TransportError, ConnectionError)
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter
+# TODO: load these only when running http tests
+import BaseHTTPServer, SimpleHTTPServer, socket, time
+import threading
+from bzrlib.ui import ui_factory
+
+
+def extract_auth(url, password_manager):
+    """Extract auth parameters from am HTTP/HTTPS url and add them to the given
+    password manager.  Return the url, minus those auth parameters (which
+    confuse urllib2).
+    """
+    assert re.match(r'^(https?)(\+\w+)?://', url), \
+            'invalid absolute url %r' % url
+    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
+    
+    if '@' in netloc:
+        auth, netloc = netloc.split('@', 1)
+        if ':' in auth:
+            username, password = auth.split(':', 1)
+        else:
+            username, password = auth, None
+        if ':' in netloc:
+            host = netloc.split(':', 1)[0]
+        else:
+            host = netloc
+        username = urllib.unquote(username)
+        if password is not None:
+            password = urllib.unquote(password)
+        else:
+            password = ui_factory.get_password(prompt='HTTP %(user)@%(host) password',
+                                               user=username, host=host)
+        password_manager.add_password(None, host, username, password)
+    url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
+    return url
+
+
+class HttpTransportBase(Transport):
+    """Base class for http implementations.
+
+    Does URL parsing, etc, but not any network IO.
+
+    The protocol can be given as e.g. http+urllib://host/ to use a particular
+    implementation.
+    """
+
+    # _proto: "http" or "https"
+    # _qualified_proto: may have "+pycurl", etc
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        proto_match = re.match(r'^(https?)(\+\w+)?://', base)
+        if not proto_match:
+            raise AssertionError("not a http url: %r" % base)
+        self._proto = proto_match.group(1)
+        impl_name = proto_match.group(2)
+        if impl_name:
+            impl_name = impl_name[1:]
+        self._impl_name = impl_name
+        if base[-1] != '/':
+            base = base + '/'
+        super(HttpTransportBase, self).__init__(base)
+        # In the future we might actually connect to the remote host
+        # rather than using get_url
+        # self._connection = None
+        (apparent_proto, self._host,
+            self._path, self._parameters,
+            self._query, self._fragment) = urlparse.urlparse(self.base)
+        self._qualified_proto = apparent_proto
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+
+        This can be supplied with a string or a list.
+
+        The URL returned always has the protocol scheme originally used to 
+        construct the transport, even if that includes an explicit
+        implementation qualifier.
+        """
+        assert isinstance(relpath, basestring)
+        if isinstance(relpath, basestring):
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+            if relpath_parts[-1] == '':
+                raise ValueError("path %r within branch %r seems to be a directory"
+                                 % (relpath, self._path))
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+        for p in relpath_parts:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.' or p == '':
+                continue # No-op
+            else:
+                basepath.append(p)
+        # Possibly, we could use urlparse.urljoin() here, but
+        # I'm concerned about when it chooses to strip the last
+        # portion of the path, and when it doesn't.
+        path = '/'.join(basepath)
+        if path == '':
+            path = '/'
+        result = urlparse.urlunparse((self._qualified_proto,
+                                    self._host, path, '', '', ''))
+        return result
+
+    def _real_abspath(self, relpath):
+        """Produce absolute path, adjusting protocol if needed"""
+        abspath = self.abspath(relpath)
+        qp = self._qualified_proto
+        rp = self._proto
+        if self._qualified_proto != self._proto:
+            abspath = rp + abspath[len(qp):]
+        if not isinstance(abspath, str):
+            # escaping must be done at a higher level
+            abspath = abspath.encode('ascii')
+        return abspath
+
+    def has(self, relpath):
+        raise NotImplementedError("has() is abstract on %r" % self)
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        code, response_file = self._get(relpath, None)
+        return response_file
+
+    def _get(self, relpath, ranges):
+        """Get a file, or part of a file.
+
+        :param relpath: Path relative to transport base URL
+        :param byte_range: None to get the whole file;
+            or [(start,end)] to fetch parts of a file.
+
+        :returns: (http_code, result_file)
+
+        Note that the current http implementations can only fetch one range at
+        a time through this call.
+        """
+        raise NotImplementedError(self._get)
+
+    def readv(self, relpath, offsets):
+        """Get parts of the file at the given relative path.
+
+        :param offsets: A list of (offset, size) tuples.
+        :param return: A list or generator of (offset, data) tuples
+        """
+        # Ideally we would pass one big request asking for all the ranges in
+        # one go; however then the server will give a multipart mime response
+        # back, and we can't parse them yet.  So instead we just get one range
+        # per region, and try to coallesce the regions as much as possible.
+        #
+        # The read-coallescing code is not quite regular enough to have a
+        # single driver routine and
+        # helper method in Transport.
+        def do_combined_read(combined_offsets):
+            # read one coalesced block
+            total_size = 0
+            for offset, size in combined_offsets:
+                total_size += size
+            mutter('readv coalesced %d reads.', len(combined_offsets))
+            offset = combined_offsets[0][0]
+            byte_range = (offset, offset + total_size - 1)
+            code, result_file = self._get(relpath, [byte_range])
+            if code == 206:
+                for off, size in combined_offsets:
+                    result_bytes = result_file.read(size)
+                    assert len(result_bytes) == size
+                    yield off, result_bytes
+            elif code == 200:
+                data = result_file.read(offset + total_size)[offset:offset + total_size]
+                pos = 0
+                for offset, size in combined_offsets:
+                    yield offset, data[pos:pos + size]
+                    pos += size
+                del data
+        if not len(offsets):
+            return
+        pending_offsets = deque(offsets)
+        combined_offsets = []
+        while len(pending_offsets):
+            offset, size = pending_offsets.popleft()
+            if not combined_offsets:
+                combined_offsets = [[offset, size]]
+            else:
+                if (len (combined_offsets) < 500 and
+                    combined_offsets[-1][0] + combined_offsets[-1][1] == offset):
+                    # combatible offset:
+                    combined_offsets.append([offset, size])
+                else:
+                    # incompatible, or over the threshold issue a read and yield
+                    pending_offsets.appendleft((offset, size))
+                    for result in do_combined_read(combined_offsets):
+                        yield result
+                    combined_offsets = []
+        # whatever is left is a single coalesced request
+        if len(combined_offsets):
+            for result in do_combined_read(combined_offsets):
+                yield result
+
+    def put(self, relpath, f, mode=None):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        raise TransportNotPossible('http PUT not supported')
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        raise TransportNotPossible('http does not support mkdir()')
+
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        raise TransportNotPossible('http does not support rmdir()')
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        raise TransportNotPossible('http does not support append()')
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('http does not support copy()')
+
+    def copy_to(self, relpaths, other, mode=None, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+
+        TODO: if other is LocalTransport, is it possible to
+              do better than put(get())?
+        """
+        # At this point HttpTransport might be able to check and see if
+        # the remote location is the same, and rather than download, and
+        # then upload, it could just issue a remote copy_this command.
+        if isinstance(other, HttpTransportBase):
+            raise TransportNotPossible('http cannot be the target of copy_to()')
+        else:
+            return super(HttpTransportBase, self).\
+                    copy_to(relpaths, other, mode=mode, pb=pb)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('http does not support move()')
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise TransportNotPossible('http does not support delete()')
+
+    def is_readonly(self):
+        """See Transport.is_readonly."""
+        return True
+
+    def listable(self):
+        """See Transport.listable."""
+        return False
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        raise TransportNotPossible('http does not support stat()')
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # The old RemoteBranch ignore lock for reading, so we will
+        # continue that tradition and return a bogus lock object.
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        raise TransportNotPossible('http does not support lock_write()')
+
+    def clone(self, offset=None):
+        """Return a new HttpTransportBase with root at self.base + offset
+        For now HttpTransportBase does not actually connect, so just return
+        a new HttpTransportBase object.
+        """
+        if offset is None:
+            return self.__class__(self.base)
+        else:
+            return self.__class__(self.abspath(offset))
+
+#---------------- test server facilities ----------------
+# TODO: load these only when running tests
+
+
+class WebserverNotAvailable(Exception):
+    pass
+
+
+class BadWebserverPath(ValueError):
+    def __str__(self):
+        return 'path %s is not in %s' % self.args
+
+
+class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
+
+    def log_message(self, format, *args):
+        self.server.test_case.log('webserver - %s - - [%s] %s "%s" "%s"',
+                                  self.address_string(),
+                                  self.log_date_time_string(),
+                                  format % args,
+                                  self.headers.get('referer', '-'),
+                                  self.headers.get('user-agent', '-'))
+
+    def handle_one_request(self):
+        """Handle a single HTTP request.
+
+        You normally don't need to override this method; see the class
+        __doc__ string for information on how to handle specific HTTP
+        commands such as GET and POST.
+
+        """
+        for i in xrange(1,11): # Don't try more than 10 times
+            try:
+                self.raw_requestline = self.rfile.readline()
+            except socket.error, e:
+                if e.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
+                    # omitted for now because some tests look at the log of
+                    # the server and expect to see no errors.  see recent
+                    # email thread. -- mbp 20051021. 
+                    ## self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
+                    time.sleep(0.01)
+                    continue
+                raise
+            else:
+                break
+        if not self.raw_requestline:
+            self.close_connection = 1
+            return
+        if not self.parse_request(): # An error code has been sent, just exit
+            return
+        mname = 'do_' + self.command
+        if not hasattr(self, mname):
+            self.send_error(501, "Unsupported method (%r)" % self.command)
+            return
+        method = getattr(self, mname)
+        method()
+
+
+class TestingHTTPServer(BaseHTTPServer.HTTPServer):
+    def __init__(self, server_address, RequestHandlerClass, test_case):
+        BaseHTTPServer.HTTPServer.__init__(self, server_address,
+                                                RequestHandlerClass)
+        self.test_case = test_case
+
+class HttpServer(Server):
+    """A test server for http transports."""
+
+    # used to form the url that connects to this server
+    _url_protocol = 'http'
+
+    def _http_start(self):
+        httpd = None
+        httpd = TestingHTTPServer(('localhost', 0),
+                                  TestingHTTPRequestHandler,
+                                  self)
+        host, port = httpd.socket.getsockname()
+        self._http_base_url = '%s://localhost:%s/' % (self._url_protocol, port)
+        self._http_starting.release()
+        httpd.socket.settimeout(0.1)
+
+        while self._http_running:
+            try:
+                httpd.handle_request()
+            except socket.timeout:
+                pass
+
+    def _get_remote_url(self, path):
+        path_parts = path.split(os.path.sep)
+        if os.path.isabs(path):
+            if path_parts[:len(self._local_path_parts)] != \
+                   self._local_path_parts:
+                raise BadWebserverPath(path, self.test_dir)
+            remote_path = '/'.join(path_parts[len(self._local_path_parts):])
+        else:
+            remote_path = '/'.join(path_parts)
+
+        self._http_starting.acquire()
+        self._http_starting.release()
+        return self._http_base_url + remote_path
+
+    def log(self, format, *args):
+        """Capture Server log output."""
+        self.logs.append(format % args)
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        self._home_dir = os.getcwdu()
+        self._local_path_parts = self._home_dir.split(os.path.sep)
+        self._http_starting = threading.Lock()
+        self._http_starting.acquire()
+        self._http_running = True
+        self._http_base_url = None
+        self._http_thread = threading.Thread(target=self._http_start)
+        self._http_thread.setDaemon(True)
+        self._http_thread.start()
+        self._http_proxy = os.environ.get("http_proxy")
+        if self._http_proxy is not None:
+            del os.environ["http_proxy"]
+        self.logs = []
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        self._http_running = False
+        self._http_thread.join()
+        if self._http_proxy is not None:
+            import os
+            os.environ["http_proxy"] = self._http_proxy
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_remote_url(self._home_dir)
+        
+    def get_bogus_url(self):
+        """See bzrlib.transport.Server.get_bogus_url."""
+        # this is chosen to try to prevent trouble with proxies, wierd dns,
+        # etc
+        return 'http://127.0.0.1:1/'
+

=== added file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-04-06 05:28:00 +0000
@@ -0,0 +1,164 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""http/https transport using pycurl"""
+
+# TODO: test reporting of http errors
+
+# TODO: Transport option to control caching of particular requests; broadly we
+# would want to offer "caching allowed" or "must revalidate", depending on
+# whether we expect a particular file will be modified after it's committed.
+# It's probably safer to just always revalidate.  mbp 20060321
+
+import os
+from StringIO import StringIO
+
+import bzrlib
+from bzrlib.errors import (TransportNotPossible, NoSuchFile,
+                           TransportError, ConnectionError,
+                           DependencyNotPresent)
+from bzrlib.trace import mutter
+from bzrlib.transport import register_urlparse_netloc_protocol
+from bzrlib.transport.http import HttpTransportBase, extract_auth, HttpServer
+
+try:
+    import pycurl
+except ImportError, e:
+    mutter("failed to import pycurl: %s", e)
+    raise DependencyNotPresent('pycurl', e)
+
+
+register_urlparse_netloc_protocol('http+pycurl')
+
+
+class PyCurlTransport(HttpTransportBase):
+    """http client transport using pycurl
+
+    PyCurl is a Python binding to the C "curl" multiprotocol client.
+
+    This transport can be significantly faster than the builtin Python client. 
+    Advantages include: DNS caching, connection keepalive, and ability to 
+    set headers to allow caching.
+    """
+
+    def __init__(self, base):
+        super(PyCurlTransport, self).__init__(base)
+        mutter('using pycurl %s' % pycurl.version)
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def has(self, relpath):
+        curl = pycurl.Curl()
+        abspath = self._real_abspath(relpath)
+        curl.setopt(pycurl.URL, abspath)
+        curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
+        self._set_curl_options(curl)
+        # don't want the body - ie just do a HEAD request
+        curl.setopt(pycurl.NOBODY, 1)
+        self._curl_perform(curl)
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        if code == 404: # not found
+            return False
+        elif code in (200, 302): # "ok", "found"
+            return True
+        elif code == 0:
+            self._raise_curl_connection_error(curl)
+        else:
+            self._raise_curl_http_error(curl)
+        
+    def _get(self, relpath, ranges):
+        curl = pycurl.Curl()
+        abspath = self._real_abspath(relpath)
+        sio = StringIO()
+        curl.setopt(pycurl.URL, abspath)
+        self._set_curl_options(curl)
+        curl.setopt(pycurl.WRITEFUNCTION, sio.write)
+        curl.setopt(pycurl.NOBODY, 0)
+        if ranges is not None:
+            assert len(ranges) == 1
+            # multiple ranges not supported yet because we can't decode the
+            # response
+            curl.setopt(pycurl.RANGE, '%d-%d' % ranges[0])
+        self._curl_perform(curl)
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        if code == 404:
+            raise NoSuchFile(abspath)
+        elif code == 200:
+            sio.seek(0)
+            return code, sio
+        elif code == 206 and (ranges is not None):
+            sio.seek(0)
+            return code, sio
+        elif code == 0:
+            self._raise_curl_connection_error(curl)
+        else:
+            self._raise_curl_http_error(curl)
+
+    def _raise_curl_connection_error(self, curl):
+        curl_errno = curl.getinfo(pycurl.OS_ERRNO)
+        url = curl.getinfo(pycurl.EFFECTIVE_URL)
+        raise ConnectionError('curl connection error (%s) on %s'
+                              % (os.strerror(curl_errno), url))
+
+    def _raise_curl_http_error(self, curl):
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        url = curl.getinfo(pycurl.EFFECTIVE_URL)
+        raise TransportError('http error %d probing for %s' %
+                             (code, url))
+
+    def _set_curl_options(self, curl):
+        """Set options for all requests"""
+        # There's no way in http/1.0 to say "must revalidate"; we don't want
+        # to force it to always retrieve.  so just turn off the default Pragma
+        # provided by Curl.
+        headers = ['Cache-control: max-age=0',
+                   'Pragma: no-cache']
+        ## curl.setopt(pycurl.VERBOSE, 1)
+        # TODO: maybe include a summary of the pycurl version
+        ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__)
+        curl.setopt(pycurl.USERAGENT, ua_str)
+        curl.setopt(pycurl.HTTPHEADER, headers)
+        curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
+
+    def _curl_perform(self, curl):
+        """Perform curl operation and translate exceptions."""
+        try:
+            curl.perform()
+        except pycurl.error, e:
+            # XXX: There seem to be no symbolic constants for these values.
+            if e[0] == 6:
+                # couldn't resolve host
+                raise NoSuchFile(curl.getinfo(pycurl.EFFECTIVE_URL), e)
+
+
+class HttpServer_PyCurl(HttpServer):
+    """Subclass of HttpServer that gives http+pycurl urls.
+
+    This is for use in testing: connections to this server will always go
+    through pycurl where possible.
+    """
+
+    # urls returned by this server should require the pycurl client impl
+    _url_protocol = 'http+pycurl'
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(PyCurlTransport, HttpServer_PyCurl),
+            ]

=== added file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http/_urllib.py	2006-04-06 05:28:00 +0000
@@ -0,0 +1,166 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import urllib, urllib2
+
+import bzrlib  # for the version
+from bzrlib.errors import BzrError
+from bzrlib.trace import mutter
+from bzrlib.transport import register_urlparse_netloc_protocol
+from bzrlib.transport.http import HttpTransportBase, extract_auth, HttpServer
+from bzrlib.errors import (TransportNotPossible, NoSuchFile,
+                           TransportError, ConnectionError)
+
+
+register_urlparse_netloc_protocol('http+urllib')
+
+
+class Request(urllib2.Request):
+    """Request object for urllib2 that allows the method to be overridden."""
+
+    method = None
+
+    def get_method(self):
+        if self.method is not None:
+            return self.method
+        else:
+            return urllib2.Request.get_method(self)
+
+
+class HttpTransport_urllib(HttpTransportBase):
+    """Python urllib transport for http and https.
+    """
+
+    # TODO: Implement pipelined versions of all of the *_multi() functions.
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        super(HttpTransport_urllib, self).__init__(base)
+
+    def _get(self, relpath, ranges):
+        path = relpath
+        try:
+            path = self._real_abspath(relpath)
+            response = self._get_url_impl(path, method='GET', ranges=ranges)
+            return response.code, response
+        except urllib2.HTTPError, e:
+            mutter('url error code: %s for has url: %r', e.code, path)
+            if e.code == 404:
+                raise NoSuchFile(path, extra=e)
+            raise
+        except (BzrError, IOError), e:
+            if hasattr(e, 'errno'):
+                mutter('io error: %s %s for has url: %r',
+                    e.errno, errno.errorcode.get(e.errno), path)
+                if e.errno == errno.ENOENT:
+                    raise NoSuchFile(path, extra=e)
+            raise ConnectionError(msg = "Error retrieving %s: %s" 
+                             % (self.abspath(relpath), str(e)),
+                             orig_error=e)
+
+    def _get_url_impl(self, url, method, ranges):
+        """Actually pass get request into urllib
+
+        :returns: urllib Response object
+        """
+        if ranges:
+            range_string = ranges
+        else:
+            range_string = 'all'
+        mutter("get_url %s [%s]" % (url, range_string))
+        manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+        url = extract_auth(url, manager)
+        auth_handler = urllib2.HTTPBasicAuthHandler(manager)
+        opener = urllib2.build_opener(auth_handler)
+        request = Request(url)
+        request.method = method
+        request.add_header('Pragma', 'no-cache')
+        request.add_header('Cache-control', 'max-age=0')
+        request.add_header('User-Agent', 'bzr/%s (urllib)' % bzrlib.__version__)
+        if ranges:
+            assert len(ranges) == 1
+            request.add_header('Range', 'bytes=%d-%d' % ranges[0])
+        response = opener.open(request)
+        return response
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def has(self, relpath):
+        """Does the target location exist?
+        """
+        abspath = self._real_abspath(relpath)
+        try:
+            f = self._get_url_impl(abspath, 'HEAD', [])
+            # Without the read and then close()
+            # we tend to have busy sockets.
+            f.read()
+            f.close()
+            return True
+        except urllib2.URLError, e:
+            mutter('url error code: %s for has url: %r', e.code, abspath)
+            if e.code == 404:
+                return False
+            raise
+        except IOError, e:
+            mutter('io error: %s %s for has url: %r',
+                e.errno, errno.errorcode.get(e.errno), abspath)
+            if e.errno == errno.ENOENT:
+                return False
+            raise TransportError(orig_error=e)
+
+    def copy_to(self, relpaths, other, mode=None, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+
+        TODO: if other is LocalTransport, is it possible to
+              do better than put(get())?
+        """
+        # At this point HttpTransport_urllib might be able to check and see if
+        # the remote location is the same, and rather than download, and
+        # then upload, it could just issue a remote copy_this command.
+        if isinstance(other, HttpTransport_urllib):
+            raise TransportNotPossible('http cannot be the target of copy_to()')
+        else:
+            return super(HttpTransport_urllib, self).copy_to(relpaths, other, mode=mode, pb=pb)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('http does not support move()')
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise TransportNotPossible('http does not support delete()')
+
+
+class HttpServer_urllib(HttpServer):
+    """Subclass of HttpServer that gives http+urllib urls.
+
+    This is for use in testing: connections to this server will always go
+    through urllib where possible.
+    """
+
+    # urls returned by this server should require the urllib client impl
+    _url_protocol = 'http+urllib'
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(HttpTransport_urllib, HttpServer_urllib),
+            ]

=== added file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/local.py	2006-04-12 04:45:32 +0000
@@ -0,0 +1,334 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Transport for the local filesystem.
+
+This is a fairly thin wrapper on regular file IO."""
+
+import os
+import shutil
+import sys
+from stat import ST_MODE, S_ISDIR, ST_SIZE
+import tempfile
+import urllib
+
+from bzrlib.trace import mutter
+from bzrlib.transport import Transport, Server
+from bzrlib.osutils import abspath, realpath, normpath, pathjoin, rename
+
+
+class LocalTransport(Transport):
+    """This is the transport agent for local filesystem access."""
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        if base.startswith('file://'):
+            base = base[len('file://'):]
+        # realpath is incompatible with symlinks. When we traverse
+        # up we might be able to normpath stuff. RBC 20051003
+        base = normpath(abspath(base))
+        if base[-1] != '/':
+            base = base + '/'
+        super(LocalTransport, self).__init__(base)
+
+    def should_cache(self):
+        return False
+
+    def clone(self, offset=None):
+        """Return a new LocalTransport with root at self.base + offset
+        Because the local filesystem does not require a connection, 
+        we can just return a new object.
+        """
+        if offset is None:
+            return LocalTransport(self.base)
+        else:
+            return LocalTransport(self.abspath(offset))
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative URL."""
+        assert isinstance(relpath, basestring), (type(relpath), relpath)
+        result = normpath(pathjoin(self.base, urllib.unquote(relpath)))
+        #if result[-1] != '/':
+        #    result += '/'
+        return result
+
+    def relpath(self, abspath):
+        """Return the local path portion from a given absolute path.
+        """
+        from bzrlib.osutils import relpath
+        if abspath is None:
+            abspath = u'.'
+        if len(abspath) > 1 and abspath.endswith('/'):
+            abspath = abspath[:-1]
+        if self.base == '/':
+            root = '/'
+        else:
+            root = self.base[:-1]
+        return relpath(root, abspath)
+
+    def has(self, relpath):
+        return os.access(self.abspath(relpath), os.F_OK)
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            path = self.abspath(relpath)
+            return open(path, 'rb')
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+
+    def put(self, relpath, f, mode=None):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        from bzrlib.atomicfile import AtomicFile
+
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            fp = AtomicFile(path, 'wb', new_mode=mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+        try:
+            self._pump(f, fp)
+            fp.commit()
+        finally:
+            fp.close()
+
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree."""
+        queue = list(self.list_dir(u'.'))
+        while queue:
+            relpath = queue.pop(0)
+            st = self.stat(relpath)
+            if S_ISDIR(st[ST_MODE]):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            os.mkdir(path)
+            if mode is not None:
+                os.chmod(path, mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        try:
+            fp = open(self.abspath(relpath), 'ab')
+        except (IOError, OSError),e:
+            self._translate_error(e, relpath)
+        # win32 workaround (tell on an unwritten file returns 0)
+        fp.seek(0, 2)
+        result = fp.tell()
+        self._pump(f, fp)
+        return result
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        import shutil
+        path_from = self.abspath(rel_from)
+        path_to = self.abspath(rel_to)
+        try:
+            shutil.copy(path_from, path_to)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path_from)
+
+    def rename(self, rel_from, rel_to):
+        path_from = self.abspath(rel_from)
+        try:
+            # *don't* call bzrlib.osutils.rename, because we want to 
+            # detect errors on rename
+            os.rename(path_from, self.abspath(rel_to))
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path_from)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self.abspath(rel_from)
+        path_to = self.abspath(rel_to)
+
+        try:
+            # this version will delete the destination if necessary
+            rename(path_from, path_to)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path_from)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            os.remove(path)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path)
+
+    def copy_to(self, relpaths, other, mode=None, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        """
+        if isinstance(other, LocalTransport):
+            # Both from & to are on the local filesystem
+            # Unfortunately, I can't think of anything faster than just
+            # copying them across, one by one :(
+            import shutil
+
+            total = self._get_total(relpaths)
+            count = 0
+            for path in relpaths:
+                self._update_pb(pb, 'copy-to', count, total)
+                try:
+                    mypath = self.abspath(path)
+                    otherpath = other.abspath(path)
+                    shutil.copy(mypath, otherpath)
+                    if mode is not None:
+                        os.chmod(otherpath, mode)
+                except (IOError, OSError),e:
+                    self._translate_error(e, path)
+                count += 1
+            return count
+        else:
+            return super(LocalTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def list_dir(self, relpath):
+        """Return a list of all files at the given location.
+        WARNING: many transports do not support this, so trying avoid using
+        it if at all possible.
+        """
+        path = self.abspath(relpath)
+        try:
+            return [urllib.quote(entry) for entry in os.listdir(path)]
+        except (IOError, OSError), e:
+            self._translate_error(e, path)
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            return os.stat(path)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        from bzrlib.lock import ReadLock
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            return ReadLock(path)
+        except (IOError, OSError), e:
+            self._translate_error(e, path)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        from bzrlib.lock import WriteLock
+        return WriteLock(self.abspath(relpath))
+
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        path = relpath
+        try:
+            path = self.abspath(relpath)
+            os.rmdir(path)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+
+    def _can_roundtrip_unix_modebits(self):
+        if sys.platform == 'win32':
+            # anyone else?
+            return False
+        else:
+            return True
+
+
+class ScratchTransport(LocalTransport):
+    """A transport that works in a temporary dir and cleans up after itself.
+    
+    The dir only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+
+    def __init__(self, base=None):
+        if base is None:
+            base = tempfile.mkdtemp()
+        super(ScratchTransport, self).__init__(base)
+
+    def __del__(self):
+        shutil.rmtree(self.base, ignore_errors=True)
+        mutter("%r destroyed" % self)
+
+
+class LocalRelpathServer(Server):
+    """A pretend server for local transports, using relpaths."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        return "."
+
+
+class LocalAbspathServer(Server):
+    """A pretend server for local transports, using absolute paths."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        return os.path.abspath("")
+
+
+class LocalURLServer(Server):
+    """A pretend server for local transports, using file:// urls."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        # FIXME: \ to / on windows
+        return "file://%s" % os.path.abspath("")
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(LocalTransport, LocalRelpathServer),
+            (LocalTransport, LocalAbspathServer),
+            (LocalTransport, LocalURLServer),
+            ]

=== added file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/memory.py	2006-04-05 05:43:06 +0000
@@ -0,0 +1,295 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport that uses memory for its storage.
+
+The contents of the transport will be lost when the object is discarded,
+so this is primarily useful for testing.
+"""
+
+from copy import copy
+import os
+import errno
+import re
+from stat import *
+from cStringIO import StringIO
+
+from bzrlib.trace import mutter
+from bzrlib.errors import TransportError, NoSuchFile, FileExists, LockError
+from bzrlib.transport import Transport, register_transport, Server
+
+
+class MemoryStat(object):
+
+    def __init__(self, size, is_dir, perms):
+        self.st_size = size
+        if not is_dir:
+            if perms is None:
+                perms = 0644
+            self.st_mode = S_IFREG | perms
+        else:
+            if perms is None:
+                perms = 0755
+            self.st_mode = S_IFDIR | perms
+
+
+class MemoryTransport(Transport):
+    """This is an in memory file system for transient data storage."""
+
+    def __init__(self, url=""):
+        """Set the 'base' path where files will be stored."""
+        if url == "":
+            url = "memory:/"
+        if url[-1] != '/':
+            url = url + '/'
+        super(MemoryTransport, self).__init__(url)
+        self._cwd = url[url.find(':') + 1:]
+        # dictionaries from absolute path to file mode
+        self._dirs = {}
+        self._files = {}
+        self._locks = {}
+
+    def clone(self, offset=None):
+        """See Transport.clone()."""
+        if offset is None or offset == '':
+            return copy(self)
+        segments = offset.split('/')
+        cwdsegments = self._cwd.split('/')[:-1]
+        while len(segments):
+            segment = segments.pop(0)
+            if segment == '.':
+                continue
+            if segment == '..':
+                if len(cwdsegments) > 1:
+                    cwdsegments.pop()
+                continue
+            cwdsegments.append(segment)
+        url = self.base[:self.base.find(':') + 1] + '/'.join(cwdsegments) + '/'
+        result = MemoryTransport(url)
+        result._dirs = self._dirs
+        result._files = self._files
+        result._locks = self._locks
+        return result
+
+    def abspath(self, relpath):
+        """See Transport.abspath()."""
+        # while a little slow, this is sufficiently fast to not matter in our
+        # current environment - XXX RBC 20060404 move the clone '..' handling
+        # into here and call abspath from clone
+        temp_t = self.clone(relpath)
+        if temp_t.base.count('/') == 1:
+            return temp_t.base
+        else:
+            return temp_t.base[:-1]
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        orig_content, orig_mode = self._files.get(_abspath, ("", None))
+        self._files[_abspath] = (orig_content + f.read(), orig_mode)
+        return len(orig_content)
+
+    def _check_parent(self, _abspath):
+        dir = os.path.dirname(_abspath)
+        if dir != '/':
+            if not dir in self._dirs:
+                raise NoSuchFile(_abspath)
+
+    def has(self, relpath):
+        """See Transport.has()."""
+        _abspath = self._abspath(relpath)
+        return _abspath in self._files or _abspath in self._dirs
+
+    def delete(self, relpath):
+        """See Transport.delete()."""
+        _abspath = self._abspath(relpath)
+        if not _abspath in self._files:
+            raise NoSuchFile(relpath)
+        del self._files[_abspath]
+
+    def get(self, relpath):
+        """See Transport.get()."""
+        _abspath = self._abspath(relpath)
+        if not _abspath in self._files:
+            raise NoSuchFile(relpath)
+        return StringIO(self._files[_abspath][0])
+
+    def put(self, relpath, f, mode=None):
+        """See Transport.put()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        self._files[_abspath] = (f.read(), mode)
+
+    def mkdir(self, relpath, mode=None):
+        """See Transport.mkdir()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        if _abspath in self._dirs:
+            raise FileExists(relpath)
+        self._dirs[_abspath]=mode
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def iter_files_recursive(self):
+        for file in self._files:
+            if file.startswith(self._cwd):
+                yield file[len(self._cwd):]
+    
+    def list_dir(self, relpath):
+        """See Transport.list_dir()."""
+        _abspath = self._abspath(relpath)
+        if _abspath != '/' and _abspath not in self._dirs:
+            raise NoSuchFile(relpath)
+        result = []
+        for path in self._files:
+            if (path.startswith(_abspath) and 
+                path[len(_abspath) + 1:].find('/') == -1 and
+                len(path) > len(_abspath)):
+                result.append(path[len(_abspath) + 1:])
+        for path in self._dirs:
+            if (path.startswith(_abspath) and 
+                path[len(_abspath) + 1:].find('/') == -1 and
+                len(path) > len(_abspath) and
+                path[len(_abspath)] == '/'):
+                result.append(path[len(_abspath) + 1:])
+        return result
+
+    def rename(self, rel_from, rel_to):
+        """Rename a file or directory; fail if the destination exists"""
+        abs_from = self._abspath(rel_from)
+        abs_to = self._abspath(rel_to)
+        def replace(x):
+            if x == abs_from:
+                x = abs_to
+            elif x.startswith(abs_from + '/'):
+                x = abs_to + x[len(abs_from):]
+            return x
+        def do_renames(container):
+            for path in container:
+                new_path = replace(path)
+                if new_path != path:
+                    if new_path in container:
+                        raise FileExists(new_path)
+                    container[new_path] = container[path]
+                    del container[path]
+        do_renames(self._files)
+        do_renames(self._dirs)
+    
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        _abspath = self._abspath(relpath)
+        if _abspath in self._files:
+            self._translate_error(IOError(errno.ENOTDIR, relpath), relpath)
+        for path in self._files:
+            if path.startswith(_abspath):
+                self._translate_error(IOError(errno.ENOTEMPTY, relpath),
+                                      relpath)
+        for path in self._dirs:
+            if path.startswith(_abspath) and path != _abspath:
+                self._translate_error(IOError(errno.ENOTEMPTY, relpath), relpath)
+        if not _abspath in self._dirs:
+            raise NoSuchFile(relpath)
+        del self._dirs[_abspath]
+
+    def stat(self, relpath):
+        """See Transport.stat()."""
+        _abspath = self._abspath(relpath)
+        if _abspath in self._files:
+            return MemoryStat(len(self._files[_abspath][0]), False, 
+                              self._files[_abspath][1])
+        elif _abspath == '':
+            return MemoryStat(0, True, None)
+        elif _abspath in self._dirs:
+            return MemoryStat(0, True, self._dirs[_abspath])
+        else:
+            raise NoSuchFile(_abspath)
+
+    def lock_read(self, relpath):
+        """See Transport.lock_read()."""
+        return _MemoryLock(self._abspath(relpath), self)
+
+    def lock_write(self, relpath):
+        """See Transport.lock_write()."""
+        return _MemoryLock(self._abspath(relpath), self)
+
+    def _abspath(self, relpath):
+        """Generate an internal absolute path."""
+        if relpath.find('..') != -1:
+            raise AssertionError('relpath contains ..')
+        if relpath == '.':
+            return self._cwd[:-1]
+        if relpath.endswith('/'):
+            relpath = relpath[:-1]
+        if relpath.startswith('./'):
+            relpath = relpath[2:]
+        return self._cwd + relpath
+
+
+class _MemoryLock(object):
+    """This makes a lock."""
+
+    def __init__(self, path, transport):
+        assert isinstance(transport, MemoryTransport)
+        self.path = path
+        self.transport = transport
+        if self.path in self.transport._locks:
+            raise LockError('File %r already locked' % (self.path,))
+        self.transport._locks[self.path] = self
+
+    def __del__(self):
+        # Should this warn, or actually try to cleanup?
+        if self.transport:
+            warn("MemoryLock %r not explicitly unlocked" % (self.path,))
+            self.unlock()
+
+    def unlock(self):
+        del self.transport._locks[self.path]
+        self.transport = None
+
+
+class MemoryServer(Server):
+    """Server for the MemoryTransport for testing with."""
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        self._dirs = {}
+        self._files = {}
+        self._locks = {}
+        self._scheme = "memory+%s:" % id(self)
+        def memory_factory(url):
+            result = MemoryTransport(url)
+            result._dirs = self._dirs
+            result._files = self._files
+            result._locks = self._locks
+            return result
+        register_transport(self._scheme, memory_factory)
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        # unregister this server
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._scheme
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(MemoryTransport, MemoryServer),
+            ]

=== added file 'bzrlib/transport/readonly.py'
--- bzrlib/transport/readonly.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/readonly.py	2006-04-02 22:42:19 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport that adapts another transport to be readonly."""
+
+from bzrlib.errors import TransportNotPossible
+from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
+
+
+class ReadonlyTransportDecorator(TransportDecorator):
+    """A decorator that can convert any transport to be readonly.
+
+    This is requested via the 'readonly+' prefix to get_transport().
+    """
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        raise TransportNotPossible('readonly transport')
+    
+    @classmethod
+    def _get_url_prefix(self):
+        """Readonly transport decorators are invoked via 'readonly+'"""
+        return 'readonly+'
+
+    def delete(self, relpath):
+        """See Transport.delete()."""
+        raise TransportNotPossible('readonly transport')
+
+    def delete_tree(self, relpath):
+        """See Transport.delete_tree()."""
+        raise TransportNotPossible('readonly transport')
+
+    def put(self, relpath, f, mode=None):
+        """See Transport.put()."""
+        raise TransportNotPossible('readonly transport')
+
+    def mkdir(self, relpath, mode=None):
+        """See Transport.mkdir()."""
+        raise TransportNotPossible('readonly transport')
+
+    def is_readonly(self):
+        """See Transport.is_readonly."""
+        return True
+
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        raise TransportNotPossible('readonly transport')
+
+    def lock_write(self, relpath):
+        """See Transport.lock_write."""
+        raise TransportNotPossible('readonly transport')
+
+
+class ReadonlyServer(DecoratorServer):
+    """Server for the ReadonlyTransportDecorator for testing with."""
+
+    def get_decorator_class(self):
+        return ReadonlyTransportDecorator
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(ReadonlyTransportDecorator, ReadonlyServer),
+            ]

=== added file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/sftp.py	2006-04-19 05:22:38 +0000
@@ -0,0 +1,1023 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport over SFTP, using paramiko."""
+
+import errno
+import getpass
+import os
+import random
+import re
+import stat
+import subprocess
+import sys
+import time
+import urllib
+import urlparse
+import weakref
+
+from bzrlib.config import config_dir, ensure_config_dir_exists
+from bzrlib.errors import (ConnectionError,
+                           FileExists, 
+                           TransportNotPossible, NoSuchFile, PathNotChild,
+                           TransportError,
+                           LockError, ParamikoNotPresent
+                           )
+from bzrlib.osutils import pathjoin, fancy_rename
+from bzrlib.trace import mutter, warning, error
+from bzrlib.transport import (
+    register_urlparse_netloc_protocol,
+    Server,
+    Transport,
+    urlescape,
+    )
+import bzrlib.ui
+
+try:
+    import paramiko
+except ImportError, e:
+    raise ParamikoNotPresent(e)
+else:
+    from paramiko.sftp import (SFTP_FLAG_WRITE, SFTP_FLAG_CREATE,
+                               SFTP_FLAG_EXCL, SFTP_FLAG_TRUNC,
+                               CMD_HANDLE, CMD_OPEN)
+    from paramiko.sftp_attr import SFTPAttributes
+    from paramiko.sftp_file import SFTPFile
+    from paramiko.sftp_client import SFTPClient
+
+
+register_urlparse_netloc_protocol('sftp')
+
+
+# don't use prefetch unless paramiko version >= 1.5.2 (there were bugs earlier)
+_default_do_prefetch = False
+if getattr(paramiko, '__version_info__', (0, 0, 0)) >= (1, 5, 2):
+    _default_do_prefetch = True
+
+
+_close_fds = True
+if sys.platform == 'win32':
+    # close_fds not supported on win32
+    _close_fds = False
+
+_ssh_vendor = None
+
+def _get_ssh_vendor():
+    """Find out what version of SSH is on the system."""
+    global _ssh_vendor
+    if _ssh_vendor is not None:
+        return _ssh_vendor
+
+    _ssh_vendor = 'none'
+
+    if 'BZR_SSH' in os.environ:
+        _ssh_vendor = os.environ['BZR_SSH']
+        if _ssh_vendor == 'paramiko':
+            _ssh_vendor = 'none'
+        return _ssh_vendor
+
+    try:
+        p = subprocess.Popen(['ssh', '-V'],
+                             close_fds=_close_fds,
+                             stdin=subprocess.PIPE,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        returncode = p.returncode
+        stdout, stderr = p.communicate()
+    except OSError:
+        returncode = -1
+        stdout = stderr = ''
+    if 'OpenSSH' in stderr:
+        mutter('ssh implementation is OpenSSH')
+        _ssh_vendor = 'openssh'
+    elif 'SSH Secure Shell' in stderr:
+        mutter('ssh implementation is SSH Corp.')
+        _ssh_vendor = 'ssh'
+
+    if _ssh_vendor != 'none':
+        return _ssh_vendor
+
+    # XXX: 20051123 jamesh
+    # A check for putty's plink or lsh would go here.
+
+    mutter('falling back to paramiko implementation')
+    return _ssh_vendor
+
+
+class SFTPSubprocess:
+    """A socket-like object that talks to an ssh subprocess via pipes."""
+    def __init__(self, hostname, vendor, port=None, user=None):
+        assert vendor in ['openssh', 'ssh']
+        if vendor == 'openssh':
+            args = ['ssh',
+                    '-oForwardX11=no', '-oForwardAgent=no',
+                    '-oClearAllForwardings=yes', '-oProtocol=2',
+                    '-oNoHostAuthenticationForLocalhost=yes']
+            if port is not None:
+                args.extend(['-p', str(port)])
+            if user is not None:
+                args.extend(['-l', user])
+            args.extend(['-s', hostname, 'sftp'])
+        elif vendor == 'ssh':
+            args = ['ssh', '-x']
+            if port is not None:
+                args.extend(['-p', str(port)])
+            if user is not None:
+                args.extend(['-l', user])
+            args.extend(['-s', 'sftp', hostname])
+
+        self.proc = subprocess.Popen(args, close_fds=_close_fds,
+                                     stdin=subprocess.PIPE,
+                                     stdout=subprocess.PIPE)
+
+    def send(self, data):
+        return os.write(self.proc.stdin.fileno(), data)
+
+    def recv_ready(self):
+        # TODO: jam 20051215 this function is necessary to support the
+        # pipelined() function. In reality, it probably should use
+        # poll() or select() to actually return if there is data
+        # available, otherwise we probably don't get any benefit
+        return True
+
+    def recv(self, count):
+        return os.read(self.proc.stdout.fileno(), count)
+
+    def close(self):
+        self.proc.stdin.close()
+        self.proc.stdout.close()
+        self.proc.wait()
+
+
+class LoopbackSFTP(object):
+    """Simple wrapper for a socket that pretends to be a paramiko Channel."""
+
+    def __init__(self, sock):
+        self.__socket = sock
+ 
+    def send(self, data):
+        return self.__socket.send(data)
+
+    def recv(self, n):
+        return self.__socket.recv(n)
+
+    def recv_ready(self):
+        return True
+
+    def close(self):
+        self.__socket.close()
+
+
+SYSTEM_HOSTKEYS = {}
+BZR_HOSTKEYS = {}
+
+# This is a weakref dictionary, so that we can reuse connections
+# that are still active. Long term, it might be nice to have some
+# sort of expiration policy, such as disconnect if inactive for
+# X seconds. But that requires a lot more fanciness.
+_connected_hosts = weakref.WeakValueDictionary()
+
+def clear_connection_cache():
+    """Remove all hosts from the SFTP connection cache.
+
+    Primarily useful for test cases wanting to force garbage collection.
+    """
+    _connected_hosts.clear()
+
+
+def load_host_keys():
+    """
+    Load system host keys (probably doesn't work on windows) and any
+    "discovered" keys from previous sessions.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    try:
+        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
+    except Exception, e:
+        mutter('failed to load system host keys: ' + str(e))
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+    try:
+        BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
+    except Exception, e:
+        mutter('failed to load bzr host keys: ' + str(e))
+        save_host_keys()
+
+
+def save_host_keys():
+    """
+    Save "discovered" host keys in $(config)/ssh_host_keys/.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+    ensure_config_dir_exists()
+
+    try:
+        f = open(bzr_hostkey_path, 'w')
+        f.write('# SSH host keys collected by bzr\n')
+        for hostname, keys in BZR_HOSTKEYS.iteritems():
+            for keytype, key in keys.iteritems():
+                f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
+        f.close()
+    except IOError, e:
+        mutter('failed to save bzr host keys: ' + str(e))
+
+
+class SFTPLock(object):
+    """This fakes a lock in a remote location."""
+    __slots__ = ['path', 'lock_path', 'lock_file', 'transport']
+    def __init__(self, path, transport):
+        assert isinstance(transport, SFTPTransport)
+
+        self.lock_file = None
+        self.path = path
+        self.lock_path = path + '.write-lock'
+        self.transport = transport
+        try:
+            # RBC 20060103 FIXME should we be using private methods here ?
+            abspath = transport._remote_path(self.lock_path)
+            self.lock_file = transport._sftp_open_exclusive(abspath)
+        except FileExists:
+            raise LockError('File %r already locked' % (self.path,))
+
+    def __del__(self):
+        """Should this warn, or actually try to cleanup?"""
+        if self.lock_file:
+            warning("SFTPLock %r not explicitly unlocked" % (self.path,))
+            self.unlock()
+
+    def unlock(self):
+        if not self.lock_file:
+            return
+        self.lock_file.close()
+        self.lock_file = None
+        try:
+            self.transport.delete(self.lock_path)
+        except (NoSuchFile,):
+            # What specific errors should we catch here?
+            pass
+
+class SFTPTransport (Transport):
+    """
+    Transport implementation for SFTP access.
+    """
+    _do_prefetch = _default_do_prefetch
+
+    def __init__(self, base, clone_from=None):
+        assert base.startswith('sftp://')
+        self._parse_url(base)
+        base = self._unparse_url()
+        if base[-1] != '/':
+            base += '/'
+        super(SFTPTransport, self).__init__(base)
+        if clone_from is None:
+            self._sftp_connect()
+        else:
+            # use the same ssh connection, etc
+            self._sftp = clone_from._sftp
+        # super saves 'self.base'
+    
+    def should_cache(self):
+        """
+        Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """
+        Return a new SFTPTransport with root at self.base + offset.
+        We share the same SFTP session between such transports, because it's
+        fairly expensive to set them up.
+        """
+        if offset is None:
+            return SFTPTransport(self.base, self)
+        else:
+            return SFTPTransport(self.abspath(offset), self)
+
+    def abspath(self, relpath):
+        """
+        Return the full url to the given relative path.
+        
+        @param relpath: the relative path or path components
+        @type relpath: str or list
+        """
+        return self._unparse_url(self._remote_path(relpath))
+    
+    def _remote_path(self, relpath):
+        """Return the path to be passed along the sftp protocol for relpath.
+        
+        relpath is a urlencoded string.
+        """
+        # FIXME: share the common code across transports
+        assert isinstance(relpath, basestring)
+        relpath = urllib.unquote(relpath).split('/')
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+
+        for p in relpath:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.':
+                continue # No-op
+            else:
+                basepath.append(p)
+
+        path = '/'.join(basepath)
+        return path
+
+    def relpath(self, abspath):
+        username, password, host, port, path = self._split_url(abspath)
+        error = []
+        if (username != self._username):
+            error.append('username mismatch')
+        if (host != self._host):
+            error.append('host mismatch')
+        if (port != self._port):
+            error.append('port mismatch')
+        if (not path.startswith(self._path)):
+            error.append('path mismatch')
+        if error:
+            extra = ': ' + ', '.join(error)
+            raise PathNotChild(abspath, self.base, extra=extra)
+        pl = len(self._path)
+        return path[pl:].strip('/')
+
+    def has(self, relpath):
+        """
+        Does the target location exist?
+        """
+        try:
+            self._sftp.stat(self._remote_path(relpath))
+            return True
+        except IOError:
+            return False
+
+    def get(self, relpath):
+        """
+        Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            path = self._remote_path(relpath)
+            f = self._sftp.file(path, mode='rb')
+            if self._do_prefetch and (getattr(f, 'prefetch', None) is not None):
+                f.prefetch()
+            return f
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': error retrieving')
+
+    def get_partial(self, relpath, start, length=None):
+        """
+        Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        # TODO: implement get_partial_multi to help with knit support
+        f = self.get(relpath)
+        f.seek(start)
+        if self._do_prefetch and hasattr(f, 'prefetch'):
+            f.prefetch()
+        return f
+
+    def put(self, relpath, f, mode=None):
+        """
+        Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        :param mode: The final mode for the file
+        """
+        final_path = self._remote_path(relpath)
+        self._put(final_path, f, mode=mode)
+
+    def _put(self, abspath, f, mode=None):
+        """Helper function so both put() and copy_abspaths can reuse the code"""
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (abspath, time.time(),
+                        os.getpid(), random.randint(0,0x7FFFFFFF))
+        fout = self._sftp_open_exclusive(tmp_abspath, mode=mode)
+        closed = False
+        try:
+            try:
+                fout.set_pipelined(True)
+                self._pump(f, fout)
+            except (IOError, paramiko.SSHException), e:
+                self._translate_io_exception(e, tmp_abspath)
+            if mode is not None:
+                self._sftp.chmod(tmp_abspath, mode)
+            fout.close()
+            closed = True
+            self._rename_and_overwrite(tmp_abspath, abspath)
+        except Exception, e:
+            # If we fail, try to clean up the temporary file
+            # before we throw the exception
+            # but don't let another exception mess things up
+            # Write out the traceback, because otherwise
+            # the catch and throw destroys it
+            import traceback
+            mutter(traceback.format_exc())
+            try:
+                if not closed:
+                    fout.close()
+                self._sftp.remove(tmp_abspath)
+            except:
+                # raise the saved except
+                raise e
+            # raise the original with its traceback if we can.
+            raise
+
+    def iter_files_recursive(self):
+        """Walk the relative paths of all files in this transport."""
+        queue = list(self.list_dir('.'))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        try:
+            path = self._remote_path(relpath)
+            # In the paramiko documentation, it says that passing a mode flag 
+            # will filtered against the server umask.
+            # StubSFTPServer does not do this, which would be nice, because it is
+            # what we really want :)
+            # However, real servers do use umask, so we really should do it that way
+            self._sftp.mkdir(path)
+            if mode is not None:
+                self._sftp.chmod(path, mode=mode)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, path, ': unable to mkdir',
+                failure_exc=FileExists)
+
+    def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):
+        """Translate a paramiko or IOError into a friendlier exception.
+
+        :param e: The original exception
+        :param path: The path in question when the error is raised
+        :param more_info: Extra information that can be included,
+                          such as what was going on
+        :param failure_exc: Paramiko has the super fun ability to raise completely
+                           opaque errors that just set "e.args = ('Failure',)" with
+                           no more information.
+                           This sometimes means FileExists, but it also sometimes
+                           means NoSuchFile
+        """
+        # paramiko seems to generate detailless errors.
+        self._translate_error(e, path, raise_generic=False)
+        if hasattr(e, 'args'):
+            if (e.args == ('No such file or directory',) or
+                e.args == ('No such file',)):
+                raise NoSuchFile(path, str(e) + more_info)
+            if (e.args == ('mkdir failed',)):
+                raise FileExists(path, str(e) + more_info)
+            # strange but true, for the paramiko server.
+            if (e.args == ('Failure',)):
+                raise failure_exc(path, str(e) + more_info)
+            mutter('Raising exception with args %s', e.args)
+        if hasattr(e, 'errno'):
+            mutter('Raising exception with errno %s', e.errno)
+        raise e
+
+    def append(self, relpath, f):
+        """
+        Append the text in the file-like object into the final
+        location.
+        """
+        try:
+            path = self._remote_path(relpath)
+            fout = self._sftp.file(path, 'ab')
+            result = fout.tell()
+            self._pump(f, fout)
+            return result
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, relpath, ': unable to append')
+
+    def rename(self, rel_from, rel_to):
+        """Rename without special overwriting"""
+        try:
+            self._sftp.rename(self._remote_path(rel_from),
+                              self._remote_path(rel_to))
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, rel_from,
+                    ': unable to rename to %r' % (rel_to))
+
+    def _rename_and_overwrite(self, abs_from, abs_to):
+        """Do a fancy rename on the remote server.
+        
+        Using the implementation provided by osutils.
+        """
+        try:
+            fancy_rename(abs_from, abs_to,
+                    rename_func=self._sftp.rename,
+                    unlink_func=self._sftp.remove)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, abs_from, ': unable to rename to %r' % (abs_to))
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self._remote_path(rel_from)
+        path_to = self._remote_path(rel_to)
+        self._rename_and_overwrite(path_from, path_to)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        path = self._remote_path(relpath)
+        try:
+            self._sftp.remove(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': unable to delete')
+            
+    def listable(self):
+        """Return True if this store supports listing."""
+        return True
+
+    def list_dir(self, relpath):
+        """
+        Return a list of all files at the given location.
+        """
+        # does anything actually use this?
+        path = self._remote_path(relpath)
+        try:
+            return self._sftp.listdir(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': failed to list_dir')
+
+    def rmdir(self, relpath):
+        """See Transport.rmdir."""
+        path = self._remote_path(relpath)
+        try:
+            return self._sftp.rmdir(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': failed to rmdir')
+
+    def stat(self, relpath):
+        """Return the stat information for a file."""
+        path = self._remote_path(relpath)
+        try:
+            return self._sftp.stat(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': unable to stat')
+
+    def lock_read(self, relpath):
+        """
+        Lock the given file for shared (read) access.
+        :return: A lock object, which has an unlock() member function
+        """
+        # FIXME: there should be something clever i can do here...
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """
+        Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which has an unlock() member function
+        """
+        # This is a little bit bogus, but basically, we create a file
+        # which should not already exist, and if it does, we assume
+        # that there is a lock, and if it doesn't, the we assume
+        # that we have taken the lock.
+        return SFTPLock(relpath, self)
+
+    def _unparse_url(self, path=None):
+        if path is None:
+            path = self._path
+        path = urllib.quote(path)
+        # handle homedir paths
+        if not path.startswith('/'):
+            path = "/~/" + path
+        netloc = urllib.quote(self._host)
+        if self._username is not None:
+            netloc = '%s@%s' % (urllib.quote(self._username), netloc)
+        if self._port is not None:
+            netloc = '%s:%d' % (netloc, self._port)
+        return urlparse.urlunparse(('sftp', netloc, path, '', '', ''))
+
+    def _split_url(self, url):
+        if isinstance(url, unicode):
+            url = url.encode('utf-8')
+        (scheme, netloc, path, params,
+         query, fragment) = urlparse.urlparse(url, allow_fragments=False)
+        assert scheme == 'sftp'
+        username = password = host = port = None
+        if '@' in netloc:
+            username, host = netloc.split('@', 1)
+            if ':' in username:
+                username, password = username.split(':', 1)
+                password = urllib.unquote(password)
+            username = urllib.unquote(username)
+        else:
+            host = netloc
+
+        if ':' in host:
+            host, port = host.rsplit(':', 1)
+            try:
+                port = int(port)
+            except ValueError:
+                # TODO: Should this be ConnectionError?
+                raise TransportError('%s: invalid port number' % port)
+        host = urllib.unquote(host)
+
+        path = urllib.unquote(path)
+
+        # the initial slash should be removed from the path, and treated
+        # as a homedir relative path (the path begins with a double slash
+        # if it is absolute).
+        # see draft-ietf-secsh-scp-sftp-ssh-uri-03.txt
+        # RBC 20060118 we are not using this as its too user hostile. instead
+        # we are following lftp and using /~/foo to mean '~/foo'.
+        # handle homedir paths
+        if path.startswith('/~/'):
+            path = path[3:]
+        elif path == '/~':
+            path = ''
+        return (username, password, host, port, path)
+
+    def _parse_url(self, url):
+        (self._username, self._password,
+         self._host, self._port, self._path) = self._split_url(url)
+
+    def _sftp_connect(self):
+        """Connect to the remote sftp server.
+        After this, self._sftp should have a valid connection (or
+        we raise an TransportError 'could not connect').
+
+        TODO: Raise a more reasonable ConnectionFailed exception
+        """
+        global _connected_hosts
+
+        idx = (self._host, self._port, self._username)
+        try:
+            self._sftp = _connected_hosts[idx]
+            return
+        except KeyError:
+            pass
+        
+        vendor = _get_ssh_vendor()
+        if vendor == 'loopback':
+            sock = socket.socket()
+            sock.connect((self._host, self._port))
+            self._sftp = SFTPClient(LoopbackSFTP(sock))
+        elif vendor != 'none':
+            sock = SFTPSubprocess(self._host, vendor, self._port,
+                                  self._username)
+            self._sftp = SFTPClient(sock)
+        else:
+            self._paramiko_connect()
+
+        _connected_hosts[idx] = self._sftp
+
+    def _paramiko_connect(self):
+        global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+        
+        load_host_keys()
+
+        try:
+            t = paramiko.Transport((self._host, self._port or 22))
+            t.set_log_channel('bzr.paramiko')
+            t.start_client()
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to reach SSH host %s:%d' %
+                                  (self._host, self._port), e)
+            
+        server_key = t.get_remote_server_key()
+        server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
+        keytype = server_key.get_name()
+        if SYSTEM_HOSTKEYS.has_key(self._host) and SYSTEM_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = SYSTEM_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        elif BZR_HOSTKEYS.has_key(self._host) and BZR_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = BZR_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        else:
+            warning('Adding %s host key for %s: %s' % (keytype, self._host, server_key_hex))
+            if not BZR_HOSTKEYS.has_key(self._host):
+                BZR_HOSTKEYS[self._host] = {}
+            BZR_HOSTKEYS[self._host][keytype] = server_key
+            our_server_key = server_key
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            save_host_keys()
+        if server_key != our_server_key:
+            filename1 = os.path.expanduser('~/.ssh/known_hosts')
+            filename2 = pathjoin(config_dir(), 'ssh_host_keys')
+            raise TransportError('Host keys for %s do not match!  %s != %s' % \
+                (self._host, our_server_key_hex, server_key_hex),
+                ['Try editing %s or %s' % (filename1, filename2)])
+
+        self._sftp_auth(t)
+        
+        try:
+            self._sftp = t.open_sftp_client()
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to start sftp client %s:%d' %
+                                  (self._host, self._port), e)
+
+    def _sftp_auth(self, transport):
+        # paramiko requires a username, but it might be none if nothing was supplied
+        # use the local username, just in case.
+        # We don't override self._username, because if we aren't using paramiko,
+        # the username might be specified in ~/.ssh/config and we don't want to
+        # force it to something else
+        # Also, it would mess up the self.relpath() functionality
+        username = self._username or getpass.getuser()
+
+        # Paramiko tries to open a socket.AF_UNIX in order to connect
+        # to ssh-agent. That attribute doesn't exist on win32 (it does in cygwin)
+        # so we get an AttributeError exception. For now, just don't try to
+        # connect to an agent if we are on win32
+        if sys.platform != 'win32':
+            agent = paramiko.Agent()
+            for key in agent.get_keys():
+                mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
+                try:
+                    transport.auth_publickey(username, key)
+                    return
+                except paramiko.SSHException, e:
+                    pass
+        
+        # okay, try finding id_rsa or id_dss?  (posix only)
+        if self._try_pkey_auth(transport, paramiko.RSAKey, username, 'id_rsa'):
+            return
+        if self._try_pkey_auth(transport, paramiko.DSSKey, username, 'id_dsa'):
+            return
+
+        if self._password:
+            try:
+                transport.auth_password(username, self._password)
+                return
+            except paramiko.SSHException, e:
+                pass
+
+            # FIXME: Don't keep a password held in memory if you can help it
+            #self._password = None
+
+        # give up and ask for a password
+        password = bzrlib.ui.ui_factory.get_password(
+                prompt='SSH %(user)s@%(host)s password',
+                user=username, host=self._host)
+        try:
+            transport.auth_password(username, password)
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to authenticate to SSH host as %s@%s' %
+                                  (username, self._host), e)
+
+    def _try_pkey_auth(self, transport, pkey_class, username, filename):
+        filename = os.path.expanduser('~/.ssh/' + filename)
+        try:
+            key = pkey_class.from_private_key_file(filename)
+            transport.auth_publickey(username, key)
+            return True
+        except paramiko.PasswordRequiredException:
+            password = bzrlib.ui.ui_factory.get_password(
+                    prompt='SSH %(filename)s password',
+                    filename=filename)
+            try:
+                key = pkey_class.from_private_key_file(filename, password)
+                transport.auth_publickey(username, key)
+                return True
+            except paramiko.SSHException:
+                mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except paramiko.SSHException:
+            mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except IOError:
+            pass
+        return False
+
+    def _sftp_open_exclusive(self, abspath, mode=None):
+        """Open a remote path exclusively.
+
+        SFTP supports O_EXCL (SFTP_FLAG_EXCL), which fails if
+        the file already exists. However it does not expose this
+        at the higher level of SFTPClient.open(), so we have to
+        sneak away with it.
+
+        WARNING: This breaks the SFTPClient abstraction, so it
+        could easily break against an updated version of paramiko.
+
+        :param abspath: The remote absolute path where the file should be opened
+        :param mode: The mode permissions bits for the new file
+        """
+        path = self._sftp._adjust_cwd(abspath)
+        attr = SFTPAttributes()
+        if mode is not None:
+            attr.st_mode = mode
+        omode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
+                | SFTP_FLAG_TRUNC | SFTP_FLAG_EXCL)
+        try:
+            t, msg = self._sftp._request(CMD_OPEN, path, omode, attr)
+            if t != CMD_HANDLE:
+                raise TransportError('Expected an SFTP handle')
+            handle = msg.get_string()
+            return SFTPFile(self._sftp, handle, 'wb', -1)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, abspath, ': unable to open',
+                failure_exc=FileExists)
+
+
+# ------------- server test implementation --------------
+import socket
+import threading
+
+from bzrlib.tests.stub_sftp import StubServer, StubSFTPServer
+
+STUB_SERVER_KEY = """
+-----BEGIN RSA PRIVATE KEY-----
+MIICWgIBAAKBgQDTj1bqB4WmayWNPB+8jVSYpZYk80Ujvj680pOTh2bORBjbIAyz
+oWGW+GUjzKxTiiPvVmxFgx5wdsFvF03v34lEVVhMpouqPAYQ15N37K/ir5XY+9m/
+d8ufMCkjeXsQkKqFbAlQcnWMCRnOoPHS3I4vi6hmnDDeeYTSRvfLbW0fhwIBIwKB
+gBIiOqZYaoqbeD9OS9z2K9KR2atlTxGxOJPXiP4ESqP3NVScWNwyZ3NXHpyrJLa0
+EbVtzsQhLn6rF+TzXnOlcipFvjsem3iYzCpuChfGQ6SovTcOjHV9z+hnpXvQ/fon
+soVRZY65wKnF7IAoUwTmJS9opqgrN6kRgCd3DASAMd1bAkEA96SBVWFt/fJBNJ9H
+tYnBKZGw0VeHOYmVYbvMSstssn8un+pQpUm9vlG/bp7Oxd/m+b9KWEh2xPfv6zqU
+avNwHwJBANqzGZa/EpzF4J8pGti7oIAPUIDGMtfIcmqNXVMckrmzQ2vTfqtkEZsA
+4rE1IERRyiJQx6EJsz21wJmGV9WJQ5kCQQDwkS0uXqVdFzgHO6S++tjmjYcxwr3g
+H0CoFYSgbddOT6miqRskOQF3DZVkJT3kyuBgU2zKygz52ukQZMqxCb1fAkASvuTv
+qfpH87Qq5kQhNKdbbwbmd2NxlNabazPijWuphGTdW0VfJdWfklyS2Kr+iqrs/5wV
+HhathJt636Eg7oIjAkA8ht3MQ+XSl9yIJIS8gVpbPxSw5OMfw0PjVE7tBdQruiSc
+nvuQES5C9BMHjF39LZiGH1iLQy7FgdHyoP+eodI7
+-----END RSA PRIVATE KEY-----
+"""
+    
+
+class SingleListener(threading.Thread):
+
+    def __init__(self, callback):
+        threading.Thread.__init__(self)
+        self._callback = callback
+        self._socket = socket.socket()
+        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        self._socket.bind(('localhost', 0))
+        self._socket.listen(1)
+        self.port = self._socket.getsockname()[1]
+        self.stop_event = threading.Event()
+
+    def run(self):
+        s, _ = self._socket.accept()
+        # now close the listen socket
+        self._socket.close()
+        try:
+            self._callback(s, self.stop_event)
+        except socket.error:
+            pass #Ignore socket errors
+        except Exception, x:
+            # probably a failed test
+            warning('Exception from within unit test server thread: %r' % x)
+
+    def stop(self):
+        self.stop_event.set()
+        # use a timeout here, because if the test fails, the server thread may
+        # never notice the stop_event.
+        self.join(5.0)
+
+
+class SFTPServer(Server):
+    """Common code for SFTP server facilities."""
+
+    def __init__(self):
+        self._original_vendor = None
+        self._homedir = None
+        self._server_homedir = None
+        self._listener = None
+        self._root = None
+        self._vendor = 'none'
+        # sftp server logs
+        self.logs = []
+
+    def _get_sftp_url(self, path):
+        """Calculate an sftp url to this server for path."""
+        return 'sftp://foo:bar@localhost:%d/%s' % (self._listener.port, path)
+
+    def log(self, message):
+        """StubServer uses this to log when a new server is created."""
+        self.logs.append(message)
+
+    def _run_server(self, s, stop_event):
+        ssh_server = paramiko.Transport(s)
+        key_file = os.path.join(self._homedir, 'test_rsa.key')
+        file(key_file, 'w').write(STUB_SERVER_KEY)
+        host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        ssh_server.add_server_key(host_key)
+        server = StubServer(self)
+        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
+                                         StubSFTPServer, root=self._root,
+                                         home=self._server_homedir)
+        event = threading.Event()
+        ssh_server.start_server(event, server)
+        event.wait(5.0)
+        stop_event.wait(30.0)
+    
+    def setUp(self):
+        global _ssh_vendor
+        self._original_vendor = _ssh_vendor
+        _ssh_vendor = self._vendor
+        self._homedir = os.getcwdu()
+        if self._server_homedir is None:
+            self._server_homedir = self._homedir
+        self._root = '/'
+        # FIXME WINDOWS: _root should be _server_homedir[0]:/
+        self._listener = SingleListener(self._run_server)
+        self._listener.setDaemon(True)
+        self._listener.start()
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        global _ssh_vendor
+        self._listener.stop()
+        _ssh_vendor = self._original_vendor
+
+
+class SFTPFullAbsoluteServer(SFTPServer):
+    """A test server for sftp transports, using absolute urls and ssh."""
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_sftp_url(urlescape(self._homedir[1:]))
+
+
+class SFTPServerWithoutSSH(SFTPServer):
+    """An SFTP server that uses a simple TCP socket pair rather than SSH."""
+
+    def __init__(self):
+        super(SFTPServerWithoutSSH, self).__init__()
+        self._vendor = 'loopback'
+
+    def _run_server(self, sock, stop_event):
+        class FakeChannel(object):
+            def get_transport(self):
+                return self
+            def get_log_channel(self):
+                return 'paramiko'
+            def get_name(self):
+                return '1'
+            def get_hexdump(self):
+                return False
+
+        server = paramiko.SFTPServer(FakeChannel(), 'sftp', StubServer(self), StubSFTPServer,
+                                     root=self._root, home=self._server_homedir)
+        server.start_subsystem('sftp', None, sock)
+        server.finish_subsystem()
+
+
+class SFTPAbsoluteServer(SFTPServerWithoutSSH):
+    """A test server for sftp transports, using absolute urls."""
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_sftp_url(urlescape(self._homedir[1:]))
+
+
+class SFTPHomeDirServer(SFTPServerWithoutSSH):
+    """A test server for sftp transports, using homedir relative urls."""
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_sftp_url("~/")
+
+
+class SFTPSiblingAbsoluteServer(SFTPAbsoluteServer):
+    """A test servere for sftp transports, using absolute urls to non-home."""
+
+    def setUp(self):
+        self._server_homedir = '/dev/noone/runs/tests/here'
+        super(SFTPSiblingAbsoluteServer, self).setUp()
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(SFTPTransport, SFTPAbsoluteServer),
+            (SFTPTransport, SFTPHomeDirServer),
+            (SFTPTransport, SFTPSiblingAbsoluteServer),
+            ]

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-08-17 08:32:56 +0000
+++ bzrlib/tree.py	2006-04-13 05:51:35 +0000
@@ -18,15 +18,13 @@
 """
 
 import os
+from cStringIO import StringIO
 
 import bzrlib
 from bzrlib.trace import mutter, note
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.inventory import Inventory
-from bzrlib.osutils import pumpfile, appendpath, fingerprint_file
-
-
-exporters = {}
+from bzrlib.osutils import appendpath, fingerprint_file
 
 class Tree(object):
     """Abstract file tree.
@@ -57,6 +55,11 @@
     def has_id(self, file_id):
         return self.inventory.has_id(file_id)
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
+
     __contains__ = has_id
 
     def __iter__(self):
@@ -65,6 +68,9 @@
     def id2path(self, file_id):
         return self.inventory.id2path(file_id)
 
+    def kind(self, file_id):
+        raise NotImplementedError("subclasses must implement kind")
+
     def _get_inventory(self):
         return self._inventory
     
@@ -75,6 +81,8 @@
                          doc="Inventory of this Tree")
 
     def _check_retrieved(self, ie, f):
+        if not __debug__:
+            return  
         fp = fingerprint_file(f)
         f.seek(0)
         
@@ -92,23 +100,29 @@
                      "store is probably damaged/corrupt"])
 
 
-    def print_file(self, fileid):
-        """Print file with id `fileid` to stdout."""
+    def print_file(self, file_id):
+        """Print file with id `file_id` to stdout."""
         import sys
-        pumpfile(self.get_file(fileid), sys.stdout)
-        
-        
-    def export(self, dest, format='dir', root=None):
-        """Export this tree."""
-        try:
-            exporter = exporters[format]
-        except KeyError:
-            from bzrlib.errors import BzrCommandError
-            raise BzrCommandError("export format %r not supported" % format)
-        exporter(self, dest, root)
-
-
-
+        sys.stdout.write(self.get_file_text(file_id))
+
+    def lock_read(self):
+        pass
+
+    def unlock(self):
+        pass
+
+    def filter_unversioned_files(self, paths):
+        """Filter out paths that are not versioned.
+
+        :return: set of paths.
+        """
+        # NB: we specifically *don't* call self.has_filename, because for
+        # WorkingTrees that can indicate files that exist on disk but that 
+        # are not versioned.
+        pred = self.inventory.has_filename
+        return set((p for p in paths if not pred(p)))
+        
+        
 class RevisionTree(Tree):
     """Tree viewing a previous revision.
 
@@ -119,16 +133,26 @@
            or at least passing a description to the constructor.
     """
     
-    def __init__(self, store, inv):
-        self._store = store
+    def __init__(self, branch, inv, revision_id):
+        self._branch = branch
+        self._weave_store = branch.weave_store
         self._inventory = inv
+        self._revision_id = revision_id
+
+    def get_weave(self, file_id):
+        return self._weave_store.get_weave(file_id,
+                self._branch.get_transaction())
+
+    def get_file_lines(self, file_id):
+        ie = self._inventory[file_id]
+        weave = self.get_weave(file_id)
+        return weave.get_lines(ie.revision)
+
+    def get_file_text(self, file_id):
+        return ''.join(self.get_file_lines(file_id))
 
     def get_file(self, file_id):
-        ie = self._inventory[file_id]
-        f = self._store[ie.text_id]
-        mutter("  get fileid{%s} from %r" % (file_id, self))
-        self._check_retrieved(ie, f)
-        return f
+        return StringIO(self.get_file_text(file_id))
 
     def get_file_size(self, file_id):
         return self._inventory[file_id].text_size
@@ -138,25 +162,50 @@
         if ie.kind == "file":
             return ie.text_sha1
 
+    def is_executable(self, file_id):
+        ie = self._inventory[file_id]
+        if ie.kind != "file":
+            return None 
+        return self._inventory[file_id].executable
+
     def has_filename(self, filename):
         return bool(self.inventory.path2id(filename))
 
     def list_files(self):
         # The only files returned by this are those from the version
         for path, entry in self.inventory.iter_entries():
-            yield path, 'V', entry.kind, entry.file_id
+            yield path, 'V', entry.kind, entry.file_id, entry
+
+    def get_symlink_target(self, file_id):
+        ie = self._inventory[file_id]
+        return ie.symlink_target;
+
+    def kind(self, file_id):
+        return self._inventory[file_id].kind
+
+    def lock_read(self):
+        self._branch.lock_read()
+
+    def unlock(self):
+        self._branch.unlock()
 
 
 class EmptyTree(Tree):
     def __init__(self):
         self._inventory = Inventory()
 
+    def get_symlink_target(self, file_id):
+        return None
+
     def has_filename(self, filename):
         return False
 
+    def kind(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return "root_directory"
+
     def list_files(self):
-        if False:  # just to make it a generator
-            yield None
+        return iter([])
     
     def __contains__(self, file_id):
         return file_id in self._inventory
@@ -166,8 +215,6 @@
         return None
 
 
-
-
 ######################################################################
 # diff
 
@@ -234,128 +281,3 @@
             
 
 
-######################################################################
-# export
-
-def dir_exporter(tree, dest, root):
-    """Export this tree to a new directory.
-
-    `dest` should not exist, and will be created holding the
-    contents of this tree.
-
-    TODO: To handle subdirectories we need to create the
-           directories first.
-
-    :note: If the export fails, the destination directory will be
-           left in a half-assed state.
-    """
-    import os
-    os.mkdir(dest)
-    mutter('export version %r' % tree)
-    inv = tree.inventory
-    for dp, ie in inv.iter_entries():
-        kind = ie.kind
-        fullpath = appendpath(dest, dp)
-        if kind == 'directory':
-            os.mkdir(fullpath)
-        elif kind == 'file':
-            pumpfile(tree.get_file(ie.file_id), file(fullpath, 'wb'))
-        else:
-            raise BzrError("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-        mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
-exporters['dir'] = dir_exporter
-
-try:
-    import tarfile
-except ImportError:
-    pass
-else:
-    def get_root_name(dest):
-        """Get just the root name for a tarball.
-
-        >>> get_root_name('mytar.tar')
-        'mytar'
-        >>> get_root_name('mytar.tar.bz2')
-        'mytar'
-        >>> get_root_name('tar.tar.tar.tgz')
-        'tar.tar.tar'
-        >>> get_root_name('bzr-0.0.5.tar.gz')
-        'bzr-0.0.5'
-        >>> get_root_name('a/long/path/mytar.tgz')
-        'mytar'
-        >>> get_root_name('../parent/../dir/other.tbz2')
-        'other'
-        """
-        endings = ['.tar', '.tar.gz', '.tgz', '.tar.bz2', '.tbz2']
-        dest = os.path.basename(dest)
-        for end in endings:
-            if dest.endswith(end):
-                return dest[:-len(end)]
-
-    def tar_exporter(tree, dest, root, compression=None):
-        """Export this tree to a new tar file.
-
-        `dest` will be created holding the contents of this tree; if it
-        already exists, it will be clobbered, like with "tar -c".
-        """
-        from time import time
-        now = time()
-        compression = str(compression or '')
-        if root is None:
-            root = get_root_name(dest)
-        try:
-            ball = tarfile.open(dest, 'w:' + compression)
-        except tarfile.CompressionError, e:
-            raise BzrError(str(e))
-        mutter('export version %r' % tree)
-        inv = tree.inventory
-        for dp, ie in inv.iter_entries():
-            mutter("  export {%s} kind %s to %s" % (ie.file_id, ie.kind, dest))
-            item = tarfile.TarInfo(os.path.join(root, dp))
-            # TODO: would be cool to actually set it to the timestamp of the
-            # revision it was last changed
-            item.mtime = now
-            if ie.kind == 'directory':
-                item.type = tarfile.DIRTYPE
-                fileobj = None
-                item.name += '/'
-                item.size = 0
-                item.mode = 0755
-            elif ie.kind == 'file':
-                item.type = tarfile.REGTYPE
-                fileobj = tree.get_file(ie.file_id)
-                item.size = _find_file_size(fileobj)
-                item.mode = 0644
-            else:
-                raise BzrError("don't know how to export {%s} of kind %r" %
-                        (ie.file_id, ie.kind))
-
-            ball.addfile(item, fileobj)
-        ball.close()
-    exporters['tar'] = tar_exporter
-
-    def tgz_exporter(tree, dest, root):
-        tar_exporter(tree, dest, root, compression='gz')
-    exporters['tgz'] = tgz_exporter
-
-    def tbz_exporter(tree, dest, root):
-        tar_exporter(tree, dest, root, compression='bz2')
-    exporters['tbz2'] = tbz_exporter
-
-
-def _find_file_size(fileobj):
-    offset = fileobj.tell()
-    try:
-        fileobj.seek(0, 2)
-        size = fileobj.tell()
-    except TypeError:
-        # gzip doesn't accept second argument to seek()
-        fileobj.seek(0)
-        size = 0
-        while True:
-            nread = len(fileobj.read())
-            if nread == 0:
-                break
-            size += nread
-    fileobj.seek(offset)
-    return size

=== added file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tsort.py	2006-03-28 14:29:13 +0000
@@ -0,0 +1,465 @@
+# (C) 2005, 2006 Canonical Limited.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Topological sorting routines."""
+
+
+import bzrlib.errors as errors
+
+
+__all__ = ["topo_sort", "TopoSorter", "merge_sort", "MergeSorter"]
+
+
+def topo_sort(graph):
+    """Topological sort a graph.
+
+    graph -- sequence of pairs of node->parents_list.
+
+    The result is a list of node names, such that all parents come before
+    their children.
+
+    node identifiers can be any hashable object, and are typically strings.
+    """
+    return TopoSorter(graph).sorted()
+
+
+class TopoSorter(object):
+
+    def __init__(self, graph):
+        """Topological sorting of a graph.
+    
+        :param graph: sequence of pairs of node_name->parent_names_list.
+                      i.e. [('C', ['B']), ('B', ['A']), ('A', [])]
+                      For this input the output from the sort or
+                      iter_topo_order routines will be:
+                      'A', 'B', 'C'
+        
+        node identifiers can be any hashable object, and are typically strings.
+
+        If you have a graph like [('a', ['b']), ('a', ['c'])] this will only use
+        one of the two values for 'a'.
+
+        The graph is sorted lazily: until you iterate or sort the input is
+        not processed other than to create an internal representation.
+
+        iteration or sorting may raise GraphCycleError if a cycle is present 
+        in the graph.
+        """
+        # a dict of the graph.
+        self._graph = dict(graph)
+        ### if debugging:
+        # self._original_graph = dict(graph)
+        
+        # this is a stack storing the depth first search into the graph.
+        self._node_name_stack = []
+        # at each level of 'recursion' we have to check each parent. This
+        # stack stores the parents we have not yet checked for the node at the 
+        # matching depth in _node_name_stack
+        self._pending_parents_stack = []
+        # this is a set of the completed nodes for fast checking whether a
+        # parent in a node we are processing on the stack has already been
+        # emitted and thus can be skipped.
+        self._completed_node_names = set()
+
+    def sorted(self):
+        """Sort the graph and return as a list.
+        
+        After calling this the sorter is empty and you must create a new one.
+        """
+        return list(self.iter_topo_order())
+
+###        Useful if fiddling with this code.
+###        # cross check
+###        sorted_names = list(self.iter_topo_order())
+###        for index in range(len(sorted_names)):
+###            rev = sorted_names[index]
+###            for left_index in range(index):
+###                if rev in self.original_graph[sorted_names[left_index]]:
+###                    print "revision in parent list of earlier revision"
+###                    import pdb;pdb.set_trace()
+
+    def iter_topo_order(self):
+        """Yield the nodes of the graph in a topological order.
+        
+        After finishing iteration the sorter is empty and you cannot continue
+        iteration.
+        """
+        while self._graph:
+            # now pick a random node in the source graph, and transfer it to the
+            # top of the depth first search stack.
+            node_name, parents = self._graph.popitem()
+            self._push_node(node_name, parents)
+            while self._node_name_stack:
+                # loop until this call completes.
+                parents_to_visit = self._pending_parents_stack[-1]
+                # if all parents are done, the revision is done
+                if not parents_to_visit:
+                    # append the revision to the topo sorted list
+                    # all the nodes parents have been added to the output, now
+                    # we can add it to the output.
+                    yield self._pop_node()
+                else:
+                    while self._pending_parents_stack[-1]:
+                        # recurse depth first into a single parent 
+                        next_node_name = self._pending_parents_stack[-1].pop()
+                        if next_node_name in self._completed_node_names:
+                            # this parent was completed by a child on the
+                            # call stack. skip it.
+                            continue
+                        # otherwise transfer it from the source graph into the
+                        # top of the current depth first search stack.
+                        try:
+                            parents = self._graph.pop(next_node_name)
+                        except KeyError:
+                            # if the next node is not in the source graph it has
+                            # already been popped from it and placed into the
+                            # current search stack (but not completed or we would
+                            # have hit the continue 4 lines up.
+                            # this indicates a cycle.
+                            raise errors.GraphCycleError(self._node_name_stack)
+                        self._push_node(next_node_name, parents)
+                        # and do not continue processing parents until this 'call' 
+                        # has recursed.
+                        break
+
+    def _push_node(self, node_name, parents):
+        """Add node_name to the pending node stack.
+        
+        Names in this stack will get emitted into the output as they are popped
+        off the stack.
+        """
+        self._node_name_stack.append(node_name)
+        self._pending_parents_stack.append(list(parents))
+
+    def _pop_node(self):
+        """Pop the top node off the stack 
+
+        The node is appended to the sorted output.
+        """
+        # we are returning from the flattened call frame:
+        # pop off the local variables
+        node_name = self._node_name_stack.pop()
+        self._pending_parents_stack.pop()
+
+        self._completed_node_names.add(node_name)
+        return node_name
+
+
+def merge_sort(graph, branch_tip, mainline_revisions=None):
+    """Topological sort a graph which groups merges.
+
+    :param graph: sequence of pairs of node->parents_list.
+    :param branch_tip: the tip of the branch to graph. Revisions not 
+                       reachable from branch_tip are not included in the
+                       output.
+    :param mainline_revisions: If not None this forces a mainline to be
+                               used rather than synthesised from the graph.
+                               This must be a valid path through some part
+                               of the graph. If the mainline does not cover all
+                               the revisions, output stops at the start of the
+                               old revision listed in the mainline revisions
+                               list.
+                               The order for this parameter is oldest-first.
+
+    The result is a list of node names, such that all parents come before
+    their children.
+
+    node identifiers can be any hashable object, and are typically strings.
+    """
+    return MergeSorter(graph, branch_tip, mainline_revisions).sorted()
+
+
+class MergeSorter(object):
+
+    def __init__(self, graph, branch_tip, mainline_revisions=None):
+        """Merge-aware topological sorting of a graph.
+    
+        :param graph: sequence of pairs of node_name->parent_names_list.
+                      i.e. [('C', ['B']), ('B', ['A']), ('A', [])]
+                      For this input the output from the sort or
+                      iter_topo_order routines will be:
+                      'A', 'B', 'C'
+        :param branch_tip: the tip of the branch to graph. Revisions not 
+                       reachable from branch_tip are not included in the
+                       output.
+        :param mainline_revisions: If not None this forces a mainline to be
+                               used rather than synthesised from the graph.
+                               This must be a valid path through some part
+                               of the graph. If the mainline does not cover all
+                               the revisions, output stops at the start of the
+                               old revision listed in the mainline revisions
+                               list.
+                               The order for this parameter is oldest-first.
+
+        
+        node identifiers can be any hashable object, and are typically strings.
+
+        If you have a graph like [('a', ['b']), ('a', ['c'])] this will only use
+        one of the two values for 'a'.
+
+        The graph is sorted lazily: until you iterate or sort the input is
+        not processed other than to create an internal representation.
+
+        iteration or sorting may raise GraphCycleError if a cycle is present 
+        in the graph.
+
+        Background information on the design:
+        -------------------------------------
+        definition: the end of any cluster or 'merge' occurs when:
+            1 - the next revision has a lower merge depth than we do.
+              i.e.
+              A 0
+              B  1
+              C   2
+              D  1
+              E 0
+              C, D are the ends of clusters, E might be but we need more data.
+            2 - or the next revision at our merge depth is not our left most
+              ancestor.
+              This is required to handle multiple-merges in one commit.
+              i.e.
+              A 0    [F, B, E]
+              B  1   [D, C]
+              C   2  [D]
+              D  1   [F]
+              E  1   [F]
+              F 0
+              C is the end of a cluster due to rule 1.
+              D is not the end of a cluster from rule 1, but is from rule 2: E 
+                is not its left most ancestor
+              E is the end of a cluster due to rule 1
+              F might be but we need more data.
+              
+        we show connecting lines to a parent when:
+         - The parent is the start of a merge within this cluster.
+           That is, the merge was not done to the mainline before this cluster 
+           was merged to the mainline.
+           This can be detected thus:
+            * The parent has a higher merge depth and is the next revision in 
+              the list.
+          
+          The next revision in the list constraint is needed for this case:
+          A 0   [D, B]   
+          B  1  [C, F]   # we do not want to show a line to F which is depth 2 
+                           but not a merge
+          C  1  [H]      # note that this is a long line to show back to the 
+                           ancestor - see the end of merge rules.
+          D 0   [G, E]
+          E  1  [G, F]
+          F   2 [G]
+          G  1  [H]
+          H 0
+         - Part of this merges 'branch':
+          The parent has the same merge depth and is our left most parent and we
+           are not the end of the cluster.
+          A 0   [C, B] lines: [B, C]
+          B  1  [E, C] lines: [C]
+          C 0   [D]    lines: [D]
+          D 0   [F, E] lines: [E, F]
+          E  1  [F]    lines: [F]
+          F 0
+         - The end of this merge/cluster:
+          we can ONLY have multiple parents at the end of a cluster if this
+          branch was previously merged into the 'mainline'.
+          - if we have one and only one parent, show it
+            Note that this may be to a greater merge depth - for instance if
+            this branch continued from a deeply nested branch to add something
+            to it.
+          - if we have more than one parent - show the second oldest (older ==
+            further down the list) parent with
+            an equal or lower merge depth
+             XXXX revisit when awake. ddaa asks about the relevance of each one
+             - maybe more than one parent is relevant
+        """
+        # a dict of the graph.
+        self._graph = dict(graph)
+        # if there is an explicit mainline, alter the graph to match. This is
+        # easier than checking at every merge whether we are on the mainline and
+        # if so which path to take.
+        if mainline_revisions is None:
+            self._mainline_revisions = []
+            self._stop_revision = None
+        else:
+            self._mainline_revisions = list(mainline_revisions)
+            self._stop_revision = self._mainline_revisions[0]
+        # skip the first revision, its what we reach and its parents are 
+        # therefore irrelevant
+        for index, revision in enumerate(self._mainline_revisions[1:]):
+            # NB: index 0 means self._mainline_revisions[1]
+            # if the mainline matches the graph, nothing to do.
+            parent = self._mainline_revisions[index]
+            if parent is None:
+                # end of mainline_revisions history
+                continue
+            if self._graph[revision][0] == parent:
+                continue
+            # remove it from its prior spot
+            self._graph[revision].remove(parent)
+            # insert it into the start of the mainline
+            self._graph[revision].insert(0, parent)
+        # we need to do a check late in the process to detect end-of-merges
+        # which requires the parents to be accessible: its easier for now
+        # to just keep the original graph around.
+        self._original_graph = dict(self._graph.items())
+        
+        # this is a stack storing the depth first search into the graph.
+        self._node_name_stack = []
+        # at each level of recursion we need the merge depth this node is at:
+        self._node_merge_depth_stack = []
+        # at each level of 'recursion' we have to check each parent. This
+        # stack stores the parents we have not yet checked for the node at the 
+        # matching depth in _node_name_stack
+        self._pending_parents_stack = []
+        # this is a set of the nodes who have been completely analysed for fast
+        # membership checking
+        self._completed_node_names = set()
+        # this is the scheduling of nodes list.
+        # Nodes are scheduled
+        # from the bottom left of the tree: in the tree
+        # A 0  [D, B]
+        # B  1 [C]
+        # C  1 [D]
+        # D 0  [F, E]
+        # E  1 [F]
+        # F 0
+        # the scheduling order is: F, E, D, C, B, A 
+        # that is - 'left subtree, right subtree, node'
+        # which would mean that when we schedule A we can emit the entire tree.
+        self._scheduled_nodes = []
+        # This records for each node when we have processed its left most 
+        # unmerged subtree. After this subtree is scheduled, all other subtrees
+        # have their merge depth increased by one from this nodes merge depth.
+        self._left_subtree_done_stack = []
+
+        # seed the search with the tip of the branch
+        if branch_tip is not None:
+            parents = self._graph.pop(branch_tip)
+            self._push_node(branch_tip, 0, parents)
+
+    def sorted(self):
+        """Sort the graph and return as a list.
+        
+        After calling this the sorter is empty and you must create a new one.
+        """
+        return list(self.iter_topo_order())
+
+    def iter_topo_order(self):
+        """Yield the nodes of the graph in a topological order.
+        
+        After finishing iteration the sorter is empty and you cannot continue
+        iteration.
+        """
+        while self._node_name_stack:
+            # loop until this call completes.
+            parents_to_visit = self._pending_parents_stack[-1]
+            # if all parents are done, the revision is done
+            if not parents_to_visit:
+                # append the revision to the topo sorted scheduled list:
+                # all the nodes parents have been scheduled added, now
+                # we can add it to the output.
+                self._pop_node()
+            else:
+                while self._pending_parents_stack[-1]:
+                    if not self._left_subtree_done_stack[-1]:
+                        # recurse depth first into the primary parent
+                        next_node_name = self._pending_parents_stack[-1].pop(0)
+                    else:
+                        # place any merges in right-to-left order for scheduling
+                        # which gives us left-to-right order after we reverse
+                        # the scheduled queue. XXX: This has the effect of 
+                        # allocating common-new revisions to the right-most
+                        # subtree rather than the left most, which will 
+                        # display nicely (you get smaller trees at the top
+                        # of the combined merge).
+                        next_node_name = self._pending_parents_stack[-1].pop()
+                    if next_node_name in self._completed_node_names:
+                        # this parent was completed by a child on the
+                        # call stack. skip it.
+                        continue
+                    # otherwise transfer it from the source graph into the
+                    # top of the current depth first search stack.
+                    try:
+                        parents = self._graph.pop(next_node_name)
+                    except KeyError:
+                        # if the next node is not in the source graph it has
+                        # already been popped from it and placed into the
+                        # current search stack (but not completed or we would
+                        # have hit the continue 4 lines up.
+                        # this indicates a cycle.
+                        raise errors.GraphCycleError(self._node_name_stack)
+                    next_merge_depth = 0
+                    if self._left_subtree_done_stack[-1]:
+                        next_merge_depth = 1
+                    else:
+                        next_merge_depth = 0
+                        self._left_subtree_done_stack[-1] = True
+                    next_merge_depth = (
+                        self._node_merge_depth_stack[-1] + next_merge_depth)
+                    self._push_node(
+                        next_node_name,
+                        next_merge_depth,
+                        parents)
+                    # and do not continue processing parents until this 'call' 
+                    # has recursed.
+                    break
+        # We have scheduled the graph. Now deliver the ordered output:
+        sequence_number = 0
+        while self._scheduled_nodes:
+            node_name, merge_depth = self._scheduled_nodes.pop()
+            if node_name == self._stop_revision:
+                return
+            if not len(self._scheduled_nodes):
+                end_of_merge = True
+            elif self._scheduled_nodes[-1][1] < merge_depth:
+                # the next node is to our left
+                end_of_merge = True
+            elif (self._scheduled_nodes[-1][1] == merge_depth and
+                  (self._scheduled_nodes[-1][0] not in
+                   self._original_graph[node_name])):
+                # the next node was part of a multiple-merge.
+                end_of_merge = True
+            else:
+                end_of_merge = False
+            yield (sequence_number, node_name, merge_depth, end_of_merge)
+            sequence_number += 1
+
+    def _push_node(self, node_name, merge_depth, parents):
+        """Add node_name to the pending node stack.
+        
+        Names in this stack will get emitted into the output as they are popped
+        off the stack.
+        """
+        self._node_name_stack.append(node_name)
+        self._node_merge_depth_stack.append(merge_depth)
+        self._left_subtree_done_stack.append(False)
+        self._pending_parents_stack.append(list(parents))
+
+    def _pop_node(self):
+        """Pop the top node off the stack 
+
+        The node is appended to the sorted output.
+        """
+        # we are returning from the flattened call frame:
+        # pop off the local variables
+        node_name = self._node_name_stack.pop()
+        merge_depth = self._node_merge_depth_stack.pop()
+        self._left_subtree_done_stack.pop()
+        self._pending_parents_stack.pop()
+
+        self._completed_node_names.add(node_name)
+        self._scheduled_nodes.append((node_name, merge_depth))
+        return node_name

=== added file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tuned_gzip.py	2006-04-18 04:22:52 +0000
@@ -0,0 +1,292 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+# Written by Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Bzrlib specific gzip tunings. We plan to feed these to the upstream gzip."""
+
+# make GzipFile faster:
+import gzip
+from gzip import U32, LOWU32, FEXTRA, FCOMMENT, FNAME, FHCRC
+import sys
+import struct
+import zlib
+
+__all__ = ["GzipFile"]
+
+
+class GzipFile(gzip.GzipFile):
+    """Knit tuned version of GzipFile.
+
+    This is based on the following lsprof stats:
+    python 2.4 stock GzipFile write:
+    58971      0   5644.3090   2721.4730   gzip:193(write)
+    +58971     0   1159.5530   1159.5530   +<built-in method compress>
+    +176913    0    987.0320    987.0320   +<len>
+    +58971     0    423.1450    423.1450   +<zlib.crc32>
+    +58971     0    353.1060    353.1060   +<method 'write' of 'cStringIO.
+                                            StringO' objects>
+    tuned GzipFile write:
+    58971      0   4477.2590   2103.1120   bzrlib.knit:1250(write)
+    +58971     0   1297.7620   1297.7620   +<built-in method compress>
+    +58971     0    406.2160    406.2160   +<zlib.crc32>
+    +58971     0    341.9020    341.9020   +<method 'write' of 'cStringIO.
+                                            StringO' objects>
+    +58971     0    328.2670    328.2670   +<len>
+
+
+    Yes, its only 1.6 seconds, but they add up.
+    """
+
+    def _add_read_data(self, data):
+        # 4169 calls in 183
+        # temp var for len(data) and switch to +='s.
+        # 4169 in 139
+        len_data = len(data)
+        self.crc = zlib.crc32(data, self.crc)
+        self.extrabuf += data
+        self.extrasize += len_data
+        self.size += len_data
+
+    def _read(self, size=1024):
+        # various optimisations:
+        # reduces lsprof count from 2500 to 
+        # 8337 calls in 1272, 365 internal
+        if self.fileobj is None:
+            raise EOFError, "Reached EOF"
+
+        if self._new_member:
+            # If the _new_member flag is set, we have to
+            # jump to the next member, if there is one.
+            #
+            # First, check if we're at the end of the file;
+            # if so, it's time to stop; no more members to read.
+            next_header_bytes = self.fileobj.read(10)
+            if next_header_bytes == '':
+                raise EOFError, "Reached EOF"
+
+            self._init_read()
+            self._read_gzip_header(next_header_bytes)
+            self.decompress = zlib.decompressobj(-zlib.MAX_WBITS)
+            self._new_member = False
+
+        # Read a chunk of data from the file
+        buf = self.fileobj.read(size)
+
+        # If the EOF has been reached, flush the decompression object
+        # and mark this object as finished.
+
+        if buf == "":
+            self._add_read_data(self.decompress.flush())
+            assert len(self.decompress.unused_data) >= 8, "what does flush do?"
+            self._read_eof()
+            # tell the driving read() call we have stuffed all the data
+            # in self.extrabuf
+            raise EOFError, 'Reached EOF'
+
+        self._add_read_data(self.decompress.decompress(buf))
+
+        if self.decompress.unused_data != "":
+            # Ending case: we've come to the end of a member in the file,
+            # so seek back to the start of the data for the next member which
+            # is the length of the decompress objects unused data - the first
+            # 8 bytes for the end crc and size records.
+            #
+            # so seek back to the start of the unused data, finish up
+            # this member, and read a new gzip header.
+            # (The number of bytes to seek back is the length of the unused
+            # data, minus 8 because those 8 bytes are part of this member.
+            seek_length = len (self.decompress.unused_data) - 8
+            if seek_length > 0:
+                # we read too much data
+                self.fileobj.seek(-seek_length, 1)
+            elif seek_length < 0:
+                # we haven't read enough to check the checksum.
+                assert -8 < seek_length, "too great a seek."
+                buf = self.fileobj.read(-seek_length)
+                self.decompress.decompress(buf)
+
+            # Check the CRC and file size, and set the flag so we read
+            # a new member on the next call
+            self._read_eof()
+            self._new_member = True
+
+    def _read_eof(self):
+        """tuned to reduce function calls and eliminate file seeking:
+        pass 1:
+        reduces lsprof count from 800 to 288
+        4168 in 296 
+        avoid U32 call by using struct format L
+        4168 in 200
+        """
+        # We've read to the end of the file, so we should have 8 bytes of 
+        # unused data in the decompressor. If we dont, there is a corrupt file.
+        # We use these 8 bytes to calculate the CRC and the recorded file size.
+        # We then check the that the computed CRC and size of the
+        # uncompressed data matches the stored values.  Note that the size
+        # stored is the true file size mod 2**32.
+        crc32, isize = struct.unpack("<LL", self.decompress.unused_data[0:8])
+        # note that isize is unsigned - it can exceed 2GB
+        if crc32 != U32(self.crc):
+            raise IOError, "CRC check failed %d %d" % (crc32, U32(self.crc))
+        elif isize != LOWU32(self.size):
+            raise IOError, "Incorrect length of data produced"
+
+    def _read_gzip_header(self, bytes=None):
+        """Supply bytes if the minimum header size is already read.
+        
+        :param bytes: 10 bytes of header data.
+        """
+        """starting cost: 300 in 3998
+        15998 reads from 3998 calls
+        final cost 168
+        """
+        if bytes is None:
+            bytes = self.fileobj.read(10)
+        magic = bytes[0:2]
+        if magic != '\037\213':
+            raise IOError, 'Not a gzipped file'
+        method = ord(bytes[2:3])
+        if method != 8:
+            raise IOError, 'Unknown compression method'
+        flag = ord(bytes[3:4])
+        # modtime = self.fileobj.read(4) (bytes [4:8])
+        # extraflag = self.fileobj.read(1) (bytes[8:9])
+        # os = self.fileobj.read(1) (bytes[9:10])
+        # self.fileobj.read(6)
+
+        if flag & FEXTRA:
+            # Read & discard the extra field, if present
+            xlen = ord(self.fileobj.read(1))
+            xlen = xlen + 256*ord(self.fileobj.read(1))
+            self.fileobj.read(xlen)
+        if flag & FNAME:
+            # Read and discard a null-terminated string containing the filename
+            while True:
+                s = self.fileobj.read(1)
+                if not s or s=='\000':
+                    break
+        if flag & FCOMMENT:
+            # Read and discard a null-terminated string containing a comment
+            while True:
+                s = self.fileobj.read(1)
+                if not s or s=='\000':
+                    break
+        if flag & FHCRC:
+            self.fileobj.read(2)     # Read & discard the 16-bit header CRC
+
+    def readline(self, size=-1):
+        """Tuned to remove buffer length calls in _unread and...
+        
+        also removes multiple len(c) calls, inlines _unread,
+        total savings - lsprof 5800 to 5300
+        phase 2:
+        4168 calls in 2233
+        8176 calls to read() in 1684
+        changing the min chunk size to 200 halved all the cache misses
+        leading to a drop to:
+        4168 calls in 1977
+        4168 call to read() in 1646
+        - i.e. just reduced the function call overhead. May be worth 
+          keeping.
+        """
+        if size < 0: size = sys.maxint
+        bufs = []
+        readsize = min(200, size)    # Read from the file in small chunks
+        while True:
+            if size == 0:
+                return "".join(bufs) # Return resulting line
+
+            # c is the chunk
+            c = self.read(readsize)
+            # number of bytes read
+            len_c = len(c)
+            i = c.find('\n')
+            if size is not None:
+                # We set i=size to break out of the loop under two
+                # conditions: 1) there's no newline, and the chunk is
+                # larger than size, or 2) there is a newline, but the
+                # resulting line would be longer than 'size'.
+                if i==-1 and len_c > size: i=size-1
+                elif size <= i: i = size -1
+
+            if i >= 0 or c == '':
+                # if i>= 0 we have a newline or have triggered the above
+                # if size is not None condition.
+                # if c == '' its EOF.
+                bufs.append(c[:i+1])    # Add portion of last chunk
+                # -- inlined self._unread --
+                ## self._unread(c[i+1:], len_c - i)   # Push back rest of chunk
+                self.extrabuf = c[i+1:] + self.extrabuf
+                self.extrasize = len_c - i + self.extrasize
+                self.offset -= len_c - i
+                # -- end inlined self._unread --
+                return ''.join(bufs)    # Return resulting line
+
+            # Append chunk to list, decrease 'size',
+            bufs.append(c)
+            size = size - len_c
+            readsize = min(size, readsize * 2)
+
+    def readlines(self, sizehint=0):
+        # optimise to avoid all the buffer manipulation
+        # lsprof changed from:
+        # 4168 calls in 5472 with 32000 calls to readline()
+        # to :
+        # 4168 calls in 417.
+        # Negative numbers result in reading all the lines
+        if sizehint <= 0:
+            sizehint = -1
+        content = self.read(sizehint)
+        return content.splitlines(True)
+
+    def _unread(self, buf, len_buf=None):
+        """tuned to remove unneeded len calls.
+        
+        because this is such an inner routine in readline, and readline is
+        in many inner loops, this has been inlined into readline().
+
+        The len_buf parameter combined with the reduction in len calls dropped
+        the lsprof ms count for this routine on my test data from 800 to 200 - 
+        a 75% saving.
+        """
+        if len_buf is None:
+            len_buf = len(buf)
+        self.extrabuf = buf + self.extrabuf
+        self.extrasize = len_buf + self.extrasize
+        self.offset -= len_buf
+
+    def write(self, data):
+        if self.mode != gzip.WRITE:
+            import errno
+            raise IOError(errno.EBADF, "write() on read-only GzipFile object")
+
+        if self.fileobj is None:
+            raise ValueError, "write() on closed GzipFile object"
+        data_len = len(data)
+        if data_len > 0:
+            self.size = self.size + data_len
+            self.crc = zlib.crc32(data, self.crc)
+            self.fileobj.write( self.compress.compress(data) )
+            self.offset += data_len
+
+    def writelines(self, lines):
+        # profiling indicated a significant overhead 
+        # calling write for each line.
+        # this batch call is a lot faster :).
+        # (4 seconds to 1 seconds for the sample upgrades I was testing).
+        self.write(''.join(lines))
+
+

=== added directory 'bzrlib/ui'
=== added file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui/__init__.py	2006-03-18 21:02:22 +0000
@@ -0,0 +1,112 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""UI abstraction.
+
+This tells the library how to display things to the user.  Through this
+layer different applications can choose the style of UI.
+
+At the moment this layer is almost trivial: the application can just
+choose the style of progress bar.
+
+Set the ui_factory member to define the behaviour.  The default
+displays no output.
+"""
+
+
+import bzrlib.progress
+from bzrlib.symbol_versioning import *
+
+
+class UIFactory(object):
+    """UI abstraction.
+
+    This tells the library how to display things to the user.  Through this
+    layer different applications can choose the style of UI.
+    """
+
+    def __init__(self):
+        super(UIFactory, self).__init__()
+        self._progress_bar_stack = None
+
+    @deprecated_method(zero_eight)
+    def progress_bar(self):
+        """See UIFactory.nested_progress_bar()."""
+        raise NotImplementedError(self.progress_bar)
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        raise NotImplementedError(self.get_password)
+        
+    def nested_progress_bar(self):
+        """Return a nested progress bar.
+
+        When the bar has been finished with, it should be released bu calling
+        bar.finished().
+        """
+        raise NotImplementedError(self.nested_progress_bar)
+
+    def clear_term(self):
+        """Prepare the terminal for output.
+
+        This will, for example, clear text progress bars, and leave the
+        cursor at the leftmost position."""
+        raise NotImplementedError(self.clear_term)
+
+
+class SilentUIFactory(UIFactory):
+    """A UI Factory which never prints anything.
+
+    This is the default UI, if another one is never registered.
+    """
+
+    @deprecated_method(zero_eight)
+    def progress_bar(self):
+        """See UIFactory.nested_progress_bar()."""
+        return bzrlib.progress.DummyProgress()
+
+    def get_password(self, prompt='', **kwargs):
+        return None
+
+    def nested_progress_bar(self):
+        if self._progress_bar_stack is None:
+            self._progress_bar_stack = bzrlib.progress.ProgressBarStack(
+                klass=bzrlib.progress.DummyProgress)
+        return self._progress_bar_stack.get_nested()
+
+    def clear_term(self):
+        pass
+
+
+def clear_decorator(func, *args, **kwargs):
+    """Decorator that clears the term"""
+    ui_factory.clear_term()
+    func(*args, **kwargs)
+
+
+ui_factory = SilentUIFactory()
+"""IMPORTANT: never import this symbol directly. ONLY ever access it as 
+ui.ui_factory."""

=== added file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui/text.py	2006-03-18 21:02:22 +0000
@@ -0,0 +1,75 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""Text UI, write output to the console.
+"""
+
+import getpass
+import sys
+
+import bzrlib.progress
+from bzrlib.symbol_versioning import *
+from bzrlib.ui import UIFactory
+
+
+class TextUIFactory(UIFactory):
+
+    @deprecated_method(zero_eight)
+    def progress_bar(self):
+        """See UIFactory.nested_progress_bar()."""
+        # this in turn is abstract, and creates either a tty or dots
+        # bar depending on what we think of the terminal
+        return bzrlib.progress.ProgressBar()
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        prompt = (prompt % kwargs).encode(sys.stdout.encoding, 'replace')
+        prompt += ': '
+        try:
+            return getpass.getpass(prompt)
+        except KeyboardInterrupt:
+            return None
+
+    def nested_progress_bar(self):
+        """Return a nested progress bar.
+        
+        The actual bar type returned depends on the progress module which
+        may return a tty or dots bar depending on the terminal.
+        """
+        if self._progress_bar_stack is None:
+            self._progress_bar_stack = bzrlib.progress.ProgressBarStack()
+        return self._progress_bar_stack.get_nested()
+
+    def clear_term(self):
+        """Prepare the terminal for output.
+
+        This will, clear any progress bars, and leave the cursor at the
+        leftmost position."""
+        if self._progress_bar_stack is None:
+            return
+        overall_pb = self._progress_bar_stack.bottom()
+        if overall_pb is not None:
+            overall_pb.clear()

=== added file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/uncommit.py	2006-03-23 05:14:03 +0000
@@ -0,0 +1,61 @@
+#!/usr/bin/env python
+"""\
+Remove the last revision from the history of the current branch.
+"""
+
+import os
+import bzrlib
+from bzrlib.errors import BoundBranchOutOfDate
+
+def test_remove(filename):
+    if os.path.exists(filename):
+        os.remove(filename)
+    else:
+        print '* file does not exist: %r' % filename
+
+
+def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
+    """Remove the last revision from the supplied branch.
+
+    :param dry_run: Don't actually change anything
+    :param verbose: Print each step as you do it
+    :param revno: Remove back to this revision
+    """
+    from bzrlib.atomicfile import AtomicFile
+    unlockable = []
+    try:
+        if tree is not None:
+            tree.lock_write()
+            unlockable.append(tree)
+        
+        branch.lock_write()
+        unlockable.append(branch)
+
+        master = branch.get_master_branch()
+        if master is not None:
+            master.lock_write()
+            unlockable.append(master)
+        rh = branch.revision_history()
+        if master is not None and rh[-1] != master.last_revision():
+            raise BoundBranchOutOfDate(branch, master)
+        if revno is None:
+            revno = len(rh)
+
+        files_to_remove = []
+        for r in range(revno-1, len(rh)):
+            rev_id = rh.pop()
+            if verbose:
+                print 'Removing revno %d: %s' % (len(rh)+1, rev_id)
+
+
+        # Committing before we start removing files, because
+        # once we have removed at least one, all the rest are invalid.
+        if not dry_run:
+            if master is not None:
+                master.set_revision_history(rh)
+            branch.set_revision_history(rh)
+            if tree is not None:
+                tree.set_last_revision(branch.last_revision())
+    finally:
+        for item in reversed(unlockable):
+            item.unlock()

=== renamed file 'tools/history2weaves.py' => 'bzrlib/upgrade.py'
--- tools/history2weaves.py	2005-08-22 14:14:01 +0000
+++ bzrlib/upgrade.py	2006-03-23 14:51:52 +0000
@@ -1,147 +1,75 @@
-#! /usr/bin/python
-
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Experiment in converting existing bzr branches to weaves."""
-
-try:
-    import psyco
-    psyco.full()
-except ImportError:
-    pass
-
-
-import bzrlib.branch
-from bzrlib.revfile import Revfile
-from bzrlib.weave import Weave
-from bzrlib.weavefile import read_weave, write_weave
-from bzrlib.progress import ProgressBar
-from bzrlib.atomicfile import AtomicFile
-import tempfile
-import hotshot, hotshot.stats
-import sys
-
-def convert():
-    pb = ProgressBar()
-
-    inv_weave = Weave()
-
-    last_text_sha = {}
-
-    # holds in-memory weaves for all files
-    text_weaves = {}
-
-    b = bzrlib.branch.find_branch('.')
-
-    revno = 1
-    rev_history = b.revision_history()
-    last_idx = None
-    inv_parents = []
-    text_count = 0
-    
-    for rev_id in rev_history:
-        pb.update('converting revision', revno, len(rev_history))
-        
-        inv_xml = b.get_inventory_xml(rev_id).readlines()
-
-        new_idx = inv_weave.add(rev_id, inv_parents, inv_xml)
-        inv_parents = [new_idx]
-
-        tree = b.revision_tree(rev_id)
-        inv = tree.inventory
-
-        # for each file in the inventory, put it into its own revfile
-        for file_id in inv:
-            ie = inv[file_id]
-            if ie.kind != 'file':
-                continue
-            if last_text_sha.get(file_id) == ie.text_sha1:
-                # same as last time
-                continue
-            last_text_sha[file_id] = ie.text_sha1
-
-            # new text (though possibly already stored); need to store it
-            text_lines = tree.get_file(file_id).readlines()
-
-            # if the file's created for the first time in this
-            # revision then make a new weave; else find the old one
-            if file_id not in text_weaves:
-                text_weaves[file_id] = Weave()
-                
-            w = text_weaves[file_id]
-
-            # base the new text version off whatever was last
-            # (actually it'd be better to track this, to allow for
-            # files that are deleted and then reappear)
-            last = len(w)
-            if last == 0:
-                parents = []
-            else:
-                parents = [last-1]
-
-            w.add(rev_id, parents, text_lines)
-            text_count += 1
-
-        revno += 1
-
-    pb.clear()
-    print '%6d revisions and inventories' % revno
-    print '%6d texts' % text_count
-
-    i = 0
-    # TODO: commit them all atomically at the end, not one by one
-    write_atomic_weave(inv_weave, 'weaves/inventory.weave')
-    for file_id, file_weave in text_weaves.items():
-        pb.update('writing weave', i, len(text_weaves))
-        write_atomic_weave(file_weave, 'weaves/%s.weave' % file_id)
-        i += 1
-
-    pb.clear()
-
-
-def write_atomic_weave(weave, filename):
-    inv_wf = AtomicFile(filename)
-    try:
-        write_weave(weave, inv_wf)
-        inv_wf.commit()
-    finally:
-        inv_wf.close()
-
-    
-
-
-def profile_convert(): 
-    prof_f = tempfile.NamedTemporaryFile()
-
-    prof = hotshot.Profile(prof_f.name)
-
-    prof.runcall(convert) 
-    prof.close()
-
-    stats = hotshot.stats.load(prof_f.name)
-    #stats.strip_dirs()
-    stats.sort_stats('time')
-    ## XXX: Might like to write to stderr or the trace file instead but
-    ## print_stats seems hardcoded to stdout
-    stats.print_stats(20)
-            
-
-if '-p' in sys.argv[1:]:
-    profile_convert()
-else:
-    convert()
-    
+"""bzr upgrade logic."""
+
+# change upgrade from .bzr to create a '.bzr-new', then do a bait and switch.
+
+
+from bzrlib.bzrdir import ConvertBzrDir4To5, ConvertBzrDir5To6, BzrDir, BzrDirFormat4, BzrDirFormat5
+import bzrlib.errors as errors
+from bzrlib.transport import get_transport
+import bzrlib.ui as ui
+
+
+class Convert(object):
+
+    def __init__(self, url, format):
+        self.format = format
+        self.bzrdir = BzrDir.open_unsupported(url)
+        if self.bzrdir.root_transport.is_readonly():
+            raise errors.UpgradeReadonly
+        self.transport = self.bzrdir.root_transport
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            self.convert()
+        finally:
+            self.pb.finished()
+
+    def convert(self):
+        try:
+            branch = self.bzrdir.open_branch()
+            if branch.bzrdir.root_transport.base != \
+                self.bzrdir.root_transport.base:
+                self.pb.note("This is a checkout. The branch (%s) needs to be "
+                             "upgraded separately.",
+                             branch.bzrdir.root_transport.base)
+        except errors.NotBranchError:
+            pass
+        if not self.bzrdir.needs_format_conversion(self.format):
+            raise errors.UpToDateFormat(self.bzrdir._format)
+        if not self.bzrdir.can_convert_format():
+            raise errors.BzrError("cannot upgrade from branch format %s" %
+                           self.bzrdir._format)
+        self.pb.note('starting upgrade of %s', self.transport.base)
+        self._backup_control_dir()
+        while self.bzrdir.needs_format_conversion(self.format):
+            converter = self.bzrdir._format.get_converter(self.format)
+            self.bzrdir = converter.convert(self.bzrdir, self.pb)
+        self.pb.note("finished")
+
+    def _backup_control_dir(self):
+        self.pb.note('making backup of tree history')
+        self.transport.copy_tree('.bzr', '.bzr.backup')
+        self.pb.note('%s.bzr has been backed up to %s.bzr.backup',
+             self.transport.base,
+             self.transport.base)
+        self.pb.note('if conversion fails, you can move this directory back to .bzr')
+        self.pb.note('if it succeeds, you can remove this directory if you wish')
+
+def upgrade(url, format=None):
+    """Upgrade to format, or the default bzrdir format if not supplied."""
+    Convert(url, format)

=== removed file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-08-22 17:52:11 +0000
+++ bzrlib/upgrade.py	1970-01-01 00:00:00 +0000
@@ -1,141 +0,0 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-import bzrlib.errors
-
-
-def upgrade(branch):
-    """
-    Upgrade branch to current format.
-
-    This causes objects to be rewritten into the current format.
-
-    If they change, their SHA-1 will of course change, which might
-    break any later signatures, or backreferences that check the
-    SHA-1.
-
-    TODO: Check non-mainline revisions.
-    """
-    import sys
-
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
-    from bzrlib.progress import ProgressBar
-
-    branch.lock_write()
-
-    try:
-        pb = ProgressBar(show_spinner=True)
-        last_rev_id = None
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-
-        updated_revisions = []
-
-        # Set to True in the case that the previous revision entry
-        # was updated, since this will affect future revision entries
-        updated_previous_revision = False
-
-        for rev_id in history:
-            revno += 1
-            pb.update('upgrading revision', revno, revcount)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            last_rev_id = rev_id
-
-            # if set to true, revision must be written out
-            updated = False
-
-            if rev.inventory_sha1 is None:
-                rev.inventory_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                updated = True
-                mutter("  set inventory_sha1 on {%s}" % rev_id)
-
-            for prr in rev.parents:
-                try:
-                    actual_sha1 = branch.get_revision_sha1(prr.revision_id)
-                except bzrlib.errors.NoSuchRevision:
-                    mutter("parent {%s} of {%s} not present in branch; skipped"
-                           % (prr.revision_id, rev_id))
-                    continue
-                    
-                if actual_sha1 != prr.revision_sha1:
-                    mutter("parent {%s} of {%s} sha1 mismatch: "
-                           "%s vs %s; fixed"
-                           % (prr.revision_id, rev_id,
-                              actual_sha1, prr.revision_sha1))
-                    prr.revision_sha1 = actual_sha1
-                    updated = True
-
-            if updated:
-                updated_previous_revision = True
-                # We had to update this revision entries hashes
-                # Now we need to write out a new value
-                # This is a little bit invasive, since we are *rewriting* a
-                # revision entry. I'm not supremely happy about it, but
-                # there should be *some* way of making old entries have
-                # the full meta information.
-                import tempfile, os, errno
-                from bzrlib.xml import pack_xml
-                
-                rev_tmp = tempfile.TemporaryFile()
-                pack_xml(rev, rev_tmp)
-                rev_tmp.seek(0)
-
-                tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',
-                    dir=branch.controlfilename('revision-store'))
-                os.close(tmpfd)
-                def special_rename(p1, p2):
-                    if sys.platform == 'win32':
-                        try:
-                            os.remove(p2)
-                        except OSError, e:
-                            if e.errno != errno.ENOENT:
-                                raise
-                    os.rename(p1, p2)
-
-                try:
-                    # TODO: We may need to handle the case where the old revision
-                    # entry was not compressed (and thus did not end with .gz)
-
-                    # Remove the old revision entry out of the way
-                    rev_path = branch.controlfilename(['revision-store', rev_id+'.gz'])
-                    special_rename(rev_path, tmp_path)
-                    branch.revision_store.add(rev_tmp, rev_id) # Add the new one
-                    os.remove(tmp_path) # Remove the old name
-                    mutter('    Updated revision entry {%s}' % rev_id)
-                except:
-                    # On any exception, restore the old entry
-                    special_rename(tmp_path, rev_path)
-                    raise
-                rev_tmp.close()
-                updated_revisions.append(rev_id)
-            else:
-                updated_previous_revision = False
-
-    finally:
-        branch.unlock()
-
-    pb.clear()
-
-    if updated_revisions:
-        print '%d revisions updated to current format' % len(updated_revisions)

=== added directory 'bzrlib/util/configobj'
=== added file 'bzrlib/util/configobj/__init__.py'
=== added file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/configobj.py	2006-02-21 02:26:30 +0000
@@ -0,0 +1,3498 @@
+# configobj.py
+# A config file reader/writer that supports nested sections in config files.
+# Copyright (C) 2005 Michael Foord, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         nico AT tekNico DOT net
+
+# ConfigObj 4
+# http://www.voidspace.org.uk/python/configobj.html
+
+# Released subject to the BSD License
+# Please see http://www.voidspace.org.uk/python/license.shtml
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+from __future__ import generators
+
+"""
+    >>> z = ConfigObj()
+    >>> z['a'] = 'a'
+    >>> z['sect'] = {
+    ...    'subsect': {
+    ...         'a': 'fish',
+    ...         'b': 'wobble',
+    ...     },
+    ...     'member': 'value',
+    ... }
+    >>> x = ConfigObj(z.write())
+    >>> z == x
+    1
+"""
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import os, re
+from types import StringTypes
+from warnings import warn
+from codecs import BOM_UTF8, BOM_UTF16, BOM_UTF16_BE, BOM_UTF16_LE
+
+# A dictionary mapping BOM to
+# the encoding to decode with, and what to set the
+# encoding attribute to.
+BOMS = {
+    BOM_UTF8: ('utf_8', None),
+    BOM_UTF16_BE: ('utf16_be', 'utf_16'),
+    BOM_UTF16_LE: ('utf16_le', 'utf_16'),
+    BOM_UTF16: ('utf_16', 'utf_16'),
+    }
+# All legal variants of the BOM codecs.
+# TODO: the list of aliases is not meant to be exhaustive, is there a
+#   better way ?
+BOM_LIST = {
+    'utf_16': 'utf_16',
+    'u16': 'utf_16',
+    'utf16': 'utf_16',
+    'utf-16': 'utf_16',
+    'utf16_be': 'utf16_be',
+    'utf_16_be': 'utf16_be',
+    'utf-16be': 'utf16_be',
+    'utf16_le': 'utf16_le',
+    'utf_16_le': 'utf16_le',
+    'utf-16le': 'utf16_le',
+    'utf_8': 'utf_8',
+    'u8': 'utf_8',
+    'utf': 'utf_8',
+    'utf8': 'utf_8',
+    'utf-8': 'utf_8',
+    }
+
+# Map of encodings to the BOM to write.
+BOM_SET = {
+    'utf_8': BOM_UTF8,
+    'utf_16': BOM_UTF16,
+    'utf16_be': BOM_UTF16_BE,
+    'utf16_le': BOM_UTF16_LE,
+    None: BOM_UTF8
+    }
+
+try:
+    from validate import VdtMissingValue
+except ImportError:
+    VdtMissingValue = None
+
+try:
+    enumerate
+except NameError:
+    def enumerate(obj):
+        """enumerate for Python 2.2."""
+        i = -1
+        for item in obj:
+            i += 1
+            yield i, item
+
+try:
+    True, False
+except NameError:
+    True, False = 1, 0
+
+
+__version__ = '4.2.0beta2'
+
+__revision__ = '$Id: configobj.py 156 2006-01-31 14:57:08Z fuzzyman $'
+
+__docformat__ = "restructuredtext en"
+
+# NOTE: Does it make sense to have the following in __all__ ?
+# NOTE: DEFAULT_INDENT_TYPE, NUM_INDENT_SPACES, MAX_INTERPOL_DEPTH
+# NOTE: If used as from configobj import...
+# NOTE: They are effectively read only
+__all__ = (
+    '__version__',
+    'DEFAULT_INDENT_TYPE',
+    'NUM_INDENT_SPACES',
+    'MAX_INTERPOL_DEPTH',
+    'ConfigObjError',
+    'NestingError',
+    'ParseError',
+    'DuplicateError',
+    'ConfigspecError',
+    'ConfigObj',
+    'SimpleVal',
+    'InterpolationError',
+    'InterpolationDepthError',
+    'MissingInterpolationOption',
+    'RepeatSectionError',
+    '__docformat__',
+    'flatten_errors',
+)
+
+DEFAULT_INDENT_TYPE = ' '
+NUM_INDENT_SPACES = 4
+MAX_INTERPOL_DEPTH = 10
+
+OPTION_DEFAULTS = {
+    'interpolation': True,
+    'raise_errors': False,
+    'list_values': True,
+    'create_empty': False,
+    'file_error': False,
+    'configspec': None,
+    'stringify': True,
+    # option may be set to one of ('', ' ', '\t')
+    'indent_type': None,
+    'encoding': None,
+    'default_encoding': None,
+}
+
+class ConfigObjError(SyntaxError):
+    """
+    This is the base class for all errors that ConfigObj raises.
+    It is a subclass of SyntaxError.
+    
+    >>> raise ConfigObjError
+    Traceback (most recent call last):
+    ConfigObjError
+    """
+    def __init__(self, message='', line_number=None, line=''):
+        self.line = line
+        self.line_number = line_number
+        self.message = message
+        SyntaxError.__init__(self, message)
+
+class NestingError(ConfigObjError):
+    """
+    This error indicates a level of nesting that doesn't match.
+    
+    >>> raise NestingError
+    Traceback (most recent call last):
+    NestingError
+    """
+
+class ParseError(ConfigObjError):
+    """
+    This error indicates that a line is badly written.
+    It is neither a valid ``key = value`` line,
+    nor a valid section marker line.
+    
+    >>> raise ParseError
+    Traceback (most recent call last):
+    ParseError
+    """
+
+class DuplicateError(ConfigObjError):
+    """
+    The keyword or section specified already exists.
+    
+    >>> raise DuplicateError
+    Traceback (most recent call last):
+    DuplicateError
+    """
+
+class ConfigspecError(ConfigObjError):
+    """
+    An error occured whilst parsing a configspec.
+    
+    >>> raise ConfigspecError
+    Traceback (most recent call last):
+    ConfigspecError
+    """
+
+class InterpolationError(ConfigObjError):
+    """Base class for the two interpolation errors."""
+
+class InterpolationDepthError(InterpolationError):
+    """Maximum interpolation depth exceeded in string interpolation."""
+
+    def __init__(self, option):
+        """
+        >>> raise InterpolationDepthError('yoda')
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "yoda".
+        """
+        InterpolationError.__init__(
+            self,
+            'max interpolation depth exceeded in value "%s".' % option)
+
+class RepeatSectionError(ConfigObjError):
+    """
+    This error indicates additional sections in a section with a
+    ``__many__`` (repeated) section.
+    
+    >>> raise RepeatSectionError
+    Traceback (most recent call last):
+    RepeatSectionError
+    """
+
+class MissingInterpolationOption(InterpolationError):
+    """A value specified for interpolation was missing."""
+
+    def __init__(self, option):
+        """
+        >>> raise MissingInterpolationOption('yoda')
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "yoda" in interpolation.
+        """
+        InterpolationError.__init__(
+            self,
+            'missing option "%s" in interpolation.' % option)
+
+class Section(dict):
+    """
+    A dictionary-like object that represents a section in a config file.
+    
+    It does string interpolation if the 'interpolate' attribute
+    of the 'main' object is set to True.
+    
+    Interpolation is tried first from the 'DEFAULT' section of this object,
+    next from the 'DEFAULT' section of the parent, lastly the main object.
+    
+    A Section will behave like an ordered dictionary - following the
+    order of the ``scalars`` and ``sections`` attributes.
+    You can use this to change the order of members.
+    
+    Iteration follows the order: scalars, then sections.
+    """
+
+    _KEYCRE = re.compile(r"%\(([^)]*)\)s|.")
+
+    def __init__(self, parent, depth, main, indict=None, name=None):
+        """
+        * parent is the section above
+        * depth is the depth level of this section
+        * main is the main ConfigObj
+        * indict is a dictionary to initialise the section with
+        """
+        if indict is None:
+            indict = {}
+        dict.__init__(self)
+        # used for nesting level *and* interpolation
+        self.parent = parent
+        # used for the interpolation attribute
+        self.main = main
+        # level of nesting depth of this Section
+        self.depth = depth
+        # the sequence of scalar values in this Section
+        self.scalars = []
+        # the sequence of sections in this Section
+        self.sections = []
+        # purely for information
+        self.name = name
+        # for comments :-)
+        self.comments = {}
+        self.inline_comments = {}
+        # for the configspec
+        self.configspec = {}
+        # for defaults
+        self.defaults = []
+        #
+        # we do this explicitly so that __setitem__ is used properly
+        # (rather than just passing to ``dict.__init__``)
+        for entry in indict:
+            self[entry] = indict[entry]
+
+    def _interpolate(self, value):
+        """Nicked from ConfigParser."""
+        depth = MAX_INTERPOL_DEPTH
+        # loop through this until it's done
+        while depth:
+            depth -= 1
+            if value.find("%(") != -1:
+                value = self._KEYCRE.sub(self._interpolation_replace, value)
+            else:
+                break
+        else:
+            raise InterpolationDepthError(value)
+        return value
+
+    def _interpolation_replace(self, match):
+        """ """
+        s = match.group(1)
+        if s is None:
+            return match.group()
+        else:
+            # switch off interpolation before we try and fetch anything !
+            self.main.interpolation = False
+            # try the 'DEFAULT' member of *this section* first
+            val = self.get('DEFAULT', {}).get(s)
+            # try the 'DEFAULT' member of the *parent section* next
+            if val is None:
+                val = self.parent.get('DEFAULT', {}).get(s)
+            # last, try the 'DEFAULT' member of the *main section*
+            if val is None:
+                val = self.main.get('DEFAULT', {}).get(s)
+            self.main.interpolation = True
+            if val is None:
+                raise MissingInterpolationOption(s)
+            return val
+
+    def __getitem__(self, key):
+        """Fetch the item and do string interpolation."""
+        val = dict.__getitem__(self, key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def __setitem__(self, key, value):
+        """
+        Correctly set a value.
+        
+        Making dictionary values Section instances.
+        (We have to special case 'Section' instances - which are also dicts)
+        
+        Keys must be strings.
+        Values need only be strings (or lists of strings) if
+        ``main.stringify`` is set.
+        """
+        if not isinstance(key, StringTypes):
+            raise ValueError, 'The key "%s" is not a string.' % key
+        # add the comment
+        if not self.comments.has_key(key):
+            self.comments[key] = []
+            self.inline_comments[key] = ''
+        # remove the entry from defaults
+        if key in self.defaults:
+            self.defaults.remove(key)
+        #
+        if isinstance(value, Section):
+            if not self.has_key(key):
+                self.sections.append(key)
+            dict.__setitem__(self, key, value)
+        elif isinstance(value, dict):
+            # First create the new depth level,
+            # then create the section
+            if not self.has_key(key):
+                self.sections.append(key)
+            new_depth = self.depth + 1
+            dict.__setitem__(
+                self,
+                key,
+                Section(
+                    self,
+                    new_depth,
+                    self.main,
+                    indict=value,
+                    name=key))
+        else:
+            if not self.has_key(key):
+                self.scalars.append(key)
+            if not self.main.stringify:
+                if isinstance(value, StringTypes):
+                    pass
+                elif isinstance(value, (list, tuple)):
+                    for entry in value:
+                        if not isinstance(entry, StringTypes):
+                            raise TypeError, (
+                                'Value is not a string "%s".' % entry)
+                else:
+                    raise TypeError, 'Value is not a string "%s".' % value
+            dict.__setitem__(self, key, value)
+
+    def __delitem__(self, key):
+        """Remove items from the sequence when deleting."""
+        dict. __delitem__(self, key)
+        if key in self.scalars:
+            self.scalars.remove(key)
+        else:
+            self.sections.remove(key)
+        del self.comments[key]
+        del self.inline_comments[key]
+
+    def get(self, key, default=None):
+        """A version of ``get`` that doesn't bypass string interpolation."""
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def update(self, indict):
+        """
+        A version of update that uses our ``__setitem__``.
+        """
+        for entry in indict:
+            self[entry] = indict[entry]
+
+
+    def pop(self, key, *args):
+        """ """
+        val = dict.pop(self, key, *args)
+        if key in self.scalars:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.scalars.remove(key)
+        elif key in self.sections:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.sections.remove(key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def popitem(self):
+        """Pops the first (key,val)"""
+        sequence = (self.scalars + self.sections)
+        if not sequence:
+            raise KeyError, ": 'popitem(): dictionary is empty'"
+        key = sequence[0]
+        val =  self[key]
+        del self[key]
+        return key, val
+
+    def clear(self):
+        """
+        A version of clear that also affects scalars/sections
+        Also clears comments and configspec.
+        
+        Leaves other attributes alone :
+            depth/main/parent are not affected
+        """
+        dict.clear(self)
+        self.scalars = []
+        self.sections = []
+        self.comments = {}
+        self.inline_comments = {}
+        self.configspec = {}
+
+    def setdefault(self, key, default=None):
+        """A version of setdefault that sets sequence if appropriate."""
+        try:
+            return self[key]
+        except KeyError:
+            self[key] = default
+            return self[key]
+
+    def items(self):
+        """ """
+        return zip((self.scalars + self.sections), self.values())
+
+    def keys(self):
+        """ """
+        return (self.scalars + self.sections)
+
+    def values(self):
+        """ """
+        return [self[key] for key in (self.scalars + self.sections)]
+
+    def iteritems(self):
+        """ """
+        return iter(self.items())
+
+    def iterkeys(self):
+        """ """
+        return iter((self.scalars + self.sections))
+
+    __iter__ = iterkeys
+
+    def itervalues(self):
+        """ """
+        return iter(self.values())
+
+    def __repr__(self):
+        return '{%s}' % ', '.join([('%s: %s' % (repr(key), repr(self[key])))
+            for key in (self.scalars + self.sections)])
+
+    __str__ = __repr__
+
+    # Extra methods - not in a normal dictionary
+
+    def dict(self):
+        """
+        Return a deepcopy of self as a dictionary.
+        
+        All members that are ``Section`` instances are recursively turned to
+        ordinary dictionaries - by calling their ``dict`` method.
+        
+        >>> n = a.dict()
+        >>> n == a
+        1
+        >>> n is a
+        0
+        """
+        newdict = {}
+        for entry in self:
+            this_entry = self[entry]
+            if isinstance(this_entry, Section):
+                this_entry = this_entry.dict()
+            elif isinstance(this_entry, (list, tuple)):
+                # create a copy rather than a reference
+                this_entry = list(this_entry)
+            newdict[entry] = this_entry
+        return newdict
+
+    def merge(self, indict):
+        """
+        A recursive update - useful for merging config files.
+        
+        >>> a = '''[section1]
+        ...     option1 = True
+        ...     [[subsection]]
+        ...     more_options = False
+        ...     # end of file'''.splitlines()
+        >>> b = '''# File is user.ini
+        ...     [section1]
+        ...     option1 = False
+        ...     # end of file'''.splitlines()
+        >>> c1 = ConfigObj(b)
+        >>> c2 = ConfigObj(a)
+        >>> c2.merge(c1)
+        >>> c2
+        {'section1': {'option1': 'False', 'subsection': {'more_options': 'False'}}}
+        """
+        for key, val in indict.items():
+            if (key in self and isinstance(self[key], dict) and
+                                isinstance(val, dict)):
+                self[key].merge(val)
+            else:   
+                self[key] = val
+
+    def rename(self, oldkey, newkey):
+        """
+        Change a keyname to another, without changing position in sequence.
+        
+        Implemented so that transformations can be made on keys,
+        as well as on values. (used by encode and decode)
+        
+        Also renames comments.
+        """
+        if oldkey in self.scalars:
+            the_list = self.scalars
+        elif oldkey in self.sections:
+            the_list = self.sections
+        else:
+            raise KeyError, 'Key "%s" not found.' % oldkey
+        pos = the_list.index(oldkey)
+        #
+        val = self[oldkey]
+        dict.__delitem__(self, oldkey)
+        dict.__setitem__(self, newkey, val)
+        the_list.remove(oldkey)
+        the_list.insert(pos, newkey)
+        comm = self.comments[oldkey]
+        inline_comment = self.inline_comments[oldkey]
+        del self.comments[oldkey]
+        del self.inline_comments[oldkey]
+        self.comments[newkey] = comm
+        self.inline_comments[newkey] = inline_comment
+
+    def walk(self, function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+        """
+        Walk every member and call a function on the keyword and value.
+        
+        Return a dictionary of the return values
+        
+        If the function raises an exception, raise the errror
+        unless ``raise_errors=False``, in which case set the return value to
+        ``False``.
+        
+        Any unrecognised keyword arguments you pass to walk, will be pased on
+        to the function you pass in.
+        
+        Note: if ``call_on_sections`` is ``True`` then - on encountering a
+        subsection, *first* the function is called for the *whole* subsection,
+        and then recurses into it's members. This means your function must be
+        able to handle strings, dictionaries and lists. This allows you
+        to change the key of subsections as well as for ordinary members. The
+        return value when called on the whole subsection has to be discarded.
+        
+        See  the encode and decode methods for examples, including functions.
+        
+        .. caution::
+        
+            You can use ``walk`` to transform the names of members of a section
+            but you mustn't add or delete members.
+        
+        >>> config = '''[XXXXsection]
+        ... XXXXkey = XXXXvalue'''.splitlines()
+        >>> cfg = ConfigObj(config)
+        >>> cfg
+        {'XXXXsection': {'XXXXkey': 'XXXXvalue'}}
+        >>> def transform(section, key):
+        ...     val = section[key]
+        ...     newkey = key.replace('XXXX', 'CLIENT1')
+        ...     section.rename(key, newkey)
+        ...     if isinstance(val, (tuple, list, dict)):
+        ...         pass
+        ...     else:
+        ...         val = val.replace('XXXX', 'CLIENT1')
+        ...         section[newkey] = val
+        >>> cfg.walk(transform, call_on_sections=True)
+        {'CLIENT1section': {'CLIENT1key': None}}
+        >>> cfg
+        {'CLIENT1section': {'CLIENT1key': 'CLIENT1value'}}
+        """
+        out = {}
+        # scalars first
+        for i in range(len(self.scalars)):
+            entry = self.scalars[i]
+            try:
+                val = function(self, entry, **keywargs)
+                # bound again in case name has changed
+                entry = self.scalars[i]
+                out[entry] = val
+            except Exception:
+                if raise_errors:
+                    raise
+                else:
+                    entry = self.scalars[i]
+                    out[entry] = False
+        # then sections
+        for i in range(len(self.sections)):
+            entry = self.sections[i]
+            if call_on_sections:
+                try:
+                    function(self, entry, **keywargs)
+                except Exception:
+                    if raise_errors:
+                        raise
+                    else:
+                        entry = self.sections[i]
+                        out[entry] = False
+                # bound again in case name has changed
+                entry = self.sections[i]
+            # previous result is discarded
+            out[entry] = self[entry].walk(
+                function,
+                raise_errors=raise_errors,
+                call_on_sections=call_on_sections,
+                **keywargs)
+        return out
+
+    def decode(self, encoding):
+        """
+        Decode all strings and values to unicode, using the specified encoding.
+        
+        Works with subsections and list values.
+        
+        Uses the ``walk`` method.
+        
+        Testing ``encode`` and ``decode``.
+        >>> m = ConfigObj(a)
+        >>> m.decode('ascii')
+        >>> def testuni(val):
+        ...     for entry in val:
+        ...         if not isinstance(entry, unicode):
+        ...             print >> sys.stderr, type(entry)
+        ...             raise AssertionError, 'decode failed.'
+        ...         if isinstance(val[entry], dict):
+        ...             testuni(val[entry])
+        ...         elif not isinstance(val[entry], unicode):
+        ...             raise AssertionError, 'decode failed.'
+        >>> testuni(m)
+        >>> m.encode('ascii')
+        >>> a == m
+        1
+        """
+        def decode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.decode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.decode(encoding)
+            newkey = key.decode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        # using ``call_on_sections`` allows us to modify section names
+        self.walk(decode, call_on_sections=True)
+
+    def encode(self, encoding):
+        """
+        Encode all strings and values from unicode,
+        using the specified encoding.
+        
+        Works with subsections and list values.
+        Uses the ``walk`` method.
+        """
+        def encode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.encode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.encode(encoding)
+            newkey = key.encode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        self.walk(encode, call_on_sections=True)
+
+    def istrue(self, key):
+        """A deprecated version of ``as_bool``."""
+        warn('use of ``istrue`` is deprecated. Use ``as_bool`` method '
+                'instead.', DeprecationWarning)
+        return self.as_bool(key)
+
+    def as_bool(self, key):
+        """
+        Accepts a key as input. The corresponding value must be a string or
+        the objects (``True`` or 1) or (``False`` or 0). We allow 0 and 1 to
+        retain compatibility with Python 2.2.
+        
+        If the string is one of  ``True``, ``On``, ``Yes``, or ``1`` it returns 
+        ``True``.
+        
+        If the string is one of  ``False``, ``Off``, ``No``, or ``0`` it returns 
+        ``False``.
+        
+        ``as_bool`` is not case sensitive.
+        
+        Any other input will raise a ``ValueError``.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_bool('a')
+        Traceback (most recent call last):
+        ValueError: Value "fish" is neither True nor False
+        >>> a['b'] = 'True'
+        >>> a.as_bool('b')
+        1
+        >>> a['b'] = 'off'
+        >>> a.as_bool('b')
+        0
+        """
+        val = self[key]
+        if val == True:
+            return True
+        elif val == False:
+            return False
+        else:
+            try:
+                if not isinstance(val, StringTypes):
+                    raise KeyError
+                else:
+                    return self.main._bools[val.lower()]
+            except KeyError:
+                raise ValueError('Value "%s" is neither True nor False' % val)
+
+    def as_int(self, key):
+        """
+        A convenience method which coerces the specified value to an integer.
+        
+        If the value is an invalid literal for ``int``, a ``ValueError`` will
+        be raised.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_int('a')
+        Traceback (most recent call last):
+        ValueError: invalid literal for int(): fish
+        >>> a['b'] = '1'
+        >>> a.as_int('b')
+        1
+        >>> a['b'] = '3.2'
+        >>> a.as_int('b')
+        Traceback (most recent call last):
+        ValueError: invalid literal for int(): 3.2
+        """
+        return int(self[key])
+
+    def as_float(self, key):
+        """
+        A convenience method which coerces the specified value to a float.
+        
+        If the value is an invalid literal for ``float``, a ``ValueError`` will
+        be raised.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_float('a')
+        Traceback (most recent call last):
+        ValueError: invalid literal for float(): fish
+        >>> a['b'] = '1'
+        >>> a.as_float('b')
+        1.0
+        >>> a['b'] = '3.2'
+        >>> a.as_float('b')
+        3.2000000000000002
+        """
+        return float(self[key])
+    
+
+class ConfigObj(Section):
+    """
+    An object to read, create, and write config files.
+    
+    Testing with duplicate keys and sections.
+    
+    >>> c = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member = value
+    ... [ "hello" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(c.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate section name at line 5.
+    
+    >>> d = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member1 = value
+    ... member2 = value
+    ... 'member1' = value
+    ... [ "and again" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(d.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate keyword name at line 6.
+    """
+
+    _keyword = re.compile(r'''^ # line start
+        (\s*)                   # indentation
+        (                       # keyword
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"=].*?)       # no quotes
+        )
+        \s*=\s*                 # divider
+        (.*)                    # value (including list values and comments)
+        $   # line end
+        ''',
+        re.VERBOSE)
+
+    _sectionmarker = re.compile(r'''^
+        (\s*)                     # 1: indentation
+        ((?:\[\s*)+)              # 2: section marker open
+        (                         # 3: section name open
+            (?:"\s*\S.*?\s*")|    # at least one non-space with double quotes
+            (?:'\s*\S.*?\s*')|    # at least one non-space with single quotes
+            (?:[^'"\s].*?)        # at least one non-space unquoted
+        )                         # section name close
+        ((?:\s*\])+)              # 4: section marker close
+        \s*(\#.*)?                # 5: optional comment
+        $''',
+        re.VERBOSE)
+
+    # this regexp pulls list values out as a single string
+    # or single values and comments
+    _valueexp = re.compile(r'''^
+        (?:
+            (?:
+                (
+                    (?:
+                        (?:
+                            (?:".*?")|              # double quotes
+                            (?:'.*?')|              # single quotes
+                            (?:[^'",\#][^,\#]*?)       # unquoted
+                        )
+                        \s*,\s*                     # comma
+                    )*      # match all list items ending in a comma (if any)
+                )
+                (
+                    (?:".*?")|                      # double quotes
+                    (?:'.*?')|                      # single quotes
+                    (?:[^'",\#\s][^,]*?)             # unquoted
+                )?          # last item in a list - or string value
+            )|
+            (,)             # alternatively a single comma - empty list
+        )
+        \s*(\#.*)?          # optional comment
+        $''',
+        re.VERBOSE)
+
+    # use findall to get the members of a list value
+    _listvalueexp = re.compile(r'''
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'",\#].*?)       # unquoted
+        )
+        \s*,\s*                 # comma
+        ''',
+        re.VERBOSE)
+
+    # this regexp is used for the value
+    # when lists are switched off
+    _nolistvalue = re.compile(r'''^
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"\#].*?)      # unquoted
+        )
+        \s*(\#.*)?              # optional comment
+        $''',
+        re.VERBOSE)
+
+    # regexes for finding triple quoted values on one line
+    _single_line_single = re.compile(r"^'''(.*?)'''\s*(#.*)?$")
+    _single_line_double = re.compile(r'^"""(.*?)"""\s*(#.*)?$')
+    _multi_line_single = re.compile(r"^(.*?)'''\s*(#.*)?$")
+    _multi_line_double = re.compile(r'^(.*?)"""\s*(#.*)?$')
+
+    _triple_quote = {
+        "'''": (_single_line_single, _multi_line_single),
+        '"""': (_single_line_double, _multi_line_double),
+    }
+
+    # Used by the ``istrue`` Section method
+    _bools = {
+        'yes': True, 'no': False,
+        'on': True, 'off': False,
+        '1': True, '0': False,
+        'true': True, 'false': False,
+        }
+
+    def __init__(self, infile=None, options=None, **kwargs):
+        """
+        Parse or create a config file object.
+        
+        ``ConfigObj(infile=None, options=None, **kwargs)``
+        """
+        if infile is None:
+            infile = []
+        if options is None:
+            options = {}
+        # keyword arguments take precedence over an options dictionary
+        options.update(kwargs)
+        # init the superclass
+        Section.__init__(self, self, 0, self)
+        #
+        defaults = OPTION_DEFAULTS.copy()
+        for entry in options.keys():
+            if entry not in defaults.keys():
+                raise TypeError, 'Unrecognised option "%s".' % entry
+        # TODO: check the values too.
+        #
+        # Add any explicit options to the defaults
+        defaults.update(options)
+        #
+        # initialise a few variables
+        self.filename = None
+        self._errors = []
+        self.raise_errors = defaults['raise_errors']
+        self.interpolation = defaults['interpolation']
+        self.list_values = defaults['list_values']
+        self.create_empty = defaults['create_empty']
+        self.file_error = defaults['file_error']
+        self.stringify = defaults['stringify']
+        self.indent_type = defaults['indent_type']
+        self.encoding = defaults['encoding']
+        self.default_encoding = defaults['default_encoding']
+        self.BOM = False
+        self.newlines = None
+        #
+        self.initial_comment = []
+        self.final_comment = []
+        #
+        if isinstance(infile, StringTypes):
+            self.filename = infile
+            if os.path.isfile(infile):
+                infile = open(infile).read() or []
+            elif self.file_error:
+                # raise an error if the file doesn't exist
+                raise IOError, 'Config file not found: "%s".' % self.filename
+            else:
+                # file doesn't already exist
+                if self.create_empty:
+                    # this is a good test that the filename specified
+                    # isn't impossible - like on a non existent device
+                    h = open(infile, 'w')
+                    h.write('')
+                    h.close()
+                infile = []
+        elif isinstance(infile, (list, tuple)):
+            infile = list(infile)
+        elif isinstance(infile, dict):
+            # initialise self
+            # the Section class handles creating subsections
+            if isinstance(infile, ConfigObj):
+                # get a copy of our ConfigObj
+                infile = infile.dict()
+            for entry in infile:
+                self[entry] = infile[entry]
+            del self._errors
+            if defaults['configspec'] is not None:
+                self._handle_configspec(defaults['configspec'])
+            else:
+                self.configspec = None
+            return
+        elif hasattr(infile, 'read'):
+            # This supports file like objects
+            infile = infile.read() or []
+            # needs splitting into lines - but needs doing *after* decoding
+            # in case it's not an 8 bit encoding
+        else:
+            raise TypeError, ('infile must be a filename,'
+                ' file like object, or list of lines.')
+        #
+        if infile:
+            # don't do it for the empty ConfigObj
+            infile = self._handle_bom(infile)
+            # infile is now *always* a list
+            #
+            # Set the newlines attribute (first line ending it finds)
+            # and strip trailing '\n' or '\r' from lines
+            for line in infile:
+                if (not line) or (line[-1] not in '\r\n'):
+                    continue
+                for end in ('\r\n', '\n', '\r'):
+                    if line.endswith(end):
+                        self.newlines = end
+                        break
+                break
+            infile = [line.rstrip('\r\n') for line in infile]
+        #
+        self._parse(infile)
+        # if we had any errors, now is the time to raise them
+        if self._errors:
+            error = ConfigObjError("Parsing failed.")
+            # set the errors attribute; it's a list of tuples:
+            # (error_type, message, line_number)
+            error.errors = self._errors
+            # set the config attribute
+            error.config = self
+            raise error
+        # delete private attributes
+        del self._errors
+        #
+        if defaults['configspec'] is None:
+            self.configspec = None
+        else:
+            self._handle_configspec(defaults['configspec'])
+
+    def _handle_bom(self, infile):
+        """
+        Handle any BOM, and decode if necessary.
+        
+        If an encoding is specified, that *must* be used - but the BOM should
+        still be removed (and the BOM attribute set).
+        
+        (If the encoding is wrongly specified, then a BOM for an alternative
+        encoding won't be discovered or removed.)
+        
+        If an encoding is not specified, UTF8 or UTF16 BOM will be detected and
+        removed. The BOM attribute will be set. UTF16 will be decoded to
+        unicode.
+        
+        NOTE: This method must not be called with an empty ``infile``.
+        
+        Specifying the *wrong* encoding is likely to cause a
+        ``UnicodeDecodeError``.
+        
+        ``infile`` must always be returned as a list of lines, but may be
+        passed in as a single string.
+        """
+        if ((self.encoding is not None) and
+            (self.encoding.lower() not in BOM_LIST)):
+            # No need to check for a BOM
+            # encoding specified doesn't have one
+            # just decode
+            return self._decode(infile, self.encoding)
+        #
+        if isinstance(infile, (list, tuple)):
+            line = infile[0]
+        else:
+            line = infile
+        if self.encoding is not None:
+            # encoding explicitly supplied
+            # And it could have an associated BOM
+            # TODO: if encoding is just UTF16 - we ought to check for both
+            # TODO: big endian and little endian versions.
+            enc = BOM_LIST[self.encoding.lower()]
+            if enc == 'utf_16':
+                # For UTF16 we try big endian and little endian
+                for BOM, (encoding, final_encoding) in BOMS.items():
+                    if not final_encoding:
+                        # skip UTF8
+                        continue
+                    if infile.startswith(BOM):
+                        ### BOM discovered
+                        ##self.BOM = True
+                        # Don't need to remove BOM
+                        return self._decode(infile, encoding)
+                #
+                # If we get this far, will *probably* raise a DecodeError
+                # As it doesn't appear to start with a BOM
+                return self._decode(infile, self.encoding)
+            #
+            # Must be UTF8
+            BOM = BOM_SET[enc]
+            if not line.startswith(BOM):
+                return self._decode(infile, self.encoding)
+            #
+            newline = line[len(BOM):]
+            #
+            # BOM removed
+            if isinstance(infile, (list, tuple)):
+                infile[0] = newline
+            else:
+                infile = newline
+            self.BOM = True
+            return self._decode(infile, self.encoding)
+        #
+        # No encoding specified - so we need to check for UTF8/UTF16
+        for BOM, (encoding, final_encoding) in BOMS.items():
+            if not line.startswith(BOM):
+                continue
+            else:
+                # BOM discovered
+                self.encoding = final_encoding
+                if not final_encoding:
+                    self.BOM = True
+                    # UTF8
+                    # remove BOM
+                    newline = line[len(BOM):]
+                    if isinstance(infile, (list, tuple)):
+                        infile[0] = newline
+                    else:
+                        infile = newline
+                    # UTF8 - don't decode
+                    if isinstance(infile, StringTypes):
+                        return infile.splitlines(True)
+                    else:
+                        return infile
+                # UTF16 - have to decode
+                return self._decode(infile, encoding)
+        #
+        # No BOM discovered and no encoding specified, just return
+        if isinstance(infile, StringTypes):
+            # infile read from a file will be a single string
+            return infile.splitlines(True)
+        else:
+            return infile
+
+    def _a_to_u(self, string):
+        """Decode ascii strings to unicode if a self.encoding is specified."""
+        if not self.encoding:
+            return string
+        else:
+            return string.decode('ascii')
+
+    def _decode(self, infile, encoding):
+        """
+        Decode infile to unicode. Using the specified encoding.
+        
+        if is a string, it also needs converting to a list.
+        """
+        if isinstance(infile, StringTypes):
+            # can't be unicode
+            # NOTE: Could raise a ``UnicodeDecodeError``
+            return infile.decode(encoding).splitlines(True)
+        for i, line in enumerate(infile):
+            if not isinstance(line, unicode):
+                # NOTE: The isinstance test here handles mixed lists of unicode/string
+                # NOTE: But the decode will break on any non-string values
+                # NOTE: Or could raise a ``UnicodeDecodeError``
+                infile[i] = line.decode(encoding)
+        return infile
+
+    def _decode_element(self, line):
+        """Decode element to unicode if necessary."""
+        if not self.encoding:
+            return line
+        if isinstance(line, str) and self.default_encoding:
+            return line.decode(self.default_encoding)
+        return line
+
+    def _str(self, value):
+        """
+        Used by ``stringify`` within validate, to turn non-string values
+        into strings.
+        """
+        if not isinstance(value, StringTypes):
+            return str(value)
+        else:
+            return value
+
+    def _parse(self, infile):
+        """
+        Actually parse the config file
+        
+        Testing Interpolation
+        
+        >>> c = ConfigObj()
+        >>> c['DEFAULT'] = {
+        ...     'b': 'goodbye',
+        ...     'userdir': 'c:\\\\home',
+        ...     'c': '%(d)s',
+        ...     'd': '%(c)s'
+        ... }
+        >>> c['section'] = {
+        ...     'a': '%(datadir)s\\\\some path\\\\file.py',
+        ...     'b': '%(userdir)s\\\\some path\\\\file.py',
+        ...     'c': 'Yo %(a)s',
+        ...     'd': '%(not_here)s',
+        ...     'e': '%(c)s',
+        ... }
+        >>> c['section']['DEFAULT'] = {
+        ...     'datadir': 'c:\\\\silly_test',
+        ...     'a': 'hello - %(b)s',
+        ... }
+        >>> c['section']['a'] == 'c:\\\\silly_test\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == 'c:\\\\home\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo hello - goodbye'
+        1
+        
+        Switching Interpolation Off
+        
+        >>> c.interpolation = False
+        >>> c['section']['a'] == '%(datadir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == '%(userdir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo %(a)s'
+        1
+        
+        Testing the interpolation errors.
+        
+        >>> c.interpolation = True
+        >>> c['section']['d']
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "not_here" in interpolation.
+        >>> c['section']['e']
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "%(c)s".
+        
+        Testing our quoting.
+        
+        >>> i._quote('\"""\'\'\'')
+        Traceback (most recent call last):
+        SyntaxError: EOF while scanning triple-quoted string
+        >>> try:
+        ...     i._quote('\\n', multiline=False)
+        ... except ConfigObjError, e:
+        ...    e.msg
+        'Value "\\n" cannot be safely quoted.'
+        >>> k._quote(' "\' ', multiline=False)
+        Traceback (most recent call last):
+        SyntaxError: EOL while scanning single-quoted string
+        
+        Testing with "stringify" off.
+        >>> c.stringify = False
+        >>> c['test'] = 1
+        Traceback (most recent call last):
+        TypeError: Value is not a string "1".
+        """
+        comment_list = []
+        done_start = False
+        this_section = self
+        maxline = len(infile) - 1
+        cur_index = -1
+        reset_comment = False
+        while cur_index < maxline:
+            if reset_comment:
+                comment_list = []
+            cur_index += 1
+            line = infile[cur_index]
+            sline = line.strip()
+            # do we have anything on the line ?
+            if not sline or sline.startswith('#'):
+                reset_comment = False
+                comment_list.append(line)
+                continue
+            if not done_start:
+                # preserve initial comment
+                self.initial_comment = comment_list
+                comment_list = []
+                done_start = True
+            reset_comment = True
+            # first we check if it's a section marker
+            mat = self._sectionmarker.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a section line
+                (indent, sect_open, sect_name, sect_close, comment) = (
+                    mat.groups())
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                cur_depth = sect_open.count('[')
+                if cur_depth != sect_close.count(']'):
+                    self._handle_error(
+                        "Cannot compute the section depth at line %s.",
+                        NestingError, infile, cur_index)
+                    continue
+                if cur_depth < this_section.depth:
+                    # the new section is dropping back to a previous level
+                    try:
+                        parent = self._match_depth(
+                            this_section,
+                            cur_depth).parent
+                    except SyntaxError:
+                        self._handle_error(
+                            "Cannot compute nesting level at line %s.",
+                            NestingError, infile, cur_index)
+                        continue
+                elif cur_depth == this_section.depth:
+                    # the new section is a sibling of the current section
+                    parent = this_section.parent
+                elif cur_depth == this_section.depth + 1:
+                    # the new section is a child the current section
+                    parent = this_section
+                else:
+                    self._handle_error(
+                        "Section too nested at line %s.",
+                        NestingError, infile, cur_index)
+                #
+                sect_name = self._unquote(sect_name)
+                if parent.has_key(sect_name):
+##                    print >> sys.stderr, sect_name
+                    self._handle_error(
+                        'Duplicate section name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # create the new section
+                this_section = Section(
+                    parent,
+                    cur_depth,
+                    self,
+                    name=sect_name)
+                parent[sect_name] = this_section
+                parent.inline_comments[sect_name] = comment
+                parent.comments[sect_name] = comment_list
+##                print >> sys.stderr, parent[sect_name] is this_section
+                continue
+            #
+            # it's not a section marker,
+            # so it should be a valid ``key = value`` line
+            mat = self._keyword.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a keyword value
+                # value will include any inline comment
+                (indent, key, value) = mat.groups()
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                # check for a multiline value
+                if value[:3] in ['"""', "'''"]:
+                    try:
+                        (value, comment, cur_index) = self._multiline(
+                            value, infile, cur_index, maxline)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                else:
+                    # extract comment and lists
+                    try:
+                        (value, comment) = self._handle_value(value)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                #
+##                print >> sys.stderr, sline
+                key = self._unquote(key)
+                if this_section.has_key(key):
+                    self._handle_error(
+                        'Duplicate keyword name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # add the key
+##                print >> sys.stderr, this_section.name
+                this_section[key] = value
+                this_section.inline_comments[key] = comment
+                this_section.comments[key] = comment_list
+##                print >> sys.stderr, key, this_section[key]
+##                if this_section.name is not None:
+##                    print >> sys.stderr, this_section
+##                    print >> sys.stderr, this_section.parent
+##                    print >> sys.stderr, this_section.parent[this_section.name]
+                continue
+            #
+            # it neither matched as a keyword
+            # or a section marker
+            self._handle_error(
+                'Invalid line at line "%s".',
+                ParseError, infile, cur_index)
+        if self.indent_type is None:
+            # no indentation used, set the type accordingly
+            self.indent_type = ''
+        # preserve the final comment
+        if not self and not self.initial_comment:
+            self.initial_comment = comment_list
+        else:
+            self.final_comment = comment_list
+
+    def _match_depth(self, sect, depth):
+        """
+        Given a section and a depth level, walk back through the sections
+        parents to see if the depth level matches a previous section.
+        
+        Return a reference to the right section,
+        or raise a SyntaxError.
+        """
+        while depth < sect.depth:
+            if sect is sect.parent:
+                # we've reached the top level already
+                raise SyntaxError
+            sect = sect.parent
+        if sect.depth == depth:
+            return sect
+        # shouldn't get here
+        raise SyntaxError
+
+    def _handle_error(self, text, ErrorClass, infile, cur_index):
+        """
+        Handle an error according to the error settings.
+        
+        Either raise the error or store it.
+        The error will have occured at ``cur_index``
+        """
+        line = infile[cur_index]
+        message = text % cur_index
+        error = ErrorClass(message, cur_index, line)
+        if self.raise_errors:
+            # raise the error - parsing stops here
+            raise error
+        # store the error
+        # reraise when parsing has finished
+        self._errors.append(error)
+
+    def _unquote(self, value):
+        """Return an unquoted version of a value"""
+        if (value[0] == value[-1]) and (value[0] in ('"', "'")):
+            value = value[1:-1]
+        return value
+
+    def _quote(self, value, multiline=True):
+        """
+        Return a safely quoted version of a value.
+        
+        Raise a ConfigObjError if the value cannot be safely quoted.
+        If multiline is ``True`` (default) then use triple quotes
+        if necessary.
+        
+        Don't quote values that don't need it.
+        Recursively quote members of a list and return a comma joined list.
+        Multiline is ``False`` for lists.
+        Obey list syntax for empty and single member lists.
+        
+        If ``list_values=False`` then the value is only quoted if it contains
+        a ``\n`` (is multiline).
+        """
+        if isinstance(value, (list, tuple)):
+            if not value:
+                return ','
+            elif len(value) == 1:
+                return self._quote(value[0], multiline=False) + ','
+            return ', '.join([self._quote(val, multiline=False)
+                for val in value])
+        if not isinstance(value, StringTypes):
+            if self.stringify:
+                value = str(value)
+            else:
+                raise TypeError, 'Value "%s" is not a string.' % value
+        squot = "'%s'"
+        dquot = '"%s"'
+        noquot = "%s"
+        wspace_plus = ' \r\t\n\v\t\'"'
+        tsquot = '"""%s"""'
+        tdquot = "'''%s'''"
+        if not value:
+            return '""'
+        if (not self.list_values and '\n' not in value) or not (multiline and
+                ((("'" in value) and ('"' in value)) or ('\n' in value))):
+            if not self.list_values:
+                # we don't quote if ``list_values=False``
+                quot = noquot
+            # for normal values either single or double quotes will do
+            elif '\n' in value:
+                # will only happen if multiline is off - e.g. '\n' in key
+                raise ConfigObjError, ('Value "%s" cannot be safely quoted.' %
+                    value)
+            elif ((value[0] not in wspace_plus) and
+                    (value[-1] not in wspace_plus) and
+                    (',' not in value)):
+                quot = noquot
+            else:
+                if ("'" in value) and ('"' in value):
+                    raise ConfigObjError, (
+                        'Value "%s" cannot be safely quoted.' % value)
+                elif '"' in value:
+                    quot = squot
+                else:
+                    quot = dquot
+        else:
+            # if value has '\n' or "'" *and* '"', it will need triple quotes
+            if (value.find('"""') != -1) and (value.find("'''") != -1):
+                raise ConfigObjError, (
+                    'Value "%s" cannot be safely quoted.' % value)
+            if value.find('"""') == -1:
+                quot = tdquot
+            else:
+                quot = tsquot
+        return quot % value
+
+    def _handle_value(self, value):
+        """
+        Given a value string, unquote, remove comment,
+        handle lists. (including empty and single member lists)
+        
+        Testing list values.
+        
+        >>> testconfig3 = '''
+        ... a = ,
+        ... b = test,
+        ... c = test1, test2   , test3
+        ... d = test1, test2, test3,
+        ... '''
+        >>> d = ConfigObj(testconfig3.split('\\n'), raise_errors=True)
+        >>> d['a'] == []
+        1
+        >>> d['b'] == ['test']
+        1
+        >>> d['c'] == ['test1', 'test2', 'test3']
+        1
+        >>> d['d'] == ['test1', 'test2', 'test3']
+        1
+        
+        Testing with list values off.
+        
+        >>> e = ConfigObj(
+        ...     testconfig3.split('\\n'),
+        ...     raise_errors=True,
+        ...     list_values=False)
+        >>> e['a'] == ','
+        1
+        >>> e['b'] == 'test,'
+        1
+        >>> e['c'] == 'test1, test2   , test3'
+        1
+        >>> e['d'] == 'test1, test2, test3,'
+        1
+        
+        Testing creating from a dictionary.
+        
+        >>> f = {
+        ...     'key1': 'val1',
+        ...     'key2': 'val2',
+        ...     'section 1': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 1b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...     'section 2': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 2b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...      'key3': 'val3',
+        ... }
+        >>> g = ConfigObj(f)
+        >>> f == g
+        1
+        
+        Testing we correctly detect badly built list values (4 of them).
+        
+        >>> testconfig4 = '''
+        ... config = 3,4,,
+        ... test = 3,,4
+        ... fish = ,,
+        ... dummy = ,,hello, goodbye
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig4.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        
+        Testing we correctly detect badly quoted values (4 of them).
+        
+        >>> testconfig5 = '''
+        ... config = "hello   # comment
+        ... test = 'goodbye
+        ... fish = 'goodbye   # comment
+        ... dummy = "hello again
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig5.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        """
+        # do we look for lists in values ?
+        if not self.list_values:
+            mat = self._nolistvalue.match(value)
+            if mat is None:
+                raise SyntaxError
+            (value, comment) = mat.groups()
+            # NOTE: we don't unquote here
+            return (value, comment)
+        mat = self._valueexp.match(value)
+        if mat is None:
+            # the value is badly constructed, probably badly quoted,
+            # or an invalid list
+            raise SyntaxError
+        (list_values, single, empty_list, comment) = mat.groups()
+        if (list_values == '') and (single is None):
+            # change this if you want to accept empty values
+            raise SyntaxError
+        # NOTE: note there is no error handling from here if the regex
+        # is wrong: then incorrect values will slip through
+        if empty_list is not None:
+            # the single comma - meaning an empty list
+            return ([], comment)
+        if single is not None:
+            single = self._unquote(single)
+        if list_values == '':
+            # not a list value
+            return (single, comment)
+        the_list = self._listvalueexp.findall(list_values)
+        the_list = [self._unquote(val) for val in the_list]
+        if single is not None:
+            the_list += [single]
+        return (the_list, comment)
+
+    def _multiline(self, value, infile, cur_index, maxline):
+        """
+        Extract the value, where we are in a multiline situation
+        
+        Testing multiline values.
+        
+        >>> i == {
+        ...     'name4': ' another single line value ',
+        ...     'multi section': {
+        ...         'name4': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name2': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name3': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name1': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...     },
+        ...     'name2': ' another single line value ',
+        ...     'name3': ' a single line value ',
+        ...     'name1': ' a single line value ',
+        ... }
+        1
+        """
+        quot = value[:3]
+        newvalue = value[3:]
+        single_line = self._triple_quote[quot][0]
+        multi_line = self._triple_quote[quot][1]
+        mat = single_line.match(value)
+        if mat is not None:
+            retval = list(mat.groups())
+            retval.append(cur_index)
+            return retval
+        elif newvalue.find(quot) != -1:
+            # somehow the triple quote is missing
+            raise SyntaxError
+        #
+        while cur_index < maxline:
+            cur_index += 1
+            newvalue += '\n'
+            line = infile[cur_index]
+            if line.find(quot) == -1:
+                newvalue += line
+            else:
+                # end of multiline, process it
+                break
+        else:
+            # we've got to the end of the config, oops...
+            raise SyntaxError
+        mat = multi_line.match(line)
+        if mat is None:
+            # a badly formed line
+            raise SyntaxError
+        (value, comment) = mat.groups()
+        return (newvalue + value, comment, cur_index)
+
+    def _handle_configspec(self, configspec):
+        """Parse the configspec."""
+        try:
+            configspec = ConfigObj(
+                configspec,
+                raise_errors=True,
+                file_error=True,
+                list_values=False)
+        except ConfigObjError, e:
+            # FIXME: Should these errors have a reference
+            # to the already parsed ConfigObj ?
+            raise ConfigspecError('Parsing configspec failed: %s' % e)
+        except IOError, e:
+            raise IOError('Reading configspec failed: %s' % e)
+        self._set_configspec_value(configspec, self)
+
+    def _set_configspec_value(self, configspec, section):
+        """Used to recursively set configspec values."""
+        if '__many__' in configspec.sections:
+            section.configspec['__many__'] = configspec['__many__']
+            if len(configspec.sections) > 1:
+                # FIXME: can we supply any useful information here ?
+                raise RepeatSectionError
+        for entry in configspec.scalars:
+            section.configspec[entry] = configspec[entry]
+        for entry in configspec.sections:
+            if entry == '__many__':
+                continue
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._set_configspec_value(configspec[entry], section[entry])
+
+    def _handle_repeat(self, section, configspec):
+        """Dynamically assign configspec for repeated section."""
+        try:
+            section_keys = configspec.sections
+            scalar_keys = configspec.scalars
+        except AttributeError:
+            section_keys = [entry for entry in configspec 
+                                if isinstance(configspec[entry], dict)]
+            scalar_keys = [entry for entry in configspec 
+                                if not isinstance(configspec[entry], dict)]
+        if '__many__' in section_keys and len(section_keys) > 1:
+            # FIXME: can we supply any useful information here ?
+            raise RepeatSectionError
+        scalars = {}
+        sections = {}
+        for entry in scalar_keys:
+            val = configspec[entry]
+            scalars[entry] = val
+        for entry in section_keys:
+            val = configspec[entry]
+            if entry == '__many__':
+                scalars[entry] = val
+                continue
+            sections[entry] = val
+        #
+        section.configspec = scalars
+        for entry in sections:
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._handle_repeat(section[entry], sections[entry])
+
+    def _write_line(self, indent_string, entry, this_entry, comment):
+        """Write an individual line, for the write method"""
+        # NOTE: the calls to self._quote here handles non-StringType values.
+        return '%s%s%s%s%s' % (
+            indent_string,
+            self._decode_element(self._quote(entry, multiline=False)),
+            self._a_to_u(' = '),
+            self._decode_element(self._quote(this_entry)),
+            self._decode_element(comment))
+
+    def _write_marker(self, indent_string, depth, entry, comment):
+        """Write a section marker line"""
+        return '%s%s%s%s%s' % (
+            indent_string,
+            self._a_to_u('[' * depth),
+            self._quote(self._decode_element(entry), multiline=False),
+            self._a_to_u(']' * depth),
+            self._decode_element(comment))
+
+    def _handle_comment(self, comment):
+        """
+        Deal with a comment.
+        
+        >>> filename = a.filename
+        >>> a.filename = None
+        >>> values = a.write()
+        >>> index = 0
+        >>> while index < 23:
+        ...     index += 1
+        ...     line = values[index-1]
+        ...     assert line.endswith('# comment ' + str(index))
+        >>> a.filename = filename
+        
+        >>> start_comment = ['# Initial Comment', '', '#']
+        >>> end_comment = ['', '#', '# Final Comment']
+        >>> newconfig = start_comment + testconfig1.split('\\n') + end_comment
+        >>> nc = ConfigObj(newconfig)
+        >>> nc.initial_comment
+        ['# Initial Comment', '', '#']
+        >>> nc.final_comment
+        ['', '#', '# Final Comment']
+        >>> nc.initial_comment == start_comment
+        1
+        >>> nc.final_comment == end_comment
+        1
+        """
+        if not comment:
+            return ''
+        if self.indent_type == '\t':
+            start = self._a_to_u('\t')
+        else:
+            start = self._a_to_u(' ' * NUM_INDENT_SPACES)
+        if not comment.startswith('#'):
+            start += _a_to_u('# ')
+        return (start + comment)
+
+    def _compute_indent_string(self, depth):
+        """
+        Compute the indent string, according to current indent_type and depth
+        """
+        if self.indent_type == '':
+            # no indentation at all
+            return ''
+        if self.indent_type == '\t':
+            return '\t' * depth
+        if self.indent_type == ' ':
+            return ' ' * NUM_INDENT_SPACES * depth
+        raise SyntaxError
+
+    # Public methods
+
+    def write(self, outfile=None, section=None):
+        """
+        Write the current ConfigObj as a file
+        
+        tekNico: FIXME: use StringIO instead of real files
+        
+        >>> filename = a.filename
+        >>> a.filename = 'test.ini'
+        >>> a.write()
+        >>> a.filename = filename
+        >>> a == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> b.filename = 'test.ini'
+        >>> b.write()
+        >>> b == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> i.filename = 'test.ini'
+        >>> i.write()
+        >>> i == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> a = ConfigObj()
+        >>> a['DEFAULT'] = {'a' : 'fish'}
+        >>> a['a'] = '%(a)s'
+        >>> a.write()
+        ['a = %(a)s', '[DEFAULT]', 'a = fish']
+        """
+        if self.indent_type is None:
+            # this can be true if initialised from a dictionary
+            self.indent_type = DEFAULT_INDENT_TYPE
+        #
+        out = []
+        cs = self._a_to_u('#')
+        csp = self._a_to_u('# ')
+        if section is None:
+            int_val = self.interpolation
+            self.interpolation = False
+            section = self
+            for line in self.initial_comment:
+                line = self._decode_element(line)
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith(cs):
+                    line = csp + line
+                out.append(line)
+        #
+        indent_string = self._a_to_u(
+            self._compute_indent_string(section.depth))
+        for entry in (section.scalars + section.sections):
+            if entry in section.defaults:
+                # don't write out default values
+                continue
+            for comment_line in section.comments[entry]:
+                comment_line = self._decode_element(comment_line.lstrip())
+                if comment_line and not comment_line.startswith(cs):
+                    comment_line = csp + comment_line
+                out.append(indent_string + comment_line)
+            this_entry = section[entry]
+            comment = self._handle_comment(section.inline_comments[entry])
+            #
+            if isinstance(this_entry, dict):
+                # a section
+                out.append(self._write_marker(
+                    indent_string,
+                    this_entry.depth,
+                    entry,
+                    comment))
+                out.extend(self.write(section=this_entry))
+            else:
+                out.append(self._write_line(
+                    indent_string,
+                    entry,
+                    this_entry,
+                    comment))
+        #
+        if section is self:
+            for line in self.final_comment:
+                line = self._decode_element(line)
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith(cs):
+                    line = csp + line
+                out.append(line)
+            self.interpolation = int_val
+        #
+        if section is not self:
+            return out
+        #
+        if (self.filename is None) and (outfile is None):
+            # output a list of lines
+            # might need to encode
+            # NOTE: This will *screw* UTF16, each line will start with the BOM
+            if self.encoding:
+                out = [l.encode(self.encoding) for l in out]
+            if (self.BOM and ((self.encoding is None) or
+                (BOM_LIST.get(self.encoding.lower()) == 'utf_8'))):
+                # Add the UTF8 BOM
+                if not out:
+                    out.append('')
+                out[0] = BOM_UTF8 + out[0]
+            return out
+        #
+        # Turn the list to a string, joined with correct newlines
+        output = (self._a_to_u(self.newlines or os.linesep)
+            ).join(out)
+        if self.encoding:
+            output = output.encode(self.encoding)
+        if (self.BOM and ((self.encoding is None) or
+            (BOM_LIST.get(self.encoding.lower()) == 'utf_8'))):
+            # Add the UTF8 BOM
+            output = BOM_UTF8 + output
+        if outfile is not None:
+            outfile.write(output)
+        else:
+            h = open(self.filename, 'w')
+            h.write(output)
+            h.close()
+
+    def validate(self, validator, preserve_errors=False, section=None):
+        """
+        Test the ConfigObj against a configspec.
+        
+        It uses the ``validator`` object from *validate.py*.
+        
+        To run ``validate`` on the current ConfigObj, call: ::
+        
+            test = config.validate(validator)
+        
+        (Normally having previously passed in the configspec when the ConfigObj
+        was created - you can dynamically assign a dictionary of checks to the
+        ``configspec`` attribute of a section though).
+        
+        It returns ``True`` if everything passes, or a dictionary of
+        pass/fails (True/False). If every member of a subsection passes, it
+        will just have the value ``True``. (It also returns ``False`` if all
+        members fail).
+        
+        In addition, it converts the values from strings to their native
+        types if their checks pass (and ``stringify`` is set).
+        
+        If ``preserve_errors`` is ``True`` (``False`` is default) then instead
+        of a marking a fail with a ``False``, it will preserve the actual
+        exception object. This can contain info about the reason for failure.
+        For example the ``VdtValueTooSmallError`` indeicates that the value
+        supplied was too small. If a value (or section) is missing it will
+        still be marked as ``False``.
+        
+        You must have the validate module to use ``preserve_errors=True``.
+        
+        You can then use the ``flatten_errors`` function to turn your nested
+        results dictionary into a flattened list of failures - useful for
+        displaying meaningful error messages.
+        
+        >>> try:
+        ...     from validate import Validator
+        ... except ImportError:
+        ...     print >> sys.stderr, 'Cannot import the Validator object, skipping the related tests'
+        ... else:
+        ...     config = '''
+        ...     test1=40
+        ...     test2=hello
+        ...     test3=3
+        ...     test4=5.0
+        ...     [section]
+        ...         test1=40
+        ...         test2=hello
+        ...         test3=3
+        ...         test4=5.0
+        ...         [[sub section]]
+        ...             test1=40
+        ...             test2=hello
+        ...             test3=3
+        ...             test4=5.0
+        ... '''.split('\\n')
+        ...     configspec = '''
+        ...     test1= integer(30,50)
+        ...     test2= string
+        ...     test3=integer
+        ...     test4=float(6.0)
+        ...     [section ]
+        ...         test1=integer(30,50)
+        ...         test2=string
+        ...         test3=integer
+        ...         test4=float(6.0)
+        ...         [[sub section]]
+        ...             test1=integer(30,50)
+        ...             test2=string
+        ...             test3=integer
+        ...             test4=float(6.0)
+        ...     '''.split('\\n')
+        ...     val = Validator()
+        ...     c1 = ConfigObj(config, configspec=configspec)
+        ...     test = c1.validate(val)
+        ...     test == {
+        ...         'test1': True,
+        ...         'test2': True,
+        ...         'test3': True,
+        ...         'test4': False,
+        ...         'section': {
+        ...             'test1': True,
+        ...             'test2': True,
+        ...             'test3': True,
+        ...             'test4': False,
+        ...             'sub section': {
+        ...                 'test1': True,
+        ...                 'test2': True,
+        ...                 'test3': True,
+        ...                 'test4': False,
+        ...             },
+        ...         },
+        ...     }
+        1
+        >>> val.check(c1.configspec['test4'], c1['test4'])
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "5.0" is too small.
+        
+        >>> val_test_config = '''
+        ...     key = 0
+        ...     key2 = 1.1
+        ...     [section]
+        ...     key = some text
+        ...     key2 = 1.1, 3.0, 17, 6.8
+        ...         [[sub-section]]
+        ...         key = option1
+        ...         key2 = True'''.split('\\n')
+        >>> val_test_configspec = '''
+        ...     key = integer
+        ...     key2 = float
+        ...     [section]
+        ...     key = string
+        ...     key2 = float_list(4)
+        ...        [[sub-section]]
+        ...        key = option(option1, option2)
+        ...        key2 = boolean'''.split('\\n')
+        >>> val_test = ConfigObj(val_test_config, configspec=val_test_configspec)
+        >>> val_test.validate(val)
+        1
+        >>> val_test['key'] = 'text not a digit'
+        >>> val_res = val_test.validate(val)
+        >>> val_res == {'key2': True, 'section': True, 'key': False}
+        1
+        >>> configspec = '''
+        ...     test1=integer(30,50, default=40)
+        ...     test2=string(default="hello")
+        ...     test3=integer(default=3)
+        ...     test4=float(6.0, default=6.0)
+        ...     [section ]
+        ...         test1=integer(30,50, default=40)
+        ...         test2=string(default="hello")
+        ...         test3=integer(default=3)
+        ...         test4=float(6.0, default=6.0)
+        ...         [[sub section]]
+        ...             test1=integer(30,50, default=40)
+        ...             test2=string(default="hello")
+        ...             test3=integer(default=3)
+        ...             test4=float(6.0, default=6.0)
+        ...     '''.split('\\n')
+        >>> default_test = ConfigObj(['test1=30'], configspec=configspec)
+        >>> default_test
+        {'test1': '30', 'section': {'sub section': {}}}
+        >>> default_test.validate(val)
+        1
+        >>> default_test == {
+        ...     'test1': 30,
+        ...     'test2': 'hello',
+        ...     'test3': 3,
+        ...     'test4': 6.0,
+        ...     'section': {
+        ...         'test1': 40,
+        ...         'test2': 'hello',
+        ...         'test3': 3,
+        ...         'test4': 6.0,
+        ...         'sub section': {
+        ...             'test1': 40,
+        ...             'test3': 3,
+        ...             'test2': 'hello',
+        ...             'test4': 6.0,
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Now testing with repeated sections : BIG TEST
+        
+        >>> repeated_1 = '''
+        ... [dogs]
+        ...     [[__many__]] # spec for a dog
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=long)
+        ...         name = string(default=rover)
+        ...         [[[__many__]]]  # spec for a puppy
+        ...             name = string(default="son of rover")
+        ...             age = float(default=0.0)
+        ... [cats]
+        ...     [[__many__]] # spec for a cat
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=short)
+        ...         name = string(default=pussy)
+        ...         [[[__many__]]] # spec for a kitten
+        ...             name = string(default="son of pussy")
+        ...             age = float(default=0.0)
+        ...         '''.split('\\n')
+        >>> repeated_2 = '''
+        ... [dogs]
+        ... 
+        ...     # blank dogs with puppies
+        ...     # should be filled in by the configspec
+        ...     [[dog1]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog2]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog3]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ... [cats]
+        ... 
+        ...     # blank cats with kittens
+        ...     # should be filled in by the configspec
+        ...     [[cat1]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat2]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat3]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ... '''.split('\\n')
+        >>> repeated_3 = '''
+        ... [dogs]
+        ... 
+        ...     [[dog1]]
+        ...     [[dog2]]
+        ...     [[dog3]]
+        ... [cats]
+        ... 
+        ...     [[cat1]]
+        ...     [[cat2]]
+        ...     [[cat3]]
+        ... '''.split('\\n')
+        >>> repeated_4 = '''
+        ... [__many__]
+        ... 
+        ...     name = string(default=Michael)
+        ...     age = float(default=0.0)
+        ...     sex = option(m, f, default=m)
+        ... '''.split('\\n')
+        >>> repeated_5 = '''
+        ... [cats]
+        ... [[__many__]]
+        ...     fleas = boolean(default=True)
+        ...     tail = option(long, short, default=short)
+        ...     name = string(default=pussy)
+        ...     [[[description]]]
+        ...         height = float(default=3.3)
+        ...         weight = float(default=6)
+        ...         [[[[coat]]]]
+        ...             fur = option(black, grey, brown, "tortoise shell", default=black)
+        ...             condition = integer(0,10, default=5)
+        ... '''.split('\\n')
+        >>> from validate import Validator
+        >>> val= Validator()
+        >>> repeater = ConfigObj(repeated_2, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {
+        ...         'dog1': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog2': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog3': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...     },
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'cats': {
+        ...         'cat1': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat2': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat3': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...     },
+        ...     'dogs': {
+        ...         'dog1': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog2': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog3': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(configspec=repeated_4)
+        >>> repeater['Michael'] = {}
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'Michael': {'age': 0.0, 'name': 'Michael', 'sex': 'm'},
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_5)
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {'cat1': {}, 'cat2': {}, 'cat3': {}},
+        ... }
+        1
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Test that interpolation is preserved for validated string values.
+        Also check that interpolation works in configspecs.
+        >>> t = ConfigObj()
+        >>> t['DEFAULT'] = {}
+        >>> t['DEFAULT']['test'] = 'a'
+        >>> t['test'] = '%(test)s'
+        >>> t['test']
+        'a'
+        >>> v = Validator()
+        >>> t.configspec = {'test': 'string'}
+        >>> t.validate(v)
+        1
+        >>> t.interpolation = False
+        >>> t
+        {'test': '%(test)s', 'DEFAULT': {'test': 'a'}}
+        >>> specs = [
+        ...    'interpolated string  = string(default="fuzzy-%(man)s")',
+        ...    '[DEFAULT]',
+        ...    'man = wuzzy',
+        ...    ]
+        >>> c = ConfigObj(configspec=specs)
+        >>> c.validate(v)
+        1
+        >>> c['interpolated string']
+        'fuzzy-wuzzy'
+        
+        FIXME: Above tests will fail if we couldn't import Validator (the ones
+        that don't raise errors will produce different output and still fail as
+        tests)
+        """
+        if section is None:
+            if self.configspec is None:
+                raise ValueError, 'No configspec supplied.'
+            if preserve_errors:
+                if VdtMissingValue is None:
+                    raise ImportError('Missing validate module.')
+            section = self
+        #
+        spec_section = section.configspec
+        if '__many__' in section.configspec:
+            many = spec_section['__many__']
+            # dynamically assign the configspecs
+            # for the sections below
+            for entry in section.sections:
+                self._handle_repeat(section[entry], many)
+        #
+        out = {}
+        ret_true = True
+        ret_false = True
+        for entry in spec_section:
+            if entry == '__many__':
+                continue
+            if (not entry in section.scalars) or (entry in section.defaults):
+                # missing entries
+                # or entries from defaults
+                missing = True
+                val = None
+            else:
+                missing = False
+                val = section[entry]
+            try:
+                check = validator.check(spec_section[entry],
+                                        val,
+                                        missing=missing
+                                        )
+            except validator.baseErrorClass, e:
+                if not preserve_errors or isinstance(e, VdtMissingValue):
+                    out[entry] = False
+                else:
+                    # preserve the error
+                    out[entry] = e
+                    ret_false = False
+                ret_true = False
+            else:
+                ret_false = False
+                out[entry] = True
+                if self.stringify or missing:
+                    # if we are doing type conversion
+                    # or the value is a supplied default
+                    if not self.stringify:
+                        if isinstance(check, (list, tuple)):
+                            # preserve lists
+                            check = [self._str(item) for item in check]
+                        elif missing and check is None:
+                            # convert the None from a default to a ''
+                            check = ''
+                        else:
+                            check = self._str(check)
+                    if (check != val) or missing:
+                        section[entry] = check
+                if missing and entry not in section.defaults:
+                    section.defaults.append(entry)
+        #
+        # FIXME: Will this miss missing sections ?
+        for entry in section.sections:
+            if section is self and entry == 'DEFAULT':
+                continue
+            check = self.validate(validator, preserve_errors=preserve_errors,
+                section=section[entry])
+            out[entry] = check
+            if check == False:
+                ret_true = False
+            elif check == True:
+                ret_false = False
+            else:
+                ret_true = False
+                ret_false = False
+        #
+        if ret_true:
+            return True
+        elif ret_false:
+            return False
+        else:
+            return out
+
+class SimpleVal(object):
+    """
+    A simple validator.
+    Can be used to check that all members expected are present.
+    
+    To use it, provide a configspec with all your members in (the value given
+    will be ignored). Pass an instance of ``SimpleVal`` to the ``validate``
+    method of your ``ConfigObj``. ``validate`` will return ``True`` if all
+    members are present, or a dictionary with True/False meaning
+    present/missing. (Whole missing sections will be replaced with ``False``)
+    
+    >>> val = SimpleVal()
+    >>> config = '''
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ... [section]
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ...     [[sub section]]
+    ...     test1=40
+    ...     test2=hello
+    ...     test3=3
+    ...     test4=5.0
+    ... '''.split('\\n')
+    >>> configspec = '''
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ... [section]
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ...     [[sub section]]
+    ...     test1=''
+    ...     test2=''
+    ...     test3=''
+    ...     test4=''
+    ... '''.split('\\n')
+    >>> o = ConfigObj(config, configspec=configspec)
+    >>> o.validate(val)
+    1
+    >>> o = ConfigObj(configspec=configspec)
+    >>> o.validate(val)
+    0
+    """
+    
+    def __init__(self):
+        self.baseErrorClass = ConfigObjError
+    
+    def check(self, check, member, missing=False):
+        """A dummy check method, always returns the value unchanged."""
+        if missing:
+            raise self.baseErrorClass
+        return member
+
+# Check / processing functions for options
+def flatten_errors(cfg, res, levels=None, results=None):
+    """
+    An example function that will turn a nested dictionary of results
+    (as returned by ``ConfigObj.validate``) into a flat list.
+    
+    ``cfg`` is the ConfigObj instance being checked, ``res`` is the results
+    dictionary returned by ``validate``.
+    
+    (This is a recursive function, so you shouldn't use the ``levels`` or
+    ``results`` arguments - they are used by the function.
+    
+    Returns a list of keys that failed. Each member of the list is a tuple :
+    ::
+    
+        ([list of sections...], key, result)
+    
+    If ``validate`` was called with ``preserve_errors=False`` (the default)
+    then ``result`` will always be ``False``.
+
+    *list of sections* is a flattened list of sections that the key was found
+    in.
+    
+    If the section was missing then key will be ``None``.
+    
+    If the value (or section) was missing then ``result`` will be ``False``.
+    
+    If ``validate`` was called with ``preserve_errors=True`` and a value
+    was present, but failed the check, then ``result`` will be the exception
+    object returned. You can use this as a string that describes the failure.
+    
+    For example *The value "3" is of the wrong type*.
+    
+    # FIXME: is the ordering of the output arbitrary ?
+    >>> import validate
+    >>> vtor = validate.Validator()
+    >>> my_ini = '''
+    ...     option1 = True
+    ...     [section1]
+    ...     option1 = True
+    ...     [section2]
+    ...     another_option = Probably
+    ...     [section3]
+    ...     another_option = True
+    ...     [[section3b]]
+    ...     value = 3
+    ...     value2 = a
+    ...     value3 = 11
+    ...     '''
+    >>> my_cfg = '''
+    ...     option1 = boolean()
+    ...     option2 = boolean()
+    ...     option3 = boolean(default=Bad_value)
+    ...     [section1]
+    ...     option1 = boolean()
+    ...     option2 = boolean()
+    ...     option3 = boolean(default=Bad_value)
+    ...     [section2]
+    ...     another_option = boolean()
+    ...     [section3]
+    ...     another_option = boolean()
+    ...     [[section3b]]
+    ...     value = integer
+    ...     value2 = integer
+    ...     value3 = integer(0, 10)
+    ...         [[[section3b-sub]]]
+    ...         value = string
+    ...     [section4]
+    ...     another_option = boolean()
+    ...     '''
+    >>> cs = my_cfg.split('\\n')
+    >>> ini = my_ini.split('\\n')
+    >>> cfg = ConfigObj(ini, configspec=cs)
+    >>> res = cfg.validate(vtor, preserve_errors=True)
+    >>> errors = []
+    >>> for entry in flatten_errors(cfg, res):
+    ...     section_list, key, error = entry
+    ...     section_list.insert(0, '[root]')
+    ...     if key is not None:
+    ...        section_list.append(key)
+    ...     else:
+    ...         section_list.append('[missing]')
+    ...     section_string = ', '.join(section_list)
+    ...     errors.append((section_string, ' = ', error))
+    >>> errors.sort()
+    >>> for entry in errors:
+    ...     print entry[0], entry[1], (entry[2] or 0)
+    [root], option2  =  0
+    [root], option3  =  the value "Bad_value" is of the wrong type.
+    [root], section1, option2  =  0
+    [root], section1, option3  =  the value "Bad_value" is of the wrong type.
+    [root], section2, another_option  =  the value "Probably" is of the wrong type.
+    [root], section3, section3b, section3b-sub, [missing]  =  0
+    [root], section3, section3b, value2  =  the value "a" is of the wrong type.
+    [root], section3, section3b, value3  =  the value "11" is too big.
+    [root], section4, [missing]  =  0
+    """
+    if levels is None:
+        # first time called
+        levels = []
+        results = []
+    if res is True:
+        return results
+    if res is False:
+        results.append((levels[:], None, False))
+        if levels:
+            levels.pop()
+        return results
+    for (key, val) in res.items():
+        if val == True:
+            continue
+        if isinstance(cfg.get(key), dict):
+            # Go down one level
+            levels.append(key)
+            flatten_errors(cfg[key], val, levels, results)
+            continue
+        results.append((levels[:], key, val))
+    #
+    # Go up one level
+    if levels:
+        levels.pop()
+    #
+    return results
+
+
+# FIXME: test error code for badly built multiline values
+# FIXME: test handling of StringIO
+# FIXME: test interpolation with writing
+
+def _doctest():
+    """
+    Dummy function to hold some of the doctests.
+    
+    >>> a.depth
+    0
+    >>> a == {
+    ...     'key2': 'val',
+    ...     'key1': 'val',
+    ...     'lev1c': {
+    ...         'lev2c': {
+    ...             'lev3c': {
+    ...                 'key1': 'val',
+    ...             },
+    ...         },
+    ...     },
+    ...     'lev1b': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...         'lev2ba': {
+    ...             'key1': 'val',
+    ...         },
+    ...         'lev2bb': {
+    ...             'key1': 'val',
+    ...         },
+    ...     },
+    ...     'lev1a': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...     },
+    ... }
+    1
+    >>> b.depth
+    0
+    >>> b == {
+    ...     'key3': 'val3',
+    ...     'key2': 'val2',
+    ...     'key1': 'val1',
+    ...     'section 1': {
+    ...         'keys11': 'val1',
+    ...         'keys13': 'val3',
+    ...         'keys12': 'val2',
+    ...     },
+    ...     'section 2': {
+    ...         'section 2 sub 1': {
+    ...             'fish': '3',
+    ...     },
+    ...     'keys21': 'val1',
+    ...     'keys22': 'val2',
+    ...     'keys23': 'val3',
+    ...     },
+    ... }
+    1
+    >>> t = '''
+    ... 'a' = b # !"$%^&*(),::;'@~#= 33
+    ... "b" = b #= 6, 33
+    ... ''' .split('\\n')
+    >>> t2 = ConfigObj(t)
+    >>> assert t2 == {'a': 'b', 'b': 'b'}
+    >>> t2.inline_comments['b'] = ''
+    >>> del t2['a']
+    >>> assert t2.write() == ['','b = b', '']
+    
+    # Test ``list_values=False`` stuff
+    >>> c = '''
+    ...     key1 = no quotes
+    ...     key2 = 'single quotes'
+    ...     key3 = "double quotes"
+    ...     key4 = "list", 'with', several, "quotes"
+    ...     '''
+    >>> cfg = ConfigObj(c.splitlines(), list_values=False)
+    >>> cfg == {'key1': 'no quotes', 'key2': "'single quotes'", 
+    ... 'key3': '"double quotes"', 
+    ... 'key4': '"list", \\'with\\', several, "quotes"'
+    ... }
+    1
+    >>> cfg = ConfigObj(list_values=False)
+    >>> cfg['key1'] = 'Multiline\\nValue'
+    >>> cfg['key2'] = '''"Value" with 'quotes' !'''
+    >>> cfg.write()
+    ["key1 = '''Multiline\\nValue'''", 'key2 = "Value" with \\'quotes\\' !']
+    >>> cfg.list_values = True
+    >>> cfg.write() == ["key1 = '''Multiline\\nValue'''",
+    ... 'key2 = \\'\\'\\'"Value" with \\'quotes\\' !\\'\\'\\'']
+    1
+    
+    Test flatten_errors:
+    
+    >>> from validate import Validator, VdtValueTooSmallError
+    >>> config = '''
+    ...     test1=40
+    ...     test2=hello
+    ...     test3=3
+    ...     test4=5.0
+    ...     [section]
+    ...         test1=40
+    ...         test2=hello
+    ...         test3=3
+    ...         test4=5.0
+    ...         [[sub section]]
+    ...             test1=40
+    ...             test2=hello
+    ...             test3=3
+    ...             test4=5.0
+    ... '''.split('\\n')
+    >>> configspec = '''
+    ...     test1= integer(30,50)
+    ...     test2= string
+    ...     test3=integer
+    ...     test4=float(6.0)
+    ...     [section ]
+    ...         test1=integer(30,50)
+    ...         test2=string
+    ...         test3=integer
+    ...         test4=float(6.0)
+    ...         [[sub section]]
+    ...             test1=integer(30,50)
+    ...             test2=string
+    ...             test3=integer
+    ...             test4=float(6.0)
+    ...     '''.split('\\n')
+    >>> val = Validator()
+    >>> c1 = ConfigObj(config, configspec=configspec)
+    >>> res = c1.validate(val)
+    >>> flatten_errors(c1, res) == [([], 'test4', False), (['section', 
+    ...     'sub section'], 'test4', False), (['section'], 'test4', False)]
+    True
+    >>> res = c1.validate(val, preserve_errors=True)
+    >>> check = flatten_errors(c1, res)
+    >>> check[0][:2]
+    ([], 'test4')
+    >>> check[1][:2]
+    (['section', 'sub section'], 'test4')
+    >>> check[2][:2]
+    (['section'], 'test4')
+    >>> for entry in check:
+    ...     isinstance(entry[2], VdtValueTooSmallError)
+    ...     print str(entry[2])
+    True
+    the value "5.0" is too small.
+    True
+    the value "5.0" is too small.
+    True
+    the value "5.0" is too small.
+    
+    Test unicode handling, BOM, write witha file like object and line endings :
+    >>> u_base = '''
+    ... # initial comment
+    ...     # inital comment 2
+    ... 
+    ... test1 = some value
+    ... # comment
+    ... test2 = another value    # inline comment
+    ... # section comment
+    ... [section]    # inline comment
+    ...     test = test    # another inline comment
+    ...     test2 = test2
+    ... 
+    ... # final comment
+    ... # final comment2
+    ... '''
+    >>> u = u_base.encode('utf_8').splitlines(True)
+    >>> u[0] = BOM_UTF8 + u[0]
+    >>> uc = ConfigObj(u)
+    >>> uc.encoding = None
+    >>> uc.BOM == True
+    1
+    >>> uc == {'test1': 'some value', 'test2': 'another value',
+    ... 'section': {'test': 'test', 'test2': 'test2'}}
+    1
+    >>> uc = ConfigObj(u, encoding='utf_8', default_encoding='latin-1')
+    >>> uc.BOM
+    1
+    >>> isinstance(uc['test1'], unicode)
+    1
+    >>> uc.encoding
+    'utf_8'
+    >>> uc.newlines
+    '\\n'
+    >>> uc['latin1'] = "This costs lot's of "
+    >>> a_list = uc.write()
+    >>> len(a_list)
+    15
+    >>> isinstance(a_list[0], str)
+    1
+    >>> a_list[0].startswith(BOM_UTF8)
+    1
+    >>> u = u_base.replace('\\n', '\\r\\n').encode('utf_8').splitlines(True)
+    >>> uc = ConfigObj(u)
+    >>> uc.newlines
+    '\\r\\n'
+    >>> uc.newlines = '\\r'
+    >>> from cStringIO import StringIO
+    >>> file_like = StringIO()
+    >>> uc.write(file_like)
+    >>> file_like.seek(0)
+    >>> uc2 = ConfigObj(file_like)
+    >>> uc2 == uc
+    1
+    >>> uc2.filename == None
+    1
+    >>> uc2.newlines == '\\r'
+    1
+    """
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    #
+    testconfig1 = """\
+    key1= val    # comment 1
+    key2= val    # comment 2
+    # comment 3
+    [lev1a]     # comment 4
+    key1= val    # comment 5
+    key2= val    # comment 6
+    # comment 7
+    [lev1b]    # comment 8
+    key1= val    # comment 9
+    key2= val    # comment 10
+    # comment 11
+        [[lev2ba]]    # comment 12
+        key1= val    # comment 13
+        # comment 14
+        [[lev2bb]]    # comment 15
+        key1= val    # comment 16
+    # comment 17
+    [lev1c]    # comment 18
+    # comment 19
+        [[lev2c]]    # comment 20
+        # comment 21
+            [[[lev3c]]]    # comment 22
+            key1 = val    # comment 23"""
+    #
+    testconfig2 = """\
+                        key1 = 'val1'
+                        key2 =   "val2"
+                        key3 = val3
+                        ["section 1"] # comment
+                        keys11 = val1
+                        keys12 = val2
+                        keys13 = val3
+                        [section 2]
+                        keys21 = val1
+                        keys22 = val2
+                        keys23 = val3
+                        
+                            [['section 2 sub 1']]
+                            fish = 3
+    """
+    #
+    testconfig6 = '''
+    name1 = """ a single line value """ # comment
+    name2 = \''' another single line value \''' # comment
+    name3 = """ a single line value """
+    name4 = \''' another single line value \'''
+        [ "multi section" ]
+        name1 = """
+        Well, this is a
+        multiline value
+        """
+        name2 = \'''
+        Well, this is a
+        multiline value
+        \'''
+        name3 = """
+        Well, this is a
+        multiline value
+        """     # a comment
+        name4 = \'''
+        Well, this is a
+        multiline value
+        \'''  # I guess this is a comment too
+    '''
+    #
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    a = ConfigObj(testconfig1.split('\n'), raise_errors=True)
+    b = ConfigObj(testconfig2.split('\n'), raise_errors=True)
+    i = ConfigObj(testconfig6.split('\n'), raise_errors=True)
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'a': a,
+        'b': b,
+        'i': i,
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    BUGS
+    ====
+    
+    None known.
+    
+    TODO
+    ====
+    
+    Better support for configuration from multiple files, including tracking
+    *where* the original file came from and writing changes to the correct
+    file.
+    
+    
+    Make ``newline`` an option (as well as an attribute) ?
+    
+    ``UTF16`` encoded files, when returned as a list of lines, will have the
+    BOM at the start of every line. Should this be removed from all but the
+    first line ?
+    
+    Option to set warning type for unicode decode ? (Defaults to strict).
+    
+    A method to optionally remove uniform indentation from multiline values.
+    (do as an example of using ``walk`` - along with string-escape)
+    
+    Should the results dictionary from validate be an ordered dictionary if
+    `odict <http://www.voidspace.org.uk/python/odict.html>`_ is available ?
+    
+    Implement a better ``__repr__`` ? (``ConfigObj({})``)
+    
+    Implement some of the sequence methods (which include slicing) from the
+    newer ``odict`` ?
+    
+    INCOMPATIBLE CHANGES
+    ====================
+    
+    (I have removed a lot of needless complications - this list is probably not
+    conclusive, many option/attribute/method names have changed)
+    
+    Case sensitive
+    
+    The only valid divider is '='
+    
+    We've removed line continuations with '\'
+    
+    No recursive lists in values
+    
+    No empty section
+    
+    No distinction between flatfiles and non flatfiles
+    
+    Change in list syntax - use commas to indicate list, not parentheses
+    (square brackets and parentheses are no longer recognised as lists)
+    
+    ';' is no longer valid for comments and no multiline comments
+    
+    No attribute access
+    
+    We don't allow empty values - have to use '' or ""
+    
+    In ConfigObj 3 - setting a non-flatfile member to ``None`` would
+    initialise it as an empty section.
+    
+    The escape entities '&mjf-lf;' and '&mjf-quot;' have gone
+    replaced by triple quote, multiple line values.
+    
+    The ``newline``, ``force_return``, and ``default`` options have gone
+    
+    The ``encoding`` and ``backup_encoding`` methods have gone - replaced
+    with the ``encode`` and ``decode`` methods.
+    
+    ``fileerror`` and ``createempty`` options have become ``file_error`` and
+    ``create_empty``
+    
+    Partial configspecs (for specifying the order members should be written
+    out and which should be present) have gone. The configspec is no longer
+    used to specify order for the ``write`` method.
+    
+    Exceeding the maximum depth of recursion in string interpolation now
+    raises an error ``InterpolationDepthError``.
+    
+    Specifying a value for interpolation which doesn't exist now raises an
+    error ``MissingInterpolationOption`` (instead of merely being ignored).
+    
+    The ``writein`` method has been removed.
+    
+    The comments attribute is now a list (``inline_comments`` equates to the
+    old comments attribute)
+    
+    ISSUES
+    ======
+    
+    ``validate`` doesn't report *extra* values or sections.
+    
+    You can't have a keyword with the same name as a section (in the same
+    section). They are both dictionary keys - so they would overlap.
+    
+    ConfigObj doesn't quote and unquote values if ``list_values=False``.
+    This means that leading or trailing whitespace in values will be lost when
+    writing. (Unless you manually quote).
+    
+    Interpolation checks first the 'DEFAULT' subsection of the current
+    section, next it checks the 'DEFAULT' section of the parent section,
+    last it checks the 'DEFAULT' section of the main section.
+    
+    Logically a 'DEFAULT' section should apply to all subsections of the *same
+    parent* - this means that checking the 'DEFAULT' subsection in the
+    *current section* is not necessarily logical ?
+    
+    In order to simplify unicode support (which is possibly of limited value
+    in a config file) I have removed automatic support and added the
+    ``encode`` and ``decode methods, which can be used to transform keys and
+    entries. Because the regex looks for specific values on inital parsing
+    (i.e. the quotes and the equals signs) it can only read ascii compatible
+    encodings. For unicode use ``UTF8``, which is ASCII compatible.
+    
+    Does it matter that we don't support the ':' divider, which is supported
+    by ``ConfigParser`` ?
+    
+    The regular expression correctly removes the value -
+    ``"'hello', 'goodbye'"`` and then unquote just removes the front and
+    back quotes (called from ``_handle_value``). What should we do ??
+    (*ought* to raise exception because it's an invalid value if lists are
+    off *sigh*. This is not what you want if you want to do your own list
+    processing - would be *better* in this case not to unquote.)
+    
+    String interpolation and validation don't play well together. When
+    validation changes type it sets the value. This will correctly fetch the
+    value using interpolation - but then overwrite the interpolation reference.
+    If the value is unchanged by validation (it's a string) - but other types
+    will be.
+    
+    
+    List Value Syntax
+    =================
+    
+    List values allow you to specify multiple values for a keyword. This
+    maps to a list as the resulting Python object when parsed.
+    
+    The syntax for lists is easy. A list is a comma separated set of values.
+    If these values contain quotes, the hash mark, or commas, then the values
+    can be surrounded by quotes. e.g. : ::
+    
+        keyword = value1, 'value 2', "value 3"
+    
+    If a value needs to be a list, but only has one member, then you indicate
+    this with a trailing comma. e.g. : ::
+    
+        keyword = "single value",
+    
+    If a value needs to be a list, but it has no members, then you indicate
+    this with a single comma. e.g. : ::
+    
+        keyword = ,     # an empty list
+    
+    Using triple quotes it will be possible for single values to contain
+    newlines and *both* single quotes and double quotes. Triple quotes aren't
+    allowed in list values. This means that the members of list values can't
+    contain carriage returns (or line feeds :-) or both quote values.
+      
+    CHANGELOG
+    =========
+    
+    2006/02/04
+    ----------
+    
+    Removed ``BOM_UTF8`` from ``__all__``.
+    
+    The ``BOM`` attribute has become a boolean. (Defaults to ``False``.) It is
+    *only* ``True`` for the ``UTF16`` encoding.
+    
+    File like objects no longer need a ``seek`` attribute.
+    
+    ConfigObj no longer keeps a reference to file like objects. Instead the
+    ``write`` method takes a file like object as an optional argument. (Which
+    will be used in preference of the ``filename`` attribute if htat exists as
+    well.)
+    
+    Full unicode support added. New options/attributes ``encoding``,
+    ``default_encoding``.
+    
+    utf16 files decoded to unicode.
+    
+    If ``BOM`` is ``True``, but no encoding specified, then the utf8 BOM is
+    written out at the start of the file. (It will normally only be ``True`` if
+    the utf8 BOM was found when the file was read.)
+    
+    File paths are *not* converted to absolute paths, relative paths will
+    remain relative as the ``filename`` attribute.
+    
+    Fixed bug where ``final_comment`` wasn't returned if ``write`` is returning
+    a list of lines.
+    
+    2006/01/31
+    ----------
+    
+    Added ``True``, ``False``, and ``enumerate`` if they are not defined.
+    (``True`` and ``False`` are needed for *early* versions of Python 2.2,
+    ``enumerate`` is needed for all versions ofPython 2.2)
+    
+    Deprecated ``istrue``, replaced it with ``as_bool``.
+    
+    Added ``as_int`` and ``as_float``.
+    
+    utf8 and utf16 BOM handled in an endian agnostic way.
+    
+    2005/12/14
+    ----------
+    
+    Validation no longer done on the 'DEFAULT' section (only in the root
+    level). This allows interpolation in configspecs.
+    
+    Change in validation syntax implemented in validate 0.2.1
+    
+    4.1.0
+    
+    2005/12/10
+    ----------
+    
+    Added ``merge``, a recursive update.
+    
+    Added ``preserve_errors`` to ``validate`` and the ``flatten_errors``
+    example function.
+    
+    Thanks to Matthew Brett for suggestions and helping me iron out bugs.
+    
+    Fixed bug where a config file is *all* comment, the comment will now be
+    ``initial_comment`` rather than ``final_comment``.
+    
+    2005/12/02
+    ----------
+    
+    Fixed bug in ``create_empty``. Thanks to Paul Jimenez for the report.
+    
+    2005/11/04
+    ----------
+    
+    Fixed bug in ``Section.walk`` when transforming names as well as values.
+    
+    Added the ``istrue`` method. (Fetches the boolean equivalent of a string
+    value).
+    
+    Fixed ``list_values=False`` - they are now only quoted/unquoted if they
+    are multiline values.
+    
+    List values are written as ``item, item`` rather than ``item,item``.
+    
+    4.0.1
+    
+    2005/10/09
+    ----------
+    
+    Fixed typo in ``write`` method. (Testing for the wrong value when resetting
+    ``interpolation``).
+
+    4.0.0 Final
+    
+    2005/09/16
+    ----------
+    
+    Fixed bug in ``setdefault`` - creating a new section *wouldn't* return
+    a reference to the new section.
+    
+    2005/09/09
+    ----------
+    
+    Removed ``PositionError``.
+    
+    Allowed quotes around keys as documented.
+    
+    Fixed bug with commas in comments. (matched as a list value)
+    
+    Beta 5
+    
+    2005/09/07
+    ----------
+    
+    Fixed bug in initialising ConfigObj from a ConfigObj.
+    
+    Changed the mailing list address.
+    
+    Beta 4
+    
+    2005/09/03
+    ----------
+    
+    Fixed bug in ``Section.__delitem__`` oops.
+    
+    2005/08/28
+    ----------
+    
+    Interpolation is switched off before writing out files.
+    
+    Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+    "Gustavo Niemeyer" <gustavo@niemeyer.net>)
+    
+    Moved the doctests from the ``__init__`` method to a separate function.
+    (For the sake of IDE calltips).
+    
+    Beta 3
+    
+    2005/08/26
+    ----------
+    
+    String values unchanged by validation *aren't* reset. This preserves
+    interpolation in string values.
+    
+    2005/08/18
+    ----------
+    
+    None from a default is turned to '' if stringify is off - because setting 
+    a value to None raises an error.
+    
+    Version 4.0.0-beta2
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Actually added the RepeatSectionError class ;-)
+    
+    2005/08/15
+    ----------
+    
+    If ``stringify`` is off - list values are preserved by the ``validate``
+    method. (Bugfix)
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Fixed ``simpleVal``.
+    
+    Added ``RepeatSectionError`` error if you have additional sections in a
+    section with a ``__many__`` (repeated) section.
+    
+    By Nicola Larosa
+    
+    Reworked the ConfigObj._parse, _handle_error and _multiline methods:
+    mutated the self._infile, self._index and self._maxline attributes into
+    local variables and method parameters
+    
+    Reshaped the ConfigObj._multiline method to better reflect its semantics
+    
+    Changed the "default_test" test in ConfigObj.validate to check the fix for
+    the bug in validate.Validator.check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Michael Foord
+    
+    Implemented repeated sections.
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    2005/08/10
+    ----------
+   
+    By Michael Foord
+     
+    Implemented default values in configspecs.
+    
+    By Nicola Larosa
+    
+    Fixed naked except: clause in validate that was silencing the fact
+    that Python2.2 does not have dict.pop
+    
+    2005/08/08
+    ----------
+    
+    By Michael Foord
+    
+    Bug fix causing error if file didn't exist.
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility
+    
+    2005/08/04
+    ----------
+    
+    By Michael Foord
+    
+    Added the inline_comments attribute
+    
+    We now preserve and rewrite all comments in the config file
+    
+    configspec is now a section attribute
+    
+    The validate method changes values in place
+    
+    Added InterpolationError
+    
+    The errors now have line number, line, and message attributes. This
+    simplifies error handling
+    
+    Added __docformat__
+    
+    2005/08/03
+    ----------
+    
+    By Michael Foord
+    
+    Fixed bug in Section.pop (now doesn't raise KeyError if a default value
+    is specified)
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    Removed the ``writein`` method
+    
+    Added __version__
+    
+    2005/07/29
+    ----------
+    
+    By Nicola Larosa
+    
+    Indentation in config file is not significant anymore, subsections are
+    designated by repeating square brackets
+    
+    Adapted all tests and docs to the new format
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Added more tests
+    
+    2005/07/23
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    Code tests converted to doctest format, and scattered them around
+    in various docstrings
+    
+    Walk method rewritten using scalars and sections attributes
+    
+    2005/07/22
+    ----------
+    
+    By Nicola Larosa
+    
+    Changed Validator and SimpleVal "test" methods to "check"
+    
+    More code cleanup
+    
+    2005/07/21
+    ----------
+    
+    Changed Section.sequence to Section.scalars and Section.sections
+    
+    Added Section.configspec
+    
+    Sections in the root section now have no extra indentation
+    
+    Comments now better supported in Section and preserved by ConfigObj
+    
+    Comments also written out
+    
+    Implemented initial_comment and final_comment
+    
+    A scalar value after a section will now raise an error
+    
+    2005/07/20
+    ----------
+    
+    Fixed a couple of bugs
+    
+    Can now pass a tuple instead of a list
+    
+    Simplified dict and walk methods
+    
+    Added __str__ to Section
+    
+    2005/07/10
+    ----------
+    
+    By Nicola Larosa
+    
+    More code cleanup
+    
+    2005/07/08
+    ----------
+    
+    The stringify option implemented. On by default.
+    
+    2005/07/07
+    ----------
+    
+    Renamed private attributes with a single underscore prefix.
+    
+    Changes to interpolation - exceeding recursion depth, or specifying a
+    missing value, now raise errors.
+    
+    Changes for Python 2.2 compatibility. (changed boolean tests - removed
+    ``is True`` and ``is False``)
+    
+    Added test for duplicate section and member (and fixed bug)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+    
+    2005/07/02
+    ----------
+    
+    Version 0.1.0
+    
+    Now properly handles values including comments and lists.
+    
+    Better error handling.
+    
+    String interpolation.
+    
+    Some options implemented.
+    
+    You can pass a Section a dictionary to initialise it.
+    
+    Setting a Section member to a dictionary will create a Section instance.
+    
+    2005/06/26
+    ----------
+    
+    Version 0.0.1
+    
+    Experimental reader.
+    
+    A reasonably elegant implementation - a basic reader in 160 lines of code.
+    
+    *A programming language is a medium of expression.* - Paul Graham
+"""
+

=== added directory 'bzrlib/util/configobj/docs'
=== added file 'bzrlib/util/configobj/docs/BSD-LICENSE.txt'
--- bzrlib/util/configobj/docs/BSD-LICENSE.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/BSD-LICENSE.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,60 @@
+===================================
+ The Voidspace Open Source License
+===================================
+
+.. note::
+
+    See this document online at http://www.voidspace.org.uk/python/license.shtml
+
+This is the OSI_ Approved license that the Voidspace modules and programs are
+available under [#]_. It's often reffered to as the *BSD License*.
+
+It is a very unrestrictive license but it comes with the usual disclaimer. 
+This is free software: test it, break it, just don't blame me if it eats your 
+data ! Of course if it does, let me know and I'll fix the problem so that it 
+doesn't happen to anyone else {sm;:-)}.
+
+.. _osi: http://www.opensource.org/
+
+::
+
+    Copyright (c) 2003-2005, Michael Foord
+    All rights reserved.
+    E-mail : fuzzyman AT voidspace DOT org DOT uk
+    
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+    
+    
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+    
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+    
+        * Neither the name of Michael Foord nor the name of Voidspace 
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+    
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+If you use any of my code in your projects, then a link back would be nice. If
+you let me know about any of your projects that use them - then I can link to
+you as examples.
+
+-----------
+
+.. [#] Unless specifically stated otherwise of course.

=== added file 'bzrlib/util/configobj/docs/configobj.txt'
--- bzrlib/util/configobj/docs/configobj.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/configobj.txt	2006-02-21 02:26:30 +0000
@@ -0,0 +1,2399 @@
+==================================
+ Reading and Writing Config Files
+==================================
+
+----------------------------------------
+ ConfigObj 4 Introduction and Reference
+----------------------------------------
+
+:Authors: Michael Foord, Nicola Larosa
+:Version: ConfigObj 4.2.0
+:Date: 2006/02/16
+:Homepage: `ConfigObj Homepage`_
+:Sourceforge: Sourceforge_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+
+.. meta::
+   :description: ConfigObj - a Python module for easy reading and writing of 
+                 config files.
+   :keywords: python, script, module, config, configuration, data, persistence,
+              developer, configparser
+
+.. contents:: ConfigObj Manual
+.. sectnum::
+
+Introduction
+============
+
+**ConfigObj** is a simple but powerful config file reader and writer: an *ini
+file round tripper*. Its main feature is that it is very easy to use, with a
+straightforward programmer's interface and a simple syntax for config files.
+It has lots of other features though :
+
+.. sidebar:: ConfigObj in Bazaar
+
+    **ConfigObj** is now used as the config file parser for `Bazaar <http://bazaar-ng.org>`_.
+    
+    Bazaar is *the* Python  distributed {acro;VCS;Version Control System}. 
+    ConfigObj is used to read ``bazaar.conf`` and ``branches.conf``.
+    
+    Other projects that use **ConfigObj** include :
+    
+    * `Planet Plus <http://planetplus.python-hosting.com/>`_
+    
+        A new web application version of `Planet <http://www.planetplanet.org/>`_, 
+        the web aggregator.
+        
+    * `NeuroImaging in Python <http://projects.scipy.org/neuroimaging/ni/wiki>`_
+    
+        BrainSTAT is a project with the ultimate goal to produce a
+        platform-independent python environment for the analysis of brain
+        imaging data.
+        
+    * `Gruik <http://www.tracos.org/gruik/wiki>`_
+    
+        Gruik is a free software network packet sniffer.
+
+* Nested sections (subsections), to any level
+* List values
+* Multiple line values
+* String interpolation (substitution)
+* Integrated with a powerful validation system
+
+    - including automatic type checking/conversion
+    - repeated sections
+    - and allowing default values
+
+* All comments in the file are preserved
+* The order of keys/sections is preserved
+* No external dependencies
+* Full unicode support.
+
+ConfigObj 4 is a complete rewrite of ConfigObj. ConfigObj now has a barrage [#]_
+of doctests built into it, testing almost every feature. Run ``python configobj.py -v``
+to see them in action. Despite the tests, ConfigObj 4 is smaller than version 3
+and has no external dependencies.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. hint::
+
+    ConfigObj provides a convenient API for storing all sorts of data, not just
+    config files. There is an article on using `ConfigObj for Data Persistence`_.
+    
+    The code from that article is available as ConfigPersist.py_.
+
+.. _ConfigObj for Data Persistence: http://www.voidspace.org.uk/python/articles/configobj_for_data_persistence.shtml
+.. _ConfigPersist.py: http://www.voidspace.org.uk/python/configpersist.html
+
+
+Downloading
+===========
+
+The current version is **4.2.0**, dated 16th February 2006. ConfigObj 4 is
+now stable. We still expect to pick up a few bugs along the way though [#]_.
+{sm;:-)}
+
+You can get ConfigObj in the following ways :
+
+Files
+-----
+
+* configobj.py_ from Voidspace
+
+    ConfigObj has no external dependencies. This file is sufficient to access
+    all the functionality except Validation_.
+
+* configobj.zip_ from Voidspace
+
+    This also contains validate.py_ , the `API Docs`_ and `this document`_.
+
+* The latest development version can be obtained from the `Subversion
+  Repository`_.
+
+* validate.py_ from Voidspace
+
+* You can also download *configobj.zip* from Sourceforge_
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated by
+the EpyDoc_ program.
+
+* You can view `this document`_ online as the `ConfigObj Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+ConfigObj is also part of the Pythonutils_ set of modules. This contains
+various other useful modules, and is required by many of the `Voidspace Python
+Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.2.0.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _this document:
+.. _configobj homepage: http://www.voidspace.org.uk/python/configobj.html
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+
+
+Getting Started
+===============
+
+The outstanding feature of using ConfigObj is simplicity. Most functions can be
+performed with single line commands.
+
+Reading a Config File
+---------------------
+
+The normal way to read a config file, is to give ConfigObj the filename :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+
+    {-coloring}
+
+You can also pass the config file in as a list of lines, or a ``StringIO``
+instance, so it doesn't matter where your config data comes from.
+
+You can then access members of your config file as a dictionary. Subsections
+will also be dictionaries.
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+    #
+    value1 = config['keyword1']
+    value2 = config['keyword2']
+    #
+    section1 = config['section1']
+    value3 = section1['keyword3']
+    value4 = section1['keyword4']
+    #
+    # you could also write
+    value3 = config['section1']['keyword3']
+    value4 = config['section1']['keyword4']
+
+    {-coloring}
+
+Writing a Config File
+---------------------
+
+Creating a new config file is just as easy as reading one. You can specify a
+filename when you create the ConfigObj, or do it later [#]_.
+
+If you *don't* set a filename, then the ``write`` method will return a list of
+lines instead of writing to file. See the write_ method for more details.
+
+Here we show creating an empty ConfigObj, setting a filename and some values,
+and then writing to file :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj()
+    config.filename = filename
+    #
+    config['keyword1'] = value1
+    config['keyword2'] = value2
+    #
+    config['section1'] = {}
+    config['section1']['keyword3'] = value3
+    config['section1']['keyword4'] = value4
+    #
+    section2 = {
+        'keyword5': value5,
+        'keyword6': value6,
+        'sub-section': {
+            'keyword7': value7
+            }
+    }
+    config['section2'] = section2
+    #
+    config['section3'] = {}
+    config['section3']['keyword 8'] = [value8, value9, value10]
+    config['section3']['keyword 9'] = [value11, value12, value13]
+    #
+    config.write()
+
+    {-coloring}
+
+.. caution::
+
+    Keywords and section names can only be strings [#]_. Attempting to set
+    anything else will raise a ``ValueError``.
+
+Config Files
+------------
+
+The config files that ConfigObj will read and write are based on the 'INI'
+format. This means it will read and write files created for ``ConfigParser``
+[#]_.
+
+Keywords and values are separated by an ``'='``, and section markers are
+between square brackets. Keywords, values, and section names can be surrounded
+by single or double quotes. Indentation is not significant, but can be
+preserved.
+
+Subsections are indicated by repeating the square brackets in the section
+marker. You nest levels by using more brackets.
+
+You can have list values by separating items with a comma, and values spanning
+multiple lines by using triple quotes (single or double).
+
+For full details on all these see `the config file format`_. Here's an example
+to illustrate : ::
+
+    # This is the 'initial_comment'
+    # Which may be several lines
+    keyword1 = value1
+    'keyword 2' = 'value 2'
+
+    [ "section 1" ]
+    # This comment goes with keyword 3
+    keyword 3 = value 3
+    'keyword 4' = value4, value 5, 'value 6'
+
+        [[ sub-section ]]    # an inline comment
+        # sub-section is inside "section 1"
+        'keyword 5' = 'value 7'
+        'keyword 6' = '''A multiline value,
+    that spans more than one line :-)
+    The line breaks are included in the value.'''
+
+            [[[ sub-sub-section ]]]
+            # sub-sub-section is *in* 'sub-section'
+            # which is in 'section 1'
+            'keyword 7' = 'value 8'
+
+    [section 2]    # an inline comment
+    keyword8 = "value 9"
+    keyword9 = value10     # an inline comment
+    # The 'final_comment'
+    # Which also may be several lines
+
+
+ConfigObj specifications
+========================
+
+.. raw:: html
+
+    {+coloring}
+
+    config = ConfigObj(infile=None, options=None, **keywargs)
+
+    {-coloring}
+
+infile
+------
+
+You don't need to specify an infile. If you omit it, an empty ConfigObj will be
+created. ``infile`` *can* be :
+
+* Nothing. In which case the ``filename`` attribute of your ConfigObj will be
+  ``None``. You can set a filename at any time.
+
+* A filename. What happens if the file doesn't already exist is determined by
+  the options_ ``file_error`` and ``create_empty``. The filename will be
+  preserved as the ``filename`` attribute. This can be changed at any time.
+
+* A list of lines. Any trailing newlines will be removed from the lines. The
+  ``filename`` attribute of your ConfigObj will be ``None``.
+
+* A ``StringIO`` instance or file object, or any object with a ``read`` method.
+  The ``filename`` attribute of your ConfigObj will be ``None`` [#]_.
+
+* A dictionary. You can initialise a ConfigObj from a dictionary [#]_. The
+  ``filename`` attribute of your ConfigObj will be ``None``. All keys must be
+  strings. In this case, the order of values and sections is arbitrary.
+
+options
+-------
+
+There are various options that control the way ConfigObj behaves. They can be
+passed in as a dictionary of options, or as keyword arguments. Explicit keyword
+arguments override the dictionary.
+
+All of the options are available as attributes after the config file has been
+parsed.
+
+ConfigObj has the following options (with the default values shown) :
+
+* 'raise_errors': ``False``
+
+    When parsing, it is possible that the config file will be badly formed. The
+    default is to parse the whole file and raise a single error at the end. You
+    can set ``raise_errors = True`` to have errors raised immediately. See the
+    exceptions_ section for more details.
+
+    Altering this value after initial parsing has no effect.
+
+* 'list_values': ``True``
+
+    If ``True`` (the default) then list values are possible. If ``False``, the
+    values are not parsed for lists.
+	
+	If ``list_values = False`` then single line values are not quoted or
+	unqouted when reading and writing.
+
+    Changing this value affects whether single line values will be quoted or 
+    not when writing.
+
+* 'create_empty': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will create an empty file. This can be a useful test that
+    the filename makes sense: an impossible filename will cause an error.
+
+    Altering this value after initial parsing has no effect.
+
+* 'file_error': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will raise an ``IOError``.
+
+    Altering this value after initial parsing has no effect.
+
+* 'interpolation': ``True``
+
+    Whether string interpolation is switched on or not. It is on (``True``) by
+    default.
+
+    You can set this attribute to change whether string interpolation is done
+    when values are fetched. See the interpolation_ section for more details.
+
+* 'configspec': ``None``
+
+    If you want to use the validation system, you supply a configspec. This is
+    effectively a type of config file that specifies a check for each member.
+    This check can be used to do type conversion as well as check that the
+    value is within your required parameters.
+
+    You provide a configspec in the same way as you do the initial file: a
+    filename, or list of lines, etc. See the validation_ section for full
+    details on how to use the system.
+
+    When parsed, every section has a ``configspec`` with a dictionary of
+    configspec checks for *that section*.
+
+* 'stringify': ``True``
+
+    If you use the validation scheme, it can do type checking *and* conversion
+    for you. This means you may want to set members to integers, or other
+    non-string values.
+
+    If 'stringify' is set to ``True`` (default) then non-string values will
+    be converted to strings when you write the config file. The validation_
+    process converts values from strings to the required type.
+
+    If 'stringify' is set to ``False``, attempting to set a member to a
+    non-string value [#]_ will raise a ``TypeError`` (no type conversion is
+    done by validation).
+
+* 'indent_type': ``' '``
+
+    Indentation is not significant; it can however be present in the output
+    config. Allowable values are: ``''`` (no indentation), ``' '`` (indentation
+    with spaces, fixed at four per level), or ``'\t'`` (indentation with tabs,
+    one tab per level).
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    dictionary, the indentation used in the output is the default one, that is,
+    spaces.
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    list of lines or a file, the indentation used in the first indented line is
+    selected and used in all output lines. If no input line is indented, no
+    output line will be either.
+
+    If this option *is* specified, the option value is used in the output
+    config, overriding the type of indentation in the input config (if any).
+
+* 'encoding': ``None``
+
+    By default **ConfigObj** does not decode the file/strings you pass it into
+    Unicode [#]_. If you want your config file as Unicode (keys and members)
+    you need to provide an encoding to decode the file with. This encoding will
+    also be used to encode the config file when writing.
+    
+    You can change the encoding attribute at any time.
+    
+    Any characters in your strings that can't be encoded with the specified
+    encoding will raise a ``UnicodeEncodeError``.
+    
+    .. note::
+    
+        ``UTF16`` encoded files will automatically be detected and decoded,
+        even if ``encoding`` is ``None``.
+        
+        This is because it is a 16-bit encoding, and ConfigObj will mangle it
+        (split characters on byte boundaries) if it parses it without decoding.
+
+* 'default_encoding': ``None``
+
+    When using the ``write`` method, **ConfigObj** uses the ``encoding``
+    attribute to encode the Unicode strings. If any members (or keys) have
+    been set as byte strings instead of Unicode, these must first be decoded
+    to Unicode before outputting in the specified encoding.
+
+    ``default_encoding``, if specified, is the encoding used to decode byte
+    strings in the **ConfigObj** before writing. If this is ``None``, then
+    the Python default encoding (``sys.defaultencoding`` - usually ascii) is
+    used.
+    
+    For most Western European users, a value of ``latin-1`` is sensible.
+    
+    ``default_encoding`` is *only* used if an ``encoding`` is specified.
+    
+    Any characters in byte-strings that can't be decoded using the
+    ``default_encoding`` will raise a ``UnicodeDecodeError``.
+
+
+Methods
+-------
+
+The ConfigObj is a subclass of an object called ``Section``, which is itself a
+subclass of ``dict``, the builtin dictionary type. This means it also has
+**all** the normal dictionary methods.
+
+In addition, the following `Section Methods`_ may be useful :
+
+* *encode*
+* *decode*
+* *walk*
+* *merge*
+* *dict* 
+* *as_bool*
+* *as_float*
+* *as_int*
+
+Read about Sections_ for details of all the methods.
+
+.. hint::
+
+    The *merge* method of sections is a recursive update.
+    
+    You can use this to merge sections, or even whole ConfigObjs, into each
+    other.
+    
+    You would typically use this to create a default ConfigObj and then merge
+    in user settings. This way users only need to specify values that are
+    different from the default.
+    
+
+The public methods available on ConfigObj are :
+
+* 'write'
+* 'validate'
+
+write
+~~~~~
+
+::
+
+    write(file_object=None)
+
+This method writes the current ConfigObj and takes a single, optional argument
+[#]_.
+
+If you pass in a file like object to the ``write`` method, the config file will
+be written to this. (The only method of this object that is used is its
+``write`` method, so a ``StringIO`` instance, or any other file like object
+will work.)
+
+Otherwise, the behaviour of this method depends on the ``filename`` attribute
+of the ConfigObj.
+
+``filename``
+    ConfigObj will write the configuration to the file specified.
+
+``None``
+    ``write`` returns a list of lines. (Not ``'\n'`` terminated)
+
+First the 'initial_comment' is written, then the config file, followed by the
+'final_comment'. Comment lines and inline comments are written with each
+key/value.
+
+validate
+~~~~~~~~
+
+::
+
+    validate(validator, preserve_errors=False)
+
+.. raw:: html
+
+    {+coloring}
+
+    # filename is the config file
+    # filename2 is the configspec
+    # (which could also be hardcoded into your program)
+    config = ConfigObj(filename, configspec=filename2)
+    #
+    from validate import Validator
+    val = Validator()
+    test = config.validate(val)
+    if test == True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+The validate method uses the `validate 
+<http://www.voidspace.org.uk/python/validate.html>`__ module to do the
+validation.
+    
+This method validates the ConfigObj against the configspec. By doing type
+conversion as well, it can abstract away the config file altogether and present
+the config *data* to your application (in the types it expects it to be).
+
+If the ``configspec`` attribute of the ConfigObj is ``None``, it raises a
+``ValueError``.
+
+If the stringify_ attribute is set, this process will convert values to the
+type defined in the configspec.
+
+The validate method uses checks specified in the configspec and defined in the
+``Validator`` object. It is very easy to extend.
+
+The configspec looks like the config file, but instead of the value, you
+specify the check (and any default value). See the validation_ section for
+details.
+
+.. hint::
+
+    If your ConfigObj is only comprised of basic data types, then you can use
+    a function from the ConfigPersist.py_ module to auto-generate your
+    configspec.
+    
+    See `ConfigObj for Data Persistence`_.
+
+
+Return Value
+############
+
+By default, the validate method either returns ``True`` (everything passed) 
+or a dictionary of ``True``/``False`` representing pass/fail. The dictionary 
+follows the structure of the ConfigObj.
+
+If a whole section passes then it is replaced with the value ``True``. If a 
+whole section fails, then it is replaced with the value ``False``.
+
+If a value is missing, and there is no default in the check, then the check 
+automatically fails.
+
+The ``validate`` method takes an optional keyword argument ``preserve_errors``.
+If you set this to ``True``, instead of getting ``False`` for failed checks you
+get the actual error object from the **validate** module. This usually contains
+useful information about why the check failed.
+
+See the `flatten_errors`_ function for how to turn your results dictionary into
+a useful list of error messages.
+
+Even if ``preserve_errors`` is ``True``, missing keys or sections will still be
+represented by a ``False`` in the results dictionary.
+
+
+Mentioning Default Values
+#########################
+
+In the check in your configspec, you can specify a default to be used - by 
+using the ``default`` keyword. E.g. ::
+
+    key1 = integer(0, 30, default=15)
+    key2 = integer(default=15)
+    key3 = boolean(default=True)
+    key4 = option('Hello', 'Goodbye', 'Not Today', default='Not Today')
+
+If the configspec check supplies a default and the value is missing in the
+config, then the default will be set in your ConfigObj. (It is still passed to
+the ``Validator`` so that type conversion can be done: this means the default
+value must still pass the check.)
+
+ConfigObj keeps a record of which values come from defaults, using the
+``defaults`` attribute of sections_. Any key in this list isn't written out by
+the ``write`` method. If a key is set from outside (even to the same value)
+then it is removed from the ``defaults`` list.
+
+.. note:
+
+    Even if all the keys in a section are in the defaults list, the section
+    marker is still written out.
+
+There is additionally a special case default value of ``None``. If you set the
+default value to ``None`` and the value is missing, the value will always be
+set to ``None``. As the other checks don't return ``None`` (unless you
+implement your own that do), you can tell that this value came from a default
+value (and was missing from the config file). It allows an easy way of
+implementing optional values. Simply check (and ignore) members that are set
+to ``None``.
+
+.. note::
+
+    If stringify_ is ``False`` then ``default=None`` returns ``''`` instead of
+    ``None``. This is because setting a value to a non-string raises an error
+    if stringify is unset.
+
+The default value can be a list. See `List Values`_ for the way to do this.
+
+Writing invalid default values is a *guaranteed* way of confusing your users.
+Default values **must** pass the check.
+
+Mentioning Repeated Sections
+############################
+
+In the configspec it is possible to cause *every* sub-section in a section to
+be validated using the same configspec. You do this with a section in the
+configspec  called ``__many__``. Every sub-section in that section has the
+``__many__`` configspec applied to it (without you having to explicitly name
+them in advance).
+
+If you define a ``__many__`` type section it must the only sub-section in that
+section. Having a ``__many__`` *and* other sub-sections defined in the same
+section will raise a ``RepeatSectionError``.
+
+Your ``__many__`` section can have nested subsections, which can also include
+``__many__`` type sections.
+
+See `Repeated Sections`_ for examples.
+
+
+Mentioning SimpleVal
+####################
+
+If you just want to check if all members are present, then you can use the
+``SimpleVal`` object that comes with ConfigObj. It only fails members if they
+are missing.
+
+Write a configspec that has all the members you want to check for, but set
+every section to ``''``.
+
+.. raw:: html
+
+    {+coloring}
+
+    val = SimpleVal()
+    test = config.validate(val)
+    if test is True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+    
+Attributes
+----------
+
+A ConfigObj has the following attributes :
+
+* indent_type
+* interpolate
+* stringify
+* BOM
+* initial_comment
+* final_comment
+* list_values
+* encoding
+* default_encoding
+
+.. note::
+
+    This doesn't include *comments*, *inline_comments*, *defaults*, or
+    *configspec*. These are actually attributes of Sections_.
+
+It also has the following attributes as a result of parsing. They correspond to
+options_ when the ConfigObj was created, but changing them has no effect.
+
+* raise_errors
+* create_empty
+* file_error
+
+interpolate
+~~~~~~~~~~~
+
+ConfigObj can perform string interpolation in a *similar* way to
+``ConfigParser``. See the interpolation_ section for full details.
+
+If ``interpolate`` is set to ``False``, then interpolation is *not* done when
+you fetch values.
+
+stringify
+~~~~~~~~~
+
+If this attribute is set (``True``) then the validate_ method changes the
+values in the ConfigObj. These are turned back into strings when write_ is
+called.
+
+If stringify is unset (``False``) then attempting to set a value to a non
+string (or a list of strings) will raise a ``TypeError``.
+
+BOM
+~~~
+
+If the initial config file *started* with the UTF8 Unicode signature (known
+slightly incorrectly as the {acro;BOM;Byte Order Mark}), or the UTF16 BOM, then
+this attribute is set to ``True``. Otherwise it is ``False``.
+
+If it is set to ``True`` when ``write`` is called then, if ``encoding`` is set
+to ``None`` *or* to ``utf_8`` (and variants) a UTF BOM will be written.
+
+For UTF16 encodings, a BOM is *always* written.
+
+initial_comment
+~~~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments before the start of the members.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The write method puts these lines before it starts writing out the members.
+
+final_comment
+~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments after the last member.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The ``write`` method puts these lines after it finishes writing out the
+members.
+
+list_values
+~~~~~~~~~~~
+
+This attribute is ``True`` or ``False``. If set to ``False`` then values are
+not parsed for list values. In addition single line values are not unquoted.
+
+This allows you to do your own parsing of values. It exists primarily to
+support the reading of the configspec_ - but has other use cases.
+
+For example you could use the ``LineParser`` from the
+`listquote module <http://www.voidspace.org.uk/python/listquote.html#lineparser>`_ 
+to read values for nested lists.
+
+Single line values aren't quoted when writing - but multiline values are
+handled as normal.
+
+.. caution::
+
+    Because values aren't quoted, leading or trailing whitespace can be
+	lost.
+
+    This behaviour was changed in version 4.0.1.
+	
+	Prior to this, single line values might have been quoted; even with
+	``list_values=False``. This means that files written by **ConfigObj**
+	*could* now be incompatible - and need the quotes removing by hand.
+
+encoding
+~~~~~~~~
+
+This is the encoding used to encode the output, when you call ``write``. It
+must be a valid encoding `recognised by Python <http://docs.python.org/lib/standard-encodings.html>`_.
+
+If this value is ``None`` then no encoding is done when ``write`` is called.
+
+
+default_encoding
+~~~~~~~~~~~~~~~~
+
+If encoding is set, any byte-strings in your ConfigObj instance (keys or
+members) will first be decoded to unicode using the encoding specified by the
+``default_encoding`` attribute. This ensures that the output is in the encoding
+specified.
+
+If this value is ``None`` then ``sys.defaultencoding`` is used instead.
+
+The Config File Format
+======================
+
+You saw an example config file in the `Config Files`_ section. Here is a fuller
+specification of the config files used and created by ConfigObj.
+
+The basic pattern for keywords is : ::
+
+    # comment line
+    # comment line
+    keyword = value # inline comment
+
+Both keyword and value can optionally be surrounded in quotes. The equals sign
+is the only valid divider.
+
+Values can have comments on the lines above them, and an inline comment after
+them. This, of course, is optional. See the comments_ section for details.
+
+If a keyword or value starts or ends with whitespace, or contains a quote mark
+or comma, then it should be surrounded by quotes. Quotes are not necessary if
+whitespace is surrounded by non-whitespace.
+
+Values can also be lists. Lists are comma separated. You indicate a single
+member list by a trailing comma. An empty list is shown by a single comma : ::
+
+    keyword1 = value1, value2, value3
+    keyword2 = value1, # a single member list
+    keyword3 = , # an empty list
+
+Values that contain line breaks (multi-line values) can be surrounded by triple
+quotes. These can also be used if a value contains both types of quotes. List
+members cannot be surrounded by triple quotes : ::
+
+    keyword1 = ''' A multi line value
+    on several
+    lines'''     # with a comment
+    keyword2 = '''I won't be "afraid".'''
+    #
+    keyword3 = """ A multi line value
+    on several
+    lines"""     # with a comment
+    keyword4 = """I won't be "afraid"."""
+
+.. warning::
+
+    There is no way of safely quoting values that contain both types of triple
+    quotes.
+
+A line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+New sections are indicated by a section marker line. That is the section name
+in square brackets. Whitespace around the section name is ignored. The name can
+be quoted with single or double quotes. The marker can have comments before it
+and an inline comment after it : ::
+
+    # The First Section
+    [ section name 1 ] # first section
+    keyword1 = value1
+
+    # The Second Section
+    [ "section name 2" ] # second section
+    keyword2 = value2
+
+Any subsections (sections that are *inside* the current section) are
+designated by repeating the square brackets before and after the section name.
+The number of square brackets represents the nesting level of the sub-section.
+Square brackets may be separated by whitespace; such whitespace, however, will
+not be present in the output config written by the ``write`` method.
+
+Indentation is not significant, but can be preserved. See the description of
+the ``indent_type`` option, in the `ConfigObj specifications`_ chapter, for the
+details.
+
+A *NestingError* will be raised if the number of the opening and the closing
+brackets in a section marker is not the same, or if a sub-section's nesting
+level is greater than the nesting level of it parent plus one.
+
+In the outer section, single values can only appear before any sub-section.
+Otherwise they will belong to the sub-section immediately before them. ::
+
+    # initial comment
+    keyword1 = value1
+    keyword2 = value2
+
+    [section 1]
+    keyword1 = value1
+    keyword2 = value2
+
+        [[sub-section]]
+        # this is in section 1
+        keyword1 = value1
+        keyword2 = value2
+
+            [[[nested section]]]
+            # this is in sub section
+            keyword1 = value1
+            keyword2 = value2
+
+        [[sub-section2]]
+        # this is in section 1 again
+        keyword1 = value1
+        keyword2 = value2
+
+    [[sub-section3]]
+    # this is also in section 1, indentation is misleading here
+    keyword1 = value1
+    keyword2 = value2
+
+    # final comment
+
+When parsed, the above config file produces the following data structure :
+
+.. raw:: html
+
+    {+coloring}
+
+    {
+        'keyword1': 'value1',
+        'keyword2': 'value2',
+        'section 1': {
+            'keyword1': 'value1',
+            'keyword2': 'value2',
+            'sub-section': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+                'nested section': {
+                    'keyword1': 'value1',
+                    'keyword2': 'value2',
+                },
+            },
+            'sub-section2': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+            'sub-section3': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+        },
+    }
+
+    {-coloring}
+
+Sections are ordered: note how the structure of the resulting ConfigObj is in
+the same order as the original file.
+
+Sections
+========
+
+Every section in a ConfigObj has certain properties. The ConfigObj itself also
+has these properties, because it too is a section (sometimes called the *root
+section*).
+
+``Section`` is a subclass of the standard new-class dictionary, therefore it
+has **all** the methods of a normal dictionary. This means you can ``update``
+and ``clear`` sections.
+
+.. note::
+
+    You create a new section by assigning a member to be a dictionary.
+    
+    The new ``Section`` is created *from* the dictionary, but isn't the same
+    thing as the dictionary. (So references to the dictionary you use to create
+    the section *aren't* references to the new section).
+    
+    Note the following.
+
+    .. raw:: html
+    
+        {+coloring}
+        
+        config = ConfigObj()
+        vals = {'key1': 'value 1', 
+                'key2': 'value 2'
+               }
+        config['vals'] = vals
+        config['vals'] == vals
+        True
+        config['vals'] is vals
+        False
+        
+        {-coloring}
+     
+    If you now change ``vals``, the changes won't be reflected in ``config['vals']``.
+
+A section is ordered, following its ``scalars`` and ``sections``
+attributes documented below. This means that the following dictionary
+attributes return their results in order.
+
+* '__iter__'
+
+    More commonly known as ``for member in section:``.
+
+* '__repr__' and '__str__'
+
+    Any time you print or display the ConfigObj.
+
+* 'items'
+
+* 'iteritems'
+
+* 'iterkeys'
+
+* 'itervalues'
+
+* 'keys'
+
+* 'popitem'
+
+* 'values'
+
+Section Attributes
+------------------
+
+* main
+
+    A reference to the main ConfigObj.
+
+* parent
+
+    A reference to the 'parent' section, the section that this section is a
+    member of.
+
+    On the ConfigObj this attribute is a reference to itself. You can use this
+    to walk up the sections, stopping when ``section.parent is section``.
+
+* depth
+
+    The nesting level of the current section.
+
+    If you create a new ConfigObj and add sections, 1 will be added to the
+    depth level between sections.
+
+* defaults
+
+    This attribute is a list of scalars that came from default values. Values
+    that came from defaults aren't written out by the ``write`` method.
+    Setting any of these values in the section removes them from the defaults
+    list.
+
+* scalars, sections
+
+    These attributes are normal lists, representing the order that members,
+    single values and subsections appear in the section. The order will either
+    be the order of the original config file, *or* the order that you added
+    members.
+
+    The order of members in this lists is the order that ``write`` creates in
+    the config file. The ``scalars`` list is output before the ``sections``
+    list.
+
+    Adding or removing members also alters these lists. You can manipulate the
+    lists directly to alter the order of members.
+
+    .. warning::
+
+        If you alter the ``scalars``, ``sections``, or ``defaults`` attributes
+        so that they no longer reflect the contents of the section, you will
+        break your ConfigObj.
+
+    See also the ``rename`` method.
+
+* comments
+
+    This is a dictionary of comments associated with each member. Each entry is
+    a list of lines. These lines are written out before the member.
+
+* inline_comments
+
+    This is *another* dictionary of comments associated with each member. Each
+    entry is a string that is put inline with the member.
+
+* configspec
+
+    The configspec attribute is a dictionary mapping scalars to *checks*. A
+    check defines the expected type and possibly the allowed values for a
+    member.
+
+    The configspec has the same format as a config file, but instead of values
+    it has a specification for the value (which may include a default value).
+    The validate_ method uses it to check the config file makes sense. If a
+    configspec is passed in when the ConfigObj is created, then it is parsed
+    and broken up to become the ``configspec`` attribute of each section.
+
+    If you didn't pass in a configspec, this attribute will be ``None`` on the
+    root section (the main ConfigObj).
+
+    You can set the configspec attribute directly on a section.
+
+    See the validation_ section for full details of how to write configspecs.
+
+Section Methods
+---------------
+
+* **dict**
+
+    This method takes no arguments. It returns a deep copy of the section as a
+    dictionary. All subsections will also be dictionaries, and list values will
+    be copies, rather than references to the original [#]_.
+
+* **rename**
+
+    ``rename(oldkey, newkey)``
+
+    This method renames a key, without affecting its position in the sequence.
+
+    It is mainly implemented for the ``encode`` and ``decode`` methods, which
+    provide some Unicode support.
+
+* **merge**
+
+    ``merge(indict)``
+    
+    This method is a *recursive update* method. It allows you to merge two
+    config files together.
+    
+    You would typically use this to create a default ConfigObj and then merge
+    in user settings. This way users only need to specify values that are
+    different from the default.
+    
+    For example :
+    
+    .. raw:: html
+    
+        {+coloring}
+        
+        # def_cfg contains your default config settings
+        # user_cfg contains the user settings
+        cfg = ConfigObj(def_cfg)
+        usr = ConfigObj(user_cfg)
+        #
+        cfg.merge(usr)
+        
+        """
+        cfg now contains a combination of the default settings and the user
+        settings.
+        
+        The user settings will have overwritten any of the default ones.
+        """
+    
+        {-coloring}
+    
+* **walk**
+
+    This method can be used to transform values and names. See `walking a
+    section`_ for examples and explanation.
+
+* **decode**
+
+    ``decode(encoding)``
+
+    This method decodes names and values into Unicode objects, using the
+    supplied encoding.
+
+* **encode**
+
+    ``encode(encoding)``
+
+    This method is the opposite of ``decode`` {sm;:!:}.
+
+    It encodes names and values using the supplied encoding. If any of your
+    names/values are strings rather than Unicode, Python will have to do an
+    implicit decode first. (This method uses ``sys.defaultencoding`` for
+    implicit decodes.)
+
+* **as_bool**
+
+    ``as_bool(key)``
+    
+    Returns ``True`` if the key contains a string that represents ``True``, or
+    is the ``True`` object.
+    
+    Returns ``False`` if the key contains a string that represents ``False``, 
+    or is the ``False`` object. 
+
+    Raises a ``ValueError`` if the key contains anything else.
+    
+    Strings that represent ``True`` are (not case sensitive) : ::
+    
+        true, yes, on, 1
+        
+    Strings that represent ``False`` are : ::
+    
+        false, no, off, 0
+    
+    .. note::
+    
+        In ConfigObj 4.1.0, this method was called ``istrue``. That method is
+        now deprecated and will issue a warning when used. It will go away
+        in the next release.
+        
+* **as_int**
+
+    ``as_int(key)``
+    
+    This returns the value contained in the specified key as an integer.
+    
+    It raises a ``ValueError`` if the conversion can't be done.
+
+* **as_float**
+
+    ``as_float(key)``
+    
+    This returns the value contained in the specified key as a float.
+    
+    It raises a ``ValueError`` if the conversion can't be done.
+    
+Walking a Section
+-----------------
+
+.. note::
+
+    The walk method allows you to call a function on every member/name.
+
+.. raw:: html
+
+    {+coloring}
+
+        walk(function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+
+    {-coloring}
+
+``walk`` is a method of the ``Section`` object. This means it is also a method
+of ConfigObj.
+
+It walks through every member and calls a function on the keyword and value. It
+walks recursively through subsections.
+
+It returns a dictionary of all the computed values.
+
+If the function raises an exception, the default is to propagate the error, and
+stop. If ``raise_errors=False`` then it sets the return value for that keyword
+to ``False`` instead, and continues. This is similar to the way validation_
+works.
+
+Your function receives the arguments ``(section, key)``. The current value is
+then ``section[key]`` [#]_. Any unrecognised keyword arguments you pass to
+walk, are passed on to the function.
+
+Normally ``walk`` just recurses into subsections. If you are transforming (or
+checking) names as well as values, then you want to be able to change the names
+of sections. In this case set ``call_on_sections`` to ``True``. Now, on
+encountering a sub-section, *first* the function is called for the *whole*
+sub-section, and *then* it recurses into it's members. This means your function
+must be able to handle receiving dictionaries as well as strings and lists.
+
+If you are using the return value from ``walk`` *and* ``call_on_sections``,
+note that walk discards the return value when it calls your function.
+
+.. caution::
+
+    You can use ``walk`` to transform the names of members of a section
+    but you mustn't add or delete members.
+
+
+Examples
+--------
+
+Examples that use the walk method are the ``encode`` and ``decode`` methods.
+They both define a function and pass it to walk. Because these functions
+transform names as well as values (from byte strings to Unicode) they set
+``call_on_sections=True``.
+
+To see how they do it, *read the source Luke* {sm;:cool:}.
+
+You can use this for transforming all values in your ConfigObj. For example
+you might like the nested lists from ConfigObj 3. This was provided by the
+listquote_ module. You could switch off the parsing for list values
+(``list_values=False``) and use listquote to parse every value.
+
+Another thing you might want to do is use the Python escape codes in your
+values. You might be *used* to using ``\n`` for line feed and ``\t`` for tab.
+Obviously we'd need to decode strings that come from the config file (using the
+escape codes). Before writing out we'll need to put the escape codes back in
+encode.
+
+As an example we'll write a function to use with walk, that encodes or decodes
+values using the ``string-escape`` codec.
+
+The function has to take each value and set the new value. As a bonus we'll
+create one function that will do decode *or* encode depending on a keyword
+argument.
+
+We don't want to work with section names, we're only transforming values, so
+we can leave ``call_on_sections`` as ``False``. This means the two datatypes we
+have to handle are strings and lists, we can ignore everything else. (We'll
+treat tuples as lists as well).
+
+We're not using the return values, so it doesn't need to return anything, just
+change the values if appropriate.
+
+.. raw:: html
+
+    {+coloring}
+
+    def string_escape(section, key, encode=False):
+        """
+        A function to encode or decode using the 'string-escape' codec.
+        To be passed to the walk method of a ConfigObj.
+        By default it decodes.
+        To encode, pass in the keyword argument ``encode=True``.
+        """
+        val = section[key]
+        # is it a type we can work with
+        # NOTE: for platforms where Python > 2.2
+        # you can use basestring instead of (str, unicode)
+        if not isinstance(val, (str, unicode, list, tuple)):
+            # no !
+            return
+        elif isinstance(val, (str, unicode)):
+            # it's a string !
+            if not encode:
+                section[key] = val.decode('string-escape')
+            else:
+                section[key] = val.encode('string-escape')
+        else:
+            # it must be a list or tuple!
+            # we'll be lazy and create a new list
+            newval = []
+            # we'll check every member of the list
+            for entry in val:
+                if isinstance(entry, (str, unicode)):
+                    if not encode:
+                        newval.append(entry.decode('string-escape'))
+                    else:
+                       newval.append(entry.encode('string-escape'))
+                else:
+                    newval.append(entry)
+            # done !
+            section[key] =  newval
+
+    # assume we have a ConfigObj called ``config``
+    #
+    # To decode
+    config.walk(string_escape)
+    #
+    # To encode.
+    # Because ``walk`` doesn't recognise the ``encode`` argument
+    # it passes it to our function.
+    config.walk(string_escape, encode=True)
+
+    {-coloring}
+
+Here's a simple example of using ``walk`` to transform names and values. One
+usecase of this would be to create a *standard* config file with placeholders
+for section and keynames. You can then use walk to create new config files
+and change values and member names :
+
+.. raw:: html
+
+    {+coloring}
+
+    # We use 'XXXX' as a placeholder
+    config = '''
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+    [XXXXsection1]
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+    [XXXXsection2]
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+        [[XXXXsection1]]
+        XXXXkey1 = XXXXvalue1
+        XXXXkey2 = XXXXvalue2
+        XXXXkey3 = XXXXvalue3
+    '''.splitlines()
+    cfg = ConfigObj(config)
+    #
+    def transform(section, key):
+        val = section[key]
+        newkey = key.replace('XXXX', 'CLIENT1')
+        section.rename(key, newkey)
+        if isinstance(val, (tuple, list, dict)):
+            pass
+        else:
+            val = val.replace('XXXX', 'CLIENT1')
+            section[newkey] = val
+    #
+    cfg.walk(transform, call_on_sections=True)
+    print cfg
+    {'CLIENT1key1': 'CLIENT1value1', 'CLIENT1key2': 'CLIENT1value2', 
+    'CLIENT1key3': 'CLIENT1value3', 
+    'CLIENT1section1': {'CLIENT1key1': 'CLIENT1value1', 
+        'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3'}, 
+    'CLIENT1section2': {'CLIENT1key1': 'CLIENT1value1', 
+        'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3', 
+        'CLIENT1section1': {'CLIENT1key1': 'CLIENT1value1', 
+            'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3'}}}
+            
+    {-coloring}
+
+
+Exceptions
+==========
+
+There are several places where ConfigObj may raise exceptions (other than
+because of bugs).
+
+1) If a configspec filename you pass in doesn't exist, or a config file
+    filename doesn't exist *and* ``file_error=True``, an ``IOError`` will be
+    raised.
+
+2) If you try to set a non-string key, or a non string value when
+    ``stringify=False``, a ``TypeError`` will be raised.
+
+3) A badly built config file will cause parsing errors.
+
+4) A parsing error can also occur when reading a configspec.
+
+5) In string interpolation you can specify a value that doesn't exist, or
+    create circular references (recursion).
+
+6) If you have a ``__many__`` repeated section with other section definitions
+    (in a configspec), a ``RepeatSectionError`` will be raised.
+
+Number 5 (which is actually two different types of exceptions) is documented
+    in interpolation_.
+
+Number 6 is explained in the validation_ section.
+
+*This* section is about errors raised during parsing.
+
+The base error class is ``ConfigObjError``. This is a subclass of
+``SyntaxError``, so you can trap for ``SyntaxError`` without needing to
+directly import any of the ConfigObj exceptions.
+
+The following other exceptions are defined (all deriving from
+``ConfigObjError``) :
+
+* ``NestingError``
+
+    This error indicates either a mismatch in the brackets in a section marker,
+    or an excessive level of nesting.
+
+* ``ParseError``
+
+    This error indicates that a line is badly written. It is neither a valid
+    ``key = value`` line, nor a valid section marker line, nor a comment line.
+
+* ``DuplicateError``
+
+    The keyword or section specified already exists.
+
+* ``ConfigspecError``
+
+    An error occured whilst parsing a configspec.
+
+When parsing a configspec, ConfigObj will stop on the first error it
+encounters.  It will raise a ``ConfigspecError``. This will have an ``error``
+attribute, which is the actual error that was raised.
+
+Behavior when parsing a config file depends on the option ``raise_errors``.
+If ConfigObj encounters an error while parsing a config file:
+
+    If ``raise_errors=True`` then ConfigObj will raise the appropriate error
+    and parsing will stop.
+
+    If ``raise_errors=False`` (the default) then parsing will continue to the
+    end and *all* errors will be collected.
+
+In the second case a ``ConfigObjError`` is raised after parsing has stopped.
+The error raised has a ``config`` attribute, which is the parts of the
+ConfigObj that parsed successfully. It also has an attribute ``errors``, which
+is a list of *all* the errors raised. Each entry in the list is an instance of
+the appropriate error type. Each one has the following attributes (useful for
+delivering a sensible error message to your user) :
+
+* ``line``: the original line that caused the error.
+
+* ``line_number``: its number in the config file.
+
+* ``message``: the error message that accompanied the error.
+
+.. note::
+
+    One wrongly written line could break the basic structure of your config
+    file. This could cause every line after it to flag an error, so having a
+    list of all the lines that caused errors may not be as useful as it sounds.
+    {sm;:-(}.
+
+
+Validation
+==========
+
+Validation is done through a combination of the configspec_ and a ``Validator``
+object. For this you need *validate.py* [#]_. See downloading_ if you don't
+have a copy.
+
+Validation can perform two different operations :
+
+1) Check that a value meets a specification. For example, check that a value
+    is an integer between one and six, or is a choice from a specific set of
+    options.
+
+2) It can convert the value into the type required. For example, if one of
+    your values is a port number, validation will turn it into an integer for
+    you.
+
+So validation can act as a transparent layer between the datatypes of your
+application configuration (boolean, integers, floats, etc) and the text format
+of your config file.
+
+configspec
+----------
+
+The ``validate`` method checks members against an entry in the configspec. Your
+configspec therefore resembles your config file, with a check for every member.
+
+In order to perform validation you need a ``Validator`` object. This has
+several useful built-in check functions. You can also create your own custom
+functions and register them with your Validator object.
+
+Each check is the name of one of these functions, including any parameters and
+keyword arguments. The configspecs look like function calls, and they map to
+function calls.
+
+The basic datatypes that an un-extended Validator can test for are :
+
+* boolean values (True and False)
+* integers (including minimum and maximum values)
+* floats (including min and max)
+* strings (including min and max length)
+* IP addresses (v4 only)
+
+It can also handle lists of these types and restrict a value to being one from
+a set of options.
+
+An example configspec is going to look something like : ::
+
+    port = integer(0, 100)
+    user = string(max=25)
+    mode = option('quiet', 'loud', 'silent')
+
+You can specify default values, and also have the same configspec applied to
+several sections. This is called `repeated sections`_.
+
+For full details on writing configspecs, please refer to the `validate.py
+documentation`_.
+
+.. important::
+
+    Your configspec is read by ConfigObj in the same way as a config file.
+    
+    That means you can do interpolation *within* your configspec.
+    
+    In order to allow this, checks in the 'DEFAULT' section (of the root level
+    of your configspec) are *not* used.
+
+.. _validate.py documentation: http://www.voidspace.org.uk/python/validate.html
+
+Type Conversion
+---------------
+
+By default, validation does type conversion. This means that if you specify
+``integer`` as the check, then calling validate_ will actually change the value
+to an integer (so long as the check succeeds).
+
+It also means that when you call the write_ method, the value will be converted
+back into a string using the ``str`` function.
+
+To switch this off, and leave values as strings after validation, you need to
+set the stringify_ attribute to ``False``. If this is the case, attempting to
+set a value to a non-string will raise an error.
+
+Default Values
+--------------
+
+You can set a default value in your check. If the value is missing from the
+config file then this value will be used instead. This means that your user
+only has to supply values that differ from the defaults.
+
+If you *don't* supply a default then for a value to be missing is an error,
+and this will show in the `return value`_ from validate.
+
+Additionally you can set the default to be ``None``. This means the value will
+be set to ``None`` (the object) *whichever check is used*. (It will be set to
+``''`` rather than ``None`` if stringify_ is ``False``). You can use this
+to easily implement optional values in your config files. ::
+
+    port = integer(0, 100, default=80)
+    user = string(max=25, default=0)
+    mode = option('quiet', 'loud', 'silent', default='loud')
+    nick = string(default=None)
+
+.. note::
+
+    Because the default goes through type conversion, it also has to pass the
+    check.
+
+    Note that ``default=None`` is case sensitive.
+
+
+List Values
+~~~~~~~~~~~
+
+It's possible that you will want to specify a list as a default value. To avoid
+confusing syntax with commas and quotes you use a list constructor to specify 
+that keyword arguments are lists. This includes the ``default`` value. This 
+makes checks look something like : ::
+
+    checkname(default=list('val1', 'val2', 'val3'))
+
+This works with all keyword arguments, but is most useful for default values.
+
+
+Repeated Sections
+-----------------
+
+Repeated sections are a way of specifying a configspec for a section that
+should be applied to *all* subsections in the same section.
+
+The easiest way of explaining this is to give an example. Suppose you have a
+config file that describes a dog. That dog has various attributes, but it can
+also have many fleas. You don't know in advance how many fleas there will be,
+or what they will be called, but you want each flea validated against the same
+configspec.
+
+We can define a section called *fleas*. We want every flea in that section
+(every sub-section) to have the same configspec applied to it. We do this by
+defining a single section called ``__many__``. ::
+
+    [dog]
+    name = string(default=Rover)
+    age = float(0, 99, default=0)
+
+        [[fleas]]
+
+            [[[__many__]]]
+            bloodsucker = boolean(default=True)
+            children = integer(default=10000)
+            size = option(small, tiny, micro, default=tiny)
+
+Every flea on our dog will now be validated using the ``__many__`` configspec.
+
+If you define another sub-section in a section *as well as* a ``__many__`` then
+you will get an error.
+
+``__many__`` sections can have sub-sections, including their own ``__many__``
+sub-sections. Defaults work in the normal way in repeated sections.
+
+
+Validation and Interpolation
+----------------------------
+
+String interpolation and validation don't play well together. When validation
+changes type it sets the value. If the value uses interpolation, then the 
+interpolation reference would normally be overwritten. Calling ``write`` would
+then use the absolute value and the interpolation reference would be lost.
+
+As a compromise - if the value is unchanged by validation then it is not reset.
+This means strings that pass through validation unmodified will not be 
+overwritten. If validation changes type - the value has to be overwritten, and
+any interpolation references are lost {sm;:-(}.
+
+SimpleVal
+---------
+
+You may not need a full validation process, but still want to check if all the
+expected values are present.
+
+Provided as part of the ConfigObj module is the ``SimpleVal`` object. This has
+a dummy ``test`` method that always passes.
+
+The only reason a test will fail is if the value is missing. The return value
+from ``validate`` will either be ``True``, meaning all present, or a dictionary
+with ``False`` for all missing values/sections.
+
+To use it, you still need to pass in a valid configspec when you create the
+ConfigObj, but just set all the values to ``''``. Then create an instance of
+``SimpleVal`` and pass it to the ``validate`` method.
+
+As a trivial example if you had the following config file : ::
+
+    # config file for an application
+    port = 80
+    protocol = http
+    domain = voidspace
+    top_level_domain = org.uk
+
+You would write the following configspec : ::
+
+    port = ''
+    protocol = ''
+    domain = ''
+    top_level_domain = ''
+
+.. raw:: html
+
+    {+coloring}
+
+    config = Configobj(filename, configspec=configspec)
+    val = SimpleVal()
+    test = config.validate(val)
+    if test == True:
+        print 'All values present.'
+    elif test == False:
+        print 'No values present!'
+    else:
+        for entry in test:
+            if test[entry] == False:
+                print '"%s" missing.' % entry
+
+    {-coloring}
+
+
+Interpolation
+=============
+
+ConfigObj allows string interpolation *similar* to the way ``ConfigParser``
+
+You specify a value to be substituted by including ``%(name)s`` in the value.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section to
+see if ``name`` is the key to a value. ('name' is case sensitive).
+
+If it doesn't find it, next it checks the 'DEFAULT' section of the parent
+section, last it checks the 'DEFAULT' section of the main section.
+
+If the value specified isn't found then a ``MissingInterpolationOption`` error
+is raised (a subclass of ``ConfigObjError``).
+
+If it is found then the returned value is also checked for substitutions. This
+allows you to make up compound values (for example directory paths) that use
+more than one default value. It also means it's possible to create circular
+references. If after ten replacements there are still values to substitute, an
+``InterpolationDepthError`` is raised.
+
+Both of these errors are subclasses of ``InterpolationError``, which is a
+subclass of ``ConfigObjError``.
+
+String interpolation and validation don't play well together. This is because 
+validation overwrites values - and so may erase the interpolation references.
+See `Validation and Interpolation`_. (This can only happen if validation
+has to *change* the value).
+
+
+Comments
+========
+
+Any line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+If a config file starts with comments then these are preserved as the
+initial_comment_.
+
+If a config file ends with comments then these are preserved as the
+final_comment_.
+
+Every key or section marker may have lines of comments immediately above it.
+These are saved as the ``comments`` attribute of the section. Each member is a
+list of lines.
+
+You can also have a comment inline with a value. These are saved as the
+``inline_comments`` attribute of the section, with one entry per member of the
+section.
+
+Subsections (section markers in the config file) can also have comments.
+
+See `Section Attributes`_ for more on these attributes.
+
+These comments are all written back out by the ``write`` method.
+
+
+flatten_errors
+==============
+
+::
+
+    flatten_errors(cfg, res)
+
+Validation_ is a powerful way of checking that the values supplied by the user
+make sense.
+
+The validate_ method returns a results dictionary that represents pass or fail
+for each value. This doesn't give you any information about *why* the check
+failed.
+
+``flatten_errors`` is an example function that turns a results dictionary into
+a flat list, that only contains vaues that *failed*.
+
+``cfg`` is the ConfigObj instance being checked, ``res`` is the results
+dictionary returned by ``validate``.
+
+It returns a list of keys that failed. Each member of the list is a tuple : ::
+
+    ([list of sections...], key, result)
+
+If ``validate`` was called with ``preserve_errors=False`` (the default)
+then ``result`` will always be ``False``.
+
+*list of sections* is a flattened list of sections that the key was found
+in.
+
+If the section was missing then key will be ``None``.
+
+If the value (or section) was missing then ``result`` will be ``False``.
+
+If ``validate`` was called with ``preserve_errors=True`` and a value
+was present, but failed the check, then ``result`` will be the exception
+object returned. You can use this as a string that describes the failure.
+
+For example :
+
+    *The value "3" is of the wrong type*.
+
+
+Example Usage
+-------------
+
+The output from ``flatten_errors`` is a list of tuples.
+
+Here is an example of how you could present this information to the user.
+
+.. raw:: html
+
+    {+coloring}
+    
+    vtor = validate.Validator()
+    # ini is your config file - cs is the configspec
+    cfg = ConfigObj(ini, configspec=cs)
+    res = cfg.validate(vtor, preserve_errors=True)
+    for entry in flatten_errors(cfg, res):
+        # each entry is a tuple
+        section_list, key, error = entry
+        if key is not None:
+           section_list.append(key)
+        else:
+            section_list.append('[missing section]')
+        section_string = ', '.join(section_list)
+        if error == False:
+            error = 'Missing value or section.'
+        print section_string, ' = ', error
+
+    {-coloring}
+
+Backwards Compatibility
+=======================
+
+There have been a lot of changes since ConfigObj 3. The core parser is now
+based on regular expressions, and is a lot faster and smaller. There is now no
+difference in the way we treat flat files and non-flatfiles, that is, no empty
+sections. This means some of the code can be a lot simpler, less code does
+more of the work [#]_.
+
+There have been other simplifications: for example we only have eight options
+instead of seventeen.
+
+Most config files created for ConfigObj 3 will be read with no changes and many
+programs will work without having to alter code. Some of the changes do break
+backwards compatibility: for example, code that uses the previous options will
+now raise an error. It should be very easy to fix these, though.
+
+Below is a list of all the changes that affect backwards compatibility. This
+doesn't include details of method signatures that have changed, because almost
+all of them have.
+
+Incompatible Changes
+--------------------
+
+(I have removed a lot of needless complications: this list is probably not
+conclusive, many option/attribute/method names have changed.)
+
+Case sensitive.
+
+The only valid divider is '='.
+
+Line continuations with ``\`` removed.
+
+No recursive lists in values.
+
+No empty sections.
+
+No distinction between flatfiles and non flatfiles.
+
+Change in list syntax: use commas to indicate list, not parentheses (square
+brackets and parentheses are no longer recognised as lists).
+
+';' is no longer valid for comments, and no multiline comments.
+
+No attribute-style access to values.
+
+Empty values not allowed: use '' or "".
+
+In ConfigObj 3, setting a non-flatfile member to ``None`` would initialise it
+as an empty section.
+
+The escape entities '&mjf-lf;' and '&mjf-quot;' have gone, replaced by triple
+quote, multiple line values.
+
+The ``newline``, ``force_return``, and ``default`` options have gone.
+
+``fileerror`` and ``createempty`` options have become ``file_error`` and
+``create_empty``.
+
+Partial configspecs (for specifying the order members should be written out,
+and which should be present) have gone. The configspec is no longer used to
+specify order for the ``write`` method.
+
+Exceeding the maximum depth of recursion in string interpolation now raises an
+error ``InterpolationDepthError``.
+
+Specifying a value for interpolation which doesn't exist now raises a
+``MissingInterpolationOption`` error (instead of merely being ignored).
+
+The ``writein`` method has been removed.
+
+The comments attribute is now a list (``inline_comments`` equates to the old
+comments attribute).
+
+ConfigObj 3
+-----------
+
+ConfigObj 3 is now deprecated in favour of ConfigObj 4. I can fix bugs in
+ConfigObj 3 if needed, though.
+
+For anyone who still needs it, you can download it here: `ConfigObj 3.3.1`_
+
+You can read the old docs at : `ConfigObj 3 Docs`_
+
+.. _ConfigObj 3.3.1: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj3.zip
+.. _ConfigObj 3 Docs: http://www.voidspace.org.uk/python/configobj3.html
+
+
+CREDITS
+=======
+
+ConfigObj 4 is written by (and copyright) `Michael Foord`_ and 
+`Nicola Larosa`_.
+
+Particularly thanks to Nicola Larosa for help on the config file spec, the
+validation system and the doctests.
+
+*validate.py* was originally written by Michael Foord and `Mark Andrews`_.
+
+Thanks to others for input and bugfixes.
+
+
+LICENSE
+=======
+
+ConfigObj, and related files, are licensed under the BSD license. This is a
+very unrestrictive license, but it comes with the usual disclaimer. This is
+free software: test it, break it, just don't blame us if it eats your data !
+Of course if it does, let us know and we'll fix the problem so it doesn't
+happen to anyone else {sm;:-)}. ::
+
+    Copyright (c) 2004 - 2006, Michael Foord & Nicola Larosa
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+
+
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+
+        * Neither the name of Michael Foord nor Nicola Larosa
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+You should also be able to find a copy of this license at : `BSD License`_
+
+.. _BSD License: http://www.voidspace.org.uk/python/license.shtml
+
+
+TODO
+====
+
+Fix any bugs (and resolvable issues).
+
+Do an example for the 'walk' which removes uniform indentation in multiline
+values.
+
+When initialising a section from a ConfigObj *or* an ``OrderedDictionary``
+we could preserve ordering.
+
+Add an *odict* method which returns an ``OrderedDictionary``.
+
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the **ConfigObj**
+    `Mailing List`_.
+
+You can't have a keyword with the same name as a section (in the same section).
+They are both dictionary keys, so they would overlap.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section,
+next it checks the 'DEFAULT' section of the parent section, last it checks the
+'DEFAULT' section of the main section.
+
+Logically a 'DEFAULT' section should apply to all subsections of the *same
+parent*: this means that checking the 'DEFAULT' sub-section in the *current
+section* is not necessarily logical ?
+
+Does it matter that we don't support the ':' divider, which is supported by
+``ConfigParser`` ?
+
+String interpolation and validation don't play well together. See 
+`Validation and Interpolation`_.
+
+Validation is no longer done on the 'DEFAULT' section (on the root level). This
+allows interpolation from within your configspec - but also prevents you
+validating the 'DEFAULT' section.
+
+
+CHANGELOG
+=========
+
+This is an abbreviated changelog showing the major releases up to version 4.
+From version 4 it lists all releases and changes. *More* data on individual
+changes may be found in the source code.
+
+2006/02/16 - Version 4.2.0
+--------------------------
+
+Removed ``BOM_UTF8`` from ``__all__``.
+
+The ``BOM`` attribute has become a boolean. (Defaults to ``False``.) It is
+*only* ``True`` for the ``UTF16`` encoding.
+
+File like objects no longer need a ``seek`` attribute.
+
+Full unicode support added. New options/attributes ``encoding``,
+``default_encoding``.
+
+ConfigObj no longer keeps a reference to file like objects. Instead the
+``write`` method takes a file like object as an optional argument. (Which
+will be used in preference of the ``filename`` attribute if that exists as
+well.)
+
+utf16 files decoded to unicode.
+
+If ``BOM`` is ``True``, but no encoding specified, then the utf8 BOM is
+written out at the start of the file. (It will normally only be ``True`` if
+the utf8 BOM was found when the file was read.)
+
+File paths are *not* converted to absolute paths, relative paths will
+remain relative as the ``filename`` attribute.
+
+Fixed bug where ``final_comment`` wasn't returned if ``write`` is returning
+a list of lines.
+
+Deprecated ``istrue``, replaced it with ``as_bool``.
+
+Added ``as_int`` and ``as_float``.
+
+2005/12/14 - Version 4.1.0
+--------------------------
+
+Added ``merge``, a recursive update.
+
+Added ``preserve_errors`` to ``validate`` and the ``flatten_errors``
+example function.
+
+Thanks to Matthew Brett for suggestions and helping me iron out bugs.
+    
+Fixed bug where a config file is *all* comment, the comment will now be
+``initial_comment`` rather than ``final_comment``.
+
+Validation no longer done on the 'DEFAULT' section (only in the root level).
+This allows interpolation in configspecs.
+
+Also use the new list syntax in validate_ 0.2.1. (For configspecs).
+
+2005/12/02 - Version 4.0.2
+--------------------------
+
+Fixed bug in ``create_empty``. Thanks to Paul Jimenez for the report.
+
+
+2005/11/05 - Version 4.0.1
+--------------------------
+
+Fixed bug in ``Section.walk`` when transforming names as well as values.
+
+Added the ``istrue`` method. (Fetches the boolean equivalent of a string
+value).
+
+Fixed ``list_values=False`` - they are now only quoted/unquoted if they
+are multiline values.
+
+List values are written as ``item, item`` rather than ``item,item``.
+
+
+2005/10/17 - Version 4.0.0
+--------------------------
+
+**ConfigObj 4.0.0 Final**
+
+Fixed bug in ``setdefault``. When creating a new section with setdefault the
+reference returned would be to the dictionary passed in *not* to the new 
+section. Bug fixed and behaviour documented.
+
+Obscure typo/bug fixed in ``write``. Wouldn't have affected anyone though.
+
+2005/09/09 - Version 4.0.0 beta 5
+---------------------------------
+
+Removed ``PositionError``.
+
+Allowed quotes around keys as documented.
+
+Fixed bug with commas in comments. (matched as a list value)
+
+2005/09/07 - Version 4.0.0 beta 4
+---------------------------------
+
+Fixed bug in ``__delitem__``. Deleting an item no longer deletes the 
+``inline_comments`` attribute.
+
+Fixed bug in initialising ConfigObj from a ConfigObj.
+
+Changed the mailing list address.
+
+2005/08/28 - Version 4.0.0 beta 3
+---------------------------------
+
+Interpolation is switched off before writing out files.
+
+Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+"Gustavo Niemeyer" <gustavo@niemeyer.net>)
+
+Moved the doctests from the ``__init__`` method to a separate function.
+(For the sake of IDE calltips).
+
+2005/08/25 - Version 4.0.0 beta 2
+---------------------------------
+
+Amendments to *validate.py*.
+
+Official release.
+
+2005/08/21 - Version 4.0.0 beta 1
+---------------------------------
+
+Reads nested subsections to any depth.
+
+Multiline values.
+
+Simplified options and methods.
+
+New list syntax.
+
+Faster, smaller, and better parser.
+
+Validation greatly improved. Includes:
+
+    * type conversion
+    * default values
+    * repeated sections
+
+Improved error handling.
+
+Plus lots of other improvements {sm;:grin:}.
+
+2004/05/24 - Version 3.0.0
+--------------------------
+
+Several incompatible changes: another major overhaul and change. (Lots of
+improvements though).
+
+Added support for standard config files with sections. This has an entirely
+new interface: each section is a dictionary of values.
+
+Changed the update method to be called writein: update clashes with a dict
+method.
+
+Made various attributes keyword arguments, added several.
+
+Configspecs and orderlists have changed a great deal.
+
+Removed support for adding dictionaries: use update instead.
+
+Now subclasses a new class called caselessDict. This should add various
+dictionary methods that could have caused errors before.
+
+It also preserves the original casing of keywords when writing them back out.
+
+Comments are also saved using a ``caselessDict``.
+
+Using a non-string key will now raise a ``TypeError`` rather than converting 
+the key.
+
+Added an exceptions keyword for *much* better handling of errors.
+
+Made ``creatempty=False`` the default.
+
+Now checks indict *and* any keyword args. Keyword args take precedence over
+indict.
+
+``' ', ':', '=', ','`` and ``'\t'`` are now all valid dividers where the 
+keyword is unquoted.
+
+ConfigObj now does no type checking against configspec when you set items.
+
+delete and add methods removed (they were unnecessary).
+
+Docs rewritten to include all this gumph and more; actually ConfigObj is
+*really* easy to use.
+
+Support for stdout was removed.
+
+A few new methods added.
+
+Charmap is now incorporated into ConfigObj.
+
+2004/03/14 - Version 2.0.0 beta
+-------------------------------
+
+Re-written it to subclass dict. My first forays into inheritance and operator
+overloading.
+
+The config object now behaves like a dictionary.
+
+I've completely broken the interface, but I don't think anyone was really
+ using it anyway.
+
+This new version is much more 'classy' {sm;:wink:}
+
+It will also read straight from/to a filename and completely parse a config
+file without you *having* to supply a config spec.
+
+Uses listparse, so can handle nested list items as values.
+
+No longer has getval and setval methods: use normal dictionary methods, or add
+and delete.
+
+2004/01/29 - Version 1.0.5
+--------------------------
+
+Version 1.0.5 has a couple of bugfixes as well as a couple of useful additions
+over previous versions.
+
+Since 1.0.0 the buildconfig function has been moved into this distribution,
+and the methods reset, verify, getval and setval have been added.
+
+A couple of bugs have been fixed.
+
+Origins
+-------
+
+ConfigObj originated in a set of functions for reading config files in the
+atlantibots_ project. The original functions were written by Rob McNeur...
+
+.. _atlantibots: http://www.voidspace.org.uk/atlantibots
+
+----------
+
+Footnotes
+=========
+
+.. [#] 253 of them, at the time of writing.
+
+.. [#] And if you discover any bugs, let us know. We'll fix them quickly.
+
+.. [#] If you specify a filename that doesn't exist, ConfigObj will assume you
+    are creating a new one. See the *create_empty* and *file_error* options_.
+
+.. [#] They can be byte strings ('ordinary' strings) or Unicode.
+
+.. [#] This is a change in ConfigObj 4.2.0. Note that ConfigObj doesn't call
+    the seek method of any file like object you pass in. You may want to call
+    ``file_object.seek(0)`` yourself, first.
+
+.. [#] Except we don't support the RFC822 style line continuations, nor ':' as
+    a divider.
+
+.. [#] For a file object that will depend what mode it was opened with. You
+    can read *and* write to a ``StringIO`` instance, but not always to a
+    ``cStringIO`` instance.
+
+.. [#] A side effect of this is that it enables you to copy a ConfigObj :
+
+    .. raw:: html
+    
+        {+coloring}
+    
+        # only copies members
+        # not attributes/comments
+        config2 = ConfigObj(config1)
+    
+        {-coloring}
+
+    The order of values and sections will not be preserved, though.
+
+.. [#] The exception is if it detects a ``UTF16`` encoded file which it
+    must decode before parsing.
+     
+.. [#] Other than lists of strings.
+
+.. [#] The method signature in the API docs will show that this method takes
+    two arguments. The second is the section to be written. This is because the
+    ``write`` method is called recursively.
+
+.. [#] The dict method doesn't actually use the deepcopy mechanism. This means
+    if you add nested lists (etc) to your ConfigObj, then the dictionary
+    returned by dict may contain some references. For all *normal* ConfigObjs
+    it will return a deepcopy.
+
+.. [#] Passing ``(section, key)`` rather than ``(value, key)`` allows you to
+    change the value by setting ``section[key] = newval``. It also gives you
+    access to the *rename* method of the section.
+
+.. [#] Minimum required version of *validate.py* 0.2.0 .
+
+.. [#] There's nothing to stop you decoding the whole config file to Unicode
+    *first*.
+
+.. [#] It also makes ConfigObj a lot simpler to *use*.
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://sourceforge.net/donate/index.php?group_id=123265">
+            <img src="http://images.sourceforge.net/images/project-support.jpg" width="88" height="32" border="0" alt="Support This Project" /> 
+        </a>
+        <a href="http://sourceforge.net">
+            <img src="http://sourceforge.net/sflogo.php?group_id=123265&amp;type=1" width="88" height="31" border="0" alt="SourceForge.net Logo" />
+        </a>
+        <br />
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a><br /><br />
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+    </div>
+
+.. _listquote: http://www.voidspace.org.uk/python/modules.shtml#listquote
+.. _Michael Foord: http://www.voidspace.org.uk/index2.shtml
+.. _Nicola Larosa: http://www.teknico.net
+.. _Mark Andrews: http://www.la-la.com

=== added file 'bzrlib/util/configobj/docs/validate.txt'
--- bzrlib/util/configobj/docs/validate.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/validate.txt	2006-02-21 02:26:30 +0000
@@ -0,0 +1,625 @@
+===================================
+ Validation Schema with validate.py
+===================================
+
+--------------------------
+ Using the Validator class
+--------------------------
+
+
+:Authors: `Michael Foord`_, `Nicola Larosa`_, `Mark Andrews`_
+:Version: Validate 0.2.1
+:Date: 2005/12/16
+:Homepage: `Validate Homepage`_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+.. _Michael Foord: fuzzyman@voidspace.org.uk
+.. _Nicola Larosa: nico@teknico.net
+.. _Mark Andrews: mark@la-la.com
+.. _This Document:
+.. _Validate Homepage: http://www.voidspace.org.uk/python/validate.html
+.. _BSD License: http://www.voidspace.org.uk/python/license.shtml
+
+
+.. contents:: Validate Manual
+
+Introduction
+============
+
+Validation is used to check that supplied values conform to a specification.
+
+The value can be supplied as a string, e.g. from a config file. In this case
+the check will also *convert* the value to the required type. This allows you
+to add validation as a transparent layer to access data stored as strings. The
+validation checks that the data is correct *and* converts it to the expected
+type.
+
+Checks are also strings, and are easy to write. One generic system can be used
+to validate information from different sources, via a single consistent
+mechanism.
+
+Checks look like function calls, and map to function calls. They can include
+parameters and keyword arguments. These arguments are passed to the relevant
+function by the ``Validator`` instance, along with the value being checked.
+
+The syntax for checks also allows for specifying a default value. This default
+value can be ``None``, no matter what the type of the check. This can be used
+to indicate that a value was missing, and so holds no useful value.
+
+Functions either return a new value, or raise an exception. See `Validator
+Exceptions`_ for the low down on the exception classes that ``validate.py``
+defines.
+
+Some standard functions are provided, for basic data types; these come built
+into every validator. Additional checks are easy to write: they can be provided
+when the ``Validator`` is instantiated, or added afterwards.
+
+Validate was primarily written to support ConfigObj_, but is designed to be 
+applicable to many other situations.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. _ConfigObj: http://www.voidspace.org.uk/python/configobj.html
+
+Downloading
+===========
+
+The current version is **0.2.1**, dated 16th December 2005. 
+
+You can get obtain validate in the following ways :
+
+Files
+-----
+
+* validate.py_ from Voidspace
+
+* configobj.zip from Voidspace - See the homepage of ConfigObj_ for the latest 
+  version and download links.
+
+    This contains validate.py and `this document`_. (As well as ConfigObj_ and 
+    the ConfigObj documentation).
+    
+* The latest development version can be obtained from the `Subversion Repository`_.
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated 
+automatically by EpyDoc_.
+
+* You can view `this document`_ online as the `Validate Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+Validate_ is also part of the Pythonutils_ set of modules. This contains 
+various other useful helper modules, and is required by many of the `Voidspace 
+Python Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.1.0.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4/
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+.. _validate: http://www.voidspace.org.uk/python/validate.html
+
+
+The standard functions
+======================
+
+The standard functions come built-in to every ``Validator`` instance. They work
+with the following basic data types :
+
+* integer
+* float
+* boolean
+* string
+* ip_addr
+
+plus lists of these datatypes.
+
+Adding additional checks is done through coding simple functions.
+
+The full set of standard checks are :
+
+:'integer': matches integer values (including negative). Takes optional 'min'
+            and 'max' arguments : ::
+
+                integer()
+                integer(3, 9)    # any value from 3 to 9
+                integer(min=0) # any positive value
+                integer(max=9)
+
+:'float': matches float values
+          Has the same parameters as the integer check.
+
+:'bool': matches boolean values: ``True`` or ``False``.
+         Acceptable string values for True are : ::
+
+             true, on, yes, 1
+
+         Acceptable string values for False are : ::
+
+             false, off, no, 0
+
+         Any other value raises an error.
+
+:'string': matches any string. Takes optional keyword args 'min' and 'max' to
+           specify min and max length of string.
+
+:'ip_addr': matches an Internet Protocol address, v.4, represented by a
+            dotted-quad string, i.e. '1.2.3.4'.
+
+:'list': matches any list. Takes optional keyword args 'min', and 'max' to
+         specify min and max sizes of the list.
+
+:'int_list': Matches a list of integers. Takes the same arguments as list.
+
+:'float_list': Matches a list of floats. Takes the same arguments as list.
+
+:'bool_list': Matches a list of boolean values. Takes the same arguments as
+              list.
+
+:'string_list': Matches a list of strings. Takes the same arguments as list.
+
+:'ip_addr_list': Matches a list of IP addresses. Takes the same arguments as
+                 list.
+
+:'mixed_list': Matches a list with different types in specific positions.
+               List size must match the number of arguments.
+
+               Each position can be one of : ::
+
+                   int, str, bool, float, ip_addr
+
+               So to specify a list with two strings followed by two integers,
+               you write the check as : ::
+
+                   mixed_list(str, str, int, int)
+
+:'pass': matches everything: it never fails and the value is unchanged. It is
+         also the default if no check is specified.
+
+:'option': matches any from a list of options.
+           You specify this test with : ::
+
+               option('option 1', 'option 2', 'option 3')
+
+The following code will work without you having to specifically add the
+functions yourself.
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    newval1 = vtor.check('integer', value1)
+    newval2 = vtor.check('bool', value2)
+    # etc ...
+
+    {-coloring}
+
+.. note::
+
+    Of course, if these checks fail they raise exceptions. So you should wrap
+    them in ``try...except`` blocks. Better still,  use ConfigObj for a higher
+    level interface.
+
+
+Using Validator
+===============
+
+Using ``Validator`` is very easy. It has one public attribute and one public
+method.
+
+Shown below are the different steps in using ``Validator``.
+
+The only additional thing you need to know, is about `Writing check
+functions`_.
+
+Instantiate
+-----------
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    vtor = Validator()
+
+    {-coloring}
+
+or even :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    fdict = {
+        'check_name1': function1,
+        'check_name2': function2,
+        'check_name3': function3,
+    }
+    #
+    vtor = Validator(fdict)
+
+    {-coloring}
+
+The second method adds a set of your functions as soon as your validator is
+created. They are stored in the ``vtor.functions`` dictionary. The 'key' you
+give them in this dictionary is the name you use in your checks (not the
+original function name).
+
+Dictionary keys/functions you pass in can override the built-in ones if you
+want.
+
+Adding functions
+----------------
+
+The code shown above, for adding functions on instantiation, has exactly the
+same effect as the following code :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    vtor.functions['check_name1'] = function1
+    vtor.functions['check_name2'] = function2
+    vtor.functions['check_name3'] = function3
+
+    {-coloring}
+
+``vtor.functions``is just a dictionary that maps names to functions, so we
+could also have called ``vtor.functions.update(fdict)``.
+
+Writing the check
+-----------------
+
+As we've heard, the checks map to the names in the ``functions`` dictionary.
+You've got a full list of `The standard functions`_ and the arguments they
+take.
+
+If you're using ``Validator`` from ConfigObj, then your checks will look like
+: ::
+
+    keyword = int_list(max=6)
+
+but the check part will be identical .
+
+The check method
+----------------
+
+If you're not using ``Validator`` from ConfigObj, then you'll need to call the
+``check`` method yourself.
+
+If the check fails then it will raise an exception, so you'll want to trap
+that. Here's the basic example :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator, ValidateError
+    #
+    vtor = Validator()
+    check = "integer(0, 9)"
+    value = 3
+    try:
+        newvalue = vtor.check(check, value)
+    except ValidateError:
+        print 'Check Failed.'
+    else:
+        print 'Check passed.'
+
+    {-coloring}
+
+.. caution::
+
+    Although the value can be a string, if it represents a list it should
+    already have been turned into a list of strings.
+
+
+Default Values
+~~~~~~~~~~~~~~
+
+Some values may not be available, and you may want to be able to specify a
+default as part of the check.
+
+You do this by passing the keyword ``missing=True`` to the ``check`` method, as
+well as a ``default=value`` in the check. (Constructing these checks is done
+automatically by ConfigObj: you only need to know about the ``default=value``
+part) :
+
+.. raw:: html
+
+    {+coloring}
+
+    check1 = 'integer(default=50)'
+    check2 = 'option("val 1", "val 2", "val 3", default="val 1")'
+
+    assert vtor.check('', check1, missing=True) == 50
+    assert vtor.check('', check2, missing=True) == "val 1"
+
+    {-coloring}
+
+If you pass in ``missing=True`` to the check method, then the actual value is
+ignored. If no default is specified in the check, a ``ValidateMissingValue``
+exception is raised. If a default is specified then that is passed to the
+check instead.
+
+If the check has ``default=None`` (case sensitive) then ``vtor.check`` will
+*always* return ``None`` (the object). This makes it easy to tell your program
+that this check contains no useful value when missing, i.e. the value is
+optional, and may be omitted without harm.
+
+
+List Values
+~~~~~~~~~~~
+
+It's possible that you would like your default value to be a list. It's even
+possible that you will write your own check functions - and would like to pass
+them keyword arguments as lists from within the check.
+
+To avoid confusing syntax with commas and quotes you use a list constructor to
+specify that keyword arguments are lists. This includes the ``default`` value.
+This makes checks look something like : ::
+
+    checkname(default=list('val1', 'val2', 'val3'))
+
+Validator Exceptions
+====================
+
+.. note::
+
+    If you only use Validator through ConfigObj, it traps these Exceptions for
+    you. You will still need to know about them for writing your own check
+    functions.
+
+``vtor.check`` indicates that the check has failed by raising an exception.
+The appropriate error should be raised in the check function.
+
+The base error class is ``ValidateError``. All errors (except for ``VdtParamError``) 
+raised are sub-classes of this.
+
+If an unrecognised check is specified then ``VdtUnknownCheckError`` is
+raised.
+
+There are also ``VdtTypeError`` and ``VdtValueError``.
+
+If incorrect parameters are passed to a check function then it will (or should)
+raise ``VdtParamError``. As this indicates *programmer* error, rather than an error
+in the value, it is a subclass of ``SyntaxError`` instead of ``ValidateError``. 
+
+.. note::
+
+    This means it *won't* be caught by ConfigObj - but propagated instead.
+
+If the value supplied is the wrong type, then the check should raise
+``VdtTypeError``. e.g. the check requires the value to be an integer (or
+representation of an integer) and something else was supplied.
+
+If the value supplied is the right type, but an unacceptable value, then the
+check should raise ``VdtValueError``. e.g. the check requires the value to
+be an integer (or representation of an integer) less than ten and a higher
+value was supplied.
+
+Both ``VdtTypeError`` and ``VdtValueError`` are initialised with the
+incorrect value. In other words you raise them like this :
+
+.. raw:: html
+
+    {+coloring}
+
+    raise VdtTypeError(value)
+    #
+    raise VdtValueError(value)
+
+    {-coloring}
+
+``VdtValueError`` has the following subclasses, which should be raised if
+they are more appropriate.
+
+* ``VdtValueTooSmallError``
+* ``VdtValueTooBigError``
+* ``VdtValueTooShortError``
+* ``VdtValueTooLongError``
+
+Writing check functions
+=======================
+
+Writing check functions is easy.
+
+The check function will receive the value as its first argument, followed by
+any other parameters and keyword arguments.
+
+If the check fails, it should raise a ``VdtTypeError`` or a
+``VdtValueError`` (or an appropriate subclass).
+
+All parameters and keyword arguments are *always* passed as strings. (Parsed
+from the check string).
+
+The value might be a string (or list of strings) and need
+converting to the right type - alternatively it might already be a list of 
+integers. Our function needs to be able to handle either.
+
+If the check passes then it should return the value (possibly converted to the
+right type).
+
+And that's it !
+
+Example
+-------
+
+Here is an example function that requires a list of integers. Each integer
+must be between 0 and 99.
+
+It takes a single argument specifying the length of the list. (Which allows us
+to use the same check in more than one place). If the length can't be converted
+to an integer then we need to raise ``VdtParamError``.
+
+Next we check that the value is a list. Anything else should raise a
+``VdtTypeError``. The list should also have 'length' entries. If the list
+has more or less entries then we will need to raise a
+``VdtValueTooShortError`` or a ``VdtValueTooLongError``.
+
+Then we need to check every entry in the list. Each entry should be an integer
+between 0 and 99, or a string representation of an integer between 0 and 99.
+Any other type is a ``VdtTypeError``, any other value is a
+``VdtValueError`` (either too big, or too small).
+
+.. raw:: html
+
+    {+coloring}
+
+    def special_list(value, length):
+        """
+        Check that the supplied value is a list of integers,
+        with 'length' entries, and each entry between 0 and 99.
+        """
+        # length is supplied as a string
+        # we need to convert it to an integer
+        try:
+            length = int(length)
+        except ValueError:
+            raise VdtParamError('length', length)
+        #
+        # Check the supplied value is a list
+        if not isinstance(value, list):
+            raise VdtTypeError(value)
+        #
+        # check the length of the list is correct
+        if len(value) > length:
+            raise VdtValueTooLongError(value)
+        elif len(value) < length:
+            raise VdtValueTooShortError(value)
+        #
+        # Next, check every member in the list
+        # converting strings as necessary
+        out = []
+        for entry in value:
+            if not isinstance(entry, (str, unicode, int)):
+                # a value in the list
+                # is neither an integer nor a string
+                raise VdtTypeError(value)
+            elif isinstance(entry, (str, unicode)):
+                if not entry.isdigit():
+                    raise VdtTypeError(value)
+                else:
+                    entry = int(entry)
+            if entry < 0:
+                raise VdtValueTooSmallError(value)
+            elif entry > 99:
+                raise VdtValueTooBigError(value)
+            out.append(entry)
+        #
+        # if we got this far, all is well
+        # return the new list
+        return out
+
+    {-coloring}
+
+If you are only using validate from ConfigObj then the error type (*TooBig*, 
+*TooSmall*, etc) is lost - so you may only want to raise ``VdtValueError``.
+
+.. caution::
+
+    If your function raises an exception that isn't a subclass of 
+    ``ValidateError``, then ConfigObj won't trap it. This means validation will 
+    fail.
+    
+    This is why our function starts by checking the type of the value. If we 
+    are passed the wrong type (e.g. an integer rather than a list) we get a 
+    ``VdtTypeError`` rather than bombing out when we try to iterate over 
+    the value.
+
+If you are using validate in another circumstance you may want to create your 
+own subclasses of ``ValidateError``, that convey more specific information.
+
+TODO
+====
+
+* A regex check function ?
+* A timestamp check function ? (Using the ``parse`` function from ``DateUtil``).
+* Allow triple quotes ? (getting a bit heavy for a regex)
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    `Mailing List`_.
+
+If we could pull tuples out of arguments, it would be easier
+to specify arguments for 'mixed_lists'.
+
+CHANGELOG
+=========
+
+2005/12/16      Version 0.2.1
+-----------------------------
+
+Fixed bug so we can handle keyword argument values with commas.
+
+We now use a list constructor for passing list values to keyword arguments
+(including ``default``) : ::
+
+    default=list("val", "val", "val")
+
+Added the ``_test`` test. {sm;:-)}
+
+Moved a function call outside a try...except block.
+
+2005/08/18      Version 0.2.0
+-----------------------------
+
+Updated by `Michael Foord`_ and `Nicola Larosa`_
+
+Does type conversion as well.
+
+2005/02/01      Version 0.1.0
+-----------------------------
+
+Initial version developed by `Michael Foord`_
+and `Mark Andrews`_
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a>
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+    </div>

=== added file 'bzrlib/util/configobj/validate.py'
--- bzrlib/util/configobj/validate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/validate.py	2006-02-21 02:26:30 +0000
@@ -0,0 +1,1445 @@
+# validate.py
+# A Validator object
+# Copyright (C) 2005 Michael Foord, Mark Andrews, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         mark AT la-la DOT com
+#         nico AT tekNico DOT net
+
+# This software is licensed under the terms of the BSD license.
+# http://www.voidspace.org.uk/python/license.shtml
+# Basically you're free to copy, modify, distribute and relicense it,
+# So long as you keep a copy of the license with it.
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+"""
+    The Validator object is used to check that supplied values 
+    conform to a specification.
+    
+    The value can be supplied as a string - e.g. from a config file.
+    In this case the check will also *convert* the value to
+    the required type. This allows you to add validation
+    as a transparent layer to access data stored as strings.
+    The validation checks that the data is correct *and*
+    converts it to the expected type.
+    
+    Some standard checks are provided for basic data types.
+    Additional checks are easy to write. They can be
+    provided when the ``Validator`` is instantiated or
+    added afterwards.
+    
+    The standard functions work with the following basic data types :
+    
+    * integers
+    * floats
+    * booleans
+    * strings
+    * ip_addr
+    
+    plus lists of these datatypes
+    
+    Adding additional checks is done through coding simple functions.
+    
+    The full set of standard checks are : 
+    
+    * 'integer': matches integer values (including negative)
+                 Takes optional 'min' and 'max' arguments : ::
+    
+                   integer()
+                   integer(3, 9)  # any value from 3 to 9
+                   integer(min=0) # any positive value
+                   integer(max=9)
+    
+    * 'float': matches float values
+               Has the same parameters as the integer check.
+    
+    * 'boolean': matches boolean values - ``True`` or ``False``
+                 Acceptable string values for True are :
+                   true, on, yes, 1
+                 Acceptable string values for False are :
+                   false, off, no, 0
+    
+                 Any other value raises an error.
+    
+    * 'ip_addr': matches an Internet Protocol address, v.4, represented
+                 by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    * 'string': matches any string.
+                Takes optional keyword args 'min' and 'max'
+                to specify min and max lengths of the string.
+    
+    * 'list': matches any list.
+              Takes optional keyword args 'min', and 'max' to specify min and
+              max sizes of the list.
+    
+    * 'int_list': Matches a list of integers.
+                  Takes the same arguments as list.
+    
+    * 'float_list': Matches a list of floats.
+                    Takes the same arguments as list.
+    
+    * 'bool_list': Matches a list of boolean values.
+                   Takes the same arguments as list.
+    
+    * 'ip_addr_list': Matches a list of IP addresses.
+                     Takes the same arguments as list.
+    
+    * 'string_list': Matches a list of strings.
+                     Takes the same arguments as list.
+    
+    * 'mixed_list': Matches a list with different types in 
+                    specific positions. List size must match
+                    the number of arguments.
+    
+                    Each position can be one of :
+                    'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+                    So to specify a list with two strings followed
+                    by two integers, you write the check as : ::
+    
+                      mixed_list('string', 'string', 'integer', 'integer')
+    
+    * 'pass': This check matches everything ! It never fails
+              and the value is unchanged.
+    
+              It is also the default if no check is specified.
+    
+    * 'option': This check matches any from a list of options.
+                You specify this check with : ::
+    
+                  option('option 1', 'option 2', 'option 3')
+    
+    You can supply a default value (returned if no value is supplied)
+    using the default keyword argument.
+    
+    You specify a list argument for default using a list constructor syntax in
+    the check : ::
+    
+        checkname(arg1, arg2, default=list('val 1', 'val 2', 'val 3'))
+    
+    A badly formatted set of arguments will raise a ``VdtParamError``.
+"""
+
+__docformat__ = "restructuredtext en"
+
+__version__ = '0.2.1'
+
+__revision__ = '$Id: validate.py 123 2005-09-08 08:54:28Z fuzzyman $'
+
+__all__ = (
+    '__version__',
+    'dottedQuadToNum',
+    'numToDottedQuad',
+    'ValidateError',
+    'VdtUnknownCheckError',
+    'VdtParamError',
+    'VdtTypeError',
+    'VdtValueError',
+    'VdtValueTooSmallError',
+    'VdtValueTooBigError',
+    'VdtValueTooShortError',
+    'VdtValueTooLongError',
+    'VdtMissingValue',
+    'Validator',
+    'is_integer',
+    'is_float',
+    'is_bool',
+    'is_list',
+    'is_ip_addr',
+    'is_string',
+    'is_int_list',
+    'is_bool_list',
+    'is_float_list',
+    'is_string_list',
+    'is_ip_addr_list',
+    'is_mixed_list',
+    'is_option',
+    '__docformat__',
+)
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import re
+StringTypes = (str, unicode)
+
+
+_list_arg = re.compile(r'''
+    (?:
+        ([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*list\(
+            (
+                (?:
+                    \s*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s\)][^,\)]*?)  # unquoted
+                    )
+                    \s*,\s*
+                )*
+                (?:
+                    (?:".*?")|              # double quotes
+                    (?:'.*?')|              # single quotes
+                    (?:[^'",\s\)][^,\)]*?)  # unquoted
+                )?                          # last one
+            )
+        \)
+    )
+''', re.VERBOSE)    # two groups
+
+_list_members = re.compile(r'''
+    (
+        (?:".*?")|              # double quotes
+        (?:'.*?')|              # single quotes
+        (?:[^'",\s=][^,=]*?)       # unquoted
+    )
+    (?:
+    (?:\s*,\s*)|(?:\s*$)            # comma
+    )
+''', re.VERBOSE)    # one group
+
+_paramstring = r'''
+    (?:
+        (
+            (?:
+                [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*list\(
+                    (?:
+                        \s*
+                        (?:
+                            (?:".*?")|              # double quotes
+                            (?:'.*?')|              # single quotes
+                            (?:[^'",\s\)][^,\)]*?)       # unquoted
+                        )
+                        \s*,\s*
+                    )*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s\)][^,\)]*?)       # unquoted
+                    )?                              # last one
+                \)
+            )|
+            (?:
+                (?:".*?")|              # double quotes
+                (?:'.*?')|              # single quotes
+                (?:[^'",\s=][^,=]*?)|       # unquoted
+                (?:                         # keyword argument
+                    [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s=][^,=]*?)       # unquoted
+                    )
+                )
+            )
+        )
+        (?:
+            (?:\s*,\s*)|(?:\s*$)            # comma
+        )
+    )
+    '''
+
+_matchstring = '^%s*' % _paramstring
+
+# Python pre 2.2.1 doesn't have bool
+try:
+    bool
+except NameError:
+    def bool(val):
+        """Simple boolean equivalent function. """
+        if val:
+            return 1
+        else:
+            return 0
+
+def dottedQuadToNum(ip):
+    """
+    Convert decimal dotted quad string to long integer
+    
+    >>> dottedQuadToNum('1 ')
+    1L
+    >>> dottedQuadToNum(' 1.2')
+    16777218L
+    >>> dottedQuadToNum(' 1.2.3 ')
+    16908291L
+    >>> dottedQuadToNum('1.2.3.4')
+    16909060L
+    >>> dottedQuadToNum('1.2.3. 4')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 1.2.3. 4
+    >>> dottedQuadToNum('255.255.255.255')
+    4294967295L
+    >>> dottedQuadToNum('255.255.255.256')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 255.255.255.256
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    try:
+        return struct.unpack('!L',
+            socket.inet_aton(ip.strip()))[0]
+    except socket.error:
+        # bug in inet_aton, corrected in Python 2.3
+        if ip.strip() == '255.255.255.255':
+            return 0xFFFFFFFFL
+        else:
+            raise ValueError('Not a good dotted-quad IP: %s' % ip)
+    return
+
+def numToDottedQuad(num):
+    """
+    Convert long int to dotted quad string
+    
+    >>> numToDottedQuad(-1L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: -1
+    >>> numToDottedQuad(1L)
+    '0.0.0.1'
+    >>> numToDottedQuad(16777218L)
+    '1.0.0.2'
+    >>> numToDottedQuad(16908291L)
+    '1.2.0.3'
+    >>> numToDottedQuad(16909060L)
+    '1.2.3.4'
+    >>> numToDottedQuad(4294967295L)
+    '255.255.255.255'
+    >>> numToDottedQuad(4294967296L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: 4294967296
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    # no need to intercept here, 4294967295L is fine
+    try:
+        return socket.inet_ntoa(
+            struct.pack('!L', long(num)))
+    except (socket.error, struct.error, OverflowError):
+        raise ValueError('Not a good numeric IP: %s' % num)
+
+class ValidateError(Exception):
+    """
+    This error indicates that the check failed.
+    It can be the base class for more specific errors.
+    
+    Any check function that fails ought to raise this error.
+    (or a subclass)
+    
+    >>> raise ValidateError
+    Traceback (most recent call last):
+    ValidateError
+    """
+
+class VdtMissingValue(ValidateError):
+    """No value was supplied to a check that needed one."""
+
+class VdtUnknownCheckError(ValidateError):
+    """An unknown check function was requested"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtUnknownCheckError('yoda')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        ValidateError.__init__(
+            self,
+            'the check "%s" is unknown.' % value)
+
+class VdtParamError(SyntaxError):
+    """An incorrect parameter was passed"""
+
+    def __init__(self, name, value):
+        """
+        >>> raise VdtParamError('yoda', 'jedi')
+        Traceback (most recent call last):
+        VdtParamError: passed an incorrect value "jedi" for parameter "yoda".
+        """
+        SyntaxError.__init__(
+            self,
+            'passed an incorrect value "%s" for parameter "%s".' % (
+                value, name))
+
+class VdtTypeError(ValidateError):
+    """The value supplied was of the wrong type"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtTypeError('jedi')
+        Traceback (most recent call last):
+        VdtTypeError: the value "jedi" is of the wrong type.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is of the wrong type.' % value)
+
+class VdtValueError(ValidateError):
+    """
+    The value supplied was of the correct type, but was not an allowed value.
+    """
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueError('jedi')
+        Traceback (most recent call last):
+        VdtValueError: the value "jedi" is unacceptable.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is unacceptable.' % value)
+
+class VdtValueTooSmallError(VdtValueError):
+    """The value supplied was of the correct type, but was too small."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooSmallError('0')
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "0" is too small.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too small.' % value)
+
+class VdtValueTooBigError(VdtValueError):
+    """The value supplied was of the correct type, but was too big."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooBigError('1')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "1" is too big.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too big.' % value)
+
+class VdtValueTooShortError(VdtValueError):
+    """The value supplied was of the correct type, but was too short."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooShortError('jed')
+        Traceback (most recent call last):
+        VdtValueTooShortError: the value "jed" is too short.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too short.' % (value,))
+
+class VdtValueTooLongError(VdtValueError):
+    """The value supplied was of the correct type, but was too long."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooLongError('jedie')
+        Traceback (most recent call last):
+        VdtValueTooLongError: the value "jedie" is too long.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too long.' %  (value,))
+
+class Validator(object):
+    """
+        Validator is an object that allows you to register a set of 'checks'.
+        These checks take input and test that it conforms to the check.
+        
+        This can also involve converting the value from a string into
+        the correct datatype.
+        
+        The ``check`` method takes an input string which configures which
+        check is to be used and applies that check to a supplied value.
+        
+        An example input string would be:
+        'int_range(param1, param2)'
+        
+        You would then provide something like:
+        
+        >>> def int_range_check(value, min, max):
+        ...     # turn min and max from strings to integers
+        ...     min = int(min)
+        ...     max = int(max)
+        ...     # check that value is of the correct type.
+        ...     # possible valid inputs are integers or strings
+        ...     # that represent integers
+        ...     if not isinstance(value, (int, long, StringTypes)):
+        ...         raise VdtTypeError(value)
+        ...     elif isinstance(value, StringTypes):
+        ...         # if we are given a string
+        ...         # attempt to convert to an integer
+        ...         try:
+        ...             value = int(value)
+        ...         except ValueError:
+        ...             raise VdtValueError(value)
+        ...     # check the value is between our constraints
+        ...     if not min <= value:
+        ...          raise VdtValueTooSmallError(value)
+        ...     if not value <= max:
+        ...          raise VdtValueTooBigError(value)
+        ...     return value
+        
+        >>> fdict = {'int_range': int_range_check}
+        >>> vtr1 = Validator(fdict)
+        >>> vtr1.check('int_range(20, 40)', '30')
+        30
+        >>> vtr1.check('int_range(20, 40)', '60')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "60" is too big.
+        
+        New functions can be added with : ::
+        
+        >>> vtr2 = Validator()       
+        >>> vtr2.functions['int_range'] = int_range_check
+        
+        Or by passing in a dictionary of functions when Validator 
+        is instantiated.
+        
+        Your functions *can* use keyword arguments,
+        but the first argument should always be 'value'.
+        
+        If the function doesn't take additional arguments,
+        the parentheses are optional in the check.
+        It can be written with either of : ::
+        
+            keyword = function_name
+            keyword = function_name()
+        
+        The first program to utilise Validator() was Michael Foord's
+        ConfigObj, an alternative to ConfigParser which supports lists and
+        can validate a config file using a config schema.
+        For more details on using Validator with ConfigObj see:
+        http://www.voidspace.org.uk/python/configobj.html
+    """
+
+    # this regex does the initial parsing of the checks
+    _func_re = re.compile(r'(.+?)\((.*)\)')
+
+    # this regex takes apart keyword arguments
+    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$')
+
+
+    # this regex finds keyword=list(....) type values
+    _list_arg = _list_arg
+
+    # this regex takes individual values out of lists - in one pass
+    _list_members = _list_members
+
+    # These regexes check a set of arguments for validity
+    # and then pull the members out
+    _paramfinder = re.compile(_paramstring, re.VERBOSE)
+    _matchfinder = re.compile(_matchstring, re.VERBOSE)
+
+
+    def __init__(self, functions=None):
+        """
+        >>> vtri = Validator()
+        """
+        self.functions = {
+            '': self._pass,
+            'integer': is_integer,
+            'float': is_float,
+            'boolean': is_bool,
+            'ip_addr': is_ip_addr,
+            'string': is_string,
+            'list': is_list,
+            'int_list': is_int_list,
+            'float_list': is_float_list,
+            'bool_list': is_bool_list,
+            'ip_addr_list': is_ip_addr_list,
+            'string_list': is_string_list,
+            'mixed_list': is_mixed_list,
+            'pass': self._pass,
+            'option': is_option,
+        }
+        if functions is not None:
+            self.functions.update(functions)
+        # tekNico: for use by ConfigObj
+        self.baseErrorClass = ValidateError
+
+    def check(self, check, value, missing=False):
+        """
+        Usage: check(check, value)
+        
+        Arguments:
+            check: string representing check to apply (including arguments)
+            value: object to be checked
+        Returns value, converted to correct type if necessary
+        
+        If the check fails, raises a ``ValidateError`` subclass.
+        
+        >>> vtor.check('yoda', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        >>> vtor.check('yoda()', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        fun_match = self._func_re.match(check)
+        if fun_match:
+            fun_name = fun_match.group(1)
+            arg_string = fun_match.group(2)
+            arg_match = self._matchfinder.match(arg_string)
+            if arg_match is None:
+                # Bad syntax
+                raise VdtParamError
+            fun_args = []
+            fun_kwargs = {}
+            # pull out args of group 2
+            for arg in self._paramfinder.findall(arg_string):
+                # args may need whitespace removing (before removing quotes)
+                arg = arg.strip()
+                listmatch = self._list_arg.match(arg)
+                if listmatch:
+                    key, val = self._list_handle(listmatch)
+                    fun_kwargs[key] = val
+                    continue
+                keymatch = self._key_arg.match(arg)
+                if keymatch:
+                    val = self._unquote(keymatch.group(2))
+                    fun_kwargs[keymatch.group(1)] = val
+                    continue
+                #
+                fun_args.append(self._unquote(arg))
+        else:
+            # allows for function names without (args)
+            (fun_name, fun_args, fun_kwargs) = (check, (), {})
+        #
+        if missing:
+            try:
+                value = fun_kwargs['default']
+            except KeyError:
+                raise VdtMissingValue
+            if value == 'None':
+                value = None
+        if value is None:
+            return None
+# tekNico: default must be deleted if the value is specified too,
+# otherwise the check function will get a spurious "default" keyword arg
+        try:
+            del fun_kwargs['default']
+        except KeyError:
+            pass
+        try:
+            fun = self.functions[fun_name]
+        except KeyError:
+            raise VdtUnknownCheckError(fun_name)
+        else:
+##            print fun_args
+##            print fun_kwargs
+            return fun(value, *fun_args, **fun_kwargs)
+
+    def _unquote(self, val):
+        """Unquote a value if necessary."""
+        if (len(val) > 2) and (val[0] in ("'", '"')) and (val[0] == val[-1]):
+            val = val[1:-1]
+        return val
+
+    def _list_handle(self, listmatch):
+        """Take apart a ``keyword=list('val, 'val')`` type string."""
+        out = []
+        name = listmatch.group(1)
+        args = listmatch.group(2)
+        for arg in self._list_members.findall(args):
+            out.append(self._unquote(arg))
+        return name, out
+
+    def _pass(self, value):
+        """
+        Dummy check that always passes
+        
+        >>> vtor.check('', 0)
+        0
+        >>> vtor.check('', '0')
+        '0'
+        """
+        return value
+
+
+def _is_num_param(names, values, to_float=False):
+    """
+    Return numbers from inputs or raise VdtParamError.
+    
+    Lets ``None`` pass through.
+    Pass in keyword argument ``to_float=True`` to
+    use float for the conversion rather than int.
+    
+    >>> _is_num_param(('', ''), (0, 1.0))
+    [0, 1]
+    >>> _is_num_param(('', ''), (0, 1.0), to_float=True)
+    [0.0, 1.0]
+    >>> _is_num_param(('a'), ('a'))
+    Traceback (most recent call last):
+    VdtParamError: passed an incorrect value "a" for parameter "a".
+    """
+    fun = to_float and float or int
+    out_params = []
+    for (name, val) in zip(names, values):
+        if val is None:
+            out_params.append(val)
+        elif isinstance(val, (int, long, float, StringTypes)):
+            try:
+                out_params.append(fun(val))
+            except ValueError, e:
+                raise VdtParamError(name, val)
+        else:
+            raise VdtParamError(name, val)
+    return out_params
+
+# built in checks
+# you can override these by setting the appropriate name
+# in Validator.functions
+# note: if the params are specified wrongly in your input string,
+#       you will also raise errors.
+
+def is_integer(value, min=None, max=None):
+    """
+    A check that tests that a given value is an integer (int, or long)
+    and optionally, between bounds. A negative value is accepted, while
+    a float will fail.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    >>> vtor.check('integer', '-1')
+    -1
+    >>> vtor.check('integer', '0')
+    0
+    >>> vtor.check('integer', 9)
+    9
+    >>> vtor.check('integer', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('integer', '2.2')
+    Traceback (most recent call last):
+    VdtTypeError: the value "2.2" is of the wrong type.
+    >>> vtor.check('integer(10)', '20')
+    20
+    >>> vtor.check('integer(max=20)', '15')
+    15
+    >>> vtor.check('integer(10)', '9')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(10)', 9)
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(max=20)', '35')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(max=20)', 35)
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(0, 9)', False)
+    0
+    """
+#    print value, type(value)
+    (min_val, max_val) = _is_num_param(('min', 'max'), (min, max))
+    if not isinstance(value, (int, long, StringTypes)):
+        raise VdtTypeError(value)
+    if isinstance(value, StringTypes):
+        # if it's a string - does it represent an integer ?
+        try:
+            value = int(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+def is_float(value, min=None, max=None):
+    """
+    A check that tests that a given value is a float
+    (an integer will be accepted), and optionally - that it is between bounds.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    This can accept negative values.
+    
+    >>> vtor.check('float', '2')
+    2.0
+    
+    From now on we multiply the value to avoid comparing decimals
+    
+    >>> vtor.check('float', '-6.8') * 10
+    -68.0
+    >>> vtor.check('float', '12.2') * 10
+    122.0
+    >>> vtor.check('float', 8.4) * 10
+    84.0
+    >>> vtor.check('float', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('float(10.1)', '10.2') * 10
+    102.0
+    >>> vtor.check('float(max=20.2)', '15.1') * 10
+    151.0
+    >>> vtor.check('float(10.0)', '9.0')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9.0" is too small.
+    >>> vtor.check('float(max=20.0)', '35.0')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35.0" is too big.
+    """
+    (min_val, max_val) = _is_num_param(
+        ('min', 'max'), (min, max), to_float=True)
+    if not isinstance(value, (int, long, float, StringTypes)):
+        raise VdtTypeError(value)
+    if not isinstance(value, float):
+        # if it's a string - does it represent a float ?
+        try:
+            value = float(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+bool_dict = {
+    True: True, 'on': True, '1': True, 'true': True, 'yes': True, 
+    False: False, 'off': False, '0': False, 'false': False, 'no': False,
+}
+
+def is_bool(value):
+    """
+    Check if the value represents a boolean.
+    
+    >>> vtor.check('boolean', 0)
+    0
+    >>> vtor.check('boolean', False)
+    0
+    >>> vtor.check('boolean', '0')
+    0
+    >>> vtor.check('boolean', 'off')
+    0
+    >>> vtor.check('boolean', 'false')
+    0
+    >>> vtor.check('boolean', 'no')
+    0
+    >>> vtor.check('boolean', 'nO')
+    0
+    >>> vtor.check('boolean', 'NO')
+    0
+    >>> vtor.check('boolean', 1)
+    1
+    >>> vtor.check('boolean', True)
+    1
+    >>> vtor.check('boolean', '1')
+    1
+    >>> vtor.check('boolean', 'on')
+    1
+    >>> vtor.check('boolean', 'true')
+    1
+    >>> vtor.check('boolean', 'yes')
+    1
+    >>> vtor.check('boolean', 'Yes')
+    1
+    >>> vtor.check('boolean', 'YES')
+    1
+    >>> vtor.check('boolean', '')
+    Traceback (most recent call last):
+    VdtTypeError: the value "" is of the wrong type.
+    >>> vtor.check('boolean', 'up')
+    Traceback (most recent call last):
+    VdtTypeError: the value "up" is of the wrong type.
+    
+    """
+    if isinstance(value, StringTypes):
+        try:
+            return bool_dict[value.lower()]
+        except KeyError:
+            raise VdtTypeError(value)
+    # we do an equality test rather than an identity test
+    # this ensures Python 2.2 compatibilty
+    # and allows 0 and 1 to represent True and False
+    if value == False:
+        return False
+    elif value == True:
+        return True
+    else:
+        raise VdtTypeError(value)
+
+
+def is_ip_addr(value):
+    """
+    Check that the supplied value is an Internet Protocol address, v.4,
+    represented by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    >>> vtor.check('ip_addr', '1 ')
+    '1'
+    >>> vtor.check('ip_addr', ' 1.2')
+    '1.2'
+    >>> vtor.check('ip_addr', ' 1.2.3 ')
+    '1.2.3'
+    >>> vtor.check('ip_addr', '1.2.3.4')
+    '1.2.3.4'
+    >>> vtor.check('ip_addr', '0.0.0.0')
+    '0.0.0.0'
+    >>> vtor.check('ip_addr', '255.255.255.255')
+    '255.255.255.255'
+    >>> vtor.check('ip_addr', '255.255.255.256')
+    Traceback (most recent call last):
+    VdtValueError: the value "255.255.255.256" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3.4.5')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3.4.5" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3. 4')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3. 4" is unacceptable.
+    >>> vtor.check('ip_addr', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    value = value.strip()
+    try:
+        dottedQuadToNum(value)
+    except ValueError:
+        raise VdtValueError(value)
+    return value
+
+def is_list(value, min=None, max=None):
+    """
+    Check that the value is a list of values.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    It does no check on list members.
+    
+    >>> vtor.check('list', ())
+    ()
+    >>> vtor.check('list', [])
+    []
+    >>> vtor.check('list', (1, 2))
+    (1, 2)
+    >>> vtor.check('list', [1, 2])
+    [1, 2]
+    >>> vtor.check('list', '12')
+    '12'
+    >>> vtor.check('list(3)', (1, 2))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2)" is too short.
+    >>> vtor.check('list(max=5)', (1, 2, 3, 4, 5, 6))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2, 3, 4, 5, 6)" is too long.
+    >>> vtor.check('list(min=3, max=5)', (1, 2, 3, 4))
+    (1, 2, 3, 4)
+    >>> vtor.check('list', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    (min_len, max_len) = _is_num_param(('min', 'max'), (min, max))
+    try:
+        num_members = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if min_len is not None and num_members < min_len:
+        raise VdtValueTooShortError(value)
+    if max_len is not None and num_members > max_len:
+        raise VdtValueTooLongError(value)
+    return value
+
+def is_string(value, min=None, max=None):
+    """
+    Check that the supplied value is a string.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    >>> vtor.check('string', '0')
+    '0'
+    >>> vtor.check('string', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    >>> vtor.check('string(2)', '12')
+    '12'
+    >>> vtor.check('string(2)', '1')
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "1" is too short.
+    >>> vtor.check('string(min=2, max=3)', '123')
+    '123'
+    >>> vtor.check('string(min=2, max=3)', '1234')
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "1234" is too long.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    return is_list(value, min, max)
+
+def is_int_list(value, min=None, max=None):
+    """
+    Check that the value is a list of integers.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an integer.
+    
+    >>> vtor.check('int_list', ())
+    []
+    >>> vtor.check('int_list', [])
+    []
+    >>> vtor.check('int_list', (1, 2))
+    [1, 2]
+    >>> vtor.check('int_list', [1, 2])
+    [1, 2]
+    >>> vtor.check('int_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_integer(mem) for mem in is_list(value, min, max)]
+
+def is_bool_list(value, min=None, max=None):
+    """
+    Check that the value is a list of booleans.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a boolean.
+    
+    >>> vtor.check('bool_list', ())
+    []
+    >>> vtor.check('bool_list', [])
+    []
+    >>> check_res = vtor.check('bool_list', (True, False))
+    >>> check_res == [True, False]
+    1
+    >>> check_res = vtor.check('bool_list', [True, False])
+    >>> check_res == [True, False]
+    1
+    >>> vtor.check('bool_list', [True, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_bool(mem) for mem in is_list(value, min, max)]
+
+def is_float_list(value, min=None, max=None):
+    """
+    Check that the value is a list of floats.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a float.
+    
+    >>> vtor.check('float_list', ())
+    []
+    >>> vtor.check('float_list', [])
+    []
+    >>> vtor.check('float_list', (1, 2.0))
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 2.0])
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_float(mem) for mem in is_list(value, min, max)]
+
+def is_string_list(value, min=None, max=None):
+    """
+    Check that the value is a list of strings.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a string.
+    
+    >>> vtor.check('string_list', ())
+    []
+    >>> vtor.check('string_list', [])
+    []
+    >>> vtor.check('string_list', ('a', 'b'))
+    ['a', 'b']
+    >>> vtor.check('string_list', ['a', 1])
+    Traceback (most recent call last):
+    VdtTypeError: the value "1" is of the wrong type.
+    """
+    return [is_string(mem) for mem in is_list(value, min, max)]
+
+def is_ip_addr_list(value, min=None, max=None):
+    """
+    Check that the value is a list of IP addresses.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an IP address.
+    
+    >>> vtor.check('ip_addr_list', ())
+    []
+    >>> vtor.check('ip_addr_list', [])
+    []
+    >>> vtor.check('ip_addr_list', ('1.2.3.4', '5.6.7.8'))
+    ['1.2.3.4', '5.6.7.8']
+    >>> vtor.check('ip_addr_list', ['a'])
+    Traceback (most recent call last):
+    VdtValueError: the value "a" is unacceptable.
+    """
+    return [is_ip_addr(mem) for mem in is_list(value, min, max)]
+
+fun_dict = {
+    'integer': is_integer,
+    'float': is_float,
+    'ip_addr': is_ip_addr,
+    'string': is_string,
+    'boolean': is_bool,
+}
+
+def is_mixed_list(value, *args):
+    """
+    Check that the value is a list.
+    Allow specifying the type of each member.
+    Work on lists of specific lengths.
+    
+    You specify each member as a positional argument specifying type
+    
+    Each type should be one of the following strings :
+      'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+    So you can specify a list of two strings, followed by
+    two integers as :
+    
+      mixed_list('string', 'string', 'integer', 'integer')
+    
+    The length of the list must match the number of positional
+    arguments you supply.
+    
+    >>> mix_str = "mixed_list('integer', 'float', 'ip_addr', 'string', 'boolean')"
+    >>> check_res = vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', True))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> check_res = vtor.check(mix_str, ('1', '2.0', '1.2.3.4', 'a', 'True'))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> vtor.check(mix_str, ('b', 2.0, '1.2.3.4', 'a', True))
+    Traceback (most recent call last):
+    VdtTypeError: the value "b" is of the wrong type.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a'))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2.0, '1.2.3.4', 'a')" is too short.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', 1, 'b'))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2.0, '1.2.3.4', 'a', 1, 'b')" is too long.
+    >>> vtor.check(mix_str, 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    
+    This test requires an elaborate setup, because of a change in error string
+    output from the interpreter between Python 2.2 and 2.3 .
+    
+    >>> res_seq = (
+    ...     'passed an incorrect value "',
+    ...     'yoda',
+    ...     '" for parameter "mixed_list".',
+    ... )
+    >>> if INTP_VER == (2, 2):
+    ...     res_str = "".join(res_seq)
+    ... else:
+    ...     res_str = "'".join(res_seq)
+    >>> try:
+    ...     vtor.check('mixed_list("yoda")', ('a'))
+    ... except VdtParamError, err:
+    ...     str(err) == res_str
+    1
+    """
+    try:
+        length = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if length < len(args):
+        raise VdtValueTooShortError(value)
+    elif length > len(args):
+        raise VdtValueTooLongError(value)
+    try:
+        return [fun_dict[arg](val) for arg, val in zip(args, value)]
+    except KeyError, e:
+        raise VdtParamError('mixed_list', e)
+
+def is_option(value, *options):
+    """
+    This check matches the value to any of a set of options.
+    
+    >>> vtor.check('option("yoda", "jedi")', 'yoda')
+    'yoda'
+    >>> vtor.check('option("yoda", "jedi")', 'jed')
+    Traceback (most recent call last):
+    VdtValueError: the value "jed" is unacceptable.
+    >>> vtor.check('option("yoda", "jedi")', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    if not value in options:
+        raise VdtValueError(value)
+    return value
+
+def _test(value, *args, **keywargs):
+    """
+    A function that exists for test purposes.
+    
+    >>> checks = [
+    ...     '3, 6, min=1, max=3, test=list(a, b, c)',
+    ...     '3',
+    ...     '3, 6',
+    ...     '3,',
+    ...     'min=1, test="a b c"',
+    ...     'min=5, test="a, b, c"',
+    ...     'min=1, max=3, test="a, b, c"',
+    ...     'min=-100, test=-99',
+    ...     'min=1, max=3',
+    ...     '3, 6, test="36"',
+    ...     '3, 6, test="a, b, c"',
+    ...     '3, max=3, test=list("a", "b", "c")',
+    ...     '''3, max=3, test=list("'a'", 'b', "x=(c)")''',
+    ...     "test='x=fish(3)'",
+    ...    ]
+    >>> v = Validator({'test': _test})
+    >>> for entry in checks:
+    ...     print v.check(('test(%s)' % entry), 3)
+    (3, ('3', '6'), {'test': ['a', 'b', 'c'], 'max': '3', 'min': '1'})
+    (3, ('3',), {})
+    (3, ('3', '6'), {})
+    (3, ('3',), {})
+    (3, (), {'test': 'a b c', 'min': '1'})
+    (3, (), {'test': 'a, b, c', 'min': '5'})
+    (3, (), {'test': 'a, b, c', 'max': '3', 'min': '1'})
+    (3, (), {'test': '-99', 'min': '-100'})
+    (3, (), {'max': '3', 'min': '1'})
+    (3, ('3', '6'), {'test': '36'})
+    (3, ('3', '6'), {'test': 'a, b, c'})
+    (3, ('3',), {'test': ['a', 'b', 'c'], 'max': '3'})
+    (3, ('3',), {'test': ["'a'", 'b', 'x=(c)'], 'max': '3'})
+    (3, (), {'test': 'x=fish(3)'})
+    """
+    return (value, args, keywargs)
+
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'vtor': Validator(),
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    TODO
+    ====
+    
+    Consider which parts of the regex stuff to put back in
+    
+    Can we implement a timestamp datatype ? (check DateUtil module)
+    
+    ISSUES
+    ======
+    
+    If we could pull tuples out of arguments, it would be easier
+    to specify arguments for 'mixed_lists'.
+    
+    CHANGELOG
+    =========
+    
+    2005/12/16
+    ----------
+    
+    Fixed bug so we can handle keyword argument values with commas.
+    
+    We now use a list constructor for passing list values to keyword arguments
+    (including ``default``) : ::
+    
+        default=list("val", "val", "val")
+    
+    Added the ``_test`` test. {sm;:-)}
+    
+    0.2.1
+    
+    2005/12/12
+    ----------
+    
+    Moved a function call outside a try...except block.
+    
+    2005/08/25
+    ----------
+    
+    Most errors now prefixed ``Vdt``
+    
+    ``VdtParamError`` no longer derives from ``VdtError``
+    
+    Finalised as version 0.2.0
+    
+    2005/08/21
+    ----------
+    
+    By Nicola Larosa
+    
+    Removed the "length" argument for lists and strings, and related tests
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Deleted the "none" and "multiple" types and checks
+    
+    Added the None value for all types in Validation.check
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Removed timestamp.
+    
+    By Nicola Larosa
+    
+    Fixed bug in Validator.check: when a value that has a default is also
+    specified in the config file, the default must be deleted from fun_kwargs
+    anyway, otherwise the check function will get a spurious "default" keyword
+    argument
+    
+    Added "ip_addr_list" check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    Fixed last is_mixed_list test to work on Python 2.2 too
+    
+    2005/08/10
+    ----------
+    
+    By Nicola Larosa
+    
+    Restored Python2.2 compatibility by avoiding usage of dict.pop
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility, one test still fails
+    for trivial reasons (string output delimiters)
+    
+    2005/08/05
+    ----------
+    
+    By Michael Foord
+    
+    Added __version__, __all__, and __docformat__
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    2005/07/20
+    ----------
+    
+    By Nicola Larosa
+    
+    Added an 'ip_addr' IPv4 address value check, with tests
+    
+    Updated the tests for mixed_list to include IP addresses
+    
+    Changed all references to value "tests" into value "checks", including
+    the main Validator method, and all code tests
+    
+    2005/07/19
+    ----------
+    
+    By Nicola Larosa
+    
+    Added even more code tests
+    
+    Refined the mixed_list check
+    
+    2005/07/18
+    ----------
+    
+    By Nicola Larosa
+    
+    Introduced more VdtValueError subclasses
+    
+    Collapsed the ``_function_test`` and ``_function_parse`` methods into the
+    ``check`` one
+    
+    Refined the value checks, using the new VdtValueError subclasses
+    
+    Changed "is_string" to use "is_list"
+    
+    Added many more code tests
+    
+    Changed the "bool" value type to "boolean"
+    
+    Some more code cleanup
+    
+    2005/07/17
+    ----------
+    
+    By Nicola Larosa
+    
+    Code tests converted to doctest format and placed in the respective
+    docstrings, so they are automatically checked, and easier to update
+    
+    Changed local vars "min" and "max" to "min_len", "max_len", "min_val" and
+    "max_val", to avoid shadowing the builtin functions (but left function
+    parameters alone)
+    
+    Uniformed value check function names to is_* convention
+    
+    ``date`` type name changed to ``timestamp``
+    
+    Avoided some code duplication in list check functions
+    
+    Some more code cleanup
+    
+    2005/07/09
+    ----------
+    
+    Recoded the standard functions
+    
+    2005/07/08
+    ----------
+    
+    Improved paramfinder regex
+    
+    Ripped out all the regex stuff, checks, and the example functions
+    (to be replaced !)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+"""
+

=== added file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/versionedfile.py	2006-04-19 02:55:17 +0000
@@ -0,0 +1,629 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# Authors:
+#   Johan Rydberg <jrydberg@gnu.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# Remaing to do is to figure out if get_graph should return a simple
+# map, or a graph object of some kind.
+
+
+"""Versioned text file storage api."""
+
+
+from copy import deepcopy
+from unittest import TestSuite
+
+
+import bzrlib.errors as errors
+from bzrlib.inter import InterObject
+from bzrlib.symbol_versioning import *
+from bzrlib.textmerge import TextMerge
+from bzrlib.transport.memory import MemoryTransport
+from bzrlib.tsort import topo_sort
+from bzrlib import ui
+
+
+class VersionedFile(object):
+    """Versioned text file storage.
+    
+    A versioned file manages versions of line-based text files,
+    keeping track of the originating version for each line.
+
+    To clients the "lines" of the file are represented as a list of
+    strings. These strings will typically have terminal newline
+    characters, but this is not required.  In particular files commonly
+    do not have a newline at the end of the file.
+
+    Texts are identified by a version-id string.
+    """
+
+    def __init__(self, access_mode):
+        self.finished = False
+        self._access_mode = access_mode
+
+    def copy_to(self, name, transport):
+        """Copy this versioned file to name on transport."""
+        raise NotImplementedError(self.copy_to)
+    
+    @deprecated_method(zero_eight)
+    def names(self):
+        """Return a list of all the versions in this versioned file.
+
+        Please use versionedfile.versions() now.
+        """
+        return self.versions()
+
+    def versions(self):
+        """Return a unsorted list of versions."""
+        raise NotImplementedError(self.versions)
+
+    def has_ghost(self, version_id):
+        """Returns whether version is present as a ghost."""
+        raise NotImplementedError(self.has_ghost)
+
+    def has_version(self, version_id):
+        """Returns whether version is present."""
+        raise NotImplementedError(self.has_version)
+
+    def add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
+        """Add a text to the versioned file via a pregenerated delta.
+
+        :param version_id: The version id being added.
+        :param parents: The parents of the version_id.
+        :param delta_parent: The parent this delta was created against.
+        :param sha1: The sha1 of the full text.
+        :param delta: The delta instructions. See get_delta for details.
+        """
+        self._check_write_ok()
+        if self.has_version(version_id):
+            raise errors.RevisionAlreadyPresent(version_id, self)
+        return self._add_delta(version_id, parents, delta_parent, sha1, noeol, delta)
+
+    def _add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
+        """Class specific routine to add a delta.
+
+        This generic version simply applies the delta to the delta_parent and
+        then inserts it.
+        """
+        # strip annotation from delta
+        new_delta = []
+        for start, stop, delta_len, delta_lines in delta:
+            new_delta.append((start, stop, delta_len, [text for origin, text in delta_lines]))
+        if delta_parent is not None:
+            parent_full = self.get_lines(delta_parent)
+        else:
+            parent_full = []
+        new_full = self._apply_delta(parent_full, new_delta)
+        # its impossible to have noeol on an empty file
+        if noeol and new_full[-1][-1] == '\n':
+            new_full[-1] = new_full[-1][:-1]
+        self.add_lines(version_id, parents, new_full)
+
+    def add_lines(self, version_id, parents, lines, parent_texts=None):
+        """Add a single text on top of the versioned file.
+
+        Must raise RevisionAlreadyPresent if the new version is
+        already present in file history.
+
+        Must raise RevisionNotPresent if any of the given parents are
+        not present in file history.
+        :param parent_texts: An optional dictionary containing the opaque 
+             representations of some or all of the parents of 
+             version_id to allow delta optimisations. 
+             VERY IMPORTANT: the texts must be those returned
+             by add_lines or data corruption can be caused.
+        :return: An opaque representation of the inserted version which can be
+                 provided back to future add_lines calls in the parent_texts
+                 dictionary.
+        """
+        self._check_write_ok()
+        return self._add_lines(version_id, parents, lines, parent_texts)
+
+    def _add_lines(self, version_id, parents, lines, parent_texts):
+        """Helper to do the class specific add_lines."""
+        raise NotImplementedError(self.add_lines)
+
+    def add_lines_with_ghosts(self, version_id, parents, lines,
+                              parent_texts=None):
+        """Add lines to the versioned file, allowing ghosts to be present.
+        
+        This takes the same parameters as add_lines.
+        """
+        self._check_write_ok()
+        return self._add_lines_with_ghosts(version_id, parents, lines,
+                                           parent_texts)
+
+    def _add_lines_with_ghosts(self, version_id, parents, lines, parent_texts):
+        """Helper to do class specific add_lines_with_ghosts."""
+        raise NotImplementedError(self.add_lines_with_ghosts)
+
+    def check(self, progress_bar=None):
+        """Check the versioned file for integrity."""
+        raise NotImplementedError(self.check)
+
+    def _check_write_ok(self):
+        """Is the versioned file marked as 'finished' ? Raise if it is."""
+        if self.finished:
+            raise errors.OutSideTransaction()
+        if self._access_mode != 'w':
+            raise errors.ReadOnlyObjectDirtiedError(self)
+
+    def clear_cache(self):
+        """Remove any data cached in the versioned file object."""
+
+    def clone_text(self, new_version_id, old_version_id, parents):
+        """Add an identical text to old_version_id as new_version_id.
+
+        Must raise RevisionNotPresent if the old version or any of the
+        parents are not present in file history.
+
+        Must raise RevisionAlreadyPresent if the new version is
+        already present in file history."""
+        self._check_write_ok()
+        return self._clone_text(new_version_id, old_version_id, parents)
+
+    def _clone_text(self, new_version_id, old_version_id, parents):
+        """Helper function to do the _clone_text work."""
+        raise NotImplementedError(self.clone_text)
+
+    def create_empty(self, name, transport, mode=None):
+        """Create a new versioned file of this exact type.
+
+        :param name: the file name
+        :param transport: the transport
+        :param mode: optional file mode.
+        """
+        raise NotImplementedError(self.create_empty)
+
+    def fix_parents(self, version, new_parents):
+        """Fix the parents list for version.
+        
+        This is done by appending a new version to the index
+        with identical data except for the parents list.
+        the parents list must be a superset of the current
+        list.
+        """
+        self._check_write_ok()
+        return self._fix_parents(version, new_parents)
+
+    def _fix_parents(self, version, new_parents):
+        """Helper for fix_parents."""
+        raise NotImplementedError(self.fix_parents)
+
+    def get_delta(self, version):
+        """Get a delta for constructing version from some other version.
+        
+        :return: (delta_parent, sha1, noeol, delta)
+        Where delta_parent is a version id or None to indicate no parent.
+        """
+        raise NotImplementedError(self.get_delta)
+
+    def get_deltas(self, versions):
+        """Get multiple deltas at once for constructing versions.
+        
+        :return: dict(version_id:(delta_parent, sha1, noeol, delta))
+        Where delta_parent is a version id or None to indicate no parent, and
+        version_id is the version_id created by that delta.
+        """
+        result = {}
+        for version in versions:
+            result[version] = self.get_delta(version)
+        return result
+
+    def get_suffixes(self):
+        """Return the file suffixes associated with this versioned file."""
+        raise NotImplementedError(self.get_suffixes)
+    
+    def get_text(self, version_id):
+        """Return version contents as a text string.
+
+        Raises RevisionNotPresent if version is not present in
+        file history.
+        """
+        return ''.join(self.get_lines(version_id))
+    get_string = get_text
+
+    def get_lines(self, version_id):
+        """Return version contents as a sequence of lines.
+
+        Raises RevisionNotPresent if version is not present in
+        file history.
+        """
+        raise NotImplementedError(self.get_lines)
+
+    def get_ancestry(self, version_ids):
+        """Return a list of all ancestors of given version(s). This
+        will not include the null revision.
+
+        Must raise RevisionNotPresent if any of the given versions are
+        not present in file history."""
+        if isinstance(version_ids, basestring):
+            version_ids = [version_ids]
+        raise NotImplementedError(self.get_ancestry)
+        
+    def get_ancestry_with_ghosts(self, version_ids):
+        """Return a list of all ancestors of given version(s). This
+        will not include the null revision.
+
+        Must raise RevisionNotPresent if any of the given versions are
+        not present in file history.
+        
+        Ghosts that are known about will be included in ancestry list,
+        but are not explicitly marked.
+        """
+        raise NotImplementedError(self.get_ancestry_with_ghosts)
+        
+    def get_graph(self):
+        """Return a graph for the entire versioned file.
+        
+        Ghosts are not listed or referenced in the graph.
+        """
+        result = {}
+        for version in self.versions():
+            result[version] = self.get_parents(version)
+        return result
+
+    def get_graph_with_ghosts(self):
+        """Return a graph for the entire versioned file.
+        
+        Ghosts are referenced in parents list but are not
+        explicitly listed.
+        """
+        raise NotImplementedError(self.get_graph_with_ghosts)
+
+    @deprecated_method(zero_eight)
+    def parent_names(self, version):
+        """Return version names for parents of a version.
+        
+        See get_parents for the current api.
+        """
+        return self.get_parents(version)
+
+    def get_parents(self, version_id):
+        """Return version names for parents of a version.
+
+        Must raise RevisionNotPresent if version is not present in
+        file history.
+        """
+        raise NotImplementedError(self.get_parents)
+
+    def get_parents_with_ghosts(self, version_id):
+        """Return version names for parents of version_id.
+
+        Will raise RevisionNotPresent if version_id is not present
+        in the history.
+
+        Ghosts that are known about will be included in the parent list,
+        but are not explicitly marked.
+        """
+        raise NotImplementedError(self.get_parents_with_ghosts)
+
+    def annotate_iter(self, version_id):
+        """Yield list of (version-id, line) pairs for the specified
+        version.
+
+        Must raise RevisionNotPresent if any of the given versions are
+        not present in file history.
+        """
+        raise NotImplementedError(self.annotate_iter)
+
+    def annotate(self, version_id):
+        return list(self.annotate_iter(version_id))
+
+    def _apply_delta(self, lines, delta):
+        """Apply delta to lines."""
+        lines = list(lines)
+        offset = 0
+        for start, end, count, delta_lines in delta:
+            lines[offset+start:offset+end] = delta_lines
+            offset = offset + (start - end) + count
+        return lines
+
+    def join(self, other, pb=None, msg=None, version_ids=None,
+             ignore_missing=False):
+        """Integrate versions from other into this versioned file.
+
+        If version_ids is None all versions from other should be
+        incorporated into this versioned file.
+
+        Must raise RevisionNotPresent if any of the specified versions
+        are not present in the other files history unless ignore_missing
+        is supplied when they are silently skipped.
+        """
+        self._check_write_ok()
+        return InterVersionedFile.get(other, self).join(
+            pb,
+            msg,
+            version_ids,
+            ignore_missing)
+
+    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+        """Iterate over the lines in the versioned file from version_ids.
+
+        This may return lines from other versions, and does not return the
+        specific version marker at this point. The api may be changed
+        during development to include the version that the versioned file
+        thinks is relevant, but given that such hints are just guesses,
+        its better not to have it if we dont need it.
+
+        NOTES: Lines are normalised: they will all have \n terminators.
+               Lines are returned in arbitrary order.
+        """
+        raise NotImplementedError(self.iter_lines_added_or_present_in_versions)
+
+    def transaction_finished(self):
+        """The transaction that this file was opened in has finished.
+
+        This records self.finished = True and should cause all mutating
+        operations to error.
+        """
+        self.finished = True
+
+    @deprecated_method(zero_eight)
+    def walk(self, version_ids=None):
+        """Walk the versioned file as a weave-like structure, for
+        versions relative to version_ids.  Yields sequence of (lineno,
+        insert, deletes, text) for each relevant line.
+
+        Must raise RevisionNotPresent if any of the specified versions
+        are not present in the file history.
+
+        :param version_ids: the version_ids to walk with respect to. If not
+                            supplied the entire weave-like structure is walked.
+
+        walk is deprecated in favour of iter_lines_added_or_present_in_versions
+        """
+        raise NotImplementedError(self.walk)
+
+    @deprecated_method(zero_eight)
+    def iter_names(self):
+        """Walk the names list."""
+        return iter(self.versions())
+
+    def plan_merge(self, ver_a, ver_b):
+        """Return pseudo-annotation indicating how the two versions merge.
+
+        This is computed between versions a and b and their common
+        base.
+
+        Weave lines present in none of them are skipped entirely.
+
+        Legend:
+        killed-base Dead in base revision
+        killed-both Killed in each revision
+        killed-a    Killed in a
+        killed-b    Killed in b
+        unchanged   Alive in both a and b (possibly created in both)
+        new-a       Created in a
+        new-b       Created in b
+        ghost-a     Killed in a, unborn in b    
+        ghost-b     Killed in b, unborn in a
+        irrelevant  Not in either revision
+        """
+        raise NotImplementedError(VersionedFile.plan_merge)
+        
+    def weave_merge(self, plan, a_marker=TextMerge.A_MARKER, 
+                    b_marker=TextMerge.B_MARKER):
+        return PlanWeaveMerge(plan, a_marker, b_marker).merge_lines()[0]
+
+
+class PlanWeaveMerge(TextMerge):
+    """Weave merge that takes a plan as its input.
+    
+    This exists so that VersionedFile.plan_merge is implementable.
+    Most callers will want to use WeaveMerge instead.
+    """
+
+    def __init__(self, plan, a_marker=TextMerge.A_MARKER,
+                 b_marker=TextMerge.B_MARKER):
+        TextMerge.__init__(self, a_marker, b_marker)
+        self.plan = plan
+
+    def _merge_struct(self):
+        lines_a = []
+        lines_b = []
+        ch_a = ch_b = False
+
+        def outstanding_struct():
+            if not lines_a and not lines_b:
+                return
+            elif ch_a and not ch_b:
+                # one-sided change:
+                yield(lines_a,)
+            elif ch_b and not ch_a:
+                yield (lines_b,)
+            elif lines_a == lines_b:
+                yield(lines_a,)
+            else:
+                yield (lines_a, lines_b)
+       
+        # We previously considered either 'unchanged' or 'killed-both' lines
+        # to be possible places to resynchronize.  However, assuming agreement
+        # on killed-both lines may be too agressive. -- mbp 20060324
+        for state, line in self.plan:
+            if state == 'unchanged':
+                # resync and flush queued conflicts changes if any
+                for struct in outstanding_struct():
+                    yield struct
+                lines_a = []
+                lines_b = []
+                ch_a = ch_b = False
+                
+            if state == 'unchanged':
+                if line:
+                    yield ([line],)
+            elif state == 'killed-a':
+                ch_a = True
+                lines_b.append(line)
+            elif state == 'killed-b':
+                ch_b = True
+                lines_a.append(line)
+            elif state == 'new-a':
+                ch_a = True
+                lines_a.append(line)
+            elif state == 'new-b':
+                ch_b = True
+                lines_b.append(line)
+            else:
+                assert state in ('irrelevant', 'ghost-a', 'ghost-b', 
+                                 'killed-base', 'killed-both'), state
+        for struct in outstanding_struct():
+            yield struct
+
+
+class WeaveMerge(PlanWeaveMerge):
+    """Weave merge that takes a VersionedFile and two versions as its input"""
+
+    def __init__(self, versionedfile, ver_a, ver_b, 
+        a_marker=PlanWeaveMerge.A_MARKER, b_marker=PlanWeaveMerge.B_MARKER):
+        plan = versionedfile.plan_merge(ver_a, ver_b)
+        PlanWeaveMerge.__init__(self, plan, a_marker, b_marker)
+
+
+class InterVersionedFile(InterObject):
+    """This class represents operations taking place between two versionedfiles..
+
+    Its instances have methods like join, and contain
+    references to the source and target versionedfiles these operations can be 
+    carried out on.
+
+    Often we will provide convenience methods on 'versionedfile' which carry out
+    operations with another versionedfile - they will always forward to
+    InterVersionedFile.get(other).method_name(parameters).
+    """
+
+    _optimisers = set()
+    """The available optimised InterVersionedFile types."""
+
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
+        """Integrate versions from self.source into self.target.
+
+        If version_ids is None all versions from source should be
+        incorporated into this versioned file.
+
+        Must raise RevisionNotPresent if any of the specified versions
+        are not present in the other files history unless ignore_missing is 
+        supplied when they are silently skipped.
+        """
+        # the default join: 
+        # - if the target is empty, just add all the versions from 
+        #   source to target, otherwise:
+        # - make a temporary versioned file of type target
+        # - insert the source content into it one at a time
+        # - join them
+        if not self.target.versions():
+            target = self.target
+        else:
+            # Make a new target-format versioned file. 
+            temp_source = self.target.create_empty("temp", MemoryTransport())
+            target = temp_source
+        graph = self.source.get_graph()
+        order = topo_sort(graph.items())
+        pb = ui.ui_factory.nested_progress_bar()
+        parent_texts = {}
+        try:
+            # TODO for incremental cross-format work:
+            # make a versioned file with the following content:
+            # all revisions we have been asked to join
+            # all their ancestors that are *not* in target already.
+            # the immediate parents of the above two sets, with 
+            # empty parent lists - these versions are in target already
+            # and the incorrect version data will be ignored.
+            # TODO: for all ancestors that are present in target already,
+            # check them for consistent data, this requires moving sha1 from
+            # 
+            # TODO: remove parent texts when they are not relevant any more for 
+            # memory pressure reduction. RBC 20060313
+            # pb.update('Converting versioned data', 0, len(order))
+            # deltas = self.source.get_deltas(order)
+            for index, version in enumerate(order):
+                pb.update('Converting versioned data', index, len(order))
+                parent_text = target.add_lines(version,
+                                               self.source.get_parents(version),
+                                               self.source.get_lines(version),
+                                               parent_texts=parent_texts)
+                parent_texts[version] = parent_text
+                #delta_parent, sha1, noeol, delta = deltas[version]
+                #target.add_delta(version,
+                #                 self.source.get_parents(version),
+                #                 delta_parent,
+                #                 sha1,
+                #                 noeol,
+                #                 delta)
+                #target.get_lines(version)
+            
+            # this should hit the native code path for target
+            if target is not self.target:
+                return self.target.join(temp_source,
+                                        pb,
+                                        msg,
+                                        version_ids,
+                                        ignore_missing)
+        finally:
+            pb.finished()
+
+
+class InterVersionedFileTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple inter versioned-file classes.
+
+    This is done by copying the test once for each interversionedfile provider
+    and injecting the transport_server, transport_readonly_server,
+    versionedfile_factory and versionedfile_factory_to classes into each copy.
+    Each copy is also given a new id() to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for (interversionedfile_class,
+             versionedfile_factory,
+             versionedfile_factory_to) in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.interversionedfile_class = interversionedfile_class
+            new_test.versionedfile_factory = versionedfile_factory
+            new_test.versionedfile_factory_to = versionedfile_factory_to
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), interversionedfile_class.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    @staticmethod
+    def default_test_list():
+        """Generate the default list of interversionedfile permutations to test."""
+        from bzrlib.weave import WeaveFile
+        from bzrlib.knit import KnitVersionedFile
+        result = []
+        # test the fallback InterVersionedFile from weave to annotated knits
+        result.append((InterVersionedFile, 
+                       WeaveFile,
+                       KnitVersionedFile))
+        for optimiser in InterVersionedFile._optimisers:
+            result.append((optimiser,
+                           optimiser._matching_file_factory,
+                           optimiser._matching_file_factory
+                           ))
+        # if there are specific combinations we want to use, we can add them 
+        # here.
+        return result

=== modified file 'bzrlib/weave.py' (properties changed: -x to +x)
--- bzrlib/weave.py	2005-08-18 07:59:20 +0000
+++ bzrlib/weave.py	2006-04-11 05:21:53 +0000
@@ -21,37 +21,6 @@
 
 """Weave - storage of related text file versions"""
 
-# before intset (r923) 2000 versions in 41.5s
-# with intset (r926) 2000 versions in 93s !!!
-# better to just use plain sets.
-
-# making _extract build and return a list, rather than being a generator
-# takes 37.94s
-
-# with python -O, r923 does 2000 versions in 36.87s
-
-# with optimizations to avoid mutating lists - 35.75!  I guess copying
-# all the elements every time costs more than the small manipulations.
-# a surprisingly small change.
-
-# r931, which avoids using a generator for extract, does 36.98s
-
-# with memoized inclusions, takes 41.49s; not very good
-
-# with slots, takes 37.35s; without takes 39.16, a bit surprising
-
-# with the delta calculation mixed in with the add method, rather than
-# separated, takes 36.78s
-
-# with delta folded in and mutation of the list, 36.13s
-
-# with all this and simplification of add code, 33s
-
-
-
-
-
-# TODO: Perhaps have copy method for Weave instances?
 
 # XXX: If we do weaves this way, will a merge still behave the same
 # way if it's done in a different order?  That's a pretty desirable
@@ -74,31 +43,54 @@
 
 # TODO: Parallel-extract that passes back each line along with a
 # description of which revisions include it.  Nice for checking all
-# shas in parallel.
+# shas or calculating stats in parallel.
 
 # TODO: Using a single _extract routine and then processing the output
 # is probably inefficient.  It's simple enough that we can afford to
 # have slight specializations for different ways its used: annotate,
 # basis for add, get, etc.
 
-# TODO: Perhaps the API should work only in names to hide the integer
-# indexes from the user?
-
-
-
+# TODO: Probably the API should work only in names to hide the integer
+# indexes from the user.
+
+# TODO: Is there any potential performance win by having an add()
+# variant that is passed a pre-cooked version of the single basis
+# version?
+
+# TODO: Reweave can possibly be made faster by remembering diffs
+# where the basis and destination are unchanged.
+
+# FIXME: Sometimes we will be given a parents list for a revision
+# that includes some redundant parents (i.e. already a parent of 
+# something in the list.)  We should eliminate them.  This can 
+# be done fairly efficiently because the sequence numbers constrain
+# the possible relationships.
+
+# FIXME: the conflict markers should be *7* characters
+
+from copy import copy
+from cStringIO import StringIO
+from difflib import SequenceMatcher
+import os
 import sha
-
-
-
-class WeaveError(Exception):
-    """Exception in processing weave"""
-
-
-class WeaveFormatError(WeaveError):
-    """Weave invariant violated"""
-    
-
-class Weave(object):
+import time
+
+from bzrlib.trace import mutter
+from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
+        RevisionAlreadyPresent,
+        RevisionNotPresent,
+        WeaveRevisionAlreadyPresent,
+        WeaveRevisionNotPresent,
+        )
+import bzrlib.errors as errors
+from bzrlib.osutils import sha_strings
+from bzrlib.symbol_versioning import *
+from bzrlib.tsort import topo_sort
+from bzrlib.versionedfile import VersionedFile, InterVersionedFile
+from bzrlib.weavefile import _read_weave_v5, write_weave_v5
+
+
+class Weave(VersionedFile):
     """weave - versioned text file storage.
     
     A Weave manages versions of line-based text files, keeping track
@@ -113,7 +105,7 @@
 
     * a nonnegative index number.
 
-    * a version-id string. (not implemented yet)
+    * a version-id string.
 
     Typically the index number will be valid only inside this weave and
     the version-id is used to reference it in the larger world.
@@ -181,17 +173,39 @@
 
     _name_map
         For each name, the version number.
+
+    _weave_name
+        Descriptive name of this weave; typically the filename if known.
+        Set by read_weave.
     """
 
-    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map']
+    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
+                 '_weave_name']
     
-    def __init__(self):
+    def __init__(self, weave_name=None, access_mode='w'):
+        super(Weave, self).__init__(access_mode)
         self._weave = []
         self._parents = []
         self._sha1s = []
         self._names = []
         self._name_map = {}
-
+        self._weave_name = weave_name
+
+    def __repr__(self):
+        return "Weave(%r)" % self._weave_name
+
+    def copy(self):
+        """Return a deep copy of self.
+        
+        The copy can be modified without affecting the original weave."""
+        other = Weave()
+        other._weave = self._weave[:]
+        other._parents = self._parents[:]
+        other._sha1s = self._sha1s[:]
+        other._names = self._names[:]
+        other._name_map = self._name_map.copy()
+        other._weave_name = self._weave_name
+        return other
 
     def __eq__(self, other):
         if not isinstance(other, Weave):
@@ -199,53 +213,273 @@
         return self._parents == other._parents \
                and self._weave == other._weave \
                and self._sha1s == other._sha1s 
-
     
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
+    @deprecated_method(zero_eight)
+    def idx_to_name(self, index):
+        """Old public interface, the public interface is all names now."""
+        return index
+
+    def _idx_to_name(self, version):
+        return self._names[version]
+
+    @deprecated_method(zero_eight)
     def lookup(self, name):
+        """Backwards compatability thunk:
+
+        Return name, as name is valid in the api now, and spew deprecation
+        warnings everywhere.
+        """
+        return name
+
+    def _lookup(self, name):
+        """Convert symbolic version name to index."""
         try:
             return self._name_map[name]
         except KeyError:
-            raise WeaveError("name %s not present in weave" % name)
-
-        
-    def add(self, name, parents, text):
+            raise RevisionNotPresent(name, self._weave_name)
+
+    @deprecated_method(zero_eight)
+    def iter_names(self):
+        """Deprecated convenience function, please see VersionedFile.names()."""
+        return iter(self.names())
+
+    @deprecated_method(zero_eight)
+    def names(self):
+        """See Weave.versions for the current api."""
+        return self.versions()
+
+    def versions(self):
+        """See VersionedFile.versions."""
+        return self._names[:]
+
+    def has_version(self, version_id):
+        """See VersionedFile.has_version."""
+        return self._name_map.has_key(version_id)
+
+    __contains__ = has_version
+
+    def get_delta(self, version_id):
+        """See VersionedFile.get_delta."""
+        return self.get_deltas([version_id])[version_id]
+
+    def get_deltas(self, version_ids):
+        """See VersionedFile.get_deltas."""
+        version_ids = self.get_ancestry(version_ids)
+        for version_id in version_ids:
+            if not self.has_version(version_id):
+                raise RevisionNotPresent(version_id, self)
+        # try extracting all versions; parallel extraction is used
+        nv = self.num_versions()
+        sha1s = {}
+        deltas = {}
+        texts = {}
+        inclusions = {}
+        noeols = {}
+        last_parent_lines = {}
+        parents = {}
+        parent_inclusions = {}
+        parent_linenums = {}
+        parent_noeols = {}
+        current_hunks = {}
+        diff_hunks = {}
+        # its simplest to generate a full set of prepared variables.
+        for i in range(nv):
+            name = self._names[i]
+            sha1s[name] = self.get_sha1(name)
+            parents_list = self.get_parents(name)
+            try:
+                parent = parents_list[0]
+                parents[name] = parent
+                parent_inclusions[name] = inclusions[parent]
+            except IndexError:
+                parents[name] = None
+                parent_inclusions[name] = set()
+            # we want to emit start, finish, replacement_length, replacement_lines tuples.
+            diff_hunks[name] = []
+            current_hunks[name] = [0, 0, 0, []] # #start, finish, repl_length, repl_tuples
+            parent_linenums[name] = 0
+            noeols[name] = False
+            parent_noeols[name] = False
+            last_parent_lines[name] = None
+            new_inc = set([name])
+            for p in self._parents[i]:
+                new_inc.update(inclusions[self._idx_to_name(p)])
+            # debug only, known good so far.
+            #assert set(new_inc) == set(self.get_ancestry(name)), \
+            #    'failed %s != %s' % (set(new_inc), set(self.get_ancestry(name)))
+            inclusions[name] = new_inc
+
+        nlines = len(self._weave)
+
+        for lineno, inserted, deletes, line in self._walk_internal():
+            # a line is active in a version if:
+            # insert is in the versions inclusions
+            # and
+            # deleteset & the versions inclusions is an empty set.
+            # so - if we have a included by mapping - version is included by
+            # children, we get a list of children to examine for deletes affect
+            # ing them, which is less than the entire set of children.
+            for version_id in version_ids:  
+                # The active inclusion must be an ancestor,
+                # and no ancestors must have deleted this line,
+                # because we don't support resurrection.
+                parent_inclusion = parent_inclusions[version_id]
+                inclusion = inclusions[version_id]
+                parent_active = inserted in parent_inclusion and not (deletes & parent_inclusion)
+                version_active = inserted in inclusion and not (deletes & inclusion)
+                if not parent_active and not version_active:
+                    # unrelated line of ancestry
+                    continue
+                elif parent_active and version_active:
+                    # shared line
+                    parent_linenum = parent_linenums[version_id]
+                    if current_hunks[version_id] != [parent_linenum, parent_linenum, 0, []]:
+                        diff_hunks[version_id].append(tuple(current_hunks[version_id]))
+                    parent_linenum += 1
+                    current_hunks[version_id] = [parent_linenum, parent_linenum, 0, []]
+                    parent_linenums[version_id] = parent_linenum
+                    try:
+                        if line[-1] != '\n':
+                            noeols[version_id] = True
+                    except IndexError:
+                        pass
+                elif parent_active and not version_active:
+                    # deleted line
+                    current_hunks[version_id][1] += 1
+                    parent_linenums[version_id] += 1
+                    last_parent_lines[version_id] = line
+                elif not parent_active and version_active:
+                    # replacement line
+                    # noeol only occurs at the end of a file because we 
+                    # diff linewise. We want to show noeol changes as a
+                    # empty diff unless the actual eol-less content changed.
+                    theline = line
+                    try:
+                        if last_parent_lines[version_id][-1] != '\n':
+                            parent_noeols[version_id] = True
+                    except (TypeError, IndexError):
+                        pass
+                    try:
+                        if theline[-1] != '\n':
+                            noeols[version_id] = True
+                    except IndexError:
+                        pass
+                    new_line = False
+                    parent_should_go = False
+
+                    if parent_noeols[version_id] == noeols[version_id]:
+                        # no noeol toggle, so trust the weaves statement
+                        # that this line is changed.
+                        new_line = True
+                        if parent_noeols[version_id]:
+                            theline = theline + '\n'
+                    elif parent_noeols[version_id]:
+                        # parent has no eol, we do:
+                        # our line is new, report as such..
+                        new_line = True
+                    elif noeols[version_id]:
+                        # append a eol so that it looks like
+                        # a normalised delta
+                        theline = theline + '\n'
+                        if parents[version_id] is not None:
+                        #if last_parent_lines[version_id] is not None:
+                            parent_should_go = True
+                        if last_parent_lines[version_id] != theline:
+                            # but changed anyway
+                            new_line = True
+                            #parent_should_go = False
+                    if new_line:
+                        current_hunks[version_id][2] += 1
+                        current_hunks[version_id][3].append((inserted, theline))
+                    if parent_should_go:
+                        # last hunk last parent line is not eaten
+                        current_hunks[version_id][1] -= 1
+                    if current_hunks[version_id][1] < 0:
+                        current_hunks[version_id][1] = 0
+                        # import pdb;pdb.set_trace()
+                    # assert current_hunks[version_id][1] >= 0
+
+        # flush last hunk
+        for i in range(nv):
+            version = self._idx_to_name(i)
+            if current_hunks[version] != [0, 0, 0, []]:
+                diff_hunks[version].append(tuple(current_hunks[version]))
+        result = {}
+        for version_id in version_ids:
+            result[version_id] = (
+                                  parents[version_id],
+                                  sha1s[version_id],
+                                  noeols[version_id],
+                                  diff_hunks[version_id],
+                                  )
+        return result
+
+    def get_parents(self, version_id):
+        """See VersionedFile.get_parent."""
+        return map(self._idx_to_name, self._parents[self._lookup(version_id)])
+
+    def _check_repeated_add(self, name, parents, text, sha1):
+        """Check that a duplicated add is OK.
+
+        If it is, return the (old) index; otherwise raise an exception.
+        """
+        idx = self._lookup(name)
+        if sorted(self._parents[idx]) != sorted(parents) \
+            or sha1 != self._sha1s[idx]:
+            raise RevisionAlreadyPresent(name, self._weave_name)
+        return idx
+
+    @deprecated_method(zero_eight)
+    def add_identical(self, old_rev_id, new_rev_id, parents):
+        """Please use Weave.clone_text now."""
+        return self.clone_text(new_rev_id, old_rev_id, parents)
+
+    def _add_lines(self, version_id, parents, lines, parent_texts):
+        """See VersionedFile.add_lines."""
+        return self._add(version_id, lines, map(self._lookup, parents))
+
+    @deprecated_method(zero_eight)
+    def add(self, name, parents, text, sha1=None):
+        """See VersionedFile.add_lines for the non deprecated api."""
+        return self._add(name, text, map(self._maybe_lookup, parents), sha1)
+
+    def _add(self, version_id, lines, parents, sha1=None):
         """Add a single text on top of the weave.
   
         Returns the index number of the newly added version.
 
-        name
+        version_id
             Symbolic name for this version.
             (Typically the revision-id of the revision that added it.)
 
         parents
             List or set of direct parent version numbers.
             
-        text
-            Sequence of lines to be added in the new version."""
-
-        assert isinstance(name, basestring)
-        if name in self._name_map:
-            raise WeaveError("name %r already present in weave" % name)
-        
+        lines
+            Sequence of lines to be added in the new version.
+        """
+
+        assert isinstance(version_id, basestring)
+        if not sha1:
+            sha1 = sha_strings(lines)
+        if version_id in self._name_map:
+            return self._check_repeated_add(version_id, parents, lines, sha1)
+
         self._check_versions(parents)
-        ## self._check_lines(text)
+        ## self._check_lines(lines)
         new_version = len(self._parents)
 
-        s = sha.new()
-        map(s.update, text)
-        sha1 = s.hexdigest()
-        del s
-
         # if we abort after here the (in-memory) weave will be corrupt because only
         # some fields are updated
+        # XXX: FIXME implement a succeed-or-fail of the rest of this routine.
+        #      - Robert Collins 20060226
         self._parents.append(parents[:])
         self._sha1s.append(sha1)
-        self._names.append(name)
-        self._name_map[name] = new_version
+        self._names.append(version_id)
+        self._name_map[version_id] = new_version
 
             
         if not parents:
@@ -253,11 +487,10 @@
             # this more quickly by just appending unconditionally.
             # even more specially, if we're adding an empty text we
             # need do nothing at all.
-            if text:
+            if lines:
                 self._weave.append(('{', new_version))
-                self._weave.extend(text)
+                self._weave.extend(lines)
                 self._weave.append(('}', None))
-        
             return new_version
 
         if len(parents) == 1:
@@ -267,7 +500,7 @@
                 return new_version
             
 
-        ancestors = self.inclusions(parents)
+        ancestors = self._inclusions(parents)
 
         l = self._weave
 
@@ -280,7 +513,7 @@
 
         # another small special case: a merge, producing the same text
         # as auto-merge
-        if text == basis_lines:
+        if lines == basis_lines:
             return new_version            
 
         # add a sentinal, because we can also match against the final line
@@ -293,8 +526,7 @@
         #print 'basis_lines:', basis_lines
         #print 'new_lines:  ', lines
 
-        from difflib import SequenceMatcher
-        s = SequenceMatcher(None, basis_lines, text)
+        s = SequenceMatcher(None, basis_lines, lines)
 
         # offset gives the number of lines that have been inserted
         # into the weave up to the current point; if the original edit instruction
@@ -311,7 +543,7 @@
             i1 = basis_lineno[i1]
             i2 = basis_lineno[i2]
 
-            assert 0 <= j1 <= j2 <= len(text)
+            assert 0 <= j1 <= j2 <= len(lines)
 
             #print tag, i1, i2, j1, j2
 
@@ -328,50 +560,45 @@
                 # we don't destroy ourselves
                 i = i2 + offset
                 self._weave[i:i] = ([('{', new_version)] 
-                                    + text[j1:j2] 
+                                    + lines[j1:j2] 
                                     + [('}', None)])
                 offset += 2 + (j2 - j1)
-
         return new_version
 
+    def _clone_text(self, new_version_id, old_version_id, parents):
+        """See VersionedFile.clone_text."""
+        old_lines = self.get_text(old_version_id)
+        self.add_lines(new_version_id, parents, old_lines)
 
-    def inclusions(self, versions):
+    def _inclusions(self, versions):
         """Return set of all ancestors of given version(s)."""
+        if not len(versions):
+            return []
         i = set(versions)
-        v = max(versions)
-        try:
-            while v >= 0:
-                if v in i:
-                    # include all its parents
-                    i.update(self._parents[v])
-                v -= 1
-            return i
-        except IndexError:
-            raise ValueError("version %d not present in weave" % v)
-
-
-    def minimal_parents(self, version):
-        """Find the minimal set of parents for the version."""
-        included = self._parents[version]
-        if not included:
+        for v in xrange(max(versions), 0, -1):
+            if v in i:
+                # include all its parents
+                i.update(self._parents[v])
+        return i
+        ## except IndexError:
+        ##     raise ValueError("version %d not present in weave" % v)
+
+    @deprecated_method(zero_eight)
+    def inclusions(self, version_ids):
+        """Deprecated - see VersionedFile.get_ancestry for the replacement."""
+        if not version_ids:
             return []
-        
-        li = list(included)
-        li.sort(reverse=True)
-
-        mininc = []
-        gotit = set()
-
-        for pv in li:
-            if pv not in gotit:
-                mininc.append(pv)
-                gotit.update(self.inclusions(pv))
-
-        assert mininc[0] >= 0
-        assert mininc[-1] < version
-        return mininc
-
-
+        if isinstance(version_ids[0], int):
+            return [self._idx_to_name(v) for v in self._inclusions(version_ids)]
+        else:
+            return self.get_ancestry(version_ids)
+
+    def get_ancestry(self, version_ids):
+        """See VersionedFile.get_ancestry."""
+        if isinstance(version_ids, basestring):
+            version_ids = [version_ids]
+        i = self._inclusions([self._lookup(v) for v in version_ids])
+        return [self._idx_to_name(v) for v in i]
 
     def _check_lines(self, text):
         if not isinstance(text, list):
@@ -392,26 +619,63 @@
             except IndexError:
                 raise IndexError("invalid version number %r" % i)
 
+    def _compatible_parents(self, my_parents, other_parents):
+        """During join check that other_parents are joinable with my_parents.
+
+        Joinable is defined as 'is a subset of' - supersets may require 
+        regeneration of diffs, but subsets do not.
+        """
+        return len(other_parents.difference(my_parents)) == 0
+
+    def annotate(self, version_id):
+        if isinstance(version_id, int):
+            warn('Weave.annotate(int) is deprecated. Please use version names'
+                 ' in all circumstances as of 0.8',
+                 DeprecationWarning,
+                 stacklevel=2
+                 )
+            result = []
+            for origin, lineno, text in self._extract([version_id]):
+                result.append((origin, text))
+            return result
+        else:
+            return super(Weave, self).annotate(version_id)
     
-    def annotate(self, index):
-        return list(self.annotate_iter(index))
-
-
-    def annotate_iter(self, version):
-        """Yield list of (index-id, line) pairs for the specified version.
+    def annotate_iter(self, version_id):
+        """Yield list of (version-id, line) pairs for the specified version.
 
         The index indicates when the line originated in the weave."""
-        for origin, lineno, text in self._extract([version]):
-            yield origin, text
-
-
+        incls = [self._lookup(version_id)]
+        for origin, lineno, text in self._extract(incls):
+            yield self._idx_to_name(origin), text
+
+    @deprecated_method(zero_eight)
     def _walk(self):
-        """Walk the weave.
-
-        Yields sequence of
-        (lineno, insert, deletes, text)
-        for each literal line.
-        """
+        """_walk has become visit, a supported api."""
+        return self._walk_internal()
+
+    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+        """See VersionedFile.iter_lines_added_or_present_in_versions()."""
+        if version_ids is None:
+            version_ids = self.versions()
+        version_ids = set(version_ids)
+        for lineno, inserted, deletes, line in self._walk_internal(version_ids):
+            # if inserted not in version_ids then it was inserted before the
+            # versions we care about, but because weaves cannot represent ghosts
+            # properly, we do not filter down to that
+            # if inserted not in version_ids: continue
+            if line[-1] != '\n':
+                yield line + '\n'
+            else:
+                yield line
+
+    #@deprecated_method(zero_eight)
+    def walk(self, version_ids=None):
+        """See VersionedFile.walk."""
+        return self._walk_internal(version_ids)
+
+    def _walk_internal(self, version_ids=None):
+        """Helper method for weave actions."""
         
         istack = []
         dset = set()
@@ -419,28 +683,79 @@
         lineno = 0         # line of weave, 0-based
 
         for l in self._weave:
-            if isinstance(l, tuple):
+            if l.__class__ == tuple:
                 c, v = l
                 isactive = None
                 if c == '{':
-                    istack.append(v)
+                    istack.append(self._names[v])
                 elif c == '}':
                     istack.pop()
                 elif c == '[':
-                    assert v not in dset
-                    dset.add(v)
+                    assert self._names[v] not in dset
+                    dset.add(self._names[v])
                 elif c == ']':
-                    dset.remove(v)
+                    dset.remove(self._names[v])
                 else:
-                    raise WeaveFormatError('unexpected instruction %r'
-                                           % v)
+                    raise WeaveFormatError('unexpected instruction %r' % v)
             else:
-                assert isinstance(l, basestring)
+                assert l.__class__ in (str, unicode)
                 assert istack
-                yield lineno, istack[-1], dset, l
+                yield lineno, istack[-1], frozenset(dset), l
             lineno += 1
 
-
+        if istack:
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
+        if dset:
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
+
+    def plan_merge(self, ver_a, ver_b):
+        """Return pseudo-annotation indicating how the two versions merge.
+
+        This is computed between versions a and b and their common
+        base.
+
+        Weave lines present in none of them are skipped entirely.
+        """
+        inc_a = set(self.get_ancestry([ver_a]))
+        inc_b = set(self.get_ancestry([ver_b]))
+        inc_c = inc_a & inc_b
+
+        for lineno, insert, deleteset, line in\
+            self.walk([ver_a, ver_b]):
+            if deleteset & inc_c:
+                # killed in parent; can't be in either a or b
+                # not relevant to our work
+                yield 'killed-base', line
+            elif insert in inc_c:
+                # was inserted in base
+                killed_a = bool(deleteset & inc_a)
+                killed_b = bool(deleteset & inc_b)
+                if killed_a and killed_b:
+                    yield 'killed-both', line
+                elif killed_a:
+                    yield 'killed-a', line
+                elif killed_b:
+                    yield 'killed-b', line
+                else:
+                    yield 'unchanged', line
+            elif insert in inc_a:
+                if deleteset & inc_a:
+                    yield 'ghost-a', line
+                else:
+                    # new in A; not in B
+                    yield 'new-a', line
+            elif insert in inc_b:
+                if deleteset & inc_b:
+                    yield 'ghost-b', line
+                else:
+                    yield 'new-b', line
+            else:
+                # not in either revision
+                yield 'irrelevant', line
+
+        yield 'unchanged', ''           # terminator
 
     def _extract(self, versions):
         """Yield annotation of lines in included set.
@@ -451,9 +766,14 @@
 
         The set typically but not necessarily corresponds to a version.
         """
-        included = self.inclusions(versions)
+        for i in versions:
+            if not isinstance(i, int):
+                raise ValueError(i)
+            
+        included = self._inclusions(versions)
 
         istack = []
+        iset = set()
         dset = set()
 
         lineno = 0         # line of weave, 0-based
@@ -464,15 +784,43 @@
 
         WFE = WeaveFormatError
 
+        # wow. 
+        #  449       0   4474.6820   2356.5590   bzrlib.weave:556(_extract)
+        #  +285282   0   1676.8040   1676.8040   +<isinstance>
+        # 1.6 seconds in 'isinstance'.
+        # changing the first isinstance:
+        #  449       0   2814.2660   1577.1760   bzrlib.weave:556(_extract)
+        #  +140414   0    762.8050    762.8050   +<isinstance>
+        # note that the inline time actually dropped (less function calls)
+        # and total processing time was halved.
+        # we're still spending ~1/4 of the method in isinstance though.
+        # so lets hard code the acceptable string classes we expect:
+        #  449       0   1202.9420    786.2930   bzrlib.weave:556(_extract)
+        # +71352     0    377.5560    377.5560   +<method 'append' of 'list' 
+        #                                          objects>
+        # yay, down to ~1/4 the initial extract time, and our inline time
+        # has shrunk again, with isinstance no longer dominating.
+        # tweaking the stack inclusion test to use a set gives:
+        #  449       0   1122.8030    713.0080   bzrlib.weave:556(_extract)
+        # +71352     0    354.9980    354.9980   +<method 'append' of 'list' 
+        #                                          objects>
+        # - a 5% win, or possibly just noise. However with large istacks that
+        # 'in' test could dominate, so I'm leaving this change in place -
+        # when its fast enough to consider profiling big datasets we can review.
+
+              
+             
+
         for l in self._weave:
-            if isinstance(l, tuple):
+            if l.__class__ == tuple:
                 c, v = l
                 isactive = None
                 if c == '{':
-                    assert v not in istack
+                    assert v not in iset
                     istack.append(v)
+                    iset.add(v)
                 elif c == '}':
-                    istack.pop()
+                    iset.remove(istack.pop())
                 elif c == '[':
                     if v in included:
                         assert v not in dset
@@ -483,62 +831,89 @@
                         assert v in dset
                         dset.remove(v)
             else:
-                assert isinstance(l, basestring)
+                assert l.__class__ in (str, unicode)
                 if isactive is None:
                     isactive = (not dset) and istack and (istack[-1] in included)
                 if isactive:
                     result.append((istack[-1], lineno, l))
             lineno += 1
-
         if istack:
-            raise WFE("unclosed insertion blocks at end of weave",
-                                   istack)
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
         if dset:
-            raise WFE("unclosed deletion blocks at end of weave",
-                                   dset)
-
-        return result
-    
-
-
-    def get_iter(self, version):
-        """Yield lines for the specified version."""
-        for origin, lineno, line in self._extract([version]):
-            yield line
-
-
-    def get(self, index):
-        return list(self.get_iter(index))
-
-
-    def mash_iter(self, included):
-        """Return composed version of multiple included versions."""
-        for origin, lineno, text in self._extract(included):
-            yield text
-
-
-    def dump(self, to_file):
-        from pprint import pprint
-        print >>to_file, "Weave._weave = ",
-        pprint(self._weave, to_file)
-        print >>to_file, "Weave._parents = ",
-        pprint(self._parents, to_file)
-
-
-
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
+        return result
+
+    @deprecated_method(zero_eight)
+    def get_iter(self, name_or_index):
+        """Deprecated, please do not use. Lookups are not not needed.
+        
+        Please use get_lines now.
+        """
+        return iter(self.get_lines(self._maybe_lookup(name_or_index)))
+
+    @deprecated_method(zero_eight)
+    def maybe_lookup(self, name_or_index):
+        """Deprecated, please do not use. Lookups are not not needed."""
+        return self._maybe_lookup(name_or_index)
+
+    def _maybe_lookup(self, name_or_index):
+        """Convert possible symbolic name to index, or pass through indexes.
+        
+        NOT FOR PUBLIC USE.
+        """
+        if isinstance(name_or_index, (int, long)):
+            return name_or_index
+        else:
+            return self._lookup(name_or_index)
+
+    @deprecated_method(zero_eight)
+    def get(self, version_id):
+        """Please use either Weave.get_text or Weave.get_lines as desired."""
+        return self.get_lines(version_id)
+
+    def get_lines(self, version_id):
+        """See VersionedFile.get_lines()."""
+        int_index = self._maybe_lookup(version_id)
+        result = [line for (origin, lineno, line) in self._extract([int_index])]
+        expected_sha1 = self._sha1s[int_index]
+        measured_sha1 = sha_strings(result)
+        if measured_sha1 != expected_sha1:
+            raise errors.WeaveInvalidChecksum(
+                    'file %s, revision %s, expected: %s, measured %s' 
+                    % (self._weave_name, version_id,
+                       expected_sha1, measured_sha1))
+        return result
+
+    def get_sha1(self, name):
+        """Get the stored sha1 sum for the given revision.
+        
+        :param name: The name of the version to lookup
+        """
+        return self._sha1s[self._lookup(name)]
+
+    @deprecated_method(zero_eight)
     def numversions(self):
+        """How many versions are in this weave?
+
+        Deprecated in favour of num_versions.
+        """
+        return self.num_versions()
+
+    def num_versions(self):
+        """How many versions are in this weave?"""
         l = len(self._parents)
         assert l == len(self._sha1s)
         return l
 
-
-    def __len__(self):
-        return self.numversions()
-
+    __len__ = num_versions
 
     def check(self, progress_bar=None):
-        # check no circular inclusions
-        for version in range(self.numversions()):
+        # TODO evaluate performance hit of using string sets in this routine.
+        # TODO: check no circular inclusions
+        # TODO: create a nested progress bar
+        for version in range(self.num_versions()):
             inclusions = list(self._parents[version])
             if inclusions:
                 inclusions.sort()
@@ -546,167 +921,304 @@
                     raise WeaveFormatError("invalid included version %d for index %d"
                                            % (inclusions[-1], version))
 
-        # try extracting all versions; this is a bit slow and parallel
-        # extraction could be used
-        nv = self.numversions()
-        for version in range(nv):
+        # try extracting all versions; parallel extraction is used
+        nv = self.num_versions()
+        sha1s = {}
+        texts = {}
+        inclusions = {}
+        for i in range(nv):
+            # For creating the ancestry, IntSet is much faster (3.7s vs 0.17s)
+            # The problem is that set membership is much more expensive
+            name = self._idx_to_name(i)
+            sha1s[name] = sha.new()
+            texts[name] = []
+            new_inc = set([name])
+            for p in self._parents[i]:
+                new_inc.update(inclusions[self._idx_to_name(p)])
+
+            assert set(new_inc) == set(self.get_ancestry(name)), \
+                'failed %s != %s' % (set(new_inc), set(self.get_ancestry(name)))
+            inclusions[name] = new_inc
+
+        nlines = len(self._weave)
+
+        update_text = 'checking weave'
+        if self._weave_name:
+            short_name = os.path.basename(self._weave_name)
+            update_text = 'checking %s' % (short_name,)
+            update_text = update_text[:25]
+
+        for lineno, insert, deleteset, line in self._walk_internal():
             if progress_bar:
-                progress_bar.update('checking text', version, nv)
-            s = sha.new()
-            for l in self.get_iter(version):
-                s.update(l)
-            hd = s.hexdigest()
-            expected = self._sha1s[version]
+                progress_bar.update(update_text, lineno, nlines)
+
+            for name, name_inclusions in inclusions.items():
+                # The active inclusion must be an ancestor,
+                # and no ancestors must have deleted this line,
+                # because we don't support resurrection.
+                if (insert in name_inclusions) and not (deleteset & name_inclusions):
+                    sha1s[name].update(line)
+
+        for i in range(nv):
+            version = self._idx_to_name(i)
+            hd = sha1s[version].hexdigest()
+            expected = self._sha1s[i]
             if hd != expected:
-                raise WeaveError("mismatched sha1 for version %d; "
-                                 "got %s, expected %s"
-                                 % (version, hd, expected))
+                raise errors.WeaveInvalidChecksum(
+                        "mismatched sha1 for version %s: "
+                        "got %s, expected %s"
+                        % (version, hd, expected))
 
         # TODO: check insertions are properly nested, that there are
         # no lines outside of insertion blocks, that deletions are
         # properly paired, etc.
 
-
-
-    def merge(self, merge_versions):
-        """Automerge and mark conflicts between versions.
-
-        This returns a sequence, each entry describing alternatives
-        for a chunk of the file.  Each of the alternatives is given as
-        a list of lines.
-
-        If there is a chunk of the file where there's no diagreement,
-        only one alternative is given.
-        """
-
-        # approach: find the included versions common to all the
-        # merged versions
-        raise NotImplementedError()
-
-
-
-    def _delta(self, included, lines):
-        """Return changes from basis to new revision.
-
-        The old text for comparison is the union of included revisions.
-
-        This is used in inserting a new text.
-
-        Delta is returned as a sequence of
-        (weave1, weave2, newlines).
-
-        This indicates that weave1:weave2 of the old weave should be
-        replaced by the sequence of lines in newlines.  Note that
-        these line numbers are positions in the total weave and don't
-        correspond to the lines in any extracted version, or even the
-        extracted union of included versions.
-
-        If line1=line2, this is a pure insert; if newlines=[] this is a
-        pure delete.  (Similar to difflib.)
-        """
-
-
-            
-    def plan_merge(self, ver_a, ver_b):
-        """Return pseudo-annotation indicating how the two versions merge.
-
-        This is computed between versions a and b and their common
-        base.
-
-        Weave lines present in none of them are skipped entirely.
-        """
-        inc_a = self.inclusions([ver_a])
-        inc_b = self.inclusions([ver_b])
-        inc_c = inc_a & inc_b
-
-        for lineno, insert, deleteset, line in self._walk():
-            if deleteset & inc_c:
-                # killed in parent; can't be in either a or b
-                # not relevant to our work
-                yield 'killed-base', line
-            elif insert in inc_c:
-                # was inserted in base
-                killed_a = bool(deleteset & inc_a)
-                killed_b = bool(deleteset & inc_b)
-                if killed_a and killed_b:
-                    yield 'killed-both', line
-                elif killed_a:
-                    yield 'killed-a', line
-                elif killed_b:
-                    yield 'killed-b', line
-                else:
-                    yield 'unchanged', line
-            elif insert in inc_a:
-                if deleteset & inc_a:
-                    yield 'ghost-a', line
-                else:
-                    # new in A; not in B
-                    yield 'new-a', line
-            elif insert in inc_b:
-                if deleteset & inc_b:
-                    yield 'ghost-b', line
-                else:
-                    yield 'new-b', line
-            else:
-                # not in either revision
-                yield 'irrelevant', line
-
-        yield 'unchanged', ''           # terminator
-
-
-
-    def weave_merge(self, plan):
-        lines_a = []
-        lines_b = []
-        ch_a = ch_b = False
-
-        for state, line in plan:
-            if state == 'unchanged' or state == 'killed-both':
-                # resync and flush queued conflicts changes if any
-                if not lines_a and not lines_b:
-                    pass
-                elif ch_a and not ch_b:
-                    # one-sided change:                    
-                    for l in lines_a: yield l
-                elif ch_b and not ch_a:
-                    for l in lines_b: yield l
-                elif lines_a == lines_b:
-                    for l in lines_a: yield l
-                else:
-                    yield '<<<<\n'
-                    for l in lines_a: yield l
-                    yield '====\n'
-                    for l in lines_b: yield l
-                    yield '>>>>\n'
-
-                del lines_a[:]
-                del lines_b[:]
-                ch_a = ch_b = False
-                
-            if state == 'unchanged':
-                if line:
-                    yield line
-            elif state == 'killed-a':
-                ch_a = True
-                lines_b.append(line)
-            elif state == 'killed-b':
-                ch_b = True
-                lines_a.append(line)
-            elif state == 'new-a':
-                ch_a = True
-                lines_a.append(line)
-            elif state == 'new-b':
-                ch_b = True
-                lines_b.append(line)
-            else:
-                assert state in ('irrelevant', 'ghost-a', 'ghost-b', 'killed-base',
-                                 'killed-both'), \
-                       state
-
-                
-
-
-
+    def _join(self, other, pb, msg, version_ids, ignore_missing):
+        """Worker routine for join()."""
+        if not other.versions():
+            return          # nothing to update, easy
+
+        if version_ids:
+            for version_id in version_ids:
+                if not other.has_version(version_id) and not ignore_missing:
+                    raise RevisionNotPresent(version_id, self._weave_name)
+        else:
+            version_ids = other.versions()
+
+        # two loops so that we do not change ourselves before verifying it
+        # will be ok
+        # work through in index order to make sure we get all dependencies
+        names_to_join = []
+        processed = 0
+        # get the selected versions only that are in other.versions.
+        version_ids = set(other.versions()).intersection(set(version_ids))
+        # pull in the referenced graph.
+        version_ids = other.get_ancestry(version_ids)
+        pending_graph = [(version, other.get_parents(version)) for
+                         version in version_ids]
+        for name in topo_sort(pending_graph):
+            other_idx = other._name_map[name]
+            # returns True if we have it, False if we need it.
+            if not self._check_version_consistent(other, other_idx, name):
+                names_to_join.append((other_idx, name))
+            processed += 1
+
+
+        if pb and not msg:
+            msg = 'weave join'
+
+        merged = 0
+        time0 = time.time()
+        for other_idx, name in names_to_join:
+            # TODO: If all the parents of the other version are already
+            # present then we can avoid some work by just taking the delta
+            # and adjusting the offsets.
+            new_parents = self._imported_parents(other, other_idx)
+            sha1 = other._sha1s[other_idx]
+
+            merged += 1
+
+            if pb:
+                pb.update(msg, merged, len(names_to_join))
+           
+            lines = other.get_lines(other_idx)
+            self._add(name, lines, new_parents, sha1)
+
+        mutter("merged = %d, processed = %d, file_id=%s; deltat=%d"%(
+                merged, processed, self._weave_name, time.time()-time0))
+ 
+    def _imported_parents(self, other, other_idx):
+        """Return list of parents in self corresponding to indexes in other."""
+        new_parents = []
+        for parent_idx in other._parents[other_idx]:
+            parent_name = other._names[parent_idx]
+            if parent_name not in self._name_map:
+                # should not be possible
+                raise WeaveError("missing parent {%s} of {%s} in %r" 
+                                 % (parent_name, other._name_map[other_idx], self))
+            new_parents.append(self._name_map[parent_name])
+        return new_parents
+
+    def _check_version_consistent(self, other, other_idx, name):
+        """Check if a version in consistent in this and other.
+
+        To be consistent it must have:
+
+         * the same text
+         * the same direct parents (by name, not index, and disregarding
+           order)
+        
+        If present & correct return True;
+        if not present in self return False; 
+        if inconsistent raise error."""
+        this_idx = self._name_map.get(name, -1)
+        if this_idx != -1:
+            if self._sha1s[this_idx] != other._sha1s[other_idx]:
+                raise errors.WeaveTextDiffers(name, self, other)
+            self_parents = self._parents[this_idx]
+            other_parents = other._parents[other_idx]
+            n1 = set([self._names[i] for i in self_parents])
+            n2 = set([other._names[i] for i in other_parents])
+            if not self._compatible_parents(n1, n2):
+                raise WeaveParentMismatch("inconsistent parents "
+                    "for version {%s}: %s vs %s" % (name, n1, n2))
+            else:
+                return True         # ok!
+        else:
+            return False
+
+    @deprecated_method(zero_eight)
+    def reweave(self, other, pb=None, msg=None):
+        """reweave has been superceded by plain use of join."""
+        return self.join(other, pb, msg)
+
+    def _reweave(self, other, pb, msg):
+        """Reweave self with other - internal helper for join().
+
+        :param other: The other weave to merge
+        :param pb: An optional progress bar, indicating how far done we are
+        :param msg: An optional message for the progress
+        """
+        new_weave = _reweave(self, other, pb=pb, msg=msg)
+        self._copy_weave_content(new_weave)
+
+    def _copy_weave_content(self, otherweave):
+        """adsorb the content from otherweave."""
+        for attr in self.__slots__:
+            if attr != '_weave_name':
+                setattr(self, attr, copy(getattr(otherweave, attr)))
+
+
+class WeaveFile(Weave):
+    """A WeaveFile represents a Weave on disk and writes on change."""
+
+    WEAVE_SUFFIX = '.weave'
+    
+    def __init__(self, name, transport, filemode=None, create=False, access_mode='w'):
+        """Create a WeaveFile.
+        
+        :param create: If not True, only open an existing knit.
+        """
+        super(WeaveFile, self).__init__(name, access_mode)
+        self._transport = transport
+        self._filemode = filemode
+        try:
+            _read_weave_v5(self._transport.get(name + WeaveFile.WEAVE_SUFFIX), self)
+        except errors.NoSuchFile:
+            if not create:
+                raise
+            # new file, save it
+            self._save()
+
+    def _add_lines(self, version_id, parents, lines, parent_texts):
+        """Add a version and save the weave."""
+        result = super(WeaveFile, self)._add_lines(version_id, parents, lines,
+                                                   parent_texts)
+        self._save()
+        return result
+
+    def _clone_text(self, new_version_id, old_version_id, parents):
+        """See VersionedFile.clone_text."""
+        super(WeaveFile, self)._clone_text(new_version_id, old_version_id, parents)
+        self._save
+
+    def copy_to(self, name, transport):
+        """See VersionedFile.copy_to()."""
+        # as we are all in memory always, just serialise to the new place.
+        sio = StringIO()
+        write_weave_v5(self, sio)
+        sio.seek(0)
+        transport.put(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
+
+    def create_empty(self, name, transport, filemode=None):
+        return WeaveFile(name, transport, filemode, create=True)
+
+    def _save(self):
+        """Save the weave."""
+        self._check_write_ok()
+        sio = StringIO()
+        write_weave_v5(self, sio)
+        sio.seek(0)
+        self._transport.put(self._weave_name + WeaveFile.WEAVE_SUFFIX,
+                            sio,
+                            self._filemode)
+
+    @staticmethod
+    def get_suffixes():
+        """See VersionedFile.get_suffixes()."""
+        return [WeaveFile.WEAVE_SUFFIX]
+
+    def join(self, other, pb=None, msg=None, version_ids=None,
+             ignore_missing=False):
+        """Join other into self and save."""
+        super(WeaveFile, self).join(other, pb, msg, version_ids, ignore_missing)
+        self._save()
+
+
+@deprecated_function(zero_eight)
+def reweave(wa, wb, pb=None, msg=None):
+    """reweaving is deprecation, please just use weave.join()."""
+    _reweave(wa, wb, pb, msg)
+
+def _reweave(wa, wb, pb=None, msg=None):
+    """Combine two weaves and return the result.
+
+    This works even if a revision R has different parents in 
+    wa and wb.  In the resulting weave all the parents are given.
+
+    This is done by just building up a new weave, maintaining ordering 
+    of the versions in the two inputs.  More efficient approaches
+    might be possible but it should only be necessary to do 
+    this operation rarely, when a new previously ghost version is 
+    inserted.
+
+    :param pb: An optional progress bar, indicating how far done we are
+    :param msg: An optional message for the progress
+    """
+    wr = Weave()
+    ia = ib = 0
+    queue_a = range(wa.num_versions())
+    queue_b = range(wb.num_versions())
+    # first determine combined parents of all versions
+    # map from version name -> all parent names
+    combined_parents = _reweave_parent_graphs(wa, wb)
+    mutter("combined parents: %r", combined_parents)
+    order = topo_sort(combined_parents.iteritems())
+    mutter("order to reweave: %r", order)
+
+    if pb and not msg:
+        msg = 'reweave'
+
+    for idx, name in enumerate(order):
+        if pb:
+            pb.update(msg, idx, len(order))
+        if name in wa._name_map:
+            lines = wa.get_lines(name)
+            if name in wb._name_map:
+                lines_b = wb.get_lines(name)
+                if lines != lines_b:
+                    mutter('Weaves differ on content. rev_id {%s}', name)
+                    mutter('weaves: %s, %s', wa._weave_name, wb._weave_name)
+                    import difflib
+                    lines = list(difflib.unified_diff(lines, lines_b,
+                            wa._weave_name, wb._weave_name))
+                    mutter('lines:\n%s', ''.join(lines))
+                    raise errors.WeaveTextDiffers(name, wa, wb)
+        else:
+            lines = wb.get_lines(name)
+        wr._add(name, lines, [wr._lookup(i) for i in combined_parents[name]])
+    return wr
+
+def _reweave_parent_graphs(wa, wb):
+    """Return combined parent ancestry for two weaves.
+    
+    Returned as a list of (version_name, set(parent_names))"""
+    combined = {}
+    for weave in [wa, wb]:
+        for idx, name in enumerate(weave._names):
+            p = combined.setdefault(name, set())
+            p.update(map(weave._idx_to_name, weave._parents[idx]))
+    return combined
 
 
 def weave_toc(w):
@@ -715,7 +1227,7 @@
     for i in (6, 50, 10, 10):
         print '-' * i,
     print
-    for i in range(w.numversions()):
+    for i in range(w.num_versions()):
         sha1 = w._sha1s[i]
         name = w._names[i]
         parent_str = ' '.join(map(str, w._parents[i]))
@@ -723,14 +1235,11 @@
 
 
 
-def weave_stats(weave_file):
-    from bzrlib.progress import ProgressBar
+def weave_stats(weave_file, pb):
     from bzrlib.weavefile import read_weave
 
-    pb = ProgressBar()
-
     wf = file(weave_file, 'rb')
-    w = read_weave(wf)
+    w = read_weave(wf, WeaveVersionedFile)
     # FIXME: doesn't work on pipes
     weave_size = wf.tell()
 
@@ -738,7 +1247,7 @@
     vers = len(w)
     for i in range(vers):
         pb.update('checking sizes', i, vers)
-        for line in w.get_iter(i):
+        for origin, lineno, line in w._extract([i]):
             total += len(line)
 
     pb.clear()
@@ -771,10 +1280,10 @@
         Add NEWTEXT, with specified parent versions.
     weave annotate WEAVEFILE VERSION
         Display origin of each line.
-    weave mash WEAVEFILE VERSION...
-        Display composite of all selected versions.
     weave merge WEAVEFILE VERSION1 VERSION2 > OUT
         Auto-merge two versions and display conflicts.
+    weave diff WEAVEFILE VERSION1 VERSION2 
+        Show differences between two versions.
 
 example:
 
@@ -805,7 +1314,13 @@
 def main(argv):
     import sys
     import os
-    from weavefile import write_weave, read_weave
+    try:
+        import bzrlib
+    except ImportError:
+        # in case we're run directly from the subdirectory
+        sys.path.append('..')
+        import bzrlib
+    from bzrlib.weavefile import write_weave, read_weave
     from bzrlib.progress import ProgressBar
 
     try:
@@ -844,10 +1359,18 @@
         w = readit()
         sys.stdout.writelines(w.get_iter(int(argv[3])))
         
-    elif cmd == 'mash': # get composite
+    elif cmd == 'diff':
+        from difflib import unified_diff
         w = readit()
-        sys.stdout.writelines(w.mash_iter(map(int, argv[3:])))
-
+        fn = argv[2]
+        v1, v2 = map(int, argv[3:5])
+        lines1 = w.get(v1)
+        lines2 = w.get(v2)
+        diff_gen = unified_diff(lines1, lines2,
+                                '%s version %d' % (fn, v1),
+                                '%s version %d' % (fn, v2))
+        sys.stdout.writelines(diff_gen)
+            
     elif cmd == 'annotate':
         w = readit()
         # newline is added to all lines regardless; too hard to get
@@ -865,14 +1388,14 @@
         weave_toc(readit())
 
     elif cmd == 'stats':
-        weave_stats(argv[2])
+        weave_stats(argv[2], ProgressBar())
         
     elif cmd == 'check':
         w = readit()
         pb = ProgressBar()
         w.check(pb)
         pb.clear()
-        print '%d versions ok' % w.numversions()
+        print '%d versions ok' % w.num_versions()
 
     elif cmd == 'inclusions':
         w = readit()
@@ -883,42 +1406,22 @@
         print ' '.join(map(str, w._parents[int(argv[3])]))
 
     elif cmd == 'plan-merge':
+        # replaced by 'bzr weave-plan-merge'
         w = readit()
         for state, line in w.plan_merge(int(argv[3]), int(argv[4])):
             if line:
                 print '%14s | %s' % (state, line),
-
     elif cmd == 'merge':
+        # replaced by 'bzr weave-merge-text'
         w = readit()
         p = w.plan_merge(int(argv[3]), int(argv[4]))
         sys.stdout.writelines(w.weave_merge(p))
-            
-    elif cmd == 'mash-merge':
-        if len(argv) != 5:
-            usage()
-            return 1
-
-        w = readit()
-        v1, v2 = map(int, argv[3:5])
-
-        basis = w.inclusions([v1]).intersection(w.inclusions([v2]))
-
-        base_lines = list(w.mash_iter(basis))
-        a_lines = list(w.get(v1))
-        b_lines = list(w.get(v2))
-
-        from bzrlib.merge3 import Merge3
-        m3 = Merge3(base_lines, a_lines, b_lines)
-
-        name_a = 'version %d' % v1
-        name_b = 'version %d' % v2
-        sys.stdout.writelines(m3.merge_lines(name_a=name_a, name_b=name_b))
     else:
         raise ValueError('unknown command %r' % cmd)
     
 
 
-def profile_main(argv): 
+def profile_main(argv):
     import tempfile, hotshot, hotshot.stats
 
     prof_f = tempfile.NamedTemporaryFile()
@@ -938,12 +1441,52 @@
     return ret
 
 
+def lsprofile_main(argv): 
+    from bzrlib.lsprof import profile
+    ret,stats = profile(main, argv)
+    stats.sort()
+    stats.pprint()
+    return ret
+
+
 if __name__ == '__main__':
     import sys
     if '--profile' in sys.argv:
         args = sys.argv[:]
         args.remove('--profile')
         sys.exit(profile_main(args))
+    elif '--lsprof' in sys.argv:
+        args = sys.argv[:]
+        args.remove('--lsprof')
+        sys.exit(lsprofile_main(args))
     else:
         sys.exit(main(sys.argv))
 
+
+class InterWeave(InterVersionedFile):
+    """Optimised code paths for weave to weave operations."""
+    
+    _matching_file_factory = staticmethod(WeaveFile)
+    
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with weaves."""
+        try:
+            return (isinstance(source, Weave) and
+                    isinstance(target, Weave))
+        except AttributeError:
+            return False
+
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
+        """See InterVersionedFile.join."""
+        if self.target.versions() == []:
+            # optimised copy
+            self.target._copy_weave_content(self.source)
+            return
+        try:
+            self.target._join(self.source, pb, msg, version_ids, ignore_missing)
+        except errors.WeaveParentMismatch:
+            self.target._reweave(self.source, pb, msg)
+
+
+InterVersionedFile.register_optimiser(InterWeave)

=== added file 'bzrlib/weave_commands.py'
--- bzrlib/weave_commands.py	1970-01-01 00:00:00 +0000
+++ bzrlib/weave_commands.py	2006-03-24 11:12:04 +0000
@@ -0,0 +1,90 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""builtin bzr commands relating to individual weave files
+
+These should never normally need to be used by end users, but might be
+of interest in debugging or data recovery.
+"""
+
+import sys
+
+from bzrlib.commands import Command
+from bzrlib.trace import mutter, warning
+
+class cmd_weave_list(Command):
+    """List the revision ids present in a weave, in alphabetical order"""
+
+    hidden = True
+    takes_args = ['weave_filename']
+
+    def run(self, weave_filename):
+        from bzrlib.weavefile import read_weave
+        weave = read_weave(file(weave_filename, 'rb'))
+        names = weave.versions()
+        names.sort()
+        print '\n'.join(names)
+
+
+class cmd_weave_join(Command):
+    """Join the contents of two weave files.
+
+    The resulting weave is sent to stdout.
+
+    This command is only intended for bzr developer use.
+    """
+
+    hidden = True
+    takes_args = ['weave1', 'weave2']
+
+    def run(self, weave1, weave2):
+        from bzrlib.weavefile import read_weave, write_weave
+        w1 = read_weave(file(weave1, 'rb'))
+        w2 = read_weave(file(weave2, 'rb'))
+        w1.join(w2)
+        write_weave(w1, sys.stdout)
+
+
+class cmd_weave_plan_merge(Command):
+    """Show the plan for merging two versions within a weave"""
+    hidden = True
+    takes_args = ['weave_file', 'revision_a', 'revision_b']
+
+    def run(self, weave_file, revision_a, revision_b):
+        from bzrlib.weavefile import read_weave, write_weave
+        w = read_weave(file(weave_file, 'rb'))
+        for state, line in w.plan_merge(revision_a, revision_b):
+            # make sure to print every line with a newline, even if it doesn't
+            # really have one
+            if not line:
+                continue
+            if line[-1] != '\n':
+                state += '!eol'
+                line += '\n'
+            if '\n' in line[:-1]:
+                warning("line in weave contains embedded newline: %r" % line)
+            print '%15s | %s' % (state, line),
+
+class cmd_weave_merge_text(Command):
+    """Debugging command to merge two texts of a weave"""
+    hidden = True
+    takes_args = ['weave_file', 'revision_a', 'revision_b']
+
+    def run(self, weave_file, revision_a, revision_b):
+        from bzrlib.weavefile import read_weave, write_weave
+        w = read_weave(file(weave_file, 'rb'))
+        p = w.plan_merge(revision_a, revision_b)
+        sys.stdout.writelines(w.weave_merge(p))

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2005-08-18 07:51:58 +0000
+++ bzrlib/weavefile.py	2006-03-11 13:58:48 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/python
-
 # Copyright (C) 2005 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
@@ -95,39 +93,63 @@
 
 
 def read_weave(f):
-    return read_weave_v5(f)
-
-
-def read_weave_v5(f):
-    from weave import Weave, WeaveFormatError
-    w = Weave()
-
-    wfe = WeaveFormatError
-    l = f.readline()
+    # FIXME: detect the weave type and dispatch
+    from bzrlib.trace import mutter
+    from weave import Weave
+    w = Weave(getattr(f, 'name', None))
+    _read_weave_v5(f, w)
+    return w
+
+
+def _read_weave_v5(f, w):
+    """Private helper routine to read a weave format 5 file into memory.
+    
+    This is only to be used by read_weave and WeaveFile.__init__.
+    """
+    #  200   0   2075.5080   1084.0360   bzrlib.weavefile:104(_read_weave_v5)
+    # +60412 0    366.5900    366.5900   +<method 'readline' of 'file' objects>
+    # +59982 0    320.5280    320.5280   +<method 'startswith' of 'str' objects>
+    # +59363 0    297.8080    297.8080   +<method 'append' of 'list' objects>
+    # replace readline call with iter over all lines ->
+    # safe because we already suck on memory.
+    #  200   0   1492.7170    802.6220   bzrlib.weavefile:104(_read_weave_v5)
+    # +59982 0    329.9100    329.9100   +<method 'startswith' of 'str' objects>
+    # +59363 0    320.2980    320.2980   +<method 'append' of 'list' objects>
+    # replaced startswith with slice lookups:
+    #  200   0    851.7250    501.1120   bzrlib.weavefile:104(_read_weave_v5)
+    # +59363 0    311.8780    311.8780   +<method 'append' of 'list' objects>
+    # +200   0     30.2500     30.2500   +<method 'readlines' of 'file' objects>
+                  
+    from weave import WeaveFormatError
+
+    lines = iter(f.readlines())
+    
+    l = lines.next()
     if l != FORMAT_1:
         raise WeaveFormatError('invalid weave file header: %r' % l)
 
     ver = 0
+    # read weave header.
     while True:
-        l = f.readline()
+        l = lines.next()
         if l[0] == 'i':
             if len(l) > 2:
                 w._parents.append(map(int, l[2:].split(' ')))
             else:
                 w._parents.append([])
 
-            l = f.readline()[:-1]
-            assert l.startswith('1 ')
+            l = lines.next()[:-1]
+            assert '1 ' == l[0:2]
             w._sha1s.append(l[2:])
                 
-            l = f.readline()
-            assert l.startswith('n ')
+            l = lines.next()
+            assert 'n ' == l[0:2]
             name = l[2:-1]
             assert name not in w._name_map
             w._names.append(name)
             w._name_map[name] = ver
                 
-            l = f.readline()
+            l = lines.next()
             assert l == '\n'
 
             ver += 1
@@ -136,13 +158,14 @@
         else:
             raise WeaveFormatError('unexpected line %r' % l)
 
+    # read weave body
     while True:
-        l = f.readline()
+        l = lines.next()
         if l == 'W\n':
             break
-        elif l.startswith('. '):
+        elif '. ' == l[0:2]:
             w._weave.append(l[2:])  # include newline
-        elif l.startswith(', '):
+        elif ', ' == l[0:2]:
             w._weave.append(l[2:-1])        # exclude newline
         elif l == '}\n':
             w._weave.append(('}', None))

=== added file 'bzrlib/win32console.py'
--- bzrlib/win32console.py	1970-01-01 00:00:00 +0000
+++ bzrlib/win32console.py	2005-10-21 03:33:18 +0000
@@ -0,0 +1,40 @@
+
+"""
+Set of functions to work with console on Windows.
+Author: Alexander Belchenko (e-mail: bialix AT ukr.net)
+License: Public domain
+"""
+
+import struct
+
+try:
+   import ctypes
+except ImportError:
+   ctypes = None
+
+
+def get_console_size(defaultx=80, defaulty=25):
+   """ Return size of current console.
+
+   This function try to determine actual size of current working
+   console window and return tuple (sizex, sizey) if success,
+   or default size (defaultx, defaulty) otherwise.
+
+   Dependencies: ctypes should be installed.
+   """
+   if ctypes is None:
+       # no ctypes is found
+       return (defaultx, defaulty)
+
+   h = ctypes.windll.kernel32.GetStdHandle(-11)
+   csbi = ctypes.create_string_buffer(22)
+   res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
+
+   if res:
+       (bufx, bufy, curx, cury, wattr,
+        left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
+       sizex = right - left + 1
+       sizey = bottom - top + 1
+       return (sizex, sizey)
+   else:
+       return (defaultx, defaulty)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-07-22 19:32:52 +0000
+++ bzrlib/workingtree.py	2006-04-19 05:22:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,17 +14,181 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# TODO: Don't allow WorkingTrees to be constructed for remote branches.
-
-# FIXME: I don't know if writing out the cache from the destructor is really a
-# good idea, because destructors are considered poor taste in Python, and
-# it's not predictable when it will be written out.
-
+"""WorkingTree object and friends.
+
+A WorkingTree represents the editable working copy of a branch.
+Operations which represent the WorkingTree are also done here, 
+such as renaming or adding files.  The WorkingTree has an inventory 
+which is updated by these operations.  A commit produces a 
+new revision based on the workingtree and its inventory.
+
+At the moment every WorkingTree has its own branch.  Remote
+WorkingTrees aren't supported.
+
+To get a WorkingTree, call bzrdir.open_workingtree() or
+WorkingTree.open(dir).
+"""
+
+MERGE_MODIFIED_HEADER_1 = "BZR merge-modified list format 1"
+CONFLICT_HEADER_1 = "BZR conflict list format 1"
+
+# TODO: Give the workingtree sole responsibility for the working inventory;
+# remove the variable and references to it from the branch.  This may require
+# updating the commit code so as to update the inventory within the working
+# copy, and making sure there's only one WorkingTree for any directory on disk.
+# At the moment they may alias the inventory and have old copies of it in
+# memory.  (Now done? -- mbp 20060309)
+
+from copy import deepcopy
+from cStringIO import StringIO
+import errno
+import fnmatch
 import os
-    
+import stat
+ 
+
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.branch import (Branch,
+                           quotefn)
+from bzrlib.conflicts import Conflict, ConflictList, CONFLICT_SUFFIXES
+import bzrlib.bzrdir as bzrdir
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrCheckError,
+                           BzrError,
+                           ConflictFormatError,
+                           DivergedBranches,
+                           WeaveRevisionNotPresent,
+                           NotBranchError,
+                           NoSuchFile,
+                           NotVersionedError,
+                           MergeModifiedFormatError,
+                           UnsupportedOperation,
+                           )
+from bzrlib.inventory import InventoryEntry, Inventory
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
+from bzrlib.merge import merge_inner, transform_tree
+from bzrlib.osutils import (
+                            abspath,
+                            appendpath,
+                            compact_date,
+                            file_kind,
+                            isdir,
+                            getcwd,
+                            pathjoin,
+                            pumpfile,
+                            safe_unicode,
+                            splitpath,
+                            rand_bytes,
+                            normpath,
+                            realpath,
+                            relpath,
+                            rename,
+                            supports_executable,
+                            )
+from bzrlib.progress import DummyProgress, ProgressPhase
+from bzrlib.revision import NULL_REVISION
+from bzrlib.rio import RioReader, rio_file, Stanza
+from bzrlib.symbol_versioning import *
+from bzrlib.textui import show_status
 import bzrlib.tree
-from errors import BzrCheckError
-from trace import mutter
+from bzrlib.transform import build_tree
+from bzrlib.trace import mutter, note
+from bzrlib.transport import get_transport
+from bzrlib.transport.local import LocalTransport
+import bzrlib.ui
+import bzrlib.xml5
+
+
+def gen_file_id(name):
+    """Return new file id.
+
+    This should probably generate proper UUIDs, but for the moment we
+    cope with just randomness because running uuidgen every time is
+    slow."""
+    import re
+    from binascii import hexlify
+    from time import time
+
+    # get last component
+    idx = name.rfind('/')
+    if idx != -1:
+        name = name[idx+1 : ]
+    idx = name.rfind('\\')
+    if idx != -1:
+        name = name[idx+1 : ]
+
+    # make it not a hidden file
+    name = name.lstrip('.')
+
+    # remove any wierd characters; we don't escape them but rather
+    # just pull them out
+    name = re.sub(r'[^\w.]', '', name)
+
+    s = hexlify(rand_bytes(8))
+    return '-'.join((name, compact_date(time()), s))
+
+
+def gen_root_id():
+    """Return a new tree-root file id."""
+    return gen_file_id('TREE_ROOT')
+
+
+class TreeEntry(object):
+    """An entry that implements the minium interface used by commands.
+
+    This needs further inspection, it may be better to have 
+    InventoryEntries without ids - though that seems wrong. For now,
+    this is a parallel hierarchy to InventoryEntry, and needs to become
+    one of several things: decorates to that hierarchy, children of, or
+    parents of it.
+    Another note is that these objects are currently only used when there is
+    no InventoryEntry available - i.e. for unversioned objects.
+    Perhaps they should be UnversionedEntry et al. ? - RBC 20051003
+    """
+ 
+    def __eq__(self, other):
+        # yes, this us ugly, TODO: best practice __eq__ style.
+        return (isinstance(other, TreeEntry)
+                and other.__class__ == self.__class__)
+ 
+    def kind_character(self):
+        return "???"
+
+
+class TreeDirectory(TreeEntry):
+    """See TreeEntry. This is a directory in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeDirectory)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return "/"
+
+
+class TreeFile(TreeEntry):
+    """See TreeEntry. This is a regular file in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeFile)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
+
+class TreeLink(TreeEntry):
+    """See TreeEntry. This is a symlink in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeLink)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
 
 class WorkingTree(bzrlib.tree.Tree):
     """Working copy tree.
@@ -35,29 +199,139 @@
     It is possible for a `WorkingTree` to have a filename which is
     not listed in the Inventory and vice versa.
     """
-    def __init__(self, basedir, inv):
+
+    def __init__(self, basedir='.',
+                 branch=DEPRECATED_PARAMETER,
+                 _inventory=None,
+                 _control_files=None,
+                 _internal=False,
+                 _format=None,
+                 _bzrdir=None):
+        """Construct a WorkingTree for basedir.
+
+        If the branch is not supplied, it is opened automatically.
+        If the branch is supplied, it must be the branch for this basedir.
+        (branch.base is not cross checked, because for remote branches that
+        would be meaningless).
+        """
+        self._format = _format
+        self.bzrdir = _bzrdir
+        if not _internal:
+            # not created via open etc.
+            warn("WorkingTree() is deprecated as of bzr version 0.8. "
+                 "Please use bzrdir.open_workingtree or WorkingTree.open().",
+                 DeprecationWarning,
+                 stacklevel=2)
+            wt = WorkingTree.open(basedir)
+            self.branch = wt.branch
+            self.basedir = wt.basedir
+            self._control_files = wt._control_files
+            self._hashcache = wt._hashcache
+            self._set_inventory(wt._inventory)
+            self._format = wt._format
+            self.bzrdir = wt.bzrdir
         from bzrlib.hashcache import HashCache
         from bzrlib.trace import note, mutter
-
-        self._inventory = inv
-        self.basedir = basedir
-        self.path2id = inv.path2id
-
+        assert isinstance(basedir, basestring), \
+            "base directory %r is not a string" % basedir
+        basedir = safe_unicode(basedir)
+        mutter("opening working tree %r", basedir)
+        if deprecated_passed(branch):
+            if not _internal:
+                warn("WorkingTree(..., branch=XXX) is deprecated as of bzr 0.8."
+                     " Please use bzrdir.open_workingtree() or"
+                     " WorkingTree.open().",
+                     DeprecationWarning,
+                     stacklevel=2
+                     )
+            self.branch = branch
+        else:
+            self.branch = self.bzrdir.open_branch()
+        assert isinstance(self.branch, Branch), \
+            "branch %r is not a Branch" % self.branch
+        self.basedir = realpath(basedir)
+        # if branch is at our basedir and is a format 6 or less
+        if isinstance(self._format, WorkingTreeFormat2):
+            # share control object
+            self._control_files = self.branch.control_files
+        else:
+            # only ready for format 3
+            assert isinstance(self._format, WorkingTreeFormat3)
+            assert isinstance(_control_files, LockableFiles), \
+                    "_control_files must be a LockableFiles, not %r" \
+                    % _control_files
+            self._control_files = _control_files
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
-        hc = self._hashcache = HashCache(basedir)
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
+        cache_filename = self.bzrdir.get_workingtree_transport(None).abspath('stat-cache')
+        hc = self._hashcache = HashCache(basedir, cache_filename, self._control_files._file_mode)
         hc.read()
+        # is this scan needed ? it makes things kinda slow.
         hc.scan()
 
         if hc.needs_write:
             mutter("write hc")
             hc.write()
-            
-            
-    def __del__(self):
-        if self._hashcache.needs_write:
-            self._hashcache.write()
-
+
+        if _inventory is None:
+            self._set_inventory(self.read_working_inventory())
+        else:
+            self._set_inventory(_inventory)
+
+    def _set_inventory(self, inv):
+        self._inventory = inv
+        self.path2id = self._inventory.path2id
+
+    def is_control_filename(self, filename):
+        """True if filename is the name of a control file in this tree.
+        
+        This is true IF and ONLY IF the filename is part of the meta data
+        that bzr controls in this tree. I.E. a random .bzr directory placed
+        on disk will not be a control file for this tree.
+        """
+        try:
+            self.bzrdir.transport.relpath(self.abspath(filename))
+            return True
+        except errors.PathNotChild:
+            return False
+
+    @staticmethod
+    def open(path=None, _unsupported=False):
+        """Open an existing working tree at path.
+
+        """
+        if path is None:
+            path = os.path.getcwdu()
+        control = bzrdir.BzrDir.open(path, _unsupported)
+        return control.open_workingtree(_unsupported)
+        
+    @staticmethod
+    def open_containing(path=None):
+        """Open an existing working tree which has its root about path.
+        
+        This probes for a working tree at path and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into /.  If there isn't one, raises NotBranchError.
+        TODO: give this a new exception.
+        If there is one, it is returned, along with the unused portion of path.
+        """
+        if path is None:
+            path = os.getcwdu()
+        control, relpath = bzrdir.BzrDir.open_containing(path)
+        return control.open_workingtree(), relpath
+
+    @staticmethod
+    def open_downlevel(path=None):
+        """Open an unsupported working tree.
+
+        Only intended for advanced situations like upgrading part of a bzrdir.
+        """
+        return WorkingTree.open(path, _unsupported=True)
 
     def __iter__(self):
         """Iterate through file_ids for this tree.
@@ -67,21 +341,75 @@
         """
         inv = self._inventory
         for path, ie in inv.iter_entries():
-            if os.path.exists(self.abspath(path)):
+            if bzrlib.osutils.lexists(self.abspath(path)):
                 yield ie.file_id
 
-
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__,
                                getattr(self, 'basedir', None))
 
-
-
     def abspath(self, filename):
-        return os.path.join(self.basedir, filename)
+        return pathjoin(self.basedir, filename)
+    
+    def basis_tree(self):
+        """Return RevisionTree for the current last revision."""
+        revision_id = self.last_revision()
+        if revision_id is not None:
+            try:
+                xml = self.read_basis_inventory()
+                inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+            except NoSuchFile:
+                inv = None
+            if inv is not None and inv.revision_id == revision_id:
+                return bzrlib.tree.RevisionTree(self.branch.repository, inv,
+                                                revision_id)
+        # FIXME? RBC 20060403 should we cache the inventory here ?
+        return self.branch.repository.revision_tree(revision_id)
+
+    @staticmethod
+    @deprecated_method(zero_eight)
+    def create(branch, directory):
+        """Create a workingtree for branch at directory.
+
+        If existing_directory already exists it must have a .bzr directory.
+        If it does not exist, it will be created.
+
+        This returns a new WorkingTree object for the new checkout.
+
+        TODO FIXME RBC 20060124 when we have checkout formats in place this
+        should accept an optional revisionid to checkout [and reject this if
+        checking out into the same dir as a pre-checkout-aware branch format.]
+
+        XXX: When BzrDir is present, these should be created through that 
+        interface instead.
+        """
+        warn('delete WorkingTree.create', stacklevel=3)
+        transport = get_transport(directory)
+        if branch.bzrdir.root_transport.base == transport.base:
+            # same dir 
+            return branch.bzrdir.create_workingtree()
+        # different directory, 
+        # create a branch reference
+        # and now a working tree.
+        raise NotImplementedError
+ 
+    @staticmethod
+    @deprecated_method(zero_eight)
+    def create_standalone(directory):
+        """Create a checkout and a branch and a repo at directory.
+
+        Directory must exist and be empty.
+
+        please use BzrDir.create_standalone_workingtree
+        """
+        return bzrdir.BzrDir.create_standalone_workingtree(directory)
+
+    def relpath(self, abs):
+        """Return the local path portion from a given absolute path."""
+        return relpath(self.basedir, abs)
 
     def has_filename(self, filename):
-        return os.path.exists(self.abspath(filename))
+        return bzrlib.osutils.lexists(self.abspath(filename))
 
     def get_file(self, file_id):
         return self.get_file_byname(self.id2path(file_id))
@@ -89,32 +417,235 @@
     def get_file_byname(self, filename):
         return file(self.abspath(filename), 'rb')
 
+    def get_root_id(self):
+        """Return the id of this trees root"""
+        inv = self.read_working_inventory()
+        return inv.root.file_id
+        
     def _get_store_filename(self, file_id):
-        ## XXX: badly named; this isn't in the store at all
-        return self.abspath(self.id2path(file_id))
-
-                
+        ## XXX: badly named; this is not in the store at all
+        return self.abspath(self.id2path(file_id))
+
+    @needs_read_lock
+    def clone(self, to_bzrdir, revision_id=None, basis=None):
+        """Duplicate this working tree into to_bzr, including all state.
+        
+        Specifically modified files are kept as modified, but
+        ignored and unknown files are discarded.
+
+        If you want to make a new line of development, see bzrdir.sprout()
+
+        revision
+            If not None, the cloned tree will have its last revision set to 
+            revision, and and difference between the source trees last revision
+            and this one merged in.
+
+        basis
+            If not None, a closer copy of a tree which may have some files in
+            common, and which file content should be preferentially copied from.
+        """
+        # assumes the target bzr dir format is compatible.
+        result = self._format.initialize(to_bzrdir)
+        self.copy_content_into(result, revision_id)
+        return result
+
+    @needs_read_lock
+    def copy_content_into(self, tree, revision_id=None):
+        """Copy the current content and user files of this tree into tree."""
+        if revision_id is None:
+            transform_tree(tree, self)
+        else:
+            # TODO now merge from tree.last_revision to revision
+            transform_tree(tree, self)
+            tree.set_last_revision(revision_id)
+
+    @needs_write_lock
+    def commit(self, message=None, revprops=None, *args, **kwargs):
+        # avoid circular imports
+        from bzrlib.commit import Commit
+        if revprops is None:
+            revprops = {}
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = self.branch.nick
+        # args for wt.commit start at message from the Commit.commit method,
+        # but with branch a kwarg now, passing in args as is results in the
+        #message being used for the branch
+        args = (DEPRECATED_PARAMETER, message, ) + args
+        Commit().commit(working_tree=self, revprops=revprops, *args, **kwargs)
+        self._set_inventory(self.read_working_inventory())
+
+    def id2abspath(self, file_id):
+        return self.abspath(self.id2path(file_id))
+
     def has_id(self, file_id):
         # files that have been deleted are excluded
         inv = self._inventory
         if not inv.has_id(file_id):
             return False
         path = inv.id2path(file_id)
-        return os.path.exists(self.abspath(path))
+        return bzrlib.osutils.lexists(self.abspath(path))
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
 
     __contains__ = has_id
-    
 
     def get_file_size(self, file_id):
-        # is this still called?
-        raise NotImplementedError()
-
-
+        return os.path.getsize(self.id2abspath(file_id))
+
+    @needs_read_lock
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
 
+    def is_executable(self, file_id):
+        if not supports_executable():
+            return self._inventory[file_id].executable
+        else:
+            path = self._inventory.id2path(file_id)
+            mode = os.lstat(self.abspath(path)).st_mode
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
+
+    @needs_write_lock
+    def add(self, files, ids=None):
+        """Make files versioned.
+
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
+
+        This adds the files to the inventory, so that they will be
+        recorded by the next commit.
+
+        files
+            List of paths to add, relative to the base of the tree.
+
+        ids
+            If set, use these instead of automatically generated ids.
+            Must be the same length as the list of files, but may
+            contain None for ids that are to be autogenerated.
+
+        TODO: Perhaps have an option to add the ids even if the files do
+              not (yet) exist.
+
+        TODO: Perhaps callback with the ids and paths as they're added.
+        """
+        # TODO: Re-adding a file that is removed in the working copy
+        # should probably put it back with the previous ID.
+        if isinstance(files, basestring):
+            assert(ids is None or isinstance(ids, basestring))
+            files = [files]
+            if ids is not None:
+                ids = [ids]
+
+        if ids is None:
+            ids = [None] * len(files)
+        else:
+            assert(len(ids) == len(files))
+
+        inv = self.read_working_inventory()
+        for f,file_id in zip(files, ids):
+            if self.is_control_filename(f):
+                raise BzrError("cannot add control file %s" % quotefn(f))
+
+            fp = splitpath(f)
+
+            if len(fp) == 0:
+                raise BzrError("cannot add top-level %r" % f)
+
+            fullpath = normpath(self.abspath(f))
+
+            try:
+                kind = file_kind(fullpath)
+            except OSError, e:
+                if e.errno == errno.ENOENT:
+                    raise NoSuchFile(fullpath)
+                # maybe something better?
+                raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
+
+            if not InventoryEntry.versionable_kind(kind):
+                raise BzrError('cannot add: not a versionable file ('
+                               'i.e. regular file, symlink or directory): %s' % quotefn(f))
+
+            if file_id is None:
+                file_id = gen_file_id(f)
+            inv.add_path(f, kind=kind, file_id=file_id)
+
+            mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def add_pending_merge(self, *revision_ids):
+        # TODO: Perhaps should check at this point that the
+        # history of the revision is actually present?
+        p = self.pending_merges()
+        updated = False
+        for rev_id in revision_ids:
+            if rev_id in p:
+                continue
+            p.append(rev_id)
+            updated = True
+        if updated:
+            self.set_pending_merges(p)
+
+    @needs_read_lock
+    def pending_merges(self):
+        """Return a list of pending merges.
+
+        These are revisions that have been merged into the working
+        directory but not yet committed.
+        """
+        try:
+            merges_file = self._control_files.get_utf8('pending-merges')
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+            return []
+        p = []
+        for l in merges_file.readlines():
+            p.append(l.rstrip('\n'))
+        return p
+
+    @needs_write_lock
+    def set_pending_merges(self, rev_list):
+        self._control_files.put_utf8('pending-merges', '\n'.join(rev_list))
+
+    @needs_write_lock
+    def set_merge_modified(self, modified_hashes):
+        def iter_stanzas():
+            for file_id, hash in modified_hashes.iteritems():
+                yield Stanza(file_id=file_id, hash=hash)
+        self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
+
+    @needs_write_lock
+    def _put_rio(self, filename, stanzas, header):
+        my_file = rio_file(stanzas, header)
+        self._control_files.put(filename, my_file)
+
+    @needs_read_lock
+    def merge_modified(self):
+        try:
+            hashfile = self._control_files.get('merge-hashes')
+        except NoSuchFile:
+            return {}
+        merge_hashes = {}
+        try:
+            if hashfile.next() != MERGE_MODIFIED_HEADER_1 + '\n':
+                raise MergeModifiedFormatError()
+        except StopIteration:
+            raise MergeModifiedFormatError()
+        for s in RioReader(hashfile):
+            file_id = s.get("file_id")
+            if file_id not in self.inventory:
+                continue
+            hash = s.get("hash")
+            if hash == self.get_file_sha1(file_id):
+                merge_hashes[file_id] = hash
+        return merge_hashes
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.id2abspath(file_id))
 
     def file_class(self, filename):
         if self.path2id(filename):
@@ -124,7 +655,6 @@
         else:
             return '?'
 
-
     def list_files(self):
         """Recursively list all files as (path, class, kind, id).
 
@@ -135,9 +665,6 @@
 
         Skips the control directory.
         """
-        from osutils import appendpath, file_kind
-        import os
-
         inv = self._inventory
 
         def descend(from_dir_relpath, from_dir_id, dp):
@@ -147,7 +674,9 @@
                 ## TODO: If we find a subdirectory with its own .bzr
                 ## directory, then that is a separate tree and we
                 ## should exclude it.
-                if bzrlib.BZRDIR == f:
+
+                # the bzrdir for this tree
+                if self.bzrdir.transport.base.endswith(f + '/'):
                     continue
 
                 # path within tree
@@ -172,7 +701,20 @@
                                             "now of kind %r"
                                             % (fap, f_ie.kind, f_ie.file_id, fk))
 
-                yield fp, c, fk, (f_ie and f_ie.file_id)
+                # make a last minute entry
+                if f_ie:
+                    entry = f_ie
+                else:
+                    if fk == 'directory':
+                        entry = TreeDirectory()
+                    elif fk == 'file':
+                        entry = TreeFile()
+                    elif fk == 'symlink':
+                        entry = TreeLink()
+                    else:
+                        entry = TreeEntry()
+                
+                yield fp, c, fk, (f_ie and f_ie.file_id), entry
 
                 if fk != 'directory':
                     continue
@@ -184,16 +726,198 @@
                 for ff in descend(fp, f_ie.file_id, fap):
                     yield ff
 
-        for f in descend('', inv.root.file_id, self.basedir):
+        for f in descend(u'', inv.root.file_id, self.basedir):
             yield f
-            
-
-
+
+    @needs_write_lock
+    def move(self, from_paths, to_name):
+        """Rename files.
+
+        to_name must exist in the inventory.
+
+        If to_name exists and is a directory, the files are moved into
+        it, keeping their old names.  
+
+        Note that to_name is only the last component of the new name;
+        this doesn't change the directory.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
+        """
+        result = []
+        ## TODO: Option to move IDs only
+        assert not isinstance(from_paths, basestring)
+        inv = self.inventory
+        to_abs = self.abspath(to_name)
+        if not isdir(to_abs):
+            raise BzrError("destination %r is not a directory" % to_abs)
+        if not self.has_filename(to_name):
+            raise BzrError("destination %r not in working directory" % to_abs)
+        to_dir_id = inv.path2id(to_name)
+        if to_dir_id == None and to_name != '':
+            raise BzrError("destination %r is not a versioned directory" % to_name)
+        to_dir_ie = inv[to_dir_id]
+        if to_dir_ie.kind not in ('directory', 'root_directory'):
+            raise BzrError("destination %r is not a directory" % to_abs)
+
+        to_idpath = inv.get_idpath(to_dir_id)
+
+        for f in from_paths:
+            if not self.has_filename(f):
+                raise BzrError("%r does not exist in working tree" % f)
+            f_id = inv.path2id(f)
+            if f_id == None:
+                raise BzrError("%r is not versioned" % f)
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            if self.has_filename(dest_path):
+                raise BzrError("destination %r already exists" % dest_path)
+            if f_id in to_idpath:
+                raise BzrError("can't move %r to a subdirectory of itself" % f)
+
+        # OK, so there's a race here, it's possible that someone will
+        # create a file in this interval and then the rename might be
+        # left half-done.  But we should have caught most problems.
+        orig_inv = deepcopy(self.inventory)
+        try:
+            for f in from_paths:
+                name_tail = splitpath(f)[-1]
+                dest_path = appendpath(to_name, name_tail)
+                result.append((f, dest_path))
+                inv.rename(inv.path2id(f), to_dir_id, name_tail)
+                try:
+                    rename(self.abspath(f), self.abspath(dest_path))
+                except OSError, e:
+                    raise BzrError("failed to rename %r to %r: %s" %
+                                   (f, dest_path, e[1]),
+                            ["rename rolled back"])
+        except:
+            # restore the inventory on error
+            self._set_inventory(orig_inv)
+            raise
+        self._write_inventory(inv)
+        return result
+
+    @needs_write_lock
+    def rename_one(self, from_rel, to_rel):
+        """Rename one file.
+
+        This can change the directory or the filename or both.
+        """
+        inv = self.inventory
+        if not self.has_filename(from_rel):
+            raise BzrError("can't rename: old working file %r does not exist" % from_rel)
+        if self.has_filename(to_rel):
+            raise BzrError("can't rename: new working file %r already exists" % to_rel)
+
+        file_id = inv.path2id(from_rel)
+        if file_id == None:
+            raise BzrError("can't rename: old name %r is not versioned" % from_rel)
+
+        entry = inv[file_id]
+        from_parent = entry.parent_id
+        from_name = entry.name
+        
+        if inv.path2id(to_rel):
+            raise BzrError("can't rename: new name %r is already versioned" % to_rel)
+
+        to_dir, to_tail = os.path.split(to_rel)
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id == None and to_dir != '':
+            raise BzrError("can't determine destination directory id for %r" % to_dir)
+
+        mutter("rename_one:")
+        mutter("  file_id    {%s}" % file_id)
+        mutter("  from_rel   %r" % from_rel)
+        mutter("  to_rel     %r" % to_rel)
+        mutter("  to_dir     %r" % to_dir)
+        mutter("  to_dir_id  {%s}" % to_dir_id)
+
+        inv.rename(file_id, to_dir_id, to_tail)
+
+        from_abs = self.abspath(from_rel)
+        to_abs = self.abspath(to_rel)
+        try:
+            rename(from_abs, to_abs)
+        except OSError, e:
+            inv.rename(file_id, from_parent, from_name)
+            raise BzrError("failed to rename %r to %r: %s"
+                    % (from_abs, to_abs, e[1]),
+                    ["rename rolled back"])
+        self._write_inventory(inv)
+
+    @needs_read_lock
     def unknowns(self):
+        """Return all unknown files.
+
+        These are files in the working directory that are not versioned or
+        control files or ignored.
+        
+        >>> from bzrlib.bzrdir import ScratchDir
+        >>> d = ScratchDir(files=['foo', 'foo~'])
+        >>> b = d.open_branch()
+        >>> tree = d.open_workingtree()
+        >>> map(str, tree.unknowns())
+        ['foo']
+        >>> tree.add('foo')
+        >>> list(b.unknowns())
+        []
+        >>> tree.remove('foo')
+        >>> list(b.unknowns())
+        [u'foo']
+        """
         for subp in self.extras():
             if not self.is_ignored(subp):
                 yield subp
 
+    @deprecated_method(zero_eight)
+    def iter_conflicts(self):
+        """List all files in the tree that have text or content conflicts.
+        DEPRECATED.  Use conflicts instead."""
+        return self._iter_conflicts()
+
+    def _iter_conflicts(self):
+        conflicted = set()
+        for path in (s[0] for s in self.list_files()):
+            stem = get_conflicted_stem(path)
+            if stem is None:
+                continue
+            if stem not in conflicted:
+                conflicted.add(stem)
+                yield stem
+
+    @needs_write_lock
+    def pull(self, source, overwrite=False, stop_revision=None):
+        top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        source.lock_read()
+        try:
+            pp = ProgressPhase("Pull phase", 2, top_pb)
+            pp.next_phase()
+            old_revision_history = self.branch.revision_history()
+            basis_tree = self.basis_tree()
+            count = self.branch.pull(source, overwrite, stop_revision)
+            new_revision_history = self.branch.revision_history()
+            if new_revision_history != old_revision_history:
+                pp.next_phase()
+                if len(old_revision_history):
+                    other_revision = old_revision_history[-1]
+                else:
+                    other_revision = None
+                repository = self.branch.repository
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    merge_inner(self.branch,
+                                self.branch.basis_tree(),
+                                basis_tree, 
+                                this_tree=self, 
+                                pb=pb)
+                finally:
+                    pb.finished()
+                self.set_last_revision(self.branch.last_revision())
+            return count
+        finally:
+            source.unlock()
+            top_pb.finished()
 
     def extras(self):
         """Yield all unknown files in this WorkingTree.
@@ -205,10 +929,8 @@
         Currently returned depth-first, sorted by name within directories.
         """
         ## TODO: Work from given directory downwards
-        from osutils import isdir, appendpath
-        
         for path, dir_entry in self.inventory.directories():
-            mutter("search for unknowns in %r" % path)
+            mutter("search for unknowns in %r", path)
             dirabs = self.abspath(path)
             if not isdir(dirabs):
                 # e.g. directory deleted
@@ -269,9 +991,6 @@
         # Eventually it should be replaced with something more
         # accurate.
         
-        import fnmatch
-        from osutils import splitpath
-        
         for pat in self.get_ignore_list():
             if '/' in pat or '\\' in pat:
                 
@@ -290,4 +1009,660 @@
                     return pat
         else:
             return None
-        
\ No newline at end of file
+
+    def kind(self, file_id):
+        return file_kind(self.id2abspath(file_id))
+
+    @needs_read_lock
+    def last_revision(self):
+        """Return the last revision id of this working tree.
+
+        In early branch formats this was == the branch last_revision,
+        but that cannot be relied upon - for working tree operations,
+        always use tree.last_revision().
+        """
+        return self.branch.last_revision()
+
+    def lock_read(self):
+        """See Branch.lock_read, and WorkingTree.unlock."""
+        self.branch.lock_read()
+        try:
+            return self._control_files.lock_read()
+        except:
+            self.branch.unlock()
+            raise
+
+    def lock_write(self):
+        """See Branch.lock_write, and WorkingTree.unlock."""
+        self.branch.lock_write()
+        try:
+            return self._control_files.lock_write()
+        except:
+            self.branch.unlock()
+            raise
+
+    def _basis_inventory_name(self):
+        return 'basis-inventory'
+
+    @needs_write_lock
+    def set_last_revision(self, new_revision):
+        """Change the last revision in the working tree."""
+        if self._change_last_revision(new_revision):
+            self._cache_basis_inventory(new_revision)
+
+    def _change_last_revision(self, new_revision):
+        """Template method part of set_last_revision to perform the change.
+        
+        This is used to allow WorkingTree3 instances to not affect branch
+        when their last revision is set.
+        """
+        if new_revision is None:
+            self.branch.set_revision_history([])
+            return False
+        # current format is locked in with the branch
+        revision_history = self.branch.revision_history()
+        try:
+            position = revision_history.index(new_revision)
+        except ValueError:
+            raise errors.NoSuchRevision(self.branch, new_revision)
+        self.branch.set_revision_history(revision_history[:position + 1])
+        return True
+
+    def _cache_basis_inventory(self, new_revision):
+        """Cache new_revision as the basis inventory."""
+        try:
+            # this double handles the inventory - unpack and repack - 
+            # but is easier to understand. We can/should put a conditional
+            # in here based on whether the inventory is in the latest format
+            # - perhaps we should repack all inventories on a repository
+            # upgrade ?
+            inv = self.branch.repository.get_inventory(new_revision)
+            inv.revision_id = new_revision
+            xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
+
+            path = self._basis_inventory_name()
+            self._control_files.put_utf8(path, xml)
+        except WeaveRevisionNotPresent:
+            pass
+
+    def read_basis_inventory(self):
+        """Read the cached basis inventory."""
+        path = self._basis_inventory_name()
+        return self._control_files.get_utf8(path).read()
+        
+    @needs_read_lock
+    def read_working_inventory(self):
+        """Read the working inventory."""
+        # ElementTree does its own conversion from UTF-8, so open in
+        # binary.
+        result = bzrlib.xml5.serializer_v5.read_inventory(
+            self._control_files.get('inventory'))
+        self._set_inventory(result)
+        return result
+
+    @needs_write_lock
+    def remove(self, files, verbose=False):
+        """Remove nominated files from the working inventory..
+
+        This does not remove their text.  This does not run on XXX on what? RBC
+
+        TODO: Refuse to remove modified files unless --force is given?
+
+        TODO: Do something useful with directories.
+
+        TODO: Should this remove the text or not?  Tough call; not
+        removing may be useful and the user can just use use rm, and
+        is the opposite of add.  Removing it is consistent with most
+        other tools.  Maybe an option.
+        """
+        ## TODO: Normalize names
+        ## TODO: Remove nested loops; better scalability
+        if isinstance(files, basestring):
+            files = [files]
+
+        inv = self.inventory
+
+        # do this before any modifications
+        for f in files:
+            fid = inv.path2id(f)
+            if not fid:
+                # TODO: Perhaps make this just a warning, and continue?
+                # This tends to happen when 
+                raise NotVersionedError(path=f)
+            mutter("remove inventory entry %s {%s}", quotefn(f), fid)
+            if verbose:
+                # having remove it, it must be either ignored or unknown
+                if self.is_ignored(f):
+                    new_status = 'I'
+                else:
+                    new_status = '?'
+                show_status(new_status, inv[fid].kind, quotefn(f))
+            del inv[fid]
+
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def revert(self, filenames, old_tree=None, backups=True, 
+               pb=DummyProgress()):
+        from transform import revert
+        from conflicts import resolve
+        if old_tree is None:
+            old_tree = self.basis_tree()
+        conflicts = revert(self, old_tree, filenames, backups, pb)
+        if not len(filenames):
+            self.set_pending_merges([])
+            resolve(self)
+        else:
+            resolve(self, filenames, ignore_misses=True)
+        return conflicts
+
+    # XXX: This method should be deprecated in favour of taking in a proper
+    # new Inventory object.
+    @needs_write_lock
+    def set_inventory(self, new_inventory_list):
+        from bzrlib.inventory import (Inventory,
+                                      InventoryDirectory,
+                                      InventoryEntry,
+                                      InventoryFile,
+                                      InventoryLink)
+        inv = Inventory(self.get_root_id())
+        for path, file_id, parent, kind in new_inventory_list:
+            name = os.path.basename(path)
+            if name == "":
+                continue
+            # fixme, there should be a factory function inv,add_?? 
+            if kind == 'directory':
+                inv.add(InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(InventoryFile(file_id, name, parent))
+            elif kind == 'symlink':
+                inv.add(InventoryLink(file_id, name, parent))
+            else:
+                raise BzrError("unknown kind %r" % kind)
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def set_root_id(self, file_id):
+        """Set the root id for this tree."""
+        inv = self.read_working_inventory()
+        orig_root_id = inv.root.file_id
+        del inv._byid[inv.root.file_id]
+        inv.root.file_id = file_id
+        inv._byid[inv.root.file_id] = inv.root
+        for fid in inv:
+            entry = inv[fid]
+            if entry.parent_id == orig_root_id:
+                entry.parent_id = inv.root.file_id
+        self._write_inventory(inv)
+
+    def unlock(self):
+        """See Branch.unlock.
+        
+        WorkingTree locking just uses the Branch locking facilities.
+        This is current because all working trees have an embedded branch
+        within them. IF in the future, we were to make branch data shareable
+        between multiple working trees, i.e. via shared storage, then we 
+        would probably want to lock both the local tree, and the branch.
+        """
+        # FIXME: We want to write out the hashcache only when the last lock on
+        # this working copy is released.  Peeking at the lock count is a bit
+        # of a nasty hack; probably it's better to have a transaction object,
+        # which can do some finalization when it's either successfully or
+        # unsuccessfully completed.  (Denys's original patch did that.)
+        # RBC 20060206 hookinhg into transaction will couple lock and transaction
+        # wrongly. Hookinh into unllock on the control files object is fine though.
+        
+        # TODO: split this per format so there is no ugly if block
+        if self._hashcache.needs_write and (
+            # dedicated lock files
+            self._control_files._lock_count==1 or 
+            # shared lock files
+            (self._control_files is self.branch.control_files and 
+             self._control_files._lock_count==3)):
+            self._hashcache.write()
+        # reverse order of locking.
+        result = self._control_files.unlock()
+        try:
+            self.branch.unlock()
+        finally:
+            return result
+
+    @needs_write_lock
+    def update(self):
+        """Update a working tree along its branch.
+
+        This will update the branch if its bound too, which means we have multiple trees involved:
+        The new basis tree of the master.
+        The old basis tree of the branch.
+        The old basis tree of the working tree.
+        The current working tree state.
+        pathologically all three may be different, and non ancestors of each other.
+        Conceptually we want to:
+        Preserve the wt.basis->wt.state changes
+        Transform the wt.basis to the new master basis.
+        Apply a merge of the old branch basis to get any 'local' changes from it into the tree.
+        Restore the wt.basis->wt.state changes.
+
+        There isn't a single operation at the moment to do that, so we:
+        Merge current state -> basis tree of the master w.r.t. the old tree basis.
+        Do a 'normal' merge of the old branch basis if it is relevant.
+        """
+        old_tip = self.branch.update()
+        if old_tip is not None:
+            self.add_pending_merge(old_tip)
+        self.branch.lock_read()
+        try:
+            result = 0
+            if self.last_revision() != self.branch.last_revision():
+                # merge tree state up to new branch tip.
+                basis = self.basis_tree()
+                to_tree = self.branch.basis_tree()
+                result += merge_inner(self.branch,
+                                      to_tree,
+                                      basis,
+                                      this_tree=self)
+                self.set_last_revision(self.branch.last_revision())
+            if old_tip and old_tip != self.last_revision():
+                # our last revision was not the prior branch last reivison
+                # and we have converted that last revision to a pending merge.
+                # base is somewhere between the branch tip now
+                # and the now pending merge
+                from bzrlib.revision import common_ancestor
+                try:
+                    base_rev_id = common_ancestor(self.branch.last_revision(),
+                                                  old_tip,
+                                                  self.branch.repository)
+                except errors.NoCommonAncestor:
+                    base_rev_id = None
+                base_tree = self.branch.repository.revision_tree(base_rev_id)
+                other_tree = self.branch.repository.revision_tree(old_tip)
+                result += merge_inner(self.branch,
+                                      other_tree,
+                                      base_tree,
+                                      this_tree=self)
+            return result
+        finally:
+            self.branch.unlock()
+
+    @needs_write_lock
+    def _write_inventory(self, inv):
+        """Write inventory as the current inventory."""
+        sio = StringIO()
+        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+        sio.seek(0)
+        self._control_files.put('inventory', sio)
+        self._set_inventory(inv)
+        mutter('wrote working inventory')
+
+    def set_conflicts(self, arg):
+        raise UnsupportedOperation(self.set_conflicts, self)
+
+    @needs_read_lock
+    def conflicts(self):
+        conflicts = ConflictList()
+        for conflicted in self._iter_conflicts():
+            text = True
+            try:
+                if file_kind(self.abspath(conflicted)) != "file":
+                    text = False
+            except OSError, e:
+                if e.errno == errno.ENOENT:
+                    text = False
+                else:
+                    raise
+            if text is True:
+                for suffix in ('.THIS', '.OTHER'):
+                    try:
+                        kind = file_kind(self.abspath(conflicted+suffix))
+                    except OSError, e:
+                        if e.errno == errno.ENOENT:
+                            text = False
+                            break
+                        else:
+                            raise
+                    if kind != "file":
+                        text = False
+                        break
+            ctype = {True: 'text conflict', False: 'contents conflict'}[text]
+            conflicts.append(Conflict.factory(ctype, path=conflicted,
+                             file_id=self.path2id(conflicted)))
+        return conflicts
+
+
+class WorkingTree3(WorkingTree):
+    """This is the Format 3 working tree.
+
+    This differs from the base WorkingTree by:
+     - having its own file lock
+     - having its own last-revision property.
+
+    This is new in bzr 0.8
+    """
+
+    @needs_read_lock
+    def last_revision(self):
+        """See WorkingTree.last_revision."""
+        try:
+            return self._control_files.get_utf8('last-revision').read()
+        except NoSuchFile:
+            return None
+
+    def _change_last_revision(self, revision_id):
+        """See WorkingTree._change_last_revision."""
+        if revision_id is None or revision_id == NULL_REVISION:
+            try:
+                self._control_files._transport.delete('last-revision')
+            except errors.NoSuchFile:
+                pass
+            return False
+        else:
+            try:
+                self.branch.revision_history().index(revision_id)
+            except ValueError:
+                raise errors.NoSuchRevision(self.branch, revision_id)
+            self._control_files.put_utf8('last-revision', revision_id)
+            return True
+
+    @needs_write_lock
+    def set_conflicts(self, conflicts):
+        self._put_rio('conflicts', conflicts.to_stanzas(), 
+                      CONFLICT_HEADER_1)
+
+    @needs_read_lock
+    def conflicts(self):
+        try:
+            confile = self._control_files.get('conflicts')
+        except NoSuchFile:
+            return ConflictList()
+        try:
+            if confile.next() != CONFLICT_HEADER_1 + '\n':
+                raise ConflictFormatError()
+        except StopIteration:
+            raise ConflictFormatError()
+        return ConflictList.from_stanzas(RioReader(confile))
+
+
+def get_conflicted_stem(path):
+    for suffix in CONFLICT_SUFFIXES:
+        if path.endswith(suffix):
+            return path[:-len(suffix)]
+
+@deprecated_function(zero_eight)
+def is_control_file(filename):
+    """See WorkingTree.is_control_filename(filename)."""
+    ## FIXME: better check
+    filename = normpath(filename)
+    while filename != '':
+        head, tail = os.path.split(filename)
+        ## mutter('check %r for control file' % ((head, tail),))
+        if tail == '.bzr':
+            return True
+        if filename == head:
+            break
+        filename = head
+    return False
+
+
+class WorkingTreeFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in an dict by their format string for reference 
+    during workingtree opening. Its not required that these be instances, they
+    can be classes themselves with class methods - it simply depends on 
+    whether state is needed for a given format or not.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every time regardless.
+    """
+
+    _default_format = None
+    """The default format used for new trees."""
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def find_format(klass, a_bzrdir):
+        """Return the format for the working tree object in a_bzrdir."""
+        try:
+            transport = a_bzrdir.get_workingtree_transport(None)
+            format_string = transport.get("format").read()
+            return klass._formats[format_string]
+        except NoSuchFile:
+            raise errors.NoWorkingTree(base=transport.base)
+        except KeyError:
+            raise errors.UnknownFormatError(format_string)
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def get_format_description(self):
+        """Return the short description for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats can be initialized and opened.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+
+
+class WorkingTreeFormat2(WorkingTreeFormat):
+    """The second working tree format. 
+
+    This format modified the hash cache from the format 1 hash cache.
+    """
+
+    def get_format_description(self):
+        """See WorkingTreeFormat.get_format_description()."""
+        return "Working tree format 2"
+
+    def initialize(self, a_bzrdir, revision_id=None):
+        """See WorkingTreeFormat.initialize()."""
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        branch = a_bzrdir.open_branch()
+        if revision_id is not None:
+            branch.lock_write()
+            try:
+                revision_history = branch.revision_history()
+                try:
+                    position = revision_history.index(revision_id)
+                except ValueError:
+                    raise errors.NoSuchRevision(branch, revision_id)
+                branch.set_revision_history(revision_history[:position + 1])
+            finally:
+                branch.unlock()
+        revision = branch.last_revision()
+        inv = Inventory() 
+        wt = WorkingTree(a_bzrdir.root_transport.base,
+                         branch,
+                         inv,
+                         _internal=True,
+                         _format=self,
+                         _bzrdir=a_bzrdir)
+        wt._write_inventory(inv)
+        wt.set_root_id(inv.root.file_id)
+        wt.set_last_revision(revision)
+        wt.set_pending_merges([])
+        build_tree(wt.basis_tree(), wt)
+        return wt
+
+    def __init__(self):
+        super(WorkingTreeFormat2, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirFormat6()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        return WorkingTree(a_bzrdir.root_transport.base,
+                           _internal=True,
+                           _format=self,
+                           _bzrdir=a_bzrdir)
+
+
+class WorkingTreeFormat3(WorkingTreeFormat):
+    """The second working tree format updated to record a format marker.
+
+    This format:
+        - exists within a metadir controlling .bzr
+        - includes an explicit version marker for the workingtree control
+          files, separate from the BzrDir format
+        - modifies the hash cache format
+        - is new in bzr 0.8
+        - uses a LockDir to guard access to the repository
+    """
+
+    def get_format_string(self):
+        """See WorkingTreeFormat.get_format_string()."""
+        return "Bazaar-NG Working Tree format 3"
+
+    def get_format_description(self):
+        """See WorkingTreeFormat.get_format_description()."""
+        return "Working tree format 3"
+
+    _lock_file_name = 'lock'
+    _lock_class = LockDir
+
+    def _open_control_files(self, a_bzrdir):
+        transport = a_bzrdir.get_workingtree_transport(None)
+        return LockableFiles(transport, self._lock_file_name, 
+                             self._lock_class)
+
+    def initialize(self, a_bzrdir, revision_id=None):
+        """See WorkingTreeFormat.initialize().
+        
+        revision_id allows creating a working tree at a differnet
+        revision than the branch is at.
+        """
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        transport = a_bzrdir.get_workingtree_transport(self)
+        control_files = self._open_control_files(a_bzrdir)
+        control_files.create_lock()
+        control_files.lock_write()
+        control_files.put_utf8('format', self.get_format_string())
+        branch = a_bzrdir.open_branch()
+        if revision_id is None:
+            revision_id = branch.last_revision()
+        inv = Inventory() 
+        wt = WorkingTree3(a_bzrdir.root_transport.base,
+                         branch,
+                         inv,
+                         _internal=True,
+                         _format=self,
+                         _bzrdir=a_bzrdir,
+                         _control_files=control_files)
+        wt.lock_write()
+        try:
+            wt._write_inventory(inv)
+            wt.set_root_id(inv.root.file_id)
+            wt.set_last_revision(revision_id)
+            wt.set_pending_merges([])
+            build_tree(wt.basis_tree(), wt)
+        finally:
+            wt.unlock()
+            control_files.unlock()
+        return wt
+
+    def __init__(self):
+        super(WorkingTreeFormat3, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        control_files = self._open_control_files(a_bzrdir)
+        return WorkingTree3(a_bzrdir.root_transport.base,
+                           _internal=True,
+                           _format=self,
+                           _bzrdir=a_bzrdir,
+                           _control_files=control_files)
+
+    def __str__(self):
+        return self.get_format_string()
+
+
+# formats which have no format string are not discoverable
+# and not independently creatable, so are not registered.
+__default_format = WorkingTreeFormat3()
+WorkingTreeFormat.register_format(__default_format)
+WorkingTreeFormat.set_default_format(__default_format)
+_legacy_formats = [WorkingTreeFormat2(),
+                   ]
+
+
+class WorkingTreeTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple workingtree formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and workingtree_format
+    classes into each copy. Each copy is also given a new id() to make it
+    easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        from bzrlib.tests import TestSuite
+        result = TestSuite()
+        for workingtree_format, bzrdir_format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = bzrdir_format
+            new_test.workingtree_format = workingtree_format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), workingtree_format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result

=== added file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml4.py	2006-01-19 21:25:01 +0000
@@ -0,0 +1,198 @@
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml_serializer import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        kind = elt.get('kind')
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_id = elt.get('text_id')
+            ie.text_sha1 = elt.get('text_sha1')
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+
+        ## mutter("read inventoryentry: %r", elt.attrib)
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for i, parent_id in enumerate(rev.parents):
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert parent_id
+                p.set('revision_id', parent_id)
+                if i < len(rev.parent_sha1s):
+                    p.set('revision_sha1', rev.parent_sha1s[i])
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev.parent_ids.append(p.get('revision_id'))
+                rev.parent_sha1s.append(p.get('revision_sha1'))
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parent_ids[0]
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev.parent_ids.append(precursor)
+            rev.parent_sha1s.append(precursor_sha1)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+

=== added file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml5.py	2006-04-06 10:21:12 +0000
@@ -0,0 +1,211 @@
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml_serializer import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+class Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+    """
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory',
+                    format='5')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        if inv.revision_id is not None:
+            e.set('revision_id', inv.revision_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        if not InventoryEntry.versionable_kind(ie.kind):
+            raise AssertionError('unsupported entry kind %s' % ie.kind)
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_sha1', 'revision', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        if ie.executable:
+            e.set('executable', 'yes')
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       format='5',
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+        if rev.parent_ids:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for parent_id in rev.parent_ids:
+                assert isinstance(parent_id, basestring)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                p.set('revision_id', parent_id)
+        if rev.properties:
+            self._pack_revision_properties(rev, root)
+        return root
+
+
+    def _pack_revision_properties(self, rev, under_element):
+        top_elt = SubElement(under_element, 'properties')
+        for prop_name, prop_value in sorted(rev.properties.items()):
+            assert isinstance(prop_name, basestring) 
+            assert isinstance(prop_value, basestring) 
+            prop_elt = SubElement(top_elt, 'property')
+            prop_elt.set('name', prop_name)
+            prop_elt.text = prop_value
+            prop_elt.tail = '\n'
+        top_elt.tail = '\n'
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        revision_id = elt.get('revision_id')
+        inv = Inventory(root_id, revision_id=revision_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        if not InventoryEntry.versionable_kind(kind):
+            raise AssertionError('unsupported entry kind %s' % kind)
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_sha1 = elt.get('text_sha1')
+            if elt.get('executable') == 'yes':
+                ie.executable = True
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+        ie.revision = elt.get('revision')
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+        parents = elt.find('parents') or []
+        for p in parents:
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev.parent_ids.append(p.get('revision_id'))
+        self._unpack_revision_properties(elt, rev)
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+    def _unpack_revision_properties(self, elt, rev):
+        """Unpack properties onto a revision."""
+        props_elt = elt.find('properties')
+        assert len(rev.properties) == 0
+        if not props_elt:
+            return
+        for prop_elt in props_elt:
+            assert prop_elt.tag == 'property', \
+                "bad tag under properties list: %r" % p.tag
+            name = prop_elt.get('name')
+            value = prop_elt.text
+            assert name not in rev.properties, \
+                "repeated property %r" % p.name
+            rev.properties[name] = value
+
+
+serializer_v5 = Serializer_v5()

=== renamed file 'bzrlib/xml.py' => 'bzrlib/xml_serializer.py'
--- bzrlib/xml.py	2005-08-17 02:06:19 +0000
+++ bzrlib/xml_serializer.py	2006-01-19 21:25:01 +0000
@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
@@ -20,21 +19,55 @@
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
-# importing this module is fairly slow because it has to load several ElementTree bits
+# importing this module is fairly slow because it has to load several
+# ElementTree bits
+
+from bzrlib.trace import mutter, warning
+
 try:
-    from util.cElementTree import ElementTree, SubElement, Element
+    from cElementTree import (ElementTree, SubElement, Element,
+                              XMLTreeBuilder, fromstring, tostring)
 except ImportError:
-    from util.elementtree.ElementTree import ElementTree, SubElement, Element
-
-
-def pack_xml(o, f):
-    """Write object o to file f as XML.
-
-    o must provide a to_element method.
-    """
-    ElementTree(o.to_element()).write(f, 'utf-8')
-    f.write('\n')
-
-
-def unpack_xml(cls, f):
-    return cls.from_element(ElementTree().parse(f))
+    mutter('WARNING: using slower ElementTree; consider installing cElementTree'
+           " and make sure it's on your PYTHONPATH")
+    from util.elementtree.ElementTree import (ElementTree, SubElement,
+                                              Element, XMLTreeBuilder,
+                                              fromstring, tostring)
+
+from bzrlib.errors import BzrError
+
+
+class Serializer(object):
+    """Abstract object serialize/deserialize"""
+    def write_inventory(self, inv, f):
+        """Write inventory to a file"""
+        elt = self._pack_inventory(inv)
+        self._write_element(elt, f)
+
+    def write_inventory_to_string(self, inv):
+        return tostring(self._pack_inventory(inv)) + '\n'
+
+    def read_inventory_from_string(self, xml_string):
+        return self._unpack_inventory(fromstring(xml_string))
+
+    def read_inventory(self, f):
+        return self._unpack_inventory(self._read_element(f))
+
+    def write_revision(self, rev, f):
+        self._write_element(self._pack_revision(rev), f)
+
+    def write_revision_to_string(self, rev):
+        return tostring(self._pack_revision(rev)) + '\n'
+
+    def read_revision(self, f):
+        return self._unpack_revision(self._read_element(f))
+
+    def read_revision_from_string(self, xml_string):
+        return self._unpack_revision(fromstring(xml_string))
+
+    def _write_element(self, elt, f):
+        ElementTree(elt).write(f, 'utf-8')
+        f.write('\n')
+
+    def _read_element(self, f):
+        return ElementTree().parse(f)

=== renamed file 'bzrlib/newinventory.py' => 'contrib/newinventory.py'
--- bzrlib/newinventory.py	2005-07-11 07:33:45 +0000
+++ contrib/newinventory.py	2005-09-27 07:24:40 +0000
@@ -14,13 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from xml import ElementTree, Element
+from bzrlib.xml import ElementTree, Element
 
 
 def write_inventory(inv, f):
     el = Element('inventory', {'version': '2'})
     el.text = '\n'
-    
+
     root = Element('root_directory', {'id': inv.root.file_id})
     root.tail = root.text = '\n'
     el.append(root)

=== modified file 'contrib/pwk'
--- contrib/pwk	2005-07-07 10:31:36 +0000
+++ contrib/pwk	2006-02-28 14:45:51 +0000
@@ -10,9 +10,6 @@
 PWK_ROOT='http://patchwork.ozlabs.org/bazaar-ng'
 PWK_AUTH_ROOT='https://patchwork.ozlabs.org/bazaar-ng'
 
-# bzr uses -p0 style; others use -p1
-PATCH_OPTS='-p0'
-
 usage() {
     cat <<EOF
 usage: 
@@ -42,12 +39,12 @@
     catpatch $2 | ${PAGER:-less}
     ;;
 try)
-    catpatch $2 | patch -p0 --dry-run
+    catpatch $2 | patch -p1 --dry-run
     ;;
 apply)
-    catpatch $2 | patch -p0
+    catpatch $2 | patch -p1
     ;;
 *)
     usage
     exit 1
-esac
\ No newline at end of file
+esac

=== modified file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-05-09 04:30:00 +0000
+++ contrib/zsh/_bzr	2005-11-18 08:10:07 +0000
@@ -9,13 +9,308 @@
 # how to get zsh to reload this file when it changes, other than by
 # starting a new zsh.
 
-# This is not very useful at the moment because it only completes on
-# commands and breaks filename expansion for other arguments.
-
-
-_arguments -S "1::bzr command:($(bzr help commands | grep -v '^   '))"
-
-
-# Local variables:
-# mode: shell-script
-# End:
+local _bzr_subcommands expl curcontext="$curcontext" state line
+local bzrRoot helpArgs parentFile
+typeset -A opt_args
+
+helpArgs=( '(--help)-h[show help message]' '(-h)--help[show help message]' )
+bzrRoot=$(bzr root 2> /dev/null)
+parentFile=$bzrRoot/.bzr/parent
+
+if [[ $service == "bzr" ]]; then
+    _arguments -C -A "-*" \
+    '*::command:->subcmd' && return 0
+
+    if (( CURRENT == 1 )); then
+      _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+      _describe -t subcommand 'subcommand' _bzr_subcommands
+      return
+    fi
+
+    service="$words[1]"
+    curcontext="${curcontext%:*}=$service:"
+fi
+
+_unknownFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_unknownRoot() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --from-root --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_versionedFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --versioned)"})
+    compadd -af fileList
+    return 0
+}
+
+case $service in
+    (add)
+       _arguments $helpArgs \
+       '--no-recurse[do not recurse into subdirectories]' \
+       '(-q)--quiet' \
+       '(--quiet)-q' \
+       '*:unknown files:_unknownFiles'
+    ;;
+
+    (annotate|praise|blame)
+        _arguments $helpArgs '*:files:_versionedFiles'
+    ;;
+
+    (clone|branch|get)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:FROM_LOCATION:_files -/'
+        elif (( CURRENT == 3 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:TO_LOCATION:_files -/'
+        fi
+    ;;
+
+    (rename|mv)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:old name:_versionedFiles'
+        else
+            _arguments $helpArgs '*:new name:'
+        fi
+    ;;
+
+    (cat)
+        _arguments $helpArgs \
+        '(-r)--revision:rev:' \
+        '(--revision)-r:rev:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (root)
+        _arguments $helpArgs '*:file:_files'
+    ;;
+
+    (log)
+        _arguments $helpArgs \
+        '(-r)--revision[revision or range]:rev or rev range:' \
+        '(--revision)-r[revision or range]:rev or rev range:' \
+        '(--verbose)-v[show revision manifest]' \
+        '(-v)--verbose[show revision manifest]' \
+        '(--short --long)-l[use long format (default)]' \
+        '(--short -l)--long[use long format (default)]' \
+        '(-l --long)--short[use short format]' \
+        '(--message)-m[specify regexp]:regexp:' \
+        '(-m)--message[specify regexp]:regexp:' \
+        '--show-ids[show file IDs]' \
+        '--forward[reverse direction of revisions]' \
+        '--timezoe[specify timezone for dates]:timezone:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (resolve)
+        _arguments $helpArgs '--all' '*:file:_versionedFiles'
+    ;;
+
+    (status|st|stat)
+        _arguments $helpArgs \
+        '--all[include unchanged versioned files]' \
+        '--show-ids[show file IDs]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (check)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:DIR:_files -/'
+    ;;
+
+    (mkdir|upgrade|renames)
+        _arguments $helpArgs '*:DIR:_files -/'
+    ;;
+
+    (remove|rm)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (pull)
+        _arguments $helpArgs \
+        '--remember[Remember the specified location as a default]' \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (missing|miss|mis)
+        _arguments $helpArgs \
+        '(--verbose -q --quiet)-v' \
+        '(-v -q --quiet)--verbose' \
+        '(--verbose -v --quiet)-q' \
+        '(-v -q --verbose)--quiet' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (commit|checkin|ci)
+        _arguments $helpArgs \
+        '(--message)-m[commit message]:message text:' \
+        '(-m)--message[commit message]:message text:' \
+        '(--file)-F[commit message from file]:message file:' \
+        '(-F)--file[commit message from file]:message file:' \
+        '--unchanged[include unchanged files]' \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:modified files:_versionedFiles'
+    ;;
+
+    (conflicts|added|deleted|modified|unknowns|directories|ignored)
+        _arguments $helpArgs
+    ;;
+
+    (revno|init|version)
+        _arguments $helpArgs
+    ;;
+
+    (whoami)
+        _arguments $helpArgs '--email[only show e-mail address]' 
+    ;;
+
+    (inventory)
+        _arguments $helpArgs \
+        '(-r)--revision[show inventory of a revision]:revision:' \
+        '(--revision)-r[show inventory of a revision]:revision:' \
+        '--show-ids[show file IDs]'
+    ;;
+
+    (diff|dif|di)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--diff-options[options to pass to gdiff]:diff options:' \
+        '*:files:_files'
+    ;;
+
+    (export)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--format=[format of exported file]:format:(dir tar tgz tbz2)' \
+        '--root[root directory of patch]:_files -/' \
+        '*:destination:_files'
+    ;;
+
+    (ignore)
+        _arguments $helpArgs '*:NAME_PATTERN:_unknownRoot'
+    ;;
+
+    (info)
+        _arguments $helpArgs '*:branch:_files -/'
+    ;;
+
+    (testament)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:branch:_files -/'
+    ;;
+
+    (revert|merge-revert)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--no-backup[skip generation of backup~ files]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (merge)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--reprocess[Reprocess to reduce spurious conflicts]' \
+        '--show-base[Show base revision text in conflicts]' \
+        '--force[ignore uncommitted changes]' \
+        '--merge-type:merge type:(diff3 merge3 weave)' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (ls)
+        _arguments $helpArgs \
+        '(--verbose)-v[display more information]' \
+        '(-v)--verbose[display more information]' \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--from-root[Print all paths from the root of the branch]' \
+        '--non-recursive[Do not recurse into subdirectories]' \
+        '--null[Null separate the files]' \
+        '--ignored[Print ignored files]' \
+        '--unknown[Print unknown files]' \
+        '--versioned[Print versioned files]'
+    ;;
+
+    (move)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:files:_versionedFiles'
+        else
+            _arguments '*:destination dir:_files -/'
+        fi
+    ;;
+
+    (help)
+        _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+        _arguments $helpArgs \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:subcmds:->cmds'
+        _describe -t subcommand 'subcommand' _bzr_subcommands
+    ;;
+
+    # Plugins
+
+    (visualize|visualise|viz|vis)
+        _arguments $helpArgs \
+            '(-r)--revision[starting revision]:rev:' \
+            '(--revision)-r[starting revision]:rev:'
+    ;;
+
+    (gannotate|gblame|gpraise)
+        _arguments $helpArgs \
+            '--all[show annotations on all lines]' \
+            '--plain[do not hightlight annotation lines]' \
+            '*:files:_versionedFiles'
+    ;;
+
+    (push)
+        _arguments $helpArgs \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (clean-tree)
+        _arguments $helpArgs \
+        '--dry-run[show files to delete instead of deleting them]' \
+        '--ignored[delete all ignored files]' \
+        '--detritus[delete conflict files, merge backups, failed self-tests, *~, *.tmp, etc]'
+    ;;
+
+
+    (*)
+        _message "unknown bzr command completion: $service"
+    ;;
+esac

=== removed file 'doc/Makefile'
--- doc/Makefile	2005-05-02 07:29:28 +0000
+++ doc/Makefile	1970-01-01 00:00:00 +0000
@@ -1,20 +0,0 @@
-all: $(addsuffix .html,$(basename $(wildcard *.txt))) todo.html
-
-clean:
-	-rm *.html
-
-%.html: %.txt
-	rest2html $^ > $@.tmp && mv $@.tmp $@
-
-todo.html: ../TODO
-	rest2html $^ > $@.tmp && mv $@.tmp $@
-
-
-
-upload: all
-	rsync -av \
-	./ \
-	escudero.ubuntu.com:/srv/www.bazaar-ng.org/www/doc/ \
-	--delete \
-	--include \*.html --exclude \* 
-

=== removed file 'doc/adoption.txt'
--- doc/adoption.txt	2005-03-09 04:51:05 +0000
+++ doc/adoption.txt	1970-01-01 00:00:00 +0000
@@ -1,118 +0,0 @@
-****************
-Driving adoption
-****************
-
-Getting adoption means persuading people that it's a good choice.
-
-
-I think the key is to have something that key project leaders see as
-worth using.  Imagine what it would take to get tridge, havoc, or akpm
-to switch.  Or not even to switch, but to even just try it out.
-
-* Simple operations must be simple.
-
-* The project and the implementation must not have bad smells.
-
-* Given their current understanding of the problem, there must be at
-  one feature that's clearly better than what they're currently
-  using.
-
-
-What holds it back now?
-
-* Too complex on initial impression
-
-* Bad smell from having so many forks/wrappers/kooky opinions
-
-* Some of the more exotic features can only be appreciated on
-  familiarity
-
-* It doesn't actually achieve by default a lot of the advantages that
-  it ought to: for example it still blocks on the network
-
-
-Good features at the moment
-
-* Archive storage is clean; probably makes a favorable impression on
-  people who look at it
-
-* Relatively few dependencies (if you don't look too closely at
-  hackerlab, etc)
-
-
-From `Ben Collins-Sussman`__
-
-__ http://www.red-bean.com/sussman/svn-anti-fud.html
-
-   If you're learning about Subversion and thinking of using it in
-   your group or company, please approach it the way you'd approach
-   any new product: with caution. This isn't to say that Subversion is
-   unreliable... but that doesn't mean you shouldn't use some common
-   sense either. Don't blindly jump into the deep end without a
-   test-drive. No user wants a new product forced upon them, and if
-   you're going to be responsible for administering the system, you
-   better have some familiarity with it before rolling it out to
-   everyone. Find a smallish project, and set it up as a "pilot" for
-   Subversion. Ask for enthusiastic volunteers to test-drive the
-   experiment. In the end, if Subversion turns out to be a good fit,
-   you'll have much happier developers (who have been part of the
-   process from the start) and you'll be ready to support a larger
-   installation as well. [...]
-
-   When Subversion hit "alpha" it was already being used by dozens of private developers and shops for real work. Any other project probably would have called the product "1.0" at that point, but we deliberately decided to delay that label as long as possible. Because we're talking managing people's irreplaceable data, the project was extremely conservative about labeling something 1.0. We were aware that many people were waiting for that label before using Subversion, and had very specific expectations about the meaning of that label. So we stuck to that standard. All it takes is one high-profile case of data loss to destroy an SCM's reputation.
-
-
-`John S. Yates, Jr.`__:
-
-__ http://lists.gnu.org/archive/html/gnu-arch-users/2004-10/msg00370.html
-
-  First let me say that I have nothing but increasing respect for
-  Tom's skills and accomplishments.
-  
-  That said I see Gnu Arch as really just emerging from a period
-  of prototype development.  If the project really wants to take
-  over the world, and especially supplant projects with momentum
-  and commercial customers (e.g. Subversion, BitKeeper, etc) then
-  I would warn against two mistakes I have experienced repeatedly
-  in my career:
-  
-  1) Deferring to a tiny installed base instead of focusing on
-     eliminating barriers to adoption
-  
-  2) Believing that great technology will be irresistible no matter
-     how it is presented
-  
-  Appearances matter.  Expectations matter.  Standards (official
-  or de facto) matter.
-
-There is also a `reply from Tom`__.
-
-__ http://lists.gnu.org/archive/html/gnu-arch-users/2004-10/msg00430.html
-
-Clear wins
-----------
-
-To convince people to use Baz, there has to be some feature they can
-clearly understand which will be much better under Baz.  It must be
-something they do today.
-
-* Offline support
-
-* Almost no network delays
-
-* Atomic changes (svn already has this)
-
-* Correct repeated merges
-
-* Read-only mirroring archives (not really important) 
-
-My model is that people will consider changing if 
-
-1. it's at least as good as cvs/svn
-2. AND there are no big concerns about implementation/safety
-3. AND there is at least one feature which is easy to use and a big win
-
-This gets you to some people at least trying it.  Will people migrate
-big projects to it?  Maybe, if it looks safe, it fixes there problem,
-and it doesn't look like something substantially better is on the
-horizon.

=== removed file 'doc/bitkeeper.txt'
--- doc/bitkeeper.txt	2005-04-15 01:31:21 +0000
+++ doc/bitkeeper.txt	1970-01-01 00:00:00 +0000
@@ -1,24 +0,0 @@
-Bitkeeper compared to Arch
-==========================
-
-BK has a default GUI, which is good, but it's ugly and not all that
-friendly to the new user.  It does pay attention to allowing quick
-keyboard navigation.
-
-The tool itself is also a bit unfriendly in terms of emitting a lot of
-noise messages and having lots of weird commands.
-
-Bitkeeper always requires both per-file and per-changeset commands.
-It seems bad to always require this; at most per-file messages should
-be optional.  Are they really the best option?
-
-Claimed to have extremely space-efficient storage, keeping 19 years of
-history in slightly more than twice the size of the working copy. 
-
-Hashes: does Baz always have these even without signing?  It really
-should.
-
-Fine-grained event triggers, pre- and post-event.
-
-Can remotely find status of a tree, e.g. parent, number of comitters,
-versioned files, extras, modified, etc. 

=== removed file 'doc/changelogs.txt'
--- doc/changelogs.txt	2005-03-09 04:51:05 +0000
+++ doc/changelogs.txt	1970-01-01 00:00:00 +0000
@@ -1,30 +0,0 @@
-**********
-ChangeLogs
-**********
-
-Changelogs have an interesting relation to version control systems.
-They are, for some projects, the primary way of examining history.
-They provide a pattern and standard for writing commit history, which
-may encourage people to enter more details (function names, etc) in a
-systematic way than they otherwise might.  The ChangeLog is available
-offline, in tarfiles, or even when the project has been switched into
-a different VCS.
-
-GNU ChangeLog format is only the most important, not the only such
-file.  Debian uses a different format for packages.  At a higher
-level, 
-
-At the same time the information there is some redundancy: both the
-ChangeLog and the VCS want to hold the description of what has been
-done, including descriptive text, names of files changed, etc.
-
-Some people say__ that ChangeLogs are mostly needed because of the
-limitations of CVS (and originally RCS): if it was easy to read the VC
-history when disconnected and at local speed, and if there are atomic
-commits then they might be needed rather less.
-
-__ http://gcc.gnu.org/ml/gcc/2004-06/msg00270.html
-
-There should be hooks called to create the log message template and
-before committing it.  These might, for example, populate it with
-space for per-file comments, or check that it is in the right format.

=== removed file 'doc/cherry-picking.txt'
--- doc/cherry-picking.txt	2005-03-09 04:51:05 +0000
+++ doc/cherry-picking.txt	1970-01-01 00:00:00 +0000
@@ -1,42 +0,0 @@
-**************
-Cherry picking
-**************
-
-Bazaar-NG allows you to try to 'cherry pick' changes from one branch
-to another.  That is to say that you can merge across only the changes
-from a particular commit, without getting anything else from that
-branch.
-
-However, because of dependencies in the source, it's not always very
-practical to pick changes out of a widely-diverged branch.  The
-general principle is that it's easier to keep things separate and
-combine them later than to mix things up and try to separate them.
-
-Therefore, we suggest using branches for each line of development that
-might want to separately merge.  Create a branch for each major
-feature; create branches for code that is ready to go upstream and for
-code that is very experimental.  Branches are cheap.  As changes pass
-through these branches they accumulate interesting metadata.
-
-Design
-------
-
-There is little that a tool can do to help with merges once you start
-cherry-picking changes; we need to rely on the merge tool detecting
-that some changes have already been applied.
-
-We should note that the changes have already come across, but I don't
-see many places where that will be terribly helpful.
-
-Sometimes a succession of cherry picks will be equivalent to merging
-the whole history up to some point, in which case we can fall back to
-a proper merge.
-
-When choosing a merge ancestor I think knowing non-contiguous
-cherry-picked changes will not help.
-
-Cherry-picking is equivalent to putting the merge ancestor just before
-the changeset(s) to be picked in.  So we can do this regardless of
-whether we have the whole history of the tree.
-
-

=== removed file 'doc/cmdref.txt'
--- doc/cmdref.txt	2005-03-09 04:51:05 +0000
+++ doc/cmdref.txt	1970-01-01 00:00:00 +0000
@@ -1,789 +0,0 @@
-***********************
-Reference for Bazaar-NG
-***********************
-
-:Author: Martin Pool
-:Copyright: Copyright 2004, 2005 Martin Pool; GNU GPL v2
-
-.. contents::
-
-
-
-Generalities
-------------
-
-Note that this document refers to many features which are not
-implemented yet.  Some may never be implemented.
-
-The top level command is ``bzr``.  Everything else is invoked as a
-subcommand of that, as is common in version control systems.
-
-A source directory containing some files to be versioned is called a
-*tree*; this contains a number of *files* and possibly some
-subdirectories.  The point of bzr is to keep track of the *versions*
-(or *revisions*) of the tree at various points in time.  A sequence of
-versions of a tree form a *branch*.  All branches begin with the empty
-directory, called the null revision.  Two branches may have some
-revisions in common and then *diverge* later -- which is why they're
-called branches.  We can distinguish the *working copy* of the files,
-which is available for modification, from their previous versions.
-
-Each file has a unique *file-id*, which remains the same for the life
-of the file, even when files are renamed.  Files also have a *kind*,
-which can currently be *file* or *directory*, and which cannot change
-for any file-id.
-
-The difference between two revisions is a *changeset*.  Changesets
-include the textual changes to files, the affected file-ids, the date
-of the change, the author, the name of the branch.  Changesets have a
-globally unique identifier.
-
-A changeset committed to a branch is *local* to that branch; otherwise
-it is foreign.  (Note that a changeset may be local to several
-different branches if it was created prior to their divergence.)
-
-Changesets may include a note that they *incorporate* other
-changesets; this is used when changes are merged from one branch into
-another.  After changes have been merged but before they are committed
-they are listed as *pending merges*; when they are committed they are
-listed within their changeset.
-
-All files in a versioned directory can be divided into four classes:
-
-Versioned
-  Anything assigned a file-id.  Changes to these files are tracked.
-Control
-  Anything under ``.bzr/``.  Should not be edited by users.
-Ignored
-  Anything matching an ignore pattern.  As the name suggests, ignored
-  by Bazaar.
-Unknown
-  Everything else.  Generally ignored, except that they are reported
-  as 'unknown' and can be added by a recursive add_.
-
-We say a tree, or a file, is *unmodified* if it is the same as in the
-most recent version, or *modified* otherwise.
-
-*Committing* creates a new revision equal to the current state of the
-tree.  Immediately after committing, all files are unmodified, since
-they are the same as the just-committed state.
-
-
-
-
-File states
------------
-
-A file is in exactly one of these *states*.  These are identified by
-single characters which should be familiar to people used to svn/cvs.
-
-?, Unknown
-    Not versioned, ignored, or control.
-    Typically new files that should be versioned or
-    ignored but have not been marked as either yet.
-., Up-to-date
-    This versioned file is the same as in the last revision on this
-    branch.  (The file had the same name and text and properties in
-    the previous revision.)
-A, Added
-    This file will be added by the next commit.  (The file ID was not
-    present in the previous revision.)
-M, Modified
-    This file has been changed from the previous revision and will be
-    recorded in the next commit.  (The file had the same name but
-    different text in the previous revision.)  Directories have no 
-    text and so can never be in this state.
-D, Deleted
-    This file has been removed from the inventory or deleted in the 
-    working copy and will be removed in the next revision.
-R, Renamed
-    This file had a different name in the previous revision; the next
-    commit will record the new name.  The file text may also have been
-    modified.
-
-These are only summaries, not complete descriptions.  For example the
-'D' state does not distinguish between a file removed by the remove_
-command and one whose text has been deleted.  (There should perhaps be
-a variation of the ``info`` command, or a ``file-info`` that shows
-enough details.)
-
-A directory may be up-to-date even if some files inside it have
-different states.
-
-.. _pinned: pinned.html
-
-
-
-Global options
---------------
-
-These can be applied to many or all commands.
-
---help     Show help, either globally or for the selected command.
-
--v, --verbose  Show progress/explanatory information.  This is good if
-               for example using a slow network and you want to see
-               that something is happening.
-
---debug    Show way too much detail about files being opened, locking,
-           etc.  Intended for
-
---silent   Show nothing but errors.  (Perhaps unnecessary; you could
-           just redirect stdout to /dev/null.)
-
---version  Show version and quit.
-
---dry-run         Show what would be done, but don't make any
-                  permanent changes.
-
---directory DIR   Operate in given directory rather than cwd.
-                  (Perhaps this should be ``-d``, but that might be
-                  better saved for ``--delete``.  Perhaps -b?)
-
---recurse, -R     When a directory name is given, operate not only on
-                  that directory but also any files or directories
-                  contained within it.
-
-                  (XXX: Should this be on by default?  It will often
-                  be what people want, but also possibly more
-                  surprising.  If it is, we will want a
-                  ``--no-recurse``.)
-
---force           Relax safety checks.
-
---format=FORMAT
-   Choose output format; options might include XML, YAML, text.
-
---id-only
-    Only update the inventory, don't touch the working copy.  (May
-    need a better name.)
-
---show-ids
-    List file ids, as well as names.
-
-
-
-init
-----
-
-Create bzr control files in the current directory::
-
-    bzr init
-
-Use this, followed by add_ and then commit_ to import or start a new
-project.
-
-
-
-add
----
-
-Add one or more files or directories to the branch::
-
-    bzr add FILE...
-
-Each added file is assigned a new file-id, and will be in the Added
-state.  They will be recorded in the next commit that covers that
-file, and then in the Up-to-date state.
-
-+-------------+-------------------------------------------------+
-|File state   |Action                                           |
-+=============+=================================================+
-|?            |New file id given to file; now in A state        |
-+-------------+-------------------------------------------------+
-|I            |"Adding previously ignored file"; now in A       |
-+-------------+-------------------------------------------------+
-|Z, ZM        |Add file with new ID; now in A state.            |
-|             |(To get the old ID back, use revert_ on that     |
-|             |file.)                                           |
-+-------------+-------------------------------------------------+
-|U, A, M, R,  |Warning "already added"; no change               |
-|RM           |                                                 |
-+-------------+-------------------------------------------------+
-|!, D         |Error "no such file"; no change                  |
-+-------------+-------------------------------------------------+
-|#            |Error "cannot add control file"; no change       |
-+-------------+-------------------------------------------------+
-
---id ID
-      Add file with given ID rather than a random UUID.  Error if this
-      id is assigned to any other file.  (not implemented yet)
-
---recurse
-    Add directory and all Unknown children, recursively.  This
-    includes U children of previously-added subdirectories.
-    (not implemented yet)
-
-This command will add any new source files, except for those matching
-ignore rules::
-
-  $ bzr add --recurse .
-
-(Note: I hope this might satisfy people who are fond of arch
-name-based tagging, and who dislike individually adding & removing
-files.  All they need to do is set up the appropriate ignore patterns,
-then 'add -R .' and they're done.)
-
-
-remove
-------
-
-Make a file no longer be versioned, and record its deletion from the
-inventory::
-
-    $ bzr remove FILE...
-
-This does not remove the working copy.  If you wish to both remove the
-working copy and unregister the file, you can simply delete it using
-regular ``rm``.  This is the opposite of add_.
-
-:State table:
-
-+--------------+--------------------------------------------------+
-| File state   | Action                                           |
-+==============+==================================================+
-| M, R, RM, .  | File to D state.                                 |
-+--------------+--------------------------------------------------+
-| A            | File back to I or ? state                        |
-+--------------+--------------------------------------------------+
-| I, ?, Z, ZM  | Error, no change                                 |
-+--------------+--------------------------------------------------+
-| !            | Change to D                                      |
-+--------------+--------------------------------------------------+
-| D            | Warning "already deleted"                        |
-+--------------+--------------------------------------------------+
-| #            | "Cannot remove control file"                     |
-+--------------+--------------------------------------------------+
-
-
-
-diff
-----
-
-Show all changes compared to the pristine as a text changeset.
-
---full      Include details for even binary files, uuencoded.  Makes the
-            diff long but lossless.
-
-
-
-export
-------
-
-::
-
-    bzr export TO-DIR
-
-Copy the tree, but leave out Bazaar-NG control files.  This includes copying
-uncommitted changes.
-
-This is equivalent to copying the branch and then deleting the control
-directory (except more efficient).
-
-Should this also delete any other control files like ``.bzrignore``?
-
-
-
-status
-------
-
-Show which files are added/deleted/modified/unknown/missing/etc, as in
-subversion.
-
-Given the ``--check`` option, returns non-zero if there are any
-uncommitted changes.
-
-
-
-info
-----
-
-Display various information items about the branch.  Can be given
-various options to pull out particular fields for easier use in
-scripts.
-
-Should include:
-
-* branch name
-* parent
-* number of revisions
-* number of files that are versioned/modified/deleted/added/unknown
-* number of versioned directories
-* branch format version
-* number of people who have made commits
-* date of last commit
-
-
-
-delta
------
-
-Compute a two-way non-history-sensitive delta from one branch or
-version to another.  Basically a smart diff between the two.
-
-(Perhaps this should just be a ``--format`` option to diff_?)
-
-
-
-merge
------
-
-Merge changes in from another branch, and leave them uncommitted in
-this tree::
-
-   merge [FROM-BRANCH]
-
-This makes a note of the revisions which were merged.
-
-A range of revisions may be specified to cherry-pick changes from that
-branch, or to merge changes only up to a certain point.
-
-Merge refuses to run if there are uncommitted changes unless forced,
-so that merged changes don't get mixed up with your own changes.
-
-You can use a merge rather than an update to accomplish any of several
-things:
-
-* Merge in a patch, but modify it to either suit your taste or fix
-  textual or semantic conflicts.
-
-* Collapse several merged patches into a single changeset.  A feature
-  may go through many revisions when being developed on its own
-  branch, but you might want to hide that detail when it merges onto a
-  main branch.
-
---revision RANGE  Merge only selected revisions, rather than
-                  everything that's not present yet.
-
-Before a merge is committed, it may be reversed with the revert_
-command.
-
-
-
-sync
-----
-
-::
-
-    sync [OTHER-BRANCH]
-
---revision RANGE  Pull only selected revisions.
-
-Synchronize mirrored branches.
-
-A mirror branch is a branch that strictly follows a parent branch,
-without any changes being committed to it.  This is useful in several
-ways:
-
-* Moving a backup of a branch onto another machine to protect against
-  disk failure or laptop theft.
-
-* Making the complete history of a upstream branch available for
-  offline use.
-
-The result is the same as copying the whole branch, but it is more
-efficient for remote branches because only newly-added changesets are
-moved.   The result is similar to rsync except it respects Bzr locking.
-
-The same command can be used for push mirrors (changesets are moved
-from this branch to the other) or pull mirrors (vice versa).  Bzr
-automatically determines which to do by looking at which branch has
-more patches.   (Perhaps it would be clearer to have separate *push*
-and *pull* commands?)
-
-This command can only be used when the history of one branch is a
-subset of the other.  If you commit different changes to both
-branches, then ``sync`` will say that the branches have diverged and
-it will refuse to run.  This command also refuses to run if the
-destination branch has any uncommitted changes.  Uncommitted changes
-on the origin are not copied.
-
-Method:
-
-* Get the ordered list of change ids on both branches.
-
-* One list should be a prefix of the other; if not, fail.  The shorter
-  list will be the destination branch.
-
-* Check the destination has no uncommitted changes.
-
-* For each change present only in the origin, download it to the
-  destination, add to the changeset history and update the tree and
-  control files.
-
-
-
-commit
-------
-
-::
-
-   commit MESSAGE [WHAT]
-
-Commit changes from the working copy into the branch.  By default
-commits everything, but can be given a list of files directories or
-files.
-
-Partial commits (not implemented yet) should ideally be allowed even when the partial thing
-you want to commit is an add, delete or rename.  Partial commits are
-**not** allowed if patches have been merged in this change.
-
-Method:
-
-* check preconditions
-* get log, if not present or given on command line
-* re-check preconditions
-* calculate diff from pristine to working copy
-* store this diff, plus headers, into the patches directory
-* add the diff to the list of applied patches
-* update pristine tree
-
-Commit can optionally run some pre-commit and post-commit hooks,
-passing them the delta which will be applied; this might mail the
-delta or apply it to an export and run a test case there.
-
-:State table:
-
-+-------------+-------------------------------------------------+
-|File state   |After commit                                     |
-+=============+=================================================+
-|?, I, #      |Unchanged                                        |
-+-------------+-------------------------------------------------+
-|A, M, R, RM  |Recorded, U                                      |
-+-------------+-------------------------------------------------+
-|D            |Gone                                             |
-+-------------+-------------------------------------------------+
-|Z, ZM        |Recorded, working copy remains as I or ?         |
-+-------------+-------------------------------------------------+
-|!            |Error "file is missing"; must be deleted or      |
-|             |reverted before commit                           |
-+-------------+-------------------------------------------------+
-
-
-
-lint
-----
-
-Check for problems in the working copy.
-
-+-------------+-------------------------------------------------+
-|File state   |Lint output                                      |
-+=============+=================================================+
-|?            |"Unknown file, please add/ignore/remove"         |
-+-------------+-------------------------------------------------+
-|Z, ZM        |"Zombie file, please add/ignore/remove/revert"   |
-+-------------+-------------------------------------------------+
-|!            |"File is missing, please recreate, delete or     |
-|             |revert."                                         |
-+-------------+-------------------------------------------------+
-|A, M, D, R,  |No output                                        |
-|RM, #, I     |                                                 |
-+-------------+-------------------------------------------------+
-
-
-
-check
------
-
-Run various consistency/sanity checks on the branch.  These might
-include:
-
-* Make sure all patches named by the inventory exist.
-
-* No patch IDs are duplicated.
-
-* Make sure hash of each patch is correct.
-
-* Starting at zero, apply each patch and make sure it does not
-  conflict.
-
-* All files named in inventory are reasonable.
-
-* No file IDs are duplicated in inventory.
-
-* Possibly many more.
-
-* No patches are in both ``patch-history`` and ``merged-patches``.
-
-* The patches in the bag are exactly those listed in ``patch-history``.
-
-Maybe add a separate option to say that you believe the tree is clean.
-
-
-
-backout
--------
-
-Reverse the changes made by previous changesets::
-
-   bzr backout REVISION
-   bzr backout REV1 REV2
-
-If a single revision is given, that single changeset is backed out.
-If two revisions are given, all changes in that range are backed out.
-
-The change that is reversed need not be the most recently committed
-change, but if there are revisions after the ones to be reverse which
-depend on them this command may cause conflicts.
-
-This undoes the changes but remembers that it was once applied, so it
-will not be merged again.  Anyone who pulls from later versions of
-this tree will also have that patch reversed.
-
-You can backout a backout patch, etc, which will restore the previous changes.
-
-This leaves the changeset prepared but not committed; after doing this
-you should commit it if you want.
-
-You can also backout only the parts of  a changeset touching a
-particular file or subdirectory::
-
-  bzr reverse foo.c@31
-
-
-revert
-------
-
-Undo changes to the working copy of files or directories, and discard
-any pending merge notes::
-
-  $ bzr revert [FILE...]
-
-If no files are specified, the entire tree is reverted, which is
-equivalent to specifying the top-level directory.  In either case, the
-list of pending merges is also cleared.
-
-This does not affect history, only the working copy.  A corollary is
-that if no changes have been made to the working copy, this does
-nothing.
-
-If a file was Modified, it is returned to the last committed state,
-reversing changes to the working copy.  If the file has been Added, it
-is un-added but the working copy is not removed, so it returns to
-either the Unknown or Ignored state.  If the file has been Deleted, it
-is resurrected and returns to the Up-to-date state.  If the file is
-Unknown, Ignored, or a Control file then it is not changed and a
-warning is issued::
-
-  bzr: warning: cannot revert ignored file foo.c
-
-If the file has been Renamed, it is returned to its original name and
-any textual changes are reversed.  This may cause an error if the
-rename clashes with an existing file::
-
-  bzr: error: cannot revert foo.c to old name bar.c: file exists
-
-If a directory is listed, by default only changes to the directory
-itself are undone.
-
-Ideally this would not lose any changes, but rather just set them
-aside, so that the revert command would be undoable.  One way is to
-follow Arch and write out the discarded changes to a changeset file
-which can be either re-applied or discarded at a later date.  This
-very nicely allows for arbitrarily nested undo.  A simpler
-intermediate mechanism would be to just move the discarded files to
-GNU-style tilde backups.
-
---merges        Clear the list of pending merges.  If files are
-                specified, their text is also reverted, otherwise no
-                files are changed.
-
---id-only       Don't touch the working text, only the inventory.
-
-
-+-------------+-------------------------------------------------+
-| File state  | Actions                                         |
-+=============+=================================================+
-| A           | Returned to ? or I state, working text unchanged|
-+-------------+-------------------------------------------------+
-| D           | Working copy restored, returned to U            |
-+-------------+-------------------------------------------------+
-| Z           | Returned to U                                   |
-+-------------+-------------------------------------------------+
-| ZM          | Working copy restored to previous               |
-|             | version, returned to U                          |
-+-------------+-------------------------------------------------+
-| R           | Moved back to old                               |
-|             | name.                                           |
-+-------------+-------------------------------------------------+
-| RM          | Moved back to old name and restored to previous |
-|             | text.                                           |
-+-------------+-------------------------------------------------+
-|I, #, ?      | "Cannot revert"                                 |
-+-------------+-------------------------------------------------+
-
-
-log
----
-
-Show a log of changes::
-
-    bzr log
-
-By default shows all changes on this branch.
-
-The default format shows merged changes indented under the change that
-merged them.
-
-Alternatively changes can be sorted by date disregarding merges, which
-shows the order in which they were written not the order they were
-merged.  Changes which were merged are flagged as such.  Such an order
-is needed for a GNU-style ChangeLog.  The option is comparable to
-choosing a threaded or unthreaded display in an email client, and
-should perhaps have those names.
-
-Another option is to show just short descriptions of the merged
-changes, similar to arch.
-
-The log can be filtered in any of these ways:
-
-* Logs touching a particular file or directory.
-
-* Changes touching a particular file-id, regardless of what name it
-  had in the past.
-
-* Changes by a particular author.
-
-* Changes from a particular branch name (not necessarily the same
-  branch).
-
-Another option is to also include diffs (which may make it quite
-large).
-
-
-ignore
-------
-
-Mark a file pattern to be ignored and not versioned::
-
-    bzr ignore PATTERN
-
-The pattern should be quoted on Unix to protect it against shell
-expansion.
-
-The pattern is appended to ``.bzr-ignore``.  This file is created if
-it does not already exist, and added if it is not already versioned.
-bzr prints a message showing the pattern added so that people can see
-where to go to remove it::
-
-    $ bzr ignore \*.pyc
-    bzr: notice: created .bzr-ignored
-    bzr: notice: added pattern '*.pyc' to .bzr-ignore
-    $ bzr status
-    A      .bzr-ignore
-
-If the wrong pattern is added it can be removed by either editing
-``.bzr-ignore`` or by reverting__ that file.
-
-__ revert_
-
-
-is
---
-
-Test various predicates against a branch, similar to the Unix shell
-``test`` command::
-
-    bzr is TEST [ARGS]
-
-Takes a third-level command:
-
-``clean``
-  Tree has no uncommitted changes.
-
-``in-tree``
-  Pwd is in a versioned tree.
-
-``in-control-dir``
-  Pwd is inside a Bazaar-NG control directory (and therefore should
-  not be modified directly).
-
-``tree-top``
-  Pwd is the top of a working tree.
-
-
-protect
--------
-
-(Proposed idea, may not be implemented or may need a better name.)
-
-Sets a voluntary write-protect flag on a branch, to protect against
-accidental changes::
-
-    bzr protect
-    bzr unprotect
-
-This is typically used on branches functioning as tags, which should
-not normally be committed to or updated.
-
-There may, in the future, be a mechanism to allow only particular
-users to protect/unprotect a branch.
-
-
-uncommit
---------
-
-(It is not certain this command will be implemented.)
-
-This command removes the most recent revision from the branch::
-
-    bzr uncommit
-
-This does not affect the working copy, which can be fixed up and a new
-commit run.  The new commit will have a different revision ID.
-
-Removal of the revision will not propagate to any other branches.
-
-The command may be repeated to successively remove more and more
-revisions.
-
-This command should perhaps have a more obviously dangerous name,
-since it can lose information or cause confusion.
-
-By default the revision is removed from the history, but its text is
-left in the store, which allows some chance of recovery.
-
-You cannot partially uncommit; you can however uncommit the whole
-revision and then re-commit just part of it.
-
-:Use cases:
-    `Wrong commit message`_
-
-
-.. _`Wrong commit message`: use-cases.html#wrong-commit-message
-
-
-
-find
-----
-
-Finds files, versions, etc in a branch::
-
-   bzr find [OPERATORS]
-
-The behaviour is similar to regular unix *find*, but this understands
-about bzr versioning.  Eventually this may gain all the
-and/or/grouping options of Unix find, but not yet.
-
-This does not have the quirky syntax of unix find, but rather just
-specifies commands as regular words.
-
-Operators:
-
-* ``directory``
-* ``file``
-* ``unknown``
-* ``ignored``
-
-By default the operators are anded together; there is also an ``or``
-operator.
-
-If no action is specified, just prints the file name.  Other actions:
-
-``print``
-    Print just the filename.
-``printf``
-    Print various fields about the object, using a formating system.
-``exec``
-    Execute a command on the file.
-

=== removed file 'doc/common-format.txt'
--- doc/common-format.txt	2005-03-09 04:51:05 +0000
+++ doc/common-format.txt	1970-01-01 00:00:00 +0000
@@ -1,39 +0,0 @@
-***********************
-Common changeset format
-***********************
-
-It might be useful to have a common changeset format for interchange
-between projects.
-
-It is not clear that you would be able to capture everything that
-every tool produces, but perhaps a lot could be done.
-
-Scenarios: (in order of difficulty?)
-
- * dump/load
-
- * convert repo from one tool to another
-
- * one-way sync from exotic tool into read-only CVS or Svn repo
-
- * two way sync between writable archives
-
-Perforce's RevML and ``vcp`` may be some use but apparently don't go
-all the way.
-
-Things like darcs and monotone may have such a different model that it
-may be hard to map them.
-
-Two-way sync may require keeping external state and is probably pretty
-hard.
-
-Apparently tlord tried to do something about this before but it
-stalled.
-
-Eventually perhaps we could arrange for someone to export key
-bitkeeper archives in this format, allowing all tools to read them in.
-
-Colin__ points the existing mailing list about this which is just a
-bunch of spam.
-
-__ http://web.verbum.org/blog/freesoftware/fsrc-responses

=== removed file 'doc/compared-aegis.txt'
--- doc/compared-aegis.txt	2005-03-09 04:51:05 +0000
+++ doc/compared-aegis.txt	1970-01-01 00:00:00 +0000
@@ -1,110 +0,0 @@
-****************
-Ideas from Aegis
-****************
-
-* Very mature -- in use for 14 years, many large projects, etc.
-
-* "Essential process can be learned in a day" -- which is still kind
-  of a long time; i'd like it to be well under an hour.
-
-* Good process integration; show who is supposed to be working on what
-  and how far through they are.
-
-* Very poor Windows support.
-
-* Distributed repositories. (??)
-
-* Very focussed on security -- can reproduce any previous revision;
-  availability/integrity/confidentiality; uses Unix permissions and
-  seteuid() to prevent users changing the database.
-
-* Does not itself track history -- assumes this will be done by some
-  other tool such as RCS operating on the baseline.
-
-* I think every individual project needs a single baseline. (??)
-
-* The baseline is always working and always releasable -- to the
-  extent that you have scripts which can enforce this.
-
-* Integration step is somewhat similar to that used by distributed
-  systems.  It seems that you could build some features of aegis as
-  policy macros on top of Bazaar-NG.
-
-* The baseline also contains object files built from current source,
-  which can be used to pre-populate working directories.  Also people
-  only need a copy of the source files they're changing.  I think the
-  economics of this have changed a bit.  Also tends to assume all
-  developers are on the same Unix host, which is no longer generally
-  true.
-
-* Only one review/integration can be in process at a time.
-
-* One process difference is that developers produce all changes;
-  reviewers/integrators can only accept or reject them.  This is
-  different from the integrator-makes-right model of many distributed
-  tools.  (Though the integrator still has the choice to reject, but
-  they have the option of fixing it too.)
-
-* Can automatically append Signed-off-by field.  Interesting idea.  I
-  wonder if we should have a metadata facility to include licence
-  data?
-
-* Much of the functionality of Aegis is to prevent people doing things
-  they could otherwise do.  That can be useful in enforcing a healthy
-  process but bazaar-ng is not the place for it. 
-
-* Can serialize an (in-progress) changeset to text, and then mail from
-  one place to another.
-
-* Branches are an extension of the 'change' concept; they can be
-  merged into the parent in the same way that a change can be.
-
-  Merging branches onto the mainline seems to hit a similar problem to
-  that of centralized branches.  If someone else has committed, you
-  need to make a new changeset reconciling all their changes, commit
-  that to the child branch, then commit everything to the parent.
-
-  This suggests a different way to do shared branches in Bazaar-NG:
-
-    You can push to the parent if you incorporate either directly or
-    by merger all changes on the parent.  That means that everything
-    someone has committed to the parent is present in some way in your
-    branch.  By extension, it is safe to transform the parent text
-    into your branch without losing anything.  We can therefore
-    remotely record that changeset to the parent.  This is more or
-    less what Aegis does.
-
-    The problem with this in the Bazaar-NG model is that then the new
-    commit will be only in the parent, and not on the child.  So if
-    you run ``bzr log`` on the child, you won't see what you just
-    committed.  We can't apply it to both because the predecessor is
-    different.  (darcs could do that, but it has a looser patch
-    history than I want.)
-
-
-
-
-Overall, the process model is good for a particular type of
-organization.  It would be good to build this on top of Bazaar-NG.  To
-support that we need:
-
- - patches are submitted, rather than being directly written in 
-
- - arbitrary levels of branching/review
-
- - users can submit changes to branches they are not directly allowed
-   to write to 
-
- - branches can be cleanly removed when they're no longer necessary
-
- - strong audit trail
-
-
-Interestingly, the BitKeeper model which is criticized__ by Greg
-Hudson is similar to that of Aegis: a single integrator (or small
-team?) who ultimately decide what gets into the main tree.
-
-__ http://web.mit.edu/ghudson/thoughts/bitkeeper.whynot
-
-
-The Aegis workflow can probably be `emulated in bzr <workflow.html>`_.

=== removed file 'doc/compared-codeville.txt'
--- doc/compared-codeville.txt	2005-03-12 08:54:12 +0000
+++ doc/compared-codeville.txt	1970-01-01 00:00:00 +0000
@@ -1,106 +0,0 @@
-Codeville
-*********
-
-Documentation on how this actually works is pretty scarce to say the
-least.
-
-I *think* I understand their merge algorithm though, and it's pretty
-clever.  Basically we do a two-way merge between annotated forms of
-the two files: that is, with each line marked with the revision in
-which it last changed.  (I am simplifying here by speaking of lines
-and changes, but I don't think it causes any essential problem.)
-
-Now we walk through each file, line by line.  If the change that
-introduced the line state in branch A is already merged into branch B,
-then we can just take B.
-
-Now is this actually better?
-
-It may be better in several ways:
-
-* Do not need to choose just a single ancestor, but rather can
-  take advantage of all possible previous changes.
-
-* Can handle OTHER containing changes which have been merged into
-  THIS, but have then been overwritten.
-
-* Can handle cherrypicks(!) by remembering which lines came in from
-  that cherrypick; then we don't need to merge them again.
-
-Some questions:
-
-* Do we actually need to store the annotations, or can we just infer
-  it at the time we do the merge?
-
-* Can this be accomodated in something like an SCCS weave format?  I
-  think something like a weave may work, in as much as it is basically
-  a concatenation of annotations, but I don't know if it represents
-  merges well enough to cope.
-
-Can this handle binaries or type-specific merges, and if so how?
-Unmergeable binaries are easy: just get the user to pick one.  Things
-like XML are harder; we probably need to punt out to a type-specific
-three-way merge.  Of course this approach does not forbid also
-offering a 3-way merge.
-
-----
-
-I suppose this could be accomodated by an annotation cache on top of
-plain history storage, or by using a storage format such as a weave
-that can efficiently produce annotation information.
-
-That is to say there is nothing inherently necessary about remembering
-the line history at the point when it is committed, except that it
-might be more efficient to do this once and remember it than to 
-
-----
-
-There is another interesting approach that can be used even in a tool
-that does not inherently remember annotations:
-
-Given two files to merge, find all regions of difference.  For each
-such, try to find a common ancestor having the same content for the
-region.  Subdivide the region if necessary.
-
-This naive approach is probably infeasible, since it would mean
-checking every possible predecessor.
-
-----
-
-Rather than storing or calculating annotations, we could try using a
-complex weave, which allows one file version to be represented as a
-weave of multiple disjoint previous versions.  It sounds complex but
-it might work.
-
-Essentially we store each file as a selection of lines that should be
-turned on in that file.  These files might come from any of the
-predecessors that were merged into that file.  Complex to get right
-but it might work.
-
-This is written in terms of lines, but it might make more sense to
-just use byte ranges: perhaps more efficient when handling long files,
-and makes binaries less of a special case.
-
-codeville in fact does *not* seem to do this, though to me it seems
-like a fairly natural corollary of their design.
-
-This seems to imply holding the file text and ancestry of every branch
-that ever merged into this one, rather than just finding them if we
-later want them.  Hm.  That is nice in terms of doing smart merges.
-That possibly causes trouble in terms of having a name for these
-branches floating around inside our space, and making sure we don't
-clash with them.  It may make sense in terms of having a working
-directory be just a view into a shared database, looking at a
-particular line of development.
-
-Indeed the main difficulty seems to be of naming branches in this
-space.  Perhaps we should move back to using repositories and named
-branches within them, but not rely on branch names being unique out of
-the context of a single repository.
-
-Wow, this seems to open a big can of worms.
-
-----
-
-So the conclusion is that this is very cool, but it does not require a
-fundamental change of model and can be implemented later.

=== removed file 'doc/compared-cvsnt.txt'
--- doc/compared-cvsnt.txt	2005-03-09 04:51:05 +0000
+++ doc/compared-cvsnt.txt	1970-01-01 00:00:00 +0000
@@ -1,32 +0,0 @@
-*****
-CVSNT
-*****
-
-
-http://www.cvsnt.com/cvspro/
-
- * Remembers last-merge-point__ between two branches, and uses this to
-   do smarter incremental merges.
-
-__ http://www.cvsnt.org/wiki/MergePoint
-
-http://www.cvsnt.org/wiki/CvsntAdvantages
-
- * SSPI authentication on Windows, and built-in Putty SSH code.
-
- * Branch ACLs.
- 
- * Remote user administration using cvs passwd command.
-
- * Repository browsing via cvs ls command (update: now provided in
-   standard CVS since version 1.12.9)
-
- * LockServer on a second port replaces filesystem-based locks &
-   provides file level locking
-
-
-Various other improvements relative to CVS, particularly for Windows
-users.  Worth looking at, but nothing apparently earth-shaking.
-
-Interesting that the advanced merge support is basically that it
-remembers what was pulled across.
\ No newline at end of file

=== removed file 'doc/compared-opencm.txt'
--- doc/compared-opencm.txt	2005-03-09 04:51:05 +0000
+++ doc/compared-opencm.txt	1970-01-01 00:00:00 +0000
@@ -1,36 +0,0 @@
-******************
-Compared to OpenCM
-******************
-
-  http://opencm.org/
-
-
-Not very stable; apparently inactive.  
-
-* Separated working copy & repository model.
-
-* Files have long-lived identifiers.
-
-* Files and revisions identified by the SHA-1 hash of their content
-  (as in monotone); explicitly makes it easier to be sure we have the
-  right one and prevents some tricks.
-
-* Binding of objects to external names done on the client, so you can
-  version e.g. database objects, instead of files.
-
-* Directories are inferred by having files that exist under them;
-  empty directories are a special case with an object of type DIR.
-  
-  This is a bit ugly.  I might rather have files given a name only
-  relative to their parent directory.  So renaming a directory will
-  only update the entry for the directory, and everything will move
-  with it.
-
-* Access-control rules about who can write to a central server. 
-
-* Their design is somewhat similar to ours and used a lot of disk
-  space -- enough to be a significant problem.
-
-* Assigning human names to branches proved problematic -- i think this
-  is a good reason to rely on the filesystem/URL space, which people
-  already know how to manage and deal with.

=== removed file 'doc/compared-prcs.txt'
--- doc/compared-prcs.txt	2005-03-09 04:51:05 +0000
+++ doc/compared-prcs.txt	1970-01-01 00:00:00 +0000
@@ -1,49 +0,0 @@
-********************
-Comparison with PRCS
-********************
-
-  http://prcs.sourceforge.net/
-
-* No network support; no Windows or Mac support.
-
-* Long-lived file IDs, generated from .  Files can be reintroduced
-  after they are deleted. 
-
-* Named branches.
-
-* Custom keyword replacement: can specify keywords such as the release
-  number.  Commands to update keywords, and to strip them out.
-  Separate ``rekey`` command can be used after a checkin to bring
-  keywords up to date.
-
-* Users are allowed to edit the manifest file, which also contains
-  some system-maintained data.  This apparently works OK but I think
-  it makes people uncomfotable.
-
-* ``populate`` command adds all unversioned files and unregister files
-  which are no longer present.
-
-* Merge algorithm is similar to the best-parent three-way merge we're
-  planning on using; in particular it remembers merged parents to help
-  with later choices.
-
-* ``$Format$`` feature useful in generating arbitrary tags inside text,
-  example__::
-
-    /* $Format: "static char* version = \"$ProjectVersion$\";"$ */
-    static char* version = "x.x";
-
-  transformed to::
-
-    /* $Format: "static char* version = \"$ProjectVersion$\";"$ */
-    static char* version = "1.2";
-
-  Also a good bundle of other keywords.
-
-* ``prcs execute`` is like find-xargs; perhaps easier to have a
-  find/exec pattern.
-
-__ http://svn.haxx.se/dev/archive-2004-12/1065.shtml
-
-
-Should have a look at their storage format; probably pretty efficient.
\ No newline at end of file

=== removed file 'doc/compared-teamware.txt'
--- doc/compared-teamware.txt	2005-03-09 04:51:05 +0000
+++ doc/compared-teamware.txt	1970-01-01 00:00:00 +0000
@@ -1,34 +0,0 @@
-Compared to Sun Teamware
-************************
-
-(**note**: I have never used Sun Teamware, so this document is just
-based on public documentation and information from other people.
-Corrections would be appreciated.)
-
-http://docs.sun.com/app/docs/doc/806-3573
-
-Needless to say bazaar-ng is currently far less mature, and some of
-the advantages listed below don't work in the current pre-1.0 code.
-But they are accounted for in the design.
-
-TeamWare has file locking.  Distributed systems can't easily support
-file locking because you can't prevent people diverging.  But perhaps
-we can have scripts or a higher-level tool to communicate that changes
-to a particular file are in train.  (One possibility: there is a
-development branch where that file has been changed, but not yet
-checked in.  Or if Bob can see Alice's work area, he can see that a
-particular file has been fetched read/write.)
-
-parent/child workspaces.
-
-Advantages of bazaar-ng:
-
-* Free / open source software
-
-* Prior tree revisions are always exactly reproducible (atomic
-  changesets, etc)
-
-* Can get whole-tree diff
-
-* Versioned, mergeable renames.
-

=== removed file 'doc/compression.txt'
--- doc/compression.txt	2005-03-09 04:51:05 +0000
+++ doc/compression.txt	1970-01-01 00:00:00 +0000
@@ -1,66 +0,0 @@
-Compression
-***********
-
-At the moment bzr just stores the full text of every file state ever.
-(Files unmodified from one revision are stored only once.)
-
-This is simple to write, and adequate for even reasonably large trees
-with many versions.  Disk is very cheap.  
-
-Eventually we might like something more compressed, but this is
-neither an interesting nor urgent problem.  (Not "interesting" in the
-sense that doing it is just a matter of coding; there is no
-theoretical problem or risk.)  
-
-There are various possibilities:
-
-* Store history of each file in RCS, relying on RCS to do line-by-line
-  delta compression.  (Does not handle binaries very well.)
-
-  OpenCMS paper has a horror story about using RCS for file storage.
-
-* Store full copies of each file in a container with gzip compression,
-  which should fairly efficiently eliminate unchanged areas.  This
-  works on binaries, and gives compression of file text as a side
-  benefit.
-
-  (Note that ``.zip`` will *not* do for this, because every file is
-  compressed independently.)
-  
-  The OpenCMS paper notes that RCS storage is only 20% more efficient
-  than gzip'd storage of individual file versions.
-
-* Store history of each file in SCCS; allows quick retrieval of any
-  previous state and may give more efficient storage than RCS.  Allows
-  for divergent branches within a single file.
-
-* Store xdeltas between consecutive file states.
-
-* Store xdeltas according to a spanning delta algorithm; this probably
-  requires files are stored with some kind of sequence number so that
-  we can predict related version names.
-
-* Store in something like XDFS.
-
-* Any of the above, but with the final storage in some kind of
-  database: psql, sqlite, mysql, whatever is convenient.   
-
-  It should be something that is safe across system crashes, which
-  rules out tdb at the moment.
-
-----
-
-These properties are seen as desirable in darcs and arch:
-
-* Passive HTTP downloads: without requiring any server-side
-  intelligence, a client can get updates from one version to the next
-  by requesting a set of self-contained files.  
-
-  The number of files necessary to do this must not be unfeasibly
-  large, and the size of each of those files should be proportionate
-  to the amount of actual change.  In other words the data downloaded
-  should be of comparable size to the actual edits between the trees.
-
-
-* Write-once storage: once a file is written to the repository, it is
-  not modified.

=== removed file 'doc/config-specs.txt'
--- doc/config-specs.txt	2005-03-09 04:51:05 +0000
+++ doc/config-specs.txt	1970-01-01 00:00:00 +0000
@@ -1,47 +0,0 @@
-************
-Config specs
-************
-
-A *config spec* is a set of instructions for assembling a source tree
-from separately maintained branches.  This is like CVS's "modules"
-format.  Config specs are very commonly used in Clearcase, and the
-main way of getting a new checkout.
-
-Scenarios:
-
- * One module is shared by several projects, and it needs to be
-   present in a subdirectory of their trees to build.
-
- * There are different commit or stability rules for different parts
-   of a tree.
-
-The simplest form is just a specification of branch sources, and where
-they should be assembled into the tree.  Example::
-
-  .              http://foo.net/fooapp
-  ./libbaz       http://foo.net/libbaz
-
-Normally the entire branch will be placed at this point, but sometimes
-subdirectory should be selected.  Selecting just a single file rather
-than subdirectory might be problematic because we wouldn't have
-anywhere to put the control files.
-
-Each branch will have a .bzr directory at the top level.  Changes on
-each branch can be pushed back independently.
-
-The tool needs to support (and be tested with) nested checkouts.
-Commands should normally only apply to a single tree, but there might
-be an option to descend into nested trees.
-
-One very important use case is a naive user checking out a tree that
-happens to use a configspec.  This is transparent in cvs, but not at
-all so in Arch.
-
-The ``svn:externals`` mechanism in Subversion works like this.
-
-One interesting idea from Aaron is that all branches should be like
-this: the latest-patch pointer can be generalized to in fact name
-several patches that should be assembled in subdirectories.  A
-simplest branch names just one, assembled to the base directory.  You
-might want to specify the parent branches for each one as well....
-

=== removed file 'doc/conflicts.txt'
--- doc/conflicts.txt	2005-03-09 04:51:05 +0000
+++ doc/conflicts.txt	1970-01-01 00:00:00 +0000
@@ -1,45 +0,0 @@
-Types of conflict
-*****************
-
-The following types of conflict can be encountered during a merge, or
-a merge-like operation (such as resyncing local changes with
-upstream):
-
-Aaron's list:
-
-* Parent directory missing when attempting to add a file
-
-* Attempt to create directory that already exists
-
-* Some patch hunks failed to apply
-
-* While changing permissions, the "old" permissions in the changeset do
-  not match the file.
-
-* While replacing contents of a file, the old contents of a file do not
-  match the changeset
-
-* Attempt to remove non-empty directory: This only occurs when the
-  directory has contents that it didn't have the last time it was deleted.
-  It's pretty easy to handle adds, renames and deletes for exact
-  patching.  There's no need to rm -rf directories as tla does.
-
-* Attempt to create a symlink when there is already a file/dir/etc with
-  that name
-
-* Attempt to apply a patch to a file that does not exist
-
-* Attempt to remove a file that does not exist
-
-* Attempt to rename a file that does not exist
-
-* Attempt to change permissions of a file that does not exist
-
-* Can't determine filename during three-way merging
-
-* Can't determine file parent directory during three-way merging
-
-* Can't determine file permissions during three-way merging
-
-Aaaron says: I have no doubt there are more.  For example, attempt to
-change permissions of a symlink.

=== removed file 'doc/costs.txt'
--- doc/costs.txt	2005-03-09 04:51:05 +0000
+++ doc/costs.txt	1970-01-01 00:00:00 +0000
@@ -1,21 +0,0 @@
-Costs
-=====
-
-User thought is most expensive.  User time spent waiting is next.
-Developer time for the version control system is also relatively
-expensive.  
-
-Machine resources are cheap.  In particular; disk is generally very
-cheap; an ordinary programmer earns enough to buy hundreds of GB of
-disk per day.  However, it is sometimes limited, as on a laptop.
-Therefore it is generally OK to trade off disk space for anything
-else, but it should be possible to be compact.
-
-Network round trips are very bad.
-
-Consider scalability also; avoid anything worse than O(n).  Try to
-avoid anything that needs e.g. to hold the whole tree in memory at any
-time, or to hold the entire history of the project.
-
-
-

=== removed file 'doc/darcs.txt'
--- doc/darcs.txt	2005-03-23 23:52:10 +0000
+++ doc/darcs.txt	1970-01-01 00:00:00 +0000
@@ -1,81 +0,0 @@
-Darcs compared to Arch
-======================
-
-Simpler to use; perhaps harder to completely understand.
-
-Always local; always fast.
-
-Patch commution is slow and perhaps doesn't clearly do what people
-want.  
-
-Too slow!
-
-Can't reliably get back to any previous point.  Explicitly not
-addressing source archive/librarian function.
-
-Loads everything into memory.
-
-Written in Haskell.
-
-A really simple pre-commit check hook is remarkably useful.
-
-http://www.scannedinavian.org/DarcsWiki/DifferencesFromArch
-
-
-
-Sometimes useful to be able to set email per-branch, for people who
-work on different projects under different personas.
-
-
-Token replace
--------------
-
-Very cute; possibly handy; not absolutely necessary in most places.
-Somewhat limited by the requirement that it be reversible.
-
-This is one of very few cases where it does seem necessary that we
-store deltas, rather than tree states.  But that seems to cause other
-problems in terms of being able to reliably sign revisions.
-
-This can perhaps be inferred by a smart 3-way merge tool.  Certainly
-you could have it do sub-line merges.
-
-Partial commit
---------------
-
-darcs allows you to commit only some of the changes to a single file.
-This is like the common feature of commiting only a subset of changed
-files, but taken to a higher level.
-
-It is useful more often than one might think: it is common to fix some
-documentation 'on the wing' and while strictly it should be in a
-separate commit it is not always worth the hassle to back out changes,
-fix the docs, then do the real change.  Similarly for making a
-separate branch.
-
-Although the idea is very good, the current darcs implementation is
-limited to selecting by patch hunk, which means that neighbouring
-changes cannot be separated.  Fixing this probably means having some
-kind of pluggable GUI to build the file-to-be-committed or an edited
-patch, possibly using something like meld, emacs, or dirdiff.  Another
-approach some people might like is editing the diff file to chop out
-hunks.
-
-I don't think this needs to be on by default, as it is in darcs.  It
-is usual to commit all the changes.
-
-For this to work safely, it is good to have a commit hook that
-builds/tests the tree.  Of course this needs to be evaluated against
-the tree as it will be committed (taking account of partial commits),
-not the working tree.
-
-
-Schwern
--------
-
-* http://www.scannedinavian.org/DarcsWiki/SchwernLikesDarcs
-* http://www.scannedinavian.org/DarcsWiki/SchwernHatesDarcs
-
-I think we hit most of these; more evidence for the hypothesis that
-what people really like about darcs is the simple interface not the
-patch-commutation model.
\ No newline at end of file

=== removed file 'doc/deadly-sins.txt'
--- doc/deadly-sins.txt	2005-03-09 04:51:05 +0000
+++ doc/deadly-sins.txt	1970-01-01 00:00:00 +0000
@@ -1,50 +0,0 @@
-**************************
-Deadly sins in tool design
-**************************
-
-http://gcc.gnu.org/wiki/DeadlySins
-
-They don't directly apply, but many do correspond.
-
-  The "Deadly Sins" from P. J. Brown's *Writing Interactive Compilers and Interpreters*, Wiley 1979. We've committed them all at least once in GCC.
-
-  The deadly sins are:
-
-  1. to code before you think.
-
-  2. to assume the user has all the knowledge the compiler writer has.
-
-  3. to not write proper documentation.
-
-  4. to ignore language standards.
-
-  5. to treat error diagnosis as an afterthought.
-
-  6. to equate the unlikely with the impossible.
-
-  7. to make the encoding of the compiler dependent on its data formats.
-
-  8. to use numbers for objects that are not numbers.
-
-  9. to pretend you are catering to everyone at the same time.
-
-  10. to have no strategy for processing break-ins.
-
-      (A break-in is when you interrupt an interactive compiler, and
-      then possibly continue it later. This is meaningful in an
-      environment in which the compiler is run dynamically, such as
-      many LISP and some BASIC environments. It is not meaningful for
-      typical uses of C/C++ (although there was at least one
-      interactive C environment, from Sabre).)
-
-      (Perhaps this corresponds to handling user interrupts during
-      operation -- they should not leave anything in an inconsistent
-      state.)
-
-  11. to rate the beauty of mathematics above the usability of your
-      compiler.
-
-  12. to let any error go undetected.
-
-  13. to leave users to find the errors in your compiler.
-

=== removed file 'doc/default.css'
--- doc/default.css	2005-03-09 06:49:00 +0000
+++ doc/default.css	1970-01-01 00:00:00 +0000
@@ -1,227 +0,0 @@
-/*
-:Author: David Goodger
-:Contact: goodger@users.sourceforge.net
-:date: $Date: 2004/01/28 20:46:38 $
-:version: $Revision: 1.36 $
-:copyright: This stylesheet has been placed in the public domain.
-
-Default cascading style sheet for the HTML output of Docutils.
-*/
-
-.first {
-  margin-top: 0 }
-
-.last {
-  margin-bottom: 0 }
-
-a.toc-backref {
-  text-decoration: none ;
-  color: black }
-
-blockquote.epigraph {
-  margin: 2em 5em ; }
-
-dd {
-  margin-bottom: 0.5em }
-
-div.abstract {
-  margin: 2em 5em }
-
-div.abstract p.topic-title {
-  font-weight: bold ;
-  text-align: center }
-
-div.attention, div.caution, div.danger, div.error, div.hint,
-div.important, div.note, div.tip, div.warning, div.admonition {
-  margin: 2em ;
-  border: medium outset ;
-  padding: 1em }
-
-div.attention p.admonition-title, div.caution p.admonition-title,
-div.danger p.admonition-title, div.error p.admonition-title,
-div.warning p.admonition-title {
-  color: red ;
-  font-weight: bold ;
-  font-family: sans-serif }
-
-div.hint p.admonition-title, div.important p.admonition-title,
-div.note p.admonition-title, div.tip p.admonition-title,
-div.admonition p.admonition-title {
-  font-weight: bold ;
-  font-family: sans-serif }
-
-div.dedication {
-  margin: 2em 5em ;
-  text-align: center ;
-  font-style: italic }
-
-div.dedication p.topic-title {
-  font-weight: bold ;
-  font-style: normal }
-
-div.figure {
-  margin-left: 2em }
-
-div.footer, div.header {
-  font-size: smaller }
-
-div.sidebar {
-  margin-left: 1em ;
-  border: medium outset ;
-  padding: 0em 1em ;
-  background-color: #ffffee ;
-  width: 40% ;
-  float: right ;
-  clear: right }
-
-div.sidebar p.rubric {
-  font-family: sans-serif ;
-  font-size: medium }
-
-div.system-messages {
-  margin: 5em }
-
-div.system-messages h1 {
-  color: red }
-
-div.system-message {
-  border: medium outset ;
-  padding: 1em }
-
-div.system-message p.system-message-title {
-  color: red ;
-  font-weight: bold }
-
-div.topic {
-  margin: 2em }
-
-h1.title {
-  text-align: center }
-
-h2.subtitle {
-  text-align: center }
-
-hr {
-  width: 75% }
-
-ol.simple, ul.simple {
-  margin-bottom: 1em }
-
-ol.arabic {
-  list-style: decimal }
-
-ol.loweralpha {
-  list-style: lower-alpha }
-
-ol.upperalpha {
-  list-style: upper-alpha }
-
-ol.lowerroman {
-  list-style: lower-roman }
-
-ol.upperroman {
-  list-style: upper-roman }
-
-p.attribution {
-  text-align: right ;
-  margin-left: 50% }
-
-p.caption {
-  font-style: italic }
-
-p.credits {
-  font-style: italic ;
-  font-size: smaller }
-
-p.label {
-  white-space: nowrap }
-
-p.rubric {
-  font-weight: bold ;
-  font-size: larger ;
-  color: maroon ;
-  text-align: center }
-
-p.sidebar-title {
-  font-family: sans-serif ;
-  font-weight: bold ;
-  font-size: larger }
-
-p.sidebar-subtitle {
-  font-family: sans-serif ;
-  font-weight: bold }
-
-p.topic-title {
-  font-weight: bold }
-
-pre.address {
-  margin-bottom: 0 ;
-  margin-top: 0 ;
-  font-family: serif ;
-  font-size: 100% }
-
-pre.line-block {
-  font-family: serif ;
-  font-size: 100% }
-
-pre.literal-block, pre.doctest-block {
-  margin-left: 2em ;
-  margin-right: 2em ;
-  background-color: #eeeeee }
-
-span.classifier {
-  font-family: sans-serif ;
-  font-style: oblique }
-
-span.classifier-delimiter {
-  font-family: sans-serif ;
-  font-weight: bold }
-
-span.interpreted {
-  font-family: sans-serif }
-
-span.option {
-  white-space: nowrap }
-
-span.option-argument {
-  font-style: italic }
-
-span.pre {
-  white-space: pre }
-
-span.problematic {
-  color: red }
-
-table {
-  margin-top: 0.5em ;
-  margin-bottom: 0.5em }
-
-table.citation {
-  border-left: solid thin gray ;
-  padding-left: 0.5ex }
-
-table.docinfo {
-  margin: 2em 4em }
-
-table.footnote {
-  border-left: solid thin black ;
-  padding-left: 0.5ex }
-
-td, th {
-  padding-left: 0.5em ;
-  padding-right: 0.5em ;
-  vertical-align: top }
-
-th.docinfo-name, th.field-name {
-  font-weight: bold ;
-  text-align: left ;
-  white-space: nowrap }
-
-h1 tt, h2 tt, h3 tt, h4 tt, h5 tt, h6 tt {
-  font-size: 100% }
-
-tt {
-  background-color: #eeeeee }
-
-ul.auto-toc {
-  list-style-type: none }

=== removed file 'doc/design.txt'
--- doc/design.txt	2005-03-09 04:51:05 +0000
+++ doc/design.txt	1970-01-01 00:00:00 +0000
@@ -1,1172 +0,0 @@
-****************
-Bazaar-NG design
-****************
-
-
-:Author:
-  Martin Pool <mbp@sourcefrog.net>
-
-:Date: December 2004, Noosa.
-
-.. sectnum::
-.. contents::
-
-
-Abstract
---------
-
-  *Bazaar-NG should be a joy to use.*
-
-  What if we started from scratch and tried to take the best features
-  from darcs, svn, arch, quilt, and bk?
-
-  Don't get the sum of all features; rather get the minimum features
-  that make it a joy to use.  Choose simplicity, in both interface and model.
-  Do not multiply entities
-  beyond necessity.
-
-  *Make it work; make it correct; make it fast* -- Ritchie(?)
-
-
-
-Design model
-------------
-
-* Unify archives and branches; one archive holds one branch.  If you
-  want to publish multiple branches, just put up multiple directories.
-
-* Explicitly add/remove files only; no names or tagline tagging.  If
-  someone wants to do heuristic detection of renames that's fine, but
-  it's not in the core model.
-
-Quilt indicates an interesting approach: patches themselves are the
-thing we're trying to build.  We don't just want a record of what
-happened, but we want to build up a good description of the change
-that will be implied when it's integrated.  This implies that we want
-to be able to change history quite a lot before merging upstream; or
-at least change our description of what will go up.
-
-
-Principles
-----------
-
-* Unix design philosophy (via Peter Miller), tempered by modern
-  expectations:
-
-  - least unnecessary output
-
-  - little dependence on *specific* external tools
-
-  - short command lines
-
-  - least overlap with cooperating tools
-
-* `Worse is better`__
-
-__ http://www.jwz.org/doc/worse-is-better.html
-
- - *Simplicity: the design must be simple, both in implementation and
-   interface. It is more important for the implementation to be
-   simple than the interface. Simplicity is the most important
-   consideration in a design.*
-
- - *Correctness: the design must be correct in all observable
-   aspects. It is slightly better to be simple than correct.*
-
-
- - *Consistency: the design must not be overly inconsistent. Consistency
-   can be sacrificed for simplicity in some cases, but it is better to
-   drop those parts of the design that deal with less common
-   circumstances than to introduce either implementational complexity
-   or inconsistency.*
-
- - *Completeness: the design must cover as many important situations as
-   is practical. All reasonably expected cases should be
-   covered. Completeness can be sacrificed in favor of any other
-   quality. In fact, completeness must sacrificed whenever implementation
-   simplicity is jeopardized. Consistency can be sacrificed to achieve
-   completeness if simplicity is retained; especially worthless is
-   consistency of interface.*
-
-* Try to get a reasonably tasteful balance between having something
-  that works out of the box but also has composable parts.  Provide
-  mechanism rather than policy but not to excess.
-
-* Files have ids to let us detect renames without having to walk the
-  whole path.  
-
-  If there are conflicts in ids they can in principle be resolved.
-  There might be a ``merge --by-name`` to allow you to force two trees
-  into agreement on IDs.  If the merge sees two files with the same
-  name and text then it should conclude that the files merged.
-
-  It would be nice if there were some way to make repeated imports of
-  the same tree give the same ids, but I don't think there is a safe
-  feasible way.  Sometimes files start out the same but really should
-  diverge; boilerplate files are one example. 
-
-* Archives are just directories; if you can read/write the files in
-  them you can do what you need.  This works even over http/sftp/etc.
-  Or at least this should work for read-only access; perhaps for
-  writing it is reasonable to require a svn+ssh style server invoked
-  over a socket.  
-
-  Of course people should not edit the files in there by hand but in
-  an emergency it should be possible.
-
-* Storing archives in plain directories means making some special
-  effort to make sure they can be rolled back if the commit is
-  interrupted any time.  On truly malicious filesystems (NFS) this may
-  be quite difficult, but at a minimum it should be possible to roll
-  back whatever was uncommitted and get to a reasonable state.  It
-  should also be reasonably possible to mirror branches using rsync,
-  which may transfer files in arbitrary order and cannot handle files
-  changing while in flight.  
-
-  Recovering from an interrupted commit may require a special ``bzr
-  fix`` command, which should write the results to a new branch to
-  avoid losing anything.
-
-* Branches carry enough information to recreate any previous state of
-  the branch (including its ancestors).  
-
-  This does not necessarily mean holding the complete text of all
-  those patches, but we do store at least a globally unique identifier
-  so that we can retrieve them.
-
-* Commands should correspond to svn or cvs as much as possible: add,
-  get, copy, commit, diff, status, log, merge. 
-
-* We have all the power of mirroring, but without needing to introduce
-  special concepts or commands.  If you want somebody's branch
-  available offline just copy it and keep updating to pull in their
-  changes; if you never make any changes the updates will always
-  succeed.
-
-* It is useful to be able to easily undo a previous change by
-  committing the opposite.  I had previously imagined requiring all
-  patches to be stored in a reversible form but it's enough to just do
-  backwards three-way merges.
-
-* Patches have globally unique IDs which uniquely identify them.
-
-* As a general principle we separate identification (which must be
-  globally unique) from naming (which must be meaningful to users).
-  Arch fuses them, which makes the human names long and prevents them
-  ever being reused.  Monotone doesn't have human-friendly names.
-
-* Users are identified by something like an email address;
-  ``user@domain``.  This need not actually be a working email
-  address; the point is just to piggyback on domain names to get
-  human-readable globally unique names.
-
-* Everything will be designed from the beginning to be safe and
-  reasonable on Windows and Unix.
-
-* History is append-only.  Patches are recorded along with the time at
-  which they were committed; if time steps backwards then we give a
-  warning (but probably commit anyhow.)  This means we can reliably
-  reproduce the state of the branch at any previous point, just by
-  backing out patches until we get back there.
-  
-  This is also true at a physical level as much as possible; once a
-  patch is committed we do not overwrite it.  This should make it less
-  likely that a failure will corrupt past history.  However, we may
-  need some indexes which are updated rather than replaced; they
-  should probably be atomically updated.
-
-* Storage should be reasonably transparent, as much as possible.  (ie
-  don't use SQLite or BDB.)  At the same time it should be reasonably
-  efficient on a wide range of systems (ie don't require reiserfs to
-  work well.)
-  
-  Programmers who look behind the covers should feel comfortable that
-  their data is safe, and hopefully pleased that the design is
-  elegant.
-
-* Unrecognized files cause a warning when you try to commit, but you
-  can still commit.  (Same behavior as CVS/Subversion; less discipline
-  than Arch.)
-
-  If you wish, you can change this to fail rather than just warn; this
-  can be done as tree policy or as an option (eg ``commit --strict``)
-
-* Files may be ignored by a glob; this can be applied globally (across
-  the whole tree) or for a particular directory.  As a special
-  convenience there is ``bzr ignore``. 
-
-* If branches move location (e.g. to a new host or a different
-  directory), then everyone who uses them needs to know the new URL by
-  some out-of-band method.
-
-* All operations on a branch or pair of branches can be done entirely
-  with the information stored in those branches.  Bazaar-NG never needs to
-  go and look at another branch, so we don't need unique branch names
-  or to remember the location of branches.
-
-* Store SHA-1 hashes of all patches, also store hashes of the tree
-  state in each revision.  (We need some defined way to make a hash of
-  a tree of files; for a start we can just cat them together in order
-  by filename.)
-  
-  Hashes are stored in such a way that we can switch hash algorithms
-  later if needed if SHA-1 is insecure.
-
-* You can also sign the hashes of patches or trees.
-
-* All branches carry all the patches leading up to their current
-  state, so you can recreate any previous state of that branch,
-  including the branches leading up to it.
-
-* A branch has an append-only history of patches committed on this
-  branch, and also an append-only history of patches that have been
-  merged in.
-
-* A commit log message file is present in .bzr-log all the time; you
-  can add notes to it as you go along.  Some commands automatically
-  add information to this file, such as when merging or reversing
-  changes. 
-  
-  The first line of the message is used as the summary.
-
-* Commands that make changes to the working copy will by default baulk
-  if you have any uncommitted changes.  Such commands include ``merge``
-  and ``reverse``.  This is done for two reasons: to avoid losing your
-  changes in the case where the merge causes problems, and to try to
-  keep merges relatively pure.  You can force it if you wish.
-
-  (*pull* is possibly a special case; perhaps it should set aside
-  local changes, update, and then reapply them/remerge them?)
-
-* Within a branch, you can refer to commits by their sequence number;
-  it's nice and friendly for the common case of looking at your
-  commits in order.
-
-* You can generate a changelog any time by looking at only local
-  files.  Automatically including a changelog in every commit is
-  redundant and so can be eliminated.  Of course if you want to
-  manually maintain a changelog you can do that too.
-
-* At the very least we should have ``undo`` as a reversible
-  ``revert``.  It might be even better to have a totally general undo
-  which will undo any operation; this is possible by keeping a journal
-  of all changes.
-
-* Perhaps eventually move to storing changesets in single text files,
-  containing file diffs and also information on renames, etc.  The
-  format should be similar to that of ``tla show-changeset``, but
-  lossless. 
-
-* Pristines are kept in the control directory; pristines are
-  relatively expensive to recreate so we might want to keep more than
-  one.  
-
-  (Robert says that keeping pristines under there can cause trouble
-  with people running recursive commands across the source tree, so
-  there should probably be some other way to do it.  If pristines are
-  identified by their hash then we can have a revlib without needing
-  unique branch names.)
-  
-* Can probably still have cacherevs for revisions; ideally
-  autogenerated in some sensible way.  We know the tree checksum for
-  each revision and can make sure we cached the right thing.
-
-* Bazaar-NG should ideally combine the best merging features of
-  Bitkeeper and Arch: both cherry-picking and arbitrary merging within
-  a graph.  The metaphor of a bazaar or souk is appropriate: many
-  independent agents, exchanging selected patches at will.
-
-* Code should be structured as a library plus a command-line client;
-  the library could be called from any other client.  Therefore
-  communication with the user should go through a layer, the library
-  should not arbitrarily exit() or abort(), etc.
-
-* Any of these details are open to change.  If you disagree, write and
-  say so, sooner rather than later.  There will be a day in the future
-  where we commit to compatibility, but that is a while off.
-
-* Timestamps obviously need to be in UTC to be meaningful on the
-  network.  I guess they should be displayed in localtime by default
-  and you can change that by setting $TZ or perhaps some option like
-  ``--utc``.  It might be cool to also capture the local time as an
-  indicator of what the committer was doing.
-
-* Should probably have some kind of progress indicator like --showdots
-  that is easy to ignore when run from a program (especially an
-  editor); that probably means avoiding tricks with carriage return.
-  (That might be a problem on Windows too.)
-
-* What date should be present on restored files?  We don't remember
-  the date of individual files, but we could set the date for the
-  entire commit.
-
-* One important layer is concerned with reproducing a previous
-  revision from a given branch; either the whole thing or just a
-  particular file or subdirectory.  This is used in many different
-  places.  We can potentially plug in different storage mechanisms
-  that can do this; either a very transparent and simple file-based
-  store as in darcs and arch, or perhaps a more tricky/fast
-  database-based system.         
-
-
-Entities and terminology
-------------------------
-
-The name of the project is *Bazaar-NG*; the top-level command is
-``bzr``.
-
-Branch
-''''''
-
-Development in Bazaar-NG takes places on branches.  A branch records
-the progress of a *tree* through various *revisions* by the
-accumulation of a series of *patches*.
-
-We can point to a branch by specifying its *location*.  At first this
-will be just a local directory name but it might grow to allow remote
-URLs with various schemes.
-
-Branches have a *name* which is for human convenience only; changesets
-are permanently labelled with the name of the branch on which they
-originated.  Branch names complement change descriptions by providing
-a broader context for the purpose of the change.  Typically the branch
-name will be the same as the last component of the directory or path.
-
-There is no higher-level grouping than branches.  (Nothing that
-corresponds to repositories in CVS or Subversion, or
-archives/categories/versions in Arch.)  Of course it may be a good
-practice to keep your branches organized into directories for each
-project, just as you might do with tarballs or cvs working
-directories.  
-
-Bazaar-NG makes forking branches very easy and common.  
-
-
-
-Revision
-''''''''
-
-The tree in a branch at a particular moment, after applying all the
-patches up to that point.
-
-
-File id
-'''''''
-
-A UUID for a versioned file, assigned by ``bzr add``.
-
-
-Delta
-'''''
-
-A smart diff, containing:
-
-* unidiff hunks for textual changes
-
-* for each affected file, the file id and the name of that file before
-  and after the delta (they will be the same if the file was not renamed)
-
-* in future, possibly other information describing symlinks,
-  permissions, etc
-
-A delta can be generated by comparing two trees without needing any
-additional input. 
-
-Although deltas have some diff context that would allow fuzzy
-application they are (almost?) always exactly applied to the correct
-predecessor. 
-
-
-Changeset
-'''''''''
-
-(also known as a patch)
-
-A changeset represents a commit to a particular branch; it
-incorporates a *delta* plus some header information such as the name
-of the committer, the date of the commit, and the commit message.
-
-
-Tree
-''''
-
-A tree of files and directories.  A branch minus the Bazaar-NG control files.
-
-
-
-
-Syntax
-------
-
-Branches
-''''''''
-
-Branches are identified by their directory name or URL::
-
-  bzr branch http://kernel.org/bzr/linux/linux-2.6
-  bzr branch ./linux-2.6 ./linux-2.6-mbp-partitions
-
-Branches have human-specified names used for tracing patches to their
-origin.  By default this is the last component of the directory name. 
-
-
-Revisions
-'''''''''
-
-Revisions within a branch may be identified by their sequence number
-on that branch, or by a tag name::
-
-  bzr branch ./linux-2.6@43 ./linux-2.6-old
-  bzr branch ./linux-2.6@rel6.8.1 ./linux-2.6.8.1
-
-You may also use the UUID of the patch or by the hash of that
-revision, though sane humans should never (need to) use these::
-
-  bzr log ./linux-2.6@uuid:6eaa1c41-34b8-4e0e-8819-acb5dfcabb78
-  bzr log ./linux-2.6@hash:4bf00930372cce9716411b266d2e03494f7fe7aa
-
-Revision ranges are given as two revisions separated by a colon (same
-as Svn):
-
-  bzr merge ../distcc-doc@4:10 
-
-
-Authors
-'''''''
-
-Authors are identified by their email address, taken from ``$EMAIL``
-or ``$BZR_EMAIL``.
-
-
-
-
-Tree inventory
---------------
-
-When a revision is committed, Bazaar-NG records an "inventory"  which
-essentially says which version of each file should be assembled into
-which location in the tree.  It also includes the SHA-1 hash and the
-size of each file.
-
-
-
-Merging
--------
-
-Merges are carried out in Bazaar-NG by a three-way merge of trees.  Users
-can choose to merge all changes from another branch, or a particular
-subset of changes.  In either case Bazaar-NG chooses an appropriate
-common base appropriately, although there should perhaps also be an
-option to specify a different base.
-
-I have not solved all the merge problems here.  I do think that this
-design preserves as much information as possible about the history of
-the code and so gives a good foundation for smart merging.
-The basic merge operation is a 3-way diff: we have three files *BASE*,
-*OTHER* and *MINE* and want to produce a result.  There are many
-different tools that could be used to resolve this interactively or
-automatically. 
-
-There are some cases where the best base is not a state that ever
-occurred on the two branches.  One such case is when there are two
-branches that have both tracked an upstream branch but have never
-previously synced with each other.  In this case we suggest that
-people manually specify the base::
-
-  bzr merge --base linus-2.6 my-2.6 
-
-Merges most commonly happen on files, but can also occur on metadata.
-For example we may need to resolve a conflict between file ids to
-decide what name a file should have, or conversely which id it should
-have.
-
-When merging an entire branch, the base is chosen as the last revision
-in which the trees manifests were identical.
-
-If merging only selected revisions from a branch (ie cherry picking)
-then the base is set just before the revisions to be merged.
-
-A three-way merge operates on three inputs: THIS, OTHER, and a BASE.
-Any regions which have been changed in only one of THIS and OTHER, or
-changed the same way in both will be carried across automatically.
-Regions which differ in all three trees are conflicts and must be
-manually resolved.
-
-The merge does not depend upon any states the trees may have
-passed through in between the revisions that are merged. 
-
-After the merge, the destination tree incorporates all the patches
-from the branch region that was merged in.  
-
-
-
-Sending patches by email
-------------------------
-
-Patches can be sent to someone else by email, just by posting the
-string representation of the changeset.  Could also post the GPG
-signature.
-
-The changeset cannot itself contain its uniquely-identifying hash.
-Therefore I suppose it needs some kind of super-header which says what
-the patch id is; this can be verified by comparing it to the hash of
-the actual changeset.  This in turn applies that the text must be
-exactly preserved in email, so possibly we need some kind of
-quoted-printable or ascii-armoured form.
-
-Another approach would be to not use the hash as the id, but rather
-something else which allows us to check the patch is actually what it
-claims to be.  For example giving a GPG key id and a UUID would do
-just as well, and *would* allow the id to be included within the
-patch, as would giving an arch-style revision ID, assuming we can
-either map the userid to a GPG key and/or check against a trusted
-archive. 
-
-There are two ways to apply such a received patch.  Ideally it tells
-us a revision of our branch from which it was based, probably by
-specifying the content hash.  We can use that as the base, make a
-branch there, apply the patch perfectly, and then merge that branch
-back in through a 3-way merge.  This gives a clean reconciliation of
-changes in the patch against any local changes in the branch since the
-base.
-
-If we do not have the base for the patch we can try apply it using
-a similar mechanism to regular patch, which might cause conflicts.  Or
-maybe it is not worth special-casing this; we could just require
-people to have the right basis to accept a patch.
-
-
-
-Rewriting history
------------------
-
-History is generally append-only; once something is committed it
-cannot be undone.  We need this to make several important guarantees
-about being able to reconstruct previous versions, about patches being
-consistent, and so on and on.
-
-However, pragmatically, there are a few cases where people will insist
-on being able to fudge it.  We need to accommodate those as best we
-can, within the limits of causality.  In other words, what is
-physically and logically possible should not be arbitrarily forbidden
-by the software (though it might be enormously discouraged).
-
-The basic transaction is a changeset/patch/commit.  There is little
-value and hellish complexity in introducing meta-changesets or trying
-to update already-committed changes.
-
-
-Wrong commit message
-''''''''''''''''''''
-
-*Oops, I pressed Save too soon, and the commit message is wrong.*  This
-happens all the time.
-
-If no other branch has taken that change, there is no harm in fixing
-the message.  Noticing the problem right away is probably a very
-common case.
-
-Therefore, you can change the descriptive text (but not any other
-metadata) of a changeset in your tree.  This will not propagate to
-anyone else who has already accepted the change.  Nothing will break,
-but they'll still see the original (incorrect/incomplete) commit.
-
-
-Committed confidential information
-''''''''''''''''''''''''''''''''''
-
-If you just added a file you didn't mean to add then you can simply
-commit a second changeset to remove it again.  However, sometimes
-people will accidentally commit sensitive/confidential information,
-and they need to remove it from the history. 
-
-If anyone else has already taken the changeset we can't prevent them
-seeing or keeping the information.  You need to find them and ask them
-nicely to remove it as well.  Similarly, if you've mirrored your
-branch elsewhere you need to fix it up by hand.  This additional
-manual work is a feature because it gives you some protection against
-accidentally destroying the wrong thing.
-
-A similar but related case is accidentally committing an enormous
-file; you don't want it to hang around in the archive for ever.  (In
-fact, it would need to be stored twice, once for the original commit
-and again for a reversible remove changeset.)
-
-Here is our suggestion for how to fix this: make a second branch from
-just before the undesired commit, typically by specifying a timestamp.
-If there are any later commits that need to be preserved, they can be
-merged in too.  Possibly that will cause conflicts if they depended on
-the removed changeset, and those changes then need to be resolved.
-
-
-
-
-
-
-History truncation
-------------------
-
-(I don't think we should implement this soon, if at all, but people
-might want to know it's possible.)
-
-Bazaar-NG relies on each branch being able to recreate any of its
-predecessor states.  This is needed to do really intelligent merging.
-
-However, you might eventually get sick of keeping all the history
-around forever.  Therefore, we can set a history horizon, ignoring all
-patches before that point. 
-
-The patches are still recorded as being merged but we do not keep the
-text of the patches.  Perhaps we add them to a special list.  
-
-Merges with a tree that have no history in common since the horizon
-will be somewhat harder.
-
-
-A development path
-------------------
-
-**See also work-log.txt for what I'm currently doing.**
-
-* Start by still using Arch changeset format, do-changeset and delta
-  commands, possibly also for merge.
-
-* Don't do any merges automatically at first but rather just build
-  some trees and let the user run dirdiff or something.
-
-* Don't handle renames at first.
-
-* Don't worry about actually being distributed yet; just work between
-  local directories.  There are no conceptual problems with accessing
-  remote directories.
-
-
-Compared to others
-------------------
-
-* History cannot be rewritten, aside from a couple of special
-  pragmatic cases.
-
-* Allows cherry-picking, which is not possible on bk or most others.
-
-* Allows merges within an arbitrary graph (rather than a line, star or
-  tree), which can be done by bk but not by arch or others.
-
-* History-sensitive merges allow safe repeated merges and mutual
-  merges between parallel lines.
-
-* Patches are labelled with the history of branches they traversed to
-  their current location, which is previously unique to Arch.
-
-* Would aim to be almost as small and simple as Quilt.
-
-* Does not need archives to be registered.  
-
-* Like darcs and bk, remembers the last archive you pulled from and
-  uses this as the default.  Also as a bonus remembers all branches
-  you previously pulled and their name, so that it is as if they were
-  registered.
-
-* Because patches do not change when they move around (as in Darcs),
-  they can be cryptographically signed.
-
-* Recognizes that textually non-conflicting merges may not be a
-  correct merge and may not work, and so should not be auto-committed.
-  The developer must have a chance to intervene after the merge and
-  before a commit.  (I think Monotone is wrong on this.)
-
-
-
-
-Best practices
---------------
-
-We recommend that people using Bazaar-NG follow these practices and
-protocols:
-
-* Develop independent features in separate branches.  It's easier to
-  keep them separate and merge later than to mix things together and
-  then try to separate them.  Although cherry picking is possible,
-  it's generally harder than keeping the code separate in the first
-  place.
-
-* Although you can merge in a graph, it can be easier to understand
-  things if you keep them roughly sorted into a star of downstream and
-  upstream branches.
-
-* Merge off your laptop/workstation into a personal stable tree at
-  regular changes; this protects against accidentally losing your
-  development branch for any reason. 
-
-* Try to have relatively "pure" merges: a single changeset that merges
-  changes should make only those merges and any edits needed to fix
-  them up. 
-
-* You can use reStructuredText (like this document) for commit
-  messages to allow nicer formatting and automatic detection of URLs,
-  email addreses, lists, etc.  Nothing requires this.
-
-
-
-Mechanics
----------
-
-Patch format
-''''''''''''
-
-A patch (i.e. commit to a branch) exists at three levels:
-
-* the hash of the patch, which is used as its globally-unique name
-
-* the headers of the patch, including:
-
-  - the human-readable name of the branch to which the changeset was committed
-
-  - free-form comments about the changeset
-
-  - the email address and name of the user who committed the changeset
-
-  - the date when the changeset was committed to the branch
-
-  - the UUIDs of any patches merged by this change
-
-  - the hash of the before and after trees
-
-  - the IDs of any files affected by the change, and their names
-    before and after the change, and their hash before and after the
-    change
-
-* the actual text of the patch, which may include
-
-  - unidiff hunks
-
-  - xdeltas (in reversible pairs?)
-
-  - complete files for adds/deletes, or for binaries
-
-At the simplest level a branch knows just the IDs of all of the
-patches committed to it.  More usually it will also have all their
-logs or all their text.
-
-Using the IDs, it can retrieve the patches when necessary from a
-shared or external store.  By this means we can have many checkouts,
-each of which looks like it holds all of its history, without actually
-using a lot of space.  When pulling down a remote branch by default
-everything will be mirrored, but there might be an option to only get
-the inventory or only the logs. 
-
-Keeping the relatively small header separate from the text makes it
-easy to get only the header information from a remote machine.  One
-might also when offline like to see only the logs but not necessarily
-have the text.
-
-Only the basic policy (keep everything everywhere) needs to be done in
-the first release of course.
-
-The headers need to be stored in some format that allows moderately
-structured data.  Ideally it would be both human readable and
-accessible from various languages.  In the prototype I think I'll use
-Python data format, but that's probably not good in the long term.  It
-may be better to use XML (tasteless though that is) or perhaps YAML or
-RFC-2822 style.  Python data is probably not secure in the face of
-untrusted patches.
-
-The date should probably be shown in ISO form (unoptimal though that
-is in some ways.)
-
-
-
-  
-
-Unresolved questions and other ideas
-------------------------------------
-
-
-Pulling in inexact matches
-''''''''''''''''''''''''''
-
-If ``update`` pulls in patches noninteractively onto the history, then
-there are some issues with patches that do not exactly match.  Some
-consequences:
-
-* You may pull in a patch which causes your tree to semantically
-  break.  This might be avoided by having a test case which is checked
-  before committing.
-
-* The patch may fuzzily apply; this is OK. 
-
-If we pull in a patch from elsewhere then we will have a signature on
-the patch but not a signature for the whole cacherev. 
-
-
-
-
-Have pristines/working directory by default?
-''''''''''''''''''''''''''''''''''''''''''''
-
-It seems a bit redundant to have two copies of the current version of
-each file in every repository, even ones in which you'll never edit.
-Some fixes are possible:
-
-* don't create working copy files 
-
-* hard link working copies into pristine directory (can detect
-  corruption by having SHA-1 sums for all pristine files)
-
-I think it's reasonable to have 
-
-
-
-Directory name
-''''''''''''''
-
-We have a single metadata directory at the top-level of the tree: ``.bzr``.
-There is no value in having it non-hidden, because it can't be seen
-from subdirectories anyhow.  Apparently three-letter names after a dot
-are fine on Windows -- it works for ``.svn``.
-
-
-File encodings
-''''''''''''''
-
-Unicode, line endings, etc.  Ignore this for now?
-
-Case-insensitive file names?  Maybe follow Darcs in forbidding files
-that differ only in case.
-
-
-Always use 3-way merge
-''''''''''''''''''''''
-
-I think using .rej files and fuzzy patches is confusing/unhelpful.  
-
-I would like to use 3-way merges between appropriate coordinates as
-the fundamental mechanism for all 'merge'-type operations.  
-
-Is there any case where .rej files are more useful?  Why would you
-ever want that?  Some people seem to `prefer them`__ in Arch.
-
-__ http://wiki.gnuarch.org/moin.cgi/Process_20_2a_2erej_20files 
- 
-I guess when cherry-picking you might not be able to find an
-appropriate ancestor for diff3?  I think you can; anyhow wiggle can
-transform rejects into diff3-style conflicts so why not do that?
-
-Miles says there that he prefers .rej files to conflict markers
-because they give better results for complex conflicts. 
-
-Perhaps we should just always produce both and let people use whatever
-they want.
-
-Another suggestion is the *rej_* tool, which helps fix up simple
-rejects:
-
-  There are four basic rejects fixable via rej.
-
-  1) missing context at the top or bottom of the hunk
-  2) different context in the middle of the hunk
-  3) slightly different lines removed by the hunk than exist in the file
-  4) Large hunks that might apply if they were broken up into smaller ones.
-
-.. _rej: ftp://ftp.suse.com/pub/people/mason/rej/
-
-
-Mirroring
-'''''''''
-
-One reason people say they like archives is that all new work in that
-archive will be automatically mirrored off your laptop, if it's
-already set up to mirror that archive.
-
-
-
-Control files out of tree
-'''''''''''''''''''''''''
-
-Some people would like to have absolutely no control files in their
-tree.  This is conceptually easy as long as we can find both the
-control files and working directory when a command is run.
-
-As a first step, the ``.bzr`` directory can be replaced by a symlink,
-which will prevent recursive commands looking into it.  Another
-approach is to put all actual source in a subdirectory of the tree, so
-that you never need to see the directory unless you look above the
-ceiling.
-
-If this is not enough, we might ask them to have an environment
-variable point to the control files, or have a map somewhere
-associating working directories with their control files.
-Unfortunately both of those seem likely to come loose and whip around
-dangerously.
-
-
-Representation of changesets
-''''''''''''''''''''''''''''
-
-Using patches is nice for plain text files.  In general we want the
-old and new names to correspond, but these are only for decoration;
-the file id determines where the patch really goes.
-
-* Should they be reversible?
-
-* How to represent binary diffs?
-
-* How to represent adds/removes?
-
-* How to zip up multiple changes into a single bundle?
-
-Reversibility is very important.  We do not need it for regular
-merges, since we can always recover the previous state.  We do need it
-for application of isolated patches, since we may not be able to
-recover the prior state.  It might also help when building a previous
-tree state.  
-
-Of course we can have an option to show deletes or to make the diff
-reversible even if it normally is not.
-
-It is very nice that plain diffs can be concatenated into a single
-text file.  This is not easily possible with binary files, xdeltas,
-etc.  Of course it is uncommon to display binary deltas directly or
-mail them, but if mailing is really required we could use base64 or
-MIME.
-
-Perhaps it would be reasonable to just store xdeltas between versions.
-
-Perhaps each changeset body should be a tar or zip holding the
-patches, though in simpler form than Arch.
-
-(Since these are free choices, perhaps stick closely to what Arch
-does?)
-
-
-Continuations
-'''''''''''''
-
-Do we need the generalized continuations currently present in Arch, or
-will a more restricted type do?  
-
-One use case for arch continuation tags is to make a release branch
-which contains only tags from the development branch.
-
-Maybe want darcs-style tags which just label the tree at various
-points; more familiar to users perhaps?
-
-::
-
-    bzr fork http://samba.org/bzr/samba/main ./my-samba
- 
-1. creates directory my-samba
-2. copies contents of samba main branch
-3. the parent becomes samba-main
-4. parent is the default place you'll pull from & push to
-   
-Is there a difference between "contains stuff from samba-main" and "is
-branched from samba-main"?
-
-
-
-File split/merge
-''''''''''''''''
-
-Is there any sense in having a command to copy a file, or to rejoin
-several files with different IDs?
-
-Joining might be useful when the same tree is imported several times,
-or the same new-file operation is done in different trees.
-
-
-
-Time skew
-'''''''''
-
-Local clocks can be wrong when they record a commit.  This means that
-changes may be irrevocably recorded with the wrong time, and that in
-turn means that later changes may seem to come from before earlier
-changes.  We can give a warning at the later time, but short of
-refusing the commit there is not much we can do about it.
-
-
-Annotate/blame/praise
----------------------
-
-``cvs annotate`` is pretty useful for understanding the history of
-development.  At the same time it is not quite trivial to implement,
-so I plan to make sure all the necessary data is easily accessible and
-then defer actually writing it.  Possibly the most complicated part is
-something to read in a diff and find which lines came from where. 
-
-What we need is a way to easily follow back through the history of a
-file, this is easily done by walking back along the branch.  Since we
-have revision numbers within a branch we have a short label which can
-be put against each line; we can also put a key at the bottom with
-some fields from each revision showing the committer and comment.
-
-For the case of merge commits people might be interested to know which
-merged patch brought in a change.  We cannot do this completely
-accurately since we don't know what the person did during the manual
-resolution of the merge, but by looking for identical lines we can
-probably get very close.  We can at the very least tell people the
-hash of all patches that were merged in so they can go and have a look
-at them.
-
-
-
-Performance
------------
-
-I think nothing here requires loading the whole tree into memory, as
-Darcs does.  We can detect renames and then diff files one by one.
-
-Because patches cannot change or be removed once they are committed or
-merged, we do not need to diff the patch-log, which is a problem in
-Arch.
-
-We do need to hold the whole list of patches in memory at various
-points but that should be at most perhaps 100,000 commits.
-
-We do need to pull down all patches since forever but that's not too
-unreasonable.
-
-Most heavy lifting can be done by GNU diff, patch and diff3, which are
-hopefully fast.
-
-Patches should be reasonably proportionate to the actual size of
-changes, not to the total size of the tree -- we should only list the
-hash and id for files that were touched by the change.  This implies
-that generating the manifest for any given revision means walking from
-the start of history to that revision.   Of course we can cache that
-manifest without necessarily caching the whole revision.
-
-* The dominant effect on performance in many cases will be network
-  round-trips; as Tom says "every one is like punching your user in
-  the face."
-
-  The network protocol can/should try to avoid them.
-
-  However, here's an even lazier idea: by making it possible to use
-  rsync for moving trees around, we get an insanely pipelined protocol
-  *for free*.
-
-  It's not always suitable (as when committing to a central tree), but
-  it will often work.  Cool!
-
-  Safely using rsync probably requires user intervention to make sure
-  that the tree is idle at the time the command runs; otherwise the
-  ordering of files arriving makes it really hard to know that we have
-  a consistent state.  I guess we can just ignore patches that are
-  missing...
-
-
-Hashing
--------
-
-It might be nice to present hashes in BubbleBabble or some similar
-form to make it a bit easier on humans who have to see them.  This can
-of course be translated to and from binary.  On the other hand there
-is something in favour of regular strings that can be easily verified
-with other tools.
-
-We can have a Henson Mode in which it never trusts that files with the
-same hash are identical but always checks it.  Of course if SHA-1 is
-broken then GPG will probably be broken too...
-
-Comparison:
-
-binary:
-  20 bytes
-bubblebabble
-  > xizif-segim-vipyz-dyzak-gatas-sifet-dynir-gegon-borad-cetit-tixux
-  65 bytes
-base64:
-  > qvTGHdzF6KLavt4PO0gs2a6pQ00=
-  28 bytes
-hex:
-  > aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d
-  40 bytes
-
-Hex is probably the most reasonable tradeoff.
-
-
-File metadata
--------------
-
-I don't want to get into general versioning of file metadata like
-permissions, at least in the first version; it's hard to say what
-should be propagated and what should not be.  This is a source code
-control system.
-
-It may be useful to carry some very restricted bits, like *read only*
-or *executable*; I think these are harmless.
-
-The only case where people generally want to remember permissions and
-ownership is when versioning ``/etc``, which is quite a special case.
-Perhaps this should be deferred to a special script such as the
-``cvs-conf`` package.
-
-
-Faster comparisons
-------------------
-
-There are many cases where we need to compare trees; perhaps the most
-common is just diffing the tree to see what changed.  For small to
-medium trees it is OK to just diff everything in the tree, and we can
-do just this in the first version.  This runs into trouble for
-kernel-sized trees, where reading every 
-
-
-Fear of forking
----------------
-
-There is some fear that distributed version control (many branches)
-will encourage projects to fork.  I don't think this is necessarily
-true of Bazaar.
-
-A fundamental principle of Bazaar is that is not the tool's place to
-make you run a project a particular way.  The tool enables you to do
-what you want.  The documentation and community might suggest some
-practices that have been useful for other projects, but the choice is
-up to you.  There are principles for running open source projects that
-are useful regardless of tool, and Bazaar supports them.  They include
-encouraging new contributors, building community, managing a good
-release schedule and so on, but I won't enumerate them all here (and I
-don't claim to know them all.)
-
-Bazaar reduces some pressures that can lead to forking.  There need
-not be fights about who gets commit access: everyone can have a branch
-and they can contribute their changes.  Radical new development can
-occur on one branch while stabilization occurs on another and a new
-feature or port on a third.  Both creating the branches and merging
-between them should be easier in the Bazaar than with existing
-systems.  (Though of course there may be technical difficulties that
-no tool can totally remove.)
-
-Sometimes there really is a time for a fork, for various reasons:
-irreconcilable differences on technical direction or personality.  If
-that happens, Bazaar makes the break less total: the projects can
-still merge patches, share bug fixes and features, and even eventually
-reunite.
-
-
-Why a new project?
-------------------
-
-A key goal is simplicity and user-friendliness; this is easier to
-build into a new tool than to fix in an existing tool.  Nevertheless
-we want to provide a smooth upgrade path from Arch, CVS, and other
-systems.
-
-
-References
-----------
-
-* http://www.dwheeler.com/essays/scm.html
-
-  Good analysis; should try to address everything there in a way he will like.
-
-
-.. Local variables:
-.. mode: indented-text
-.. End:
-
-.. Would like to use rst-mode, but it's too slow on a document of this
-.. size.
\ No newline at end of file

=== removed file 'doc/extra-commands.txt'
--- doc/extra-commands.txt	2005-03-09 04:51:05 +0000
+++ doc/extra-commands.txt	1970-01-01 00:00:00 +0000
@@ -1,33 +0,0 @@
-************************
-Bazaar-NG Extra Commands
-************************
-
-:Copyright: Copyright 2005 Canonical Ltd.
-
-.. contents::
-
-This document lists some "extra" commands that are intended for
-developer, debugging, or advanced-hacker use.  You should not need to
-know about them for normal operations.
-
-
-
-inventory
-    List files in the working copy, in an XML format including the
-    filename, id and content hash.
-
-unknowns
-    List unknown files.
-
-uuid
-    Emit a new UUID.
-
-add-to-store
-    Add given files to the full-text store, indexed by their hash.
-
-doctest
-    Run test suite.
-
-revno
-    Print number of revisions on this branch (or equivalently, the
-    latest revision number.)
\ No newline at end of file

=== removed file 'doc/formats.txt'
--- doc/formats.txt	2005-07-04 12:26:02 +0000
+++ doc/formats.txt	1970-01-01 00:00:00 +0000
@@ -1,261 +0,0 @@
-*****************
-Bazaar-NG formats
-*****************
-
-.. contents::
-
-Since branches are working directories there is just a single
-directory format.
-
-There is one metadata directory called ``.bzr`` at the top of each
-tree.  Control files inside ``.bzr`` are never touched by patches and
-should not normally be edited by the user.
-
-These files are designed so that repository-level operations are ACID
-without depending on atomic operations spanning multiple files.  There
-are two particular cases: aborting a transaction in the middle, and
-contention from multiple processes.  We also need to be careful to
-flush files to disk at appropriate points; even this may not be
-totally safe if the filesystem does not guarantee ordering between
-multiple file changes, so we need to be sure to roll back.
-
-The design must also be such that the directory can simply be copied
-and that hardlinked directories will work.  (So we must always replace
-files, never just append.)
-
-A cache is kept under here of easily-accessible information about
-previous revisions.  This should be under a single directory so that
-it can be easily identified, excluded from backups, removed, etc.
-This might contain pristine tree from previous revisions, manifests
-and inventories, etc.  It might also contain working directories when
-building a commit, etc.  Call this maybe ``cache`` or ``tmp``.
-
-I wonder if we should use .zip files for revisions and cacherevs
-rather than tar files so that random access is easier/more efficient.
-There is a Python library ``zipfile``.
-
-
-Signing XML files
-*****************
-
-bzr relies on storing hashes or GPG signatures of various XML files.
-There can be multiple equivalent representations of the same XML tree,
-but these will have different byte-by-byte hashes.
-
-Once signed files are written out, they must be stored byte-for-byte
-and never re-encoded or renormalized, because that would break their
-hash or signature.
-
-
-
-
-Branch metadata
-***************
-
-All inside ``.bzr``
-
-``README``
-  Tells people not to touch anything here.
-
-``branch-format``
-  Identifies the parent as a Bazaar-NG branch; contains the overall
-  branch metadata format as a string.
-
-``pristine-directory``
-  Identifies that this is a pristine directory and may not be
-  committed to.
-
-``patches/``
-  Directory containing all patches applied to this branch, one per
-  file.  Patches are stored as compressed deltas.  We also store the
-  hash of the delta, hash of the before and after manifests, and
-  optionally a GPG signature.
-
-``cache/``
-  Contains various cached data that can be destroyed and will be
-  recreated.  (It should not be modified.)
-
-``cache/pristine/``
-  Contains cached full trees for selected previous revisions, used
-  when generating diffs, etc.
-
-``cache/inventory/``
-  Contains cached inventories of previous revisions.
-
-``cache/snapshot/``
-  Contains tarballs of cached revisions of the tree, named by their
-  revision id.  These can also be removed, but 
-
-``patch-history``
-  File containing the UUIDs of all patches taken in this branch,
-  in the order they were taken.
-  Each commit adds exactly one line to this file; lines are
-  never removed or reordered.
-
-``merged-patches``
-  List of foreign patches that have been merged into this branch.
-  Must have no entries in common with ``patch-history``.  Commits that
-  include merges add to this file; lines are never removed or
-  reordered.
-
-``pending-merges`` 
-  List (one per line) of non-mainline revisions that
-  have been merged and are waiting to be committed.
-
-``branch-name``
-  User-qualified name of the branch, for the purpose of describing the
-  origin of patches, e.g. ``mbp@sourcefrog.net/distcc--main``.
-
-``friends``
-  List of branches from which we have pulled; file containing a list
-  of pairs of branch-name and location.
-
-``parent``
-  Default pull/push target.
-
-``pending-inventory``
-  Mapping from UUIDs to file name in the current working directory.  
-
-``branch-lock``
-  Lock held while modifying the branch, to protect against clashing
-  updates.
-
-
-Locking
-*******
-
-Is locking a good strategy?  Perhaps somekind of read-copy-update or
-seq-lock based mechanism would work better?
-
-If we do use a locking algorithm, is it OK to rely on filesystem
-locking or do we need our own mechanism?  I think most hosts should
-have reasonable ``flock()`` or equivalent, even on NFS.  One risk is
-that on NFS it is easy to have broken locking and not know it, so it
-might be better to have something that will fail safe.
-
-Filesystem locks go away if the machine crashes or the process is
-terminated; this can be a feature in that we do not need to deal with
-stale locks but also a feature in that the lock itself does not
-indicate cleanup may be needed.
-
-robertc points out that tla converged on renaming a directory as a
-mechanism: this is one thing which is known to be atomic on almost all
-filesystems.  Apparently renaming files, creating directories, making
-symlinks etc are not good enough.
-
-
-
-Delta
-*****
-
-XML document plus a bag of patches, expressing the difference between
-two revisions.  May be a partial delta.
-
-* list of entries
-
-  * entry
-  
-    * parent directory (if any)
-    * before-name or null if new
-    * after-name or null if deleted
-    * uuid
-    * type (dir, file, symlink, ...)
-    * patch type (patch, full-text, xdelta, ...)
-    * patch filename (?)
-
-
-Inventory
-*********
-
-XML document; series of entries.  (Quite similar to the svn
-``entries`` file; perhaps should even have that name.)
-Stored identified by its hash.
-
-An inventory is stored for recorded revisions, also a
-``pending-inventory`` for a working directory.
-
-Inventories always have the same id as the revision they correspond
-to.  bzr up to 0.0.5 explicitly stores an inventory-id; in future
-versions this may be implied.
-
-
-
-Revision
-********
-
-XML document.  Stored identified by its hash.
-
-committer
-  RFC-2822-style name of the committer.  Should match the key used to
-  sign the revision.
-
-comment
-  multi-line free-form text; whitespace and line breaks preserved
-
-timestamp
-  As floating-point seconds since epoch.
-
-branch name
-  Name of the branch to which this was originally committed.    
-
-  (I'm not totally satisfied that this is the right way to do it; the
-  results will be a bit weird when a series of revisions pass through
-  variously named branches.)
-
-inventory_hash
-  Acts as a pointer to the inventory for this revision.
-
-parents
-  Zero, one, or more references to parent revisions.   For each 
-  the revision-id and the revision file's hash are given.  The first
-  parent is by convention the revision in whose working tree the
-  new revision was created.
-
-precursor
-  Must be equal to the first parent, if any are given.  For
-  compatibility with bzr 0.0.5 and earlier; eventually will be
-  removed. 
-
-merged-branches
-  Revision ids of complete branches merged into this revision.  If a
-  revision is listed, that revision and transitively its predecessor
-  and all other merged-branches are merged.  This is empty except
-  where cherry-picks have occurred.
-
-merged-patches
-  Revision ids of cherry-picked patches.  Patches whose branches are
-  merged need not be listed here.  Listing a revision ID implies that
-  only the change of that particular revision from its predecessor has
-  been merged in.   This is empty except where cherry-picks have
-  occurred.
-
-The transitive closure avoids Arch's problem of needing to list a
-large number of previous revisions.  As ddaa writes:
-
-    Continuation revisions (created by tla tag or baz branch) are associated
-    to a patchlog whose New-patches header lists the revisions associated to
-    all the patchlogs present in the tree. That was introduced as an
-    optimisation so the set of patchlogs in any revision could be determined
-    solely by examining the patchlogs of ancestor revisions in the same
-    branch. This behaves well as long as the total count of patchlog is
-    reasonably small or new branches are not very frequent.
-
-    A continuation revision on $tree currently creates a patchlog of
-    about 500K. This patchlog is present in all descendent of the revision,
-    and all revisions that merges it.
-
-It may be useful at some times to keep a cache of all the branches, or
-all the revisions, present in the history of a branch, so that we do
-need to walk the whole history of the branch to build this list.
-  
-
-----
-
-Proposed changes
-****************
-
-* Don't store parent-id in all revisions, but rather have <DIRECTORY>
-  nodes that contain entries for children?
-
-* Assign an id to the root of the tree, perhaps listed in the top of
-  the inventory?

=== removed file 'doc/hashes.txt'
--- doc/hashes.txt	2005-03-09 04:51:05 +0000
+++ doc/hashes.txt	1970-01-01 00:00:00 +0000
@@ -1,73 +0,0 @@
-Use of hashes in Bazaar-NG
-**************************
-
-* http://infohost.nmt.edu/~val/review/hash.html
-* http://infohost.nmt.edu/~val/review/hash2.html
-
-The main attraction of hashes in bazaar-ng is as an easy way to get
-universally-unique IDs, or at least with a low chance of collision:
-
-The first paper is a bit paranoid; the second has some sensible
-advice:
-
-1. Will compare-by-hash provide significant benefit -- save time,
-   bandwidth, etc?
-
-2. Is the system usable if hash collisions can be generated at will?
-
-3. Can the hashes be regenerated with a different algorithm at any
-   time?
-
-We should try to abide by these rules.  I think they are possibly too
-paranoid -- a real break of SHA-1 would have much wider security
-implications -- but if a design that respects them is practical, it
-should be preferred.
-
-The first is probably true; the third is just a matter of making sure
-we allow for the choice of hash to be varied in the format.
-
-There are actually two variations on the second:
-
-2a. Is the system safe if an attacker can generate hash collisions?
-
-2b. Is the system safe if a user's own files contain collisions.
-
-Regardless of cryptographic weakness, SHA-1 is unlikely to
-"accidentally" collide, but it's possible that someone will
-intentionally generate collisions (in research on SHA) and then want
-to store them.  It would be unfortunate if that did not work.
-
-An advantage of naming by hash is that it lets us store only a single
-copy of identical files, but we have already decided__ that disk space
-is pretty cheap.  It is perhaps enough to have a single copy of files
-that do not change from one tree revision to the next.
-
-__ costs.html
-
-As far as an attacker: we will not automatically trust that ids from
-one branch have the same value in another.  It is possible for a
-branch to contain "lies" about its history or contents, but that
-doesn't corrupt anything else.  It may confuse or mislead someone who
-looks at the branch, but there is no substitute for human review
-anyhow.
-
--------
-
-The safest position may be to never rely on identifying content by
-hash.  Rather, things which need a universally unique ID should get a
-UUID instead.
-
-This has a slight advantage that the id can be stored directly in the
-object it refers to, when that's useful.
-
-So a `Revision` holds a UUID for the `Inventory`.  
-
-An inventory holds `InventoryEntry` objects, each with
-
-* file-id
-* filename (location in tree)
-* type (file, dir, etc)
-* text-id (uuid identifying the text)
-* text-sha1
-* text-length (for catching bugs)
-* parent-file-id

=== removed file 'doc/ignore.txt'
--- doc/ignore.txt	2005-03-28 02:24:18 +0000
+++ doc/ignore.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-Ignore patterns
-***************
-
-Ignore patterns need to be flexible but simple.
-
-Based on the system from rsync, which is in turn based on that of
-CVS.  However, I won't implement all of this at first but rather add
-bits as they seem to be needed.
-
-First build a list by concatenating:
-
-* per-user configuration
-* per-tree configuration
-* command line options
-
-Just one ``.bzrignore`` at tree root?
-
-File can contain '+ ' and '- ' prefixes to include and exclude things,
-also '#' for comments.
- 
-Should have a ``--ignore`` command line option that specifies ignore
-patterns; by putting a ! in this you can turn off any other includes.
-(Similar to CVS.)

=== removed file 'doc/index.txt'
--- doc/index.txt	2005-05-17 00:33:59 +0000
+++ doc/index.txt	1970-01-01 00:00:00 +0000
@@ -1,189 +0,0 @@
-Bazaar-NG
-*********
-
-..   These documents are formatted as ReStructuredText.  You can    ..
-..   convert them to HTML, PDF, etc using the ``python-docutils``   ..
-..   package.                                                       ..
-
-
-*Bazaar-NG* (``bzr``) is a project of `Canonical Ltd`__ to develop an
-open source distributed version control system that is powerful,
-friendly, and scalable.  The project is at an early stage of
-development.
-
-__ http://canonical.com/
-
-
-**Note:** These documents are in a very preliminary state, and so may
-be internally or externally inconsistent or redundant.  Comments are
-still very welcome.  Please send them to <mbp@sourcefrog.net>.
-
-
-For more information, see the homepage at http://bazaar-ng.org/
-
-
-
-User documentation
-------------------
-
-* `Project overview/introduction <intro.html>`__
-
-* `Command reference <cmdref.html>`__ -- intended to be user
-  documentation, and gives the best overview at the moment of what the
-  system will feel like to use.  Fairly complete.
-
-
-Requirements and general design
--------------------------------
-
-* `Various purposes of a VCS <purpose.html>`__ -- taking snapshots and
-  helping with merges is not the whole story.
-
-* `Requirements <requirements.html>`__
-
-* `Costs <costs.html>`__ of various factors: time, disk, network, etc.
-
-* `Deadly sins <deadly-sins.html>`__ that gcc maintainers suggest we avoid.
-
-* `Overview of the whole design <design.html>`__ and miscellaneous
-  small design points.
-
-* `File formats <formats.html>`__
-
-* `Random observations <random.html>`__ that don't fit anywhere else yet.
-
-
-
-Design of particular features
------------------------------
-
-* `Automatic generation of ChangeLogs <changelogs.html>`__
-
-* `Cherry picking <cherry-picking.html>`__ -- merge just selected non-contiguous changes  from a branch.
-
-* `Common changeset format <common-format.html>`__ for  interchange
-  format between VCS.
-
-* `Compression <compression.html>`__ of file text for more efficient storage.
-
-* `Config specs <config-specs.html>`__ assemble a tree from several places.
-
-* `Conflicts <conflicts.html>`_ that can occur during merge-like
-  operations.
-
-* `Ignored files <ignore.html>`__
-
-* `Recovering from interrupted operations <interrupted.html>`__
-
-* `Inventory command <inventory.html>`__
-
-* `Branch joins <join-branches.html>`__ represent that all the changes
-  from one branch are integrated into another.
-
-* `Kill a version <kill-version.html>`__ to fix a broken commit or
-  wrong message, or to
-  remove confidential information from the history.
-
-* `Hash collisions <hashes.html>`__ and weaknesses, and the security
-  implications thereof.
-
-* `Layers <layers.html>`__ within the design
-
-* `Library interface <library-interface.html>`__ for Python.
-
-* `Merge <merge.html>`__
-
-* `Mirroring <mirroring.html>`__
-
-* `Optional edit command <optional-edit.html>`__: sometimes people
-  want to make the  working copy read-only, or not present at all.
-
-* `Partial commits <partial-commit.html>`__
-
-* `Patch pools <pool.html>`__ to efficiently store related branches.
-
-* `Revfiles <revfile.html>`__ store the text history of files.
-
-* `Revfiles storing annotations <revfile-annotation.html>`__
-
-* `Revision syntax <revision-syntax.html>`__ -- ``hello.c@12``, etc.
-
-* `Roll-up commits <rollup.html>`__ -- a single revision incorporates
-  the changes from several others.
-
-* `Scalability <scalability.html>`__
-
-* `Security <security.html>`__
-
-* `Shared branches <shared-branches.html>`__ maintained by more than
-  one person
-
-* `Supportability <supportability.html>`__ -- how to handle any bugs
-  or problems in the field.
-
-* `Place tags on revisions for easy reference <tagging.html>`__
-
-* `Detecting unchanged files <unchanged.html>`__
-   
-* `Merging previously-unrelated branches <unrelated-merge.html>`__
-
-* `Usability principles <usability.html>`__ (very small at the moment)
-
-* `<use-cases.html>`__
-
-* `<web-interface.html>`__
-
-* `<workflow.html>`__ Modelling/controlling flow of patches.
-
-* `<yaml.html>`__ --  Discussion of using YAML_ as a storage or transmission format.
-
-.. _YAML: http://www.yaml.org/
-
-
-
-Comparisons to other systems
-----------------------------
-
-* `Taxonomy <taxonomy.html>`__: basic questions a VCS must answer.
-
-* `Bitkeeper <bitkeeper.html>`__, the proprietary system used by some
-  kernel developers.
-
-* `Aegis <compared-aegis.html>`__, a tool focussed on enforcing
-  process and workflow.
-
-* `Codeville <compared-codeville.html>`__ has an intruiging but
-  scarcely-documented merge algorithm.
-
-* `CVSNT <compared-cvsnt.html>`__, with more Windows support and some
-  merge enhancements.
-
-* `OpenCM <compared-opencm.html>`__, another hash-based tool with a
-  good whitepaper.
-
-* `PRCS <compared-prcs.html>`__, a non-distributed inventory-based tool.
-
-* `GNU Arch <todo-from-arch.html>`__, with many pros and cons.
-
-* `Darcs <darcs.html>`__, a merge-focussed tool with good usability.
-
-* `Quilt <quilt.html>`__ -- Andrew Morton's patch scripts, popular with kernel maintainers.
-
-* `Monotone <monotone.html>`__, Graydon Hoare's hash-based distributed system.
-
-* `SVK <svk.html>`__ --     distributed operation stacked on Subversion.
-
-* `Sun Teamware <compared-teamware.html>`__
-
-
-Project management and organization
------------------------------------
-
-* `Notes on how to get a VCS adopted <adoption.html>`__
-
-* `Thanks <thanks.html>`__ to various people
-
-* `Extra commands <extra-commands.html>`__ for
-  internal/developer/debugger use.
-
-* `Choice of Python as a development language <python.html>`__

=== removed file 'doc/interrupted.txt'
--- doc/interrupted.txt	2005-04-15 01:31:21 +0000
+++ doc/interrupted.txt	1970-01-01 00:00:00 +0000
@@ -1,113 +0,0 @@
-Interrupted operations
-**********************
-
-Problem: interrupted operations
-===============================
-
-Many version control systems tend to have trouble when operations are
-interrupted.  This can happen in various ways:
-
- * user hits Ctrl-C
-
- * program hits a bug and aborts
-
- * machine crashes
-
- * network goes down
-
- * tree is naively copied (e.g. by cp/tar) while an operation is in
-   progress
-
-We can reduce the window during which operations can be interrupted:
-most importantly, by receiving everything off the network into a
-staging area, so that network interruptions won't leave a job half
-complete.  But it is not possible to totally avoid this, because the
-power can always fail.
-
-I think we can reasonably rely on flushing to stable storage at
-various points, and trust that such files will be accessible when we
-come back up.
-
-I think by using this and building from the bottom up there are never
-any broken pointers in the branch metadata: first we add the file
-versions, then the inventory, then the revision and signature, then
-link them into the revision history.  The worst that can happen is
-that there will be some orphaned files if this is interrupted at any
-point. 
-
-rsync is just impossible in the general case: it reads the files in a
-fairly unpredictable order, so what it copies may not be a tree that
-existed at any particular point in time.  If people want to make
-backups or replicate using rsync they need to treat it like any other
-database and either
-
- * make a copy which will not be updated, and rsync from that
-
- * lock the database while rsyncing
-
-The operating system facilities are not sufficient to protect against
-all of these.  We cannot satisfactorily commit a whole atomic
-transaction in one step.
-
-Operations might be updating either the metadata or the working copy.
-
-The working copy is in some ways more difficult:
-
- * Other processes are allowed to modify it from time to time in
-   arbitrary ways.
-
-   If they modify it while bazaar is working then they will lose, but
-   we should at least try to make sure there is no corruption.
-
- * We can't atomically replace the whole working copy.  We can
-   (semi) atomically updated particular files.
-
- * If the working copy files are in a weird state it is hard to know
-   whether that occurred because bzr's work was interrupted or because
-   the user changed them.
-
-   (A reasonable user might run ``bzr revert`` if they notice
-   something like this has happened, but it would be nice to avoid
-   it.)
-
-We don't want to leave things in a broken state.
-
-
-Solution: write-ahead journaling?
-=================================
-
-One possibly solution might be write-ahead journaling:
-
-  Before beginning a change, write and flush to disk a description of
-  what change will be made.
-
-  Every bzr operation checks this journal; if there are any pending
-  operations waiting then they are completed first, before proceeding
-  with whatever the user wanted.  (Perhaps this should be in a
-  separate ``bzr recover``, but I think it's better to just do it,
-  perhaps with a warning.)
-
-  The descriptions written into the journal need to be simple enough
-  that they can safely be re-run in a totally different context.  They
-  must not depend on any external resources which might have gone
-  away.
-
-  If we can do anything without depending on journalling we should.
-
-  It may be that the only case where we cannot get by with just
-  ordering is in updating the working copy; the user might get into a
-  difficult situation where they have pulled in a change and only half
-  the working copy has been updated.  One solution would be to remove
-  the working copy files, or mark them readonly, while this is in
-  progress.  We don't want people accidentally writing to a file that
-  needs to be overwritten.
-
-  Or perhaps, in this particular case, it is OK to leave them in
-  pointing to an old state, and let people revert if they're sure they
-  want the new one?  Sounds dangerous.
-
-Aaron points out that this basically sounds like changesets.  So
-before updating the history, we first calculate the changeset and
-write it out to stable storage as a single file.  We then apply the
-changeset, possibly updating several files.  Each command should check
-whether such an application was in progress.

=== removed file 'doc/intro.txt'
--- doc/intro.txt	2005-03-09 04:51:05 +0000
+++ doc/intro.txt	1970-01-01 00:00:00 +0000
@@ -1,109 +0,0 @@
-Introduction to Bazaar
-======================
-
-Bazaar's goal is *to make a distributed version control system that
-open source developers will love to use*.  Using Bazaar should feel
-good.
-
-.. epigraph::
-
-  Language designers want to design the perfect language. They want to
-  be able to say, "My language is perfect. It can do everything." But
-  it's just plain impossible to design a perfect language, because
-  there are two ways to look at a language. One way is by looking at
-  what can be done with that language. The other is by looking at how
-  we feel using that language -- how we feel while programming.
-
-  Because of the Turing completeness theory, everything one
-  Turing-complete language can do can theoretically be done by another
-  Turing-complete language, but at a different cost. You can do
-  everything in assembler, but no one wants to program in assembler
-  anymore. From the viewpoint of what you can do, therefore, languages
-  do differ -- but the differences are limited. For example, Python
-  and Ruby provide almost the same power to the programmer.
-
-  Instead of emphasizing the what, I want to emphasize the how part: how
-  we feel while programming.
-
-  -- `Yukihiro Matsumoto`__
-
-__ http://www.artima.com/intv/ruby.html
-
-Bazaar tries to make simple things simple, and complex things
-possible.  In particular:
-
- * Distributed operation is easy: you can work while disconnected; you
-   can fork any other project; you can contribute changes back
-   easily.  
-
- * The system is designed to scale to supporting very large trees with
-   a lot of history.  No operations require downloading the entire
-   history of the project.
-
- * Changes can be "cherry-picked" out of branches as needed.   Because
-   of dependencies between 
-
-
-
-History-sensitive merging
-=========================
-
-Baz keeps track of what changes have been merged into a branch.  You
-can repeatedly merge from one branch into another and Baz will pull
-across only the new changes since you last merged.
-
-
-Speed
-=====
-
-For most users, the most important factor for performance is to avoid
-unnecessary network round trips.  Baz tries hard to avoid ever
-downloading the same data twice.  
-
-Remote archives are automatically cached on your local machine by
-default.  If you have ever accessed a remote revision you should be
-able to get it again without going to the network.  The cache policy
-may be tuned.
-
-
-
-Code history
-============
-
-One important function of a revision control system is to maintain a
-record of when, why, how and by whom changes were made. 
-
-Baz requires that branches and archives be named.
-
-Unlike most other systems, Baz keeps a record for each changeset of
-which branches and archives it passed through on its way to its
-eventual destination.  This allows people to go back later and see the
-context in which the patch was written or merged.
-
-
-
-Scalability
-===========
-
-We regularly test Baz on projects with tens of thousands of commits,
-and tens of thousands of files.
-
-
-
-Merging
-=======
-
-The basic method of integration is a three-way merge.  Baz selects an
-appropriate basis version
-
-
-
-File renaming
-=============
-
-Baz allows files and directories to be renamed in a project.  Unlike
-Subversion, Baz will correctly merge changes spanning file renames.
-This is done by automatically assigning a unique ID to each file,
-which is persistent across renames.
-
-

=== removed file 'doc/inventory.txt'
--- doc/inventory.txt	2005-03-09 04:51:05 +0000
+++ doc/inventory.txt	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-Should be a simple command that shows something similar to 'svn
-status':
-
-  what is not added but perhaps should be
-  what is source, backup, precious, junk, etc
-
-Or a nice simple command like 'bk extras' to show what's not added but
-should be.
-  
-baz inventory almost does this but it way too hard to understand
\ No newline at end of file

=== removed file 'doc/join-branches.txt'
--- doc/join-branches.txt	2005-03-09 04:51:05 +0000
+++ doc/join-branches.txt	1970-01-01 00:00:00 +0000
@@ -1,81 +0,0 @@
-****************
-Joining branches
-****************
-
-(I think this is pretty brilliant. :-)
-
-Branches diverge when people create more than one changeset following
-on from a common ancestor::
-
-  A:   0 ------- 1
-  B:    \------- 2
-
-We also allow branches to reunite.  This means that all the decisions
-taken on multiple branches have been reconciled and unified into a
-single successor::
-
-  A: 0 ------- 1 ----- 3
-      \               /
-  B:   \------ 2 ----/
-
-The predecessor of 3 is 1, in the sense that it was created on that
-branch.  We could have created the exact same state as a successor to
-2, and we can move that state onto branch 2 without any loss of
-information.
-
-(One thing we can do here is just delete B.  Because all of the work
-there has been merged onto A, this will not lose anything.  We might
-do this if the purpose of B has been achieved, such as completing a
-feature or bug.  But if the work is still in progress, we might keep
-it around.  It makes little difference whether we decide to do new
-work in a branch called B or make a new one called C.)
-
-That is to say that 3 can be perfectly (trivially) merged onto B,
-with, say ``bzr push``, ``bzr pull`` or ``bzr update`` (whatever name
-works best).  Perfectly merged means that we know there will be no
-conflicts or need for manual intervention, and that we can just
-directly store it without forming a roll-up changeset.
-
-I think we might also like the choice of merging A onto B, rather than
-pulling the changeset.  That causes a new changeset to be created on
-B, noted as a successor of 2 and 3::
-
-  A: 0 ------- 1 ----- 3 ------+
-      \               /         \
-  B:   \------ 2 ----+---------- 4
-
-One complication is that 3 is probably stored in A's history as a
-patch relative to 1; we can't just move this representation across.
-Instead, we need to recalculate the delta from 2 to 3 and store that.
-
-Despite that the delta is stored differently, the original signature
-on 3 should still be valid.  So it must be a signature of the tree
-state, not the diff.
-
-Note from `Kernel Traffic discussion`__:
-
-__ http://www.kerneltraffic.org/kernel-traffic/kt20030323_210.txt
-
-    But anyway, what made Bitkeeper suck less is the real DAG
-    structure.  Neither arch (http://arch.fifthvision.net/bin/view)
-    nor subversion seem to have understood that and, as a result,
-    don't and won't provide the same level of semantics. Zero hope for
-    Linus to use them, ever. They're needed for any decently
-    distributed development process.
-
-This in turn suggests that possibly deltas should be stored separately
-from the commits that create them.  Commits name points of
-development; deltas describe how to get from one to the other.
-
-The separation is nice in allowing us to send just a delta when
-diffing trees or recording for undo.  We might want to compute many
-deltas between different trees.
-
-Is this a problem?  Does this ignore Tom's advice about the primacy of
-storing changesets?
-
-Splitting them is probably good, but then what manifest is stored in
-changesets?  We don't want to store the manifest of the whole tree if
-we can avoid it.  So I suppose the changeset just gives the hash of
-the manifest, and the manifest then can be stored separately, possibly
-delta-encoded.

=== removed file 'doc/kill-version.txt'
--- doc/kill-version.txt	2005-03-09 04:51:05 +0000
+++ doc/kill-version.txt	1970-01-01 00:00:00 +0000
@@ -1,128 +0,0 @@
-Killing versions
-****************
-
-Sometimes people really need to rewrite history.  The canonical
-example is that they have accidentally checked confidential
-information into the wrong tree.
-
-There is a tension between two imperatives:
-
-* Accurately record all history.
-
-* Remove critical information.
-
-Of course we cannot control who may have made use of the information
-in the time it was public, but often these things are caught early enough.
-
-After history has been rewritten, anything that depends on that
-history may be inconsistent.  We cannot consistently continue on a
-version that has had some revisions knocked out; therefore it seems
-simplest to kill the version over all.  
-
-You can make a tag onto a new version from a previous consistent
-revision.  
-
-Downstream users will need to switch on to the new version.
-
-We can mark the version as killed so that mirroring will delete the
-revisions and clients looking at the version can suggest switching.
-
-This should be an extremely infrequent operation so it is OK that it
-is simple as long as it fulfils the basic requirement.
-
-A related problem is that people sometimes commit the wrong log
-message, or (more seriously) the wrong files.  It is common to notice
-immediately afterwards.
-
-
-Uncommit
---------
-
-I think this is the best and most useful option at the moment: the
-`uncommit command`_ undoes the last commit, and nothing else.  It can
-be repeated to successively remove more and more history.
-
-.. _`uncommit command`: cmdref.html#uncommit
-
-Example::
-
-    % cd my-project
-    % bzr add launch-codes.txt
-    % bzr commit -m 'add launch codes'
-    A     launch-codes.txt
-    recorded r23
-    [oops!]
-    % bzr uncommit
-    [now back to just before the commit command]
-    
-    % bzr status
-    A     launch-codes.txt
-    % bzr revert launch-codes.txt
-    % bzr status
-    ?     launch-codes.txt
-
-If anyone else has depended on the option then the branches will have
-diverged and that needs to be resolved somehow, probably by the other
-people starting a new branch with the correct commit.  In particular
-uncommitting from a shared branch will break anyone who has seen that
-revision; possibly there needs to be a policy option on such branches
-to deny uncommit.
-
-If someone else on that shared branch did a the equivalent of a
-switch_ command then they would bring their changes onto the correct
-basis.  Would it be reasonable to do that automatically?
-
-.. _switch: cmdref.html#switch
-
-
-
-UI proposals
-------------
-
-Aaron's proposal::
-
-    % cd my-project
-    % baz add launch-codes.txt
-    % baz commit -s 'add launch codes'
-    recorded patch-4
-    % baz add good-code.c
-    % baz commit -s 'add good code'
-    recorded patch-5
-    [oops!]
-    % baz undo patch-3
-    % baz pull patch-5
-    % baz commit --as-latest
-    renamed old branch to my-project-old-1
-    % baz eradicate my-project-old-1
-
-Long form::
-
-  % cd my-project
-  % baz add launch-codes.txt
-  % baz commit -s 'add launch codes'
-  recorded patch-4
-  % baz add good-code.c
-  % baz commit -s 'add good code'
-  recorded patch-5
-  [oops!]
-  % cd ../
-  % baz fork my-project--patch3 my-project-fixed
-  % cd my-project-fixed
-  % baz pull ../my-project patch5
-  % cd ../
-  % rm -rf my-project
-  % mv my-project-fixed my-project
-
-Robert's proposal::
-
-  % cd my-project
-  % baz add launch-codes.txt
-  % baz commit -s 'add launch codes'
-  recorded patch-4
-  % baz add good-code.c
-  % baz commit -s 'add good code'
-  recorded patch-5
-  [oops!]
-  % baz branch patch-3 $(baz tree-version)
-  % baz pull . patch-5
-  % baz eliminate patch-4

=== removed file 'doc/layers.txt'
--- doc/layers.txt	2005-03-09 04:51:05 +0000
+++ doc/layers.txt	1970-01-01 00:00:00 +0000
@@ -1,88 +0,0 @@
- * Write-once store of files identified by globally-unique names
-   (e.g. hashes or UUIDs).
-
-   Simplest option is to just dump them in a directory.  
-
-   Optionally do delta-compression between similar/related files; or
-   build this on top of deltas.
-
- * Tree manifests/inventories which say which files should be assembled at
-   particular points to build a tree.  These too can be stored indexed
-   by hash.
-
-   - Reconstruct a revision by pulling out the manifest and then all
-     the individual files. 
-  
-   - Manipulate working copy of inventory by add/mv/remove/commands.
-
- * Retrieve deltas 
- 
-   - Calculate diffs between two file versions, just by getting them
-     from the store.
-
-   - Retrieve deltas between any two revisions: requires looking for
-     changes to the structure of the tree, and then text changes for
-     anything inside it.
-
-   - Deltas may be either stored or calculated on demand.  They can be
-     put in the store just indexed by the from and to manifest id.
-
-   - Calculate diff between a previous revision and the working copy. 
-
- * Commit and retrieve revisions
-
-   - Revisions hold metadata (date, committer, comment, optional
-     parents, merged patches), and a pointer to the inventory.
-
-   - Stored in a write-once store.
-
- * Branch holds a linear history of revisions.
-
-   - This is mostly redundant; we could just remember the current base
-     revision and walk backwards from there.  But it still seems
-     possibly useful to hold; we can check that the two are always
-     consistent.
-
-     (This suggests that we actually *could* do ``switch`` if we
-     really wanted to, by replacing the revision history and head
-     revision.  But I don't think it's a good idea.)
-
-   - Can add a new revision to the end.
-
-   - By indexing into this can translate between 0-based revision
-     numbers and revision ids. 
-
-   - By walking through and looking at revision dates can find
-     revisions in a particular date range.
-
- * Calculations on branch histories:
-
-   - Find if one branch is a prefix of another.
-
-   - Find the latest common ancestor of another.
-
- * Three-way merge between revisions
-
-   - Resolve shape of directory
-   
-   - Then resolve textual conflicts
-
- * Pull/push changes when they perfectly match
-
-   - Possible when the destination is a prefix of the source
-
-   - Just move all revisions, manifests and texts across, and 
-
- * Merge all changes from one branch into another
-
- * Signatures applied to revisions
-
-   - There is a separable module for checking a signature: this is
-     passed the claimed author, changeset, date.  This needs to fetch
-     an appropriate key, decide if it is trusted to correspond to that
-     author, is not revoked, etc.
-
-   - If it is unknown, untrusted, revoked, etc, that is reported.
-     Depending on a paranoia level it may cause the operation to
-     abort.
-

=== removed file 'doc/library-interface.txt'
--- doc/library-interface.txt	2005-03-09 04:51:05 +0000
+++ doc/library-interface.txt	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-*****************
-Library interface
-*****************
-
-Should have a pleasant library interface.
-
-* Be clear what is supposed to be in a stable public interface and
-  what is not.  (A common failing of Python programs.)
-
-* If a function changes the working directory (as may be necessary to
-  run patch) it should restore it.

=== removed file 'doc/merge.txt'
--- doc/merge.txt	2005-05-02 04:24:33 +0000
+++ doc/merge.txt	1970-01-01 00:00:00 +0000
@@ -1,191 +0,0 @@
-=======
-Merging
-=======
-
-There should be one merge command which does the right thing, and
-which is called 'merge'.
-
-The merge command pulls a changeset or a range of changesets into your
-tree.  It knows what changes have already been integrated and avoids
-pulling them again.
-
-There should be some intelligence about working out what changes have
-already been merged. 
-
-The tool intelligently chooses (or perhaps synthesizes) an ancestor
-and two trees to merge.  These are then intelligently merged.
-
-Merge should refuse to run (unless forced) if there are any
-uncommitted changes in your tree beforehand.  This has two purposes:
-if you mess up the merge you won't lose anything important; secondly
-this makes it more likely that the merge will be relatively pure.
-
-It is a good idea to commit as soon as a merge is complete and
-satisfactorily resolved, so as to protect the work you did in the
-merge and to keep it separate from later development.  (Mark suggests
-an option to automatically commit when the merge is complete.)
-
-
-Recording merges
-----------------
-
-bzr records what branches have been merged so far.  This is useful as
-historical information and also for later choosing a merge ancestor.
-
-For each revision we record all the other revisions which have come
-into this tree, either by being completely merged or as cherry-picks.
-
-(This design is similar to the PatchLogPruning__ draft from baz.)
-
-__ http://wiki.gnuarch.org/PatchLogPruning
-
-This list of merged revisions is generally append-only, but can be
-reduced if changes are taken back out.  Changes can be
-anti-cherry-picked, which causes any successors to change from being
-fully-merged to being cherry-picked.
-
-The list of merged patches is stored delta-compressed.
-
-
-``tla update``
---------------
-
-``tla update`` performs a useful but slightly subtle change: it pulls
-in only changes that have been made on the other branch since you last
-merged.  That is to say, it sets the merge basis as the most recent
-merged-from point on the other branch.
-
-This means that any changes which were taken from your branch into the
-other and then reversed or modified will not be reversed.  Those
-changes will always be considered as new in your branch and will have
-precedence.
-
-The basic idea of a merge that only brings in remote work and doesn't
-revert your own changes is good.  It could be handled by a three way
-merge with a specified version but perhaps there is a better way.
-
-
-Merging tree shape
-------------------
-
-Merge is conducted at two levels: merging the tree shape, and merging
-the file contents.  Merging the tree shape means accounting for
-renames, adds, deletes, etc.  This is almost the same as merging the
-two inventories, but we need to do a smart merge on them to enforce
-structural invariants.
-
-
-
-Interrupting a merge
---------------------
-
-Some tools insist that you complete the entire merge while the
-``merge`` command is running; you cannot exit the program or restart
-the computer because state is held in memory.  We should avoid that.
-At least when the tool is waiting for user input it should have
-written everything to disk sufficient to pick up and continue the
-merge from that point.
-
-This suggests that there should be a command to continue a merge;
-perhaps ``bzr resolve`` should look for any unresolved changes and
-start resolving them.  ``bzr merge`` can (by default) automatically
-start this process.
-
-One hard aspect is transformation of the tree state such as renames,
-directory creation, etc.  This might include files swapping place,
-etc.  We would like to do atomically but cannot.
-
-
-
-Aborting a merge
-----------------
-
-If a merge has been begun but not committed then ``bzr revert`` should
-put everything back as it was in the previous revision.  This includes
-resetting the tree state and texts, and also clearing the list of
-pending-merged revisions.
-
-
-
-Offline merge
--------------
-
-It should be possible to download all the data necessary to do a merge
-from a remote branch, then disconnect and complete the merge.  It
-should be possible to interrupt and continue the merge during this
-process.
-
-This implies that all the data is pulled down and stored somewhere
-locally before the actual merge begins.  It could be pulled either
-into the revision history on non-trunk revisions, or into temporary
-files.
-
-It seems useful to move all revisions and texts from the other branch
-into the storage of this branch, in concordance with the general idea
-of every branch moving towards complete knowledge.  This allows the
-most options for an offline merge, and also for later looking back to
-see what was merged in and what decisions were made during the merge.
-
-
-
-Merge metadata
---------------
-
-What does cherry-picking mean here?  It means we merged the changes
-from a revision relative to its predecessor?  But what if we actually
-want to merge the delta relative to something else?  Can that be
-represented?
-
-
-Rejected merges
----------------
-
-ddaa says perhaps we should have three states: with respect to a
-branch any foreign revision can be *merged*, *not-merged*, or *rejected*.
-The difference between *not-merged* and *rejected* is that not-merged
-patches will try to merge in when you next sync from their
-branch, whereas rejected changes will not.
-
-'rejected' seems technically equivalent to it merged with the text
-changes not present.  But perhaps there should be something more?
-
-
-Excluded changes
-----------------
-
-Bitkeeper has the very interesting feature of *excluded* changesets,
-where something previously committed to or merged into this branch is
-treated as if it never occurred.  Because of their representation of
-changes as a weave they can do this without dependency problems on
-later merges.
-
-Although the later changes will not mechanically conflict, there is of
-course no guarantee that excluding the patch will generate anything
-semantically valid.
-
-Cool though this is, it seems that few people understand it well
-enough to use it confidently.  It complicates the merge algorithms and
-seems to have been a source of some bugs.  It may be better just to
-apply a reversed patch.
-
-
-
-Implementation
---------------
-
-Each revision has a pointer to two sets, represented as lists of
-revision-ids:
-
-* ``merged-revisions``
-* ``picked-revisions``
-
-When a branch is merged in, its revision history plus merged-revisions
-are added to the merged-revisions file.  When changes are
-cherry-picked in they are added to the picked-revisions.
-
-These lists are stored in the text store and their sha1 and id is
-stored in the revision.
-
-While a merge is underway, these are stored in
-``.bzr/pending-merged-revisions`` and
-``.bzr/pending-picked-revisions``.

=== removed file 'doc/mirroring.txt'
--- doc/mirroring.txt	2005-03-09 04:51:05 +0000
+++ doc/mirroring.txt	1970-01-01 00:00:00 +0000
@@ -1,14 +0,0 @@
-Mirroring
-=========
-
-The basic idea of mirroring is good, but it's too hard to understand.
-
-Push mirrors and pull mirrors.
-
-Suggest that people commit to their local machine and then regularly
-mirror elsewhere as a backup.
-
-bk and darcs's design inherently avoids this; just make another repo
-and decide to use it that way.
-
-On the other hand they do not keep track of the progress of a patch.
\ No newline at end of file

=== removed file 'doc/monotone.txt'
--- doc/monotone.txt	2005-03-09 04:51:05 +0000
+++ doc/monotone.txt	1970-01-01 00:00:00 +0000
@@ -1,16 +0,0 @@
-********************
-Thoughts on Monotone
-********************
-
-Great project, but not perfect.
-
-My main feeling is that it exposes too much mechanism and in the wrong
-ways.  For example, hashes are all very well for representing
-identity, but I don't want to see them; I just want to say "r43 of
-this branch".  
-
-Cryptographic signing is good for making sure revisions are really
-what they claim to be, but I don't want to directly deal with it: I
-want it as a check that things are really happening as they should
-be.  It should be barely seen and not heard unless/until something
-goes wrong.

=== removed file 'doc/news.txt'
--- doc/news.txt	2005-03-09 04:51:05 +0000
+++ doc/news.txt	1970-01-01 00:00:00 +0000
@@ -1,147 +0,0 @@
-Bazaar-NG Development News
-**************************
-
-
-2005-02-20
-==========
-
-* Added documentation of `patch pools <pool.html>`_, explication of
-  commands in terms of file state transitions, `uncommit
-  <kill-version.html>`_, `journalling <interrupted.html>`_ through
-  recording changesets.  Feedback from Aaron on some of the design
-  documentation.
-
-* Much cleaner ``diff`` and ``status`` commands, based on a new Tree
-  class that abstracts access to either a current working copy or a
-  historical revision.
-
-* ``remove`` command now both removes the working copy and updates the
-  inventory, and has a new alias ``rm``.  
-
-* Basic local versioning works well::
-
-    % bzr init   
-    % echo 'int main() {}' > hello.c
-    % bzr add hello.c
-    % bzr commit 'my test program'
-    % bzr status
-    U    hello.c
-    % echo 'int main() { return 0; }' > hello.c 
-    % bzr status                               
-    M    hello.c
-    % bzr diff        
-    --- old/hello.c 
-    +++ new/hello.c 
-    @@ -1,1 +1,1 @@
-    -int main() {}
-    +int main() { return 0; }
-    % bzr commit 'fix return code'
-    % bzr log
-    ----------------------------------------
-    revno: 1
-    committer: mbp@sourcefrog.net
-    timestamp: Wed 2005-02-16 11:39:13.003095 EST +1100
-
-    my test program
-    ----------------------------------------
-    revno: 2
-    committer: mbp@sourcefrog.net
-    timestamp: Wed 2005-02-16 11:43:13.010274 EST +1100
-
-    fix return code
-    % bzr status
-    U    hello.c
-    % bzr rm hello.c
-    % ls
-    % bzr status    
-    D    hello.c
-    % bzr commit 'this program sucks!'
-    % bzr stauts
-    bzr: error: unknown command 'stauts'
-    exit 1
-    % bzr status
-    %
-
-
-
-* Many internal/hacker commands: ``revno``, ``get-inventory``,
-  ``get-file``, ...
-
-* Most of the implementation is split into reasonably clean objects:
-  `Branch`, `Revision`, `Tree`, `Inventory`.  I think these will give
-  a good foundation for non-intrusive remote operations.  They 
-  live in submodules of `bzrlib`.
-
- ====== ==================================
-    596 bzr.py
-    608 bzrlib/branch.py
-     39 bzrlib/errors.py
-     25 bzrlib/__init__.py
-    320 bzrlib/inventory.py
-    126 bzrlib/osutils.py
-     66 bzrlib/revision.py
-    173 bzrlib/store.py
-    107 bzrlib/tests.py
-     62 bzrlib/trace.py
-    239 bzrlib/tree.py
-     41 bzrlib/xml.py
- ------ ----------------------------------
-   2402 total
- ====== ==================================
-
-
-* After feedback and reflection I have decided not to use `SHA-1
-  hashes`__ as the main identifier of texts, inventories, revisions,
-  etc.  It would probably be safe, but there is a certain amount of
-  concern (or even FUD) about the security of these hashes.  Since
-  they cannot be trusted to be safe in the long term, it is better
-  just to use something cheaper to compute, and equally unique in the
-  absence of malice: UUIDs.
-
-__ hashes.html
-
-  We now store the SHA-1 of files, but use it only as a check that the
-  files were stored safely.  At tridge's suggestion we also keep the
-  size, which can catch some classes of bug or data corruption::
-
-    % bzr diff
-    bzr: error: wrong SHA-1 for file
-    '680757bf-2f6e-4ef3-9fb4-1b81ba04b73f' 
-    in ImmutableStore('/home/mbp/work/bazaar-ng/toy/.bzr/text-store')
-      inventory expects d643a377c01d3e29f3e6e05b1618eb6833992dd0
-      file is actually 352586b84597ea8915ef9b1fb5c9c6c5cdd26d7b
-      store is probably damaged/corrupt
-
-
-2005-02-27
-==========
-
-Revisions are now named by something based on the username and date,
-plus some random bytes to make it unique.  This may be a reasonable
-tradeoff against uniqueness and readability.   This same id is used by
-default for revision inventories::
-
-  mbp@sourcefrog.net-20050220113441-34e486671a10f75297e03986
-
-Keeping them separate is still a good thing, because the inventory may
-be large and we often want only the revision and not the inventory.
-It is possible for the revision to point to an older inventory if it
-has not changed (but this is not implemented yet.)
-
-Tridge pointed out some `possible performance problems`__ with the
-assumption that we can simply compare all files to check for changes. 
-
-__ unchanged.html
-
-XML has newlines to make it a bit more readable.
-
-Nested subdirectories are now supported down to an arbitrary depth.
-
-bzr can successfully import and reproduce itself.
-
-
-
-ongoing
-=======
-
-* Systematic command-line handling and option parsing, etc.
\ No newline at end of file

=== removed file 'doc/optional-edit.txt'
--- doc/optional-edit.txt	2005-03-09 04:51:05 +0000
+++ doc/optional-edit.txt	1970-01-01 00:00:00 +0000
@@ -1,30 +0,0 @@
-******************************
-Optional explicit edit command
-******************************
-
-The default is for each branch to have an editable working copy, which
-can be modified and committed.  However, in some cases (following RCS
-and BK) people might prefer to have either a read-only working copy,
-or none at all.
-
-Reasons:
-
- - Branches which are never edited don't need it; removing it saves
-   some time and space.  In particular this applies to branches on a
-   server which are meant to be branched from or committed too, not
-   edited.
-
- - Files for which there is no working copy or a read-only copy are
-   known not to be edited, and this can make some operations such as
-   diffing a bit simpler.
-
-Since this is mostly an optimization and requires user knowledge it
-perhaps should not be added.  Certainly not yet.
-
-Sketch::
-
-  $ bzr get --no-wc SOURCE TARGET
-  $ bzr edit --remove
-  $ bzr edit --read-only .
-  $ bzr edit --write foo.c
-

=== removed file 'doc/partial-commit.txt'
--- doc/partial-commit.txt	2005-03-09 04:51:05 +0000
+++ doc/partial-commit.txt	1970-01-01 00:00:00 +0000
@@ -1,20 +0,0 @@
-Partial commit
-**************
-
-In the first cut, commit covers the whole tree.
-
-Secondly, people might want to commit only particular files or
-subdirectories.  Semantics of this in the case of renames/adds/deletes
-may be a bit complex, but I think it can be done.  The basic point is
-to build an inventory which includes only the specified changes from
-the working copy. 
-
-Beyond that, it can be very nice in darcs to commit only selected
-changes to a file.  I think this should not be the default though;
-perhaps only do it with ``-i``.  Even better (for some users) kick off
-a graphical tool to select the particular regions.  Beyond a certain
-point it may become easier for the user to explicitly set aside some
-changes and commit others first.
-
-In any case, it is a good idea to run a test suite on the revision to
-be committed, to make sure there were no missed dependencies.
\ No newline at end of file

=== removed file 'doc/pool.txt'
--- doc/pool.txt	2005-03-09 04:51:05 +0000
+++ doc/pool.txt	1970-01-01 00:00:00 +0000
@@ -1,65 +0,0 @@
-Patch pools
-***********
-
-*Patch pools* are an optimization for efficient storage of related
-branches.  They are not required for the first release.
-
-Revisions, inventories, and file states are identified in Bazaar-NG by
-universally unique hashes, and they are never modified once they are
-created.  Objects which are common between branches may therefore be
-stored only once and referenced from each branch.  Various strategies
-are available for doing this.
-
-Objects can be held hard-linked between related branches on
-filesystems that support hardlinks.  This provides automatic reference
-counting as branches are deleted.
-
-It is common to have several branches of the same project on a
-machine, with many objects in common.  These can be configured with
-each other on the pool path.  The parent should be the default pool
-path when creating a new branch.
-
-Each user might also have a pool that acts as a cache of all remote
-revisions.  Such a cache might use some kind of least-recently-used
-policy to limit its size.
-
-The user might nominate a series or hierarchy of pools to be searched for a
-patch; these might be progressively on the local machine, local
-network and remotely.  
-
-A system like the supermirror might make good use of a pool that
-gradually accumulates all public objects in the world, and stores
-branches very cheaply.
-
-One complication is garbage collection.  Naive implementations that
-store references from branches into the pool will not be able to
-detect objects that are no longer referenced by any active branch; as
-branches are created and deleted over time such objects will
-accumulate.  This may not be a problem in many cases, given the
-relative abundance of disk compared to programmer time, and the
-relatively small number of long branches that are discarded.  There
-are some partial solutions:
-
-* Keep a reference count for each object.  There is no problem of
-  circular references.  However, keeping the count accurately requires
-  that branches are never lost or deleted other than through the
-  correct mechanism.
-
-* From time to time, building a new pool including only objects from
-  active branches.
-
-* Keeping a pool that holds only patches known to be available from
-  elsewhere, so the pool is only a cache and never the single source
-  of a particular object.  Such a pool can then be discarded at will
-  and the objects will be re-fetched from their original source.
-
-This last point suggests that new objects should never be written
-solely into a pool, because of the risk that they might be
-accidentally lost.
-
-Using the parent on the default pool path allows for varying
-greed or laziness in fetching objects.  By default, objects might be
-read only as necessary, and then stored in the local cache.  If the
-user wants to keep the whole history available locally that could be
-specified with a ``--greedy`` option when making the branch, or
-through later pulling down the history.
\ No newline at end of file

=== removed file 'doc/purpose.txt'
--- doc/purpose.txt	2005-03-21 22:29:49 +0000
+++ doc/purpose.txt	1970-01-01 00:00:00 +0000
@@ -1,78 +0,0 @@
-What is the purpose of version control?
-***************************************
-
-There are several overlapping purposes:
-
-* Allowing concurrent development by several people.  Primarily,
-  helping resolve changes when they are integrated, but also making
-  each person aware of what the others have been doing, allowing them
-  to talk about changes, etc.
-
-* To allow different lines of development, with sensible
-  reintegration.  e.g. stable/development, or branches for
-  experimental features.
-
-* To record a history of development, so that you can find out why a
-  feature went in or who added it, or which releases might be affected
-  by a bug.
-
-* In particular, as a *record of decisions* about the project made by
-  the developers.
-
-* Help in writing a gloss on that history; not simply a record of
-  events but an explanation of what happened and why.  Such a record
-  may need to be written at several levels: a very detailed
-  explanation of changes to a function; a note that a bug was fixed
-  and why; and then a brief NEWS file for the whole release.  
-
-  While past events can never change, the intepretation placed upon
-  them may change, even after the event.  For example, even after a
-  release has gone out, one might want to go back and note that a bug
-  was actually fixed.
-
-  The system is helping the developers tell a story about the
-  development of the project.
-
-* As an aid to thinking about the project.  (Much as a personal
-  journal is not merely a record or even analysis of events, but also
-  a chance to reflect and to think of the future.)
-
-  People can review diffs, and then write a description of what
-  changed, and in doing so perhaps realize something else they should
-  do, or realize they made a mistake.  Making branches helps work out
-  the order of feature integration and the stability of different
-  lines of development.
-
-* As an 'undo' protection mechanism.  This is one reason why version
-  control can be useful on projects that have only a single developer
-  and never branch.
-
-* Incidentally, as a backup mechanism.  Version control systems,
-  particularly distributed systems, tend to cause code to exist on
-  several machines, which gives some protection against loss of any
-  single copy.  It's still a good idea to use a separate backup system
-  as well.
-
-* As a file-sharing mechanism: even with just a single developer and
-  line of development it can be useful to keep files synchronized
-  between several machines, which may not always be connected.
-
-
-----
-
-Steve Berczuk says__:
-
-__ http://www.bell-labs.com/cgi-user/OrgPatterns/OrgPatterns?ConfigurationManagementPatterns
-
-    A successful configuration management process allows:
-
-    * Developers to work together on a project, sharing common code.
-
-    * Developers to share development effort on a module.
-
-    * Developers to have access to the current stable (tested) version of a system.
-
-    * The ability to back up to a previous stable version (one of a number of NamedStableBases), of a system
-
-    * The ability of a developer to checkpoint changes to a module and
-      to back off to a previous version of that module

=== removed file 'doc/python.txt'
--- doc/python.txt	2005-04-15 01:31:21 +0000
+++ doc/python.txt	1970-01-01 00:00:00 +0000
@@ -1,101 +0,0 @@
-Choice of Python
-----------------
-
-This will be written in Python, at least for the first cut, just for
-ease of development -- I think I am at least 2-3 times faster than in
-C or C++, and bugs may be less severe.  I am open to the idea of
-switching to C at some time in the future, but because that is
-enormously expensive I want to avoid it until it's clearly necessary.
-Python is also a good platform to handle cross-platform portability.
-
-Possible reasons to go to C:
-
-Audience acceptance
-  If Linus says "I'd use it if it were written in
-  C" that would be persuasive.  I think the good developers we want to
-  do not consider implementation language as a dominant factor.  A few
-  related but separate questions are important to them: modest
-  dependencies, easy installation, presence in distributions (or as
-  packages), active support, etc.  A few queries show that Python is
-  seen as relatively safe and acceptable even by people who don't
-  actually use it.
-
-Speed
-  Having scalable designs is much more important.
-  Secondly, we will do most of the heavy lifting in external
-  C programs in the first cut, and perhaps move these into native
-  libraries later.  (Subversion people had trouble in relying on GNU
-  diff on legacy platforms and they had to integrate the code eventually.)
-
-Bindings to other languages
-  If we have only a Python interpretation then it can be run as a
-  shell script from emacs or similar tools.  It can also be natively
-  called from Python scripts, which would allow GUI bindings to almost
-  every toolkit, and it can possibly be called from Java and
-  .NET/Mono.    By the time this is
-  mature, it's possible that Python code will be able to cross-call
-  Perl and other languages through Parrot.  
-  There should be enough options there to support a good
-  infrastructure there of additional tools.
-  If it was necessary to provide a C API that can perhaps be wrapped around a 
-  Python library.
-
-Reuse of tla code
-  That may be useful, if there are substantial sections that approach
-  or meet our goals for both design and implementation (e.g. being
-  good to use from a library.)
-
-  This does not necessarily mean doing the whole thing in C; we could
-  call out to tla or could wrap particular bits into libraries.
-
-
-----
-
-Erik Bgfors:
-
-  However, I think it's very important that a VCS can be wrapped in other
-  languages so that it can be integrated in IDE's and have tools written
-  for them.
-  
-  A library written in c would be simple to wrap in other languages and
-  therefore could be used from for example monodevelop and friends. I
-  really believe this is important for a VCS.
-
-I agree; this is a more important argument against Python than speed,
-where I think we can be entirely adequate just using smart design.
-But there are some partial answers:
-
-We can design bzr to be easily called as an external process -- not
-depending on interactive input, having systematically parsed output,
---format=xml output, etc.  This is the only mode CVS supports, and
-people have built many interesting tools on top of it, and it's still
-popular for svn and tla.  For things like editor integration this is
-often the easiest way.
-
-Secondly, there is a good chance of calling into Python from other
-languages.  There are projects like Jython, IronPython, Parrot and so
-on that may well fix this.
-
-Thirdly, we can present a Python library through a C interface; this
-might seem a bit weird but I think it will work fine.  Python is
-easily embeddable; this might be the best way for Windows IDE
-integration.
-
-Finally, if none of these work, then we can always recode in C,
-treating Python only as a prototype.
-
-I think working in Python I can develop it at least twice as fast as
-in C, particularly in this early phase where the design is still being
-worked out.  Although all other things being equal it might be nice to
-be in pure C, but I don't think it's worth paying that price.
-
-  One of the problems with darcs is that it's such a mess wrapping it.
-
-Yes.
-
-----
-
-Experiments to date on large trees show that even with little
-optimization, bzr is mostly disk-bound, and the CPU time usage is only
-a few seconds.  That supports the position that Python performance
-will be adequate.

=== removed file 'doc/quilt.txt'
--- doc/quilt.txt	2005-03-09 04:51:05 +0000
+++ doc/quilt.txt	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-Quilt
-=====
-
-Descendent of Andrew Morton's patch scripts.
-
-Demonstrates that quite a powerful tool can be built in a relatively
-short time.
-
-Is largely not concerned with preserving a history record.  Allows 
-
-Pop patches off, similar to excluding them.
\ No newline at end of file

=== removed file 'doc/quotes.txt'
--- doc/quotes.txt	2005-04-05 08:24:51 +0000
+++ doc/quotes.txt	1970-01-01 00:00:00 +0000
@@ -1,2 +0,0 @@
-  If you're just quilting the good ideas of others, then that seems more
-  like standing on the shoulders of giants -- bje

=== removed file 'doc/random.txt'
--- doc/random.txt	2005-04-26 05:20:17 +0000
+++ doc/random.txt	1970-01-01 00:00:00 +0000
@@ -1,257 +0,0 @@
-I think Ruby's point is right: we need to think about how a tool
-*feels* as you're using it.
-
-Making regular commits gives a nice rhythm to to working; in some
-ways it's nicer to just commit single files with C-x v v than to build
-complex changesets.  (See gmane.c.v-c.arch.devel post 19 Nov, Tom
-Lord.)
-
-
-* Would like to generate an activity report, to e.g. mail to your boss
-  or post to your blog.  "What did I change today, across all these
-  specified branches?"
-
-
-* It is possibly nice that tla by default forbids you from committing
-  if emacs autosave or lock files exist -- I find it confusing to
-  commit somethin other than what is shown in the editor window
-  because there are unsaved changes. 
-
-  However, grumbling about unknown files is annoying, and requiring
-  people to edit regexps in the id-tagging-method file to fix it is
-  totally unreasonable.
-
-  Perhaps there should be a preference to abort on unknown files, or
-  perhaps it should be possible to specify forbidden files.
-
-  Perhaps this is related to a mechanism to detect conflicted files:
-  should refuse to commit if there are any .rej files lying around.
-
-
-
-   *Those who lose history are doomed to recreate it.*
-         -- broked (on #gnu.arch.users)
-
-   *A universal convention supplies all of maintainability, clarity, consistency, and a foundation for good programming habits too. What it doesn't do is insist that you follow it against your will. That's Python!*
-
-    -- Tim Peters on comp.lang.python, 2001-06-16
-
-    (Bazaar provides mechanism and convention, but it is up to you
-    whether you wish to follow or enforce that convention.)
-
-----
-
-jblack asks for 
-
-   A way to subtract merges, so that you can see the work you've done
-   to a branch since conception.
-
-----
-
-:: 
-
-  <mpool> now that is a neat idea: advertise branches over zeroconf
-  <mpool> should make lca fun :-)
-
-----
-
-http://thedailywtf.com/ShowPost.aspx?PostID=24281
-
-  Source control is necessary and useful, but in a team of one (or even two) people the setup overhead isn't always worth it--especially if you're going to join source control in a month, and you don't want to have to migrate everything out of your existing (in my case, skunkworks) system before you can use it.
-
-  At least that was my experience--I putzed with CVS a bit and knew other source control systems pretty well, but in the day-to-day it wasn't worth the bother (granted, I was a bit offended at having to wait to use the mainline source control, but that's another matter).
-
-I think Bazaar-NG will have such low setup overhead (just ``init``,
-``add``) that it can be easily used for even tiny projects.  The
-ability to merge previously-unrelated trees means they can fold their
-project in later.
-
-
-----
-
-From tridge:
-
-* cope without $EMAIL better
-
-* notes at start of .bzr.log:
-  * you can delete this
-  * or include it in bug reports
-
-* should you be able to remove things from the default ignore list?
-
-* headers at start of diff, giving some comments, perhaps dates
- 
-  * is diff against /dev/null really OK?  I think so.
-
-* separate remove/delete commands?  
-
-* detect files which were removed and now in 'missing' state
-
-* should we actually compare files for 'status', or check mtime and
-  size; reading every file in the samba source tree can take a long
-  time.
-
-  without this, doing a status on a large tree can be very slow.  but
-  relying on mtime/size is a bit dangerous.
-
-  people really do work on trees which take a large chunk of memory
-  and which will not stay in memory
-
-* status up-to-date files: not 'U', and don't list without --all
-
-* if status does compare file text, then it should be quick when
-  checking just a single file
-
-* wrapper for svn that every time run logs
-
-  - command
-  - all inputs
-  - time it took
-  - sufficient to replay everything
-  - record all files
-
-* status crashes if a file is missing
-
-* option for -p1 level on diff, etc.  perhaps
-
-* commit without message should start $EDITOR
-
-* don't duplicate all files on commit
-
-* start importing tridge-junkcode
-
-* perhaps need xdelta storage sooner rather than later, to handle very
-  large file
-
-----
-
-The first operation most people do with a new version-control system
-is *not* making their own project, but rather getting a checkout of an
-existing project, building it, and possibly submitting a patch.  So
-those operations should be *extremely* easy.
-
-----
-
-* Way to check that a branch is fully merged, and no longer needed:
-  should mean all its changes have been integrated upstream, no
-  uncommitted changes or rejects or unknown files.
-
-* Filter revisions by containing a particular word (as for log).
-  Perhaps have key-value fields that might be used for
-  e.g. line-of-development or bug nr?
-
-* List difference in the revisions on one branch vs another.
-
-* Perhaps use a partially-readable but still hopefully unique ID for
-  revisions/inventories?
-
-* Preview what will happen in a merge before it is applied
-
-* When a changeset deletes a file, should have the option to just make
-  it unknown/ignored.
-
-  Perhaps this is best handled by an interactive merge.  If the file
-  is unchanged locally and deleted remotely, it will by default be
-  deleted (but the user has the option to reject the delete, or to
-  make it just unversioned, or to save a copy.)  If it is modified
-  locall then the user still needs to choose between those options but
-  there is no default (or perhaps the default is to reject the delete.)
-
-* interactive commit, prompting whether each hunk should be sent (as
-  for darcs)
-
-* Write up something about detection of unmodified files
-
-* Preview a merge so as to get some idea what will happen:
-
-  * What revisions will be merged (log entries, etc)
-
-  * What files will be affected?
-
-  * Are those simple updates, or have they been updated locally as
-    well.
-
-  * Any renames or metadata clashes?
-
-  * Show diffs or conflict markers.
-
-  * Do the merge, but write into a second directory.
-
-* "Show me all changesets that touch this file"
-
-  Can be done by walking back through all revisions, and filtering out
-  those where the file-id either gets a new name or a new text.  
-
-* Way to commit backdated revisions or pretend to be something by
-  someone else, for the benefit of import tools; in general allow
-  everything taken from the current environment to be overridden.
-
-* Cope well when trying to checkout or update over a flaky
-  connection.  Passive HTTP possibly helps with this: we can fetch all
-  the file texts first, then the inventory, and can even retry
-  interrupted connections.
-
-* Use readline for reading log messages, and store a history of
-  previous commit messages!
-
-* Warn when adding huge files(?) - more than say 10MB?  On the other
-  hand, why not just cope?
-
-* Perhaps allow people to specify a revision-id, much as people have
-  unique but human-assigned names for patches at the moment?
-
-----
-
-20050218090900.GA2071@opteron.random
-
-    Subject:      Re: [darcs-users] Re: [BK] upgrade will be needed
-    From:         Andrea Arcangeli <andrea@suse.de>
-    Newsgroups:   gmane.linux.kernel
-    Date:         Fri, 18 Feb 2005 10:09:00 +0100
-
-    On Thu, Feb 17, 2005 at 06:24:53PM -0800, Tupshin Harper wrote:
-    > small to medium sized ones). Last I checked, Arch was still too slow in 
-    > some areas, though that might have changed in recent months. Also, many 
-
-    IMHO someone needs to rewrite ARCH using the RCS or SCCS format for the
-    backend and a single file for the changesets and with sane parameters
-    conventions miming SVN. The internal algorithms of arch seems the most
-    advanced possible. It's just the interface and the fs backend that's so
-    bad and doesn't compress in the backups either.  SVN bsddb doesn't
-    compress either by default, but at least the new fsfs compresses pretty
-    well, not as good as CVS, but not as badly as bsddb and arch either.
-
-    I may be completely wrong, so take the above just as a humble
-    suggestion.
-
-    darcs scares me a bit because it's in haskell, I don't believe very much
-    in functional languages for compute intensive stuff, ram utilization
-    skyrockets sometime (I wouldn't like to need >1G of ram to manage the
-    tree). Other languages like python or perl are much slower than C/C++
-    too but at least ram utilization can be normally dominated to sane
-    levels with them and they can be greatly optimized easily with C/C++
-    extensions of the performance critical parts.
-
-
------
-
-When sending patches by email (optionall) send each one as a separate
-mail, with a sequence number [%03d/%03d] at the start of the Subject.
-See mail from linus 2005-04-07.
-
-http://linux.yyz.us/patch-format.html
-
-http://www.zip.com.au/~akpm/linux/patches/stuff/tpp.txt
-
-----
-
-dwmw2 (2005-04-07) 
-
-reorder patches by cherry-picking them from a main development tree
-before sending them on.
-
-----
-
-ignore BitKeeper
-
-re-add should give the same id as before

=== removed file 'doc/requirements.txt'
--- doc/requirements.txt	2005-03-09 04:51:05 +0000
+++ doc/requirements.txt	1970-01-01 00:00:00 +0000
@@ -1,48 +0,0 @@
-************
-Requirements
-************
-
-gcc requirements
-----------------
-
-* http://lists.gnu.org/archive/html/gnu-arch-users/2004-06/msg00082.html
-* http://gcc.gnu.org/ml/gcc/2004-06/msg00264.html
-
-Some of these are:
-
-* Must preserve existing history from CVS; must be able to export back
-  out to CVS in case they change their mind.
-
-* Support a single central blessed version.
-
-* Cheap branches and tagging.
-
-* Handle repeated merges; remember the last time something was merged.
-
-* Easy cloning.
-
-* Copy a named patch set (one or more changes) onto a particular
-  branch.
-
-* Unix, Windows, Mac.
-
-* What patches changed a particular symbol?
-
-* annotate function that shows deleted lines.
-
-* Reliable backups.  (bzr can do this simply by cloning, then
-  mirror-sync.)
-
-* Scale to enormously wide and long history.
-
-* Easy to get back to last-known-good state.
-
-
-
-kernel
-------
-
- * http://www.kerneltraffic.org/kernel-traffic/kt20030323_210.txt
-
-   Discussion of BitKeeper, strengths and weaknesses, and
-   interoperation tools.

=== removed file 'doc/revfile-annotation.txt'
--- doc/revfile-annotation.txt	2005-05-06 03:20:15 +0000
+++ doc/revfile-annotation.txt	1970-01-01 00:00:00 +0000
@@ -1,155 +0,0 @@
-==============================
-Extension to store annotations
-==============================
-
-We might extend the revfile format in a future version to also store
-annotations.  *This is not implemented yet.*
-
-In previous versions, the  index file identified texts by their
-SHA-1 digest.  This was unsatisfying for two reasons.  Firstly it
-assumes that SHA-1 will not collide, which is not an assumption we
-wish to make in long-lived files.  Secondly for annotations we need
-to be able to map from file versions back to a revision.
-
-Texts are identified by the name of the revfile and a UUID
-corresponding to the first revision in which they were first
-introduced.  This means that given a text we can identify which
-revision it belongs to, and annotations can use the index within the
-revfile to identify where a region was first introduced.
-
-  We cannot identify texts by the integer revision number, because
-  that would limit us to only referring to a file in a particular
-  branch.
-
-  I'd like to just use the revision-id, but those are variable-length
-  strings, and I'd like the revfile index to be fixed-length and
-  relatively short.  UUIDs can be encoded in binary as only 16 bytes.
-  Perhaps we should just use UUIDs for revisions and be done?
-
-
-
-Annotations
------------
-
-Annotations indicate which revision of a file first inserted a line
-(or region of bytes).
-
-Given a string, we can write annotations on it like so: a sequence of
-*(index, length)* pairs, giving the *index* of the revision which
-introduced the next run of *length* bytes.  The sum of the lengths
-must equal the length of the string.  For text files the regions will
-typically fall on line breaks.  This can be transformed in memory to
-other structures, such as a list of *(index, content)* pairs.
-
-When a line was inserted from a merge revision then the annotation for
-that line should still be the source in the merged branch, rather than
-just being the revision in which the merge took place.
-
-They can cheaply be calculated when inserting a new text, but are
-expensive to calculate after the fact because that requires searching
-back through all previous text and all texts which were merged in.  It
-therefore seems sensible to calculate them once and store them.
-
-To do this we need two operators which update an existing annotated
-file:
-
-A. Given an annotated file and a working text, update the annotation to
-   mark regions inserted in the working file as new in this revision.
-
-B. Given two annotated files, merge them to produce an annotated
-   result.    When there are conflicts, both texts should be included
-   and annotated.
-
-These may be repeated: after a merge there may be another merge, or
-there may be manual fixups or conflict resolutions.
-
-So what we require is given a diff or a diff3 between two files, map
-the regions of bytes changed into corresponding updates to the origin
-annotations.
-
-Annotations can also be delta-compressed; we only need to add new
-annotation data when there is a text insertion.
-
-    (It is possible in a merge to have a change of annotation when
-    there is no text change, though this seems unlikely.  This can
-    still be represented as a "pointless" delta, plus an update to the
-    annotations.)
-
-Index file
-----------
-
-In a proposed (not implemented) storage with annotations, the index
-file is a series of fixed-length records::
-
-  byte[16]     UUID of revision
-  byte[20]     SHA-1 of expanded text (as binary, not hex)
-  uint32       flags: 1=zlib compressed
-  uint32       sequence number this is based on, or -1 for full text
-  uint32       offset in text file of start
-  uint32       length of compressed delta in text file
-  uint32[3]    reserved
-
-Total 64 bytes.
-
-The header is also 64 bytes, for tidyness and easy calculation.  For
-this format the header must be ``bzr revfile v2\n`` padded with
-``\xff`` to 64 bytes.
-
-The first record after the header is index 0.  A record's base index
-must be less than its own index.
-
-The SHA-1 is redundant with the inventory but stored just as a check
-on the compression methods and so that the file can be validated
-without reference to any other information.
-
-Each byte in the text file should be included by at most one delta.
-
-
-Deltas
-------
-
-In a proposed (not implemented) storage with annotations, deltas to
-the text are stored as a series of variable-length records::
-
-  uint32        idx
-  uint32        m
-  uint32        n
-  uint32        l
-  byte[l]       new
-
-This describes a change originally introduced in the revision
-described by *idx* in the index.
-
-This indicates that the region [m:n] of the input file should be
-replaced by the text *new*.  If m==n this is a pure insertion of l
-bytes.  If l==0 this is a pure deletion of (n-m) bytes.
-
-
-
-
-
-Open issues
------------
-
-
-* Storing the annotations with the text is reasonably simple and
-  compact, but means that we always need to process the annotation
-  structure even when we only want the text.  In particular it means
-  that full-texts cannot just simply be copied out but rather composed
-  from chunks.  That seems inefficient since it is probably common to
-  only want the text.
-
-* Should annotations also indicate where text was deleted?
-
-* This design calls for only one annotation per line, which seems
-  standard.  However, this is lacking in at least two cases:
-
-  - Lines which originate in the same way in more than one revision,
-    through being independently introduced.  In this case we would
-    apparently have to make an arbitrary choice; I suppose branches
-    could prefer to assume lines originated in their own history.
-
-  - It might be useful to directly indicate which mergers included
-    which lines.  We do have that information in the revision history
-    though, so there seems no need to store it for every line.
-

=== removed file 'doc/revfile.txt'
--- doc/revfile.txt	2005-05-06 03:20:15 +0000
+++ doc/revfile.txt	1970-01-01 00:00:00 +0000
@@ -1,148 +0,0 @@
-********
-Revfiles
-********
-
-The unit for compressed storage in bzr is a *revfile*, whose design
-was suggested by Matt Mackall.
-
-This document describes version 1 of the file, and has some notes on
-what might be done in version 2.
-
-
-Requirements
-============
-
-Compressed storage is a tradeoff between several goals:
-
-* Reasonably compact storage of long histories.
-
-* Robustness and simplicity.
-
-* Fast extraction of versions and addition of new versions (preferably
-  without rewriting the whole file, or reading the whole history.)
-
-* Fast and precise annotations.
-
-* Storage of files of at least a few hundred MB.
-
-* Lossless in useful ways: we can extract a series of texts and write
-  them back out without losing any information.
-
-
-Design
-======
-
-revfiles store the history of a single logical file, which is
-identified in bzr by its file-id.  In this sense they are similar to
-an RCS or CVS ``,v`` file or an SCCS sfile.
-
-Each state of the file is called a *text*. 
-
-Renaming, adding and deleting this file is handled at a higher level
-by the inventory system, and is outside the scope of the revfile.  The
-revfile name is typically based on the file id which is itself
-typically based on the name the file had when it was first added.  But
-this is purely cosmetic.
-
-    For example a file now called ``frob.c`` may have the id
-    ``frobber.c-12873`` because it was originally called
-    ``frobber.c``.  Its texts are kept in the revfile
-    ``.bzr/revfiles/frobber.c-12873.revs``.
-
-When the file is deleted from the inventory the revfile does not
-change.  It's just not used in reproducing trees from that point
-onwards.
-
-The revfile does not record the date when the text was added, a commit
-message, properties, or any other metadata.  That is handled in the
-higher-level revision history.
-
-Inventories and other metadata files that vary from one version to the
-next can themselves be stored in revfiles.
-
-revfiles store files as simple byte streams, with no consideration of
-translating character sets, line endings, or keywords.  Those are also
-handled at a higher level.  However, the revfile may make use of
-knowledge that a file is line-based in generating a diff.  
-
-   (The Python builtin difflib is too slow when generating a purely
-   byte-by-byte delta so we always make a line-by-line diff; when this
-   is fixed it may be feasible to use line-by-line diffs for all
-   files.)
-
-Files whose text does not change from one revision to the next are
-stored as just a single text in the revfile.  This can happen even if
-the file was renamed or other properties were changed in the
-inventory.
-
-The revfile is held on disk as two files: an *index* and a *data*
-file.  The index file is short and always read completely into memory;
-the data file is much longer and only the relevant bits of it,
-identified by the index file, need to be read.
-
-  This design is similar to that of Netscape `mail summary files`_, in
-  that there is a small index which can always be read into memory and
-  that quickly identifies where to look in the main file.  They differ
-  in many other ways though, most particularly that the index is not
-  just a cache but holds precious data in its own right.
-
-.. _`mail summary files`: http://www.jwz.org/doc/mailsum.html
-
-This is meant to scale to hold 100,000 revisions of a single file, by
-which time the index file will be ~4.8MB and a bit big to read
-sequentially.
-
-Some of the reserved fields could be used to implement a (semi?)
-balanced tree indexed by SHA1 so we can much more efficiently find the
-index associated with a particular hash.  For 100,000 revs we would be
-able to find it in about 17 random reads, which is not too bad.  On
-the other hand that would compromise the append-only indexing, and
-100,000 revs is a fairly extreme case.
-
-This performs pretty well except when trying to calculate deltas of
-really large files.  For that the main thing would be to plug in
-something faster than difflib, which is after all pure Python.
-Another approach is to just store the gzipped full text of big files,
-though perhaps that's too perverse?
-
-
-Identifying texts
------------------
-
-In the current version, texts are identified by their SHA-1.  
-
-
-Skip-deltas
------------
-
-Because the basis of a delta does not need to be the text's logical
-predecessor, we can adjust the deltas to avoid ever needing to apply
-too many deltas to reproduce a particular file.  
-
-
-Tools
------
-
-The revfile module can be invoked as a program to give low-level
-access for data recovery, debugging, etc.
-
-
-
-
-Open issues
-===========
-
-* revfiles use unsigned 32-bit integers both in diffs and the index.
-  This should be more than enough for any reasonable source file but
-  perhaps not enough for large binaries that are frequently committed.
-
-  Perhaps for those files there should be an option to continue to use
-  the text-store.  There is unlikely to be any benefit in holding
-  deltas between them, and deltas will anyhow be hard to calculate. 
-
-* The append-only design does not allow for destroying committed data,
-  as when confidential information is accidentally added.  That could
-  be fixed by creating the fixed repository as a separate branch, into
-  which only the preserved revisions are exported.
-
-* Should we also store full-texts as a transitional step?

=== removed file 'doc/revision-syntax.txt'
--- doc/revision-syntax.txt	2005-03-09 04:51:05 +0000
+++ doc/revision-syntax.txt	1970-01-01 00:00:00 +0000
@@ -1,77 +0,0 @@
-Syntax for identifying revisions
-********************************
-
-There seem to be two main options: use a separate ``-r`` option, or
-put a revision identifier within the filename.
-
-``-r`` options are familiar from CVS and Subversion, and easy to
-specify, and cover almost all situations.
-
-Special identifiers in the filename are sometimes more concise, and
-allow for more complete specification of filenames in the presence of
-renames.  They may be more complex to implement, and will cause some
-filenames to be either forbidden or to require escaping.
-
-----
-
-
-How to interpret revisions in the presence of renamed files?  Suppose
-hello.c was renamed in r3; when we do ``bzr diff -r2 hello.c`` should
-we compare to 
-
-* the file currently called ``hello.c``, as it was in revision 2, even
-  if it then had a different name
-
-or
-
-* the file that was called ``hello.c`` in revision 2, even if that is
-  a different file-id?
-
-Often only one will work, but it is possible to have situations where
-in revision 2 there was a file called ``hello.c`` but it is not the
-same one that now has that name.
-
-If we can have a simple syntax that accomodates either that would be
-good.  One might be to mix revision numbers into the path, rather than
-a separate ``-r`` option.  For example::
-
-   hello.c@2
-   @2/hello.c
-
-   src/@2/hello.c
-
-The simplest case is to just append the number, but you can vary
-them.  It might even be possible to chain them, getting the file id
-from one revision and its state from another:
-
-  @70/hello.c@31
-
-This means "the file that was called ``hello.c`` in revision 70, give
-me its state in revision 31."  (If you wanted to do this across
-branches, I think you might need to get the file id and then look it
-up, etc.)
-
-Some more syntax: you can refer to a revision by its hash just by
-specifying that; we can distinguish them from anything else by their
-length::
-
-  hello.c@09fac8dbfd27bd9b4d23a00eb648aa751789536d 
-
-(Is that really safe?  Perhaps some punctuation is needed to
-distinguish it from a label.  But then I think people should rarely or
-never use them; perhaps it need not be allowed at all.)
-
-You can also give a date, which is used to find the most recent
-revision no later than that date::
-
-  hello.c@{2004-12-01}
-  hello.c@{yesterday}
-  ./@{10:30}/
-
-(Braces can be a bit scary for shell, but if there are no commas they
-are somewhat safe.)
-
-Or, once we have labels, you can use a label::
-
-  hello.c@release-2.2
-

=== removed file 'doc/rollup.txt'
--- doc/rollup.txt	2005-03-09 04:51:05 +0000
+++ doc/rollup.txt	1970-01-01 00:00:00 +0000
@@ -1,68 +0,0 @@
-Roll-up changesets
-******************
-
-Sometimes it is useful to collapse changes; sometimes it is useful to
-pull in patches while keeping their original identity.  So (uniquely?)
-Bazaar-NG allows either option at the user's discretion.
-
-I'm not completely sure yet how this should be controlled: should it
-be an option at every stage, or should mergers generate roll-up
-changesets?  I think possibly the second.
-
-Rolled-up changesets can be represented as so::
-
-  mbp, 2005-04-01
-  import new quantum-computing feature
-
-    jrh, 2005-03-02
-    stub implementation for quantum computing
-
-    jrh, 2005-03-03
-    plus operator
-    
-    jrh, 2005-03-04
-    regexps
-
-    jrh, 2005-03-03
-    fix bugs, ready to go
-
-down to an arbitrary level of nesting.
-    
-When bringing changes into a branch, we have the choice of either
-rolling them up into a single changeset or bringing them across
-individually.  I'm not sure, but I think this may be best handled by
-having two separate commands:
-
-* *update* (or maybe *pull*) brings across revisions onto the current
-  branch, adding them one-by-one into the revision history.  This can
-  only be executed if the destination branch is a prefix of the source
-  branch and therefore we know the patches will apply perfectly.
-
-* *merge* pulls in one or more changes from another branch, and allows
-   the user to edit the merged state or fix any conflicts.  These
-   changes are then all marked as rolled-up by a new commit.  It is
-   possible to look up those changes inside the commit, if the
-   information is still accessible.
-
-In this model, Arch is missing the first command: you cannot put
-patches from one branch onto another -- and it would be hard to add
-it, since the unique branch id is part of the name of the patch name.
-
-Most other tools are missing the second: it is impossible to combine
-changesets and therefore the total history of any branch can grow very
-long; there is no way to make summaries within the tool itself.
-
-People might also want to retrospectively zip up several changes
-existing on a branch, but I think it may be better to make a new
-branch from before the changes and pull them across onto that.
-
-----
-
-One interesting application of this is in maintaining the ``NEWS``
-file present in many software distributions.  This summarizes the
-changes to the software in each release at a very high user-visible
-level.  Although this is very much version-control data it is not
-normally known to the VC system, except as another versioned file.
-
-One way to handle this is to have a *release* branch, where the commit
-message for each release the NEWS notes for that release.

=== removed file 'doc/scalability.txt'
--- doc/scalability.txt	2005-03-09 04:51:05 +0000
+++ doc/scalability.txt	1970-01-01 00:00:00 +0000
@@ -1,20 +0,0 @@
-***********
-Scalability
-***********
-
-bzr needs to scale up very well: projects with tens of thousands of
-commits, tens of thousands of files, and tens of thousands of
-branches.
-
-We are concerned with both the big-O performance of the design, and
-the multiplicative factors of the implementation.  Both is important.
-
-For example, darcs, svn and arch use more than one inode per working
-file (pristine, id file, etc).  This is only a constant factor, but
-enough to more than double the space used by a typical tree.  We would
-like to avoid it if we can.
-
-From a early stage in development the features which do work should be
-tested on large trees.
-
-

=== removed file 'doc/security.txt'
--- doc/security.txt	2005-03-09 04:51:05 +0000
+++ doc/security.txt	1970-01-01 00:00:00 +0000
@@ -1,409 +0,0 @@
-*****************************
-Security aspects of Bazaar-NG
-*****************************
-
-
-* Good security is required.
-
-* Usability is required for good security.
-
-    Being too strict "because it's the secure way" just means that people will
-    disable you altogether, or start doing things that they know is wrong,
-    because the right way of doing this may be secure, but [..] also very
-    inconvenient. 
-
-    -- Linus Torvalds
-
-.. contents:
-
-Requirements
-============
-
-David Wheeler gives some good requirements__:
- 
-    Problem is, the people who develop SCM tools often don't think about what kind of security requirements they need to support. This mini-paper describes briefly the kinds of security requirements an SCM tool should support. 
-
-__ http://www.dwheeler.com/essays/scm-security.html
-
-    confidentiality_
-      Are only those who should be able to read information able to do so?
-
-    integrity
-      Are only those who should be able to write/change information able to do so? This includes not only limiting access rights for writing, but also protecting against repository corruption.
-
-    availability
-      Is the system available to those who need it? (I.E., is it resistant to denial-of-service attacks?)
-
-    identification/authentication
-      Does the system safely authenticate its users? If it uses tokens (like passwords), are they protected when stored and while being sent over a network, or are they exposed as cleartext?
-
-    audit
-      Are actions recorded?
-
-    non-repudiation
-      Can the system "prove" that a certain user/key did an action later?
-
-    self-protection
-      Does the system protect itself, and can its own
-      data (like timestamps) be trusted?
-
-    trusted paths
-      Can the system make sure that its communication with users is
-      protected?
-
-Attacker categories
--------------------
-
-* Unprivileged outsiders.
-
-  (Almost always read-only, but people might want to allow them to
-  write in some cases, e.g. for wikis.)
-
-* Non-malicious developers with privilege.
-
-* Malicious developers with privilege.
-
-* Attackers who have stolen a privileged developer's identity.
-
-
-Access control
---------------
-
-Dan Nicolaescu gives these examples of access control:
-
-  - security related code that is still emabargoed, only select few
-    are allowed to see it, it is not desirable to release this
-    information to the public because a fix is still being worked
-    on. It would be nice to be able to have this kind of code under
-    the same version control system used for normal development for
-    ease of use and easy merging, yet it is crucial to restrict access
-    to a branches, files or directories to certain people.
-
-  - feature freeze before a release. It would be good if the release
-    manager could disable writing to the release branch, so that the
-    last tests are run, and not have someone commit stuff by mistake. 
-
-  - documentation/translation writers don't need write access to the
-    whole source code, just to the documentation directories. 
-
-  - For proprietary companies restricting access is even more
-    important, for example only some engineers should access the
-    latest development version of some code in order to keep some
-    trade secrets, etc, etc.
-
-In Bazaar-NG, the basic unit of access control is the branch.  If
-people are not supposed to read a branch, or know of its existence,
-put it somewhere where they can't see it.  If people are allowed to
-read from but not write to a branch then set those permissions.  The
-code can later be merged into a public branch if desired with no loss
-of function.
-
-We largely rely on lower-level security measures controlling who can
-get read or write access to a branch.  If you have a branch that
-should be confidential, then put it on an appropriately-secured
-machine, with only people in a particular group allowed to read it.
-
-Not having separate repositories is probably a feature here -- unlike
-Subversion, no features depend on having branches be in the same
-repository.  Each repository can have different group ownership.
-(The directories should usually be setgid.)  It also makes it easier
-to see just what the access control is; there is only one object that
-can meaningfully have an ACL.
-
-The existence of a secret branch can be fairly well hidden from the
-world.  When its changes are merged in, all that is visible is the
-name, date, and branch name of the commit, not anything about the
-location of the source branch.
-
-The documentation case I would handle by having a separate
-documentation branch, which could perhaps be checked out into a
-subdirectory when it is required.  I think this is fairly common for
-larger projects even in CVS.
-
-
-
-
-Confidentiality
----------------
-
-As dwheeler points out, this can be important even for open source
-projects, such as when preparing a security patch.
-Mechanisms that send email should have an option to encrypt the mail.
-
-I can't think of anywhere encrypted archives would be useful.  If you
-want to store it on an encrypted filesystem you can.  If you want to
-store encrypted files you can do that too, though that will leak some
-information in the metadata and branch structure.
-
-
-Security in distributed systems
--------------------------------
-
-If I have a branch on my laptop, the software ultimately cannot
-prevent me doing anything to that branch -- physical access trumps
-software controls.  We can, at most, try to prevent non-malicious
-mistakes.
-
-The purpose of the software here is to protect other people, whose
-machines I do not control.  In particular, it should be hard for me to
-lie to them; the software should detect any false statements.
-
-In particular, these should be prevented:
-
- * Claiming to be someone else.
-
- * Attempting to rewrite history.
-
-
-Revocation
-----------
-
-Suppose Alice's code-signing key is stolen by an attacker Charles.
-Charles can sign changesets purporting to come from Alice.  
-
-Alice needs to revoke that key; hopefully she has saved a copy of the
-key elsewhere and can use that to revoke it.  Failing that she can
-mail everyone and ask them to delete it.  This can propagate through
-the usual GPG mechanism, which is very nice.
-
-Alice also needs to make a new key and get it trusted.
-
-This revocation does not distinguish between changesets genuinely
-signed by Alice in the past, and changesets fraudulently signed by
-Charles. 
-
-What can Alice do now?  First of all, she needs to work out what
-changesets signed by her key can still be trusted.  One good way to do
-this is to check against another branch signed by Bob.  If Bob's key
-is safe, we know his copy of Alice's changesets are OK and the full
-tree at various points is OK.
-
-Then:
-
- * Go through her old changesets, check that they're OK -- perhaps
-   restore from a trusted backup.  Re-sign those changesets with a new
-   key bound to the same email address.  Publish the new signatures
-   instead.
-
-   (This seems to indicate it is a good idea to bind signatures to
-   changeset by author name/address rather than by key ID.)
-
- * Roll-up all previous development into a new tree, then sign that.
-   This means there is no safe access to the previous individual
-   changes, but in some cases it may be OK.
-
-If a key is revoked at a particular time then perhaps we could still
-trust commits made before that time.  I don't know if GPG revocations
-can support that.
-
-
-Old keys
---------
-
-Keys also expire, rather than being revoked.  What does this mean?
-
-Ideally we would check that the date when a changeset claims to have
-been signed is within the validity period of the key.  This requires
-more GPG integration than may at the moment be possible, but in theory
-we can do it.
-
-Also need to make sure that commits are in order by date, or at least
-reasonably close to being in order (to allow for some clock skew).
-
-One interesting case is when version is committed for which both the
-public and private keys have been lost.  This will always be
-untrusted, but that should not prevent people continuing to use the
-archive if they can accept that.
-
-This suggests that perhaps we should allow for multiple signatures on
-a single revision.
-
-
-Encumbrance attacks
--------------------
-
-A special case where we need to be able to destroy history to avoid a
-legal problem.  Allowed as discussed elsewhere: either destroy commits
-from the tail backwards, or equivalently branch from a previous
-revision and replace with that.
-
-People who saw the original branch can still prove it happened; people
-who look in the future will not see any record.  
-
-Either way, probably requires physical branch access.
-
-
-Multiple signature keys
------------------------
-
-Should we allow for several signatures on a single changeset?  What
-would that mean?  How do we know what signatures are meaningful or
-worthwhile?
-
-
-Forensics
----------
-
-dwheeler:
-
-   [O]nce you find out who did a malicious act, the SCM should make it
-   easy to identify all of their actions. In short, if you make it
-   easy to catch someone, you increase the attackers' risk... and that
-   means the attacker is less likely to do it.
-
-dwheeler asks that the committer's IP address be recorded.  Putting
-this in the changeset seems to cause too much of a
-privacy/confidentiality problem.  However, an active server might
-reasonably record the IPs of all clients.
-
-
-Non-repudiation
----------------
-
-If a changeset has propagated to Bob, signed by Alice's key, then Bob
-can prove that someone possessing Alice's key signed it.  Alice's only
-way out is to claim her key was stolen.
-
-
-Trusted review
---------------
-
-Can be handled by importing onto another branch.  Can have various
-levels for "quickly checked", "deeply trusted", etc.
-
-(Is it really necessary to import onto a new branch rather than add
-anotations to existing branches?  Copying the whole text seems a bit
-redundant.  This might be a nice place for arch-style taggings, where
-we just add a reference to another branch.)
-
-
-Hooks
------
-
-Automatically running hooks downloaded from someone else is
-dangerous.  In particular, the user may not have the chance to check
-the hooks are reasonable before they are run.
-
-Conversely, users can subvert client-side hooks.  If we want to run a
-check before accepting code onto a shared branch, that must run on the
-server.
-
-Enforcing server-side checks gives a good way to run build,
-formatting, suspiciousness checks, etc.  This implies that write
-access to a repository is through a mediating daemon rather than by
-directly writing. 
-
-
-
-Signing
--------
-
-We use signing to prove that a particular person (or 'principal',
-possibly a robot) committed a particular changeset.
-
-It is the job of external signing software to help work out whether
-this is true or not.  This has several parts:
-
- * Mathematical verification that a signature on a particular 
-   changeset header document is correct
-
- * Determining that the signature corresponds to a particular public
-   key
-
- * Determining that the public key corresponds to the person claimed
-   to have authored the changeset (identified by email address.)
-
-The second two are really PKI functions, and somewhat harder than the
-first.
-
-The canonical implementation is to use GPG/OpenPGP, but anything will
-do.  There are simpler RSA/DSA implementations which assume each user
-manually builds a list of trusted keys.
-
-This leaves open the question of which people should be trusted to
-provide software on a particular branch or at all.  This is not a very
-easy question for software to answer.  We assume that people will know
-by other means.  For public code, it may be that all changesets are
-re-signed by say samba-team@samba.org.
-
-I think it is fair to distinguish people by an email address, or at
-least by $ID@$DOMAIN.  There is no need to have this actually receive
-email, so spam need not be a problem.
-
-The signing design is inspired by the very usable security afforded by
-OpenSSH: it automatically protects where it can, and allows higher
-security to users who want to do some work (by offline verification of
-signatures).
-
-Using a signing mechanism other than GPG when key developers already
-have GPG and there is a big infrastructure to support it seems
-undesirable.  It is true that GPG is quite complex.
-
-The purpose of signing is to protect against unauthorized modification
-of archives. 
-
-Bazaar-NG can apply a GPG signature to both patches and manifests.  This
-vallows a later proof that the revision and the changeset were produced
-by the author they claim to have been written by.
-
-We cannot cryptographically prove that a particular patch was merged
-into a branch, because the person doing the merge might have subverted
-the patch in the process of merging it.  All we can prove
-cryptographically is that the merge committer asserts they took the
-patch.
-
-GPGME and PyMe seem to give a reasonable interface for doing this:
-there is a function to check a signature, and the return indicates the
-signing name, with possible errors including a missing key, etc.
-
-
-Sign branches, not revisions
-''''''''''''''''''''''''''''
-
-Aaron Bentley suggested the interesting idea of signing the mapping of
-revisions onto branches, rather than revisions themselves.  For
-example a branch could contain just a signed pointer to the most
-recent revision. 
-
-(It probably is useful to be able to check signatures on previous
-revisions, for example when recovering from an intrusion.)
-
-
-Protocol attacks
-----------------
-
-Both client and server should be resistant to malicious changesets,
-network requests, etc.  There's no easy solution.
-
-* Defense in depth.  Check reasonablenes at various points.
-
-* Disallow changesets that try to change files outside of the branch.
-
-
-Availability
-------------
-
-bzr can be configured so as to have no single point of failure to a
-denial-of-service attack (or at least nearly none):
-
-* Can have any number of mirrors of a branch.
-
-* If a central server is taken out, developers can continue working
-  with state they already have (unbind their branches), and can
-  collaborate by email or other means until the server is repaired or
-  replaced.
-
-* The origin branch can be on a machine whose location is secret and
-  which is not directly publicly accessible.
-
-* Branches can be moved between machines or IP addresses without
-  disrupting anything else.
-
-* Branches can be moved around out-of-band, as tarballs over
-  bittorrent, etc.
-
-I think the only possible denial of service attacks are those that aim
-to shut down the entire network, or block communication with
-individual developers, for example by flooding their email address.
-But if those people can get connected through some other means, they
-can continue.

=== removed file 'doc/shared-branches.txt'
--- doc/shared-branches.txt	2005-03-09 04:51:05 +0000
+++ doc/shared-branches.txt	1970-01-01 00:00:00 +0000
@@ -1,137 +0,0 @@
-***************
-Shared branches
-***************
-
-How much can we simulate the single-branch model of svn/cvs?  GNU arch
-does reasonably well, but at the cost of making disconnected/connected
-operation leave ugly merge marks in the project history.
-
-This does seem to be a drawback of the otherwise-good branch/tree
-fusion.
-
-`Greg Hudson`__ makes the very good point that the Linux kernel is
-quite unusual among free software projects in relying on human
-integrators, rather than having a shared branch between the
-committers.  The other case is very small projects (distcc?) where
-there is a single person with commit access.
-
-__ http://web.mit.edu/ghudson/thoughts/bitkeeper.whynot
-
-I think decentralized systems can still be very good, but they need to
-support this model very well: a team of committers, working on a
-single trunk branch, but merging from other branches as well.  The
-Samba team is more typical of a demanding open-source project than the
-kernel is.
-
-People fundamentally want to be able to share a tree and all commit
-into it.  It's fine to support other models (like sending patches to
-an owner/integrator/maintainer), but it is not reasonable to require
-that there be exactly one such person.
-
-arch-pqm is a clever kludge; shouldn't be required.  Setting up email
-on all clients, and scripts to automatically process email on the
-server is certainly possible.  But for some people it may not be easy;
-it is never trivial to debug; and particularly for Windows users it
-may be very hard.  If you don't control your own email server or
-cannot  easily run scripts it will be even more messy.
-
-The model of Bitkeeper, and the default model of Bazaar-NG, is that
-each branch has one owner and exists in one place.  People submit
-their patches to the owner of the branch.  This is a pretty good
-model, but has some limitations.
-
-It is a tough transition from CVS; it requires that people learn a
-whole new way of working as well as a new tool.
-
-If the person who owns the branch is away, should all integration
-work stop?
-
-I'm not totally confident in Darcs; it seems like the shared branch
-can reach a state that is not exactly the same as any of the
-checkouts.  Maybe not?  In other respects it is a good model.
-
-Possible solution: bound branches
----------------------------------
-
-You can make a branch which is a write-through mirror of another
-branch.  They work like CVS checkouts in that you must be connected
-and fully up-to-date to commit.  These can hold as much history as you
-like from the main branch, so you can view history, annotate, make new
-branches when offline.  If you are offline for a long time then make a
-new branch, then integrate it back later.
-
-The effect is similar to systems which have separate working copies
-and storage: we have one branch used primarily as a workarea, and
-another used as storage.  I think that making them both branches
-is possibly cleaner: it expresses the way some state may be held
-locally, allows offline branching from the bound branch, etc.
-
-:: 
-
-    $ bzr get http://foo.net/foo ./foo-main
-    $ cd foo-main 
-    (make some changes)
-    $ bzr commit -m 'Add my neat feature'
-    bzr: error: this branch is not-up-to-date with master 
-        http://foo.net/foo
-    - run 'bzr update'
-    - or detach this branch from its parent to work independently
-    $ bzr update
-    - setting aside local changes
-    - pulling changes from parent
-    - putting back local changes
-    $ bzr commit -m 'Add my neat feature'
-
-The ``get`` command is very similar to ``branch``, but means that
-commits will be written back.
-
-This means that the ``pull`` or ``update`` command must be able to
-work even when there are local changes, by setting them aside.  This
-in turn means that local changes may conflict with remote changes, and
-that has to be resolved -- that is no worse than in pulling in changes
-from elsewhere.
-
-commit to a master repository should fail if the slave's patch history
-is not exactly equal to that of the master -- the slave has diverged.
-If the slave history is a prefix of the master history then an update
-will do it.  Otherwise, they have diverged and the slave gets
-detached.
-
-We can have shared server-side hooks, even for access control, at
-least in principle.
-
-Perhaps ``bind`` is a nicer word than ``slave``.
-
-It is conceptually possible to take an existing branch and bind it to
-another, as long as the history of the first is a prefix of that of
-the second.  One could also unbind a branch.  These should at most be
-advanced commands, because in general it is simpler just to make a new
-branch with the desired properties.
-
-Difficulties
-~~~~~~~~~~~~
-
-It may be a bit hard to get a remote commit exactly right,
-particularly if we want to keep a working copy there.  If working
-copies are optional, then turning it off will keep things simple but
-not totally avoid the problem.
-
-I think this is just an example of a general problem though: there is
-no totally satisfactory way to atomically update a working copy.
-(What happens to cvs or svn if you interrupt while it's doing this?
-Generally you get a broken wc.)  What we can do is create a lock while
-the update is taking place, which can be used to at least detect the
-problem. 
-
-If changes are allowed to the working copy of the master branch then
-they might conflict with what is committed by the slave.  Should those
-changes be merged into the working copy of the parent?  If so, they
-might conflict.  
-
-  (darcs handles this case by inserting conflict markers in the remote
-  file, which seems unsatisfactory to me.)
-
-So I am inclined to say that at the moment of pushing to a master
-branch, the destination should be clean.  If we have explicit edits
-then probably there should be no editable files there.  (We could
-perhaps make all the files read-only for the duration of the update.)

=== removed file 'doc/short-demo.txt'
--- doc/short-demo.txt	2005-03-09 04:51:05 +0000
+++ doc/short-demo.txt	1970-01-01 00:00:00 +0000
@@ -1,152 +0,0 @@
-******************************
-Learn Bazaar in twenty minutes
-******************************
-
-
-:Author:
-  Martin Pool <mbp@sourcefrog.net>
-
-:Date: November 2004, London
-
-
-*Bazaar* is an open source distributed version control system.  A
-version control system helps you develop software (or other documents)
-
-In Bazaar, branches hold a history of their changes.  All working
-copies are branches, and all branches are working copies.
-
-Let's get a copy of someone else's published project::
-
-    $ baz branch http://baz.alice.org/hello-2.6
-
-This creates a directory called ``hello-2.6`` in your current
-directory, containing Alice's *Hello World* 2.6 tree and a history of
-her changes.  We'll leave this directory containing a pristine copy of
-her work for future reference.
-
-We want to add a new feature.  Branching and merging is are cheap and
-easy in Baz, so development of each new feature should be done on a
-new branch -- this lets you merge them in different orders, and leave
-aside new development to fix a bug if you want.  It's good to choose a
-meaningful name for your development branches so that you can keep
-them straight.  So let's make an additional branch, based off our
-local copy of Alice's branch::
-
-    $ baz branch ./hello-2.6 ./hello-2.6-goodbye
-
-This command completed quickly: we didn't need to fetch anything from
-Alice's server to make a new branch.
-
-Now we have a second directory containing another copy of the source,
-with a different name.  Of course both are still the same as Alice's
-source.  Time to hack::
-
-    $ cd hello-2.6-goodbye
-    $ vi hello.c
-
-We add a new and much-desired ``--goodbye`` option, build and test
-it.  Having got to a good state, we think we're ready to commit our
-changes.  Many people find it helpful to review the changes before
-committing them, just to make sure there were no accidental mistakes::
-
-    $ baz diff | less
-
-This shows the changes relative to the version we started with.  If
-that looks OK, we're ready to commit::
-
-    $ baz commit -m 'Add --goodbye option'
-
-This records the changes into this branch.  We can see that they got
-recorded::
-
-    $ baz log | less
-
-This will show our change as the most recent, preceded by all our
-development.  If we only want to see our own changes, we can filter
-the log in various ways::
-
-    $ baz log --author bruce@ 
-    $ baz log --date today 
-
-(That second one might include anything Alice did before we started.)
-
-Since our changes were committed, we don't have anything outstanding,
-as we can see with this command::
-
-    $ baz diff 
-    No changes
-
-Thinking about it a bit more, we decide that it's too messy to mix the
-"Goodbye" implementation in with "Hello world".  We start up our
-editor and split it our into a new separate file, ``bye.c``.  By
-default, Baz assumes unknown files in the working directory are
-scratch files -- the same behaviour as CVS and Subversion.  We need to
-tell Baz that this file should be versioned, and then we can commit::
-
-    $ baz add bye.c
-    $ baz commit -s 'Refactor bye() into separate file'
-
-Of course this commits all our changes as a single atomic
-transaction.  If we wanted, we could tell Baz to build the program
-before committing to help ensure we never get a broken build.
-
-We're ready to publish our improved Hello World, but first we want to
-make sure we've integrated anything Alice has done while we were
-working.  First we update our mirror of Alice's branch::
-
-    $ baz sync ../hello-2.6
-
-This pulls the history of her branch onto our machine; we could
-disconnect from the network at this point and do all the merging work
-without needing to connect to ``alice.org``.   
-
-Now we'll pull Alice's new changes into our branch.  We can use a
-mirror of her branch, or we can go straight to her public branch.  In
-fact, we can merge from different copies of the same branch at
-different times, and Baz will still understand which changesets have
-already been merged.  Either of these commands will work::
-
-    $ baz merge ../hello-2.6
-    $ baz merge http://baz.alice.org/hello-2.6
-
-This pulls down any changesets on Alice's branch that aren't in ours,
-and applies them to our tree.  Since we've made changes in parallel
-with Alice, those changes might conflict.  Baz by default will insert
-CVS-style conflict markers into your branch if that happens, but you
-can also ask it to run a graphical merge tool or to tell emacs to do
-the merge.  Baz will help you merge changes other than file text, such
-as a file being renamed in one branch.  
-
-By running ``baz diff`` we can see the effect of her changes on our
-tree.  By running ``baz log --pending`` we can see a description of
-all of the changes that were pulled in.  Once the changes have been
-reconciled (which will often happen automatically), we can commit this
-to our branch::
-
-    $ baz commit -s 'Merge from Alice'
-
-Baz records that our change incorporates those patches from Alice::
-
-    $ baz log | less
-
-Now that we have the feature we wanted, we can publish our branch on
-the web.  We could copy it up using ``baz branch``, but because Baz
-branches are simply directories we can just rsync it onto a web
-server, even if the web server doesn't have Baz installed::
-
-    $ rsync -av ./hello-2.6-goodbye webserver.com:public_html/
-
-We send an email to Alice asking her to merge from this location.
-Typically she'll want to have a look at those changes before accepting
-them, and she can do that by running ``merge`` then ``diff``, as we
-did before.  If she likes the change, she can merge it, possibly
-applying some fixes in the process.  Next time we merge from her,
-we'll see our changes are in.
-
-
-
-.. Local variables:
-.. mode: rst
-.. compile-command: "rest2html short-demo.txt > short-demo.html"
-.. End:
-

=== removed file 'doc/split-join-files.txt'
--- doc/split-join-files.txt	2005-07-29 19:53:21 +0000
+++ doc/split-join-files.txt	1970-01-01 00:00:00 +0000
@@ -1,59 +0,0 @@
-Split and join files
-====================
-
-Version control systems tend to have trouble when files are split or
-joined.  Here is one possibility for handling that.
-
-
-Splitting files
----------------
-
-This can work like 'svn cp': copy with history::
-
-  % bzr cp a b
-
-This creates a new file b, whose file-history is the same as a, but
-with a different file-id, and present at a different name.
-
-This creates the slightly strange situation that b claims to have text
-versions for revisions where b was not actually present.
-
-
-Joining files
--------------
-
-We might want to also join files in various cases:
-
- * Multiple imports of the same file
-
- * Two source files are being refactored into a single one
-
- * Two people apply an external patch that creates the same file
-
-I'm not aware of any other system that handles this.
-
-  % bzr join a b
-
-a and b must be versioned files.  b is removed from the working tree
-and inventory.  b's file id is added as a secondary id as a, so that
-later merges between them will detect them as being the same.  
-
-b's weave is (???) incorporated into a.  
-
-This means that we have a file which has multiple history paths.  For
-example, asking to compare that file to its state in a previous
-version has to arbitrarily pick one to use.  That's not necessarily
-fatal if we assign one as primary, but it might be a bit confusing.
-Trying to merge from another tree might can't really choose a good
-basis.
-
-
-
-File aliases
-------------
-
-Rather than trying to join files we might just add notes that files
-are also known by a different file-id in other trees.
-
-
-

=== removed file 'doc/supportability.txt'
--- doc/supportability.txt	2005-03-09 04:51:05 +0000
+++ doc/supportability.txt	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-************************
-Bazaar-NG Supportability
-************************
-
-The goal is to be able to diagnose and fix a bug the first time it is
-reported, even if the reporter or developer can't reproduce it.
-
-Done:
-
-* Verbose logs are always written to ``.bzr.log`` in the current
-  directory.  This include all error messages and some trace messages
-  that are not normally displayed.  It also includes backtraces if
-  an exception occurs.
-
-To do:
-
-* The --verbose option should send debug output to stderr.
-
-* Re-read changesets after writing them to a temporary file, but
-  before permanently committing them.  Make sure that they are valid
-  XML (against RELAX-NG schema), that they can be parsed, and that
-  they recreate the right revision.  This should help prevent the
-  archive ever getting corrupted, which is a pain to reverse.

=== removed file 'doc/svk.txt'
--- doc/svk.txt	2005-04-07 02:40:18 +0000
+++ doc/svk.txt	1970-01-01 00:00:00 +0000
@@ -1,32 +0,0 @@
-svk
-===
-
-The strategic strength is that it can trivially and reliably
-interoperate with upstream projects using Subversion.  This tends to
-satisfy people who need disconnected operation, and so to allow
-projects to feel safe about switching to Subversion.
-
-On the other hand it may be a bit flaky in implementation -- when I
-tried it (dec 04), it crashed in confusing ways several times.  And
-certainly Subversion's reputation for reliability is mixed -- some
-people think it's very solid, but I've seen many db crashes at HP.
-
-Being written in Perl on top of Svn bindings may not inspire
-confidence.  robertc says he's worked with the libsvn bindings and
-they're a mess.  Relatively little documentation.  In general a
-feeling of a very tall stack.
-
-There is some fluff about defining multiple repositories, which seems
-like an argument for history-in-branch.
-
-Keeps track of merge arrows to do smart merges.
-
-They follow Perforce in not having any control files in the tree --
-nice in some ways but you must use the right tool to move or delete a
-working area.  (In fact the whole thing seems to be inspired a bit by
-Perforce?)  I think keeping just one dotfile at the top level may be a
-fair compromise.
-
-
-
-(If this is unfair or inaccurate mail me dammit.)
\ No newline at end of file

=== removed file 'doc/switch-in-branch.txt'
--- doc/switch-in-branch.txt	2005-05-03 07:48:35 +0000
+++ doc/switch-in-branch.txt	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-Trees which are annoying to set up.  Want to switch to a different
-branch without needing to set everything up again.
-
-Uses for switch: keeps around unknown/ignored files and any local
-changes.

=== removed file 'doc/tagging.txt'
--- doc/tagging.txt	2005-05-13 00:57:32 +0000
+++ doc/tagging.txt	1970-01-01 00:00:00 +0000
@@ -1,107 +0,0 @@
-*******
-Tagging
-*******
-
-
-It is useful to be able to point to particular revisions on a branch
-by symbolic names, rather than revision numbers or hashes.
-
-
-Proposal: just use branches
----------------------------
-
-This is probably the simplest model.
-
-In both Subversion and Arch, tags are basically just branches that do
-not differ from their parent revision.  This has a few advantages:
-
-* We do not need to introduce another type of entity, which can
-  simplify both understanding and implementation.  We also do not need
-  a special 
-
-* If branches are cheap (through shared storage, etc) then tags are
-  also appropriately cheap.
-
-* The general idea is familiar to people from those two systems (and
-  probably some others).
-
-* People often do put tags in the wrong place, and need to update
-  them, or want tags that move along from one state to another
-  (e.g. ``STABLE``.)  Tags-as-branches capture this through the usual
-  versioning mechanism, rather than needing to invent a parallel
-  versioning system for tags.
-
-There are some problems:
-
-* Users want tags to stay fixed, but branches are things that can be
-  committed-to.  So at one level there is a mismatch.
-
-* In particular, it can be easy to accidentally commit onto a tag
-  branch rather than a "branch branch", and that commit can easily be
-  'lost' and never integrated back to the right place.
-
-One possible resolution is to have a voluntary write-protect bit on a
-branch, which prevents accidental updates.  (Similar to the unix owner
-write bit.)  When it is necessary to update the branch, that bit can
-be removed or bypassed.
-
-This is an alternative to the Arch ``--seal`` mechanism.
-
-
-
-Proposal: tags within a branch
-------------------------------
-
-You can place tags on a branch as shorthand for a particular
-revision::
-
-  bzr tag rel3.14.18
-
-  bzr branch foo-project--main@rel3.14.18
-
-Tags are alphanumeric identifiers that do not begin with a digit.
-Tags will cover an entire revision, not particular files.
-
-Another term sometimes used is "labels"; I think we're close enough to
-CVS's "tags" that it's good to be consistent.  However, it does
-possibly clash with Arch's ``tag`` command and ``id-tagging-method``
-(sheesh).
-
-In Subversion a tag is just a branch you don't commit to.  You *can*
-work this way in Bazaar if you want to.  (And until tags are
-implemented, this will be the way to do it.)
-
-I'm not sure if tags should be versioned objects or not.  Options:
-
-* Tags are added or changed by a commit; they mark previous revisions
-  but are only visible when looking from a later commit.
-
-  This has the fairly strange effect that if you check out a previous
-  tag, you can't see the tag anymore (or you see the value it
-  previously had.)
-
-* Tags are not versioned; if you move them they're gone.
-
-* Tags exist within their own versioning space.
-
-It is useful to have mutable tags, in case they're incorrectly placed
-or need to be updated.  At the same time we do not want to lose
-history.  I think in this model it is not helpful to update tags
-within revisions.  One approach would be to version tags within a
-separate namespace, so 
-
-  | STABLE.0
-  | STABLE.1
-  | STABLE.2
-
-as just STABLE it finds the most recent tag.
-
-Should tags propagate across merges, and if so how?
-
-
-
-Implementation Plan
--------------------
-
-Add tags as non-versioned objects, that for the moment do not
-propagate across branches.

=== removed file 'doc/taxonomy.txt'
--- doc/taxonomy.txt	2005-03-09 04:51:05 +0000
+++ doc/taxonomy.txt	1970-01-01 00:00:00 +0000
@@ -1,64 +0,0 @@
-Taxonomy of version control systems
-===================================
-
-(ie a look at "cutting questions" that describe different design
-approaches.)
-
-
-History in working copy
------------------------
-
-Is history stored with working directory (bk, rcs, darcs), or
-separately (cvs, baz)?
-
-Storing in the working directory may be simpler in some respects;
-it's one less concept people need to learn.  On the other hand
-people typically have more working copies than branches, and this
-approach means having more copies of the history.
-
-Naively, storing everything in the working copy can mean that you
-keep multiple copies of the history, which would use a lot of disk,
-or 
-
-Rewriting history
------------------
-
-Can you change recorded history (darcs); only do this through a
-special mechanism (cvs, svn); or not do it at all?
-
-Being absolutely able to reproduce any point in time reliably is
-highly attractive.  On the other hand, sometimes people commit
-something (e.g. confidential information) that really should not be
-recorded.  And at a smaller level people might just commit the wrong
-message.
-
-
-Explicit edits
---------------
-
-Can you edit any file in a working copy (cvs, tla) or must you
-specially mark the before editing (rcs, bk)?
-
-There are two reasons to mark files for editing.  One is so that
-people can be notified that a file is about to
-
-
-Track history of patches
-------------------------
-
-Can you see how a patch got to its current destination?  As far as I
-know only arch can do this -- you can see what branches it
-incorporates changes from.  Well, kind of; not in an entirely
-straightforward manner.
-
-
-
-Collapsing patches
-------------------
-
-When patches from other people are integrated, can they be rolled up
-into larger patches (arch) or do they retain their individual identity
-(bk, darcs)?
-
-
-

=== removed file 'doc/thanks.txt'
--- doc/thanks.txt	2005-03-19 01:41:44 +0000
+++ doc/thanks.txt	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-******
-Thanks
-******
-
-Inspiration, suggestions, advice, bug reports
----------------------------------------------
-
-* Mark Shuttleworth
-* Robert Collins
-* David Allouche
-* Robert Weir
-* James Blackwell
-* Garrett Rooney
-* Andrew Tridgell
-* Paul Russell
-* Aaron Bentley
-* Faheem Mitha
-* Luke Gorrie
-* Dan Nicolaescu
-* Tim Parkin
-* Scott James Remnant
-
-Sponsor
--------
-
-* Canonical_
-
-.. _Canonical: http://www.canonical.com/
\ No newline at end of file

=== removed file 'doc/todo-from-arch.txt'
--- doc/todo-from-arch.txt	2005-04-15 01:31:21 +0000
+++ doc/todo-from-arch.txt	1970-01-01 00:00:00 +0000
@@ -1,372 +0,0 @@
-***************************************** 
-Opportunities for improvement on GNU Arch
-***************************************** 
-
-
-Bazaar-NG is based on the GNU Arch system, and inherits a lot of its
-design from Arch.  However, there are several things we will change in
-Baz to (we hope) improve the user experience.
-
-The core design of Arch is good, brilliant even.  It can scale from
-small projects too large ones, and is a good foundation for building
-tools on top.  However, the design is far too complex, both in
-concepts and execution.  So the plan is to cut out as many things as
-we can, add a few other good concepts from other systems, and try to
-make it into a whole that is consistent and understandable.
-
-
-Good bits to keep
------------------
-
-* Roll-up changesets
-
-  No other system is able to express this valuable idea: "I merged all
-  these changes from other people; here is the result."
-
-  However, it should *also* be possible to bring in perfect-fit
-  patches without creating a new commit.
-
-* Star-merge
-
-  Find a common ancestor on diverged and cross-merged branches.
-
-* Apply isolated changesets.
-
-  We should extend this by having a good way to send changesets by
-  email, preferably readable even by people who are not using Arch.
-
-* GPG signing of commits.
-
-  Open source hackers almost all have GPG keys already, and GPG deals
-  with a lot of PKI functions to do with propagating, signing and
-  revoking keys.
-
-  Signed commits are interesting in many ways, not least of which in
-  detecting intrusion to code servers.
-
-* Anonymous downloads can be done without an active server.
-
-  Good for security; also very good for people who do not have a
-  permnanently-connected machine on which they can install their own
-  software, or which is very tightly secured.
-
-  It's neat that you can upload over only sftp/ftp, but I'm not sure
-  it's really worth the hassle; getting properly atomic operations
-  over remote-file protocols is hard.
-
-* Clean and transparent storage format.
-
-  This is a neat hack, and gives people assurance that they can get
-  their data back out again even if the tool disappears.  Very nice.
-  (Bazaar-NG won't keep the exact same format, but the ideas will be
-  similar.) 
-
-* Relatively easily parseable/scriptable shell interface.  Good for
-  people writing web/emacs/editor/IDE interfaces, or scripts based it.
-
-* Automatically build (and hardlink) revision libraries, with
-  consistency checks.
-
-  I don't know how many people want *every* revision in a library, but
-  it can be handy to have a few key ones.
-
-  In general making use of hardlinks when they are available and safe
-  is nice.
-
-* Rely on ssh for remote access, authentication, and confidentiality.
-
-* Patch headers separate from patch bodies.  (Sometimes you only want
-  one.)
-
-* Autogeneration of Changelogs -- but should be in GNU format, at
-  least optionally.  I'm not convinced auto-updating them in the tree
-  is worthwhile; it makes merges weird.
-
-* Sealing branches.
-
-  It seems useful to prevent accidental commits to things that are
-  meant to be stable.  However, the set-once nature of sealing is
-  undesirable, because people can make mistakes or want to seal more
-  than once.
-
-  One possibility is to have a voluntary write-protect flag set on
-  branches that should not normally be updated.  One can remove the
-  flag if it turns out it was set wrongly.
- 
-* ``resolved`` command in Bazaar-1.1
-
-  Good for preventing accidental breakage.
-
-* Multi-level undo -- though could perhaps be more understandable,
-  perhaps through ``undo-history``.
-
-
-Bits to cut out
----------------
-
-One lesson from usability design is that it does not always work to
-have a complex model and then try to hide complexity in the user
-interface.  If you want something to be a joy to use, that must be
-designed in from the bottom up.
-
-  (Some developers may react to tla by thinking "eww, how gross" on
-  particular points.  As much as possible we might like to fix these.)
-
-* General impression that the tool is telling you how to run your life.
-
-* Non-standard terminology
-
-  Arch uses terms like "version" and "category" in ways that are
-  confusing to people accustomed to other version control systems.
-  This is not helpful.
-
-  Therefore: development proceeds on a *branch*, which is a series of
-  *revisions*.  Simple and obvious.
-
-* Too many commands.
-
-* Command-line options are wierdly inconsistent with both other
-  systems, with each others, and with what people would like to do.
-  For example, I would think the obvious usage is ``bzr diff [FILE]``,
-  but ``tla diff`` does not let you specify a file at all.
-
-  Most commands should take filenames as their argument: log, diff,
-  add, commit, etc.
-
-* Despite having too many commands, there are massive and glaring
-  gaps, such reverting a single file or a tree.
-
-* Commands are too different from what people are used to in CVS, and
-  often not for a good reason.
-
-* Identifiers are too long.  In part this is because Arch tries to
-  have identifiers which are both human-assigned and universally unique. 
-
-* Archive names are probably unnecessary.
-
-* Part of the reason for complexity in archives is that the Arch
-  design wants to be able to go and find patches on other branches at
-  a later time.  (This is not really implemented or used at the
-  moment.)
-
-  I think the complexity is unjustified: changesets and revisions have
-  universally unique names so they can simply be archived, either on
-  the machine of the person who wants them or on a central site like
-  supermirror.
-
-* The tool is *unforgiving*; if people create a branch with the wrong
-  name it will be around forever.
-
-* Branches are heaviweight; a record always persists in the archive.
-  Sometimes it is good to create micro-branches, try something out,
-  and then discard them.  If nobody wants the changes, there is no
-  reason for the tool to keep them.
-
-* Working offline requires creating a new branch and merging back and
-  forth.  This is both more work than it should be, and also polutes
-  the "story" told by branching.
-
-  As much as possible, the *accidental* difference of the location of
-  the repository should not effect the *semantics* of branches.
-  
-  (However, some merging may obviously be necessary when there is
-  divergence.) 
- 
-* Archive registration.  This causes confusion and is unnecessary.
-
-  Proposed solutions such as archive aliases or an additional command
-  to register-and-get make it worse.
-
-* Wierd file names (``++`` and ``,,``, which persist in user
-  directories and cause breakage of many tools.  Gives a bad
-  impression, and it's even worse when people have to interact with
-  them.
-
-* Overly-long identifiers.  (One advantage of pointing to branches
-  using filenames or URLs is that the length of the path depends on
-  how close it is to the users location, and they can more easily use 
-
-* Too slow by default.
-
-  Arch can be made fast, but in the hands of a nonexpert user it is
-  often slow.  For most users, disk is cheaper than CPU time, which is
-  cheaper than network roundtrips.  The performance model should be
-  transparent -- users should not be surprised that something is slow.
-
-* Tagging onto branches.
-
-  Unifying tags and commits is interesting, but the result is hard to
-  mentally model; even Arch maintainers can't say exactly how it is
-  supposed to work in some cases.
-
-* Reinventing the world from scratch in libhackerlab/frob/pika/xl.
-
-  Those are all fine projects and may be useful in the future, but
-  they are totally unnecessary to write a great version control
-  system.  It is not an enormous project; it is not CPU-cycle
-  critical; something like Python will be fine.
-
-* Lack (for the moment) of an active server.
-
-  Given that network traffic is the most expensive thing, we can
-  possibly get a better solution by having intelligence on both sides
-  of the link.  Suppose we want to get just one file from a previous
-  revision...
-
-* Poor Windows/Mac support.
-
-  Even though many developers only work on Linux, this still holds a
-  tool back.  The reason is this: at least some projects have some
-  developers on Windows some of the time.  Those projects can't switch
-  to Arch.  Most people want to only learn one tool deeply, so it
-  won't be Arch.
-
-  Don't make any overly Unixy assumptions.  Avoid too-cute filesystem
-  dependencies.
-
-  Being in Python should help with portability: people do need to
-  install it, but many developers will already have it and the total
-  burden is possibly less than that of installing C requisite
-  libraries.
-
-* Quirky filename support.
-
-  Files with non-ascii names, or names containing whitespace tend to
-  be handled poorly, perhaps partly because of arch's shell heritage.
-
-  By swallowing XML we do at least get automatic quoting of wierd
-  strings, and we will always use UTF-8 for internal storage.
-
-* Complex file-id-tagging 
-
-  Nobody should be expected to understand this.  There are two basic
-  cases: people want to auto-add everything, and want to add by hand.
-  Both can be reasonably accomodated in a simpler system.
-
-* Complex naming-convention regexps in ``.arch-inventory`` and 
-  ``{arch}/id-tagging-method``.  (The fact that there are two
-  overlapping mechanisms with very different names is also bad.)
-
-  All this complexity basically just comes down to versioned, ignored,
-  unknown, the same as in every other system.  So we might as well
-  just have that.
-
-  There are relatively few cases where regexps help more than globs,
-  and people do find them more complex.  Even experienced users can
-  forget to escape ``\.``.  We can have a bit of flexibility with
-  (say) zsh-style extended globs like ``*.(pyo|pyc)``.
-
-* Some files inside ``{arch}`` are meant to be edited by the user, and
-  some are not.  This is a flaw common to other systems, including
-  Bitkeeper.  The user should be clear on whether they should touch
-  things in a directory or not.
-
-* Source-librarian function works poorly.
-
-  It is not the place of a tool to force people to stay organized; it
-  should just facilitate it.  In any case, a library without
-  descriptive text is of little use.  So bazaar-ng does not force
-  three-level naming but rather lets people arrange their own trees,
-  and put on their own descriptions (either within the tree, or by
-  e.g. having a wiki page listing branches, descriptions and URLs.)
-
-* Whining about inode mismatches on pristines/revlibs.
-
-  It's fine that there is validation, but the tool should not show off
-  its limitations.  Just do the right thing.
-
-* More generally, not quite enough consistency/safety checking.
-
-* Unclear what commands work on subdirs and what works on the whole
-  tree.
-
-* Hard to share work on a single branch -- though still not really too
-  bad.
-
-* Lack of partial commits of added/deleted files.
-
-* Separate id tags for each file; simple implementation but probably
-  costs too much disk space.
-
-* Way too many deeply-nested directories; should be just one.
-
-* ``.listing`` files are ugly and a point of failure.  They can cause
-  trouble on some servers which limit access to dot files.
-
-  Isn't it possible to have the top-level file be predictable and find
-  everything else needed from there?
-
-* Summary separate from log message.
-
-  Simpler to just have one message, and let people extract the first
-  line/sentence if they wish.
-
-  Rather than 'keywords', let arbitrary properties be attached to the
-  revision at the time of commit.
-
-
-
-Simpler disconnected operation
-------------------------------
-
-A basic distributed VCS operation is to make it easy to work on an
-offline laptop.  Arch can do this in a few ways, but none of them are
-really simple.
-
-http://wiki.gnuarch.org/moin.cgi/mini_5fTravellingOftenWithArch
-
-Yaron Minsky writes (2005-01-18):
-
-    I was wondering what people considered to be a good setup for using
-    Arch on a laptop.  Here's the basic situation.  I have a few projects
-    that reside in arch repositories on my desktop computer.    Basically,
-    I'd like to be able to do commits from my laptop, and have those
-    commits eventually migrate up to the main repository.  I understand
-    that the right way of doing this is to set up archives on the laptop. 
-    But what's the cleanest way of doing this?  And is there some way of
-    making the commits I do on the laptop show up cleanly and individually
-    on the desktop once they are merged in?
-
-
-Tagging-method
---------------
-
-baz default is much less strict.  
-
-Much of tla depends on being able to categorize files.  Some hangovers
-from larch -- eg precious and backup are essentially the same.  junk
-is never deleted today.  
-
-Automatic version control with 'untagged-source source'.  But this is
-deprecated for baz?
-
-Annoyed by
-
- - defaults
- - having the feature at all
- - complex way to define it
-
-Default of 166 lines.
-
-Remove id-tagging-method command or at most make it read-only.  If
-people really want to use deprecated methods they can just edit the
-file.
-
-So we can ship a default id-tagging which works the same as CVS/Svn:
-give warnings for files that are not known to be junk.  This is the
-default in baz right now.
-
-Also we have .arch-inventory, which is per-directory.
-
-
-
-Why not have 'baz ignore FILENAME'?  To remove ignores, perhaps you
-have to edit the .arch-inventory.  Print "FILTER added to
-PATH/.arch-inventory"; create and baz-add this file if it doesn't.
-
-Docs should perhaps emphasize .arch-inventory as the basic method and
-only mention =tagging-method as an advanced topic.
-
-
-
-Should this really be regexps, or just file globs?
\ No newline at end of file

=== removed file 'doc/unchanged.txt'
--- doc/unchanged.txt	2005-03-09 04:51:05 +0000
+++ doc/unchanged.txt	1970-01-01 00:00:00 +0000
@@ -1,102 +0,0 @@
-Detecting unchanged files
-*************************
-
-Many operations need to know which working files have changed compared
-to the basis revision.  (We also sometimes want to know which files
-have changed between two revisions, but since we know the text-ids
-and hashes that is much easier.)  
-
-The simplest way is to just directly compare the files.  This is
-simple and reliable, but has the disadvantage that we need to read in
-both files.  For a large tree like the kernel or even samba, this can
-use a lot of cache memory and/or be slow.  Some people have machines
-that do not have enough memory to hold even one copy of the tree at a
-time, and this would use two copies.
-
-So it is nice to know which files have not changed without actually
-reading them.  Possibilities:
-
-* Make the working files be hardlinks to the file store.  Easy to see
-  if they are still the same file or not by simply stat'ing them.  For
-  extra protection, make the stored files readonly.  Has the
-  additional advantage of reducing the disk usage of the working copy.
-
-  Disadvantage is that some people have editors that do not handle
-  this safely.  In that case the changes will go undetected, and they
-  could corrupt history.  Pretty bad.
-
-  We can provide a ``bzr edit`` command that breaks the link and makes
-  the working copy writable.
-
-* As above, but link to a temporary pristine directory, not to the
-  real store.  They can get a wrong answer, but at least cannot
-  corrupt the store.
-
-* Check the mtime and size of the file; compare them to those of the
-  previous stored version.  
-
-  The mtime doesn't need to be the time the previous revision was
-  committed.
-
-  There is a possibility of a race here where the file is modified but
-  does not change size, all in the second after the checkout.  Many
-  filesystems don't report sub-second modification times, but Linux
-  now allows for it and it may be supported in future.
-
-* Read in all the working files, but first compare them to the
-  size and text-hash of the previous file revision; only do the diff
-  if they have actually changed.  Means only reading one tree, not
-  two, but we still have to scan all the source.
-
-* Copy the file, but use an explicit edit command to remember which
-  ones have been changed.  Uneditable files should be readonly to
-  prevent inadvertent changes.
-
-The problem with almost all of these models is that it is possible for
-people to change a file without tripping them.  The only way to make
-this perfectly safe is to actually compare.  So perhaps there should
-be a paranoia option.  
-
-It is crucial that no failure can lose history.  Does that mean
-hardlinks directly into the file store are just too risky?
-
-It is most important that ``diff``, ``status`` and similar things be
-fast, because they are invoked often.  It may be that the ``commit``
-command can tolerate being somewhat slower -- but then it would be
-confusing if ``commit`` saw something different to what ``diff`` does,
-so they should be the same.
-
-For the mooted `edit command`__, we will know whether a file is
-checked out as r/o or r/w; if a file is known to be read-only it can
-be assumed to be unmodified.
-
-__ optional-edit.html
-
-The ``check`` command can make sure that any files that would be
-assumed to be unmodified are actually unmodified.
-
-File times can skew, particularly on network filesystems.  We should
-not make any assumptions that mtimes are the same as the system clock
-at the time of creation (and that would probably be racy anyhow).
-
-Proposal
---------
-
-::
-  
-    if file is not editable:
-        unmodified
-
-    if not paranoid:
-        if same inum and device as basis:
-            unmodified
-        elif present as read-only:
-            unmodified
-        elif same mtime and size:
-            unmodified
-
-    read working file, calculate sha1
-    if same size and sha-1 as previous inventory-entry:
-        unmodified
-
-    possibly-modified

=== removed file 'doc/unrelated-merge.txt'
--- doc/unrelated-merge.txt	2005-03-09 04:51:05 +0000
+++ doc/unrelated-merge.txt	1970-01-01 00:00:00 +0000
@@ -1,8 +0,0 @@
-****************************
-Merging unrelated changesets
-****************************
-
-It ought to be possible to merge in even totally unrelated changesets,
-by just treating them as patches.  It is possible that the file ids
-will mismatch, in which case we could perhaps assume the filenames are
-compatible. 
\ No newline at end of file

=== removed file 'doc/usability.txt'
--- doc/usability.txt	2005-03-09 04:51:05 +0000
+++ doc/usability.txt	1970-01-01 00:00:00 +0000
@@ -1,4 +0,0 @@
-* http://www.asktog.com/basics/firstPrinciples.html
-
-* http://wiki.gnuarch.org/moin.cgi/BazaarMockupUI
-

=== removed file 'doc/use-cases.txt'
--- doc/use-cases.txt	2005-03-09 04:51:05 +0000
+++ doc/use-cases.txt	1970-01-01 00:00:00 +0000
@@ -1,132 +0,0 @@
-Bazaar-NG Use Cases
-*******************
-
-.. contents::
-
-
-
-Review changes 
-==============
-
-Look at somebody else's tree or email submissions.  
-
-Looking at the patch alone may not be enough; we might need to apply
-it to a tree, build it and see if we like it.
-
-
-
-
-Changes on branches
-===================
-
-Clearcase allows you to put all new development onto branches that are
-later merged back.
-
-Can we detect which development branches have unmerged changes?  Can
-we dispose of those branches?
-
-
-
-unmerge
-=======
-
-Get rid of any changes that have been merged in but not yet
-committed.  Shouldn't this just be ``bzr revert``?
-
-
-
-cross damage problem with PQM 
-=============================
-
-Lock 
-
-
-
-undo some uncommitted changes
-=============================
-
-If you've made some changes and don't want them::
-
-  baz undo foo.c
-
-This stores them as a changeset in a directory that you can move
-around.  You can set a name for it::
-
-  baz undo --name blargh-refactor foo.c bar.c
-
-You can get it back::
-
-  baz redo foo.c
-
-
-move some in-progress changes onto a local branch
-=================================================
-
-This is useful if we decide some changes on a bound branch should be
-done on a separate branch; in particular people will want to do this
-if they want to work in only one subdirectory of a complex config.
-
-Possibly this should be the default with no arguments for ``bzr
-branch``.  Or possibly there should be a separate ``bzr unbind``.
-
-
-
-ignore some files
-=================
-
-I'm working on a Python project, which leaves bytecode files in the
-working directory::
-
-  baz ignore \*.pyc
-  baz ignore \*.pyo
-
-or::
-
-  baz ignore '*.py[co]'
-
-OK, there is some danger here that people always forget to quote globs
-on the command line but maybe this will be enough.
-
-Maybe take only one at a time so that we can catch unquoted globs like
-this::
- 
-  baz ignore *.pyc                      # wrong!
-
-If they do this, they see all '* added foo.pyc to .arch-inventory';
-then they can do this to get back::
-
-  baz undo .arch-inventory
-
-This is potentially much more pleasant than Subversion.
-
-
-
-Wrong commit message
-====================
-
-I accidentally commit some files with the wrong message and want to
-change them::
-
-   % bzr status
-   M    foo.c
-   % bzr commit -s 'fix foo'
-   M    foo.c
-   [oops!]
-   % bzr uncommit
-   % bzr status
-   M    foo.c
-   % bzr commit -s 'fix foo and bar'
-
-This fix should be done as soon as possible, before anything else
-depends on the change.
-   
-
-
-Monday morning
-==============
-
-Come in Monday morning; can't remember what you were doing.
-
-* log; look at what was committed
-* diff against upstream, or recent revisions
-* what else?

=== removed file 'doc/web-interface.txt'
--- doc/web-interface.txt	2005-03-09 04:51:05 +0000
+++ doc/web-interface.txt	1970-01-01 00:00:00 +0000
@@ -1,24 +0,0 @@
-*************
-Web interface
-*************
-
-The web interface may be the first way many people interact with a new
-VC system.  It may not be core functionality but it is important.
-
-mpe says he likes the bkbits.net one, in particular that you can
-quickly search the comments and pull out individual patches.  However,
-it does not really give people a good sense of how the tool works.
-
-The `Aegis web interface`__ is rather better, giving a good sense (for
-better or worse) of the ideas behind the tool, as well as the
-particular tree being viewed.  For example you can see that all
-changes in progress are registered, that one can identify conflicts
-before a merge is attempted and so on.  In part this is because there
-are many links ("Information available") and they are explained in
-some detail:
-
-  "This item will provide you with a list of changes to this project,
-  including those which have not yet commenced, those in progress, and
-  those which have been completed."
-
-__ http://aegis.sourceforge.net/webiface.html
\ No newline at end of file

=== removed file 'doc/workflow.txt'
--- doc/workflow.txt	2005-03-09 04:51:05 +0000
+++ doc/workflow.txt	1970-01-01 00:00:00 +0000
@@ -1,117 +0,0 @@
-********
-Workflow
-********
-
-People want to manage submissions of patches; see `Havoc's post`__.
-
-__ http://log.ometer.com/2004-11.html#19
-
-The ideal interface is::
-
-    baz branch http://project.org/bzr/project-2.0
-    [hackety hack]
-    baz submit
-
-It would be nice if people could submit simple changes without needing
-to set up their own public branches.  I don't think people will want
-to allow random strangers to create branches on their machine, so this
-probably means submitting the changes by email.  To form such a patch
-we need to know what branch point counts as "upstream", and who to
-send the patches to.  As a reasonable default, we might use the last
-time we branched and the last 
-
-Some of this should be better done in integration with e.g. mail
-clients or external robots or bug trackers. 
-
-Bazaar-NG allows redrafting rejected patches in an interesting way: 
-
-  Person writes a feature on a new feature branch.  They can commit
-  several times, merge up to date, even have sub-forks to an arbitrary
-  extent.  When they're ready, they submit their changes to the
-  maintainer, either by mailing the diffs relative to the main branch,
-  or by the maintainer pulling from their tree.   If they don't like
-  it, they can not commit it to their tree, and hopefully give some
-  feedback to the contributor.  (I don't think that feedback needs to
-  go back through the tool; email or some other communication
-  mechanism is probably fine.)  The contributor can then keep working
-  in their branch, until it eventually gets merged.
-
-  Alternatively, the maintainer might want to merge the change but fix
-  it up themselves.  We keep track of the fact that it was merged, and
-  the maintainer can make arbitrary fixups either in the course of
-  merging it or afterwards.  When the contributor later merges back
-  everything will work.
-
-  Another case is that the maintainer wants to improve the patch but
-  not take it into their main tree.  What they can do here is take it
-  into a separate feature branch, fix it up, and then ask the
-  contributor to merge back from there.
-
-Maintainers__ would also like to keep track of patches that have been
-submitted but not yet accepted, so they're not lost and can be
-updated.  One way to do this would be to create a branch on the
-maintainer's machine for a submitted patch, and apply the submission
-to that.  The maintainer can fix it if they want, or take updates to
-it.  The submitter can see what, if anything, was done.  Because this
-branch is identified by a URL it can be cited in bug reports, and it
-might make sense to name the branch by the bug it is supposed to fix.
-
-__ http://gcc.gnu.org/ml/gcc/2002-12/msg00444.html
-
-
-
-Aegis-style review and integration
-----------------------------------
-
-Some projects might want all changes to be submitted for review before
-merging onto the mainline.  This might be done either by convention,
-or perhaps by not allowing individual developers to merge to the
-mainline but rather having specific privileged integrators.
-
-Aegis_ enforces a lot of workflow/process; it would be good to be able
-to do something similar on top of bazaar-ng either manually or as a
-higher-level tool.  Aegis's model is that each proposed change is
-essentially on a branch that later merges into the mainline, which
-makes a lot of sense.
-
-.. _Aegis: compared-aegis.html
-
-To do something like Aegis, follow this process:
-
-* Developer makes a new branch from the trunk to develop a feature,
-  called say ``project--devel--bug-123``.
-
-* When they have almost finished development, they re-merge from the
-  trunk to make sure they're up to date.
-
-* By some mechanism they ask a reviewer to consider their changes --
-  perhaps by sending email, or using a bug tracking system, or
-  something else.  They tell the reviewer the location of their
-  branch, which might be on an HTTP server for a public project, or on
-  a directory on a shared server.
-
-* The reviewer makes a new branch for review based off the trunk,
-  ``project--review--bug-123``, and merges the development branch into
-  it.  The merge should be perfect if the developer was up to date
-  with the trunk.  If the merge fails they can either bounce it back
-  to the developer or fix the merge themselves according to local
-  policy or their own discretion.  They then review, build and test
-  the branch.  If it's OK, they commit to their review branch and send
-  a note asking for it to be integrated (or perhaps they integrate
-  themselves.)
-
-* The integrator merges from the review branch onto the trunk,
-  builds/tests and commits.  Since they pull from the reviewer's
-  branch there is no way unreviewed changes can sneak through even if
-  the developer adds to their work branch after the review.
-  
-  This can be done by a robot, or by a reviewer.
-
-* The developer can look at the review, integration and trunk branches
-  to see that their changes have merged.
-
-This model is practiced by some people at Canonical using tla.  Since
-people work within a complex configspec, they like very much to be
-able to branch in-place so that they do not need to rebuild the whole
-config to start new development.  (Though perhaps the real fix is to
-make assembling a config simpler...)

=== removed file 'doc/yaml.txt'
--- doc/yaml.txt	2005-03-09 04:51:05 +0000
+++ doc/yaml.txt	1970-01-01 00:00:00 +0000
@@ -1,40 +0,0 @@
-YAML
-****
-
-I am thinking about keeping changeset headers in YAML_, as a nice
-tradeoff:
-
- * Standard data format, so we don't need to invent, write and debug
-   an ad-hoc format.
-
- * More concise than XML, which is useful if we're storing a few.
-
- * Reasonably readable for humans; thus suitable for sending in email
-   and giving people at least some chance to read and understand it.
-   Seeing this at the top of a patch would not be actively horrible as
-   long as we keep it reasonably small; that's not true for XML.
-
- * Since YAML documents have a document header and terminator ('---'
-   and '...') we can pick them out of a text document and append
-   text patches directly afterwards.
-
- * Fits well with Python internal data structures.
-
- * There are parsers for various languages, so people can write their
-   own implementation if they really want.
-
- * There is space to specify a document format/version at the top
-   (though this is not currently handled in ydump).  
-
-The ydump module is pretty primitive, but it should have enough.
-
-.. _YAML: http://www.yaml.org/
-
-
-Conclusion
-----------
-
-On the whole, I think this is not a good idea.  It is a nicer syntax
-for XML, but there are less bindings, and it does not achieve the goal
-of having the storage format look reassuring and giving confidence
-that the data could be retrieved by an alternative implementation.
\ No newline at end of file

=== added file 'generate_docs.py'
--- generate_docs.py	1970-01-01 00:00:00 +0000
+++ generate_docs.py	2006-02-13 01:28:03 +0000
@@ -0,0 +1,93 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""%(prog)s - generate information from built-in bzr help
+
+%(prog)s creates a file with information on bzr in one of
+several different output formats:
+
+    man              man page
+    bash_completion  bash completion script
+    ...
+
+Examples: 
+
+    python2.4 generated-docs.py man
+    python2.4 generated-docs.py bash_completion
+
+Run "%(prog)s --help" for the option reference.
+"""
+
+import sys
+from optparse import OptionParser
+
+import tools.doc_generate
+
+def main(argv):
+    parser = OptionParser(usage="%prog [options] OUTPUT_FORMAT")
+
+    parser.add_option("-s", "--show-filename",
+                      action="store_true", dest="show_filename", default=False,
+                      help="print default filename on stdout")
+
+    parser.add_option("-o", "--output", dest="filename", metavar="FILE",
+                      help="write output to FILE")
+
+    parser.add_option("-b", "--bzr-name",
+                      dest="bzr_name", default="bzr", metavar="EXEC_NAME",
+                      help="name of bzr executable")
+
+    parser.add_option("-e", "--examples",
+                      action="callback", callback=print_extended_help,
+                      help="Examples of ways to call generate_doc")
+
+
+    (options, args) = parser.parse_args(argv)
+
+    if len(args) != 2:
+        parser.print_help()
+        sys.exit(1)
+
+    infogen_type = args[1]
+    infogen_mod = tools.doc_generate.get_module(infogen_type)
+
+    if options.filename:
+        outfilename = options.filename
+    else:
+        outfilename = infogen_mod.get_filename(options)
+
+    if outfilename == "-":
+        outfile = sys.stdout
+    else:
+        outfile = open(outfilename,"w")
+
+    if options.show_filename and (outfilename != "-"):
+        print >>sys.stdout, outfilename
+    
+    infogen_mod.infogen(options, outfile)
+
+def print_extended_help(option, opt, value, parser):
+    """ Program help examples
+
+    Prints out the examples stored in the docstring. 
+
+    """
+    print >>sys.stdout, __doc__ % {"prog":sys.argv[0]}
+    sys.exit(0)
+
+if __name__ == '__main__':
+    main(sys.argv)

=== removed file 'notes/inventory-v2-sample.xml'
--- notes/inventory-v2-sample.xml	2005-08-19 21:34:32 +0000
+++ notes/inventory-v2-sample.xml	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-<inventory>
-  <root>
-    <file name="readme.txt" id="readme-123" size="400" sha1="abcdef" 
-          revision="2050808"/>
-    <directory name="src" id="src-123"> 
-      <directory name="libwibble" id="libwibble-123">
-      </directory>
-    </directory>
-  </root>
-</inventory>
\ No newline at end of file

=== removed file 'notes/inventory-v2.rnc'
--- notes/inventory-v2.rnc	2005-08-19 21:34:32 +0000
+++ notes/inventory-v2.rnc	1970-01-01 00:00:00 +0000
@@ -1,30 +0,0 @@
-grammar {
-  file = element file {
-    attribute id { text },
-    attribute name { text }, 
-
-    # the following attributes are absent on the working-inventory
-    attribute sha1 { text }?,
-    attribute size { text }?,
-
-    attribute revision { text }?,
-
-    empty
-  }
-
-  directory = element directory {
-    attribute id { text },
-    attribute name { text },
-    
-    entry*
-  }
-
-  entry = 
-    (file | directory )
-
-  start = element inventory {
-    element root {
-      entry*
-    }
-  }
-}

=== removed file 'notes/new-inventory-sample.xml'
--- notes/new-inventory-sample.xml	2005-03-30 22:27:17 +0000
+++ notes/new-inventory-sample.xml	1970-01-01 00:00:00 +0000
@@ -1,108 +0,0 @@
-<?xml version="1.0"?>
-<inventory version="2">
-  <root_directory id="bogus-root-id">
-    <file id="bzrignore-20050311232317-81f7b71efa2db11a" name=".bzrignore" text_id="bzrignore-20050329080130-e781c2df6eadf795" text_sha1="0c5a363290308bfad3cf7c307dc95bf4e495a39d" text_size="81"/>
-    <file id="NEWS-20050323055033-4e00b5db738777ff" name="NEWS" text_id="NEWS-20050329080130-24324023f7567123" text_sha1="11823c9e5ee9da113c6eeb6baae32097ec9673cc" text_size="1419"/>
-    <file id="README-20050309040720-8f368abf9f346b9d" name="README" text_id="README-20050326030328-7adde2dfd2e0dbbb" text_sha1="b68ae05d135d67a400245ab3aa9806b6a28abe11" text_size="1019"/>
-    <file id="build-api-20050309064830-a0be0dc1452cda87" name="build-api" text_id="build-api-20050309064900-aa74414b35b5e281" text_sha1="450452fd581a1f1ba1fb4bec349f49a8e8a226b3" text_size="75"/>
-    <file id="bzr.py-20050313053754-5485f144c7006fa6" name="bzr" text_id="bzr-20050319081938-21c30193cb7b9e70" text_sha1="afde3aee636065c59d4af92fe035b9d7fbdf6cac" text_size="919"/>
-    <directory id="bzrlib-20050309040749-4ac9a0e211602846" name="bzrlib">
-      <file id="__init__.py-20050309040759-33e65acf91bbcd5d" name="__init__.py" text_id="__init__.py-20050328023935-fe6007f48dde344b" text_sha1="3f4bbd0540a1f6c3dfde11b65e59c11e266d779a" text_size="1485"/>
-      <file id="add.py-20050323030017-3a77d63feda58e33" name="add.py" text_id="add.py-20050327095302-b3edf909f0f81011" text_sha1="53ed9ff67fcc9e1af47d6a4d484fb75b385a6ed4" text_size="2905"/>
-      <file id="branch.py-20050309040759-e4baf4e0d046576e" name="branch.py" text_id="branch.py-20050326134153-f3dff98b574258fd" text_sha1="6a6e9163a74bbdaa3e842019b4cc7258aa80e7ad" text_size="27666"/>
-      <file id="check.py-20050309040759-f3a679400c06bcc1" name="check.py" text_id="check.py-20050329003252-a7262eda3a08f596" text_sha1="21c85347e72f617da56a621e77b30bf0f98a9f05" text_size="4136"/>
-      <file id="bzr.py-20050309040720-d10f4714595cf8c3" name="commands.py" text_id="commands.py-20050329114950-3ea1014fdadfe60e" text_sha1="ec9f6768ec4c729b58af4e08c11b18608a065b21" text_size="24888"/>
-      <file id="diff.py-20050309040759-26944fbbf2ebbf36" name="diff.py" text_id="diff.py-20050309040815-f0112c3b71bc202c" text_sha1="84729acb8afb74b285e635a7c09a38f63b14fee4" text_size="5290"/>
-      <file id="errors.py-20050309040759-20512168c4e14fbd" name="errors.py" text_id="errors.py-20050309040815-75b5f741f053d362" text_sha1="ca6731c62e3d746d1d0a90d9a4ff4c744ac98e83" text_size="1170"/>
-      <file id="info.py-20050323235939-6bbfe7d9700b0b9b" name="info.py" text_id="info.py-20050328024049-e66d2a23d130aa83" text_sha1="e331ce0bc51cfab9aa6eec18d7a1976e384b3a35" text_size="3304"/>
-      <file id="inventory.py-20050309040759-6648b84ca2005b37" name="inventory.py" text_id="inventory.py-20050329072929-15ad4d139ebe9368" text_sha1="87ecb4851198f1b41a397e2f9708376a69edde19" text_size="15054"/>
-      <file id="osutils.py-20050309040759-eeaff12fbf77ac86" name="osutils.py" text_id="osutils.py-20050329023134-4968f3ab1cb80d22" text_sha1="bec87e1bcdfc50cd8b2ee5c4f014d4a3650fb12a" text_size="7405"/>
-      <file id="revision.py-20050309040759-e77802c08f3999d5" name="revision.py" text_id="revision.py-20050312073831-a24ec3db352a0db5" text_sha1="f3013ccb7c45a9bb340328eced6dc4d11d0bc61e" text_size="2830"/>
-      <file id="store.py-20050309040759-164dc5173d6406c2" name="store.py" text_id="store.py-20050329012619-9187f36afe5f02fe" text_sha1="f8fe399336db9b608ec7d866a042c3c84e1f6212" text_size="5285"/>
-      <file id="tests.py-20050309040759-b2da7bf735c06b1f" name="tests.py" text_id="tests.py-20050326134153-0a5950308e43bf54" text_sha1="a1715e90a1a4568b1464a290057435c3ef4fdfaa" text_size="5574"/>
-      <file id="textui.py-20050309040759-5d0cebe09c528e7d" name="textui.py" text_id="textui.py-20050309040815-3605bf769ab4eaf1" text_sha1="a5cb825012e2b2e7797b25139e72d1240675c9de" text_size="1017"/>
-      <file id="trace.py-20050309040759-c8ed824bdcd4748a" name="trace.py" text_id="trace.py-20050328041243-e744e720fe028769" text_sha1="7ca91f974741f05b1ecda92fb2a9b06210e33da8" text_size="3549"/>
-      <file id="tree.py-20050309040759-9d5f2496be663e77" name="tree.py" text_id="tree.py-20050329080130-b62bcff71cd143be" text_sha1="fdf9556da1b2de8b6354aa5bd4c9813123835b96" text_size="13679"/>
-      <file id="xml.py-20050309040759-57d51586fdec365d" name="xml.py" text_id="xml.py-20050315051954-b18360320fd01696" text_sha1="ed94ba5b8af981d007e3ad06083cfe7cc1f1c172" text_size="1556"/>
-    </directory>
-    <directory id="doc-20050309044934-a811c79dd26eef58" name="doc">
-      <file id="Makefile-20050309064830-17b60435388a9520" name="Makefile" text_id="Makefile-20050328022353-1fac836fcad1c3ab" text_sha1="04c5ac50d22caa71223dce1c1723f4927c2eada4" text_size="192"/>
-      <file id="adoption.txt-20050309044946-816486d97a97c5c5" name="adoption.txt" text_id="adoption.txt-20050309045105-1a9c10fb45e803e8" text_sha1="d8a42556a2bf39fdde914ea496761df591e76b56" text_size="4504"/>
-      <file id="bitkeeper.txt-20050309044946-9e207c5bfce2670f" name="bitkeeper.txt" text_id="bitkeeper.txt-20050309045105-fd4be07e5b1e2776" text_sha1="4e33c092641f44a71cd96617e7a20f9e4b6386f6" text_size="898"/>
-      <file id="changelogs.txt-20050309044946-cdc40a9f9ddece03" name="changelogs.txt" text_id="changelogs.txt-20050309045105-f3425da69c90bc3f" text_sha1="650b8344a9e1af3afaa0b5d72b3775f036d9a54a" text_size="1320"/>
-      <file id="cherry-picking.txt-20050309044946-d14f736fdb588e8f" name="cherry-picking.txt" text_id="cherry-picking.txt-20050309045105-3ab30173396c9944" text_sha1="03832e529e9a319b0a2bf5a6157691d2025204aa" text_size="1632"/>
-      <file id="cmdref.txt-20050309044946-d9a444af4c8e2d8b" name="cmdref.txt" text_id="cmdref.txt-20050309045105-af9ab0e5f5d15df0" text_sha1="4b5b544493a96eaa24cda0e41602772591b085f2" text_size="25271"/>
-      <file id="common-format.txt-20050309044946-48fc9f627ac98c0a" name="common-format.txt" text_id="common-format.txt-20050309045105-82969da54b78f851" text_sha1="518e2a235b398bc0aa6e26a9a9bcaa227ce8d5eb" text_size="1092"/>
-      <file id="compared-aegis.txt-20050309044946-2c8b94a645654397" name="compared-aegis.txt" text_id="compared-aegis.txt-20050309045105-70ed007ce0982df5" text_sha1="a0f8e26276a435165d919a4d77eaca4871df88d9" text_size="4275"/>
-      <file id="compared-codeville.txt-20050309044946-1cb208cab910fa50" name="compared-codeville.txt" text_id="compared-codeville.txt-20050312085410-65820cfb547ac6ad" text_sha1="0e5452aae2c7d0dc2530a871df54564d09544bf4" text_size="4145"/>
-      <file id="compared-cvsnt.txt-20050309044946-5396218e1b8351fb" name="compared-cvsnt.txt" text_id="compared-cvsnt.txt-20050309045105-b2dd9d47ab90b632" text_sha1="491675da9b7cb26f222af6dcb2b00406fd06a4e6" text_size="816"/>
-      <file id="compared-opencm.txt-20050309044946-e7ddc908a19c87f5" name="compared-opencm.txt" text_id="compared-opencm.txt-20050309045105-92619d5a43a54851" text_sha1="957b510ff6e162b2b8a9ffd23478059c41ab3889" text_size="1220"/>
-      <file id="compared-prcs.txt-20050309044946-f7c399f458e1fb59" name="compared-prcs.txt" text_id="compared-prcs.txt-20050309045105-61f3870a3561bcae" text_sha1="b126391a028eac75db56b6c2d4a8fc65fd2287f1" text_size="1483"/>
-      <file id="compared-teamware.txt-20050309044946-8d9e164d94ba5a7c" name="compared-teamware.txt" text_id="compared-teamware.txt-20050309045105-1be6ce2995f6fe09" text_sha1="3c50a4d42a10b12d19590a4569a7ae80dbc2b43b" text_size="1123"/>
-      <file id="compression.txt-20050309044946-a73f1e60c40ee57d" name="compression.txt" text_id="compression.txt-20050309045105-033c43c42527006d" text_sha1="a7a0896daff05d98a0b44b29d332113854c25a4c" text_size="2453"/>
-      <file id="config-specs.txt-20050309044946-d9820c58b7f23437" name="config-specs.txt" text_id="config-specs.txt-20050309045105-6cd36c4900778e23" text_sha1="11ce81de2dce0e3e44c854cc607ccfeb009c14a3" text_size="1788"/>
-      <file id="conflicts.txt-20050309044946-b9f2aded67790400" name="conflicts.txt" text_id="conflicts.txt-20050309045105-b625f78a9d1aa3e9" text_sha1="f058f8352787dea8b41111a0a327f04daae76fad" text_size="1387"/>
-      <file id="costs.txt-20050309044946-57b65806cab34815" name="costs.txt" text_id="costs.txt-20050309045105-b164407881870dd5" text_sha1="1e15b93fae5ad627f256f8f4e44dbec7df9171c4" text_size="701"/>
-      <file id="darcs.txt-20050309044946-5dbd16bd2da081a4" name="darcs.txt" text_id="darcs.txt-20050323235210-62ff177fabaa5967" text_sha1="7db04d391bc6eb074ce5c2a705877d006d923190" text_size="2638"/>
-      <file id="deadly-sins.txt-20050309044946-b0453a1c48093054" name="deadly-sins.txt" text_id="deadly-sins.txt-20050309045105-38cb25a50d3b1157" text_sha1="00566c41dc686ce45e8b58c88afb34611b15ef6c" text_size="1585"/>
-      <file id="default.css-20050309064830-2b764767f5f259c8" name="default.css" text_id="default.css-20050309064900-d96676d92f48fd8c" text_sha1="7bcb9fe51bc6c900862cd7567bb8458cb9526644" text_size="3962"/>
-      <file id="design.txt-20050309044946-54973dca2e7aa16b" name="design.txt" text_id="design.txt-20050309045105-88fecfcf589e6fd6" text_sha1="841a4f912b726e7f0f6cb9e6c062e2703bdf6635" text_size="41787"/>
-      <file id="extra-commands.txt-20050309044946-74af949c5805df22" name="extra-commands.txt" text_id="extra-commands.txt-20050309045105-9c07bdbe483793bd" text_sha1="20472f13679beddf9366dbf218bea3cb90d00509" text_size="695"/>
-      <file id="faq.txt-20050309044946-87b18fde82cf414a" name="faq.txt" text_id="faq.txt-20050309045105-adc8655244525a7a" text_sha1="88713c58c3162cbedfd14c57469829aec6041b58" text_size="231"/>
-      <file id="formats.txt-20050309044946-7ea328cdb5cae6c5" name="formats.txt" text_id="formats.txt-20050322000245-4982990c48231f44" text_sha1="55106e6b47f6966dc5b3b72bfc2f3573bd2eb8d0" text_size="8161"/>
-      <file id="hashes.txt-20050309044946-7a7382d0d715151f" name="hashes.txt" text_id="hashes.txt-20050309045105-63587dc15501b6d2" text_sha1="c4d42f8209bec59ef8fb90b730f6d06448e27efe" text_size="2560"/>
-      <file id="ignore.txt-20050328001329-d89006518f61dd00" name="ignore.txt" text_id="ignore.txt-20050328022418-43a7fd05366b06f2" text_sha1="0e45bf3560ed78053ec7b088c11173aa08195ef0" text_size="654"/>
-      <file id="index.txt-20050309044946-be91bf1c15711e5f" name="index.txt" text_id="index.txt-20050328001404-0eab78b497aeade5" text_sha1="00c098809e185488d567226d48c49df498c0cdc1" text_size="5790"/>
-      <file id="interrupted.txt-20050309044946-b6074892ab231e63" name="interrupted.txt" text_id="interrupted.txt-20050309045105-fd981a331efdf680" text_sha1="0ca1b2820e2c99656315211ccd70f5a8a52a2898" text_size="4293"/>
-      <file id="intro.txt-20050309044946-5bb22ffbfa8bfc14" name="intro.txt" text_id="intro.txt-20050309045105-d98d5268094fa375" text_sha1="b97bea4c5baf0a826683fe2ce0b823f8f9a608c2" text_size="3379"/>
-      <file id="inventory.txt-20050309044946-07534ab6d8b3c324" name="inventory.txt" text_id="inventory.txt-20050309045105-fa6637d533fd707e" text_sha1="c4c2fb6973502b1a4b18ab53332cb727c465c340" text_size="312"/>
-      <file id="join-branches.txt-20050309044946-f635037a889c0388" name="join-branches.txt" text_id="join-branches.txt-20050309045105-fea7e1f5b49b41c2" text_sha1="b8148e34e1546e83acf1ed32f4ec4bb4cc3502ae" text_size="3260"/>
-      <file id="kill-version.txt-20050309044946-1b9d175ca38c872c" name="kill-version.txt" text_id="kill-version.txt-20050309045105-ff4fe197ed67a40d" text_sha1="65084715a68dc4a22f60ed46cfa1c81441f14100" text_size="3560"/>
-      <file id="layers.txt-20050309044946-a48f19d7e271ec87" name="layers.txt" text_id="layers.txt-20050309045105-cd42b07df59a768d" text_sha1="34f97443035d3f286a136eaf485169b355483efa" text_size="2867"/>
-      <file id="library-interface.txt-20050309044946-97a60acc594383cc" name="library-interface.txt" text_id="library-interface.txt-20050309045105-27a374e4890d74c9" text_sha1="d5c6c86b8dcbcc0a06f01ddb680466d19e0e2329" text_size="326"/>
-      <file id="merge.txt-20050309044946-5d1ff42572fb3086" name="merge.txt" text_id="merge.txt-20050309045105-495a4a5b8553fe6f" text_sha1="f95f7a04bca711cab1ca528a79ba3174275b95dd" text_size="758"/>
-      <file id="mirroring.txt-20050309044946-eb411f73887829b1" name="mirroring.txt" text_id="mirroring.txt-20050309045105-63ff2af43ce824c7" text_sha1="f9473f6f98e5d139af3489169459b33f238c6d09" text_size="393"/>
-      <file id="monotone.txt-20050309044946-dd2abf24d99d201f" name="monotone.txt" text_id="monotone.txt-20050309045105-8fb6ac30e651a5b8" text_sha1="0b3c1d7d437ceb3b4f9964bae9846b5aa3a69953" text_size="595"/>
-      <file id="news.txt-20050309044946-02b97194f31e86e0" name="news.txt" text_id="news.txt-20050309045105-e3cc2094a8bd32ac" text_sha1="709e86f1545aaf38d2caed6db3b150ac65a72a55" text_size="4434"/>
-      <file id="optional-edit.txt-20050309044946-eddc8e1d07ae10a8" name="optional-edit.txt" text_id="optional-edit.txt-20050309045105-9c7c7616d9ed5c58" text_sha1="c5bbe7b8d2cd7ed53c3a4e989213915a3419654d" text_size="951"/>
-      <file id="partial-commit.txt-20050309044946-76fa39222d4fcc58" name="partial-commit.txt" text_id="partial-commit.txt-20050309045105-ebd00caf089392ea" text_sha1="4a4825f373c3d6ccb1614d8c9a717bdc23267603" text_size="880"/>
-      <file id="pool.txt-20050309044946-4a2400814cb5087a" name="pool.txt" text_id="pool.txt-20050309045105-a146ad389b36ecef" text_sha1="68002fb614f9f22e07774f259d54e0e17f361b3d" text_size="2930"/>
-      <file id="purpose.txt-20050309044946-a4132525edac8d4c" name="purpose.txt" text_id="purpose.txt-20050321222949-ba749c2169e3dba0" text_sha1="711070a417d92af93d99430a478c593e6565dfe3" text_size="3145"/>
-      <file id="python.txt-20050309044946-9eeeab2c3546abe6" name="python.txt" text_id="python.txt-20050309045105-73f03ec782191b18" text_sha1="067a8357e841178ec89dbb76073c0c429e7633a8" text_size="4230"/>
-      <file id="quickref.txt-20050309044946-e118485499fadee0" name="quickref.txt" text_id="quickref.txt-20050309045105-af9c9832b3ab7fec" text_sha1="30f46dd441dbbd797e4dd111e106071bc24423de" text_size="549"/>
-      <file id="quilt.txt-20050309044946-00df099b997fed5e" name="quilt.txt" text_id="quilt.txt-20050309045105-f9eb948a305b247f" text_sha1="3d8613f245558097d280b756eec3cd2e19fbaa36" text_size="253"/>
-      <file id="random.txt-20050309044946-269c29512b381f5b" name="random.txt" text_id="random.txt-20050321222949-4cbd779bfebd936a" text_sha1="0c9400360472e0305b120dbbebf12788880f7ceb" text_size="8529"/>
-      <file id="requirements.txt-20050309044946-f01e5318ade63900" name="requirements.txt" text_id="requirements.txt-20050309045105-91208951f02f4d5c" text_sha1="7118ac308a4799ed8146c94fb5fd9dde2cf2789d" text_size="1030"/>
-      <file id="revision-syntax.txt-20050309044946-86b3a8e902a15fa4" name="revision-syntax.txt" text_id="revision-syntax.txt-20050309045105-ca2db9854ac402c0" text_sha1="c84b833a1dd795f4a65b9adabfdc9b3fb1159112" text_size="2445"/>
-      <file id="roadmap.txt-20050309044946-fd8a18ed2f6049f1" name="roadmap.txt" text_id="roadmap.txt-20050309045105-aa821d48d65a3646" text_sha1="86d8823ae029e7617a5527597602412c5aed744f" text_size="1480"/>
-      <file id="rollup.txt-20050309044946-e551f4f85db5a5a0" name="rollup.txt" text_id="rollup.txt-20050309045105-5801a84a4015957e" text_sha1="25d713bab1a5abc198358fedf6a414a00bf3f0df" text_size="2640"/>
-      <file id="scalability.txt-20050309044946-646ac26c7fd65298" name="scalability.txt" text_id="scalability.txt-20050309045105-572fbf8f67f12009" text_sha1="50b3aadd75b7478e68adc7818733783730d8ac3f" text_size="650"/>
-      <file id="security.txt-20050309044946-41f7c6000b1124d0" name="security.txt" text_id="security.txt-20050309045105-68e6d4d9a4574959" text_sha1="cb20cb44a6d5e421721396cf1ab5c648dd4d72ae" text_size="14662"/>
-      <file id="shared-branches.txt-20050309044946-20521acf9bda9528" name="shared-branches.txt" text_id="shared-branches.txt-20050309045105-7cbd7b176765a7b5" text_sha1="1b7eaedcaed0af15f431a398366b5a4f58a17927" text_size="5834"/>
-      <file id="short-demo.txt-20050309044946-2145761a6909b03e" name="short-demo.txt" text_id="short-demo.txt-20050309045105-acdbb8f1a3cde539" text_sha1="d5d95a0760231aa75584669546b1b29de26696f7" text_size="5732"/>
-      <file id="supportability.txt-20050309044946-92aeba826c90201b" name="supportability.txt" text_id="supportability.txt-20050309045105-3b42b97911c05906" text_sha1="2a393e5c052cfd6aafdec621185fd7415ef1152f" text_size="841"/>
-      <file id="svk.txt-20050309044946-c62acd7cee590d22" name="svk.txt" text_id="svk.txt-20050309045105-dec97635b569da41" text_sha1="a485ca74f87cc0baca211c8344bf31c11ee87389" text_size="422"/>
-      <file id="tagging.txt-20050309044946-66adf967ad987472" name="tagging.txt" text_id="tagging.txt-20050309045105-3477d840eb290fe3" text_sha1="39da37ca4d619169492365c917452127074e4aeb" text_size="3135"/>
-      <file id="taxonomy.txt-20050309044946-922fe1158868767f" name="taxonomy.txt" text_id="taxonomy.txt-20050309045105-d0be06847dc3a1ff" text_sha1="0ce01e87ad97edc9d25ba3c5b45dad4cad6b2084" text_size="1819"/>
-      <file id="testing.txt-20050309044946-ea100200833c223a" name="testing.txt" text_id="testing.txt-20050309045105-a7b901188c51574b" text_sha1="633cab6bde0535401ac9eabead45d96c90e600bc" text_size="773"/>
-      <file id="thanks.txt-20050309044946-58141ea3091846d8" name="thanks.txt" text_id="thanks.txt-20050319014144-66b28b029f489391" text_sha1="c5ad6319f69f8e31d15bc36d4522a036493f6bd3" text_size="420"/>
-      <file id="todo-from-arch.txt-20050309044946-4cb8ccdf8b3fcd07" name="todo-from-arch.txt" text_id="todo-from-arch.txt-20050309045105-a0d73cedaa3cb912" text_sha1="fa95080711fd9a116643867d6e7b7dbd6458c01e" text_size="13588"/>
-      <file id="unchanged.txt-20050309044946-fe7d1ec585585702" name="unchanged.txt" text_id="unchanged.txt-20050309045105-b4a697341c602735" text_sha1="db90f5d3210f1392f45c27ba8eb3528847f9ee02" text_size="3920"/>
-      <file id="unrelated-merge.txt-20050309044946-ec165531264f1c99" name="unrelated-merge.txt" text_id="unrelated-merge.txt-20050309045105-dbe2b17d18728121" text_sha1="62b6d77942c44f16951219e7838f56f13dd0ae37" text_size="310"/>
-      <file id="usability.txt-20050309044946-57e59739a429628b" name="usability.txt" text_id="usability.txt-20050309045105-e9515f46e755dc0f" text_sha1="692295f407ced0cbad7bc2af2d42ae2aa4f2bec2" text_size="104"/>
-      <file id="use-cases.txt-20050309044946-19871d18448c91fc" name="use-cases.txt" text_id="use-cases.txt-20050309045105-c18c9210fa9c667b" text_sha1="4463f0219146a6d18801d8c90e046b8ce338662b" text_size="2679"/>
-      <file id="web-interface.txt-20050309044946-65ca85c7f551bf88" name="web-interface.txt" text_id="web-interface.txt-20050309045105-28c3ac23357d1194" text_sha1="90dcbc996c2d0187c774732c3947010e27204646" text_size="1024"/>
-      <file id="work-order.txt-20050309044946-4cfe50fc6e5258fe" name="work-order.txt" text_id="work-order.txt-20050309045105-fe2baee64418083c" text_sha1="c03413dbaff3e33536b02e7e41dc52ced07037c3" text_size="1778"/>
-      <file id="workflow.txt-20050309044946-d214d0e0429e719e" name="workflow.txt" text_id="workflow.txt-20050309045105-05ebdf3422fdcef6" text_sha1="69429a38371b5d3b6b850beb373467b4e6943a3b" text_size="5225"/>
-      <file id="yaml.txt-20050309044946-8d597bdf670d9095" name="yaml.txt" text_id="yaml.txt-20050309045105-65f6ca2247754aef" text_sha1="7d9b5a23639d0f1a03c97f983f17b41b79142db0" text_size="1376"/>
-    </directory>
-    <directory id="elementtree-20050323061618-ec49a4621bc7e4f4" name="elementtree">
-      <file id="ElementTree.py-20050323061623-cfa8eefe86f35391" name="ElementTree.py" text_id="ElementTree.py-20050323062555-7c291904c45fd0b9" text_sha1="aa97bbce740f09213293096bd1a9b8085999048b" text_size="36765"/>
-      <file id="__init__.py-20050323061623-61888e93152e298d" name="__init__.py" text_id="__init__.py-20050323062555-2183ef6f9f77dc96" text_sha1="6ecf754d86258e81ad6f76b01059dbd500cb3bfb" text_size="1491"/>
-    </directory>
-    <directory id="notes-20050309064844-c41b58b77e2b0876" name="notes">
-      <file id="performance.txt-20050309064844-d9647fd301942365" name="performance.txt" text_id="performance.txt-20050329114950-6c0e608dfda43e40" text_sha1="2598fc4b749b2e3d0ac12a44eff4a1fba48cf786" text_size="9288"/>
-    </directory>
-    <file id="setup.py-20050314065409-02f8a0a6e3f9bc70" name="setup.py" text_id="setup.py-20050314070724-fbe6913928cfa373" text_sha1="4a08c1c4b5d08bdec22d450e43686b12cf0cc210" text_size="472"/>
-  </root_directory>
-</inventory>

=== removed file 'notes/performance.txt'
--- notes/performance.txt	2005-03-30 22:27:17 +0000
+++ notes/performance.txt	1970-01-01 00:00:00 +0000
@@ -1,321 +0,0 @@
-For a tree holding 2.4.18 (two copies), 2.4.19, 2.4.20
-
-With gzip -9:
-
-mbp@hope% du .bzr
-195110  .bzr/text-store
-20      .bzr/revision-store
-12355   .bzr/inventory-store
-216325  .bzr
-mbp@hope% du -s .
-523128  .
-
-Without gzip:
-
-This is actually a pretty bad example because of deleting and
-re-importing 2.4.18, but still not totally unreasonable.
-
-----
-
-linux-2.4.0: 116399 kB
-after addding everything: 119505kB
-bzr status  2.68s user 0.13s system 84% cpu 3.330 total
-bzr commit 'import 2.4.0'  4.41s user 2.15s system 11% cpu 59.490 total
-
-242446  .
-122068  .bzr
-
-
-----
-
-Performance (2005-03-01)
-
-To add all files from linux-2.4.18: about 70s, mostly inventory
-serialization/deserialization.
-
-To commit:
-- finished, 6.520u/3.870s cpu, 33.940u/10.730s cum
--     134.040 elapsed
-
-Interesting that it spends so long on external processing!  I wonder
-if this is for running uuidgen?  Let's try generating things
-internally.
-
-Great, this cuts it to 17.15s user 0.61s system 83% cpu 21.365 total
-to add, with no external command time.  The commit now seems to spend
-most of its time copying to disk.
-
-- finished, 6.550u/3.320s cpu, 35.050u/9.870s cum
--     89.650 elapsed
-
-I wonder where the external time is now?  We were also using uuids()
-for revisions.
-
-Let's remove everything and re-add.  Detecting everything was removed
-takes 
-- finished, 2.460u/0.110s cpu, 0.000u/0.000s cum
--     3.430 elapsed
-
-which may be mostly XML deserialization?
-
-Just getting the previous revision takes about this long:
-
-bzr invoked at Tue 2005-03-01 15:53:05.183741 EST +1100
-  by mbp@sourcefrog.net on hope
-  arguments: ['/home/mbp/bin/bzr', 'get-revision-inventory', 'mbp@sourcefrog.net-20050301044608-8513202ab179aff4-44e8cd52a41aa705']
-  platform: Linux-2.6.10-4-686-i686-with-debian-3.1
-- finished, 3.910u/0.390s cpu, 0.000u/0.000s cum
--     6.690 elapsed
-
-Now committing the revision which removes all files should be fast.
-
-- finished, 1.280u/0.030s cpu, 0.000u/0.000s cum
--     1.320 elapsed
-
-Now re-add with new code that doesn't call uuidgen:
-
-- finished, 1.990u/0.030s cpu, 0.000u/0.000s cum
--     2.040 elapsed
-
-16.61s user 0.55s system 74% cpu 22.965 total
-
-Status::
-
-  - finished, 2.500u/0.110s cpu, 0.010u/0.000s cum
-  -     3.350 elapsed
-
-And commit::
-
-Now patch up to 2.4.19.  There were some bugs in handling missing
-directories, but with that fixed we do much better::
-
-  bzr status  5.86s user 1.06s system 10% cpu 1:05.55 total
-
-This is slow because it's diffing every file; we should use mtimes etc
-to make this faster.  The cpu time is reasonable.
-
-I see difflib is pure Python; it might be faster to shell out to GNU
-diff when we need it.
-
-Export is very fast::
-
-  - finished, 4.220u/1.480s cpu, 0.010u/0.000s cum
-  -     10.810 elapsed
-
-  bzr export 1 ../linux-2.4.18.export1  3.92s user 1.72s system 21% cpu 26.030 total
-
-
-Now to find and add the new changes::
-
-  - finished, 2.190u/0.030s cpu, 0.000u/0.000s cum
-  -     2.300 elapsed
-
-
-:: 
-  bzr commit 'import 2.4.19'  9.36s user 1.91s system 23% cpu 47.127 total
-
-And the result is exactly right.  Try exporting::
-
-  mbp@hope%  bzr export 4 ../linux-2.4.19.export4
-  bzr export 4 ../linux-2.4.19.export4  4.21s user 1.70s system 18% cpu 32.304 total
-
-and the export is exactly the same as the tarball.
-
-Now we can optimize the diff a bit more by not comparing files that
-have the right SHA-1 from within the commit
-
-For comparison::
-
-  patch -p1 < ../kernel.pkg/patch-2.4.20  1.61s user 1.03s system 13% cpu 19.106 total
-
-
-Now status after applying the .20 patch.  With full-text verification::
-
-  bzr status  7.07s user 1.32s system 13% cpu 1:04.29 total
-
-with that turned off::
-
-  bzr status  5.86s user 0.56s system 25% cpu 25.577 total
-
-After adding:
-
-  bzr status  6.14s user 0.61s system 25% cpu 26.583 total
-
-Should add some kind of profile counter for quick compares vs slow
-compares.
-
-  bzr commit 'import 2.4.20'  7.57s user 1.36s system 20% cpu 43.568
-  total
-
-export:  finished, 3.940u/1.820s cpu, 0.000u/0.000s cum,  50.990 elapsed
-
-also exports correctly
-
-now .21
-
-bzr commit 'import 2.4.1'  5.59s user 0.51s system 60% cpu 10.122 total
-
-265520  .
-137704  .bzr
-
-import 2.4.2
-317758  .
-183463  .bzr
-
-
-with everything through to 2.4.29 imported, the .bzr directory is
-1132MB, compared to 185MB for one tree.  The .bzr.log is 100MB!.  So
-the storage is 6.1 times larger, although we're holding 30 versions.
-It's pretty large but I think not ridiculous.  By contrast the tarball
-for 2.4.0 is 104MB, and the tarball plus uncompressed patches are
-315MB.
-
-Uncompressed, the text store is 1041MB.  So it is only three times
-worse than patches, and could be compressed at presumably roughly
-equal efficiency.  It is large, but also a very simple design and
-perhaps adequate for the moment.  The text store with each file
-individually gziped is 264MB, which is also a very simple format and
-makes it less than twice the size of the source tree.
-
-This is actually rather pessimistic because I think there are some
-orphaned texts in there.
-
-Measured by du, the compressed full-text store is 363MB; also probably
-tolerable.
-
-The real fix is perhaps to use some kind of weave, not so much for
-storage efficiency as for fast annotation and therefore possible
-annotation-based merge.
-
------
-
-
-2005-03-25
-
-Now we have recursive add, add is much faster.  Adding all of the
-linux 2.4.19 kernel tree takes only 
-
-finished, 5.460u/0.610s cpu, 0.010u/0.000s cum, 6.710 elapsed
-
-
-However, the store code currently flushes to disk after every write,
-which is probably excessive.  So a commit takes
-
-finished, 8.740u/3.950s cpu, 0.010u/0.000s cum, 156.420 elapsed
-
-
-Status is now also quite fast, depsite that it still has to read all
-the working copies:
-
-mbp@hope% bzr status                                                            ~/work/linux-2.4.19
-bzr status  5.51s user 0.79s system 99% cpu 6.337 total
-
-strace shows much of this is in write(2), probably because of
-logging.  With more buffering on that file, removing all the explicit
-flushes, that is reduced to 
-
-mbp@hope% time bzr status  
-bzr status  5.23s user 0.42s system 97% cpu 5.780 total
-
-which is mostly opening, stating and reading files, as it should be.
-Still a few too many stat calls.
-
-Now fixed up handling of root directory.
-
-Without flushing everything to disk as it goes into the store:
-
-mbp@hope% bzr commit -m 'import linux 2.4.19'
-bzr commit -m 'import linux 2.4.19'  8.15s user 2.09s system 53% cpu 19.295 total
-
-mbp@hope% time bzr diff
-bzr diff  5.80s user 0.52s system 69% cpu 9.128 total
-mbp@hope% time bzr status
-bzr status  5.64s user 0.43s system 68% cpu 8.848 total
-
-patch -p1 < ../linux.pkg/patch-2.4.20  1.67s user 0.96s system 90% cpu 2.905 total
-
-The diff changes 3462 files according to diffstat.
-
-branch format: Bazaar-NG branch, format 0.0.4
-
-in the working tree:
-   8674 unchanged
-   2463 modified
-    818 added
-    229 removed
-      0 renamed
-      0 unknown
-      4 ignored
-    614 versioned subdirectories
-
-That is, 3510 entries have changed, but there are 48 changed
-directories so the count is exactly right!
-
-bzr commit -v -m 'import 2.4.20'  8.23s user 1.09s system 48% cpu 19.411 total
-
-Kind of strange that this takes as much time as committing the whole
-thing; I suppose it has to read every file.  
-
-This shows many files as being renamed; I don't know why that would
-be.
-
-
-Patch to 2.4.21:
-
- 2969 files changed, 366643 insertions(+), 147759 deletions(-)
-
-After auto-add:
-
- 2969 files changed, 372168 insertions(+), 153284 deletions(-)
-
-I wonder why it is not exactly the same?  Maybe because the python
-diff algorithm is a bit differnt to GNU diff.
-
-----
-
-2005-03-29  
-
-full check, retrieving all file texts once for the 2.4 kernel branch
-takes 10m elapsed, 1m cpu time.  lots of random IO and seeking.
-
-----
-
-
-mbp@hope% time python =bzr deleted --show-ids 
-README                                             README-fa1d8447b4fd0140-adbf4342752f0fc3
-python =bzr deleted --show-ids  1.55s user 0.09s system 96% cpu 1.701 total
-mbp@hope% time python -O =bzr deleted --show-ids 
-README                                             README-fa1d8447b4fd0140-adbf4342752f0fc3
-python -O =bzr deleted --show-ids  1.47s user 0.10s system 101% cpu 1.547 total
-mbp@hope% time python -O =bzr deleted --show-ids
-README                                             README-fa1d8447b4fd0140-adbf4342752f0fc3
-python -O =bzr deleted --show-ids  1.49s user 0.07s system 99% cpu 1.565 total
-mbp@hope% time python =bzr deleted --show-ids   
-README                                             README-fa1d8447b4fd0140-adbf4342752f0fc3
-python =bzr deleted --show-ids  1.55s user 0.08s system 99% cpu 1.637 total
-
-small but significant improvement from Python -O
-
-----
-
-Loading a large inventory through cElementTree is pretty quick; only
-about 0.117s.  By contrast reading the inventory into our data
-structure takes about 0.7s.  
-
-So I think the problem must be in converting everything to
-InventoryEntries and back again every time.
-
-Thought about that way it seems pretty inefficient: why create all
-those objects when most of them aren't called on most invocations?
-Instead perhaps the Inventory object should hold the ElementTree and
-pull things out of it only as necessary?  We can even have an index
-pointing into the ElementTree by id, path, etc.
-
-
-as of r148
-
-bzr deleted  1.46s user 0.08s system 98% cpu 1.561 total
-
-
-Alternatively maybe keep an id2path and path2id cache?  Keeping it
-coherent may be hard...

=== removed file 'notes/revfile.txt'
--- notes/revfile.txt	2005-08-12 15:41:44 +0000
+++ notes/revfile.txt	1970-01-01 00:00:00 +0000
@@ -1,45 +0,0 @@
-importing all inventories:
-
-with 50% chance of choosing earlier parent:
-
--rw-r--r--  1 mbp mbp 296961 2005-04-09 22:37 inventory.drev
--rw-r--r--  1 mbp mbp  11184 2005-04-09 22:37 inventory.irev
-
-imported in 2.490s
-
-
-with no chance:
-
-clearly slows down, 11.163s to import
-
--rw-r--r--  1 mbp mbp 63590 2005-04-09 22:39 inventory.drev
--rw-r--r--  1 mbp mbp 11184 2005-04-09 22:39 inventory.irev
-
-
-
-with 25% chance:
-
-revfile.py add-series inventory 0 mbp*  7.86s user 0.12s system 96% cpu 8.247 total
--rw-r--r--  1 mbp mbp 79535 2005-04-09 22:42 inventory.drev
--rw-r--r--  1 mbp mbp 11184 2005-04-09 22:42 inventory.irev
-
-
-
-with full text inserted every 20 revisions:
-
-revfile.py add-series inventory 0 mbp*  1.98s user 0.06s system 95% cpu 2.139 total
-
--rw-r--r--  1 mbp mbp 133719 2005-04-09 22:44 inventory.drev
--rw-r--r--  1 mbp mbp  11184 2005-04-09 22:44 inventory.irev
-
-So pretty quick, simple, and somewhat easily justified.
-
-
-
-every 50 revisions:
-
-revfile.py add-series inventory 0 mbp*  3.35s user 0.08s system 93% cpu 3.667 total
-
--rw-r--r--  1 mbp mbp 91895 2005-04-09 22:46 inventory.drev
--rw-r--r--  1 mbp mbp 11184 2005-04-09 22:46 inventory.irev
-

=== removed file 'notes/schemas.xml'
--- notes/schemas.xml	2005-08-19 21:34:32 +0000
+++ notes/schemas.xml	1970-01-01 00:00:00 +0000
@@ -1,4 +0,0 @@
-<?xml version="1.0"?>
-<locatingRules xmlns="http://thaiopensource.com/ns/locating-rules/1.0">
-  <uri resource="inventory-v2-sample.xml" uri="inventory-v2.rnc"/>
-</locatingRules>

=== removed directory 'patches'
=== removed file 'patches/annotate3.patch'
--- patches/annotate3.patch	2005-06-10 06:29:35 +0000
+++ patches/annotate3.patch	1970-01-01 00:00:00 +0000
@@ -1,7263 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -802,3 +804,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,27 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        for result in patches.iter_annotate_file(lines, anno_d_iter):
-+            if isinstance(result, progress.Progress):
-+                result.total = total
-+                progress.progress_bar(result)
-+            else:
-+                progress.clear_progress_bar()
-+                anno_lines = result
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== removed file 'patches/annotate4.patch'
--- patches/annotate4.patch	2005-06-10 06:34:26 +0000
+++ patches/annotate4.patch	1970-01-01 00:00:00 +0000
@@ -1,7270 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -864,3 +866,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,34 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        if not os.path.exists(filename):
-+            raise BzrCommandError("The file %s does not exist." % filename)
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        if file_id is None:
-+            raise BzrCommandError("The file %s is not versioned." % filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        progress_bar = progress.ProgressBar()
-+        try:
-+            for result in patches.iter_annotate_file(lines, anno_d_iter):
-+                if isinstance(result, progress.Progress):
-+                    result.total = total
-+                    progress_bar(result)
-+                else:
-+                    anno_lines = result
-+        finally:
-+            progress.clear_progress_bar()
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== removed file 'patches/cache-remote-revisions.diff'
--- patches/cache-remote-revisions.diff	2005-06-15 04:17:12 +0000
+++ patches/cache-remote-revisions.diff	1970-01-01 00:00:00 +0000
@@ -1,105 +0,0 @@
-***************
-*** 738,777 ****
-          revisions = []
-          pb = ProgressBar(show_spinner=True)
-          total = len(revision_ids)
--         for i,f in enumerate(revision_ids):
--             revisions.append(other.get_revision(f))
--             pb.update('retrieving revisions', i+1, total)
--         pb.clear()
--         
--         needed_texts = sets.Set()
-- 
--         for index, rev in enumerate(revisions):
--             pb.update('Scanning revisions for file contents', index, total)
--             inv = other.get_inventory(str(rev.inventory_id))
--             for key, entry in inv.iter_entries():
--                 if entry.text_id is None:
--                     continue
--                 if entry.text_id not in self.text_store:
--                     needed_texts.add(entry.text_id)
--         pb.clear()
--         count = self.text_store.copy_multi(other.text_store, needed_texts, pb,
--                                            "Copying file contents")
--         pb.clear()
--         print "Added %d file contents." % count 
--         inventory_ids = [ f.inventory_id for f in revisions ]
--         count = self.inventory_store.copy_multi(other.inventory_store, 
--                                                 inventory_ids, pb,
--                                                 "Copying inventories")
--         pb.clear()
--         print "Added %d inventories." % count 
--         revision_ids = [ f.revision_id for f in revisions]
--         count = self.revision_store.copy_multi(other.revision_store, 
--                                                revision_ids, pb, 
--                                                "Copying revisions")
--         pb.clear()
--         for revision_id in revision_ids:
--             self.append_revision(revision_id)
--         print "Added %d revisions." % count
-                      
-          
-      def commit(self, *args, **kw):
---- 738,799 ----
-          revisions = []
-          pb = ProgressBar(show_spinner=True)
-          total = len(revision_ids)
-+         tmp_dir = tempfile.mkdtemp(prefix = "temp-stores-")
-+         try:
-+             tmp_rev_dir = os.path.join(tmp_dir, "revisions")
-+             os.mkdir(tmp_rev_dir)
-+             tmp_revs = ImmutableStore(tmp_rev_dir)
-+             count = tmp_revs.copy_multi(other.revision_store, revision_ids, pb, 
-+                                         "Caching revisions")
-+             #EVIL! Substituting a local partial store for a complete one
-+             #This is a significant performance boost when complete one is
-+             #a remote store. 
-+             other.revision_store = tmp_revs
-+             pb.clear()
-+ 
-+             for i,f in enumerate(revision_ids):
-+                 revisions.append(other.get_revision(f))
-+                 pb.update("Parsing revisions", i, len(revision_ids))
-+             
-+             needed_texts = sets.Set()
-+ 
-+             #Again with the EVIL.
-+             tmp_rev_dir = os.path.join(tmp_dir, "inventories")
-+             os.mkdir(tmp_rev_dir)
-+             inv_ids = [r.inventory_id for r in revisions]
-+             tmp_revs = ImmutableStore(tmp_rev_dir)
-+             count = tmp_revs.copy_multi(other.inventory_store, inv_ids, pb, 
-+                                         "Caching inventories")
-+             other.inventory_store = tmp_revs
-+             pb.clear()
-+             for index, rev in enumerate(revisions):
-+                 pb.update('Scanning revisions for file contents', index, total)
-+                 inv = other.get_inventory(str(rev.inventory_id))
-+                 for key, entry in inv.iter_entries():
-+                     if entry.text_id is None:
-+                         continue
-+                     if entry.text_id not in self.text_store:
-+                         needed_texts.add(entry.text_id)
-+             pb.clear()
-+             count = self.text_store.copy_multi(other.text_store, needed_texts, pb,
-+                                                "Copying file contents")
-+             pb.clear()
-+             print "Added %d file contents." % count 
-+             inventory_ids = [ f.inventory_id for f in revisions ]
-+             count = self.inventory_store.copy_multi(other.inventory_store, 
-+                                                     inventory_ids, pb,
-+                                                     "Copying inventories")
-+             pb.clear()
-+             print "Added %d inventories." % count 
-+             revision_ids = [ f.revision_id for f in revisions]
-+             count = self.revision_store.copy_multi(other.revision_store, 
-+                                                    revision_ids)
-+             pb.clear()
-+             for revision_id in revision_ids:
-+                 self.append_revision(revision_id)
-+             print "Added %d revisions." % count
-+         finally:
-+             shutil.rmtree(tmp_dir)
-                      
-          
-      def commit(self, *args, **kw):

=== removed file 'patches/find-touching-from-seq.diff'
--- patches/find-touching-from-seq.diff	2005-06-06 12:33:11 +0000
+++ patches/find-touching-from-seq.diff	1970-01-01 00:00:00 +0000
@@ -1,83 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -610,11 +610,13 @@
-     hidden = True
-     takes_args = ["filename"]
-     def run(self, filename):
-+        from bzrlib.log import find_touching_revisions
-         b = Branch(filename, lock_mode='r')
-         inv = b.read_working_inventory()
-         file_id = inv.path2id(b.relpath(filename))
--        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
--            print "%6d %s" % (revno, what)
-+        rh = b.revision_history()
-+        for revision_id, what in find_touching_revisions(b, file_id, rh):
-+            print "%-40s %s" % (revision_id, what)
- 
- 
- class cmd_ls(Command):
-
-*** modified file 'bzrlib/log.py'
---- bzrlib/log.py 
-+++ bzrlib/log.py 
-@@ -17,7 +17,7 @@
- 
- 
- 
--def find_touching_revisions(branch, file_id):
-+def find_touching_revisions(branch, file_id, revisions):
-     """Yield a description of revisions which affect the file_id.
- 
-     Each returned element is (revno, revision_id, description)
-@@ -25,13 +25,20 @@
-     This is the list of revisions where the file is either added,
-     modified, renamed or deleted.
- 
--    TODO: Perhaps some way to limit this to only particular revisions,
--    or to traverse a non-mainline set of revisions?
-+    branch
-+        Branch to examine
-+
-+    file_id
-+        File to consider
-+
-+    revisions
-+        Sequence of revisions to search, can be
-+        branch.revision_history() or a filtered version of that
-+        or some sequence of non-mailine revisions.
-     """
-     last_ie = None
-     last_path = None
--    revno = 1
--    for revision_id in branch.revision_history():
-+    for revision_id in revisions:
-         this_inv = branch.get_revision_inventory(revision_id)
-         if file_id in this_inv:
-             this_ie = this_inv[file_id]
-@@ -46,19 +53,19 @@
-             # not present in either
-             pass
-         elif this_ie and not last_ie:
--            yield revno, revision_id, "added " + this_path
-+            yield revision_id, "added " + this_path
-         elif not this_ie and last_ie:
-             # deleted here
--            yield revno, revision_id, "deleted " + last_path
-+            yield revision_id, "deleted " + last_path
-         elif this_path != last_path:
--            yield revno, revision_id, ("renamed %s => %s" % (last_path, this_path))
-+            yield revision_id, ("renamed %s => %s" % (last_path, this_path))
-         elif (this_ie.text_size != last_ie.text_size
-               or this_ie.text_sha1 != last_ie.text_sha1):
--            yield revno, revision_id, "modified " + this_path
-+            yield revision_id, "modified " + this_path
- 
-         last_ie = this_ie
-         last_path = this_path
--        revno += 1
-+
- 
- 
- def show_log(branch,
-

=== removed file 'patches/meta-data-in-inventory.patch'
--- patches/meta-data-in-inventory.patch	2005-06-13 12:14:52 +0000
+++ patches/meta-data-in-inventory.patch	1970-01-01 00:00:00 +0000
@@ -1,300 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -1175,7 +1175,68 @@
-         b = Branch('.')
-         statcache.update_cache(b.base, b.read_working_inventory())
- 
--
-+class cmd_meta(Command):
-+    """Get or set the meta information properties about a file.
-+
-+    bzr meta FILENAME # Display all meta information
-+    bzr meta FILENAME prop # Display the value of meta-info named prop, return error if not found
-+    bzr meta FILENAME --set prop value # Set the value of prop to value
-+    echo "the value" | bzr meta FILENAME --set prop # Set the value to whatever is read from stdin
-+    bzr meta FILENAME --unset prop # Remove the property
-+
-+    bzr meta FILENAME --revision=10 # Display the meta information for a given revision
-+        (Not supported yet)
-+    """
-+    hidden = True
-+    takes_args = ['filename', 'property?', 'value?']
-+    takes_options = ['revision', 'set', 'unset']
-+
-+    def run(self, filename, property=None, value=None, revision=None, set=False, unset=False):
-+        if isinstance(revision, list) and len(revision) > 1:
-+            raise BzrCommandError('bzr meta takes at most 1 revision.')
-+        if revision is not None and (set or unset):
-+            raise BzrCommandError('Cannot set/unset meta information in an old version.')
-+        if set and unset:
-+            raise BzrCommandError('Cannot set and unset at the same time')
-+        if not set and value:
-+            raise BzrCommandError('You must supply --set if you want to set the value of a property.')
-+
-+        b = Branch(filename)
-+        inv = b.inventory
-+        file_id = inv.path2id(b.relpath(filename))
-+        inv_entry = inv[file_id]
-+        if not property:
-+            meta = inv_entry.meta
-+            if meta: # Not having meta is the same as having an empty meta
-+                keys = meta.properties.keys()
-+                keys.sort()
-+                # The output really needs to be parseable
-+                for key in keys:
-+                    print '%s: %r' % (key, meta.properties[key])
-+        else:
-+            meta = inv_entry.meta
-+            if set:
-+                if value is None:
-+                    value = sys.stdin.read()
-+                if not meta:
-+                    from bzrlib.inventory import Meta
-+                    inv_entry.meta = Meta({property:value})
-+                else:
-+                    inv_entry.meta.properties[property] = value
-+                b.inventory = inv # This should cause it to be saved
-+            elif unset:
-+                if not meta or not meta.properties.has_key(property):
-+                    return 3 # Cannot unset a property that doesn't exist
-+                    # I wonder if this should be a different return code
-+                del inv_entry.meta.properties[property]
-+                b.inventory = inv
-+            else:
-+                if not meta or not meta.properties.has_key(property):
-+                    return 3 # Missing property
-+                # Probably this should not be print, but 
-+                # sys.stdout.write() so that you get back exactly
-+                # what was given. But I'm leaving it this way for now
-+                print meta.properties[property]
- 
- # list of all available options; the rhs can be either None for an
- # option that takes no argument, or a constructor function that checks
-@@ -1196,6 +1257,8 @@
-     'verbose':                None,
-     'version':                None,
-     'email':                  None,
-+    'set':                    None,
-+    'unset':                  None,
-     }
- 
- SHORT_OPTIONS = {
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -226,8 +226,10 @@
-                       new_tree.get_file(file_id).readlines(),
-                       to_file)
- 
--    for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-+    for old_path, new_path, file_id, kind, text_modified, meta_modified in delta.renamed:
-         print '*** renamed %s %r => %r' % (kind, old_path, new_path)
-+        if meta_modified:
-+            print '## Meta-info modified'
-         if text_modified:
-             diff_file(old_label + old_path,
-                       old_tree.get_file(file_id).readlines(),
-@@ -235,9 +237,11 @@
-                       new_tree.get_file(file_id).readlines(),
-                       to_file)
- 
--    for path, file_id, kind in delta.modified:
-+    for path, file_id, kind, text_modified, meta_modified in delta.modified:
-         print '*** modified %s %r' % (kind, path)
--        if kind == 'file':
-+        if meta_modified:
-+            print '## Meta-info modified'
-+        if kind == 'file' and text_modified:
-             diff_file(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + path,
-@@ -316,7 +320,7 @@
- 
-         if self.renamed:
-             print >>to_file, 'renamed:'
--            for oldpath, newpath, fid, kind, text_modified in self.renamed:
-+            for oldpath, newpath, fid, kind, text_modified, meta_modified in self.renamed:
-                 if show_ids:
-                     print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
-                 else:
-@@ -324,7 +328,16 @@
-                     
-         if self.modified:
-             print >>to_file, 'modified:'
--            show_list(self.modified)
-+            for path, fid, kind, text_modified, meta_modified in self.modified:
-+                if kind == 'directory':
-+                    path += '/'
-+                elif kind == 'symlink':
-+                    path += '@'
-+                    
-+                if show_ids:
-+                    print >>to_file, '  %-30s %s' % (path, fid)
-+                else:
-+                    print >>to_file, ' ', path
-             
-         if show_unchanged and self.unchanged:
-             print >>to_file, 'unchanged:'
-@@ -388,6 +401,14 @@
-                 ## mutter("no text to check for %r %r" % (file_id, kind))
-                 text_modified = False
- 
-+            old_meta = old_inv[file_id].meta
-+            new_meta = new_inv[file_id].meta
-+
-+            if old_meta != new_meta:
-+                meta_modified = True
-+            else:
-+                meta_modified = False
-+
-             # TODO: Can possibly avoid calculating path strings if the
-             # two files are unchanged and their names and parents are
-             # the same and the parents are unchanged all the way up.
-@@ -395,9 +416,10 @@
-             
-             if old_path != new_path:
-                 delta.renamed.append((old_path, new_path, file_id, kind,
--                                      text_modified))
--            elif text_modified:
--                delta.modified.append((new_path, file_id, kind))
-+                                      text_modified, meta_modified))
-+            elif text_modified or meta_modified:
-+                delta.modified.append((new_path, file_id, kind,
-+                                       text_modified, meta_modified))
-             elif want_unchanged:
-                 delta.unchanged.append((new_path, file_id, kind))
-         else:
-
-*** modified file 'bzrlib/inventory.py'
---- bzrlib/inventory.py 
-+++ bzrlib/inventory.py 
-@@ -33,6 +33,67 @@
- import bzrlib
- from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
- from bzrlib.trace import mutter
-+
-+class Meta(XMLMixin):
-+    """Meta information about a single inventory entry.
-+
-+    This is basically just a set of key->value pairs.
-+
-+    In general, bzr does not handle this information, it only provides
-+    a location for plugins and other data sources to store this data.
-+
-+    """
-+    def __init__(self, properties):
-+        self.properties = properties
-+
-+    def __eq__(self, other):
-+        if other is None:
-+            return False
-+        if not isinstance(other, Meta):
-+            return NotImplemented
-+
-+        return (self.properties == other.properties)
-+
-+    def __ne__(self, other):
-+        return not (self == other)
-+
-+    def __hash__(self):
-+        raise ValueError('not hashable')
-+
-+    def from_element(cls, elt):
-+        assert elt.tag == 'meta'
-+
-+        properties = {}
-+        for child in elt:
-+            if child.tag == 'property':
-+                if child.text is None:
-+                    properties[child.get('name')] = ''
-+                else:
-+                    properties[child.get('name')] = child.text
-+        self = cls(properties)
-+        return self
-+
-+    from_element = classmethod(from_element)
-+
-+    def to_element(self):
-+        e = Element('meta')
-+        keys = self.properties.keys()
-+        keys.sort()
-+
-+        # The blah.text is just to make things look pretty in the files
-+        # We may want to remove it
-+        if len(keys) > 0:
-+            e.text = '\n'
-+
-+        for key in keys:
-+            prop = Element('property')
-+            prop.set('name', key)
-+            prop.text=self.properties[key]
-+            prop.tail='\n'
-+            e.append(prop)
-+
-+        return e
-+
- 
- class InventoryEntry(XMLMixin):
-     """Description of a versioned file.
-@@ -100,6 +161,7 @@
- 
-     text_sha1 = None
-     text_size = None
-+    meta = None
-     
-     def __init__(self, file_id, name, kind, parent_id, text_id=None):
-         """Create an InventoryEntry
-@@ -124,6 +186,7 @@
-         self.kind = kind
-         self.text_id = text_id
-         self.parent_id = parent_id
-+        self.meta = None
-         if kind == 'directory':
-             self.children = {}
-         elif kind == 'file':
-@@ -146,6 +209,10 @@
-         other.text_size = self.text_size
-         # note that children are *not* copied; they're pulled across when
-         # others are added
-+        if self.meta:
-+            other.meta = Meta(self.meta.properties)
-+        else:
-+            other.meta = None
-         return other
- 
- 
-@@ -182,6 +249,9 @@
-             e.set('parent_id', self.parent_id)
- 
-         e.tail = '\n'
-+
-+        if self.meta:
-+            e.append(self.meta.to_element())
-             
-         return e
- 
-@@ -205,6 +275,11 @@
-         v = elt.get('text_size')
-         self.text_size = v and int(v)
- 
-+        self.meta = None
-+        for child in elt:
-+            if child.tag == 'meta':
-+                self.meta = Meta.from_element(child)
-+
-         return self
-             
- 
-@@ -220,7 +295,8 @@
-                and (self.text_size == other.text_size) \
-                and (self.text_id == other.text_id) \
-                and (self.parent_id == other.parent_id) \
--               and (self.kind == other.kind)
-+               and (self.kind == other.kind) \
-+               and (self.meta == other.meta)
- 
- 
-     def __ne__(self, other):
-

=== removed file 'patches/ndiff.patch'
--- patches/ndiff.patch	2005-07-04 12:11:40 +0000
+++ patches/ndiff.patch	1970-01-01 00:00:00 +0000
@@ -1,149 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -721,10 +721,11 @@
-     """
-     
-     takes_args = ['file*']
--    takes_options = ['revision', 'diff-options']
-+    takes_options = ['revision', 'diff-options', 'ndiff']
-     aliases = ['di', 'dif']
- 
--    def run(self, revision=None, file_list=None, diff_options=None):
-+    def run(self, revision=None, file_list=None, diff_options=None,
-+            ndiff=None):
-         from bzrlib.diff import show_diff
- 
-         if file_list:
-@@ -735,9 +736,15 @@
-                 file_list = None
-         else:
-             b = find_branch('.')
--    
-+
-+        if ndiff:
-+            format = 'ndiff'
-+        else:
-+            format = None
-+
-         show_diff(b, revision, specific_files=file_list,
--                  external_diff_options=diff_options)
-+                  external_diff_options=diff_options,
-+                  format=format)
- 
- 
-         
-@@ -1344,6 +1351,7 @@
-     'format':                 unicode,
-     'forward':                None,
-     'message':                unicode,
-+    'ndiff':                  None,
-     'no-recurse':             None,
-     'profile':                None,
-     'revision':               _parse_revision_str,
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -70,6 +70,13 @@
-     print >>to_file
- 
- 
-+
-+def internal_ndiff(old_label, oldlines,
-+                   new_label, newlines,
-+                   to_file):
-+    """Show diff in python-specific ndiff format."""
-+    from difflib import ndiff
-+    to_file.writelines(ndiff(oldlines, newlines))
- 
- 
- def external_diff(old_label, oldlines, new_label, newlines, to_file,
-@@ -152,7 +159,8 @@
-     
- 
- 
--def show_diff(b, revision, specific_files, external_diff_options=None):
-+def show_diff(b, revision, specific_files, external_diff_options=None,
-+              format=None):
-     """Shortcut for showing the diff to the working tree.
- 
-     b
-@@ -160,6 +168,9 @@
- 
-     revision
-         None for each, or otherwise the old revision to compare against.
-+
-+    format
-+        'unified', 'context', 'ndiff', 'external'
-     
-     The more general form is show_diff_trees(), where the caller
-     supplies any two trees.
-@@ -174,12 +185,13 @@
-     new_tree = b.working_tree()
- 
-     show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
--                    external_diff_options)
-+                    external_diff_options, format)
- 
- 
- 
- def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
--                    external_diff_options=None):
-+                    external_diff_options=None,
-+                    format=None):
-     """Show in text form the changes from one tree to another.
- 
-     to_files
-@@ -204,10 +216,12 @@
-     if external_diff_options:
-         assert isinstance(external_diff_options, basestring)
-         opts = external_diff_options.split()
--        def diff_file(olab, olines, nlab, nlines, to_file):
-+        def diff_fn(olab, olines, nlab, nlines, to_file):
-             external_diff(olab, olines, nlab, nlines, to_file, opts)
-+    elif format == 'ndiff':
-+        diff_fn = internal_ndiff
-     else:
--        diff_file = internal_diff
-+        diff_fn = internal_diff
-     
- 
-     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
-@@ -216,7 +230,7 @@
-     for path, file_id, kind in delta.removed:
-         print >>to_file, '*** removed %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(old_label + path,
-+            diff_fn(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       DEVNULL, 
-                       [],
-@@ -225,7 +239,7 @@
-     for path, file_id, kind in delta.added:
-         print >>to_file, '*** added %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(DEVNULL,
-+            diff_fn(DEVNULL,
-                       [],
-                       new_label + path,
-                       new_tree.get_file(file_id).readlines(),
-@@ -234,7 +248,7 @@
-     for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-         print >>to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
-         if text_modified:
--            diff_file(old_label + old_path,
-+            diff_fn(old_label + old_path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + new_path,
-                       new_tree.get_file(file_id).readlines(),
-@@ -243,7 +257,7 @@
-     for path, file_id, kind in delta.modified:
-         print >>to_file, '*** modified %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(old_label + path,
-+            diff_fn(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + path,
-                       new_tree.get_file(file_id).readlines(),
-

=== removed file 'patches/pending-merge.patch'
--- patches/pending-merge.patch	2005-08-04 22:04:40 +0000
+++ patches/pending-merge.patch	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-*** modified file 'bzrlib/merge.py'
---- bzrlib/merge.py 
-+++ bzrlib/merge.py 
-@@ -105,14 +105,21 @@
-     location, revno = treespec
-     branch = find_branch(location)
-     if revno is None:
-+        rev_id = None
-+    elif revno == -1:
-+        rev_id = branch.last_patch()
-+    else:
-+        rev_id = branch.lookup_revision(revno)
-+    return branch, get_revid_tree(branch, rev_id, temp_root, label)
-+
-+def get_revid_tree(branch, rev_id, temp_root, label):
-+    if rev_id is None:
-         base_tree = branch.working_tree()
--    elif revno == -1:
--        base_tree = branch.basis_tree()
-     else:
--        base_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        base_tree = branch.revision_tree(rev_id)
-     temp_path = os.path.join(temp_root, label)
-     os.mkdir(temp_path)
--    return branch, MergeTree(base_tree, temp_path)
-+    return MergeTree(base_tree, temp_path)
- 
- 
- def abspath(tree, file_id):
-@@ -178,30 +185,68 @@
-         If true, this_dir must have no uncommitted changes before the
-         merge begins.
-     """
-+    from bzrlib.revision import common_ancestor, MultipleRevisionSources
-+    from bzrlib.branch import NoSuchRevision
-     tempdir = tempfile.mkdtemp(prefix="bzr-")
-     try:
-         if this_dir is None:
-             this_dir = '.'
-         this_branch = find_branch(this_dir)
-+        this_rev_id = this_branch.last_patch()
-+        if this_rev_id is None:
-+            raise BzrCommandError("This branch has no commits")
-         if check_clean:
-             changes = compare_trees(this_branch.working_tree(), 
-                                     this_branch.basis_tree(), False)
-             if changes.has_changed():
-                 raise BzrCommandError("Working tree has uncommitted changes.")
-         other_branch, other_tree = get_tree(other_revision, tempdir, "other")
-+        if other_revision[1] == -1:
-+            other_rev_id = other_branch.last_patch()
-+            other_basis = other_rev_id
-+        elif other_revision[1] is not None:
-+            other_rev_id = other_branch.lookup_revision(other_revision[1])
-+            other_basis = other_rev_id
-+        else:
-+            other_rev_id = None
-+            other_basis = other_branch.last_patch()
-         if base_revision == [None, None]:
-             if other_revision[1] == -1:
-                 o_revno = None
-             else:
-                 o_revno = other_revision[1]
--            base_revno = this_branch.common_ancestor(other_branch, 
--                                                     other_revno=o_revno)[0]
--            if base_revno is None:
-+            base_rev_id = common_ancestor(this_rev_id, other_basis,
-+                                          MultipleRevisionSources(this_branch, 
-+                                          other_branch))
-+            if base_rev_id is None:
-                 raise UnrelatedBranches()
--            base_revision = ['.', base_revno]
--        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
-+            try:
-+                base_revision = this_branch.get_revision(base_rev_id)
-+                base_branch = this_branch
-+            except NoSuchRevision:
-+                base_branch = other_branch
-+            base_tree = get_revid_tree(base_branch, base_rev_id, tempdir, 
-+                                       "base")
-+            base_is_ancestor = True
-+        else:
-+            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
-+            if base_revision[1] == -1:
-+                base_rev_id = base_branch.last_patch()
-+            elif base_revision[1] is None:
-+                base_rev_id = None
-+            else:
-+                base_rev_id = base_branch.lookup_revision(base_revision[1])
-+            if base_rev_id is not None:
-+                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
-+                                               MultipleRevisionSources(
-+                                               this_branch, 
-+                                               base_branch))
-+            else:
-+                base_is_ancestor = False
-         merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                     ignore_zero=ignore_zero)
-+        if base_is_ancestor and other_rev_id is not None:
-+            this_branch.add_pending_merge(other_rev_id)
-     finally:
-         shutil.rmtree(tempdir)
- 
-

=== removed file 'patches/plugins-no-plugins.patch'
--- patches/plugins-no-plugins.patch	2005-06-22 06:38:18 +0000
+++ patches/plugins-no-plugins.patch	1970-01-01 00:00:00 +0000
@@ -1,519 +0,0 @@
-*** added file 'bzrlib/plugin.py'
---- /dev/null
-+++ bzrlib/plugin.py
-@@ -0,0 +1,92 @@
-+# Copyright (C) 2004, 2005 by Canonical Ltd
-+
-+# This program is free software; you can redistribute it and/or modify
-+# it under the terms of the GNU General Public License as published by
-+# the Free Software Foundation; either version 2 of the License, or
-+# (at your option) any later version.
-+
-+# This program is distributed in the hope that it will be useful,
-+# but WITHOUT ANY WARRANTY; without even the implied warranty of
-+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+# GNU General Public License for more details.
-+
-+# You should have received a copy of the GNU General Public License
-+# along with this program; if not, write to the Free Software
-+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+
-+# This module implements plug-in support.
-+# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-+# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-+# update any bzrlib registries it wants to extend; for example, to add new
-+# commands, import bzrlib.commands and add your new command to the
-+# plugin_cmds variable.
-+
-+import sys, os, imp
-+try:
-+    set
-+except NameError:
-+    from sets import Set as set
-+from bzrlib.trace import log_error
-+
-+
-+def load_plugins():
-+    """Find all python files which are plugins, and load them
-+
-+    The environment variable BZR_PLUGIN_PATH is considered a delimited set of
-+    paths to look through. Each entry is searched for *.py files (and whatever
-+    other extensions are used in the platform, such as *.pyd).
-+    """
-+    bzrpath = os.environ.get('BZR_PLUGIN_PATH', os.path.expanduser('~/.bzr/plugins'))
-+
-+    # The problem with imp.get_suffixes() is that it doesn't include
-+    # .pyo which is technically valid
-+    # It also means that "testmodule.so" will show up as both test and testmodule
-+    # though it is only valid as 'test'
-+    # but you should be careful, because "testmodule.py" loads as testmodule.
-+    suffixes = imp.get_suffixes()
-+    suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
-+    package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
-+    for d in bzrpath.split(os.pathsep):
-+        # going trough them one by one allows different plugins with the same
-+        # filename in different directories in the path
-+        if not d:
-+            continue
-+        plugin_names = set()
-+        if not os.path.isdir(d):
-+            continue
-+        for f in os.listdir(d):
-+            path = os.path.join(d, f)
-+            if os.path.isdir(path):
-+                for entry in package_entries:
-+                    # This directory should be a package, and thus added to
-+                    # the list
-+                    if os.path.isfile(os.path.join(path, entry)):
-+                        break
-+                else: # This directory is not a package
-+                    continue
-+            else:
-+                for suffix_info in suffixes:
-+                    if f.endswith(suffix_info[0]):
-+                        f = f[:-len(suffix_info[0])]
-+                        if suffix_info[2] == imp.C_EXTENSION and f.endswith('module'):
-+                            f = f[:-len('module')]
-+                        break
-+                else:
-+                    continue
-+            plugin_names.add(f)
-+
-+        plugin_names = list(plugin_names)
-+        plugin_names.sort()
-+        for name in plugin_names:
-+            try:
-+                plugin_info = imp.find_module(name, [d])
-+                try:
-+                    plugin = imp.load_module('bzrlib.plugin.' + name,
-+                                             *plugin_info)
-+                finally:
-+                    if plugin_info[0] is not None:
-+                        plugin_info[0].close()
-+            except Exception, e:
-+                log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
-+
-
-*** modified file 'bzrlib/__init__.py'
---- bzrlib/__init__.py
-+++ bzrlib/__init__.py
-@@ -23,6 +23,7@@
- from diff import compare_trees
- from trace import mutter, warning, open_tracefile
- from log import show_log
-+from plugin import load_plugins
- import add
-
- BZRDIR = ".bzr"
-@@ -62,4 +63,4 @@
-             return None
-     except BzrError:
-         return None
--
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py
-+++ bzrlib/commands.py
-@@ -24,6 +24,24 @@
- from bzrlib.osutils import quotefn
- from bzrlib import Branch, Inventory, InventoryEntry, BZRDIR, \
-      format_date
-+
-+
-+plugin_cmds = {}
-+
-+
-+def register_plugin_command(cmd):
-+    "Utility function to help register a command"
-+    global plugin_cmds
-+    k = cmd.__name__
-+    if k.startswith("cmd_"):
-+        k_unsquished = _unsquish_command_name(k)
-+    else:
-+        k_unsquished = k
-+    if not plugin_cmds.has_key(k_unsquished):
-+        plugin_cmds[k_unsquished] = cmd
-+    else:
-+        log_error('Two plugins defined the same command: %r' % k)
-+        log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
-
-
- def _squish_command_name(cmd):
-@@ -68,100 +86,34 @@
-         revs = int(revstr)
-     return revs
-
--def _find_plugins():
--    """Find all python files which are plugins, and load their commands
--    to add to the list of "all commands"
--
--    The environment variable BZRPATH is considered a delimited set of
--    paths to look through. Each entry is searched for *.py files.
--    If a directory is found, it is also searched, but they are
--    not searched recursively. This allows you to revctl the plugins.
--
--    Inside the plugin should be a series of cmd_* function, which inherit from
--    the bzrlib.commands.Command class.
--    """
--    bzrpath = os.environ.get('BZRPLUGINPATH', '')
--
--    plugin_cmds = {}
--    if not bzrpath:
--        return plugin_cmds
--    _platform_extensions = {
--        'win32':'.pyd',
--        'cygwin':'.dll',
--        'darwin':'.dylib',
--        'linux2':'.so'
--        }
--    if _platform_extensions.has_key(sys.platform):
--        platform_extension = _platform_extensions[sys.platform]
--    else:
--        platform_extension = None
--    for d in bzrpath.split(os.pathsep):
--        plugin_names = {} # This should really be a set rather than a dict
--        for f in os.listdir(d):
--            if f.endswith('.py'):
--                f = f[:-3]
--            elif f.endswith('.pyc') or f.endswith('.pyo'):
--                f = f[:-4]
--            elif platform_extension and f.endswith(platform_extension):
--                f = f[:-len(platform_extension)]
--                if f.endswidth('module'):
--                    f = f[:-len('module')]
--            else:
--                continue
--            if not plugin_names.has_key(f):
--                plugin_names[f] = True
--
--        plugin_names = plugin_names.keys()
--        plugin_names.sort()
--        try:
--            sys.path.insert(0, d)
--            for name in plugin_names:
--                try:
--                    old_module = None
--                    try:
--                        if sys.modules.has_key(name):
--                            old_module = sys.modules[name]
--                            del sys.modules[name]
--                        plugin = __import__(name, locals())
--                        for k in dir(plugin):
--                            if k.startswith('cmd_'):
--                                k_unsquished = _unsquish_command_name(k)
--                                if not plugin_cmds.has_key(k_unsquished):
--                                    plugin_cmds[k_unsquished] = getattr(plugin, k)
--                                else:
--                                    log_error('Two plugins defined the same command: %r' % k)
--                                    log_error('Not loading the one in %r in dir %r' % (name, d))
--                    finally:
--                        if old_module:
--                            sys.modules[name] = old_module
--                except ImportError, e:
--                    log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
--        finally:
--            sys.path.pop(0)
--    return plugin_cmds
--
--def _get_cmd_dict(include_plugins=True):
-+def _get_cmd_dict(plugins_override=True):
-     d = {}
-     for k, v in globals().iteritems():
-         if k.startswith("cmd_"):
-             d[_unsquish_command_name(k)] = v
--    if include_plugins:
--        d.update(_find_plugins())
-+    # If we didn't load plugins, the plugin_cmds dict will be empty
-+    if plugins_override:
-+        d.update(plugin_cmds)
-+    else:
-+        d2 = {}
-+        d2.update(plugin_cmds)
-+        d2.update(d)
-+        d = d2
-     return d
-
--def get_all_cmds(include_plugins=True):
-+def get_all_cmds(plugins_override=True):
-     """Return canonical name and class for all registered commands."""
--    for k, v in _get_cmd_dict(include_plugins=include_plugins).iteritems():
-+    for k, v in _get_cmd_dict(plugins_override=plugins_override).iteritems():
-         yield k,v
-
-
--def get_cmd_class(cmd,include_plugins=True):
-+def get_cmd_class(cmd, plugins_override=True):
-     """Return the canonical name and command class for a command.
-     """
-     cmd = str(cmd)                      # not unicode
-
-     # first look up this command under the specified name
--    cmds = _get_cmd_dict(include_plugins=include_plugins)
-+    cmds = _get_cmd_dict(plugins_override=plugins_override)
-     try:
-         return cmd, cmds[cmd]
-     except KeyError:
-@@ -1461,6 +1413,75 @@
-     return argdict
-
-
-+def _parse_master_args(argv):
-+    """Parse the arguments that always go with the original command.
-+    These are things like bzr --no-plugins, etc.
-+
-+    There are now 2 types of option flags. Ones that come *before* the command,
-+    and ones that come *after* the command.
-+    Ones coming *before* the command are applied against all possible commands.
-+    And are generally applied before plugins are loaded.
-+
-+    The current list are:
-+        --builtin   Allow plugins to load, but don't let them override builtin commands,
-+                    they will still be allowed if they do not override a builtin.
-+        --no-plugins    Don't load any plugins. This lets you get back to official source
-+                        behavior.
-+        --profile   Enable the hotspot profile before running the command.
-+                    For backwards compatibility, this is also a non-master option.
-+        --version   Spit out the version of bzr that is running and exit.
-+                    This is also a non-master option.
-+        --help      Run help and exit, also a non-master option (I think that should stay, though)
-+
-+    >>> argv, opts = _parse_master_args(['bzr', '--test'])
-+    Traceback (most recent call last):
-+    ...
-+    BzrCommandError: Invalid master option: 'test'
-+    >>> argv, opts = _parse_master_args(['bzr', '--version', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['version']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--profile', 'command', '--more-options'])
-+    >>> print argv
-+    ['command', '--more-options']
-+    >>> print opts['profile']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--no-plugins', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['no-plugins']
-+    True
-+    >>> print opts['profile']
-+    False
-+    >>> argv, opts = _parse_master_args(['bzr', 'command', '--profile'])
-+    >>> print argv
-+    ['command', '--profile']
-+    >>> print opts['profile']
-+    False
-+    """
-+    master_opts = {'builtin':False,
-+        'no-plugins':False,
-+        'version':False,
-+        'profile':False,
-+        'help':False
-+    }
-+
-+    # This is the point where we could hook into argv[0] to determine
-+    # what front-end is supposed to be run
-+    # For now, we are just ignoring it.
-+    cmd_name = argv.pop(0)
-+    for arg in argv[:]:
-+        if arg[:2] != '--': # at the first non-option, we return the rest
-+            break
-+        arg = arg[2:] # Remove '--'
-+        if arg not in master_opts:
-+            # We could say that this is not an error, that we should
-+            # just let it be handled by the main section instead
-+            raise BzrCommandError('Invalid master option: %r' % arg)
-+        argv.pop(0) # We are consuming this entry
-+        master_opts[arg] = True
-+    return argv, master_opts
-
- def run_bzr(argv):
-     """Execute a command.
-@@ -1470,22 +1491,21 @@
-     """
-     argv = [a.decode(bzrlib.user_encoding) for a in argv]
-
--    include_plugins=True
-     try:
--        args, opts = parse_args(argv[1:])
--        if 'help' in opts:
-+        argv, master_opts = _parse_master_args(argv)
-+        if not master_opts['no-plugins']:
-+            bzrlib.load_plugins()
-+        args, opts = parse_args(argv)
-+        if 'help' in opts or master_opts['help']:
-             import help
-             if args:
-                 help.help(args[0])
-             else:
-                 help.help()
-             return 0
--        elif 'version' in opts:
-+        elif 'version' in opts or master_opts['version']:
-             show_version()
-             return 0
--        elif args and args[0] == 'builtin':
--            include_plugins=False
--            args = args[1:]
-         cmd = str(args.pop(0))
-     except IndexError:
-         import help
-@@ -1493,14 +1513,15 @@
-         return 1
-
-
--    canonical_cmd, cmd_class = get_cmd_class(cmd,include_plugins=include_plugins)
--
--    # global option
-+    plugins_override = not (master_opts['builtin'])
-+    canonical_cmd, cmd_class = get_cmd_class(cmd, plugins_override=plugins_override)
-+
-+    profile = master_opts['profile']
-+    # For backwards compatibility, I would rather stick with --profile being a
-+    # master/global option
-     if 'profile' in opts:
-         profile = True
-         del opts['profile']
--    else:
--        profile = False
-
-     # check options are reasonable
-     allowed = cmd_class.takes_options
-
-*** modified file 'testbzr'
---- testbzr
-+++ testbzr
-@@ -149,6 +149,7 @@
-     """Run a test involving creating a plugin to load,
-     and making sure it is seen properly.
-     """
-+    orig_help = backtick('bzr help commands') # No plugins yet
-     mkdir('plugin_test')
-     f = open(os.path.join('plugin_test', 'myplug.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
-@@ -157,24 +158,36 @@
-     aliases = ['mplg']
-     def run(self):
-         print 'Hello from my plugin'
-+class cmd_myplug_with_opt(bzrlib.commands.Command):
-+    '''A simple plugin that requires a special option'''
-+    takes_options = ['aspecialoptionthatdoesntexist']
-+    def run(self, aspecialoptionthatdoesntexist=None):
-+        print 'Found: %s' % aspecialoptionthatdoesntexist
-+
-+bzrlib.commands.register_plugin_command(cmd_myplug)
-+bzrlib.commands.register_plugin_command(cmd_myplug_with_opt)
-+bzrlib.commands.OPTIONS['aspecialoptionthatdoesntexist'] = str
- """)
-     f.close()
-
--    os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
--    help = backtick('bzr help commands')
-+    os.environ['BZR_PLUGIN_PATH'] = os.path.abspath('plugin_test')
-+    help = backtick('bzr help commands') #Help with user-visible plugins
-     assert help.find('myplug') != -1
-     assert help.find('Just a simple test plugin.') != -1
-
-
-     assert backtick('bzr myplug') == 'Hello from my plugin\n'
-     assert backtick('bzr mplg') == 'Hello from my plugin\n'
-+    assert backtick('bzr myplug-with-opt') == 'Found: None\n'
-+    assert backtick('bzr myplug-with-opt --aspecialoptionthatdoesntexist=2') == 'Found: 2\n'
-
-     f = open(os.path.join('plugin_test', 'override.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
--class cmd_commit(bzrlib.commands.cmd_commit):
--    '''Commit changes into a new revision.'''
-+class cmd_revno(bzrlib.commands.cmd_revno):
-+    '''Show current revision number.'''
-     def run(self, *args, **kwargs):
-         print "I'm sorry dave, you can't do that"
-+        return 1
-
- class cmd_help(bzrlib.commands.cmd_help):
-     '''Show help on a command or other topic.'''
-@@ -182,16 +195,67 @@
-         print "You have been overridden"
-         bzrlib.commands.cmd_help.run(self, *args, **kwargs)
-
-+bzrlib.commands.register_plugin_command(cmd_revno)
-+bzrlib.commands.register_plugin_command(cmd_help)
- """)
-     f.close()
-
--    newhelp = backtick('bzr help commands')
-+    newhelp = backtick('bzr help commands') # Help with no new commands,
-     assert newhelp.startswith('You have been overridden\n')
-     # We added a line, but the rest should work
-     assert newhelp[25:] == help
--
--    assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
--
-+    # Make sure we can get back to the original command
-+    # Not overridden, and no extra commands present
-+    assert backtick('bzr --builtin help commands') == help
-+    assert backtick('bzr --no-plugins help commands') == orig_help
-+
-+    assert backtick('bzr revno', retcode=1) == "I'm sorry dave, you can't do that\n"
-+
-+    print_txt = '** Loading noop plugin'
-+    f = open(os.path.join('plugin_test', 'loading.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class cmd_noop(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        pass
-+
-+print %r
-+bzrlib.commands.register_plugin_command(cmd_noop)
-+""" % print_txt)
-+    f.close()
-+    print_txt += '\n'
-+
-+    # Check that --builtin still loads the plugin, and enables it as
-+    # an extra command, but not as an override
-+    # and that --no-plugins doesn't load the command at all
-+    assert backtick('bzr noop') == print_txt
-+    assert backtick('bzr --builtin help')[:len(print_txt)] == print_txt
-+    assert backtick('bzr --no-plugins help')[:len(print_txt)] != print_txt
-+    runcmd('bzr revno', retcode=1)
-+    runcmd('bzr --builtin revno', retcode=0)
-+    runcmd('bzr --no-plugins revno', retcode=0)
-+    runcmd('bzr --builtin noop', retcode=0)
-+    runcmd('bzr --no-plugins noop', retcode=1)
-+
-+    # Check that packages can also be loaded
-+    test_str = 'packages work'
-+    os.mkdir(os.path.join('plugin_test', 'testpkg'))
-+    f = open(os.path.join('plugin_test', 'testpkg', '__init__.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class testpkgcmd(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        print %r
-+
-+bzrlib.commands.register_plugin_command(testpkgcmd)
-+""" % test_str)
-+    f.close()
-+    test_str += '\n'
-+    assert backtick('bzr testpkgcmd') == print_txt + test_str
-+    runcmd('bzr --no-plugins testpkgcmd', retcode=1)
-+
-+    # Make sure that setting BZR_PLUGIN_PATH to empty is the same as using --no-plugins
-+    os.environ['BZR_PLUGIN_PATH'] = ''
-+    assert backtick('bzr help commands') == orig_help
-+
-     shutil.rmtree('plugin_test')
-
- try:
-@@ -221,6 +285,9 @@
-
-     runcmd(['mkdir', TESTDIR])
-     cd(TESTDIR)
-+    # This means that any command that is naively run in this directory
-+    # Won't affect the parent directory.
-+    runcmd('bzr init')
-     test_root = os.getcwd()
-
-     progress("introductory commands")
-

=== removed file 'patches/progress.diff'
--- patches/progress.diff	2005-06-10 06:34:26 +0000
+++ patches/progress.diff	1970-01-01 00:00:00 +0000
@@ -1,143 +0,0 @@
-*** added file 'bzrlib/progress.py'
---- /dev/null 
-+++ bzrlib/progress.py 
-@@ -0,0 +1,138 @@
-+# Copyright (C) 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import datetime
-+
-+class Progress(object):
-+    def __init__(self, units, current, total=None):
-+        self.units = units
-+        self.current = current
-+        self.total = total
-+
-+    def _get_percent(self):
-+        if self.total is not None and self.current is not None:
-+            return 100.0 * self.current / self.total
-+
-+    percent = property(_get_percent)
-+
-+    def __str__(self):
-+        if self.total is not None:
-+            return "%i of %i %s %.1f%%" % (self.current, self.total, self.units,
-+                                         self.percent)
-+        else:
-+            return "%i %s" (self.current, self.units) 
-+
-+class ProgressBar(object):
-+    def __init__(self):
-+        self.start = None
-+        object.__init__(self)
-+
-+    def __call__(self, progress):
-+        if self.start is None:
-+            self.start = datetime.datetime.now()
-+        progress_bar(progress, start_time=self.start)
-+        
-+def divide_timedelta(delt, divisor):
-+    """Divides a timedelta object"""
-+    return datetime.timedelta(float(delt.days)/divisor, 
-+                              float(delt.seconds)/divisor, 
-+                              float(delt.microseconds)/divisor)
-+
-+def str_tdelta(delt):
-+    if delt is None:
-+        return "-:--:--"
-+    return str(datetime.timedelta(delt.days, delt.seconds))
-+
-+def get_eta(start_time, progress, enough_samples=20):
-+    if start_time is None or progress.current == 0:
-+        return None
-+    elif progress.current < enough_samples:
-+        return None
-+    elapsed = datetime.datetime.now() - start_time
-+    total_duration = divide_timedelta((elapsed) * long(progress.total), 
-+                                      progress.current)
-+    if elapsed < total_duration:
-+        eta = total_duration - elapsed
-+    else:
-+        eta = total_duration - total_duration
-+    return eta
-+
-+def progress_bar(progress, start_time=None):
-+    eta = get_eta(start_time, progress)
-+    if start_time is not None:
-+        eta_str = " "+str_tdelta(eta)
-+    else:
-+        eta_str = ""
-+
-+    fmt = " %i of %i %s (%.1f%%)"
-+    f = fmt % (progress.total, progress.total, progress.units, 100.0)
-+    max = len(f)
-+    cols = 77 - max
-+    if start_time is not None:
-+        cols -= len(eta_str)
-+    markers = int (float(cols) * progress.current / progress.total)
-+    txt = fmt % (progress.current, progress.total, progress.units,
-+                 progress.percent)
-+    sys.stderr.write("\r[%s%s]%s%s" % ('='*markers, ' '*(cols-markers), txt, 
-+                                       eta_str))
-+
-+def clear_progress_bar():
-+    sys.stderr.write('\r%s\r' % (' '*79))
-+
-+def spinner_str(progress, show_text=False):
-+    """
-+    Produces the string for a textual "spinner" progress indicator
-+    :param progress: an object represinting current progress
-+    :param show_text: If true, show progress text as well
-+    :return: The spinner string
-+
-+    >>> spinner_str(Progress("baloons", 0))
-+    '|'
-+    >>> spinner_str(Progress("baloons", 5))
-+    '/'
-+    >>> spinner_str(Progress("baloons", 6), show_text=True)
-+    '- 6 baloons'
-+    """
-+    positions = ('|', '/', '-', '\\')
-+    text = positions[progress.current % 4]
-+    if show_text:
-+        text+=" %i %s" % (progress.current, progress.units)
-+    return text
-+
-+def spinner(progress, show_text=False, output=sys.stderr):
-+    """
-+    Update a spinner progress indicator on an output
-+    :param progress: The progress to display
-+    :param show_text: If true, show text as well as spinner
-+    :param output: The output to write to
-+
-+    >>> spinner(Progress("baloons", 6), show_text=True, output=sys.stdout)
-+    \r- 6 baloons
-+    """
-+    output.write('\r%s' % spinner_str(progress, show_text))
-+
-+def run_tests():
-+    import doctest
-+    result = doctest.testmod()
-+    if result[1] > 0:
-+        if result[0] == 0:
-+            print "All tests passed"
-+    else:
-+        print "No tests to run"
-+if __name__ == "__main__":
-+    run_tests()
-

=== removed file 'patches/symlink-support.patch'
--- patches/symlink-support.patch	2005-05-17 07:01:47 +0000
+++ patches/symlink-support.patch	1970-01-01 00:00:00 +0000
@@ -1,629 +0,0 @@
-Return-Path: <erik@tntech.dk>
-X-Original-To: mbp@sourcefrog.net
-Delivered-To: mbp@ozlabs.org
-X-Greylist: delayed 1826 seconds by postgrey-1.21 at ozlabs; Sun, 15 May 2005 06:59:11 EST
-Received: from upstroke.tntech.dk (cpe.atm2-0-1041078.0x503eaf62.odnxx4.customer.tele.dk [80.62.175.98])
-	by ozlabs.org (Postfix) with ESMTP id B968E679EA
-	for <mbp@sourcefrog.net>; Sun, 15 May 2005 06:59:11 +1000 (EST)
-Received: by upstroke.tntech.dk (Postfix, from userid 1001)
-	id 63F83542FF; Sat, 14 May 2005 22:28:37 +0200 (CEST)
-To: Martin Pool <mbp@sourcefrog.net>
-Subject: [PATCH] symlink support patch
-From: Erik Toubro Nielsen <erik@tntech.dk>
-Date: Sat, 14 May 2005 22:28:37 +0200
-Message-ID: <86u0l57dsa.fsf@upstroke.tntech.dk>
-User-Agent: Gnus/5.1006 (Gnus v5.10.6) XEmacs/21.4 (Security Through
- Obscurity, linux)
-MIME-Version: 1.0
-Content-Type: multipart/mixed; boundary="=-=-="
-X-Spam-Checker-Version: SpamAssassin 3.0.3 (2005-04-27) on ozlabs.org
-X-Spam-Level: 
-X-Spam-Status: No, score=-1.4 required=3.2 tests=BAYES_00,NO_MORE_FUNN,
-	RCVD_IN_BLARS_RBL autolearn=no version=3.0.3
-Content-Length: 19688
-Lines: 604
-
---=-=-=
-
-I'm not sending this to the list as it is pretty large. 
-
-Let me know if its usefull and if I should rework anything.
-
-Erik
-
-
-Overview:
-
-        (Bugfix: in class TreeDelta I've moved kind= one level up, since
-        kind is also used in the else part)
-
-        Since both the InventoryEntry and stat cache is changed, perhaps
-        the branch format number should be increased?
-
-        Added test cases for symlinks to testbzr
-
-        Cannot use realpath since it expands path/L to path/LinkTarget
-        Cannot use exists, use new osutils.lexists
-
-        I'm overloading the text_modified to signify that a link        
-        target is changed. Perhaps text_modified should be renamed
-        content_modified?
-
-        InventoryEntry has a new member "symlink_target", 
-
-        The stat cache entry has been extended to contain the symlink
-        target and the st_mode. I try to ignore an old format cache
-        file.
-
-
-
---=-=-=
-Content-Type: text/x-patch
-Content-Disposition: inline; filename=symlinksupport.patch
-
-*** modified file 'bzrlib/add.py'
---- bzrlib/add.py 
-+++ bzrlib/add.py 
-@@ -38,7 +38,7 @@
-     for f in file_list:
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             if f not in user_list:
-                 print "Skipping %s (can't add file of kind '%s')" % (f, kind)
-                 continue
-@@ -56,7 +56,7 @@
- 
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             bailout("can't add file '%s' of kind %r" % (f, kind))
-             
-         versioned = (inv.path2id(rf) != None)
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -58,11 +58,9 @@
-     run into the root."""
-     if f == None:
-         f = os.getcwd()
--    elif hasattr(os.path, 'realpath'):
--        f = os.path.realpath(f)
-     else:
--        f = os.path.abspath(f)
--    if not os.path.exists(f):
-+        f = bzrlib.osutils.normalizepath(f)
-+    if not bzrlib.osutils.lexists(f):
-         raise BzrError('%r does not exist' % f)
-         
- 
-@@ -189,7 +187,7 @@
-         """Return path relative to this branch of something inside it.
- 
-         Raises an error if path is not in this branch."""
--        rp = os.path.realpath(path)
-+        rp = bzrlib.osutils.normalizepath(path)
-         # FIXME: windows
-         if not rp.startswith(self.base):
-             bailout("path %r is not within branch %r" % (rp, self.base))
-@@ -531,7 +529,9 @@
-             file_id = entry.file_id
-             mutter('commit prep file %s, id %r ' % (p, file_id))
- 
--            if not os.path.exists(p):
-+            # it should be enough to use os.lexists instead of exists
-+            # but lexists in an 2.4 function
-+            if not bzrlib.osutils.lexists(p):
-                 mutter("    file is missing, removing from inventory")
-                 if verbose:
-                     show_status('D', entry.kind, quotefn(path))
-@@ -554,6 +554,10 @@
-             if entry.kind == 'directory':
-                 if not isdir(p):
-                     bailout("%s is entered as directory but not a directory" % quotefn(p))
-+            elif entry.kind == 'symlink':
-+                if not os.path.islink(p):
-+                    bailout("%s is entered as symbolic link but is not a symbolic link" % quotefn(p))
-+                entry.read_symlink_target(p)
-             elif entry.kind == 'file':
-                 if not isfile(p):
-                     bailout("%s is entered as file but is not a file" % quotefn(p))
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -66,6 +66,11 @@
-         print >>to_file, "\\ No newline at end of file"
-     print >>to_file
- 
-+def _diff_symlink(old_tree, new_tree, file_id):
-+    t1 = old_tree.get_symlink_target(file_id)
-+    t2 = new_tree.get_symlink_target(file_id)
-+    print '*** *** target changed %r => %r' % (t1, t2)
-+    
- 
- def show_diff(b, revision, specific_files):
-     import sys
-@@ -112,12 +117,15 @@
- 
-     for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-         print '*** renamed %s %r => %r' % (kind, old_path, new_path)
--        if text_modified:
-+        if kind == 'file' and text_modified:
-             _diff_one(old_tree.get_file(file_id).readlines(),
-                    new_tree.get_file(file_id).readlines(),
-                    sys.stdout,
-                    fromfile=old_label + old_path,
-                    tofile=new_label + new_path)
-+
-+        elif kind == 'symlink' and text_modified:
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
-     for path, file_id, kind in delta.modified:
-         print '*** modified %s %r' % (kind, path)
-@@ -128,6 +136,8 @@
-                    fromfile=old_label + path,
-                    tofile=new_label + path)
- 
-+        elif kind == 'symlink':
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
- 
- class TreeDelta:
-@@ -149,7 +159,9 @@
-     Each id is listed only once.
- 
-     Files that are both modified and renamed are listed only in
--    renamed, with the text_modified flag true.
-+    renamed, with the text_modified flag true. The text_modified
-+    applies either to the the content of the file or the target of the
-+    symbolic link, depending of the kind of file.
- 
-     The lists are normally sorted when the delta is created.
-     """
-@@ -224,8 +236,8 @@
-         specific_files = ImmutableSet(specific_files)
- 
-     for file_id in old_tree:
-+        kind = old_inv.get_file_kind(file_id)
-         if file_id in new_tree:
--            kind = old_inv.get_file_kind(file_id)
-             assert kind == new_inv.get_file_kind(file_id)
-             
-             assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
-@@ -246,6 +258,14 @@
-                 old_sha1 = old_tree.get_file_sha1(file_id)
-                 new_sha1 = new_tree.get_file_sha1(file_id)
-                 text_modified = (old_sha1 != new_sha1)
-+            elif kind == 'symlink':
-+                t1 = old_tree.get_symlink_target(file_id)
-+                t2 = new_tree.get_symlink_target(file_id)
-+                if t1 != t2:
-+                    mutter("    symlink target changed")
-+                    text_modified = True
-+                else:
-+                    text_modified = False
-             else:
-                 ## mutter("no text to check for %r %r" % (file_id, kind))
-                 text_modified = False
-
-*** modified file 'bzrlib/inventory.py'
---- bzrlib/inventory.py 
-+++ bzrlib/inventory.py 
-@@ -125,14 +125,22 @@
-         self.kind = kind
-         self.text_id = text_id
-         self.parent_id = parent_id
-+        self.symlink_target = None
-         if kind == 'directory':
-             self.children = {}
-         elif kind == 'file':
-             pass
-+        elif kind == 'symlink':
-+            pass
-         else:
-             raise BzrError("unhandled entry kind %r" % kind)
- 
--
-+    def read_symlink_target(self, path):
-+        if self.kind == 'symlink':
-+            try:
-+                self.symlink_target = os.readlink(path)
-+            except OSError,e:
-+                raise BzrError("os.readlink error, %s" % e)
- 
-     def sorted_children(self):
-         l = self.children.items()
-@@ -145,6 +153,7 @@
-                                self.parent_id, text_id=self.text_id)
-         other.text_sha1 = self.text_sha1
-         other.text_size = self.text_size
-+        other.symlink_target = self.symlink_target
-         return other
- 
- 
-@@ -168,7 +177,7 @@
-         if self.text_size != None:
-             e.set('text_size', '%d' % self.text_size)
-             
--        for f in ['text_id', 'text_sha1']:
-+        for f in ['text_id', 'text_sha1', 'symlink_target']:
-             v = getattr(self, f)
-             if v != None:
-                 e.set(f, v)
-@@ -198,6 +207,7 @@
-         self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-         self.text_id = elt.get('text_id')
-         self.text_sha1 = elt.get('text_sha1')
-+        self.symlink_target = elt.get('symlink_target')
-         
-         ## mutter("read inventoryentry: %r" % (elt.attrib))
- 
-
-*** modified file 'bzrlib/osutils.py'
---- bzrlib/osutils.py 
-+++ bzrlib/osutils.py 
-@@ -58,7 +58,30 @@
-     else:
-         raise BzrError("can't handle file kind with mode %o of %r" % (mode, f)) 
- 
--
-+def lexists(f):
-+    try:
-+        if hasattr(os, 'lstat'):
-+            os.lstat(f)
-+        else:
-+            os.stat(f)
-+        return True
-+    except OSError,e:
-+        if e.errno == errno.ENOENT:
-+            return False;
-+        else:
-+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
-+
-+def normalizepath(f):
-+    if hasattr(os.path, 'realpath'):
-+        F = os.path.realpath
-+    else:
-+        F = os.path.abspath
-+    [p,e] = os.path.split(f)
-+    if e == "" or e == "." or e == "..":
-+        return F(f)
-+    else:
-+        return os.path.join(F(p), e)
-+    
- 
- def isdir(f):
-     """True if f is an accessible directory."""
-
-*** modified file 'bzrlib/statcache.py'
---- bzrlib/statcache.py 
-+++ bzrlib/statcache.py 
-@@ -53,7 +53,7 @@
- to use a tdb instead.
- 
- The cache is represented as a map from file_id to a tuple of (file_id,
--sha1, path, size, mtime, ctime, ino, dev).
-+sha1, path, symlink target, size, mtime, ctime, ino, dev, mode).
- """
- 
- 
-@@ -62,11 +62,13 @@
- FP_CTIME = 2
- FP_INO   = 3
- FP_DEV   = 4
--
-+FP_ST_MODE=5
- 
- SC_FILE_ID = 0
- SC_SHA1    = 1 
--
-+SC_SYMLINK_TARGET = 3
-+
-+CACHE_ENTRY_SIZE = 10
- 
- def fingerprint(abspath):
-     try:
-@@ -79,7 +81,7 @@
-         return None
- 
-     return (fs.st_size, fs.st_mtime,
--            fs.st_ctime, fs.st_ino, fs.st_dev)
-+            fs.st_ctime, fs.st_ino, fs.st_dev, fs.st_mode)
- 
- 
- def _write_cache(basedir, entry_iter, dangerfiles):
-@@ -93,7 +95,9 @@
-                 continue
-             outf.write(entry[0] + ' ' + entry[1] + ' ')
-             outf.write(b2a_qp(entry[2], True))
--            outf.write(' %d %d %d %d %d\n' % entry[3:])
-+            outf.write(' ')
-+            outf.write(b2a_qp(entry[3], True)) # symlink_target
-+            outf.write(' %d %d %d %d %d %d\n' % entry[4:])
- 
-         outf.commit()
-     finally:
-@@ -114,10 +118,13 @@
-     
-     for l in cachefile:
-         f = l.split(' ')
-+        if len(f) != CACHE_ENTRY_SIZE:
-+            mutter("cache is in old format, must recreate it")
-+            return {}
-         file_id = f[0]
-         if file_id in cache:
-             raise BzrError("duplicated file_id in cache: {%s}" % file_id)
--        cache[file_id] = (f[0], f[1], a2b_qp(f[2])) + tuple([long(x) for x in f[3:]])
-+        cache[file_id] = (f[0], f[1], a2b_qp(f[2]), a2b_qp(f[3])) + tuple([long(x) for x in f[4:]])
-     return cache
- 
- 
-@@ -125,7 +132,7 @@
- 
- def _files_from_inventory(inv):
-     for path, ie in inv.iter_entries():
--        if ie.kind != 'file':
-+        if ie.kind != 'file' and ie.kind != 'symlink':
-             continue
-         yield ie.file_id, path
-     
-@@ -190,17 +197,24 @@
-         if (fp[FP_MTIME] >= now) or (fp[FP_CTIME] >= now):
-             dangerfiles.add(file_id)
- 
--        if cacheentry and (cacheentry[3:] == fp):
-+        if cacheentry and (cacheentry[4:] == fp):
-             continue                    # all stat fields unchanged
- 
-         hardcheck += 1
- 
--        dig = sha.new(file(abspath, 'rb').read()).hexdigest()
--
-+        mode = fp[FP_ST_MODE]
-+        if stat.S_ISREG(mode):
-+            link_target = '-' # can be anything, but must be non-empty
-+            dig = sha.new(file(abspath, 'rb').read()).hexdigest()
-+        elif stat.S_ISLNK(mode):
-+            link_target = os.readlink(abspath)
-+            dig = sha.new(link_target).hexdigest()
-+        else:
-+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
-         if cacheentry == None or dig != cacheentry[1]: 
-             # if there was no previous entry for this file, or if the
-             # SHA has changed, then update the cache
--            cacheentry = (file_id, dig, path) + fp
-+            cacheentry = (file_id, dig, path, link_target) + fp
-             cache[file_id] = cacheentry
-             change_cnt += 1
- 
-
-*** modified file 'bzrlib/tree.py'
---- bzrlib/tree.py 
-+++ bzrlib/tree.py 
-@@ -125,6 +125,11 @@
-                 os.mkdir(fullpath)
-             elif kind == 'file':
-                 pumpfile(self.get_file(ie.file_id), file(fullpath, 'wb'))
-+            elif kind == 'symlink':
-+                try:
-+                    os.symlink(ie.symlink_target, fullpath)
-+                except OSError,e:
-+                    bailout("Failed to create symlink %r -> %r, error: %s" % (fullpath, ie.symlink_target, e))
-             else:
-                 bailout("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-             mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
-@@ -167,6 +172,9 @@
-         for path, entry in self.inventory.iter_entries():
-             yield path, 'V', entry.kind, entry.file_id
- 
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]
-+        return ie.symlink_target;
- 
- class EmptyTree(Tree):
-     def __init__(self):
-@@ -179,7 +187,8 @@
-         if False:  # just to make it a generator
-             yield None
-     
--
-+    def get_symlink_target(self, file_id):
-+        return None
- 
- ######################################################################
- # diff
-@@ -245,3 +254,7 @@
-         if old_name != new_name:
-             yield (old_name, new_name)
-             
-+
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]        
-+        return ie.symlink_target
-
-*** modified file 'bzrlib/workingtree.py'
---- bzrlib/workingtree.py 
-+++ bzrlib/workingtree.py 
-@@ -53,7 +53,7 @@
-             ie = inv[file_id]
-             if ie.kind == 'file':
-                 if ((file_id in self._statcache)
--                    or (os.path.exists(self.abspath(inv.id2path(file_id))))):
-+                    or (bzrlib.osutils.lexists(self.abspath(inv.id2path(file_id))))):
-                     yield file_id
- 
- 
-@@ -66,7 +66,7 @@
-         return os.path.join(self.basedir, filename)
- 
-     def has_filename(self, filename):
--        return os.path.exists(self.abspath(filename))
-+        return bzrlib.osutils.lexists(self.abspath(filename))
- 
-     def get_file(self, file_id):
-         return self.get_file_byname(self.id2path(file_id))
-@@ -86,7 +86,7 @@
-         self._update_statcache()
-         if file_id in self._statcache:
-             return True
--        return os.path.exists(self.abspath(self.id2path(file_id)))
-+        return bzrlib.osutils.lexists(self.abspath(self.id2path(file_id)))
- 
- 
-     __contains__ = has_id
-@@ -108,6 +108,12 @@
-         return self._statcache[file_id][statcache.SC_SHA1]
- 
- 
-+    def get_symlink_target(self, file_id):
-+        import statcache
-+        self._update_statcache()
-+        target = self._statcache[file_id][statcache.SC_SYMLINK_TARGET]
-+        return target
-+        
-     def file_class(self, filename):
-         if self.path2id(filename):
-             return 'V'
-
-*** modified file 'testbzr'
---- testbzr 
-+++ testbzr 
-@@ -1,4 +1,4 @@
--#! /usr/bin/python
-+#! /usr/bin/env python
- 
- # Copyright (C) 2005 Canonical Ltd
- 
-@@ -113,6 +113,17 @@
-     logfile.write('   at %s:%d\n' % stack[:2])
- 
- 
-+def has_symlinks():
-+    import os;
-+    if hasattr(os, 'symlink'):
-+        return True
-+    else:
-+        return False
-+
-+def listdir_sorted(dir):
-+    L = os.listdir(dir)
-+    L.sort()
-+    return L
- 
- # prepare an empty scratch directory
- if os.path.exists(TESTDIR):
-@@ -320,8 +331,105 @@
-     runcmd('bzr ignore *.blah')
-     assert backtick('bzr unknowns') == ''
-     assert file('.bzrignore', 'rt').read() == '*.blah\n'
--
--
-+    cd("..")
-+    
-+    if has_symlinks():
-+        progress("symlinks")
-+        mkdir('symlinks')
-+        cd('symlinks')
-+        runcmd('bzr init')
-+        os.symlink("NOWHERE1", "link1")
-+        runcmd('bzr add link1')
-+        assert backtick('bzr unknowns') == ''
-+        runcmd(['bzr', 'commit', '-m', '1: added symlink link1'])
-+
-+        mkdir('d1')
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE2", "d1/link2")
-+        assert backtick('bzr unknowns') == 'd1/link2\n'
-+        # is d1/link2 found when adding d1
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE3", "d1/link3")
-+        assert backtick('bzr unknowns') == 'd1/link3\n'
-+        runcmd(['bzr', 'commit', '-m', '2: added dir, symlink'])
-+
-+        runcmd('bzr rename d1 d2')
-+        runcmd('bzr move d2/link2 .')
-+        runcmd('bzr move link1 d2')
-+        assert os.readlink("./link2") == "NOWHERE2"
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        runcmd('bzr add d2/link3')
-+        runcmd('bzr diff')
-+        runcmd(['bzr', 'commit', '-m', '3: rename of dir, move symlinks, add link3'])
-+
-+        os.unlink("link2")
-+        os.symlink("TARGET 2", "link2")
-+        os.unlink("d2/link1")
-+        os.symlink("TARGET 1", "d2/link1")
-+        runcmd('bzr diff')
-+        assert backtick("bzr relpath d2/link1") == "d2/link1\n"
-+        runcmd(['bzr', 'commit', '-m', '4: retarget of two links'])
-+
-+        runcmd('bzr remove d2/link1')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '5: remove d2/link1'])
-+
-+        os.mkdir("d1")
-+        runcmd('bzr add d1')
-+        runcmd('bzr rename d2/link3 d1/link3new')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '6: remove d2/link1, move/rename link3'])
-+        
-+        runcmd(['bzr', 'check'])
-+        
-+        runcmd(['bzr', 'export', '-r', '1', 'exp1.tmp'])
-+        cd("exp1.tmp")
-+        assert listdir_sorted(".") == [ "link1" ]
-+        assert os.readlink("link1") == "NOWHERE1"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '2', 'exp2.tmp'])
-+        cd("exp2.tmp")
-+        assert listdir_sorted(".") == [ "d1", "link1" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '3', 'exp3.tmp'])
-+        cd("exp3.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        assert os.readlink("link2")    == "NOWHERE2"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '4', 'exp4.tmp'])
-+        cd("exp4.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.readlink("d2/link1") == "TARGET 1"
-+        assert os.readlink("link2")    == "TARGET 2"
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '5', 'exp5.tmp'])
-+        cd("exp5.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.path.islink("link2")
-+        assert listdir_sorted("d2")== [ "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '6', 'exp6.tmp'])
-+        cd("exp6.tmp")
-+        assert listdir_sorted(".") == [ "d1", "d2", "link2" ]
-+        assert listdir_sorted("d1") == [ "link3new" ]
-+        assert listdir_sorted("d2") == []
-+        assert os.readlink("d1/link3new") == "NOWHERE3"
-+        cd("..")
-+
-+        cd("..")
-+    else:
-+        progress("skipping symlink tests")
-+        
-     progress("all tests passed!")
- except Exception, e:
-     sys.stderr.write('*' * 50 + '\n'
-
-
---=-=-=--

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2005-08-17 23:26:41 +0000
+++ setup.py	2006-03-22 18:03:25 +0000
@@ -4,20 +4,116 @@
 # './setup.py install', or
 # './setup.py --help' for more options
 
+# Reinvocation stolen from bzr, we need python2.4 by virtue of bzr_man
+# including bzrlib.help
+
+import os, sys
+
+try:
+    version_info = sys.version_info
+except AttributeError:
+    version_info = 1, 5 # 1.5 or older
+
+REINVOKE = "__BZR_REINVOKE"
+NEED_VERS = (2, 4)
+KNOWN_PYTHONS = ('python2.4',)
+
+if version_info < NEED_VERS:
+    if not os.environ.has_key(REINVOKE):
+        # mutating os.environ doesn't work in old Pythons
+        os.putenv(REINVOKE, "1")
+        for python in KNOWN_PYTHONS:
+            try:
+                os.execvp(python, [python] + sys.argv)
+            except OSError:
+                pass
+    print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
+    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
+    sys.exit(1)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
+
+
 from distutils.core import setup
+from distutils.command.install_scripts import install_scripts
+from distutils.command.build import build
+
+###############################
+# Overridden distutils actions
+###############################
+
+class my_install_scripts(install_scripts):
+    """ Customized install_scripts distutils action.
+    Create bzr.bat for win32.
+    """
+    def run(self):
+        import os
+        import sys
+
+        install_scripts.run(self)   # standard action
+
+        if sys.platform == "win32":
+            try:
+                scripts_dir = self.install_dir
+                script_path = os.path.join(scripts_dir, "bzr")
+                batch_str = "@%s %s %%*\n" % (sys.executable, script_path)
+                batch_path = script_path + ".bat"
+                f = file(batch_path, "w")
+                f.write(batch_str)
+                f.close()
+                print "Created:", batch_path
+            except Exception, e:
+                print "ERROR: Unable to create %s: %s" % (batch_path, e)
+
+
+class bzr_build(build):
+    """Customized build distutils action.
+    Generate bzr.1.
+    """
+    def run(self):
+        build.run(self)
+
+        import generate_docs
+        generate_docs.main(argv=["bzr", "man"])
+
+########################
+## Setup
+########################
 
 setup(name='bzr',
-      version='0.0.0',
+      version='0.8pre',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',
       description='Friendly distributed version control system',
       license='GNU GPL v2',
       packages=['bzrlib',
+                'bzrlib.doc',
+                'bzrlib.doc.api',
+                'bzrlib.export',
                 'bzrlib.plugins',
-                'bzrlib.selftest',
+                'bzrlib.store',
+                'bzrlib.store.revision',
+                'bzrlib.store.versioned',
+                'bzrlib.tests',
+                'bzrlib.tests.blackbox',
+                'bzrlib.tests.branch_implementations',
+                'bzrlib.tests.bzrdir_implementations',
+                'bzrlib.tests.interrepository_implementations',
+                'bzrlib.tests.interversionedfile_implementations',
+                'bzrlib.tests.repository_implementations',
+                'bzrlib.tests.revisionstore_implementations',
+                'bzrlib.tests.workingtree_implementations',
+                'bzrlib.transport',
+                'bzrlib.transport.http',
+                'bzrlib.ui',
                 'bzrlib.util',
                 'bzrlib.util.elementtree',
                 'bzrlib.util.effbot.org',
-                'bzrlib.util.urlgrabber'],
-      scripts=['bzr'])
+                'bzrlib.util.configobj',
+                ],
+      scripts=['bzr'],
+      cmdclass={'install_scripts': my_install_scripts, 'build': bzr_build},
+      data_files=[('man/man1', ['bzr.1'])],
+    #   todo: install the txt files from bzrlib.doc.api.
+     )

=== removed file 'testbzr'
--- testbzr	2005-06-22 06:37:43 +0000
+++ testbzr	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-#! /usr/bin/python
-# -*- coding: utf-8 -*-
-
-# Copyright (C) 2005 Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-print 'please use "bzr selftest" instead'
-import sys
-sys.exit(1)
-

=== removed file 'testsweet.py'
--- testsweet.py	2005-08-23 06:52:09 +0000
+++ testsweet.py	1970-01-01 00:00:00 +0000
@@ -1,365 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Enhanced layer on unittest.
-
-This does several things:
-
-* nicer reporting as tests run
-
-* test code can log messages into a buffer that is recorded to disk
-  and displayed if the test fails
-
-* tests can be run in a separate directory, which is useful for code that
-  wants to create files
-
-* utilities to run external commands and check their return code
-  and/or output
-
-Test cases should normally subclass testsweet.TestCase.  The test runner should
-call runsuite().
-
-This is meant to become independent of bzr, though that's not quite
-true yet.
-"""  
-
-import unittest
-import sys
-
-# XXX: Don't need this anymore now we depend on python2.4
-def _need_subprocess():
-    sys.stderr.write("sorry, this test suite requires the subprocess module\n"
-                     "this is shipped with python2.4 and available separately for 2.3\n")
-    
-
-class CommandFailed(Exception):
-    pass
-
-
-class TestSkipped(Exception):
-    """Indicates that a test was intentionally skipped, rather than failing."""
-    # XXX: Not used yet
-
-
-class TestCase(unittest.TestCase):
-    """Base class for bzr test cases.
-
-    Just defines some useful helper functions; doesn't actually test
-    anything.
-    """
-    
-    # TODO: Special methods to invoke bzr, so that we can run it
-    # through a specified Python intepreter
-
-    OVERRIDE_PYTHON = None # to run with alternative python 'python'
-    BZRPATH = 'bzr'
-
-    _log_buf = ""
-
-
-    def setUp(self):
-        super(TestCase, self).setUp()
-        # save stdout & stderr so there's no leakage from code-under-test
-        self.real_stdout = sys.stdout
-        self.real_stderr = sys.stderr
-        sys.stdout = sys.stderr = TestCase.TEST_LOG
-        self.log("%s setup" % self.id())
-
-    def tearDown(self):
-        sys.stdout = self.real_stdout
-        sys.stderr = self.real_stderr
-        self.log("%s teardown" % self.id())
-        self.log('')
-        super(TestCase, self).tearDown()
-
-    def formcmd(self, cmd):
-        if isinstance(cmd, basestring):
-            cmd = cmd.split()
-
-        if cmd[0] == 'bzr':
-            cmd[0] = self.BZRPATH
-            if self.OVERRIDE_PYTHON:
-                cmd.insert(0, self.OVERRIDE_PYTHON)
-
-        self.log('$ %r' % cmd)
-
-        return cmd
-
-
-    def runcmd(self, cmd, retcode=0):
-        """Run one command and check the return code.
-
-        Returns a tuple of (stdout,stderr) strings.
-
-        If a single string is based, it is split into words.
-        For commands that are not simple space-separated words, please
-        pass a list instead."""
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-
-        cmd = self.formcmd(cmd)
-
-        self.log('$ ' + ' '.join(cmd))
-        actual_retcode = call(cmd, stdout=self.TEST_LOG, stderr=self.TEST_LOG)
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-
-    def backtick(self, cmd, retcode=0):
-        """Run a command and return its output"""
-        try:
-            import shutil
-            from subprocess import Popen, PIPE
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-        cmd = self.formcmd(cmd)
-        child = Popen(cmd, stdout=PIPE, stderr=self.TEST_LOG)
-        outd, errd = child.communicate()
-        self.log(outd)
-        actual_retcode = child.wait()
-
-        outd = outd.replace('\r', '')
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-        return outd
-
-
-
-    def build_tree(self, shape):
-        """Build a test tree according to a pattern.
-
-        shape is a sequence of file specifications.  If the final
-        character is '/', a directory is created.
-
-        This doesn't add anything to a branch.
-        """
-        # XXX: It's OK to just create them using forward slashes on windows?
-        import os
-        for name in shape:
-            assert isinstance(name, basestring)
-            if name[-1] == '/':
-                os.mkdir(name[:-1])
-            else:
-                f = file(name, 'wt')
-                print >>f, "contents of", name
-                f.close()
-
-
-    def log(self, msg):
-        """Log a message to a progress file"""
-        # XXX: The problem with this is that code that writes straight
-        # to the log file won't be shown when we display the log
-        # buffer; would be better to not have the in-memory buffer and
-        # instead just a log file per test, which is read in and
-        # displayed if the test fails.  That seems to imply one log
-        # per test case, not globally.  OK?
-        self._log_buf = self._log_buf + str(msg) + '\n'
-        print >>self.TEST_LOG, msg
-
-
-    def check_inventory_shape(self, inv, shape):
-        """
-        Compare an inventory to a list of expected names.
-
-        Fail if they are not precisely equal.
-        """
-        extras = []
-        shape = list(shape)             # copy
-        for path, ie in inv.entries():
-            name = path.replace('\\', '/')
-            if ie.kind == 'dir':
-                name = name + '/'
-            if name in shape:
-                shape.remove(name)
-            else:
-                extras.append(name)
-        if shape:
-            self.fail("expected paths not found in inventory: %r" % shape)
-        if extras:
-            self.fail("unexpected paths found in inventory: %r" % extras)
-
-
-    def check_file_contents(self, filename, expect):
-        self.log("check contents of file %s" % filename)
-        contents = file(filename, 'r').read()
-        if contents != expect:
-            self.log("expected: %r" % expect)
-            self.log("actually: %r" % contents)
-            self.fail("contents of %s not as expected")
-            
-
-
-class InTempDir(TestCase):
-    """Base class for tests run in a temporary branch."""
-    def setUp(self):
-        super(InTempDir, self).setUp()
-        import os
-        self.test_dir = os.path.join(self.TEST_ROOT, self.__class__.__name__)
-        os.mkdir(self.test_dir)
-        os.chdir(self.test_dir)
-        
-    def tearDown(self):
-        import os
-        os.chdir(self.TEST_ROOT)
-        super(InTempDir, self).tearDown()
-
-
-class _MyResult(unittest._TextTestResult):
-    """
-    Custom TestResult.
-
-    No special behaviour for now.
-    """
-    def __init__(self, out, style):
-        super(_MyResult, self).__init__(out, False, 0)
-        self.out = out
-        assert style in ('none', 'progress', 'verbose')
-        self.style = style
-
-    def startTest(self, test):
-        super(_MyResult, self).startTest(test)
-        # TODO: Maybe show test.shortDescription somewhere?
-        what = test.id()
-        # python2.3 has the bad habit of just "runit" for doctests
-        if what == 'runit':
-            what = test.shortDescription()
-        if self.style == 'verbose':
-            print >>self.out, '%-60.60s' % what,
-            self.out.flush()
-
-    def addError(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'ERROR'
-        elif self.style == 'progress':
-            self.stream.write('E')
-        self.stream.flush()
-        super(_MyResult, self).addError(test, err)
-
-    def addFailure(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'FAILURE'
-        elif self.style == 'progress':
-            self.stream.write('F')
-        self.stream.flush()
-        super(_MyResult, self).addFailure(test, err)
-
-    def addSuccess(self, test):
-        if self.style == 'verbose':
-            print >>self.out, 'OK'
-        elif self.style == 'progress':
-            self.stream.write('~')
-        self.stream.flush()
-        super(_MyResult, self).addSuccess(test)
-
-    def printErrors(self):
-        if self.style == 'progress':
-            self.stream.writeln()
-        super(_MyResult, self).printErrors()
-
-    def printErrorList(self, flavour, errors):
-        for test, err in errors:
-            self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
-            self.stream.writeln(self.separator2)
-            self.stream.writeln("%s" % err)
-            if isinstance(test, TestCase):
-                self.stream.writeln()
-                self.stream.writeln('log from this test:')
-                print >>self.stream, test._log_buf
-
-
-class TestSuite(unittest.TestSuite):
-    
-    def __init__(self, tests=(), name='test'):
-        super(TestSuite, self).__init__(tests)
-        self._name = name
-
-    def run(self, result):
-        import os
-        import shutil
-        import time
-        
-        self._setup_test_log()
-        self._setup_test_dir()
-        print
-    
-        return super(TestSuite,self).run(result)
-
-    def _setup_test_log(self):
-        import time
-        import os
-        
-        log_filename = os.path.abspath(self._name + '.log')
-        # line buffered
-        TestCase.TEST_LOG = open(log_filename, 'wt', buffering=1)
-    
-        print >>TestCase.TEST_LOG, "tests run at " + time.ctime()
-        print '%-30s %s' % ('test log', log_filename)
-
-    def _setup_test_dir(self):
-        import os
-        import shutil
-        
-        TestCase.ORIG_DIR = os.getcwdu()
-        TestCase.TEST_ROOT = os.path.abspath(self._name + '.tmp')
-    
-        print '%-30s %s' % ('running tests in', TestCase.TEST_ROOT)
-    
-        if os.path.exists(TestCase.TEST_ROOT):
-            shutil.rmtree(TestCase.TEST_ROOT)
-        os.mkdir(TestCase.TEST_ROOT)
-        os.chdir(TestCase.TEST_ROOT)
-    
-        # make a fake bzr directory there to prevent any tests propagating
-        # up onto the source directory's real branch
-        os.mkdir(os.path.join(TestCase.TEST_ROOT, '.bzr'))
-
-
-class TextTestRunner(unittest.TextTestRunner):
-
-    def __init__(self, stream=sys.stderr, descriptions=1, verbosity=0, style='progress'):
-        super(TextTestRunner, self).__init__(stream, descriptions, verbosity)
-        self.style = style
-
-    def _makeResult(self):
-        return _MyResult(self.stream, self.style)
-
-    # If we want the old 4 line summary output (count, 0 failures, 0 errors)
-    # we can override run() too.
-
-
-def run_suite(a_suite, name='test', verbose=False):
-    suite = TestSuite((a_suite,),name)
-    if verbose:
-        style = 'verbose'
-    else:
-        style = 'progress'
-    runner = TextTestRunner(stream=sys.stdout, style=style)
-    result = runner.run(suite)
-    return result.wasSuccessful()

=== added file 'tools/__init__.py'
=== added file 'tools/biobench.py'
--- tools/biobench.py	1970-01-01 00:00:00 +0000
+++ tools/biobench.py	2005-11-04 01:46:31 +0000
@@ -0,0 +1,92 @@
+#! /usr/bin/env python
+
+# (C) 2005 Canonical Ltd
+
+"""Benchmark for basicio
+
+Tries serializing an inventory to basic_io repeatedly.
+"""
+
+if True:
+    import psyco
+    psyco.full()
+
+
+import sys
+from timeit import Timer
+from tempfile import TemporaryFile, NamedTemporaryFile
+
+from bzrlib.branch import Branch
+from bzrlib.xml5 import serializer_v5
+from bzrlib.basicio import write_inventory, BasicWriter, \
+        read_inventory
+from bzrlib.inventory import Inventory, InventoryEntry, InventoryFile, ROOT_ID
+
+## b = Branch.open('.')
+## inv = b.get_inventory(b.last_revision())
+
+nrepeats = 3
+ntimes = 5
+NFILES = 30000
+
+def make_inventory():
+    inv = Inventory()
+    for i in range(NFILES):
+        ie = InventoryFile('%08d-id' % i, '%08d-file' % i, ROOT_ID)
+        ie.text_sha1='1212121212121212121212121212121212121212'
+        ie.text_size=12312
+        inv.add(ie)
+    return inv
+
+inv = make_inventory()
+
+bio_tmp = NamedTemporaryFile()
+xml_tmp = NamedTemporaryFile()
+
+def bio_test():
+    bio_tmp.seek(0)
+    w = BasicWriter(bio_tmp)
+    write_inventory(w, inv)
+    bio_tmp.seek(0)
+    new_inv = read_inventory(bio_tmp)
+
+def xml_test():
+    xml_tmp.seek(0)
+    serializer_v5.write_inventory(inv, xml_tmp)
+    xml_tmp.seek(0)
+    new_inv = serializer_v5.read_inventory(xml_tmp)
+
+def run_benchmark(function_name, tmp_file):
+    t = Timer(function_name + '()', 
+              'from __main__ import ' + function_name)
+    times = t.repeat(nrepeats, ntimes)
+    tmp_file.seek(0, 2)
+    size = tmp_file.tell()
+    print 'wrote inventory to %10s %5d times, each %6d bytes, total %6dkB' \
+            % (function_name, ntimes, size, (size * ntimes)>>10), 
+    each = (min(times)/ntimes*1000)
+    print 'in %.1fms each' % each 
+    return each
+
+def profileit(fn): 
+    import hotshot, hotshot.stats
+    prof_f = NamedTemporaryFile()
+    prof = hotshot.Profile(prof_f.name)
+    prof.runcall(fn) 
+    prof.close()
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('time')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+
+if '-p' in sys.argv[1:]:
+    profileit(bio_test)
+else:
+    bio_each = run_benchmark('bio_test', bio_tmp)
+    xml_each = run_benchmark('xml_test', xml_tmp)
+    print 'so bio is %.1f%% faster' % (100 * ((xml_each / bio_each) - 1))
+
+# make sure it was a fair comparison
+# assert 'cElementTree' in sys.modules

=== added file 'tools/capture_tree.py'
--- tools/capture_tree.py	1970-01-01 00:00:00 +0000
+++ tools/capture_tree.py	2005-10-04 11:13:33 +0000
@@ -0,0 +1,40 @@
+#! /usr/bin/env python2.4
+
+# Copyright (C) 2005 Canonical Ltd
+
+"""Print to stdout a description of the current directory, 
+formatted as a Python data structure.
+
+This can be useful in tests that need to recreate directory
+contents."""
+
+import sys
+import os
+
+from bzrlib.trace import enable_default_logging
+enable_default_logging()
+from bzrlib.selftest.treeshape import capture_tree_contents
+
+def main(argv):
+    # a lame reimplementation of pformat that splits multi-line
+    # strings into concatenated string literals.
+    print '['
+    for tt in capture_tree_contents('.'):
+        assert isinstance(tt, tuple)
+        print '    (', repr(tt[0]) + ',',
+        if len(tt) == 1:
+            print '),'
+        else:
+            assert len(tt) == 2
+            val = tt[1]
+            print
+            if val == '':
+                print "        ''"
+            else:
+                for valline in val.splitlines(True):
+                    print '       ', repr(valline)
+            print '    ),'
+    print ']'
+
+if __name__ == '__main__':
+    sys.exit(main(sys.argv))

=== added directory 'tools/doc_generate'
=== added file 'tools/doc_generate/__init__.py'
--- tools/doc_generate/__init__.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/__init__.py	2006-02-13 01:28:03 +0000
@@ -0,0 +1,32 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""tools/doc_generate/__init__.py - main program for bzr information generation stuff"""
+
+import sys
+
+
+def get_module(target):
+    mod_name = "tools.doc_generate.autodoc_%s" % (target)
+    mod = __import__(mod_name)
+    components = mod_name.split('.')
+    for comp in components[1:]:
+        mod = getattr(mod, comp)
+    return mod
+

=== added file 'tools/doc_generate/autodoc_bash_completion.py'
--- tools/doc_generate/autodoc_bash_completion.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/autodoc_bash_completion.py	2006-02-13 01:28:03 +0000
@@ -0,0 +1,54 @@
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"bash_completion.py - create bash completion script from built-in bzr help"
+
+import os
+import sys
+import time
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+
+def get_filename(options):
+    return "%s.bash_completion" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+
+    outfile.write(preamble % params)
+
+
+preamble = """\
+# bash completion functions for for %(bzrcmd)s (bazaar-ng)
+#
+# Large parts of this file are autogenerated from the internal
+# bazaar-ng documentation and data structures.
+#
+# Generation time: %(timestamp)s
+"""
+

=== renamed file 'bzr-man.py' => 'tools/doc_generate/autodoc_man.py'
--- bzr-man.py	2005-06-06 13:24:18 +0000
+++ tools/doc_generate/autodoc_man.py	2006-02-13 01:28:03 +0000
@@ -1,7 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2005 by Hans Ulrich Niedermann
-# Portions Copyright (C) 2005 by Canonical Ltd
+# Copyright 2005 Canonical Ltd.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,44 +14,47 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-#<<< code taken from bzr (C) Canonical
-
-import os, sys
-
-try:
-    version_info = sys.version_info
-except AttributeError:
-    version_info = 1, 5 # 1.5 or older
-
-
-REINVOKE = "__BZR_REINVOKE"    
-NEED_VERS = (2, 3)
-
-if version_info < NEED_VERS:
-    if not os.environ.has_key(REINVOKE):
-        # mutating os.environ doesn't work in old Pythons
-        os.putenv(REINVOKE, "1")
-        for python in 'python2.4', 'python2.3':
-            try:
-                os.execvp(python, [python] + sys.argv)
-            except OSError:
-                pass
-    print >>sys.stderr, "bzr-man.py: error: cannot find a suitable python interpreter"
-    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
-    sys.exit(1)
-os.unsetenv(REINVOKE)
-
-import bzrlib, bzrlib.help
-
-#>>> code taken from bzr (C) Canonical
-
-#<<< code by HUN
-
+"""man.py - create man page from built-in bzr help and static text
+
+TODO:
+  * use usage information instead of simple "bzr foo" in COMMAND OVERVIEW
+  * add command aliases
+"""
+
+import os
+import sys
+import textwrap
 import time
-import re
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+
+def get_filename(options):
+    """Provides name of manpage"""
+    return "%s.1" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    """Assembles a man page"""
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+    outfile.write(man_preamble % params)
+    outfile.write(man_escape(man_head % params))
+    outfile.write(man_escape(getcommand_list(params)))
+    outfile.write(man_escape(getcommand_help(params)))
+    outfile.write(man_escape(man_foot % params))
 
 
 def man_escape(string):
+    """Escapes strings for man page compatibility"""
     result = string.replace("\\","\\\\")
     result = result.replace("`","\\`")
     result = result.replace("'","\\'")
@@ -62,94 +62,72 @@
     return result
 
 
-class Parser:
-
-    def parse_line(self, line):
-        pass
-
-
-class CommandListParser(Parser):
-
-    """Parser for output of "bzr help commands".
-
-    The parsed content can then be used to
-    - write a "COMMAND OVERVIEW" section into a man page
-    - provide a list of all commands
-    """
-
-    def __init__(self,params):
-        self.params = params
-        self.command_usage = []
-        self.all_commands = []
-        self.usage_exp = re.compile("([a-z0-9-]+).*")
-        self.descr_exp = re.compile("    ([A-Z].*)\s*")
-        self.state = 0
-        self.command = None
-        self.usage = None
-        self.descr = None
-
-    def parse_line(self, line):
-        m = self.usage_exp.match(line)
-        if m:
-            if self.state == 0:
-                if self.usage:
-                    self.command_usage.append((self.command,self.usage,self.descr))
-                    self.all_commands.append(self.command)
-                self.usage = line
-                self.command = m.groups()[0]
-            else:
-                raise Error, "matching usage line in state %d" % state
-            self.state = 1
-            return
-        m = self.descr_exp.match(line)
-        if m:
-            if self.state == 1:
-                self.descr = m.groups()[0]
-            else:
-                raise Error, "matching descr line in state %d" % state
-            self.state = 0
-            return
-        raise Error, "Cannot parse this line"
-
-    def end_parse(self):
-        if self.state == 0:
-            if self.usage:
-                self.command_usage.append((self.command,self.usage,self.descr))
-                self.all_commands.append(self.command)
+def command_name_list():
+    """Builds a list of command names from bzrlib"""
+    command_names = bzrlib.commands.builtin_command_names()
+    command_names.sort()
+    return command_names
+
+
+def getcommand_list (params):
+    """Builds summary help for command names in manpage format"""
+    bzrcmd = params["bzrcmd"]
+    output = '.SH "COMMAND OVERVIEW"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            usage = bzrlib.help.command_usage(cmd_object)
+            tmp = '.TP\n.B "%s"\n%s\n' % (usage, firstline)
+            output = output + tmp
         else:
-            raise Error, "ending parse in state %d" % state
-
-    def write_to_manpage(self, outfile):
-        bzrcmd = self.params["bzrcmd"]
-        outfile.write('.SH "COMMAND OVERVIEW"\n')
-        for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n\n' % (bzrcmd, usage, descr))
-
-
-class HelpReader:
-
-    def __init__(self, parser):
-        self.parser = parser
-
-    def write(self, data):
-        if data[-1] == '\n':
-            data = data[:-1]
-        for line in data.split('\n'):
-            self.parser.parse_line(line)
-
-
-def write_command_details(params, command, usage, descr, outfile):
-    x = ('.SS "%s %s"\n.B "%s"\n.PP\n.B "Usage:"\n%s %s\n\n' %
-         (params["bzrcmd"],
-          command,
-          descr,
-          params["bzrcmd"],
-          usage))
-    outfile.write(man_escape(x))
+            raise RuntimeError, "Command '%s' has no help text" % (cmd_name)
+    return output
+
+
+def getcommand_help(params):
+    """Shows individual options for a bzr command"""
+    output='.SH "COMMAND REFERENCE"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        output = output + format_command(params, cmd_object)
+    return output
+
+
+def format_command (params, cmd):
+    """Provides long help for each public command"""
+    subsection_header = '.SS "%s"\n' % (bzrlib.help.command_usage(cmd))
+    doc = "%s\n" % (cmd.__doc__)
+    docsplit = cmd.__doc__.split('\n')
+    doc = '\n'.join([docsplit[0]] + [line[4:] for line in docsplit[1:]])
+    option_str = ""
+    options = cmd.options()
+    if options:
+        option_str = "\nOptions:\n"
+        for option_name, option in sorted(options.items()):
+            l = '    --' + option_name
+            if option.type is not None:
+                l += ' ' + option.argname.upper()
+            short_name = option.short_name()
+            if short_name:
+                assert len(short_name) == 1
+                l += ', -' + short_name
+            l += (30 - len(l)) * ' ' + option.help
+            # TODO: Split help over multiple lines with
+            # correct indenting and wrapping.
+            wrapped = textwrap.fill(l, initial_indent='',
+                                    subsequent_indent=30*' ')
+            option_str = option_str + wrapped + '\n'       
+    return subsection_header + option_str + "\n" + doc + "\n"
 
 
 man_preamble = """\
-.\\\" Man page for %(bzrcmd)s (bazaar-ng)
+Man page for %(bzrcmd)s (bazaar-ng)
 .\\\"
 .\\\" Large parts of this file are autogenerated from the output of
 .\\\"     \"%(bzrcmd)s help commands\"
@@ -159,8 +137,7 @@
 .\\\"
 """
 
-# The DESCRIPTION text was taken from http://www.bazaar-ng.org/
-# and is thus (C) Canonical
+
 man_head = """\
 .TH bzr 1 "%(datestamp)s" "%(version)s" "bazaar-ng"
 .SH "NAME"
@@ -193,77 +170,26 @@
 Path where
 .B "%(bzrcmd)s"
 is to look for external command.
-
 .TP
 .I "BZREMAIL"
-E-Mail address of the user. Overrides
-.I "~/.bzr.conf/email" and
-.IR "EMAIL" .
-Example content:
-.I "John Doe <john@example.com>"
-
+E-Mail address of the user. Overrides default user config.
 .TP
 .I "EMAIL"
-E-Mail address of the user. Overridden by the content of the file
-.I "~/.bzr.conf/email"
-and of the environment variable
-.IR "BZREMAIL" .
-
+E-Mail address of the user. Overriddes default user config.
 .SH "FILES"
 .TP
-.I "~/.bzr.conf/"
-Directory where all the user\'s settings are stored.
-.TP
-.I "~/.bzr.conf/email"
-Stores name and email address of the user. Overrides content of
-.I "EMAIL"
-environment variable. Example content:
-.I "John Doe <john@example.com>"
-
+.I "~/.bazaar/bazaar.conf/"
+Contains the default user config. Only one section, [DEFAULT] is allowed. A 
+typical default config file may be similiar to:
+.br
+.br
+.B [DEFAULT]
+.br
+.B email=John Doe <jdoe@isp.com>
 .SH "SEE ALSO"
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/,
 .UR http://www.bazaar-ng.org/
-.BR http://www.bazaar-ng.org/,
-.UR http://www.bazaar-ng.org/doc/
-.BR http://www.bazaar-ng.org/doc/
+.BR http://www.bazaar-ng.org/
 """
 
-def main():
-    t = time.time()
-    tt = time.gmtime(t)
-    params = \
-           { "bzrcmd": "bzr",
-             "datestamp": time.strftime("%Y-%m-%d",tt),
-             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
-             "version": bzrlib.__version__,
-             }
-
-    clp = CommandListParser(params)
-    bzrlib.help.help("commands", outfile=HelpReader(clp))
-    clp.end_parse()
-
-    filename = "bzr.1"
-    if len(sys.argv) == 2:
-        filename = sys.argv[1]
-    if filename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(filename,"w")
-
-    outfile.write(man_preamble % params)
-    outfile.write(man_escape(man_head % params))
-    clp.write_to_manpage(outfile)
-
-    # FIXME:
-    #   This doesn't do more than the summary so far.
-    #outfile.write('.SH "DETAILED COMMAND DESCRIPTION"\n')
-    #for (command,usage,descr) in clp.command_usage:
-    #    write_command_details(params, command, usage, descr, outfile = outfile)
-
-    outfile.write(man_escape(man_foot % params))
-
-
-if __name__ == '__main__':
-    main()
-
-
-#>>> code by HUN

=== modified file 'tools/http_client.py'
--- tools/http_client.py	2005-08-19 22:42:40 +0000
+++ tools/http_client.py	2005-09-19 06:05:19 +0000
@@ -68,12 +68,22 @@
 # The content is written into temporary files.  It returns a list of
 # readable file objects.
 
+# TODO: If we try pipelined or keepalive and the connection drop out
+# then retry the request on a new connection; eventually we should perhaps
+# learn that a given host or network just won't allow keepalive.
+
+
 import asyncore
 import socket, string, time, sys
 import StringIO
 import mimetools, urlparse, urllib
 import logging
 
+logging.basicConfig(level=logging.DEBUG,
+                    format='%(asctime)s %(levelname)s %(message)s',
+                    filename='/tmp/http_client.log',
+                    filemode='w')
+
 logger = logging.getLogger('bzr.http_client')
 debug = logger.debug
 info = logger.info
@@ -115,7 +125,7 @@
         self.channels = []
         self.try_pipelined = False
         self.try_keepalive = False
-        self.max_channels = 3
+        self.max_channels = 5
 
 
     def enqueue(self, url, consumer):
@@ -134,10 +144,9 @@
 
     def _make_channel(self):
         # proxy2 203.17.154.69
-        # bazaar-ng.org 
-        return HttpChannel('82.211.81.161', 80, self)
+        # return HttpChannel('82.211.81.161', 80, self)         # bazaar-ng.org 
         # return HttpChannel('203.17.154.69', 8080, self)
-        # return HttpChannel('localhost', 8000, self)
+        return HttpChannel('127.0.0.1', 8000, self)  # forwarded
             
 
     def _wake_up_channel(self):
@@ -172,7 +181,7 @@
             choice(self.channels).take_one()
 
 
-        debug("request left until a channel's idle")
+        # debug("request postponed until a channel's idle")
         
 
 
@@ -394,15 +403,20 @@
                 
                 assert got_data
 
-                debug('pass back %d bytes of %s' % (len(got_data),
-                                                    self.sent_requests[0][0]))
+                self.content_remaining -= len(got_data)
+
+                debug('pass back %d bytes of %s, %d remain'
+                      % (len(got_data),
+                         self.sent_requests[0][0],
+                         self.content_remaining))
                 consumer.feed(data)
 
-                self.content_remaining -= len(got_data)
-
             if self.content_remaining == 0:
                 del self.sent_requests[0]
 
+                debug('content complete')
+                consumer.content_complete()
+                
                 # reset lots of things and try to get the next response header
                 if self.response.connection_reply == 'close':
                     debug('server requested close')
@@ -414,7 +428,6 @@
                     self.close()
                 else:
                     debug("ready for next header...")
-                    consumer.content_complete()
                     self.take_one()
                 self.response = None
 
@@ -428,44 +441,61 @@
         self.close()
 
 
+class DummyConsumer:
+    def __init__(self, url, pb):
+        self.url = url
+        self.outf = None
+        self._pb = pb
+
+    def feed(self, data):
+        # print "feed", repr(data)
+        # print "feed", repr(data[:20]), repr(data[-20:]), len(data)
+        if not self.outf:
+            base = self.url[self.url.rindex('/')+1:]
+            self.outf = file('/tmp/download/' + base, 'wb')
+        self.outf.write(data)
+
+    def error(self, err_info):
+        import traceback
+        error('error reported to consumer')
+        traceback.print_exception(err_info[0], err_info[1], err_info[2])
+        sys.exit(1)
+
+    def content_complete(self):
+        info('content complete from %s' % self.url)
+        self.outf.close()
+        self.outf = None
+        # using last_cnt is cheating
+        self._pb.update('downloading inventory',
+                        self._pb.last_cnt+1,
+                        self._pb.last_total)
+
+
+
 if __name__ == "__main__":
-    class dummy_consumer:
-        def __init__(self, url):
-            self.url = url
-
-        def feed(self, data):
-            # print "feed", repr(data)
-            # print "feed", repr(data[:20]), repr(data[-20:]), len(data)
-            pass
-            
-        def error(self, err_info):
-            import traceback
-            traceback.print_exception(err_info[0], err_info[1], err_info[2])
-
-        def content_complete(self):
-            debug('content complete from %s' % self.url)
-            
-
     logging.basicConfig(level=logging.DEBUG)
 
     mgr = DownloadManager()
 
     from bzrlib.branch import Branch
+    from bzrlib.progress import ProgressBar
+
+    pb = ProgressBar()
     revs = Branch('/home/mbp/work/bzr').revision_history()
-
-        
+    pb.update('downloading inventories', 0, len(revs))
+
+    for rev in revs:
+        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/inventory-store/' \
+              + rev + '.gz'
+        mgr.enqueue(url, DummyConsumer(url, pb))
+
+    mgr.run()
+    
+
+
     
 #     for url in ['http://www.bazaar-ng.org/',
 #                 'http://www.bazaar-ng.org/tutorial.html',
 #                 'http://www.bazaar-ng.org/download.html',
 #                 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/revision-store/mbp@hope-20050415013653-3b3c9c3d33fae0a6.gz',
 #                 ]:
-
-    for rev in revs:
-#        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/revision-store/' \
-        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/inventory-store/' \
-              + rev + '.gz'
-        mgr.enqueue(url, dummy_consumer(url))
-
-    mgr.run()
-    

=== added file 'tools/riodemo.py'
--- tools/riodemo.py	1970-01-01 00:00:00 +0000
+++ tools/riodemo.py	2005-11-28 08:03:42 +0000
@@ -0,0 +1,60 @@
+
+
+
+# \subsection{Example usage}
+# 
+# \textbf{XXX:} Move these to object serialization code. 
+
+def write_revision(writer, revision):
+    s = Stanza(revision=revision.revision_id,
+               committer=revision.committer, 
+               timezone=long(revision.timezone),
+               timestamp=long(revision.timestamp),
+               inventory_sha1=revision.inventory_sha1,
+               message=revision.message)
+    for parent_id in revision.parent_ids:
+        s.add('parent', parent_id)
+    for prop_name, prop_value in revision.properties.items():
+        s.add(prop_name, prop_value)
+    writer.write_stanza(s)
+
+def write_inventory(writer, inventory):
+    s = Stanza(inventory_version=7)
+    writer.write_stanza(s)
+
+    for path, ie in inventory.iter_entries():
+        s = Stanza()
+        s.add(ie.kind, ie.file_id)
+        for attr in ['name', 'parent_id', 'revision', \
+                     'text_sha1', 'text_size', 'executable', \
+                     'symlink_target', \
+                     ]:
+            attr_val = getattr(ie, attr, None)
+            if attr == 'executable' and attr_val == 0:
+                continue
+            if attr == 'parent_id' and attr_val == 'TREE_ROOT':
+                continue
+            if attr_val is not None:
+                s.add(attr, attr_val)
+        writer.write_stanza(s)
+
+
+def read_inventory(inv_file):
+    """Read inventory object from rio formatted inventory file"""
+    from bzrlib.inventory import Inventory, InventoryFile
+    s = read_stanza(inv_file)
+    assert s['inventory_version'] == 7
+    inv = Inventory()
+    for s in read_stanzas(inv_file):
+        kind, file_id = s.items[0]
+        parent_id = None
+        if 'parent_id' in s:
+            parent_id = s['parent_id']
+        if kind == 'file':
+            ie = InventoryFile(file_id, s['name'], parent_id)
+            ie.text_sha1 = s['text_sha1']
+            ie.text_size = s['text_size']
+        else:
+            raise NotImplementedError()
+        inv.add(ie)
+    return inv

=== added file 'tools/trace-revisions'
--- tools/trace-revisions	1970-01-01 00:00:00 +0000
+++ tools/trace-revisions	2005-11-07 16:21:12 +0000
@@ -0,0 +1,19 @@
+#! /usr/bin/env python2.4
+
+import sys
+
+from bzrlib.branch import Branch
+
+b = Branch.open('.')
+rev_id = sys.argv[1]
+rh = []
+while True:
+    rh.append(rev_id)
+    rev = b.get_revision(rev_id)
+    if rev.parent_ids:
+        rev_id = rev.parent_ids[0]
+    else:
+        break
+rh.reverse()
+for rev_id in rh:
+    print rev_id

=== modified file 'tools/weavebench.py'
--- tools/weavebench.py	2005-07-18 11:23:40 +0000
+++ tools/weavebench.py	2005-09-19 06:05:19 +0000
@@ -51,7 +51,7 @@
             if lines:
                 del lines[randrange(0, len(lines))]
 
-        rev_id = wf.add(parents, lines)
+        rev_id = wf.add("%s" % i, parents, lines)
         parents = [rev_id]
 
     write_weave(wf, file(WEAVE_NAME, 'wb'))

=== modified file 'tutorial.txt'
--- tutorial.txt	2005-08-12 15:24:19 +0000
+++ tutorial.txt	2006-04-12 04:57:34 +0000
@@ -1,79 +1,84 @@
+.. This file is in Python ReStructuredText format - it can be formatted
+.. into HTML or text.  In the future we plan to extract the example commands
+.. and automatically test them.
+
+.. This text was previously on the wiki at http://bazaar.canonical.com/IntroductionToBzr
+.. but has been moved into the source tree so it can be kept in sync with
+.. the source and possibly automatically checked.
+
 ==================
 Bazaar-NG Tutorial
 ==================
 
-current for bzr 0.0.6pre, July 2005
-
-
-*Note:* This tutorial is a work in 
-progress, and describes code that is itself still evolving. 
-If you have comments on either the design or the tutorial, 
-please send them to the bazaar-ng@lists.canonical.com mailing list.
-
+Current for bzr-0.7pre, 2006-01-06.
 
 
 Introduction
 ============
 
-Bazaar-NG is a version control tool.  It manages trees of files and subdirectories.  In particular, it records *revisions* of trees, representing their state at a particular point in time, and information about those revisions and their relationships.  Recording and retrieving tree revisions is useful in several ways if you are writing software or documents or doing similar creative work.
-
- * Keeping previous revisions lets you go back if you make a mistake or want to check your work.  It acts as a high-level unlimited undo.
-
- * By recording comments on every revision, you produce an annotated history of the project, describing what, who, why, and when.
-
- * Using a version control tool can be an aid to thinking about a project: getting to a stable state at regular intervals and then writing a description of what you did is an easy way to stay organized and on track.
-
-Bazaar-NG remembers the *ancestry* of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree.  
-By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict
-lock-step.  Branching can be useful even for a single developer.
-
-Bazaar-NG installs a single new command,
-*bzr*.  Everything else is a subcommand of this.  You can get
-some help with ``bzr help``.  There will be more in the future.
-
-
-
-Introducing yourself to bzr
-===========================
-
-One function of a version control system is to keep track of who
-changed what.  In a distributed system that requires an identifier for
-each author that is globally unique.  Most people already have one of
-these: an email address.
-
-[after 0.0.4] To tell bzr which email address to use, put it in the file
-``$HOME/.bzr.conf/email``, or the environment variable ``$BZREMAIL``.
-If neither of these are set, bzr will use the ``$EMAIL``
-variable, or use your username and hostname.
-
-To check this has taken effect, or if you forget your own name, use
-the ``whoami`` ("who am i?") command::
-
-  % bzr whoami
-
-Some people want to avoid sharing their email address so as not to
-get spam.  bzr will never
-disclose your email address unless you tell it to by publishing an
-archive or transmiting a changeset.  It's recommended that you do use
-a real address, so that people can contact you about your work, but
-it's not required.  You can use an address which is obfuscated, which
-bounces, or which goes through an anti-spam service such as spamgourmet.com.
-
-
+If you are already familiar with decentralized revision control, then please feel free to skip ahead to "Introducing Yourself to Bazaar-NG". If, on the other hand, you are familiar with revision control but not decentralized revision control, then please start at "How DRCS is different." Otherwise, get some coffee or tea, get comfortable and get ready to catch up. 
+
+The Purposes of Revision Control
+================================
+
+Odds are that you have worked on some sort of textual data -- the sources to a program, web sites or the config files that Unix system administrators have to deal with in /etc. The chances are also good that you have made some sort of mistake that you deeply regretted. Perhaps you deleted the configuration file for your mailserver or perhaps mauled the source code for a pet project. Whatever happened, you have just deleted important information that you would desperately like to get back. If this has ever happened to you, then you are probably ready for Bazaar-NG.
+
+Revision control systems (which I'll henceforth call RCS) such as Bazaar-NG give you the ability to track changes for a directory by turning it into something slightly more complicated than a directory that we call a **branch**. The branch not only stores how the directory looks right now, but also how it looked at various points in the past. Then, when you do something you wish you hadn't, you can restore the directory to the way it looked at some point in the past.
+
+Revision control systems give users the ability to save changes to a branch by "committing a **revision**". The revision created is essentially a summary of the changes that were made since the last time the tree was saved. 
+
+These revisions have other uses as well. For example, one can comment revisions to record what the recent set of changes meant by providing an optional log message. Real life log messages include things like "Fixed the web template to close the table" and "Added sftp suppport. Fixes #595"
+	
+We keep these logs so that if later there is some sort of problem with sftp, we can figure out when the problem probably happened. 
+
+How DRCS is Different
+---------------------
+
+Many Revision Control Systems (RCS) are stored on servers. If one wants to work on the code stored within an RCS, then one needs to connect to the server and "checkout" the code. Doing so gives one a directory in which a person can make changes and then commit. The RCS client then connects to the RCS server and stores the changes. This method is known as the centralized model. 
+
+The centralized model can have some drawbacks. A centralized RCS requires that one is able to connect to the server whenever one wants to do version control work. This can be a bit of a problem if your server on some other machine on the internet and you are not. Or, worse yet, you ''are'' on the internet but the server is missing!
+
+Decentralized Revision Control Systems (which I'll call DRCS after this point) deal with this problem by keeping branches on the same machine as the client. In Bazaar-NG's case, the branch is kept in the same place as the code that is being version controlled. This allows the user to save his changes (**commit**) whenever he wants -- even if he is offline. The user only needs internet access when he wants to access the changes in someone else's branch that are somewhere else.
+
+ 
+A common requirement that many people have is the need to keep track of the changes for a directory such as file and subdirectory changes. Performing this tracking by hand is a awkward process that over time becomes unwieldy. That is, until one considers version control tools such as Bazaar-NG. These tools automate the process of storing data by creating a **revision** of the directory tree whenever the user asks. 
+
+Revision control software such as Bazaar-NG can do much more than just storage and performing undo.  For example, with Bazaar-NG developer can take the modifications in one branch of software and apply them to another, related, branch -- even if those changes exist in a branch owned by somebody else. This allows developers to cooperate without giving write access to repository.
+
+Bazaar-NG remembers the ''ancestry'' of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree. By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict lock-step.  Branching can be useful even for a single developer.
+
+Introducing yourself to Bazaar-NG
+=================================
+
+Bazaar-NG installs a single new command, **bzr**.  Everything else is a subcommand of this.  You can get some help with `bzr help`.  There will be more in the future.
+
+One function of a version control system is to keep track of who changed what.  In a decentralized system, that requires an identifier for each author that is globally unique.  Most people already have one of these: an email address. Bzr is smart enough to automatically generate an email address by looking up your username and hostname. If you don't like the guess that Bazaar-NG makes, then three options exist:
+
+ 1. (**Bazaar-NG 0.6 and later**). Setting the email address in the
+ ``~/.bazaar/bazaar.conf`` by adding the following lines. Please note that
+ ``[DEFAULT]`` is case sensitive::
+
+    [DEFAULT]
+    email= Your Name <email@isp.com>
+ 
+ 1. (**Bazaar-NG 0.6 and later**) Override the previous setting on a
+ branch by branch basis by creating a branch section in
+ ``~/.bazaar/branches.conf`` by adding the following lines::
+
+    [/the/directory/to/the/branch]
+    email=Your Name <email@isp.com>
+ 
+ 1. Overriding the two previous options by setting the global environment
+ variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) to your full email address.
 
 Creating a branch
-==================
-
-
-History is by default stored in the .bzr directory of the branch.
-There will be a facility to store it in a separate repository, which
-may be remote.  We create a new branch by running *bzr init* in
-an existing directory::
+=================
+
+History is by default stored in the .bzr directory of the branch. There will be a facility to store it in a separate repository, which may be remote.  We create a new branch by running **bzr init** in an existing directory::
 
     % mkdir tutorial
     % cd tutorial
     % ls -a
-    ls -a
     ./  ../
     % pwd
     /home/mbp/work/bzr.test/tutorial
@@ -83,51 +88,57 @@
     ./  ../  .bzr/
     %
 
-As for CVS, there are three classes of file: unknown, ignored, and
-versioned.  The *add* command makes a file versioned: that is,
-changes to it will be recorded by the system::
+As for CVS, there are three classes of file: unknown, ignored, and versioned.  The **add** command makes a file versioned: that is, changes to it will be recorded by the system::
 
     % echo 'hello world' > hello.txt
     % bzr status
-    ?       hello.txt
+    unknown:
+      hello.txt
     % bzr unknowns
     hello.txt
-    % bzr add -v hello.txt
-    A       hello.txt
+    % bzr add hello.txt
+    added hello.txt
     % bzr unknowns
 
 
-If you add the wrong file, simply use ``bzr remove`` to make
-it unversioned again.  This does not delete the working copy.
-
+If you add the wrong file, simply use **bzr remove** to make it unversioned again.  This does not delete the working copy.
+
+Branch locations
+================
+
+All history is stored in a branch, which is just an on-disk directory containing control files.  By default there is no separate repository or database as used in svn or svk. You can choose to create a repository if you want to (see the **bzr init-repo** command). You may wish to do this if you have very large branches, or many branches of a moderate sized project.
+
+You'll usually refer to branches on your computer's filesystem just by
+giving the name of the directory containing the branch.  bzr also supports
+accessing branches over http, for example::
+
+    % bzr log http://bazaar-ng.org/bzr/bzr.dev/
+
+By installing bzr plugins you can also access branches over the sftp or rsync protocols.
 
 Reviewing changes
 =================
 
-Once you have completed some work, you will want to *commit*
-it to the version history.  It is good to commit fairly often:
-whenever you get a new feature working, fix a bug, or improve some
-code or documentation.  It's also a good practice to make sure that
-the code compiles and passes its test suite before committing, to make
-sure that every revision is a known-good state.  You can also review
-your changes, to make sure you're committing what you intend to, and
-as a chance to rethink your work before you permanently record it. 
-
-Two bzr commands are particularly useful here: *status* and
-*diff*.  The *status* command
-shows a listing with one line per file, indicating whether it has been
-Added, Deleted, Modified, or Renamed in the current revision.  Unknown
-files are shown as '?'.  With the ``--all`` option, the status
-command also shows unmodified versioned files as '.', and ignored
-files as 'I'::
+Once you have completed some work, you will want to **commit** it to the version history.  It is good to commit fairly often: whenever you get a new feature working, fix a bug, or improve some code or documentation.  It's also a good practice to make sure that the code compiles and passes its test suite before committing, to make sure that every revision is a known-good state.  You can also review your changes, to make sure you're committing what you intend to, and as a chance to rethink your work before you permanently record it. 
+
+Two bzr commands are particularly useful here: **status** and **diff**.  
+
+bzr status
+----------
+
+The **status** command tells you what changes have been made to the
+working directory since the last revision::
 
     % bzr status
-    A       hello.txt
-
-The *diff* command shows the full text of changes to all
-files as a standard unified diff.  This can be piped through many
-programs such as ``patch``, ``diffstat``,
-``filterdiff`` and ``colordiff``::
+    modified:
+       foo
+
+By default **bzr status** hides "boring" files that are either unchanged or ignored.  To see them too, use the --all option.  The status command can optionally be given the name of some files or directories to check.
+
+bzr diff
+--------
+
+The **diff** command shows the full text of changes to all files as a standard unified diff.  This can be piped through many programs such as ''patch'', ''diffstat'', ''filterdiff'' and ''colordiff''::
 
     % bzr diff
     *** added file 'hello.txt'
@@ -136,93 +147,88 @@
     @@ -1,0 +1,1 @@
     +hello world
 
-With the ``-r`` option, the tree is compared to an earlier
-revision.
-
-[TODO: options to run external diff; to get context diff or other
-formats; to diff only selected files; to compare two historical
-revisions.]
-
-
+
+With the ''-r'' option, the tree is compared to an earlier revision, or
+the differences between two versions are shown::
+
+    % bzr diff -r 1000..          # everything since r1000
+    % bzr diff -r 1000..1100      # changes from 1000 to 1100
+
+The --diff-options option causes bzr to run the external diff program,
+passing options.  For example::
+
+    % bzr diff --diff-options --side-by-side foo
 
 Committing changes
 ==================
 
-When the working tree state is satisfactory, it can be
-*committed* to the branch, creating a new revision holding a
-snapshot of that state.  
-
-The ``commit`` command takes a message describing the changes
-in the revision.  It also records your userid, the current time and
-timezone, and the inventory and contents of the tree.  The commit
-message is specified by the ``-m`` or ``--message`` option.
-You can enter a multi-line commit message; in most shells you can
-enter this just by leaving the quotes open at the end of the line. ::
+When the working tree state is satisfactory, it can be **committed** to the branch, creating a new revision holding a snapshot of that state.  
+
+bzr commit
+----------
+
+The **commit** command takes a message describing the changes in the revision.  It also records your userid, the current time and timezone, and the inventory and contents of the tree.  The commit message is specified by the ''-m'' or ''--message'' option. You can enter a multi-line commit message; in most shells you can enter this just by leaving the quotes open at the end of the line.
+
+::
 
     % bzr commit -m "added my first file"
 
-[TODO: commit message interactively, through an editor or from a
-file.]
-
-[TODO: commit only selected files, including renamed/added/deleted
-files.]
-
+You can also use the -F option to take the message from a file.  Some people like to make notes for a commit message while they work, then review the diff to make sure they did what they said they did.  (This file can also be useful when you pick up your work after a break.)
+
+Message from an editor
+======================
+
+If you use neither the `-m` nor the `-F` option then bzr will open an editor for you to enter a message.  The editor to run is controlled by your `$EDITOR` environment variable or (Post Bazaar-NG 0.6) email setting in .  If you quit the editor without making any changes, the commit will be cancelled.
+
+Selective commit
+----------------
+
+If you give file or directory names on the commit command line then only
+the changes to those files will be committed.  For example::
+
+    % bzr commit -m "documentation fix" commit.py
+
+By default bzr always commits all changes to the tree, even if run from a
+subdirectory.  To commit from only the current directory down, use::
+
+    % bzr commit .
 
 
 Removing uncommitted changes
 ============================
 
-If you've made some changes and don't want to keep them, use the
-``revert`` command to go back to the previous head version.  It's a
-good idea to use ``bzr diff`` first to see what will be removed. 
-By default the revert command reverts the whole tree; if file or
-directory names are given then only those ones will be affected.
-revert also clears the list of pending merges revisions.
-
-
-
-
+If you've made some changes and don't want to keep them, use the **revert** command to go back to the previous head version.  It's a good idea to use **bzr diff** first to see what will be removed. By default the revert command reverts the whole tree; if file or directory names are given then only those ones will be affected. **revert** also clears the list of pending merges revisions.
 
 Ignoring files
 ==============
 
-Many source trees contain some files that do not need to be
-versioned, such as editor backups, object or bytecode files, and built
-programs.  You can simply not add them, but then they'll always crop
-up as unknown files.  You can also tell bzr to ignore these files by
-adding them to a file called ``.bzrignore`` at the top of the
-tree.
+Many source trees contain some files that do not need to be versioned, such as editor backups, object or bytecode files, and built programs.  You can simply not add them, but then they'll always crop up as unknown files.  You can also tell bzr to ignore these files by adding them to a file called ''.bzrignore'' at the top of the tree.
 
-This file contains a list of file wildcards (or "globs"), one
-per line.  Typical contents are like this::
+This file contains a list of file wildcards (or "globs"), one per line.  Typical contents are like this::
 
     *.o
     *~
     *.tmp
     *.py[co]
 
-If a glob contains a slash, it is matched against the whole path
-from the top of the tree; otherwise it is matched against only the
-filename.  So the previous example ignores ``*.o`` in all
-subdirectories, but this example ignores only config.h at the top
-level and HTML files in ``doc/``::
+If a glob contains a slash, it is matched against the whole path from the
+top of the tree; otherwise it is matched against only the filename.  So
+the previous example ignores files with extension ``.o`` in all
+subdirectories, but this example ignores only config.h at the top level
+and HTML files in ``doc/``::
 
     ./config.h
     doc/*.html
 
-To get a list of which files are ignored and what pattern they matched, use ``bzr ignored``::
+To get a list of which files are ignored and what pattern they matched, use ''bzr ignored''::
 
     % bzr ignored
-    config.h                                           ./config.h
-    configure.in~                                      *~
-
-It is OK to have an ignore pattern match a versioned file, or to
-add an ignored file.  Ignore patterns have no effect on versioned
-files; they only determine whether unversioned files are reported as
-unknown or ignored.
-
-The ``.bzrignore`` file should normally be versioned, so that new
-copies of the branch see the same patterns::
+    config.h                 ./config.h
+    configure.in~            *~
+
+It is OK to have either an ignore pattern match a versioned file, or to add an ignored file.  Ignore patterns have no effect on versioned files; they only determine whether unversioned files are reported as unknown or ignored.
+
+The ''.bzrignore'' file should normally be versioned, so that new copies of the branch see the same patterns::
 
     % bzr add .bzrignore
     % bzr commit -m "Add ignore patterns"
@@ -234,21 +240,26 @@
 bzr log
 -------
 
-The ``log`` command shows a list of previous revisions.
+The **bzr log** command shows a list of previous revisions. The **bzr log --forward** command does the same in chronological order to get most recent revisions printed at last.
+
+As with bzr diff, bzr log supports the -r argument::
+
+    % bzr log -r 1000..          # Revision 1000 and everything after it
+    % bzr log -r ..1000          # Everything up to and including revision % 1000
+    % bzr log -r 1000..1100      # changes from 1000 to 1100
+    % bzr log -r 1000            # The changes in only revision 1000
 
 
 Branch statistics
 =================
 
-The ``bzr info`` command shows some summary information about
-the working tree and the branch history.  
+The **bzr info** command shows some summary information about the working tree and the branch history.  
 
 
 Versioning directories
 ======================
 
-bzr versions files and directories in a way that can keep track of
-renames and intelligently merge them::
+bzr versions files and directories in a way that can keep track of renames and intelligently merge them::
 
     % mkdir src
     % echo 'int main() {}' > src/simple.c
@@ -264,14 +275,11 @@
 
 Deleting and removing files
 ===========================
-
-You can delete files or directories by just deleting them from the
-working directory.  This is a bit different to CVS, which requires
-that you also do *cvs remove*.
-
-*bzr remove* makes the file un-versioned, but does not
-delete the working copy.  This is useful when you add the wrong file,
-or decide that a file should actually not be versioned. ::
+You can delete files or directories by just deleting them from the working directory.  This is a bit different to CVS, which requires that you also do **cvs remove**.
+
+	**bzr remove** makes the file un-versioned, but does not delete the working copy.  This is useful when you add the wrong file, or decide that a file should actually not be versioned. 
+
+::
 
     % rm -r src
     % bzr remove -v hello.txt
@@ -281,35 +289,56 @@
     D       src/
     D       src/simple.c
 
+If you remove the wrong file by accident, you can use **bzr revert** to restore it.
+
 
 Branching
 =========
 
-Often rather than starting your own project, you will want to
-submit a change to an existing project.  You can get a copy of an
-existing branch by copying its directory, expanding a tarball, or by a
-remote copy using something like rsync.  You can also use bzr to fetch
-a copy.  Because this new copy is potentially a new branch, the
-command is called *branch*::
-
-    % bzr branch http://bazaar-ng.org/bzr/main ./bzr-main
-    % cd bzr-main
-
-This copies down the complete history of this branch, so we can
-do all operations on it locally: log, annotate, making and merging
-branches.  There will be an option to get only part of the history if
-you wish.
-
-
+Often rather than starting your own project, you will want to submit a change to an existing project.  You can get a copy of an existing branch by copying its directory, expanding a tarball, or by a remote copy using something like rsync.  You can also use bzr to fetch a copy.  Because this new copy is potentially a new branch, the command is called *branch*::
+
+    % bzr branch http://bazaar-ng.org/bzr/bzr.dev 
+    % cd bzr.dev
+
+This copies down the complete history of this branch, so we can do all operations on it locally: log, annotate, making and merging branches.  There will be an option to get only part of the history if you wish.
 
 Following upstream changes
 ==========================
 
-You can stay up-to-date with the parent branch by *pulling*
-in their changes::
+You can stay up-to-date with the parent branch by "pulling" in their changes::
 
     % bzr pull
 
-This only works if the local branch if your branch includes only
-changes from the parent branch.  Otherwise, the branches are said to
-have *diverged*, and they must be merged instead.
+After this change, the local directory will be a mirror of the source. This
+includes the ''revision-history'' - which is a list of the commits done in 
+this branch, rather than merged from other branches.
+
+This command only works if your local (destination) branch is either an
+older copy of the parent branch with no new commits of its own, or if the
+most recent commit in your local branch has been merged into the parent
+branch.
+
+Merging from related branches
+=============================
+
+If two branches have diverged (both have unique changes) then **bzr merge** is the appropriate command to use. Merge will automatically calculate the changes that exist in the branch you're merging from that are not in your branch and attempt to apply them in your branch.
+
+::
+
+  % bzr merge URL
+
+
+If there is a conflict during a merge, 3 files with the same basename are created. The filename of the common base is appended with .BASE, the filename of the file containing your changes is appended .THIS and the filename with the changes from the other tree is appended .OTHER.
+Using a program such as kdiff3, you can now comfortably merge them into one file. To commit you have to rename it to the original basename and delete the other two files. As long as there exist files with .BASE, .THIS or .OTHER the commit command will complain.
+
+[**TODO**: explain conflict markers within files]
+
+
+Publishing your branch
+======================
+You don't need a special server to publish a bzr branch, just a normal web server.  Just mirror the files to your server, including the .bzr directory.  One can push a branch (or the changes for a branch) by one of the following three methods:
+
+ * Rsync:   rsync -avrz LOCALBRANCH  servername.com/this/directory/here
+ * (Post 0.6): bzr push sftp://servername.com/this/directory/here (sftp://servername.com/this/directory/ must already exist)
+ * The push plugin that comes with BzrTools
+

